<?php

class P2PStatusCheckActivity extends ExecuteObjectFunctionActivity
{
    function __construct($activity_history_resource)
    {
        $this->activity_history_resource = $activity_history_resource;
        parent::SplickitActivity($activity_history_resource);
    }

    function doit() {
        // first get list of all pending P2P charges
        $bca = new BalanceChangeAdapter(getM());
        $options[TONIC_FIND_BY_METADATA] = ['process'=>'P2P','notes'=>'PENDING'];
        if ($balance_change_resources = Resource::findAll($bca,null,$options)) {
            foreach ($balance_change_resources as $balance_change_resource) {

                $order_id = $balance_change_resource->order_id;
//                $this->order = new Order($order_id);
//                $ucid = $this->order->getUcid();
                $service_name = $balance_change_resource->cc_processor;
                if ($payment_service = PaymentGod::getPaymentServiceByServiceName($service_name)) {
                    $transaction_id = $balance_change_resource->cc_transaction_id;
                    $status_results = $payment_service->checkStatusOfTransactionId($transaction_id);
                    logData($status_results,"WORLD PAY RESULTS");
                    if ($status_results['status'] == 'completed') {
                        $balance_change_resource->notes = 'COMPLETED-' . getRawStamp();
                        $balance_change_resource->save();
                        $this->getOrderMessagesAndSetLocked($order_id,'N');
                    } else if ($status_results['status'] == 'pending') {
                        // check on how long, fail after 90 minutes
                        if ($balance_change_resource->created < (time() - (90*60))) {
                            if ($payment_service->cancelTransaction($transaction_id)) {
                                $balance_change_resource->notes = 'EXPIRED';
                                $balance_change_resource->save();
                                $this->getOrderMessagesAndSetLocked($order_id,'C');
                                $this->sendEmailToUser('expired',$order_id);
                            }
                        }
                    } else if ($status_results['status'] == 'declined') {
                        // cancel order messages
                        $balance_change_resource->notes = 'DECLINED' . getRawStamp();
                        $balance_change_resource->save();

                        $this->getOrderMessagesAndSetLocked($order_id,'C');
                        // email consumer that they have declined the payment request and the order has been cancelled
                        $this->sendEmailToUser('declined',$order_id);
                    } else {
                        $message = "ERROR!!!  transaction_id could not be validated";
                        myerror_log($message);
                    }
                }
            }
        } else {
            myerror_log("There are no pending P2P Charges");
            return true;
        }
    }

    function sendEmailToUser($method,$order_id)
    {
        $full_order = CompleteOrder::staticGetCompleteOrder($order_id);
        $to_addr = $full_order['user_email'];
        $merchant_name = $full_order['merchant_name'];
        $subject = "Update on your online order to $merchant_name";
        $first_name = $full_order['first_name'];
        if ($method == 'declined') {
            $message = "Hello $first_name \r\r  It seems you have delined the charge for your order to $merchant_name. The order has been cancelled.";
        } else if ($method == 'expired') {
            $message = "Hello $first_name \r\r  It seems we have been unable to reach you to confirm the charge for your order to $merchant_name. The order has been cancelled.";
        }
        myerror_log('ERROR EMAIL SENT TO USER!: '.$to_addr.' : '.$subject.' : '.$message);
        MailIt::sendUserEmailMandrill($to_addr, $subject, $message, $this->full_order['merchant_name'], null, null);
    }

    function getOrderMessagesAndSetLocked($order_id,$locked)
    {
        $merchant_message_history_adapter = new MerchantMessageHistoryAdapter(getM());
        $message_options[TONIC_FIND_BY_METADATA] = ['order_id' => $order_id, 'locked' => 'X'];
        if ($message_resources = Resource::findAll($merchant_message_history_adapter, null, $message_options)) {
            foreach ($message_resources as $message_resource) {
                $message_resource->locked = $locked;
                $message_resource->save();
            }
        } else {
            $message = "ERROR!!!!  couldnt find messages to SET to $locked for P2P  payment.  order_id: $order_id";
            myerror_log($message);
            MailIt::sendErrorEmail("P2P Message Release Error", $message);
            SmsSender2::sendEngineeringAlert($message);
        }
    }

}
?>