<?php

require_once "smarty" . DIRECTORY_SEPARATOR . "libs" . DIRECTORY_SEPARATOR . "Smarty.class.php";

class StatsEmailActivity extends SplickitActivity
{
	
	var $number_of_days_back;
	var $skin;
	var $brands_to_send = array();
	var $merchant_message_history_id; // for testing puposes
	var $date_of_end;
	
	function __construct($resource)
	{
		parent::SplickitActivity($resource);
		$string_of_brands = "150,152,248,333,366,346,390,393,395,372,409,421,425";
		$brands = explode(',',$string_of_brands);
		foreach ($brands as $brand_id) {
			$this->brands_to_send["$brand_id"] = 1;
		}
		$this->date_of_end = getMySqlFormattedDateTimeFromTimeStampAndTimeZone(time(), 'GMT');
	}

	function doit() {
		$skin_records = $this->getActiveBrandsFromSkins();
		foreach ($skin_records as $skin) {
			$brand_id = $skin['brand_id'];
			$this->skin = $skin;
			if ($this->brands_to_send["$brand_id"]) {
				if ($email_body = $this->processBrand($brand_id)) {
					myerror_logging(5,$email_body);
					$email_list = 'arosenthal@order140.com;krussell@order140.com;tarek@order140.com';
					$this->merchant_message_history_id = MailIt::sendEmailToList($email_list, "Stats Email", $email_body);
				} else {
					myerror_log("No email body was generated");
				}
			} else {
				myerror_log('skipping this brand: '.$brand_id);
			}
		}
		return true;
	}
	
	function processBrand($brand_id)
	{
		if ($data = $this->getAllBrandData($brand_id)) {
			$this->saveStats($brand_id, $data);
			$resource = Resource::dummyfactory($data);
			$email = EmailService::staticGetCrmTemplate();
			$smarty = new Smarty();
            $smarty->assign('resource',$resource);
            $content = $smarty->fetch("eval:$email");
            return $content;
		} 
	}
	
	function getAllBrandData($brand_id)
	{
		$data['skin_external_identifier'] = $this->skin['external_identifier'];
		$data['total_web_customers'] = number_format($this->getTotalWebCustomers($brand_id),0);
		$data['total_mobile_customers'] = number_format($this->getTotalMobileCustomers($brand_id),0);
		$this->setDaysBack(30);
		$data['total_new_customers'] = number_format($this->getNewCustomers($brand_id),0);
		$data['total_orders_for_period'] = number_format($this->getTotalOrdersForPeriod($brand_id),0);
		$data['total_dollars_for_period'] = number_format($this->getTotalDollarsForPeriod($brand_id),0);
		$data['average_ticket_size'] = number_format($this->getAverageTicketSize($brand_id),2);
		$data['lifetime_transacted_dollars'] = number_format($this->getLifeTimeTransactedDollars($brand_id),0);
		$this->setDaysBack(30);
		$data['top_locations'] = $this->getTopLocations($brand_id);
		$this->setDaysBack(90);
		$data['top_ordering_customers'] = $this->getTopOrderingCustomersForPeriod($brand_id);
		$data['top_spending_customers'] = $this->getTopSpendingCustomersForPeriod($brand_id);
		$week_ending = date('F j');
		$data['week_ending'] = $week_ending;
		return $data;
	}
	
	/**
	 * @desc Will convert the querried data to json and store in the db
	 * @param $data
	 */
	function saveStats($brand_id,$data)
	{
		$json = json_encode($data);
		myerror_log($json);
		$bswera = new BrandStatsEmailWeeklyRecordAdapter($mimetypes);
		if ($bswera->read_only) {
			$resource = Resource::dummyfactory(array("brand_id"=>$brand_id,"for_week_ending"=>date('Y-m-d'),"data_as_json"=>$json));
		} else {
			$resource = Resource::createByData($bswera, array("brand_id"=>$brand_id,"for_week_ending"=>date('Y-m-d'),"data_as_json"=>$json));
		}
		return $resource;
	}

	function getActiveBrandsFromSkins()
	{
		$brand_adapter = new BrandAdapter($mimetypes);
		$sql = "SELECT * FROM Skin Where brand_id IS NOT NULL AND mobile_app_type != 'A' AND in_production = 'Y' AND logical_delete = 'N'";
		$options[TONIC_FIND_BY_SQL] = $sql;
		$records = $brand_adapter->getRecords($data, $options);
		return $records;
	}
	
	function getNumberFromUserSql($sql)
	{
		$user_adapter = new UserAdapter($mimetypes);
		$options[TONIC_FIND_BY_SQL] = $sql;
		if ($results = $user_adapter->getRecord($data, $options)) {
			$number = $results['number'];
		}
		return $number;
	}

	function getTotalUsersWebAndMobile($brand_id)
	{
		$total_mobile_users = $this->getTotalMobileCustomers($brand_id);
		$total_web_users = $this->getTotalWebCustomers($brand_id);
		return $total_mobile_users + $total_web_users;
	}
	
	function getTotalMobileCustomers($brand_id)
	{
		$sql = "SELECT count(distinct a.device_id) as number FROM User a JOIN Skin b ON a.skin_id = b.skin_id WHERE b.brand_id = $brand_id AND a.created <= '".$this->date_of_end."' and a.logical_delete = 'N'";
		return $this->getNumberFromUserSql($sql);
	}
	
	function getTotalWebCustomers($brand_id)
	{
		$sql = "SELECT count(*) as number FROM User a JOIN Skin b ON a.skin_id = b.skin_id WHERE b.brand_id = $brand_id AND a.created <= '".$this->date_of_end."' AND a.device_id IS NULL and a.logical_delete = 'N'";
		return $this->getNumberFromUserSql($sql);
	}
	
	function getTotalNewCusomtersInTheLastWeek($brand_id)
	{
		$this->setDaysBack(7);
		return $this->getNewCustomers($brand_id);
	}
	
	function getNewCustomers($brand_id)
	{
		$days_back = $this->number_of_days_back;
		$sql = "SELECT count(*) as number FROM User a JOIN Skin b ON a.skin_id = b.skin_id WHERE b.brand_id = $brand_id AND a.created <= '".$this->date_of_end."' AND a.created > DATE_SUB('".$this->date_of_end."',INTERVAL $days_back DAY) AND a.logical_delete = 'N'";
		return $this->getNumberFromUserSql($sql);
	}
	
	function getAverageTicketSize($brand_id)
	{
		$days_back = $this->number_of_days_back;
		$sql = "SELECT AVG(a.grand_total) as number FROM Orders a JOIN Skin b ON a.skin_id = b.skin_id WHERE b.brand_id = $brand_id AND a.created <= '".$this->date_of_end."' AND a.created > DATE_SUB('".$this->date_of_end."',INTERVAL $days_back DAY) AND a.status = 'E' AND a.logical_delete = 'N'";
		return $this->getNumberFromUserSql($sql);
		
	}
	
	function getTotalOrdersForPeriod($brand_id)
	{
		$days_back = $this->number_of_days_back;
		$sql = "SELECT count(order_id) as number FROM Orders a JOIN Skin b ON a.skin_id = b.skin_id WHERE b.brand_id = $brand_id AND a.created <= '".$this->date_of_end."' AND a.created > DATE_SUB('".$this->date_of_end."',INTERVAL $days_back DAY) AND a.status = 'E' AND a.logical_delete = 'N'";
		return $this->getNumberFromUserSql($sql);
		
	}
	
	function getTotalDollarsForPeriod($brand_id)
	{
		$days_back = $this->number_of_days_back;
		$sql = "SELECT SUM(a.grand_total) as number FROM Orders a JOIN Skin b ON a.skin_id = b.skin_id WHERE b.brand_id = $brand_id AND a.created <= '".$this->date_of_end."' AND a.created > DATE_SUB('".$this->date_of_end."',INTERVAL $days_back DAY) AND a.status = 'E' AND a.logical_delete = 'N'";
		return $this->getNumberFromUserSql($sql);
		
	}
	
	function getLifeTimeTransactedDollars($brand_id)
	{
		$sql = "SELECT SUM(a.grand_total) as number FROM Orders a JOIN Skin b ON a.skin_id = b.skin_id WHERE b.brand_id = $brand_id AND a.created <= '".$this->date_of_end."' AND a.status = 'E' AND a.logical_delete = 'N'";
		return $this->getNumberFromUserSql($sql);
	}
	
	function getTopLocations($brand_id)
	{
		//$sql = "Select distinct merchant_id FROM Orders JOIN Skin b ON a.skin_id = b.skin_id WHERE b.brand_id = $brand_id
		$days_back = $this->number_of_days_back;
		$sql = "SELECT w.merchant_id, w.address1, w.city, w.state, w.weekly_order_total, w.weekly_revenue_total FROM (SELECT a.merchant_id, b.address1, b.city, b.state,count(a.order_id) weekly_order_total, SUM(a.grand_total) weekly_revenue_total FROM Orders a JOIN Merchant b ON a.merchant_id = b.merchant_id AND b.brand_id = $brand_id AND a.created <= '".$this->date_of_end."' AND a.created > DATE_SUB('".$this->date_of_end."',INTERVAL $days_back DAY) AND a.status = 'E' AND a.logical_delete = 'N' GROUP BY a.merchant_id) w ORDER BY w.weekly_revenue_total desc limit 10";
		$locations = $this->getOrderSql($sql);
		foreach ($locations as &$location) {
			$weekly_revenue_total = $location['weekly_revenue_total'];
			$location['weekly_revenue_total'] = number_format($weekly_revenue_total,2);
		}
		return $locations;
	}
	
	function getTopOrderingCustomersForPeriod($brand_id)
	{
		$days_back = $this->number_of_days_back;
		$sql = "SELECT y.user_id, y.first_name, y.last_name, y.email, y.number_of_orders, y.period_total FROM (SELECT a.user_id, b.first_name, b.last_name, b.email, count(a.order_id) number_of_orders, SUM(a.grand_total) period_total FROM Orders a JOIN Skin c ON a.skin_id = c.skin_id JOIN User b ON a.user_id = b.user_id WHERE c.brand_id = $brand_id AND a.created <= '".$this->date_of_end."' AND a.created > DATE_SUB('".$this->date_of_end."',INTERVAL $days_back DAY) AND a.status = 'E' AND a.logical_delete = 'N' GROUP BY a.user_id) y ORDER BY y.number_of_orders desc limit 10";
		$customers = $this->getOrderSql($sql);
		foreach ($customers as &$customer) {
			$period_total = $customer['period_total'];
			$customer['period_total'] = number_format($period_total,2);
		}
		return $customers;
	}
	
	function getTopSpendingCustomersForPeriod($brand_id)
	{
		$days_back = $this->number_of_days_back;
		$sql = "SELECT y.user_id, y.first_name, y.last_name, y.email, y.number_of_orders, y.period_total FROM (SELECT a.user_id, b.first_name, b.last_name, b.email, count(a.order_id) number_of_orders, SUM(a.grand_total) period_total FROM Orders a JOIN Skin c ON a.skin_id = c.skin_id JOIN User b ON a.user_id = b.user_id WHERE c.brand_id = $brand_id AND a.created <= '".$this->date_of_end."' AND a.created > DATE_SUB('".$this->date_of_end."',INTERVAL $days_back DAY) AND a.status = 'E' AND a.logical_delete = 'N' GROUP BY a.user_id) y ORDER BY y.period_total desc limit 10";
		return $this->getOrderSql($sql);
	}
	
	function getOrderSql($sql)
	{
		$order_adapter = new OrderAdapter($mimetypes);
		$options[TONIC_FIND_BY_SQL] = $sql;
		if ($results = $order_adapter->getRecords($data, $options)) {
			return $results;
		}
	}
	
	function setDaysBack($number_of_days_back)
	{
		$this->number_of_days_back = $number_of_days_back;
	}
	
}
?>