<?php

class DeviceCallInHistoryAdapter extends MySQLAdapter
{
    var $pulled_message_types;

	function DeviceCallInHistoryAdapter($mimetypes)
	{
		parent::MysqlAdapter(
			$mimetypes,
			'device_call_in_history',
			'%([0-9]{4,15})%',
			'%d',
			array('merchant_id'),
			NULL,
			array('last_call_in')
			);
        $this->log_level = 0;
	}

	function getCallInResourceByMerchantId($merchant_id)
    {
        return Resource::find($this,"$merchant_id");
    }
    function getAutoTurnedOffByMerchantId($merchant_id)
    {
        if ($record = $this->getRecord(array('merchant_id'=>$merchant_id))) {
            if ( $record['auto_turned_off'] == 1) {
                return true;
            }
        }
        return false;
    }

    function getLastCallInByMerchantId($merchant_id)
    {
        $record = $this->getRecord(array('merchant_id'=>$merchant_id));
        $last_call_in = $record['last_call_in'];
        return $last_call_in;
    }

    function getLastCallInAsIntegerByMerchantId($merchant_id)
    {
        $record = $this->getRecord(array('merchant_id'=>$merchant_id));
        $last_call_in_as_integer = $record['last_call_in_as_integer'];
        return $last_call_in_as_integer;
    }

    function createRecord($merchant_id)
	{
		$merchant_id = intval($merchant_id);
		myerror_logging(3,"about to retrieve or create record for merchant_id: ".$merchant_id);
		// had to do this because of 2 digit demo merchants
		$merchant_data['merchant_id'] = $merchant_id;
		$m_options[TONIC_FIND_BY_METADATA] = $merchant_data;
		$cih_resource = Resource::findOrCreateIfNotExists($this, null, $m_options);
		$this->saveLastCallIn($cih_resource);
	}
	
	private function saveLastCallIn($cih_resource,$device_base_format = 'Z')
	{
        $cih_resource->last_call_in = time();
        $cih_resource->last_call_in_as_integer = time();
        $cih_resource->device_base_format = $device_base_format;
        if ($cih_resource->auto_turned_off == 1) {
            $merchant_id = $cih_resource->merchant_id;
            myerror_log("about to turn merchant_id: $merchant_id back to Orderin On from call in history adapter");
            $sql = "UPDATE Merchant SET ordering_on = 'Y' WHERE merchant_id = $merchant_id LIMIT 1";
            $this->_query($sql);
            if ($this->rows_updated == 1) {
                if($this->isGrubhubMerchant($merchant_id)){
                    $this->activateGrubhubMerchant($merchant_id, true);
                }
                myerror_log("Merchant is AUTO turned back on since printer has now called in");
                $text = "Merchant device is auto turned back on for merchant_id: $merchant_id, device_format: $device_base_format";
                MailIt::sendErrorEmailSupport("Merchant Device Back On Line, Turning Ordering Back On",$text);
                $cih_resource->auto_turned_off = 0;
            } else {
                $merchant_resource = Resource::find(new MerchantAdapter(getM()),"$merchant_id");
                if ($merchant_resource->ordering_on == 'Y') {
                    $cih_resource->auto_turned_off = 0;
                }
                myerror_log("COULD NOT UPDATE MERCHANT TO TURN ORDERING BACK ON");
            }
        }
        $this->setLogLevel(0);
		return $cih_resource->save();		
		
	}
	
	/**
	 * @desc will record the last call in for the device
	 */
	static function recordPullCallIn($merchant_id,$device_base_format = 'Z')
	{
		$dciha = new DeviceCallInHistoryAdapter(getM());
		$merchant_data['merchant_id'] = $merchant_id;
		$m_options[TONIC_FIND_BY_METADATA] = $merchant_data;
		$cih_resource = Resource::findOrCreateIfNotExists($dciha, null, $m_options);
        $dciha->saveLastCallIn($cih_resource,$device_base_format);
	}

    function noActiveMerchantMessageMapRecord($merchant_id,$base_format)
    {
        return false == $this->checkForValidMerchantMessageMapRecord($merchant_id,$base_format);
    }

    function checkForValidMerchantMessageMapRecord($merchant_id,$base_format)
    {
        $sql = "SELECT * FROM Merchant_Message_Map WHERE merchant_id = $merchant_id AND message_format LIKE '$base_format%' AND logical_delete = 'N'";
        $options[TONIC_FIND_BY_SQL] = $sql;
        if ($message_map_resource = Resource::find(new MerchantMessageMapAdapter(getM()),null, $options)) {
            return true;
        } else {
            return false;
        }
    }

	function getNonRecentlyCalledInDevicesOtherThanGPRS($minutes)
    {
        $now = time();
        $results = [];
        $threshold_time = $now - ($minutes*60);
        $sql = "SELECT a.* FROM device_call_in_history a JOIN Merchant b ON a.merchant_id = b.merchant_id WHERE a.last_call_in_as_integer < $threshold_time AND a.auto_turned_off = 0 AND b.active = 'Y' AND b.ordering_on = 'Y' AND b.logical_delete = 'N'";
        $options[TONIC_FIND_BY_SQL] = $sql;
        if ($late_call_in_resources = Resource::findAll($this,null,$options)) {
            $pulled_message_types = $this->loadPulledMessageFormats();
            foreach ($late_call_in_resources as $late_call_in_resource) {
                // validate that there is an active record in merchant_message_map table
                $merchant_id = $late_call_in_resource->merchant_id;
                $base_format = $late_call_in_resource->device_base_format;
                if ($this->noActiveMerchantMessageMapRecord($merchant_id,$base_format)) {
                    // first delete call-in record
                    $delete_sql = "DELETE FROM device_call_in_history WHERE merchant_id = $merchant_id AND device_base_format = '$base_format'";
                    $this->_query($delete_sql);
                    // now skip to the next record
                    continue;
                }
                if ($record = $pulled_message_types[$late_call_in_resource->device_base_format]) {
                    $name = $record['name'];
                    $results["$name"][] = $late_call_in_resource;
                }
            }
        }
        return $results;
    }

    function loadPulledMessageFormats()
    {
        $message_controller_template_infos_adapter = new MessageControllerTemplateInfosAdapter(getM());
        $this->pulled_message_types = $message_controller_template_infos_adapter->getPulledMessageFormats();
        return $this->pulled_message_types;
    }

    function deactivateMerchantDueToNoCallInByDeviceCallInResource($device_call_in_history_resource)
    {
        $merchant_adapter = new MerchantAdapter(getM());
        $merchant_id = $device_call_in_history_resource->merchant_id;
        if ($merchant_resource = Resource::find($merchant_adapter,"$merchant_id")) {
            $merchant_resource->ordering_on = 'N';
            $merchant_resource->inactive_reason = 'F';
            $merchant_resource->save();
            $merchant_adapter->auditTrailForInternalFunction("INTERNAL: Auto Shut OFf /merchants/".$merchant_resource->merchant_id);
            $device_call_in_history_resource->auto_turned_off = 1;
            $device_call_in_history_resource->save();
            $key = $device_call_in_history_resource->device_base_format;
            //$auto_turned_off_infos[] = "merchant_id: $merchant_id, device_format: $key";

            // test for closed hours and if so, then skip the email
            $hour_adapter = new HourAdapter(getM());

            if (! $hour_adapter->isMerchantOpenAtThisTime($merchant_id,$merchant_resource->time_zone,'R')) {
                // merchant is closed,  so test if this is RRT and dont send the email
                if (getProperty('bucket_prefix') == 'net.rrtusa') {
                    return false;
                }
            }
            if($this->isGrubhubMerchant($merchant_id)){
                $this->activateGrubhubMerchant($merchant_id, false);
            }
            return $merchant_resource;
        }
    }
    function isGrubhubMerchant($merchant_id){
        $options = array();
        $options[TONIC_FIND_BY_METADATA]['merchant_id'] = $merchant_id;
        $adapter = new SkinMerchantMapAdapter(getM());
        $skinRecords = $adapter->select('',$options);

        $skinIdsArray = array_column($skinRecords,skin_id);
        $skinIds =implode($skinIdsArray,',');

        $skinAdapter = new SkinAdapter(getM());
        $sql = "SELECT count(*) as result FROM Skin s WHERE s.skin_id in (".$skinIds.") AND s.skin_type = 'GH'";
        $skin_options[TONIC_FIND_BY_SQL] = $sql;
        $skin_select_resource = Resource::find($skinAdapter, null, $skin_options);
        $counter = (int)$skin_select_resource->result;
        if($counter > 0){
            return true;
        }
        return false;
    }


    function activateGrubhubMerchant($merchant_id, $activate){
	    try{
            $grubhub = new GrubhubService();
            $grubhub_merchant_id = $grubhub->getGHMerchant($merchant_id);
            if(!$grubhub_merchant_id){
                return;
            }
            $grubhub->setUri($grubhub_merchant_id);
            $uri = $grubhub->getUri();
            if($activate){
                $grubhub->generateOnlineBody();
            }else{
                $grubhub->generateOfflineBody();
            }
            $body = $grubhub->getBody();
            $hostUrl = $grubhub->getHostUrl();
            $host = explode('//',$hostUrl)[1];
            $port = $grubhub->getPort();
            $ext = $grubhub->getExt();
            $grubhub->setRequestMethod("PUT");
            $request_method = $grubhub ->getRequestMethod();
            $secret = $grubhub->getSecret();
            $client_id = $grubhub->getClientId();
            $grubhub->generateNonce();
            $nonce = $grubhub->getNonce();
            $body_hash = $grubhub->hashBody($body);
            # Normalize the pieces of the request to ensure the hash, in the next step, matches what the service expects
            $normalized_request = $grubhub->normalizeRequest($nonce, $request_method, $uri, $host, $port, $body_hash, $ext);
            # HMAC SHA256 hash the normalized request using the client secret and base64 encode the result
            $hashed_normalized_request = $grubhub->hashNormalizedRequest($normalized_request, $secret);
            # Generate the value that should actually be used in the "Authorization" HTTP header
            $header_value = $grubhub->createHeaderValue($client_id, $nonce, $body_hash, $ext, $hashed_normalized_request);
            $grubhub_url = $grubhub->getRequestUrl();
            return $grubhub->setMerchantStatus($grubhub_url, $header_value, $body);

        }catch (Exception $e){
            myerror_log("ERROR!  could not get Grubhub for Merchant_id> ".$merchant_id);
        }
    }
}
?>