<?php

require_once 'lib'.DIRECTORY_SEPARATOR.'adapters'.DIRECTORY_SEPARATOR.'holidayhouradapter.php';

class HourAdapter extends MySQLAdapter
{
	private $this_weeks_hours;
	private $this_weeks_hours_type;
	private $closed_message;
	private $merchant_status_message;
	private $holiday;
	private $day_array = array("Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday");

	public $merchant_hour_records;
	
	protected $current_time;
	
	private $local_order_and_pickup_time_array = array();
	function HourAdapter($mimetypes)
	{
		parent::MysqlAdapter(
			$mimetypes,
			'Hour',
			'%([0-9]{1,15})%',
			'%d',
			array('hour_id')
			);
		$this->log_level = $_SERVER['log_level'];
		$this->current_time = time();
	}
	
	function setCurrentTime($time_stamp)
	{
		$this->current_time = $time_stamp;
	}
	
	// just a test function
	function getNowAsInt()
	{
		$now_as_int = $this->current_time;
		return $now_as_int;
	}
	 
	function &select($url, $options = NULL)
    {
    	$options[TONIC_FIND_BY_METADATA]['logical_delete'] = 'N';
    	return parent::select($url,$options);
    }
    
    function getMerchantHoursForWeek($starting_time_stamp,$merchant_id,$hour_type,$reload = false)
    {
    	if ($this->this_weeks_hours && $this->this_weeks_hours_type == $hour_type && !$reload)
    		return $this->this_weeks_hours;
    	return $this->loadMerchantHoursForWeek($starting_time_stamp, $merchant_id, $hour_type);
    }
    
    function getAllMerchantHourRecords($merchant_id)
    {
    	$hours = array();
		$hours['pickup'] = $this->getHours($merchant_id, 'R');
		$hours['delivery'] = $this->getHours($merchant_id, 'D');
		return $hours;
		
    }

    /**
     * 
     * @desc For pickup right now. gets an array of days of the week with human readable hours    "Monday"=>"08:00am-05:00pm"
     * 
     * @param $merchant_id
     */
    function getAllMerchantHoursHumanReadable($merchant_id)
    {
    	if ($hours = $this->getAllMerchantHourRecords($merchant_id)) {
	    	if ($pickuphours = $hours['pickup']) {
	    		$the_array = $this->getHumanReadableHoursForFullWeeksHourRecords($pickuphours);
				$sunday = array_shift($the_array);
				$the_array[] = $sunday;
				return $the_array;
	    	}
    	} else {
    		return false;
    	}
    }

    function newGetAllMerchantHoursHumanReadable($merchant_id)
    {
        if ($hours = $this->getAllMerchantHourRecords($merchant_id)) {

            if ($pickuphours = $hours['pickup']) {
                $the_array = $this->getHumanReadableHoursForFullWeeksHourRecords($pickuphours);
                $sunday = array_shift($the_array);
                $the_array[] = $sunday;
                $hours['pickup'] = $the_array;
            }

            if ($delivery_hours = $hours['delivery']) {
                $the_array = $this->getHumanReadableHoursForFullWeeksHourRecords($delivery_hours);
                $sunday = array_shift($the_array);
                $the_array[] = $sunday;
                $hours['delivery'] = $the_array;
            }
            return $hours;
        } else {
            return false;
        }
    }

    function getAllMerchantHoursHumanReadableV2($merchant_id)
	{
		if ($hours = $this->getAllMerchantHourRecords($merchant_id)) {
			$human_readables = array();
			$hole_hours_adapter = new HoleHoursAdapter(getM());
			$hole_hours = $hole_hours_adapter->getByMerchantIdOrganizedByHourType($merchant_id);
			foreach ($hours as $type=>$day_array){
				$base_array = $this->getHumanReadableHoursForFullWeeksHourRecords($day_array,$hole_hours[$type]);
				foreach ($base_array as $index=>$row) {
					$day = (string)$index+1;
					$human_readables[$type]["$day"] = $row;
				}
			}
			return $human_readables;
		} else {
			return false;
		}
	}

	function getHumanReadableHoursForFullWeeksHourRecords($hour_records,$hole_hour_records = null)
	{
		for($i=0;$i<7;$i++) {
			$open = $hour_records[$i]['open'];
			$close = $hour_records[$i]['close'];
			
			// now convert to ampm
			$open = $this->getAmPmStringFromTimeOfDayMilitary($open);
			$close = $this->getAmPmStringFromTimeOfDayMilitary($close);
			$day_index = $hour_records[$i]['day_of_week'];
			$day = $this->day_array[$day_index-1];
			if ($hour_records[$i]['day_open'] == 'Y') {
			    if ($_SERVER['IS_SODEXO_LEVELUP']) {
			        // do not show human readable hole hours to level up, they cant handle mid day closures
                    $the_array[] = array($day=>$open."-".$close);
                } else if ($hole = $hole_hour_records[$day_index]) {
                    $hole_start = $this->getAmPmStringFromTimeOfDayMilitary($hole['start_time']);
                    $hole_end = $this->getAmPmStringFromTimeOfDayMilitary($hole['end_time']);
                    if ($hole['second_start_time'] > 0 && $hole['second_end_time'] > 0) {
                        $second_hole_start = $this->getAmPmStringFromTimeOfDayMilitary($hole['second_start_time']);
                        $second_hole_end = $this->getAmPmStringFromTimeOfDayMilitary($hole['second_end_time']);
                        $the_array[] = array($day=>$open."-".$hole_start.":".$hole_end."-".$second_hole_start.":".$second_hole_end."-".$close);
                    } else {
                        $the_array[] = array($day=>$open."-".$hole_start.":".$hole_end."-".$close);
                    }

                } else {
                    $the_array[] = array($day=>$open."-".$close);
                }
			} else {
				$the_array[] = array($day => "closed");
			}
		}
		return $the_array;
	}

    /**
     * 
     * @desc will take a military time and convert to an am/pm time   05:45  ->  5:45am    or     15:45  ->  3:45pm
     * 
     * @param $time  
     */
    function getAmPmStringFromTimeOfDayMilitary($time)
    {
    	$time_split = explode(":",$time);
    	if ($time_split[0] < 12)
    		$string = $time_split[0].":".$time_split[1]."am";
    	else if ($time_split[0] == 12)
    		$string = $time_split[0].":".$time_split[1]."pm";
    	else if ($time_split[0] > 12)
    	{
    		$new_hour = $time_split[0] - 12;
    		$string = $new_hour.":".$time_split[1]."pm";
    	}
    	if (substr($string, 0,1) == "0")
    		$string = substr($string, 1);
    	return $string;
    }

    function getHours($merchant_id,$hour_type)
    {
		$hours = $this->getRecords(array("merchant_id"=>$merchant_id,"hour_type"=>$hour_type));
		return $hours;
    }
    
    function getHolidaysThatFallOnTheseDays($merchant_id,$days)
    {
    	$holiday_array = array();
    	$holiday_hour_adapter = new HolidayHourAdapter($this->mimetypes);
		$holiday_hour_options[TONIC_FIND_BY_METADATA]['merchant_id'] = $merchant_id;
		$holiday_hour_options[TONIC_FIND_BY_METADATA]['the_date'] = array("IN"=>$days);
		if ($holiday_hours = $holiday_hour_adapter->select('',$holiday_hour_options))
		{
			myerror_logging(3, "we have a holiday during the next week");
			$holiday = true;
			foreach ($holiday_hours as $holiday_hour)
			{
				//resolve to date
				$the_date = date('Y-m-d',$holiday_hour['the_date']);
				myerror_logging(3, "THe holiday is on day: ".$the_date);
				$holiday_array[$the_date] = $holiday_hour;
			}
		} else {
			myerror_logging(3, "we DO NOT have a holiday during the next week");
		}    	
		return $holiday_array;
    }

    /**
     * 
     * @desc Will return an array for the next 7 days open,close,day_open,holiday
     * @param $days 
     * @param $hours (hour records from the db)
     * @param $holiday_array (array of which days are holidays in the next 7 with the hours for that day)
     */
    function getTheWeeksHoursFromDaysHoursAndHolidaysDuringThisWeek($days,$hours,$holiday_array)
    {
		foreach ($days as $day)
		{
			// get day of week
			$day_of_week = date('w',strtotime($day))+1;
			// loop through hours
			foreach ($hours as $hour)
			{
				if ($hour['day_of_week'] == $day_of_week)
				{
					$hour['date'] = $day;
					//check for holiday
					if (isset($holiday_array[$day]))
					{
						myerror_logging(3, "now setting the holiday in the date array on: ".$day);
						$holiday_day = $holiday_array[$day];
						$hour['open'] = $holiday_day['open'];
						$hour['close'] = $holiday_day['close'];
						$hour['day_open'] = $holiday_day['day_open'];
						$hour['holiday'] = 'Y';
					}	else {
						$hour['holiday'] = 'N';
					}
					$this_weeks_hours[] = $hour;
				}	
			}
		}
		return $this_weeks_hours;
    }
    
    function getDaysArrayFromStartingTimeStamp($starting_time_stamp)
    {
		//myerror_logging(3,"the starting time stamp is: ".$testing);
		$m = date("m",$starting_time_stamp);
		$d = date("d",$starting_time_stamp);
		$y = date("Y",$starting_time_stamp);
		
		// load days array for starting time stamp
		for ($inc = 0; $inc < 7; $inc++)
			$days[] = date('Y-m-d',mktime(0, 0, 0, $m , $d+$inc, $y));
    	return $days;
    }
    
    /**
     * 
     * @desc Will load up the merchant hours array for the next 7 days open,close,day_open,holiday
     * @param $days 
     * @param $hours (hour records from the db)
     * @param $holiday_array (array of which days are holidays in the next 7 with the hours for that day)
     * @return array or false
     */
    
	function loadMerchantHoursForWeek($starting_time_stamp,$merchant_id,$hour_type)
	{
		
		$starting_local_date_time = date("l F j, Y, g:i a e",$starting_time_stamp);
		myerror_logging(3, "starting loadMerchantHoursForWeek with a starting local time of: ".$starting_local_date_time);
		$hours = $this->getHours($merchant_id, $hour_type);
		if (sizeof($hours) < 7)
		{
			$message = "There are no $hour_type hours set up for this merchant.  Please check the setup.    merchant_id: ".$merchant_id. "    user: ".$_SERVER['PHP_AUTH_USER'];
			//MailIt::sendErrorEmailSupport('ERROR no hours set up for merchant', 'merchant_id: '.$merchant_id.' --  for hour_type: '.$hour_type.' --  user: '.$_SERVER['PHP_AUTH_USER'].'  --  from: '.getRequestingDevicesAppType().' -- v'.getRequestClientVersion().' --  '.$_SERVER['STAMP']);
			MailIt::sendErrorEmailSupport("ERROR no hours set up for merchant", $message);
			return false;
		}
		
		$days = $this->getDaysArrayFromStartingTimeStamp($starting_time_stamp);
		
		$holiday_array = $this->getHolidaysThatFallOnTheseDays($merchant_id, $days);
		$this_weeks_hours = $this->getTheWeeksHoursFromDaysHoursAndHolidaysDuringThisWeek($days, $hours, $holiday_array);
		$this->this_weeks_hours = $this_weeks_hours;
		$this->this_weeks_hours_type = $hour_type;
		return $this_weeks_hours;	
	}

	/**
	 * @desc Will return the timestamp of the close for COB scheduling of the passed in timestamp as the day in question.  basically finds out which hour type closes last.  not passing in a timestamp results in find the COB for now (today)
	 * 
	 * @param $merchant_mixed
	 * @param $time_stamp
	 */
	function getOpenAndCloseForCOBInTimeStampFormForMerchantId($merchant_mixed,$time_stamp = 0)
	{
		// if a time is not passed in, then set to now (today)
		if ($time_stamp == 0) {
            $time_stamp = $this->current_time;
        }
		if (is_a($merchant_mixed,'Resource')) {
		    $merchant_resource = $merchant_mixed;
		    $merchant_id = $merchant_resource->merchant_id;
        } else if (is_numeric($merchant_mixed)) {
		    $merchant_id = $merchant_mixed;
            $merchant_resource = Resource::find(new MerchantAdapter($this->mimetypes),''.$merchant_id);
        } else {
		    throw new Exception("unknown object passed in to getOpenAndCloseForCOBInTimeStampFormForMerchantId: $merchant_mixed");
        }
		$time_zone = $merchant_resource->time_zone;
		$tz = date_default_timezone_get();
		setTheDefaultTimeZone($time_zone,$merchant_resource->state);

		$pickup_hours = $this->getTSForOpenAndCloseForHourType($merchant_id,'R',$time_stamp);
		$delivery_hours = $this->getTSForOpenAndCloseForHourType($merchant_id,'D',$time_stamp);
		
		if ($pickup_hours && $delivery_hours)
		{
			$combined_hours['local_open_date'] = $pickup_hours['local_open_date'];
			if ($pickup_hours['open_ts'] <= $delivery_hours['open_ts'])
			{
				$combined_hours['open_ts'] = $pickup_hours['open_ts'];
				$combined_hours['local_open'] = $pickup_hours['local_open'];
				$combined_hours['local_open_dt_tm'] = $pickup_hours['local_open_dt_tm'];
			} else {
				$combined_hours['open_ts'] = $delivery_hours['open_ts'];
				$combined_hours['local_open'] = $delivery_hours['local_open'];
				$combined_hours['local_open_dt_tm'] = $delivery_hours['local_open_dt_tm'];
			}
			
			if ($pickup_hours['close_ts'] >= $delivery_hours['close_ts'])
			{
				$combined_hours['close_ts'] = $pickup_hours['close_ts'];
				$combined_hours['local_close'] = $pickup_hours['local_close'];
				$combined_hours['local_close_dt_tm'] = $pickup_hours['local_close_dt_tm'];
			} else {
				$combined_hours['close_ts'] = $delivery_hours['close_ts'];
				$combined_hours['local_close'] = $delivery_hours['local_close'];
				$combined_hours['local_close_dt_tm'] = $delivery_hours['local_close_dt_tm'];
			}
			return $combined_hours;
		} else if ($pickup_hours) {
			return $pickup_hours;
		} else if ($delivery_hours) {
			return $delivery_hours;
		} else {
			return false;
		}
		
	}
	
	/**
	 * 
	 * @desc will generate an epoch time stamp for the open and close for the given merchant and hour type and number of days. 1 = today, 2 = today and tomorrow, etc.  
	 * @desc PLEASE NOTE:  time zone MUST BE SET before calling this function
	 * 
	 * @param $merchant_id
	 * @param $hour_type
	 * @param $total_days
	 */
	function getNextOpenAndCloseTimeStamps($merchant_id,$hour_type,$total_days,$right_now = 0)
	{
		if ($right_now == 0) {
            $right_now = $this->current_time;
        }
		$open_close_ts = array();
		if ($this->merchant_delivery_price_distance_resource) {
		    $total_days = $this->merchant_delivery_price_distance_resource->max_days_out;
        }
		for ($i = 0 ; $i < $total_days ; $i++)
		{
			$time_stamp = $right_now + ($i * 24 * 60 * 60);
			if ($hours = $this->getTSForOpenAndCloseForHourType($merchant_id, $hour_type,$time_stamp))
			{
				$open_close_ts[$i] = array("open"=>$hours['open_ts'],"close"=>$hours['close_ts']);
			} else {
			    // need to add a blank so we dont blow up the expectation of 7 days of hours in CompleteMerchant line 145. i probably need to change that at some point?
			    $open_close_ts[$i] = array();

			}
			
		}
		if ($_SERVER['log_level'] > 3)
		{
			myerror_log("results for getNextOpenAndCloseTimeStamps in houradapter");
			$i= 1;
			foreach($open_close_ts as $day)
			{
			    if ($day) {
                    if ($open = $day['open']) {
                        $open_string = date("Y-m-d H:i:s",$open);
                    }

                    if ($close = $day['close']) {
                        $close_string = date("Y-m-d H:i:s",$close);
                    }

                    myerror_log("day $i:  o - $open_string     c - $close_string");

                } else {
                    myerror_log("day $i:  o - CLOSED     c - CLOSED");
                }
                $i++;

			}

		}
		return $open_close_ts;
	}
	
	/**
	 * 
	 * @desc will generate an epoch time stamp for the todays open and close for the given merchant and hour type.  PLEASE NOTE:  time zone MUST BE SET before calling this function
	 * 
	 * @param $merchant_id
	 * @param $hour_type
	 */
	function getTSForOpenAndCloseForHourType($merchant_id,$hour_type,$time_stamp=0)
	{
		// if a time is not passed in, then set to now (today)
		if ($time_stamp == 0)
			$time_stamp = $this->current_time;
		$date = date('Y-m-d',$time_stamp);
		$m = date("m",$time_stamp);
		$d = date("d",$time_stamp);
		$Y = date("Y",$time_stamp);

		$current_time = date('H:i:s',$time_stamp);

		$todays_hours = $this->getHourRecordForDate($date, $merchant_id, $hour_type);
		$open = $todays_hours['open'];
		$close = $todays_hours['close'];
        $day_open = $todays_hours['day_open'];
        $using_yesterdays_hours = false;
		if ($day_open == 'N' || $current_time < $open) {
		    if ($day_open == 'N') {
		        // if its closed today lets set open to be noon just so we dont blow things up
                $open = '12:00';
            }
		    // check to see if we are still in yesterdays hours if after midnight close
            $yesterday_time_stamp = $time_stamp-(24*3600);
            $yesterdays_date = date('Y-m-d', $yesterday_time_stamp);
            $yesterdays_hours = $this->getHourRecordForDate($yesterdays_date, $merchant_id, $hour_type);
            $yesterday_open = $yesterdays_hours['open'];
            $yesterday_close = $yesterdays_hours['close'];
            $yesterday_is_open = $yesterdays_hours['day_open'];
            if ($yesterday_is_open == 'Y' && $yesterday_close < $open && $current_time < $yesterday_close) {
                // use yesterday's info
                $open = $yesterday_open;
                $close = $yesterday_close;
                $day_open = $yesterday_is_open;
                $using_yesterdays_hours = true;
                $date = $yesterdays_date;
                $m = date("m",$yesterday_time_stamp);
                $d = date("d",$yesterday_time_stamp);
                $Y = date("Y",$yesterday_time_stamp);
            }
        }
		if ($day_open == 'Y') {
			
			$o = explode(':',$open);
			$open_ts = mktime($o[0], $o[1], '0', $m , $d, $Y);

			$c = explode(':',$close);

			$close_ts = mktime($c[0], $c[1], '0', $m, $d, $Y);
		    if ($close_ts < $open_ts) {
		        // after midnight close so add 24 hours to the close ts
                $close_ts = $close_ts + (24 * 3600);
            }
			$close_date = date('Y-m-d',$close_ts);
			$hours['local_open_date'] = $date;
			$hours['local_open'] = $open;
			$hours['local_open_dt_tm'] = $date.' '.$open;
			$hours['local_close'] = $close;
			$hours['local_close_dt_tm'] = $close_date.' '.$close;
			$hours['open_ts'] = $open_ts;
			$hours['close_ts'] = $close_ts;
			return $hours;
		}
		return false;		
	}

	function getTodaysHours($merchant_id,$hour_type,$time_zone = -100,$time = 0)
	{
		if ($time == 0)
			$time = $this->current_time;
		return $this->getHourRecordForTimeStamp($time, $merchant_id, $hour_type,$time_zone);
	}
	
	function getHourRecordForTimeStamp($timestamp,$merchant_id,$hour_type,$time_zone = -100)
	{
		myerror_logging(3, "starting getHourRecordForTimeStamp");
		$time_zone_string = date_default_timezone_get();
		if ($time_zone != -100)
			setTheDefaultTimeZone($time_zone);
		// this will resolve to the date of the merchants time zone since it should have been set already either in code somewhere else or passed in here
		//  a -100 means it was set somewhere else
		$date = date('Y-m-d',$timestamp);
		$record = $this->getHourRecordForDate($date, $merchant_id, $hour_type);
		date_default_timezone_set($time_zone_string);
		return $record;
	}
	
	function getHourRecordForDate($submitted_date,$merchant_id,$hour_type)
	{
		myerror_logging(3, "starting getHourRecordForDate");
		if ($this->this_weeks_hours)
		{
			myerror_logging(3, "in the this weeks hours");
			foreach ($this->this_weeks_hours as $day_hours)
			{
				if ($submitted_date == $day_hours['date'])
					return $day_hours;
			}
		} else {
			myerror_logging(3, "in the NOT this weeks hours (hours haven't been loaded yet)");
			$holiday_hour_adapter = new HolidayHourAdapter(getM());
			$holiday_hour_options[TONIC_FIND_BY_METADATA]['merchant_id'] = $merchant_id;
			$holiday_hour_options[TONIC_FIND_BY_METADATA]['the_date'] = $submitted_date;
			if ($hours = $holiday_hour_adapter->select('',$holiday_hour_options)) {
				//$this->holiday = true;// ok loaded up with the holiday hours
				myerror_logging(3, "we got holiday hours");
				$hour = array_pop($hours);
				$hour['holiday'] = 'Y';
			} else {
				//$this->holiday = false;
				myerror_logging(3, "did not find any holiday hours");
                $day_of_week = date('w',strtotime($submitted_date))+1;
                $hour = $this->getHoursForDayAndType($merchant_id,$hour_type,$day_of_week);
				$hour['holiday'] = 'N';
			}
			return $hour;
		}
		
	}

	function getHoursForDayAndType($merchant_id,$hour_type,$day_of_week)
    {
        $merchant_hours = $this->getTheMerchantHourRecords($merchant_id);
        return $merchant_hours[$hour_type][$day_of_week];
    }

	function getTheMerchantHourRecords($merchant_id)
    {
        if (isset($this->merchant_hour_records)) {
            return $this->merchant_hour_records;
        } else {
            $hour_hash = [];
            $hour_records = $this->getRecords(['merchant_id'=>$merchant_id]);
            foreach ($hour_records as $hour_record) {
                $hour_type = $hour_record['hour_type'];
                $day_of_week = $hour_record['day_of_week'];
                $hour_hash[$hour_type][$day_of_week] = $hour_record;
            }
            $this->merchant_hour_records = $hour_hash;
        }
        return $this->merchant_hour_records;
    }
	
	function getNext90()
	{
		myerror_logging(2,"starting get next 90 lead times");
		$gmt_available_times = array();
		$minimum_lead = isset($this->merchant_resource) ? $this->merchant_resource->lead_time : 10 ;
		$now = $this->current_time+($minimum_lead * 60);
        $now_plus_90 = $now + 5400;
		return $this->getAvailableTimesFormattedToEvenMinutes($now, $now_plus_90);
	}
	

	function getAvailableTimesFormattedToEvenMinutes($start_ts,$stop_ts)
	{
		$available_times = array();
		$i = 0;
		$interval = 1;
		while ($start_ts < $stop_ts) {
			//myerror_logging(2,"adding: ".date("l F j, Y, g:i a",$start_ts));
			$available_times[] = $start_ts;
			$min = ''.date("i",$start_ts);
			$dig = substr($min, 1);
		 	if ($i == 10)
			{ 
				if ($dig == 0 || $dig == 5)
					$interval = 5;
				else if ($dig < 5)
					$interval = 5 - $dig;
				else if ($dig > 5)
					$interval = 10 - $dig;
			} else {
                if ($i > 10) {
                    $interval = 5;
                }
            }
			$start_ts = $start_ts + ($interval*60);
			$i++;
		}
		return $available_times;
	}
	
	function buildCurrentPickupTimesArray()
	{
		;// first get next 90 min
		
		// then get remaining with an interval of 15, min delivery window of 90, max days out of 0?  could work

	}

	/**
	 * 
	 * @desc Takes a date string, eg: 2012-10-15, and returns the local open and close date time strings for a merchant in an array object 
	 * @return $local['local_open_dt_tm'] <br>$local['local_close_dt_tm']<br>$local['local_open_date'] <br>$local['merchant_id']
	 * @return $local['local_close_dt_tm']<br>
	 * @return $local['local_open_date'] <br>
	 * @return $local['merchant_id']
	 * 
	 * @param $merchant_id
	 * @param $date
	 * @param $hour_type
	 */
	function getLocalOpenAndCloseDtTmForDate($merchant_id,$date,$hour_type)
	{
    	$hour_record = $this->getHourRecordForDate($date, $merchant_id, $hour_type);
    	$open = $hour_record['open'];
		$close = $hour_record['close'];
		$local_open_dt_tm = $date.' '.$open;	
		if ($open > $close)
		{
			//inverted hours so get hour record of tomorrow
			$tomorrow = strtotime ( '+1 day' , strtotime($date) ) ;
			$tomorrow = date ( 'Y-m-d' , $tomorrow );
			$local_close_dt_tm = $tomorrow.' '.$close;
		} else {
			$local_close_dt_tm = $date.' '.$close;
		}
		$local['local_open_dt_tm'] = $local_open_dt_tm;
		$local['local_close_dt_tm'] = $local_close_dt_tm;
		$local['local_open_date'] = $date;
		$local['merchant_id'] = $merchant_id;
		return $local;
		
	}
	
	function getLocalOpenAndCloseDtTmForTimestamp($merchant_id,$time_stamp,$hour_type)
	{
		;//$this->getHour
		
	}
		
	function isMerchantOpenAtThisTime($merchant_id,$time_zone,$hour_type,$the_time = 0)
	{
        if ($the_time == 0) {
            $now = true;
            $the_time = time();
        }
        if ($hour_type == 'D') {
            $delivery_string = ' for delivery';
        } else {
            $delivery_string = '';
        }
        $tzone_string = date_default_timezone_get();
        // now set the TZ if it wasn't set before calling the function. a -100 indicates we set it already (stupid)
        if ($time_zone != -100) {
            $merchant_resource = Resource::find(new MerchantAdapter(getM()),''.$merchant_id);
            setTheDefaultTimeZone($time_zone,$merchant_resource->state);
        }

        // will return false if the merchant is not open today at all
        if ($open_close_ts = $this->getTSForOpenAndCloseForHourType($merchant_id,$hour_type,$the_time)) {
            $tz = date_default_timezone_get();
            myerror_logging(3,"the default timezone is: ".$tz);

            //set merchant local times
            $local_order_time = date("Y-m-d H:i:s",$this->current_time);
            $local_pickup_time = date("Y-m-d H:i",$the_time);
            $this->local_order_and_pickup_time_array['local_order_time'] = $local_order_time;
            $this->local_order_and_pickup_time_array['local_pickup_time'] = $local_pickup_time;
            myerror_logging(3,"local order time: ".$local_order_time);
            myerror_logging(3,"local pickup time: ".$local_pickup_time);

            $gmt_open = $open_close_ts['open_ts'];
            $gmt_close = $open_close_ts['close_ts'];

            myerror_logging(3,"local_open: ".date("g:i a",$gmt_open));
            myerror_logging(3,"local_close: ".date("g:i a",$gmt_close));
            myerror_logging(3,"local pickup time: ".date("g:i a",$the_time));
            $message = '';
            if ($gmt_open <= $the_time && $the_time <= $gmt_close) {
                myerror_logging(3,"merchant is open");
                $merchant_open = true;
                $minutes_till_close = ($gmt_close - $the_time)/60;
                myerror_logging(3,'minutes till close is: '.$minutes_till_close);
                if ($minutes_till_close < 60 && $now) {
                    $message = "Please note that this merchant will close$delivery_string at ".date('g:i a',$gmt_close)." today";
                }
            } else if ($gmt_open > $the_time) {
                myerror_logging(3,"merchant is not open yet.  merchant will open$delivery_string at ".date('g:i a',$gmt_open));
                if ($now)
                    $message = "Merchant is not open$delivery_string yet.  Merchant will open$delivery_string at ".date('g:i a',$gmt_open);
                else
                    $message = "Sorry, this merchant is closed$delivery_string at your requested pickup time. Merchant will open$delivery_string at ".date('g:i a',$gmt_open);
                $merchant_open = false;
            } else if ($gmt_close < $the_time) {
                myerror_logging(3,"merchant has closed$delivery_string for the day.  Merchant closed$delivery_string at ".date('g:i a',$gmt_close));
                if ($now)
                    $message = "Sorry, this merchant has closed$delivery_string for the day.  Merchant closed$delivery_string at ".date('g:i a',$gmt_close);
                else
                    $message = "We're sorry, this merchant is closed$delivery_string at your requested time. Merchant closes$delivery_string at ".date('g:i a',$gmt_close);
                $merchant_open = false;
            }

            if ($merchant_open) {
                //check for donut hole
                $hole_adapter = new HoleHoursAdapter($m);
                if ($row = $hole_adapter->getByMerchantIdAndDayOfWeekAndOrderType($merchant_id,date('w',$the_time)+1,$hour_type)) {
                    myerror_log("THere are hole hours");
                    $hole_start = $this->getTimeStampOfDayHour($row['start_time'],$the_time);
                    $hole_stop = $this->getTimeStampOfDayHour($row['end_time'],$the_time);
                    if ($hole_start < $the_time && $the_time < $hole_stop) {
                        myerror_log("WE ARE IN THE HOLE!!!!");
                        $merchant_open = false;
                    } else if ($row['second_start_time'] > 0 && $row['second_end_time'] > 0 ) {
                        if ($row['second_start_time'] < $the_time && $the_time < $row['second_end_time']) {
                            myerror_log("WE ARE IN THE SECOND HOLE!!!!");
                            $merchant_open = false;
                        }
                    }
                }
            }
        } else {
			$message = "Sorry this merchant is closed today$delivery_string .";
//			if ($hours['holiday'] == 'Y')
//			{
//				$message = $_SERVER['GLOBAL_PROPERTIES']['holiday_closed_message'];
//				$this->holiday = true;
//			}
			$merchant_open = false;
		}
		
//CHANGE_THIS			
		date_default_timezone_set($tzone_string);  // reset the timezone back to what it was.  before calculating the open hours.  this is only relevant during order creation.
		
		$this->merchant_status_message = $message;
		return $merchant_open;
	}

	function getTimeStampOfDayHour($day_hour,$time_stamp_of_date)
	{
		$s = explode(":",$day_hour);
		$time_stamp_of_day_hour = mktime($s[0], $s[1], '0', date("m",$time_stamp_of_date)  , date("d",$time_stamp_of_date), date("Y",$time_stamp_of_date));
		return $time_stamp_of_day_hour;
	}
	
	function getRecentlyClosedMerchants($minutes,$day = 0)
	{
		
		if ($day == 0)
			$day = date('w') + 1;
		
		$default_server_timezone_offset = $_SERVER['GLOBAL_PROPERTIES']['default_server_timezone_offset'];
		$ts = time();
		$ts_sub_min = time()-($minutes*60);
		
		// this is how we get the day
		//DAYOFWEEK(DATE_SUB(NOW(),INTERVAL ($default_server_timezone_offset-a.time_zone) HOUR ))

//		if ($_SERVER['GLOBAL_PROPERTIES']['daylight_savings'])
//			$sql = "SELECT z.* FROM (SELECT a.merchant_id,DATE(DATE_SUB(NOW(),INTERVAL ($default_server_timezone_offset - a.time_zone) HOUR)) as local_close_date, c.close, c.open, DATE_ADD(DATE_ADD(DATE(DATE_SUB(NOW(),INTERVAL ($default_server_timezone_offset - a.time_zone) HOUR)), INTERVAL c.close HOUR_SECOND), INTERVAL ($default_server_timezone_offset-a.time_zone) HOUR ) as close_stamp FROM Merchant a, `Hour` c WHERE a.active = 'Y' AND a.merchant_id = c.merchant_id AND c.day_of_week = $day AND c.day_open = 'Y' AND c.merchant_id > 1000 AND c.hour_type = 'R') z WHERE z.close_stamp > DATE_SUB(NOW(), INTERVAL $minutes MINUTE) AND z.close_stamp < NOW()";
//		else
//			$sql = "SELECT z.* FROM (SELECT a.merchant_id, c.close, DATE_ADD(DATE_ADD(DATE(DATE_SUB(NOW(),INTERVAL 8 HOUR)), INTERVAL c.close HOUR_SECOND), INTERVAL (-a.time_zone) HOUR ) as close_stamp FROM Merchant a, `Hour` c WHERE a.active = 'Y' AND a.merchant_id = c.merchant_id AND c.day_of_week = $day AND c.day_open = 'Y' AND c.merchant_id > 1000 AND c.hour_type = 'R') z WHERE z.close_stamp > DATE_SUB(NOW(), INTERVAL $minutes MINUTE) AND z.close_stamp < NOW()";
		
		//if ($_SERVER['GLOBAL_PROPERTIES']['daylight_savings'])
		//	$default_server_timezone_offset = -1;
		$sql = "SELECT z.* FROM (SELECT a.merchant_id,DATE(DATE_SUB(NOW(),INTERVAL ($default_server_timezone_offset - a.time_zone) HOUR)) as local_close_date, c.close, c.open, DATE_ADD(DATE_ADD(DATE(DATE_SUB(NOW(),INTERVAL ($default_server_timezone_offset - a.time_zone) HOUR)), INTERVAL c.close HOUR_SECOND), INTERVAL ($default_server_timezone_offset-a.time_zone) HOUR ) as close_stamp, c.day_of_week  FROM Merchant a, `Hour` c WHERE c.close IS NOT NULL AND c.close > '00:00:00' AND a.active = 'Y' AND a.merchant_id = c.merchant_id AND c.day_of_week = DAYOFWEEK(DATE_SUB(NOW(),INTERVAL ($default_server_timezone_offset-a.time_zone) HOUR )) AND c.day_open = 'Y' AND c.merchant_id > 1000 AND c.hour_type = 'R') z WHERE z.close_stamp > DATE_SUB(NOW(), INTERVAL $minutes MINUTE) AND z.close_stamp < NOW()";
		$sql .= " UNION ";
		$sql .= "SELECT z.* FROM (SELECT a.merchant_id,DATE(DATE_SUB(NOW(),INTERVAL ($default_server_timezone_offset - a.time_zone) HOUR)) as local_close_date, c.second_close, c.open, DATE_ADD(DATE_ADD(DATE(DATE_SUB(NOW(),INTERVAL ($default_server_timezone_offset - a.time_zone) HOUR)), INTERVAL c.second_close HOUR_SECOND), INTERVAL ($default_server_timezone_offset-a.time_zone) HOUR ) as close_stamp, c.day_of_week  FROM Merchant a, `Hour` c WHERE c.second_close IS NOT NULL AND c.second_close > '00:00:00' AND a.active = 'Y' AND a.merchant_id = c.merchant_id AND c.day_of_week = DAYOFWEEK(DATE_SUB(NOW(),INTERVAL ($default_server_timezone_offset-a.time_zone) HOUR )) AND c.day_open = 'Y' AND c.merchant_id > 1000 AND c.hour_type = 'R') z WHERE z.close_stamp > DATE_SUB(NOW(), INTERVAL $minutes MINUTE) AND z.close_stamp < NOW()";

		myerror_logging(2,"GET CLOSED STORES: ".$sql);
		$options[TONIC_FIND_BY_SQL] = $sql;
		if ($merchants = $this->select('',$options))
		{
			myerror_logging(1,"Size of recently closed merchants is: ".sizeof($merchants));
			
			// now get the open time stamp of the recently closed merchnat
			foreach ($merchants as &$merchant)
			{
				$merchant['local_close_dt_tm'] = $merchant['local_close_date'].' '.$merchant['close'];
				
				// check to see if we have inverted hours
				if ($merchant['close'] > $merchant['open'] && $merchant['open'] != '00:00:00')
				{
					$merchant['local_open_date'] = $merchant['local_close_date'];
					$merchant['local_open_dt_tm'] = $merchant['local_open_date'].' '.$merchant['open'];
				}
				else 
				{
					// inverted hours so get open from yesterday
					$day = $merchant['day_of_week'];
					if ($day == 1)
						$day = 7;
					else 
						$day = $day - 1;
					
					$sql = "SELECT open FROM Hour WHERE merchant_id = ".$merchant['merchant_id']." AND day_of_week = $day AND hour_type = 'R'";
						
					$options[TONIC_FIND_BY_SQL] = $sql;
					$open_hour = $this->select('',$options);
					$open_hour = array_pop($open_hour);	
					$merchant['open'] = $open_hour['open'];
					
					$merchant_local_close_date = $merchant['local_close_date'];
					
					$newdate = strtotime ( '-1 day' , strtotime ( $merchant_local_close_date ) ); 
					$date = date('Y-m-d',$newdate);
					$merchant['local_open_date'] = $date;
					
					$merchant['local_open_dt_tm'] = $date.' '.$merchant['open'];
					
				}
			
				$sectoep = strtotime($merchant['close_stamp']);
				//myerror_logging(2,$merchant['merchant_id'].' '.$merchant['local_open_dt_tm'].'   '.$merchant['local_close_dt_tm'].'  '.$merchant['close_stamp']);
				myerror_logging(2,$merchant['merchant_id'].' '.$merchant['local_open_dt_tm'].'   '.$merchant['local_close_dt_tm'].'  '.$sectoep);
			}
			return $merchants;
		}
		else
			return false;
	}
   
	function getLocalOrderAndPickupTimes()
	{
		return $this->local_order_and_pickup_time_array;
	}
	
    function getMerchantStatusMessage()
    {
    	return $this->merchant_status_message;
    }
    
    function getHoliday()
    {
    	return $this->holiday;
    }
}
?>