<?php

class MenuSchedulesAdapter extends MySQLAdapter
{
    private $uses_schedule = false;

    var $using_week_in_cycle;

    var $using_day_in_cycle;
    var $hour_records_by_type_and_day;

    function __construct($mimetypes)
    {
        parent::MysqlAdapter(
            $mimetypes,
            'Menu_Schedules',
            '%([0-9]{4,15})%',
            '%d',
            array('id'),
            null,
            array('start_dt_tm','created','modified')
        );

        $this->allow_full_table_scan = true;

    }

    function &select($url, $options = NULL)
    {
        $options[TONIC_FIND_BY_METADATA]['logical_delete'] = 'N';
        return parent::select($url,$options);
    }

    /**
     * @param $menu_id int
     * @param $current_date_time DateTime
     * @return array|null
     * @throws Exception
     */
    function getInnactiveObjectsByDate($menu_id,$current_date_time)
    {
        if ($record = $this->getRecord(['menu_id'=>$menu_id,'active'=>1])) {
            $this->uses_schedule = true;
            $current_week_in_cycle = $this->getCurrentWeekInCycle($current_date_time,$record);
            $current_day_in_week = $current_date_time->format('w')+1;
            $this->using_week_in_cycle = $current_week_in_cycle;
            $this->using_day_in_cycle = $current_day_in_week;
            $off_results = [];
            $on_results = [];
            $schedule_id = $record['id'];
            $menu_object_scheduals_by_week_and_day_adapter = new MenuObjectSchedulesByWeekAndDayMapsAdapter(getM());
            foreach ($menu_object_scheduals_by_week_and_day_adapter->getRecords(['menu_schedule_id'=>$schedule_id]) as $map) {
                if ($map['week_number_in_cycle'] == $current_week_in_cycle) {
                    if ($map['day_number'] == $current_day_in_week) {
                        // week and day match so this item is on
                        $on_results[$map['object_type']][] = $map['object_id'];
                    }
                }
                $off_results[$map['object_type']][$map['object_id']] = $map['object_id'];
            }
            foreach ($on_results as $name=>$on_menu_object) {
                foreach ($on_menu_object as $id) {
                    unset($off_results[$name][$id]);
                }
                if ($off_results[$name] == []) {
                    unset($off_results[$name]);
                }
            }
            return $off_results;
        } else {
            $this->uses_schedule = false;
        }
        return null;
    }

    /**
     * @param $start_date DateTime
     * @param $current_date DateTime
     * @return int
     */
    function getCurrentWeekInCycle($current_date,$record)
    {
        $weeks_in_cycle = $record['number_of_weeks_in_cycle'];
        $starting_time_stamp = $record['start_dt_tm'];
        $hour = 0;
        $minute = 1;
        if ($this->hour_records_by_type_and_day['R']['1']['day_open'] == 'Y' ) {
            if ($this->hour_records_by_type_and_day['R']['1']['close'] < 4) {
                $h = explode(':',$this->hour_records_by_type_and_day['R']['1']['close']);
                $hour = $h[0];
                $minute = $h[1];
            }
        }
        $resolved_timestamp = getTimeStampForDateTimeAndTimeZone($hour,$minute,0,date('m',$starting_time_stamp),date('d',$starting_time_stamp),date('Y',$starting_time_stamp),$current_date->getTimezone());
        //$diff = $starting_time_stamp-$resolved_timestamp;
        $start_date = new DateTime();
        $start_date->setTimestamp($resolved_timestamp);
        $total_complete_weeks_since_start = floor($start_date->diff($current_date)->days/7);
        $remainder = $total_complete_weeks_since_start % $weeks_in_cycle;
        return $remainder+1;
    }

    function doesLastMenuCheckedUseSchedule()
    {
        return $this->uses_schedule;
    }


}
?>