<?php

class MerchantDeliveryInfoAdapter extends MySQLAdapter
{

	private $cached_price_boolean = false;
	private $catering = false;
	private $merchant_catering_delivery_active = false;

	private $user_delivery_location_resource;
	private $base_order_details;
	private $doordash_error_result;
    private $delivery_calculation_type;

    private $request_method;


	const MERCHANT_DELIVERY_NOT_ACTIVE_ERROR_MESSAGE = "We're sorry, it appears this merchant is not accepting delivery orders at the moment. This may have just changed, sorry for the inconvenience.";
	
    function MerchantDeliveryInfoAdapter($mimetypes)
    {
        parent::MysqlAdapter(
            $mimetypes,
            'Merchant_Delivery_Info',
            '%([0-9]{2,15})%',
            '%d',
            array('merchant_delivery_id'),
            null,
            array('created','modified')
        );
    }

    /**
     * @desc static function to get a MerchantDeliveryInfo resource with lat and long of the merhant;
     * 
     * @param int $merchant_id
     * 
     * returns the resource or false if it cant get the info
     */
    
    static function getFullMerchantDeliveryInfoAsResource($merchant_id)
    {
        // NOT USED ANY MORE.
        return false;
    }
    
    /**
     * 
     * @desc Used to calculate the delivery price based on Id's.  
     * 
     * @param $user_delivery_location_id
     * @param $merchant_id
     * 
     * @throws Exception with error message to user
     */
    
    function getDeliveryPriceFromIds($user_delivery_location_id,$merchant_id)
    {    	
    	if ($resource = $this->getMerchantDeliveryPriceResourceForUserLocationAndMerchantId($user_delivery_location_id,$merchant_id)) {
    		return $resource->price;
    	}
    	return false;    	
    }
    
    /**
     * 
     * @desc will get the cheapest matching merchant delivery location that the users address falls into.  
     * @param int $user_delivery_location_id
     * @param int $merchant_id
     * @return Resource
     */
    function getMerchantDeliveryPriceResourceForUserLocationAndMerchantId($user_delivery_location_id,$merchant_id)
    {
        if ($user_delivery_location_id == null) {
            myerror_log("ERROR!!! no user delivery location id subitted to get merchant delivery price");
            return createErrorResourceWithHttpCode("Sorry there was an error and your request could not be processed",500,500);
        } else if ($merchant_id == null) {
            myerror_log("ERROR!!! no merchant id subitted to get merchant delivery price");
            return createErrorResourceWithHttpCode("Sorry there was an error and your request could not be processed",500,500);
        }
    	$this->cached_price_boolean = false;
    	//if ($mdi_resource = $this->getFullMerchantDeliveryInfoAsResource($merchant_id)) { }
        $mdpd_adapter = new MerchantDeliveryPriceDistanceAdapter(getM());
        if ($this->getCatering()) {
            $mdpd_adapter->setCatering();
        }
    	if ($merchant_delivery_price_distance_records = $this->getMerchantDeliveryPriceRecords($merchant_id,$mdpd_adapter)) {
            if ($udl_resource = Resource::find(new UserDeliveryLocationAdapter(getM()),''.$user_delivery_location_id)) {
	    	    foreach ($this->base_order_details as $name=>$value) {
	    	        myerror_log("setting order details on udl_resouce $name = $value",5);
                    $udl_resource->set($name,$value);
                }
	    	    $this->user_delivery_location_resource = $udl_resource;
	    	    $mdpd_adapter->setRequestMethod($this->request_method);
	    		$result = $mdpd_adapter->getDeliveryPriceResourceFromUserDeliveryLocationAndMerchantDeliveryInfoResources($udl_resource, $merchant_delivery_price_distance_records);
	    		if (strtolower($result->delivery_price_type) == 'doordash' && $mdpd_adapter->hasDoorDashError() ) {
	    		   myerror_log("we have a door dash fail");
	    		   $this->doordash_error_result = $mdpd_adapter->getDoordashErrorResult();
	    		   $this->delivery_calculation_type = 'doordash';
	    		   if ($this->doordash_error_result['field_errors'][0]['field'] == 'external_business_name') {
                       $this->error_text = "DoorDash is reporting that delivery has not been properly set up at this merchant and cannot schedule a delivery at this time.";
                   } else {
                       $this->error_text = "DoorDash is reporting that they currently cannot deliver to this address.";
                   }

	    		   return null;
                } else {
	    		    if ($mdpd_adapter->hasNativeDeliveryError()) {
	    		        $this->error_text = $mdpd_adapter->getLastErrorText();
                    }
                    $this->cached_price_boolean = $result->cached_price_boolean == true ? true : false;
                }
	    		return $result;
	    	} else {
	    		throw new NoMatchingUserDeliveryLocationException();
	    	}
    	} else {
    	    if ($this->catering) {
    	        $type = 'catering';
            } else {
                $type = 'retail';
            }
			myerror_log("ERROR!  Merchant delivery information does not exist for $type but delivery price distance request has been submitted");
			MailIt::sendErrorEmail('Impossible error',  "Merchant delivery information does not exist for $type but delivery price distance request has been submitted.  merchant_id: $merchant_id");
    		throw new NoMerchantDeliveryInformationException();
    	}    	
    }

    function setMerchantDeliveryPriceRecords($records)
    {
        $this->merchant_delivery_price_distance_records = $records;
    }

    /**
     * @param $merchant_id
     * @param $merchant_delivery_price_distance_adapter MerchantDeliveryPriceDistanceAdapter
     * @return mixed
     */
    function getMerchantDeliveryPriceRecords($merchant_id,$merchant_delivery_price_distance_adapter)
    {
        if ($this->merchant_delivery_price_distance_records) {
            myerror_log("price distance records are cached. number of records: ".sizeof($this->merchant_delivery_price_distance_records),5);
            return $this->merchant_delivery_price_distance_records;
        } else {
            if ($merchant_delivery_price_distance_adapter == null) {
                $merchant_delivery_price_distance_adapter = new MerchantDeliveryPriceDistanceAdapter(getM());
                if ($this->catering) {
                    $merchant_delivery_price_distance_adapter->setCatering();
                }
            }
            $this->merchant_delivery_price_distance_records = $merchant_delivery_price_distance_adapter->getDeliveryPriceDistanceRecordsForMerchantOrderedByPriorityDescending($merchant_id);
            return $this->merchant_delivery_price_distance_records;
        }
    }

    function getDeliveryTaxAmount($merchant, $delivery_price)
    {
        if ($delivery_tax_rate = $this->getDeliveryTaxRateOverride($merchant['merchant_id'])) {
            myerror_log("we have a delivery tax rate override of: " . $delivery_tax_rate);
        } else {
            $lookup_adapter = new LookupAdapter(getM());
            if (strtolower($lookup_adapter->getNameFromTypeAndValue("state_delivery_is_taxed", $merchant['state'])) == 'yes') {
                $delivery_tax_rate = TaxAdapter::staticGetTotalBaseTaxRate($merchant['merchant_id']);
            } else {
                $delivery_tax_rate = 0;
            }
        }
        $delivery_tax_amount = $delivery_price * ($delivery_tax_rate / 100);
        return number_format($delivery_tax_amount, 2);
    }

    function getDeliveryTaxRateOverride($merchant_id)
    {
        $tax_adapter = new TaxAdapter(getM());
        if ($record = $tax_adapter->getRecord(array("merchant_id" => $merchant_id, "locale" => 'Delivery'))) {
            return $record['rate'];
        }
    }

    function getCachedPriceBooleanForLastCall()
    {
    	return $this->cached_price_boolean;
    }

    function setCateringDeliveryActive()
    {
        $this->merchant_catering_delivery_active = true;
    }

    function setCatering()
    {
        $this->catering = true;
    }

    function getCatering()
    {
        return $this->catering;
    }

    function getUserDeliveryLocationResource()
    {
        return $this->user_delivery_location_resource;
    }

    function setBaseOrderDetails($base_order_detials)
    {
        $this->base_order_details = $base_order_detials;
    }

    function getDeliveryCalculationType()
    {
        return $this->delivery_calculation_type;
    }

    function getDoorDashErrorResult()
    {
        return $this->doordash_error_result;
    }

    function setRequestMethod($request_method)
    {
        $this->request_method = $request_method;
    }

}

class MerchantDeliveryException extends Exception
{
    public function __construct($message)
    {
        parent::__construct($message);
    }
}

class NoMatchingUserDeliveryLocationException extends MerchantDeliveryException
{
    public function __construct() {
        parent::__construct("We're sorry, but we can't find this delivery address record, please create a new one", 530);
    }
}

class NoMerchantDeliveryInformationException extends MerchantDeliveryException
{
    public function __construct() {
        parent::__construct("We're sorry, it appears this merchant has not set up their delivery information yet so a delivery order cannot be submitted a this time", 580);
    }
}

class MerchantDeliveryNotActiveException extends MerchantDeliveryException
{
    public function __construct() {
        parent::__construct(MerchantDeliveryInfoAdapter::MERCHANT_DELIVERY_NOT_ACTIVE_ERROR_MESSAGE, 580);
    }
}

?>