<?php

Class BrinkController extends MessageController
{
    /*
      Brink Destination Id's
        Id="3" Name="Pick Up"
        Id="4" Name="Delivery 1"
        Id="4008879" Name="Delivery 2"
        Id="4009708" Name="Catering - Pick Up"
        Id="4009709" Name="Catering - Delivery"
    */

    var $destination_ids_by_order_type = array("R"=>4067027,"D1"=>4,"D2"=>4008879);
    var $location_token;
    var $format_name = 'brink';
    var $retry_delay = 2;
    var $force_send_as_pickup_to_brink = false;

    /**
     * @var BrinkService
     */
    var $brink_service;

    /**
     * @var Holds the all the brink parameters from the third party tables
     */
    var $brink_parameters;

    const BRINK_THIRD_PARTY_PROVIDER_ID = 1001;

    const BRINK_SNARFS_LARGE_TIP_ITEM_ID = 640229484;

    function BrinkController($mt,$u,&$r,$l = 0)
    {
        parent::MessageController($mt,$u,$r,$l);
//        $this->representation = '/order_templates/brink/execute_order_brink.xml';
    }

    protected function send($message_body)
    {
        myerror_log("Brink sending with body: ".cleanUpCRLFTFromString($message_body));
        $this->initializeBrinkService($this->full_order['order_id']);
        if ($response = $this->brink_service->send($message_body)) {
            $this->message_resource->message_text = $message_body;
            $this->message_resource->response = $response['raw_result'];
            return $response;
        }
    }

    /**
     * @param $order Order
     * @return Order
     */
    public function setBrinkCheckoutInfoOnOrderObject($order)
    {
        $starting_order_resource = $order->getOrderResource();
        $order_resource = $this->setBrinkCheckoutInfoOnOrderResource($starting_order_resource);
        $order_resource->save();
        return new Order($order_resource->order_id);
    }

    public function setBrinkCheckoutInfoOnOrderResource(&$order_resource)
    {
        if ($checkout_info_as_array = $this->getBrinkCheckoutInfo($order_resource->order_id)) {
            $order_resource->total_tax_amt = $checkout_info_as_array['Tax'];
            $delivery_amount = 0.00;
            if ($order_resource->order_type == 'D') {
                if ($order_resource->skip_delivery_fee) {
                    //group order participant probably
                    $delivery_amount = 0.00;
                } else {
                    if ($order_surcharge = $checkout_info_as_array['Surcharges']['OrderSurcharge']) {
                        if (substr_count(strtolower($order_surcharge['Name']), 'delivery') > 0) {
                            $delivery_amount = round(floatval($order_surcharge['Amount']), 2);
                        }
                    }
                }
                $order_resource->delivery_amt = $delivery_amount;
            }
//            if ($order_resource->promo_amt == 0.00 || $order_resource->order_type == 'D') {
//                $order_resource->order_amt = $checkout_info_as_array['Subtotal'];
//            }
            $order_resource->grand_total = $checkout_info_as_array['Total']+$order_resource->tip_amt+$order_resource->trans_fee_amt;
            $order_resource->grand_total_to_merchant = $checkout_info_as_array['Total']+$order_resource->tip_amt;
        }
        return $order_resource;
    }

    function initializeBrinkService($order_id,$method = 'SubmitOrder')
    {
        $this->brink_service = new BrinkService($this->location_token,$method);
        if (isset($this->brink_parameters['brink_domain'])) {
            myerror_log("we are getting the domain from the 3rd party table: ".$this->brink_parameters['brink_domain']);
            $this->brink_service->setDomain($this->brink_parameters['brink_domain']);
        }
        $this->brink_service->setOrderId($order_id);
        $this->brink_service->setMerchantId($this->merchant['merchant_id']);
        $this->brink_service->setBrandId($this->merchant['brand_id']);
    }

    public function getBrinkCheckoutInfo($order_id)
    {
        $resource = $this->prepMessageForSending(Resource::dummyfactory(array("order_id"=>$order_id,"message_format"=>'BC')));
        $this->initializeBrinkService($order_id,'CalculateOrder');
        $response = $this->brink_service->send($resource->message_text);
        if ($response['http_code'] == 200) {
            return $response['Order'];
        } else {
            return false;
        }
    }

    public function prepMessageForSending($message_resource)
    {
        $resource = parent::prepMessageForSending($message_resource);
        $resource->message_text = cleanUpDoubleSpacesCRLFTFromString($resource->message_text);
        $resource->message_text = str_replace('<Modifiers> </Modifiers>','<Modifiers i:nil="true"/>',$resource->message_text);
        $resource->message_text = str_replace('<Modifiers></Modifiers>','<Modifiers i:nil="true"/>',$resource->message_text);
        $resource->message_text = str_replace('<Modifiers/>','<Modifiers i:nil="true"/>',$resource->message_text);
        $this->location_token = $this->getBrinkPickupLocation($resource->merchant_id);
        return $resource;
    }

    public function populateMessageData($message_resource)
    {
        $resource = parent::populateMessageData($message_resource);
        $this->getBrinkThirdPartyValues($resource->skin_id,$resource->merchant_id,$this->merchant['brand_id']);
        if ($this->force_send_as_pickup_to_brink) {
            myerror_log("we are forcing a pickup order on the call to brink so that paricipants do not get charged a delivery fee");
            unset($resource->delivery_info);
            $resource->order_type = 'R';
        } else if (isset($resource->delivery_info)) {
            $resource->delivery_info->address2 = str_replace('&','and',$resource->delivery_info->address2);
            $resource->delivery_info->instructions = str_replace('&','and',$resource->delivery_info->instructions);
        }
        $modifier_top_level_items = [];
        foreach ($resource->order_details as &$order_details) {
            foreach ($order_details['order_detail_complete_modifier_list_no_holds'] as &$modifier) {
                $gm = explode(":",$modifier['external_id']);
                $m = explode("-",$gm[0]);
                $modifier['external_modifier_group_id'] = $m[0];
                if (is_null($m[1])) {
                    $modifier_item_id = $modifier['modifier_item_id'];
                    throw new Exception("Bad format for brink modifier external id on modifier_item_id: $modifier_item_id. should be of the form 'external_modifier_group_id-external_modifier_item_id: ".$modifier['external_id']);
                }
                $modifier['external_id'] = $m[1];
                if ($gm[1]) {
                    $c = explode("-",$gm[1]);
                    $child_modifier = [];
                    $child_modifier['mod_price'] = 0.00;
                    $child_modifier['external_modifier_group_id'] = $c[0];
                    $child_modifier['external_id'] = $c[1];
                    $child_modifier['mod_quantity'] = 1;
                    $child_modifier['comes_with'] = 'N';
                    $modifier['child_modifier'] = $child_modifier;
                }
                if ($m[0] == 'PLU') {
                    if (isStringFieldNullOrEmptyOnArray($order_details,'external_id') || $order_details['external_id'] == 'use_modifier') {
                        $order_details['external_id'] = $m[1];
                        $order_details['price'] = $modifier['mod_price'];
                        $order_details['item_total'] = $modifier['mod_price'];
                        // this is the fix to make sure multile modifier propogate to
                        $order_details['item_total'] = $modifier['mod_total_price'];
                        if ($modifier['mod_quantity'] > 1) {
                            $order_details['quantity'] = $modifier['mod_quantity'];
                        }
                    } else {
                        $t = [];
                        $t['external_id'] = $m[1];
                        $t['price'] = $modifier['mod_price'];
                        $t['item_total'] = $modifier['mod_total_price'];
                        $t['quantity'] = $modifier['mod_quantity'];

                        $order_details['item_total'] = $order_details['item_total'] - $modifier['mod_total_price'];
                        $order_details['external_modifier_group_id'] = 'PLU';
                        $modifier_top_level_items[] = $t;
                    }
                }
            }


            foreach ($order_details['order_detail_hold_it_modifiers'] as &$modifier) {
                $m = explode("-",$modifier['external_id']);
                $modifier['external_modifier_group_id'] = $m[0];
                if (is_null($m[1])) {
                    $modifier_item_id = $modifier['modifier_item_id'];
                    throw new Exception("Bad format for brink modifier external id on modifier_item_id: $modifier_item_id. should be of the form 'external_modifier_group_id-external_modifier_item_id: ".$modifier['external_id']);
                }
                $modifier['external_id'] = $m[1];
                if ($m[0] == 'PLU') {
                    $order_details['external_id'] = $m[1];
                    $order_details['price'] = $modifier['mod_price'];
                    $order_details['item_total'] = $modifier['mod_price'];
                    //$modifier['external_id'] = '';
                }
                // get active flag
                if ($modifier_size_map_record = ModifierSizeMapAdapter::staticGetRecordByPrimaryKey($modifier['modifier_size_id'],'ModifierSizeMapAdapter')) {
                    $modifier['active'] = $modifier_size_map_record['active'];
                } else {
                    $modifier['active'] = 'N';
                }
            }
        }
        foreach ($modifier_top_level_items as $add_item_that_was_a_modifier) {
            $resource->order_details[] = $add_item_that_was_a_modifier;
        }
        //logData($resource->getDataFieldsReally(),'RESOURCE');
        myerror_log("brand_id = ".$resource->merchant['brand_id'],3);
        myerror_log("order status = ".$resource->order_status,3);



        if ($resource->points_used > 0 && $resource->amount_off_from_points > 0.00) {
            myerror_log("amount off from points: ".$resource->amount_off_from_points);
            $resource->loyalty_payment_amount = $resource->amount_off_from_points;
            $resource->loyalty_discount_id = $this->getBrinkBrandLoyaltyDiscountId($resource->merchant['brand_id']);
        }
        //get new checkout info from brink to assign promo and delivery details on SubmitOrder template
        if ($message_resource->message_format != 'BC') {
            $checkout_info = $this->getBrinkCheckoutInfo($message_resource->order_id);
            $resource->set('brink_tip_value',$resource->tip_amt);
            $resource->set("checkout_info",$checkout_info);
            $resource->set("brink_grand_total",number_format($checkout_info['Total'],2,'.', ''));
            $resource->set("brink_large_tip","false");
            if (isset($checkout_info['Surcharges'])) {
                $resource->set("surcharges","true");
            }
        }

        if ($message_resource->message_format == 'BO') {
            $resource->set("type_bo","true");
        } else {
            $resource->set("type_bo","false");
        }

        $resource = $this->setBrinkPickupAndOrderTimeStringsOnResource($resource);

        //$resource->tip_item_id = isProd()?'1234567':'4012226';
        $resource->tip_item_id = $this->getBrinkBrandTipId($resource);
        $resource->destination_id = $this->getBrinkDestinationId($resource);
        $resource->set("discount_id",$this->getBrinkBrandDiscountId($resource->merchant['brand_id']));
        $resource->set('note_id_for_brand',$this->getBrinkBrandNoteId($resource->merchant['brand_id']));
        if (isset($resource->group_order_payments)) {
            // just use the CC tender id for the whole things
            $tender_ids[] = ['id'=>$this->brink_parameters['brink_tender_id'],'amount'=>$resource->grand_total];
        } else {
            $tender_ids = $this->getBrinkSkinTenderId($resource->payments);
        }
        $resource->set('tender_id',$tender_ids[0]['id']);
        $tip_accounted_for = false;
        if ($resource->tip_amt < $tender_ids[0]['amount']) {
            $tender_ids[0]['amount'] = $tender_ids[0]['amount'] - $resource->tip_amt;
            $tip_accounted_for = true;
        }
        $resource->set('tender_id_amount',number_format($tender_ids[0]['amount'],2));
       /* if ($tender_ids[1]) {
            $resource->set('additional_tender_id',$tender_ids[1]['id']);
            if ($tip_accounted_for) {
                $resource->set('additional_tender_id_amount',number_format($tender_ids[1]['amount'],2));
            } else {
                $additional_amount = $tender_ids[1]['amount'] - $resource->tip_amt;
                $resource->set('additional_tender_id_amount',number_format($additional_amount,2));
            }

        } */
        return $resource;
    }

    public function getBrinkThirdPartyValues($skin_id,$merchant_id,$merchant_brand_id)
    {
        if (! isset($this->brink_parameters)) {
            $parameters = getAllThirdPartyParametersForMerchantSkinCombination($skin_id,$merchant_id,$merchant_brand_id,self::BRINK_THIRD_PARTY_PROVIDER_ID);
            $this->brink_parameters = $parameters['Brink'];
        }
        return $this->brink_parameters;
    }


    public function setBrinkPickupAndOrderTimeStringsOnResource($resource)
    {
        $tz = date_default_timezone_get();
        date_default_timezone_set('GMT');
        $fake_time = (getTimeStampForDateTimeAndTimeZone(12, 0, 0, date('m'), date('d'), date('Y'), 'GMT') + (24*60*60));
        $fake_brink_pickup_time_string = date("Y-m-d\TH:i:00.0000\Z",$fake_time);  // need this for the call to calculate order
        $brink_pickup_time_string = date("Y-m-d\TH:i:00.0000\Z",$resource->pickup_dt_tm);
        $brink_order_time_string = date("Y-m-d\TH:i:00.0000\Z",$resource->order_dt_tm);
        myerror_logging(3,"setting brink pickup and order time strings to: $brink_pickup_time_string    and     $brink_order_time_string     respectively");
        $resource->set("brink_pickup_time_string",$brink_pickup_time_string);
        $resource->set("brink_order_time_string",$brink_order_time_string);
        $resource->set("fake_brink_pickup_time_string",$fake_brink_pickup_time_string);
        date_default_timezone_set($tz);
        return $resource;
    }

    function getBrinkBrandTipId()
    {
        //if ($tip_id = $this-)
        if ($tip_id = $this->brink_parameters['brink_tip_id']) {
            return $tip_id;
        }
    }

    function getBrandIdFromCurrentOrderSkinId($skin_id)
    {
        if ($skin_id == 5) {
            return 150;
        } else if ($skin_id = 6) {
            return 152;
        } else if ($skin_id = 298) {
            return 578;
        } else {
            throw new Exception("NO BRAND associdate with skin_id: $skin_id  in Brink Controller");
        }
    }

    function getBrinkBrandLoyaltyDiscountId($brand_id)
    {
        if ($loyalty_id = $this->brink_parameters['brink_loyalty_discount_id']) {
            return $loyalty_id;
        }

        //backwards compatability
//        if ($brand_id == 150) {
//            return 640226525;
//        }
    }

    function getBrinkBrandDiscountId($brand_id)
    {
        if ($discount_id = $this->brink_parameters['brink_promo_discount_id']) {
            return $discount_id;
        }

        //backwards compatability
//        if ($brand_id == 152) {
//            $discount_id = 640212223;
//        } else if ($brand_id == 437) {
//            $discount_id = 3;
//        } else if ($brand_id == 150) {
//            $discount_id = 640225352;
//        }
//        return $discount_id;
    }

    function getBrinkBrandNoteId($brand_id)
    {
        if ($note_id_for_brand = $this->brink_parameters['brink_note_id']) {
            return $note_id_for_brand;
        }

        //backwards compatability
//        if ($brand_id == 152) {
//            $note_id_for_brand = 640212020;
//        } else if ($brand_id == 282) {
//            $note_id_for_brand = 4067754;
//        } else if ($brand_id == 437) {
//            $note_id_for_brand = 212223;
//        } else if ($brand_id == 150) {
//            $note_id_for_brand = 640225565;
//        }
//        return $note_id_for_brand;
    }

    function getBrinkBrandTenderId($payments)
    {
        $tender_ids = [];
        foreach ($payments as $payment) {
            if ($payment['process'] == 'Order') {
                continue;
            } else if ($payment['process'] == 'StoredValuePayment' && isset($this->brink_parameters['brink_stored_value_tender_id'])) {
                $tender_ids[] = ['id'=>$this->brink_parameters['brink_stored_value_tender_id'],'amount'=>$payment['charge_amt']];
            } else {
                $tender_ids[] = ['id'=>$this->brink_parameters['brink_tender_id'],'amount'=>$payment['charge_amt']];
            }
        }
        return $tender_ids;
    }
    function getBrinkSkinTenderId($payments)
    {
        $tender_ids = [];
        foreach ($payments as $payment) {
            if ($payment['process'] == 'Order') {
                continue;
            } else if ($payment['process'] == 'StoredValuePayment' && isset($this->brink_parameters['brink_stored_value_tender_id'])) {
                $tender_ids[] = ['id'=>$this->brink_parameters['brink_stored_value_tender_id'],'amount'=>$payment['charge_amt']];
            } else if($payment['charge_amt'] > 0){
                $tender_ids[] = ['id'=>$this->brink_parameters['brink_tender_id'],'amount'=>$payment['charge_amt']];
            } else {
                $tender_ids[] = ['id' => $this->brink_parameters['brink_tender_id'], 'amount' => $payment['grand_total']];
            }
        }
        return $tender_ids;
    }

    function getBrinkDestinationId($resource)
    {
        $order_type = $resource->order_type;

        if ($order_type == 'D') {
            if ($merchant_delivery_price_distance_id = $resource->merchant_delivery_price_distance_id) {
                $mdpda = new MerchantDeliveryPriceDistanceAdapter(getM());
                if ($merchant_delivery_price_distance_resource = Resource::find($mdpda,"$merchant_delivery_price_distance_id")) {
                        $notes = $merchant_delivery_price_distance_resource->notes;
                        $s = explode(';', $notes);
                        $notes_data = ['dummy'=>'value'];
                        foreach ($s as $tuple) {
                            $nv = explode('=', $tuple);
                            $notes_data[$nv[0]] = $nv[1];
                        }
                        if (isset($notes_data['brink_destination_id'])) {
                            $destination_id = $notes_data['brink_destination_id'];
                            return $destination_id;
                        } else {
                            myerror_log("ERROR!!! NO brink destination ID set on  Merchant_Delivery_Price_Distance record");
                        }
                } else {
                    myerror_log("ERROR!!! could not find merchant delivery price distance resource from id: $merchant_delivery_price_distance_id");
                }
            }
            // now test third party
            if ($destination_id = $this->brink_parameters['brink_delivery_destination_id']) {
                return $destination_id;
            }
        } else {
            if ($destination_id = $this->brink_parameters['brink_pickup_destination_id']) {
                return $destination_id;
            }
        }
        myerror_log("unable to get Brink destination id from third party tables. use legacy method");


        // backwards compatability
//        $sa = new SkinAdapter(getM());
//        $skin_record = $sa->getRecordFromPrimaryKey($resource->skin_id);
//        if ($skin_record['brand_id'] == 152) {
//            //pickup moose
//            $destination_id = '640212044';
//            return $destination_id;
//        } else if ($skin_record['brand_id'] == 150) {
//            // snarfs
//            if ($order_type == 'D') {
//                // return 640225350;
//                return 640225739;
//            } else {
//                // return 640223061;
//                return 640225738;
//            }
//            return $this->destination_ids_by_order_type[$order_type];
//        } else if ($skin_record['brand_id'] == 578) {
//            // doublz
//            if ($order_type == 'D') {
//                return 217254;
//            } else {
//                return 217253;
//            }
//        } else {
//            return "NO DESITINATION ID SET UP FOR THIS BRAND: ".$skin_record['brand_id'];
//        }

    }

    public function getBrinkPickupLocation($merchant_id) {
        if ($loction_token = $this->brink_parameters['brink_location_token']) {
            return $loction_token;
        }

        // backwards compatability
        $mbima = new MerchantBrinkInfoMapsAdapter(getM());
        $tpppmma = new ThirdPartyProviderParameterMerchantMapsAdapter(getM());
        if($record = $mbima->getRecord(array("merchant_id"=>$merchant_id))) {
            myerror_log("We have the brink location token: " . $record['brink_location_token']);
            return $record['brink_location_token'];
        } else if ($paramters = $tpppmma->getMerchantParameters($merchant_id,self::BRINK_THIRD_PARTY_PROVIDER_ID)) {
                if ($this->brink_location_token = $paramters['Brink']['brink_location_token']) {
                    return $this->brink_location_token;
                } else {
                    MailIt::sendErrorEmailSupport("Couldn't find Brink location mapping for $merchant_id", "Merchant $merchant_id did not receive message for order. Merchant does not have a Brink location mapping. Orders will not go through until this is corrected.");
                    throw new NoMerchantBrinkLocationException($merchant_id);
                }
        } else {
            MailIt::sendErrorEmailSupport("Couldn't find Brink location mapping for $merchant_id", "Merchant $merchant_id did not receive message for order. Merchant does not have a Brink location mapping. Orders will not go through until this is corrected.");
            throw new NoMerchantBrinkLocationException($merchant_id);
        }
    }
}

class NoMerchantBrinkLocationException extends Exception
{
    public function __construct($merchant_id) {
        parent::__construct("No Brink location data for merchant_id: $merchant_id", 100);
    }
}

?>