<?php
Class ComprisController extends MessageController
{

    var $importer;
    var $merchant_resource;
    var $store_is_currently_open;

    protected $representation = '/order_templates/compris/execute_order_compris.xml';
    protected $format = 'O';
    protected $format_name = 'compris';

    protected $compris_info = [];



    const CUSTOM_DATE_TIME_FORMAT = "Y-m-d\TH:i:00.000";
    //const CUSTOM_DATE_TIME_FORMAT = "2020-09-02\TH:i:00.000";
    const COMPRIS_THIRD_PARTY_PROVIDER_ID = '1007';

    const DELIVERY_OCCASION_VALUE = "4";
    const PICKUP_OCCASION_VALUE = "5";

    const SECONDS_BEFORE_SOD_NOT_RUN_ALERT = 3600;
    const SECONDS_IN_THE_FUTURE_TO_SET_ALERT_EMAIL_WHEN_STORE_POS_GOES_DOWN = 330;
    const SECONDS_IN_THE_FUTURE_TO_SET_NEXT_STATUS_CHECK_IF_STORE_JUST_SHUT_DOWN = 150;

    function ComprisController($mt,$u,&$r,$l = 0)
	{
		parent::MessageController($mt, $u, $r,$l);
		$this->date_time_format = self::CUSTOM_DATE_TIME_FORMAT;
	}

    function setViewed(&$message_resource)
    {
        $message_resource->viewed = 'N';
    }

    function pullNextMessageResourceByMerchant($alphanumeric_id)
    {
        if ($merchant_resource = MerchantAdapter::getMerchantResourceFromAlphaNumeric($alphanumeric_id)) {
            $merchant_id = $merchant_resource->merchant_id;
            $dciha = new DeviceCallInHistoryAdapter(getM());
            if ($dciha->getAutoTurnedOffByMerchantId($merchant_id)) {
                myerror_logging(3, "COMPRIS Merhcant is auto turned off so bypass pull of messages for merchant_id: $merchant_id");
                // return the status test but force a 60 second buffer if messages exist
                if ($future_messages = $this->getAvailablePuledMessageResourcesArrayByMerchantId($merchant_id)) {
                    $forced_status_buffer_in_seconds = 60;
                }
                return $this->getStatusCheckResponseMessage($merchant_resource,$forced_status_buffer_in_seconds);
            } else if ($message_resource = parent::pullNextMessageResourceByMerchant($merchant_id)) {
                $dciha->recordPullCallIn($merchant_id,$this->format);
                $resource = $this->prepMessageForSending($message_resource);
                $xml = $resource->message_text;
                $clean_xml = str_replace("   ", '', removeCarriageReturnsTabsLineFeedsFromString($xml));
                $resource->message_text = $clean_xml;
                //$resource->message_text = cleanUpXML($xml);
                return $resource;
            } else {
                $dciha->recordPullCallIn($merchant_id,$this->format);
                myerror_logging(3, "There are no pulled messages ready for this merchant: $merchant_id");
                // return the status test
                return $this->getStatusCheckResponseMessage($merchant_resource);
            }
        }
    }

    public function getStatusCheckResponseMessage($merchant_resource,$status_buffer_in_seconds = 0)
    {
        // first check to see when last status was sent (how?)
        if ($status_buffer_in_seconds == 0) {
            $status_buffer_in_seconds = getProperty('comprise_status_message_buffer_in_seconds');
        }
        $last_status_message_resource = Resource::findOrCreateIfNotExistsByData($this->adapter,['merchant_id'=>$merchant_resource->merchant_id,'message_format'=>'OS']);
        if ($last_status_message_resource->sent_dt_tm > date("Y-m-d H:i:s",time()-$status_buffer_in_seconds)) {
            return null;
        } else {
            $last_status_message_resource->sent_dt_tm = date("Y-m-d H:i:s",time());
            $last_status_message_resource->locked = 'S';
            $last_status_message_resource->save();
        }
        $resource = new Resource();
        $packet_id = "STATUS-".time();
        $status_text = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Request Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus/></Request></CommunicationCenter>';
        $resource->message_text = $status_text;
        $resource->_representation = $this->static_message_template;
        return $resource;
    }

    public function populateMessageData($message_resource)
    {
        $resource = parent::populateMessageData($message_resource);
        $this->compris_info = getThirdPartyParametersForMerchantSkinCombination(self::COMPRIS_THIRD_PARTY_PROVIDER_ID,$resource->skin_id,$this->merchant['merchant_id'],$this->merchant['brand_id']);
        logData($this->compris_info,"COMPRIS THIRD PARTY INFO",3);
        if ($resource->skin_id == 237 && $resource->promo_id > 0 && $resource->promo_amt < 0) {
            // figure out which item is the promo
            // get the qualifying item id from the promocontroller
            $ptffama = new PromoType4ItemAmountMapsAdapter(getM());
            if ($data = $ptffama->getRecord(['promo_id'=>$resource->promo_id])) {
                $promo_info = [];
                $qitem_id_string = $data['qualifying_object_id_list'];
                $qitem_ids_strings_array = explode(',',$qitem_id_string);
                foreach ($qitem_ids_strings_array as $promo_id_string) {
                    $qids = explode('-',$promo_id_string);
                    if ($qids['0'] == 'Item') {
                        //now get external id
                        $promo_brand_external_id_maps_adapter = new PromoBrandExternalIdMapAdapter(getM());
                        if ($record = $promo_brand_external_id_maps_adapter->getRecord(['brand_id'=>$resource->brand_id,"promo_id"=>$resource->promo_id])) {
                            $qualifying_item_id = $qids[1];
                            $promo_external_id = $record['external_id'];
                            $promo_info["$qualifying_item_id"] = $promo_external_id;
                        }
                    } else {
                        myerror_log("ERROR!!!!  NON ITEM Promo used with compris");
                    }
                }
            } else {
                myerror_log('ERROR!!!!!  Non type 4 promo used with compris');
            }


        }
        foreach ($resource->order_details as &$order_detail) {
            $e = explode(':',$order_detail['external_id']);
            $p = explode('+',$order_detail['external_id']);
            $order_detail['additional_protiens'] = [];
            if ($e[1]) {
                $order_detail['external_id'] = $e[0];
                $order_detail['adjective'] = $e[1];
            } else if ($p[1]) {
                $order_detail['external_id'] = $p[0];
                array_shift($p);
                $order_detail['additional_protiens'] = $p;
            }
            if (sizeof($promo_info) > 0) {
                if ($promo_external_id = $promo_info[$order_detail['item_id']] ) {
                    // promo item
                    $order_detail['discount_coupon_code'] = $promo_external_id;
                    $promo_info = [];
                }
            }

        }

        // custom daste time format
//        $resource->set('compris_order_date_time',$this->getDateTimeFormatForController($resource->order_dt_tm));
//        $resource->set('compris_pickup_date_time',$this->getDateTimeFormatForController($resource->pickup_dt_tm));

        $utc_date_time = new DateTime();
        $utc_date_time->setTimezone(new DateTimeZone('UTC'));
        $utc_date_time->setTimestamp($resource->pickup_dt_tm);

        $merchant_time_zone = new DateTimeZone($resource->merchant['time_zone_string']);
        $merchant_now_date_time = new DateTime("now", $merchant_time_zone);
        $offset = $merchant_now_date_time->format('P');

        $compris_pickup_string = $utc_date_time->format(self::CUSTOM_DATE_TIME_FORMAT).$offset;

        $utc_date_time->setTimestamp($resource->order_dt_tm);
        $compris_order_string = $utc_date_time->format(self::CUSTOM_DATE_TIME_FORMAT).$offset;

        $resource->set('compris_order_date_time',$compris_order_string);
        $resource->set('compris_pickup_date_time',$compris_pickup_string);

        foreach ($this->compris_info as $name=>$value) {
            $resource->set($name,$value);
        }

        if ($resource->skin_id == 237) {
            myerror_log("Native Skin with order type: ".$resource->order_type,3);
            if ($resource->order_type == 'R') {
                $resource->occasion = self::PICKUP_OCCASION_VALUE;
                if ($native_pickup_order_level_discount_id = $this->compris_info['native_pickup_order_level_discount_id']) {
                    $resource->set('order_level_discount_id',$native_pickup_order_level_discount_id);
                } else {
                    $resource->set('order_level_discount_id',0);
                }
            } else {
                $resource->occasion = self::DELIVERY_OCCASION_VALUE;
            }
        } else {
            // non native skin meaning a marketplace order
            myerror_log("MARKETPLACE ORDER setting occation to ".self::DELIVERY_OCCASION_VALUE,3);
            $resource->occasion = self::DELIVERY_OCCASION_VALUE;
        }

        $resource->set('compris_total',($resource->grand_total-$resource->total_tax_amt) * 100);

        $resource->set('message_map_id_for_callback',$message_resource->map_id);
        return $resource;
    }

    function parseCallBackXml($body)
    {
        $start = strpos($body,  '<?xml');
        $end = strpos($body,  '?>') + 2;
        $xml = substr($body,$end);
        $xml = str_ireplace("\\r","",$xml);
        $xml = str_ireplace("\\n","",$xml);
        $xml = str_ireplace("\\t","",$xml);
        //$xml = $this->getCleanPayloadFromXML($body, $end, );
        return parseXMLintoHashmap($xml);
    }

    function getAlertMessageSendOnTimeByMerchant($merchant_resource)
    {
        // determin if we are out of hours
        $hour_adapter = new HourAdapter(getM());
        $tz = date_default_timezone_get();
        date_default_timezone_set($merchant_resource->time_zone_string);
        $ts = $hour_adapter->getNextOpenAndCloseTimeStamps($merchant_resource->merchant_id,'R',2,$this->current_time);
        date_default_timezone_set($tz);
        if ($this->current_time < $ts[0]['open']+self::SECONDS_BEFORE_SOD_NOT_RUN_ALERT) {
            // we allow up to 1hr for the store to open before we alert now.
            $next_message_dt_tm = $ts[0]['open']+self::SECONDS_BEFORE_SOD_NOT_RUN_ALERT;
        } else if ($this->current_time > $ts[0]['close']-self::SECONDS_BEFORE_SOD_NOT_RUN_ALERT) {
            // stores occasionally close out their system within 1hr of closing and we get an SOD not run so added that logic to reduce the false positives
            $next_message_dt_tm = $ts[1]['open']+self::SECONDS_BEFORE_SOD_NOT_RUN_ALERT;
        } else {
            // set it to 90 seconds in the future since we'll check again in 60 and it will allow it to cancel the message if it comes back up quickly
            $next_message_dt_tm = $this->current_time+self::SECONDS_IN_THE_FUTURE_TO_SET_ALERT_EMAIL_WHEN_STORE_POS_GOES_DOWN;
            $this->store_is_currently_open = true;
        }
        return $next_message_dt_tm;
    }

    function callBack($alphanumeric_id)
    {
        myerror_log("we have a COMPRIS call back with  alpha numeric: ".$alphanumeric_id,3);
        if ($merchant_resource = MerchantAdapter::getMerchantResourceFromAlphaNumeric($alphanumeric_id)) {
            $merchant_id = $merchant_resource->merchant_id;
            myerror_log("We have the merchant on the Compris call back: ".$merchant_resource->merchant_id,3);
            $m_string = 'merchant '.$merchant_resource->merchant_id.'  '.$merchant_resource->merchant_external_id.'  '.$merchant_resource->alphanumeric_id;
            try {
                $call_back_data_hash_map = $this->parseCallBackXml($this->request->body);
            } catch (Exception $e) {
                myerror_log("ERROR ON COMPRIS CALL BACK.  could not parse response as XML: ".$this->request->body);
                myerror_log("error parsing: ".$e->getMessage());
                return false;
            }

            if ($ucid = $call_back_data_hash_map['@attributes']['PacketId'][0]) {
                if (substr($ucid,0,6) == 'STATUS') {
                    // return from teh status call
                    myerror_log("COMPRIS STATUS return for $m_string - ".$this->request->body,3);
                    if ($pos_status = $call_back_data_hash_map['Response']['PosStatus']['@attributes']['ResultText'][0]) {
                        $pos_status_lower = strtolower($pos_status);
                        if ($pos_status_lower == 'pos is ready' || $pos_status_lower == 'pos not ready, order has been started') {
                            // all is good record the call in
                            DeviceCallInHistoryAdapter::recordPullCallIn($merchant_resource->merchant_id,$this->format);
                            // check to see if there any pending emails
                            $mmha = new MerchantMessageHistoryAdapter(getM());
                            $options[TONIC_FIND_BY_METADATA] = ['merchant_id'=>$merchant_resource->merchant_id,'message_format'=>'Eoff','locked'=>'N'];
                            if ($message_resource = Resource::find($mmha,null,$options)) {
                                $message_resource->locked = 'C';
                                $message_resource->save();
                                // wondering if maybe it makes sense to delete these here
                            }
                            return true;
                        } else {
                            myerror_log($m_string.' BAD STATUS MESSAGE: '.$pos_status);
                        }
                    } else {
                        myerror_log("$m_string ERROR COULD NOT GET STATUS FROM RESPONSE: ".$this->request->body);
                    }
                    if ($merchant_resource->active == "Y" && $merchant_resource->ordering_on == "Y") {
                        return $this->setAutoTurnedOffByMerchantResource($merchant_resource);
                    }
                } else if ($order = new Order($ucid)) {
                    myerror_log("we have the order on the Compris callback: ".$order->getOrderId(),3);
                    if ($map_id = $call_back_data_hash_map['Response']['@attributes']['Id'][0]) {
                        myerror_log("we have the message id on the Compris callback: $map_id",3);
                        if ($message_resource = Resource::find(new MerchantMessageHistoryAdapter(getM()),"$map_id")) {
                            if (isset($call_back_data_hash_map['Response']['TenderOrder']['@attributes']['ResultCode'][0])) {
                                $result_code = (string) $call_back_data_hash_map['Response']['TenderOrder']['@attributes']['ResultCode'][0];
                                myerror_log("we have the result code on the Compris callback: $result_code",3);
                                if ($result_code == "0") {
                                    $message_resource->response = $this->request->body;
                                    MerchantMessageHistoryAdapter::markMessageResourceAsViewed($message_resource);
                                    $order->updateStatus('E');
                                    return true;
                                }
                            }
                            myerror_log("ERROR ON COMPRIS CALL BACK.  response: ".$this->request->body);
                            //fail the message and alert support
                            return $this->processFail($message_resource,$this->request->body);
                        } else {
                            myerror_log("ERROR!!!  Orphaned callback for merchant_id: $m_string, message id does not exist: $map_id");
                        }
                    } else {
                        myerror_log("ERROR ON COMPRIS CALL BACK.  No message id contained in response: ".$this->request->body);
                    }
                } else {
                    myerror_log("ERROR ON COMPRIS CALL BACK.  $ucid order does not exist. ".$this->request->body);
                }
                return false;
            } else {
                /*
                 *  message timeout: <?xml version="1.0" encoding="UTF-8"?><RRTService>  <Result>Timeout</Result>  <Turn>41245937</Turn>  <Id>51288478</Id></RRTService>
                 *  status timeout:   <?xml version="1.0" encoding="UTF-8"?><RRTService>  <Result>Timeout</Result>  <Turn></Turn>  <Id></Id></RRTService>
                 */
                // some error, check for a timeout
                if (substr_count(strtolower($this->request->body),'timeout') > 0  ) {
                    myerror_log("we had a timeout trying to reach the Compris comm center");
                    if ($merchant_resource->active == "Y" && $merchant_resource->ordering_on == "Y") {
                        return $this->setAutoTurnedOffByMerchantResource($merchant_resource);
                    }
                }
                if ($map_id = $call_back_data_hash_map['Id']) {
                    //$order_id = $call_back_data_hash_map['Turn'];
                    if ($message_resource = Resource::find(new MerchantMessageHistoryAdapter(getM()),"$map_id")) {
                        if ($call_back_data_hash_map['Result'] == 'timeout') {
                            if ($message_resource->tries > 2) {
                                return $this->processFail($message_resource,'Timeout trying to communicate with the CommCenter');
                            } else  {
                                //try a couple more times
                                $message_resource->next_message_dt_tm = time() + 120;
                                $message_resource->locked = 'P';
                                $message_resource->save();
                                return true;
                            }
                        } else {
                            myerror_log("COMPRIS RESPONSE ERROR!!!!  unknown error: ".$this->request->body);
                            return $this->processFail($message_resource,$this->request->body);
                        }
                    } else {
                        myerror_log("ERROR!!!  Orphaned callback for $m_string, '$map_id' message id does not exist: ".$this->request->body);
                    }
                } else {
                    // this is probably a timeout of a status check
                    myerror_log("ERROR!!!  Orphaned callback for $m_string, NO message map id submitted with callback: ".$this->request->body);
                }
            }
            return false;
        } else {
            myerror_log("COMPRIS RESPONSE ERROR!!!! Merchant does NOT exist for alphanumeric: $alphanumeric_id");
            return false;
        }
    }

    function setAutoTurnedOffByMerchantResource($merchant_resource)
    {
        $m_string = 'merchant '.$merchant_resource->merchant_id.'  '.$merchant_resource->merchant_external_id.'  '.$merchant_resource->alphanumeric_id;
        myerror_log("begin COMPRIS deactivation of $m_string");
        $merchant_resource->ordering_on = 'N';
        $merchant_resource->inactive_reason = 'F';
        $merchant_resource->save();
        $body = "Alert for ".$merchant_resource->name." at ".$merchant_resource->address1." - ".$merchant_resource->phone_no."\n\nYour POS off line. Please check your system. \n\n POS Response: ".$this->request->body;
        $mmh_data = ['merchant_id'=>$merchant_resource->merchant_id,'message_format'=>'Eoff'];
        $mmh_data['next_message_dt_tm'] = $this->getAlertMessageSendOnTimeByMerchant($merchant_resource);
        MailIt::stageEmail($merchant_resource->shop_email,"Your POS Is Offline!!!!! SiteCode: ".$merchant_resource->merchant_external_id,$body,getProperty('support_sender_name'),null,$mmh_data);
        // if we're during open hours shorten the buffer
        if ($this->store_is_currently_open) {
            // change last status to buffer - 1 minute since we just shut down and the email is set to go out in less than 2 minutes
            if ($last_status_message_resource = Resource::findOrCreateIfNotExistsByData($this->adapter,['merchant_id'=>$merchant_resource->merchant_id,'message_format'=>'OS'])) {
                $status_buffer_in_seconds = getProperty('comprise_status_message_buffer_in_seconds');
                $last_status_message_resource->sent_dt_tm = date("Y-m-d H:i:s",$this->current_time-($status_buffer_in_seconds)+self::SECONDS_IN_THE_FUTURE_TO_SET_NEXT_STATUS_CHECK_IF_STORE_JUST_SHUT_DOWN);
                $last_status_message_resource->save();
            }
        }
        $dciha = new DeviceCallInHistoryAdapter(getM());
        $data = ['merchant_id'=>$merchant_resource->merchant_id,'device_base_format'=>$this->format];
        if ($device_call_in_history_resource = Resource::findOrCreateIfNotExistsByData($dciha,$data)) {
            if ($device_call_in_history_resource->auto_turned_off == 1) {
                myerror_log("Merchant is already turned off do nothing.  merchant_id: ".$merchant_resource->merchant_id);
            } else {
                myerror_log("Auto Turning off merchant: ".$merchant_resource->merchant_id);
                $dciha->auditTrailForInternalFunction("INTERNAL: Auto Shut OFf /merchants/" . $merchant_resource->merchant_id);
                $device_call_in_history_resource->auto_turned_off = 1;
                $device_call_in_history_resource->save();
            }
        }
        return false;
    }


    function processFail($message_resource,$error_message)
    {
        $message_resource->response = $error_message;
        $message_resource->locked = 'F';
        $message_resource->save();
        $this->message_resource = $message_resource;
        $full_order = CompleteOrder::staticGetCompleteOrder($message_resource->order_id, $this->mimetypes, false);
        $this->full_order = $full_order;
        $this->merchant = $full_order['merchant'];
        $this->order_user = $full_order['user'];
        $this->emailorderexecutionerror($error_message);
        return true;
    }

    function getMainMessageSuccessOrderStatus()
    {
        return 'O';
    }

}
?>