<?php
class CurlController extends MessageController
{
	protected $format = 'C';
	protected $format_name = 'curl';
	protected $service_used;
	
	function CurlController($mt,$u,&$r,$l = 0)
	{
		parent::MessageController($mt,$u,$r,$l);
	}

    /**
     * @param $info
     * @return SplickitService
     */
	function getServiceFromInfo($info)
    {
        if ($curl_service = $info['service']) {
            $service_class_name = $curl_service.'Service';
            $info['ids_for_third_party_parameters'] = ['skin_id'=>$this->full_order['skin_id'],'brand_id'=>$this->merchant['brand_id'],'merchant_id'=>$this->merchant['merchant_id']];
            if ($service = new $service_class_name($info)) {
                $this->service_used = $service_class_name;
                return $service;
            }
        }
    }
		
	function send($message_text)
	{

		$info = $this->extractAndSetInfoData($this->message_resource->info);
		if ($service = $this->getServiceFromInfo($info)) {
		    // if we are using a custom service to send then we dont care about the message text because the service will create it from the complete order
            $result = $service->send($this->full_order);
            $this->message_resource->message_text = $service->getMessageBody();
            $this->message_resource->response = $service->getRawReturn();
            if ($result) {
                return true;
            } else {
                throw new Exception("Error Sending Curl Through ".$this->service_used,500);
            }
        }
		$url = $this->deliver_to_addr;
		if ($info) {
			if ($header_json = $info['headers']) {
				$header_list_array = json_decode($header_json,true);
				foreach ($header_list_array as $name=>$value)
					$headers[] = "$name: $value";
			}
			if ($info['post']) {
				$post = true;
            }
			if ($info['verbose']) {
				$verbose = true;
            }
		} else {
			$headers[] = 'Content-Length: ' . strlen($message);
			if (json_decode($message))
				$headers[] = 'Content-Type: Application/json';
			else if (substr_count($message, "soap-envelope") > 0)
				$headers[] = 'Content-Type: application/soap+xml';
			else if (substr_count($message, "<?xml version"))
				$headers[] = 'Content-Type: application/xml';
		}
		$response = $this->curlIt($url, $headers, $post, $message, $verbose);
		if ($raw_return = $response['raw_return'])
			$this->setMessageResourceResponse($raw_return);
		else if ($error = $response['error'])
			$this->setMessageResourceResponse($error);
		if ($response['http_code'] == 200)
			return true;
		else
			return false;
		
	}
	
	function curlIt($url,$headers,$post,$body,$verbose)
	{
		myerror_logging(3,"CurlController: about to curl with a body of: ".$body);
		if ($ch = curl_init($url))
		{
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			if ($verbose)
				curl_setopt($ch, CURLOPT_VERBOSE,$verbose);
			if ($post)
				curl_setopt($ch, CURLOPT_POST,$post);
			if ($body)
				curl_setopt($ch, CURLOPT_POSTFIELDS, $body);
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
			$response = SplickitCurl::curlIt($ch);
			curl_close($ch);
			myerror_logging(3,"the response in curl controller is: ".$response['raw_return']);
		} else {
			$response['error'] = "FAILURE. Could not connect to $url";
			myerror_log("ERROR! CurlController: could not connect to: $url");
		}
		return $response;
	}
}

