<?php

class ThanxLoyaltyController extends LoyaltyController
{
    var $access_token;
    var $thanx_id;

    const ERROR_FOR_THANX_EXISTING_USER_MESSAGE = 'Sorry, this email address exists already, please use normal login procedure.';

    function __construct($mimetypes, $user, $request, $log_level = 0)
    {
        parent::__construct($mimetypes, $user, $request, $log_level);
        $this->auto_join = true;
        $this->uses_remote_sso = true;
        $this->remote_service = 'ThanxLoyalty';
    }

    function createAccount($user_id,$points)
    {
        if ($user_resource = getUserResourceFromId($user_id)) {
            $user_data = [];
            $user_data['first_name'] = $user_resource->first_name;
            $user_data['last_name'] = $user_resource->last_name;
            $user_data['email'] = $user_resource->email;
            if ($phone = $this->data['contact_no']) {
                $phone = preg_replace('/[^0-9]/', '', $phone);
                $user_data['phone'] = '+1'.$phone;
                $user_resource->contact_no = $user_resource->_adapter->formatGoodTenDigitPhoneNumber($phone);
                $user_resource->save();
            } else {
                $user_data['phone'] = '+1'.preg_replace('/[^0-9]/', '', $user_resource->contact_no);
            }

            if ($birthday_string = $this->data['birthday']) {
                if ($birthday_string != null && trim($birthday_string) != '') {
                    $b = explode('/',$birthday_string);
                    $birthday_array = ['year'=>$b[2],'month'=>$b[0],'day'=>$b[1]];
                    $user_data['birth_date'] = $birthday_array;
                }
            }


            $thanx_loyalty_service = new ThanxLoyaltyService();
            if ($response = $thanx_loyalty_service->createUser($user_data)) {
                $authorization = $response['authorization'];
                $user = $response['user'];
                $loyalty_number = 'id='.$user['id'].':access='.$authorization['access_token'].':refresh='.$authorization['refresh_token'];
                return $this->addUserBrandLoyaltyRecord($user_id, $loyalty_number, 0);
            } else {
                myerror_log("SERIOUS ERROR TRYING TO CREATE REMOTE ThanX USER: ".$thanx_loyalty_service->getErrorFromCurlResponse());
                $this->service_response = $thanx_loyalty_service->curl_response;
            }
        }
        $error = $thanx_loyalty_service->getErrorFromCurlResponse();
        if ($error == 'User account exists') {
            $error = self::ERROR_FOR_THANX_EXISTING_USER_MESSAGE;
        }
        throw new UnableToCreateRemoteUserException($error);
    }


    function updateAccount($user_resource)
    {
        myerror_log("about to start update in teh Thanx loyalty controller");
        $user_data = [];
        // We only update the first name and last name, can't update email or phone.
        $user_data['first_name'] = $user_resource->first_name;
        $user_data['last_name'] = $user_resource->last_name;

        // Populate the birthday.
        if ($birthday_string = $user_resource->birthday) {
            if ($birthday_string != null && trim($birthday_string) != '') {
                $b = explode('/', $birthday_string);
                $birthday_array = ['year' => $b[2], 'month' => $b[0], 'day' => $b[1]];
                $user_data['birth_date'] = $birthday_array;
            }
        }
        myerror_log("about to update on the remote Thanx system");
        $thanx_loyalty_service = new ThanxLoyaltyService();
        if ($id = $this->getThanxIdFromLoyaltyNumber($this->loyalty_number)) {
            $access_token = $this->getAccessTokenFromLoyaltyNumber($this->loyalty_number);
            $user_data['thanx_id'] = $id;
            if ($response = $thanx_loyalty_service->updateUser($user_data, $access_token)) {
                if ($response['http_code'] == 201) {
                    return true;
                }
            }
            myerror_log("SERIOUS ERROR TRYING TO CREATE REMOTE ThanX USER: " . $thanx_loyalty_service->getErrorFromCurlResponse());
            $this->service_response = $thanx_loyalty_service->curl_response;
        } else {
            myerror_log("UNABLE TO GET THANX ID FROM LOYALTY NUMBER");
        }
    }

    function getRemoteAccount($access_token)
    {
        $thanx_loyalty_service = new ThanxLoyaltyService($this->complete_order);
        if ($response = $thanx_loyalty_service->getRemoteAccount($access_token)) {
            return $response;
        } else {
            myerror_log("SERIOUS ERROR TRYING TO GET REMOTE ThanX ACCOUNT: ".$thanx_loyalty_service->getErrorFromCurlResponse());
        }
    }

    /*  *** NOT USING ,  THIS I NEW API and we use  LEGACY API for loyalty
    function getRewards($access_token)
    {
        $thanx_loyalty_service = new ThanxLoyaltyService();
        if ($response = $thanx_loyalty_service->getRewards($access_token)) {
            return $response;
        }
    }
    */

    function getRemoteUserInfo($id,$access_token)
    {
        $thanx_loyalty_service = new ThanxLoyaltyService();
        if ($response = $thanx_loyalty_service->getRemoteUserInfo($id,$access_token)) {
            $authorization = $response['authorization'];
            $user = $response['user'];
            if ($this->user) {
                $loyalty_number = 'id='.$user['id'].':access='.$authorization['access_token'].':refresh='.$authorization['refresh_token'];
                return $this->addUserBrandLoyaltyRecord($this->user['user_id'], $loyalty_number, 0);
            } else {
                return $response;
            }

        } else {
            myerror_log("SERIOUS ERROR TRYING TO GET REMOTE ThanX USER: ".$thanx_loyalty_service->getErrorFromCurlResponse());
        }
    }

    // this is overridden for thanxs since they never expire so we dont want the splickit token to expire
    function createUserAuthenticationToken($user_id)
    {
        // set duration to 6 months
        $duration = 180*24*3600;
        return createUserAuthenticationToken($user_id,$duration);
    }

    function processSSO($thanx_authentication_token)
    {
        $thanx_loyalty_service = new ThanxLoyaltyService();
        if ($response = $thanx_loyalty_service->authenticateCode($thanx_authentication_token)) {
            // get user info from Thanx and validate the email exists in our system
            if ($remote_user_info = $thanx_loyalty_service->getRemoteAccount($response['access_token'])) {
                $email = $remote_user_info['email'];
                if ($user_resource = UserAdapter::doesUserExist($email)) {
                    myerror_log("THANX SSO: User Exists create or update loyalty record");
                } else {
                    myerror_log("THANX SSO: ERROR!!!!!  User does NOT exist in our system: $email .  create the user");
                    $remote_user_data = $thanx_loyalty_service->getRemoteUserFields($response['access_token']);
                    logData($remote_user_data,'Thanx Remote User Data');
                    if ($email) {
                        $new_user_data = ['email'=>$email];
                        $new_user_data['first_name'] = $remote_user_data['user']['first_name'];
                        $new_user_data['last_name'] = $remote_user_data['user']['last_name'];
                        if ($remote_user_data['user']['phone']) {
                            $new_user_data['contact_no'] = formatGoodTenDigitPhoneNumber(substr($remote_user_data['user']['phone'],2));
                        } else {
                            $new_user_data['contact_no'] = formatGoodTenDigitPhoneNumber('1234567890');
                        }
                        $user_resource = Resource::factory(new ThanxUserAdapter(getM()),$new_user_data);
                        if ($user_resource->save()) {
                            myerror_log("THANX SSO: New user created in Order140db");
                        } else {
                            myerror_log("THANX SSO: ERROR CREATING USER IN order140 DB from thanx response");
                            return false;
                        }
                    } else {
                        myerror_log("THANX SSO: No email returned from Thanx. Cannot create user in system");
                        return false;
                    }
                }
                // update the loyalty record
                $loyalty_number = 'id=' . $remote_user_info['id'] . ':access=' . $response['access_token'] . ':refresh=' . $response['refresh_token'];
                $user_brand_loyalty_adapter = new UserBrandPointsMapAdapter(getM());
                if ($loyalty_resource = $user_brand_loyalty_adapter->addUserLoyaltyRecord($user_resource->user_id, getBrandIdFromCurrentContext(), $loyalty_number, 0)) {
                    return $user_resource;
                } else {
                    myerror_log("THANX SSO: ERROR CREATING thanx loyalty record IN DB from thanx response");
                }
            }
        }
        return false;
    }

    //***    ORDER METHODS   ***//
    function sendLoyaltyOrderEvent($complete_order,$points)
    {
        $response = $this->sendThanxLoyaltyOrderEvent($complete_order,'placed');
        if ($response['error']) {
            return $response;
        }
        $response = $this->sendThanxLoyaltyOrderEvent($complete_order,'billed');
        return $response;
    }

    function sendThanxLoyaltyOrderEvent($complete_order,$state,$reward)
    {
        $thanks_loyalty_service = new ThanxLoyaltyService();
        $access_token = $this->getAccessTokenFromLoyaltyNumber($this->loyalty_number);
        $complete_order['thanx_reward'] = $reward;

        $response = $thanks_loyalty_service->sendOrderInfo($complete_order,$access_token,$state);
        $response['raw_return'] = $thanks_loyalty_service->getRawReturn();
        $this->basket_response = $response;
        if ($response['status'] == 'success') {
            return $response;
        } else {
            myerror_log("ERROR trying to submit basket to Thanx: ".$thanks_loyalty_service->getRawReturn());
            $response['error'] = $response['message'];
            return $response;
        }
    }

    function processOrderFromCompleteOrder($complete_order)
    {
        $this->complete_order = $complete_order;
        if ($this->loyalty_number) {
            $response = $this->getPointsAndSendIfApplicable($complete_order);
            $order_id = $complete_order['order_id'];
            if ($response['error']) {
                myerror_log("we had an error trying to use a thanx award after the order has been placed");
                MailIt::sendErrorEmailSupport("Thanx Basket Update Error","order_id: $order_id.  There was an error trying to update the basket on the thanx system: ".$response['error']);
                if ($complete_order['promo_amt'] < 0) {
                    // so we need to cancel this order somehow.
                    throw new LoyaltyPaymentException($response['error']);
                }
            }
            $discount = $response['discount'];
            if ($discount > 0.00 && $response['status'] == 'success') {
                $balance_change_adapter = new BalanceChangeAdapter(getM());
                $bc_resource = $balance_change_adapter->addRow($complete_order['user_id'], 0.00, $discount , 0.00,'LoyaltyBalancePayment','ThanxLoyaltyController', $complete_order['order_id'], $response['id'],$response['raw_return']);
            }
        }
    }

    function getPointsAndSendIfApplicable($complete_order)
    {
        return $this->sendLoyaltyOrderEvent($complete_order,0);
    }

    function getLoyaltyAmountOffFromAward()
    {

    }

    function getPointsEarnedFromCompleteOrder($complete_order)
    {
        return 0;
    }

    function getThanxIdFromLoyaltyNumber($loyalty_number)
    {
        $l = explode(":",$loyalty_number);
        foreach ($l as $field) {
            $f = explode('=',$field);
            if ($f[0] == 'id') {
                $this->thanx_id = $f[1];
                return $f[1];
            }
        }
    }


    function getAccessTokenFromLoyaltyNumber($loyalty_number)
    {
        $l = explode(":",$loyalty_number);
        foreach ($l as $field) {
            $f = explode('=',$field);
            if ($f[0] == 'access') {
                $this->access_token = $f[1];
                return $f[1];
            }
        }
    }

    function getAdditionalCheckoutData()
    {
        $access_token = $this->getAccessTokenFromLoyaltyNumber($this->loyalty_number);
        if ($rewards_info = $this->getRemoteAccount($access_token)) {
            return $this->processRewardsListForCheckout($rewards_info['rewards']);
        }
    }

    function processRewardsListForCheckout($raw_rewards_list)
    {
        $rewards = [];
        foreach ($raw_rewards_list as $reward) {
            if ($reward['state'] != 'redeemable') {
                continue;
            }
            $rewards[] = ['id'=>$reward['id'],'text'=>$reward['label'],'fine_print'=>$reward['fine_print']];
        }
        if (sizeof($rewards) > 0) {
            return $rewards;
        }
    }

    function validateRemotePromo($complete_order,$promo_code)
    {
        return $this->sendThanxLoyaltyOrderEvent($complete_order,'checkout',$promo_code);
    }

    function getMasterKeyWordForRemoteVariablePromo()
    {
        return 'master_thanx';
    }

    function getVariablePromoAmountFromResponse($response)
    {

        return $response['discount'] == null ? 0.00 : $response['discount'];
    }

    function reverseLoyaltyTransactionsForOrderId($order_id)
    {
        myerror_log("Starting reverseLoyaltyTransactionsForOrderId");
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id);
        $response = $this->sendThanxLoyaltyOrderEvent($complete_order,'refunded');
        if ($response['status'] == 'success') {
            $balance_change_adapter = new BalanceChangeAdapter(getM());
            $options[TONIC_FIND_BY_METADATA] = ['process'=>'LoyaltyBalancePayment','cc_processor'=>'ThanxLoyaltyController','order_id'=>$order_id];
            if ($bc_resource = Resource::find($balance_change_adapter,null,$options)) {
                $bc_resource->notes = 'REFUNDED: '.$response['raw_return'];
                $bc_resource->save();
            }
        } else {
            MailIt::sendErrorEmailSupport("ERROR REFUNDING THANX AWARD","We had an error trying to do a THANX refund of order_id: ".$order_id."  -  ".$response['raw_return']);
        }
    }

    static function getDiscountDisplay()
    {
        return "Reward Used";
    }
}

class ThanxUserAdapter extends MySQLAdapter
{

    function __construct($mimetypes)
    {
        parent::MysqlAdapter(
            $mimetypes,
            'User',
            '%([0-9]{1,15})%',
            '%d',
            array('user_id'),
            null,
            array('created', 'modified')
        );
    }

    function insert($resource)
    {
        $password = generateAlphaCode(10);
        $resource->password = Encrypter::Encrypt($password);
        $resource->uuid = generateUUID();
        return parent::insert($resource);
    }

}
?>