<?php
class BrinkImporter extends SplickitImporter
{

    protected $menu_id;
    protected $brand_id;
    protected $brand_name;

    private $build_modifier_structure = false;
    protected $modifier_group_list_hash = [];
    protected $brink_location_token;
    protected $brink_parameters;

    //private $brink_third_party_provider_id;
    const BRINK_THIRD_PARTY_PROVIDER_ID = 1001;

    function __construct($merchant_mixed)
    {
        parent::__construct($merchant_mixed);
        if ($this->merchant_resource) {
            //validate its aa brink merchant
            if ($this->isLoadedMerchantABrinkMerchant()) {
                // get brand_id and menu_id
                if ($menu_id = MerchantMenuMapAdapter::getMenuIdFromMerchantIdAndType($this->merchant_resource->merchant_id,'Pickup')) {
                    myerror_log("we have the menu_id: $menu_id");
                    $this->menu_id = $menu_id;
                    $this->brand_id = $this->merchant_resource->brand_id;
                } else {
                    myerror_log("ERROR!!!  Merchant does not hvae a pickup menu id set in merchant_menu_map");
                    throw new Exception("Merchant does not hvae a pickup menu id set in merchant_menu_map");
                }
            } else {
                throw new Exception("Merchant Is Not A Brink Merchant");
            }
        }
    }

    function stageImport()
    {
        $doit_ts = time()-2;
        // need someway to automatically do the build modifiers
        //  ;build_modifiers=1";
        $info = "merchant_alphanumeric_id=".$this->merchant_resource->alphanumeric_id;
        $activity_history_adapter = new ActivityHistoryAdapter(getM());
        if ($activity_history_resource = $activity_history_adapter->createActivityReturnActivityResource('BrinkPriceSync', $doit_ts, $info, '')) {
            return $activity_history_resource->activity_id;
        } else {
            return null;
        }

        //return $this->stageImportFromMerchantExternal($this->external_merchant_id);
    }


    function setBrandRegExForMerchantExternalIds()
    {
        $this->brand_regex_for_merchant_external_ids = '[a-zA-Z0-9_.-]+';
    }

    function getRemoteData($merchant_resource)
    {
        return $this->importBrinkRemoteData($merchant_resource);
    }

    function isLoadedMerchantABrinkMerchant()
    {
        $merchant_id = $this->merchant_resource->merchant_id;
        //$tpppmma = new ThirdPartyProviderParameterMerchantMapsAdapter(getM());
        if ($paramters = $this->getBrinkThirdPartyValues(null,$merchant_id,$this->merchant_resource->brand_id)) {
            myerror_log("getting brink info from new third party tables: ".json_encode($paramters));
            if ($this->brink_location_token = $paramters['brink_location_token']) {
                return true;
            }
            myerror_log('NO LOCATION TOKEN IN THIRD PARTY TABLES!!!!');
        }
        $mbima = new MerchantBrinkInfoMapsAdapter(getM());
        if ($record = $mbima->getRecord(array("merchant_id"=>$merchant_id))) {
            if ($this->brink_location_token = $record['brink_location_token']) {
                return true;
            }
        }
        myerror_log("ERROR!!!!!!  No association with brink for this merchant!!!!");
        return false;
    }

    function importBrinkRemoteData($merchant_resource)
    {
        if ($this->build_modifier_structure) {
            $menu_id = $this->menu_id;
            myerror_log("erasing old modifier structure for menu_id: $menu_id");
            $menu_adapter = new MenuAdapter(getM());
            $sql = "DELETE a FROM Item_Modifier_Item_Map a JOIN Item b ON a.item_id = b.item_id JOIN Menu_Type c ON b.menu_type_id = c.menu_type_id WHERE c.menu_id = $menu_id";
            $menu_adapter->_query($sql);
            $sql = "DELETE a FROM Item_Modifier_Group_Map a JOIN Item b ON a.item_id = b.item_id JOIN Menu_Type c ON b.menu_type_id = c.menu_type_id WHERE c.menu_id = $menu_id ";
            $menu_adapter->_query($sql);
            $sql = "DELETE a FROM Modifier_Size_Map a JOIN Modifier_Item b ON a.modifier_item_id = b.modifier_item_id JOIN Modifier_Group c ON b.modifier_group_id = c.modifier_group_id WHERE c.menu_id = $menu_id";
            $menu_adapter->_query($sql);
            $sql = "DELETE a FROM Modifier_Item a JOIN Modifier_Group b ON b.modifier_group_id = a.modifier_group_id WHERE b.menu_id = $menu_id";
            $menu_adapter->_query($sql);
            $sql = "DELETE FROM Modifier_Group WHERE menu_id = $menu_id";
            $menu_adapter->_query($sql);
        } else {
            myerror_log("bypass create modifiers");
        }
        if ($this->callServiceForRemoteData($merchant_resource)) {
            myerror_log("we have the brink data");
            $plu_list = array();
            foreach ($this->item_price_records_imported as $remote_item_record) {
                $plu_list['item-'.$remote_item_record['Id']] = $remote_item_record;
            }
            foreach ($this->modifier_price_records_imported as $key=>$remote_modifier_record) {
                $plu_list['modifier-'.$key] = $remote_modifier_record;
            }
        } else {
            throw new Exception("MAJOR PROBLEM WITH BRINK ".$this->getBrandName()." IMPORT");
        }
        ksort($plu_list);
        $this->imported_prices = $plu_list;

        $merchant_external = $merchant_resource->merchant_external_id;
        $file_name = "./import_logs/brink".$this->getBrandName()."_$merchant_external.txt";
        // MailIt::sendErrorEmailAdam($file_name,$request->body);
        if ($myfile = fopen($file_name, "w")) {
            foreach ($plu_list as $plu_record) {
                fwrite($myfile, $plu_record['productName'].' '.$plu_record['productId'].' $'.$plu_record['pricePerUnit']."\r\n");
            }

            fclose($myfile);
        }
        myerror_log("done");
    }

    function processPriceResource(&$splickit_price_resource)
    {
        $merchant_id = $this->merchant_resource->merchant_id;
        $imported_price_hash_by_plu = $this->imported_prices;
        $raw_plu_on_splickt_price_record = $splickit_price_resource->external_id;
        myerror_log("splickit price record external id: ".$raw_plu_on_splickt_price_record);
        $p = explode("-",$raw_plu_on_splickt_price_record);
        $plu = isset($p[1]) ? $raw_plu_on_splickt_price_record : $p[0];

        myerror_log("about to get price for: $plu",5);
        if ($p[0] == 'PLU') {
            myerror_log("doing price for modifier item price thing",5);
            $plu = 'item-'.$p[1];
        } else if (isset($splickit_price_resource->modifier_price)) {
            $plu = 'modifier-'.$plu;
        } else {
            $plu = 'item-'.$plu;
        }
        if (isset($imported_price_hash_by_plu[$plu])) {
            $imported_price = $imported_price_hash_by_plu[$plu];
            myerror_log("WE DO HAVE A PRICE: " . $imported_price_hash_by_plu[$plu]['Price']);
            if ($imported_price_hash_by_plu[$plu]['Active'] != 'true') {
                $record = $imported_price_hash_by_plu[$plu];
                myerror_log(get_class().": we have an innactive price for ".$record['Name'],5);
                $this->addToRunningImport("We have an innactive price from the POS for ".$record['Name']."  plu: $plu");
            } else {
                $this->addToRunningImport("WE DO HAVE AN ACTIVE PRICE: ".json_encode($imported_price_hash_by_plu[$plu]));
            }
            $splickit_price_resource->active = strtolower($imported_price_hash_by_plu[$plu]['Active']) == 'true' ? 'Y' : 'N';
            if (isset($splickit_price_resource->modifier_price)) {
                $splickit_price_resource->modifier_price = $this->getModifierPriceFromItemData($imported_price_hash_by_plu[$plu]);
            } else {
                $splickit_price_resource->price = $this->getObjectPriceFromItemData($imported_price_hash_by_plu[$plu]);
            }

            $this->doCustomPriceManipulationForPriceResource($splickit_price_resource);
            $splickit_price_resource->tax_group = $this->isImportedItemTaxable($imported_price_hash_by_plu[$plu]) ? 1 : 0;
            if (isset($splickit_price_resource->item_id) && ($this->build_modifier_structure)) {
                $modifier_group_priority = 200;
                foreach ($imported_price['ModifierGroupIds']['int'] as $modifier_group_id) {
                    $brink_modifier_group = $this->modifier_group_list_hash[$modifier_group_id[0]];
                    $modifier_group_name = $brink_modifier_group['Name'];
                    myerror_log("BRINK LOGGING:  about to import group: $modifier_group_name");
                    if ($modifier_group_name == $brink_modifier_group['Name']) {
                        $modifier_group_external_id = $brink_modifier_group['Id'];
                        $use_modifier_size = false;
                    } else {
                        $modifier_group_external_id = strtolower(str_replace(' ','',$modifier_group_name));
                        $use_modifier_size = true;
                    }

                    $modifier_group_resource = $this->getModifierGroupResource($this->menu_id,$modifier_group_external_id);
                    if ($modifier_group_resource->insert_id) {
                        $modifier_group_resource->modifier_group_name = $modifier_group_name;
                        $modifier_group_resource->modifier_description = $modifier_group_name;
                        // they dont send an active field so all Mod groups are active
                        //$modifier_group_resource->active = "Y";
                        $modifier_group_resource->save();
                        $modifier_group_priority = $modifier_group_priority - 5;
                    }
                    $modifier_group_id = $modifier_group_resource->modifier_group_id;

                    //create modifier_items
                    $modifier_group_item_node = $brink_modifier_group['Items']['ModifierGroupItem'];
                    // ok so there is a problem with the XML parser and if there is only 1 group then it puts it into its own hash rather than an arrya of 1 so i'm hacking this
                    if (count($modifier_group_item_node) == 4 && isset($modifier_group_item_node['ItemId'])) {
                        // ok we have a single hash
                        myerror_log("single hash");
                        $modifier_group_item_node = array($modifier_group_item_node);
                    }
                    foreach ($modifier_group_item_node as $brink_modifier_item) {
                        myerror_log("we have the brink modifier item");
                        if ($modifier_item = $imported_price_hash_by_plu['modifier-'.$modifier_group_external_id.'-'.$brink_modifier_item['ItemId']]) {
                            if ($imported_record = $this->item_price_records_imported[$brink_modifier_item['ItemId']]) {
                                myerror_log("tsop");
                            }
                            $brink_modifier_item['Name'] = $imported_record['Name'];
                            myerror_log("we ahve the modifier item");
                            myerror_log("BRINK LOGGING:  about to import modifier item: ".$modifier_item['Name']);

                            if ($use_modifier_size) {
                                $modifier_item_external_id = "$modifier_group_id-".strtolower(str_replace(' ','',$brink_modifier_item['Name']));
                            } else {
                                $modifier_item_external_id = $brink_modifier_item['ItemId'];
                            }
                            $modifier_item_resource = $this->getModifierItemResource($modifier_group_id,$modifier_item_external_id);
                            //$modifier_item_resource->modifier_item_name = ($brink_modifier_group['Id'] == '4025180') ? 'Side Sauce='.$brink_modifier_item['Name'] : $brink_modifier_item['Name'];
                            $modifier_item_resource->modifier_item_name = $brink_modifier_item['Name'];
                            $modifier_item_resource->modifier_item_print_name = $brink_modifier_item['Name'];
                            //$modifier_item_resource->modifier_item_max = $this->getModifierItemMax($menu_type_name,$modifier_group_name,$modifier_item_resource->modifier_item_name);
                            $modifier_item_resource->priority = 100 - intval($brink_modifier_item['Position']);
                            $modifier_item_resource->save();

                            $full_external_modifier_size_id = $brink_modifier_group['Id'].'-'.$brink_modifier_item['ItemId'];
                            //$full_external_modifier_size_id = $brink_modifier_item['ItemId'];
                            try {
                                $modifier_size_resource = $this->getModifierSizeResource($modifier_item_resource->modifier_item_id,0,$full_external_modifier_size_id,0);
                                $modifier_size_resource->modifier_price = $this->getObjectPriceFromItemData($modifier_item);
                                $modifier_size_resource->active = $this->getActiveFlagSetting($modifier_item);
                                $modifier_size_resource->save();
                            } catch (Exception $mse) {
                                $error_message = "ERROR ON BRINK IMPORT!!! Cannot create record in data base. we may have currupt set up for the ".$splickit_price_resource->item_id."  with ".$brink_modifier_group['Name'];
                                myerror_log($error_message);
                                throw new Exception($error_message);
                            }

                        }
                    }
                    // now add the Item Modifier Group Map
                    if ($imgm_resource = $this->getItemModifierGroupMap($splickit_price_resource->item_id,$modifier_group_id,0)) {
                        $brink_modifier_group['name'] = $brink_modifier_group['DisplayName'];
                        $this->updateItemModifierGroupMapResource($imgm_resource,$brink_modifier_group,$modifier_group_priority);
                    }
                }
                // now do the comes with
                foreach ($imported_price['IncludedModifiers']['ItemIncludedModifier'] as $brink_included_modifier) {
                    $external_id = $brink_included_modifier['ModifierGroupId'].'-'.$brink_included_modifier['ItemId'];
                    $menu_id = $this->menu_id;
                    $sql = "select a.* from Modifier_Size_Map a JOIN Modifier_Item b ON a.modifier_item_id = b.modifier_item_id JOIN Modifier_Group c on b.modifier_group_id = c.modifier_group_id WHERE c.menu_id = $menu_id and a.external_id = '$external_id'";
                    $options[TONIC_FIND_BY_SQL] = $sql;
                    if ($resource = Resource::find(new ModifierSizeMapAdapter(getM()),null,$options)) {
                        $item_modifier_item_map_adapter = new ItemModifierItemMapAdapter(getM());
                        $imim_data['item_id'] = $splickit_price_resource->item_id;
                        $imim_data['modifier_item_id'] = $resource->modifier_item_id;
                        Resource::findOrCreateIfNotExistsByData($item_modifier_item_map_adapter,$imim_data);
                    }
                }
            }
            $price_recorded = true;
        } else if ($plu == 'item-use_modifier') {
            myerror_log("we have an item where the price is set by the modifier",5);
            $this->addToRunningImport("we have an item where the price is set by the modifier: ".json_encode($splickit_price_resource->getDataFieldsReally()));
            //$splickit_price_resource->active = 'Y';
            $price_recorded = false;
        } else {
            // there was no price returned so create an innactive price record
            myerror_log("WE DO NOT HAVE A PRICE FOR PLU: $plu",5);
            $id = isset($splickit_price_resource->item_id) ? $splickit_price_resource->item_id : $splickit_price_resource->modifier_item_id;
            $this->addToRunningImport("WE DID NOT IMPORT A PRICE RECORD FOR ITEM_ID: $id   SIZE_ID: ".$splickit_price_resource->size_id."   PLU: $plu SET record to INACTIVE!");
            //$splickit_price_resource->active = 'N';
            $price_recorded = false;
        }


        $splickit_price_resource->merchant_id = $merchant_id; // in case this is a default record
        if ($p[1] == 'zero_price') {
            $splickit_price_resource->modifier_price = 0.00;
            $splickit_price_resource->price = 0.00;
        }
        if ($splickit_price_resource->save()) {
            return $price_recorded;
        }
    }

    function getObjectPriceFromItemData($item_data)
    {
        return $item_data['Price'];
    }

    function getModifierPriceFromItemData($item_data)
    {
        if ($item_data['PriceMethod'] == '0') {
            $plu = $item_data['ItemId'];
            myerror_log("BRINK IMPORT: We are getting a parent price for this plu: $plu");
            if ($new_item_data = $this->item_price_records_imported[$plu]) {
                myerror_log("BRINK IMPORT: Parent price aquired: ".$new_item_data['Price']);
                return $new_item_data['Price'];
            } else {
                myerror_log("BRINK IMPORT: ERROR No parent price aquired, using submitted item data: ".$item_data['Price']);
            }

        }
        return $item_data['Price'];
    }

    public function getBrinkThirdPartyValues($skin_id,$merchant_id,$merchant_brand_id)
    {
        if (! isset($this->brink_parameters)) {
            $parameters = getAllThirdPartyParametersForMerchantSkinCombination($skin_id,$merchant_id,$merchant_brand_id,self::BRINK_THIRD_PARTY_PROVIDER_ID);
            $this->brink_parameters = $parameters['Brink'];
        }
        return $this->brink_parameters;
    }

    function callServiceForRemoteData($merchant_resource)
    {
        if (isset($merchant_resource->merchant_id)) {
            $merchant_id  = $merchant_resource->merchant_id;
            $this->getBrinkThirdPartyValues(null,$merchant_id,$this->brand_id);
        } else {
            throw new Exception("Cannot complete request. No merchant loaded.");
        }

        $method = "GetDestinations";
        $response0 = $this->curlIt($method,$merchant_resource);
        myerror_log($response0);

        $method = "GetDayParts";
        $response0 = $this->curlIt($method,$merchant_resource);
        myerror_log($response0);

        $method = "GetModifierGroups";
        //setProperty('write_brink_file','true');
        $response = $this->curlIt($method,$merchant_resource);
        if (getProperty("write_brink_file") == 'true') { $this->writeFile('brink_modifier_groups.xml',$response); }
        $modifier_group_list = $this->parseXMLResponseIntoArray($response,$method);
        $this->modifier_group_list_hash = createHashmapFromArrayOfArraysByFieldName($modifier_group_list['ModifierGroup'],'Id');
        //$modifier_group_list_hash_by_name = createHashmapFromArrayOfArraysByFieldName($modifier_group_list['ModifierGroup'],'Name');
//        $this->modifier_price_records_imported = array();
//        foreach ($modifier_group_list['ModifierGroup'] as $modifier_group_info) {
//            foreach ($modifier_group_info['Items']['ModifierGroupItem'] as $remote_modifier_item) {
//                $this->modifier_price_records_imported[$remote_modifier_item['ItemId']] = $remote_modifier_item;
//            }
//        }
//        if (sizeof($this->modifier_price_records_imported) <1) {
//            return false;
//        }

        $method = "GetItems";
        $response2 = $this->curlIt($method,$merchant_resource);
        if (getProperty("write_brink_file") == 'true') { $this->writeFile('brink_items.xml',$response2); }
        $item_list = $this->parseXMLResponseIntoArray($response2,$method);
        $this->item_price_records_imported = array();
        foreach ($item_list['Item'] as $brink_remote_item_info) {
            $this->item_price_records_imported[$brink_remote_item_info['Id']] = $brink_remote_item_info;
        }
        if (sizeof($this->item_price_records_imported) < 1) {
            return false;
        }

        $this->modifier_price_records_imported = array();
        foreach ($modifier_group_list['ModifierGroup'] as $modifier_group_info) {
            foreach ($modifier_group_info['Items']['ModifierGroupItem'] as $remote_modifier_item) {
                $imported_item_record = $this->item_price_records_imported[$remote_modifier_item['ItemId']];
                $remote_modifier_item['Active'] = $imported_item_record['Active'];
                $this->modifier_price_records_imported[$modifier_group_info['Id'].'-'.$remote_modifier_item['ItemId']] = $remote_modifier_item;
            }
        }
        if (sizeof($this->modifier_price_records_imported) <1) {
            return false;
        }


//        $method = "GetItemGroups";
//        $response3 = $this->curlIt($method,$merchant_resource);
//        if (getProperty("write_brink_file") == 'true') { $this->writeFile('brink_item_groups.xml',$response3); }
//        $item_group_list = $this->parseXMLResponseIntoArray($response3,$method);
        return true;
    }

    function stageImportFromMerchantResource($merchant_resource,$delay)
    {
        //check it merchant is a brink merchant
        if (MerchantBrinkInfoMapsAdapter::isMechantBrinkMerchant($merchant_resource->merchant_id)) {
            return parent::stageImportFromMerchantResource($merchant_resource,$delay);
        } else {
            throw new Exception("merchant is not a Brink Merchant");
        }

    }

    function myBrinkToArray($xml) {
        $array = (array)$xml;

        foreach ( array_slice($array, 0) as $key => $value )
        {
            if ( $value instanceof SimpleXMLElement )
            {
                $ns = $value->getNamespaces(true);
                if ($ns['b']) {
                    $value = $array[$key]->children($ns['b']);
                }
                $array[$key] = empty($value) ? NULL : $this->myBrinkToArray($value);
            } else if (is_array($value)) {
                foreach ($value as $sub_key => $sub_value) {
                    $array[$key][$sub_key] = $this->myBrinkToArray($sub_value);
                }
            }
        }
        return $array;
    }

    function parseXMLResponseIntoArray($response,$method)
    {
        $start = strpos($response,"<".$method."Result");
        $end_string = "</".$method."Result>";
        $end = strpos($response,$end_string) + strlen($end_string);
        $payload = substr($response,$start,$end-$start);
        $xml=simplexml_load_string($payload, NULL, NULL, "http://schemas.datacontract.org/2004/07/Pos.Web.Service.Data") or die("Error: Cannot create object");
        $ns = $xml->getNamespaces(true);
        $res = $xml->children($ns['a']);
        $hashmap = $this->myBrinkToArray($res);
        return $hashmap;
    }

    function curlIt($method,$merchant_resource)
    {
        $soap_action = "http://tempuri.org/ISettingsWebService/$method";
        $url = getProperty("brink_api_url_settings");
        myerror_log("URL: $url");
        $access_token = getProperty("brink_access_token"); //"ssYesckeWU+3jzRwuW1IyQ==";
        if ($this->brand_id == 150) {
            //use new url
            $url = str_replace('api.brinkpos.net','api8.brinkpos.net',$url);
        } else if ($domain = $this->brink_parameters['brink_domain']) {
            $url = str_replace('api.brinkpos.net',$domain,$url);
        }
        myerror_log("URL after replacemnt: $url");
        /**********  TAKE THIS OUT AFTER RRT ENVIRONMENT IS LIVE **********/
//        // do brand exception for moose and snarfs
        if ($this->brand_id == 152) {
            $access_token = 'ssYesckeWU+3jzRwuW1IyQ==';
        }
//        else {
//            $access_token = 'PchOZWrujESdhCa94TvKuQ==';
//        }
        
        $location_token = $this->getLocationToken($merchant_resource);

        $curl = curl_init($url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 10);
        curl_setopt($curl, CURLOPT_VERBOSE, 0);

        $body = '<s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/"><s:Body><'.$method.' xmlns="http://tempuri.org/"><accessToken>'.$access_token.'</accessToken><locationToken>'.$location_token.'</locationToken></'.$method.'></s:Body></s:Envelope>';
        error_log("body: $body");
        error_log(" ");
        curl_setopt($curl, CURLOPT_POST, 1);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $body);
        $headers = array('Content-Type: text/xml; charset=utf-8', 'Content-Length: ' . strlen($body), "SOAPAction: $soap_action");
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        logCurl($url,'POST',$up,$headers,$body);
        $result = curl_exec($curl);
        //error_log($result);
        return $result;
    }

    function getLocationToken($merchant_resource)
    {
        if ($this->merchant_location_token) {
            return $this->merchant_location_token;
        } else if ($this->brink_location_token) {
            return $this->brink_location_token;
        }
        $merchant_resource = $merchant_resource == null ? $this->merchant_resource : $merchant_resource;
        $mbimr = MerchantBrinkInfoMapsAdapter::staticGetRecord(array("merchant_id"=>$merchant_resource->merchant_id),'MerchantBrinkInfoMapsAdapter');
        $location_token = $mbimr['brink_location_token'];
        myerror_logging(1,"we have the brink location token: $location_token");
        if (isNotProd() && !isLaptop()) {
            $location_token = getProperty('brink_test_location_token');
        }
        $this->merchant_location_token = $location_token;
        return $location_token;
    }

    function writeFile($file_name,$content)
    {
        if ($myfile = fopen($file_name, "w")) {
            fwrite($myfile, $content);
            fclose($myfile);
        } else {
            myerror_log("******************** COULDNT WRITE XML DUMP TO FILE $file_name ***********************");
        }
    }

    function getBrandName()
    {
        return $this->brand_name;
    }

    function setBuildModifierStructure($bool)
    {
        $this->build_modifier_structure = $bool;
    }

    function importFromActivity($mixed)
    {
        return $this->import($mixed);
    }

    function importWithModifierStructureBuild($merchant_external_id)
    {
        $this->setBuildModifierStructure(true);
        $this->import($merchant_external_id);
    }


    /* ABSTRACT METHTODS NOT USED WITH BRINK SINCE WE'RE JUST DOING PRICES */


    function getItemDescriptionFromItemData($item_data){}

    function getModifierGroupMinForThisIntegration($modifier_group_data){}

    function getModifierGroupMaxForThisIntegration($modifier_group_data)
    {
        return $modifier_group_data['Maximum'] == 0 ? 10 : $modifier_group_data['Maximum'];
    }

    function getActiveFlagSetting($data){}

    function getMenuTypeCatId($menu_type_data){}

}
?>