<?php
/**
 * Created by PhpStorm.
 * User: Elvis
 * Date: 3/1/16
 * Time: 11:07 AM
 */
class ElPolloLocoImporter extends SplickitImporter
{
    const BRAND_ID = 307;
    private $base_api_url;

    private $endpoints  =  array(
        "merchant_data" => "/USD_Data_XML.jsp?unitID={merchant_external_id}",
        "merchant_tax_data" => "/Tax_Data_XML.jsp?unitId={merchant_external_id}",
    );

    function __construct($merchant_mixed)
    {
        parent::__construct($merchant_mixed);
        $this->base_api_url = getProperty('elpolloloco_api_url');
    }

    function setBrandRegExForMerchantExternalIds()
    {
        $this->brand_regex_for_merchant_external_ids = '\d+';
    }

    function setBrandId()
    {
        $this->brand_id = self::BRAND_ID;
    }

    function importRemoteData($merchant_resource)
    {
        if($merchant_data = $this->getRemoteData($merchant_resource)){
            // save basic info
            $merchant_resource -> set("phone_no", $merchant_data["phone"]);
            $merchant_resource -> set("fax_no", $merchant_data["fax"]);
            $merchant_resource -> set("shop_email", $merchant_data["storeEmail"]);
            $merchant_resource -> set("ordering_on", $merchant_data["tempClose"] == null || $merchant_data["tempClose"]  == '0'? 'Y': 'N');
            $merchant_resource -> set("custom_menu_message", $merchant_data["tempCloseMessage"]);
            //$merchant_resource -> set("delivery", $merchant_data["isDelivery"]);

            list($lat, $lng) = explode(",", preg_replace(array("/\(/", "/\)/"), array("",""), $merchant_data["point"]));
            $merchant_resource -> set('lat', $lat);
            $merchant_resource -> set('lng', $lng);

            $timezone = $this->calculateTimeZoneFromRemoteData($merchant_data["timeZone"]);
            list($time_zone, $time_zone_string) = $timezone;
            $merchant_resource->set("time_zone", $time_zone);
            $merchant_resource->set("time_zone_string", $time_zone_string);
            $merchant_resource->save();

            //store hours
            $storeHours = $this->buildStoreHoursFromRemoteData($merchant_data["storeHours"]);
            $options[TONIC_FIND_BY_METADATA]['merchant_id'] = $merchant_resource->merchant_id;
            $options[TONIC_FIND_BY_METADATA]['hour_type'] = 'R';
            $store_hours_resource = Resource::findAll(new HourAdapter($mt), null,$options);
            foreach($store_hours_resource as $storeHour){
                $storeHour->set("open", $storeHours[$storeHour->day_of_week]["open"]);
                $storeHour->set("close", $storeHours[$storeHour->day_of_week]["close"]);
                $storeHour ->save();
            }

            //tax
            unset($options);
            $options[TONIC_FIND_BY_METADATA]['merchant_id'] = $merchant_resource->merchant_id;
            $options[TONIC_FIND_BY_METADATA]['tax_group'] = 1;
            if($tax_resource = Resource::findExact(new TaxAdapter($mt), null,$options)){
                $tax_resource->set("rate", $merchant_data["taxPercentage"]);
                $tax_resource->save();
            }

            //catering
            unset($options);
            $options[TONIC_FIND_BY_METADATA]['merchant_id'] = $merchant_resource->merchant_id;
            if($mdpd_resource = Resource::findExact(new MerchantDeliveryPriceDistanceAdapter($mt), null,$options)){
                $mdpd_resource->set("distance_up_to", $merchant_data["cateringRadius"]);
                $mdpd_resource->save();
            }

            unset($options);
            $options[TONIC_FIND_BY_METADATA]['merchant_id'] = $merchant_resource->merchant_id;
            if($mdi_resource = Resource::findExact(new MerchantDeliveryInfoAdapter($mt), null,$options)){
                $mdi_resource->set("delivery_price_type", "driving");
                $mdi_resource->save();
            }

            $this->message = "The import has been completed. The data update  merchant $merchant_resource->merchant_external_id,  is completed";

        }
    }

    function calculateTimeZoneFromRemoteData($remote_data){
        $zones = array(
            "Eastern Standard Time" => array("America/New_York",-5),
            "Central Standard Time"  => array("America/Chicago",-6),
            "Mountain Standard Time" => array("America/Denver",-7),
            "Mountain Standard Time no DST" => array("America/Phoenix",-7),
            "Pacific Standard Time"=> array("America/Los_Angeles",-8)
        );
        $timezone_data = $zones[$remote_data];
        $time_zone = $timezone_data[0];
        $offset = $timezone_data[1];
        $time_zone_string = "(GMT$offset:00) $time_zone (US & Canada)";
        return array($offset, $time_zone_string);

    }

    function buildStoreHoursFromRemoteData($remote_data){
        $time = strtotime("Thursday");
        $hour_data = explode(",", $remote_data);
        $store_hours = array();
        foreach($hour_data as $hour){
            $day_of_week = date('w', $time)+1;
            list($open, $close) = explode("-", $hour);
            $store_hours[$day_of_week] = array(
                "open" => $open,
                "close" => $close
            );
            $time = strtotime('+1 day', $time);
        }
        return $store_hours;
    }

    function getRemoteData($merchant_resource)
    {
        return $this->getRemoteMerchantData($merchant_resource);
    }

    function getRemoteMerchantData($merchant_resource)
    {
        $endpoint = $this->endpoints["merchant_data"];
        $url = $this->base_api_url . str_replace("{merchant_external_id}", $merchant_resource->merchant_external_id, $endpoint);
        $response = ElPolloLocoCurl::curlIt($url);
        if($merchant_metadata = $this->processXmlMerchantData($response, $merchant_resource)){
            $merchant = array("merchant_external_id" => $merchant_metadata['@attributes']['unitID'][0]);

            foreach($merchant_metadata['usdFieldDef'] as $fields){
                $merchant[$fields['@attributes']['name'][0]] = $fields['@attributes']['value'][0];
            }

            $taxendpoint = $this->endpoints["merchant_tax_data"];
            $taxurl = $this->base_api_url . str_replace("{merchant_external_id}", $merchant_resource->merchant_external_id, $taxendpoint);
            $tax_response = ElPolloLocoCurl::curlIt($taxurl);
            if($tax_metadata = $this->processRemoteMerchantTaxData($tax_response, $merchant_resource)){
                $merchant["taxPercentage"] = $tax_metadata['@attributes']['percentage'][0];
            }
            return $merchant;
        }
        return false;
    }


    function processXmlMerchantData($remote_data, $merchant_resource){
        if($remote_data["http_code"] == 200){
            $raw_data = $remote_data["raw_result"];
            if ($start = strpos($raw_data, '<usdStore unitID="'.$merchant_resource->merchant_external_id.'"')) {
                $end_string = "</usdStore>";
                $end = strpos($raw_data, $end_string, $start) + strlen($end_string);
                $payload = substr($raw_data, $start, $end - $start);
                if($xml = simplexml_load_string($payload)){
                    return myToArray($xml);
                } else{
                    myerror_logging(3, "Error cannot create object from merchant xml: $payload");
                    $this->message = "Error cannot create object from merchant xml: $payload";
                    return false;
                }
            }else{
                myerror_logging(3, "Error not found paylod for merchant: $merchant_resource->merchant_external_id");
                $this->message = "Error not found paylod for merchant: $merchant_resource->merchant_external_id";
                return false;
            }

        }else{
            myerror_logging(3, "Error when get merchant data: { error_code => ".$remote_data["error_no"].", error => ".$remote_data["error"]."}");
            $this->message = "Error when get merchant data: { error_code => ".$remote_data["error_no"].", error => ".$remote_data["error"]."}";
            return false;
        }

    }

    function processRemoteMerchantTaxData($remote_data, $merchant_resource){
        if($remote_data["http_code"] == 200){
            $raw_data = $remote_data["raw_result"];
            if ($start = strpos($raw_data, '<row UnitID="'.$merchant_resource->merchant_external_id.'"')) {
                $end_string = "/>";
                $end = strpos($raw_data, $end_string, $start) + strlen($end_string);
                $payload = substr($raw_data, $start, $end - $start);
                if($xml = simplexml_load_string($payload)){
                    return myToArray($xml);
                } else{
                    myerror_logging(3, "Error cannot create object from tax xml: $payload");
                    $this->message = "Error cannot create object from tax xml: $payload";
                    return false;
                }
            }else{
                myerror_logging(3, "Error not found tax paylod for $merchant_resource->merchant_external_id");
                $this->message = "Error not found tax paylod for $merchant_resource->merchant_external_id";
                return false;
            }

        }else{
            myerror_logging(3, "Error when get tax merchant data: { error_code => ".$remote_data["error_no"].", error => ".$remote_data["error"]."}");
            $this->message = "Error when get tax merchant data: { error_code => ".$remote_data["error_no"].", error => ".$remote_data["error"]."}";
            return false;
        }

    }


    /***** functions that need to be overwritten for the specific integration *******/
    function getItemDescriptionFromItemData($item_data)
    {
        // TODO: Implement getItemDescriptionFromItemData() method.
    }

    function getModifierGroupMinForThisIntegration($modifier_group_data)
    {
        // TODO: Implement getModifierGroupMinForThisIntegration() method.
    }

    function getModifierGroupMaxForThisIntegration($modifier_group_data)
    {
        // TODO: Implement getModifierGroupMaxForThisIntegration() method.
    }

    function getActiveFlagSetting($data)
    {
        // TODO: Implement getActiveFlagSetting() method.
    }

    function getMenuTypeCatId($menu_type_data)
    {
        // TODO: Implement getMenuTypeCatId() method.
    }

    function getObjectPriceFromItemData($item_data)
    {
        // TODO: Implement getObjectPriceFromItemData() method.
    }

    function setMenuId()
    {
        // TODO: Implement setMenuId() method.
    }
}
?>