<?php
class EplImporter extends SplickitImporter
{
    var $imported_prices = array();

    const MENU_ID = 102711;
    const BRAND_ID = 307;

    var $merchant_resource;
    var $raw_menu_payload;

    var $item_price_records_imported;
    var $modifier_price_records_imported;

    var $upcharges_by_item = array();
    var $large_drink_combo_upcharge = 0.00;

    var $item_group_hash_of_zero_prices = array();

    function __construct($merchant_mixed)
    {
        myerror_log("EPL IMPORT:  starting constructor is EPL importer");
        parent::__construct($merchant_mixed);
        $this->create_zero_records = false;
    }

    function setBrandId()
    {
        $this->brand_id = self::BRAND_ID;
    }

    function setBrandRegExForMerchantExternalIds()
    {
        return false;
        //$this->brand_regex_for_merchant_external_ids = '[0-9a-zA-Z\-]+';
    }

    function importMenuFromRequest($request)
    {
        myerror_log("EPL IMPORT:  starting import menu from request in eplimporter");
        myerror_log("EPL IMPORT body: ".$request->body,5);
        $this->raw_menu_payload = $request->body;
        $merchant_id = $this->merchant_resource->merchant_id;
        if (getProperty("write_epl_import_file") == 'true') {
            $file_name = "./menus/epl_menu_$merchant_id.xml";
            MailIt::sendErrorEmail("epl_menu_$merchant_id.xml",$request->body);
            if ($myfile = fopen($file_name, "w")) {
                fwrite($myfile, $request->body);
                fclose($myfile);
            } else {
                myerror_log("EPL IMPORT:  ******************** COULDNT WRITE XML DUMP TO FILE ***********************");
            }
        }
        $this->getPriceData();
        return $this->importRemoteData($this->merchant_resource);
    }

    function getRemoteData($merchant_resource)
    {
        if ($this->imported_prices == null || !is_array($this->imported_prices)) {
            myerror_log("EPL IMPORT:  Import Exception!!! No data has been loaded for EPL import");
            throw new UnsuccessfulEPLImportException($merchant_resource->merchant_external_id);
        }
    }

    function callCustomMethodsOnData()
    {
        // this really should be a method list but since only 1 thing to do i punted
        $menu_id = self::MENU_ID;
        $sql = "SELECT a.*,b.modifier_item_id,b.modifier_item_name FROM Modifier_Group a JOIN Modifier_Item b ON a.modifier_group_id = b.modifier_group_id WHERE a.menu_id = $menu_id";
        $mga = new ModifierGroupAdapter($m);
        $records = $mga->select(null,array(TONIC_FIND_BY_SQL=>$sql));
        foreach ($records as $record) {
            if (strtolower($record['external_modifier_group_id']) == 'zero_prices') {
                $name = $record['modifier_item_name'];
                $id = $record['modifier_item_id'];
                myerror_log("EPL IMPORT:  this modifier is being set to zero_price: $name - $id",3);
                $this->item_group_hash_of_zero_prices[$record['modifier_item_id']] = 1;
            }
        }
    }

    function importRemoteData($merchant_resource)
    {
        parent::importRemoteData($merchant_resource);
        $this->message = "There were ".$this->getNumberOfItemPricesImported()." item prices imported, and ".$this->getNumberOfModifierPricesImported()." modifier prices imported.";
    }

    function doCustomImgmLogic($imgm_resource)
    {
        if ($upcharges_for_item = $this->upcharges_by_item[$imgm_resource->item_id]) {
            $modifier_group = ModifierGroupAdapter::staticGetRecordByPrimaryKey($imgm_resource->modifier_group_id,'ModifierGroupAdapter');
            $short_modifier_group_name = strtolower(substr($modifier_group['modifier_group_name'],0,9));
            if ($short_modifier_group_name == 'combo 621') {
                // we need to adjust the prices of these items in the group
                $modifier_item_records_for_group = ModifierItemAdapter::staticGetRecords(array("modifier_group_id"=>$imgm_resource->modifier_group_id),'ModifierItemAdapter');
                foreach ($modifier_item_records_for_group as $modifier_item_record) {
                    foreach ($upcharges_for_item as $size_id=>$price) {
                        $msma = new ModifierSizeMapAdapter($m);
                        $default_price_record = $msma->getRecord(array("modifier_item_id"=>$modifier_item_record['modifier_item_id'],"size_id"=>"0","merchant_id"=>"0"));
                        $price_resource = Resource::findOrCreateIfNotExistsByData($msma,array("modifier_item_id"=>$modifier_item_record['modifier_item_id'],"size_id"=>$size_id,"merchant_id"=>$this->merchant_resource->merchant_id));
                        $price_resource->modifier_price = $default_price_record['external_id'] == '8019002-8019300:C' ? $price + $this->large_drink_combo_upcharge : $price;
                        $price_resource->external_id = $default_price_record['external_id'];
                        $price_resource->save();
                    }
                }
            }
        }
    }

    function getNumberOfItemPricesImported()
    {
        return $this->item_price_records_imported;
    }

    function getNumberOfModifierPricesImported()
    {
        return $this->modifier_price_records_imported;
    }

    function savePricesForPriceResources(&$splickit_price_resources)
    {
        $total_price_records_imported = 0;
        myerror_log("EPL IMPORT:  about to import EPL prices for ".count($splickit_price_resources)." splickit price resources");
        foreach ($splickit_price_resources as $splickit_price_resource) {
            if ($this->processPriceResource($splickit_price_resource)) {
                $total_price_records_imported++;
            }
        }
        return $total_price_records_imported;
    }

    function getObjectPriceFromItemData($item_data)
    {
        return $item_data['PRICE'];
    }

    function getTaxableFromImportedPriceHashByPluRecord($record,$secondary_plu)
    {
        return $record['TAXID1'];
    }

    function processPriceResource(&$splickit_price_resource)
    {
        //if (is_a($splickit_price_resource->_adapter,'ModifierSizeMapAdapter')) {
        myerror_log("EPL IMPORT:  about to update price resource: ".$splickit_price_resource->external_id);
        $merchant_id = $this->merchant_resource->merchant_id;
        $imported_price_hash_by_plu = $this->imported_prices;
        $raw_plu_on_splickt_price_record = $splickit_price_resource->external_id;
        $p = explode(":",$raw_plu_on_splickt_price_record);
        $plu = $p[0];
        if (isset($splickit_price_resource->modifier_price)) {
            if ($this->item_group_hash_of_zero_prices[$splickit_price_resource->modifier_item_id]) {
                myerror_log("EPL IMPORT:  this modifier is being set to zero_price: eplmod_id: ".$splickit_price_resource->modifier_item_id,3);
                $splickit_price_resource->modifier_price = 0.00;
                $splickit_price_resource->merchant_id = $this->merchant_resource->merchant_id;
                return $splickit_price_resource->save();
            } else {
                myerror_log("EPL IMPORT:  this modifier is being set to a PRICE: eplmod_id: ".$splickit_price_resource->modifier_item_id,3);
//                if ($p[1] == 'C') {
//                    $exploded_array = explode('-',$secondary_plu);
//                    if ($secondary_plu = $exploded_array[1]) {
//                        $forced_combo = true;
//                    }
//                }
            }
        } else {
            if ($secondary_plu = isset($p[1]) ? $p[1] : null) {
                $exploded_array = explode('-',$secondary_plu);
                $secondary_plu = $exploded_array[0];
            }
            $forced_combo = $p[2] == 'FC' ? true : false;
            $forced_meal = false;
            if (!$forced_combo && isset($p[2])) {
                $xa = explode('-',$p[2]);
                $combo_plu = $xa[0];
                $forced_meal = true;
            } else if (!$forced_combo && $secondary_plu != null) {
                $combo_plu = $secondary_plu;
            }
        }

        myerror_log("EPL IMPORT:  about to get price for: $plu");
        $plu = $this->getPrimaryPlu($plu);
        if (isset($imported_price_hash_by_plu[$plu])) {

            $imported_price_record = $forced_combo || $forced_meal ? $imported_price_hash_by_plu[$secondary_plu] : $imported_price_hash_by_plu[$plu];
            if ($secondary_plu && !$forced_combo) {
                myerror_log("EPL IMPORT:  we have a not forced combo on the import so get the price difference");
                $combo_price = floatval($this->getObjectPriceFromItemData($imported_price_hash_by_plu[$combo_plu]));
                $regular_price = $forced_meal ? floatval($this->getObjectPriceFromItemData($imported_price_hash_by_plu[$secondary_plu])) : floatval($this->getObjectPriceFromItemData($imported_price_hash_by_plu[$plu]));
                $upcharge = $combo_price - $regular_price;
                $this->upcharges_by_item[$splickit_price_resource->item_id][$splickit_price_resource->size_id] = $upcharge;
            }
            $splickit_price_resource->active = 'Y';
            $price = $this->getObjectPriceFromItemData($imported_price_record);
            myerror_log("EPL IMPORT:  price from import is: ".$price);
            $splickit_price_resource->price = $price;
            $splickit_price_resource->modifier_price = $price;
            $splickit_price_resource->tax_group = $this->getTaxableFromImportedPriceHashByPluRecord($imported_price_record) == '1' ?  1  : 0;
            $price_recorded = true;
        } else {
            // there was no price returned so create an innactive price record
            myerror_log("EPL IMPORT:  there was no price returned so create an innactive price record",3);
            $splickit_price_resource->active = 'N';
            $price_recorded = false;
        }
        $splickit_price_resource->merchant_id = $merchant_id; // in case this is a default record
        if ($splickit_price_resource->save()) {
            myerror_log("EPL IMPORT:  we ahve saved the price for $plu as: $price");
            return $price_recorded;
        }
    }

    function getPrimaryPlu($plu)
    {
        $plu = explode('+',$plu);
        $plu = explode('-',$plu[0]);
        return $plu[0];
    }

    function getPriceData()
    {
        if ($this->imported_prices) {
            return true;
        }
        $raw_data = $this->raw_menu_payload;
        $this->processRawExport($raw_data);
    }

    function processRawExport($raw_export)
    {
        if ($start = strpos($raw_export, "<MENUITEMS")) {
            $end_string = "</MENUITEMS>";


            $end = strpos($raw_export, $end_string) + strlen($end_string);
            $payload = substr($raw_export, $start, $end - $start);
            $xml = simplexml_load_string($payload) or die("error: cannot create object");
            $hashmap = $this->myXMLToArray($xml);

            $plu_list = array();
            foreach ($hashmap['MENUITEM'] as $plu_node) {
                $price_node = array();
                foreach ($plu_node['@attributes'] as $attribute=>$value_as_subarray) {
                    $price_node[$attribute] = $value_as_subarray[0];
                }
                $plu_list[$plu_node['@attributes']['ITEMID'][0]] = $price_node;
            }
            ksort($plu_list);
            if (getProperty("write_epl_import_file") == 'true') {
                $merchant_id = $this->merchant_resource->merchant_id;
                $file_name = "./menus/imported_epl_plus_and_prices_$merchant_id.txt";
                if ($myfile = fopen($file_name, "w")) {
                    foreach($plu_list as $plu_record) {
                        fwrite($myfile, json_encode($plu_record)."\r\n");
                    }
                    fclose($myfile);
                    if ($txt = file_get_contents($file_name)) {
                        MailIt::sendErrorEmail("epl_menu_$merchant_id-plu_list.xml",$txt);
                    }

                }
            }
            $this->imported_prices = $plu_list;
        } else {
            throw new UnsuccessfulEPLImportException($this->merchant_resource->merchant_id . " - " . $this->merchant_resource->alphanumeric_id);
        }
        if ($start = strpos($raw_export, "<MODIFIERGROUPS")) {
            $end_string = "</MODIFIERGROUPS>";
            $end = strpos($raw_export, $end_string) + strlen($end_string);
            $payload = substr($raw_export, $start, $end - $start);
            $xml = simplexml_load_string($payload) or die("error: cannot create object");
            $hashmap = $this->myXMLToArray($xml);
            foreach ($hashmap['MODIFIERGROUP'] as $mod_plu_node) {
                if ($mod_plu_node['@attributes']['MODGRPID'][0] == "990011") {
                    // get the price
                    foreach ($mod_plu_node['MODIFIERS']['MODIFIER'] as $mod_price_node) {
                        if ($mod_price_node['@attributes']['ITEMID'][0] == '8019002') {
                            $this->large_drink_combo_upcharge = $mod_price_node['@attributes']['PRICE'][0];
                            return;
                        }
                    }
                }
            }
        }
    }

    function myXMLToArray($xml) {
        $array = (array)$xml;

        foreach ( array_slice($array, 0) as $key => $value )
        {
            if ( $value instanceof SimpleXMLElement )
            {
                if ($key == "@attributes") {
                    continue;
                }
                $array[$key] = empty($value) ? NULL : $this->myXMLToArray($value);
            } else if (is_array($value)) {
                foreach ($value as $sub_key => $sub_value) {
                    $array[$key][$sub_key] = $this->myXMLToArray($sub_value);
                }
            }
        }
        return $array;
    }

    function getMerchant()
    {
        return $this->merchant_resource->getDataFieldsReally();
    }

    // need these to make sure not abstract. we're not useing them because we're only importing prices for ElPolloLoco

    function getItemDescriptionFromItemData($item_data) {}

    function getModifierGroupMinForThisIntegration($modifier_group_data) {}

    function getModifierGroupMaxForThisIntegration($modifier_group_data) {}

    function getActiveFlagSetting($data) {}

    function getMenuTypeCatId($menu_type_data) {}

    function setMenuId()
    {
        $this->menu_id = self::MENU_ID;
    }
}
class UnsuccessfulEPLImportException extends Exception
{
    public function __construct($merchant_ids) {
        parent::__construct("We could not import the prices for this merchant: $merchant_ids", 100);
    }
}
?>