<?php
class LjsImporter extends SplickitImporter
{

    var $merchant_site_id_map;
    var $ljs_merchant_resources;
    var $activity_info;

    function __construct($data)
    {
        parent::__construct($data);
    }

    function loadImportTableFromFile()
    {
        //$file_name = getProperty('ljs_import_file_name');
        $s3 = new AWSService();
        $bucket = getProperty('bucket_prefix').".vm";
        if ($single_file_name = $this->data['file_name']) {
            $files = [$single_file_name];
        } else {
            $files = array();
            $full_path_file_names = $s3->getFileNameList($bucket,"Exports/pricing_");
            foreach ($full_path_file_names as $full_path_file_name) {
                $k = explode('/',$full_path_file_name);
                $files[] = $k[1];
            }
        }
        $site_codes = array();
        if ($files) {
            foreach ($files as $file_name) {
                $sc = explode('_',$file_name);
                $site_code = str_replace('.csv','',$sc[1]);
                $site_codes[] = $site_code;
                $key = "Exports/$file_name";
                if ($s3->getOrElseWithLocalSave($bucket,$key,$file_name,'none')) {
                    myerror_log("ljs price file exists and we have downloaded the file to the /tmp/ folder");
                    $new_name = date("Y-m-d_") . $file_name;
                    $new_key = "Exports/$new_name";
                    if (! isLaptop()) {
                        $s3->rename($bucket, $key, $new_key);
                        $s3->move($bucket, $new_key, 'Exports/archive/' . $new_name);
                    }
                } else {
                    myerror_log("No file exists on S3 with file name: $file_name");
                    continue;
                }

                if (file_exists("/tmp/$file_name")) {
                    $handle = fopen("/tmp/$file_name", "r");
                    $lineNumber = 1;
                    $index_of_price = 0;
                    $index_of_site_code = 0;
                    $sql_adapter = new MySQLAdapter();
                    $delete_sql = "DELETE FROM `Ljs_Prices_Dump` WHERE `SiteCode` = $site_code";
                    $sql_adapter->_query($delete_sql);

                    while (($raw_string = fgets($handle)) !== false) {
                        $row = str_getcsv($raw_string);
                        if ($lineNumber == 1) {
                            $names = '';
                            foreach ($row as $index=>$name) {
                                if ($name == 'Price') {
                                    $index_of_price = $index;
                                } else if ($name == 'SiteCode') {
                                    $index_of_site_code = $index;
                                }
                                $names .= $sql_adapter->_escapeFieldName($name).', ';
                            }
                            $names = substr($names, 0, -2);
                        } else {
                            $values = '';
                            $row_site_code = null;
                            foreach ($row as $index=>$value) {

                                if ($index == $index_of_site_code) {
                                    $row_site_code = (int) $value;
                                }

                                if ($index == $index_of_price) {
                                    if (!is_numeric($value)) {
                                        myerror_log("SOMETHING OTHER THAN A NUMBER SUBMITTED FOR PRICE. skipping row: ".json_encode($row));
                                        break;
                                    }
                                    $values .= "$value, ";
                                } else {
                                    $values .= $sql_adapter->_delimitString($value).', ';
                                }

                            }
                            $values = substr($values, 0, -2);
                            $sql = sprintf(
                                'INSERT INTO %s (%s) VALUES (%s)',
                                '`Ljs_Prices_Dump`',$names,$values
                            );
                            $sql_adapter->_query($sql);
                        }

                        // Increase the current line
                        $lineNumber++;
                    }

                    myerror_log("price data for sitecode $site_code loaded");
                    $activity_history_adapter = new ActivityHistoryAdapter(getM());
                    if ($activity_history_resource = $activity_history_adapter->createActivityReturnActivityResource('LjsPriceSync', time() - 5, 'process=SavePrices;site_code='.$site_code, null)) {
                        myerror_log("Activity to load prices for site_code $site_code has been created. activity_id: ".$activity_history_resource->activity_id);
                    } else {
                        MailIt::sendErrorEmail("Unable to create price sync activity for Long John Silver after saving data in to the table");
                    }

                    //delete the file
                    fclose($handle);
                    unlink("/tmp/$file_name");

                } else {
                    myerror_log("No file in tmp folder matcing $file_name move on to next merchant");
                }
            }

            return $site_codes;


        } else {
            myerror_log("No could not get files from S3 remote drive matcing Exports/prices_");
            return false;
        }

    }

    function loadPricesForMerchantFromImportTable($site_code)
    {
        $this->imported_prices = null;
        $sql = "SELECT * FROM `Ljs_Prices_Dump` WHERE `SiteCode` = '$site_code'";
        $options[TONIC_FIND_BY_SQL] = $sql;
        $m = getM();
        $msa = new MySQLAdapter($m);
        if ($records = $msa->select(null,$options)) {
            foreach ($records as $record) {
                $this->imported_prices[$record['SystemID']] = $record;
            }
            return true;
        } else {
            myerror_log("LJS IMPORT ERROR: No data for SiteCode: $site_code");
            return false;
        }
    }

    function importDataFromS3FileToDataBase($info)
    {
        $this->activity_info = $info;
        if (! $this->loadImportTableFromFile() ) {
            return false;
        }
    }

    function importPricesForSiteCode($site_code)
    {
        $merchant_adapter = new MerchantAdapter();
        if ($merchant_resource = Resource::find($merchant_adapter,null,[TONIC_FIND_BY_METADATA=>['brand_id'=>517,'merchant_external_id'=>$site_code]])) {
            return $this->importPricesForMerchantResource($merchant_resource);
        } else {
            myerror_log("ERROR!!!! Unable to find merchant for LJS site code: $site_code");
        }
        return false;
    }

    function importPricesForMerchantResource($merchant_resource)
    {
        $this->merchant_resource = $merchant_resource;
        $merchant_id = $merchant_resource->merchant_id;

        if ($this->loadPricesForMerchantFromImportTable($merchant_resource->merchant_external_id)) {
            // get the menus
            $mmma = new MerchantMenuMapAdapter(getM());
            $sql = "SELECT DISTINCT menu_id FROM Merchant_Menu_Map WHERE merchant_id = $merchant_id AND logical_delete = 'N'";
            $options[TONIC_FIND_BY_SQL] = $sql;

            if ($results = Resource::findAll($mmma,null,$options)) {
                // put delete here
                if ($merchant_id > 0) {
                    $sql = "DELETE FROM Item_Modifier_Group_Map where merchant_id = $merchant_id";
                    $imgma = new ItemModifierGroupMapAdapter(getM());
                    $imgma->_query($sql);
                }

                foreach ($results as $result_resource) {
                    $this->menu_id = $result_resource->menu_id;
                    $this->merchant_mixed = "LJS/merchant/$merchant_id/menu/".$this->menu_id;
                    $this->importRemoteData($merchant_resource);
                    myerror_log("finshing merchant_id: $merchant_id  menu_id: ".$result_resource->menu_id);
                }
            }
        } else {
             return false;
        }
    }

    function importEntireBrandFromDataInTable()
    {
        // this is called from an Activity
        $this->loadLjsMerchants();
        foreach ($this->ljs_merchant_resources as $merchant_resource) {
            $this->importPricesForMerchantResource($merchant_resource);
        }
        return true;
    }

    function setResultsMessage()
    {
        $merchant_id = $this->merchant_resource->merchant_id;
        $menu_id = $this->menu_id;
        //$this->message = "For merchant_id: $merchant_id  and  menu_id: $menu_id,  there were ".$this->getNumberOfItemPricesImported()." item prices imported, and ".$this->getNumberOfModifierPricesImported()." modifier prices imported. \r\n";
        $this->message = "For merchant_id: $merchant_id  and  menu_id: $menu_id,  item prices: ".$this->getNumberOfItemPricesImported()."  modifier prices: ".$this->getNumberOfModifierPricesImported()."   imgm records: ".$this->number_of_imgm_records_created." \r\n" ;
        $this->prependToRunningImport($this->message);
    }


    function getRemoteData($merchant_resource)
    {
        //we're not using this method for LJS since there are muliple merchangs per import and multiple menu's per merchant
        //$this->loadImportTable();
        return true;
    }


    function loadLjsMerchants()
    {
        $merchant_adapter = new MerchantAdapter(getM());
        if ($merchant_resources = Resource::findAll($merchant_adapter,null,[TONIC_FIND_BY_METADATA=>['brand_id'=>517]])) {
            foreach ($merchant_resources as $merchant_resource) {
                $this->merchant_site_id_map[$merchant_resource->merchant_external_id] = $merchant_resource->merchant_id;
                $this->ljs_merchant_resources[$merchant_resource->merchant_external_id] = $merchant_resource;
            }
        }
    }

    function processPriceResource(&$splickit_price_resource)
    {
        $id_size_string = isset($splickit_price_resource->item_size_id) ? 'item_size_id = '.$splickit_price_resource->item_size_id : 'modifier_size_id = '.$splickit_price_resource->modifier_size_id;
        if ($splickit_price_resource->active == 'N') {
            myerror_log("innactive price record, skip import for $id_size_string",5);
            $this->addToRunningImport("Innactive record at merchant, skip import. $id_size_string");
            return false;
        } else if ($splickit_price_resource->merchant_id == 0 ) {
            myerror_log("merchant doesnt' have a daughter record for this item. skip import",5);
            return false;
        }
        $merchant_id = $this->merchant_resource->merchant_id;
        $imported_price_hash_by_plu = $this->imported_prices;
        $raw_plu_on_splickt_price_record = $splickit_price_resource->external_id;
        $a = explode(":",$raw_plu_on_splickt_price_record);
        $plu = $a[0];
        $p = explode('+',$plu);
        if (isset($p[1])) {
            $plu = $p[0];
        }

        if ($plu == null || trim($plu) == '') {
            $json_of_thing = json_encode($splickit_price_resource->getDataFieldsReally());
            myerror_log("BLANK PLU SKIP UPDATE FOR THIS ITEM!  price record: $json_of_thing",5);
            $this->addToRunningImport("BLANK PLU SKIP UPDATE FOR THIS ITEM!  price record: $json_of_thing");
            return false;
        }
        myerror_log("about to get price for: $raw_plu_on_splickt_price_record",5);
        if ($imported_price_record = $imported_price_hash_by_plu[$plu]) {
            if ($a[1]) {
                $adjective_id = $a[1];
                myerror_log("we have an item with a size associate with it so we need to get it out of the db: $adjective_id",5);
                $site_code = $this->merchant_resource->merchant_external_id;
                if ($adjective_id == '414') {
                    $product_name_string = " POSProductName LIKE 'SM %' ";
                } else if ($adjective_id == '415') {
                    $product_name_string = " (POSProductName LIKE 'MD %' OR POSProductName LIKE 'RG %')";
                } else if ($adjective_id == '416') {
                    $product_name_string = " POSProductName LIKE 'LG %' ";
                } else {
                    myerror_log("WE HAVE AN UNKNOWN ADJECTIVE IN THE MENU for mid: $merchant_id  raw plu: $raw_plu_on_splickt_price_record ");
                    //add a bad string to make sure the SQL doesn't blow up
                    $product_name_string = " POSProductName = 'BAD STRING' ";
                }

                $sql = "SELECT * FROM `Ljs_Prices_Dump` WHERE `SiteCode` = '$site_code' AND SystemID = '$plu' AND $product_name_string";
                $options[TONIC_FIND_BY_SQL] = $sql;
                $m = getM();
                $msa = new MySQLAdapter($m);
                if ($records = $msa->select(null,$options)) {
                    foreach ($records as $record) {
                        $imported_price_record = $record;
                    }
                } else {
                    $this->addToRunningImport("COULDN'T find data in price dump to match: $raw_plu_on_splickt_price_record");
                    return false;
                }
            }
//            $starting_active = $splickit_price_resource->active;
//            $starting_price = isset($splickit_price_resource->price) ? $splickit_price_resource->price : $splickit_price_resource->modifier_price;
            //$splickit_price_resource->active = 'Y';
            $splickit_price_resource->price = $this->getObjectPriceFromItemData($imported_price_record);
            $splickit_price_resource->modifier_price = $this->getObjectPriceFromItemData($imported_price_record);
            $this->doCustomPriceManipulationForPriceResource($splickit_price_resource);
            //$splickit_price_resource->tax_group = $this->isImportedItemTaxable($imported_price_record) ? 1 : 0;
            $price_recorded = true;
        } else {
            // there was no price returned so create an innactive price record
            myerror_log("WE DID NOT IMPORT A PRICE RECORD FOR PLU: $plu.  SET record to INACTIVE!",5);
            $id_string = isset($splickit_price_resource->item_id) ? 'item_id = '.$splickit_price_resource->item_id : 'modifier_id = '.$splickit_price_resource->modifier_item_id;
            $this->addToRunningImport("WE DID NOT IMPORT A PRICE RECORD FOR $id_string  SIZE_ID: ".$splickit_price_resource->size_id."   PLU: $plu SET record to INACTIVE!");
            //$splickit_price_resource->active = 'N';
            $price_recorded = false;
        }
        $splickit_price_resource->merchant_id = $merchant_id; // in case this is a default record
//        if ($a[1] == 'zero_price') {
//            $splickit_price_resource->modifier_price = 0.00;
//            $splickit_price_resource->price = 0.00;
//        }
        if  ($this->saveThePriceResource($splickit_price_resource)) {
            $this->addToRunningImport("we have recorded the price for: ".json_encode($imported_price_record));
            return $price_recorded;
        } else {
            myerror_log("ERROR!!!  we couldnn't save the price record. ".$splickit_price_resource->_adapter->getLastErrorText()());
            $this->addToRunningImport("ERROR!!!  WE COULD NOT SAVE THE PRICE RECORD. error text: ".$splickit_price_resource->_adapter->getLastErrorText()());
        }
    }

    function getObjectPriceFromItemData($item_data){
        return $item_data['Price'];
    }

    function getItemDescriptionFromItemData($item_data){}

    function getModifierGroupMinForThisIntegration($modifier_group_data){}

    function getModifierGroupMaxForThisIntegration($modifier_group_data){}

    function getActiveFlagSetting($data){}

    function getMenuTypeCatId($menu_type_data){}








}
?>