<?php

class BrinkImporter extends SplickitImporter
{

    var $merchant_location_token;
    var $brand_menu_id = array("152"=>"102745");

    function __construct($merchant_mixed)
    {
        //see if the mixed is a url with brink location token
        if (preg_match('%/import/brink/(.+)$%', $merchant_mixed, $matches)) {
            $location_id_string = $matches[1];
            $s = explode("?",$location_id_string);
            $location_id = $s[0];
            $brink_info_record = getStaticRecord(array("brink_location_token"=>$location_id),'MerchantBrinkInfoMapsAdapter');
            if ($merchant_resource = Resource::find(new MerchantAdapter($m),''.$brink_info_record['merchant_id'])) {
                $merchant_mixed = $merchant_resource;
            }
        }
        parent::__construct($merchant_mixed);
        $this->create_zero_records = true;
    }

    function setBrandRegExForMerchantExternalIds()
    {
        $this->brand_regex_for_merchant_external_ids = '[0-9a-zA-Z:\-]+';
    }

    function setCreateZeroRecordsTrue()
    {
        $this->create_zero_records = true;
    }

    function setCreateZeroRecordsFalse()
    {
        $this->create_zero_records = false;
    }


    function processV2Request($request)
    {
        if (preg_match('%/importbrinkmerchant/pitapit/([0-9a-zA-Z\-]+)%', $request->url, $matches)) {
            $merchant_external_id = $matches[1];
            $merchant_resource = MerchantAdapter::getMerchantFromExternalIdBrandId($merchant_external_id,282);
            if (MerchantBrinkInfoMapsAdapter::isMechantBrinkMerchant($merchant_resource->merchant_id)) {
                $this->assignMerchantResource($merchant_resource);
                $menu_id = $this->importRemoteMerchantMetaDataForLoadedMerchant();
                $data = array("menu_id"=>$menu_id);
                $data['message'] = "The menu was imported successfully";
                $data['http_code'] = 200;
                $data['stamp'] = getRawStamp();
            } else {
                $data = array("error"=>'Merchant does not appear to be a brink merchant');
                $data['http_code'] = 422;
                $data['stamp'] = getRawStamp();
            }
        } else {
            $data = array("error"=>'Malformed URL');
            $data['http_code'] = 422;
            $data['stamp'] = getRawStamp();
        }
        $resource = Resource::dummyfactory($data);
        return $resource;
    }

    function getItemDescriptionFromItemData($item_data)
    {
        return $item_data['Name'];
    }

    function getModifierGroupMinForThisIntegration($modifier_group_data)
    {
        $min = isset($modifier_group_data['Minimum']) ? $modifier_group_data['Minimum'] : 0;
        if (strtolower($modifier_group_data['DisplayName']) == 'pita choice') {
            return 1;
        }
        return $min;
    }

    function getModifierGroupMaxForThisIntegration($modifier_group_data)
    {
        if (isset($modifier_group_data['Maximum']) && $modifier_group_data['Maximum'] > 0) {
            $max = $modifier_group_data['Maximum'];
        } else {
            $max = count($modifier_group_data['Items']['ModifierGroupItem']);
        }
        return $max;
    }

    function getActiveFlagSetting($data)
    {
        return strtolower($data['Active'] == 'true') ? 'Y' : 'N';
    }

    function getMenuTypeCatId($menu_type_data)
    {
        $menu_type_category = substr_count($menu_type_data['Name'],'Pitas') > 0 ? 'E' : 'S';
        return $menu_type_category;
    }

//    function getPitapitBrinkMenuResource($menu_name)
//    {
//        $data['external_menu_id'] = "pitapit-brink-master";
//        $data['version'] = 3.0;
//        $data['name'] = 'PitaPit-Brink-Master_V2';
//        if ($menu_resource = $this->getOrCreateMenuObjectResource($data,new MenuAdapter($mimetypes))) {
//            error_log("we have the parent menu resource");
//            return $menu_resource;
//        } else {
//            throw new Exception("error trying to import brink menu");
//        }
//    }

    function getMenuResourceForBrand($brand_id)
    {
        if ($menu_id = $this->getMenuIdFromBrandId($brand_id)) {
            if ($menu_resource = Resource::find(new MenuAdapter(getM()),$menu_id)) {
                return $menu_resource;
            }
        }
        throw new Exception("error trying to import brink menu. Couln't get Menu from brand_id:$brand_id   and   menu_id: $menu_id");
    }

    function getMenuIdFromBrandId($brand_id)
    {
        switch ($brand_id){
            case 152 :{
                $menu_id = 102745;
            } break;
            default :{
                $menu_id = nul;
            } break;
        }
        return $menu_id;
    }

    function doSizeLogic(&$brink_item,$menu_type_id,$menu_type_name)
    {
        $name = $brink_item['Name'];
        if (substr_count(strtolower($name),' sm pita') > 0) {
            $brink_item['Name'] = str_ireplace(' Sm Pita','',$name);
            $size_resource = $this->getSizeForMenuTypeId($menu_type_id,'Small Pita');
            if ($size_resource->insert_id) {
                $size_resource->priority = 50;
                $size_resource->save();
            }
        } else if (substr_count(strtolower($name),' reg pita') > 0) {
            $brink_item['Name'] = str_ireplace(' Reg Pita', '', $name);
            $size_resource = $this->getSizeForMenuTypeId($menu_type_id, 'Regular Pita');
            if ($size_resource->insert_id) {
                $size_resource->priority = 100;
                $size_resource->save();
            }
        } else if (substr_count(strtolower($name),' reg cookie') > 0) {
            $brink_item['Name'] = str_ireplace(' reg cookie', '', $name);
            $size_resource = $this->getSizeForMenuTypeId($menu_type_id, '3 Cookies');
            if ($size_resource->insert_id) {
                $size_resource->priority = 100;
                $size_resource->save();
            }
        } else if (substr_count(strtolower($name),' sm cookie') > 0) {
            $brink_item['Name'] = str_ireplace(' sm cookie', '', $name);
            $size_resource = $this->getSizeForMenuTypeId($menu_type_id, '1 Cookie');
            if ($size_resource->insert_id) {
                $size_resource->priority = 50;
                $size_resource->save();
            }
        } else if ($menu_type_name == 'Catering Express Lunches') {
            $size_name = ($name == "Express Lunch Reg") ? "Regular" : "Small";
            $size_resource = $this->getSizeForMenuTypeId($menu_type_id,$size_name);
        } else if (substr_count(strtolower($name),' sm') > 0 && substr_count(strtolower($menu_type_name),'catering') > 0) {
            $brink_item['Name'] = str_ireplace(' sm','',$name);
            $size_resource = $this->getSizeForMenuTypeId($menu_type_id,'Small Platter');
        } else if (substr_count(strtolower($name),' reg') > 0 && substr_count(strtolower($menu_type_name),'catering') > 0) {
            $brink_item['Name'] = str_ireplace(' reg','',$name);
            $size_resource = $this->getSizeForMenuTypeId($menu_type_id,'Regular Platter');
        } else {
            $size_resource = $this->getSizeForMenuTypeId($menu_type_id,'one size');
        }
        return $size_resource;
    }

    function doSizeLogicForModifierGroup(&$brink_modifier_group,$menu_type_id)
    {
        $group_name = $brink_modifier_group['Name'];
    }

    function getRemoteData($merchant_resource)
    {
        return $this->importRemoteData($merchant_resource);
    }

    function importRemoteData($merchant_resource)
    {
        if (isset($merchant_resource->merchant_id)) {
            $merchant_id  = $merchant_resource->merchant_id;
        } else {
            throw new Exception("Cannot complete request. No merchant loaded.");
        }

        $method = "GetDestinations";
        $response0 = $this->curlIt($method,$merchant_resource);
        myerror_log($response0);

        $method = "GetDayParts";
        $response0 = $this->curlIt($method,$merchant_resource);
        myerror_log($response0);

        $method = "GetModifierGroups";
        $response = $this->curlIt($method,$merchant_resource);
        if (getProperty("write_brink_file") == 'true') { $this->writeFile('brink_modifier_groups.xml',$response); }
        $modifier_group_list = $this->parseXMLResponseIntoArray($response,$method);
        $modifier_group_list_hash = createHashmapFromArrayOfArraysByFieldName($modifier_group_list['ModifierGroup'],'Id');
        //$modifier_group_list_hash_by_name = createHashmapFromArrayOfArraysByFieldName($modifier_group_list['ModifierGroup'],'Name');

        $method = "GetItems";
        $response2 = $this->curlIt($method,$merchant_resource);
        if (getProperty("write_brink_file") == 'true') { $this->writeFile('brink_items.xml',$response2); }
        $item_list = $this->parseXMLResponseIntoArray($response2,$method);
        $item_list_hash = createHashmapFromArrayOfArraysByFieldName($item_list['Item'],'Id');
        //$item_list_hash_by_name = createHashmapFromArrayOfArraysByFieldName($item_list['Item'],'Name');

        $method = "GetItemGroups";
        $response3 = $this->curlIt($method,$merchant_resource);
        if (getProperty("write_brink_file") == 'true') { $this->writeFile('brink_item_groups.xml',$response3); }
        $item_group_list = $this->parseXMLResponseIntoArray($response3,$method);
        //$item_group_list_hash = createHashmapFromArrayOfArraysByFieldName($item_group_list['ItemGroup'],'Name');
        //$menu_resource = $this->getPitapitBrinkMenuResource($merchant_resource->merchant_external_id);
        $menu_resource = $this->getMenuResourceForBrand($merchant_resource->brand_id);
        $menu_id = $menu_resource->menu_id;
        $menu_type_priority = 100;
        $size_priority = 1000;
        foreach ($item_group_list['ItemGroup'] as $brink_menu_type) {
            if (substr_count(strtolower($brink_menu_type['Name']),'splickit') > 0) {
                if (strtolower($brink_menu_type['IsDeleted']) == 'true') {
                    continue;
                }
                $brink_menu_type['Name'] = str_ireplace('splickit - ', '', $brink_menu_type['Name']);
            } else {
                myerror_log("Skipping: ".$brink_menu_type['Name']);
                continue;
            }
            $menu_type_name = $brink_menu_type['Name'];
            /* if ($brink_menu_type['Id'] == '4023841') {
                myerror_log("Catering Platters");
                $pq_modifier_group_resource = $this->getModifierGroupResource($menu_id,"PlatterQuantity");
                if ($pq_modifier_group_resource->insert_id) {
                    $pq_modifier_group_resource->modifier_group_name = "Quantity";
                    $pq_modifier_group_resource->modifier_description = "Quantity";
                    $pq_modifier_group_resource->modifier_type = 'Q';
                    $pq_modifier_group_resource->priority = 300;
                    $pq_modifier_group_resource->save();
                }

                $pq_modifier_item_resource = $this->getModifierItemResource($pq_modifier_group_resource->modifier_group_id,"PPQofPlatters");
                if ($pq_modifier_item_resource->insert_id) {
                    $pq_modifier_item_resource->modifier_item_name = 'Number Of Platters';
                    $pq_modifier_item_resource->modifier_item_print_name = 'Number Of Platters';
                    $pq_modifier_item_resource->modifier_item_max = 100;
                    $pq_modifier_item_resource->priority = 100;
                    $pq_modifier_item_resource->save();
                }
                $catering_platter_quantity_modifier_item_id = $pq_modifier_item_resource->modifier_item_id;
                $pq_modifier_size_resource = $this->getModifierSizeResource($pq_modifier_item_resource->modifier_item_id,0,'ppqofp-price',$merchant_id);
                $pq_modifier_size_resource->modifier_price = 0.00;
                $pq_modifier_size_resource->save();
                $catering_platter_quantity_modifier_group_id = $pq_modifier_group_resource->modifier_group_id;
            } */
            myerror_log("BRINK LOGGING:  about to loop menu type: ".$brink_menu_type['Name']);
            $menu_type_resource = $this->getMenuTypeResource($menu_id,$brink_menu_type['Id']);
            $menu_type_resource->menu_type_name = $brink_menu_type['Name'];
            $menu_type_resource->menu_type_description = $brink_menu_type['Name'];
            $menu_type_resource->cat_id = $this->getMenuTypeCatId($brink_menu_type);
            $menu_type_resource->priority = $brink_menu_type['AlternateId'];
            $menu_type_resource->save();
            $menu_type_priority = $menu_type_priority - 5;
            $menu_type_id = $menu_type_resource->menu_type_id;
            $item_priority = 200;
            foreach ($brink_menu_type['Items']['ItemGroupItem'] as $item_id) {
                $item = array();
                myerror_log("we have the item_id: ".$item_id['ItemId']);
                $brink_item = $item_list_hash[$item_id['ItemId']];
                // NEED TO PULL OUT THE SIZE HERE
                $size_resource = $this->doSizeLogic($brink_item,$menu_type_id,$menu_type_name);
                // END PULL OUT SIZE
                myerror_log("BRINK LOGGING:  about to import item: ".$brink_item['Name']);
//                if ($brink_item['Id'] == '4026328') {
//                    myerror_log("we ahve a chefs choice pallter");
//                }
                if ($size_resource->size_name == 'one size') {
                    $item_external_id = $brink_item['Id'];
                } else {
                    $item_external_id = "$menu_type_id-".strtolower(str_replace(' ','',$brink_item['Name']));
                    myerror_log("creating fake external id: $item_external_id");
                }

                $item_resource = $this->getItemResource($menu_type_id,$item_external_id);
                if (isset($item_resource->insert_id)) {
                    $item_resource->item_name = $brink_item['Name'];
                    $item_resource->item_print_name = $brink_item['Name'];
                    $item_resource->description = $this->getItemDescriptionFromItemData($brink_item);
                    $item_resource->priority = $item_priority;
                    $item_resource->save();
                    $item_priority = $item_priority - 5;
                }
                $item_size_resource = $this->getItemSizePriceResource($item_resource->item_id,$size_resource->size_id,$brink_item['Id'],$merchant_id);
                $item_size_resource->price = $this->getObjectPriceFromItemData($brink_item);
                $item_size_resource->priority = ($size_resource->size_name == 'Regular Pita') ? 2000 : $size_priority;
                $size_priority = $size_priority - 5;
                if (strtolower($brink_item['Active']) != 'true') {
                    myerror_log("we have a Brink innactive item: ".$brink_item['Name']);
                }
                $item_size_resource->active = (strtolower($brink_item['Active']) == 'true') ? 'Y' : 'N';
                if ($brink_item['IsDeleted'] == 'true') {
                    $item_size_resource->active = 'N';
                }
                $item_size_resource->save();

                $modifier_group_priority = 100;
                $groups = $brink_item['ModifierGroupIds']['int'];
                if (is_numeric($groups)) {
                    // single record that the xml screwed up make it an array
                    $brink_item['ModifierGroupIds']['int'] = array(array($groups));
                }
//                if ($brink_item['Name'] == 'Buffalo Chicken') {
//                    myerror_log("Strating buffalo chicken");
//                }
                foreach ($brink_item['ModifierGroupIds']['int'] as $modifier_group_id) {
                    $brink_modifier_group = $modifier_group_list_hash[$modifier_group_id[0]];
                    if (substr_count(strtolower($brink_modifier_group['Name']),'splickit') > 0) {
                        $brink_modifier_group['Name'] = str_ireplace('splickit - ','',$brink_modifier_group['Name']);
                        if (strtolower($brink_modifier_group['Name']) == 'combo') {
                            myerror_log("we have a combo");
                            myerror_log("we are skipping combos for now");
                            continue;
                        }
                    } else {
                        continue;
                    }

                    $modifier_group_name = $this->stripSizeNameFromModiferGroup($brink_modifier_group['Name']);
                    myerror_log("BRINK LOGGING:  about to import group: $modifier_group_name");
//                    if ($modifier_group_name == 'Pita Choice') {
//                        myerror_log("satrting pita choice for item: ".$brink_item['Name']);
//                    }
                    if ($modifier_group_name == $brink_modifier_group['Name']) {
                        $modifier_group_external_id = $brink_modifier_group['Id'];
                        $use_modifier_size = false;
                    } else {
                        $modifier_group_external_id = strtolower(str_replace(' ','',$modifier_group_name));
                        $use_modifier_size = true;
                    }

                    $modifier_group_resource = $this->getModifierGroupResource($menu_id,$modifier_group_external_id);
                    if ($modifier_group_resource->insert_id) {
                        $modifier_group_resource->modifier_group_name = $modifier_group_name;
                        $modifier_group_resource->modifier_description = $modifier_group_name;
                        // they dont send an active field so all Mod groups are active
                        $modifier_group_resource->active = "Y";
                        $modifier_group_resource->save();
                        $modifier_group_priority = $modifier_group_priority - 5;
                    }
                    $modifier_group_id = $modifier_group_resource->modifier_group_id;

                    //create modifier_items
                    $modifier_group_item_node = $brink_modifier_group['Items']['ModifierGroupItem'];
                    // ok so there is a problem with the XML parser and if there is only 1 group then it puts it into its own hash rather than an arrya of 1 so i'm hacking this
                    if (count($modifier_group_item_node) == 4 && isset($modifier_group_item_node['ItemId'])) {
                        // ok we have a single hash
                        myerror_log("single hash");
                        $modifier_group_item_node = array($modifier_group_item_node);
                    }
                    foreach ($modifier_group_item_node as $brink_modifier_item) {
                        myerror_log("we have the brink modifier item");
                        if ($modifier_item = $item_list_hash[$brink_modifier_item['ItemId']]) {
                            $brink_modifier_item['Name'] = $modifier_item['Name'];
                            myerror_log("we ahve the modifier item");
                            myerror_log("BRINK LOGGING:  about to import modifier item: ".$modifier_item['Name']);

                            if ($use_modifier_size) {
                                $modifier_item_external_id = "$modifier_group_id-".strtolower(str_replace(' ','',$brink_modifier_item['Name']));
                            } else {
                                $modifier_item_external_id = $brink_modifier_item['ItemId'];
                            }
                            $modifier_item_resource = $this->getModifierItemResource($modifier_group_id,$modifier_item_external_id);
                            //$modifier_item_resource->modifier_item_name = ($brink_modifier_group['Id'] == '4025180') ? 'Side Sauce='.$brink_modifier_item['Name'] : $brink_modifier_item['Name'];
                            $modifier_item_resource->modifier_item_name = $brink_modifier_item['Name'];
                            $modifier_item_resource->modifier_item_print_name = $brink_modifier_item['Name'];
                            $modifier_item_resource->modifier_item_max = $this->getModifierItemMax($menu_type_name,$modifier_group_name,$modifier_item_resource->modifier_item_name);
                            $modifier_item_resource->priority = 100 - intval($brink_modifier_item['Position']);
                            $modifier_item_resource->save();

                            $full_external_modifier_size_id = $brink_modifier_group['Id'].'-'.$brink_modifier_item['ItemId'];
                            //$full_external_modifier_size_id = $brink_modifier_item['ItemId'];
                            try {
                                $modifier_size_resource = $this->getModifierSizeResource($modifier_item_resource->modifier_item_id,$size_resource->size_id,$full_external_modifier_size_id,$merchant_id);
                                $modifier_size_resource->modifier_price = $this->getObjectPriceFromItemData($modifier_item);
                                $modifier_size_resource->active = $this->getActiveFlagSetting($modifier_item);
                                $modifier_size_resource->save();
                            } catch (Exception $mse) {
                                $error_message = "ERROR ON BRINK IMPORT!!! Cannot create record in data base. we may have currupt set up for the ".$brink_item['KitchenName']."  with ".$brink_modifier_group['Name'];
                                myerror_log($error_message);
                                throw new Exception($error_message);
                            }

                        }
                    }
                    // now add the Item Modifier Group Map
                    if ($imgm_resource = $this->getItemModifierGroupMap($item_resource->item_id,$modifier_group_id,$merchant_id)) {
                        $brink_modifier_group['name'] = $brink_modifier_group['DisplayName'];
                        $this->updateItemModifierGroupMapResource($imgm_resource,$brink_modifier_group,$this->getModifierGroupPriority($modifier_group_name,$modifier_group_priority));
                        if ($brink_menu_type['Id'] == '4032848' && ($brink_modifier_group['Id'] == '4034281' || $brink_modifier_group['Id'] == '4034282')) {
                            // we have an express lunch sandwich choice group so need to set price override to price of 5 sandwiches (the price of the parent item)
                            $imgm_resource->price_override = $this->getObjectPriceFromItemData($brink_item);
                            $imgm_resource->save();
                        } else if ($brink_modifier_group['Id'] == '4034277' || $brink_modifier_group['Id'] == '4034289') {
                            //setting price override for cheeses and sauces repectively
                            $imgm_resource->price_override = $modifier_size_resource->modifier_price;
                            $imgm_resource->save();
                        }
                    }
                }
                /* if ($menu_type_name == 'Catering Platters') {
                    // it should come with quantity 1 pre selected
                    $item_modifier_item_map_adapter = new ItemModifierItemMapAdapter($m);
                    $imim_data['item_id'] = $item_resource->item_id;
                    $imim_data['modifier_item_id'] = $catering_platter_quantity_modifier_item_id;
                    Resource::findOrCreateIfNotExistsByData($item_modifier_item_map_adapter,$imim_data);

                    if ($platter_item_modifier_group_map_resource = $this->getItemModifierGroupMap($item_resource->item_id,$catering_platter_quantity_modifier_group_id,$merchant_id)) {
                        $platter_item_modifier_group_map_resource->display_name = 'Quantity';
                        $platter_item_modifier_group_map_resource->min = 1;
                        $platter_item_modifier_group_map_resource->max = 100;
                        $platter_item_modifier_group_map_resource->priority = 300;
                        $platter_item_modifier_group_map_resource->save();
                    }
                } */
/*   NOT USING THIS NOW BUT WE WILL BE LATER SO LEAVING IT IN
                $ingredients = array();
                foreach ($brink_item['IngredientItemIds']['int'] as $ingredient_id) {
                    myerror_log("we ahve an ingredient_id of: ".$ingredient_id[0]);
                    $brink_ingredient = $item_list_hash[$ingredient_id[0]];
                    $ingredient['Name'] = $brink_ingredient['Name'];
                    $ingredient['external_id'] = $brink_ingredient['Id'];
                    $ingredient['SortPriority'] = $brink_ingredient['SortPriority'];
                    $ingredient['RevenueCenterId'] = $brink_ingredient['RevenueCenterId'];
                    $ingredient['Price'] = $brink_ingredient['Price'];
                    $ingredient['KitchenName'] = $brink_ingredient['KitchenName'];
                    $ingredients[] = $ingredient;
                }
                $item['ingredients'] = $ingredients;
*/
            }
        }
        $merchant_menu_data = array("merchant_id"=>$merchant_id,"merchant_menu_type"=>'pickup');
        $mmm_resource = Resource::findOrCreateIfNotExistsByData(new MerchantMenuMapAdapter($m),$merchant_menu_data);
        $mmm_resource->menu_id = $menu_id;
        $mmm_resource->save();
        $merchant_menu_data = array("merchant_id"=>$merchant_id,"merchant_menu_type"=>'delivery');
        $mmm_resource = Resource::findOrCreateIfNotExistsByData(new MerchantMenuMapAdapter($m),$merchant_menu_data);
        $mmm_resource->menu_id = $menu_id;
        $mmm_resource->save();

        return $menu_id;
    }

    function getModifierItemMax($menu_type_name,$modifier_group_name,$modifier_item_name)
    {
        $max = 1;
        if (substr_count(strtolower($menu_type_name),'catering') == 1) {
           $max = 100;
        } elseif (strtolower($modifier_group_name) == 'spreads') {
           $max = 2;
        } elseif (strtolower($modifier_group_name) == 'cheeses') {
           if (strtolower($modifier_item_name) == 'parmesan') {
               $max = 1;
           } else {
               $max = 2;
           }
        } elseif (strtolower($modifier_group_name) == 'toppings') {
           myerror_log("we are in the toppings");
        } elseif (strtolower($modifier_group_name) == 'extras') {
           if (strtolower($modifier_item_name) == 'extra bacon') {
               $max = 2;
           }
        } elseif (strtolower($modifier_group_name) == 'sauces') {
           $max = 2;
        }
        return $max;
    }

    function getModifierGroupPriority($name,$modifier_group_priority)
    {
        if ($name == 'Pita Choice') {
            $priority = 200;
        } else if ($name == "Recommended Toppings") {
            $priority = 195;
        } else if ($name == 'Toppings') {
            $priority = 170;
        } else if ($name == 'Spreads') {
            $priority = 180;
        } else if ($name == 'Cheeses') {
            $priority = 140;
        } else if ($name == 'Sauces') {
            $priority = 120;
        } else if ($name == 'Extras') {
            $priority = 190;
        } else if ($name == 'Chips') {
            $priority = 20;
        } else if ($name == 'Cookies') {
            $priority = 10;
        } else {
            $priority = $modifier_group_priority;
        }
        return $priority;
    }

    function stripSizeNameFromModiferGroup($brink_name)
    {
        $brink_name = str_replace(' - Small','',$brink_name);
        $brink_name = str_replace(' (Sm)','',$brink_name);
        $brink_name = str_replace(' - Regular','',$brink_name);
        $brink_name = str_replace(' (Reg)','',$brink_name);
        return $brink_name;
    }

    function myBrinkToArray($xml) {
        $array = (array)$xml;

        foreach ( array_slice($array, 0) as $key => $value )
        {
            if ( $value instanceof SimpleXMLElement )
            {
                $ns = $value->getNamespaces(true);
                if ($ns['b']) {
                    $value = $array[$key]->children($ns['b']);
                }
                $array[$key] = empty($value) ? NULL : $this->myBrinkToArray($value);
            } else if (is_array($value)) {
                foreach ($value as $sub_key => $sub_value) {
                    $array[$key][$sub_key] = $this->myBrinkToArray($sub_value);
                }
            }
        }
        return $array;
    }

    function parseXMLResponseIntoArray($response,$method)
    {
        $start = strpos($response,"<".$method."Result");
        $end_string = "</".$method."Result>";
        $end = strpos($response,$end_string) + strlen($end_string);
        $payload = substr($response,$start,$end-$start);
        $xml=simplexml_load_string($payload, NULL, NULL, "http://schemas.datacontract.org/2004/07/Pos.Web.Service.Data") or die("Error: Cannot create object");
        $ns = $xml->getNamespaces(true);
        $res = $xml->children($ns['a']);
        $hashmap = $this->myBrinkToArray($res);
        return $hashmap;
    }

    function curlIt($method,$merchant_resource)
    {
        $soap_action = "http://tempuri.org/ISettingsWebService/$method";
        $url = getProperty("brink_api_url_settings");
        $access_token = getProperty("brink_access_token"); //"ssYesckeWU+3jzRwuW1IyQ==";
        $location_token = $this->getLocationToken($merchant_resource);

        $curl = curl_init($url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 10);
        curl_setopt($curl, CURLOPT_VERBOSE, 0);

        $body = '<s:Envelope xmlns:s="http://schemas.xmlsoap.org/soap/envelope/"><s:Body><'.$method.' xmlns="http://tempuri.org/"><accessToken>'.$access_token.'</accessToken><locationToken>'.$location_token.'</locationToken></'.$method.'></s:Body></s:Envelope>';
        error_log("body: $body");
        error_log(" ");
        curl_setopt($curl, CURLOPT_POST, 1);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $body);
        $headers = array('Content-Type: text/xml; charset=utf-8', 'Content-Length: ' . strlen($body), "SOAPAction: $soap_action");
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        logCurl($url,'POST',$up,$headers,$body);
        $result = curl_exec($curl);
        //error_log($result);
        return $result;
    }

    function getLocationToken($merchant_resource)
    {
        if ($this->merchant_location_token) {
            return $this->merchant_location_token;
        }
        $merchant_resource = $merchant_resource == null ? $this->merchant_resource : $merchant_resource;
        $mbimr = MerchantBrinkInfoMapsAdapter::staticGetRecord(array("merchant_id"=>$merchant_resource->merchant_id),'MerchantBrinkInfoMapsAdapter');
        $location_token = $mbimr['brink_location_token'];
        myerror_logging(1,"we have the brink location token: $location_token");
        if (isNotProd() && !isLaptop()) {
            $location_token = getProperty('brink_test_location_token');
        }
        $this->merchant_location_token = $location_token;
        return $location_token;
    }

    function writeFile($file_name,$content)
    {
        if ($myfile = fopen($file_name, "w")) {
            fwrite($myfile, $content);
            fclose($myfile);
        } else {
            myerror_log("******************** COULDNT WRITE XML DUMP TO FILE $file_name ***********************");
        }
    }

    function getObjectPriceFromItemData($item_data)
    {
        return $item_data['Price'];
    }

    function setMenuId()
    {
            // TODO: Implement setMenuId() method.
    }
}
?>