<?php
class VivonetImporter extends SplickitImporter
{

    function __construct($merchant_mixed)
    {
        parent::__construct($merchant_mixed);
        if ($this->merchant_resource) {
            $this->loadMenuIdFromLoadedMerchant();
            $this->brand_id = $this->merchant_resource->brand_id;
        }
    }

    function loadMenuIdFromLoadedMerchant()
    {
        $merchant_menu_map_adapter = new MerchantMenuMapAdapter(getM());
        if ($merchant_menu_map_records = $merchant_menu_map_adapter->getRecords(["merchant_id"=>$this->merchant_resource->merchant_id])) {
            $this->menu_id = $merchant_menu_map_records[0]['menu_id'];
        }
    }

    function setMenuId()
    {
        ; // do nothing we need to set from the merchant now
    }
    
    /**
     * @desc This is overridden form parent because we are NOW using our own alpha numeric key for import url
     * @return activity_id
     * @throws Exception
     */
    function stageImport()
    {
        return $this->stageImportFromMerchantExternal($this->merchant_resource->alphanumeric_id);
    }

    function importEntireSystem()
    {
        $activities = [];

        //first get merchants from third party tables
        $sql = "SELECT DISTINCT merchant_id FROM Third_Party_Provider_Parameter_Merchant_Maps a JOIN Third_Party_Provider_Parameters b ON a.third_party_provider_parameter_id = b.id where b.third_party_provider_id = 1003 AND a.logical_delete = 'N'";
        $options[TONIC_FIND_BY_SQL] = $sql;
        $merchant_adapter = new MerchantAdapter(getM());
        if ($records = $merchant_adapter->getRecords([1=>1],$options)) {
            $number_of_vivonet_merchants = sizeof($records);
            myerror_log("we have found $number_of_vivonet_merchants vivonet merchants that we will try to stage an import for");
            foreach ($records as $record) {
                $merchant_id = $record['merchant_id'];
                // get merchant record
                $time_offset_in_seconds = 0;
                if ($merchant_resource = Resource::find($merchant_adapter,"$merchant_id")) {
                    if ($merchant_resource->active == 'N') {
                        myerror_log("Skipping vivonet import for innactive store: ".$merchant_resource->merchant_id);
                        continue;
                    }
                    if ($activity_id = ActivityHistoryAdapter::createActivity('VivonetImport', time()+60+$time_offset_in_seconds, "object=VivonetImporter;method=import;thefunctiondatastring=".$merchant_resource->alphanumeric_id, null)) {
                        myerror_log("Vivonet merchant $merchant_id has been staged for import. activity_id = $activity_id",3);
                        $time_offset_in_seconds = $time_offset_in_seconds + 10;
                        $activities[] = $activity_id;
                    } else {
                        myerror_log("VIVONET IMPORT ERROR!!!!! could not stage import for merchant_id: $merchant_id");
                    }
                } else {
                    myerror_log("VIVONET IMPORT ERROR!!!!!  could not find merchant record for merchant_id: $merchant_id");
                }
            }
        } else {
            myerror_log("VIVONET IMPORT ERROR!!!!!  could not find any vivonet merchants to stage imports for");
        }
        $number_of_staged_merchants = sizeof($activities);
        myerror_log("We staged $number_of_staged_merchants merchants for vivonet");
        return $activities;
    }

    function importVivonetRemoteData($merchant_resource)
    {
        if ($data = $this->callServiceForRemoteData($merchant_resource)) {
            $plu_list = array();
            foreach ($data as $remote_product_record) {
                $plu_list[$remote_product_record['productId']] = $remote_product_record;
            }
        } else {
            throw new Exception("MAJOR PROBLEM WITH VIVONET IMPORT");
        }
        ksort($plu_list);
        $this->imported_prices = $plu_list;

        $merchant_external = $merchant_resource->merchant_external_id;
        $file_name = "./import_logs/vivonet_$merchant_external.txt";
        // MailIt::sendErrorEmailAdam($file_name,$request->body);
        if ($myfile = fopen($file_name, "w")) {
            foreach ($plu_list as $plu_record) {
                fwrite($myfile, $plu_record['productName'].' '.$plu_record['productId'].' $'.$plu_record['pricePerUnit']."\r\n");
            }

            fclose($myfile);
        }
        myerror_log("done");
    }

    function getObjectPriceFromItemData($item_data)
    {
        return $item_data['pricePerUnit'];
    }
        
    function callServiceForRemoteData($merchant_resource)
    {
        $vivonet_service = new VivonetService(array('merchant_id'=>$merchant_resource->merchant_id));
        return $vivonet_service->pullPricesForLoadedMerchant($data);
    }

    function getRemoteData($merchant_resource)
    {
        $this->importVivonetRemoteData($merchant_resource);
    }

//    protected function saveThePriceResource($splickit_price_resource) {
//        $json = json_encode($splickit_price_resource->getDataFieldsReally());
//        myerror_log("record is now saved! (but not really!)  $json");
//        return true;
//    }
    
    /* ABSTRACT METHTODS NOT USED WITH VIVONET SINCE WE'RE JUST DOING PRICES */


    function getItemDescriptionFromItemData($item_data){}

    function getModifierGroupMinForThisIntegration($modifier_group_data){}

    function getModifierGroupMaxForThisIntegration($modifier_group_data){}

    function getActiveFlagSetting($data){}

    function getMenuTypeCatId($menu_type_data){}

}
?>