<?php
class XoikosImporter extends SplickitImporter
{
    const MENU_ID = 102710;
    const BRAND_ID = 430;
    
    var $imported_prices = array();
    var $merchant_resource;

    var $item_price_records_imported;
    var $modifier_price_records_imported;

    // menu pricing help
    const COOKIE_GROUP_ID = 335966;
    const COOKIE_ITEM_ID = 299813;
    const COOKIE_EXTERNAL_ID = "336";

    const CHIP_GROUP_ID = 335965;
    const CHIP_ITEM_ID = 299812;
    const CHIP_EXTERNAL_ID = "270";

    const GOLDBOX_ITEM_ID = 299827;

    const BOTTLED_DRINK_PLU = "386";

    var $base_cookie_price;
    var $base_chip_price;
    var $base_bottled_drink_price;


    function __construct($merchant_mixed)
    {
        parent::__construct($merchant_mixed);
        $this->create_zero_records = false;
    }
    
    function setMenuId()
    {
        $this->menu_id = self::MENU_ID;
    }

    function setBrandId()
    {
        $this->brand_id = self::BRAND_ID;
    }

    function setBrandRegExForMerchantExternalIds()
    {
        $this->brand_regex_for_merchant_external_ids = '[0-9a-zA-Z\-]+';
    }
    
    function setIMGMPriceOverride(&$imgm_resource)
    {
        if ($imgm_resource->item_id == self::CHIP_ITEM_ID) {
            $imgm_resource->price_override = $this->base_chip_price;
        } else if ($imgm_resource->item_id == self::COOKIE_ITEM_ID) {
            $imgm_resource->price_override = $this->base_cookie_price;
        } /*else if ($imgm_resource->item_id == self::GOLDBOX_ITEM_ID && $imgm_resource->modifier_group_id == self::CHIP_GROUP_ID) {
            $imgm_resource->price_override = $this->base_chip_price;
        } else if ($imgm_resource->item_id == self::GOLDBOX_ITEM_ID && $imgm_resource->modifier_group_id == self::COOKIE_GROUP_ID) {
            $imgm_resource->price_override = $this->base_cookie_price;
        }*/
    }

    function processPriceResource(&$splickit_price_resource)
    {
        if (isset($splickit_price_resource->modifier_type) && $splickit_price_resource->modifier_type == 'Q') {
            return false;
        } else {
            return parent::processPriceResource($splickit_price_resource);
        }
    }

    function callCustomMethodsOnData()
    {
        $this->setMultiItemPrices();
    }

    function setMultiItemPrices()
    {
        $this->base_cookie_price = $this->imported_prices[self::COOKIE_EXTERNAL_ID]['price'];
        $this->base_chip_price = $this->imported_prices[self::CHIP_EXTERNAL_ID]['price'];
        $this->base_bottled_drink_price = $this->imported_prices['']['price'];
    }
    
    function isCookieModifier($splickit_price_resource)
    {
        return $splickit_price_resource->modifier_group_id == self::COOKIE_GROUP_ID;
    }

    function isChipModifier($splickit_price_resource)
    {
        return $splickit_price_resource->modifier_group_id == self::CHIP_GROUP_ID;
    }

    function doCustomPriceManipulationForPriceResource(&$splickit_price_resource)
    {
        if ($this->isCookieModifier($splickit_price_resource)) {
            $splickit_price_resource->modifier_price = $this->base_cookie_price;
        } else if ($this->isChipModifier($splickit_price_resource)) {
            $splickit_price_resource->modifier_price = $this->base_chip_price;
        } else if (substr($splickit_price_resource->external_id,0,3) == self::BOTTLED_DRINK_PLU) {
            // we have a bottle of soda
            // check to see if flavor is active

            // bypassing for now
            
            $raw_plu_on_splickt_price_record = $splickit_price_resource->external_id;
            $p = explode(":",$raw_plu_on_splickt_price_record);
            if ($mlu = $p[1]) {
                if ($mlu_record = $this->imported_prices[$mlu]) {
                    myerror_log("we do have a value MLU: $mlu  ".$mlu_record['description']);
                    $this->addToRunningImport("we do have a value MLU: $mlu  ".$mlu_record['description']);
                } else {
                    myerror_log("WE DO NOT HAVE A PRICE FOR PLU: ".$p[0]."  and   $mlu");
                    $this->addToRunningImport("WE DO NOT HAVE A PRICE FOR PLU: ".$p[0]."  and   $mlu");
                    $splickit_price_resource->active = 'N';
                }
            }
        }
    }

    function isImportedItemTaxable($imported_price_record)
    {
        return $imported_price_record['taxable'] == '1';
    }

    function getRemoteData($merchant_resource)
    {
        $this->importXoikosRemoteData();
    }

    function importXoikosRemoteData()
    {
        $xml = '<?xml version="1.0" encoding="utf-8"?>
                    <soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">
                      <soap:Body>
                        <getPlusByStore xmlns="http://www.subshop.com/">
                          <store>' . $this->external_merchant_id . '</store>
                        </getPlusByStore>
                      </soap:Body>
                    </soap:Envelope>';
        myerror_log("about to make request with xml: $xml");
        $xml = cleanUpDoubleSpacesCRLFTFromString($xml);
        $soap_action = "http://www.subshop.com/getPlusByStore";
        $headers[] = 'SOAPAction: "' . $soap_action . '"';
        $response = XoikosCurl::curlIt("http://www.subshop.com/onlineordering/order.asmx", $xml, $headers);
        $raw = $response['raw_result'];
//      if (isLaptop()) {
//        $file = fopen("xoikos_raw_plu_dump.txt","w");
//        fwrite($file,$raw);
//        fclose($file);
//        $raw = file_get_contents("./unit_tests_pos_integrations/resources/xoikos_raw_plu_dump.txt");
//      } else {
//        throw new Exception("ERROR!!! laptop import uncommented but NOT ON LAPTOP");
//      }
        $this->processRawExport($raw);
    }

    function processRawExport($raw_export)
    {
        if ($start = strpos($raw_export, "<DocumentElement")) {
            $end_string = "</DocumentElement>";


            $end = strpos($raw_export, $end_string) + strlen($end_string);
            $payload = substr($raw_export, $start, $end - $start);
            $xml = simplexml_load_string($payload) or die("error: cannot create object");
            $hashmap = $this->myXMLToArray($xml);

            $plu_list = array();
            foreach ($hashmap['OnlinePLUs'] as $plu_node) {
                $plu_list[$plu_node['plu']] = $plu_node;
            }
            ksort($plu_list);
            $this->imported_prices = $plu_list;
        } else {
            throw new UnsuccessfulXoikosImportException($this->merchant_resource->merchant_id . " - " . $this->external_merchant_id);
        }
    }

    function myXMLToArray($xml) {
        $array = (array)$xml;

        foreach ( array_slice($array, 0) as $key => $value )
        {
            if ( $value instanceof SimpleXMLElement )
            {
                if ($key == "@attributes") {
                    continue;
                }
                $array[$key] = empty($value) ? NULL : $this->myXMLToArray($value);
            } else if (is_array($value)) {
                foreach ($value as $sub_key => $sub_value) {
                    $array[$key][$sub_key] = $this->myXMLToArray($sub_value);
                }
            }
        }
        return $array;
    }

    function getMerchant()
    {
        return $this->merchant_resource->getDataFieldsReally();
    }

    function getObjectPriceFromItemData($item_data)
    {
        return $item_data['price'];
    }

    // need these to make sure not abstract. we're not useing them because we're only importing prices for Goodcents

    function getItemDescriptionFromItemData($item_data) {}

    function getModifierGroupMinForThisIntegration($modifier_group_data) {}

    function getModifierGroupMaxForThisIntegration($modifier_group_data) {}

    function getActiveFlagSetting($data) {}

    function getMenuTypeCatId($menu_type_data) {}

}
class UnsuccessfulXoikosImportException extends UnsuccessfulImportException
{
    public function __construct($merchant_ids) {
        parent::__construct("We could not import the prices for this merchant: $merchant_ids", 100);
    }
}
?>