<?php
class GivexPaymentService extends StoredValuePaymentService
{
    var $info;
    var $api_key;
    var $do_nonsf = false;
    var $skin_user_id;
    var $skin_password;
    var $method;

    const SPLICKIT_ACCEPTED_PAYMENT_TYPE_ID_FOR_GIVEX = 12020;
    const BALANCE_FIELD_NAME_ON_REMOTE_DATA = 'balance';
    const BALANCE_FIELD_POSITION_ON_REMOTE_DATA = 2;
    const GIVEX_THIRD_PARTY_PROVIDER_ID = 5002;

    const USER_ID_NOT_SET_ERROR = "Error!!! The GIVEX user_id for skin is not set";
    const PASSSWORD_NOT_SET_ERROR = "Error!!! The GIVEX password for skin is not set";



    function __construct($data)
    {
        parent::__construct($data);
        $this->url = getProperty('givex_url');
        if ($data['merchant_id']) {
            $this->merchant_id = $data['merchant_id'];
        }
        $this->setInfo();
    }


    function processServiceError($payment_results)
    {

        $payment_results['status'] = 'failure';
        if (($payment_results['error_no'] == 28 ||  $payment_results['error_no'] == 52) && $this->method == 'Sale') {
            // we have a timeeout if its a redemption call then we need to call the reversal
            myerror_log("we have a timeout processing a GiveX sale. About to do a reversal of the sale");
            $results = $this->reverseCurrrentSaleTransaction();
            $payment_results['reversal_info'] = $results['result'];
            $payment_results['reversal_result'] = $results['raw_result'];
            $payment_results['response_text'] = "Timeout trying to reach the stored value server";
        }
        if ($message = $payment_results['result'][2]) {
            $payment_results['response_text'] = $message;
        } else {
            myerror_log("some unknown error processing givex stored value card");
            MailIt::sendErrorEmailAdam("uncaught stored value processing error message", "check logs to find out what to create in GivexPaymentService->processServiceError: " . logData($payment_results, 'givex return'));
        }
        return $payment_results;
    }


    function getChargedAmountFromResponse($payment_results)
    {
        // its all position for GiveX
        //GiveX doesn't send us this value so we have to get it from the instance variable
        return $this->amount;
    }

    function getCardNumberFromResponse($payment_results)
    {
        //return $payment_results['AcctNo'];
    }

    function getBalancePayload($account_number)
    {
        $this->method = 'Balance';
        $data = [];
        $data['jsonrpc'] = "2.0";
        $data['id'] = $this->getTopLevelId();
        $data['method'] = 'dc_994';
        $data['params'] = $this->getBasePayloadFields($account_number);
        return $data;
    }

    function getSalePayload($account_number,$amount)
    {
        $this->method = 'Sale';
        $data = [];
        $data['jsonrpc'] = "2.0";
        $data['id'] = $this->getTopLevelId();
        $data['method'] = 'dc_901';
        $data['params'] = $this->getBasePayloadFields($account_number);
        $data['params'][] = $amount;
        return $data;
    }

    function getReturnPayload($return_amount,$order_id)
    {
        return null;
    }

    function getCreditPayload($account_number,$amount)
    {
        $this->method = 'Credit';
        $data = [];
        $data['jsonrpc'] = "2.0";
        $data['id'] = $this->getTopLevelId();
        $data['method'] = 'dc_927';
        $data['params'] = $this->getBasePayloadFields($account_number);
        $data['params'][] = $amount;
        return $data;

    }

    function getVoidTranactionPayload($auth_reference, $transaction_id)
    {
        $this->method = 'Void';
        $data = [];
        $data['jsonrpc'] = "2.0";
        $data['id'] = $this->getTopLevelId();
        $data['method'] = 'dc_907';
        $data['params'] = $this->getBasePayloadFields();
        $data['params'][] = $this->amount;
        if ($transaction_id) {
            $data['params'][] = $transaction_id;
        }
        return $data;
    }

    function getReversalTransactionPayload($order_id)
    {
        $this->method = 'Reversal';
        $data = [];
        $data['jsonrpc'] = "2.0";
        $data['id'] = $this->getTopLevelId();
        $data['method'] = 'dc_918';
        $data['params'] = $this->getBasePayloadFields();
        $data['params'][] = $this->amount;
        return $data;

    }
    
    function getTopLevelId()
    {
        return getRawStamp().'-'.time().'-'.strtoupper(generateAlphaCode(2));
    }

    function getOrderIdOrStamp()
    {
        if ($this->order_id) {
            return $this->order_id;
        } else {
            return getRawStamp();
        }
    }

    function getBasePayloadFields($account_number)
    {
        $id = $this->getOrderIdOrStamp();
        $this->skin_user_id = $this->getUserIdForSkin();
        $this->skin_password = $this->getPasswordForSkin();
        if ($account_number == null) {
            $account_number = $this->getBillingUsersAccountNumber();
        }
        $payload = ['en',$id,$this->skin_user_id,$this->skin_password,$account_number];
        return $payload;
    }

    function addCashValueToCard($account_number, $amount)
    {
        $payload = $this->getCreditPayload($account_number,$amount);
        $payment_results = $this->curl($payload);
        $results = $this->processResults($payment_results);
        if ($results['response_code'] == 100) {
            $results['balance'] = $results['result'][3];
        }
        return $results;
    }

    function partialRefund($card_number, $amount)
    {
        $auth_trans = $this->balance_change_resource_for_stored_value_payment->cc_transaction_id;
        $s = explode(':',$auth_trans);
        $transaction_id = $s[1];
        $this->amount = $amount;
        $payload = $this->getVoidTranactionPayload(null, $transaction_id);
        $payment_results = $this->curl($payload);
        $results = $this->processResults($payment_results);
        return $results;
    }

    function voidTransaction($auth_reference, $transaction_id)
    {
        $payload = $this->getVoidTranactionPayload($auth_reference, $transaction_id);
        $payment_results = $this->curl($payload);
        $results = $this->processResults($payment_results);
        return $results;
    }

    function reverseCurrrentSaleTransaction()
    {
        $payload = $this->getReversalTransactionPayload();
        $payment_results = $this->curl($payload);
        return $payment_results;
    }


    function getRemoteCardInfoAndUpdateUserSkinStoredValueMap($user_skin_stored_value_map_resource)
    {
        $card_number = $user_skin_stored_value_map_resource->card_number;
        $remote_card_info = $this->getRemoteCardInfo($card_number);
        if ($remote_card_info['status'] == 'success') {
            $user_skin_stored_value_map_resource->balance = $remote_card_info[$this->getBalanceFieldNameOnRemoteData()];
            $user_skin_stored_value_map_resource->save();
        }
        return $remote_card_info;
    }

    function getRemoteCardInfo($account_number)
    {
        $payload = $this->getBalancePayload($account_number);
        $unprocessed_payment_results = $this->curl($payload);
        $payment_results = $this->processResults($unprocessed_payment_results);
        if ($payment_results['response_code'] == 100) {
            $payment_results['balance'] = $payment_results['result'][self::BALANCE_FIELD_POSITION_ON_REMOTE_DATA];
        }
        return $payment_results;
    }

    function processResults($payment_results)
    {
        if ($payment_results['result'][1] === '0') {
            $payment_results['response_code'] = 100;
        }
        return parent::processResults($payment_results);
    }

    function curl($payload)
    {
        if ($this->skin_password == null) {
            return $this->getDummyErrorResponse(self::PASSSWORD_NOT_SET_ERROR);
        }
        if ($this->skin_user_id == null) {
            return $this->getDummyErrorResponse(self::USER_ID_NOT_SET_ERROR);
        }
        $payment_results = $this->parseJSONResponse(GivexCurl::curlIt($this->url,$payload,null));
        return $payment_results;
    }

    function getDummyErrorResponse($message)
    {
        $payment_results = json_decode('{"jsonrpc": "2.0", "id": 0, "result": ["0", "9", "'.$message.'", "00.00"]}',true);
        return $payment_results;
    }

    function isSkinAssociatedWithThisPaymentService($skin_id)
    {
        // this needs to check for the existence of the OperatorId and password in the third_party_parameter_skin_map table
        $info = $this->getInfo();
        return (validateThatStringFieldIsSetAndIsNotNullAndIsNotEmpty($info,'password') && validateThatStringFieldIsSetAndIsNotNullAndIsNotEmpty($info,'user_id'));
    }

    function setInfo()
    {
        $this->info = getThirdPartyParametersForMerchantSkinCombination(self::GIVEX_THIRD_PARTY_PROVIDER_ID,getSkinIdForContext(),$this->merchant_id);
        return $this->info;
    }

    function getInfo()
    {
        if ($this->info) {
            return $this->info;
        } else {
            $this->setInfo();
            return $this->info;
        }
    }

    function getUserIdForSkin()
    {
        $info = $this->getInfo();
        return $info['user_id'];
    }

    function getPasswordForSkin()
    {
        $info = $this->getInfo();
        return $info['password'];
    }

    function getBalanceFieldNameOnRemoteData()
    {
        return self::BALANCE_FIELD_NAME_ON_REMOTE_DATA;
    }

    function getSplickitAcceptedPayemntTypeForPaymentService()
    {
        return self::SPLICKIT_ACCEPTED_PAYMENT_TYPE_ID_FOR_GIVEX;
    }

    function getTransactionIdFromPaymentResults($payment_results)
    {
        $transaction_id = $payment_results['result'][2];
        return $transaction_id;
    }

    function getAuthCodeFromPaymentResults($payment_results)
    {
        return 'tbd';
    }

    function getOrderId()
    {
        if ($this->order_id) {
            return $this->order_id;
        } else if ($this->balance_change_resource_for_stored_value_payment) {
            return $this->balance_change_resource_for_stored_value_payment->order_id;
        } else {
            throw new Exception("ERROR!!!!  order_id is NOT set in givex payment service");
        }
    }

    function setNoNSFSale()
    {
        $this->do_nonsf = true;
    }
}

class GivexCardException extends StoredValueCardException
{
    public function __construct($message)
    {
        parent::__construct($message, 999);
    }

}

?>