<?php
abstract class StoredValuePaymentService extends SplickitPaymentService
{
    var $balance_change_resource_for_stored_value_payment;
    var $user_skin_stored_value_map_resource; // should be part of the stored value payment service abstract class
    var $url;
    private $billing_users_account_number;

    const CONNECTON_ERROR_TO_REMOTE_SERVER = "Sorry, we are having a connection error the to remote server.";
    const USER_DOES_NOT_HAVE_A_CARD_ON_RECORD_ERROR = "User does not have a stored value card on record";
    const CARD_NUMBER_IS_INVALID_ERROR_MESSAGE = "Error! This card number does not appear to be valid, please check your entry.";
    const CARD_DOES_NOT_HAVE_ENOUGH_VALUE_TO_COVER_CHARGE_ERROR = "Sorry, your card does not have enough balance to cover this amount.";
    const MERCHANT_OR_SKIN_NOT_ASSOCIATED_WITH_PAYMENT_SERVICE_ERROR = "Sorry, this organizaton has not set up this payment service and is currently not able to accept these payments";



    function __construct($data)
    {
        parent::__construct($data);
        if ($data['user_id'] && $data['skin_id']) {
            $user_skin_stored_value_maps_adapter = new UserSkinStoredValueMapsAdapter(getM());
            $this->user_skin_stored_value_map_resource = $user_skin_stored_value_maps_adapter->getUserSkinStoredValueMapResourceForUserSkinPaymentTypeCombination($data['user_id'],$data['skin_id'],$this->getSplickitAcceptedPayemntTypeForPaymentService());
        }

    }

    function getStatusForFailedCharge()
    {
        return 'P';
    }

    function processPayment($amount)
    {
        if ($account_number = $this->getBillingUsersAccountNumber()) {
            //validate that there is balance on card
            if ($this->user_skin_stored_value_map_resource->balance < $amount) {
                myerror_log("ERROR!!! Stored value card does not have enough balance");
                return $this->processResults($this->getDummyErrorResponse(self::CARD_DOES_NOT_HAVE_ENOUGH_VALUE_TO_COVER_CHARGE_ERROR));
            }
            if ($this->info) {
                $payload = $this->getSalePayload($account_number, $amount);
                $payment_results = $this->processResults($this->curl($payload));
            } else {
                myerror_log("ERROR!!!!!!   Merchant or Skin does NOT have stored Value info");
                return $this->processResults($this->getDummyErrorResponse(self::MERCHANT_OR_SKIN_NOT_ASSOCIATED_WITH_PAYMENT_SERVICE_ERROR));
            }
        } else {
            myerror_log("Could not get local stored value record to process stored value fcard or user_id: " . $this->billing_user_id);
            return $this->processResults($this->getDummyErrorResponse(self::USER_DOES_NOT_HAVE_A_CARD_ON_RECORD_ERROR));
        }
        if ($payment_results['status'] == 'failure') {
            return $payment_results;
        }
        /******  this block is for partials which currently GiveX is not using *******/
        if ($charged_amount = $this->getChargedAmountFromResponse($payment_results)) {
            if (round($amount, 2) > round($charged_amount, 2)) {
                // we have to do a split payment
                $remainder_to_bill = $amount - $charged_amount;
                myerror_log("the remainder to bill is: " . round($remainder_to_bill, 2), 3);
                if ($remainder_to_bill < .01) {
                    // trying to account for small balances due to tax rounding. just a hunch.
                    myerror_log("We have a small roudning error so ignore and assume zero");
                } else {
                    if ($this->doesBillingUserHaveCreditCardOnFile()) {
                        $cc_payment_results = $this->processVioPaymentForRemainder($remainder_to_bill, $this->billing_user_id, $this->merchant_id);
                    } else {
                        MailIt::sendErrorEmailSupport("Major Error With Stored Value Payment", "Order_id: " . $this->order_id . ". Partial charge with NO CC on file");
                    }
                    if ($cc_payment_results['status'] == 'success') {
                        myerror_log("Success running CC for remainder of balance");
                        $this->secondary_charge_payment_results = $cc_payment_results;
                    } else {
                        if ($sp_void_results = $this->voidTransaction($payment_results['Auth_Reference'], $payment_results['Transaction_ID'])) {
                            if ($sp_void_results['response_code'] == 100) {
                                myerror_log("successful void of stored value after CC fail");
                            } else {
                                myerror_log("ERROR REFUNDING STS STORED VALUE PAYMENT! cc payment failed on split. couldn't refund the original amount back to the card");
                                MailIt::sendErrorEmailSupport("THere was an error refunding the sts payment on a split", "Order_id: " . $this->order_id . " had a problem refunding the sts stored value payment");
                            }
                        } else {
                            myerror_log("ERROR REFUNDING STS STORED VALUE PAYMENT! cc payment failed on split. couldn't refund the original amount back to the card");
                            MailIt::sendErrorEmailSupport("THere was an error refunding the sts payment on a split", "Order_id: " . $this->order_id . " had a problem refunding the sts stored value payment");
                        }
                        return $this->processResults($this->getDummyErrorResponse("Serious Error with Stored Value charge"));
                    }
                }
            }
        } else {
            myerror_log("Serious Error on stored value payment service.  Unable to get charged amount from process");
            MailIt::sendErrorEmailAdam("STORED VALUE ERROR","Unable to get charged amount from process.  Order_id: ".$this->order_id);
        }

        /****** partial charge code *******/
        return $payment_results;
    }

    function processResults($payment_results)
    {
        // if successfull set the transaction_id,processor_used,authcode
        if ($payment_results['response_code'] == 100) {
            $payment_results['status'] = 'success';
            $payment_results['transactionid'] = $this->getTransactionIdFromPaymentResults($payment_results);
            $payment_results['authcode'] = $this->getAuthCodeFromPaymentResults($payment_results);
            return $payment_results;
        } else {
            return $this->processServiceError($payment_results);
        }
    }


    function parseXMlResponse($payment_results)
    {
        //$this->curl_response = $response;
        $raw_result_as_array = array();
        if ($raw_result = $payment_results['raw_result']) {
            $raw_result_as_array = parseXMLintoHashmap($raw_result);
        }
        return array_merge($payment_results,$raw_result_as_array);
    }

    function getBillingUsersAccountNumber()
    {
        if ($this->billing_users_account_number) {
            return $this->billing_users_account_number;
        }
        if ($this->user_skin_stored_value_map_resource) {
            return $this->user_skin_stored_value_map_resource->card_number;
        }
        $billing_user_id = $this->billing_user_id;
        if ($billing_user_id < 2) {
            return null;
        }
        $user_skin_stored_value_maps_adapter = new UserSkinStoredValueMapsAdapter(getM());
        if ($this->user_skin_stored_value_map_resource = $user_skin_stored_value_maps_adapter->getUserSkinStoredValueMapResourceForUserSkinPaymentTypeCombination($billing_user_id,getSkinIdForContext(),$this->getSplickitAcceptedPayemntTypeForPaymentService())) {
            return $this->user_skin_stored_value_map_resource->card_number;
        }
    }

    function recordOrderTransactionsInBalanceChangeTable($order_resource,$payment_results)
    {
        $balance_change_adapter = new BalanceChangeAdapter(getM());
        $balance_change_resource_for_order = parent::recordOrderTransactionsInBalanceChangeTable($order_resource);
        $balance_after_order = $balance_change_resource_for_order->balance_after;
        $ending_balance_after_stored_value_charge = $balance_after_order + $this->getChargedAmountFromResponse($payment_results);
        $auth_reference_transaction_id = $this->getAuthReferenceTransactionValuesForService($payment_results);
        if ($bc_resource = $balance_change_adapter->addStoredValueRow($this->billing_user_id, $balance_after_order, $this->getChargedAmountFromResponse($payment_results), $ending_balance_after_stored_value_charge, $this->name, $order_resource->order_id, $auth_reference_transaction_id ,'payment with stored value Card')) {
            $bc_resource->card_info = $this->getCardNumberFromResponse($payment_results);
            $bc_resource->save();
            $this->balance_change_resource_for_stored_value_payment = $bc_resource;

            myerror_log("successful insert of stored value balance change row");
            if ($this->secondary_charge_payment_results) {
                $cc_bc_resource = $balance_change_adapter->addAuthorizeRow($this->billing_user_id,$ending_balance_after_stored_value_charge,$this->secondary_charge_payment_results['payment']['amount'],$this->secondary_charge_payment_results['processor_used'],$order_resource->order_id,$this->secondary_charge_payment_results['transactionid'],'PENDING');
                $ending_balance_after_cc_charge = $ending_balance_after_stored_value_charge + $cc_bc_resource->charge_amt;
                $this->setUsersBalance($ending_balance_after_cc_charge);
                $card_type = $this->secondary_charge_payment_results['payment']['credit_card_brand'];
                $card_last_four = $this->secondary_charge_payment_results['payment']['credit_card_last_four'];
                $card_info = $card_type.'-'.$card_last_four;
                $cc_bc_resource->card_info = $card_info;
                $cc_bc_resource->save();
                myerror_log("successful insert of cc authorize balance change row for second part of payment");
            } else {
                $this->setUsersBalance($ending_balance_after_stored_value_charge);
            }
        } else {
            myerror_log("ERROR!  FAILED TO ADD ROW TO BALANCE CHANGE TABLE");
            MailIt::sendErrorEMail('Error thrown in StoredValuePaymentService','ERROR*****************  FAILED TO ADD ROW TO BALANCE CHANGE TABLE, user_id = '.$order_resource->user_id.', AFTER RUNNING STORED VALUE CARD AND UPDATING BALANCE: '.$balance_change_adapter->getLastErrorText());
        }
    }

    function addCardToUserAccount($card_number,$user_id,$skin_id)
    {
        // check that card number is valid
        $response = $this->getRemoteCardInfo($card_number);
        if ($response['status'] == 'success') {
            $user_skin_stored_value_maps_adapter = new UserSkinStoredValueMapsAdapter(getM());
            if ($card_resource = $user_skin_stored_value_maps_adapter->saveCardNumber($card_number, $user_id, $skin_id, $this->getSplickitAcceptedPayemntTypeForPaymentService())) {
                $card_resource->balance = $response[$this->getBalanceFieldNameOnRemoteData()];
                $card_resource->save();
                return $card_resource;
            } else {
                $error_resource = createErrorResourceWithHttpCode("There was an internal error and we were not able to add your card. Engineering has been alerted", 500);
                return $error_resource;
            }
        } else if ($response['response_text'] == 'INVALID API KEY') {
            // *********  THIS IS STS SPECIFIC NEED TO CHANGE IT  ************* ///
            $json = json_encode($response);
            MailIt::sendErrorEmail("BAD STS CONNECTION!!!!","We are having an error connection to the STS server",$json);
            $error_resource = createErrorResourceWithHttpCode(self::CONNECTON_ERROR_TO_REMOTE_SERVER,500);
            return $error_resource;
        } else {
            $error_resource = createErrorResourceWithHttpCode(self::CARD_NUMBER_IS_INVALID_ERROR_MESSAGE,422);
            return $error_resource;
        }
    }

    function getRemoteCardInfoAndUpdateUserSkinStoredValueMap($user_skin_stored_value_map_resource)
    {
        $card_number = $user_skin_stored_value_map_resource->card_number;
        $remote_card_info = $this->getRemoteCardInfo($card_number);
        if ($remote_card_info['status'] == 'success') {
            $user_skin_stored_value_map_resource->balance = $remote_card_info[$this->getBalanceFieldNameOnRemoteData()];
            $user_skin_stored_value_map_resource->save();
        }
        return $remote_card_info;
    }

    function getAuthReferenceTransactionValuesForService($payment_results)
    {
        return $payment_results['authcode'] . ':' . $payment_results['transactionid'];
    }

    function setBillingUsersAccountNumber($account_number)
    {
        $this->billing_users_account_number = $account_number;
    }

    abstract function isSkinAssociatedWithThisPaymentService($skin_id);

    abstract function getSalePayload($account_number, $amount);

    abstract function curl($payload);

    abstract function getChargedAmountFromResponse($payment_results);

    abstract function getCardNumberFromResponse($payment_results);

    abstract function addCashValueToCard($card_number,$amount);

    abstract function partialRefund($card_number,$amount);

    abstract function voidTransaction($auth_reference,$transaction_id);

    abstract function getRemoteCardInfo($account_number);

    abstract function getBalanceFieldNameOnRemoteData();

    abstract function getSplickitAcceptedPayemntTypeForPaymentService();

    abstract function getTransactionIdFromPaymentResults($payment_results);

    abstract function getAuthCodeFromPaymentResults($payment_results);

    abstract function processServiceError($payment_results);

}

class NonAssociatedMerchantException extends Exception
{
    public function __construct($merchant_id)
    {
        parent::__construct("Non associated merchant.  merchant_id = $merchant_id", 999);
    }
}

class StoredValueCardException extends Exception
{
    public function __construct($message)
    {
        parent::__construct($message, 999);
    }

}

?>