<?php
class StsPaymentService extends StoredValuePaymentService
{
    var $skin_sts_info_maps_adapter;
    var $skin_sts_info_map;
    var $api_key;
    var $forced_full_balance_payment = false;

    const SPLICKIT_ACCEPTED_PAYMENT_TYPE_ID_FOR_STS = 12000;
    const BALANCE_FIELD_NAME_ON_REMOTE_DATA = 'Amount_Balance';

    function __construct($data)
    {
        parent::__construct($data);
        $this->url = getProperty('sts_url');
        $this->skin_sts_info_maps_adapter = new SkinStsInfoMapsAdapter(getM());
        if ($record = $this->skin_sts_info_maps_adapter->getStsInfo(getSkinIdForContext())) {
            $this->skin_sts_info_map = $record;
            $this->info = $record;
        }
    }

    function getSalePayload($account_number, $amount)
    {
        return $this->getChargeCardXML($account_number, $amount);
    }

    function processResults($payment_results)
    {
        if ($payment_results['Response_Code'] == '00') {
            $payment_results['response_code'] = 100;
        }
        return parent::processResults($payment_results);
    }


    function processServiceError($payment_results)
    {
        $payment_results['status'] = 'failure';
        if ($message = $payment_results['Response_Text']) {
            $payment_results['response_text'] = $message;
        } else {
            myerror_log("some unknown error processing STS stored value card");
            MailIt::sendErrorEmailAdam("uncaught stored value processing error message", "check logs to find out what to create in StsPaymentService->processStsError: " . logData($payment_results, 'sts return'));
        }
        return $payment_results;
    }

    function getChargedAmountFromResponse($payment_results)
    {
        if ($this->forced_full_balance_payment) {
            return $this->amount;
        } else {
            return $payment_results['Approved_Amount'];
        }

    }

    function getCardNumberFromResponse($payment_results)
    {
        return $payment_results['Card_Number'];
    }

    function addCashValueToCard($card_number, $amount)
    {
        $xml = $this->getAddCashValueToCardXML($card_number, $amount);
        $payment_results = $this->curl($xml);
        return $this->processResults($payment_results);
    }

    function partialRefund($card_number, $amount)
    {
        $xml = $this->getCreditValueToCardXML($card_number, $amount);
        $payment_results = $this->curl($xml);
        return $this->processResults($payment_results);
    }

    function voidTransaction($auth_reference, $transaction_id)
    {
        $xml = $this->getVoidTransactionXML($auth_reference, $transaction_id);
        $payment_results = $this->curl($xml);
        $results = $this->processResults($payment_results);
        return $results;
    }

    function getRemoteCardInfo($account_number)
    {
        $xml = $this->getBalanceXML($account_number);
        $payment_results = $this->curl($xml);
        return $this->processResults($payment_results);
    }


    function curl($xml)
    {
        $payment_results = $this->parseXMlResponse(StsCurl::curlIt($this->url,$xml));
        return $payment_results;
    }

    function getDummyErrorResponse($message)
    {
        $payment_results = [];
        $payment_results['Response_Code'] = '01';
        $payment_results['Response_Text'] = $message;
        return $payment_results;
    }

    function isSkinAssociatedWithThisPaymentService($skin_id)
    {
        $skin_sts_info_maps_adapter = new SkinStsInfoMapsAdapter(getM());
        return $skin_sts_info_maps_adapter->isSkinStsSkin($skin_id);
    }

    function getCreditValueToCardXML($user_account_number, $amount)
    {
        $xml = $this->getRootXMl() . '<Action_Code>08</Action_Code><Trans_Type>N</Trans_Type><POS_Entry_Mode></POS_Entry_Mode><Card_Number>' . $user_account_number . '</Card_Number><Transaction_Amount>' . $amount . '</Transaction_Amount></Request>';
        return $xml;
    }

    function getAddCashValueToCardXML($user_account_number, $amount)
    {
        $xml = $this->getRootXMl() . '<Action_Code>02</Action_Code><Trans_Type>N</Trans_Type><POS_Entry_Mode></POS_Entry_Mode><Card_Number>' . $user_account_number . '</Card_Number><Transaction_Amount>' . $amount . '</Transaction_Amount></Request>';
        return $xml;
    }

    function getActivateNewCardXML($amount)
    {
        return $this->getAddCashValueToCardXML('NewAccountRQ', $amount);
    }


    function getBalanceXML($user_account_number)
    {
        $xml = $this->getRootXMl() . '<Action_Code>05</Action_Code><Trans_Type>N</Trans_Type><POS_Entry_Mode>S</POS_Entry_Mode><Card_Number>' . $user_account_number . '</Card_Number></Request>';
        return $xml;
    }

    function getVoidTransactionXML($auth_reference, $transaction_id)
    {
        $xml = $this->getRootXMl() . '<Action_Code>11</Action_Code><Trans_Type>L</Trans_Type><POS_Entry_Mode>M</POS_Entry_Mode><Auth_Reference>' . $auth_reference . '</Auth_Reference><Transaction_ID>' . $transaction_id . '</Transaction_ID></Request>';
        return $xml;
    }

    function getChargeCardXML($user_account_number, $amount)
    {
        if ($this->doesBillingUserHaveCreditCardOnFile()) {
            //allows partial charge
            $this->forced_full_balance_payment = false;
            $xml = $this->getRootXMl() . '<Action_Code>19</Action_Code><Trans_Type>N</Trans_Type><POS_Entry_Mode>S</POS_Entry_Mode><Card_Number>' . $user_account_number . '</Card_Number><Transaction_Amount>' . $amount . '</Transaction_Amount></Request>';
        } else {
            $this->forced_full_balance_payment = true;
            $xml = $this->getRootXMl() . '<Action_Code>01</Action_Code><Trans_Type>N</Trans_Type><POS_Entry_Mode>M</POS_Entry_Mode><Card_Number>' . $user_account_number . '</Card_Number><Transaction_Amount>' . $amount . '</Transaction_Amount></Request>';
        }
        return $xml;
    }

    function getRootXMl()
    {
        $root_xml = '<Request><Merchant_Number>' . $this->getMerchantNumber() . '</Merchant_Number><Terminal_ID>' . $this->getTerminalId() . '</Terminal_ID><API_Key>' . $this->getApiKey() . '</API_Key>';
        return $root_xml;
    }

    function getTransactionIdFromPaymentResults($payment_results)
    {
        return $payment_results['Transaction_ID'];
    }

    function getAuthCodeFromPaymentResults($payment_results)
    {
        return $payment_results['Auth_Reference'];
    }

    function getMerchantNumber()
    {
        return $this->skin_sts_info_map['merchant_number'];
    }

    function getTerminalId()
    {
        return $this->skin_sts_info_map['terminal_id'];
    }

    function getApiKey()
    {
        return $this->skin_sts_info_map['api_key'];;
    }

    function getBalanceFieldNameOnRemoteData()
    {
        return self::BALANCE_FIELD_NAME_ON_REMOTE_DATA;
    }

    function getSplickitAcceptedPayemntTypeForPaymentService()
    {
        return self::SPLICKIT_ACCEPTED_PAYMENT_TYPE_ID_FOR_STS;
    }


}

class NonStsMerchantException extends NonAssociatedMerchantException
{
    public function __construct($merchant_id)
    {
        parent::__construct("Non STS merchant.  merchant_id = $merchant_id", 999);
    }
}

class STSCardException extends StoredValueCardException
{
    public function __construct($message)
    {
        parent::__construct($message, 999);
    }

}

?>