<?php
class VantivePaymentService extends StoredValuePaymentService
{
    var $info;
    var $api_key;
    var $do_nonsf = false;

    const SPLICKIT_ACCEPTED_PAYMENT_TYPE_ID_FOR_VANTIVE = 12010;
    const BALANCE_FIELD_NAME_ON_REMOTE_DATA = 'Balance';
    const VANTIVE_THIRD_PARTY_PROVIDER_ID = 5001;

    const OPERATOR_ID_NOT_SET_ERROR = "Error!!! The Vantive Operator_Id is not set";
    const PASSSWORD_NOT_SET_ERROR = "Error!!! The vantive password is not set";



    function __construct($data)
    {
        parent::__construct($data);
        $this->url = getProperty('vantive_url');
        if ($data['merchant_id']) {
            $this->merchant_id = $data['merchant_id'];
            $this->setInfo();
        }
    }


    function processServiceError($payment_results)
    {
        //THIS NEEDS TO BE FIGURED OUT FOR VANTIVE. STILL STS logic
        $payment_results['status'] = 'failure';
        if ($message = $payment_results['TextResponse']) {
            $payment_results['response_text'] = $message;
        } else {
            myerror_log("some unknown error processing Vantive stored value card");
            MailIt::sendErrorEmailAdam("uncaught stored value processing error message", "check logs to find out what to create in VantivePaymentService->processServiceError: " . logData($payment_results, 'vantive return'));
        }
        return $payment_results;
    }


    function getChargedAmountFromResponse($payment_results)
    {
        return $payment_results['Purchase'];
    }

    function getCardNumberFromResponse($payment_results)
    {
        return $payment_results['AcctNo'];
    }

    function getBalancePayload($account_number)
    {
        $payload = $this->getBasePayloadFields();
        if (isStringFieldNullOrEmptyOnArray($payload,'AcctNo')) {
            myerror_log("manually setting account number to: $account_number");
            $payload['AcctNo'] = $account_number;
        }
        $payload['InvoiceNo'] = $payload['InvoiceNo'] = "1000".time();
        $payload['TranCode'] = 'Balance';
        $this->url = $this->url.'Balance';
        return $payload;
    }

    function getSalePayload($account_number,$amount)
    {
        $payload = $this->getBasePayloadFields();
        $payload['MerchantID'] = $this->getMerchantId();
        $payload['Purchase'] = number_format($amount,2);
        if ($this->do_nonsf) {
            $payload['TranCode'] = 'NoNSFSale';
        } else {
            $payload['TranCode'] = 'Sale';
        }
        $payload['InvoiceNo'] = $this->getOrderId();
        $this->url = $this->url.'Sale';
        return $payload;
    }

    function getReturnPayload($return_amount,$order_id)
    {
        $payload = $this->getBasePayloadFields();
        $payload['MerchantID'] = $this->getMerchantId();
        $payload['Amount']['Purchase'] = number_format($return_amount,2);
        $payload['TranCode'] = 'Return';
        $payload['InvoiceNo'] = $order_id;
        //reset url incase came from failed void
        $this->url = getProperty('vantive_url');
        $this->url = $this->url.'Return';
        return $payload;
    }

    function getVoidTranactionPayload($auth_reference, $transaction_id)
    {
        $payload = $this->getBasePayloadFields();
        $payload['MerchantID'] = $this->getMerchantId();
        $payload['Amount']['Purchase'] = $this->balance_change_resource_for_stored_value_payment->charge_amt;
        $payload['TranCode'] = 'VoidSale';
        $payload['RefNo'] = $auth_reference;
        $payload['AuthCode'] = $auth_reference;
        $payload['InvoiceNo'] = $this->balance_change_resource_for_stored_value_payment->order_id;
        $this->url = $this->url.'VoidSale';
        return $payload;
    }

    function getBasePayloadFields()
    {
        $payload = [];
        $payload['IpPort'] = 9100;
        $payload['OperatorID'] = $this->getOperatorId();
        $payload['TranType'] = 'PrePaid';
        $payload['RefNo'] = getRawStamp();
        $payload['Memo'] = "MPS Transact 1.2.0.5";
        $payload['AcctNo'] = $this->getBillingUsersAccountNumber();
        return $payload;
    }


    function addCashValueToCard($card_number, $amount)
    {
        throw new Exception("BUILD addCashValueToCard");
    }

    function partialRefund($card_number, $amount)
    {
        $payload = $this->getReturnPayload($amount,$this->getOrderId());
        $payment_results = $this->curl($payload);
        $results = $this->processResults($payment_results);
        return $results;
    }

    function voidTransaction($auth_reference, $transaction_id)
    {
        $payload = $this->getVoidTranactionPayload($auth_reference, $transaction_id);
        $payment_results = $this->curl($payload);
        $results = $this->processResults($payment_results);
        return $results;
    }

    function getRemoteCardInfoAndUpdateUserSkinStoredValueMap($user_skin_stored_value_map_resource)
    {
        $card_number = $user_skin_stored_value_map_resource->card_number;
        $remote_card_info = $this->getRemoteCardInfo($card_number);
        if ($remote_card_info['status'] == 'success') {
            $user_skin_stored_value_map_resource->balance = $remote_card_info[$this->getBalanceFieldNameOnRemoteData()];
            $user_skin_stored_value_map_resource->save();
        }
        return $remote_card_info;
    }

    function getRemoteCardInfo($account_number)
    {
        if ($this->merchant_id == null) {
            // to get balance payload we need to get a merchant that is associated with this skin that participates in Vantive Stored value
            $skin_id = getSkinIdForContext();
            $skin_merchant_list = SkinMerchantMapAdapter::staticGetRecords(['skin_id'=>$skin_id],'SkinMerchantMapAdapter');
            $third_party_provider_parametera_adapter = new ThirdPartyProviderParametersAdapter(getM());
            if ($record = $third_party_provider_parametera_adapter->getRecord(['third_party_provider_id'=>5001,'name'=>'MerchantID'])) {
                $third_party_provider_parameter_id = $record['id'];
            } else {
                throw new Exception("Couldnt locate parameter id for MerchantID in Vantive payment service");
            }
            foreach ($skin_merchant_list as $skin_merchant_map) {
                $merchant_id = $skin_merchant_map['merchant_id'];
                $data['merchant_id'] = $merchant_id;
                $data['third_party_provider_parameter_id'] = $third_party_provider_parameter_id;
                $options[TONIC_FIND_BY_METADATA] = $data;
                if ($resource = Resource::find(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()),null,$options)) {
                    $this->merchant_id = $merchant_id;
                    break;
                }
            }
            if ($this->merchant_id == null) {
                throw new Exception("No merchants in skin are associated with Vantive Gift Card Program");
            }
        }
        $payload = $this->getBalancePayload($account_number);
        $payment_results = $this->curl($payload);
        return $this->processResults($payment_results);
    }

    function processResults($payment_results)
    {
        if ($payment_results['CmdStatus'] == 'Approved') {
            $payment_results['response_code'] = 100;
        }
        return parent::processResults($payment_results);
    }

    function curl($payload)
    {
        if (validateThatStringFieldIsSetAndIsNotNullAndIsNotEmpty($payload,'OperatorID')) {
            if (validateThatStringFieldIsSetAndIsNotNullAndIsNotEmpty($this->info,'password')) {
                $username_password = $this->info['MerchantID'].':'.$this->info['password'];
                $payment_results = $this->parseJSONResponse(VantiveCurl::curlIt($this->url,$payload,$username_password));
                return $payment_results;
            } else {
                myerror_log("merchant password not set error");
                return $this->getDummyErrorResponse(self::PASSSWORD_NOT_SET_ERROR);
            }
        } else {
            myerror_log("skin OperatorID not set error");
            return $this->getDummyErrorResponse(self::OPERATOR_ID_NOT_SET_ERROR);
        }
    }

    function getDummyErrorResponse($message)
    {
        $payment_results = [];
        $payment_results['CmdStatus'] = 'Failed';
        $payment_results['TextResponse'] = $message;
        return $payment_results;
    }

    function isSkinAssociatedWithThisPaymentService($skin_id)
    {
        // this needs to check for the existence of the OperatorId and password in the third_party_parameter_skin_map table
        $info = $this->getInfo();
        return (validateThatStringFieldIsSetAndIsNotNullAndIsNotEmpty($info,'password') && validateThatStringFieldIsSetAndIsNotNullAndIsNotEmpty($info,'MerchantID') && validateThatStringFieldIsSetAndIsNotNullAndIsNotEmpty($info,'OperatorID'));
    }

    function setInfo()
    {
        $this->info = getThirdPartyParametersForMerchantSkinCombination(self::VANTIVE_THIRD_PARTY_PROVIDER_ID,getSkinIdForContext(),$this->merchant_id);
        return $this->info;
    }

    function getInfo()
    {
        if ($this->info) {
            return $this->info;
        } else {
            $this->setInfo();
            return $this->info;
        }
    }

    function getMerchantId()
    {
        $info = $this->getInfo();
        return $info['MerchantID'];
    }

    function getOperatorId()
    {
        $info = $this->getInfo();
        return $info['OperatorID'];
    }

    function getBalanceFieldNameOnRemoteData()
    {
        return self::BALANCE_FIELD_NAME_ON_REMOTE_DATA;
    }

    function getSplickitAcceptedPayemntTypeForPaymentService()
    {
        return self::SPLICKIT_ACCEPTED_PAYMENT_TYPE_ID_FOR_VANTIVE;
    }

    function getTransactionIdFromPaymentResults($payment_results)
    {
        return 'none';
    }

    function getAuthCodeFromPaymentResults($payment_results)
    {
        return $payment_results['AuthCode'];
    }

    function getOrderId()
    {
        if ($this->order_id) {
            return $this->order_id;
        } else if ($this->balance_change_resource_for_stored_value_payment) {
            return $this->balance_change_resource_for_stored_value_payment->order_id;
        } else {
            throw new Exception("ERROR!!!!  order_id is NOT set in Vantive payment service");
        }
    }

    function setNoNSFSale()
    {
        $this->do_nonsf = true;
    }
}

class NonVantiveMerchantException extends NonAssociatedMerchantException
{
    public function __construct($merchant_id)
    {
        parent::__construct("Non Vantive merchant.  merchant_id = $merchant_id", 999);
    }
}

class VantiveCardException extends StoredValueCardException
{
    public function __construct($message)
    {
        parent::__construct($message, 999);
    }

}

?>