<?php
class WorldpayPaymentService extends P2PPaymentService
{
    var $info;
    var $api_key;
    var $skin_user_id;
    var $method;
    var $action;

    const SPLICKIT_ACCEPTED_PAYMENT_TYPE_ID_FOR_WORLDPAY = 13010;
    const WORLDPAY_THIRD_PARTY_PROVIDER_ID = 7001;

    const WORLDPAY_MERCHANT_ID_NOT_SET_ERROR = "Error!!! The worldpay_merchant_id is not set in third party tables";



    function __construct($data)
    {
        parent::__construct($data);
        $this->url = getProperty('worldpay_url');
        if ($data['merchant_id']) {
            $this->merchant_id = $data['merchant_id'];
        }
        $this->setInfo();
        $this->service_name = 'WorldPay';
    }

    function cancelTransaction($transaction_id)
    {
        $this->action = "cancel";
        $t = explode('-',$transaction_id);
        $status_payload = ['purchasecode_id'=>$t[0],'code_table'=>$t[1]];
        $u = explode('/',$this->url);
        $last = array_pop($u);
        $url = str_replace($last,"cancel_purchase_code.php",$this->url);
        $worldpay_status_results = $this->parseJSONResponse(WorldpayCurl::curlIt($url,$status_payload,null));
        $results = $this->processResults($worldpay_status_results);

    }

    function checkStatusOfTransactionId($transaction_id)
    {
        $t = explode('-',$transaction_id);
        return $this->retrieveStatusOfPurchaseCode($t[0],$t[1]);
    }

    function retrieveStatusOfPurchaseCode($purchase_code,$code_table)
    {
        $this->action = "status_check";
        if ($purchase_code == null) {
            myerror_log("ERROR no purchase code submitted");
            return false;
        } else if ($code_table == null) {
            myerror_log("ERROR no code_table submitted");
            return false;
        }
        $status_payload = ['purchasecode_id'=>$purchase_code,'code_table'=>$code_table];
        $u = explode('/',$this->url);
        $last = array_pop($u);
        $url = str_replace($last,"check_purchase_code.php",$this->url);
        $worldpay_status_results = $this->parseJSONResponse(WorldpayCurl::curlIt($url,$status_payload,null));
        return $this->processResults($worldpay_status_results);

    }

    function processPayment($amount)
    {
        $user_phone = $res = preg_replace("/[^0-9]/", "", $this->billing_user_resource->contact_no );
        $sale_payload = $this->getSalePayload($amount,$user_phone);
        $worldpay_payment_results = $this->parseJSONResponse(WorldpayCurl::curlIt($this->url,$sale_payload,null));
        return $this->processResults($worldpay_payment_results);
    }


    function processServiceError($payment_results)
    {

        $payment_results['status'] = 'failure';
        return $payment_results;
    }

    function getSalePayload($amount,$phone_number)
    {
        // {"purchasecode":"$2y$10$Qfn5Ct4p86GaaTA0vm5iOu.evm57p7dLgbs6x1IAALgfPkaXydrme-88888-11111-95.12-USD-55555-1001001","sale_id":"199999","phonesale":"1234567890","location_country":"US","merchant_name":"Bobs Edibles","add_tip":0}
        $this->action = 'sale';
        $purchase_code = $this->generatePurchaseCode($amount);
        $data['purchasecode'] = $purchase_code;
        $data['sale_id'] = $this->getOrderIdOrStamp();
        $data['phonesale'] = $phone_number;
        $data['location_country'] = 'USA';
        $data['merchant_name'] = isset($this->complete_order['merchant']['merchant_name']) ? $this->complete_order['merchant']['merchant_name'] : 'MaryJaneMessanger';
        $data['add_tip'] = false;
        return $data;

    }

    function generatePurchaseCode($amount)
    {
        //mxtoken-merchant_id-amount-USD
        $token = getProperty('world_pay_mxtoken');
        $world_pay_merchant_id = $this->getWorldPayMerchantId();
        $merchant_id = $this->merchant_id;
        $purchase_code = "$token-$world_pay_merchant_id-$merchant_id-$amount-USD-1-1";
        return $purchase_code;
    }

    function getReturnPayload($return_amount,$order_id)
    {

    }

    function getOrderIdOrStamp()
    {
        if ($this->order_id) {
            return $this->order_id;
        } else {
            return getRawStamp();
        }
    }


    function processResults($payment_results)
    {
        myerror_log("starting process results for action: ".$this->action." in worldpayment service: ".json_encode($payment_results));
        if ($this->action == 'status_check') {
            if ($payment_results['code_status'] == 1 || $payment_results['code_status'] == 3 || $payment_results['code_status'] == 8 || $payment_results['code_status'] == 6 ) {
                $payment_results['status'] = 'pending';
            } else if ($payment_results['code_status'] == 4) {
                $payment_results['status'] = 'completed';
            } else {
                $payment_results['status'] = 'declined';
            }
        } else if ($this->action == 'sale') {
            if ($payment_results['purchasecode_id'] > 0 && $payment_results['message_id'] == 'push-sale' && $payment_results['error'] == 2) {
                $payment_results['response_code'] = 100;
                $payment_results['status'] = 'success';
                $payment_results['transaction_id'] = $payment_results['purchasecode_id'].'-'.$payment_results['code_table'];
            }
        } else if ($this->action == 'cancel') {
            if ($payment_results['code_status'] == "0") {
                $payment_results['response_code'] = 100;
                $payment_results['status'] = 'cancelled';
            }
        }

        return $payment_results;
    }

    function curlIt($payload)
    {
        if ($this->worldpay_merchant_id == null) {
            return $this->getDummyErrorResponse(self::WORLDPAY_MERCHANT_ID_NOT_SET_ERROR);
        }
        $payment_results = $this->parseJSONResponse(WorldpayCurl::curlIt($this->url,$payload,null));
        return $payment_results;
    }


    function setInfo()
    {
        $this->info = getThirdPartyParametersForMerchantSkinCombination(self::WORLDPAY_THIRD_PARTY_PROVIDER_ID,getSkinIdForContext(),$this->merchant_id);
        return $this->info;
    }

    function getInfo()
    {
        if ($this->info) {
            return $this->info;
        } else {
            $this->setInfo();
            return $this->info;
        }
    }

    function getWorldPayMerchantId()
    {
        $info = $this->getInfo();
        if ($location_id = $info['worldpay_merchant_id']) {
            return $location_id;
        } else {
            // dont think this is being used by World pay right now so we will default
            return 3;
        }
    }

    function getSplickitAcceptedPayemntTypeForPaymentService()
    {
        return self::SPLICKIT_ACCEPTED_PAYMENT_TYPE_ID_FOR_WORLDPAY;
    }

    function getTransactionIdFromPaymentResults($payment_results)
    {

    }

    function getAuthCodeFromPaymentResults($payment_results)
    {
        return 'tbd';
    }

}

class WorldPayException extends Exception
{
    public function __construct($message)
    {
        parent::__construct($message, 999);
    }

}

?>