<?php
Class JerseyMikesController extends MessageController
{
	var $the_message;
	var $price_diff_due_to_comboes;
	
	protected $representation = '/json.xml';
	protected $format = 'K';
	
	protected $format_array = array ('K'=>'/json.xml');
	protected $format_name = 'jerseymikes';
	protected $clean_card_no;
	protected $clean_json_string;

	protected $prod_url = 'https://api.jerseymikes.com:8181/v1/orders/';
	private $test_url = 'https://dev.api.jerseymikes.com:8181/v1/orders/';
	private $jm_password;
	private $jm_username;
	private $jm_error_addr;
	
	protected $retry_delay = 1;
	protected $max_retries = 3;
	
	static $taxable_delivery_plu = "PLU52313";
	static $non_taxable_delivery_plu = "PLU52413";

	function JerseyMikesController($mt,$u,&$r,$l = 0)
	{
		parent::MessageController($mt, $u, $r,$l);	
		$this->jm_password = getProperty('jm_password');
		$this->jm_username = getProperty('jm_username');
		$this->jm_error_addr = getProperty('jm_error_addr');
	}
	
	static function staticCheckOfflineStores()
	{
		$jmc = new JerseyMikesController($mt, $u, $r);
		$jmc->checkOfflineStores();
		return true;
	}
		
	function checkOfflineStores()
	{
		myerror_log("about to check offline JM stores");
		$merchant_adapter = new MerchantAdapter($mimetypes);
		$jm_merchant_data['brand_id'] = 326;
		$jm_merchant_data['active'] = 'Y';
		$jm_merchant_data['ordering_on'] = 'N';
		$jm_merchant_data_options[TONIC_FIND_BY_METADATA] = $jm_merchant_data;
		if ($merchant_result_array = Resource::findAll($merchant_adapter,'',$jm_merchant_data_options))
		{
			myerror_log("there are ".count($merchant_result_array)." off line stores to check");
			foreach ($merchant_result_array as $offline_merchant_resource)
			{
				myerror_log("about to check status of JM store id: ".$offline_merchant_resource->merchant_external_id);
				if ($this->checkNetworkStatus($offline_merchant_resource->merchant_external_id))
				{
					$offline_merchant_resource->ordering_on = 'Y';
					if ($offline_merchant_resource->save())
					{
						MailIt::sendEmailSingleRecepientWithValidation($this->jm_error_addr, "JM store has been turned back on", "store number: ".$offline_merchant_resource->merchant_external_id, "Order140 Support", $bcc, $attachments);
						myerror_log("JM merchant ".$offline_merchant_resource->merchant_external_id." is back online!");
					} else {
						MailIt::sendErrorEmail("Couldn't set JM Store Back to Ordering ON", "merchant_id: ".$offline_merchant_resource->merchant_id."    jm_id: ".$offline_merchant_resource->merchant_external_id);
						SmsSender2::sendEngineeringAlert("Couldn't set JM Store Back to Ordering ON. merchant_id: ".$offline_merchant_resource->merchant_id);
					}
				} else {
					myerror_log("JM merchant ".$offline_merchant_resource->merchant_external_id." is still not responding, leave ordering to off");
				}
			}
		} else {
			myerror_log("there were no off line stores to check");
		}	
	}
	
	function checkNetworkStatus($store_id)
    {
    	
    	$url = "https://api.jerseymikes.com:8181/v1/stores/".$store_id."?network_status=1";
		if (isTest())
			$url = "https://dev.api.jerseymikes.com:8181/v1/stores/".$store_id."?network_status=1";
    	myerror_log("the url for the store status check is: ".$url);
	    $curl = curl_init($url);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);		
		curl_setopt($curl,CURLOPT_VERBOSE,0);
		curl_setopt($curl, CURLOPT_SSL_CIPHER_LIST,'TLSv1');	
		curl_setopt($curl, CURLOPT_TIMEOUT, 5);
		curl_setopt($curl, CURLOPT_CONNECTTIMEOUT,5);
		
		if (isProd())
			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, true);
		else
			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);	
		curl_setopt($curl, CURLOPT_USERPWD, $this->jm_username.":".$this->jm_password);
		$result = curl_exec($curl);
		myerror_log("the result from the store status check is: ".$result);
		curl_close($curl);
		if ($result_array = json_decode($result,true))
			if (isset($result_array['status']) && (strtolower($result_array['status']) == 'success'))
				return true;
		// we get here its still off line
		return false;
    	
    }
	
    public function convertOrderToJerseyMikesDataSetFromOrderId($order_id)
    {
    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	return $this->convertOrderToJerseyMikesDataSet($order);
    }
		
	public function convertOrderToJerseyMikesDataSet($order,$with_cc_data = true)
	{
		// get combo information for jersey mikes
		$complete_menu = new CompleteMenu(102433);
    	$combos = $complete_menu->getCombos(102433, 'Y', $order['merchant_id'], false);
    	
    	// get loyalty number
    	$user_resource = Resource::find(new UserAdapter($mimetypes),''.$order['user_id']);
    	
    	// get loyalty  points
    	$loyalty_balance = 0;
    	$ubp_options[TONIC_FIND_BY_METADATA] = array("user_id"=>$order['user_id'],"brand_id"=>326);
    	if ($user_brand_points_resource = Resource::find(new UserBrandPointsMapAdapter($mimetypes),'',$ubp_options)) {
			$loyalty_balance = $user_brand_points_resource->points;
			$loyalty_number = $user_brand_points_resource->loyalty_number;
		}

		
    	//will need to check for gifts here
    	$billing_user_id = $order['user_id'];
    	
    	$loyalty_points_used = 0;
    	// cant use this until we have sychronos ordering and billing
//    	if (strtolower($order['promo_code']) == 'use gift')
 //   		$billing_id = $gifter_id;
		$jm_order['jm_user_id'] = $order['user_id'];
		$jm_order['user_email'] = $order['user_email'];
		$jm_order['user_phone'] = $user_resource->contact_no;
		$jm_order['splickit_order_id'] = $order['order_id'];
		$jm_order['splickit_session_id'] = $order['stamp'];
		if ($order['user_id'] < 20000) {
			$jm_order['user_email'] = 'dummy@order140.com';
		}
		$jm_order['loyalty_number'] = $loyalty_number;
		$jm_order['loyalty_balance'] = $loyalty_balance;
		$jm_order['loyalty_points_used'] = $loyalty_points_used;
		$jm_order['store_id'] = $order['merchant_external_id'];
		if (isTest() || isLaptop() || $order['user_id'] == 3) {
			$jm_order['store_id'] = 'HD';
		} 
		$jm_order['first_name'] = $order['first_name'];
		$jm_order['last_name'] = $order['last_name'];
		$jm_order['pickup_time_stamp'] = $order['pickup_dt_tm'];
		$jm_order['local_pickup_date_time'] = $order['pickup_date_time2'];
		$jm_order['taxable'] = $order['order_amt'];
		$jm_order['non_taxable'] = "0.00";
		$jm_order['tax'] = $order['total_tax_amt'];
		$jm_order['additional_tax'] = "0.00";
		$note = $order['note'];
		if (strpos($note, 'auth_data=') > 0)
		{
			$position = strpos($note, 'auth_data=');
			$data_string = substr($note, $position);
			$note = substr($note, 0,$position);
			$s = explode('=', $data_string);
			$auth_data = json_decode($s[1],true);
			$jm_order['catering_crm'] = $auth_data;
		}
		if ($order['user_id'] < 20000 && $order['user_id'] > 99) {
			$note = 'TEST ORDER DO NOT MAKE. '.$note;
		}
		$jm_order['note'] = $note;
		
		$merchant = $order['merchant'];
		$merchant_external_id = $merchant['merchant_external_id'];		
		
		//now check for delivery
		$delivery_tax = 0.00;
		$total_non_taxable = 0.00;
		if ($order_delivery_info_resource = $order['delivery_info'])
		{
			$order_delivery_info = $order_delivery_info_resource->getDataFieldsReally();
			$jm_order['delivery_order'] = 'Yes';
			$jm_order['delivery_street1'] = $order_delivery_info['address1'];
			$jm_order['delivery_street2'] = $order_delivery_info['address2'];
			$jm_order['delivery_city'] = $order_delivery_info['city'];
			$jm_order['delivery_zip'] = $order_delivery_info['zip'];
			$jm_order['delivery_instructions'] = $order_delivery_info['instructions'];
			$jm_order['delivery_phone_number'] = $order_delivery_info['phone_no'];
			
			//determine delivery
			$delivery_tax_record = $this->getDeliveryTaxRecordForStore($merchant['merchant_id']);
			$delivery_tax_rate = $delivery_tax_record['rate'];
			$is_delivery_taxable = $delivery_tax_record['is_taxable'];
			$delivery_tax = ($delivery_tax_rate/100) * $order['delivery_amt'];
			$delivery_price = $order['delivery_amt'];
			
		}

			//first get tax rates
			$tax_adapter = new TaxAdapter($mimetypes);
			$tax_options[TONIC_FIND_BY_METADATA]['merchant_id'] = $merchant['merchant_id'];
			if ($tax_resources = Resource::findAll($tax_adapter,'',$tax_options))
			{
				$base_tax_rate = 0.0;
				$total_additional_tax_rate = 0.0;
				foreach ($tax_resources as $tax_resource)
				{
					if ($tax_resource->tax_group == 1)
						$base_tax_rate = $base_tax_rate + $tax_resource->rate;
					else if ($tax_resource->tax_group == 2)
						$total_additional_tax_rate = $total_additional_tax_rate + $tax_resource->rate;
					
				}
				//now do the correction for the chicago stores.
				if ($merchant_external_id == 27014 || $merchant_external_id == 27025)
				{
					$diff_additional_tax_rate = $total_additional_tax_rate - $base_tax_rate;
					
					$new_tax_amt = $order['order_amt'] * $base_tax_rate/100;
					$new_tax_amt = number_format($new_tax_amt,2);
					//now figure out the total amout of type 2 items in the order.
					$type_2_order_amt = 0.00;
					foreach ($order['order_details'] as $order_detail)
					{
						$item_size_id = $order_detail['item_size_id'];
						$is_adapter = new ItemSizeAdapter($mimetypes);
						if ($is_resource = Resource::find($is_adapter,''.$item_size_id))
						{
							if ($is_resource->tax_group == 2)
								$type_2_order_amt = $type_2_order_amt + $order_detail['item_total_w_mods'];
						} else {
							MailIt::sendErrorEmail("SERIOUS ERROR IN JERSEY MIKES CONTROLLER", "unable to find Item record for additional tax. DATA INTEGRITY ISSUE!  item_size_id: ".$item_size_id);	
						}
					}
					$additional_tax_amt = $type_2_order_amt * $diff_additional_tax_rate/100;
					$additional_tax_amt = number_format($additional_tax_amt,2);
					//$jm_order['tax'] = $new_tax_amt;
					$jm_order['additional_tax'] = $additional_tax_amt;
				} // end chicago stores	
			} else {
				MailIt::sendErrorEmail("serious tax error in jersey mikes controller", "cant get tax rates for chicago store for sugary drink taxes");
			}		
		if (isProd() && $order['user_id'] > 19999 && $order['status'] != 'Z' && $with_cc_data)
		{
			$cc_functions = new CreditCardFunctions();
			$card_data = $cc_functions->pullCreditCardDataFromUserId($billing_user_id);
			$clean_card_no = $cc_functions->getCleanCardNumber($card_data);

			$this->clean_card_no = $clean_card_no;
			$lastfour = substr($card_data['cc_number'], -4);
			$jm_order['cc_expiration'] = $card_data['cc_exp'];
			$jm_order['cc_zipcode'] = $card_data['postal_code'];
			$jm_order['cc_number'] = $card_data['cc_number'];

		} else {
			$jm_order['cc_number'] = '4111111111111111';
			$jm_order['cc_expiration'] = '1218';
			$jm_order['cc_zipcode'] = '12345';
		}

		$items = array();
		$item_count = 1;
		foreach ($order['order_details'] as &$order_detail)
		{
			// get modifier string for dispalay for things other than chips, drinks,and deserts	
			// UGLY string matching.  SUX.
			$modifier_display_string = "";
			if ($order_detail['menu_type_name'] != 'Chips, Drinks, and Dessert') {
					$modifier_display_string = CompleteOrder::getModifierString($order_detail['order_detail_modifiers'], null);
					$modifier_display_string = substr($modifier_display_string,0,-1);
			}			
			unset($item);
			unset($components);
			$item_dependent_on_modifier_chosen = false;
			$total_price_mods = 0.00;
			$group = false;
			$item = array();
			$components = array();
			$plu = $order_detail['external_id'];
			if ($plu_list = isStringList($plu))
			{
				// we have an item that is dependent on the modifier
				$item_dependent_on_modifier_chosen = true;
			}
			
			if (substr($plu,0,3) == 'GPL')
			{
				$group = true;
				//$plu = substr($plu, 4);
			} else if (substr($plu,0,3) == 'PLU') {
				;//$plu = substr($plu,3);
			}	
				
			$item_quantity = $order_detail['quantity'];
			$single_item_raw_price = $order_detail['price'];
			$item_total = $order_detail['item_total_w_mods'];
			$single_item_total = $item_total/$item_quantity;
			
			$item = $this->createJerseyMikesItem($plu, $plu, $order_detail['item_name'], $item_quantity, $order_detail['note'], $single_item_total, $item_total, $withs, $holds, $components);
			
			if ($order_detail['points_used'] > 0)
			{
				$item['use_points'] = 'yes';
				$item['points_used'] = $order_detail['points_used'];
				$total_points_used = $total_points_used + $order_detail['points_used'];
				$single_item_raw_price = 0.00;
			}

			$with = array();
			$hold = array();
			$mod_price = 0.00;
			$first_bottled_drink = true;
			foreach ($order_detail['order_detail_added_modifiers'] as $order_detail_modifier)
			{
				myerror_logging(3, "iterating through the order_detail_added_modifiers: ".$order_detail_modifier['mod_name']);
				$material_id = $order_detail_modifier['external_id'];
				//$material_id_as_array = split(',',$material_id);
				$material_id_as_array = explode(':',$material_id);
				if ($item_dependent_on_modifier_chosen)
				{
					if ($the_new_plu = array_intersect($plu_list, $material_id_as_array))
					{
						myerror_logging(3, "we have a modifier that will change the PLU of the top level item");
						$item_dependent_on_modifier_chosen = false;
						$plu = array_pop($the_new_plu);
						$item['plu'] = $plu;
						$item['vplu'] = $plu;
					}
				} else if ($material_id_as_array[0] == 'PLU15307') {
					// bottled drink exception
						myerror_logging(3, "we are in the bottled drink exception");
						if ($first_bottled_drink)
						{
							myerror_logging(3, "first bottled drink");
							$first_bottled_drink = false;
							// we have a main bottled drink so do not move the modifier up, just use it.  create new main level from the rest
							if ($material_id_as_array[1] && substr_count($material_id_as_array[1], 'M') > 0) {
                                if ($material_id_as_array[1] != 'M8888') {
                                    $with[] = str_replace('M', '', $material_id_as_array[1]);
                                }
                            }

                            // since we're in the 'added' modifiers and Number of Bottles has a comes with record, we need to add 1 if this is the 'Number of Bottles' modifier.  the M8888 ingredient
							$quantity = $material_id_as_array[1] == 'M8888' ? $order_detail_modifier['mod_quantity'] +1 : $order_detail_modifier['mod_quantity'];
							$single_item_raw_price = $order_detail_modifier['mod_price'];
							$item['withs'] = $with;
							$item['holds'] = array();
							$item['item_name'] = $order_detail_modifier['mod_name'];
							$item['quantity'] = $quantity;
							$item['price'] = $single_item_raw_price;
							$item['item_total'] = $quantity * $single_item_raw_price;
							//$items[$item_count++] = $item;
						}
						else
						{
                            $new_item_with = array();
							myerror_logging(3, "NOT the first bottled drink so create a new first level item");
							// create new first level item since we have multiple
							if ($material_id_as_array[1] && substr_count($material_id_as_array[1], 'M') > 0) {
                                if ($material_id_as_array[1] != 'M8888') {
                                    $new_item_with[] = str_replace('M', '', $material_id_as_array[1]);
                                }
                            }

							$quantity = $order_detail_modifier['mod_quantity'];
							$single_item_raw_price = $order_detail_modifier['mod_price'];
							$bottled_drink_total = $quantity * $single_item_raw_price;
							$new_bottled_drink_item = $this->createJerseyMikesItem($plu, $plu, $order_detail_modifier['mod_name'], $quantity, '', $single_item_raw_price, $bottled_drink_total, $new_item_with, $holds, $components);
							$items[$item_count++] = $new_bottled_drink_item;
						}

						continue;

				}
				$last_object_in_material_id_list = array_pop($material_id_as_array);
				if (substr($last_object_in_material_id_list, 0, 1) == 'M') {
					$material_id = $last_object_in_material_id_list;
				} else if (strtolower(substr(trim($order_detail_modifier['modifier_group_name']),-16)) == 'steak or chicken') {
					error_log("we have a steak or chicken"); //all good
				} else {
					continue;
				}
				if (substr($material_id, 0, 1) == 'M') {
                    if ($material_id != 'M8888') {
                        $with[] = str_replace('M', '', $material_id);
                    }

				}
				$total_price_mods = $total_price_mods + $order_detail_modifier['mod_total_price'];
			}
						
 			foreach ($order_detail['order_detail_hold_it_modifiers'] as $order_detail_hold_it_modifier)
			{	
				myerror_logging(3, "iterating through the hold it modifiers: ".$order_detail_hold_it_modifier['mod_name']);
				//$material_id = intval($order_detail_hold_it_modifier['external_id']);
				$mstring = $order_detail_hold_it_modifier['external_id'];
				$material_id_as_array = explode(':',$mstring);
				$last_object_in_material_id_list = array_pop($material_id_as_array);
				if (substr($last_object_in_material_id_list, 0, 1) == 'M')
				{
					myerror_logging(3, "its an ingredient so hold it");
                    if ($last_object_in_material_id_list != 'M8888') {
                        $hold[] = str_replace('M', '', $last_object_in_material_id_list); //
                    }
				} else {
					myerror_logging(3, "NOT an ingredient so DO NOT hold it");
				}
			}
			$first_chip_cookie = true;
			foreach ($order_detail['order_detail_mealdeal'] as $meal_deal_item)
			{
				myerror_logging(3,"iterating through meal deal with: ".$meal_deal_item['mod_name']);
				unset($md_item);
				unset($md_item_with);
				$md_item_with = array();
				if (substr($meal_deal_item['external_id'],0,3) == 'PLU' || substr($meal_deal_item['external_id'],0,4) == 'GPLU' )
				{
					$split = explode(':', $meal_deal_item['external_id']);
					if ($item_dependent_on_modifier_chosen)
					{
						if ($the_new_plu = array_intersect($plu_list, $split))
						{
							myerror_logging(3,"we have a modifier that will change the PLU of the top level item");
							$item_dependent_on_modifier_chosen = false;
							$plu = array_pop($the_new_plu);
							$item['plu'] = $plu;
							$item['vplu'] = $plu;
							//now pop teh first item off the split since its the GPLU number, then we'll be left with the PLU and Material Id for the item (this is probably a drink with a lunch box)
							array_shift($split);	
						} 
					}
					
					$md_item['plu'] = $split[0];
					$md_item['vplu'] = $split[0];
					$md_item['item_name'] = $meal_deal_item['mod_name'];
					$md_item['note'] = '';
					$md_item['price'] = $meal_deal_item['mod_price'];
					$md_item['components'] = array();

					// determine if this is a Sandwich
					$is_sandwich = false;
// CHANGE_THIS  sandwich id hack
					if ($meal_deal_item['modifier_group_id'] == 328447)
						$is_sandwich = true;
					if ($group)
					{
						myerror_logging(3, "we have a group so go through group logic for the component");
						// set price to null
						$md_item['price'] = null;
						$md_item['quantity'] = $meal_deal_item['mod_quantity'];
						if ($is_sandwich)
						{
							myerror_logging(3,"we have a sandwich so attach the with and hold to this component");
							$md_item['withs'] = $with;
							$md_item['holds'] = $hold;
							//cant have more than 1 sandwich in a component so reset and then add additional below
							$additional_sandwiches_this_type = $meal_deal_item['mod_quantity'] - 1;
							$md_item['quantity'] = 1; 
						} else if ($split[1]) {
							myerror_logging(3, "its a chip cookie or drink so get the ingredient id and set it to the with of this component");
							if ($split[1] && substr_count($split[1], 'M') > 0) {
                                if ($split[1] != 'M8888') {
                                    $md_item_with[] = str_replace('M', '', $split[1]);
                                }
                            }

							$md_item['withs'] = $md_item_with;
							$md_item['holds'] = array();
							$md_item['quantity'] = 1;
						}
						myerror_logging(3,"now add the meal deal item to the components array");
						$components[] = $md_item;
						if ($is_sandwich)
						{
							while ($additional_sandwiches_this_type > 0)
							{
								$components[] = $md_item;
								$additional_sandwiches_this_type--;
							}
						}
						
					} else if ($split[0] == $plu) {
						myerror_logging(3,"not a group so determine if we need to move this item up to the top level or not");
						if ($first_chip_cookie)
						{
							// we have a main level cookie or chips so do not move the modifier up, just use it
							if ($split[1] && substr_count($split[1], 'M') > 0) {
                                if ($split[1] != 'M8888') {
                                    $with[] = str_replace('M', '', $split[1]);
                                }
                            }
							$first_chip_cookie = false;
							$quantity = $meal_deal_item['mod_quantity'];
							$item['item_name'] = $meal_deal_item['mod_name'];
							if ($quantity > 1)
							{
								$item['quantity'] = $quantity;
								$item_quantity = $meal_deal_item['mod_quantity'];
								$single_item_raw_price = $meal_deal_item['mod_price'];
							}
						}
						else
						{
							// create new first level item since we have multiple
							if ($split[1] && substr_count($split[1], 'M') > 0)
                                if ($split[1] != 'M8888') {
                                    $md_item_with[] = str_replace('M', '', $split[1]);
                                }
							$md_item['withs'] = $md_item_with;
							$md_item['holds'] = array();
							$quantity = $meal_deal_item['mod_quantity'];
							$md_item['quantity'] = $quantity;
							$md_item['item_total'] = $quantity * $meal_deal_item['mod_price'];
							//for($i=0;$i<$quantity;$i++)
								$items[$item_count++] = $md_item;							
						}
					} else {
						myerror_log("DO WE EVER GET HERE????");
						$md_item_with = array();
						if ($split[1] && substr_count($split[1], 'M') > 0) {
                            if ($split[1] != 'M8888') {
                                $md_item_with[] = str_replace('M', '', $split[1]);
                            }
                        }
						$md_item['withs'] = $md_item_with;
						$md_item['holds'] = array();
						$quantity = $meal_deal_item['mod_quantity'];
						$md_item['quantity'] = $quantity;
						$md_item['item_total'] = $quantity * $meal_deal_item['mod_price'];
						//for($i=0;$i<$quantity;$i++)
							$items[$item_count++] = $md_item;
					}
				}
			}
			if ($group)
			{
				unset($with);
				unset($hold);
			}
			
			// the cookie and brownie platter hack
				if ($plu == 'GPLU800' || $plu == 'GPLU900') {
					if ($plu == 'GPLU800')
					{
						$component_plu = "PLU11301";
						$component_name = "Cookies";
					} else {
						$component_plu = "PLU11502";
						$component_name = "Brownies";
					}
					$component = $this->createJerseyMikesItem($component_plu, $component_plu, $component_name, 12, '', 0.00, 0.00, $withs, $holds, $components);
					$components[] = $component;
				} 			

			$item['withs'] = $with;
			$item['holds'] = $hold;
			$item['components'] = $components;

			// now if the PLU is still a list, shoose the first one
			if ($item_dependent_on_modifier_chosen && $plulist2 = isStringList($plu))
			{
				$plu = $plulist2[0];
				$item['plu'] = $plu;
				$item['vplu'] = $plu;
			}
			
			// get price for this item with its mods
			// we need to redo this because meal deals have stuff removed and chips and drinks also get moved off.
			// unless! it is a lunch box!
			if ($plu == 'GPLU700' || $plu == 'GPLU500' || $plu == 'GPLU100' || $plu == 'GPLU200')
			{
				;// do nothing let original pricd stuff work
			} else if ($plu == 'PLU15307') {
				;// do nothing, let bottled drink logic work
			} else {
				$total_jm_price_for_item_with_materials = $single_item_raw_price+$total_price_mods;
				$item['price'] = $total_jm_price_for_item_with_materials;
				$item['item_total'] = $total_jm_price_for_item_with_materials * $item_quantity;
			}			
			if ($this->request->data['is_place_order'] == true) {
				$item['ingredients'] = $modifier_display_string;
			}
			if ($plu)  // the combo gets put together in the combo code below (stupid?)
				$items[$item_count++] = $item;

		}
		// now convert items hash to better items array
		// 1.  loop through the combos to see if we have any matches
		foreach ($combos as $combo)
		{
			$combo_external_ids = $combo['external_ids'];
			
			do {
				unset($order_external_ids);
				foreach ($items as $the_item)
					$order_external_ids[] = $the_item['plu'];
	
				$result = array_intersect($combo_external_ids, $order_external_ids);
				myerror_log("the result: ".$result);
				$continue_to_loop = false;
				if (sizeof($result) == sizeof($combo_external_ids))
				{
					$continue_to_loop = true;
					myerror_log("we have existence of the ".$combo['combo_name']);
					//now build the combo					
					$combo_item = $this->createJerseyMikesItem($combo['combo_external_id'], $combo['combo_external_id'], $combo['combo_name'], 1, '', $combo['price'], $combo['price'], $withs, $holds, $components);
					//pick out the components
					unset($combo_item_components);
					foreach ($items as $index=>$loop_item)
					{
						if ($combo_external_ids[0] == $loop_item['plu'])
						{
							if ($loop_item['quantity'] > 1)
							{
								$loop_item['quantity'] = 1;
								$items[$index]['quantity'] = $items[$index]['quantity'] - 1;
								$items[$index]['item_total'] = $items[$index]['quantity']*$items[$index]['price'];
								$loop_item['item_total'] = $loop_item['price'];
							} else {
								unset($items[$index]);	
							}
							$individual_combined_price_of_comboed_items = $individual_combined_price_of_comboed_items + $loop_item['price'];
							$loop_item['price'] = '0.00';
							$loop_item['item_total'] = '0.00';
							$combo_item_components[] = $loop_item;
							break;
						}
					}
					//unset($items[$index]);
					foreach ($items as $index=>$loop_item)
					{
						if ($combo_external_ids[1] == $loop_item['plu'])
						{
							if ($loop_item['quantity'] > 1)
							{
								$loop_item['quantity'] = 1;
								$items[$index]['quantity'] = $items[$index]['quantity'] - 1;
								$items[$index]['item_total'] = $items[$index]['quantity']*$items[$index]['price'];
								$loop_item['item_total'] = $loop_item['price'];
							} else {
								unset($items[$index]);	
							}
							$individual_combined_price_of_comboed_items = $individual_combined_price_of_comboed_items + $loop_item['price'];
							$loop_item['price'] = '0.00';
							$loop_item['item_total'] = '0.00';
							$combo_item_components[] = $loop_item;
							
							break;
						}
					}
					//unset($items[$index]);
					$combo_item['components'] = $combo_item_components;
					$total_new_price_of_comboed_items = $total_new_price_of_comboed_items + $combo_item['price'];
					$better_items_array[] = $combo_item;
				} 
			} while ($continue_to_loop);
			if ($individual_combined_price_of_comboed_items > $total_new_price_of_comboed_items) {
				$price_diff_due_to_comboes = $individual_combined_price_of_comboed_items - $total_new_price_of_comboed_items;
				$this->price_diff_due_to_comboes = $price_diff_due_to_comboes;
			}
		}
		foreach ($items as $final_loop_item) {
			$better_items_array[] = $final_loop_item;
		}
			
		$this->stripPLUsAndMaterials($better_items_array);
		// now recalculate the price
		$total_taxable = 0.00;
		foreach($better_items_array as $better_item)
		{
			$better_item_price = $better_item['price'] * $better_item['quantity'];
			$total_price_of_items = $total_price_of_items + $better_item_price;
			// now determin taxable
			$vplu = $better_item['vplu'];
			myerror_logging(1,"about to test plu: $vplu in the JM controller for tax");
			$ism_sql = "SELECT * FROM Item_Size_Map WHERE merchant_id = ".$merchant['merchant_id']." AND (external_id = '$vplu' OR external_id like '%:$vplu' OR external_id like '$vplu:%') AND active = 'Y' AND logical_delete = 'N' LIMIT 1";
			$ism_options[TONIC_FIND_BY_SQL] = $ism_sql;
			$ism_adapter = new ItemSizeAdapter($mimetypes);
			if ($ism_resource = Resource::find($ism_adapter,'',$ism_options))
			{
				if ($ism_resource->tax_group == 1)
					$total_taxable = $total_taxable + $better_item_price;
				else if ($ism_resource->tax_group == 2) // we need to add in the regular tax on the item.
					$total_taxable = $total_taxable + $better_item_price;
				else 
					$total_non_taxable = $total_non_taxable + $better_item_price;
			} else {
				$error_string = "unable to find item size mapping for: ".$vplu."     at merchant_id: ".$merchant['merchant_id']; 
				myerror_log($error_string);
				$jm_order['controller_error'] = $error_string;
				recordError("bad mapping for jm external id", $error_string);
				$total_taxable = $total_taxable + $better_item_price;
			}
		}
		// now add the delivery if it exists
		$pre_delivery_taxable = $total_taxable;
		if ($jm_order['delivery_order'] == 'Yes') {
			// first add it to the items
			$better_items_array[] = $this->createJerseyMikesItem($delivery_tax_record['plu'], $delivery_tax_record['vplu'], 'Delivery Charge', 1, '', $delivery_price, $delivery_price, array(), array(), array());
			if ($is_delivery_taxable) {
				$total_taxable = $total_taxable + $delivery_price;
			} else {
				$total_non_taxable = $total_non_taxable + $delivery_price;
			}
		}
		
		$jm_order['loyalty_points_used'] = $total_points_used;
		$jm_order['taxable'] = number_format($total_taxable,2);
	//	$jm_order['Taxable_Amount'] = number_format($total_taxable,2);
		$jm_order['non_taxable'] = $total_non_taxable;
		$jm_order['tax'] = number_format($pre_delivery_taxable * $base_tax_rate / 100,2) + number_format($delivery_tax,2);	
		$jm_order['items'] = $better_items_array;
		
		$this->processJMPromo($jm_order, $order);
		
		return $jm_order;
	}
	
	function processJMPromo(&$jm_order,$complete_order)
	{
		if ($jm_external_promo_id = $complete_order['promo_external_id_for_brand']) {
			$jm_order['discount_number'] = $jm_external_promo_id;
			// now the jm integration hack
			$pt1ma = new PromoType1AmtMapAdapter($mimetypes);
			$promo_details = $pt1ma->getRecord(array("promo_id"=>$complete_order['promo_id']));
			$percent_off = $promo_details['percent_off']/100;
			$taxable_discount = round($percent_off * $jm_order['taxable'],2);
			$non_taxable_discount = round($percent_off * $jm_order['non_taxable'],2);
			$tax_discount = round($percent_off * $jm_order['tax'],2);
			$additional_tax_discount = round($percent_off * $jm_order['additional_tax'],2);
			
			$jm_order['taxable'] = $jm_order['taxable'] - $taxable_discount;
			$jm_order['non_taxable'] = $jm_order['non_taxable'] - $non_taxable_discount;
			$jm_order['additional_tax'] = $jm_order['additional_tax'] - $additional_tax_discount;
			$jm_order['tax'] = $jm_order['tax'] - $tax_discount;
			$jm_order['taxable_discount'] = $taxable_discount;
			$jm_order['non_taxable_discount'] = $non_taxable_discount;
			return $jm_order;
		}
		
	}
	
	function createJerseyMikesItem($plu,$vplu,$item_name,$quantity,$note,$price,$item_total,$withs,$holds,$components)
	{
		$jm_item['plu'] = $plu;
		$jm_item['vplu'] = $vplu;
		$jm_item['item_name'] = $item_name;
		$jm_item['quantity'] = $quantity;
		$jm_item['note'] = $note;
		$jm_item['price'] = $price;
		$jm_item['item_total'] = $item_total;
		$jm_item['withs'] = ($withs) ? $withs : array();
		$jm_item['holds'] = ($holds) ? $holds : array();
		$jm_item['components'] = ($components) ? $components : array();
		return $jm_item;
	}
	
	public function stripPLUsAndMaterials(&$better_items_array)
	{
		foreach ($better_items_array as &$better_item)
		{
			$better_plu = $better_item['plu'];
			//$better_item['vplu'] = $better_plu;
			$better_plu = str_replace("GPLU", "", $better_plu);
			$better_plu = str_replace("PLU", "", $better_plu);
			$better_item['plu'] = $better_plu;
			if (sizeof($better_item['components'], $mode) > 0)
				$this->stripPLUsAndMaterials($better_item['components']);
		}	
	}
	
	public function send($message)
	{	
		$this->the_message = $message;	
		myerror_logging(1, "Jersey mikes order data: ".$this->clean_json_string);
		
		$order_info = json_decode($message);
		$url = $this->prod_url;
		if (isLaptop() || isTest() || $this->use_test_addr == true || $this->order_user['user_id'] < 100)
		{
			$url = $this->test_url;
			$test_string = 'TEST TEST ';
			$no_certificate_verify = true;
		}
		
		myerror_log("the order user id is: ".$this->order_user['user_id']);
		if ($this->order_user['user_id'] == 2)
		{
			MailIt::sendEmailMandrill("arosenthal@order140.com", $test_string."jersey mikes order info: ".$order_info->order_id, $this->clean_json_string.'  '.getStamp(), "Jersey Mikes", $bcc, $attachments);
			$this->message_resource->info = "prod tester so fake sending";
			return true;
		}
		
		$merchant_id = $this->message_resource->merchant_id;
		$merchant_resource = Resource::find(new MerchantAdapter($mimetypes),''.$merchant_id);
		myerror_log("about to curl to Jesery Mikes with url: ".$url);
		if ($curl = curl_init($url))
		{
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($curl, CURLOPT_VERBOSE, 0);
			curl_setopt($curl, CURLOPT_POST, 1);
			curl_setopt($curl, CURLOPT_POSTFIELDS,$message);
			curl_setopt($curl, CURLOPT_SSL_CIPHER_LIST,'TLSv1');	

			myerror_logging(5,"about to curl to jersey mikes with:   ".$this->jm_username.":".$this->jm_password);
			curl_setopt($curl, CURLOPT_USERPWD, $this->jm_username.":".$this->jm_password);
			//curl_setopt($curl, CURLOPT_USERPWD, "splickit:Spelunk$5marble");
			if (isLaptop() || isTest() || $no_certificate_verify) {
				curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
			}
			$headers = array('Content-Type: application/json');
			curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
			$result = curl_exec($curl);
			$return_info = curl_getinfo($curl);
			curl_close($curl);
			$clean_result = cleanUpCRLFTFromString($result);
			myerror_log("result from JerseyMikes send: ".$clean_result);
			if ($result)
			{
				$this->message_resource->response = $clean_result;
					
				$return_code = $return_info['http_code'];
				myerror_log("the http return code: ".$return_code);

				$result_array = json_decode($result,true);
				$result_return_code = $result_array['code'];

				if (strtolower($result_array['status']) == 'success')
				{
					myerror_log("we have a successfull JM order send");
					if (getProperty("jersey_mikes_ordering_on") == 'false' && $this->order_user['user_id'] > 19999)		
						setProperty("jersey_mikes_ordering_on", 'true');
					if ($merchant_resource->order_on = 'N' && $this->order_user['user_id'] > 19999)		
					{
						$merchant_resource->ordering_on = 'Y';
						$merchant_resource->save();
					}
					$this->message_resource->message_text = $this->clean_json_string;
					
					return true;
				}
				$this->message_resource->message_text = 'nullit';
				$error_message = $result_array['message'];
				myerror_log("Jersey Mikes Error Message: ".$error_message);
				if ($result_array['code'] == 503)
				{
					myerror_log("WE have a JM STore oFF LINE.  set store to inneactive");
					
					if ($merchant_resource)
					{
						MailIt::sendEmailSingleRecepientWithValidation($this->jm_error_addr, "Jersey Mikes Store ".$merchant_resource->merchant_external_id." appears down", $error_message."We are setting this store to innactive: ".$merchant_resource->merchant_external_id, "Order140 Support", $bcc, $attachments);
						if (getProperty('jersey_mikes_ordering_on') == 'true' && $merchant_resource->ordering_on == 'Y')
						{
							logError($m2,"Offline Jersey Mikes Store ".$merchant_resource->merchant_external_id, "Setting store to innactive. merchant_id:".$merchant_resource->merchant_id);
							$merchant_resource->ordering_on = 'N';
							$merchant_resource->save();
						}
					} else {
						MailIt::sendErrorEmailSupport('Unable to auto shut down Jersey Mikes Store. Please do it manually NOW!', "Set ordering_on value to 'N' for merchant_id = ".$merchant_id);
					}					
				} else {
					MailIt::sendEmailSingleRecepientWithValidation($this->jm_error_addr, "JERSEY MIKES ORDER FAILURE!", $error_message, 'Splickit Server', $bcc, $attachments);
					MailIt::sendErrorEmailAdam("JERSEY MIKES ORDER FAILURE!", $error_message);
				}
				$this->message_resource->tries = 100;
				throw new Exception("JerseyMikes Sending Error: ".$result_array['ErrorMessage'], 100);				
			} else {
				//no result from jersey mikes, we need to shut down this store
				$error_message = "Timeout trying to reach JM store: ".$merchant_resource->merchant_external_id."  Shutting down ordering for this store.";
				myerror_log("WE have a JM STore oFF LINE.  Settting store to ordering=off.");
				if ($merchant_resource)
				{
					MailIt::sendEmailSingleRecepientWithValidation($this->jm_error_addr, "Jersey Mikes Store ".$merchant_resource->merchant_external_id." appears down", "We are setting this store to innactive: ".$merchant_resource->merchant_external_id, "Order140 Support", $bcc, $attachments);
					if (getProperty('jersey_mikes_ordering_on') == 'true' && $merchant_resource->ordering_on == 'Y')
					{
						logError($m2,"Offline Jersey Mikes Store ".$merchant_resource->merchant_external_id, "Setting store to innactive, merchant_id:".$merchant_resource->merchant_id);
						$merchant_resource->ordering_on = 'N';
						$merchant_resource->save();
					}
				} else {
					MailIt::sendErrorEmailSupport('Unable to auto shut down Jersey Mikes Store. Please do it manually NOW!', "Set ordering_on value to 'N' for merchant_id = ".$merchant_id);
				}					
				
				$this->message_resource->message_text = 'nullit';
				throw new Exception("JerseyMikes Sending Error: Timeout trying to reach JM store.", 100);					
			}
		}
		$this->message_resource->message_text = 'nullit';
		// if we get here we had a problem connecting to JM
		$error_message = "Unable to reach jersey mikes server!  Will retry in 1 minute. ".getStamp();
		if ($this->message_resource->tries == 3)
		{
			setProperty("jersey_mikes_ordering_on", "false");
			$error_message = "Unable to reach jersey mikes server!  Setting jersey_mikes_ordering_on property value to false";
			throw new Exception("Unable to reach jersey mikes server after 3 attempts!  Shutting down Jersey Mikes ordering!", 100);
		}
		myerror_log("Error: ".$error_message);
		MailIt::sendErrorEmailSupport("JM ERROR!", $error_message);
		MailIt::sendErrorEmail("Unable to reach jersey mikes server!  Will retry in 1 minute.", "Order Id: ".$this->full_order['order_id']);
		SmsSender2::sendAlertListSMS($error_message);
		SmsSender2::sendEngineeringAlert($error_message);
		
		throw new Exception("Jersey Mikes Server Appears Down", 100);
	}
	
	function prepMessageForSending($message_resource)
	{
		$resource = $this->populateMessageData($message_resource);
		$resource->set('loaded','true');
		$resource->set('message_id',$message_resource->map_id);
		myerror_logging(3,"finishing prep message for sending logic");
		return $resource;
	}
	
	public function populateMessageData(&$message_resource)
	{
		$this->full_order = CompleteOrder::staticGetCompleteOrder($message_resource->order_id, $this->mimetypes);
		$this->merchant = $this->full_order['merchant'];
		$this->order_user = $this->full_order['user'];
		$jm_order = $this->convertOrderToJerseyMikesDataSet($this->full_order);
		try {
			$json_string = json_encode($jm_order);
		} catch (Exception $e) {
			myerror_log("error encoding ".$e->getMessage()); // do nothing
		}
		$jm_order['cc_number'] = $this->clean_card_no;
		$clean_json_string = json_encode($jm_order);
		myerror_log("the jersey mikes data is: ".$clean_json_string);
		$this->clean_json_string = $clean_json_string;
		$message_resource->message_text = $json_string;
		$message_resource->_representation = $this->static_message_template;
		return $message_resource;	
				
	}

	public function getOrderById($order_id)
	{
		
		myerror_logging(2,"starting JerseyMikesController->get order by id");
		$this->full_order = CompleteOrder::staticGetCompleteOrder($order_id, $this->mimetypes);
		$jm_order = $this->convertOrderToJerseyMikesDataSet($this->full_order);
		try {
			$json_string = json_encode($jm_order);
		} catch (Exception $e) {
			myerror_log("error encoding ".$e->getMessage()); // do nothing
		}
		$json_resource =& Resource::Factory(new OrderAdapter($this->mimetypes),array('json'=>$json_string));
		$json_resource->message_text = $json_string;
		$json_resource->_representation = $this->static_message_template;
		return $json_resource;	
	}	
	
	function createShorePointsAccount($phone_number)
	{
		$jersey_mikes_loyalty_controller = new JerseymikesLoyaltyController($mimetypes, $this->user, $this->request);
		$result_data = $jersey_mikes_loyalty_controller->createAccount($this->user['user_id'], $points, $phone_number);
		return $result_data;
		
	}
	
	static function setShorePointsData($loyalty_attributes)
	{
		// this doesn't exist now
		return true;
		
		if (isset($loyalty_attributes['phone_number']))
		{	
			$phone_number = $loyalty_attributes['phone_number'];
    		$phone_number = str_ireplace("-", "", $phone_number);
   			$phone_number = str_ireplace("(", "", $phone_number);
   			$phone_number = str_ireplace(")", "", $phone_number);
   			$phone_number = str_ireplace(".", "", $phone_number);
			$data['phone_number'] = $phone_number;
		}
		
		if (isset($loyalty_attributes['opted_in_to_sms']))
			$data['opted_in_to_sms'] = $loyalty_attributes['opted_in_to_sms'];
		if (isset($loyalty_attributes['opted_in_to_email']))
			$data['opted_in_to_email'] = $loyalty_attributes['opted_in_to_email'];
			
		$data_json = json_encode($data);
			
		$url = getProperty('shorepoints_url').$loyalty_number;

		myerror_log("the url for the update is: ".$url);
	    $curl = curl_init($url);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);		
		curl_setopt($curl,CURLOPT_VERBOSE,0);
		if (isLaptop() || isTest())
			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
		$username = getProperty('jm_username');	
		$password = getProperty('jm_password');
		curl_setopt($curl, CURLOPT_USERPWD, $username.":".$password);
		curl_setopt($curl, CURLOPT_POST, true);
		curl_setopt($curl, CURLOPT_POSTFIELDS, $data_json);
		curl_setopt($curl, CURLOPT_SSL_CIPHER_LIST,'TLSv1');
		myerror_log("about to curl to JM with url: ".$url);
		$result = curl_exec($curl);
		return $result;
		
	}
	
	static function getShorePointsData($loyalty_number)
	{
    	$loyalty_number = str_ireplace("-", "", $loyalty_number);
   		$loyalty_number = str_ireplace("(", "", $loyalty_number);
   		$loyalty_number = str_ireplace(")", "", $loyalty_number);
   		$loyalty_number = str_ireplace(".", "", $loyalty_number);
		$jmlc = new JerseymikesLoyaltyController($mimetypes, $user, $request);
		$response = $jmlc->getAccountInfo($loyalty_number);
		return $response;
	}
	
	function getDeliveryTaxRecordForStore($merchant_id)
	{
		$tax_adapter = new TaxAdapter($mimetypes);
		if ($record = $tax_adapter->getRecord(array('merchant_id'=>$merchant_id,'locale'=>'delivery','tax_group'=>99))) {
			myerror_logging(3,"we have aquired the jersey mikes tax delivery record");
			$record = $this->verifyTaxDataAndSetIsTaxable($record);
		} else {
			MailIt::sendErrorEmailAdam("JM DELIVERY ORDER WITH NO TAX RECORD", "setting to untaxable PLU.  merhcant_id: ".$merchant_id);
			$record = $this->createNonTaxableResponse(array());
		}
		return $record;
	}
	
	function verifyTaxDataAndSetIsTaxable($record)
	{
		
		if ($record['locale_description'] == JerseyMikesController::$non_taxable_delivery_plu) {
			if ($record['rate'] > 0.00) {
				MailIt::sendErrorEmailAdam("JM Integrity Delivery Tax integrity error","we have a JM non_taxable delivery PLU but a rate of more than 0.00.  will set to zero and contineu");
				$record = $this->createNonTaxableResponse($record);
			}
			$record['is_taxable'] = false;
		} else if ($record['locale_description'] == JerseyMikesController::$taxable_delivery_plu) {
			if ($record['rate'] == 0.00) {
				MailIt::sendErrorEmailAdam("JM Integrity Delivery Tax integrity error","we have a JM taxable delivery PLU but a rate of 0.00.  will set to non taxable and continue");
				$record = $this->createNonTaxableResponse($record);	
			}
			$record['is_taxable'] = true;
		} else {
			MailIt::sendErrorEmailAdam("JM Integrity Error Unrecognized locale_description", "we have an unrecogized local description of: ".$record['local_description']);
			$record = $this->createNonTaxableResponse($record);
		}
		$record['vplu'] = $record['locale_description'];
		$record['plu'] = substr($record['locale_description'], 3);
		return $record;
	}
	
	function createNonTaxableResponse($record)
	{
		$record['locale_description'] = $this->non_taxable_delivery_plu;
		$record['rate'] = 0.00;
		$record['is_taxable'] = false;
		return $record;
	}
}

?>