<?php
require_once 'lib/utilities/functions.inc';

Class JerseyMikesImporter extends SplickitImporter
{
	protected $jersey_mikes_menu_id = 102433; 
	protected $jersey_mikes_brand_id = 326;
	protected $jersey_mikes_skin_id = 72;
	
	private $jm_password;
	private $jm_username;
	private $jm_error_addr;
	
	protected $link_text_for_email;
	protected $force_links = false;
	protected $error_text;
	
	static $static_error_text;
	
	var $imported_merchant_ids = array();
	private $jm_price_records_from_jm_system = array();
	
	function JerseyMikesImporter()
	{
		$this->jm_password = getProperty('jm_password');
		$this->jm_username = getProperty('jm_username');
		$this->jm_error_addr = getProperty('jm_error_addr');
	}
	
	function setJerseyMikesMenuId($menu_id)
	{
		$this->jersey_mikes_menu_id = $menu_id;
	}
	
	function getJmPricesArray()
	{
		return $this->jm_price_records_from_jm_system;
	}
	
	function importAllStores($cycles = 10000)
	{
		$curl = curl_init("https://api.jerseymikes.com:8181/v1/stores/");
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);		
		if (isLaptop() || isTest())
		{
			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
		}
		curl_setopt($curl, CURLOPT_SSL_CIPHER_LIST,'TLSv1');
		curl_setopt($curl, CURLOPT_USERPWD, $this->jm_username.":".$this->jm_password);
		$result = curl_exec($curl);
		curl_close($curl);
		$result_array = json_decode($result,true);
		if ($result_array['status'] != 'success')
		{
			myerror_log("there was an error and the Jersey Mikes all store import could not run: ".$result);
			MailIt::sendErrorEmailAdam("Jersey Mikes All Store import error", "there was an error and the Jersey Mikes all store import could not run: ".$result);
			return false;
		}
		$store_list = $result_array['response'];
		$i = 0;
		$store_errors = 0;
		$sql = "SELECT merchant_id, merchant_external_id, time_zone FROM Merchant WHERE brand_id = 326 ";
		$merchant_adapter = new MerchantAdapter($mimetypes);
		$options[TONIC_FIND_BY_SQL] = $sql;
		$m_results = $merchant_adapter->select($url,$options);
		$m_list = array();
		foreach ($m_results as $this_m)
		{
			$m_list[$this_m['merchant_external_id']] = $this_m;
		}
		$already_imported_stores = array();
		for ($j=-5;$j>-9;$j--)
		{	
			myerror_log("starting to import time zone: ".$j);
			foreach ($store_list as $store)
			{
				$merchant_external_id = $store['store_number'];
				if ($already_imported_stores[$merchant_external_id])
				{
					myerror_log("we've already imported this store: $merchant_external_id, go onto the next one");
					continue;
				}
				if (isset($m_list[$merchant_external_id]))
				{
					if ($m_list[$merchant_external_id]['time_zone'] != $j)
					{
						myerror_log("Do not import this store ($merchant_external_id), its in a different time zone: ".$m_list[$merchant_external_id]['time_zone']);
						continue;
					} else {
						if (isLaptop())
						{
							$already_imported_stores[$merchant_external_id] = true;
							continue;
						}
					}
					if ($i >= $cycles)
						continue;
				} else {
					if (isProd())
						myerror_log("looks like we have a new store in the list: ".$merchant_external_id);
					else
					{
						myerror_log("we're not in prod so skip inserting new store");
						continue;
					}
				}
				myerror_log("about to import store: ".$merchant_external_id);
				$already_imported_stores[$merchant_external_id] = true;
				$result = $this->importFullStore($merchant_external_id);
				if ($result)
				{
					$i++;
					myerror_log("JM store ".$external_store_id." has been imported");
					usleep(500000); // to prevent server overload during large imports
				} else {
					myerror_log("there was an error and the store was not imported");
					$store_errors++;
				}
				
				// reset the counter
				set_time_limit(30);
			}
		}
		myerror_log("finished the import all stores call.  success: ".$i."     fails: ".$store_errors);
		MailIt::sendErrorEmailAdam("results of JM import all stores", "finished the import all stores call.  success: ".$i."     fails: ".$store_errors);
		die("finished the import all stores call.  success: ".$i."     fails: ".$store_errors);
	}
	
	static function stageImportFromRequest($request)
	{
		$jm_importer = new JerseyMikesImporter();
		$url_context = 'importjerseymikesprices';
		if ($merchant_external_id = $jm_importer->getJMExternalStoreIdFromUrl($url_context,$request->url)) {
			if ($id = $jm_importer->stageImport($merchant_external_id)) {
				$response = new Response(200,"Store id ".$merchant_external_id." has been staged for import. actid=".$id);
			} else {
				MailIt::sendErrorEmail("Problem staging Jersey Mikes Store $external_store_id for import", "Problem staging Jersey Mikes Store $external_store_id for import");
				$response = new Response(500,"There was a problem and the store was not staged for import");				
			}
		} else {
			$response = new Response(500,"No Store Id Submitted!");
		}
		return $response;
	}
	
	function stageImport($merchant_external_id)
	{
		return ActivityHistoryAdapter::createActivity('ExecuteObjectFunction', time(), "object=JerseyMikesImporter;method=importFullStoreWithLink;thefunctiondatastring=".$merchant_external_id, $activity_text);
	}
	
	function mailLinks()
	{
		if ($this->link_text_for_email)
		{
			if (isProd())
	    	{
	    		MailIt::sendEmailSingleRecepientWithValidation($this->jm_error_addr, 'Jersey Mikes Merchant Data Imported', $this->link_text_for_email, 'Order140 Support', $bcc, $attachments);
	    	} else {
	    		MailIt::sendEmailSingleRecepientWithValidation('arosenthal@order140.com', 'TEST TEST TEST Jersey Mikes Merchant Data Imported', "here are the TEST links: \r\n ".$this->link_text_for_email, 'Order140 Support', $bcc, $attachments);
	    	}
		}
	}
	
	function importFullStoreWithLink($merchant_external_id)
	{
		if ($this->importFullStore($merchant_external_id))
		{
			$this->mailLinks();
			return true;
		} else {
			return false;
		}
	}
	
	function storeImportCurl($merchant_external_id)
	{
		$url = "https://api.jerseymikes.com:8181/v1/stores/".$merchant_external_id;
		if (!isProd()) {	
			$url = "https://dev.api.jerseymikes.com:8181/v1/stores/".$merchant_external_id;
			$no_verify = true;
		}
		myerror_log("the url for the import is: ".$url);
	    $curl = curl_init($url);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);		
		curl_setopt($curl,CURLOPT_VERBOSE,0);
		curl_setopt($curl, CURLOPT_USERPWD, $this->jm_username.":".$this->jm_password);
		if ($no_verify) {
			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
		}
		$result = SplickitCurl::curlIt($curl);
		curl_close($curl);
    	myerror_log("result from JerseyMikes get store call: ".$result);
		return $result['raw_result'];
	}
	
	function importFullStore($merchant_external_id)
	{
		myerror_log("about to start the jersey mikes import");
	    $result = $this->storeImportCurl($merchant_external_id);
	    myerror_logging(5,"$result");
	    return $this->processDataPull($result,$merchant_external_id);
	}
	
	function processDataPull($raw_result,$merchant_external_id)
	{
		$menu_id = $this->jersey_mikes_menu_id;
		$result_array = json_decode($raw_result,true);
		if (strtolower($result_array['status']) == 'success')
		{
			if ($store_data = $result_array['response'])
			{
				if ($store_basic = $store_data['details']) {
					logData($store_basic, 'Store Basic');
				} else {
					$error_text .= "  ERROR!  THERE ARE NO STORE DETAILS!  ";
				}
				if ($store_hours = $store_data['hours']) {
					myerror_log("we have the store hours"); //all is good
				} else {
					$error_text .= "  ERROR!  THERE ARE NO STORE HOURS!  ";
				}
				
				if (isset($store_basic['tax_rate']) && $store_basic['tax_rate'] > -1)
				{
					$tax_rate = $store_basic['tax_rate'];
					myerror_log("we have the tax rate."); // all is good
				} else {
					$error_text .= " ERROR! THERE IS NO TAX RATE! ";
				}
				if ($store_prices = $store_data['menu']) {
					myerror_log("we have the prices");
				} else {
					$error_text .= " ERROR!  THERE ARE NO STORE PRICES!";
				}
				// *************  can get chip flavors and bottled drink flavors here too ************** //
				if ($raw_chip_drink_flavors = $store_data['chip_drink_ingredients']) {
                    myerror_log("we have the chip drink flavors");
                    foreach ($raw_chip_drink_flavors as $flavor) {
                        $chip_drink_flavors['M' . $flavor] = 1;
                    }
				} else {

					myerror_log("There were no chip drink flavors in the menu feed so we get them from the active modifiers list in the db");
                    //chips
                    $modifer_size_map_adapter = new ModifierSizeMapAdapter($m);
                    $options[TONIC_FIND_BY_SQL] = "SELECT a.external_id FROM Modifier_Size_Map a JOIN Modifier_Item b ON a.modifier_item_id = b.modifier_item_id WHERE b.modifier_group_id = 328207 AND a.merchant_id = 0 AND a.active = 'Y'";
                    $chip_records = $modifer_size_map_adapter->getRecords($data,$options);
                    foreach ($chip_records as $chip_record) {
                        $ingredient_id = str_replace("PLU11702:","",$chip_record['external_id']);
                        $chip_drink_flavors[$ingredient_id] = 1;
                    }
                    //bottled drinks
                    $options[TONIC_FIND_BY_SQL] = "SELECT a.external_id FROM Modifier_Size_Map a JOIN Modifier_Item b ON a.modifier_item_id = b.modifier_item_id WHERE b.modifier_group_id = 330278 AND a.merchant_id = 0 AND a.active = 'Y'";
                    $chip_records = $modifer_size_map_adapter->getRecords($data,$options);
                    foreach ($chip_records as $chip_record) {
                        $ingredient_id = str_replace("PLU15307:","",$chip_record['external_id']);
                        $chip_drink_flavors[$ingredient_id] = 1;
                    }
				}

				// *************  can get chip flavors and bottled drink flavors here too ************** //
			} else {
				$error_text = "ERROR! There is no store data";
			}	
			
		} else if ($error_text = $result_array['message']) {
			myerror_log("ERROR in Jersey Mikes Import: ".$error_text);
		} else {
			myerror_log("Unrecognized Error from Jersy mikes import call");
			$error_text = "Unrecognized response from Jersery Mikes import call: ".$raw_result;
		}
		
		if ($error_text)
		{
			$test = '';
			if (isTest()) {
				$test = 'TEST SERVER TEST ';
			}
			$error_message = $error_text;
			if (isProd()) {
				MailIt::sendEmailSingleRecepientWithValidation($this->jm_error_addr, $test."IMPORT ERROR! Bad Store data for import. store id: ".$merchant_external_id, $error_message, 'Order140 Support', $bcc, $attachments);
			}
			MailIt::sendErrorEmailAdam($test."IMPORT ERROR! Bad Store data for import. store id: ".$merchant_external_id, $error_message);
			$this->error_text = $error_text;
			return false;
		}
		
		$additional_tax_rate = 0.00;
		if (isset($store_basic['additional_tax']) && $store_basic['additional_tax'] > -1) {
			$additional_tax_rate = $store_basic['additional_tax'];
		}
		if ($merchant_external_id == 27014 || $merchant_external_id == 27025) {
			$total_additional = $tax_rate + $additional_tax_rate;
		} else {
			$total_additional = $tax_rate; // so bottled drinks in everywhere but downtown chicago will use the same tax rate as the rest of the items
		}
		
		// first check to see if this store exists
		$o_store_basic['merchant_external_id']=$merchant_external_id;
		$o_store_basic['brand_id'] = $this->jersey_mikes_brand_id;
		$options_store_basic[TONIC_FIND_BY_METADATA] = $o_store_basic;
		$merchant_adapter = new MerchantAdapter($mimetypes);
		if ($merchant_resource = Resource::find($merchant_adapter,'',$options_store_basic))
		{
			$new_merchant=false;
			$merchant_id = $merchant_resource->merchant_id;
			
			$merchant_resource->address1=str_replace('#', '', $store_basic['address1']);
			$merchant_resource->address2=str_replace('#', '', $store_basic['address2']);
			$merchant_resource->city=$store_basic['city'];
			$merchant_resource->state=$store_basic->State;
			$merchant_resource->zip="".$store_basic['zip'];
			
			$merchant_resource->show_tip='N';
			$merchant_resource->phone_no=$store_basic['phone'];
			if (isset($store_basic['online_ordering']) && strtolower($store_basic['online_ordering']) == 'yes') {
				$merchant_resource->active = 'Y';
			} else if (isset($store_basic['online_ordering']) && strtolower($store_basic['online_ordering']) == 'no') {
				$merchant_resource->active = 'N';
			}
			
			if (isset($store_basic['delivery']) && $store_basic['delivery'] == '1') {
				$merchant_resource->delivery = 'Y';
			} else if (isset($store_basic['delivery']) && $store_basic['delivery'] == '0') {
				$merchant_resource->delivery = 'N';
			}
								
			if (isset($store_basic['lat']) && $store_basic['lat'] != 0)
			{
				$merchant_resource->lat = $store_basic['lat'];
				$merchant_resource->lng = $store_basic['lng'];
			}
			if ($merchant_resource->save()) {
				myerror_log("we had a good JM save"); //all is good
			} else {
				myerror_log("WE HAD A SAVE ERROR importing a JM ($merchant_id)  store id: ".$merchant_external_id);
				return returnErrorResource("Error! couldn't save jersey mikes ($merchant_id) merchant: ".$merchant_resource->_adapter->getLastErrorText());
			}

			if ($tax_rate != Null)
			{
				$tax_adapter = new TaxAdapter($mimetypes);
				$tax_options[TONIC_FIND_BY_METADATA]['merchant_id'] = $merchant_id;
				if ($tax_resources = Resource::findAll($tax_adapter,'',$tax_options))
				{
					foreach ($tax_resources as $tax_resource)
					{

						if ($tax_resource->locale == 'State')
							$tax_resource->rate = $tax_rate;
						else if ($tax_resource->locale == 'City')
							$tax_resource->rate = $total_additional;
						else
							continue;
						$tax_resource->save();
					}
				} else {
					TaxAdapter::createTaxRecord($merchant_id,"State", $tax_rate,1);
					TaxAdapter::createTaxRecord($merchant_id,"City", $total_additional,2);				
				}
			}
			
		} else {
			$new_merchant=true;
			
			$merchant_data['merchant_external_id'] = $merchant_external_id;
			$merchant_data['shop_email'] = 'trish@order140.com';
			$merchant_data['name'] = 'Jersey Mikes';
			$merchant_data['brand_id'] = $this->jersey_mikes_brand_id;
			$merchant_data['display_name'] = $store_basic['city'];
			$merchant_data['address1'] = str_replace('#', '', $store_basic['address1']);
			$merchant_data['address2'] = str_replace('#', '', $store_basic['address2']);
			$merchant_data['city'] = $store_basic['city'];
			$merchant_data['zip'] = "".$store_basic['zip'];
			$merchant_data['country'] = 'US'; 

			$merchant_data['phone_no'] = $store_basic['phone'];
			if ($zip_data = ZipLookupAdapter::getZipInfo($store_basic['zip']))
			{
				$merchant_data['time_zone'] = $zip_data['time_zone_offset'];
				$merchant_data['state'] = $zip_data['state'];
			} else {
				if (substr(getRawStamp(), 0,4) == 'UUUU')
				{
					$merchant_data['time_zone'] = -5;
					$merchant_data['state'] = 'NJ';
				} else {			
					MailIt::sendEmailSingleRecepientWithValidation($this->jm_error_addr,"There were errors importing store ".$merchant_external_id, "Unknown Zip Code: ".$store_basic['zip'], "splickit import admin", $bcc, $attachments);
	    			MailIt::sendErrorEmailAdam("There were errors importing Jersey Mikes store ".$merchant_external_id, "Unknown Zip Code: ".$store_basic['zip']);
					return returnErrorResource("Error! couldn't create jersey mikes merchant: Unknown Zip Code: ".$store_basic['zip']);
				}
			}
			if (isset($store_basic['lat']) && $store_basic['lat'] != 0)
			{
				$merchant_data['lat'] = $store_basic['lat'];
				$merchant_data['lng'] = $store_basic['lng'];
			}
			
			if (isset($store_basic['delivery']) && $store_basic['delivery'] == '1') {
				$merchant_data['delivery'] = 'Y';
			} else {
				$merchant_data['delivery'] = 'N';
			}
			
			$merchant_data['show_tip'] = 'N'; 
			$merchant_data['active'] = 'Y';
			if (isset($store_basic['online_ordering']) && strtolower($store_basic['online_ordering']) == 'no')
				$merchant_data['active'] = 'N';
			
			$merchant_data['ordering_on'] = 'Y';
			$merchant_data['order_del_type'] = 'K';
			$merchant_data['lead_time'] = 15; 
			$merchant_data['immediate_message_delivery'] = 'Y';
			$merchant_resource = Resource::factory($merchant_adapter,$merchant_data);
			$merchant_resource->save();
			if ($merchant_id = $merchant_resource->merchant_id) {
				myerror_log("new JM merchant Created with id: ".$merchant_id);
			} else {
				$error_text = $merchant_adapter->getLastErrorText();
				MailIt::sendEmailSingleRecepientWithValidation($this->jm_error_addr,"There were errors importing store ".$merchant_external_id, "Server Error", "splickit import admin", $bcc, $attachments);
    			MailIt::sendErrorEmailAdam("There were errors importing Jersey Mikes store ".$merchant_external_id, $error_text);
				return returnErrorResource("Error! couldn't create jersey mikes merchant: ".$error_text);
			}
			if ($tax_rate)
			{
				TaxAdapter::createTaxRecord($merchant_id,"State", $tax_rate,1);
				TaxAdapter::createTaxRecord($merchant_id,"City", $total_additional,2);
			}
			SkinMerchantMapAdapter::createSkinMerchantMapRecord($merchant_id, $this->jersey_mikes_skin_id);
			SkinMerchantMapAdapter::createSkinMerchantMapRecord($merchant_id,1);
			MerchantMessageMapAdapter::createMerchantMessageMap($merchant_id, "K", "JerseyMikes", "X");
			MerchantPaymentTypeAdapter::createMerchantPaymentTypeRecord($merchant_id, "CreditCard");
			MerchantMenuMapAdapter::createMerchantMenuMap($merchant_id, $this->jersey_mikes_menu_id, "pickup");
			if (isset($store_basic['delivery']) && $store_basic['delivery'] == '1') {
				$merchant_resource->delivery = 'Y';
				MerchantMenuMapAdapter::createMerchantMenuMap($merchant_id, $this->jersey_mikes_menu_id, "delivery");
				$merchant_resource->save();
			}

			$sql = "INSERT INTO Holiday (merchant_id) VALUES ($merchant_id)";
			$merchant_adapter->_query($sql);
		}
		$this->imported_merchant_ids[] = $merchant_id;		
		$this->setStoreHours($merchant_id, 'R', $store_data);
		
		// new delivery code
		if ($_SERVER['FORCE_JM_DELIVERY_FOR_STORE'] == "$merchant_external_id") {
			$store_basic['delivery'] = '1';
		}
		if (isset($store_basic['delivery'])) {
			if ($store_basic['delivery'] == '1') {
				$merchant_resource->delivery = 'Y';
				$polygon_coordinates = $store_basic['polygon_coordinates'];
				$delivery_minimum_purchase = isset($store_basic['delivery_minimum_purchase'])? $store_basic['delivery_minimum_purchase']:1.00;
				$delivery_minimum_time_window = isset($store_basic['delivery_minimum_time_window_minutes'])?$store_basic['delivery_minimum_time_window_minutes']:30;
				$delivery_cost = isset($store_basic['delivery_price'])?$store_basic['delivery_price']:0;
				// now create all the delivery stuff
				Resource::findOrCreateIfNotExistsByData(new MerchantMenuMapAdapter($mimetypes), array("merchant_id"=>$merchant_resource->merchant_id,"menu_id"=>102433,"merchant_menu_type"=>'delivery'));
				$this->setStoreHours($merchant_id, 'D', $store_data);
				$this->findOrCreateMerchantDeliveryInfo($merchant_id, $polygon_coordinates, $delivery_minimum_purchase, $delivery_minimum_time_window);
				$this->findOrCreateMerchantDelvieryPriceDistance($merchant_id, $polygon_coordinates, $delivery_cost);
			} else {
				$merchant_resource->delivery = 'N';
			}
			$merchant_resource->save();
		}		
		
		foreach ($store_prices as $the_jm_price_record_for_merchant)
    	{
 	  		if (substr($the_jm_price_record_for_merchant['plu_id'], -2) == "00")
    			$plu = 'GPLU'.$the_jm_price_record_for_merchant['plu_id'];
    		else
    			$plu = 'PLU'.$the_jm_price_record_for_merchant['plu_id'];
     		$jm_price_records[$plu] = $the_jm_price_record_for_merchant;
    	}
    	
    	// determin delivery price and if taxable
    	$this->checkForDeliveryPluAndSetTaxRecordIfPresent($jm_price_records,$merchant_id);

    	$this->jm_price_records_from_jm_system = $jm_price_records;
    	
    	// get specialities items (HACK ALERT!)
   		$gplu700_price_record = $jm_price_records['GPLU700'];
   		$gplu600_price_record = $jm_price_records['GPLU600'];
   		$regular_lunch_box_drink_price_diff = $gplu700_price_record['price'] - $gplu600_price_record['price'];
    	$gplu500_price_record = $jm_price_records['GPLU500'];
    	$gplu400_price_record = $jm_price_records['GPLU400'];
    	$mini_lunch_box_drink_price_diff = $gplu500_price_record['price'] - $gplu400_price_record['price'];
    	
    	$gplu100_price_record = $jm_price_records['GPLU100'];
    	$gplu200_price_record = $jm_price_records['GPLU200'];
    	$small_large_combo_price_diff = $gplu200_price_record['price'] - $gplu100_price_record['price'];
    	
    	$raw_chip_22ozdrink_price = $jm_price_records['PLU11702']['price'] + $jm_price_records['PLU8702']['price'];
    	$raw_chip_32ozdrink_price = $jm_price_records['PLU11702']['price'] + $jm_price_records['PLU8703']['price'];
    	
    	$small_fountain_drink_price = $jm_price_records['PLU8702']['price'];
    	$large_fountain_drink_price = $jm_price_records['PLU8703']['price'];
    	
    	myerror_log("rawchip22price: ".$raw_chip_22ozdrink_price);
    	myerror_log("smallcomboprice: ".$gplu100_price_record['price']);
    	myerror_log("rawchip32price: ".$raw_chip_32ozdrink_price);
    	myerror_log("largecomboprice: ".$gplu200_price_record['price']);

    	if ($raw_chip_22ozdrink_price < $gplu100_price_record['price'])
    	{
    		myerror_log("bad small combo pricing on jersey mikes store: ".$merchant_external_id);
    		MailIt::sendEmailSingleRecepientWithValidation($this->jm_error_addr, "bad small combo pricing on jersey mikes", "store id: ".$merchant_external_id, $from_name, $bcc, $attachments);
    		MailIt::sendEmailSingleRecepientWithValidation("arosenthal@order140.com", "bad small combo pricing on jersey mikes", "store id: ".$merchant_external_id, $from_name, $bcc, $attachments);
    	}
    	if ($raw_chip_32ozdrink_price < $gplu200_price_record['price'])
    	{
    		myerror_log("bad large combo pricing on jersey mikes store: ".$merchant_external_id);
    		MailIt::sendEmailSingleRecepientWithValidation($this->jm_error_addr, "bad large combo pricing on jersey mikes", "store id: ".$merchant_external_id, $from_name, $bcc, $attachments);
    		MailIt::sendEmailSingleRecepientWithValidation("arosenthal@order140.com", "bad large combo pricing on jersey mikes", "store id: ".$merchant_external_id, $from_name, $bcc, $attachments);
    	}
    		
    	$complete_menu = new CompleteMenu($menu_id);
    	$price_resources = $complete_menu->getHashOfResourcesAllPricesForMenuWithExternalIdAsIndex($menu_id, $merchant_id);
    	$modifier_price_resources = $complete_menu->getHashOfResourcesAllModifierPricesForMenuWithExternalIdAsIndex($menu_id, $merchant_id);
    	$combo_price_resources = $complete_menu->getHashOfResourcesAllComboPricesForMenuWithExternalIdAsIndex($menu_id, $merchant_id);
    	
    	$total_price_resources = array_merge($price_resources,$modifier_price_resources,$combo_price_resources);

    	foreach ($total_price_resources as $price_resource)
    	{
    		if ($splickit_plu = $price_resource->external_id)
    		{
    			$included_merchant_menu_types = $price_resource->included_merchant_menu_types;
    			myerror_log("About to do JM price for: ".$splickit_plu);
    			if (substr($splickit_plu,-7) == 'noprice')
    			{
    				// at the time of coding this is only being used for teh steak or chicken in the cheese steaks	
    				myerror_log("we have a noprice identifier");
    			}
    			
    			// must deal with the combo drink stuff here!
    			
    			if (substr_count($splickit_plu,':') > 0)
    			{
    				$s = explode(':', $splickit_plu);
    				$splickit_plu = $s[0];
    				$last_item_in_external_id_array = array_pop($s);
    			} else {
    				$last_item_in_external_id_array = $splickit_plu;
    			}
    			//lunchbox HACK
    			if ($splickit_plu == 'GPLU600,GPLU700') // regular size box lunch
    			{	
    				$splickit_plu = 'GPLU600';
    			} 
    			else if ($splickit_plu == 'GPLU400,GPLU500') // mini size box lunch
    			{
    				$splickit_plu = 'GPLU400';
    			} 
    			$price_resource->merchant_id = $merchant_id;
  
     			if ($jm_price_record = $jm_price_records[$splickit_plu])
    			{
    				$price = $jm_price_record['price'];
    				$taxable = $jm_price_record['taxable'];
    				if (isset($jm_price_record['available_for'])) {
    					if ($jm_price_record['available_for'] == 1) {
    						$included_merchant_menu_types = 'Pickup';
    					} else if ($jm_price_record['available_for'] == 2) {
    						$included_merchant_menu_types = 'Delivery';
    					}
    				}
    				if ($splickit_plu == 'GPLU100' || $splickit_plu == 'GPLU200' || $splickit_plu == 'PLU8702' || $splickit_plu == 'PLU8703') {
    					//fountain drinks and combos
    					$included_merchant_menu_types = 'Pickup';
    				}
    				myerror_log("the aquired price for $splickit_plu is: $price");
    				myerror_log("the aquired taxable for $splickit_plu is: ".$taxable);
    				// get the chip price since we'll need this for the price override on the chip main item down below
    				$type = $price_resource->getTypeOfResource();
    			    if ($splickit_plu == 'PLU11702' &&  $type == 'Item_Size_Map')
    				{
    					$chip_price = $price;
    					$chip_item_id = $price_resource->item_id;
    				} else if ($splickit_plu == 'PLU15307' &&  $type == 'Item_Size_Map') {
    					$bottled_drink_price = $price;
    					$bottled_drink_item_id = $price_resource->item_id;
    				} else if ($splickit_plu == 'GPLU600' || $splickit_plu == 'GPLU400') {
    					$lunch_box_item_id = $price_resource->item_id; // not sure why this line is here, i dont use lunch box item id anywhere
    				} else if ($splickit_plu == 'GPLU700') {
    					$price = $regular_lunch_box_drink_price_diff;
    				} else if ($splickit_plu == 'GPLU500') {
    					$price = $mini_lunch_box_drink_price_diff;
    				} else if ($price_resource->external_id == 'GPLU200:PLU8703:xxxxx') {
    					// large combo drink
    					$price = $small_large_combo_price_diff;
    				} else if ($price_resource->external_id == 'PLU8702:xxxxx') {
    					// small combo drink
    					$price = 0;
    				}

					if ($taxable == "0") {
						$price_resource->tax_group = 0;
					} else if ($taxable == "1" && $price_resource->tax_group == 0) {
						$price_resource->tax_group = 1;
					} else {
    					// so the item is taxable so leave it as the tax_group of the existing resource or the parent price record of merchant_id = 0
    				}
                    $pulled_price = floatval($jm_price_record['price']);
    				if ($pulled_price < .01)
    				{
                        $price_resource->active = 'N';
                        $price = 0.00;
    				} else {
                        $price = floatval($price);
                        $price_resource->active = 'Y';
                    }
    				// at the time of coding this is only being used for teh steak or chicken in the cheese steaks
    				if ($last_item_in_external_id_array == 'noprice') {
                        $price = 0;
                    }
    				
    				$price_resource->price = $price;
    				$price_resource->included_merchant_menu_types = $included_merchant_menu_types;
    				$price_resource->modifier_price = $price;
    				// ******  do the chip drink hack here??? ****** //
    				if ($type == 'Modifier_Size_Map') {
    					// we have a modifier so check if its a chip flavor or bottled drink flavor
    					if ($splickit_plu == 'PLU11702' || $splickit_plu == 'PLU15307') {
    						// bottled drink or chip flavor
    						if (isset($chip_drink_flavors)) {
	    						if ($chip_drink_flavors[$last_item_in_external_id_array] || $last_item_in_external_id_array == 'M8888') {
	    							$price_resource->active = 'Y';
	    						} else {
	    							$price_resource->active = 'N';
	    						}
    						}
    					}
    				}
    			} else if (substr_count($splickit_plu, 'PLU') > 0) {
	    			// make sure to set to innactive since no data came through for this item in the dump from JM
	    			$price_resource->active = 'N';	
    			}  else  {
                    // if we get here its probably just an ingredient so the fields will inherit from either the parent '0' record or the existing record
                }
    		} else {
    			// we just need to make sure to create a child record
    			$price_resource->merchant_id = $merchant_id;
    		}
    		$price_resource->save();
    	}
    	
    	// now need to copy the Item_Modifier_Group_Map records
    	// there is a potential problem here.  if we are using a price override, it may be differnt for each merchant
    	// but we dont really have a mechanism to do that based on the jersey mikes data.
    	ItemModifierGroupMapAdapter::createChildRecordsForMerchant($this->jersey_mikes_menu_id, $merchant_id,true);
   	
    	// need to set price override for the Main Chips Item
    	if ($chip_price && $chip_item_id)
    	{
    		$imgm_adapter = new ItemModifierGroupMapAdapter($mimetypes);
    		$imgm_data2['item_id'] = $chip_item_id; 
    		$imgm_data2['merchant_id'] = $merchant_id;
    		$imgm_options2[TONIC_FIND_BY_METADATA] = $imgm_data2;
    		if ($imgm_resource2 = Resource::findExact($imgm_adapter,'',$imgm_options2))
    		{
    			$imgm_resource2->price_override = $chip_price;
    			$imgm_resource2->save();
    		}	 
    	} 
    	
    	// need to set price override for the Bottled drink choices
    	if ($bottled_drink_price && $bottled_drink_item_id)
    	{
    		$imgm_adapter = new ItemModifierGroupMapAdapter($mimetypes);
    		$imgm_data3['item_id'] = $bottled_drink_item_id; 
    		$imgm_data3['merchant_id'] = $merchant_id;
    		$imgm_options3[TONIC_FIND_BY_METADATA] = $imgm_data3;
    		if ($imgm_resource3 = Resource::findExact($imgm_adapter,'',$imgm_options3))
    		{
    			$imgm_resource3->price_override = $bottled_drink_price;
    			$imgm_resource3->save();
    		}	 
    	} 
    	
    	if (isProd() && $error_text != null && $error_text != '')
    	{
    		MailIt::sendEmailMandrill($this->jm_error_addr,"There were errors importing store ".$merchant_external_id, $error_text, "splickit import admin", $bcc, $attachments);
    		MailIt::sendErrorEmail("There were errors importing Jersey Mikes store ".$merchant_external_id, $error_text);
    	}
    	myerror_log("the value of force links is: ".$this->force_links);
    		$addr_temp = str_ireplace(' ', '-', $store_basic['address1']);    		
    		$addr_temp = str_ireplace('#', '', $addr_temp);    		
    		$better_address1 = str_ireplace('.', '', $addr_temp);    		
    		$better_city = str_ireplace(' ', '-', $store_basic['city']);
    		$better_state = str_ireplace(' ', '-', $store_basic['state']);
    		$link = 'https://ordering.jerseymikes.com/'.$better_state.'/'.$better_city.'/'.$better_address1.'/'.$merchant_id;
    		$this->link_text_for_email .= "Store Id ".$merchant_external_id." = ".$link." \r\n";
    		myerror_log("jersey mikes link: ".$link);    		
    	return true;
    }
    
    function setStoreHours($merchant_id,$hour_type,$store_data)
    {
		$day_array = array('sunday','monday','tuesday','wednesday','thursday','friday','saturday');
		if ($hour_type == 'R') {
			$store_hours = $store_data['hours'];
		} else if ($hour_type == 'D') {
			if ($store_hours = $store_data['delivery_hours']) {
				myerror_log("we have delivery hours");
			} else {
				myerror_log("no delivery hours devualt to store hours");
				$store_hours = $store_data['hours'];
				$no_submitted_delivery_hours = true;
			}
		}

		if ($store_hours)
		{
			foreach ($day_array as $index=>$value)
			{
				$hours_string = $store_hours[$value];
				$hours_string = str_replace(' ', '', $hours_string);
				
				$hours = split('-',$hours_string);
				$open = date("H:i",strtotime($hours[0]));
				$close = date("H:i",strtotime($hours[1]));
				
				if ($open > $close) {
					MailIt::sendEmailSingleRecepientWithValidation("arosenthal@order140.com", "jerey mikes inverted hours", "inverted hours for merchant_id: ".$merchant_id, $from_name, $bcc, $attachments);
				}

				$hour_data['merchant_id'] = $merchant_id;
				$hour_data['hour_type'] = $hour_type;
				$hour_data['day_of_week'] = $index+1;
				$hour_resource = Resource::findOrCreateIfNotExistsByData(new HourAdapter($mimetypes), $hour_data);
				// this is a bit whacky, but JM hours are ONLY set the first time they turn delivery on....UNLESSS they submit delivery hours.
				// so this logic prevents over writing any manual changes that were made to the delivery hours before they start sending us the actual values.
				// yep. its that weird.
				if (!isset($hour_resource->insert_id) && $hour_type == 'D' && $no_submitted_delivery_hours) {
					continue;
				} 
				$hour_resource->open = $open;
				$hour_resource->close = $close;
				$hour_resource->save();
				
			}
		} else {
			throw new Exception("No store hours submitted to set store hours method");
		}		
    }
    
    function findOrCreateMerchantDeliveryInfo($merchant_id, $polygon_coordinates, $delivery_minimum_purchase, $delivery_minimum_time_window)
    {
    	$data['merchant_id'] = $merchant_id;
    	$merchant_delivery_info_resource = Resource::findOrCreateIfNotExistsByData(new MerchantDeliveryInfoAdapter($mimetypes), $data);
    	if ($merchant_delivery_info_resource->insert_id) {
    		$new_record = true;
    	}
    	if ($polygon_coordinates) {
    		$merchant_delivery_info_resource->delivery_price_type = 'polygon';
    	} else if ($new_record) {
    		$merchant_delivery_info_resource->delivery_price_type = 'driving';
    	}
    	if ($delivery_minimum_purchase) {
    		$merchant_delivery_info_resource->minimum_order = $delivery_minimum_purchase;
    	} else if ($new_record) {
    		$merchant_delivery_info_resource->minimum_order = 1.00;
    	}
    	if ($delivery_minimum_time_window) {
    		$merchant_delivery_info_resource->minimum_delivery_time = $delivery_minimum_time_window;
    	}
    	$merchant_delivery_info_resource->save();
    }
    
    function findOrCreateMerchantDelvieryPriceDistance($merchant_id,$polygon_coordinates,$price)
    {
    	$data['merchant_id'] = $merchant_id;
    	$merchant_delivery_price_distance_resource = Resource::findOrCreateIfNotExistsByData(new MerchantDeliveryPriceDistanceAdapter($mimetypes), $data);
    	if ($merchant_delivery_price_distance_resource->insert_id) {
    		$new_record = true;
    	}
    	if ($polygon_coordinates) {
    		$merchant_delivery_price_distance_resource->polygon_coordinates = $polygon_coordinates;
    	} else if ($new_record) {
    		$merchant_delivery_price_distance_resource->distance_up_to = 1.00;
    	}
    	if ($price) {
    		$merchant_delivery_price_distance_resource->price = $price;
    	} else if ($new_record) {
    		$merchant_delivery_price_distance_resource->price = 1.00;
    	}
    	$merchant_delivery_price_distance_resource->save();
    }
    
    function checkForDeliveryPluAndSetTaxRecordIfPresent($jm_price_records,$merchant_id)
    {
    	if ($delivery_plu_record = $jm_price_records[JerseyMikesController::$non_taxable_delivery_plu]) {
    		$rate = 0;
    	} else if ($delivery_plu_record = $jm_price_records[JerseyMikesController::$taxable_delivery_plu]) {
    		$rate = TaxAdapter::staticGetTotalBaseTaxRate($merchant_id);
    	}
    	if ($delivery_plu_record) {
    		$this->setDeliveryTaxRecord($delivery_plu_record, $merchant_id,$rate);
    	}
    }
    
    function setDeliveryTaxRecord($delivery_plu_record,$merchant_id,$rate)
    {
    	$tax_adapter = new TaxAdapter($mimetypes);
   		$delivery_price = $delivery_plu_record['price'];
    	$tax_options[TONIC_FIND_BY_METADATA] = array("merchant_id"=>$merchant_id,"locale"=>"Delivery","tax_group"=>99);
    	$tax_resource = Resource::findOrCreateIfNotExists($tax_adapter, $url, $tax_options);
    	$tax_resource->rate = $rate;
    	$tax_resource->locale_description = 'PLU'.$delivery_plu_record['plu_id'];
    	$tax_resource->save();
    	
    }
    
    /**
     * 
     * creates the splickit link for the Jersey Mikes store.  returns a resource with link and external mercahnt id 
     * @param $merchant_external_id
     * @return Resource
     */
    function createStoreLinkResponse($merchant_external_id)
    {
    	if ($link = $this->getStoreLink($merchant_external_id))
    	{
    		$resource = new Resource();
    		$resource->set('jersey_mikes_merchant_id',$merchant_external_id);
    		$resource->set('link',$link);
    		$resource->_representation = '/json.xml';
    	} else {
    		$resource = returnErrorResource("No store matching that Id can be found");
    	}
    	return $resource;
    }
    
    /**
     * 
     * creates the splickit link for the Jersey Mikes store.  returns a resource with link and external mercahnt id 
     * @param Request $request
     * @return Resource
     */
    function createStoreLinkResponseFromRequest($request)
    {
    	$url_context = 'jerseymikesstore';
    	if ($merchant_external_id = $this->getJMExternalStoreIdFromUrl($url_context,$request->url)) {
    		return $this->createStoreLinkResponse($merchant_external_id);
    	} else {
    		return returnErrorResource("No store id sumitted");
    	}
    }

    function getStoreLink($merchant_external_id)
    {
    	$o_store_basic['merchant_external_id']=$merchant_external_id;
		$o_store_basic['brand_id'] = $this->jersey_mikes_brand_id;
		$options_store_basic[TONIC_FIND_BY_METADATA] = $o_store_basic;
		$merchant_adapter = new MerchantAdapter($mimetypes);
		if ($merchant_resource = Resource::find($merchant_adapter,'',$options_store_basic))
		{
    		$complete_address = $merchant_resource->state.'/'.$merchant_resource->city.'/'.$merchant_resource->address1;
    		$complete_address = str_ireplace(' ', '-', $complete_address);
    		$complete_address = str_ireplace('#', '', $complete_address);
    		$complete_address = str_ireplace('.', '', $complete_address);
    		$link = 'https://ordering.jerseymikes.com/'.$complete_address.'/'.$merchant_resource->merchant_id;
    		return $link;
		}
		return false;
    }

    function getErrorText()
    {
    	return $this->error_text;
    }
    
    function setForceLinksOn()
    {
    	$this->force_links = true;
    }
    
    function getLinkTextForEmail()
    {
    	if ($this->link_text_for_email) 
    		return $this->link_text_for_email;
    	else
    		return false;
    }

 	function getJMExternalStoreIdFromUrl($url_context,$url)
 	{
		$key_values = '%/'.$url_context.'/([0-9]{3,11})/%';
		if (preg_match($key_values, $url, $matches)) {
			$external_store_id = $matches[1];
		} else if (substr_count($url,'/HD/') > 0) {
			$external_store_id = 'HD';
		} else {
			myerror_log("NO STORE ID SUBMITTED");
			return null;		
		}
		myerror_log("we have a jersey mikes external id of: ".$external_store_id);
 		return $external_store_id;
 	}

	/*
	 * ok so the JM importer was built long before the importer framework so its 'mostly' stand alone but uses some methods from the splicit importer
	 * which is NOW an abstract method in order for force certain methods. So...... i had to add these methods to the JM importer just to allow it to be instantiated.
	 * There is currently NO plan to migrate this object to the new framework since it all works fine and there is NO benefit to the refactor.
	 */

    function getItemDescriptionFromItemData($item_data) {}

    function getModifierGroupMinForThisIntegration($modifier_group_data) {}

	function getModifierGroupMaxForThisIntegration($modifier_group_data) {}

	function getActiveFlagSetting($data) {}

	function getMenuTypeCatId($menu_type_data) {}

	function setBrandId() {}

	function setBrandRegExForMerchantExternalIds() {}

	function importRemoteData($merchant_resource) {}


}
?>
