<?php

class JerseymikesLoyaltyController extends LoyaltyController
{
	var $too_many_failed_loyalty_attempts_message = 'Sorry, but your account is now locked out of loyalty for 1 hour due to too many failed attempts.';
	
	function createAccount($user_id,$points) 
	{
		$loyalty_number = $this->getLoyaltyNumber();
		$shore_points_loyalty_service = new ShorepointsLoyaltyService();
		$result = $shore_points_loyalty_service->createShorePointsAccountFromPhoneNumber($loyalty_number);
		if ($result['status'] == 'success')
		{
   			$user_brand_loyalty_adapter = new UserBrandPointsMapAdapter($mimetypes);
   			$user_brand_loyalty_resource = $user_brand_loyalty_adapter->createUserLoyaltyAccount($user_id, $this->brand_id, $loyalty_number, $points);
			return $result;
		} else {
			$this->service_response = $shore_points_loyalty_service->curl_response;
			return false;
		}
	}
	
	function linkAccount($user_id,$points)
	{
		if ($this->validateLoadedLoyaltyNumber()) {
			$points = $this->remote_account_info['points'];
			return $this->addLoyaltyRecord($user_id, $points);
		} else {
			return false;
		}
	}
	
	/**
	 * 
	 * @description this should return the data as an array?  maybe?  hmmm..
	 * @param string $loyalty_number
	 * @return array
	 */
	function getRemoteAccountInfo($loyalty_number)
	{
		if ($this->remote_account_info) {
			return $this->remote_account_info;
		}
    	$loyalty_number = str_ireplace("-", "", $loyalty_number);
   		$loyalty_number = str_ireplace("(", "", $loyalty_number);
   		$loyalty_number = str_ireplace(")", "", $loyalty_number);
   		$loyalty_number = str_ireplace(".", "", $loyalty_number);
   		
   		$shorepoints_loyalty_service = new ShorepointsLoyaltyService();
   		if ($info = $shorepoints_loyalty_service->getShorePointsAccount($loyalty_number)) {
	   		if (isset($info['history'])) {
	   			$this->loyalty_history = $info['history'];
	   		}
	   		return $info;
   		}
	}
		
	function getAccountInfoForUserSession()
	{
		if ($user_brand_points_map_resource = $this->getLocalAccountInfoAsResource()) {
			$loyalty_number = $user_brand_points_map_resource->loyalty_number;
		} else {
			return null;
		}
		if ($raw_info_from_curl = $this->getRemoteAccountInfo($loyalty_number)) {
			$info = $this->cleanRemoteLoyaltyResponseForUserSession($raw_info_from_curl);
	    	foreach ($info as $name=>$value)
	    	{
	    		if ($name == "history" || name == "points")
	    			continue;
	    		$blai_row['key'] = $name;
	    		if ($name == "store_number")
	    			$name = "my store";
	    		if ($name == 'creation_date')
	    		{
	    			$name = "member since";
	    			$s = explode(" ", $value);
	    			$value = $s[0];
	    		}
	    		$blai_row['name'] = ucwords(str_replace("_", " ", $name)); 
	    		$blai_row['value'] = $value;
	    		$blai_row['type'] = "text";
	    		$blai_row['editable'] = "no";
	    		//if ($name == 'phone_number' || substr($name,0,5) == 'opted')
				//	$blai_row['editable'] = "yes";	
				if (substr($name,0,5) == 'opted')
					$blai_row['type'] = "boolean";		    				
	    		$attributes[] = $blai_row;
	    	}
	    	$info['loyalty_number'] = $loyalty_number;     	
	    	$info['loyalty_attributes'] = $attributes;
	    	$loyalty_transactions = (is_array($info['history'])) ? $info['history'] : array();
	    	$info['loyalty_transactions'] = $this->getNormalizedHistory($loyalty_transactions);
	    	$info['barcode_number'] = $info['card_number'];
	    	$info['loyalty_points'] = $info['points'];
	    	//$info['points_current'] = $info['points'];
	    	
	    	// record the value in  the user_brand_points table
	    	$user_brand_points_map_resource->points = $info['points'];
	    	$user_brand_points_map_resource->save();
	    	return $info;
		}
	}
	
	function getNormalizedHistory($loyalty_transactions)
	{
		foreach ($loyalty_transactions as &$transaction) {
			$transaction['description'] = $transaction['transaction_type'];
			$transaction['amount'] = $transaction['points_added']." points";
			$transaction['balance'] = $transaction['balance']." points";
		}
		return $loyalty_transactions;
	}
		
	function validateLoadedLoyaltyNumber()
	{
		if ($this->doesUserHaveTooManyFailedAttempts()) {
			$this->service_response['error'] = $this->too_many_failed_loyalty_attempts_message;
			return false;
		}
		$account_info = $this->getRemoteAccountInfo($this->getLoyaltyNumber());
		if ($account_info['status'] == 'success') {
			$this->remote_account_info = $account_info;
			return true;
		} else {
			// to do fail logic
			$this->service_response = $account_info;
			if ($this->user['user_id'] > 9999) {
				$this->recordLoyaltyFail();
			}
			$this->service_response['error'] = $this->getBadLoyaltyNumberMessage();
			return false;
		}
	}
	
	function recordLoyaltyFail()
	{
		myerror_log("BAD LOYALTY NUBMER ON Validate: ".$loyalty_number);
		$id = ($this->user['device_id'] != NULL && trim($this->user['device_id']) != '') ? $this->user['device_id'] : $this->user['user_id'];
		$brand_loyalty_fails_adapter = new BrandLoyaltyFailsAdapter($mimetypes);
		$brand_loyalty_fails_resource =  $brand_loyalty_fails_adapter->getExactResourceFromData(array("device_id"=>$id,"brand_id"=>326));
   		if ($brand_loyalty_fails_resource)
   		{
   			$brand_loyalty_fails_resource->failed_attempts = $brand_loyalty_fails_resource->failed_attempts + 1;
   			if ($brand_loyalty_fails_resource->failed_attempts > 2) {
   				$brand_loyalty_fails_resource->unlock_at_ts = time() + 3602; // the 02 is just for testing to see that is 2 hour in the future.
   				$this->service_response['error'] = $this->too_many_failed_loyalty_attempts_message;
   			}
   		} else {
   			$data['device_id'] = $id;
   			$data['brand_id'] = 326;
   			$data['failed_attempts'] = 1;
   			$brand_loyalty_fails_resource = Resource::factory(new BrandLoyaltyFailsAdapter($mimetypes),$data);
   		}
   		return $brand_loyalty_fails_resource->save();	
	}
	
	function doesUserHaveTooManyFailedAttempts()
	{
   		$id = ($this->user['device_id'] != NULL && trim($this->user['device_id']) != '') ? $this->user['device_id'] : $this->user['user_id'];
   		return BrandLoyaltyFailsAdapter::doesUserHaveMoreThanSubmittedNumberOfFails($id, 326, 2);
	}
	
	/**
	 * 
	 * @desc This method gets overridden in the custom loyalty progams.
	 * @param array $complete_order
	 * @param int $points
	 * @return mixed
	 */ 
	function sendLoyaltyOrderEvent($complete_order,$points)
	{
		return array();
	}
	
	/**
	 * @desc holds the function that creates points from an order
	 *
	 */ 
	function getPointsEarnedFromCompleteOrder($complete_order)
	{	
		return round($complete_order['order_amt']);
	}
	
	// function getHistory() {}
	
	function setLoyaltyData($data)
	{
		parent::setLoyaltyData($data);
		if (isset($this->data['loyalty_phone_number'])) {
			$this->loyalty_number = $this->data['loyalty_phone_number'];
		}
	}
}
?>
