<?php

use Aws\S3\S3Client;
use Aws\Sqs\SqsClient;
use Aws\Exception\AwsException;

class AWSService
{
	protected $client = null;
	
	function __construct() {
        $key = getProperty('S3_AWS_ACCESS_KEY_ID');
        $secret = getProperty('S3_AWS_SECRET_ACCESS_KEY');
        if ($key == null || $secret == null) {
            if (isProd()) {
                MailIt::sendErrorEmailAdam("NO AWS VALUES IN PRODUCTION","Unable to get AWS secret and key.  check credentials.conf file and/or db properties as a back up for   S3_AWS_ACCESS_KEY_ID   and   S3_AWS_SECRET_ACCESS_KEY");
            }
            myerror_log("ERROR!!!!!! we have NO AWS credentials");
            //throw new Exception("unable to get aws key and secret.");
        } else {
            $c = new S3Client(array(
                'version' => 'latest',
                'region'  => 'us-east-1',
                'credentials' => array(
                    'key' => $key,
                    'secret'  => $secret
                )
            ));
            $this->client = $c;
        }
	}

	function sendToQueue($queue, $json) {
        $key = getProperty('S3_AWS_ACCESS_KEY_ID');
        $secret = getProperty('S3_AWS_SECRET_ACCESS_KEY');
        if ($key == null || $secret == null) {
            if (isProd()) {
                MailIt::sendErrorEmailAdam("NO AWS VALUES IN PRODUCTION","Unable to get AWS secret and key.  check credentials.conf file and/or db properties as a back up for   S3_AWS_ACCESS_KEY_ID   and   S3_AWS_SECRET_ACCESS_KEY");
            }
            myerror_log("ERROR!!!!!! we have NO AWS credentials");
            //throw new Exception("unable to get aws key and secret.");
        } else {
            $c = new SqsClient(array(
                'version' => 'latest',
                'region' => (isProd()) ? 'us-east-1' : 'us-west-2',
                'credentials' => array(
                    'key' => $key,
                    'secret' => $secret
                )
            ));
        }

        $params = [
            'DelaySeconds' => 10,
            'MessageAttributes' => [
                "Title" => [
                    'DataType' => "String",
                    'StringValue' => "Maverick-Order"
                ],
                "Author" => [
                    'DataType' => "String",
                    'StringValue' => "Maverick"
                ]
            ],
            'MessageBody' => $json,
            'QueueUrl' => $queue
        ];

        try {
            $result = $c->sendMessage($params);
            return true;
        } catch (AwsException $e) {
            // output error message if fails
            myerror_log($e->getMessage());
            throw $e;
        }
    }

	function getFileNameList($bucket,$filter)
    {
        $files = array();
        $objects = $this->client->getIterator('ListObjects', array(
            'Bucket' => $bucket
        ));
        foreach ($objects as $object) {
            $key = $object['Key'];
            if (substr_count($key,"$filter")) {
                $files[] = $key;
            }
        }
        return $files;
    }

	function hasKey($bucket, $key) {
		return $this->doesObjectExist($bucket, $key); 
	}		
	
	function getKey($bucket, $key) {

		if (getProperty('do_not_call_out_to_aws') == 'true') {
			return null;
		} else if (fopen("http://www.google.com:80/","r")) {
			return $this->getOrElse($bucket, $key, null);
		} else {
			myerror_log("NO INTERNET CONNECTION!!!! CANT GET REMOTE TEMPLATES!!");
			return null;
		}

	}

	function rename($bucket, $key, $new_name)
    {
        if($this->hasKey($bucket, $key)) {
            $rename_array = ['Bucket' => $bucket, 'Key' => $new_name,'CopySource' => "$bucket/$key"];
            if ($this->client->copyObject($rename_array)) {
                $this->client->deleteObject(['Bucket' => $bucket, 'Key' => $key]);
            } else {
                myerror_log("AWS S3 ERROR. Couldnt rename object in remote bucket: ".json_encode($rename_array));
            }

        } else {
            return false;
        }
    }

    function move($bucket, $existing_key, $new_key)
    {
        if($this->hasKey($bucket, $existing_key)) {
            $move_array = ['Bucket' => $bucket, 'Key' => $new_key,'CopySource' => "$bucket/$existing_key"];
            if ($this->client->copyObject($move_array)) {
                $this->client->deleteObject(['Bucket' => $bucket, 'Key' => $existing_key]);
            } else {
                myerror_log("AWS S3 ERROR. Couldnt rename object in remote bucket: ".json_encode($rename_array));
            }

        } else {
            return false;
        }
    }

    function getOrElse($bucket, $key, $default) {
		if($this->hasKey($bucket, $key)) {
			return $this->getObject($bucket, $key);
		} else {
			return $default;
		}
	}

	function getOrElseWithLocalSave($bucket, $key, $local_file_name,$default)
    {
        if($this->hasKey($bucket, $key)) {
            if ($object = $this->client->getObject(array('Bucket' => $bucket, 'Key' => $key,'SaveAs' => '/tmp/'.$local_file_name) )) {
                return true;
            } else {
                recordError("AWS service Couldn't save file to local file system","File: $local_file_name  to /tmp/ folder. most likely trying to download an price file for import");
                myerror_log("Error! Couldn't save file  $local_file_name to local file system in tmp folder");
            }
        } else {
            myerror_log("Error! Couln't find key: $key");
        }
        return false;
    }

	function getObjectUrl($bucket, $key, $expires = null, array $args = array()){
			return $this->client->getObjectUrl($bucket,$key, $expires, $args);
	}
	
	protected function doesObjectExist($bucket, $key) {
        myerror_log("Checking for existance of key: $key, in bucket: $bucket",3);
		return $this->client->doesObjectExist($bucket, $key);
	}
	
	protected function getObject($bucket, $key) {
		$object = $this->client->getObject(array(
				'Bucket' => $bucket,
				'Key' => $key
		));
		
		return $object->get("Body")->getContents();
	}
}

?>