<?php
use Aws\S3\S3Client;
class EmailService {
	
	protected $aws_service;
	protected $template_service;
	
	function __construct($aws = null, $ts = null) {
		if(isset($aws)) {
			$this->aws_service = $aws;
		} else {
			$this->aws_service = new AWSService();
		}
		
		if(isset($ts)) {
		  $this->template_service = $ts;
		} else {
		  $this->template_service = new TemplateManager($m);
		}
	}
	
	function getGroupOrderInviteEmail($data) {
        $bucket = getProperty('bucket_prefix').".templates";
        if(isProd() || isLaptop()) {
            $key = 'group-order-email/production/group_order_invite_email.html';
	  } else {
            $key = 'group-order-email/staging/group_order_invite_email.html';
	  }
	  if($body = $this->aws_service->getKey($bucket, $key)) {
	    myerror_log("EmailService - successfully got group order invite email.");
	    //$body = $this->rewindReadbody($body);
	    return $this->template_service->populateTemplate($body, $data);
	  } else {
	    myerror_log("EmailService - could not get group order invite email from $bucket.");
	    return null;
	  }
	}
	
	function getGroupOrderAdminEmail($data) {
        $bucket = getProperty('bucket_prefix').".templates";
        if(isProd() || isLaptop()) {
            $key = 'group-order-email/production/group_order_confirmation.html';
        } else {
            $key = 'group-order-email/staging/group_order_confirmation.html';
        }

	  if($body = $this->aws_service->getKey($bucket, $key)) {
	    myerror_log("EmailService - successfully got group order admin email.");
	    //$body = $this->rewindReadbody($body);
	    return $this->template_service->populateTemplate($body, $data);
	  } else {
	    myerror_log("EmailService - could not get group order admin email from $bucket.");
	    return null;
	  }
	}

	function getMerchantWelcomeEmail($bucket,$skin_name)
    {
        $key = "merchant-welcome-email/production/".$skin_name."-welcome-email.html";
        myerror_log("EmailService - getting $bucket MERCHANT welcome template for skin: $skin_name",5);
        return $this->getWelcomeEmail($bucket,$key);
    }

    function getUserWelcomeEmail($bucket,$skin_name)
    {
        $key = "user-welcome-email/production/".$skin_name."-welcome-email.html";
        myerror_log("EmailService - getting $bucket welcome template for skin: $skin_name",5);
        return $this->getWelcomeEmail($bucket,$key);
    }

	function getWelcomeEmail($bucket, $key) {

	  if($body = $this->aws_service->getKey($bucket, $key)) {
	    return $this->template_service->populateTemplate($body, array());
	  } else {
	    myerror_log("EmailService - could not get welcome email for skin: ".getSkinNameForContext(),5);
	    return null;
	  }
	}
	
	function getUserWelcomeTemplate($skin_name) {
	  return $this->getUserWelcomeEmail(getProperty('bucket_prefix').".templates", $skin_name);
	}
	
	function getMerchantWelcomeTemplate($skin_name) {		
		return $this->getMerchantWelcomeEmail("".getProperty('bucket_prefix').".templates", $skin_name);
	}
	
	function rewindReadbody($body)
	{
		if ($body) {
			$body->rewind();			
			return $body->read($body->getContentLength());
		} else {
			return null;
		}
	}
	
	static function staticGetCrmTemplate()
	{
		$email_service = new EmailService();
		return $email_service->getCrmTemplate();
	}
	
	function getCrmTemplate() 
	{
		$bucket = "".getProperty('bucket_prefix').".templates";
		if($body = $this->aws_service->getKey($bucket, "administrative-emails/stats-email.html")) {
			return $body;
		}
	}
}