<?php

class EmmaService extends MarketingService
{
    const EMMA_THIRD_PARTY_PROVIDER_ID = 6003;
    const OPT_IN_FIELD_NAME_ON_REMOTE_SYSTEM = 'opt-in';

    public function __construct($brand_id)
    {
        $this->service_name = 'Emma';
        $this->third_party_provider_id = self::EMMA_THIRD_PARTY_PROVIDER_ID;
        $this->join_fields = ['email','first_name','last_name','birthday','opt-in'];
        parent::__construct($brand_id);
    }

    function setOptInOnResource(&$resource)
    {
        $resource->set(self::OPT_IN_FIELD_NAME_ON_REMOTE_SYSTEM,1);
    }

    function validate_optin($resource)
    {
        return isset($resource->marketing_email_opt_in);
    }

    function createJoinPayload($resource)
    {
        $payload = [];
        $payload['email'] = $resource->email;
        //$payload['fields'] = ['first_name'=>$resource->first_name,'last_name'=>$resource->last_name];
        $payload['fields'] = [];
        foreach ($this->join_fields as $field_name) {
            if ($field_name == 'email') {
                continue;
            }
            $payload['fields']["$field_name"] = $resource->$field_name;
        }
        //$payload['groups'] = $groups;
        return $payload;
    }

    function join($resource)
    {
        $results = parent::join($resource);

        if ($results) {
            $raw_results_array = json_decode($results['raw_result'],true);
            return array_merge($results,$raw_results_array);
        }
        return false;
    }

    function curlIt($data)
    {
        $account_id = $this->marketing_service_brand_info['account_id'];
        $credentials = $this->marketing_service_brand_info['public_api_key'].':'.$this->marketing_service_brand_info['private_api_key'];
        $emma_url = getProperty('emma_url');
        $url = str_replace('%%account_id%%',$account_id,$emma_url);
        return EmmaCurl::curlIt($url,$data,$credentials);
    }

    function isSuccessfulResponse($response)
    {
        $response_array  = $this->processCurlResponse($response);
        if ($response_array['added'] == true) {
            return true;
        } else {
            // do we actually do anything here ?
            myerror_log("Some error from Emma adding the user. ".$response_array['raw_result']);
            return false;
        }
    }
}
?>