<?php

class GrubhubService extends SplickitService
{
    const GRUBHUB_THIRD_PARTY_PROVIDER_ID = 1008;
    const NEW_LINE = PHP_EOL;
    private $client_id;
    private $secret;
    private $issue_date;
    private $nonce= "RRT:TEST";
    private $hostUrl;
    private $body;
    private $request_method;
    private $port;
    private $ext;
    private $uri;

    function __construct($data)
    {
        parent::__construct($data);
        $this->client_id  = getProperty("grubhub_client_id");
        $this->secret  = getProperty("grubhub_secret");
        $this->issue_date = getProperty("grubhub_issue_date");
        $this->hostUrl = getProperty('grubhub_api_host');
        $this->ext ="";
        $this->port = 443;
    }

    function getHostUrl(){
        return $this->hostUrl;
    }

    function getSecret(){
        return $this->secret;
    }

    function getClientId(){
        return $this->client_id;
    }

    function setUri($merchantId){
        $this->uri = "/pos/v1/merchant/".$merchantId."/pos-status";
    }

    function getUri(){
        return $this->uri;
    }

    function getRequestUrl(){
        return $this->hostUrl.$this->uri;
    }

    function generateOfflineBody(){
        $this->body = array("reason" => "POS communication is offline", "new_pos_status"=> "OFFLINE");
    }

    function generateOnlineBody(){
        $this->body = array("reason" => "POS communication is online", "new_pos_status"=> "ONLINE");
    }

    function getBody(){
        return $this->body;
    }

    function getPort(){
        return $this->port;
    }

    function getExt(){
        return $this->ext;
    }

    function setRequestMethod($method){
        return $this->request_method = $method;
    }

    function getRequestMethod(){
        return $this->request_method;
    }

    function generateNonce(){
        $time_seconds = time() * 24 * 60 * 60;
        $issue_seconds = intval($this->issue_date) * 24 * 60 * 60;
        $difference = $issue_seconds - $time_seconds;
        $last_seven = substr($difference, -7,-2);
        $permitted_chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $random_chars = substr(str_shuffle($permitted_chars), 0, 5);
        $this->nonce = $last_seven.':'.$random_chars;
    }

    function getNonce(){
        return $this->nonce;
    }

    function hashBody($body){
        if(!$body){
            return "";
        }
        $body = json_encode($body);
        return base64_encode(hash('SHA256', $body, true));
    }

    function normalizeRequest($nonce, $request_method, $request_uri, $host, $port, $body_hash, $ext){
        $normalized_request = $nonce;
        $normalized_request .= self::NEW_LINE;
        $normalized_request .= strtoupper($request_method);
        $normalized_request .= self::NEW_LINE;
        $normalized_request .= $request_uri;
        $normalized_request .= self::NEW_LINE;
        $normalized_request .= strtolower($host);
        $normalized_request .= self::NEW_LINE;
        $normalized_request .= $port;
        $normalized_request .= self::NEW_LINE;
        $normalized_request .= $body_hash;
        $normalized_request .= self::NEW_LINE;
        $normalized_request .= $ext;
        $normalized_request .= self::NEW_LINE;
        return $normalized_request;
    }

    function hashNormalizedRequest($request, $secret){
        return base64_encode(hash_hmac('sha256', $request, $secret,true));
    }

    function createHeaderValue($client_id, $nonce, $body_hash, $ext, $mac){
        $header_value = "";
        $header_value .= "MAC id=\"";
        $header_value .= $client_id;
        $header_value .= "\",nonce=\"";
        $header_value .= $nonce;
        if(isset($body_hash)){
            $header_value .= "\",bodyhash=\"";
            $header_value .= $body_hash;
        }
        if($ext){
            $header_value .= "\",ext=\"";
            $header_value .= $ext;
        }
        $header_value .= "\",mac=\"";
        $header_value .= utf8_decode($mac);
        $header_value .= "\"";
        return $header_value;
    }

    function setMerchantStatus($endpoint, $authHeader, $data)
    {
        $response = $this->curlIt($endpoint, $authHeader, $data);
        logData($response, "grubhub response",5);
        return $response;
    }

    function curlIt($endpoint, $authHeader, $data)
    {
        myerror_logging(3,"about to curl Grubhub to endpoint: ".$endpoint);
        logData($data, "grubhub info",5);
        $response = GrubhubCurl::curlIt($endpoint,$authHeader,$data);
        logData($response, "Grubhub raw result Response",3);
        $this->curl_response = $response;
        return $this->processCurlResponse($response);
    }

    function processCurlResponse($response)
    {
        $result_array = parent::processCurlResponse($response);
        $this->response_in_data_format = $result_array;
        return $result_array;
    }

    function processRawReturn($raw_return)
    {
        $return_array = json_decode($raw_return,true);
        $relavant_payload = $return_array['data'];
        if (isset($return_array['errors']) && sizeof($return_array['errors']) > 0) {
            $relavant_payload['error'] = $return_array['errors'];
        }
        return $relavant_payload;
    }
    function getGHMerchant($merchant_id){
        $tpppmma = new ThirdPartyProviderParameterMerchantMapsAdapter(getM());
        try{
            if ($paramters = $tpppmma->getMerchantParameters($merchant_id,GrubhubService::GRUBHUB_THIRD_PARTY_PROVIDER_ID)) {
                if ($grubhub_merchant_id = $paramters['GrubHub']['grubhub_merchant_id']) {
                    return $grubhub_merchant_id;
                } else {
                    myerror_log("Couldn't find GrubHub Merchant mapping for $merchant_id",5);
                }
            } else {
                myerror_log("Couldn't find GrubHub Merchant mapping for $merchant_id",5);
            }
            return null;
        }catch (Exception $e){
            myerror_log("Couldn't find GrubHub Merchant mapping for $merchant_id",5);
            return null;
        }
    }

}