<?php

/**
 * Created by PhpStorm.
 * User: Elvis
 * Date: 8/15/16
 * Time: 8:03 AM
 */
class MailChimpService extends MarketingService
{
    var $base_url;
    var $response = [];
    var $errors = [];
    private $mail_chimp_parameters_for_skin = [];

    const BIRTHDAY_ERROR_MESSAGE =  "Sorry, birthday must be in the form of mm/dd/YYYY. please try again";
    const MAILCHIMP_THIRD_PARTY_PROVIDER_ID = 6001;
    const OPT_IN_FIELD_NAME_ON_REMOTE_SYSTEM = 'status';

    function __construct()
    {
        $this->base_url = getProperty('mailchimp_url');
        $this->mail_chimp_parameters_for_skin = getThirdPartyParametersForMerchantSkinCombination(self::MAILCHIMP_THIRD_PARTY_PROVIDER_ID,getSkinIdForContext(),null,null);
        if (! isset($this->mail_chimp_parameters_for_skin['mailchimp_mail_list'])) {
            MailIt::sendErrorEmail("Error With Mailchimp MAIL LIST", "No value for mailchimp_mail_list set up in third party parameter skin table.  skin: " . getSkinNameForContext());
            throw new Exception("Mail chimp not set up correctly for skin.  No mailchimp_mail_list variable set in third party skin table");
        }
        if (! isset($this->mail_chimp_parameters_for_skin['mailchimp_api_key'])) {
            MailIt::sendErrorEmail("Error With Mailchimp API KEY", "No value for mailchimp_api_key set up in third party parameter skin table.  skin: " . getSkinNameForContext());
            throw new Exception("Mail chimp not set up correctly for skin.  No mailchimp_api_key variable set in third party skin table");
        }
        $data_center = $this->getDataCenterFromApiKey($this->mail_chimp_parameters_for_skin['mailchimp_api_key']);
        $this->base_url = str_replace('%%data_center%%',$data_center,$this->base_url);
    }

    function setOptInOnResource(&$resource)
    {
        $resource->set(self::OPT_IN_FIELD_NAME_ON_REMOTE_SYSTEM,'subscribed');
    }

    function createJoinPayload($resource)
    {
        $parameters["email_address"] = $resource->email;
        $parameters["status"] = "subscribed";
        $parameters["merge_fields"] = array(
            "FNAME" => $resource->first_name,
            "LNAME" => $resource->last_name,
            "BIRTHDAY" => $this->getBirthdayInCorrectFormat($resource->birthday)
        );
        return $parameters;

    }

    function getDataCenterFromApiKey($key)
    {
        $s = explode('-',$key);
        $data_center = array_pop($s);
        return $data_center;
    }

    function validateBirthday($date_of_birth)
    {
        $error_message = MailChimpService::BIRTHDAY_ERROR_MESSAGE;
        myerror_log("about to validate birthday in mail chimp: $date_of_birth");
        $valid = validateDate($date_of_birth);

        if ($valid === false) {
            $this->errors['birthday'] = $error_message;
        }
        return $valid;
    }

    function getBirthdayInCorrectFormat($birthday)
    {
        $b = explode('/',$birthday);
        return $b[0].'/'.$b[1];
    }

    function curlIt($data)
    {
        return $this->send($data);
    }

    function validateJoinFields($resource)
    {
        return true;
    }

    function send($data)
    {
//        $parameters["email_address"] = $data['email'];
//        $parameters["status"] = "subscribed";
//        $parameters["merge_fields"] = array(
//            "FNAME" => $data['first_name'],
//            "LNAME" => $data['last_name'],
//            "BIRTHDAY" => $this->getBirthdayInCorrectFormat($data["birthday"])
//        );

        $url = $this->base_url . "lists/" . $this->getMailChimpMailList() . "/members";
        $this->headers[] = "Authorization: apikey " . $this->getMailChimpApiKey();
        $response = MailChimpCurl::curlIt($url, $data, $this->headers);
        $this->response = $response;
        return $response;
    }

    function getMailChimpApiKey()
    {
        return $this->mail_chimp_parameters_for_skin['mailchimp_api_key'];
    }

    function getMailChimpMailList()
    {
        return $this->mail_chimp_parameters_for_skin['mailchimp_mail_list'];
    }
    

    function getError()
    {
        return $this->errors;
    }
}