<?php

class SessionMService extends SplickitService
{
    var $method;
    var $order_date_time_formatted;
    var $message_body;

    const SESSIONM_THIRD_PARTY_PROVIDER_ID = 3004;


    public function __construct($info)
    {
        $this->service_name = 'SessionM';
        $this->third_party_provider_id = self::SESSIONM_THIRD_PARTY_PROVIDER_ID;
        parent::__construct();
        $this->data = $info;
        if ($ids = $info['ids_for_third_party_parameters']) {
            $this->setThirdPartyParameters(self::SESSIONM_THIRD_PARTY_PROVIDER_ID,$ids['skin_id'],$ids['merchant_id'],$ids['brand_id']);
        }
        if ($method = $info['type']) {
            $this->method = $method;
        }
    }

    function createAndSendBatchUserFile()
    {
        $file_name = $this->createBatchUserFile();
        return $this->sendFile($file_name,'users/');
    }

    function createAndSendBatchMenuFile($menu_id)
    {
        $file_name = $this->createBatchMenuFile($menu_id);
        return $this->sendFile($file_name,'catalog/');
    }

    function createAndSendBatchMerchantFile()
    {
        $file_name = $this->createBatchMerchantFile();
        return $this->sendFile($file_name,'stores/');
    }

    function createAndSendBatchTransactionFile($date_string)
    {
        $d = explode(',',$date_string);
        $file_name = $this->createBatchTransactionsFile($d[0],$d[1]);
        return $this->sendFile($file_name,'transactions/');
    }

    function sendFile($file_name,$directory)
    {
        // do the stuff to send the file to the FTP site
        $command = './scripts/upload_sftp.sh '.'/usr/local/splickit/etc/keys/sftp_key '.'ljsilvers '.'sftp.ent-sessionm.com '.$file_name.' uploads/production/'.$directory;
        myerror_log("about to call this command: '$command'");
        $response = shell_exec($command);
        myerror_log("SessionM SFTP response: ".$response);
        // delete the file from the file system
        //unlink($file_name);
        return true;

        /* this is a fire and forget so there is no trapping since we dont get an error if it doesn't upload */
    }

    function createBatchUserFile()
    {
        $user_adapter = new UserAdapter(getM());
        $options[TONIC_FIND_BY_METADATA]['skin_id'] = 237;
//        $sql = "SELECT * FROM User WHERE skin_id = 237 ORDER BY 1 DESC LIMIT 10";
//        $options[TONIC_FIND_BY_SQL] = $sql;
        $user_resources = Resource::findAll($user_adapter,null,$options);
        $batch_users = [];
        $date = date("Ymd");
        $file_name = "./reportfiles/sessionm/users_$date.json";
        // delete previous copy
        unlink($file_name);
        $myfile = fopen($file_name, "w");
        fwrite($myfile, "[\r\n");
        $i = 1;
        $count = sizeof($user_resources);
        foreach ($user_resources as $user_resource) {
            $user_template = $this->createUserPayLoad($user_resource->getDataFieldsReally())['user'];
            $json = json_encode($user_template);
            $comma = ',';
            if ($i == $count) {
                $comma = '';
            }
            $r = fwrite($myfile, "$json $comma \r\n");
            $i++;
        }
        fwrite($myfile, "]");
        fclose($myfile);
        return $file_name;
    }


    function createBatchMerchantFile()
    {
        $merchant_adapter = new MerchantAdapter(getM());
        $options[TONIC_FIND_BY_METADATA]['brand_id'] = 517;
        //$options[TONIC_FIND_BY_METADATA]['active'] = 'Y';
        $merchant_resources = Resource::findAll($merchant_adapter,null,$options);
        $date = date("Ymd");
        $file_name = "./reportfiles/sessionm/venu_$date.csv";
        // delete previous copy
        unlink($file_name);
        $myfile = fopen($file_name, "w");
        //fwrite($myfile, "Venu [\r\n");
        $i = 1;
        $count = sizeof($merchant_resources);
        foreach ($merchant_resources as $merchant_resource) {
            $merchant_template = $this->createmerchantPayLoad($merchant_resource->getDataFieldsReally());
            $line = "";
            $headers = "";
            foreach ($merchant_template as $name=>$value) {
                if ($i == 1) {
                    $headers.=$name.',';
                }
                if ($name == 'store_id' || $name == 'lat' || $name == 'lng') {
                    $line.=$value.',';
                } else {
                    $line.='"'.$value.'",';
                }

            }
            if ($i == 1) {
                $headers = substr($headers,0,-1);
                $r = fwrite($myfile, $headers."\r\n");
            }
            $line = substr($line,0,-1);
            $r = fwrite($myfile, $line."\r\n");
            $i++;
        }
        fwrite($myfile, "");
        fclose($myfile);
        return $file_name;
    }

    function createBatchMenuFile($menu_id_string)
    {
        $menu_ids = explode(',',$menu_id_string);
        $menu_types = [];
        $catalog = array();
        foreach ($menu_ids as $menu_id) {
            $complete_menu = CompleteMenu::getCompleteMenu($menu_id);
            foreach ($complete_menu['menu_types'] as $menu_type) {
                $menu_type_template = $this->createMenuTypePayLoad($menu_type);
                $menu_types[] = $menu_type_template;
            }
        }

        $catalog[] = ['nodes'=>$menu_types];
        $menu_json = json_encode($catalog);

        $menu_id_string = str_replace(',','-',$menu_id_string);
        $date = date("Ymd");
        $file_name = "./reportfiles/sessionm/catalog_".$menu_id_string."_".$date.".json";
        // delete previous copy
        unlink($file_name);
        $myfile = fopen($file_name, "w");
        fwrite($myfile,$menu_json);
        //fwrite($myfile, "]");
        fclose($myfile);
        return $file_name;
    }

    function createBatchTransactionsFile($date_string,$end_date_string)
    {
        $orders_adapter = new OrderAdapter(getM());
        $brand_id = $this->data['ids_for_third_party_parameters']['brand_id'];
        $sql = "SELECT a.order_id,a.promo_id,a.promo_amt,b.promo_type,b.description as promo_description,c.external_id as promo_external_id_for_brand,a.order_dt_tm,a.total_tax_amt,a.grand_total,a.modified,a.merchant_id,m.merchant_external_id ";
        $sql .= "FROM Orders a JOIN Merchant m ON a.merchant_id = m.merchant_id LEFT OUTER JOIN Promo b ON a.promo_id = b.promo_id LEFT OUTER JOIN Promo_Brand_External_Id_Map c ON a.promo_id = c.promo_id WHERE m.brand_id = $brand_id and status = 'E' and a.created >= '$date_string 00:00:01' and a.created < '$end_date_string 00:00:01' order by a.order_id desc";
        $options[TONIC_FIND_BY_SQL] = $sql;
        $order_records = $orders_adapter->getRecords([],$options);
        $orders = [];
        foreach ($order_records as $order_record) {
            $order_id = $order_record['order_id'];
            $order_record['order_details'] = [];
            $order_record['payments'] = [];
            $orders[$order_id]= $order_record;
        }



        $sql = "SELECT b.order_id,b.order_detail_id,b.item_size_id,b.quantity,b.price,b.item_total ";
        $sql .= " FROM Orders a JOIN Merchant m ON a.merchant_id = m.merchant_id JOIN Order_Detail b ON a.order_id = b.order_id ";
        $sql .= " WHERE m.brand_id = $brand_id and status = 'E' and a.created > '$date_string 00:00:01' and a.created < '$end_date_string 00:00:01' order by a.order_id desc";
        $options[TONIC_FIND_BY_SQL] = $sql;
        $order_detail_records = $orders_adapter->getRecords([],$options);
        //$order_detail_resources = Resource::findAll($orders_adapter,null,$options);

        foreach ($order_detail_records as $order_detail_record) {
            $order_id = $order_detail_record['order_id'];
            $orders[$order_id]['order_details'][] = $order_detail_record;
        }

        $sql = "SELECT b.* ";
        $sql .= " FROM Orders a JOIN Merchant m ON a.merchant_id = m.merchant_id JOIN Balance_Change b ON a.order_id = b.order_id ";
        $sql .= " WHERE m.brand_id = $brand_id and a.status = 'E' and a.created > '$date_string 00:00:01' and a.created < '$end_date_string 00:00:01' and b.charge_amt > -.01 and (process = 'Cash' OR process = 'CCpayment') order by a.order_id desc";
        $options[TONIC_FIND_BY_SQL] = $sql;
        $balance_change_records = $orders_adapter->getRecords([],$options);
        foreach ($balance_change_records as $balance_change_record) {
            $order_id = $balance_change_record['order_id'];
            $orders[$order_id]['payments'][] = $balance_change_record;
        }

        $file_name = "./transactions_from_$date_string.json";
        $myfile = fopen($file_name, "w");
        fwrite($myfile, "[\r\n");
        $i = 1;
        $count = sizeof($orders);
        foreach ($orders as $order_info) {
//            if ($i > 10) {
//                continue;
//            }
            $transaction_payload_for_order = $this->createBatchTransactionPayload($order_info);
            $json = json_encode($transaction_payload_for_order);
            $comma = ',';
            if ($i == $count) {
                $comma = '';
            }
            $r = fwrite($myfile, "$json $comma \r\n");
            $i++;
        }
        fwrite($myfile, "]");
        fclose($myfile);
        return $file_name;
    }

    function getPaymentNode($payments_array)
    {
        $payments = [];
        foreach ($payments_array as $payment) {

            switch($payment['process']) {
                case 'CCpayment':
                case 'Authorize':
                    $payment_proccess = 'CreditCard';
                    break;
                case 'Cash': $payment_proccess = 'Instore Payment';
                    break;
                case 'LoyaltyBalancePayment': $payment_proccess = 'Loyalty';
                    break;
                case 'DoorDash': $payment_proccess = 'DoorDash';
                    break;
                case 'UberEats': $payment_proccess = 'UberEats';
                    break;
                default : $payment_proccess = 'None Listed';
            }

            $payments[] = [
                "receipt_code" => "receipt-code-".getRawStamp(),
                "payment_id" => $payment['id'],
                "amount" => $payment['charge_amt'],
                "type" => $payment_proccess,
                "payment_time" => $this->order_date_time_formatted, //(yyyy-MM-ddTHH:mm:ss.fffZ)
                "user_id" => $payment['user_id'],
                "user_id_type" => "External_ID"
            ];
        }
        return $payments;
    }

    function getOrderDateTimeFormatted($time_stamp)
    {
        $date_time = new DateTime();
        $date_time->setTimezone(new DateTimeZone('UTC'));
        $date_time->setTimestamp($time_stamp);
        $formatted = $date_time->format("Y-m-d\TH:i:s.000\Z");
        return $formatted;
    }

    function createMenuTypePayLoad($menu_type)
    {
        $subcategories = [];
        foreach ($menu_type['menu_items'] as $item) {
           $children = [];
           foreach ($item['size_prices'] as $item_size_map) {
               $children[] = ["id"=>$item_size_map["item_size_id"],
                   "name"=>$item_size_map["size_name"]."-".$item["item_name"]];
           }
           if ($menu_type['menu_id'] == 103047) {
               $suffix = '-Delivery';
           } else {
               $suffix = '-Pickup';
           }
           $subcategories[] = ["name"=>$item["item_name"].$suffix,
               "description"=>$item["description"],
               "category_id"=>$item["item_id"],"children"=>$children];
        }
        if (! validateThatStringFieldIsSetAndIsNotNullAndIsNotEmpty($menu_type,'menu_type_description')) {
            $menu_type["menu_type_description"] = $menu_type["menu_type_name"];
        }
        $template = [ "name"=>$menu_type["menu_type_name"].$suffix,
            "description"=>$menu_type["menu_type_description"],
            "category_id"=>$menu_type["menu_type_id"],
            "subcategories"=>$subcategories];
        return $template;
    }

    function createMerchantPayLoad($merchant)
    {
        $status = 'Active';
        if ($merchant['active'] == 'Y') {
            $status = 'Active';
        } else {
            $status = 'Inactive';
        }
        $merchant_id = $merchant['merchant_id'];
        $merchant_template = ["external_id"=>$merchant['merchant_external_id'],
            "store_id"=>$merchant_id,
            "brand"=>'Long John Silvers',
            "name"=>$merchant['display_name'],
            "lat"=>$merchant['lat'],
            "lng"=>$merchant['lng'],
            "address"=>$merchant['address1'].' '.$merchant['address2'],
            "city"=>$merchant['city'],
            "state"=>$merchant['state'],
            "zip"=>$merchant['zip'],
            "country"=>$merchant['city'],
            "phone_number"=>$merchant['phone_no'],
            "time_zone"=>$merchant['time_zone_string'],
            "dma"=>null,
            "status"=>$status,
            "website_url"=>"https://longjohnsilvers.rrtusa.net/merchants/$merchant_id?order_type=pickup",
            "geofence_radius"=>null
            ];
        return $merchant_template;

    }

    function createUserPayLoad($user)
    {
        $user_obj = ["external_id"=>$user['user_id'],
            "email"=>$user['email'],
            "first_name"=>$user['first_name'],
            "last_name"=>$user['last_name'],
            "external_id_type"=>"RRT", // MAV-48: Swapped to current from, // "external_ids"=>["RRT"=>[$user['user_id']]],
            "phone_numbers"=>[[
                    "phone_number"=>preg_replace('/[^0-9]/', '', $user['contact_no']),
                    "phone_type"=>"mobile",
                    "preference_flags"=>["primary"]
                    ]]
                ];
        if(isset($user['opted_in'])){
            $user_obj['opted_in']=$user['opted_in'];
        }
        return ['user'=>$user_obj];
    }

    function createTransactionPayload($complete_order)
    {
        $items = [];
        $this->order_date_time_formatted = $this->getOrderDateTimeFormatted($complete_order['order_dt_tm']);

        foreach ($complete_order['order_details'] as $item) {
            $items[] = [
                "line_id" => $item['order_detail_id'],
                "item_id" => $item['item_size_id'],
                "quantity" => $item['quantity'],
                "unit_price" => $item['price'],
                "tax_included" => 0,
                "subtotal" => $item['item_total']
            ];
        }

        $payment_node = $this->getPaymentNode($complete_order['payments']);
        $discount_node = $this->getDiscountNode($complete_order);

        $request_payload = [
            "pos_employee_id" => 1,
            "table_id" => 11,
            "channel" =>  'STORE',
            "transaction_id" => $complete_order['order_id'],
            "subtotal" => $complete_order['grand_total'] - $complete_order['total_tax_amt'],
            "tax_total" => $complete_order['total_tax_amt'],
            "open_time" => $this->order_date_time_formatted, // "A UTC time in RFC3339 ex => 2011-08-30T09 =>30 =>16.768-04 =>00."
            "modified_time" => $this->order_date_time_formatted,
            "is_closed" => true,
            "is_voided" => false,
            "items" => $items,
            "payments" => $payment_node,
            "discounts" => $discount_node
        ];

        $payload = [
            "store_id" => $this->third_party_parameters['store_guid'],
            "client_id" => $this->third_party_parameters['retailer_id'],
            "request_id" => 1,
            "request_payload" => $request_payload
        ];
        return $payload;
    }

    function createBatchTransactionPayload($complete_order)
    {
        $items = [];
        if ($complete_order['payments'][0]['created']) {
            $order_time_stamp = $complete_order['payments'][0]['created'];
        } else {
            $order_time_stamp = $complete_order['modified'];
        }
        $this->order_date_time_formatted = $this->getOrderDateTimeFormatted($order_time_stamp);

        foreach ($complete_order['order_details'] as $item) {
            $items[] = [
                "line_id" => $item['order_detail_id'],
                "item_id" => $item['item_size_id'],
                "quantity" => (int) $item['quantity'],
                "unit_price" => (float) $item['price'],
                "tax_included" => 0,
                "subtotal" => (float) $item['item_total']
            ];
        }

        $payment_node = $this->getPaymentNode($complete_order['payments']);
        $discount_node = $this->getDiscountNode($complete_order);

        $sub_total = (float) $complete_order['grand_total'] - $complete_order['total_tax_amt'];
        $total_tax = (float) $complete_order['total_tax_amt'];
        $request_payload = [
            //"store_id" => $this->third_party_parameters['store_guid'],
//            "store_id" => "c39a3b91-fd49-463c-9fae-18812158e6a6",
            "store_id" => $complete_order['merchant_external_id'],
//            "client_id" => $this->third_party_parameters['retailer_id'],
            "client_id" => "8be53b45-db1c-4d09-a876-7ee5b4b69570",
            "request_id" => "1",
            "pos_employee_id" => "1",
            "table_id" => "11",
            "channel" =>  'STORE',
            "is_closed" => true,
            "is_voided" => false,
            "transaction_id" => $complete_order['order_id'],
            "subtotal" => $sub_total,
            "tax_total" => $total_tax,
            "open_time" => $this->order_date_time_formatted, // "A UTC time in RFC3339 ex => 2011-08-30T09 =>30 =>16.768-04 =>00."
            "modified_time" => $this->order_date_time_formatted,
            "items" => $items,
            "payments" => $payment_node,
            "discounts" => $discount_node
        ];
        return $request_payload;
    }

    function getDiscountNode($complete_order)
    {
        $discount_node = [];
        if ($complete_order['promo_amt'] < 0) {
            $discount_node[] = [
                "reference_id"=> $complete_order['promo_id'],
                "reference_id_type"=> $complete_order['promo_type'],
                "pos_discount_id" =>  $complete_order['promo_external_id_for_brand'],
                "status" => "offline",
                "display_name" =>  $complete_order['promo_description'],
                "image_url" =>  null,
                "discounted_line_ids" =>  null,
                "discount_source" => "POS",
                "amount" => $complete_order['promo_amt'],
                "stack_order" => 0,
                "applied_time" => $this->order_date_time_formatted,
                "user_id" => $complete_order['user_id']
            ];
        }
        return $discount_node;
    }

    function send($complete_order)
    {
        if ($this->method == 'transaction') {
            //first need to do a create user call
            $payload_hash = $this->createUserPayLoad($complete_order['user']);
            // https://api-ljsilvers.ent-sessionm.com/priv/v1/apps/%%app_key%%/users/%%user_id%%
            $url = $this->third_party_parameters['customer_url'];
            $url = str_replace('%%app_key%%', $this->third_party_parameters['customer_app_key'], $url);
            $url = str_replace('%%user_id%%', $complete_order['user_id'], $url);

            $username_password = $this->third_party_parameters['customer_app_key'] . ':' . $this->third_party_parameters['customer_secret'];
            $raw_response = $this->curlIt($payload_hash, $url, $username_password);
            if ($raw_response['http_code'] == 200) {
                myerror_log("Success on creating user at SessionM");
            } else {
                myerror_log("ERROR CREATING SESSION M USER: ".$raw_response['raw_result']);
                ;// what to do if it fails?  user probably exists so just go ahead
            }

            $payload_hash = $this->createTransactionPayload($complete_order);
            $url = $this->third_party_parameters['transaction_url'];
            $username_password = $this->third_party_parameters['transaction_app_key'] . ':' . $this->third_party_parameters['transaction_secret'];

            $this->message_body = json_encode($payload_hash);
            $raw_response = $this->curlIt($payload_hash, $url, $username_password);
            $response = $this->processCurlResponse($raw_response);

            if ($response['status'] == 'success') {
                return true;
            } else {
                // what to do if it fails?  let the logic reschedule it i guess.
                return false;
            }
        }
    }

    function createStandardUser($user)
    {
        $payload_hash = $this->createUserPayLoad(json_decode(json_encode($user), true));
        $url = $this->third_party_parameters['new_customer_url'];
        $url = str_replace('%%app_key%%', $this->third_party_parameters['customer_app_key'], $url);
        $username_password = $this->third_party_parameters['customer_app_key'] . ':' . $this->third_party_parameters['customer_secret'];
        $raw_response = $this->curlIt($payload_hash, $url, $username_password);
        if ($raw_response['http_code'] == 200) {
            myerror_log("Success on creating user at SessionM");
            return true;
        } else {
            myerror_log("ERROR CREATING SESSION M USER: ".$raw_response['raw_result']);
            ;// what to do if it fails?  user probably exists so just go ahead
            return false;
        }

    }
    function findUser($user)
    {
        $url = $this->third_party_parameters['customer_search_url'];
        $url = str_replace('%%app_key%%', $this->third_party_parameters['customer_app_key'], $url);
        $url = str_replace('%%mailto%%', $user->email, $url);

        $username_password = $this->third_party_parameters['customer_app_key'] . ':' . $this->third_party_parameters['customer_secret'];
        $raw_response = $this->curlIt(null, $url, $username_password);
        if ($raw_response['http_code'] == 200) {
            myerror_log("Success on creating user at SessionM");
            return true;
        } else {
            myerror_log("ERROR CREATING SESSION M USER: ".$raw_response['raw_result']);
            ;// what to do if it fails?  user probably exists so just go ahead
            return false;
        }
    }
    function updateStandardUser($user)
    {
        $payload_hash = $this->createUserPayLoad(json_decode(json_encode($user), true));
        $url = $this->third_party_parameters['customer_url'];
        $url = str_replace('%%app_key%%', $this->third_party_parameters['customer_app_key'], $url);
        $url = str_replace('%%user_id%%', $user->user_id, $url);

        $username_password = $this->third_party_parameters['customer_app_key'] . ':' . $this->third_party_parameters['customer_secret'];
        $raw_response = $this->curlIt($payload_hash, $url, $username_password);
        if ($raw_response['http_code'] == 200) {
            myerror_log("Success on updating user at SessionM");
            return true;
        } else {
            myerror_log("ERROR updating SESSION M USER: ".$raw_response['raw_result']);
            ;// what to do if it fails?  user probably exists so just go ahead
            return false;
        }
    }
    function deleteStandardUser($user)
    {
        $url = $this->third_party_parameters['customer_url'];
        $url = str_replace('%%app_key%%', $this->third_party_parameters['customer_app_key'], $url);
        $url = str_replace('%%user_id%%', $user->user_id, $url);

        $username_password = $this->third_party_parameters['customer_app_key'] . ':' . $this->third_party_parameters['customer_secret'];
        $raw_response = $this->curlIt(null, $url, $username_password, 'DELETE');
        if ($raw_response['http_code'] == 200) {
            myerror_log("Success on deleting a user at SessionM");
            return true;
        } else {
            myerror_log("ERROR DELETING SESSION M USER: ".$raw_response['raw_result']);
            ;// what to do if it fails?  user probably exists so just go ahead
            return false;
        }
    }

    function curlIt($data,$url,$username_password, $method='GET')
    {
        $headers = []; // might need headers
        return SessionMCurl::curlIt($url,$data,$username_password,$headers, $method);
    }


    function processCurlResponse($response)
    {
        return parent::processCurlResponse($response); // TODO: Change the autogenerated stub
    }

    function getMessageBody()
    {
        return $this->message_body;
    }
}
?>
