<?php
class ThanxLoyaltyService extends SplickitLoyaltyService
{
    protected $thanx_client_id;
    protected $thanx_client_secret;
    protected $thanx_merchant_key;
    protected $thanx_redirect_url;
    protected $thanx_service_url;
    protected $thanx_legacy_service_url;
    protected $auth_user_info;

    var $location_uid;

    function __construct($data)
    {
        if (isset($data['merchant_id'])) {
            $merchant_id = $data['merchant_id'];
            $this->location_uid = $merchant_id;
        }
        $brand_id = isset($data['brand_id']) ? $data['brand_id'] : 0 ;
        $this->setData($data);
        if ($third_party_parameter_info = getThirdPartyParametersForMerchantSkinCombination(3003,getSkinIdForContext(),$merchant_id,$brand_id)) {
            $this->thanx_client_id = $third_party_parameter_info['client_id'];
            $this->thanx_client_secret = $third_party_parameter_info['secret'];
            $this->thanx_merchant_key = $third_party_parameter_info['merchant_key'];
            $this->thanx_redirect_url = $third_party_parameter_info['redirect_url'];
        } else {
            throw new Exception("Thanx Info Not Set Up For Skin");
        }

        $this->thanx_service_url = getProperty('thanx_url');
        $this->thanx_legacy_service_url = getProperty('thanx_legacy_url');
    }

    /** these are just for testing since the web will be doing all the authenticating and the API will just be validating the recieved token **/
    function setAuthUserInfoFromRawReturn($raw_return)
    {
        myerror_log("THE RAW RETURN IS: $raw_return",5);
        $this->auth_user_info = json_decode($raw_return,true);
        logData($this->auth_user_info,"thanx User Info",5);
    }

    function getAuthUserInfo()
    {
        return $this->auth_user_info;
    }

    function initiateAuthenticationProcess($user_info)
    {
        $uri = "/oauth/authorize";
        $payload = [];
        $payload['client_id'] = $this->getThanxClientId();
        $payload['redirect_uri'] = "https://".$this->getThanxRedirectUrl();
        $payload['response_type'] = "code";
        $payload['scope'] = 'passwordless';
        $payload['username'] = $user_info['email'];
        return $this->curlIt($uri,$payload,null, null,'POST');
    }

    function aquireAccessToken($code)
    {
        $uri = "/oauth/token";
        $payload = [];
        $payload['grant_type'] = 'authorization_code';
        $payload['client_id'] = $this->getThanxClientId();
        $payload['client_secret'] = $this->getThanxClientSecret();
        $payload['code'] = "$code";
        $payload['redirect_uri'] = "https://".$this->getThanxRedirectUrl();
        return $this->curlIt($uri,$payload,null, null,'POST');
    }

    function authenticateCode($code)
    {
        $response = $this->aquireAccessToken($code);
        if ($this->isSuccessfulResponse($response)) {
            return $this->processCurlResponse($response);
        } else {
            return false;
        }
    }
    
    
    function isValidAuthResponse($results)
    {
        if ($results['http_code'] == 200 || $results['http_code'] == 201) {
            $this->setAuthUserInfoFromRawReturn($results['raw_result']);
            return true;
        }
        return false;
    }

    function isValidAuthenticationToken($thanx_authentication_token)
    {
        return $this->isValidAuthResponse($this->authenticate($thanx_authentication_token));
    }

    /**   end testing functions  **/


    function createUser($user_data)
    {
        $uri = "/users/";
        $payload = ['user'=>$user_data];
        $response = $this->curlIt($uri,$payload,null, null,'POST');
        if ($this->isSuccessfulResponse($response)) {
            return $this->processCurlResponse($response);
        } else {
            $this->processCurlErrorResponse($response);
            return false;
        }
    }

    function updateUser($user_data, $access_token)
    {
        // code to update user on the remote Thanx system
        myerror_log("In thanx loyalty service. starting code to update user on Thanx system");

        $user = $this->getRemoteUserFields($access_token);
        $thanx_user_id = $user['user']['id'];
        $headers = ["Authorization: Bearer $access_token"];

        $uri = "/users/$thanx_user_id";
        $payload = ['user'=>$user_data];

        $response = $this->curlIt($uri,$payload, $headers, null,'PATCH');

        if ($this->isSuccessfulResponse($response)) {
            return $this->processCurlResponse($response);
        } else {
            $this->processCurlErrorResponse($response);
            return false;
        }
    }

    function processCurlErrorResponse($response)
    {
        if ($raw_return = $this->getRawResponse($response)) {
            $this->raw_return = $raw_return;
            $response['result'] = $this->processRawReturn($raw_return);
        }
        $response['error'] = $response['result']['error']['message'];
        $response['status'] = 'failed';
        logData($response, "Curl Response",3);
        $this->curl_response = $response;
        return $response;
    }

    function getRemoteUserFields($access_token)
    {
        $uri = "/users/me";
        $headers = ["Authorization: Bearer $access_token"];
        $response = $this->curlIt($uri,null,$headers, null,'GET');
        if ($this->isSuccessfulResponse($response)) {
            return $this->processCurlResponse($response);
        } else {
            return false;
        }
    }

    function getRemoteUserInfo($id,$access_token)
    {
        $uri = "/users/$id";
        $headers = ["Authorization: Bearer $access_token"];
        $response = $this->curlIt($uri,null,$headers, null,'GET');
        if ($this->isSuccessfulResponse($response)) {
            return $this->processCurlResponse($response);
        } else {
            return false;
        }
    }

    function getRemoteAccount($access_token)
    {
        $uri = "/api/account";
        if ($this->location_uid > 0 ) {
            $uri .= '?location_uid='.$this->location_uid;
        }
        $headers = ["Authorization: Bearer $access_token"];
        $response = $this->legacyCurlIt($uri,null,$headers, null,'GET');
        if ($this->isSuccessfulResponse($response)) {
            return $this->processCurlResponse($response);
        } else {
            return false;
        }
    }

    function getRewards($access_token)
    {
        $uri = "/rewards";
        $headers = ["Authorization: Bearer $access_token"];
        $response = $this->curlIt($uri,null,$headers, null,'GET');
        if ($this->isSuccessfulResponse($response)) {
            return $this->processCurlResponse($response);
        } else {
            return false;
        }
    }

    function processPromo($reward_id)
    {

    }

    function sendOrderInfo($complete_order,$access_token,$state = 'complete')
    {
        $uri = "/api/baskets";
        $headers = ["Authorization: Bearer $access_token"];
        if ($complete_order['thanx_reward']) {
            if ($complete_order['thanx_reward'] == 'remove_promo') {
                $reward = null;
            } else {
                $reward = $complete_order['thanx_reward'];
            }
        } else if (validateThatStringFieldIsSetAndIsNotNullAndIsNotEmpty($complete_order,'promo_code')) {
            $reward = $complete_order['promo_code'];
        }
        $data = $this->buildThanxBasket($complete_order,$state,$reward);
        $response = $this->legacyCurlIt($uri,$data,$headers, null,'POST');
        if ($this->isSuccessfulResponse($response)) {
            return $this->processCurlResponse($response);
        } else {
            myerror_log('Error processing Thanx order tracking: '.$response['raw_result']);
            return $this->processCurlResponse($response);
        }
    }

    function buildThanxBasket($complete_order,$state = 'complete',$reward = null)
    {
        $basket = [];
        $basket['id'] = $complete_order['ucid'];
        $basket['state'] = $state;
//        $basket["order_timestamp"] = "2019-05-08T18:02:05Z";
        $basket["location_uid"] = "".$complete_order['merchant_id'];
        if ($reward) {
            $basket['rewards'] = [$reward];
        } else {
            $basket['rewards'] = [];
        }


        if ($state == 'billed' || $state == 'completed') {
            $last_four = $complete_order['user']['last_four'];
            $basket['payments'] = [["issuer"=>"visa","last4"=>"$last_four","amount"=>floatval($complete_order['payments'][0]['charge_amt']),"authorized_at"=>date("c")]];
        }

        $basket['items'] = [];
        foreach ($complete_order['order_details'] as $order_detail) {
            $item = [];
            $item['id'] = $order_detail['external_id'];
            $item['name'] = $order_detail['item_name'];
            $item['price'] = floatval($order_detail['item_total_w_mods']);
            $item['categories'] = [$order_detail['menu_type_name']];
            $basket['items'][] = $item;
        }
        $basket['subtotal'] = floatval($complete_order['order_amt']);
        return $basket;
    }

//    function authenticate($thanx_authentication_token)
//    {
//        $uri = "/oauth/authorize";
//        $data['client'] = $this->thanx_client_id;
//        $data['security_token'] = $thanx_authentication_token;
//        return $this->curlIt($uri,$data,$headers,$user_password);
//    }
//
//    function rewardPoints($thanx_order_array,$thanx_user_authentication_token)
//    {
//        $uri = "/api/auth/checkins/online_order";
//        $user_password = "$thanx_user_authentication_token:x";
//        return $this->curlIt($uri,$thanx_order_array,$headers,$user_password);
//    }
//
//    function redemptionPurchase($thanx_order_array,$thanx_user_authentication_token)
//    {
//        $uri = "/api/auth/redemptions/online_order";
//        $user_password = "$thanx_user_authentication_token:x";
//        return $this->curlIt($uri,$thanx_order_array,$headers,$user_password);
//    }
//
//    function voidRedemption($data,$thanx_user_authentication_token)
//    {
//        $uri = "/api/auth/redemptions";
//        $user_password = "$thanx_user_authentication_token:x";
//        $data['client'] = $this->thanx_client_id;
//        return $this->curlIt($uri,$data,$headers,$user_password,'delete');
//    }
//
//    function getBalance($thanx_user_authentication_token)
//    {
//        $uri = "/api/auth/checkins/balance?client=".$this->thanx_client_id;
//        $user_password = "$thanx_user_authentication_token:x";
//        //$data['client'] = $this->thanx_client_id;
//        //$data['authentication_token'] = $thanx_user_authentication_token;
//        $response = $this->curlIt($uri,$data,$headers,$user_password);
//        if ($raw_result = $response['raw_result']) {
//            $response_array = json_decode($raw_result,true);
//            logData($response_array,"thanx Balance Response",3);
//            return $response_array;
//        }
//        myerror_log("SOME ERROR THROWN TRYING TO GET thanx POINTS: ".$response['error']);
//
//
//    }

    function generateSignature($uri,$method,$json_payload)
    {
        $uc_method = strtoupper($method);
        $string_to_sign = utf8_encode($this->getThanxClientId().",$uc_method,appliction/json,$json_payload,$uri");
        return hash_hmac("sha1", $string_to_sign, $this->thanx_client_secret);
    }

    function legacyCurlIt($uri,$data,$headers,$user_password,$method = 'GET')
    {
        $headers = $headers == null ? array() : $headers;
        $headers[] = "Accept: application/vnd.thanx-v1+json";
        $headers[] = "Merchant-Key: ".$this->thanx_merchant_key;
        $url = $this->thanx_legacy_service_url.$uri;
        $this->curl_response = ThanxCurl::curlIt($url,$data,$headers,$user_password,$method);
        return $this->curl_response;
    }

    function curlIt($uri,$data,$headers,$user_password,$method = 'GET')
    {
        $headers = $headers == null ? array() : $headers;
        if ($data) {
            $json = json_encode($data);
        }
        $x_signature = $this->generateSignature($uri,$json);
        $headers[] = 'X-Signature: '.$x_signature;

        $dt = new DateTime('now',new DateTimeZone('GMT'));
        $thanks_format_date = $dt->format("D, d M Y H:i:s e");

        $headers[] = "Date: $thanks_format_date";
        $headers[] = "Accept-Version: v4.0";
        $headers[] = "Accept: application/json";
        $headers[] = "X-ClientId: ".$this->getThanxClientId();
        $url = $this->thanx_service_url.$uri;
        $this->curl_response = ThanxCurl::curlIt($url,$data,$headers,$user_password,$method);
        return $this->curl_response;
    }

    function getThanxClientId()
    {
        return $this->thanx_client_id;
    }
    
    function getThanxClientSecret()
    {
        return $this->thanx_client_secret;
    }

    function getThanxRedirectUrl()
    {
        return $this->thanx_redirect_url;
    }

}
?>
