<?php

use phpFastCache\CacheManager;

class SplickitCache
{

    /**
     * @var string
     */
    private $key;

    /**
     * @var string
     */
    private $memcached_server;

    /**
     * @var ExtendedCacheItemPoolInterface
     */
    private $instance_cache;

    /**
     * @var ExtendedCacheItemInterface
     */
    private $cached_string;

    /**
     * @var string
     */
    private $environment_file;

    function __construct()
    {
        $this->memcached_server = getProperty('memcached_server');
        $this->environment_file = getProperty('loaded_environment');
        if ($this->byPassCache()) {
            return;
        } else {
            myerror_log("caching_log IO: About to get Cache Instance",3);
            $this->instance_cache = $this->getInstanceCache();
            myerror_log("caching_log IO: Retrieved Cache Instance",3);
        }
    }

    function processCachBustRequest($url)
    {
        if ($this->byPassCache()) {
            return null;
        }
        if (preg_match("%/brands/([0-9]{3,15})%", $url, $matches)) {
            $cache_key = "brand-".$matches[1];
            myerror_log("we have a bust cache call on the portal for $cache_key");
            $this->deleteCache($cache_key);
        } else if (preg_match("%/skins/([0-9]{3,15})%", $url, $matches)) {
            $skin_adapter = new SkinAdapter(getM());
            $cache_key = $skin_adapter->deleteSkinCacheFromSkinId($matches[1]);
        } else {
            $this->instanceFlushAll();
            $cache_key = "Entire Cache";
        }
        return Resource::dummyfactory(['result' => 'success',"resource" => $cache_key]);
    }


    function getCache($key)
    {
        if ($this->byPassCache()) {
            return null;
        }
        $this->key = $key;
        myerror_log("caching_log IO: GETTING SPLICKIT CACHE OBJECT RETRIEVER for key: $key",5);
        $this->cached_string = $this->instance_cache->getItem($key);
        myerror_log("caching_log IO: aquired SPLICKIT CACHE OBJECT RETRIEVER for key: $key",5);
        if (isLoggedInUserStoreTesterLevelOrBetter()) {
            myerror_logging(5,"caching_log: user is store tester or better. Do Not Check Cache, create new cache for $key");
            return null;
        } else if (getProperty("DO_NOT_CHECK_CACHE") == 'true') {
            if (isProd()) {
                myerror_log("caching_log: DO_NOT_CHECK_CACHE flag is set, do not use cache",3);
            }
            return null;
        } else if ($this->memcached_server == 'none') {
            myerror_log("caching_log: memcached server set to none. bypass caching",6);
            return null;
        }

        if ($value = $this->cached_string->get()) {
            if (strpos($key,'DESCRIBE') < 1) {
                if (substr($key,0,4) == 'menu') {
                    $log_string = 'menu';
                } else if (is_array($value)) {
                    $log_string = json_encode($value);
                } else if (is_a($value,'Resource')) {
                    $log_string = json_encode($value->getDataFieldsReally());
                } else {
                    $log_string = "$value";
                }
            }
            myerror_log("caching_log IO: getting SPLICKIT CACHE for key: $key - $log_string",5);
            return $value;
        } else {
            myerror_log("caching_log IO:  $key  DOES NOT EXIST.",5);
            return null;
        }

    }

    function setCache($key,$value,$expires_in_seconds)
    {
        if ($this->byPassCache()) {
            return null;
        }
        if ($_SERVER['DO_NOT_SAVE_TO_CACHE']) {
            myerror_log("bypassing set cache because server is set to do not save",6);
            return;
        } else if ($this->memcached_server == 'none') {
            myerror_log("caching_log: memcached server set to none. bypass caching",6);
            return;
        }
        $this->cached_string = $this->instance_cache->getItem($key);
        if (strpos($key,'DESCRIBE') < 1) {
            if (substr($key,0,4) == 'menu') {
                $log_string = 'menu';
                //$log_string = json_encode($value);
            } else if (is_array($value)) {
                $log_string = json_encode($value);
            } else if (is_a($value,'Resource')) {
                $log_string = json_encode($value->getDataFieldsReally());
            } else {
                $log_string = "$value";
            }


            myerror_log("caching_log IO: SAVING SPLICKIT CACHE for key: $key  -  $log_string",3);
            myerror_log("caching_log: saving to server: " . $this->memcached_server,3);
            $this->cached_string->set($value)->expiresAfter($expires_in_seconds);
            $this->instance_cache->save($this->cached_string);
            myerror_log("caching_log: saved $key to server.",3);
        } else {
            myerror_log("caching_log: saving $key to server.",5);
            $this->cached_string->set($value)->expiresAfter($expires_in_seconds);
            $this->instance_cache->save($this->cached_string);
            myerror_log("caching_log: saved $key to server.",5);
        }

    }

    function deleteCache($key)
    {
        if ($this->byPassCache()) {
            return null;
        }
        myerror_log("caching_log IO: DELETING SPLICKIT CACHE for key: $key ",3);
        $this->instance_cache->deleteItem($key);
    }

    function isLocalCache()
    {
        return $this->environment_file == "unit_test" || $this->environment_file == "unit_test_ide";
    }

    private function getInstanceCache()
    {
//        if (false && $this->isLocalCache()) {
//            // must get a new instance each time while unit tests run
//            return new phpFastCache\Drivers\Memcached\Driver(['servers' => [
//                [
//                    'host' =>$this->memcached_server,
//                    'port' => 11211,
//                    // 'sasl_user' => false, // optional
//                    // 'sasl_password' => false // optional
//                ]
//            ]]);
//       }
       return CacheManager::getInstance('memcached',['servers' => [
            [
                'host' =>$this->memcached_server,
                'port' => 11211,
                // 'sasl_user' => false, // optional
                // 'sasl_password' => false // optional
            ]
        ]]);
    }

    public function setMemcachedServer($server)
    {
        $this->memcached_server = $server;
    }

    static function getCacheFromKey($key)
    {
        $sc = new SplickitCache();
        return $sc->getCache($key);
    }

    static function deleteCacheFromKey($key)
    {
        $sc = new SplickitCache();
        return $sc->deleteCache($key);
    }

    static function flushAll()
    {
        $splickit_cache = new SplickitCache();
        $splickit_cache->instanceFlushAll();
    }

    function instanceFlushAll()
    {
        if ($this->byPassCache()) {
            return null;
        }
        $mem_server = getProperty('memcached_server');
        myerror_log("About to flush cache for $mem_server");
        $m = new Memcached();
        $m->addServer("$mem_server", 11211);
        $result = $m->flush();
        if ($result) {
            myerror_log("FLUSH CACHE RESULTS SUCCESS");
        } else {
            $code = $m->getResultCode();
            $ecode = $m->getLastErrorCode();
            myerror_log("FLUSH CACHE RESULTS ERROR code: $code   -   ecode: $ecode");
        }
        return $result;
    }

    function byPassCache()
    {
        if (getProperty('DO_NOT_CHECK_CACHE') == 'true') {
            return true;
        } else {
            return false;
        }
    }

}

?>