<?php


use Phinx\Migration\AbstractMigration;

class CreateThirdPartyProviders extends AbstractMigration
{
    /**
     * Change Method.
     *
     * Write your reversible migrations using this method.
     *
     * More information on writing migrations is available here:
     * http://docs.phinx.org/en/latest/migrations.html#the-abstractmigration-class
     *
     * The following commands can be used in this method and Phinx will
     * automatically reverse them when rolling back:
     *
     *    createTable
     *    renameTable
     *    addColumn
     *    addCustomColumn
     *    renameColumn
     *    addIndex
     *    addForeignKey
     *
     * Any other destructive changes will result in an error when trying to
     * rollback the migration.
     *
     * Remember to call "create()" or "update()" and NOT "save()" when working
     * with the Table class.
     */
    public function change()
    {
        $table = $this->table('Third_Party_Providers');
        $table
            ->addColumn('third_party_provider_type_id','integer',['null' => false])
            ->addColumn('name', 'string', ['null' => false])
            ->addColumn('description', 'string',['null' => false])
            ->addColumn('created','timestamp', array('default' => 'CURRENT_TIMESTAMP'))
            ->addColumn('modified','timestamp', array('default' => '0000-00-00 00:00:00'))
            ->addColumn('logical_delete','string',['limit' => 1,'default' => 'N'])
            ->addForeignKey('third_party_provider_type_id', 'Third_Party_Provider_Types', 'id', array('delete'=> 'CASCADE', 'update'=> 'NO_ACTION'))
            ->addIndex(array('third_party_provider_type_id','name'),array("unique"=>true,"name"=>'provider_type_name'));


        $table->create();

        $sql = "ALTER TABLE `Third_Party_Providers` AUTO_INCREMENT = 1000;";
        $this->execute($sql);

        $sql = "INSERT INTO Third_Party_Providers (`id`,`third_party_provider_type_id`,`name`,`description`)
                VALUES (1001,1000,'Brink','Brink POS'),
                (1002,1000,'Task Retail','Task Retail POS'),
                (1003,1000,'Vivonet','Vivonet POS'),
                (1004,1000,'Emagine','Emagine POS'),
                (1005,1000,'Foundry','Foundry POS'),
                (1006,1000,'Xoikos','Xoikos POS'),
                (2002,2000,'Sage','Sage'),
                (2001,2000,'Mercury','Mercury'),
                (2003,2000,'FPN','FPN'),
                (2004,2000,'PayEezy','PayEezy'),
                (2005,2000,'Heartland','Heartland'),
                (2006,2000,'Securenet','Securenet'),
                (2007,2000,'VioInstant','Vio Instant'),
                (2008,2000,'Elavon','Elavon'),
                (3001,3000,'Punch','Punch Loyalty'),
                (3002,3000,'Heartland','Loyalty'),
                (4001,4000,'Doordash','Doordash')";
        $this->execute($sql);

        $sql = "INSERT INTO Lookup_Master VALUES (null,'third_party_provider_pos','All third party pos providers',NOW(),NOW(),'N')";
        $this->execute($sql);
        $sql = "INSERT INTO Lookup_Master VALUES (null,'third_party_provider_cc_processor','All third party cc providers',NOW(),NOW(),'N')";
        $this->execute($sql);
        $sql = "INSERT INTO Lookup_Master VALUES (null,'third_party_provider_loyalty','All third party loyalty providers',NOW(),NOW(),'N')";
        $this->execute($sql);
        $sql = "INSERT INTO Lookup_Master VALUES (null,'third_party_provider_delivery','All third party delivery providers',NOW(),NOW(),'N')";
        $this->execute($sql);

        $sql = "INSERT INTO Lookup (`type_id_field`,`type_id_value`,`type_id_name`)
                VALUES ('third_party_provider_pos',1001,'Brink'),
                ('third_party_provider_pos',1002,'Task'),
                ('third_party_provider_pos',1003,'Vivonet'),
                ('third_party_provider_pos',1004,'Emagine'),
                ('third_party_provider_pos',1005,'Foundry'),
                ('third_party_provider_pos',1006,'Xoikos'),
                ('third_party_provider_cc_processor',2002,'Sage'),
                ('third_party_provider_cc_processor',2001,'Mercury'),
                ('third_party_provider_cc_processor',2003,'FPN'),
                ('third_party_provider_cc_processor',2004,'Payeezy'),
                ('third_party_provider_cc_processor',2005,'Heartland'),
                ('third_party_provider_cc_processor',2006,'Securenet'),
                ('third_party_provider_cc_processor',2007,'Vioinstant'),
                ('third_party_provider_loyalty',3001,'Punch'),
                ('third_party_provider_loyalty',3002,'Heartland'),
                ('third_party_provider_delivery',4001,'Doordash ')";
        $this->execute($sql);


    }
}
