<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class ABTaskRetailControllerTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__ . '-' . $_SERVER['STAMP'];
        $this->ids = $_SERVER['unit_test_ids'];

        $this->createSampleOrderAndMessageResource();
    }

    function tearDown()
    {
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }


    private static function assignSequentialPLUsToMenuItems($menu_id, $item_records)
    {
        $menu_item_adapter = new ItemAdapter($mimetypes);

        $curr_item_plu = 1000;
        $curr_mod_plu = 4000;
        foreach ($item_records as $item_record) {
            $menu_item_resource = Resource::find($menu_item_adapter, $item_record['item_id']);

            ABTaskRetailControllerTest::setPLUForMenuItem($item_record['item_id'], $curr_item_plu);
            $curr_item_plu++;
        }

        $modifier_size_map_adapter = new ModifierSizeMapAdapter($m);
        $modifier_size_map_adapter->allow_full_table_scan = true;
        $modifier_sizes = Resource::findAll($modifier_size_map_adapter, null);
        foreach ($modifier_sizes as $modifier_size) {
            $modifier_size->external_id = $curr_mod_plu;
            $modifier_size->save();
            $curr_mod_plu++;
        }
    }

    private function modifierItemMapsForMenuItem($item_id)
    {
        $item_modifier_item_map_adapter = new ItemModifierItemMapAdapter($mimetypes);
        $options = array();
        $options[TONIC_FIND_BY_METADATA] = array('item_id' => $item_id);
        $modifier_item_maps = Resource::findAll($item_modifier_item_map_adapter, null, $options);
        return $modifier_item_maps;
    }

    private function modifierItemsForModifierItemMenuMaps($modifier_item_maps)
    {
        $modifier_item_adapter = new ModifierItemAdapter($mimetypes);
        $ret_modifier_items = array();
        foreach ($modifier_item_maps as $modifier_item_map) {
            $options = array();
            $options[TONIC_FIND_BY_METADATA] = array('modifier_item_id' => $modifier_item_map->modifier_item_id);
            $modifier_item = Resource::findAll($modifier_item_adapter, null, $options);
            array_push($ret_modifier_items, $modifier_item);
        }
        return $ret_modifier_items;
    }

    private function modifierItemSizesForModifierItems($modifier_items)
    {
        $modifier_size_map_adapter = new ModifierSizeMapAdapter($mimetypes);
        $ret_sizes = array();
        foreach ($modifier_items as $modifier_item) {
            $options = array();
            $options[TONIC_FIND_BY_METADATA] = array('modifier_item_id' => $modifier_item[0]->modifier_item_id);
            $sizes = Resource::findAll($modifier_size_map_adapter, null, $options);
            foreach ($sizes as $size) {
                array_push($ret_sizes, $size);
            }
        }
        return $ret_sizes;
    }

    private function setPLUForMenuItem($item_id, $plu)
    {
        $item_size_adapter = new ItemSizeAdapter($mimetypes);
        $size_adapter = new SizeAdapter($mimetypes);

        $options = array();
        $options[TONIC_FIND_BY_METADATA] = array('item_id' => $item_id);
        $item_size = Resource::findExact($item_size_adapter, null, $options);
        $item_size->external_id = $plu;
        $item_size->save();
    }

    private function createSampleOrderAndMessageResource()
    {
        // User. Needs Credit card
        $user_resource = createNewUser(array("flags" => "1C20000001"));
        $user_id = $user_resource->user_id;
        logTestUserIn($user_id);

        // Menu
        $menu_id = $this->ids['menu_id'];

        // Test merchant
        $merchant_id = $this->ids['merchant_id'];

        // Order
        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'some note', 2);
        $order_data['lead_time'] = 20;
        $order_resource = placeOrderFromOrderData($order_data, getTodayTwelveNoonTimeStampDenver());
        error_log("Placed order: $order_resource");
        $order_id = $order_resource->order_id;

        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);

        // Create message resource
        $merchant_message_history_adapter = new MerchantMessageHistoryAdapter($mimetypes);
        $message_resource = $merchant_message_history_adapter->getExactResourceFromData(array("order_id" => $order_id, "message_format" => 'A'));

        // Assign instance variables
        $this->user_resource = $user_resource;
        $this->message_resource = $message_resource;
        $this->order_resource = $order_resource;
        $this->order_id = $order_id;
    }

    function testItemRecordSplit()
    {
        $task_retail_controller = new TaskRetailController($m, $user_id, $r);
        $item_record['item'] = 'item';
        $mod_record['mod_price'] = "5.00";
        $mod_record['external_id'] = '123:456';
        $item_record['order_detail_complete_modifier_list_no_holds'][] = $mod_record;
        $task_retail_controller->breakOutComboModifierFromItem($item_record);
        $this->assertCount(2, $item_record['order_detail_complete_modifier_list_no_holds']);
        $this->assertEquals('1.75', $item_record['order_detail_complete_modifier_list_no_holds'][0]['mod_price']);
        $this->assertEquals('123', $item_record['order_detail_complete_modifier_list_no_holds'][0]['external_id']);
        $this->assertEquals('3.25', $item_record['order_detail_complete_modifier_list_no_holds'][1]['mod_price']);
        $this->assertEquals('456', $item_record['order_detail_complete_modifier_list_no_holds'][1]['external_id']);
        $this->assertEquals('N', $item_record['order_detail_complete_modifier_list_no_holds'][1]['comes_with'], "Should have a NOT comes with flag");
    }

    function testLoadSettingForTaskRetailService()
    {
        $task_retail_service = new TaskRetailService($this->message_resource);
        $settings = $task_retail_service->currentTaskRetailSettings();

        $this->assertCount(4, $settings, "shoul be 4 settings");
        $this->assertEquals("http://136.179.5.40/xchangenetFR/services/xn_secureorders.asmx", $settings['task_retail_url']);
        $this->assertEquals("http://136.179.5.40/xchangenetFR/services/xn_authenticate.asmx?op=authenticateClient", $settings['task_retail_auth_url']);
        $this->assertEquals("154", $settings['task_retail_username']);
        $this->assertEquals("Splickit1", $settings['task_retail_password']);
    }

    function testLoadCustomSettingForTaskRetailService()
    {
        $menu_id = createTestMenuWithNnumberOfItems(5);
        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_resource->time_zone = -6;
        $merchant_resource->save();

        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()), array("merchant_id" => $merchant_resource->merchant_id, "message_format" => 'A', "delivery_addr" => "taskretail", "message_type" => "O"));
        attachMerchantToSkin($merchant_resource->merchant_id, $this->ids['skin_id']);


        Resource::createByData(new MerchantTaskRetailInfoMapAdapter(getM()), array(
            "brand_id" => 300,
            "merchant_id" => $merchant_resource->merchant_id,
            "location" => "97531",
            "media_id" => 5,
            "payment_id" => 18,
            'task_retail_url' => 'http://custom.domain/xchangenetFR/services/xn_custom_url_secureorders.asmx',
            'task_retail_auth_url' => 'http://custom.domain/xchangenetFR/services/xn_authenticate.asmx?op=authenticateClient',
            'task_retail_username' => 'customuser',
            'task_retail_password' => 'custompwd'));

        $sql = 'CALL load_third_party_from_taskretail_info()';
        $adapter = new ThirdPartyProviderParameterMerchantMapsAdapter(getM());
        $adapter->_query($sql);


        $order_adapter = new OrderAdapter(getM());
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id, 'pickup', 'some note', 2);
        $order_data['lead_time'] = 20;
        $order_resource = placeOrderFromOrderData($order_data, getTomorrowTwelveNoonTimeStampDenver());
        error_log("Placed order: $order_resource");
        $order_id = $order_resource->order_id;

        CompleteOrder::staticGetCompleteOrder($order_id, getM());

        // Create message resource
        $merchant_message_history_adapter = new MerchantMessageHistoryAdapter(getM());
        $message_resource = $merchant_message_history_adapter->getExactResourceFromData(array("order_id" => $order_id, "message_format" => 'A'));

        $task_retail_service = new TaskRetailService($message_resource);
        $settings = $task_retail_service->currentTaskRetailSettings();

        $this->assertCount(4, $settings, "shoul be 4 settings");
        $this->assertEquals("http://custom.domain/xchangenetFR/services/xn_custom_url_secureorders.asmx", $settings['task_retail_url']);
        $this->assertEquals("http://custom.domain/xchangenetFR/services/xn_authenticate.asmx?op=authenticateClient", $settings['task_retail_auth_url']);
        $this->assertEquals("customuser", $settings['task_retail_username']);
        $this->assertEquals("custompwd", $settings['task_retail_password']);
    }

    /**
     * @expectedException MissingSettingsForTaskRetailException
     */
    function testFailLoadSettingForTaskRetailService()
    {
        $menu_id = createTestMenuWithNnumberOfItems(5);
        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_resource->time_zone = -6;
        $merchant_resource->save();

        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()), array("merchant_id" => $merchant_resource->merchant_id, "message_format" => 'A', "delivery_addr" => "taskretail", "message_type" => "O"));
        attachMerchantToSkin($merchant_resource->merchant_id, $this->ids['skin_id']);


        Resource::createByData(new MerchantTaskRetailInfoMapAdapter(getM()), array(
            "brand_id" => 300,
            "merchant_id" => $merchant_resource->merchant_id,
            "location" => "97531",
            "media_id" => 5,
            "payment_id" => 18,
            'task_retail_url' => '',
            'task_retail_auth_url' => 'http://custom.domain/xchangenetFR/services/xn_authenticate.asmx?op=authenticateClient',
            'task_retail_username' => '',
            'task_retail_password' => 'custompwd'));

        $sql = 'CALL load_third_party_from_taskretail_info()';
        $adapter = new ThirdPartyProviderParameterMerchantMapsAdapter(getM());
        $adapter->_query($sql);


        $order_adapter = new OrderAdapter(getM());
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id, 'pickup', 'some note', 2);
        $order_data['lead_time'] = 20;
        $order_resource = placeOrderFromOrderData($order_data, getTomorrowTwelveNoonTimeStampDenver());
        error_log("Placed order: $order_resource");
        $order_id = $order_resource->order_id;

        CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);

        // Create message resource
        $merchant_message_history_adapter = new MerchantMessageHistoryAdapter($mimetypes);
        $message_resource = $merchant_message_history_adapter->getExactResourceFromData(array("order_id" => $order_id, "message_format" => 'A'));

        $task_retail_service = new TaskRetailService($message_resource);
    }

    function testComboSplitting()
    {
        $user_resource = createNewUser(array("flags" => "1C20000001"));
        $user_id = $user_resource->user_id;
        logTestUserIn($user_id);
        $merchant_id = $this->ids['merchant_id'];

        // Order
        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours', 1);
        $new_mods[] = $order_data['items'][0]['mods'][0];
        $new_mods[] = $order_data['items'][0]['mods'][1];
        $order_data['items'][0]['mods'] = $new_mods;
        $order_resource = placeOrderFromOrderData($order_data, $ts);
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id, $m, false);
        $resource = Resource::dummyFactory($complete_order);

        $resource->order_details[0]['order_detail_complete_modifier_list_no_holds'][1]['mod_price'] = "3.00";
        $resource->order_details[0]['order_detail_complete_modifier_list_no_holds'][1]['external_id'] = "12345:67890";

        $task_retail_controller = new TaskRetailController($m, $user_id, $r);
        $task_retail_controller->doComboModification($resource);
        $order_item_record = $resource->order_details[0];
        $this->assertEquals(3, count($order_item_record['order_detail_complete_modifier_list_no_holds']));
        $drink_mod = $order_item_record['order_detail_complete_modifier_list_no_holds'][1];
        $this->assertEquals(1.75, $drink_mod['mod_price'], "mod price should have been set to 1.75 since its the drink");
        $this->assertEquals("12345", $drink_mod['external_id'], "external id should have been stipped out to first one");
        $other_mod = $order_item_record['order_detail_complete_modifier_list_no_holds'][2];
        $this->assertEquals(1.25, $other_mod['mod_price'], "mod price should have been set to 1.25 since its the other item");
        $this->assertEquals("67890", $other_mod['external_id'], "external id should have been stipped out to second one");
        $this->assertEquals('N', $other_mod['comes_with'], "Should have a NOT comes with flag");
    }

    function testSetTaskRetailPickupAndOrderTimes()
    {
        $ts = getTimeStampForDateTimeAndTimeZone(12, 0, 0, 2, 14, 2014, 'America/Denver');
        $order_data['pickup_dt_tm'] = $ts + 1200;
        $order_data['order_dt_tm'] = $ts;
        $resource = Resource::dummyfactory($order_data);
        $task_retail_controller = new TaskRetailController($m, $u, $r);
        $resource = $task_retail_controller->setTaskRetailPickupAndOrderTimeStringsOnResource($resource);
        $this->assertEquals("2014-02-14T11:20:00.0000", $resource->task_retail_pickup_time_string, "pickup time should have been set to corect string for pacific");
        $this->assertEquals("2014-02-14T11:00:00.0000", $resource->task_retail_order_time_string, "order time should have been set to corect string for pacific");

    }

    function testTaskRetailControllerFactory()
    {
        $mmha = new MerchantMessageHistoryAdapter($mimetypes);
        $message_resource = $mmha->createMessageReturnResource(1080, 1234567, 'A', 'TaskRetail', time(), "O", $info, $message_text);
        $message_controller = ControllerFactory::generateFromMessageResource($message_resource, $mimetypes, $user, $request, $log_level);
        $this->assertTrue(is_a($message_controller, 'TaskRetailController'), 'Should have created a TaskRetailController');
    }


    /*
      TODO More robust template test
      TODO Test that held, comes with, and add modifiers display correctly. Holds should not be sent.

    function testPrepMessageForSending()
    {
        $request = null; // Don't need request for this test. Header/URL, the like

        $mid = $this->order_resource->merchant_id;
        Resource::createByData(new MerchantTaskRetailInfoMapAdapter($m), array("brand_id" => 300, "merchant_id" => $mid, "location" => "97531", "media_id" => 5, "payment_id" => 18));
        $task_retail_controller = new TaskRetailController($mimetypes, $user, $request, 5);
        $this->assertNotNull($task_retail_controller, 'Oops. $task_retail_controller was null');

        $date = new DateTime();
        $date->setDate(2020, 1, 1);
        $date->setTime(14, 1, 2);
        $this->order_resource->pickup_dt_tm = getMySqlFormattedDateTimeFromTimeStampAndTimeZone($date->getTimestamp());
        $this->order_resource->order_dt_tm = getMySqlFormattedDateTimeFromTimeStampAndTimeZone($date->getTimestamp());
        $this->order_resource->save();

        $this->user_resource->email = 'testy.mctesterson@order140.com';
        $this->user_resource->save();

        $merchant_message_history_adapter = new MerchantMessageHistoryAdapter($m);
        $message_resource = $merchant_message_history_adapter->getExactResourceFromData(array("order_id" => $this->order_id, "message_format" => 'A'));

        $resource = $task_retail_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($resource->message_text, 'The message text for the resource using Task Retail was null.');
        error_log("Returned message text: $resource->message_text");
        $this->assertContains("<SaleName>Web Order</SaleName>", $resource->message_text);
        $this->assertNotContains("<PLU></PLU>", $resource->message_text);
        $this->assertRegExp('/<PLU>.+<\/PLU>/', $resource->message_text);
        $this->assertContains("<MediaID>5</MediaID>", $resource->message_text);
        $this->assertContains("<MediaID>18</MediaID>", $resource->message_text);

//    $expected_string = cleanUpDoubleSpacesCRLFTFromString(file_get_contents("./unit_tests/resources/expected_task_retail_message_body_v2.txt"));
//    $expected_string = str_replace('%%grand_total%%', $this->order_resource->grand_total, $expected_string);
//    $expected_string = str_replace('%%total_tax_amt%%', $this->order_resource->total_tax_amt, $expected_string);
        $actual_string = cleanUpDoubleSpacesCRLFTFromString($resource->message_text);

        $expected_top_part = cleanUpDoubleSpacesCRLFTFromString('<SaleName>Web Order</SaleName>
				<PickupLocation>97531</PickupLocation>
				<PickUpDate>2020-01-01T12:01:00.0000</PickUpDate>
				<OrderedDate>2020-01-01T12:01:00.0000</OrderedDate>
				<Member>
					<CardNumber>124</CardNumber>
					<ExternalID>1</ExternalID>
					<FirstName>First</FirstName>
					<Surname>Last</Surname>
					<StreetAddress></StreetAddress>
					<State></State>
                    <PostCode>00000</PostCode>
					<Country></Country>
					<Sex>1</Sex>
					<EmailAddress>testy.mctesterson@order140.com</EmailAddress>
				</Member>');
        $this->assertContains($expected_top_part, $actual_string);

        $expected_body_one = cleanUpDoubleSpacesCRLFTFromString('<Item>
                            <PLU>1000</PLU>
                            <QTY>1</QTY>
                            <Value>1.50</Value>
                            <Modifiers xsi:nil="true" />
                        </Item>

                                                    <Item>
                                <PLU>4009</PLU>
                                <QTY>1</QTY>
                                <Value>0.500</Value>
                                <Modifiers xsi:nil="true" />
                            </Item>
                                                    <Item>
                                <PLU>4008</PLU>
                                <QTY>1</QTY>
                                <Value>0.500</Value>
                                <Modifiers xsi:nil="true" />
                            </Item>
                                                    <Item>
                                <PLU>4007</PLU>
                                <QTY>1</QTY>
                                <Value>0.500</Value>
                                <Modifiers xsi:nil="true" />
                            </Item>
                                                    <Item>
                                <PLU>4006</PLU>
                                <QTY>1</QTY>
                                <Value>0.500</Value>
                                <Modifiers xsi:nil="true" />
                            </Item>
                                                    <Item>
                                <PLU>4005</PLU>
                                <QTY>1</QTY>
                                <Value>0.500</Value>
                                <Modifiers xsi:nil="true" />
                            </Item>
                                                    <Item>
                                <PLU>4004</PLU>
                                <QTY>1</QTY>
                                <Value>0.500</Value>
                                <Modifiers xsi:nil="true" />
                            </Item>
                                                    <Item>
                                <PLU>4003</PLU>
                                <QTY>1</QTY>
                                <Value>0.500</Value>
                                <Modifiers xsi:nil="true" />
                            </Item>
                                                    <Item>
                                <PLU>4002</PLU>
                                <QTY>1</QTY>
                                <Value>0.500</Value>
                                <Modifiers xsi:nil="true" />
                            </Item>');
        $this->assertContains($expected_body_one, $actual_string);

        $expected_body_two = cleanUpDoubleSpacesCRLFTFromString('<Item>
                            <PLU>1001</PLU>
                            <QTY>1</QTY>
                            <Value>1.50</Value>
                            <Modifiers xsi:nil="true" />
                        </Item>
                                                    <Item>
                                <PLU>4009</PLU>
                                <QTY>1</QTY>
                                <Value>0.500</Value>
                                <Modifiers xsi:nil="true" />
                            </Item>
                                                    <Item>
                                <PLU>4008</PLU>
                                <QTY>1</QTY>
                                <Value>0.500</Value>
                                <Modifiers xsi:nil="true" />
                            </Item>
                                                    <Item>
                                <PLU>4007</PLU>
                                <QTY>1</QTY>
                                <Value>0.500</Value>
                                <Modifiers xsi:nil="true" />
                            </Item>
                                                    <Item>
                                <PLU>4006</PLU>
                                <QTY>1</QTY>
                                <Value>0.500</Value>
                                <Modifiers xsi:nil="true" />
                            </Item>
                                                    <Item>
                                <PLU>4005</PLU>
                                <QTY>1</QTY>
                                <Value>0.500</Value>
                                <Modifiers xsi:nil="true" />
                            </Item>
                                                    <Item>
                                <PLU>4004</PLU>
                                <QTY>1</QTY>
                                <Value>0.500</Value>
                                <Modifiers xsi:nil="true" />
                            </Item>');
        $this->assertContains($expected_body_two, $actual_string);

        $expected_media = cleanUpDoubleSpacesCRLFTFromString('<Medias>
					<Media>
						<MediaID>18</MediaID>
						<Value>' . number_format($this->order_resource->grand_total,2) . '</Value>
					</Media>
					<Media>
						<MediaID>5</MediaID>
						<Value>' . $this->order_resource->total_tax_amt . '</Value>
					</Media>
				</Medias>
				<IsDelivery>0</IsDelivery>');

        $this->assertContains($expected_media, $actual_string);
    }

*/
    function testSendMessage()
    {
        $request = null;
        $task_retail_controller = new TaskRetailController(getM(), $this->user_resource, $request, 5);
        $task_retail_service = new TaskRetailService($this->message_resource);

        $resource = $task_retail_controller->prepMessageForSending($this->message_resource);
        $body = $resource->message_text;

        $response = $task_retail_service->send($body);
        $this->assertTrue($response != false, 'The response from sending the message was false.');
    }

    function testSendMessageInController()
    {
        $request = null; // Don't need request for this test. Header/URL, the like

        $task_retail_controller = new TaskRetailController($mimetypes, $this->user_resource, $request, 5);
        $task_retail_controller->message_resource = $this->message_resource;

        $result = $task_retail_controller->sendTheMessage();
        $this->assertTrue($result != false, 'The result of sending the message was false.');
        $message_resource = SplickitController::getResourceFromId($this->message_resource->map_id, "MerchantMessageHistory");
        $message_text = $message_resource->message_text;
        $this->assertNotNull($message_text);
        $message_text = cleanUpCRLFTFromString($message_text);
        $message_text = str_replace("  ", '', $message_text);
        $today_date = date("Y-m-d", getTodayTwelveNoonTimeStampDenver());
        $expected_pickup = "<PickUpDate>" . $today_date . "T11:20:00.0000</PickUpDate>";
        $this->assertContains($expected_pickup, $message_text);

        $this->assertNotNull($message_resource->response, "It should save the message response on the record");
        $this->assertEquals("<Successful>true</Successful>", $message_resource->response);
    }

    /**
     * @expectedException UnsuccessfulTaskRetailAuthenticationException
     */
    function testAuthenticationException()
    {
        $_SERVER['AUTOFAIL_AUTHENTICATION'] = true;
        $request = null;
        $task_retail_controller = new TaskRetailController($mimetypes, $this->user_resource, $request, 5);
        $task_retail_service = new TaskRetailService($this->message_resource);

        $resource = $task_retail_controller->prepMessageForSending($this->message_resource);
        $body = $resource->message_text;

        $response = $task_retail_service->send($body);
        $_SERVER['AUTOFAIL_AUTHENTICATION'] = false;
    }

    /**
     * @expectedException UnsuccessfulTaskRetailPushException
     */
    function testTaskRetailPushException()
    {
        $task_retail_service = new TaskRetailService($this->message_resource);
        $task_retail_service->send('<PLUU>');
    }

    function testSendOrderWithoutPromoForSending()
    {
        $mid = $this->ids['merchant_id'];
        $uid = $this->ids['user_id'];
        $user = logTestUserIn($uid);

        Resource::createByData(new MerchantTaskRetailInfoMapAdapter($m), array("brand_id" => 300, "merchant_id" => $mid, "location" => "97531", "media_id" => 5, "payment_id" => 18));
        $task_retail_controller = new TaskRetailController($mimetypes, $user, $request, 5);
        $this->assertNotNull($task_retail_controller, 'Oops. $task_retail_controller was null');

        //create the type 1 promo
        $promo_adapter = new PromoAdapter($mimetypes);
        $ids['promo_id_type_1'] = 202;
        $sql = "INSERT INTO `Promo` VALUES(202, 'The Type1 Promo', 'Get $10 off', 1, 'Y', 'N', 0, 2, 'N', 'N','all', '2010-01-01', '2030-01-01', 1, 0, 0, 0.00, 0, 0.00, 'Y', 'N',0, 282,NOW(), NOW(), 'N')";
        $promo_adapter->_query($sql);

        $sql = "INSERT INTO `Promo_Merchant_Map` VALUES(null, 202, $mid, '2013-10-05', '2030-01-01', NULL, now())";
        $promo_adapter->_query($sql);
        $pmm_resource2 = Resource::createByData(new PromoMerchantMapAdapter($mimetypes), array("merchant_id"=>$mid,"promo_id"=>202));
        $ids['promo_merchant_map_id_type_1_alternate'] = $pmm_resource2->map_id;
        $sql = "INSERT INTO `Promo_Message_Map` VALUES(null, 202, 'Congratulations! You''re getting $%%amt%% off your order!', NULL, NULL, NULL, NULL, now())";
        $promo_adapter->_query($sql);
        $sql = "INSERT INTO `Promo_Type1_Amt_Map` VALUES(null, 202, 1.00, 10.00, 0.00,50.00, NOW())";
        $promo_adapter->_query($sql);

        $pkwm_adapter = new PromoKeyWordMapAdapter($mimetypes);
        Resource::createByData($pkwm_adapter, array("promo_id"=>202,"promo_key_word"=>"type1promo","brand_id"=>282));

        $order_data = OrderAdapter::getSimpleCartArrayByMerchantId($mid, 'pickup','notes', 5);
        $order_data['submitted_order_type'] = 'pickup';

        $json_encoded_data = json_encode($order_data);
        $request = createRequestObject('/app2/apiv2/cart/checkout','POST', $json_encoded_data, 'application/json');

        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $cart_resource = $place_order_controller->processV2Request();
        $this->assertNull($cart_resource->error);
        $cart_ucid = $cart_resource->cart_ucid;

        $submit_order_data['merchant_id'] = $mid;
        $submit_order_data['note'] = "the new cart note";
        $submit_order_data['user_id'] = $user['user_id'];
        $submit_order_data['cart_ucid'] = $cart_ucid;
        $submit_order_data['tip'] = (rand(100, 1000))/100;
        $payment_array = $cart_resource->accepted_payment_types;
        $submit_order_data['merchant_payment_type_map_id'] = $payment_array[0]['merchant_payment_type_map_id'];

        $lead_times_array = $cart_resource->lead_times_array;
        $first_time = $lead_times_array[0];

        $submit_order_data['requested_time'] = $first_time;

        $json_encoded_data = json_encode($submit_order_data);

        $request = createRequestObject("/apiv2/orders/$cart_ucid","POST",$json_encoded_data,'application/json');
        $place_order_controller = new PlaceOrderController($mt, getAuthenticatedUser(), $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $order_resource = $place_order_controller->processV2Request();
        $this->assertNull($order_resource->error);

        $merchant_message_history_adapter = new MerchantMessageHistoryAdapter($m);
        $message_resource = $merchant_message_history_adapter->getExactResourceFromData(array("order_id" => $order_resource->order_id, "message_format" => 'A'));

        $resource = $task_retail_controller->prepMessageForSending($message_resource);

        $actual_string = cleanUpDoubleSpacesCRLFTFromString($resource->message_text);

        $order_promo_amount = abs($order_resource->promo_amt);
        $order_promo_amount = number_format($order_promo_amount, 2);

        $expected_promo_media = cleanUpDoubleSpacesCRLFTFromString('
					<Media>
						<MediaID>30039</MediaID>
						<Value>' . $order_promo_amount . '</Value>
					</Media>');

        $this->assertnotContains($expected_promo_media, $actual_string);
    }


    function testSendOrderWithPromoForSending()
    {
        $mid = $this->ids['merchant_id'];
        $uid = $this->ids['user_id'];
        $user = logTestUserIn($uid);

        Resource::createByData(new MerchantTaskRetailInfoMapAdapter($m), array("brand_id" => 300, "merchant_id" => $mid, "location" => "97531", "media_id" => 5, "payment_id" => 18));
        $task_retail_controller = new TaskRetailController(getM(), $user, $request, 5);
        $this->assertNotNull($task_retail_controller, 'Oops. $task_retail_controller was null');

        //create the type 1 promo
        $promo_adapter = new PromoAdapter(getM());
        $ids['promo_id_type_1'] = 202;
        $sql = "INSERT INTO `Promo` VALUES(202, 'The Type1 Promo', 'Get $10 off', 1, 'Y', 'N', 0, 2, 'N', 'N','all', '2010-01-01', '2030-01-01', 1, 0, 0, 0.00, 0, 0.00, 'Y', 'N', 0,282,NOW(), NOW(), 'N')";
        $promo_adapter->_query($sql);

        $sql = "INSERT INTO `Promo_Merchant_Map` VALUES(null, 202, $mid, '2013-10-05', '2030-01-01', NULL, now())";
        $promo_adapter->_query($sql);
        $pmm_resource2 = Resource::createByData(new PromoMerchantMapAdapter($mimetypes), array("merchant_id"=>$mid,"promo_id"=>202));
        $ids['promo_merchant_map_id_type_1_alternate'] = $pmm_resource2->map_id;
        $sql = "INSERT INTO `Promo_Message_Map` VALUES(null, 202, 'Congratulations! You''re getting $%%amt%% off your order!', NULL, NULL, NULL, NULL, now())";
        $promo_adapter->_query($sql);
        $sql = "INSERT INTO `Promo_Type1_Amt_Map` VALUES(null, 202, 1.00, 10.00, 0.00,50.00, NOW())";
        $promo_adapter->_query($sql);

        $pkwm_adapter = new PromoKeyWordMapAdapter($mimetypes);
        Resource::createByData($pkwm_adapter, array("promo_id"=>202,"promo_key_word"=>"type1promo","brand_id"=>282));

        $order_data = OrderAdapter::getSimpleCartArrayByMerchantId($mid, 'pickup','notes', 5);
        $order_data['submitted_order_type'] = 'pickup';

        $json_encoded_data = json_encode($order_data);
        $request = createRequestObject('/app2/apiv2/cart/checkout','POST', $json_encoded_data, 'application/json');

        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        $cart_resource = $place_order_controller->processV2Request();
        $this->assertNull($cart_resource->error);
        $cart_ucid = $cart_resource->cart_ucid;

        //enter promo
        $request = createRequestObject("/app2/apiv2/cart/$cart_ucid/checkout?promo_code=type1promo", 'GET', $body, 'application/json');
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$mid,0.00);

//        $submit_order_data['merchant_id'] = $mid;
//        $submit_order_data['note'] = "the new cart note";
//        $submit_order_data['user_id'] = $user['user_id'];
//        $submit_order_data['cart_ucid'] = $cart_ucid;
//        $submit_order_data['tip'] = (rand(100, 1000))/100;
//        $payment_array = $cart_resource->accepted_payment_types;
//        $submit_order_data['merchant_payment_type_map_id'] = $payment_array[0]['merchant_payment_type_map_id'];
//
//        $lead_times_array = $cart_resource->lead_times_array;
//        $first_time = $lead_times_array[0];
//
//        $submit_order_data['requested_time'] = $first_time;
//
//        $json_encoded_data = json_encode($submit_order_data);
//
//        $request = createRequestObject("/apiv2/orders/$cart_ucid","POST",$json_encoded_data,'application/json');
//        $place_order_controller = new PlaceOrderController($mt, getAuthenticatedUser(), $request);
//        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
//        $order_resource = $place_order_controller->processV2Request();
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'A');

        $resource = $task_retail_controller->prepMessageForSending($message_resource);

        $actual_string = cleanUpDoubleSpacesCRLFTFromString($resource->message_text);

        $order_promo_amount = abs($order_resource->promo_amt);
        $order_promo_amount = number_format($order_promo_amount, 2);

        $expected_promo_media = cleanUpDoubleSpacesCRLFTFromString('
					<Media>
						<MediaID>30039</MediaID>
						<Value>' . $order_promo_amount . '</Value>
					</Media>');

        $this->assertContains($expected_promo_media, $actual_string);
    }

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
              SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();$mysqli->begin_transaction(); ;

        $skin_resource = createWorldHqSkin();
        $ids['skin_id'] = $skin_resource->skin_id;

        //map it to a menu
        $menu_id = createTestMenuWithNnumberOfItems(5);
        $ids['menu_id'] = $menu_id;

        $modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 10);
        $modifier_group_id = $modifier_group_resource->modifier_group_id;
        $item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 2);
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[1]['item_id'], $modifier_group_id, 4);
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[2]['item_id'], $modifier_group_id, 1);

        ABTaskRetailControllerTest::assignSequentialPLUsToMenuItems($menu_id, $item_records);

        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_resource->time_zone = -6;
        $merchant_resource->save();
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes), array("merchant_id" => $merchant_resource->merchant_id, "message_format" => 'A', "delivery_addr" => "taskretail", "message_type" => "O"));
        attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
        $ids['merchant_id'] = $merchant_resource->merchant_id;
        Resource::createByData(new MerchantTaskRetailInfoMapAdapter($m), array("brand_id" => 300, "merchant_id" => $merchant_resource->merchant_id, "location" => "97531", "media_id" => 5, "payment_id" => 18));


        $sql = 'CALL load_third_party_from_taskretail_info()';
        $adapter = new ThirdPartyProviderParameterMerchantMapsAdapter(getM());
        $adapter->_query($sql);

        $user_resource = createNewUser(array("flags" => "1C20000001"));
        $ids['user_id'] = $user_resource->user_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* main method for testing */
    static function main()
    {
        $suite = new PHPUnit_Framework_TestSuite(__CLASS__);
        PHPUnit_TextUI_TestRunner::run($suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    ABTaskRetailControllerTest::main();
}

?>
