<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class BrinkControllerTestNewThirdPartyStuff extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
        $this->ids = $_SERVER['unit_test_ids'];
        setContext("com.splickit.thirdpartybrinkskin");
        $_SERVER['USE_BRINK_DELIVERY_MOCK'] = false;
    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
        $_SERVER['BRINK_TIMEOUT'] = false;
    }

    function testCreateActivity()
    {
        $doit_ts = time()-2;
        $info = "merchant_alphanumeric_id=hu76yt5r4e;build_modifiers=1";
        $activity_history_adapter = new ActivityHistoryAdapter(getM());
        $activity_history_resource = $activity_history_adapter->createActivityReturnActivityResource('BrinkPriceSync', $doit_ts, $info, '');
        $this->assertNotNull($activity_history_resource);

    }

    function testCreateGroupOrderType2()
    {
        $merchant_resource = createNewTestMerchant($this->ids['menu_id'],['authorize'=>true]);
        $merchant_resource->merchant_external_id = 'Group-123456';
        $merchant_resource->group_ordering_on = 1;
        $merchant_resource->save();
        $merchant_id = $merchant_resource->merchant_id;
        Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'B',"delivery_addr"=>"Brink","message_type"=>"X"));
        Resource::createByData(new MerchantBrinkInfoMapsAdapter(getM()),array("merchant_id"=>$merchant_id,"brink_location_token"=>'sumdumtoken'));

        $note = "This is the group order note.";
        $emails = "sumdumemail1@order140.com,sumdumemail2@order140.com,sumdumemail3@order140.com,sumdumemail4@order140.com";
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $request = new Request();
        $request->data = array("merchant_id" => $merchant_id, "notes" => $note, "participant_emails" => $emails, "group_order_type" => 2, "submit_at_ts" => (getTomorrowTwelveNoonTimeStampDenver() + (3600)));
        $request->url = "app2/apiv2/grouporders";
        $request->method = "POST";
        $group_order_controller = new GroupOrderController(getM(), $user, $request, 5);
        $group_order_controller->current_time = getTomorrowTwelveNoonTimeStampDenver();
        $resource = $group_order_controller->processV2Request();
        $this->assertNull($resource->error);
        $this->assertNotNull($resource);

        $this->assertNotNull($resource->group_order_token);
        $this->assertTrue($resource->group_order_id > 1000, "shouljd have a group order id");
        $this->assertEquals($user['user_id'], $resource->admin_user_id);
        $group_order_adapter = new GroupOrderAdapter(getM());
        $group_order_record = $group_order_adapter->getRecordFromPrimaryKey($resource->group_order_id);
        $this->assertEquals($note, $group_order_record['notes']);
        $this->assertEquals($emails, $group_order_record['participant_emails']);
        $base_order_data = OrderAdapter::staticGetRecordByPrimaryKey($resource->order_id, 'OrderAdapter');
        $this->assertEquals('R', $base_order_data['order_type']);
        return $resource->group_order_token;
    }

    /**
     * @depends testCreateGroupOrderType2
     */
    function testAddToGroupOrderType2UsingCart($group_order_token)
    {
        $group_order_record = getStaticRecord(array("group_order_token" => $group_order_token), 'GroupOrderAdapter');
        $group_order_id = $group_order_record['group_order_id'];
        $merchant_id = $group_order_record['merchant_id'];
        $user_resource = createNewUserWithCCNoCVV();
        $user_resource->first_name = 'Rob';
        $user_resource->last_name = 'Zombie';
        $user_resource->save();
        $user = logTestUserResourceIn($user_resource);

        $order_data = OrderAdapter::getSimpleCartArrayByMerchantId($this->ids['merchant_id']);
        $cart_note = "sum dum cart note";
        $order_data['note'] = $cart_note;
        $order_data['group_order_token'] = $group_order_token;

        $json_encoded_data = json_encode($order_data);
        $request = createRequestObject('/app2/apiv2/cart/checkout','POST',$json_encoded_data);
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($checkout_resource->error);
        $this->assertNotNull($checkout_resource, "should have gotten a cart resource back");
        $ucid = $checkout_resource->ucid;
        $this->assertNotNull($ucid,"It should have the ucid of the child order");
        $this->assertNotNull($checkout_resource->cart_ucid,"It should have backwards compatible ucid of the child order ");

        // validate that order was added to the mapping table
        $cart_order_resource = Resource::find(new CartsAdapter(getM()),"$ucid");
        $gorma = new GroupOrderIndividualOrderMapsAdapter(getM());
        $record = $gorma->getRecord(array("user_order_id" => $cart_order_resource->order_id));
        $this->assertEquals($group_order_id, $record['group_order_id']);
        $this->assertEquals("In Process", $record['status']);

        // validate note was added
        $this->assertEquals($cart_note, $cart_order_resource->note, "note should ahve been set on the cart order record");

        // now place the order which should add it to the group order
        $place_order_result_resource = $this->placeOrderToBePartOfGroupOrder($user, $merchant_id, $checkout_resource->ucid, $checkout_resource, $time);
        $this->assertNull($place_order_result_resource->error);

        // map should have been set to subitted
        $record = $gorma->getRecord(array("user_order_id" => $cart_order_resource->order_id));
        $this->assertEquals("Submitted", $record['status']);
        return $group_order_token;
    }

    /**
     * @depends testAddToGroupOrderType2UsingCart
     */
    function testAddToGroupOrderType2UsingCartForAdminUser($group_order_token)
    {

        $time = getTomorrowTwelveNoonTimeStampDenver();
        $group_order_record = getStaticRecord(array("group_order_token" => $group_order_token), 'GroupOrderAdapter');
        $group_order_id = $group_order_record['group_order_id'];
        $merchant_id = $group_order_record['merchant_id'];
        $user_id = $group_order_record['admin_user_id'];
        $user = logTestUserIn($user_id);
        $admin_name_string = $user['first_name'] . ' ' . $user['last_name'][0] . '.';

        $order_data = OrderAdapter::getSimpleCartArrayByMerchantId($this->ids['merchant_id']);
        $cart_note = "Admin cart note";
        $order_data['note'] = $cart_note;
        $order_data['group_order_token'] = $group_order_token;


        $json_encoded_data = json_encode($order_data);
        $request = createRequestObject('/app2/apiv2/cart/checkout','POST',$json_encoded_data);
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($checkout_resource->error);

        // validate that order was added to the mapping table

        $cart_order_resource = Resource::find(new CartsAdapter($m), $checkout_resource->ucid);
        $gorma = new GroupOrderIndividualOrderMapsAdapter($m);
        $record = $gorma->getRecord(array("user_order_id" => $cart_order_resource->order_id));
        $this->assertEquals($group_order_id, $record['group_order_id']);
        $this->assertEquals("In Process", $record['status']);


        // validate note was added
        $submitted_cart_record = CartsAdapter::staticGetRecord(array("ucid" => $checkout_resource->ucid), 'CartsAdapter');
        $order_id = $submitted_cart_record['order_id'];
        $order_record = OrderAdapter::staticGetRecordByPrimaryKey($order_id, 'Order');
        $this->assertEquals($cart_note, $order_record['note'], "note should ahve been set on the cart order record");

        // now place the order which should add it to the group order
        $place_order_result_resource = $this->placeOrderToBePartOfGroupOrder($user, $merchant_id, $checkout_resource->ucid, $checkout_resource, getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($place_order_result_resource->error);

        // map should have been set to subitted
        $record = $gorma->getRecord(array("user_order_id" => $cart_order_resource->order_id));
        $this->assertEquals("Submitted", $record['status']);
        return $group_order_token;
    }

    /**
     * @depends testAddToGroupOrderType2UsingCartForAdminUser
     */
    function testBillingSectionOfCompleteOrderForGroupOrderType2($group_order_token)
    {
        $group_order_adapter = new GroupOrderAdapter(getM());
        $group_order_record = $group_order_adapter->getRecord(['group_order_token'=>$group_order_token]);
        $group_order_records = GroupOrderIndividualOrderMapsAdapter::staticGetRecords(array("group_order_id"=>$group_order_record['group_order_id'],"status"=>'Submitted'),'GroupOrderIndividualOrderMapsAdapter');
        foreach ($group_order_records as $group_order_records_map) {
            $balance_change_records[$group_order_records_map['user_order_id']] = BalanceChangeAdapter::staticGetRecord(array("order_id"=>$group_order_records_map['user_order_id'],"process"=>'CCpayment'),'BalanceChangeAdapter');
        }
        $this->assertCount(2,$balance_change_records,"there should have been 2 records of CC payments");

        // now get complete order for group order and we should see a payment section with CC info
        $group_order_token = $group_order_record['group_order_token'];
        $group_base_order = CompleteOrder::staticGetCompleteOrder($group_order_token);

        $this->assertNotNull($group_base_order['group_order_payments'],"It should have a payments section");
        foreach ($group_base_order['group_order_payments'] as $payment_node) {
            $this->assertNotNull($payment_node['charge_amt'],"It needs to have the charge amount");
            $this->assertNotNull($payment_node['order_id'],"It should have the individual order_id");
            $this->assertNotNull($payment_node['ucid'],"It shoudl have the UCID of the order");
            $ucids[$payment_node['order_id']] = $payment_node['ucid'];
            $this->assertNotNull($payment_node['last_four'],"It should have the last 4 of the user as part of the payment node on the group order");
            $this->assertEquals('1234',$payment_node['last_four']);
        }

        //validate brink template

        $goid = $group_order_record['order_id'];
        $request = createRequestObject("/app2/messagemanager/getorderbyid/$goid/brink/?format=B",'GET');
        $brink_controller = new BrinkController(getM(),null,$request,5);
        $ready_to_send_message_resource = $brink_controller->getOrderById($goid);
        $message_text = $ready_to_send_message_resource->message_text;
        myerror_log($message_text);
        $message_text = cleanUpDoubleSpacesCRLFTFromString($message_text);

        $expected_node = cleanUpDoubleSpacesCRLFTFromString('<Payments>
                                <NewOrderPayment i:type="NewExternalPayment">
                                    <Amount>4.40</Amount>
                                    <Id>5</Id>
                                    <TenderId>55555</TenderId>
                                    <TipAmount>0.00</TipAmount>
                                    <ReferenceDetails>
                                    <PaymentReferenceDetail>
                                        <Name>Splickit</Name>
                                        <Value>Splickit Order</Value>
                                    </PaymentReferenceDetail>
                                    </ReferenceDetails>
                                </NewOrderPayment>
                                </Payments>');
        $this->assertContains($expected_node,$message_text);

    }



    function testStoredValueAlternativeTenderId()
    {
        $merchant_resource = createNewTestMerchant($this->ids['menu_id'],['authorize'=>true]);
        $merchant_id = $merchant_resource->merchant_id;
        Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'B',"delivery_addr"=>"Brink","message_type"=>"X"));
        Resource::createByData(new MerchantBrinkInfoMapsAdapter(getM()),array("merchant_id"=>$merchant_id,"brink_location_token"=>'sumdumtoken'));


        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);

        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,000,$time);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $bc_resource = Resource::find(new BalanceChangeAdapter(getM()),null,[3=>['order_id'=>$order_id,'process'=>'Authorize']]);
        $bc_resource->process = 'StoredValuePayment';
        $bc_resource->save();

        $brink_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'B');
        $this->assertNotNull($brink_message_resource,"we shojdl have a message resource");
        $brink_controller = new BrinkController($m, $u, $r, 5);
        $brink_controller->sendThisMessage($brink_message_resource);
        $ready_to_send_message_resource = $brink_controller->prepMessageForSending($brink_message_resource);
        myerror_log($ready_to_send_message_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);
        //check to see that promo is in the XML
        $expected_tender_id_node = "<TenderId>33333</TenderId>";
        $this->assertContains($expected_tender_id_node,$body,"It should have the  stored value id");

        $grand_total = number_format($order_resource->grand_total,2);
        $expected_amount_node = "<Amount>$grand_total</Amount>";
        $this->assertContains($expected_amount_node,$body,"It should have the amount");

        $brink_controller = new BrinkController(getM(),null, $r,5);
        $brink_controller->sendThisMessage($brink_message_resource);
        $brink_service = $brink_controller->brink_service;
        $this->assertContains('api.brinkpos.net',$brink_service->getUrl());
    }

//    function testStoredValueSplitPayments()
//    {
//        $merchant_resource = createNewTestMerchant($this->ids['menu_id'],['authorize'=>true]);
//        $merchant_id = $merchant_resource->merchant_id;
//        Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'B',"delivery_addr"=>"Brink","message_type"=>"X"));
//        Resource::createByData(new MerchantBrinkInfoMapsAdapter(getM()),array("merchant_id"=>$merchant_id,"brink_location_token"=>'sumdumtoken'));
//
//
//        $user_resource = createNewUserWithCCNoCVV();
//        $user = logTestUserResourceIn($user_resource);
//
//        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
//        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
//        $this->assertNull($checkout_resource->error);
//
//        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,000,$time);
//        $this->assertNull($order_resource->error);
//        $order_id = $order_resource->order_id;
//
//        $bc_resource = Resource::find(new BalanceChangeAdapter(getM()),null,[3=>['order_id'=>$order_id,'process'=>'Authorize']]);
//        $bc_resource->charge_amt = "1.20";
//        $bc_resource->save();
//
//        unset($bc_resource->id);
//        $bc_resource->_exists = false;
//        $bc_resource->process = 'StoredValuePayment';
//        $bc_resource->charge_amt = "1.00";
//        $bc_resource->notes = 'payment with stored value Card';
//        $bc_resource->save();
//
//        $brink_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'B');
//        $this->assertNotNull($brink_message_resource,"we shojdl have a message resource");
//        $brink_controller = new BrinkController($m, $u, $r, 5);
//        $brink_controller->sendThisMessage($brink_message_resource);
//        $ready_to_send_message_resource = $brink_controller->prepMessageForSending($brink_message_resource);
//        myerror_log($ready_to_send_message_resource->message_text);
//        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);
//        //check to see that promo is in the XML
//        $expected_stored_value_tender_id_node = "<TenderId>33333</TenderId>";
//        $this->assertContains($expected_stored_value_tender_id_node,$body,"It should have the  stored value id");
//        $expected_stored_value_amount_node = "<Amount>1.00</Amount>";
//        $this->assertContains($expected_stored_value_amount_node,$body,"It should have the amount");
//
//        $expected_stored_value_tender_id_node = "<TenderId>55555</TenderId>";
//        $this->assertContains($expected_stored_value_tender_id_node,$body,"It should have the  stored value id");
//
//        $expected_amount_node = "<Amount>1.20</Amount>";
//        $this->assertContains($expected_amount_node,$body,"It should have the amount");
//
//        $brink_controller = new BrinkController(getM(),null, $r,5);
//        $brink_controller->sendThisMessage($brink_message_resource);
//        $brink_service = $brink_controller->brink_service;
//        $this->assertContains('api.brinkpos.net',$brink_service->getUrl());
//    }


    function testGroupOrderBrinkNativeDelivery()
    {

        $_SERVER['USE_BRINK_DELIVERY_MOCK'] = true;
        $merchant_resource = createNewTestMerchantDelivery($this->ids['menu_id']);
        $merchant_resource->merchant_external_id = 'sumdum2-123456';
        $merchant_resource->group_ordering_on = 1;
        $merchant_resource->save();
        $merchant_id = $merchant_resource->merchant_id;

        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'B','brink','X');


        $merchant_delivery_price_resource = Resource::find(new MerchantDeliveryPriceDistanceAdapter(getM()),'',array(3=>array("merchant_id"=>$merchant_id)));
        $merchant_delivery_price_resource->name = "zone_1";
        $merchant_delivery_price_resource->distance_up_to = 5;
        $destination_id_for_delivery = "delivery_override-123456";
        //$merchant_delivery_price_resource->notes = "brink_destination_id=$destination_id_for_delivery";
        $merchant_delivery_price_resource->price = 5.55;
        $merchant_delivery_price_resource->save();
        $mdpd_id = $merchant_delivery_price_resource->getPrimaryId();


        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $brink_third_party_parameters = $tpppa->getRecords(['third_party_provider_id'=>1001]);
        foreach ($brink_third_party_parameters as $brink_parameter) {
            if ($brink_parameter['name'] == 'brink_location_token') {
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()),["merchant_id"=>$merchant_id,"third_party_provider_parameter_id"=>$brink_parameter['id'],"value"=>'location-888888']);
            } else if ($brink_parameter['name'] == 'brink_delivery_destination_id') {
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()), ["merchant_id" => $merchant_id, "third_party_provider_parameter_id" => $brink_parameter['id'], "value" => "$destination_id_for_delivery"]);
            } else {
                $id = $brink_parameter['id'];
                $name = $brink_parameter['name'];
                $value = $name.'-1000'.$id;
                if ($name == 'brink_domain') {
                    $value = 'api88888.brinkpos.net';
                }
                $ids[$name] = $value;
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()),["brand_id"=>getBrandIdFromCurrentContext(),"third_party_provider_parameter_id"=>$brink_parameter['id'],"value"=>$value]);
            }
        }

        $user_resource = createNewUserWithCC();
        $user_resource->first_name = 'Tom';
        $user_resource->last_name = 'Zombie';
        $user_resource->flags = '1C20000001';
        $user_resource->save();
        $user = logTestUserResourceIn($user_resource);

        //$json = '{"user_id":"' . $user['user_id'] . '","name":"","address1":"1045 Pine Street","address2":"","city":"Boulder","state":"CO","zip":"80302","phone_no":"1234567890","lat":40.019785,"lng":-105.282509}';
        $json = '{"user_id":"'.$user['user_id'].'","name":"","address1":"1045 Pine Street","address2":"3B & 3C","city":"boulder","state":"co","zip":"80302","instructions":"come around back please","phone_no":"9709262121","lat":40.0197891,"lng":-105.284703}';

        $request = new Request();
        $request->body = $json;
        $request->mimetype = "Application/json";
        $request->_parseRequestBody();
        $request->method = 'POST';
        $request->url = "/users/" . $user['uuid'] . "/userdeliverylocation";
        $user_controller = new UserController($mt, $user, $request, 5);
        $response = $user_controller->processV2Request();
        $this->assertNull($response->error, "should not have gotten a delivery save error but did");
        $this->assertNotNull($response->user_addr_id);
        $user_address_id = $response->user_addr_id;

        $request = new Request();
        $request->data = array("merchant_id" => $merchant_id, "note" => '', "participant_emails" => $emails, "user_addr_id" => $user_address_id, "group_order_type" => 2, "submit_at_ts" => (getTomorrowTwelveNoonTimeStampDenver() + 900));
        $request->url = "app2/apiv2/grouporders";
        $request->method = "POST";
        $group_order_controller = new GroupOrderController($mt, $user, $request, 5);
        $resource = $group_order_controller->processV2Request();
        $group_order_token = $resource->group_order_token;
        $cart = CartsAdapter::staticGetRecordByPrimaryKey($group_order_token, "CartsAdapter");
        $order_id = $cart['order_id'];
        // check to make sure cart has mdpd_id on record
        $this->assertEquals($mdpd_id,$cart['merchant_delivery_price_distance_id']);
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id, $m);
        $this->assertEquals('D', $complete_order['order_type'], "shoudl have created a delivery dummy order row");
        $this->assertEquals($user_address_id, $complete_order['user_delivery_location_id'], "Should have the user addr id on the order");
        $this->assertEquals(5.55, $complete_order['delivery_amt'], "should have a delivery price on the dummy order");
        $this->assertEquals(getSkinIdForContext(),$complete_order['skin_id'],"skin id shoudl equal the context");
        //$this->assertEquals(0.00, $complete_order['delivery_amt'], "should NOT have a delivery price on the dummy order");
        $group_order_adapter = new GroupOrderAdapter(getM());
        $group_order_record = $group_order_adapter->getRecordFromPrimaryKey($resource->group_order_id);
        $this->assertEquals($user_address_id,$group_order_record['user_addr_id'],'User address id should be on the group order record');

        $group_order_token = $group_order_record['group_order_token'];
        $group_order_id = $group_order_record['group_order_id'];

        $user = logTestUserIn($group_order_record['admin_user_id']);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'Delivery',"sum dum cart note");
        $cart_data['group_order_token'] = $group_order_token;

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data);
        $this->assertNull($checkout_resource->error);
        $this->assertCount(1,$checkout_resource->lead_times_array,"It should only have 1 available lead time");


        // validate that the order record was added to mapping table
        $cart_order_resource = Resource::find(new CartsAdapter($m),$checkout_resource->ucid);
        $gorma = new GroupOrderIndividualOrderMapsAdapter($m);
        $record = $gorma->getRecord(array("user_order_id"=>$cart_order_resource->order_id));
        $this->assertEquals($group_order_id,$record['group_order_id'],"It should have added the order to the mapping table.");

        // validate that the delivery charge was added to the order record since this is the admin
        $this->assertEquals($group_order_record['user_addr_id'],$cart_order_resource->user_delivery_location_id,"It should have the user address id of the parent order");
        $this->assertEquals(4.44,$cart_order_resource->delivery_amt,"It should have the delivery charge on the order record, but it shoudl have the brink amount of 4.44");


        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $gorma = new GroupOrderIndividualOrderMapsAdapter($m);
        $record = $gorma->getRecord(array("user_order_id"=>$order_id));
        $this->assertEquals($group_order_id,$record['group_order_id']);

        //test to see if values get added to the parent order record
        $parent_order_record = OrderAdapter::staticGetRecord(array("ucid"=>$group_order_token),'OrderAdapter');
        $this->assertEquals($order_resource->order_amt,$parent_order_record['order_amt']);
        $this->assertEquals($order_resource->total_tax_amt, $parent_order_record['total_tax_amt']);
        $this->assertEquals($order_resource->grand_total, $parent_order_record['grand_total']);
        $this->assertEquals($order_resource->delivery_amt, $parent_order_record['delivery_amt']);

        // now lets add another user and see if they get the delivery fee.
//        $group_order_token = $group_order_record['group_order_token'];
//        $group_order_id = $group_order_record['group_order_id'];

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $order_data = OrderAdapter::getSimpleCartArrayByMerchantId($this->ids['merchant_id'],'Delivery',"sum dum participant cart note");
        $order_data['group_order_token'] = $group_order_token;
        $order_data["submitted_order_type"]= 'Delivery';
        $order_data['user_addr_id'] = null;

        $checkout_resource = getCheckoutResourceFromOrderData($order_data);
        $this->assertNull($checkout_resource->error);
        $this->assertEquals("Pending",$checkout_resource->order_status,"It shoudl have a status of Pending");
        $this->assertEquals(0.00,$checkout_resource->delivery_amt);


        // validate that the order record was added to mapping table
        $cart_order_resource = Resource::find(new CartsAdapter($m),$checkout_resource->ucid);
        $gorma = new GroupOrderIndividualOrderMapsAdapter($m);
        $record = $gorma->getRecord(array("user_order_id"=>$cart_order_resource->order_id));
        $this->assertEquals($group_order_id,$record['group_order_id'],"It should have added the order to the mapping table.");

        // validate that the delivery charge was NOT added to the order record since this is jsut a participant
        $this->assertEquals($group_order_record['user_addr_id'],$cart_order_resource->user_delivery_location_id,"It should have the user address id of the parent order");
        $this->assertEquals(0.00,$cart_order_resource->delivery_amt,"It should NOT have the delivery charge on the order record since this is a participant and not the admin");

        // now place the order
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $this->assertEquals($cart_order_resource->order_id, $order_id, "should have created a valid order id on teh original cart record");
        $this->assertEquals("D",$order_resource->order_type,"It should be listed as a delivery order");
        $this->assertEquals(0.00,$order_resource->delivery_amt,"There should not be a delivery fee");
        $this->assertEquals($group_order_record['user_addr_id'],$order_resource->user_delivery_location_id,"It should have the UDL of the parent order");

        // ************************ now lets submit the group order ************************

        $group_order_token = $group_order_record['group_order_token'];
        $group_order_id = $group_order_record['group_order_id'];
        $user = logTestUserIn($group_order_record['admin_user_id']);

        $request = createRequestObject("/app2/apiv2/grouporders/$group_order_token/submit",'POST',$body,$m);
        $group_order_controller = new GroupOrderController($m,$user,$request,5);
        $response = $group_order_controller->processV2request();
        $this->assertNull($response->error);

        $group_order_token = $group_order_record['group_order_token'];
        $group_base_order = CompleteOrder::staticGetCompleteOrder($group_order_token);

        // check to make sure the messages were created
        $brink_order_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($group_base_order['order_id'],'B');
        $this->assertNotNull($brink_order_message_resource,'It should have created the brink message for the parent order');

        // all status should have been set to 'O'
        $records = GroupOrderIndividualOrderMapsAdapter::staticGetRecords(array("group_order_id"=>$group_order_record['group_order_id']),'GroupOrderIndividualOrderMapsAdapter');
        $this->assertCount(2,$records,"There should be 2 orders");
        foreach($records as $record) {
            $base_order_info = CompleteOrder::getBaseOrderData($record['user_order_id']);
            $this->assertEquals('O',$base_order_info['status']);
        }

        $final_group_order_record = getStaticRecord(array("group_order_id"=>$group_order_record['group_order_id']),'GroupOrderAdapter');
        $this->assertEquals('Submitted',$final_group_order_record['status']);
        $e = explode(' ',$final_group_order_record['sent_ts']);
        $sent_date = $e[0];
        $this->assertEquals(date('Y-m-d'),$sent_date,"shoudl have today as sent");
        $mmha = new MerchantMessageHistoryAdapter(getM());
        $message_resource = $mmha->getExactResourceFromData(array("order_id"=>$group_base_order['order_id'],"message_format"=>'Econf'));
        $message_text = $message_resource->message_text;
        $this->assertTrue(substr_count(strip_tags($message_text), 'Tom, your order will be') == 1,"should have name of group order admin");

        //check to make sure requested_delivery_time propogates to parent order
        $sql = "SELECT a.* from Orders a JOIN Group_Order_Individual_Order_Maps b ON a.order_id = b.user_order_id WHERE group_order_id = $group_order_id";
        $options[TONIC_FIND_BY_SQL] = $sql;
        $order_resources = Resource::findAll(new OrderAdapter(getM()),null,$options);
        $requested_delivery_time = $order_resources[0]->requested_delivery_time;
        $this->assertNotNull($requested_delivery_time,"It should have a requested delivery time");
        $this->assertEquals($requested_delivery_time,$group_base_order['requested_delivery_time'],"It should have picked up the requested_delivery_time from the child order");


        //************************ get order thats delivered to the store************************
        $brink_controller = ControllerFactory::generateFromMessageResource($brink_order_message_resource,$m,$u,$r,5);
        $message_to_send_resource = $brink_controller->prepMessageForSending($brink_order_message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        myerror_log($message_to_send_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);
        $expected_delivery_node = cleanUpDoubleSpacesCRLFTFromString("<Delivery>
                            <Address1>1045 Pine Street</Address1>
                            <Address2>3B and 3C, come around back please</Address2>
                            <City>boulder</City>
                            <Country>USA</Country>
                            <PostalCode>80302</PostalCode>
                            <StateCode>CO</StateCode>
                        </Delivery>");
        $this->assertContains($expected_delivery_node,$body,"should have the delivery info");

        $destination_string = "<DestinationId>delivery_override-123456</DestinationId>";
        $this->assertContains($destination_string,$body,"Brink native destination id");

    }

    function testGroupOrderBrinkWithDoorDash()
    {

        $_SERVER['USE_BRINK_DELIVERY_MOCK'] = true;
        $merchant_resource = createNewTestMerchantDelivery($this->ids['menu_id']);
        $merchant_resource->merchant_external_id = 'sumdum2-123456';
        $merchant_resource->group_ordering_on = 1;
        $merchant_resource->save();
        $merchant_id = $merchant_resource->merchant_id;

        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'B','brink','X');


        $mdia = new MerchantDeliveryPriceDistanceAdapter(getM());
        $mdia_resource = $mdia->getExactResourceFromData(["merchant_id"=>$merchant_id]);
        $mdia_resource->minimum_order_amount = 0.01;
        $mdia_resource->delivery_cost = 1.00;
        $mdia_resource->delivery_increment = 15;
        $mdia_resource->max_days_out = 3;
        $mdia_resource->save();


        $mdpd_resource = Resource::find(new MerchantDeliveryPriceDistanceAdapter(getM()),'',array(3=>array("merchant_id"=>$merchant_id)));
        $mdpd_resource->delivery_price_type = 'Doordash';
        $mdpd_resource->name = 'Doordash';
        $mdpd_resource->distance_up_to = 'nullit';
        $mdpd_resource->price = -1.00;
        //$mdpd_resource->notes = "external_business_name=sumdumbiz;";
        $mdpd_resource->save();
        $mdpd_resource = $mdpd_resource->getRefreshedResource();
        $mdpd_id = $mdpd_resource->getPrimaryId();



        $tpppbma = new ThirdPartyProviderParameterBrandMapsAdapter(getM());
        $external_business_name_tppbma_resource = $tpppbma->setBrandParameterByName('external_business_name',4001,getBrandIdFromCurrentContext(),'sumdumbiz');

        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $brink_third_party_parameters = $tpppa->getRecords(['third_party_provider_id'=>1001]);
        foreach ($brink_third_party_parameters as $brink_parameter) {
            if ($brink_parameter['name'] == 'brink_location_token') {
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()), ["merchant_id" => $merchant_id, "third_party_provider_parameter_id" => $brink_parameter['id'], "value" => 'location-888888']);
            } else if ($brink_parameter['name'] == 'brink_delivery_destination_id') {
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()), ["merchant_id" => $merchant_id, "third_party_provider_parameter_id" => $brink_parameter['id'], "value" => '66666']);
            } else {
                $id = $brink_parameter['id'];
                $name = $brink_parameter['name'];
                $value = $name.'-1000'.$id;
                if ($name == 'brink_domain') {
                    $value = 'api88888.brinkpos.net';
                }
                $ids[$name] = $value;
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()),["brand_id"=>getBrandIdFromCurrentContext(),"third_party_provider_parameter_id"=>$brink_parameter['id'],"value"=>$value]);
            }
        }

        $user_resource = createNewUserWithCC();
        $user_resource->first_name = 'Tom';
        $user_resource->last_name = 'Zombie';
        $user_resource->flags = '1C20000001';
        $user_resource->save();
        $user = logTestUserResourceIn($user_resource);

//        $json = '{"user_id":"' . $user['user_id'] . '","name":"","address1":"1045 Pine Street","address2":"","city":"Boulder","state":"CO","zip":"80302","phone_no":"1234567890","lat":40.019785,"lng":-105.282509}';
        $json = '{"user_id":"'.$user['user_id'].'","name":"","address1":"1045 Pine Street","address2":"3B & 3C","city":"boulder","state":"co","zip":"80302","instructions":"come around back please","phone_no":"9709262121","lat":40.0197891,"lng":-105.284703}';

        $request = new Request();
        $request->body = $json;
        $request->mimetype = "Application/json";
        $request->_parseRequestBody();
        $request->method = 'POST';
        $request->url = "/users/" . $user['uuid'] . "/userdeliverylocation";
        $user_controller = new UserController($mt, $user, $request, 5);
        $response = $user_controller->processV2Request();
        $this->assertNull($response->error, "should not have gotten a delivery save error but did");
        $this->assertNotNull($response->user_addr_id);
        $user_address_id = $response->user_addr_id;

        $request = new Request();
        $request->data = array("merchant_id" => $merchant_id, "note" => '', "participant_emails" => $emails, "user_addr_id" => $user_address_id, "group_order_type" => 2, "submit_at_ts" => (getTomorrowTwelveNoonTimeStampDenver() + 900));
        $request->url = "app2/apiv2/grouporders";
        $request->method = "POST";
        $group_order_controller = new GroupOrderController($mt, $user, $request, 5);
        $resource = $group_order_controller->processV2Request();
        $this->assertNull($resource->error);
        $group_order_token = $resource->group_order_token;
        $cart = CartsAdapter::staticGetRecordByPrimaryKey($group_order_token, "CartsAdapter");
        $order_id = $cart['order_id'];
        // check to make sure cart has mdpd_id on record
        $this->assertEquals($mdpd_id,$cart['merchant_delivery_price_distance_id']);
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id, $m);
        $this->assertEquals('D', $complete_order['order_type'], "shoudl have created a delivery dummy order row");
        $this->assertEquals($user_address_id, $complete_order['user_delivery_location_id'], "Should have the user addr id on the order");
        $this->assertEquals(8.00, $complete_order['delivery_amt'], "should have a delivery price on the dummy order");
        $this->assertEquals(getSkinIdForContext(),$complete_order['skin_id'],"skin id shoudl equal the context");
        //$this->assertEquals(0.00, $complete_order['delivery_amt'], "should NOT have a delivery price on the dummy order");
        $group_order_adapter = new GroupOrderAdapter(getM());
        $group_order_record = $group_order_adapter->getRecordFromPrimaryKey($resource->group_order_id);
        $this->assertEquals($user_address_id,$group_order_record['user_addr_id'],'User address id should be on the group order record');

        $group_order_token = $group_order_record['group_order_token'];
        $group_order_id = $group_order_record['group_order_id'];

        $user = logTestUserIn($group_order_record['admin_user_id']);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $cart_data['group_order_token'] = $group_order_token;

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data);
        $this->assertNull($checkout_resource->error);
        $this->assertCount(1,$checkout_resource->lead_times_array,"It should only have 1 available lead time");


        // validate that the order record was added to mapping table
        $cart_order_resource = Resource::find(new CartsAdapter($m),$checkout_resource->ucid);
        $gorma = new GroupOrderIndividualOrderMapsAdapter($m);
        $record = $gorma->getRecord(array("user_order_id"=>$cart_order_resource->order_id));
        $this->assertEquals($group_order_id,$record['group_order_id'],"It should have added the order to the mapping table.");

        // validate that the delivery charge was added to the order record since this is the admin
        $this->assertEquals($group_order_record['user_addr_id'],$cart_order_resource->user_delivery_location_id,"It should have the user address id of the parent order");
        $this->assertEquals(4.44,$cart_order_resource->delivery_amt,"It should have the delivery charge on the order record, but it shoudl have the brink amount of 4.44");


        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,2.22);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $this->assertEquals(4.44,$order_resource->delivery_amt,"It should have the delivery charge on the order record, but it should have the brink amount of 4.44");

        $gorma = new GroupOrderIndividualOrderMapsAdapter($m);
        $record = $gorma->getRecord(array("user_order_id"=>$order_id));
        $this->assertEquals($group_order_id,$record['group_order_id']);

        //test to see if values get added to the parent order record
        $parent_order_record = OrderAdapter::staticGetRecord(array("ucid"=>$group_order_token),'OrderAdapter');
        $this->assertEquals($order_resource->order_amt,$parent_order_record['order_amt']);
        $this->assertEquals($order_resource->total_tax_amt, $parent_order_record['total_tax_amt']);
        $this->assertEquals($order_resource->grand_total, $parent_order_record['grand_total']);
        $this->assertEquals($order_resource->delivery_amt, $parent_order_record['delivery_amt']);

        // now lets add another user and see if they get the delivery fee.
//        $group_order_token = $group_order_record['group_order_token'];
//        $group_order_id = $group_order_record['group_order_id'];

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $order_data = OrderAdapter::getSimpleCartArrayByMerchantId($this->ids['merchant_id']);
        $order_data['group_order_token'] = $group_order_token;
        $order_data["submitted_order_type"]= 'Delivery';
        $order_data['user_addr_id'] = null;

        $cart_resource = getCartResourceFromOrderData($order_data);
        $this->assertEquals(0.00,$cart_resource->delivery_amt);
        $this->assertEquals(2.20,$cart_resource->grand_total);


        $checkout_resource = getCheckoutResourceFromCartUcid($cart_resource->ucid);
        $this->assertNull($checkout_resource->error);
        $this->assertEquals("Pending",$checkout_resource->order_status,"It shoudl have a status of Pending");
        $this->assertEquals(0.00,$checkout_resource->delivery_amt);
        $this->assertEquals(2.20,$checkout_resource->grand_total);


        // validate that the order record was added to mapping table
        $cart_order_resource = Resource::find(new CartsAdapter($m),$checkout_resource->ucid);
        $gorma = new GroupOrderIndividualOrderMapsAdapter($m);
        $record = $gorma->getRecord(array("user_order_id"=>$cart_order_resource->order_id));
        $this->assertEquals($group_order_id,$record['group_order_id'],"It should have added the order to the mapping table.");

        // validate that the delivery charge was NOT added to the order record since this is jsut a participant
        $this->assertEquals($group_order_record['user_addr_id'],$cart_order_resource->user_delivery_location_id,"It should have the user address id of the parent order");
        $this->assertEquals(0.00,$cart_order_resource->delivery_amt,"It should NOT have the delivery charge on the order record since this is a participant and not the admin");

        // now place the order
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $this->assertEquals($cart_order_resource->order_id, $order_id, "should have created a valid order id on teh original cart record");
        $this->assertEquals("D",$order_resource->order_type,"It should be listed as a delivery order");
        $this->assertEquals(0.00,$order_resource->delivery_amt,"There should not be a delivery fee");
        $this->assertEquals($group_order_record['user_addr_id'],$order_resource->user_delivery_location_id,"It should have the UDL of the parent order");


        // lets have the admin make a second order and see if we bypass the delivery fee
        $user = logTestUserIn($group_order_record['admin_user_id']);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $cart_data['group_order_token'] = $group_order_token;

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data);
        $this->assertEquals(0.00,$checkout_resource->delivery_amt,"It should not charge the delivery fee a second time");
        $this->assertEquals(2.20,$checkout_resource->grand_total);

        // validate that the order record was added to mapping table
        $cart_order_resource = Resource::find(new CartsAdapter($m),$checkout_resource->ucid);
        $gorma = new GroupOrderIndividualOrderMapsAdapter($m);
        $record = $gorma->getRecord(array("user_order_id"=>$cart_order_resource->order_id));
        $this->assertEquals($group_order_id,$record['group_order_id'],"It should have added the order to the mapping table.");

        // validate that the delivery charge was NOT added to the order record since this is the admin
        $this->assertEquals($group_order_record['user_addr_id'],$cart_order_resource->user_delivery_location_id,"It should have the user address id of the parent order");
        $this->assertEquals(0.00,$cart_order_resource->delivery_amt,"It should NOT have a delivery charge on the order record");
        $this->assertEquals(2.20,$cart_order_resource->grand_total);

        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $this->assertEquals(0.00,$order_resource->delivery_amt,"It should have the delivery charge on the order record, but it should have the brink amount of 4.44");



        // ************************ now lets submit the group order ************************

        $group_order_token = $group_order_record['group_order_token'];
        $group_order_id = $group_order_record['group_order_id'];
        $user = logTestUserIn($group_order_record['admin_user_id']);

        $request = createRequestObject("/app2/apiv2/grouporders/$group_order_token/submit",'POST',$body,$m);
        $group_order_controller = new GroupOrderController($m,$user,$request,5);
        $response = $group_order_controller->processV2request();
        $this->assertNull($response->error);

        $group_order_token = $group_order_record['group_order_token'];
        $group_base_order = CompleteOrder::staticGetCompleteOrder($group_order_token);

        // check to make sure the messages were created
        $brink_order_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($group_base_order['order_id'],'B');
        $this->assertNotNull($brink_order_message_resource,'It should have created the brink message for the parent order');

        // all status should have been set to 'O'
        $records = GroupOrderIndividualOrderMapsAdapter::staticGetRecords(array("group_order_id"=>$group_order_record['group_order_id']),'GroupOrderIndividualOrderMapsAdapter');
        $this->assertCount(3,$records,"There should be 2 orders");
        foreach($records as $record) {
            $base_order_info = CompleteOrder::getBaseOrderData($record['user_order_id']);
            $this->assertEquals('O',$base_order_info['status']);
        }

        $final_group_order_record = getStaticRecord(array("group_order_id"=>$group_order_record['group_order_id']),'GroupOrderAdapter');
        $this->assertEquals('Submitted',$final_group_order_record['status']);
        $e = explode(' ',$final_group_order_record['sent_ts']);
        $sent_date = $e[0];
        $this->assertEquals(date('Y-m-d'),$sent_date,"shoudl have today as sent");
        $mmha = new MerchantMessageHistoryAdapter(getM());
        $message_resource = $mmha->getExactResourceFromData(array("order_id"=>$group_base_order['order_id'],"message_format"=>'Econf'));
        $message_text = $message_resource->message_text;
        $this->assertTrue(substr_count(strip_tags($message_text), 'Tom, your order will be') == 1,"should have name of group order admin");

        //check to make sure requested_delivery_time propogates to parent order
        $sql = "SELECT a.* from Orders a JOIN Group_Order_Individual_Order_Maps b ON a.order_id = b.user_order_id WHERE group_order_id = $group_order_id";
        $options[TONIC_FIND_BY_SQL] = $sql;
        $order_resources = Resource::findAll(new OrderAdapter(getM()),null,$options);
        $requested_delivery_time = $order_resources[0]->requested_delivery_time;
        $this->assertNotNull($requested_delivery_time,"It should have a requested delivery time");
        $this->assertEquals($requested_delivery_time,$group_base_order['requested_delivery_time'],"It should have picked up the requested_delivery_time from the child order");


        //************************ get order thats delivered to the store************************
        $brink_controller = ControllerFactory::generateFromMessageResource($brink_order_message_resource,$m,$u,$r,5);
        $message_to_send_resource = $brink_controller->prepMessageForSending($brink_order_message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        myerror_log($message_to_send_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);
        $expected_delivery_node = cleanUpDoubleSpacesCRLFTFromString("<Delivery>
                            <Address1>1045 Pine Street</Address1>
                            <Address2>3B and 3C, come around back please</Address2>
                            <City>boulder</City>
                            <Country>USA</Country>
                            <PostalCode>80302</PostalCode>
                            <StateCode>CO</StateCode>
                        </Delivery>");
        $this->assertContains($expected_delivery_node,$body,"should have the delivery info");

        $destination_string = "<DestinationId>66666</DestinationId>";
        $this->assertContains($destination_string,$body,"Brink doordash destination id");

        $this->assertContains('---Doordash Delivery Id:',$body,"It Should contain the DoorDash delivery information");


    }

    function testMultilevelModifier()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 1);
        $modifier_group_id = $modifier_group_resource->modifier_group_id;
        $modifier_item_record = ModifierItemAdapter::staticGetRecord(array("modifier_group_id" => $modifier_group_id), 'ModifierItemAdapter');
        $modifier_size_record = ModifierSizeMapAdapter::staticGetRecord(array("modifier_item_id" => $modifier_item_record['modifier_item_id']), 'ModifierSizeMapAdapter');

        $modifier_size_resource = Resource::find(new ModifierSizeMapAdapter(getM()), "" . $modifier_size_record['modifier_size_id'], []);
        $modifier_size_resource->external_id = "2222-3333:1234-5678";
        $modifier_size_resource->save();

        $item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', getM());
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 0);

        $merchant_resource = createNewTestMerchant($menu_id);
        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'B','brink','X');
        $merchant_id = $merchant_resource->merchant_id;
        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $brink_third_party_parameters = $tpppa->getRecords(['third_party_provider_id' => 1001]);
        foreach ($brink_third_party_parameters as $brink_parameter) {
            if ($brink_parameter['name'] == 'brink_location_token') {
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()), ["merchant_id" => $merchant_id, "third_party_provider_parameter_id" => $brink_parameter['id'], "value" => 'location-77777']);
            }
        }


        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $order_adapter = new OrderAdapter(getM());
        $cart_data = $order_adapter->getCartArrayWithOneModierPerModifierGroup($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data, getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);

        $order_resource = placeOrderFromCheckoutResource($checkout_resource, $user, $merchant_id, 000, $time);
        $this->assertNull($order_resource->error);

        $brink_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id, 'B');
        $this->assertNotNull($brink_message_resource, "we should have a message resource");
        $brink_controller = new BrinkController(getM(), null, $r, 5);
        $ready_to_send_message_resource = $brink_controller->prepMessageForSending($brink_message_resource);
        myerror_log($ready_to_send_message_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);
        $this->assertContains('<ModifierGroupId>2222</ModifierGroupId>', $body);
        $this->assertContains('<ItemId>3333</ItemId>', $body);
        $this->assertContains('<ModifierGroupId>1234</ModifierGroupId>', $body);
        $this->assertContains('<ItemId>5678</ItemId>', $body);

    }

    function testIsMerchantABrinkMerchant()
    {
        $merchant_resource = createNewTestMerchant($this->ids['menu_id']);
        $merchant_id = $merchant_resource->merchant_id;
        $location_id = 'location-123456';
        $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()),["merchant_id"=>$merchant_id,"third_party_provider_parameter_id"=>1000,"value"=>$location_id]);
        $this->assertFalse(MerchantMessageMapAdapter::isMerchantBrinkMerchant($merchant_id),"It should find that this merchant is NOT a brink merchant because there is no map record");

        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'BO','brink','X');
        $this->assertTrue(MerchantMessageMapAdapter::isMerchantBrinkMerchant($this->ids['merchant_id']),"It should find that this merchant IS a brink merchant");
    }



    function testUrl()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $merchant_resource = createNewTestMerchant($this->ids['menu_id']);
        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'B','brink','X');
        $merchant_id = $merchant_resource->merchant_id;
        $location_id = 'location-123456';
        $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()),["merchant_id"=>$merchant_id,"third_party_provider_parameter_id"=>1000,"value"=>$location_id]);
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','sumdumnote');
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $checkout_resource->note = "sumdumnote";
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,1.50,time());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'B');
        $brink_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $brink_controller->sendThisMessage($message_resource);
        $brink_service = $brink_controller->brink_service;
        $this->assertNotNull($brink_service,'It should have instantiated the  brink service');
        $this->assertContains('api88888.brinkpos.net',$brink_service->getUrl());
        $message_record = MerchantMessageHistoryAdapter::staticGetRecordByPrimaryKey($message_resource->map_id,'MerchantMessageHistoryAdapter');
        $message_text = $message_record['message_text'];
        $this->assertContains('SubmitOrder',$message_text);
        $this->assertEquals('SubmitOrder',$brink_service->method);
        $this->assertContains("brink_pickup_destination_id-1000",$message_text);
        $this->assertContains("brink_note_id-1000",$message_text);
        $this->assertContains("55555",$message_text);
    }

    function testNotes()
    {
        $merchant_resource = createNewTestMerchant($this->ids['menu_id']);
        $merchant_id = $merchant_resource->merchant_id;
        Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'B',"delivery_addr"=>"Brink","message_type"=>"X"));
        Resource::createByData(new MerchantBrinkInfoMapsAdapter(getM()),array("merchant_id"=>$merchant_id,"brink_location_token"=>'sumdumtoken'));


        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $note = "sasquatch lives";
        $cart_data['note'] = $note;
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);

        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,000,$time);
        $this->assertNull($order_resource->error);

        $brink_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'B');
        $this->assertNotNull($brink_message_resource,"we shojdl have a message resource");
        $brink_controller = new BrinkController($m, $u, $r, 5);
        $brink_controller->sendThisMessage($brink_message_resource);
        $ready_to_send_message_resource = $brink_controller->prepMessageForSending($brink_message_resource);
        myerror_log($ready_to_send_message_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);
        //check to see that promo is in the XML
        $expected_note_node = cleanUpDoubleSpacesCRLFTFromString('<Items>
                                    <NewOrderItem>
                                        <Description>Order Instructions</Description>
                                        <Id>1</Id>
                                        <ItemId>'.$this->ids['brink_note_id'].'</ItemId>
                                        <Modifiers i:nil="true" />
                                        <Price>0.00</Price>
                                        <DestinationId>'.$this->ids['brink_pickup_destination_id'].'</DestinationId>
                                        <Note>'.$note.'</Note>
                                    </NewOrderItem>');
        $this->assertContains($expected_note_node,$body,"It should have the  note node");

        $brink_controller = new BrinkController(getM(),null, $r,5);
        $brink_controller->sendThisMessage($brink_message_resource);
        $brink_service = $brink_controller->brink_service;
        $this->assertContains('api.brinkpos.net',$brink_service->getUrl());

    }

    function testPromo()
    {

        $merchant_id = $this->ids['merchant_id'];

        $user_resource = createNewUserWithCCNoCVV(array("email"=>"bobnewhart@order140.com","first_name"=>"bob","last_name"=>"newhart","contact_no"=>'8888888888'));
        $user = logTestUserResourceIn($user_resource);
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $request = createRequestObject('app2/apiv2/cart','post',json_encode($cart_data),'application/json');
        $place_order_controller = new PlaceOrderController(getM(),$user,$request,5);
        $create_cart_response = $place_order_controller->processV2Request();
        $this->assertNull($create_cart_response->error);
        $complete_order = CompleteOrder::staticGetCompleteOrder($create_cart_response->oid_test_only,getM());
        $splickit_sub_total = $complete_order['order_details'][0]['item_total_w_mods'];
        $ucid = $create_cart_response->ucid;
        $request = createRequestObject("app2/apiv2/cart/$ucid/checkout?promo_code=type1promo","GET");
        $place_order_controller = new PlaceOrderController(getM(),$user,$request,5);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($checkout_resource->error,"It should not throw an error");
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00,$time);
        $this->assertNull($order_resource->error,"It should not throw an error on place order with promo");
        $this->assertEquals(-$splickit_sub_total/2,$order_resource->promo_amt,"It should have the promo amount");

        $brink_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'B');
        $this->assertNotNull($brink_message_resource,"we shojdl have a message resource");
        $brink_controller = new BrinkController($m, $u, $r, 5);
        $ready_to_send_message_resource = $brink_controller->prepMessageForSending($brink_message_resource);
        myerror_log($ready_to_send_message_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);
        //check to see that promo is in the XML
        $expected_promo_node = cleanUpDoubleSpacesCRLFTFromString('<Discounts>
						<NewOrderDiscount>
							<Amount>'.number_format($splickit_sub_total/2,2).'</Amount>
							<DiscountId>'.$this->ids['brink_promo_discount_id'].'</DiscountId>
							<Id>1</Id>
							<LoyaltyRewardId>0</LoyaltyRewardId>
							<Name i:nil="true"/>
							<OrderItemIds i:nil="true" xmlns:a="http://schemas.microsoft.com/2003/10/Serialization/Arrays"/>
							<Percent>0</Percent>
						</NewOrderDiscount>
					</Discounts>');
        $this->assertContains($expected_promo_node,$body,"It should have the promo node");
        $this->assertEquals($splickit_sub_total,$order_resource->order_amt,"When promo is used, we need OUR subtotal");

        //make sure totals are correct.
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id,$m);
        $grand_total_from_math = $complete_order['order_amt']+$complete_order['total_tax_amt']+$complete_order['delivery_amt']+$complete_order['tip_amt']+$complete_order['promo_amt'];
        $this->assertEquals($grand_total_from_math,$complete_order['grand_total'],"the set grand total should equal the sum of the parts");

    }


    function testDeliveryBrink()
    {
        setContext("com.splickit.smilingmoose");
        $user = logTestUserIn($this->ids['user_id']);
        $json = '{"user_addr_id":null,"user_id":"'.$user['user_id'].'","name":"","address1":"1045 Pine Street","address2":"3B & 3C","city":"boulder","state":"co","zip":"80302","instructions":"come around back please","phone_no":"9709262121","lat":40.0197891,"lng":-105.284703}';
        $request = new Request();
        $request->body = $json;
        $request->mimetype = "Application/json";
        $request->_parseRequestBody();
        $request->method = 'POST';
        $request->url = "/users/".$user['uuid']."/userdeliverylocation";
        $user_controller = new UserController($mt, $user, $request,5);
        //$response = $user_controller->setDeliveryAddr();
        $response = $user_controller->processV2Request();
        $this->assertNull($response->error,"should not have gotten a delivery save error but did");
        $this->assertNotNull($response->user_addr_id);
        $user_address_id = $response->user_addr_id;


        $merchant_id = $this->ids['merchant_id'];
        $order_data = OrderAdapter::staticGetSimpleOrderArrayByMerchantId($merchant_id, 'delivery', 'sum dum note');
        $order_data['items'][0]['note'] = 'item note';
        $order_data['user_addr_id'] = $user_address_id;
        $order_data['tip'] = 0.00;
        $order_resource = placeOrderFromOrderData($order_data, getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'B');

        $brink_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $brink_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        myerror_log($message_to_send_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);
        $expected_delivery_node = cleanUpDoubleSpacesCRLFTFromString("<Delivery>
                            <Address1>1045 Pine Street</Address1>
                            <Address2>3B and 3C, come around back please</Address2>
                            <City>boulder</City>
                            <Country>USA</Country>
                            <PostalCode>80302</PostalCode>
                            <StateCode>CO</StateCode>
                        </Delivery>");
        $this->assertContains($expected_delivery_node,$body,"should have the delivery info");

        $destination_string = "<DestinationId>".$this->ids['brink_delivery_destination_id']."</DestinationId>";
        $this->assertContains($destination_string,$body,"it should have the zone 1 destination id");

/*        $expected_delivery_surcharge_node = cleanUpDoubleSpacesCRLFTFromString('<Surcharges>
                  <OrderSurcharge>
                     <Amount>8.18</Amount>
                     <Id>1</Id>
                     <IsSystemApplied>true</IsSystemApplied>
                     <Name>Delivery Fee 2</Name>
                     <SurchargeId>88888888</SurchargeId>
                     <Taxes />
                  </OrderSurcharge>
               </Surcharges>');
        $expected_delivery_surcharge_node = str_ireplace('> <','><',$expected_delivery_surcharge_node);
        $this->assertContains($expected_delivery_surcharge_node,$body,"It should contain the delivery surcharge node");
*/
    }

    function testGenerateTemplate()
    {
        $user = logTestUserIn($this->ids['user_id']);
        $merchant_id = $this->ids['merchant_id'];
        $order_data = OrderAdapter::staticGetSimpleOrderArrayByMerchantId($merchant_id, 'pickup', '');
        $order_data['tip'] = 3.33;
        $order_data['note'] = '';
        $order_resource = placeOrderFromOrderData($order_data, getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $messages = MerchantMessageHistoryAdapter::getAllOrderMessages($order_id);
        $messages_hash = createHashmapFromArrayOfResourcesByFieldName($messages, 'message_format');
        $this->assertNotNull($messages_hash['B'], "SHould have created a message with the Brink format");
        return $messages_hash['B'];
    }

    /**
     * @depends testGenerateTemplate
     */
    function testCreateTemplateFromMessageResourece($message_resource)
    {
        $this->assertTrue(true);
        $expected_payload = cleanUpDoubleSpacesCRLFTFromString(file_get_contents("./unit_tests/resources/expected_brink_message_body.txt"));
        $order_id = $message_resource->order_id;
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
        $this->assertEquals(3.33,$complete_order['tip_amt']);
        $user = $complete_order['user'];
        $expected_payload = str_replace("%%email%%",$user['email'],$expected_payload);
        $expected_payload = str_replace("%%item_external_id%%",$complete_order['order_details'][0]['external_id'],$expected_payload);
        $tz = date_default_timezone_get();
        date_default_timezone_set('GMT');
        $brink_pickup_time_string = date("Y-m-d\TH:i:00.0000\Z",$complete_order['pickup_dt_tm']);
        date_default_timezone_set($tz);
        $expected_payload = str_replace("%%brink_pickup_time%%",$brink_pickup_time_string,$expected_payload);




        $brink_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $brink_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);

        $this->assertContains('<Amount>2.20</Amount>',$body);
        $this->assertContains('<TipAmount>3.33</TipAmount>',$body);


    }

    function testMulitpleItemsOnTemplate()
    {
        $user = logTestUserIn($this->ids['user_id']);
        $merchant_id = $this->ids['merchant_id'];
        $order_data = OrderAdapter::staticGetSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'sum dum note');
        $order_data['items'][0]['quantity'] = 2;
        $order_data['tip'] = 0.00;
        $order_data['note'] = '';
        $order_resource = placeOrderFromOrderData($order_data, getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id);
        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'B');
        $brink_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $brink_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        myerror_log($message_to_send_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);

        $cm = new CompleteMenu($this->ids['menu_id']);
        $isrs = $cm->getAllMenuItemSizeMapResources($this->ids['menu_id'],'Y',0);
        $item_external_id = $isrs[0]->external_id;

        $this->assertContains('<Price>1.50</Price>',$body,"It should contain the single item price on each of the nodes");
        $this->assertNotContains('<Price>3.00</Price>',$body,"It should NOT contain the full price of the ordered item");


        $expected_item_payload = cleanUpDoubleSpacesCRLFTFromString('<Items>
                  <NewOrderItem>
                     <Description i:nil="true" />
                     <Id>1</Id>
                     <ItemId>'.$item_external_id.'</ItemId>
                     <Modifiers>
                        <NewOrderItemModifier>
                           <Description i:nil="true" />
                           <Id>2</Id>
                           <ItemId>5678</ItemId>
                           <Modifiers i:nil="true" />
                           <Price>0.500</Price>
                           <ModifierCodeId>0</ModifierCodeId>
                           <ModifierGroupId>1234</ModifierGroupId>
                        </NewOrderItemModifier>
                     </Modifiers>
                     <Price>1.50</Price>
                     <DestinationId>'.$this->ids['brink_pickup_destination_id'].'</DestinationId>
                     <Note i:nil="true" />
                  </NewOrderItem>
                  <NewOrderItem>
                     <Description i:nil="true" />
                     <Id>3</Id>
                     <ItemId>'.$item_external_id.'</ItemId>
                     <Modifiers>
                        <NewOrderItemModifier>
                           <Description i:nil="true" />
                           <Id>4</Id>
                           <ItemId>5678</ItemId>
                           <Modifiers i:nil="true" />
                           <Price>0.500</Price>
                           <ModifierCodeId>0</ModifierCodeId>
                           <ModifierGroupId>1234</ModifierGroupId>
                        </NewOrderItemModifier>
                     </Modifiers>
                     <Price>1.50</Price>
                     <DestinationId>'.$this->ids['brink_pickup_destination_id'].'</DestinationId>
                     <Note i:nil="true" />
                  </NewOrderItem>
               </Items>');
        $expected_item_payload = str_ireplace('> <','><',$expected_item_payload);
        $this->assertContains($expected_item_payload,$body,"It should have 2 item nodes since the item had a quantity of 2");

    }

    function testMulitpleModifiersOnTemplate()
    {
        $user = logTestUserIn($this->ids['user_id']);
        $merchant_id = $this->ids['merchant_id'];
        $order_data = OrderAdapter::staticGetSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'sum dum note');
        $order_data['items'][0]['mods'][0]['mod_quantity'] = 3;
        $order_data['tip'] = 0.00;
        $order_data['note'] = '';
        $order_resource = placeOrderFromOrderData($order_data, getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id);
        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'B');
        $brink_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $brink_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        myerror_log($message_to_send_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);
        $expected_modifier_payload = cleanUpDoubleSpacesCRLFTFromString('
                                <NewOrderItemModifier>
                                    <Description i:nil="true" />
                                    <Id>2</Id>
                                    <ItemId>5678</ItemId>
                                    <Modifiers i:nil="true" />
                                    <Price>0.500</Price>
                                    <ModifierCodeId>0</ModifierCodeId>
                                    <ModifierGroupId>1234</ModifierGroupId>
                                </NewOrderItemModifier>
                                <NewOrderItemModifier>
                                    <Description i:nil="true" />
                                    <Id>3</Id>
                                    <ItemId>5678</ItemId>
                                    <Modifiers i:nil="true" />
                                    <Price>0.500</Price>
                                    <ModifierCodeId>0</ModifierCodeId>
                                    <ModifierGroupId>1234</ModifierGroupId>
                                </NewOrderItemModifier>
                                <NewOrderItemModifier>
                                    <Description i:nil="true" />
                                    <Id>4</Id>
                                    <ItemId>5678</ItemId>
                                    <Modifiers i:nil="true" />
                                    <Price>0.500</Price>
                                    <ModifierCodeId>0</ModifierCodeId>
                                    <ModifierGroupId>1234</ModifierGroupId>
                                </NewOrderItemModifier>
                                ');

        $this->assertContains($expected_modifier_payload,$body,"it should have all three modifier nodes since there is no such thing as quantity for the template");

    }

    function testDeliveryBrinkWithDoorDash()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $json = '{"user_addr_id":null,"user_id":"'.$user['user_id'].'","name":"","address1":"1045 Pine Street","address2":"3B & 3C","city":"boulder","state":"co","zip":"80302","instructions":"come around back please","phone_no":"9709262121","lat":40.0197891,"lng":-105.284703}';
        $request = new Request();
        $request->body = $json;
        $request->mimetype = "Application/json";
        $request->_parseRequestBody();
        $request->method = 'POST';
        $request->url = "/users/".$user['uuid']."/userdeliverylocation";
        $user_controller = new UserController($mt, $user, $request,5);
        //$response = $user_controller->setDeliveryAddr();
        $response = $user_controller->processV2Request();
        $this->assertNull($response->error,"should not have gotten a delivery save error but did");
        $this->assertNotNull($response->user_addr_id);
        $user_address_id = $response->user_addr_id;


        $merchant_resource = createNewTestMerchantDelivery($this->ids['menu_id'],['long_hours'=>true]);
        $merchant_resource->merchant_external_id = 'Dash-123456';
        $merchant_resource->save();
        $merchant_id = $merchant_resource->merchant_id;

        Resource::createByData(new MerchantBrinkInfoMapsAdapter(getM()),array("merchant_id"=>$merchant_id,"brink_location_token"=>getProperty('brink_test_location_token')));
        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_id,'B','brink','X');


        $mdia = new MerchantDeliveryPriceDistanceAdapter(getM());
        $mdia_resource = $mdia->getExactResourceFromData(["merchant_id"=>$merchant_id]);
        $mdia_resource->minimum_order_amount = 0.01;
        $mdia_resource->delivery_cost = 1.00;
        $mdia_resource->delivery_increment = 15;
        $mdia_resource->max_days_out = 3;
        $mdia_resource->save();


        $mdpd_resource = Resource::find(new MerchantDeliveryPriceDistanceAdapter(getM()),'',array(3=>array("merchant_id"=>$merchant_id)));
        $mdpd_resource->delivery_price_type = 'Doordash';
        $mdpd_resource->name = 'Doordash';
        $mdpd_resource->distance_up_to = 'nullit';
        $mdpd_resource->price = -1.00;
        $mdpd_resource->notes = "external_business_name=sumdumbiz;brink_destination_id=88888";
        $mdpd_resource->save();
        $mdpd_resource = $mdpd_resource->getRefreshedResource();



        $order_data = OrderAdapter::staticGetSimpleOrderArrayByMerchantId($merchant_id, 'delivery', 'sum dum note');
        $order_data['items'][0]['note'] = 'item note';
        $order_data['user_addr_id'] = $user_address_id;
        $order_data['tip'] = 0.00;
        $checkout_resource = getCheckoutResourceFromOrderData($order_data,time());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
//        $order_resource = placeOrderFromOrderData($order_data, getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'B');

        $brink_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $brink_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        myerror_log($message_to_send_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);
        $expected_delivery_node = cleanUpDoubleSpacesCRLFTFromString("<Delivery>
                            <Address1>1045 Pine Street</Address1>
                            <Address2>3B and 3C, come around back please</Address2>
                            <City>boulder</City>
                            <Country>USA</Country>
                            <PostalCode>80302</PostalCode>
                            <StateCode>CO</StateCode>
                        </Delivery>");
        $this->assertContains($expected_delivery_node,$body,"should have the delivery info");

        $destination_string = "<DestinationId>88888</DestinationId>";
        $this->assertContains($destination_string,$body,"Brink doordash destination id");

        /*        $expected_delivery_surcharge_node = cleanUpDoubleSpacesCRLFTFromString('<Surcharges>
                          <OrderSurcharge>
                             <Amount>8.18</Amount>
                             <Id>1</Id>
                             <IsSystemApplied>true</IsSystemApplied>
                             <Name>Delivery Fee 2</Name>
                             <SurchargeId>88888888</SurchargeId>
                             <Taxes />
                          </OrderSurcharge>
                       </Surcharges>');
                $expected_delivery_surcharge_node = str_ireplace('> <','><',$expected_delivery_surcharge_node);
                $this->assertContains($expected_delivery_surcharge_node,$body,"It should contain the delivery surcharge node");
        */
    }

    function placeOrderToBePartOfGroupOrder($user, $merchant_id, $ucid, $checkout_data_resource, $time)
    {
        $order_data = array();
        $order_data['merchant_id'] = $merchant_id;
        $order_data['user_id'] = $user['user_id'];
        $order_data['cart_ucid'] = $ucid;
        //$order_data['tip'] = (rand(100, 1000)) / 100;
        if ($payment_array = $checkout_data_resource->accepted_payment_types) {
            $order_data['merchant_payment_type_map_id'] = $payment_array[0]['id'];
        }
        if ($lead_times_array = $checkout_data_resource->lead_times_array) {
            $order_data['actual_pickup_time'] = $lead_times_array[0];
        }

        $json_encoded_data = json_encode($order_data);
        $request = createRequestObject("/apiv2/orders/$ucid",'POST',$json_encoded_data);
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $order_resource = $place_order_controller->processV2Request();
        return $order_resource;
    }

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        //$skin = getOrCreateSkinAndBrandIfNecessary("Smiling Moose","Smiling Moose",6,152);
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time',300);
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction();
        $skin_resource = getOrCreateSkinAndBrandIfNecessary('thirdpartybrinkskin','thirdpartybrinkbrand');
        setContext($skin_resource->external_identifier);
        $ids['skin_id'] = getSkinIdForContext();
        $brand_id = getBrandIdFromCurrentContext();

        //map it to a menu
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $ids['menu_id'] = $menu_id;

        $cm = new CompleteMenu($menu_id);
        $modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 1);
        $modifier_group_id = $modifier_group_resource->modifier_group_id;
        $modifier_item_record = ModifierItemAdapter::staticGetRecord(array("modifier_group_id"=>$modifier_group_id),'ModifierItemAdapter');
        $modifier_size_record = ModifierSizeMapAdapter::staticGetRecord(array("modifier_item_id"=>$modifier_item_record['modifier_item_id']),'ModifierSizeMapAdapter');

        $modifier_size_resource = Resource::find(new ModifierSizeMapAdapter(getM()),"".$modifier_size_record['modifier_size_id'],[]);
        $modifier_size_resource->external_id = "1234-5678";
        $modifier_size_resource->save();
        $item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', getM());
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 0);

        $merchant_resource = createNewTestMerchantDelivery($menu_id);
        $merchant_resource->merchant_external_id = 'sumdum-123456';
        $merchant_resource->group_ordering_on = 1;
        $merchant_resource->save();
        $merchant_id = $merchant_resource->merchant_id;

        $merchant_delivery_price_resource = Resource::find(new MerchantDeliveryPriceDistanceAdapter(getM()),'',array(3=>array("merchant_id"=>$merchant_id)));
        $merchant_delivery_price_resource->name = "zone_1";
//        $merchant_delivery_price_resource->notes = "brink_destination_id=640214764";
        $merchant_delivery_price_resource->save();

        $merchant_delivery_price_resource = Resource::createByData(new MerchantDeliveryPriceDistanceAdapter($m),array("merchant_id"=>$merchant_resource->merchant_id,"distance_up_to"=>"1000","name"=>"zone_2"));
        $merchant_delivery_price_resource->notes = "brink_destination_id=delivery_override-123456";
        $merchant_delivery_price_resource->save();


        attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
        $ids['merchant_id'] = $merchant_resource->merchant_id;
        //Resource::createByData(new MerchantBrinkInfoMapsAdapter($m),array("merchant_id"=>$merchant_resource->merchant_id,"brink_location_token"=>getProperty('brink_test_location_token')));
        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'B','brink','X');

        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $brink_third_party_parameters = $tpppa->getRecords(['third_party_provider_id'=>1001]);
        foreach ($brink_third_party_parameters as $brink_parameter) {
            if ($brink_parameter['name'] == 'brink_location_token') {
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()), ["merchant_id" => $merchant_id, "third_party_provider_parameter_id" => $brink_parameter['id'], "value" => 'location-888888']);
            } else if ($brink_parameter['name'] == 'brink_stored_value_tender_id') {
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()),["brand_id"=>$brand_id,"third_party_provider_parameter_id"=>$brink_parameter['id'],"value"=>'33333']);
            } else if ($brink_parameter['name'] == 'brink_tender_id') {
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()),["brand_id"=>$brand_id,"third_party_provider_parameter_id"=>$brink_parameter['id'],"value"=>'55555']);
            } else {
                $id = $brink_parameter['id'];
                $name = $brink_parameter['name'];
                $value = $name.'-1000'.$id;
                if ($name == 'brink_domain') {
                    $value = 'api88888.brinkpos.net';
                }
                $ids[$name] = $value;
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()),["brand_id"=>$brand_id,"third_party_provider_parameter_id"=>$brink_parameter['id'],"value"=>$value]);
            }
        }

//
//        $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()),["merchant_id"=>$merchant_id,"third_party_provider_parameter_id"=>1000,"value"=>'location-888888']);
//        $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()),["brand_id"=>$brand_id,"third_party_provider_parameter_id"=>1038,"value"=>'tip-10001038']);
//        $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()),["brand_id"=>$brand_id,"third_party_provider_parameter_id"=>1039,"value"=>'pickup-10001039']);
//        $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()),["brand_id"=>$brand_id,"third_party_provider_parameter_id"=>1040,"value"=>'delivery-10001040']);
//        $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()),["brand_id"=>$brand_id,"third_party_provider_parameter_id"=>1041,"value"=>'note-10001041']);
//        $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()),["brand_id"=>$brand_id,"third_party_provider_parameter_id"=>1042,"value"=>'tender-10001042']);
//        $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()),["brand_id"=>$brand_id,"third_party_provider_parameter_id"=>1043,"value"=>'promo-10001043']);
//        $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()),["brand_id"=>$brand_id,"third_party_provider_parameter_id"=>1044,"value"=>'loyalty-10001044']);
//        $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()),["brand_id"=>$brand_id,"third_party_provider_parameter_id"=>1045,"value"=>'api88888.brinkpos.net']);



        $user_resource = createNewUserWithCCNoCVV(array("contact_no"=>'123 456 7890'));
        $ids['user_id'] = $user_resource->user_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;


        $promo_adapter = new PromoAdapter($m);
        $promo_id = 111;
        $ids['promo_id'] = $promo_id;
        if ($promo_record = $promo_adapter->getRecordFromPrimaryKey($promo_id)) {
            ;
        } else {
            $ids['promo_id_type_1'] = $promo_id;
            $sql = "INSERT INTO `Promo` VALUES($promo_id, 'The Type1 Promo', 'Get Up to $5 off', 1, 'Y', 'N', 0, 2, 'N', 'N','all', '2010-01-01', '2030-01-01', 100000,FALSE, 0, 0.00, 0, 0.00, 'Y', 'N',0, 282, NOW(), NOW(), 'N')";
            $promo_adapter->_query($sql);
//            $sql = "INSERT INTO `Promo_Merchant_Map` VALUES(null, 201, $merchant_resource->merchant_id, '2013-10-05', '2030-01-01', NULL, now())";
//            $promo_adapter->_query($sql);
            $pmm_resource = Resource::createByData(new PromoMerchantMapAdapter(getM()), array("merchant_id"=>$merchant_resource->merchant_id,"promo_id"=>$promo_id));
            $ids['promo_merchant_map_id_type_1'] = $pmm_resource->map_id;
            $sql = "INSERT INTO `Promo_Message_Map` VALUES(null, $promo_id, 'Congratulations! You''re getting %%amt%% off your order!', NULL, NULL, NULL, NULL, now())";
            $promo_adapter->_query($sql);
            $sql = "INSERT INTO `Promo_Type1_Amt_Map` VALUES(null, $promo_id, 1.00, 0, 50,50.00, NOW())";
            $promo_adapter->_query($sql);
            $pkwm_adapter = new PromoKeyWordMapAdapter($m);
            Resource::createByData($pkwm_adapter, array("promo_id"=>$promo_id,"promo_key_word"=>"type1promo","brand_id"=>282));

        }
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    BrinkControllerTestNewThirdPartyStuff::main();
}

?>