<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class ComprisControllerTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
        $this->ids = $_SERVER['unit_test_ids'];
        setContext("com.splickit.comprisskin");

    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }

    function testNoDeliveryDiscountForPickupNative()
    {
        $merchant_resource = createNewTestMerchant($this->ids['menu_id']);
        $merchant_id = $merchant_resource->merchant_id;
        $merchant_alphanumeric = $merchant_resource->alphanumeric_id;
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'O',"delivery_addr"=>"Compris","message_type"=>"X"));

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $order_resource = OrderController::getResourceFromId($order_id,'Order');
        $this->assertEquals('O',$order_resource->status);

        $compris_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'O');
        $compris_message_resource->next_message_dt_tm = time()-10;
        $compris_message_resource->save();

        $url = "https://sumdumdomain/app2/messagemanager/getnextmessagebymerchantid/$merchant_alphanumeric/compris";
        $request = createRequestObject($url,'GET');
        $message_controller = ControllerFactory::generateFromUrl($url, getM(), null, $request, 5);
        $response_message_resource = $message_controller->pullNextMessageResourceByMerchant($merchant_alphanumeric);
        $this->assertTrue(isset($response_message_resource->message_id),'It should have the message resource');
        $this->assertNotContains('<OrderLevelDiscount SysId="77383"/>',$response_message_resource->message_text);

    }

    function testPromoForItem()
    {

        $menu_id = createTestMenuWithNnumberOfItems(2);
        $item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', getM());
        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $merchant_alphanumeric = $merchant_resource->alphanumeric_id;
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'O',"delivery_addr"=>"Compris","message_type"=>"X"));

        //create the type 4 promo
        $promo_data = [];
        $promo_data['key_word'] = 'CompPromo';
        $promo_data['promo_type'] = 4;
        $promo_data['description'] = 'Get $ off your item';
        $promo_data['start_date'] = '2010-01-01';
        $promo_data['end_date'] = '2024-01-01';
        $promo_data['max_use'] = 100;
        $promo_data['merchant_id'] = $merchant_id;
        $promo_data['message5'] = "Here's the deal, order a large this, and you'll get a discount! Limit 1";
        $promo_data['qualifying_object_array'] = ['Item-888888','Item-'.$item_records[1]['item_id']];
        //$promo_data['qualifying_object_array'] = ['Menu_Type-'.$item_records[1]['menu_type_id']];
        $promo_data['fixed_amount_off'] = .50;
        $promo_data['brand_id'] = getBrandIdFromCurrentContext();
        $promo_data['external_id'] = '12345678';

        $request = createRequestObject("/app2/portal/promos",'POST',json_encode($promo_data));
        $promo_controller = new PromoController(getM(),null,$request,5);
        $response = $promo_controller->processV2Request();
        $promo_id = $response->promo_id;

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note',3);
        $cart_data['promo_code'] = 'CompPromo';

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $this->assertEquals(-0.50,$checkout_resource->promo_amt);

        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id);

        $compris_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'O');
        $compris_message_resource->next_message_dt_tm = time()-10;
        $compris_message_resource->save();

        $url = "https://sumdumdomain/app2/messagemanager/getnextmessagebymerchantid/$merchant_alphanumeric/compris";
        $request = createRequestObject($url,'GET');
        $message_controller = ControllerFactory::generateFromUrl($url, getM(), null, $request, 5);
        $response_message_resource = $message_controller->pullNextMessageResourceByMerchant($merchant_alphanumeric);
        $this->assertTrue(isset($response_message_resource->message_id),'It should have the message resource');
        $body = $response_message_resource->message_text;
        myerror_log($body);
        $expected_discount_node = '<ItemLevelCoupon SysId="12345678"/>';
        $this->assertContains($expected_discount_node,$body,'It should have the discount node for the item');



    }

//    function testDoNotScheduleOrderMessageIfOneIsScheduledWithNsecondsOnEitherSide()
//    {
//        $merchant_resource = createNewTestMerchant($this->ids['menu_id']);
//        $merchant_id = $merchant_resource->merchant_id;
//        $merchant_alphanumeric = $merchant_resource->alphanumeric_id;
//        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'O',"delivery_addr"=>"Compris","message_type"=>"X"));
//
//        $user_resource = createNewUserWithCCNoCVV();
//        $user = logTestUserResourceIn($user_resource);
//
//        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
//        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
//        $this->assertNull($checkout_resource->error);
//        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
//        $this->assertNull($order_resource->error);
//        $order_id = $order_resource->order_id;
//
//        $first_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'O');
//
//        $user_resource = createNewUserWithCCNoCVV();
//        $user = logTestUserResourceIn($user_resource);
//
//        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
//        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
//        $this->assertNull($checkout_resource->error);
//        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
//        $this->assertNull($order_resource->error);
//        $order_id = $order_resource->order_id;
//
//        $second_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'O');
//
//        $diff = $second_message_resource->next_message_dt_tm - $first_message_resource->next_message_dt_tm;
//        $this->assertEquals(10,$diff,'There should be 10 seconds betweeb the scheduled messages');
//
//
//    }

    function testForceTimeBetweenStatusChecks()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id );
        $merchant_id = $merchant_resource->merchant_id;
        $merchant_alphanumeric = $merchant_resource->alphanumeric_id;
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'O',"delivery_addr"=>"Compris","message_type"=>"X"));

        $url = "https://sumdumdomain/app2/messagemanager/getnextmessagebymerchantid/$merchant_alphanumeric/compris";
        $request = createRequestObject($url,'GET');
        $message_controller = ControllerFactory::generateFromUrl($url, getM(), null, $request, 5);
        $response_message_resource = $message_controller->pullNextMessageResourceByMerchant($merchant_alphanumeric);
        $body = $response_message_resource->message_text;
        myerror_log($body);
        $packet_id = "STATUS-".time();
        $expected_body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Request Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus/></Request></CommunicationCenter>';
        $this->assertEquals($expected_body,$body);

        // now call again and we should get null
        $url = "https://sumdumdomain/app2/messagemanager/getnextmessagebymerchantid/$merchant_alphanumeric/compris";
        $request = createRequestObject($url,'GET');
        $message_controller = ControllerFactory::generateFromUrl($url, getM(), null, $request, 5);
        $response_message_resource = $message_controller->pullNextMessageResourceByMerchant($merchant_alphanumeric);
        $this->assertNull($response_message_resource,'It should NOT create another message untill 20 seconds has passed');

        // now set sentdttm to further in the past
        $adapter = new MerchantMessageHistoryAdapter(getM());
        $last_status_message_resource = Resource::findOrCreateIfNotExistsByData($adapter,['merchant_id'=>$merchant_resource->merchant_id,'message_format'=>'OS']);
        $offset = getProperty('comprise_status_message_buffer_in_seconds') + 1;
        $last_status_message_resource->sent_dt_tm = date("Y-m-d H:i:s",time()-$offset);
        $last_status_message_resource->locked = 'S';
        $last_status_message_resource->save();

        $url = "https://sumdumdomain/app2/messagemanager/getnextmessagebymerchantid/$merchant_alphanumeric/compris";
        $request = createRequestObject($url,'GET');
        $message_controller = ControllerFactory::generateFromUrl($url, getM(), null, $request, 5);
        $response_message_resource = $message_controller->pullNextMessageResourceByMerchant($merchant_alphanumeric);
        $body = $response_message_resource->message_text;
        myerror_log($body);
        $packet_id = "STATUS-".time();
        $expected_body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Request Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus/></Request></CommunicationCenter>';
        $this->assertEquals($expected_body,$body);


    }

    function testSetAlertMessageToOneHourBeforeOpeningTimeForMerchantAfterMidnight()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id );
        $merchant_resource->merchant_external_id = "88888888";
        $merchant_resource->state = 'NY';
        $merchant_resource->time_zone = '-5';
        $merchant_resource->time_zone_string = 'America/New_York';
        $merchant_resource->save();
        $merchant_id = $merchant_resource->merchant_id;
        $merchant_alphanumeric = $merchant_resource->alphanumeric_id;
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'O',"delivery_addr"=>"Compris","message_type"=>"X"));

        $url = "https://sumdumdomain/app2/messagemanager/getnextmessagebymerchantid/$merchant_alphanumeric/compris";
        $request = createRequestObject($url,'GET');
        $message_controller = ControllerFactory::generateFromUrl($url, getM(), null, $request, 5);
        $response_message_resource = $message_controller->pullNextMessageResourceByMerchant($merchant_alphanumeric);
        $body = $response_message_resource->message_text;
        myerror_log($body);
        $packet_id = "STATUS-".time();
        $expected_body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Request Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus/></Request></CommunicationCenter>';
        $this->assertEquals($expected_body,$body);

//        $dciha = new DeviceCallInHistoryAdapter(getM());
//        $last_call_in = $dciha->getCallInResourceByMerchantId($merchant_id);
//        $this->assertNull($last_call_in);

        $url = "https://sumdumdomain/app2/messagemanager/$merchant_alphanumeric/compris/callback.txt";
        $body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Response Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus Turn="BAT-A.00013" PosTerminal="mws" ResultCode="50" ResultText="Pos is not runnning"/></Response></CommunicationCenter>';
        $callback_request = createRequestObject($url,'POST',$body,'application/xml');
        $compris_callback_controller = ControllerFactory::getMessageControllerFromRequest($callback_request);
        $this->assertEquals('callback',$compris_callback_controller->getSubmittedAction());
        $id = $compris_callback_controller->getSubmittedIdentifier();
        $ts = getTimeStampForDateTimeAndTimeZone(1,15,00,date('m'),date('d'),date('Y'));
        $compris_callback_controller->setCurrentTime($ts);
        $response = $compris_callback_controller->callBack($id);
        $this->assertFalse($response);

        $merchant_resource = $merchant_resource->getRefreshedResource();
        $this->assertEquals('N',$merchant_resource->ordering_on,"ordering_on should have been set to 'N'");
        $this->assertEquals('F',$merchant_resource->inactive_reason,"inactive_reason should have been set to 'F'");

        $dciha = new DeviceCallInHistoryAdapter(getM());
        $last_call_in_resource = $dciha->getCallInResourceByMerchantId($merchant_id);
        $this->assertEquals(1,$last_call_in_resource->auto_turned_off);

        //check time of message
        $mmha = new MerchantMessageHistoryAdapter(getM());
        $options[TONIC_FIND_BY_METADATA] = ['merchant_id'=>$merchant_id,'message_format'=>'Eoff','locked'=>'N'];
        $message_resource = Resource::find($mmha,null,$options);
        $map_id = $message_resource->map_id;
        $this->assertNotNull($message_resource,"There should be a message resource");
        // since server is in denver we expect 7am in NY but 5am in denver
        $date = date('Y-m-d',$ts);
        $hour_adapter = new HourAdapter(getM());
        $todays_hours = $hour_adapter->getHourRecordForDate($date, $merchant_id, 'R');
        $open = $todays_hours['open'];
        $h = explode(':',$open);
        $hour = $h[0];
        $min = $h[1];
        $expected_send_on_time = getTimeStampForDateTimeAndTimeZone($hour,$min,00,date('m'),date('d'),date('Y'),'America/New_York')-ComprisController::SECONDS_BEFORE_SOD_NOT_RUN_ALERT;
        $actual_send_on_time = $message_resource->next_message_dt_tm;
        $expected_string = getMySqlFormattedDateTimeFromTimeStampAndTimeZone($expected_send_on_time);
        $actual_string = getMySqlFormattedDateTimeFromTimeStampAndTimeZone($actual_send_on_time);
        $this->assertEquals($expected_string,$actual_string,"it should be scheduled for 1 hour before opening");

        //now send a pos reeady and see if it cancels the message
        $dciha = new DeviceCallInHistoryAdapter(getM());
        $last_call_in_resource = $dciha->getCallInResourceByMerchantId($merchant_id);
        $this->assertEquals(1,$last_call_in_resource->auto_turned_off);

        $url = "https://sumdumdomain/app2/messagemanager/$merchant_alphanumeric/compris/callback.txt";
        $body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Response Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus Turn="BAT-A.00001" PosTerminal="mws" PosId="3" ResultCode="0" ResultText="Pos is ready"/></Response></CommunicationCenter>';
        $callback_request = createRequestObject($url,'POST',$body,'application/xml');
        $compris_callback_controller = ControllerFactory::getMessageControllerFromRequest($callback_request);
        $this->assertEquals('callback',$compris_callback_controller->getSubmittedAction());
        $id = $compris_callback_controller->getSubmittedIdentifier();
        $ts = getTimeStampForDateTimeAndTimeZone(3,30,00,date('m'),date('d'),date('Y'));
        $compris_callback_controller->setCurrentTime($ts);
        $response = $compris_callback_controller->callBack($id);
        $this->assertTrue($response);

        $dciha = new DeviceCallInHistoryAdapter(getM());
        $last_call_in_resource = $dciha->getCallInResourceByMerchantId($merchant_id);
        $this->assertEquals(0,$last_call_in_resource->auto_turned_off);

        $merchant_resource = $merchant_resource->getRefreshedResource();
        $this->assertEquals('Y',$merchant_resource->ordering_on,"ordering_on should have been set to 'Y'");

        // now check message
        $new_message_resource = Resource::find($mmha,$map_id);
        $this->assertEquals('C',$new_message_resource->locked,'It should have cancelled the message');
    }

    function test90SecondStatusIntervalAfterShutDown()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id,['long_hours'=>true]);
        $merchant_resource->merchant_external_id = "88888889";
        $merchant_resource->save();
        $merchant_id = $merchant_resource->merchant_id;
        $merchant_alphanumeric = $merchant_resource->alphanumeric_id;
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'O',"delivery_addr"=>"Compris","message_type"=>"X"));

        $url = "https://sumdumdomain/app2/messagemanager/getnextmessagebymerchantid/$merchant_alphanumeric/compris";
        $request = createRequestObject($url,'GET');
        $message_controller = ControllerFactory::generateFromUrl($url, getM(), null, $request, 5);
        $response_message_resource = $message_controller->pullNextMessageResourceByMerchant($merchant_alphanumeric);
        $body = $response_message_resource->message_text;
        myerror_log($body);
        $packet_id = "STATUS-".time();
        $expected_body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Request Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus/></Request></CommunicationCenter>';
        $this->assertEquals($expected_body,$body);

//        $dciha = new DeviceCallInHistoryAdapter(getM());
//        $last_call_in = $dciha->getCallInResourceByMerchantId($merchant_id);
//        $this->assertNull($last_call_in);

        $mmha = new MerchantMessageHistoryAdapter(getM());
        $last_status_message_resource = Resource::find($mmha,null,[3=>['merchant_id'=>$merchant_resource->merchant_id,'message_format'=>'OS']]);
        $ten_seconds_ago = time()-10;
        $os_sent_time = strtotime($last_status_message_resource->sent_dt_tm);
        $this->assertTrue($os_sent_time > $ten_seconds_ago);

        $url = "https://sumdumdomain/app2/messagemanager/$merchant_alphanumeric/compris/callback.txt";
        $body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Response Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus Turn="BAT-A.00013" PosTerminal="mws" ResultCode="50" ResultText="Pos is not runnning"/></Response></CommunicationCenter>';
        $callback_request = createRequestObject($url,'POST',$body,'application/xml');
        $compris_callback_controller = ControllerFactory::getMessageControllerFromRequest($callback_request);
        $this->assertEquals('callback',$compris_callback_controller->getSubmittedAction());
        $id = $compris_callback_controller->getSubmittedIdentifier();
        $time = time();
        $response = $compris_callback_controller->callBack($id);
        $this->assertFalse($response);

        $merchant_resource = $merchant_resource->getRefreshedResource();
        $this->assertEquals('N',$merchant_resource->ordering_on,"ordering_on should have been set to 'N'");
        $this->assertEquals('F',$merchant_resource->inactive_reason,"inactive_reason should have been set to 'F'");

        $dciha = new DeviceCallInHistoryAdapter(getM());
        $last_call_in_resource = $dciha->getCallInResourceByMerchantId($merchant_id);
        $this->assertEquals(1,$last_call_in_resource->auto_turned_off);

        //check time of message
        $mmha = new MerchantMessageHistoryAdapter(getM());
        $options[TONIC_FIND_BY_METADATA] = ['merchant_id'=>$merchant_id,'message_format'=>'Eoff','locked'=>'N'];
        $message_resource = Resource::find($mmha,null,$options);
        $map_id = $message_resource->map_id;
        $this->assertNotNull($message_resource,"There should be a message resource");
        $expected_send_on_time = $time+ComprisController::SECONDS_IN_THE_FUTURE_TO_SET_ALERT_EMAIL_WHEN_STORE_POS_GOES_DOWN;
        $actual_send_on_time = $message_resource->next_message_dt_tm;
        $expected_string = getMySqlFormattedDateTimeFromTimeStampAndTimeZone($expected_send_on_time);
        $actual_string = getMySqlFormattedDateTimeFromTimeStampAndTimeZone($actual_send_on_time);
        $this->assertEquals($expected_string,$actual_string,"it should be scheduled for 90 seconds in the future");

        //check to see that status was set to buffer - 60 seconds in the past
        $mmha = new MerchantMessageHistoryAdapter(getM());
        $last_status_message_resource = Resource::find($mmha,null,[3=>['merchant_id'=>$merchant_resource->merchant_id,'message_format'=>'OS']]);
        $status_buffer_in_seconds = getProperty('comprise_status_message_buffer_in_seconds');
        $expected_status_sent_time = time()-($status_buffer_in_seconds)+ComprisController::SECONDS_IN_THE_FUTURE_TO_SET_NEXT_STATUS_CHECK_IF_STORE_JUST_SHUT_DOWN;
        $expected_status_sent_string = date('Y-m-d H:i:s',$expected_status_sent_time);
        $os_sent_time = $last_status_message_resource->sent_dt_tm;
        $this->assertEquals($expected_status_sent_string,$os_sent_time);

        //now send a pos reeady and see if it cancels the message
        $dciha = new DeviceCallInHistoryAdapter(getM());
        $last_call_in_resource = $dciha->getCallInResourceByMerchantId($merchant_id);
        $this->assertEquals(1,$last_call_in_resource->auto_turned_off);

        $url = "https://sumdumdomain/app2/messagemanager/$merchant_alphanumeric/compris/callback.txt";
        $body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Response Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus Turn="BAT-A.00001" PosTerminal="mws" PosId="3" ResultCode="0" ResultText="Pos is ready"/></Response></CommunicationCenter>';
        $callback_request = createRequestObject($url,'POST',$body,'application/xml');
        $compris_callback_controller = ControllerFactory::getMessageControllerFromRequest($callback_request);
        $this->assertEquals('callback',$compris_callback_controller->getSubmittedAction());
        $id = $compris_callback_controller->getSubmittedIdentifier();
        $ts = getTimeStampForDateTimeAndTimeZone(3,30,00,date('m'),date('d'),date('Y'));
        $compris_callback_controller->setCurrentTime($ts);
        $response = $compris_callback_controller->callBack($id);
        $this->assertTrue($response);

        $dciha = new DeviceCallInHistoryAdapter(getM());
        $last_call_in_resource = $dciha->getCallInResourceByMerchantId($merchant_id);
        $this->assertEquals(0,$last_call_in_resource->auto_turned_off);

        $merchant_resource = $merchant_resource->getRefreshedResource();
        $this->assertEquals('Y',$merchant_resource->ordering_on,"ordering_on should have been set to 'Y'");

        // now check message
        $new_message_resource = Resource::find($mmha,$map_id);
        $this->assertEquals('C',$new_message_resource->locked,'It should have cancelled the message');

    }


    function testSetMerchantOffForTimeoutFromStatus()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id );
        $merchant_id = $merchant_resource->merchant_id;
        $merchant_alphanumeric = $merchant_resource->alphanumeric_id;
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'O',"delivery_addr"=>"Compris","message_type"=>"X"));

        $url = "https://sumdumdomain/app2/messagemanager/getnextmessagebymerchantid/$merchant_alphanumeric/compris";
        $request = createRequestObject($url,'GET');
        $message_controller = ControllerFactory::generateFromUrl($url, getM(), null, $request, 5);
        $response_message_resource = $message_controller->pullNextMessageResourceByMerchant($merchant_alphanumeric);
        $body = $response_message_resource->message_text;
        myerror_log($body);
        $packet_id = "STATUS-".time();
        $expected_body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Request Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus/></Request></CommunicationCenter>';
        $this->assertEquals($expected_body,$body);

//        $dciha = new DeviceCallInHistoryAdapter(getM());
//        $last_call_in = $dciha->getCallInResourceByMerchantId($merchant_id);
//        $this->assertNull($last_call_in);

        $url = "https://sumdumdomain/app2/messagemanager/$merchant_alphanumeric/compris/callback.txt";
        $body = '<?xml version="1.0" encoding="UTF-8"?><RRTService>  <Result>Timeout</Result>  <Turn></Turn>  <Id></Id></RRTService>';
        $callback_request = createRequestObject($url,'POST',$body,'application/xml');
        $compris_callback_controller = ControllerFactory::getMessageControllerFromRequest($callback_request);
        $this->assertEquals('callback',$compris_callback_controller->getSubmittedAction());
        $id = $compris_callback_controller->getSubmittedIdentifier();
        $response = $compris_callback_controller->callBack($id);
        $this->assertFalse($response);

        $merchant_resource = $merchant_resource->getRefreshedResource();
        $this->assertEquals('N',$merchant_resource->ordering_on,"ordering_on should have been set to 'N'");
        $this->assertEquals('F',$merchant_resource->inactive_reason,"inactive_reason should have been set to 'F'");

        $dciha = new DeviceCallInHistoryAdapter(getM());
        $last_call_in_resource = $dciha->getCallInResourceByMerchantId($merchant_id);
        $this->assertEquals(1,$last_call_in_resource->auto_turned_off);

    }

    function testStatusFirstCallIn()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id );
        $merchant_resource->merchant_external_id = '88888';
        $merchant_resource->save();

        $merchant_id = $merchant_resource->merchant_id;
        $merchant_alphanumeric = $merchant_resource->alphanumeric_id;
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'O',"delivery_addr"=>"Compris","message_type"=>"X"));

        $url = "https://sumdumdomain/app2/messagemanager/getnextmessagebymerchantid/$merchant_alphanumeric/compris";
        $request = createRequestObject($url,'GET');
        $message_controller = ControllerFactory::generateFromUrl($url, getM(), null, $request, 5);
        $response_message_resource = $message_controller->pullNextMessageResourceByMerchant($merchant_alphanumeric);
        $body = $response_message_resource->message_text;
        myerror_log($body);
        $packet_id = "STATUS-".time();
        $expected_body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Request Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus/></Request></CommunicationCenter>';
        $this->assertEquals($expected_body,$body);

//        $dciha = new DeviceCallInHistoryAdapter(getM());
//        $last_call_in = $dciha->getCallInResourceByMerchantId($merchant_id);
//        $this->assertNull($last_call_in);

        $url = "https://sumdumdomain/app2/messagemanager/$merchant_alphanumeric/compris/callback.txt";
        $body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Response Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus Turn="BAT-A.00013" PosTerminal="mws" ResultCode="50" ResultText="Pos is not runnning"/></Response></CommunicationCenter>';
        $callback_request = createRequestObject($url,'POST',$body,'application/xml');
        $compris_callback_controller = ControllerFactory::getMessageControllerFromRequest($callback_request);
        $this->assertEquals('callback',$compris_callback_controller->getSubmittedAction());
        $id = $compris_callback_controller->getSubmittedIdentifier();
        $response = $compris_callback_controller->callBack($id);
        $this->assertFalse($response);

        $merchant_resource = $merchant_resource->getRefreshedResource();
        $this->assertEquals('N',$merchant_resource->ordering_on,"ordering_on should have been set to 'N'");
        $this->assertEquals('F',$merchant_resource->inactive_reason,"inactive_reason should have been set to 'F'");

        $dciha = new DeviceCallInHistoryAdapter(getM());
        $last_call_in_resource = $dciha->getCallInResourceByMerchantId($merchant_id);
        $this->assertEquals(1,$last_call_in_resource->auto_turned_off);

        $url = "https://sumdumdomain/app2/messagemanager/$merchant_alphanumeric/compris/callback.txt";
        $body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Response Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus Turn="BAT-A.00001" PosTerminal="mws" PosId="3" ResultCode="0" ResultText="Pos is ready"/></Response></CommunicationCenter>';
        $callback_request = createRequestObject($url,'POST',$body,'application/xml');
        $compris_callback_controller = ControllerFactory::getMessageControllerFromRequest($callback_request);
        $this->assertEquals('callback',$compris_callback_controller->getSubmittedAction());
        $id = $compris_callback_controller->getSubmittedIdentifier();
        $response = $compris_callback_controller->callBack($id);
        $this->assertTrue($response);

        //check to see that call in history was now updated
        $dciha = new DeviceCallInHistoryAdapter(getM());
        $last_call_in = $dciha->getLastCallInAsIntegerByMerchantId($merchant_id);
        $this->assertEquals(time(),$last_call_in);

        //check to see that ordering was turned ON for this merchant
        $merchant_resource = $merchant_resource->getRefreshedResource();
        $this->assertEquals('Y',$merchant_resource->ordering_on,"ordering_on should have been set to 'Y'");

    }

    function testNoOrderDoStatus()
    {

        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id );
        $merchant_resource->ordering_on = 'N';
        $merchant_resource->inactive_reason = 'F';
        $merchant_resource->save();

        $merchant_id = $merchant_resource->merchant_id;
        $merchant_alphanumeric = $merchant_resource->alphanumeric_id;
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'O',"delivery_addr"=>"Compris","message_type"=>"X"));

        $dciha = new DeviceCallInHistoryAdapter(getM());
        $merchant_data['merchant_id'] = $merchant_id;
        $old_time = time() - 7*24*3600;
        $merchant_data['last_call_in'] = $old_time;
        $merchant_data['last_call_in_as_integer'] = $old_time;
        $merchant_data['device_base_format'] = 'O';
        $merchant_data['auto_turned_off'] = 1;
        $dcih_resource = Resource::createByData($dciha,$merchant_data);


        $url = "https://sumdumdomain/app2/messagemanager/getnextmessagebymerchantid/$merchant_alphanumeric/compris";
        $request = createRequestObject($url,'GET');
        $message_controller = ControllerFactory::generateFromUrl($url, getM(), null, $request, 5);
        $response_message_resource = $message_controller->pullNextMessageResourceByMerchant($merchant_alphanumeric);
        $body = $response_message_resource->message_text;
        myerror_log($body);
        $packet_id = "STATUS-".time();
        $expected_body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Request Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus/></Request></CommunicationCenter>';
        $this->assertEquals($expected_body,$body);

        //check to see that call in history was not updated
        $dciha = new DeviceCallInHistoryAdapter(getM());
        $last_call_in = $dciha->getLastCallInAsIntegerByMerchantId($merchant_id);
        $this->assertEquals($old_time,$last_call_in);

        $url = "https://sumdumdomain/app2/messagemanager/$merchant_alphanumeric/compris/callback.txt";
        $body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Response Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus Turn="BAT-A.00013" PosTerminal="mws" ResultCode="50" ResultText="Pos is not runnning"/></Response></CommunicationCenter>';
        $callback_request = createRequestObject($url,'POST',$body,'application/xml');
        $compris_callback_controller = ControllerFactory::getMessageControllerFromRequest($callback_request);
        $this->assertEquals('callback',$compris_callback_controller->getSubmittedAction());
        $id = $compris_callback_controller->getSubmittedIdentifier();
        $response = $compris_callback_controller->callBack($id);
        $this->assertFalse($response);

        //check to see that call in history was not updated
        $dciha = new DeviceCallInHistoryAdapter(getM());
        $last_call_in = $dciha->getLastCallInAsIntegerByMerchantId($merchant_id);
        $this->assertEquals($old_time,$last_call_in);
        $cihr = $dciha->getCallInResourceByMerchantId($merchant_id);
        $this->assertEquals(1,$cihr->auto_turned_off);

        //check to see that ordering was turned OFF for this merchant
        $merchant_resource = $merchant_resource->getRefreshedResource();
        $this->assertEquals('N',$merchant_resource->ordering_on,"ordering_on should have been set to 'N'");
        $this->assertEquals('F',$merchant_resource->inactive_reason,"inactive_reason should have been set to 'F'");


        $url = "https://sumdumdomain/app2/messagemanager/$merchant_alphanumeric/compris/callback.txt";
        $body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Response Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus Turn="BAT-A.00001" PosTerminal="mws" PosId="3" ResultCode="0" ResultText="Pos is ready"/></Response></CommunicationCenter>';
        $callback_request = createRequestObject($url,'POST',$body,'application/xml');
        $compris_callback_controller = ControllerFactory::getMessageControllerFromRequest($callback_request);
        $this->assertEquals('callback',$compris_callback_controller->getSubmittedAction());
        $id = $compris_callback_controller->getSubmittedIdentifier();
        $response = $compris_callback_controller->callBack($id);
        $this->assertTrue($response);

        //check to see that call in history was now updated
        $dciha = new DeviceCallInHistoryAdapter(getM());
        $last_call_in = $dciha->getLastCallInAsIntegerByMerchantId($merchant_id);
        $this->assertEquals(time(),$last_call_in);

        //check to see that ordering was turned ON for this merchant
        $merchant_resource = $merchant_resource->getRefreshedResource();
        $this->assertEquals('Y',$merchant_resource->ordering_on,"ordering_on should have been set to 'Y'");

        $cihr = $dciha->getCallInResourceByMerchantId($merchant_id);
        $this->assertEquals(0,$cihr->auto_turned_off);

    }

    function testDoNotSendMessageIfOrderinIsOffAtMerchant()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id );
        $merchant_id = $merchant_resource->merchant_id;
        $merchant_alphanumeric = $merchant_resource->alphanumeric_id;
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'O',"delivery_addr"=>"Compris","message_type"=>"X"));

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,time());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00,time());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $compris_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'O');
        $this->assertNotNull($compris_message_resource,'It should have the compris message resource');
        $compris_message_resource->next_message_dt_tm = (time() + 10);
        $compris_message_resource->save();

        $dciha = new DeviceCallInHistoryAdapter(getM());
        $merchant_data['merchant_id'] = $merchant_id;
        $old_time = time() - 600;
        $merchant_data['last_call_in'] = $old_time;
        $merchant_data['last_call_in_as_integer'] = $old_time;
        $merchant_data['device_base_format'] = 'O';
        $merchant_data['auto_turned_off'] = 1;
        $dcih_resource = Resource::createByData($dciha,$merchant_data);


        $url = "https://sumdumdomain/app2/messagemanager/getnextmessagebymerchantid/$merchant_alphanumeric/compris";
        $request = createRequestObject($url,'GET');
        $message_controller = ControllerFactory::generateFromUrl($url, getM(), null, $request, 5);
        $response_message_resource = $message_controller->pullNextMessageResourceByMerchant($merchant_alphanumeric);
        $body = $response_message_resource->message_text;
        myerror_log($body);
        $packet_id = "STATUS-".time();
        $expected_body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Request Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus/></Request></CommunicationCenter>';
        $this->assertEquals($expected_body,$body);
        $dcih_resource = $dcih_resource->getRefreshedResource();
        $this->assertEquals(1,$dcih_resource->auto_turned_off);

        $adapter = new MerchantMessageHistoryAdapter(getM());
        $last_status_message_resource = Resource::findOrCreateIfNotExistsByData($adapter,['merchant_id'=>$merchant_resource->merchant_id,'message_format'=>'OS']);
        $last_status_message_resource->sent_dt_tm = date("Y-m-d H:i:s",time()-61);
        $last_status_message_resource->locked = 'S';
        $last_status_message_resource->save();


        //now set the message time to in the past
        $compris_message_resource->next_message_dt_tm = time() - 10;
        $compris_message_resource->save();

        $url = "https://sumdumdomain/app2/messagemanager/getnextmessagebymerchantid/$merchant_alphanumeric/compris";
        $request = createRequestObject($url,'GET');
        $message_controller = ControllerFactory::generateFromUrl($url, getM(), null, $request, 5);
        $response_message_resource = $message_controller->pullNextMessageResourceByMerchant($merchant_alphanumeric);
        $body = $response_message_resource->message_text;
        myerror_log($body);

        // merchant is auto turned off, message will NOT be sent and we will get the status message again.
        $packet_id = "STATUS-".time();
        $expected_body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Request Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus/></Request></CommunicationCenter>';
        $this->assertEquals($expected_body,$body);
        $dcih_resource = $dcih_resource->getRefreshedResource();
        $this->assertEquals(1,$dcih_resource->auto_turned_off);

        $dcih_resource = $dcih_resource->getRefreshedResource();
        $this->assertEquals(1,$dcih_resource->auto_turned_off);

        $adapter = new MerchantMessageHistoryAdapter(getM());
        $last_status_message_resource = Resource::findOrCreateIfNotExistsByData($adapter,['merchant_id'=>$merchant_resource->merchant_id,'message_format'=>'OS']);
        $offset = getProperty('comprise_status_message_buffer_in_seconds') + 1;
        $last_status_message_resource->sent_dt_tm = date("Y-m-d H:i:s",time()-$offset);
        $last_status_message_resource->locked = 'S';
        $last_status_message_resource->save();

        //now give a successful call back
        $url = "https://sumdumdomain/app2/messagemanager/$merchant_alphanumeric/compris/callback.txt";
        $body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Response Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus Turn="BAT-A.00001" PosTerminal="mws" PosId="3" ResultCode="0" ResultText="Pos is ready"/></Response></CommunicationCenter>';
        $callback_request = createRequestObject($url,'POST',$body,'application/xml');
        $compris_callback_controller = ControllerFactory::getMessageControllerFromRequest($callback_request);
        $this->assertEquals('callback',$compris_callback_controller->getSubmittedAction());
        $id = $compris_callback_controller->getSubmittedIdentifier();
        $response = $compris_callback_controller->callBack($id);
        $this->assertTrue($response);

        $merchant_resource = $merchant_resource->getRefreshedResource();
        $this->assertEquals('Y',$merchant_resource->ordering_on,"ordering_on should have been set to 'Y'");

        $dciha = new DeviceCallInHistoryAdapter(getM());
        $last_call_in_resource = $dciha->getCallInResourceByMerchantId($merchant_id);
        $this->assertEquals(0,$last_call_in_resource->auto_turned_off);


        $url = "https://sumdumdomain/app2/messagemanager/getnextmessagebymerchantid/$merchant_alphanumeric/compris";
        $request = createRequestObject($url,'GET');
        $message_controller = ControllerFactory::generateFromUrl($url, getM(), null, $request, 5);
        $response_message_resource = $message_controller->pullNextMessageResourceByMerchant($merchant_alphanumeric);
        $body = $response_message_resource->message_text;
        myerror_log($body);
        $this->assertContains('COMPRIS TEMPLATE',$body);
        $this->assertTrue(isset($response_message_resource->message_id),'It should have the message resource');
    }

    function testAdditionalCondimentsOnExternalIdOfItem()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);

        $modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 5);
        $modifier_group_id = $modifier_group_resource->modifier_group_id;
        $item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', getM());
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id,1);

        $item_size_resources = CompleteMenu::getAllItemSizesAsResources($menu_id,0);
        $item_size_resources[0]->external_id = $item_size_resources[0]->external_id.'+123456+88888';
        $item_size_resources[0]->save();

        $merchant_resource = createNewTestMerchant($menu_id );
        $merchant_id = $merchant_resource->merchant_id;
        $merchant_alphanumeric = $merchant_resource->alphanumeric_id;
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'O',"delivery_addr"=>"Compris","message_type"=>"X"));

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','sumdumnote');
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $checkout_resource->note = "sum dum note";
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $compris_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'O');
        $compris_message_resource->next_message_dt_tm = time()-10;
        $compris_message_resource->save();

        $url = "https://sumdumdomain/app2/messagemanager/getnextmessagebymerchantid/$merchant_alphanumeric/compris";
        $request = createRequestObject($url,'GET');
        $message_controller = ControllerFactory::generateFromUrl($url, getM(), null, $request, 5);
        $response_message_resource = $message_controller->pullNextMessageResourceByMerchant($merchant_alphanumeric);
        $this->assertTrue(isset($response_message_resource->message_id),'It should have the message resource');
        if ($message_controller->message_resource->locked == 'Y') {
            $message_controller->message_resource->message_text = $response_message_resource->message_text;
            $message_controller->markMessageDelivered();
        }

        //validate that there is no discount node
        $body = $response_message_resource->message_text;
        myerror_log($body);
        $first_protien ='<Condiment SysId="123456" Quantity="1" Price="0.00" />';
        $this->assertContains($first_protien,$body);
        $second_protien ='<Condiment SysId="88888" Quantity="1" Price="0.00" />';
        $this->assertContains($second_protien,$body);




    }

    function testProcessXML()
    {
        $xml = '<?xml version="1.0" encoding="UTF-8"?><CommunicationCenter MessageCenter="123456" PacketId="2909-zpf70-bl03x-g39v9" Asynchronous="0"><Response Type="OrderHandler" Id="51484990" ExpirationTime="2055-08-15T12:05:00.000"><TenderOrder Turn="41325192" ResultCode="0" ResultText="Success" TransactionNumber="9093" Guid="{1D80488B-11D4-46BE-8E36-2B19B203A714}" Total="899" TotalTax="0" OrderNumber="9093" PosId="5" PosTerminal="mws"/></Response></CommunicationCenter>';
        $compris_controller = new ComprisController();
        $result = $compris_controller->parseCallBackXml($xml);
    }

    function testComprisMessageWithNoSkinDiscount()
    {
        $merchant_resource = createNewTestMerchant($this->ids['menu_id']);
        $merchant_id = $merchant_resource->merchant_id;
        $merchant_alphanumeric = $merchant_resource->alphanumeric_id;
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'O',"delivery_addr"=>"Compris","message_type"=>"X"));

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','sumdumnote');
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $checkout_resource->note = "sum dum note";
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $order_resource = OrderController::getResourceFromId($order_id,'Order');
        $this->assertEquals('O',$order_resource->status);

        $compris_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'O');
        $compris_message_resource->next_message_dt_tm = time()-10;
        $compris_message_resource->save();


        $url = "https://sumdumdomain/app2/messagemanager/getnextmessagebymerchantid/$merchant_alphanumeric/compris";
        $request = createRequestObject($url,'GET');
        $message_controller = ControllerFactory::generateFromUrl($url, getM(), null, $request, 5);
        $response_message_resource = $message_controller->pullNextMessageResourceByMerchant($merchant_alphanumeric);
        $this->assertTrue(isset($response_message_resource->message_id),'It should have the message resource');
        if ($message_controller->message_resource->locked == 'Y') {
            $message_controller->message_resource->message_text = $response_message_resource->message_text;
            $message_controller->markMessageDelivered();
        }

        //validate that there is no discount node
        $body = $response_message_resource->message_text;
        $this->assertNotContains('<OrderLevelDiscount SysId=',$body);

    }


    function testComprisMessagesFormatForMarketPlaceSkin()
    {

        $merchant_resource = createNewTestMerchant($this->ids['menu_id']);
        $merchant_id = $merchant_resource->merchant_id;
        $merchant_alphanumeric = $merchant_resource->alphanumeric_id;
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'O',"delivery_addr"=>"Compris","message_type"=>"X"));

        removeContext();
        setContext('com.splickit.ddljsskin');
        attachMerchantToSkin($merchant_id,888);

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $order_resource = OrderController::getResourceFromId($order_id,'Order');
        $this->assertEquals('O',$order_resource->status);

        $compris_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'O');
        $compris_message_resource->next_message_dt_tm = time()-10;
        $compris_message_resource->save();


        $url = "https://sumdumdomain/app2/messagemanager/getnextmessagebymerchantid/$merchant_alphanumeric/compris";
        $request = createRequestObject($url,'GET');
        $message_controller = ControllerFactory::generateFromUrl($url, getM(), null, $request, 5);
        $response_message_resource = $message_controller->pullNextMessageResourceByMerchant($merchant_alphanumeric);
        $this->assertTrue(isset($response_message_resource->message_id),'It should have the message resource');
        if ($message_controller->message_resource->locked == 'Y') {
            $message_controller->message_resource->message_text = $response_message_resource->message_text;
            $message_controller->markMessageDelivered();
        }

        $body = $response_message_resource->message_text;
        $this->assertContains('<OrderLevelDiscount SysId="77383"/>',$body);


        $compris_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'O');
        $this->assertNotNull($compris_message_resource);
        $this->assertEquals('S',$compris_message_resource->locked);
        $this->assertEquals('N',$compris_message_resource->viewed);


        // now send the successfull response and asert that it was 'viewed'
        $url = "https://sumdumdomain/app2/messagemanager/$merchant_alphanumeric/compris/callback.txt";
        $body = '<?xml version="1.0" encoding="UTF-8"?><CommunicationCenter MessageCenter="123456" PacketId="'.$order_resource->ucid.'" Asynchronous="0"><Response Type="OrderHandler" Id="'.$compris_message_resource->map_id.'" ExpirationTime="2055-08-15T12:05:00.000"><TenderOrder Turn="'.$order_resource->order_id.'" ResultCode="0" ResultText="Success" TransactionNumber="9093" Guid="{1D80488B-11D4-46BE-8E36-2B19B203A714}" Total="'.$order_resource->grand_total.'" TotalTax="0" OrderNumber="9093" PosId="5" PosTerminal="mws"/></Response></CommunicationCenter>';
        $callback_request = createRequestObject($url,'POST',$body,'application/xml');
        $compris_callback_controller = ControllerFactory::getMessageControllerFromRequest($callback_request);
        $this->assertEquals('callback',$compris_callback_controller->getSubmittedAction());
        $id = $compris_callback_controller->getSubmittedIdentifier();
        $resource = $compris_callback_controller->callBack($id);

        $compris_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'O');
        $this->assertNotNull($compris_message_resource);
        $this->assertEquals('S',$compris_message_resource->locked);
        $this->assertEquals('V',$compris_message_resource->viewed);

        // check that order is executed
        $order_resource = OrderController::getResourceFromId($order_id,'Order');
        $this->assertEquals('E',$order_resource->status);


    }

    function testComprisMessagesWithErrorResponse()
    {
        $merchant_resource = createNewTestMerchant($this->ids['menu_id']);
        $merchant_id = $merchant_resource->merchant_id;
        $merchant_alphanumeric = $merchant_resource->alphanumeric_id;
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'O',"delivery_addr"=>"Compris","message_type"=>"X"));

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $order_resource = OrderController::getResourceFromId($order_id,'Order');
        $this->assertEquals('O',$order_resource->status);

        $compris_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'O');
        $compris_message_resource->next_message_dt_tm = time()-10;
        $compris_message_resource->save();

        $url = "https://sumdumdomain/app2/messagemanager/getnextmessagebymerchantid/$merchant_alphanumeric/compris";
        $request = createRequestObject($url,'GET');
        $message_controller = ControllerFactory::generateFromUrl($url, getM(), null, $request, 5);
        $response_message_resource = $message_controller->pullNextMessageResourceByMerchant($merchant_alphanumeric);
        $this->assertTrue(isset($response_message_resource->message_id),'It should have the message resource');
        if ($message_controller->message_resource->locked == 'Y') {
            $message_controller->message_resource->message_text = $response_message_resource->message_text;
            $message_controller->markMessageDelivered();
        }

        $compris_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'O');
        $this->assertNotNull($compris_message_resource);
        $this->assertEquals('S',$compris_message_resource->locked);
        $this->assertEquals('N',$compris_message_resource->viewed);

        $order_resource = OrderController::getResourceFromId($order_id,'Order');
        $this->assertEquals('O',$order_resource->status);


        // now send the error response and asert that it was failed
        $url = "https://sumdumdomain/app2/messagemanager/$merchant_alphanumeric/compris/callback.txt";
        $body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$order_resource->ucid.'" Asynchronous="0">	<Response Type="OrderHandler" Id="'.$compris_message_resource->map_id.'" ExpirationTime="2055-08-15T12:05:00.000" ClientMessagesAvailable="0"><OrderChange Turn="'.$order_id.'" ResultCode="1014" ResultText="POS Error: 1067(ERROR_NO_TOTAL_FUTURE_ORDER) while attempting to Process Tender" PosId="3" PosTerminal="MWS"/></Response></CommunicationCenter>';
        $callback_request = createRequestObject($url,'POST',$body,'application/xml');
        $compris_callback_controller = ControllerFactory::getMessageControllerFromRequest($callback_request);
        $this->assertEquals('callback',$compris_callback_controller->getSubmittedAction());
        $id = $compris_callback_controller->getSubmittedIdentifier();
        $resource = $compris_callback_controller->callBack($id);

        $compris_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'O');
        $this->assertNotNull($compris_message_resource);
        $this->assertEquals('F',$compris_message_resource->locked);
    }

    function testTimeout()
    {
        $merchant_resource = createNewTestMerchant($this->ids['menu_id']);
        $merchant_id = $merchant_resource->merchant_id;
        $merchant_alphanumeric = $merchant_resource->alphanumeric_id;
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'O',"delivery_addr"=>"Compris","message_type"=>"X"));

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $order_resource = OrderController::getResourceFromId($order_id,'Order');
        $this->assertEquals('O',$order_resource->status);

        $compris_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'O');
        $compris_message_resource->next_message_dt_tm = time()-10;
        $compris_message_resource->save();

        $url = "https://sumdumdomain/app2/messagemanager/getnextmessagebymerchantid/$merchant_alphanumeric/compris";
        $request = createRequestObject($url,'GET');
        $message_controller = ControllerFactory::generateFromUrl($url, getM(), null, $request, 5);
        $response_message_resource = $message_controller->pullNextMessageResourceByMerchant($merchant_alphanumeric);
        $this->assertTrue(isset($response_message_resource->message_id),'It should have the message resource');
        if ($message_controller->message_resource->locked == 'Y') {
            $message_controller->message_resource->message_text = $response_message_resource->message_text;
            $message_controller->markMessageDelivered();
        }

        $compris_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'O');
        $this->assertNotNull($compris_message_resource);
        $this->assertEquals('S',$compris_message_resource->locked);
        $this->assertEquals('N',$compris_message_resource->viewed);

        $order_resource = OrderController::getResourceFromId($order_id,'Order');
        $this->assertEquals('O',$order_resource->status);


        // now send the timeout response and assert that it was managed correctly
        $url = "https://sumdumdomain/app2/messagemanager/$merchant_alphanumeric/compris/callback.txt";
        $body = '<?xml version="1.0" encoding="UTF-8"?><RRTService><Result>timeout</Result><Turn>'.$order_id.'</Turn><Id>'.$compris_message_resource->map_id.'</Id></RRTService>';
        $callback_request = createRequestObject($url,'POST',$body,'application/xml');
        $compris_callback_controller = ControllerFactory::getMessageControllerFromRequest($callback_request);
        $this->assertEquals('callback',$compris_callback_controller->getSubmittedAction());
        $id = $compris_callback_controller->getSubmittedIdentifier();
        $resource = $compris_callback_controller->callBack($id);

        // check that the merchant was turned off
        $merchant_resource = $merchant_resource->getRefreshedResource();
        $this->assertEquals('N',$merchant_resource->ordering_on,"ordering_on should have been set to 'N'");
        $this->assertEquals('F',$merchant_resource->inactive_reason,"inactive_reason should have been set to 'F'");

        $dciha = new DeviceCallInHistoryAdapter(getM());
        $last_call_in_resource = $dciha->getCallInResourceByMerchantId($merchant_id);
        $this->assertEquals(1,$last_call_in_resource->auto_turned_off);
    }

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        ini_set('max_execution_time',300);
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $sql = 'DELETE FROM device_call_in_history';
        $dciha = new DeviceCallInHistoryAdapter(getM());
        $dciha->_query($sql);
        $mysqli->begin_transaction(); ;
        //setProperty("comprise_message_buffer_in_seconds",10,true);
        setProperty("comprise_status_message_buffer_in_seconds",300);

        $_SERVER['request_time1'] = microtime(true);
        $_SERVER['log_level'] = 5;

        $skin_resource = getOrCreateSkinAndBrandIfNecessaryWithLoyalty('comprisskin','comprisbrand',237,517);
        setContext("com.splickit.comprisskin");
        $menu_id = createTestMenuWithNnumberOfItems(1);

        $modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 5);
        $modifier_group_id = $modifier_group_resource->modifier_group_id;
        $item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', getM());
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id,1);

        $item_size_resources = CompleteMenu::getAllItemSizesAsResources($menu_id,0);
        $item_size_resources[0]->external_id = $item_size_resources[0]->external_id.'-333';
        $item_size_resources[0]->save();

        $ids['menu_id'] = $menu_id;

        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $parameters = $tpppa->getParametersFromProviderId(ComprisController::COMPRIS_THIRD_PARTY_PROVIDER_ID);
        $tpppsma = new ThirdPartyProviderParameterSkinMapsAdapter(getM());
        foreach ($parameters as $parameter_row) {
            $data = [];
            $data['third_party_provider_parameter_id'] = $parameter_row['third_party_provider_parameter_id'];
            if ($parameter_row['name'] == 'tender_id') {
                $data['value'] = 123456;
                $data['skin_id'] = getSkinIdForContext();
                Resource::createByData($tpppsma,$data);
            } else if ($parameter_row['name'] == 'order_level_discount_id') {
                $data['value'] = 77383;
                $data['skin_id'] = getSkinIdForContext();
                //$tpppsma = new ThirdPartyProviderParameterSkinMapsAdapter(getM());
                //$ids['order_level_discount'] = Resource::createByData($tpppsma, $data);
                $ids['order_level_discount_data'] = $data;
                Resource::createByData($tpppsma,$data);
            } else {
                $data['value'] = generateCode();
                $data['skin_id'] = getSkinIdForContext();
                Resource::createByData($tpppsma,$data);
            }
        }

        $skin_resource = getOrCreateSkinAndBrandIfNecessaryWithLoyalty('ddljsskin','ddljsbrand',888,517);

        foreach ($parameters as $parameter_row) {
            $data = [];
            $data['third_party_provider_parameter_id'] = $parameter_row['third_party_provider_parameter_id'];
            if ($parameter_row['name'] == 'tender_id') {
                $data['value'] = 123456;
                $data['skin_id'] = 888;
                Resource::createByData($tpppsma,$data);
            } else if ($parameter_row['name'] == 'order_level_discount_id') {
                $data['value'] = 77383;
                $data['skin_id'] = 888;
                //$tpppsma = new ThirdPartyProviderParameterSkinMapsAdapter(getM());
                //$ids['order_level_discount'] = Resource::createByData($tpppsma, $data);
                $ids['order_level_discount_data'] = $data;
                Resource::createByData($tpppsma,$data);
            } else {
                $data['value'] = generateCode();
                $data['skin_id'] = 888;
                Resource::createByData($tpppsma,$data);
            }
        }

        $_SERVER['unit_test_ids'] = $ids;

    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    ComprisControllerTest::main();
}

?>