<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class CurlControllerTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $mmha;
    var $ids;

    function setUp()
    {
        $_SERVER['STAMP'] = __CLASS__.'-'.setNewStamp();
        $this->ids = $_SERVER['unit_test_ids'];
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        setContext('com.splickit.ljs');
    }

    function tearDown()
    {
        //delete your instance
        unset($this->ids);
    }

    /*    JSON TEMPLATE FOR TRANSACTION

{
   "retailer_id":"9BB0D964-E257-4290-967C-9F2DA5F4E07E",
   "store_id":"{Orders.merchant_id}",
   "pos_employee_id":"Request from Compris",
   "sm_employee_id":null,
   "table_id":null,
   "guest_count":null,
   "is_closed":true,
   "is_voided":false,
   "transaction_id":"{Orders.order_id}",
   "subtotal":"{Orders.grand_total} - Orders.total_tax_amt}",
   "tax_total":"{Orders.total_tax_amt}",
   "open_time":"{Orders.created}",  #A UTC time in RFC3339 ex: 2011-08-30T09:30:16.768-04:00
   "modified_time":"{Orders.order_dt_tm}",  #A UTC time in RFC3339 ex: 2011-08-30T09:30:16.768-04:00
   "guest_receipt_code":null,
   "channel":"{Skin.skin_name}",
   "items":{
      "line_id":"{Order_Detail.order_detail_id}",
      "item_id":"{Order_Detail.item_size_id}",
      "quantity":"{Order_Detail.quantity}",
      "unit_price":"{Order_Detail.price}",
      "subtotal":"{Order_Detail.item_total}",
      "tax_included":0,
      "modifies_line_id ":"0"
   },
   "payments":{
      "payment_id":"{Balance_Change.id}",
      "amount":"{Balance_Change.charge_amt}",
      "type":"
	CASE WHEN {Balance_Change.process} = '\\''CCpayment'\\'' THEN '\\''CreditCard'\\''
	CASE WHEN {Balance_Change.process} = '\\''Cash'\\'' THEN '\\''Instore Payment'\\'',
	CASE WHEN {Balance_Change.process} = '\\''LoyaltyBalancePayment'\\'' THEN '\\''Loyalty'\\''
	CASE WHEN {Balance_Change.process} = '\\''DoorDash'\\'' THEN '\\''DoorDash'\\''
	CASE WHEN {Balance_Change.process} = '\\''UberEats'\\'' THEN '\\''UberEats'\\''
	",
      "payment_time":"{Balance_Change.created}",
      "_comment":"JavaScript JSON date time format ex: (yyyy-MM-ddTHH:mm:ss.fffZ)",
      "user_id":"{Orders.user_id}",
      "user_id_type":"External_ID"
   },
   "discounts":{
              "reference_id": "{Orders.promo_id}",
              "reference_id_type": "{Promo.promo_type}",
              "pos_discount_id": "{Promo_Brand_External_Id_Map.external_id}",
              "status":"offline",
              "display_name": "{Promo.promo_description}",
              "image_url": null,
              "discounted_line_ids": null,
              "discount_source":"POS",
              "amount":"{Orders.promo_amt}",
              "stack_order":0,
              "applied_time":"{Orders.order_dt_tm}",  #A UTC time in RFC3339 ex: 2011-08-30T09:30:16.768-04:00
              "user_id":"{Orders.user_id}"
        }
   ]
}
     */


    function testNewCurlController()
    {
        setProperty('session_m_transaction_url',"http://sub.sumdomain.com",true);
        $data['message_data'] = ['message_format'=>'C','delivery_addr'=>'SessionM','info'=>'service=SessionM;type=transaction'];
        $merchant_resource = createNewTestMerchant($this->ids['menu_id'],$data);
        $merchant_id = $merchant_resource->merchant_id;
        attachMerchantToSkin($merchant_id, $this->ids['skin_id']);

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note',1);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource);
        $order_id = $order_resource->order_id;
        $this->assertNull($order_resource->error);

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'C');
        $curl_controller = ControllerFactory::generateFromMessageResource($message_resource);
        $this->assertTrue(is_a($curl_controller,'CurlController'),"It should be a curl controller");

        $response = $curl_controller->sendThisMessage($message_resource);
//        $ready_to_send_message_resource = $curl_controller->prepMessageForSending($message_resource);
//        $body_of_message = $ready_to_send_message_resource->message_text;
//        $expected_json = ''; // need to figure out what the expected json is
//        $this->assertEquals($expected_json,$body_of_message);
        $sent_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'C');
        $this->assertContains('"store_id":"c39a3b91-fd49-463c-9fae-18812158e6a6","client_id":"2460fba2-6e40-441a-ae5f-c7ca15f0a91d"',$sent_message_resource->message_text);
    }

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        set_time_limit(30);
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['default_tz'] = $tz;
        date_default_timezone_set("America/Denver");
        $sql = 'TRUNCATE TABLE `db_message_log`';
        $mmha = new MerchantMessageHistoryAdapter(getM());
        $mmha->_query($sql);
        ini_set('max_execution_time',300);
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction(); ;

        $skin_resource = getOrCreateSkinAndBrandIfNecessary('ljs','ljs',237,517);
        setContext('com.splickit.ljs');
        $ids['skin_id'] = $skin_resource->skin_id;

        $menu_id = createTestMenuWithNnumberOfItems(1);
        $ids['menu_id'] = $menu_id;

        $modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 3);
        $modifier_group_id = $modifier_group_resource->modifier_group_id;
        $item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 0);

        $merchant_resource = createNewTestMerchant($menu_id);
        attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
        $ids['merchant_id'] = $merchant_resource->merchant_id;

        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $parameters = $tpppa->getParametersFromProviderId(3004);
        foreach ($parameters as $parameter_row) {
            $data = [];
            if ($parameter_row['name'] == 'transaction_url') {
                $data['value'] = 'https://domains-connecteast1.stg-sessionm.com';
            } else if ($parameter_row['name'] == 'transaction_app_key') {
                $data['value'] = 'SessionM_ljsilvers';
            } else if ($parameter_row['name'] == 'transaction_secret') {
                $data['value'] = 'Wjwa8CJB0whTnfFbFD8S';
            } else if ($parameter_row['name'] == 'customer_url') {
                $data['value'] = 'https://api-ljsilvers.stg-sessionm.com';
            } else if ($parameter_row['name'] == 'customer_app_key') {
                $data['value'] = 'ed685deed05be9e6a0a81ab1725d640a9e6db027';
            } else if ($parameter_row['name'] == 'customer_secret') {
                $data['value'] = '97411da077908574e55e5af8e897538d5d89ff5d';
            } else if ($parameter_row['name'] == 'retailer_id') {
                $data['value'] = '2460fba2-6e40-441a-ae5f-c7ca15f0a91d';
            } else if ($parameter_row['name'] == 'store_guid') {
                $data['value'] = 'c39a3b91-fd49-463c-9fae-18812158e6a6';
            }
            $data['third_party_provider_parameter_id'] = $parameter_row['third_party_provider_parameter_id'];
            $data['brand_id'] = 517;
            $tpppbma = new ThirdPartyProviderParameterBrandMapsAdapter(getM());
            Resource::createByData($tpppbma,$data);
        }

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
    }

    /* main method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    CurlControllerTest::main();
}

?>
