<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class DeliveryNewFrameworkTest extends PHPUnit_Framework_TestCase
{

    var $stamp;
    var $ids;
    var $door_dash_delivery_price_resource;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $code = generateCode(7);
        $_SERVER['STAMP'] = __CLASS__ . '-' . $code;
        $_SERVER['RAW_STAMP'] = $code;
        $this->ids = $_SERVER['unit_test_ids'];
        $_SERVER['force_doordash_estimate_fail'] = false;
        setContext('com.splickit.worldhq');
        //setContext('com.splickit.fuddruckers');

    }

    function tearDown()
    {
        //delete your instance
        unset($this->ids);
        unset($_SERVER['max_lead']);
    }

    function testDoNotGetEstimateFromDoorDashOnIsInDeliveryArea()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchantDelivery($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $data = array("merchant_id" => $merchant_resource->merchant_id);
        $mdpd = new MerchantDeliveryPriceDistanceAdapter(getM());
        $mdpd_resource = $mdpd->getExactResourceFromData($data);
        $mdpd_resource->delivery_price_type = 'driving';
        $mdpd_resource->distance_up_to = 1;
        $mdpd_resource->price = 5.00;
        $mdpd_resource->save();
        $mdpd_resource = $mdpd_resource->getRefreshedResource();

        //add second
        $mdpd_resource->_exists = false;
        unset($mdpd_resource->map_id);
        $mdpd_resource->delivery_price_type = 'doordash';
        $mdpd_resource->notes = "external_business_name=sumdumbiz";
        $mdpd_resource->price = -1.00;
        $mdpd_resource->priority = $mdpd_resource->priority - 10;
        $mdpd_resource->distance_up_to = 100;
        $mdpd_resource->save();

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $user_id = $user['user_id'];

        $udla = new UserDeliveryLocationAdapter(getM());
        $data = array("user_id" => $user_id, "address1" => "1633 18th street", "city" => "Boulder", "state" => "CO", "zip" => "80302", "phone_no" => "1234567890", "lat" => 40.000000, "lng" => -105.271900);
        $udla_resource = Resource::createByData($udla, $data);
        $udl_id = $udla_resource->user_addr_id;


        $request = createRequestObject("/apiv2/merchants/$merchant_id/isindeliveryarea/$udl_id?log_level=5",'GET');
        $merchant_controller = new MerchantController(getM(), $user, $request, 5);
        $resource = $merchant_controller->processV2Request();
        $this->assertTrue($resource->is_in_delivery_range," the is in delivery range should be true");
        $this->assertEquals(8.00,$resource->price, "It shoudl have the fixed doordash price because we dont call out");


    }

    function testDeliveryPrice()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchantDelivery($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $data = array("merchant_id" => $merchant_resource->merchant_id);
        $mdpd = new MerchantDeliveryPriceDistanceAdapter(getM());
        $mdpd_resource = $mdpd->getExactResourceFromData($data);
        $mdpd_resource->delivery_price_type = 'driving';
        $mdpd_resource->name = 'SumDumName';
        $mdpd_resource->distance_up_to = 5;
        $mdpd_resource->zip_codes = '10029,80223';
        $mdpd_resource->price = 5.00;
        $mdpd_resource->max_days_out = 3;
        $mdpd_resource->delivery_increment = 0;
        $mdpd_resource->save();
        $mdpd_resource = $mdpd_resource->getRefreshedResource();

        //add second
        $mdpd_resource->_exists = false;
        unset($mdpd_resource->map_id);
        $mdpd_resource->price = 10.00;
        $mdpd_resource->priority = $mdpd_resource->priority - 10;
        $mdpd_resource->distance_up_to = 100;
        $mdpd_resource->save();

        $options[TONIC_FIND_BY_METADATA] = ["merchant_id"=>$merchant_id];
        $resources = Resource::findAll($mdpd,null,$options);

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $user_id = $user['user_id'];

        $udla = new UserDeliveryLocationAdapter(getM());
        $data = array("user_id" => $user_id, "address1" => "1633 18th street", "city" => "Boulder", "state" => "CO", "zip" => "80302", "phone_no" => "1234567890", "lat" => 40.016101, "lng" => -105.271900);
        $udla_resource = Resource::createByData($udla, $data);
        $udl_id_near = $udla_resource->user_addr_id;

        $request = createRequestObject("/apiv2/merchants/$merchant_id/isindeliveryarea/$udl_id_near?log_level=5",'GET');
        $merchant_controller = new MerchantController(getM(), $user, $request, 5);
        $resource = $merchant_controller->processV2Request();
        $this->assertTrue($resource->is_in_delivery_range," the is in delivery range should be true");
        $this->assertEquals(5.00,$resource->price, "It should have a price equal to the amount set no the record");



        $udla = new UserDeliveryLocationAdapter(getM());
        $data = array("user_id"=>$user_id,"address1"=>"450 main street","city"=>"Lyons","state"=>"CO","zip"=>"80540","phone_no"=>"1234567890","lat"=>40.224877,"lng"=> -105.271122);
        $udla_resource = Resource::createByData($udla, $data);
        $udl_id_far = $udla_resource->user_addr_id;

        $request = createRequestObject("/apiv2/merchants/$merchant_id/isindeliveryarea/$udl_id_far?log_level=5",'GET');
        $merchant_controller = new MerchantController(getM(), $user, $request, 5);
        $resource = $merchant_controller->processV2Request();
        $this->assertTrue($resource->is_in_delivery_range," the is in delivery range should be true");
        $this->assertEquals(10.00,$resource->price, "It should have a price equal to the amount set no the record");

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $cart_data['user_addr_id'] = $udl_id_far;
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
    }

    function testMaximum()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchantDelivery($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $data = array("merchant_id" => $merchant_resource->merchant_id);
        $mdpd = new MerchantDeliveryPriceDistanceAdapter(getM());
        $mdpd_resource = $mdpd->getExactResourceFromData($data);
        $mdpd_resource->distance_up_to = 1;
        $mdpd_resource->price = 2.00;
        $mdpd_resource->save();

        $mdpd_resource->_exists = false;
        unset($mdpd_resource->map_id);
        unset($mdpd_resource->distance_up_to);
        $mdpd_resource->price = -1.00;
        $mdpd_resource->notes = "external_business_name=sumdumbiz";
        $mdpd_resource->maximum_order_amount = 20.00;
        $mdpd_resource->priority = $mdpd_resource->priority - 10;
        $mdpd_resource->delivery_price_type = 'doordash';
        $mdpd_resource->save();
        $mdpd_resource = $mdpd_resource->getRefreshedResource();
        $door_dash_mdpd_id = $mdpd_resource->map_id;

        $mdpd_resource->_exists = false;
        unset($mdpd_resource->map_id);
        unset($mdpd_resource->maximum_order_amount);
        unset($mdpd_resource->notes);
        $mdpd_resource->price = 10.00;
        $mdpd_resource->priority = $mdpd_resource->priority - 10;
        $mdpd_resource->delivery_price_type = 'driving';
        $mdpd_resource->distance_up_to = 20;
        $mdpd_resource->save();
        $mdpd_resource = $mdpd_resource->getRefreshedResource();
        $native_large_order_mdpd_id = $mdpd_resource->map_id;

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $user_id = $user['user_id'];

        $udla = new UserDeliveryLocationAdapter(getM());
        $data = array("user_id" => $user_id, "address1" => "1633 18th street", "city" => "Boulder", "state" => "CO", "zip" => "80302", "phone_no" => "1234567890", "lat" => 40.000000, "lng" => -105.271900);
        $udla_resource = Resource::createByData($udla, $data);
        $udl_id = $udla_resource->user_addr_id;

        $request = createRequestObject("/apiv2/merchants/$merchant_id/isindeliveryarea/$udl_id?log_level=5",'GET');
        $merchant_controller = new MerchantController(getM(), $user, $request, 5);
        $resource = $merchant_controller->processV2Request();
        $this->assertTrue($resource->is_in_delivery_range," the is in delivery range should be true");
        $this->assertEquals(8.00,$resource->price, "It shoudl have the estimate price record");

        // now create an order
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','the note',2);
        $cart_data['user_addr_id'] = $udl_id;
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $this->assertEquals(5.60,$checkout_resource->delivery_amt);
        $this->assertEquals($door_dash_mdpd_id,$checkout_resource->merchant_delivery_price_distance_id);

        // now add a LOT more items
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','the note',20);
        $cart_data['ucid'] = $checkout_resource->ucid;
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $this->assertTrue($checkout_resource->order_amt > 20);
        $this->assertEquals(10,$checkout_resource->delivery_amt);
        $this->assertEquals($native_large_order_mdpd_id,$checkout_resource->merchant_delivery_price_distance_id);
        return ['merchant_id'=>$merchant_id,'user_id'=>$user_id,'udl_id'=>$udl_id,'door_dash_mdpd_id'=>$door_dash_mdpd_id];
    }

    /**
     * @depends testMaximum
     */
    function testDoNotUseCachedUserDeliveryLocationMerchantDeliveryPrice($data)
    {
        $user_id = $data['user_id'];
        $merchant_id = $data['merchant_id'];
        $user = logTestUserIn($user_id);
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','the note',2);
        $cart_data['user_addr_id'] = $data['udl_id'];
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $this->assertEquals(5.60,$checkout_resource->delivery_amt);
        $this->assertEquals($data['door_dash_mdpd_id'],$checkout_resource->merchant_delivery_price_distance_id);
        return $data;
    }


    /**
     * @depends testMaximum
     */
    function testDoorDashFailDoNotRollOver($data)
    {
        $user_id = $data['user_id'];
        $user = logTestUserIn($user_id);

        $merchant_id = $data['merchant_id'];
        $udl_id = $data['udl_id'];
        $door_dash_mdpd_id = $data['door_dash_mdpd_id'];

        $_SERVER['force_doordash_estimate_fail'] = true;
        $request = createRequestObject("/apiv2/merchants/$merchant_id/isindeliveryarea/$udl_id?log_level=5",'GET');
        $merchant_controller = new MerchantController(getM(), $user, $request, 5);
        $resource = $merchant_controller->processV2Request();
        $this->assertFalse($resource->is_in_delivery_range," the is in delivery range should be false");
        $_SERVER['force_doordash_estimate_fail'] = false;
        $request = createRequestObject("/apiv2/merchants/$merchant_id/isindeliveryarea/$udl_id?log_level=5",'GET');
        $merchant_controller = new MerchantController(getM(), $user, $request, 5);
        $resource = $merchant_controller->processV2Request();
        $this->assertTrue($resource->is_in_delivery_range," the is in delivery range should be true");
        $this->assertEquals(8.00,$resource->price, "It shoudl have the estimate price record");

        // now create an order
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','the note',2);
        $cart_data['user_addr_id'] = $udl_id;
        $_SERVER['force_doordash_estimate_fail'] = true;
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNotNull($checkout_resource->error);
        $this->assertEquals(DeliveryController::DOORDASH_CANNOT_DELIVER_MESSAGE,$checkout_resource->error);

    }

    static function setUpBeforeClass()
    {
        ini_set('max_execution_time', 0);
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction(); ;


        $skin_resource = getOrCreateSkinAndBrandIfNecessary('newdeliveryskin','newdeliverybrand',null,null);
        $ids['skin_id'] = $skin_resource->skin_id;

        $skin_resource = getOrCreateSkinAndBrandIfNecessary('fuddruckers','fuddruckers',114,372);

        //map it to a menu
        $menu_id = createTestMenuWithNnumberOfItems(3);
        $ids['menu_id'] = $menu_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main()
    {
        $suite = new PHPUnit_Framework_TestSuite(__CLASS__);
        PHPUnit_TextUI_TestRunner::run($suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    DeliveryNewFrameworkTest::main();
}

?>