<?php
 
$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class DoorDashTest extends PHPUnit_Framework_TestCase
{

    var $stamp;
    var $ids;
    var $door_dash_delivery_price_resource;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $code = generateCode(7);
        $_SERVER['STAMP'] = __CLASS__ . '-' . $code;
        $_SERVER['RAW_STAMP'] = $code;
        $this->ids = $_SERVER['unit_test_ids'];
        $_SERVER['force_doordash_estimate_fail'] = false;
        setContext('com.splickit.worldhq');
        //setContext('com.splickit.fuddruckers');
        $_SERVER['force_doordash_estimate_fail'] = false;

    }

    function tearDown()
    {
        //delete your instance
        unset($this->ids);
        unset($_SERVER['max_lead']);
    }

    function testMenuTypeTimeRangeWithDoorDash()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $menu_type_adapter = new MenuTypeAdapter(getM());
        $menu_type_resource = $menu_type_adapter->getExactResourceFromData(array("menu_id" => $menu_id));
        $menu_type_resource->end_time = '03:15:00';
        $menu_type_resource->save();

        $merchant_resource = createNewTestMerchantDelivery($menu_id, ['long_hours' => true]);
        $merchant_id = $merchant_resource->merchant_id;
        MerchantMessageMapAdapter::createMerchantMessageMap($merchant_id, 'FUC', '1234567890', 'X');
        $data = array("merchant_id" => $merchant_resource->merchant_id);

        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $dd_third_party_parameters = $tpppa->getRecords(['third_party_provider_id'=>4001]);
        foreach ($dd_third_party_parameters as $dd_parameter) {
            if ($dd_parameter['name'] == 'external_business_name') {
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()),["merchant_id"=>$merchant_id,"third_party_provider_parameter_id"=>$dd_parameter['id'],"value"=>'sumdumbiz']);
            } else {
                $id = $dd_parameter['id'];
                $name = $dd_parameter['name'];
                $value = $name.'-1000'.$id;
                $ids[$name] = $value;
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()),["brand_id"=>getBrandIdFromCurrentContext(),"third_party_provider_parameter_id"=>$dd_parameter['id'],"value"=>$value]);
            }
        }

        // set merchant delivery info
        $mdia = new MerchantDeliveryPriceDistanceAdapter(getM());
        $mdia_resource = $mdia->getExactResourceFromData($data);
        $mdia_resource->minimum_order_amount = 0.01;
        $mdia_resource->delivery_cost = 1.00;
        $mdia_resource->delivery_increment = 15;
        $mdia_resource->max_days_out = 3;
        $mdia_resource->save();

        // now edit the distance price records.
        $mdpd = new MerchantDeliveryPriceDistanceAdapter(getM());
        $mdpd_resource = $mdpd->getExactResourceFromData($data);
        $mdpd_resource->delivery_price_type = 'Doordash';
        $mdpd_resource->name = 'Doordash';
        $mdpd_resource->distance_up_to = 'nullit';
        $mdpd_resource->price = -1.00;
        //$mdpd_resource->notes = "external_business_name=sumdumbiz";
        $mdpd_resource->save();

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $user_id = $user['user_id'];

        $udla = new UserDeliveryLocationAdapter(getM());
        $data = array("user_id" => $user_id, "address1" => "450 main street", "city" => "Lyons", "state" => "CO", "zip" => "80540", "phone_no" => "8888888888", "lat" => 40.224877, "lng" => -105.271122);
        $udla_resource = Resource::createByData($udla, $data);
        $udl_id = $udla_resource->user_addr_id;

        $request = createRequestObject("/apiv2/merchants/$merchant_id/isindeliveryarea/$udl_id?log_level=5", 'GET');
        $merchant_controller = new MerchantController(getM(), $user, $request, 5);
        $resource = $merchant_controller->processV2Request();
        $udl_id = $udla_resource->user_addr_id;

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id, 'pickup', 'the note', 2);
        $cart_data['user_addr_id'] = $udl_id;
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data, getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNotNull($checkout_resource->error, "Should have gotten an error because the item is not availble anymore");
        $this->assertEquals("Sorry the Test Item 1 is not available after 3:15 am. Please remove it from your cart before placing your order.", $checkout_resource->error);


        // make test for not available untill
        $menu_type_resource->start_time = '14:00:00';
        $menu_type_resource->end_time = '23:59:00';
        $menu_type_resource->save();

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data, getTomorrowTwelveNoonTimeStampDenver());
        $first_time_available = $checkout_resource->lead_times_array[0];
        $expected_first_time_available = getTomorrowTwelveNoonTimeStampDenver() + 7200 + (50 * 60); // this shoudl be 2:00pm
        $this->assertNull($checkout_resource->error);
        $this->assertEquals(date("Y-m-d H:i", $expected_first_time_available), date("Y-m-d H:i", $first_time_available), 'First time available should have been 2:50pm');
    }

    function testDoordashfailoncartcreation()
    {
        $merchant_resource = createNewTestMerchantDelivery($this->ids['menu_id'], ['long_hours' => true]);
        $merchant_id = $merchant_resource->merchant_id;
        MerchantMessageMapAdapter::createMerchantMessageMap($merchant_id, 'FUC', '1234567890', 'X');
        $data = array("merchant_id" => $merchant_resource->merchant_id);

        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $dd_third_party_parameters = $tpppa->getRecords(['third_party_provider_id'=>4001]);
        foreach ($dd_third_party_parameters as $dd_parameter) {
            if ($dd_parameter['name'] == 'external_business_name') {
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()),["merchant_id"=>$merchant_id,"third_party_provider_parameter_id"=>$dd_parameter['id'],"value"=>'sumdumbiz']);
            } else {
                $id = $dd_parameter['id'];
                $name = $dd_parameter['name'];
                $value = $name.'-1000'.$id;
                $ids[$name] = $value;
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()),["brand_id"=>getBrandIdFromCurrentContext(),"third_party_provider_parameter_id"=>$dd_parameter['id'],"value"=>$value]);
            }
        }

        // now edit the distance price records.
        $mdpd = new MerchantDeliveryPriceDistanceAdapter(getM());
        $mdpd_resource = $mdpd->getExactResourceFromData($data);
        $mdpd_resource->delivery_price_type = 'doordash';
        $mdpd_resource->distance_up_to = 'nullit';
        $mdpd_resource->price = -1.00;
        //$mdpd_resource->notes = "external_business_name=sumdumbiz";
        $mdpd_resource->save();

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $user_id = $user['user_id'];

        $udla = new UserDeliveryLocationAdapter(getM());
        $data = array("user_id" => $user_id, "address1" => "450 main street", "city" => "Lyons", "state" => "CO", "zip" => "80540", "phone_no" => "1234567890", "lat" => 40.224877, "lng" => -105.271122);
        $udla_resource = Resource::createByData($udla, $data);
        $udl_id = $udla_resource->user_addr_id;
        $_SERVER['force_doordash_estimate_fail'] = true;
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id, 'pickup', 'the note', 2);
        $cart_data['user_addr_id'] = $udl_id;
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data, time());
        $this->assertNotNull($checkout_resource->error);
    }

    function testDeliveryPriceBasedOnDoorDashOnlyBadPhoneNumber()
    {
        $merchant_resource = createNewTestMerchantDelivery($this->ids['menu_id'], ['long_hours' => true]);
        $merchant_id = $merchant_resource->merchant_id;
        MerchantMessageMapAdapter::createMerchantMessageMap($merchant_id, 'FUC', '1234567890', 'X');
        $data = array("merchant_id" => $merchant_resource->merchant_id);

        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $dd_third_party_parameters = $tpppa->getRecords(['third_party_provider_id'=>4001]);
        foreach ($dd_third_party_parameters as $dd_parameter) {
            if ($dd_parameter['name'] == 'external_business_name') {
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()),["merchant_id"=>$merchant_id,"third_party_provider_parameter_id"=>$dd_parameter['id'],"value"=>'sumdumbiz']);
            } else {
                $id = $dd_parameter['id'];
                $name = $dd_parameter['name'];
                $value = $name.'-1000'.$id;
                $ids[$name] = $value;
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()),["brand_id"=>getBrandIdFromCurrentContext(),"third_party_provider_parameter_id"=>$dd_parameter['id'],"value"=>$value]);
            }
        }

        // set merchant delivery info
        $mdia = new MerchantDeliveryPriceDistanceAdapter(getM());
        $mdia_resource = $mdia->getExactResourceFromData($data);
        $mdia_resource->minimum_order_amount = 0.01;
        $mdia_resource->delivery_cost = 1.00;
        $mdia_resource->delivery_increment = 15;
        $mdia_resource->max_days_out = 3;
        $mdia_resource->save();

        // now edit the distance price records.
        $mdpd = new MerchantDeliveryPriceDistanceAdapter(getM());
        $mdpd_resource = $mdpd->getExactResourceFromData($data);
        $mdpd_resource->delivery_price_type = 'Doordash';
        $mdpd_resource->name = 'Doordash';
        $mdpd_resource->distance_up_to = 'nullit';
        $mdpd_resource->price = -1.00;
        //$mdpd_resource->notes = "external_business_name=sumdumbiz";
        $mdpd_resource->save();

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $user_id = $user['user_id'];

        $udla = new UserDeliveryLocationAdapter(getM());
        $data = array("user_id" => $user_id, "address1" => "450 main street", "city" => "Lyons", "state" => "CO", "zip" => "80540", "phone_no" => "8888888888", "lat" => 40.224877, "lng" => -105.271122);
        $udla_resource = Resource::createByData($udla, $data);
        $udl_id = $udla_resource->user_addr_id;

        $request = createRequestObject("/apiv2/merchants/$merchant_id/isindeliveryarea/$udl_id?log_level=5", 'GET');
        $merchant_controller = new MerchantController(getM(), $user, $request, 5);
        $resource = $merchant_controller->processV2Request();
        $udl_id = $udla_resource->user_addr_id;

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id, 'pickup', 'the note', 2);
        $cart_data['user_addr_id'] = $udl_id;
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data, time());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource, $user, $merchant_id, 3.33, time());
        $this->assertNotNull($order_resource->error);
        $this->assertEquals("Phone number provided does not seem to be a valid one", $order_resource->error);
    }



    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        ini_set('max_execution_time', 0);
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction(); ;


        $skin_resource = createWorldHqSkin();
        $ids['skin_id'] = $skin_resource->skin_id;

        $skin_resource = getOrCreateSkinAndBrandIfNecessary('fuddruckers','fuddruckers',114,372);

        //map it to a menu
        $menu_id = createTestMenuWithNnumberOfItems(3);
        $ids['menu_id'] = $menu_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main()
    {
        $suite = new PHPUnit_Framework_TestSuite(__CLASS__);
        PHPUnit_TextUI_TestRunner::run($suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    DoorDashTest::main();
}

?>