<?php
ini_set('max_execution_time', 300);

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class DoubleDonutHoleTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp() {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $code = generateCode(7);
        $_SERVER['STAMP'] = __CLASS__ . '-' . $code;
        $_SERVER['RAW_STAMP'] = $code;
        $this->ids = $_SERVER['unit_test_ids'];
        setContext('com.splickit.doubledonutskin');
    }

    function tearDown() {
        unset($this->ids);
    }
    

	function testCreateDoubleDonutHole()
	{
	    $merchant_id = $this->ids['merchant_id'];
		$hole_hours_adapter = new HoleHoursAdapter(getM());
		$day_of_week = date('w',getTomorrowTwelveNoonTimeStampDenver())+1;
        $hole_one = Resource::createByData($hole_hours_adapter,
            array(
                "merchant_id" => $merchant_id,
                "day_of_week" => $day_of_week,
                'order_type' => 'Pickup',
                'start_time' => '10:00:00',
                'end_time' => '12:00:00',
                'second_start_time' => '14:00:00',
                'second_end_time' => '16:00:00',
            ));

        $this->assertTrue(is_a($hole_one,'Resource'));
        $url = "/app2/apiv2/merchants/$merchant_id?order_type=pickup";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(),null,$request);
        $response = makeMockRequest($merchant_controller);
        $response_array = json_decode($response,true);
        // check ot see if hole hours are in dump
        $data = $response_array['data'];
        $readable_hours = $data['readable_hours'];
        $day_of_week_string = date('l',getTomorrowTwelveNoonTimeStampDenver());
        $tomorrow = $readable_hours['pickup'][$day_of_week][$day_of_week_string];
        $this->assertEquals('7:00am-10:00am:12:00pm-2:00pm:4:00pm-8:00pm',$tomorrow,"It should show the double donut hole");
	}

	function testGetCheckoutWithDoubleDonutHole()
    {
        $_SERVER['max_lead'] = 360;
        $user_resource = createNewUser(array("flags" => "1C20000001"));
        $user = logTestUserResourceIn($user_resource);
        $merchant_id = $this->ids['merchant_id'];

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver()-(3*3600));
        $leadtime_array = $checkout_resource->lead_times_array;
        foreach ($leadtime_array as $ts) {
            myerror_log("available time: " . date('Y-m-d H:i:s', $ts));
            $this->assertFalse($ts > getTomorrowTwelveNoonTimeStampDenver()-7200 && $ts < getTomorrowTwelveNoonTimeStampDenver(), "There shoudl have been no times in teh donut hole");
            if ($ts > getTomorrowTwelveNoonTimeStampDenver() && $ts < getTomorrowTwelveNoonTimeStampDenver() + 7200) {
                $there_are_available_times_outside_of_donut_hole_after = true;
            }
            if ($ts < getTomorrowTwelveNoonTimeStampDenver()-7200) {
                $there_are_available_times_outside_of_donut_hole_before = true;
            }
        }

        $this->assertTrue($there_are_available_times_outside_of_donut_hole_before, "It should have available times before donut hole");
        $this->assertTrue($there_are_available_times_outside_of_donut_hole_after, "It should have available times after donut hole");

        // now see if the second hole is being adhered to
        foreach ($leadtime_array as $ts) {
            myerror_log("available time: " . date('Y-m-d H:i:s', $ts));
            $this->assertFalse($ts > getTomorrowTwelveNoonTimeStampDenver()+7200 && $ts < getTomorrowTwelveNoonTimeStampDenver()+14400, "There shoudl have been no times in teh donut hole");
            if ($ts > getTomorrowTwelveNoonTimeStampDenver()+14400) {
                $there_are_available_times_outside_of_donut_hole_after = true;
            }

        }

        $this->assertTrue($there_are_available_times_outside_of_donut_hole_before, "It should have available times before donut hole");
        $this->assertTrue($there_are_available_times_outside_of_donut_hole_after, "It should have available times after donut hole");
    }

    function testGetCheckoutWhileInSecondDonutHole()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $merchant_id = $this->ids['merchant_id'];

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver()+ ( 3 * 3600 ));
        $leadtime_array = $checkout_resource->lead_times_array;
        foreach ($leadtime_array as $ts) {
            myerror_log("available time: " . date('Y-m-d H:i:s', $ts));
            $this->assertFalse($ts < getTomorrowTwelveNoonTimeStampDenver() + 14400, "There should have been no times in the donut hole");
        }
        // first time shoudl be end of the donut hole
        $first_time = $leadtime_array[0];
        $first_time_string = date('Y-m-d H:i:m',$first_time);
        $expected_first_time_string = date('Y-m-d H:i:m',getTomorrowTwelveNoonTimeStampDenver() + (4 * 3600));
        $this->assertEquals($expected_first_time_string, $first_time_string);
    }

    function testAdvancedOrderingWithSeconDonutHole()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $merchant_id = $this->ids['merchant_id'];

        // set advanced ordering
        $merchant_resource = Resource::find(new MerchantAdapter(getM()),$merchant_id);
        $merchant_resource->advanced_ordering = 'Y';
        $merchant_resource->save();

        $mao_data = array("merchant_id"=>$merchant_id);
        $mao_data['max_days_out'] = 3;
        $maoia = new MerchantAdvancedOrderingInfoAdapter(getM());
        $maoi_resource = Resource::createByData($maoia,$mao_data);

        $hole_hours_adapter = new HoleHoursAdapter(getM());
        $day_of_week = date('w',getTomorrowTwelveNoonTimeStampDenver()+(24*3600)) + 1;
        $hole_one = Resource::createByData($hole_hours_adapter,
            array(
                "merchant_id" => $merchant_id,
                "day_of_week" => $day_of_week,
                'order_type' => 'Pickup',
                'start_time' => '10:00:00',
                'end_time' => '12:00:00',
                'second_start_time' => '14:00:00',
                'second_end_time' => '16:00:00',
            ));

        $this->assertTrue(is_a($hole_one,'Resource'));

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver()+ ( 3 * 3600 ));
        $tomorrow_leadtime_array = $checkout_resource->lead_times_by_day_array['Tomorrow'];

        $m = Date("m",getTomorrowTwelveNoonTimeStampDenver() + (24*3600));
        $d = Date("d",getTomorrowTwelveNoonTimeStampDenver() + (24*3600));
        $y = Date("Y",getTomorrowTwelveNoonTimeStampDenver() + (24*3600));
        $open_ts = getTimeStampForDateTimeAndTimeZone(7,0,0,$m,$d,$y);
        $first_start = getTimeStampForDateTimeAndTimeZone(10,0,0,$m,$d,$y);
        $first_end = getTimeStampForDateTimeAndTimeZone(12,0,0,$m,$d,$y);
        $second_start = getTimeStampForDateTimeAndTimeZone(14,0,0,$m,$d,$y);
        $second_end = getTimeStampForDateTimeAndTimeZone(16,0,0,$m,$d,$y);
        $close_ts = getTimeStampForDateTimeAndTimeZone(20,0,0,$m,$d,$y);

        myerror_log('open_ts: '.date("Y-m-d h:i:s",$open_ts));
        myerror_log('first_start: '.date("Y-m-d h:i:s",$first_start));
        myerror_log('first_end: '.date("Y-m-d h:i:s",$first_end));
        myerror_log('second_start: '.date("Y-m-d h:i:s",$second_start));
        myerror_log('second_end: '.date("Y-m-d h:i:s",$second_end));
        myerror_log('close_ts: '.date("Y-m-d h:i:s",$first_start));


        foreach ($tomorrow_leadtime_array as $ts) {
            $available_string = date('Y-m-d H:i:s',$ts);
            myerror_log("pickup available: $available_string");
            $this->assertTrue($ts >= $open_ts,"All Times on this day should be equal to or after opening time: ".date('Y-m-d H:i:s',$open_ts));
            $this->assertTrue($ts <= $close_ts,"All Times on this day should be equal to or before closing time: ".date('Y-m-d H:i:s',$close_ts));
            $this->assertFalse(($first_start < $ts) && ($ts < $first_end),"It should not have any times in teh first donut hole");
            $this->assertFalse(($second_start < $ts) && ($ts < $second_end),"It should not have any times in teh first donut hole");
        }

    }

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

    	ini_set('max_execution_time',300);
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction(); ;
        $skin_resource = getOrCreateSkinAndBrandIfNecessaryWithLoyalty('doubledonutskin', 'doubledonutbrand');
        setContext('com.splickit.doubledonutskin');
    	$_SERVER['request_time1'] = microtime(true);    	
		$menu_id = createTestMenuWithOneItem("Test Item 1");
    	$ids['menu_id'] = $menu_id;
    	
    	$merchant_resource = createNewTestMerchant($menu_id);
    	$merchant_id = $merchant_resource->merchant_id;
    	$ids['merchant_id'] = $merchant_id;
    	
    	$_SERVER['log_level'] = 5;
		$_SERVER['unit_test_ids'] = $ids;

    }
    
	static function tearDownAfterClass()
    {
    	SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
    }
    
	static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}

 }

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    DoubleDonutHoleTest::main();
}

?>