<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class EmailControllerTestAmazon extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $user1_email;
	var $merchant_id;
	
	function setUp()
	{
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
		setProperty("use_amazon_ses", "true");
		$this->merchant_id = $_SERVER['unit_test_ids']['merchant_id'];
       	
	}
	
	function tearDown() 
	{
		unset($this->merchant);
		$_SERVER['STAMP'] = $this->stamp;
    }


    
    function testEmailControllerSend()
    {
    	$merchant_id = $this->merchant_id;
    	
    	// first get message resource
    	$mmha = new MerchantMessageHistoryAdapter($mimetypes);
    	$map_id = $mmha->createMessage($merchant_id, $order_id, 'E', 'dummy@order140.com', time()-5, 'I', $info, 'Hello World');
    	$message_resource = Resource::find($mmha,''.$map_id);
    	$this->assertNotNull($message_resource);
    	
    	//get message controller
    	$email_controller = new EmailController($mt, $u, $r);
    	$result = $email_controller->sendThisMessage($message_resource);
    	$this->assertEquals(true, $result);
    	
    	// check for mock send
    	$m_resource = Resource::find($mmha,''.$map_id);
    	$response = $m_resource->response;
    	$this->assertContains("success", $response);
    }
    
    function testFailAmazonEmailSent()
    {
        setProperty('aws_ses_host','sumdumhost');
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'E');
        $email_controller = ControllerFactory::generateFromMessageResource($message_resource);
        $email_controller->sendThisMessage($message_resource);

        // check for mock send
        $mmha = new MerchantMessageHistoryAdapter(getM());
        $m_resource = Resource::find($mmha,''.$message_resource->map_id);
        $response = $m_resource->response;
        $this->assertContains("failure", $response);


    }

   	static function setUpBeforeClass()
    {
        $code = setNewStamp();
    	ini_set('max_execution_time',300);
    	SplickitCache::flushAll();
    	$db = DataBase::getInstance();
    	$mysqli = $db->getConnection();
    	$mysqli->begin_transaction(); ;
    	createWorldHqSkin();
    	$_SERVER['request_time1'] = microtime(true);
        $merchant_resource = createNewTestMerchant();
    	$ids['merchant_id'] = $merchant_resource->merchant_id;
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    	
    }
    
	static function tearDownAfterClass()
    {
    	SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
    }

	/* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    EmailControllerTestAmazon::main();
}
    
?>