<?php


$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';


class EmmaMarketingTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
        $this->ids = $_SERVER['unit_test_ids'];
        setContext('com.splickit.emmaskin');
    }

    function tearDown()
    {
        //delete your instance
        unset($this->ids);
        unset($_SERVER['max_lead']);
    }

    function testFactory()
    {
        $brand_id = getBrandIdFromCurrentContext();
        $marketing_service = MarketingServiceFactory::getMarketingServiceFromBrandId($brand_id);
        $this->assertEquals('EmmaService',get_class($marketing_service));
    }

    function testJoinOnCreateUser()
    {
        $birthday = '01/01/2000';
        $user_resource = createNewUser(array("marketing_email_opt_in"=>"Y",'birthday'=>$birthday));
        $this->assertNull($user_resource->error);
        $user_id = $user_resource->user_id;
        $this->assertTrue($user_id > 20000);
        $this->assertNotNull($user_resource->marketing_results);
    }

    function testDoNotJoinOnCreateUser()
    {
        $birthday = '01/01/2000';
        $user_resource = createNewUserWithCCNoCVV();
        $this->assertNull($user_resource->error);
        $user_id = $user_resource->user_id;
        $this->assertTrue($user_id > 20000);
        $this->assertNull($user_resource->marketing_results);
    }

    function testDoNotJoinOnCreateUserWithZeroValue()
    {
        $birthday = '01/01/2000';
        $user_resource = createNewUser(array("marketing_email_opt_in"=>0,'birthday'=>$birthday));
        $this->assertNull($user_resource->error);
        $user_id = $user_resource->user_id;
        $this->assertTrue($user_id > 20000);
        $this->assertNull($user_resource->marketing_results);
    }


    static function setUpBeforeClass()
    {
        $code = setNewStamp();
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time',300);
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction(); ;

        $skin_resource = getOrCreateSkinAndBrandIfNecessaryWithLoyalty("emmaskin","emmabrand",null, null);
        $ids['skin_id'] = $skin_resource->skin_id;
        setContext('com.splickit.emmaskin');

//        $simple_menu_id = createTestMenuWithOneItem("item_one");
//        $ids['menu_id'] = $simple_menu_id;
//
//        $merchant_resource = createNewTestMerchant($simple_menu_id,['authorize'=>true]);
//        $merchant_resource->save();
//        $ids['merchant_id'] = $merchant_resource->merchant_id;
//
//        $merchant_payment_type_map_resource = MerchantPaymentTypeMapsAdapter::createMerchantPaymentTypeMap($merchant_resource->merchant_id, 12020, null);

        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $parameters = $tpppa->getParametersFromProviderId(6003);
        foreach ($parameters as $parameter_row) {
            $data = [];
            if ($parameter_row['name'] == 'active_flag') {
                $data['value'] = 'Y';
            } else {
                $data['value'] = $parameter_row['name'].'-'.rand(11111,99999);
            }
            $data['third_party_provider_parameter_id'] = $parameter_row['third_party_provider_parameter_id'];
            $data['brand_id'] = getBrandIdFromCurrentContext();
            $tpppbma = new ThirdPartyProviderParameterBrandMapsAdapter(getM());
            Resource::createByData($tpppbma,$data);
        }

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    EmmaMarketingTest::main();
}
?>