<?php

/**
 * Created by PhpStorm.
 * User: Elvis
 * Date: 8/18/16
 * Time: 5:41 PM
 */

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class EnterpriseControllerTest extends PHPUnit_Framework_TestCase
{
    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__ . '-' . $_SERVER['STAMP'];
        $this->ids = $_SERVER['unit_test_ids'];
        setContext("com.splickit.snarfs");
    }

    function tearDown()
    {
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }

    function testIsEnterpriseMerchant() {
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id);
        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'L','enterprise','X');

        $isEnterprise = MerchantMessageMapAdapter::isMerchantEnterpriseMerchant($merchant_resource->merchant_id);

        $this->assertTrue($isEnterprise, "Should be an enterprise controller.");
    }

    function testEnterpriseGetquote() {
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id);
        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'L','enterprise','X');

        $merchant_id = $merchant_resource->merchant_id;

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $this->assertEquals(5.00, $checkout_resource->grand_total, "Enterprise quote grand_total not updated.");
        $this->assertEquals(0.5, $checkout_resource->total_tax_amt, "Enterprise quote total_tax_amt not updated.");
        $this->assertEquals(4.50, $checkout_resource->order_amt, "Enterprise quote order_amt not updated.");

    }

    function testEnterpriseEndToEnd()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $menu_id = createTestMenuWithNnumberOfItems(1);
        $data['message_data'] = [['message_format'=>'L','delivery_addr'=>'Enterprise','info'=>'service=SessionM;type=transaction','message_type'=>'X']];
        $merchant_resource = createNewTestMerchant($menu_id,$data);
        $merchant_id = $merchant_resource->merchant_id;

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource);
        $this->assertNull($order_resource->error);

        $enterprise_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,"L");
        $enterprise_message_controller = ControllerFactory::generateFromMessageResource($enterprise_message_resource);
        $this->assertTrue(is_a($enterprise_message_controller,'EnterpriseController'),"It should be an Enterprise controller");
        // This fails because the credentials have to be configured to run this test.
        setProperty('S3_AWS_ACCESS_KEY_ID','test');
        setProperty('S3_AWS_SECRET_ACCESS_KEY','test');
        $response = $enterprise_message_controller->sendThisMessage($enterprise_message_resource);

        $enterprise_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,"L");
        $this->assertEquals('S', $enterprise_message_resource->locked);
    }

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time', 300);
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction();

        setContext('com.splickit.snarfs');
        $ids['skin_id'] = getSkinIdForContext();

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;

    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main()
    {
        $suite = new PHPUnit_Framework_TestSuite(__CLASS__);
        PHPUnit_TextUI_TestRunner::run($suite);
    }
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    EnterpriseControllerTest::main();
}