<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class FoundryControllerTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
        $this->ids = $_SERVER['unit_test_ids'];
        setContext($this->ids['context']);
    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }

    function testGenerteImportUrl()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $menu_resource = Resource::find(new MenuAdapter(getM()),$menu_id);
        $menu_resource->version = '3.0';
        $menu_resource->save();
        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;

//        $sql = "INSERT INTO Merchant_Pos_Maps VALUES (NULL,$merchant_id,6000,NOW(),NOW(),'N')";
//        $adapter = new MySQLAdapter(getM());
//        $adapter->_query($sql);


        $brand_id = getBrandIdFromCurrentContext();
        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $foundry_third_party_parameters = $tpppa->getRecords(['third_party_provider_id' => 1005]);
        foreach ($foundry_third_party_parameters as $foundry_parameter) {
            $name = $foundry_parameter['name'];
            if ($name == 'tender_id_visa') {
                $value = '888';
            } else if ($name == 'get_menu') {
                $value = '0';
            } else if ($name == 'uses_price_sync') {
                $value = '1';
            } else {
                continue;
            }
            $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()), ["merchant_id" => $merchant_resource->merchant_id, "third_party_provider_parameter_id" => $foundry_parameter['id'], "value" => $value]);
        }

        $url = "http://127.0.0.1:" . $this->api_port . "/app2/portal/menus/$menu_id/getpricelist?merchant_id=$merchant_id";
        $request = createRequestObject($url,'GET');
        $menu_controller = new MenuController(getM(),null,$request,getBaseLogLevel());
        $resource = $menu_controller->processV2Request();
        //$this->assertTrue(isset($resource->import_url));
        $import_prices_url = $resource->import_url;
        $this->assertEquals("/app2/portal/import/foundry/".$merchant_resource->alphanumeric_id,$import_prices_url);
    }

    function testTenderIdMerchantOverride()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id,["authorize"=>true]);
        $merchant_id = $merchant_resource->merchant_id;

        #create the Merchant_Message_Map resource for the foundty group. Lets go with 'U'
        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'UE','Foundry','X');

        $brand_id = getBrandIdFromCurrentContext();
        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $foundry_third_party_parameters = $tpppa->getRecords(['third_party_provider_id' => 1005]);
        foreach ($foundry_third_party_parameters as $foundry_parameter) {
            $name = $foundry_parameter['name'];
            if ($name == 'tender_id_visa') {
                $value = '888';
            } else if ($name == 'get_menu') {
                $value = '0';
            } else {
                continue;
            }
            $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()), ["merchant_id" => $merchant_resource->merchant_id, "third_party_provider_parameter_id" => $foundry_parameter['id'], "value" => $value]);
        }



        $order_data = OrderAdapter::staticGetSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'sum dum note',1);

        $checkout_resource = getCheckoutResourceFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id);
        $order_id = $order_resource->order_id;
        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'UE');
        $foundry_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $foundry_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);
        $this->assertContains('<ADDTENDER TENDERID="888" />',$body);

    }

    function testMultipleModifiers()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 1);
        foreach ($modifier_group_resource->modifier_items as $mod_item) {
            $modifier_size_map = $mod_item->modifier_size_map;
            $mod_external = $modifier_size_map->external_id;
            $modifier_size_map->external_id = '12345:'.$modifier_size_map->external_id;
            $modifier_size_map->save();
        }
        $item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', getM());
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_resource->modifier_group_id, 0);
        $merchant_resource = createNewTestMerchant($menu_id,["authorize"=>true]);
        $merchant_id = $merchant_resource->merchant_id;
        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'U','Foundry','X');

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSuperSimpleCartArrayByMerchantId($merchant_id);
        $cart_data['items'][0]['mods'] = [];
        $cart_data['items'][0]['mods'][] = array("mod_quantity"=>2,"modifier_item_id"=>$modifier_group_resource->modifier_items[0]->modifier_item_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'U');
        $Foundry_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $Foundry_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);
        myerror_log($body);
        $expected_modifier_node = '<MODITEM MODGROUPID="12345" ITEMID="'.$mod_external.'" MODCODE="1"/>';
        $count = substr_count($body,$expected_modifier_node);
        $this->assertEquals(2,$count,'It should have 2 occurances of the modifier');

    }

    function testExceptionsTemplate()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);

        $modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 2);
        foreach ($modifier_group_resource->modifier_items as $mod_item) {
            $modifier_size_map = $mod_item->modifier_size_map;
            $modifier_size_map->external_id = '12345:'.$modifier_size_map->external_id;
            $modifier_size_map->save();
        }
        $modifier_group_resource2 = createModifierGroupWithNnumberOfItems($menu_id, 1);
        foreach ($modifier_group_resource2->modifier_items as $mod_item) {
            $modifier_size_map = $mod_item->modifier_size_map;
            $modifier_size_map->external_id = '67890:'.$modifier_size_map->external_id;
            $modifier_size_map->save();
        }
       // $modifier_group_resource3 = createModifierGroupWithNnumberOfItems($menu_id, 3);

        $item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', getM());
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_resource->modifier_group_id, 2);


        #create the test merchant
        $merchant_resource = createNewTestMerchant($menu_id,["authorize"=>true]);
        $merchant_id = $merchant_resource->merchant_id;

        #create the Merchant_Message_Map resource for the foundty group. Lets go with 'U'
        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'UE','Foundry','X');

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        // super simple should just get the first mod which is a comws with and will leave out the second which becomeasa a hold.
        $cart_data = OrderAdapter::getSuperSimpleCartArrayByMerchantId($merchant_id);
        $cart_data['items'][0]['mods'] = [];
        // add a comes with mod
        $cart_data['items'][0]['mods'][] = array("mod_quantity"=>1,"modifier_item_id"=>$modifier_group_resource->modifier_items[0]->modifier_item_id);

        // now lets add one from the other group which will be an add
        $cart_data['items'][0]['mods'][] = array("mod_quantity"=>1,"modifier_item_id"=>$modifier_group_resource2->modifier_items[0]->modifier_item_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $checkout_resource->note = 'This & That';
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'UE');
        $Foundry_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $Foundry_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);
        myerror_log($body);
        $this->assertNotContains('This & That',$body,"It should have escaped the ampersand");

        $held_external = $modifier_group_resource->modifier_items[1]->modifier_size_map->external_id;
        $he = explode(':',$held_external);
        $expected_held_node = '<MODITEM MODGROUPID="'.$he[0].'" ITEMID="'.$he[1].'" MODCODE="2"/>';
        $this->assertContains($expected_held_node,$body);

        $added_external = $modifier_group_resource2->modifier_items[0]->modifier_size_map->external_id;
        $ae = explode(':',$added_external);

        $expected_added_node = '<MODITEM MODGROUPID="'.$ae[0].'" ITEMID="'.$ae[1].'" MODCODE="1"/>';
        $this->assertContains($expected_added_node,$body);

        $comes_with_external = $modifier_group_resource->modifier_items[0]->modifier_size_map->external_id;
        $cwe = explode(':',$comes_with_external);
        $this->assertNotContains($cwe[1],$body,'It should not have the comes with external id');
    }

    function testCateringWithFoundryTemplate()
    {
        $menu_id = createTestCateringMenuWithOneItem();
        $item_size_resources = CompleteMenu::getAllItemSizesAsResources($menu_id,0);
        $item_size_resource = $item_size_resources[0];
        $item_size_resource->price = 50.00;
        $item_size_resource->save();

        $merchant_resource = createNewTestMerchantWithCatering($menu_id, null);
        $merchant_id = $merchant_resource->merchant_id;


        #create the Merchant_Message_Map resource for the foundty group. Lets go with 'U'
        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_id,'U','Foundry','X');


        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $catering_data = [];
        $catering_data['number_of_people'] = 10;
        $catering_data['merchant_id'] = $merchant_id;
        $catering_data['event'] = 'business lunch';
        $timestamp_of_event = getTomorrowTwelveNoonTimeStampDenver()+3600;
        $catering_data['timestamp_of_event'] = $timestamp_of_event;
        $date_time_of_event = date('n/d g:iA',$timestamp_of_event);
        $catering_data['contact_name'] = 'adam';
        $catering_data['contact_phone'] = '123 456 7890';
        $catering_data['notes'] = "Please make sure that there are plenty of napkins";
        $info = $catering_data['contact_name'].' '.$catering_data['contact_phone'];
        $notes = $catering_data['notes'];


        $json = json_encode($catering_data);
        $request = createRequestObject("/apiv2/catering","POST",$json);
        $catering_controller = new CateringController($m,$user,$request);
        $response = $catering_controller->processV2Request();
        $this->assertTrue(is_a($response,'Resource'),"The response should be a resource");
        $this->assertNull($response->error);
        $ucid = $response->ucid;
        $this->assertNotNull($ucid);
        $order_id = $response->order_id;

        $this->assertTrue($order_id > 1000,"we have a valid order id");

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $cart_data['ucid'] = $ucid;

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,time());
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,5.00,time());
        $catering_order_record = CateringOrdersAdapter::staticGetRecord(array("order_id"=>$order_id),'CateringOrdersAdapter');
        $this->assertEquals('Submitted',$catering_order_record['status']);

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'U');
        $Foundry_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $Foundry_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);
        myerror_log($body);

        $date = date('n/j',$timestamp_of_event);
        $expected_ready_string = "ORDER READY: $date 1:00PM";
        $this->assertContains($expected_ready_string,$body,"It should have the date becuase its not today");

        $expected_catering_info = "10 ppl, $date_time_of_event, $info, $notes";
        $this->assertContains($expected_catering_info,$body,'It should have the catering info');




    }

    function testTemplateWithCash()
    {
        $skin_resource = getOrCreateSkinAndBrandIfNecessary('osf','osfbrand',199,480);
        setContext('com.splickit.osf');
        $merchant_resource = createNewTestMerchant($this->ids['menu_id'],["authorize"=>true,"no_payment"=>true,"cash"=>true]);
        attachMerchantToSkin($merchant_resource->merchant_id,getSkinIdForContext());
        $merchant_id = $merchant_resource->merchant_id;

        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'U','Foundry','X');

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'U');
        $Foundry_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $Foundry_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);
        myerror_log($body);

        $this->assertNotContains('<ADDTENDER TENDERID',$body);
        $this->assertNotContains('<PAYMENTREQUESTS>',$body);

        $bca = new BalanceChangeAdapter(getM());
        $records = $bca->getRecords(['order_id'=>$order_id]);
        $this->assertCount(1,$records);
        $hash = createHashmapFromArrayOfArraysByFieldName($records,'process');
        $this->assertNotNull($hash['Cash'],"it should only have the cash row");


    }


    function testDelieryWithDoorDash()
    {
        $skin_resource = getOrCreateSkinAndBrandIfNecessary('foundryddskin','foundryddbrand');
        setContext('com.splickit.foundryddskin');
        $merchant_resource = createNewTestMerchantDelivery($this->ids['menu_id'],["authorize"=>true,'long_hours' => true]);
        $merchant_id = $merchant_resource->merchant_id;
        $data = array("merchant_id" => $merchant_resource->merchant_id);

        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $foundry_third_party_parameters = $tpppa->getRecords(['third_party_provider_id' => 1005]);
        foreach ($foundry_third_party_parameters as $foundry_parameter) {
            $name = $foundry_parameter['name'];
            if ($name == 'modecode') {
                $value = '1';
            } else if ($name == 'modecode_holds') {
                $value = '2';
            } else if ($name == 'promo_id') {
                $value = '12345';
            } else if ($name == 'tender_id_visa') {
                $value = '215';
            } else if ($name == 'tender_id_mastercard') {
                $value = '216';
            } else if ($name == 'tender_id_american_express') {
                $value = '217';
            } else if ($name == 'tender_id_discover') {
                $value = '218';
            } else if ($name == 'brand_order_mode_delivery') {
                $value = '3';
            } else if ($name == 'brand_order_mode_pickup') {
                $value = '3';
            } else if ($name == 'get_menu') {
                $value = '0';
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()), ["merchant_id" => $merchant_resource->merchant_id, "third_party_provider_parameter_id" => $foundry_parameter['id'], "value" => $value]);
                continue;
            }
            $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()), ["brand_id" => getBrandIdFromCurrentContext(), "third_party_provider_parameter_id" => $foundry_parameter['id'], "value" => $value]);
        }




        // set merchant delivery info
        $mdia = new MerchantDeliveryPriceDistanceAdapter(getM());
        $mdia_resource = $mdia->getExactResourceFromData($data);
        $mdia_resource->minimum_order_amount = 0.01;
        $mdia_resource->delivery_cost = 1.00;
        $mdia_resource->delivery_increment = 15;
        $mdia_resource->max_days_out = 3;
        $mdia_resource->save();

        // now edit the distance price records.
        $mdpd = new MerchantDeliveryPriceDistanceAdapter(getM());
        $mdpd_resource = $mdpd->getExactResourceFromData($data);
        $mdpd_resource->delivery_price_type = 'Doordash';
        $mdpd_resource->name = 'Doordash';
        $mdpd_resource->distance_up_to = 'nullit';
        $mdpd_resource->price = -1.00;
        $mdpd_resource->notes = "external_business_name=sumdumbiz";
        $mdpd_resource->save();



        attachMerchantToSkin($merchant_id,getSkinIdForContext());

        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'U','Foundry','X');

        $user_resource = createNewUserWithCCNoCVV();
        $contact_no = $user_resource->contact_no;
        $user = logTestUserResourceIn($user_resource);
        $json = '{"user_addr_id":null,"user_id":"' . $user['user_id'] . '","name":"","address1":"1045 Pine Street","address2":"","city":"boulder","state":"co","zip":"80302","phone_no":"'.$contact_no.'","lat":40.0197891,"lng":-105.284703,"instructions":"beware of dog"}';

        $url = "/users/" . $user['uuid'] . "/userdeliverylocation";
        $request = createRequestObject($url,'POST',$json);
        $user_controller = new UserController(getM(), $user, $request, 5);
        $response = $user_controller->processV2Request();
        $this->assertNull($response->error, "should not have gotten a delivery save error but did");
        $this->assertNotNull($response->user_addr_id);
        $user_address_id = $response->user_addr_id;
        $user_delivery_location_resource = SplickitController::getResourceFromId($user_address_id, 'UserDeliveryLocation');
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'delivery');
        $cart_data['user_addr_id'] = $user_address_id;
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $this->assertContains('---Doordash Delivery Id:',$order_resource->note);
        $foundry_message = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'U');
        $foundry_controller = ControllerFactory::generateFromMessageResource($foundry_message);
        $message_resource = $foundry_controller->prepMessageForSending($foundry_message);
        $message_text = $message_resource->message_text;

        $this->assertContains('ORDERMODE="3"',$message_text);
        $this->assertContains('---Doordash Delivery Id:',$message_text);
        $this->assertNotContains('<DELIVERY ',$message_text);

    }



    function testDelivery()
    {
        $skin_resource = getOrCreateSkinAndBrandIfNecessary('osf','osfbrand',199,480);
        setContext('com.splickit.osf');
        $merchant_resource = createNewTestMerchantDelivery($this->ids['menu_id'],["authorize"=>true]);
        $merchant_id = $merchant_resource->merchant_id;
        attachMerchantToSkin($merchant_id,getSkinIdForContext());

        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'U','Foundry','X');

        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $foundry_third_party_parameters = $tpppa->getRecords(['third_party_provider_id' => 1005]);
        foreach ($foundry_third_party_parameters as $foundry_parameter) {
            $name = $foundry_parameter['name'];
            if ($name == 'modecode') {
                $value = '1';
            } else if ($name == 'modecode_holds') {
                $value = '2';
            } else if ($name == 'promo_id') {
                $value = '12345';
            } else if ($name == 'tender_id_visa') {
                $value = '215';
            } else if ($name == 'tender_id_mastercard') {
                $value = '216';
            } else if ($name == 'tender_id_american_express') {
                $value = '217';
            } else if ($name == 'tender_id_discover') {
                $value = '218';
            } else if ($name == 'brand_order_mode_delivery') {
                $value = '7';
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()), ["merchant_id" => $merchant_resource->merchant_id, "third_party_provider_parameter_id" => $foundry_parameter['id'], "value" => $value]);
                continue;
            } else if ($name == 'brand_order_mode_pickup') {
                $value = '3';
            } else if ($name == 'get_menu') {
                $value = '0';
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()), ["merchant_id" => $merchant_resource->merchant_id, "third_party_provider_parameter_id" => $foundry_parameter['id'], "value" => $value]);
                continue;
            }
            $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()), ["brand_id" => getBrandIdFromCurrentContext(), "third_party_provider_parameter_id" => $foundry_parameter['id'], "value" => $value]);
        }

        $user_resource = createNewUserWithCCNoCVV();
        $contact_no = $user_resource->contact_no;
        $user = logTestUserResourceIn($user_resource);
        $json = '{"user_addr_id":null,"user_id":"' . $user['user_id'] . '","name":"","address1":"1045 Pine Street","address2":"","city":"boulder","state":"co","zip":"80302","phone_no":"'.$contact_no.'","lat":40.0197891,"lng":-105.284703,"instructions":"beware of dog"}';

        $url = "/users/" . $user['uuid'] . "/userdeliverylocation";
        $request = createRequestObject($url,'POST',$json);
        $user_controller = new UserController(getM(), $user, $request, 5);
        $response = $user_controller->processV2Request();
        $this->assertNull($response->error, "should not have gotten a delivery save error but did");
        $this->assertNotNull($response->user_addr_id);
        $user_address_id = $response->user_addr_id;
        $user_delivery_location_resource = SplickitController::getResourceFromId($user_address_id, 'UserDeliveryLocation');
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'delivery');
        $cart_data['user_addr_id'] = $user_address_id;
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $foundry_message = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'U');
        $foundry_controller = ControllerFactory::generateFromMessageResource($foundry_message);
        $message_resource = $foundry_controller->prepMessageForSending($foundry_message);
        $message_text = $message_resource->message_text;

        $this->assertContains('ORDERMODE="7"',$message_text);
        $this->assertContains('<DELIVERY ',$message_text);
        $this->assertContains('DELIVERYNOTE="beware of dog"',$message_text);

        //now set to sent
        $foundry_message->locked = 'S';
        $foundry_message->viewed = 'N';
        $foundry_message->stamp = getStamp();
        $foundry_message->sent_dt_tm = getMySqlFormattedDateTimeFromTimeStampAndTimeZone(time());
        $foundry_message->save();

        return $order_id;
    }

    /**
     * @depends testDelivery
     */
    function testIgnoreErrorIfDeliveryTicketPrinted($order_id)
    {
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,getM());
        $alphanumeric_id = $complete_order['merchant']['alphanumeric_id'];
        $call_back_payload = '<POSRESPONSE><ERRORS COUNT="2"><ERROR ERRNUM="9" TEXT="04/24 07:10:59 Aloha Order Bridge: getCurrentCheckObj(): STEP:4; Error HRESULT E_FAIL has been returned from a call to a COM component." /><LOG TEXT="04/24 07:10:59 PRINTED DELIVERY TICKET:&lt;PRINT&gt;&lt;COMMANDS&gt;&lt;SETCODEPAGE /&gt;&lt;STOPJOURNAL /&gt;&lt;STOPJOURNAL /&gt;&lt;PRINTSTYLE&gt;&lt;CPI&gt;1&lt;/CPI&gt;&lt;STYLE&gt;0&lt;/STYLE&gt;&lt;/PRINTSTYLE&gt;&lt;PRELINEFEEDS&gt;1&lt;/PRELINEFEEDS&gt;&lt;PRINTCENTERED&gt;Spokane&lt;/PRINTCENTERED&gt;&lt;PRINTCENTERED&gt;152 S. Monroe&lt;/PRINTCENTERED&gt;&lt;PRINTCENTERED&gt;&lt;/PRINTCENTERED&gt;&lt;PRINTCENTERED&gt;&lt;/PRINTCENTERED&gt;&lt;LINEFEED&gt;1&lt;/LINEFEED&gt;&lt;PRINTSTYLE&gt;&lt;CPI&gt;1&lt;/CPI&gt;&lt;STYLE&gt;3&lt;/STYLE&gt;&lt;/PRINTSTYLE&gt;&lt;PRINTCENTERED&gt;DELIVERY TICKET&lt;/PRINTCENTERED&gt;&lt;PRINTSTYLE&gt;&lt;CPI&gt;1&lt;/CPI&gt;&lt;STYLE&gt;0&lt;/STYLE&gt;&lt;/PRINTSTYLE&gt;&lt;LINEFEED&gt;1&lt;/LINEFEED&gt;&lt;PRINTLEFTRIGHT&gt;&lt;LEFT&gt;&lt;/LEFT&gt;&lt;RIGHT&gt;04/24/2020&lt;/RIGHT&gt;&lt;/PRINTLEFTRIGHT&gt;&lt;PRINTLEFTRIGHT&gt;&lt;LEFT&gt;CHK NAME:&lt;/LEFT&gt;&lt;RIGHT&gt;07:10 AM&lt;/RIGHT&gt;&lt;/PRINTLEFTRIGHT&gt;&lt;PRINTLEFTRIGHT&gt;&lt;LEFT&gt;ONLINE ORDER ID:'.$order_id.'&lt;/LEFT&gt;&lt;RIGHT&gt;#990001&lt;/RIGHT&gt;&lt;/PRINTLEFTRIGHT&gt;&lt;PRINTSTYLE&gt;&lt;CPI&gt;1&lt;/CPI&gt;&lt;STYLE&gt;0&lt;/STYLE&gt;&lt;/PRINTSTYLE&gt;&lt;LINEFEED&gt;1&lt;/LINEFEED&gt;&lt;PRINTCENTERED&gt;CONTACT INFORMATION&lt;/PRINTCENTERED&gt;&lt;PRINTLINE&gt;CONTACT:Test DO NOT MAKE&lt;/PRINTLINE&gt;&lt;PRINTLINE&gt;PHONE..:720-841-1460&lt;/PRINTLINE&gt;&lt;PRINTLINE&gt;EMAIL..:store_Tester@order140.com&lt;/PRINTLINE&gt;&lt;PRINTSTYLE&gt;&lt;CPI&gt;1&lt;/CPI&gt;&lt;STYLE&gt;0&lt;/STYLE&gt;&lt;/PRINTSTYLE&gt;&lt;LINEFEED&gt;1&lt;/LINEFEED&gt;&lt;PRINTCENTERED&gt;DELIVERY ADDRESS&lt;/PRINTCENTERED&gt;&lt;PRINTLINE&gt;160 S Monroe St,&lt;/PRINTLINE&gt;&lt;PRINTLINE&gt;CROSS ST:none&lt;/PRINTLINE&gt;&lt;PRINTLINE&gt;NOTE:beware of the dog!!!&lt;/PRINTLINE&gt;&lt;PRINTLINE&gt;Spokane, wa 99201&lt;/PRINTLINE&gt;&lt;LINEFEED&gt;1&lt;/LINEFEED&gt;&lt;PRINTSTYLE&gt;&lt;CPI&gt;1&lt;/CPI&gt;&lt;STYLE&gt;0&lt;/STYLE&gt;&lt;/PRINTSTYLE&gt;&lt;PRINTLEFTRIGHT&gt;&lt;LEFT&gt;(4) Coca-Cola&lt;/LEFT&gt;&lt;RIGHT&gt;$11.96&lt;/RIGHT&gt;&lt;/PRINTLEFTRIGHT&gt;&lt;PRINTLEFTRIGHT&gt;&lt;LEFT&gt;(4) Sprite&lt;/LEFT&gt;&lt;RIGHT&gt;$11.96&lt;/RIGHT&gt;&lt;/PRINTLEFTRIGHT&gt;&lt;PRINTLEFTRIGHT&gt;&lt;LEFT&gt;Dr. Pepper&lt;/LEFT&gt;&lt;RIGHT&gt;$2.99&lt;/RIGHT&gt;&lt;/PRINTLEFTRIGHT&gt;&lt;PRINTLEFTRIGHT&gt;&lt;LEFT&gt;Root Beer&lt;/LEFT&gt;&lt;RIGHT&gt;$2.99&lt;/RIGHT&gt;&lt;/PRINTLEFTRIGHT&gt;&lt;PRINTLEFTRIGHT&gt;&lt;LEFT&gt;Raspberry Iced Tea&lt;/LEFT&gt;&lt;RIGHT&gt;$2.99&lt;/RIGHT&gt;&lt;/PRINTLEFTRIGHT&gt;&lt;PRINTLEFTRIGHT&gt;&lt;LEFT&gt;Hot Tea&lt;/LEFT&gt;&lt;RIGHT&gt;$1.75&lt;/RIGHT&gt;&lt;/PRINTLEFTRIGHT&gt;&lt;LINEFEED&gt;1&lt;/LINEFEED&gt;&lt;PRINTLINE&gt;ITEM COUNT: 12&lt;/PRINTLINE&gt;&lt;PRINTSTYLE&gt;&lt;CPI&gt;1&lt;/CPI&gt;&lt;STYLE&gt;0&lt;/STYLE&gt;&lt;/PRINTSTYLE&gt;&lt;LINEFEED&gt;1&lt;/LINEFEED&gt;&lt;PRINTSTYLE&gt;&lt;CPI&gt;1&lt;/CPI&gt;&lt;STYLE&gt;0&lt;/STYLE&gt;&lt;/PRINTSTYLE&gt;&lt;LINEFEED&gt;1&lt;/LINEFEED&gt;&lt;PRINTLEFTRIGHT&gt;&lt;LEFT&gt;Sub Total&lt;/LEFT&gt;&lt;RIGHT&gt;$34.64&lt;/RIGHT&gt;&lt;/PRINTLEFTRIGHT&gt;&lt;PRINTLEFTRIGHT&gt;&lt;LEFT&gt;Tax&lt;/LEFT&gt;&lt;RIGHT&gt;$3.08&lt;/RIGHT&gt;&lt;/PRINTLEFTRIGHT&gt;&lt;LINEFEED&gt;1&lt;/LINEFEED&gt;&lt;PRINTLEFTRIGHT&gt;&lt;LEFT&gt;Total&lt;/LEFT&gt;&lt;RIGHT&gt;$37.72&lt;/RIGHT&gt;&lt;/PRINTLEFTRIGHT&gt;&lt;LINEFEED&gt;1&lt;/LINEFEED&gt;&lt;PRINTSTYLE&gt;&lt;CPI&gt;1&lt;/CPI&gt;&lt;STYLE&gt;2&lt;/STYLE&gt;&lt;/PRINTSTYLE&gt;&lt;PRINTLEFTRIGHT&gt;&lt;LEFT&gt;Balance Due&lt;/LEFT&gt;&lt;RIGHT&gt;$0.00&lt;/RIGHT&gt;&lt;/PRINTLEFTRIGHT&gt;&lt;PRINTSTYLE&gt;&lt;CPI&gt;1&lt;/CPI&gt;&lt;STYLE&gt;1&lt;/STYLE&gt;&lt;/PRINTSTYLE&gt;&lt;POSTLINEFEEDS&gt;3&lt;/POSTLINEFEEDS&gt;&lt;LINEFEED&gt;1&lt;/LINEFEED&gt;&lt;CUT&gt;PARTIAL&lt;/CUT&gt;&lt;POSTLINEFEEDS&gt;1&lt;/POSTLINEFEEDS&gt;&lt;/COMMANDS&gt;&lt;DOCTYPE&gt;2&lt;/DOCTYPE&gt;&lt;PRINTER&gt;1&lt;/PRINTER&gt;&lt;/PRINT&gt;" /></ERRORS><DEBUGS COUNT="0" /><LOGS COUNT="0" /><CHECKRESPONSES><ADDCHECK SYSTEMERRORS="0" ITEMERRORS="0" TENDERERRORS="0" EXTCHECKID="Test DO NOT MAKE'.$order_id.'" ORDERID="'.$order_id.'" INTCHECKID="990001" POSCHECKSUBTOTAL="34.64" POSCHECKTAX="3.08" POSCHECKTOTAL="37.72" DONOTRESEND="TRUE"><LOGS COUNT="4"><LOG TEXT="04/24 07:10:58 Starting Online Order ID '.$order_id.' with Ready Time of 2020/04/24 16:05" /><LOG TEXT="04/24 07:10:58 Online Order ID:'.$order_id.' Order Name:Test DO NOT '.$order_id.' is Aloha Check ID:990001" /><LOG TEXT="04/24 07:10:59 Looking for Lock file:D:\\BOOTDRV\\Aloha\\DATA\\06300001.OLK" /><LOG TEXT="04/24 07:10:59 Deleting Lock File:D:\\BOOTDRV\\Aloha\\DATA\\06300001.OLK" /></LOGS><DEBUGS COUNT="32"><DEBUG TEXT="04/24 07:10:58 D3: Login: Term:99;Employee:5678" /><DEBUG TEXT="04/24 07:10:58 D3: Clock In: Term:99;Job Code:10" /><DEBUG TEXT="04/24 07:10:58 D4: BEGIN Aloha Item:100 ItemPrice:POS Price/ Aloha Entry:103809043" /><DEBUG TEXT="04/24 07:10:58 D4: END Item:103809043" /><DEBUG TEXT="04/24 07:10:58 D4: BEGIN Aloha Item:105 ItemPrice:POS Price/ Aloha Entry:103809048" /><DEBUG TEXT="04/24 07:10:58 D4: END Item:103809048" /><DEBUG TEXT="04/24 07:10:58 D4: BEGIN Aloha Item:105 ItemPrice:POS Price/ Aloha Entry:103809053" /><DEBUG TEXT="04/24 07:10:58 D4: END Item:103809053" /><DEBUG TEXT="04/24 07:10:58 D4: BEGIN Aloha Item:105 ItemPrice:POS Price/ Aloha Entry:103809058" /><DEBUG TEXT="04/24 07:10:58 D4: END Item:103809058" /><DEBUG TEXT="04/24 07:10:59 D4: BEGIN Aloha Item:103 ItemPrice:POS Price/ Aloha Entry:103809063" /><DEBUG TEXT="04/24 07:10:59 D4: END Item:103809063" /><DEBUG TEXT="04/24 07:10:59 D4: BEGIN Aloha Item:100 ItemPrice:POS Price/ Aloha Entry:103809068" /><DEBUG TEXT="04/24 07:10:59 D4: END Item:103809068" /><DEBUG TEXT="04/24 07:10:59 D4: BEGIN Aloha Item:100 ItemPrice:POS Price/ Aloha Entry:103809073" /><DEBUG TEXT="04/24 07:10:59 D4: END Item:103809073" /><DEBUG TEXT="04/24 07:10:59 D4: BEGIN Aloha Item:100 ItemPrice:POS Price/ Aloha Entry:103809078" /><DEBUG TEXT="04/24 07:10:59 D4: END Item:103809078" /><DEBUG TEXT="04/24 07:10:59 D4: BEGIN Aloha Item:102 ItemPrice:POS Price/ Aloha Entry:103809083" /><DEBUG TEXT="04/24 07:10:59 D4: END Item:103809083" /><DEBUG TEXT="04/24 07:10:59 D4: BEGIN Aloha Item:106 ItemPrice:POS Price/ Aloha Entry:103809088" /><DEBUG TEXT="04/24 07:10:59 D4: END Item:103809088" /><DEBUG TEXT="04/24 07:10:59 D4: BEGIN Aloha Item:110 ItemPrice:POS Price/ Aloha Entry:103809093" /><DEBUG TEXT="04/24 07:10:59 D4: END Item:103809093" /><DEBUG TEXT="04/24 07:10:59 D4: BEGIN Aloha Item:105 ItemPrice:POS Price/ Aloha Entry:103809098" /><DEBUG TEXT="04/24 07:10:59 D4: END Item:103809098" /><DEBUG TEXT="04/24 07:10:59 D4: PaymentRequests greater than 0" /><DEBUG TEXT="04/24 07:10:59 D4: No Tender Amount Found - Getting Check Total." /><DEBUG TEXT="04/24 07:10:59 D4: Apply Payment:100 Amount:37.72 Tip:0" /><DEBUG TEXT="04/24 07:10:59 D4: Payment Approved" /><DEBUG TEXT="04/24 07:10:59 D3: CHECK PRINT REQUESTED" /><DEBUG TEXT="04/24 07:10:59 D3: Logout: Term:99" /></DEBUGS><ERRORS COUNT="0" /></ADDCHECK></CHECKRESPONSES><PRINTRESPONSES /></POSRESPONSE>';
        $request = new Request();
        $request->body = $call_back_payload;
        $request->method = 'POST';
        $request->mimetype = 'application/xml';
        $request->url = 'http://localhost/app2/messagemanager/foundry/getnextmessagebymerchantid?reqtype=04&siteid='.$alphanumeric_id.'&reqfmt=01';
        $_SERVER['QUERY_STRING'] = 'reqtype=04&siteid='.$alphanumeric_id.'&reqfmt=01';
        $request->_parseRequestBody();
        $message_controller = ControllerFactory::generateFromUrl($request->url, getM(), null, $request, 5);
        $message_controller->pullNextMessageResourceByMerchant($alphanumeric_id);
        $message = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'U');
        $this->assertEquals($call_back_payload,$message->response,"It should have saved the response on the message object");
        $this->assertEquals('V',$message->viewed,"It should have marked the message as viewed");

        // make sure order is now marked as 'E'
        $order_record = CompleteOrder::getBaseOrderData($message->order_id,$m);
        $this->assertEquals('E',$order_record['status'],"Status should have been marked as 'E'");

    }

    function testCounterPickup()
    {
        $skin_resource = getOrCreateSkinAndBrandIfNecessary('osf','osfbrand',199,480);
        setContext('com.splickit.osf');
        $merchant_resource = createNewTestMerchant($this->ids['menu_id'],["authorize"=>true]);
        attachMerchantToSkin($merchant_resource->merchant_id,getSkinIdForContext());
        $merchant_id = $merchant_resource->merchant_id;

        #create the Merchant_Message_Map resource for the foundty group. Lets go with 'U'
        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'U','Foundry','X');

        $merchant_menu_map_resource = Resource::find(new MerchantMenuMapAdapter(getM()),'',[3=>['merchant_id'=>$merchant_id,'merchant_menu_type'=>'pickup']]);
        $merchant_menu_map_resource->allows_dine_in_orders = 1;
        $merchant_menu_map_resource->save();

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $checkout_resource->dine_in = true;
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'U');
        $Foundry_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $Foundry_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);
        $this->assertContains('LASTNAME="COUNTER PICKUP"', $body,"It should contain the countin pickup in the last name");


    }


    function testPromoOnTemplate()
    {
        $user = logTestUserIn($this->ids['user_id']);
        $merchant_id = $this->ids['merchant_id'];
        $order_data = OrderAdapter::staticGetSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'sum dum note',3);
        $order_data['promo_code'] = 'FoundryPromo';
        $checkout_resource = getCheckoutResourceFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $this->assertTrue($checkout_resource->promo_amt < 0, "It should have a valid promo amount'");

        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id);
        $order_id = $order_resource->order_id;
        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'U');
        $Foundry_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $Foundry_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);
        $this->assertContains('<ADDITEM ITEMID="54321" PRICE="-1.00" QTY="1"/>',$body);


    }

    function testGetMenuPayload()
    {
        $merchant_resource = createNewTestMerchant();
        $alphanumeric = $merchant_resource->alphanumeric_id;
        $data['merchant_id'] = $merchant_resource->merchant_id;
        $data['menu_id'] = $this->ids['menu_id'];
        // we do it this way to prevent the creation of daughter records till the import
        Resource::createByData(new MerchantMenuMapAdapter(getM()),$data);
        $maps_resource = Resource::createByData(new MerchantFoundryInfoMapsAdapter($m),$data);
        $maps_resource->logical_delete = 'Y';
        $maps_resource->save();

        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $foundry_third_party_parameters = $tpppa->getRecords(['third_party_provider_id' => 1005]);
        foreach ($foundry_third_party_parameters as $foundry_parameter) {
            $name = $foundry_parameter['name'];
            if ($name == 'modecode') {
                $value = '1';
            } else if ($name == 'modecode_holds') {
                $value = '2';
            } else if ($name == 'promo_id') {
                $value = '54321';
            } else if ($name == 'tender_id_visa') {
                $value = '215';
            } else if ($name == 'tender_id_mastercard') {
                $value = '216';
            } else if ($name == 'tender_id_american_express') {
                $value = '217';
            } else if ($name == 'tender_id_discover') {
                $value = '218';
            } else if ($name == 'brand_order_mode_delivery') {
                $value = '3';
            } else if ($name == 'brand_order_mode_pickup') {
                $value = '3';
            } else if ($name == 'get_menu') {
                $value = '0';
                $tpppmmap_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()), ["merchant_id" => $merchant_resource->merchant_id, "third_party_provider_parameter_id" => $foundry_parameter['id'], "value" => $value]);
                continue;
            }
            $tpppbmap_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()), ["brand_id" => getBrandIdFromCurrentContext(), "third_party_provider_parameter_id" => $foundry_parameter['id'], "value" => $value]);
        }

        $url = "/messagemanager/foundry/getnextmessagebymerchantid?reqtype=01&siteid=$alphanumeric&reqfmt=01";
        $request = createRequestObject($url,'GET');

        $foundry_controller = new FoundryController($m,$u,$request,5);
        $response = $foundry_controller->pullNextMessageResourceByMerchant($merchant_resource->alphanumeric_id);
        $this->assertEquals('<SVRRESPONSE>1</SVRRESPONSE>',$response->message_text,'It should return the no messages xml');

        $tpppmmap_resource->value = 1;
        $tpppmmap_resource->save();

        $foundry_controller = new FoundryController($m,$u,$request,5);
        $response2 = $foundry_controller->pullNextMessageResourceByMerchant($merchant_resource->alphanumeric_id);
        $expected = '<SVRRESPONSE REQTYPE="01" SITEID="'.$merchant_resource->alphanumeric_id.'" SVRREQ="1"  >1</SVRRESPONSE>';
        $this->assertEquals($expected,$response2->message_text,'It should return the no messages xml');

        // check to see if flag was reset
        $tpppmmap_resource = $tpppmmap_resource->getRefreshedResource();
        $this->assertEquals("0",$tpppmmap_resource->value);

    }


    function testCreateFoundryMessageControllerFromFactory()
    {
        $message_resource = Resource::dummyfactory(array("message_format" => 'U'));
        $controller_factory = new ControllerFactory();
        $controller_name = $controller_factory->getControllerNameFromMessageResource($message_resource);
        $this->assertEquals('FoundryController',$controller_name,"It should return Foundry as the controller name");

        $controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $this->assertEquals('FoundryController', get_class($controller), "It should return a Foundry Controller");
    }

    function testCreateFoundryMessageControllerFromUrl()
    {
        //$url = "/getnextmessagebymerchantid/Foundry/3456wert3456ert";
        $url = "/messagemanager/foundry/getnextmessagebymerchantid?reqtype=01&siteid=3456wert3456ert&reqfmt=01";
        $request = createRequestObject($url,"GET");
        $controller = ControllerFactory::generateFromUrl($url,getM(),null,$request,5);
        $this->assertEquals('FoundryController', get_class($controller), "It should return a Foundry Controller");
    }

    function testGenerateTemplate()
    {
        $user = logTestUserIn($this->ids['user_id']);
        $merchant_id = $this->ids['merchant_id'];
        //$order_data = OrderAdapter::staticGetSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'sum dum note',3);
        $order_data = OrderAdapter::staticGetSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'sum dum note',1);
        $order_data['tip'] = 0.00;
        $order_resource = placeOrderFromOrderData($order_data, getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id);
        $order_id = $order_resource->order_id;
        $messages = MerchantMessageHistoryAdapter::getAllOrderMessages($order_id);
        $messages_hash = createHashmapFromArrayOfResourcesByFieldName($messages, 'message_format');
        $this->assertNotNull($messages_hash['U'], "SHould have created a message with the Foundry format");
        return $messages_hash['U'];
    }

    /**
     * @depends testGenerateTemplate
     */
    function testCreateTemplateFromMessageResourece($message_resource)
    {
        $this->assertTrue(true);
        $expected_payload = cleanUpDoubleSpacesCRLFTFromString(file_get_contents("./unit_tests/resources/expected_Foundry_message_body.txt"));
        $order_id = $message_resource->order_id;
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
        $user = $complete_order['user'];

        $expected_payload = str_replace("%%ready_time%%",$complete_order['pickup_time_military_with_seconds'],$expected_payload);
        $expected_payload = str_replace("%%ready_date_time%%",$complete_order['pickup_date_time_foundry'],$expected_payload);
        $expected_payload = str_replace("%%pickup_time_ampm%%",$complete_order['pickup_time_ampm'],$expected_payload);
        $expected_payload = str_replace("%%order_id%%",$order_id,$expected_payload);
        $expected_payload = str_replace("%%note%%",$complete_order['note'],$expected_payload);
        $expected_payload = str_replace("%%user_email%%",$user['email'],$expected_payload);
        $expected_payload = str_replace("%%user_phone_no%%",$user['contact_no'],$expected_payload);
        $expected_payload = str_replace("%%item_external_id%%",$complete_order['order_details'][0]['external_id'],$expected_payload);

        $mods_stuff = explode(':',$complete_order['order_details'][0]['order_detail_complete_modifier_list_no_holds'][0]['external_id']);
        $expected_payload = str_replace("%%modifier_external_id%%",$mods_stuff[1],$expected_payload);
        $expected_payload = str_replace("%%modifier_group_external_id%%",$mods_stuff[0],$expected_payload);
        $Foundry_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $Foundry_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);

        $expected_payload = str_replace("> <","><",$expected_payload);
        $this->assertEquals($expected_payload,$body,"should have created the expected Foundry payload");

        // now save the message as if it was sent
//        $message_resource->message_text = $body;
//        $message_resource->locked = 'S';
//        $message_resource->sent_dt_tm = date("Y-m-d H:i:s",time());
//        $message_resource->viewed = 'N';
//        $message_resource->stamp = $_SERVER['STAMP'];
//        $message_resource->tries = 1;
//        $message_resource->save();

        $message_resource->next_message_dt_tm = time() - 100;
        $message_resource->save();
        return $expected_payload;
    }

    /**
     * @depends testCreateTemplateFromMessageResourece
     */
    function testPullMessageResource($expected_payload)
    {

        $merchant_id = $this->ids['merchant_id'];
        $merchant_resource = $this->ids['merchant_resource'];
        $alphanumeric = $merchant_resource->alphanumeric_id;
        $request = new Request();
        $request->url = "/messagemanager/foundry/getnextmessagebymerchantid?reqtype=02&siteid=$alphanumeric&reqfmt=01";
        $_GET = true;
        $_SERVER['QUERY_STRING'] = "reqtype=02&siteid=$alphanumeric&reqfmt=01";
        $request->_parseRequestBody();
        $message_controller = ControllerFactory::generateFromUrl($request->url, $mimetypes, $user, $request, 5);

        $message_response_resource = $message_controller->pullNextMessageResourceByMerchant($alphanumeric);
        myerror_log("pulled message is: ".$message_response_resource->message_text);
        $clean_payload = cleanUpDoubleSpacesCRLFTFromString($message_response_resource->message_text);
        $this->assertEquals($expected_payload,$clean_payload);
        //$this->assertContains('', substr($message_resource->message_text));
        $message_controller->markMessageDelivered();

        $message = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($message_response_resource->order_id,'U');
        $this->assertEquals($message->message_text,$clean_payload);

        // make sure order is not marked as 'E' yet
        $order_record = CompleteOrder::getBaseOrderData($message->order_id,$m);
        $this->assertEquals('O',$order_record['status'],"Status shoudl still be open till the reply comes in");
        return $message->order_id;
    }

    /**
     * @depends testPullMessageResource
     */
    function testFoundryCallBack($order_id)
    {
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
        $alphanumeric_id = $complete_order['merchant']['alphanumeric_id'];
        $call_back_payload = cleanUpDoubleSpacesCRLFTFromString('<POSRESPONSE><ERRORS COUNT="0" /><DEBUGS COUNT="0" /><LOGS COUNT="0" /><CHECKRESPONSES>
        <ADDCHECK SYSTEMERRORS="0" ITEMERRORS="0" TENDERERRORS="0" EXTCHECKID="Wyatt1'.$order_id.'" ORDERID="'.$order_id.'" INTCHECKID="110017" POSCHECKSUBTOTAL="'.$complete_order['order_amt'].'" POSCHECKTAX="'.$complete_order['total_tax'].'" POSCHECKTOTAL="'.$complete_order['grand_total'].'" DONOTRESEND="TRUE">
			<LOGS COUNT="0" />
			<DEBUGS COUNT="2">
				<DEBUG TEXT="D150, Enhanced External Check ID: MYORDER-123" />
				<DEBUG TEXT="D150, MicrosTransaction.PostTransaction ChkNum=1924, ChkId=MYORDER-123, SubTtl=14.95, Tax=1.30, TtlDue=16.25" />
			</DEBUGS><ERRORS COUNT="0" />
		</ADDCHECK>
	</CHECKRESPONSES>
	<PRINTRESPONSES />
	<ERRORS COUNT="0" />
	<DEBUGS COUNT="2">
		<DEBUG TEXT="D150, Enhanced External Check ID: MYORDER-123" />
		<DEBUG TEXT="D150, MicrosTransaction.PostTransaction ChkNum=1924, ChkId=MYORDER-123, SubTtl=14.95, Tax=1.30, TtlDue=16.25" />
	</DEBUGS>
	<LOGS COUNT="0" />
</POSRESPONSE>');

        $url = 'http://localhost/app2/messagemanager/foundry/getnextmessagebymerchantid?reqtype=04&siteid='.$alphanumeric_id.'&reqfmt=01';
        $request = createRequestObject($url,'POST',$call_back_payload,'application/xml');
        $message_controller = ControllerFactory::generateFromUrl($request->url, $mimetypes, $user, $request, 5);
        $message_controller->pullNextMessageResourceByMerchant($alphanumeric_id);
        $message = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'U');
        $this->assertEquals($call_back_payload,$message->response,"It should have saved the response on the message object");
        $this->assertEquals('V',$message->viewed,"It should have marked the message as viewed");

        // make sure order is now marked as 'E'
        $order_record = CompleteOrder::getBaseOrderData($message->order_id,$m);
        $this->assertEquals('E',$order_record['status'],"Status should have been marked as 'E'");

    }

    /**
     * @depends testPullMessageResource
     */
    function testFoundryCallBackFailure($order_id)
    {
        //$sql = "UPDATE Orders SET status = 'O' WHERE order_id = $order_id";
        //$order_adapter = new OrderAdapter($m);
        $this->assertTrue(OrderAdapter::updateOrderStatus('O',$order_id));
        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'U');
        $message_resource->viewed = 'N';
        $message_resource->save();

        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
        $alphanumeric = $complete_order['merchant']['alphanumeric_id'];
        $this->assertEquals('O',$complete_order['status']);
        $call_back_payload = cleanUpDoubleSpacesCRLFTFromString('<?xml version="1.0" encoding="UTF-8"?>
<POSRESPONSE>
   <ERRORS COUNT="1">
      <ERROR TEXT="MicrosTransaction.PostTransaction Try: System.Web.Services.Protocols.SoapException: [-956235746] This item is not a combo meal main item, it was not found in the main item group &#xA;   at ResPosApiWeb.ResPosApiWebService.PostTransactionEx(ResPosAPI_GuestCheck&amp; pGuestCheck, ResPosAPI_MenuItem[]&amp; ppMenuItems, ResPosAPI_ComboMeal[]&amp; ppComboMeals, ResPosAPI_SvcCharge&amp; pServiceChg, ResPosAPI_Discount&amp; pSubTotalDiscount, ResPosAPI_TmedDetailItemEx&amp; pTmedDetail, ResPosAPI_TotalsResponse&amp; pTotalsResponse, String[]&amp; ppCheckPrintLines, String[]&amp; ppVoucherOutput)" ERRNUM="9" ERRORDATETIME="3/30/2016 18:33:39" />
   </ERRORS>
   <DEBUGS COUNT="13">
      <DEBUG TEXT="D150, Enhanced External Check ID: SP-12200688-318" />
      <DEBUG TEXT="D150, MicrosTransaction.IsComboMeal: ObjNum=32, GrpSeq=32, MainItem=8011001" />
      <DEBUG TEXT="D50, Compose Sides for Combo Meal: 8011002" />
      <DEBUG TEXT="D50, Adding Combo meal side item: 8026311" />
      <DEBUG TEXT="D50, Adding Combo meal side item: 8026002" />
      <DEBUG TEXT="D50, Adding Combo meal side item: 9100022" />
      <DEBUG TEXT="D50, Adding Combo meal side item: 8026005" />
      <DEBUG TEXT="D50, Adding Combo meal side item: 9100006" />
      <DEBUG TEXT="D50, Adding Combo meal side item: 8026007" />
      <DEBUG TEXT="D50, Adding Combo meal side item: 8005047" />
      <DEBUG TEXT="D50, Adding Combo meal side item: 8018002" />
      <DEBUG TEXT="D150, FullName: Test Do Not Make" />
      <DEBUG TEXT="D150, Email: store_tester@order140.com" />
   </DEBUGS>
   <LOGS COUNT="0" />
   <CHECKRESPONSES>
      <ADDCHECK ORDERID="'.$order_id.'" EXTCHECKID="SP-'.$order_id.'-318" ITEMERRORS="0" TENDERERRORS="0" SYSTEMERRORS="1">
         <LOGS COUNT="0" />
         <DEBUGS COUNT="13">
            <DEBUG TEXT="D150, Enhanced External Check ID: SP-12200688-318" />
            <DEBUG TEXT="D150, MicrosTransaction.IsComboMeal: ObjNum=32, GrpSeq=32, MainItem=8011001" />
            <DEBUG TEXT="D50, Compose Sides for Combo Meal: 8011002" />
            <DEBUG TEXT="D50, Adding Combo meal side item: 8026311" />
            <DEBUG TEXT="D50, Adding Combo meal side item: 8026002" />
            <DEBUG TEXT="D50, Adding Combo meal side item: 9100022" />
            <DEBUG TEXT="D50, Adding Combo meal side item: 8026005" />
            <DEBUG TEXT="D50, Adding Combo meal side item: 9100006" />
            <DEBUG TEXT="D50, Adding Combo meal side item: 8026007" />
            <DEBUG TEXT="D50, Adding Combo meal side item: 8005047" />
            <DEBUG TEXT="D50, Adding Combo meal side item: 8018002" />
            <DEBUG TEXT="D150, FullName: Test Do Not Make" />
            <DEBUG TEXT="D150, Email: store_tester@order140.com" />
         </DEBUGS>
         <ERRORS COUNT="1">
            <ERROR TEXT="MicrosTransaction.PostTransaction Try: System.Web.Services.Protocols.SoapException: [-956235746] This item is not a combo meal main item, it was not found in the main item group &#xA;   at ResPosApiWeb.ResPosApiWebService.PostTransactionEx(ResPosAPI_GuestCheck&amp; pGuestCheck, ResPosAPI_MenuItem[]&amp; ppMenuItems, ResPosAPI_ComboMeal[]&amp; ppComboMeals, ResPosAPI_SvcCharge&amp; pServiceChg, ResPosAPI_Discount&amp; pSubTotalDiscount, ResPosAPI_TmedDetailItemEx&amp; pTmedDetail, ResPosAPI_TotalsResponse&amp; pTotalsResponse, String[]&amp; ppCheckPrintLines, String[]&amp; ppVoucherOutput)" ERRNUM="9" ERRORDATETIME="3/30/2016 18:33:39" />
         </ERRORS>
      </ADDCHECK>
   </CHECKRESPONSES>
   <PRINTRESPONSES />
</POSRESPONSE>');

        $url = "http://localhost/app2/messagemanager/foundry/getnextmessagebymerchantid?reqtype=04&siteid=$alphanumeric&reqfmt=01";
        $request = createRequestObject($url,"POST",$call_back_payload,'application/xml');
        $message_controller = ControllerFactory::generateFromUrl($request->url, getM(),null, $request, 5);
        $message_controller->pullNextMessageResourceByMerchant($alphanumeric);
        $message = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'U');
        $this->assertEquals($call_back_payload,$message->response,"It should have saved the response on the message object");
        $this->assertEquals('F',$message->viewed,"It should have marked the message viewed as FAILED");

        // make sure order stayed in the open state 'O'
        $order_record = CompleteOrder::getBaseOrderData($message->order_id,$m);
        $this->assertEquals('O',$order_record['status'],"Status should still be open 'O'");

    }

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time',300);
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction();

        $skin_resource = getOrCreateSkinAndBrandIfNecessary('foundryskin','foundrybrand',null,null);
        setContext($skin_resource->external_identifier);
//        getOrCreateSkinAndBrandIfNecessary('laparrilla','laparrilla',203,484);
//        setContext('com.splickit.laparrilla');
        $ids['skin_id'] = getSkinIdForContext();
        $ids['context'] = $skin_resource->external_identifier;

        //map it to a menu
        $menu_id = createTestMenuWithNnumberOfItems(3);
        $ids['menu_id'] = $menu_id;

        $modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 1);
        //$modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 2);
//        $modifier_group_resource2 = createModifierGroupWithNnumberOfItems($menu_id, 1);
//        $modifier_group_resource3 = createModifierGroupWithNnumberOfItems($menu_id, 1);
        $modifier_size_map = $modifier_group_resource->modifier_items[0]->modifier_size_map;
        $modifier_size_map->external_id = '12345:'.$modifier_size_map->external_id;
        //$modifier_size_map->external_id = '98765:2-12345:'.$modifier_size_map->external_id;
        $modifier_size_map->save();

        $item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_resource->modifier_group_id, 0);
//        assignModifierGroupToItemWithFirstNAsComesWith($item_records[1]['item_id'], $modifier_group_resource2->modifier_group_id, 0);
//        assignModifierGroupToItemWithFirstNAsComesWith($item_records[2]['item_id'], $modifier_group_resource3->modifier_group_id, 0);

        #create the test merchant
        $merchant_resource = createNewTestMerchant($menu_id,["authorize"=>true]);
        attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
        $ids['merchant_id'] = $merchant_resource->merchant_id;
        $ids['merchant_resource'] = $merchant_resource;
//        $resource = Resource::createByData(new MerchantFoundryInfoMapsAdapter(getM()),array("merchant_id"=>$merchant_resource->merchant_id,"promo_id"=>12345));

        #create the Merchant_Message_Map resource for the foundty group. Lets go with 'U'
        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'U','Foundry','X');

        #create the foundry brand tender_ids table
        $brand_id = getBrandIdFromCurrentContext();
//        $sql = "INSERT INTO Foundry_Brand_Card_Tender_Ids (`brand_id`,`visa`,`master`,`american_express`,`discover`) VALUES ($brand_id,215,216,217,218)";
//        $fbctia = new FoundryBrandCardTenderIdsAdapter(getM());
//        $fbctia->_query($sql);
        $user_resource = createNewUser(array("flags"=>"1C20000001"));
        $ids['user_id'] = $user_resource->user_id;


        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $foundry_third_party_parameters = $tpppa->getRecords(['third_party_provider_id' => 1005]);
        foreach ($foundry_third_party_parameters as $foundry_parameter) {
            $name = $foundry_parameter['name'];
            if ($name == 'modecode') {
                $value = '1';
            } else if ($name == 'modecode_holds') {
                $value = '2';
            } else if ($name == 'promo_id') {
                $value = '54321';
            } else if ($name == 'tender_id_visa') {
                $value = '215';
            } else if ($name == 'tender_id_mastercard') {
                $value = '216';
            } else if ($name == 'tender_id_american_express') {
                $value = '217';
            } else if ($name == 'tender_id_discover') {
                $value = '218';
            } else if ($name == 'brand_order_mode_delivery') {
                $value = '3';
            } else if ($name == 'brand_order_mode_pickup') {
                $value = '3';
            } else if ($name == 'get_menu') {
                $value = '0';
                $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()), ["merchant_id" => $merchant_resource->merchant_id, "third_party_provider_parameter_id" => $foundry_parameter['id'], "value" => $value]);
                continue;
            }
            $tpppmap_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()), ["brand_id" => $brand_id, "third_party_provider_parameter_id" => $foundry_parameter['id'], "value" => $value]);
        }


        // create a promo
        //create the type 1 promo
        $promo_data = [];
        $key_word = "FoundryPromo";
        $promo_data['key_word'] = "$key_word";
        $promo_data['promo_type'] = 1;
        $promo_data['description'] = 'Get $ off';
        $promo_data['start_date'] = '2010-01-01';
        $promo_data['end_date'] = '2030-01-01';
        $promo_data['max_use'] = 100;
        $promo_data['allow_multiple_use_per_order'] = false;
        $promo_data['valid_on_first_order_only'] = 'N';
        $promo_data['order_type'] = 'pickup';
        $promo_data['merchant_id'] = 0;
        $promo_data['qualifying_amt'] = 1.00;
        $promo_data['promo_amt'] = 1.00;
        $promo_data['percent_off'] = 10;
        $promo_data['max_amt_off'] = 50.00;
        $promo_data['brand_id'] = getBrandIdFromCurrentContext();
        $request = createRequestObject("/app2/admin/promo",'POST',json_encode($promo_data));
        $promo_controller = new PromoController(getM(),null,$request,5);
        $response = $promo_controller->createPromo();
        $ids['promo_key_word'] = $key_word;



        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    FoundryControllerTest::main();
}

?>