<?php
ini_set('max_execution_time', 300);
$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class GroupOrderIndividualPayTestDoorDash extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $user;
    var $merchant_id;
    var $merchant;
    var $users;
    var $ids;

    function setUp()
    {
        // we dont want to call to inspirepay
        setContext("com.splickit.gotwodd");
        $_SERVER['SKIN']['show_notes_fields'] = true;
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $_SERVER['HTTP_X_SPLICKIT_CLIENT_VERSION'] = '100.0.0';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__ . '-' . $_SERVER['STAMP'];
        $this->users = $_SERVER['users'];
        $this->ids = $_SERVER['unit_test_ids'];
        $user_id = $_SERVER['unit_test_ids']['user_id'];
        $user_resource = SplickitController::getResourceFromId($user_id, 'User');
        $this->user = $user_resource->getDataFieldsReally();

        $this->merchant_id = $_SERVER['unit_test_ids']['merchant_id'];
        $this->merchant = SplickitController::getResourceFromId($this->merchant_id, 'Merchant');
        setProperty('do_not_call_out_to_aws', 'true');
    }

    function tearDown()
    {
        $_SERVER['STAMP'] = $this->stamp;
        setProperty('do_not_call_out_to_aws', 'true');
    }


    function testCreateGroupOrderWithDeliveryLocation()
    {
        // delivery minimum is set to 20

        $user_resource = createNewUserWithCCNoCVV();
        $user_resource->first_name = 'Tom';
        $user_resource->last_name = 'Zombie';
        $user_resource->save();
        $user = logTestUserResourceIn($user_resource);

        $json = '{"user_id":"' . $user['user_id'] . '","name":"","address1":"1045 Pine Street","address2":"","city":"Boulder","state":"CO","zip":"80302","phone_no":"1234567890","lat":40.019785,"lng":-105.282509}';
        $request = new Request();
        $request->body = $json;
        $request->mimetype = "Application/json";
        $request->_parseRequestBody();
        $request->method = 'POST';
        $request->url = "/users/" . $user['uuid'] . "/userdeliverylocation";
        $user_controller = new UserController($mt, $user, $request, 5);
        $response = $user_controller->processV2Request();
        $this->assertNull($response->error, "should not have gotten a delivery save error but did");
        $this->assertNotNull($response->user_addr_id);
        $user_address_id = $response->user_addr_id;

        $request = new Request();
        $request->data = array("merchant_id" => $this->merchant_id, "note" => 'sumdum', "participant_emails" => '', "user_addr_id" => $user_address_id, "group_order_type" => 2, "submit_at_ts" => (getTomorrowTwelveNoonTimeStampDenver() + 900));
        $request->url = "app2/apiv2/grouporders";
        $request->method = "POST";
        $group_order_controller = new GroupOrderController($mt, $user, $request, 5);
        $resource = $group_order_controller->processV2Request();
        $group_order_token = $resource->group_order_token;
        $cart = CartsAdapter::staticGetRecordByPrimaryKey($group_order_token, "CartsAdapter");
        $order_id = $cart['order_id'];
        // check to make sure cart has mdpd_id on record
        $this->assertEquals($this->ids['mdpd_id'],$cart['merchant_delivery_price_distance_id']);
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id, $m);
        $this->assertEquals('D', $complete_order['order_type'], "shoudl have created a delivery dummy order row");
        $this->assertEquals($user_address_id, $complete_order['user_delivery_location_id'], "Should have the user addr id on the order");
        $this->assertEquals(8.00, $complete_order['delivery_amt'], "should have the default Doordash price");
        $this->assertEquals(getSkinIdForContext(),$complete_order['skin_id'],"skin id shoudl equal the context");
        //$this->assertEquals(0.00, $complete_order['delivery_amt'], "should NOT have a delivery price on the dummy order");
        $group_order_adapter = new GroupOrderAdapter(getM());
        $group_order_record = $group_order_adapter->getRecordFromPrimaryKey($resource->group_order_id);
        $this->assertEquals($user_address_id,$group_order_record['user_addr_id'],'User address id should be on the group order record');
        $group_order_record['user_address_id'] = $user_address_id;
        //$group_order_record['delivery_cost'] = $complete_order['delivery_amt'];
        return $group_order_record;
    }

    /**
     * @depends testCreateGroupOrderWithDeliveryLocation
     */
    function testAddToDeliveryGroupOrderAsParticipantSoShouldNOTGetChargedDeliveryFee($group_order_record)
    {
        $group_order_token = $group_order_record['group_order_token'];
        $group_order_id = $group_order_record['group_order_id'];

        $parent_order_record_before = OrderAdapter::staticGetRecord(array("ucid"=>$group_order_token),'OrderAdapter');

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $order_data = OrderAdapter::getSimpleCartArrayByMerchantId($this->ids['merchant_id'],'Delivery',"sum dum participant cart note",4);
        $order_data['group_order_token'] = $group_order_token;
        $order_data["submitted_order_type"]= 'Delivery';
        $order_data['user_addr_id'] = null;

        $cart_resource = $this->addtoCart($user,$order_data);
        $this->assertNotNull($cart_resource, "should have gotten a cart resource back");
        $this->assertNull($cart_resource->error);
        $this->assertEquals("Y",$cart_resource->status,"It shoudl have a status of 'Y'");

        // validate that the order record was added to mapping table
        $cart_order_resource = Resource::find(new CartsAdapter($m),$cart_resource->ucid);
        $gorma = new GroupOrderIndividualOrderMapsAdapter($m);
        $record = $gorma->getRecord(array("user_order_id"=>$cart_order_resource->order_id));
        $this->assertEquals($group_order_id,$record['group_order_id'],"It should have added the order to the mapping table.");

        // validate that the delivery charge was NOT added to the order record since this is jsut a participant
        $this->assertEquals($group_order_record['user_address_id'],$cart_order_resource->user_delivery_location_id,"It should have the user address id of the parent order");
        $this->assertEquals(0.00,$cart_order_resource->delivery_amt,"It should NOT have the delivery charge on the order record since this is a participant and not the admin");

        // get checkout data to make sure there is only 1 time choice
        $checkout_data_resource = $this->getCheckoutForGroupOrder($user,$cart_resource->ucid,$group_order_token,getTomorrowTwelveNoonTimeStampDenver()-1800);
        $this->assertNull($checkout_data_resource->error);
        $this->assertCount(1,$checkout_data_resource->lead_times_array,"It should only have 1 available lead time");
        $this->assertEquals(0.00,$checkout_data_resource->delivery_amt,"There should not be a delivery fee");
        //$this->assertfalse($checkout_data_resource->show_lead_times,"It should have the flag set to to not show lead time");
        $this->assertEquals(date('Y-m-d H:i:s',getTomorrowTwelveNoonTimeStampDenver() + 3600),date('Y-m-d H:i:s',$checkout_data_resource->lead_times_array[0]));

        // now place the order
        $order_resource = $this->placeOrderToBePartOfTypeTwoGroupOrder($user,$this->ids['merchant_id'],$cart_resource->ucid,$group_order_token,$checkout_data_resource,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $this->assertEquals($cart_order_resource->order_id, $order_id, "should have created a valid order id on teh original cart record");
        $this->assertEquals("D",$order_resource->order_type,"It should be listed as a delivery order");
        $this->assertEquals(0.00,$order_resource->delivery_amt,"There should not be a delivery fee");
        $this->assertEquals($group_order_record['user_addr_id'],$order_resource->user_delivery_location_id,"It should have the UDL of the parent order");

        $complete_order = CompleteOrder::getBaseOrderData($order_id,getM());

        $order_messages = MerchantMessageHistoryAdapter::getAllOrderMessages($order_id);
        $this->assertCount(1, $order_messages);
        $order_message = $order_messages[0];
        $this->assertEquals('Econf', $order_message->message_format);
        $this->assertEquals('G', $order_resource->status);

        $gorma = new GroupOrderIndividualOrderMapsAdapter($m);
        $record = $gorma->getRecord(array("user_order_id"=>$order_id));
        $this->assertEquals($group_order_id,$record['group_order_id']);

        $parent_order_record_after = OrderAdapter::staticGetRecord(array("ucid"=>$group_order_token),'OrderAdapter');
        $this->assertEquals($parent_order_record_after['order_amt'],$parent_order_record_before['order_amt']+$complete_order['order_amt']);
        $this->assertEquals($parent_order_record_after['total_tax_amt'],$parent_order_record_before['total_tax_amt']+$complete_order['total_tax_amt']);
        $this->assertEquals($parent_order_record_after['order_qty'],$parent_order_record_before['order_qty']+$complete_order['order_qty']);
        $this->assertEquals($parent_order_record_after['tip_amt'],$parent_order_record_before['tip_amt']+$complete_order['tip_amt']);

        return $group_order_record;
    }

    /**
     * @depends testCreateGroupOrderWithDeliveryLocation
     */
    function testAddToDeliveryGroupOrderAsSecondParticipant($group_order_record)
    {
        $group_order_token = $group_order_record['group_order_token'];
        $group_order_id = $group_order_record['group_order_id'];

        $parent_order_record_before = OrderAdapter::staticGetRecord(array("ucid"=>$group_order_token),'OrderAdapter');

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $order_data = OrderAdapter::getSimpleCartArrayByMerchantId($this->ids['merchant_id'],'Delivery',"sum dum participant cart note",3);
        $order_data['group_order_token'] = $group_order_token;
        $order_data["submitted_order_type"]= 'Delivery';
        $order_data['user_addr_id'] = null;

        $cart_resource = $this->addtoCart($user,$order_data);
        $this->assertNotNull($cart_resource, "should have gotten a cart resource back");
        $this->assertNull($cart_resource->error);
        $checkout_data_resource = $this->getCheckoutForGroupOrder($user,$cart_resource->ucid,$group_order_token,getTomorrowTwelveNoonTimeStampDenver()-1800);
        $this->assertNull($checkout_data_resource->error);
        $this->assertCount(1,$checkout_data_resource->lead_times_array,"It should only have 1 available lead time");
        $this->assertEquals(0.00,$checkout_data_resource->delivery_amt,"There should not be a delivery fee");
        //$this->assertfalse($checkout_data_resource->show_lead_times,"It should have the flag set to to not show lead time");
        $this->assertEquals(date('Y-m-d H:i:s',getTomorrowTwelveNoonTimeStampDenver() + 3600),date('Y-m-d H:i:s',$checkout_data_resource->lead_times_array[0]));

        // now place the order
        $order_resource = $this->placeOrderToBePartOfTypeTwoGroupOrder($user,$this->ids['merchant_id'],$cart_resource->ucid,$group_order_token,$checkout_data_resource,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $this->assertEquals("D",$order_resource->order_type,"It should be listed as a delivery order");
        $this->assertEquals(0.00,$order_resource->delivery_amt,"There should not be a delivery fee");
        $this->assertEquals($group_order_record['user_addr_id'],$order_resource->user_delivery_location_id,"It should have the UDL of the parent order");

        $complete_order = CompleteOrder::getBaseOrderData($order_id,getM());

        $order_messages = MerchantMessageHistoryAdapter::getAllOrderMessages($order_id);
        $this->assertCount(1, $order_messages);
        $order_message = $order_messages[0];
        $this->assertEquals('Econf', $order_message->message_format);
        $this->assertEquals('G', $order_resource->status);

        $gorma = new GroupOrderIndividualOrderMapsAdapter($m);
        $record = $gorma->getRecord(array("user_order_id"=>$order_id));
        $this->assertEquals($group_order_id,$record['group_order_id']);

        $parent_order_record_after = OrderAdapter::staticGetRecord(array("ucid"=>$group_order_token),'OrderAdapter');

        $this->assertEquals($parent_order_record_after['order_amt'],$parent_order_record_before['order_amt']+$complete_order['order_amt']);
        $this->assertEquals($parent_order_record_after['total_tax_amt'],$parent_order_record_before['total_tax_amt']+$complete_order['total_tax_amt']);
        $this->assertEquals($parent_order_record_after['order_qty'],$parent_order_record_before['order_qty']+$complete_order['order_qty']);
        $this->assertEquals($parent_order_record_after['tip_amt'],$parent_order_record_before['tip_amt']+$complete_order['tip_amt']);

        return $group_order_record;

    }

    /**
     * @depends testAddToDeliveryGroupOrderAsSecondParticipant
     */
    function testAddToDeliveryGroupOrderAsAdminSoShouldGetChargedDeliveryFee($group_order_record)
    {
        $group_order_token = $group_order_record['group_order_token'];
        $group_order_id = $group_order_record['group_order_id'];

        $parent_order_record_before = OrderAdapter::staticGetRecord(array("ucid"=>$group_order_token),'OrderAdapter');
        $this->assertEquals(8.00, $parent_order_record_before['delivery_amt'],'Should still have the default delivery amount since the admin hasnt submitted yet');

        $user = logTestUserIn($group_order_record['admin_user_id']);

        $order_data = OrderAdapter::getSimpleCartArrayByMerchantId($this->ids['merchant_id'],'Delivery',"sum dum cart note",3);
        $order_data['group_order_token'] = $group_order_token;

        $cart_resource = $this->addtoCart($user,$order_data);
        $this->assertNotNull($cart_resource, "should have gotten a cart resource back");
        $this->assertNull($cart_resource->error);
        $this->assertEquals("Y",$cart_resource->status,"It shoudl have a status of 'Y'");


        // validate that the order record was added to mapping table
        $cart_order_resource = Resource::find(new CartsAdapter($m),$cart_resource->ucid);
        $gorma = new GroupOrderIndividualOrderMapsAdapter($m);
        $record = $gorma->getRecord(array("user_order_id"=>$cart_order_resource->order_id));
        $this->assertEquals($group_order_id,$record['group_order_id'],"It should have added the order to the mapping table.");

        // validate that the delivery charge was added to the order record since this is the admin
        $this->assertEquals($group_order_record['user_address_id'],$cart_order_resource->user_delivery_location_id,"It should have the user address id of the parent order");


        // get checkout data to make sure there is only 1 time choice
        $checkout_data_resource = $this->getCheckoutForGroupOrder($user,$cart_resource->ucid,$group_order_token,getTomorrowTwelveNoonTimeStampDenver()-1800);
        $this->assertNull($checkout_data_resource->error);
        $expected_message = "Also Note: Since you are the admin for this group order, once you submit your order, the group order time may no longer be changed, so please plan accordingly.";
        $this->assertContains($expected_message,$checkout_data_resource->user_message);
        $this->assertCount(1,$checkout_data_resource->lead_times_array,"It should only have 1 available lead time");
        $expected_delivery_amt = .2 * ($checkout_data_resource->order_amt+$parent_order_record_before['order_amt']) + 5.00;
        $this->assertEquals($expected_delivery_amt,$checkout_data_resource->delivery_amt,"It should be a delivery amount from the combined amounts of the submitted orders");

        //$this->assertfalse($checkout_data_resource->show_lead_times,"It should have the flag set to to not show lead time");
        $this->assertEquals(date('Y-m-d H:i:s',getTomorrowTwelveNoonTimeStampDenver() + 3600),date('Y-m-d H:i:s',$checkout_data_resource->lead_times_array[0]));
        $this->assertTrue(count($checkout_data_resource->accepted_payment_types) == 1,"We should have some accepted payment types");
        $payment_array = createHashmapFromArrayOfArraysByFieldName($checkout_data_resource->accepted_payment_types,'name');
        $this->assertNull($payment_array['Cash'],"There should not be a cash option for the payment array");


        // now place the order
        $order_resource = $this->placeOrderToBePartOfTypeTwoGroupOrder($user,$this->ids['merchant_id'],$cart_resource->ucid,$group_order_token,$checkout_data_resource,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id, $m);
        $this->assertEquals($cart_order_resource->order_id, $order_id, "should have created a valid order id on teh original cart record");

        $order_messages = MerchantMessageHistoryAdapter::getAllOrderMessages($order_id);
        $this->assertCount(1, $order_messages);
        $order_message = $order_messages[0];
        $this->assertEquals('Econf', $order_message->message_format);
        $this->assertEquals('G', $order_resource->status);

        $gorma = new GroupOrderIndividualOrderMapsAdapter($m);
        $record = $gorma->getRecord(array("user_order_id"=>$order_id));
        $this->assertEquals($group_order_id,$record['group_order_id']);

        //test to see if values get added to the parent order record
        $parent_order_record_after = OrderAdapter::staticGetRecord(array("ucid"=>$group_order_token),'OrderAdapter');

        $this->assertEquals($parent_order_record_after['order_amt'],$parent_order_record_before['order_amt']+$complete_order['order_amt']);
        $this->assertEquals($parent_order_record_after['total_tax_amt'],$parent_order_record_before['total_tax_amt']+$complete_order['total_tax_amt']);
        $this->assertEquals($parent_order_record_after['order_qty'],$parent_order_record_before['order_qty']+$complete_order['order_qty']);
        $this->assertEquals($parent_order_record_after['tip_amt'],$parent_order_record_before['tip_amt']+$complete_order['tip_amt']);

        $expected_delivery_amt = .2 * ($parent_order_record_after['order_amt']) + 5.00;


        $this->assertEquals($expected_delivery_amt,$order_resource->delivery_amt,"It should have the delivery charge on the order record and it shoudl be based on the complete group order so far.");
        $this->assertEquals($expected_delivery_amt,$parent_order_record_after['delivery_amt'],"The delivery charge should be the same on the parent order");
        return $group_order_record;
    }


    /**
     * @depends testAddToDeliveryGroupOrderAsAdminSoShouldGetChargedDeliveryFee
     */
    function testDoNotAllowChangeOfGroupOrderTimeDueToThirdPartyDeliveryServiceSinceAdminHasAlreadySubmitted($group_order_record)
    {
        //$this->assertTrue(false,"Build it");
        return $group_order_record;
    }


    /**
     * @depends testDoNotAllowChangeOfGroupOrderTimeDueToThirdPartyDeliveryServiceSinceAdminHasAlreadySubmitted
     */
    function testSubmitDeliveryGroupOrder($group_order_record)
    {
        $group_order_token = $group_order_record['group_order_token'];
        $group_order_id = $group_order_record['group_order_id'];
        $user = logTestUserIn($group_order_record['admin_user_id']);

        $request = createRequestObject("/app2/apiv2/grouporders/$group_order_token/submit",'POST',$body,$m);
        $group_order_controller = new GroupOrderController($m,$user,$request,5);
        $response = $group_order_controller->processV2request();
        $this->assertNull($response->error);

        $group_order_token = $group_order_record['group_order_token'];
        $group_base_order = CompleteOrder::staticGetCompleteOrder($group_order_token);

        // check to make sure the messages were created
        $order_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($group_base_order['order_id'],'E');
        $this->assertEquals($group_base_order['order_id'],$order_message_resource->order_id,"the order id of the message should equal that of the parent group order");

        // all status should have been set to 'O'
        $records = GroupOrderIndividualOrderMapsAdapter::staticGetRecords(array("group_order_id"=>$group_order_record['group_order_id']),'GroupOrderIndividualOrderMapsAdapter');
        $this->assertCount(3,$records,"There should be 3 orders");
        foreach($records as $record) {
            $base_order_info = CompleteOrder::getBaseOrderData($record['user_order_id']);
            $this->assertEquals('O',$base_order_info['status']);
        }

        $final_group_order_record = getStaticRecord(array("group_order_id"=>$group_order_record['group_order_id']),'GroupOrderAdapter');
        $this->assertEquals('Submitted',$final_group_order_record['status']);
        $e = explode(' ',$final_group_order_record['sent_ts']);
        $sent_date = $e[0];
        $this->assertEquals(date('Y-m-d'),$sent_date,"shoudl have today as sent");
        $mmha = new MerchantMessageHistoryAdapter(getM());
        $message_resource = $mmha->getExactResourceFromData(array("order_id"=>$group_base_order['order_id'],"message_format"=>'Econf'));
        $message_text = $message_resource->message_text;
        $this->assertTrue(substr_count(strip_tags($message_text), 'Tom, your order will be') == 1,"should have name of group order admin");

        //check to make sure requested_delivery_time propogates to parent order
        $sql = "SELECT a.* from Orders a JOIN Group_Order_Individual_Order_Maps b ON a.order_id = b.user_order_id WHERE group_order_id = $group_order_id";
        $options[TONIC_FIND_BY_SQL] = $sql;
        $order_resources = Resource::findAll(new OrderAdapter(getM()),null,$options);
        $requested_delivery_time = $order_resources[0]->requested_delivery_time;
        $this->assertNotNull($requested_delivery_time,"It should have a requested delivery time");
        $this->assertEquals($requested_delivery_time,$group_base_order['requested_delivery_time'],"It should have picked up the requested_delivery_time from the child order");

        return $group_order_record;
    }


    /**
     * @depends testSubmitDeliveryGroupOrder
     */
    function testSendGroupOrderExecutionMessage($group_order_record)
    {
        $order_id = $group_order_record['order_id'];
        $order_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'E');
        $message_controller = ControllerFactory::generateFromMessageResource($order_message_resource);
        $response = $message_controller->sendThisMessage($order_message_resource);

        // validate message
        $sent_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'E');
        $this->assertEquals('S',$sent_message_resource->locked,"It should have listed the messages as sent");
        $body = $sent_message_resource->message_text;
        myerror_log($body);
        $this->assertContains('1045 Pine Street',$body,"It should have the delivery addres on the message somewhere");

        // now check to see if the individual orders were set to 'E'
        $records = GroupOrderIndividualOrderMapsAdapter::staticGetRecords(array("group_order_id"=>$group_order_record['group_order_id']),'GroupOrderIndividualOrderMapsAdapter');
        foreach ($records as $record) {
            $base_order_info = CompleteOrder::getBaseOrderData($record['user_order_id']);
            $this->assertEquals('E',$base_order_info['status']);
        }
        // ---Doordash Delivery Id:149637531
        // check on parent order
        $order = getStaticRecord(array("order_id"=>$order_id),'OrderAdapter');
        $this->assertEquals('G',$order['status'],"The parent order should stay as 'G' so it doesn't get included in accounting");
        $this->assertContains('---Doordash Delivery Id:',$order['note'],"It should have the doordash info");
        $this->assertContains($order['note'],$body,"The execution message should contain the doordash information");
        return $group_order_record['admin_user_id'];
    }


    /********************  helper functions **************************/

    function addtoCart($user,$order_data)
    {
        $json_encoded_data = json_encode($order_data);
        $url = isset($order_data['ucid']) ? "/app2/apiv2/cart".$order_data['ucid'] : "/app2/apiv2/cart";
        $request = createRequestObject($url,"POST",$json_encoded_data,'application/json');
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        $cart_resource = $place_order_controller->processV2Request();
        return $cart_resource;
    }

    function getCheckoutForGroupOrder($user,$ucid,$group_order_token,$time)
    {
        $request = new Request();
        $request->url = "http://localhost/app2/apiv2/cart/$ucid/checkout";
        $request->method = "get";
        $request->mimetype = 'application/json';
        $place_order_controller = new PlaceOrderController($mt, $user, $request,5);
        $place_order_controller->setCurrentTime($time);
        $checkout_data_resource = $place_order_controller->processV2Request();
        return $checkout_data_resource;
    }

    function placeOrderToBePartOfTypeTwoGroupOrder($user,$merchant_id,$ucid,$group_order_token,$checkout_data_resource,$time)
    {
        $order_data = array();
        $order_data['merchant_id'] = $merchant_id;
        $order_data['user_id'] = $user['user_id'];
        $order_data['cart_ucid'] = $ucid;
        $order_data['tip'] = (rand(100, 1000)) / 100;
        $payment_array = $checkout_data_resource->accepted_payment_types;
        $order_data['merchant_payment_type_map_id'] = $payment_array[0]['merchant_payment_type_map_id'];
        $lead_times_array = $checkout_data_resource->lead_times_array;
        $order_data['actual_pickup_time'] = $lead_times_array[0];
        $order_data['requested_time'] = $lead_times_array[0];

        //$order_data['group_order_token'] = $group_order_token;
        $json_encoded_data = json_encode($order_data);
        $request = new Request();
        $request->url = "/apiv2/orders/$ucid";
        $request->method = "post";
        $request->body = $json_encoded_data;
        $request->mimetype = 'application/json';
        $request->_parseRequestBody();
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        $place_order_controller->setCurrentTime($time);
        $order_resource = $place_order_controller->processV2Request();
        return $order_resource;
    }

    function addToGroupOrderFullProcess($user,$merchant_id,$order_data,$group_order_token,$time)
    {
        $cart_resource = $this->addtoCart($user,$order_data);
        if ($cart_resource->error) {
            return $cart_resource;
        }
        $ucid = $cart_resource->ucid;

        $checkout_data_resource = $this->getCheckoutForGroupOrder($user,$ucid,$group_order_token,$time);
        if ($checkout_data_resource->error) {
            return $checkout_data_resource;
        }
        $order_resource = $this->placeOrderToBePartOfTypeTwoGroupOrder($user,$merchant_id,$ucid,$group_order_token,$checkout_data_resource,$time);
        return $order_resource;
    }

    /********************  end helper functions **************************/

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        ini_set('max_execution_time', 300);

        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction(); ;
        getOrCreateSkinAndBrandIfNecessaryWithLoyalty("gotwodd","gotwodd",$skin_id,$brand_id);
        $skin_resource = setContext("com.splickit.gotwodd");

        $blr_data['brand_id'] = $skin_resource->brand_id;
        $blr_data['loyalty_type'] = 'splickit_earn';
        $blr_data['earn_value_amount_multiplier'] = 1;
        $blr_data['cliff_value'] = 10;
        $brand_loyalty_rules_resource = Resource::factory(new BrandLoyaltyRulesAdapter($m),$blr_data);
        $brand_loyalty_rules_resource->save();

        $_SERVER['request_time1'] = microtime(true);
        $menu_id = $menu_id = createTestMenuWithNnumberOfItems(3);
        $ids['menu_id'] = $menu_id;

        $modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 4);
        $modifier_group_id = $modifier_group_resource->modifier_group_id;
        $item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 2);
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[1]['item_id'], $modifier_group_id, 4);
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[2]['item_id'], $modifier_group_id, 1);

        $merchant_resource = createNewTestMerchantDelivery($menu_id);
        $data = array("merchant_id" => $merchant_resource->merchant_id);

        // now edit the distance price records.
        $mdpd = new MerchantDeliveryPriceDistanceAdapter(getM());
        $mdpd_resource = $mdpd->getExactResourceFromData($data);
        $mdpd_resource->name = 'Doordash';
        $mdpd_resource->delivery_price_type = 'doordash';
        $mdpd_resource->distance_up_to = 'nullit';
        $mdpd_resource->price = -1.00;
        $mdpd_resource->delivery_increment = 15;
        $mdpd_resource->max_days_out = 3;
        $mdpd_resource->minimum_order_amount = 20.00;
        $mdpd_resource->notes = "external_business_name=sumdumbiz";
        $mdpd_resource->save();





        $merchant_resource->group_ordering_on = 1;
        $merchant_resource->save();
        $merchant_id = $merchant_resource->merchant_id;
        $ids['merchant_id'] = $merchant_id;
        $ids['mdpd_id'] = $mdpd_resource->map_id;

        $map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_resource->merchant_id,"message_format"=>'FUA',"delivery_addr"=>"1234567890","message_type"=>"X"));

        $billing_entity_resource = createSageBillingEntity($merchant_resource->brand_id);

        $merchant_payment_type_map_adapter = new MerchantPaymentTypeMapsAdapter($mimetypes);
        $cc_merchant_payment_type_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_id, 2000, $billing_entity_resource->id);
        $ids['cc_billing_entity_id'] = $cc_merchant_payment_type_resource->billing_entity_id;

        // create cash merchang payment type record
        $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_id, 1000, $billing_entity_id);

        //loyalty paymeent types
        $merchant_payment_type_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 8000, $billing_entity_id);
        $merchant_payment_type_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 9000, $billing_entity_id);


//        $user_resource = createNewUser(array('flags' => '1C20000001', 'first_name' => 'adam', 'last_name' => 'zmopolis', 'last_four'=>'1234'));
        $user_resource = createNewUserWithCCNoCVV(array('first_name' => 'adam', 'last_name' => 'zmopolis'));
        $ids['user_id'] = $user_resource->user_id;
        $users[1] = $user_resource->user_id;
        $user_resource2 = createNewUserWithCCNoCVV(array('first_name' => 'rob', 'last_name' => 'zmopolis'));
        $user_resource3 = createNewUserWithCCNoCVV(array('first_name' => 'ty', 'last_name' => 'zmopolis'));
        $user_resource4 = createNewUserWithCCNoCVV(array('first_name' => 'ty', 'last_name' => 'zmopolis'));
//        $user_resource3 = createNewUser(array('flags' => '1C20000001', 'first_name' => 'ty', 'last_name' => 'zmopolis', 'last_four'=>'1234'));
//        $user_resource4 = createNewUser(array('flags' => '1C20000001', 'first_name' => 'jason', 'last_name' => 'zmopolis', 'last_four'=>'1234'));
        $users[2] = $user_resource2->user_id;
        $users[3] = $user_resource3->user_id;
        $users[4] = $user_resource4->user_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
        $_SERVER['users'] = $users;

    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
    }

    static function main()
    {
        $suite = new PHPUnit_Framework_TestSuite(__CLASS__);
        PHPUnit_TextUI_TestRunner::run($suite);
    }
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    GroupOrderIndividualPayTestDoorDash::main();
}

?>