<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class IvrControllerTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
		$this->ids = $_SERVER['unit_test_ids'];
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }

	function testCorrectResponse()
	{
		$message = '<?xml version="1.0" encoding="UTF-8"?>
            <Response>
                <Message>280743</Message>
                <Message>123456789</Message>
                <Play loop="2">https://s3.amazonaws.com/splickit-api-assets/audio/you_have_a_pending_order.mp3</Play>
            </Response>';
		$url_encoded_message = urlencode(cleanUpDoubleSpacesCRLFTFromString($message));
		$message_response_url= getProperty('twimlets_server').$url_encoded_message;
		$response = TwilioService::InitializeOutboundCall('1234567890', $message_response_url);
		$this->assertEquals('queued',$response['twilio_status']);
	}

	function testErrorResponse()
	{
		$message = '<?xml version="1.0" encoding="UTF-8"?>
            <Response>
                <Message>280743</Message>
                <Message>1234</Message>
                <Play loop="2">https://s3.amazonaws.com/splickit-api-assets/audio/you_have_a_pending_order.mp3</Play>
            </Response>';
		$url_encoded_message = urlencode(cleanUpDoubleSpacesCRLFTFromString($message));
		$message_response_url= getProperty('twimlets_server').$url_encoded_message;
		try {
			$ts = new TwilioService();
			$response = $ts->doOutboundCall("1234", $message_response_url);
		} catch (UnsuccessfulIvrPushException $e) {
			$message = $e->getMessage();
			$code = $e->getCode();
		}
		$this->assertEquals("IVR message Falure: The 'To' number 11234 is not a valid phone number.",$message);
		$this->assertEquals(100,$code);
	}

	function testCreateIvrMessageControllerFromFactory()
	{
		$message_resource = Resource::dummyfactory(array("message_format" => 'I'));
		$controller_factory = new ControllerFactory();
		$controller_name = $controller_factory->getControllerNameFromMessageResource($message_resource);
		$this->assertEquals('IvrController',$controller_name,"It should return Ivr as the controller name");
		$controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
		$this->assertEquals('IvrController', get_class($controller), "It should return a Ivr Controller");
	}

	function testRepresentationForMessageFormatIC(){
		$user_resource = createNewUserWithCC();
		$user = logTestUserResourceIn($user_resource);
		$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'some dum note');
		$order_resource = placeOrderFromOrderData($order_data, getTodayTwelveNoonTimeStampDenver());
		$order_id = $order_resource->order_id;

		$merchant_message_history = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'IC');
		$ivr_controller = new IvrController($mt, $user, $r);
		$representation_string = $ivr_controller->getRepresentationFromMessageFormatOfMessageResource($merchant_message_history);
		$this->assertEquals("/utility_templates/twilio_outbound_response_ic.xml", $representation_string);
	}

	function testBuildMessageIC()
	{
		$user_resource = createNewUserWithCC();
		$user = logTestUserResourceIn($user_resource);
		$ivr_controller = new IvrController($mt, $u, $r);
		$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'some dum note');
		$order_resource = placeOrderFromOrderData($order_data, getTodayTwelveNoonTimeStampDenver());
		$this->assertNull($order_resource->error);
		$order_id = $order_resource->order_id;
		$this->assertTrue($order_id > 9999,"should have create a valid order id");

		$message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'IC');
		$resource = $ivr_controller->prepMessageForSending($message_resource);
		$message_body = $resource->message_text;
		$expected_message_body = '<?xml version="1.0" encoding="UTF-8"?><Response><Message>'.$order_id.'</Message><Message>1234567890</Message><Play loop="2">https://s3.amazonaws.com/splickit-api-assets/audio/you_have_a_pending_order.mp3</Play></Response>';
		$this->assertEquals($expected_message_body,cleanUpDoubleSpacesCRLFTFromString($message_body));
	}

	function testBuildMessageIA()
	{
		$user_resource = createNewUserWithCC();
		$user = logTestUserResourceIn($user_resource);
		$mmha = new MerchantMessageHistoryAdapter($m);
		$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'],'pickup','some dum note');
		$order_resource = placeOrderFromOrderData($order_data, getTodayTwelveNoonTimeStampDenver());
		$order_id = $order_resource->order_id;
		$new_message = $mmha->createMessageReturnResource($this->ids['merchant_id'],$order_id,'IA','1234567890',time()-1);
		$ivr_controller = new IvrController($mt, $u, $r);
		$this->assertNotFalse($new_message);
		$ivr_controller->setTestDeliveryAddress('1234567890');
		$resource = $ivr_controller->prepMessageForSending($new_message);
		$message_body = $resource->message_text;
		$expected_IA_text = '<?xml version="1.0" encoding="UTF-8"?><Response><Message>'.$order_id.'</Message><Message>1234567890</Message><Play loop="2">https://s3.amazonaws.com/splickit-api-assets/audio/cycleprinter.mp3</Play></Response>';
		$this->assertEquals($expected_IA_text,cleanUpDoubleSpacesCRLFTFromString($message_body));
	}

	function testBuildMessageIB()
	{
		$user_resource = createNewUserWithCC();
		$user = logTestUserResourceIn($user_resource);
		$mmha = new MerchantMessageHistoryAdapter($m);
		$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'],'pickup','some dum note');
		$order_resource = placeOrderFromOrderData($order_data, getTodayTwelveNoonTimeStampDenver());
		$order_id = $order_resource->order_id;
		$new_message = $mmha->createMessageReturnResource($this->ids['merchant_id'],$order_id,'IB','1234567890',time()-1);
		$ivr_controller = new IvrController($mt, $u, $r);
		$this->assertNotFalse($new_message);
		$ivr_controller->setTestDeliveryAddress('1234567890');
		$resource = $ivr_controller->prepMessageForSending($new_message);
		$message_body = $resource->message_text;
		$expected_IA_text = '<?xml version="1.0" encoding="UTF-8"?><Response><Message>'.$order_id.'</Message><Message>1234567890</Message><Play loop="2">https://s3.amazonaws.com/splickit-api-assets/audio/restart_printer.mp3</Play></Response>';
		$this->assertEquals($expected_IA_text,cleanUpDoubleSpacesCRLFTFromString($message_body));

	}


	function testIvrSendMessage(){
		$user_resource = createNewUserWithCC();
		$user = logTestUserResourceIn($user_resource);
		setProperty('do_not_call_out_to_aws','false');
		$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'some dum note');
		$order_resource = placeOrderFromOrderData($order_data, getTodayTwelveNoonTimeStampDenver());
		$order_id = $order_resource->order_id;
		$message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'IC');
		$ivr_controller = new IvrController($mt, $u, $r);
		$result = $ivr_controller->sendThisMessage($message_resource);
		$this->assertTrue($result);
		// now check that message was updated with values
		$message_record = MerchantMessageHistoryAdapter::staticGetRecordByPrimaryKey($message_resource->map_id,'MerchantMessageHistoryAdapter');
		$expected_message_body = '<?xml version="1.0" encoding="UTF-8"?><Response><Message>'.$order_id.'</Message><Message>1234567890</Message><Play loop="2">https://s3.amazonaws.com/splickit-api-assets/audio/you_have_a_pending_order.mp3</Play></Response>';
		$this->assertEquals($expected_message_body,cleanUpDoubleSpacesCRLFTFromString($message_record['message_text']));
		$this->assertContains('"status": "queued"',$message_record['response']);
	}
    
    function testCreateDuplicateIVRmessages2mintuesApart()
    {
    	$user_resource = createNewUserWithCC();
    	$user = logTestUserResourceIn($user_resource);
    	
    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'some dum note');
    	$order_resource = placeOrderFromOrderData($order_data, getTomorrowTwelveNoonTimeStampDenver());
    	$this->assertNull($order_resource->error);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 9999,"shoudl have create a valid order id");
    	
    	$mmha = new MerchantMessageHistoryAdapter($mimetypes);
    	$records = $mmha->getRecords(array("order_id"=>$order_id,"message_format"=>"IC","locked"=>"N"), $options); 
    	$this->assertCount(3, $records," should have found 3 IVR messages.");
    	
    	$diff1 = $records[1]['next_message_dt_tm'] - $records[0]['next_message_dt_tm'];
    	$this->assertEquals(120, $diff1," should be 2 mintues between messages");
    	$diff2 = $records[2]['next_message_dt_tm'] - $records[1]['next_message_dt_tm'];
    	$this->assertEquals(120, $diff2," should be 2 mintues between messages");
    	return $order_id;
    }   
    
//    /**
//     * @depends testCreateDuplicateIVRmessages2mintuesApart
//     */
//    function testCancelOpenIVrMEssagesForOrderOnSuccessfullCallBack($order_id)
//    {
//    	$mmha = new MerchantMessageHistoryAdapter($mimetypes);
//    	$options[TONIC_FIND_BY_METADATA] = array("order_id"=>$order_id,"message_format"=>"IC","locked"=>"N");
//    	$resources = Resource::findAll($mmha, null, $options);
//    	$this->assertCount(3, $resources);
//    	$message_resource = $resources[0];
//    	$message_resource->sent_dt_tm = time();
//    	$message_resource->locked = 'S';
//    	$message_resource->save();
//    	$resources = Resource::findAll($mmha, null, $options);
//    	$this->assertCount(2, $resources);
//
//    	//first set call back status to 3 and verify it doesnt' do anything
//
//    	$call_back_url = getProperty('localhost_url_for_testing')."/messagemanager/updateivrorderstatus?orderid=$order_id&CallStatus=3&epoch=1406727164&DTN=15052427482&VXML_ID=10&";
//    	$response = $this->makeRequest($call_back_url, $userpassword);
//    	$records1 = $mmha->getRecords(array("order_id"=>$order_id,"message_format"=>"IC","locked"=>"N"), $options);
//    	$this->assertCount(2, $records1,"there shoujld still be 2 open messages since status was a 3 and NOT a 2");
//
//    	$call_back_url = "http://localhost/app2/messagemanager/updateivrorderstatus?orderid=$order_id&CallStatus=2&epoch=1406727164&DTN=15052427482&VXML_ID=10&";
//    	$this->makeRequest($call_back_url, $userpassword);
//
//    	$records2 = $mmha->getRecords(array("order_id"=>$order_id,"message_format"=>"IC","locked"=>"N"), $options);
//    	$this->assertCount(0, $records2,"there shoujld be no more open message");
//
//    	$records3 = $mmha->getRecords(array("order_id"=>$order_id,"message_format"=>"IC","locked"=>"S"), $options);
//    	$this->assertCount(1, $records3,"should still have the one sent message that got picked up and caused the successful call back");
//
//    	$records4 = $mmha->getRecords(array("order_id"=>$order_id,"message_format"=>"IC","locked"=>"C"), $options);
//    	$this->assertCount(2, $records4," should have two cancelled messages");
//    }
      
    function makeRequest($url,$userpassword,$method = 'GET',$data = null)
    {
    	$curl = curl_init($url);
    	if ($userpassword) {
    		curl_setopt($curl, CURLOPT_USERPWD, $userpassword);
    	}
    	$headers = array("X_SPLICKIT_CLIENT_ID:com.splickit.vtwoapi","X_SPLICKIT_CLIENT_DEVICE:unit_testing","X_SPLICKIT_CLIENT:APIDispatchTest","NO_CC_CALL:true");
    	if ($method == 'POST') {
    		$json = json_encode($data);
    		curl_setopt($curl, CURLOPT_POST, 1);
			curl_setopt($curl, CURLOPT_POSTFIELDS,$json);
			$headers[] = 'Content-Type: application/json';
			$headers[] = 'Content-Length: ' . strlen($json);
    	} else if ($method == 'DELETE') {
   			curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "DELETE"); 		
    	}
		curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
    	curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        myerror_log("Abougt to curl to: $url");
		$result = curl_exec($curl);
        myerror_log("The curl result is: $result");
		$this->info = curl_getinfo($curl);
        if (curl_errno($curl) > 0) {
            $e = curl_error($curl);
            myerror_log("WE HAVE A CURL ERROR: $e");
            die("curl error: ".$e);
        }

		curl_close($curl);
    	return $result;
    }

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

    	ini_set('max_execution_time',0);
    	$_SERVER['request_time1'] = microtime(true);
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	ini_set('max_execution_time',300);
    	SplickitCache::flushAll();
    	$db = DataBase::getInstance();
    	$mysqli = $db->getConnection();
    	$mysqli->begin_transaction(); ;

		$skin_resource = createWorldHqSkin();


		//map it to a menu
    	$menu_id = createTestMenuWithNnumberOfItems(1);
    	$ids['menu_id'] = $menu_id;
    	
    	$merchant_resource = createNewTestMerchant($menu_id);
    	$merchant_id = $merchant_resource->merchant_id;
    	$ids['merchant_id'] = $merchant_id;
    	
    	$map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_id,"message_format"=>'FA',"delivery_addr"=>"1234567890","message_type"=>"O"));
    	$map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_id,"message_format"=>'IC',"delivery_addr"=>"1234567890","message_type"=>"I","delay"=>1));
		$map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_id,"message_format"=>'IC',"delivery_addr"=>"1234567890","message_type"=>"I","delay"=>3));
		$map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_id,"message_format"=>'IC',"delivery_addr"=>"1234567890","message_type"=>"I","delay"=>5));
		 
    	$sql = "DELETE FROM Merchant_Message_History WHERE message_format = 'IC'";
    	$mmha = new MerchantMessageHistoryAdapter($mimetypes);
    	$mmha->_query($sql);

    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {

    	//SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    IvrControllerTest::main();
}

?>