<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class LoyaltyGoodcentsTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
        $this->ids = $_SERVER['unit_test_ids'];
        setContext("com.splickit.goodcentssubs");

    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->stamp);
        unset($this->ids);
    }

    function testDoNotEarnLoyaltyForCash()
    {
        $user_resource = createNewUser();
        $phone_number = str_ireplace('-','',$user_resource->contact_no);
        $user = logTestUserResourceIn($user_resource);
        $user_session_controller = new UsersessionController($m,$user,$r,5);
        $user_session = $user_session_controller->getUserSession($user_resource);
        $user_brand_points_map_adapter = new UserBrandPointsMapAdapter(getM());
        $user_brand_points_record = $user_brand_points_map_adapter->getRecord(array("user_id"=>$user_resource->user_id,"brand_id"=>getBrandIdFromCurrentContext()));

        $merchant_resource = createNewTestMerchant($this->ids['menu_id']);
        $merchant_resource->merchant_external_id = '22222';
        $merchant_resource->save();
        $merchant_id = $merchant_resource->merchant_id;
        $map_resource = Resource::findOrCreateIfNotExistsByData(new MerchantPaymentTypeMapsAdapter(getM()),['merchant_id'=>$merchant_id]);
        $map_resource->splickit_accepted_payment_type_id = 1000;
        $map_resource->billing_entity_id = 'nullit';
        $map_resource->save();

        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_resource->merchant_id,"message_format"=>'X',"delivery_addr"=>"Xoikos","message_type"=>"X"));


        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $ubp_resource = Resource::find(new UserBrandPointsMapAdapter($m),"",array(TONIC_FIND_BY_METADATA=>array("user_id"=>$user_id,"brand_id"=>getBrandIdFromCurrentContext())));
        $this->assertEquals(1.00,$ubp_resource->dollar_balance);

        $this->assertEquals(GoodCentsSubsLoyaltyController::NO_LOYALTY_FOR_CASH_MESSAGE,$order_resource->loyalty_message);
        $this->assertEquals(GoodCentsSubsLoyaltyController::NO_LOYALTY_FOR_CASH_MESSAGE,$order_resource->loyalty_earned_label);
        $this->assertEquals('',$order_resource->loyalty_earned_message);
        $this->assertEquals('Rewards Balance',$order_resource->loyalty_balance_label);
        $this->assertEquals('$1.00',$order_resource->loyalty_balance_message);

        $xoikos_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'X');
        $xoikos_controller = ControllerFactory::generateFromMessageResource($xoikos_message_resource);
        $ready_to_send_message_resource = $xoikos_controller->prepMessageForSending($xoikos_message_resource);
        $message_text = $ready_to_send_message_resource->message_text;
        myerror_log($message_text);
        $this->assertContains("<LoyaltyAwarded>False</LoyaltyAwarded>",$message_text);


    }

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        ini_set('max_execution_time',300);
        $goodcents_skin_resource = getOrCreateSkinAndBrandIfNecessary("Goodcents Subs","Goodcents Subs",140,430);
        $brand_resource = Resource::find(new BrandAdapter($m),"".$goodcents_skin_resource->brand_id);
        $brand_resource->loyalty = 'Y';
        $brand_resource->save();

        $blra = new BrandLoyaltyRulesAdapter($m);
        $gcblr = array("brand_id"=>430);
        $goodcents_brand_loyalty_rules_resource = Resource::findOrCreateIfNotExistsByData($blra,$gcblr);
        $goodcents_brand_loyalty_rules_resource->starting_point_value = 100;
        $goodcents_brand_loyalty_rules_resource->earn_value_amount_multiplier = 10;
        $goodcents_brand_loyalty_rules_resource->loyalty_type = 'splickit_earn';
        $goodcents_brand_loyalty_rules_resource->save();

        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction(); ;
        $_SERVER['request_time1'] = microtime(true);

        $menu_id = createTestMenuWithNnumberOfItems(5);
        $ids['menu_id'] = $menu_id;

        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $ids['merchant_id'] = $merchant_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;

    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    LoyaltyGoodcentsTest::main();
}

?>