<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class LoyaltyHeartlandTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
		$this->ids = $_SERVER['unit_test_ids'];
		
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }
    
    function testGetPitapitLoyaltyController()
    {
    	setContext("com.splickit.pitapit");
    	$loyalty_controller = LoyaltyControllerFactory::getLoyaltyControllerForContext();
    	$this->assertEquals("PitapitLoyaltyController", get_class($loyalty_controller));
    }
    
    function testJoinLoyaltyPitaPit()
    {
		setContext("com.splickit.pitapit");
    	$user_resource = createNewUser(array("create_loyalty_account"=>true));
    	
    	$ubpm_adapter = new UserBrandPointsMapAdapter($mimetypes);
    	$record = $ubpm_adapter->getRecord(array("user_id"=>$user_resource->user_id,"brand_id"=>282));
    	$this->assertNotNull($record,"Should have found a loyalty record");
    	$loyalty_number = $record['loyalty_number'];
    	$_SERVER['heartland_test_loyalty_number'] = $loyalty_number;
    	
    	$user = logTestUserIn($user_resource->user_id);
    	$user_session_controller = new UsersessionController($mt, $user, $request,5);
    	$user_session_resource = $user_session_controller->getUserSession($user_resource);
    	$this->assertNotNull($user_session_resource);
    	$this->assertNotNull($user_session_resource->brand_loyalty);
    	$this->assertEquals($loyalty_number, $user_session_resource->brand_loyalty['loyalty_number'].':'.$user_session_resource->brand_loyalty['pin'].':'.$user_session_resource->brand_loyalty['qrcode_number']);
    }
    
    function testLinkExistingPitCard()
    {
    	setContext("com.splickit.pitapit");
    	$user_resource = createNewUser();
    	$user = logTestUserIn($user_resource->user_id);
    	$request = new Request();
    	$request->data['loyalty_number'] = "8888888888";
    	$request->data['pin'] = '1234';
    	$loyalty_controller = new PitapitLoyaltyController($mimetypes, $user, $request, 5);
    	$user_brand_points_resource = $loyalty_controller->linkAccount($user['user_id'], $points);
    	$this->assertTrue($user_brand_points_resource->_exists);
		$this->assertEquals(282, $user_brand_points_resource->brand_id);
		$this->assertEquals(1500, $user_brand_points_resource->points);
    }
    
    function testLinkExistingOnUserCreation()
    {
		setContext("com.splickit.pitapit");    	
    	$user_resource = createNewUser(array("loyalty_number"=>"1234588888","pin"=>'1234'));
    	$this->assertEquals(true, $user_resource->_exists);
    	
    	$ubpa = new UserBrandPointsMapAdapter($mimetypes);
    	$record = $ubpa->getRecord(array("user_id"=>$user_resource->user_id,"brand_id"=>282));
    	$this->assertEquals("1234588888:1234:;5022440100000000118=380100071527KATANA?", $record['loyalty_number']);
    	$this->assertEquals(1500, $record['points']);
    	return $user_resource->user_id;    	
    }
    
    /**
     * @depends testLinkExistingOnUserCreation
     */
    function testGetLoyaltySessionDataWithRemoteInfo($user_id)
	{
    	setContext('com.splickit.pitapit');
    	$user = logTestUserIn($user_id);
    	$loyalty_controller = new PitapitLoyaltyController($mimetypes, $user, $request,5);
    	
    	$loyalty_user_session_data = $loyalty_controller->getAccountInfoForUserSession();
    	$this->assertEquals($loyalty_controller->getLoyaltyNumber(), $loyalty_user_session_data['loyalty_number']);
    	$this->assertTrue($loyalty_user_session_data['points'] == '1500');
    	$this->assertTrue(isset($loyalty_user_session_data['usd']));
    	$this->assertTrue($loyalty_user_session_data['usd'] == '0.00');
    	//$this->assertTrue(is_array($loyalty_user_session_data['history']));
		return $user_id;
	}
    
    /**
     * @depends testGetLoyaltySessionDataWithRemoteInfo
     */
	function testGetUserSessionWithLoyaltyPointsAndHistory($user_id)
    {
    	setContext('com.splickit.pitapit');
		$user_resource = Resource::find(new UserAdapter($mimetypes), "$user_id", $options);
    	// reset points
    	$user_brand_points_map_adapter = new UserBrandPointsMapAdapter($mimetypes);
    	$ubpm_resource = $user_brand_points_map_adapter->getExactResourceFromData(array("user_id"=>$user_id,"brand_id"=>282));
    	$ubpm_resource->points = 500;
    	$ubpm_resource->save();

    	$user = logTestUserIn($user_id);
    	
    	$_SERVER['REQUEST_METHOD'] = 'GET';
    	$usc = new UsersessionController($mt, $user, $r, 5);
    	$user_session_resource = $usc->getUserSession($user_resource);
    	$this->assertNotNull($user_session_resource->brand_loyalty);
    	$brand_loyalty = $user_session_resource->brand_loyalty;
    	$this->assertNotNull($brand_loyalty['loyalty_number']);
    	$this->assertEquals('1234588888', $brand_loyalty['loyalty_number']);
    	$this->assertEquals('1234', $brand_loyalty['pin']);
    	//$this->assertEquals('3C154325E7544F', $brand_loyalty['barcode_number']);
    	$this->assertEquals('1500', $brand_loyalty['points']);
    	$remote_history = $brand_loyalty['loyalty_transactions'];
    	$this->assertTrue(is_array($remote_history));
    	
    	//no check to make sure the poinst were reset to the remote value
    	$ubpm_resource = $user_brand_points_map_adapter->getExactResourceFromData(array("user_id"=>$user_resource->user_id,"brand_id"=>282));
    	$this->assertEquals('1500', $ubpm_resource->points);
    }

    function testLinkExistingLoyaltyPitCardBadCard()
    {
    	setContext("com.splickit.pitapit");    	
    	$user_resource = createNewUser(array("loyalty_number"=>"7777777777","pin"=>'1234'));
    	$this->assertNotNull($user_resource->error,"Should have thrown an error creating the user since the loyalty number is bad");
    	$this->assertEquals("We're sorry, the pit card number you entered, 7777777777:1234, appears to be invalid, please check your entry.", $user_resource->error);
    }

/*
    function testRedemption()
    {
		$this->assertTrue(false);		
    }
*/    
    static function setUpBeforeClass()
    {
        $code = setNewStamp();

    	set_time_limit(30);
    	$_SERVER['request_time1'] = microtime(true);
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	ini_set('max_execution_time',300);
    	      SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();$mysqli->begin_transaction(); ;
    	
    	$skin_resource = createWorldHqSkin();
    	$ids['skin_id'] = $skin_resource->skin_id;
    	
		//map it to a menu
    	$menu_id = createTestMenuWithNnumberOfItems(5);
    	$ids['menu_id'] = $menu_id;
    	
    	$modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 10);
    	$modifier_group_id = $modifier_group_resource->modifier_group_id;
    	$item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 2);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[1]['item_id'], $modifier_group_id, 4);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[2]['item_id'], $modifier_group_id, 1);

    	$merchant_resource = createNewTestMerchant($menu_id);
    	attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
    	$ids['merchant_id'] = $merchant_resource->merchant_id;
    	
    	$user_resource = createNewUser(array("flags"=>"1C20000001"));
    	$ids['user_id'] = $user_resource->user_id;
    	
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    LoyaltyHeartlandTest::main();
}

?>