<?php

/**
 * Created by PhpStorm.
 * User: Elvis
 * Date: 8/16/16
 * Time: 4:57 PM
 */

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class MailChimpServiceTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
        $this->ids = $_SERVER['unit_test_ids'];
        setContext("com.splickit.chimpskin");

    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }

    function testValidateDate()
    {
        $mailchimp_service = new MailChimpService();
        $date_of_birth = "05/04/1983";
        $this->assertTrue($mailchimp_service->validateBirthday($date_of_birth));
        $date_of_birth = "5/4/1983";
        $this->assertTrue($mailchimp_service->validateBirthday($date_of_birth));
        $date_of_birth = "0504";
        $this->assertFalse($mailchimp_service->validateBirthday($date_of_birth));
        $date_of_birth = "may 4";
        $this->assertFalse($mailchimp_service->validateBirthday($date_of_birth));
        $date_of_birth = "06/31/1983";
        $this->assertFalse($mailchimp_service->validateBirthday($date_of_birth));
        $date_of_birth = "06/21/2008";
        $this->assertTrue($mailchimp_service->validateBirthday($date_of_birth));
        $date_of_birth = "happpy";
        $this->assertFalse($mailchimp_service->validateBirthday($date_of_birth));
        $date_of_birth = "14/06/1983";
        $this->assertFalse($mailchimp_service->validateBirthday($date_of_birth));
        $date_of_birth = "10/O6/1983";
        $this->assertFalse($mailchimp_service->validateBirthday($date_of_birth));
        $date_of_birth = "03/07/1992";
        $this->assertTrue($mailchimp_service->validateBirthday($date_of_birth));

    }


    function testCreateUserWithEmailMarketingJoin()
    {
        $user_resource = createNewUser(array("birthday" => "05/04/1983", "marketing_email_opt_in" => "Y"));
        $this->assertNull($user_resource->error);
        $user_id = $user_resource->user_id;
        $this->assertTrue($user_id > 20000);
        $this->assertNotNull($user_resource->marketing_results);

        // now check to see if the records were updated in the user extr data table
        /***** not doing this any more *****/
//        $ueda = new UserExtraDataAdapter(getM());
//        $record = $ueda->getRecord(array('user_id' => $user_id));
//        $this->assertEquals("mailchimp", $record['process']);
    }

    function testCreateUserWithEmailMarketingBadData()
    {
        $user_resource = createNewUser(array("birthday" => "0698", "marketing_email_opt_in" => "Y"));
        $this->assertNotNull($user_resource->error, "Should have gotten an error on save from bad birthday");
        $this->assertNull($user_resource->user_id);
        $this->assertEquals("Sorry, birthday must be in the form of mm/dd/YYYY. please try again", $user_resource->error);
    }

    function testCreateUserWithEmailMarketingShortDate()
    {
        $user_resource = createNewUser(array("birthday" => "5/4", "marketing_email_opt_in" => "Y"));
        $this->assertNull($user_resource->error, "Should NOT have gotten an error on save from bad birthday");
    }

//    function testUserUpdateWithBadEmailMarketingData()
//    {
//        $ur = createNewUser();
//        $user = logTestUserIn($ur->user_id);
//        $data = array("birth_date" => "0504", "marketing_email_opt_in" => "Y");
//        $request = new Request();
//        $request->method = "post";
//        $request->data = $data;
//        $user_controller = new UserController(getM(), $user, $request, 5);
//        $user_resource = $user_controller->updateUser();
//        $this->assertNotNull($user_resource->error);
//        $this->assertEquals("Sorry, birthday must be in the form of mm/dd/YYYY. please try again", $user_resource->error);
//
//    }

    function testCreateUserWIthNOData()
    {
        $user_resource = createNewUser();
        $this->assertNull($user_resource->error);
        $user_id = $user_resource->user_id;
        $this->assertTrue($user_id > 20000);

        $user = logTestUserIn($user_id);
        $data = array("first_name" => "roberts");
        $request = new Request();
        $request->method = "post";
        $request->data = $data;
        $user_controller = new UserController(getM(), $user, $request, 5);
        $user_resource = $user_controller->updateUser();

        // now check to see if the records were updated in the user extr data table
        $ueda = new UserExtraDataAdapter(getM());
        $record = $ueda->getRecord(array('user_id' => $user_id));
        $this->assertNull($record);
    }

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time', 300);
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction();
        $_SERVER['log_level'] = 5;
        $ids = array();
        $_SERVER['unit_test_ids'] = $ids;

        $skin_resource = getOrCreateSkinAndBrandIfNecessary("chimpskin", "chimpbrand", null, null);
        setContext("com.splickit.chimpskin");
        $brand_id = $skin_resource->brand_id;

        //enable brand field options for test
        $options[TONIC_FIND_BY_METADATA]['brand_id'] = $skin_resource->brand_id;
        $bsf = Resource::findAll(new BrandSignupFieldsAdapter(getM()), '', $options);

        foreach ($bsf as $item){
            $item->logical_delete = 'N';
            $item->save();
        }

        foreach (ThirdPartyProviderParametersAdapter::staticGetRecords(['third_party_provider_id'=>6001],'ThirdPartyProviderParametersAdapter') as $index=>$tpp) {
            $data = [];
            $data['third_party_provider_parameter_id'] = $tpp['id'];
            $data['skin_id'] = getSkinIdForContext();

            if ($tpp['name'] == 'mailchimp_mail_list') {
                $data['value'] = '8888abcd8888';
            } else if ($tpp['name'] == 'mailchimp_api_key') {
                $data['value'] = "1234567890abcdefghijkl-us3";
            } else if ($tpp['name'] == 'active_flag') {
                $data['value'] = 'Y';
                $data['brand_id'] = getBrandIdFromCurrentContext();
                $resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()), $data);
                continue;
            }
            $resource = Resource::createByData(new ThirdPartyProviderParameterSkinMapsAdapter(getM()), $data);
        }

        $sql = "INSERT INTO `Brand_Signup_Fields` VALUES (null, 'marketing_email_opt_in', 'boolean', '$brand_id', 'Yes, send me exclusive offers & news from Deli Fresh Subs', '', 'MailChimp', 'N', '2020-05-18 18:02:15', '0000-00-00 00:00:00'), (null, 'birthday', 'date', '$brand_id', 'BIRTHDAY (REQUIRED)', 'mm/dd/YYYY', 'MailChimp', 'N', '2020-05-18 18:02:15', '0000-00-00 00:00:00');";
        $bsfa = new BrandSignupFieldsAdapter(getM());
        $bsfa->_query($sql);

    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main()
    {
        $suite = new PHPUnit_Framework_TestSuite(__CLASS__);
        PHPUnit_TextUI_TestRunner::run($suite);
    }
}


if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    MailChimpServiceTest::main();
}
?>
