<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class MenuControllerTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
		$this->ids = $_SERVER['unit_test_ids'];
		
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }

    function testAddingMenus()
    {
        $merchant_resource = createNewTestMerchant();
        $merchant_id = $merchant_resource->merchant_id;


        $menu_id = createTestMenuWithNnumberOfItems(1);
        $url = "http://127.0.0.1:" . $this->api_port . "/app2/portal/menus/$menu_id/merchants/$merchant_id";
        $data = ["merchant_menu_type"=>'pickup'];
        $request = createRequestObject($url,'POST',json_encode($data));
        $menu_controller = new MenuController(getM(),null,$request,5);
        $response = makeMockPortalRequest($menu_controller);
        $response_as_array = json_decode($response,true);
        $this->assertEquals(200,$response_as_array['http_code']);

        $merchant_menu_map_adapter = new MerchantMenuMapAdapter(getM());
        $records = $merchant_menu_map_adapter->getRecords(['merchant_id'=>$merchant_id]);
        $this->assertCount(1,$records,"it should have one MMM record");
        $this->assertEquals($menu_id,$records[0]['menu_id']);

        $menu_id2 = createTestMenuWithNnumberOfItems(1);
        $url = "http://127.0.0.1:" . $this->api_port . "/app2/portal/menus/$menu_id2/merchants/$merchant_id";
        $data = ["merchant_menu_type"=>'pickup'];
        $request = createRequestObject($url,'POST',json_encode($data));
        $menu_controller = new MenuController(getM(),null,$request,5);
        $response = makeMockPortalRequest($menu_controller);
        $response_as_array = json_decode($response,true);

        $records = $merchant_menu_map_adapter->getRecords(['merchant_id'=>$merchant_id]);
        $this->assertCount(1,$records,"it should have one MMM record");
        $this->assertEquals($menu_id2,$records[0]['menu_id']);

        $this->assertEquals(200,$response_as_array['http_code']);

        // now try adding delivery record
        $url = "http://127.0.0.1:" . $this->api_port . "/app2/portal/menus/$menu_id/merchants/$merchant_id";
        $data = ["merchant_menu_type"=>'delivery'];
        $request = createRequestObject($url,'POST',json_encode($data));
        $menu_controller = new MenuController(getM(),null,$request,5);
        $response = makeMockPortalRequest($menu_controller);
        $response_as_array = json_decode($response,true);
        $this->assertEquals(200,$response_as_array['http_code']);

        $records = $merchant_menu_map_adapter->getRecords(['merchant_id'=>$merchant_id]);
        $this->assertCount(2,$records,"it should have two MMM record");
        $hash = createHashmapFromArrayOfArraysByFieldName($records,'merchant_menu_type');
        $this->assertEquals($menu_id,$hash['delivery']['menu_id']);
    }

    function testAttachMenuToMerchantReplacingBoth()
    {
        $merchant_resource = createNewTestMerchant();
        $merchant_id = $merchant_resource->merchant_id;


        $pickup_menu_id = createTestMenuWithNnumberOfItems(1);
        $url = "http://127.0.0.1:" . $this->api_port . "/app2/portal/menus/$pickup_menu_id/merchants/$merchant_id";
        $data = ["merchant_menu_type"=>'pickup'];
        $request = createRequestObject($url,'POST',json_encode($data));
        $menu_controller = new MenuController(getM(),null,$request,5);
        $response = makeMockPortalRequest($menu_controller);
        $response_as_array = json_decode($response,true);
        $this->assertEquals(200,$response_as_array['http_code']);

        $delivery_menu_id = createTestMenuWithNnumberOfItems(1);
        $url = "http://127.0.0.1:" . $this->api_port . "/app2/portal/menus/$delivery_menu_id/merchants/$merchant_id";
        $data = ["merchant_menu_type"=>'delivery'];
        $request = createRequestObject($url,'POST',json_encode($data));
        $menu_controller = new MenuController(getM(),null,$request,5);
        $response = makeMockPortalRequest($menu_controller);
        $response_as_array = json_decode($response,true);
        $this->assertEquals(200,$response_as_array['http_code']);



        $menu_id = createTestMenuWithNnumberOfItems(1);
        $url = "http://127.0.0.1:" . $this->api_port . "/app2/portal/menus/$menu_id/merchants/$merchant_id";
        $data = ["merchant_menu_type"=>'both'];
        $request = createRequestObject($url,'POST',json_encode($data));
        $menu_controller = new MenuController(getM(),null,$request,5);
        $response = makeMockPortalRequest($menu_controller);
        $response_as_array = json_decode($response,true);
        $this->assertEquals(200,$response_as_array['http_code']);


        $merchant_menu_map_adapter = new MerchantMenuMapAdapter(getM());
        $records = $merchant_menu_map_adapter->getRecords(['merchant_id'=>$merchant_id]);
        $this->assertCount(2,$records,"it should have two MMM records");
        $this->assertEquals($menu_id,$records[0]['menu_id']);
        $this->assertEquals($menu_id,$records[1]['menu_id']);

    }



    static function setUpBeforeClass()
    {
        $code = setNewStamp();

    	ini_set('max_execution_time',0);
    	$_SERVER['request_time1'] = microtime(true);
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	ini_set('max_execution_time',300);
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction(); ;
    	
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    MenuControllerTest::main();
}

?>