<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class MenuSchedulerTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
        setProperty("DO_NOT_CHECK_CACHE",'true',true);
		$this->ids = $_SERVER['unit_test_ids'];
		setContext("com.splickit.scheduleskin");
		
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }

    function testGetCurrentWeekInCycleMondayIssueWithCOBChangeAndLateHoursCLoseToo()
    {
        $menu_id = createTestMenuWithNnumberOfItems(5);

        $start_timestamp = getTimeStampForDateTimeAndTimeZone(19,1,0,5,4,2020);
        $data = ['menu_id'=>$menu_id,'number_of_weeks_in_cycle'=>2,'start_dt_tm'=>$start_timestamp];
        $ms_resource = Resource::createByData(new MenuSchedulesAdapter(getM()),$data);

        $user = logTestUserIn(101);
        $_SERVER['HTTP_X_SPLICKIT_CLIENT_VERSION'] = 10.0;
        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $hours_resources = Resource::findAll(new HourAdapter(getM()),null,[3=>['merchant_id'=>$merchant_id]]);
        foreach ($hours_resources as $hour_resource) {
            $day = $hour_resource->day_of_week;
            if ($day == 1 || $day == 7) {
                $hour_resource->day_open = 'N';
                $hour_resource->save();
                if ($day == 1) {
                    $monday_hour_resource = $hour_resource;
                }
            }
        }

        $time_stamp = getTimeStampForDateTimeAndTimeZone(0,15,0,5,11,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();
        $menu = $resource->menu;
        $this->assertEquals(2,$menu['using_week_in_cycle']);
        $this->assertEquals(2,$menu['using_day_in_cycle']);

        // now set sunday close to 3am
        $monday_hour_resource->day_open = 'Y';
        $monday_hour_resource->close = '02:00:00';
        $monday_hour_resource->save();

        $time_stamp = getTimeStampForDateTimeAndTimeZone(0,15,0,5,11,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();
        $menu = $resource->menu;
        $this->assertEquals(1,$menu['using_week_in_cycle']);
        $this->assertEquals(1,$menu['using_day_in_cycle']);

        $time_stamp = getTimeStampForDateTimeAndTimeZone(1,45,0,5,11,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();
        $menu = $resource->menu;
        $this->assertEquals(1,$menu['using_week_in_cycle']);
        $this->assertEquals(1,$menu['using_day_in_cycle']);

        $time_stamp = getTimeStampForDateTimeAndTimeZone(2,15,0,5,11,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();
        $menu = $resource->menu;
        $this->assertEquals(2,$menu['using_week_in_cycle']);
        $this->assertEquals(2,$menu['using_day_in_cycle']);

    }

    function testGetCurrentWeekInCycleMondayIssueWithCOBChange()
    {
        $menu_id = createTestMenuWithNnumberOfItems(5);

        $start_timestamp = getTimeStampForDateTimeAndTimeZone(19,1,0,5,4,2020);
        $data = ['menu_id'=>$menu_id,'number_of_weeks_in_cycle'=>2,'start_dt_tm'=>$start_timestamp];
        $ms_resource = Resource::createByData(new MenuSchedulesAdapter(getM()),$data);

        $user = logTestUserIn(101);
        $_SERVER['HTTP_X_SPLICKIT_CLIENT_VERSION'] = 10.0;
        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $hours_resources = Resource::findAll(new HourAdapter(getM()),null,[3=>['merchant_id'=>$merchant_id]]);
        foreach ($hours_resources as $hour_resource) {
            $day = $hour_resource->day_of_week;
            if ($day == 1 || $day == 7) {
                $hour_resource->day_open = 'N';
                $hour_resource->save();
            }
        }

        $time_stamp = getTimeStampForDateTimeAndTimeZone(21,0,0,5,8,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();
        $menu = $resource->menu;
        $this->assertEquals(2,$menu['using_week_in_cycle']);
        $this->assertEquals(2,$menu['using_day_in_cycle']);

        $time_stamp = getTimeStampForDateTimeAndTimeZone(19,0,0,5,8,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();
        $menu = $resource->menu;
        $this->assertEquals(1,$menu['using_week_in_cycle']);
        $this->assertEquals(6,$menu['using_day_in_cycle']);
        $this->assertEquals(3600,$menu['cache_expires_in_seconds']);

        $time_stamp = getTimeStampForDateTimeAndTimeZone(12,0,0,5,9,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();
        $menu = $resource->menu;
        $this->assertEquals(2,$menu['using_week_in_cycle']);
        $this->assertEquals(2,$menu['using_day_in_cycle']);

        $time_stamp = getTimeStampForDateTimeAndTimeZone(12,0,0,5,10,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();
        $menu = $resource->menu;
        $this->assertEquals(2,$menu['using_week_in_cycle']);
        $this->assertEquals(2,$menu['using_day_in_cycle']);

        $time_stamp = getTimeStampForDateTimeAndTimeZone(21,0,0,5,15,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();
        $menu = $resource->menu;
        $this->assertEquals(1,$menu['using_week_in_cycle']);
        $this->assertEquals(2,$menu['using_day_in_cycle']);

        $time_stamp = getTimeStampForDateTimeAndTimeZone(19,0,0,5,15,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();
        $menu = $resource->menu;
        $this->assertEquals(2,$menu['using_week_in_cycle']);
        $this->assertEquals(6,$menu['using_day_in_cycle']);
        $this->assertEquals(3600,$menu['cache_expires_in_seconds']);


    }

    function testGetCurrentWeekInCycleMondayIssue()
    {
        $time_stamp_start = getTimeStampForDateTimeAndTimeZone(19,0,0,2,15,2021);
//        $start_date = new DateTime();
//        $start_date->setTimestamp($time_stamp_start);

        $time_stamp_current = getTimeStampForDateTimeAndTimeZone(10,00,12,2,22,2021);
        $current_date = new DateTime();
        $current_date->setTimestamp($time_stamp_current);

        $record = ['start_dt_tm'=>$time_stamp_start,'number_of_weeks_in_cycle'=>4];
        $msa = new MenuSchedulesAdapter(getM());
        $result = $msa->getCurrentWeekInCycle($current_date,$record);
        $this->assertEquals(2,$result);
    }

    function testGetCurrentWeekInCycle()
    {
        $current_date = new DateTime();
        $time = time();
        $record = ['start_dt_tm'=>$time,'number_of_weeks_in_cycle'=>4];
        $msa = new MenuSchedulesAdapter(getM());
        $result = $msa->getCurrentWeekInCycle($current_date,$record);

        $this->assertEquals(1,$result);

        $time = $time - (7*24*3600);
        $record['start_dt_tm'] = $time;
        $result = $msa->getCurrentWeekInCycle($current_date,$record);
        $this->assertEquals(2,$result);

        $time = $time - (7*24*3600);
        $record['start_dt_tm'] = $time;
        $result = $msa->getCurrentWeekInCycle($current_date,$record);
        $this->assertEquals(3,$result);

        $time = $time - (7*24*3600);
        $record['start_dt_tm'] = $time;
        $result = $msa->getCurrentWeekInCycle($current_date,$record);
        $this->assertEquals(4,$result);

        $time = $time - (7*24*3600);
        $record['start_dt_tm'] = $time;
        $result = $msa->getCurrentWeekInCycle($current_date,$record);
        $this->assertEquals(1,$result);

    }

//    function testGetCurrentWeekInCycleWithNonMidnightStart()
//    {
//        $msa = new MenuSchedulesAdapter(getM());
//        $current_date = new DateTime();
//        $current_date->setTimestamp(getTimeStampForDateTimeAndTimeZone(3,00,39,5,4,2020));
//        $start_date = new DateTime();
//        $start_date->setTimestamp(getTimeStampForDateTimeAndTimeZone(6,35,0,5,4,2020));
//        $result = $msa->getCurrentWeekInCycle($start_date,$current_date,4);
//        $this->assertEquals(1,$result);
//
//        $start_date->sub(new DateInterval('P7D'));
//        $result = $msa->getCurrentWeekInCycle($start_date,$current_date,4);
//        $this->assertEquals(2,$result);
//
//        $start_date->sub(new DateInterval('P7D'));
//        $result = $msa->getCurrentWeekInCycle($start_date,$current_date,4);
//        $this->assertEquals(3,$result);
//
//        $start_date->sub(new DateInterval('P7D'));
//        $result = $msa->getCurrentWeekInCycle($start_date,$current_date,4);
//        $this->assertEquals(4,$result);
//
//        $start_date->sub(new DateInterval('P7D'));
//        $result = $msa->getCurrentWeekInCycle($start_date,$current_date,4);
//        $this->assertEquals(1,$result);
//
//    }



    function testGetArrayOfOnAndOffItemsByWeek()
    {
        $menu_id = createTestMenuWithNnumberOfItems(5);
        $modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id,5,'Schedule Group');

        $start_timestamp = getTimeStampForDateTimeAndTimeZone(19,1,0,5,4,2020);
        $data = ['menu_id'=>$menu_id,'number_of_weeks_in_cycle'=>4,'start_dt_tm'=>$start_timestamp];
        $ms_resource = Resource::createByData(new MenuSchedulesAdapter(getM()),$data);

        $items = CompleteMenu::getAllMenuItemsAsArray($menu_id);
        foreach ($items as $item) {
            assignModifierGroupToItemWithFirstNAsComesWith($item['item_id'], $modifier_group_resource->modifier_group_id, 0);
        }

        $item1_id = $items[0]['item_id'];
        $item2_id = $items[1]['item_id'];
        $item3_id = $items[2]['item_id'];
        $item4_id = $items[3]['item_id'];

        $mosbw = new MenuObjectSchedulesByWeekAndDayMapsAdapter(getM());

        // week 1
        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>1,'day_number'=>3,'object_type'=>'Item','object_id'=>$item1_id];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>1,'day_number'=>4,'object_type'=>'Item','object_id'=>$item2_id];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>1,'day_number'=>5,'object_type'=>'Item','object_id'=>$item3_id];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>1,'day_number'=>6,'object_type'=>'Item','object_id'=>$item4_id];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        // week 2
        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>2,'day_number'=>3,'object_type'=>'Item','object_id'=>$item4_id];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>2,'day_number'=>4,'object_type'=>'Item','object_id'=>$item1_id];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>2,'day_number'=>5,'object_type'=>'Item','object_id'=>$item2_id];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>2,'day_number'=>6,'object_type'=>'Item','object_id'=>$item3_id];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        // week 3
        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>3,'day_number'=>5,'object_type'=>'Item','object_id'=>$item1_id];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        // week 4
        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>4,'day_number'=>3,'object_type'=>'Item','object_id'=>$item3_id];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>4,'day_number'=>4,'object_type'=>'Item','object_id'=>$item4_id];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>4,'day_number'=>5,'object_type'=>'Item','object_id'=>$item2_id];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>4,'day_number'=>6,'object_type'=>'Item','object_id'=>$item1_id];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);



        $menu_schedules_adapter = new MenuSchedulesAdapter(getM());
        $current_date_time = new DateTime();
        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,4,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(4,$menu_object_schedules['Item'],"it should have 4 records of items scheduled to be off");

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,5,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(3,$menu_object_schedules['Item'],"it should have 3 records of items scheduled to be off");
        foreach ($menu_object_schedules['Item'] as $item_id) {
            $this->assertNotEquals($item1_id,$item_id);
        }

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,6,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(3,$menu_object_schedules['Item'],"it should have 3 records of items scheduled to be off");
        foreach ($menu_object_schedules['Item'] as $item_id) {
            $this->assertNotEquals($item2_id,$item_id);
        }

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,7,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(3,$menu_object_schedules['Item'],"it should have 3 records of items scheduled to be off");
        foreach ($menu_object_schedules['Item'] as $item_id) {
            $this->assertNotEquals($item3_id,$item_id);
        }

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,8,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(3,$menu_object_schedules['Item'],"it should have 3 records of items scheduled to be off");
        foreach ($menu_object_schedules['Item'] as $item_id) {
            $this->assertNotEquals($item4_id,$item_id);
        }

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,9,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(4,$menu_object_schedules['Item'],"it should have 4 records of items scheduled to be off");


        // week 2

        $menu_schedules_adapter = new MenuSchedulesAdapter(getM());
        $current_date_time = new DateTime();
        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,11,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(4,$menu_object_schedules['Item'],"it should have 4 records of items scheduled to be off");

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,12,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(3,$menu_object_schedules['Item'],"it should have 3 records of items scheduled to be off");
        foreach ($menu_object_schedules['Item'] as $item_id) {
            $this->assertNotEquals($item4_id,$item_id);
        }

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,13,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(3,$menu_object_schedules['Item'],"it should have 3 records of items scheduled to be off");
        foreach ($menu_object_schedules['Item'] as $item_id) {
            $this->assertNotEquals($item1_id,$item_id);
        }

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,14,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(3,$menu_object_schedules['Item'],"it should have 3 records of items scheduled to be off");
        foreach ($menu_object_schedules['Item'] as $item_id) {
            $this->assertNotEquals($item2_id,$item_id);
        }

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,15,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(3,$menu_object_schedules['Item'],"it should have 3 records of items scheduled to be off");
        foreach ($menu_object_schedules['Item'] as $item_id) {
            $this->assertNotEquals($item3_id,$item_id);
        }

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,16,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(4,$menu_object_schedules['Item'],"it should have 4 records of items scheduled to be off");

        //week 3
        $menu_schedules_adapter = new MenuSchedulesAdapter(getM());
        $current_date_time = new DateTime();
        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,18,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(4,$menu_object_schedules['Item'],"it should have 4 records of items scheduled to be off");

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,19,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(4,$menu_object_schedules['Item'],"it should have 4 records of items scheduled to be off");

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,20,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(4,$menu_object_schedules['Item'],"it should have 4 records of items scheduled to be off");

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,21,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(3,$menu_object_schedules['Item'],"it should have 3 records of items scheduled to be off");
        foreach ($menu_object_schedules['Item'] as $item_id) {
            $this->assertNotEquals($item1_id,$item_id);
        }

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,22,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(4,$menu_object_schedules['Item'],"it should have 4 records of items scheduled to be off");

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,23,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(4,$menu_object_schedules['Item'],"it should have 4 records of items scheduled to be off");


        //lets try june 22nd which would be week 4
        $menu_schedules_adapter = new MenuSchedulesAdapter(getM());
        $current_date_time = new DateTime();
        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,6,22,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(4,$menu_object_schedules['Item'],"it should have 4 records of items scheduled to be off");

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,6,23,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(3,$menu_object_schedules['Item'],"it should have 3 records of items scheduled to be off");
        foreach ($menu_object_schedules['Item'] as $item_id) {
            $this->assertNotEquals($item3_id,$item_id);
        }

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,6,24,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(3,$menu_object_schedules['Item'],"it should have 3 records of items scheduled to be off");
        foreach ($menu_object_schedules['Item'] as $item_id) {
            $this->assertNotEquals($item4_id,$item_id);
        }

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,6,25,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(3,$menu_object_schedules['Item'],"it should have 3 records of items scheduled to be off");
        foreach ($menu_object_schedules['Item'] as $item_id) {
            $this->assertNotEquals($item2_id,$item_id);
        }

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,6,26,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(3,$menu_object_schedules['Item'],"it should have 3 records of items scheduled to be off");
        foreach ($menu_object_schedules['Item'] as $item_id) {
            $this->assertNotEquals($item1_id,$item_id);
        }

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,6,27,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNotNull($menu_object_schedules['Item'],"it should have had the item ids");
        $this->assertCount(4,$menu_object_schedules['Item'],"it should have 4 records of items scheduled to be off");

        // now lets deactivate the parent record
        $ms_resource->active = false;
        $ms_resource->save();
        $ms_resource = $ms_resource->refreshResource();
        $this->assertEquals(0,$ms_resource->active);

        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,6,27,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNull($menu_object_schedules['Item'],"it should NOT have had any off objects");

        $ms_resource->active = 1;
        $ms_resource->save();

        return $menu_id;
    }

    /**
     * @depends testGetArrayOfOnAndOffItemsByWeek
     */
    function testGetMenu($menu_id)
    {
        $_SERVER['HTTP_X_SPLICKIT_CLIENT_VERSION'] = 10.0;
        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;

        $msa = new MenuSchedulesAdapter(getM());
        $options[TONIC_FIND_BY_METADATA] = ['menu_id'=>$menu_id];
        $ms_resource = Resource::find($msa,null,$options);

        // week 2 day 1

        $time_stamp = getTimeStampForDateTimeAndTimeZone(12,0,0,5,11,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();

        // menu should only have 1 item since all 4 others are off
        $menu = $resource->menu;
        $menu_types = $menu['menu_types'];
        $items = $menu_types[0]['menu_items'];
        $this->assertCount(1,$items);
        $this->assertEquals('Test Item 5',$items[0]['item_name']);

        // week 2 day 2
        $time_stamp = getTimeStampForDateTimeAndTimeZone(12,0,0,5,12,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();

        // menu should only have 2 items since 3 others are off
        $menu = $resource->menu;
        $menu_types = $menu['menu_types'];
        $items = $menu_types[0]['menu_items'];
        $this->assertCount(2,$items);
        $this->assertEquals('Test Item 4',$items[0]['item_name']);
        $this->assertEquals('Test Item 5',$items[1]['item_name']);

        // week 2 day 1 but after close of store
        $time_stamp = getTimeStampForDateTimeAndTimeZone(21,0,0,5,11,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();

        // should have the same menu as week 2 day 2
        $menu = $resource->menu;
        $menu_types = $menu['menu_types'];
        $items = $menu_types[0]['menu_items'];
        $this->assertCount(2,$items,"It should have the same number of items as if it was day 2");
        $this->assertEquals('Test Item 4',$items[0]['item_name'],"It should have the item 4 as if it was day 2");
        $this->assertEquals('Test Item 5',$items[1]['item_name'],"It should have the item 5 as if it was day 2");




        return $menu_id;
    }

    /**
     * @depends testGetMenu
     */
    function testAfterMidnightClose($menu_id)
    {
        // log in as store tester to prevent using cached menu
        $user = logTestUserIn(101);
        $_SERVER['HTTP_X_SPLICKIT_CLIENT_VERSION'] = 10.0;
        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $hours_resources = Resource::findAll(new HourAdapter(getM()),null,[3=>['merchant_id'=>$merchant_id]]);
        foreach ($hours_resources as $hour_resource) {
            $day = $hour_resource->day_of_week;
            if ($day == 1 || $day == 7) {
                ; //do nothing
            } else {
                $hour_resource->close = '03:00:00';
                $hour_resource->save();
            }
        }

        // test the expiration for sunday with a normal close. cache experiation shoudl be the close .

        $time_stamp = getTimeStampForDateTimeAndTimeZone(12,0,0,5,10,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();
        $menu = $resource->menu;
        $expected_cache_expiration = '2020-05-10 20:00:00';
        $this->assertEquals($expected_cache_expiration,$menu['cache_expires_at']);

        // test the expiration for monday with after midnight close. the cache expiration shoudl be set to the close at 3am.
        $time_stamp = getTimeStampForDateTimeAndTimeZone(12,0,0,5,11,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();
        $menu = $resource->menu;
        $expected_cache_expiration = '2020-05-12 03:00:00';
        $this->assertEquals($expected_cache_expiration,$menu['cache_expires_at']);


        // test getting a menu on tuesday morning at 2am. we should still be showing the monday menu
        $time_stamp = getTimeStampForDateTimeAndTimeZone(2,0,0,5,12,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();
        $menu = $resource->menu;
        $expected_cache_expiration = '2020-05-12 03:00:00';
        $this->assertEquals($expected_cache_expiration,$menu['cache_expires_at']);

        //make sure we are still getting the monday menu since it is an after midnight close
        // menu should only have 1 item since all 4 others are off
        $menu_types = $menu['menu_types'];
        $items = $menu_types[0]['menu_items'];
        $this->assertCount(1,$items);
        $this->assertEquals('Test Item 5',$items[0]['item_name']);

    }

    /**
     * @depends testGetMenu
     */
    function testClosedWeekend($menu_id)
    {
        // log in as store tester to prevent using cached menu
        $user = logTestUserIn(101);
        $_SERVER['HTTP_X_SPLICKIT_CLIENT_VERSION'] = 10.0;
        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $hours_resources = Resource::findAll(new HourAdapter(getM()),null,[3=>['merchant_id'=>$merchant_id]]);
        foreach ($hours_resources as $hour_resource) {
            $day = $hour_resource->day_of_week;
            if ($day == 1 || $day == 2) {
                $hour_resource->day_open = 'N';
                $hour_resource->save();
            }
        }

        // test getting a menu on saturday evening after close. we should be showing the tuesday menu
        $time_stamp = getTimeStampForDateTimeAndTimeZone(22,0,0,5,9,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();
        $menu = $resource->menu;

        // menu should  have 2 items
        $menu_types = $menu['menu_types'];
        $items = $menu_types[0]['menu_items'];
        $this->assertCount(2,$items);
        $this->assertEquals('Test Item 4',$items[0]['item_name']);
        $this->assertEquals('Test Item 5',$items[1]['item_name']);

        // test getting a menu on sunday. we should be showing the tuesday menu
        $time_stamp = getTimeStampForDateTimeAndTimeZone(12,0,0,5,10,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();
        $menu = $resource->menu;

        // menu should  have 2 items
        $menu_types = $menu['menu_types'];
        $items = $menu_types[0]['menu_items'];
        $this->assertCount(2,$items);
        $this->assertEquals('Test Item 4',$items[0]['item_name']);
        $this->assertEquals('Test Item 5',$items[1]['item_name']);

        // test getting a menu on monday. we should be showing the tuesday menu
        $time_stamp = getTimeStampForDateTimeAndTimeZone(12,0,0,5,11,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();
        $menu = $resource->menu;

        // menu should  have 2 items
        $menu_types = $menu['menu_types'];
        $items = $menu_types[0]['menu_items'];
        $this->assertCount(2,$items);
        $this->assertEquals('Test Item 4',$items[0]['item_name']);
        $this->assertEquals('Test Item 5',$items[1]['item_name']);

        // test getting a menu on Tuesday. we should be showing the tuesday menu
        $time_stamp = getTimeStampForDateTimeAndTimeZone(12,0,0,5,12,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();
        $menu = $resource->menu;

        // menu should  have 2 items
        $menu_types = $menu['menu_types'];
        $items = $menu_types[0]['menu_items'];
        $this->assertCount(2,$items);
        $this->assertEquals('Test Item 4',$items[0]['item_name']);
        $this->assertEquals('Test Item 5',$items[1]['item_name']);

        // test getting a menu on Tuesday after close. we should be showing the wednesday menu
        $time_stamp = getTimeStampForDateTimeAndTimeZone(22,0,0,5,12,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();
        $menu = $resource->menu;

        // menu should  have X items
        $menu_types = $menu['menu_types'];
        $items = $menu_types[0]['menu_items'];
        $this->assertCount(2,$items);
        $this->assertEquals('Test Item 1',$items[0]['item_name']);
        $this->assertEquals('Test Item 5',$items[1]['item_name']);



    }


    function testMenuScheduleForModifiers()
    {
        $_SERVER['HTTP_X_SPLICKIT_CLIENT_VERSION'] = 10.0;
        $menu_id = createTestMenuWithNnumberOfItems(2);
        $modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id,5,'Schedule Group');

        $start_timestamp = getTimeStampForDateTimeAndTimeZone(0,1,0,5,4,2020);
        $data = ['menu_id'=>$menu_id,'number_of_weeks_in_cycle'=>2,'start_dt_tm'=>$start_timestamp];
        $ms_resource = Resource::createByData(new MenuSchedulesAdapter(getM()),$data);

        $items = CompleteMenu::getAllMenuItemsAsArray($menu_id);
        foreach ($items as $item) {
            assignModifierGroupToItemWithFirstNAsComesWith($item['item_id'], $modifier_group_resource->modifier_group_id, 0);
        }

        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;

        $modifiers = CompleteMenu::getAllModifierItemsAsArray($menu_id,'Y',$merchant_id);

        $modifier_id1 = $modifiers[0]['modifier_item_id'];
        $modifier_id2 = $modifiers[1]['modifier_item_id'];
        $modifier_id3 = $modifiers[2]['modifier_item_id'];
        $modifier_id4 = $modifiers[3]['modifier_item_id'];
        $modifier_id5 = $modifiers[4]['modifier_item_id'];

        $mosbw = new MenuObjectSchedulesByWeekAndDayMapsAdapter(getM());

        // week 1
        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>1,'day_number'=>2,'object_type'=>'Modifier_Item','object_id'=>$modifier_id1];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>1,'day_number'=>4,'object_type'=>'Modifier_Item','object_id'=>$modifier_id2];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>1,'day_number'=>4,'object_type'=>'Modifier_Item','object_id'=>$modifier_id3];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>1,'day_number'=>6,'object_type'=>'Modifier_Item','object_id'=>$modifier_id3];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>1,'day_number'=>6,'object_type'=>'Modifier_Item','object_id'=>$modifier_id4];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>1,'day_number'=>6,'object_type'=>'Modifier_Item','object_id'=>$modifier_id5];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        // week 2
        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>2,'day_number'=>2,'object_type'=>'Modifier_Item','object_id'=>$modifier_id1];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>2,'day_number'=>3,'object_type'=>'Modifier_Item','object_id'=>$modifier_id2];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>2,'day_number'=>4,'object_type'=>'Modifier_Item','object_id'=>$modifier_id3];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>2,'day_number'=>5,'object_type'=>'Modifier_Item','object_id'=>$modifier_id4];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>2,'day_number'=>6,'object_type'=>'Modifier_Item','object_id'=>$modifier_id5];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);


        // week 1 day 2

        $time_stamp = getTimeStampForDateTimeAndTimeZone(12,0,0,5,4,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();

        // menu should only have 1 modifier_item since all 4 others are off
        $menu = $resource->menu;
        $menu_types = $menu['menu_types'];
        $items = $menu_types[0]['menu_items'];
        foreach ($items as $item) {
            $this->assertCount(1,$item['modifier_groups'][0]['modifier_items']);
            $this->assertEquals($modifier_id1,$item['modifier_groups'][0]['modifier_items'][0]['modifier_item_id']);

        }

        // week 1 day 4

        $time_stamp = getTimeStampForDateTimeAndTimeZone(12,0,0,5,6,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();

        // menu should  have 2 modifier_item since all 3 others are off
        $menu = $resource->menu;
        $menu_types = $menu['menu_types'];
        $items = $menu_types[0]['menu_items'];
        foreach ($items as $item) {
            $this->assertCount(2,$item['modifier_groups'][0]['modifier_items']);
            $modifier_hash_map = createHashmapFromArrayOfArraysByFieldName($item['modifier_groups'][0]['modifier_items'],'modifier_item_id');
            $this->assertTrue(isset($modifier_hash_map[$modifier_id2]));
            $this->assertTrue(isset($modifier_hash_map[$modifier_id3]));
        }

        // week 1 day 6

        $time_stamp = getTimeStampForDateTimeAndTimeZone(12,0,0,5,8,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();

        // menu should  have 2 modifier_item since all 3 others are off
        $menu = $resource->menu;
        $menu_types = $menu['menu_types'];
        $items = $menu_types[0]['menu_items'];
        foreach ($items as $item) {
            $this->assertCount(3,$item['modifier_groups'][0]['modifier_items']);
            $modifier_hash_map = createHashmapFromArrayOfArraysByFieldName($item['modifier_groups'][0]['modifier_items'],'modifier_item_id');
            $this->assertTrue(isset($modifier_hash_map[$modifier_id3]));
            $this->assertTrue(isset($modifier_hash_map[$modifier_id4]));
            $this->assertTrue(isset($modifier_hash_map[$modifier_id5]));
        }

        // week 2 day 5

        $time_stamp = getTimeStampForDateTimeAndTimeZone(12,0,0,5,14,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();

        // menu should only have 1 modifier_item since all 4 others are off
        $menu = $resource->menu;
        $menu_types = $menu['menu_types'];
        $items = $menu_types[0]['menu_items'];
        foreach ($items as $item) {
            $this->assertCount(1,$item['modifier_groups'][0]['modifier_items']);
            $this->assertEquals($modifier_id4,$item['modifier_groups'][0]['modifier_items'][0]['modifier_item_id']);

        }
    }



    function testMenuScheduleForMenuType()
    {
        $menu_id = createTestMenuWithNnumberOfItemsAndMenuTypes(2,0,3);
        $complete_menu = CompleteMenu::getCompleteMenu($menu_id);
        $menu_type1_id = $complete_menu['menu_types'][0]['menu_type_id'];
        $menu_type1_name = $complete_menu['menu_types'][0]['menu_type_name'];
        $menu_type2_id = $complete_menu['menu_types'][1]['menu_type_id'];
        $menu_type2_name = $complete_menu['menu_types'][1]['menu_type_name'];

        $start_timestamp = getTimeStampForDateTimeAndTimeZone(0,0,1,5,4,2020);
        $data = ['menu_id'=>$menu_id,'number_of_weeks_in_cycle'=>3,'start_dt_tm'=>$start_timestamp];
        $ms_resource = Resource::createByData(new MenuSchedulesAdapter(getM()),$data);


        $mosbw = new MenuObjectSchedulesByWeekAndDayMapsAdapter(getM());

        // week 1
        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>1,'day_number'=>3,'object_type'=>'Menu_Type','object_id'=>$menu_type1_id];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>1,'day_number'=>4,'object_type'=>'Menu_Type','object_id'=>$menu_type2_id];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        //week 2
        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>2,'day_number'=>5,'object_type'=>'Menu_Type','object_id'=>$menu_type1_id];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $week_item_data = ['menu_schedule_id'=>$ms_resource->id,'week_number_in_cycle'=>2,'day_number'=>5,'object_type'=>'Menu_Type','object_id'=>$menu_type2_id];
        $week_item_schedule_resource = Resource::createByData($mosbw,$week_item_data);

        $menu_schedules_adapter = new MenuSchedulesAdapter(getM());
        $current_date_time = new DateTime();
        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,4,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNull($menu_object_schedules['Item']);
        $this->assertNotNull($menu_object_schedules['Menu_Type'],"it should have had the menu_type ids");
        $this->assertCount(2,$menu_object_schedules['Menu_Type'],"it should have 2 records of menu_Types scheduled to be off");

        $menu_schedules_adapter = new MenuSchedulesAdapter(getM());
        $current_date_time = new DateTime();
        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,5,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNull($menu_object_schedules['Item']);
        $this->assertNotNull($menu_object_schedules['Menu_Type'],"it should have had the menu_type ids");
        $this->assertCount(1,$menu_object_schedules['Menu_Type'],"it should have 1 records of menu_Types scheduled to be off");
        $this->assertEquals($menu_type2_id,$menu_object_schedules['Menu_Type'][$menu_type2_id]);

        $menu_schedules_adapter = new MenuSchedulesAdapter(getM());
        $current_date_time = new DateTime();
        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,6,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNull($menu_object_schedules['Item']);
        $this->assertNotNull($menu_object_schedules['Menu_Type'],"it should have had the menu_type ids");
        $this->assertCount(1,$menu_object_schedules['Menu_Type'],"it should have 1 records of menu_Types scheduled to be off");
        $this->assertEquals($menu_type1_id,$menu_object_schedules['Menu_Type'][$menu_type1_id]);

        $menu_schedules_adapter = new MenuSchedulesAdapter(getM());
        $current_date_time = new DateTime();
        $current_date_time->setTimestamp(getTimeStampForDateTimeAndTimeZone(12,0,0,5,14,2020));
        $menu_object_schedules = $menu_schedules_adapter->getInnactiveObjectsByDate($menu_id,$current_date_time);
        $this->assertNull($menu_object_schedules['Item']);
        $this->assertNull($menu_object_schedules['Menu_Type'],"it should have had no menu_type ids");

        return $menu_id;
    }

    /**
     * @depends testMenuScheduleForMenuType
     */
    function testGetMerchantMenuWithMenuTypeSchedule($menu_id)
    {
        $_SERVER['HTTP_X_SPLICKIT_CLIENT_VERSION'] = 10.0;
        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;

        $msa = new MenuSchedulesAdapter(getM());
        $options[TONIC_FIND_BY_METADATA] = ['menu_id'=>$menu_id];
        $ms_resource = Resource::find($msa,null,$options);

        // week 1 day 1
        $time_stamp = getTimeStampForDateTimeAndTimeZone(12,0,0,5,4,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();

        // menu should only have 1 Menu_Type since both others are off
        $menu = $resource->menu;
        $menu_types = $menu['menu_types'];
        $this->assertCount(1,$menu_types);
        $this->assertEquals('Test Menu Type 3',$menu_types[0]['menu_type_name']);

        // now lets deactivate the parent record
        $ms_resource->active = false;
        $ms_resource->save();
        $ms_resource = $ms_resource->refreshResource();
        $this->assertEquals(0,$ms_resource->active);

        $time_stamp = getTimeStampForDateTimeAndTimeZone(12,0,0,5,4,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();

        // menu should  have all 3 Menu_Types since parent record is not active
        $menu = $resource->menu;
        $menu_types = $menu['menu_types'];
        $this->assertCount(3,$menu_types);

        // reactivate
        $ms_resource->active = 1;
        $ms_resource->save();


        // week 1 day 3
        $time_stamp = getTimeStampForDateTimeAndTimeZone(12,0,0,5,6,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();

        // menu should  have 2 Menu_Types
        $menu = $resource->menu;
        $menu_types = $menu['menu_types'];
        $this->assertCount(2,$menu_types);
        foreach ($menu_types as $menu_type) {
            $this->assertNotEquals('Test Menu Type 1',$menu_type['menu_type_name']);
        }

        // week 2 day 4
        $time_stamp = getTimeStampForDateTimeAndTimeZone(12,0,0,5,14,2020);
        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET',null);
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $merchant_controller->setTheTime($time_stamp);
        $resource = $merchant_controller->processV2Request();

        // menu should  have 2 Menu_Types
        $menu = $resource->menu;
        $menu_types = $menu['menu_types'];
        $this->assertCount(3,$menu_types);


    }

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

    	ini_set('max_execution_time',0);
    	$_SERVER['request_time1'] = microtime(true);
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	ini_set('max_execution_time',300);
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction(); ;
        $skin_resource = getOrCreateSkinAndBrandIfNecessary("ScheduleSkin",'ScheduleBrand',null,null);
        setContext($skin_resource->external_identifier);

    	$_SERVER['log_level'] = 5; 
		//$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	SplickitCache::flushAll();
    	$db = DataBase::getInstance();
    	$mysqli = $db->getConnection();
    	$mysqli->rollback();
    	date_default_timezone_set($_SERVER['starting_tz']);

    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    MenuSchedulerTest::main();
}

?>