<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class MessageFactoryTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $code = generateCode(7);
        $_SERVER['STAMP'] = __CLASS__ . '-' . $code;
        $_SERVER['RAW_STAMP'] = $code;
        $this->ids = $_SERVER['unit_test_ids'];

    }

    function tearDown()
    {
        //delete your instance
        unset($this->ids);
        unset($_SERVER['max_lead']);
    }

    function testBadIdMusUseAlphOrNumericNOTid()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id);
        $alphanumeric = $merchant_resource->alphanumeric_id;
        $numeric_id = $merchant_resource->numeric_id;
        $merchant_id = $merchant_resource->merchant_id;

        $url = "https://sumdumdomain/app2/messagemanager/$merchant_id/compris/callback.txt";
        $request = createRequestObject($url,'POST');
        $compris_controller = ControllerFactory::getMessageControllerFromRequest($request);
        $this->assertNull($compris_controller);

    }

    function testComprisCallBack()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id);
        $alphanumeric = $merchant_resource->alphanumeric_id;
        $numeric_id = $merchant_resource->numeric_id;

        $url = "https://sumdumdomain/app2/messagemanager/$alphanumeric/compris/callback.txt";
        $request = createRequestObject($url,'POST');
        $compris_controller = ControllerFactory::getMessageControllerFromRequest($request);
        $this->assertEquals('ComprisController',get_class($compris_controller));
        $this->assertEquals($alphanumeric,$compris_controller->getSubmittedIdentifier());
        $this->assertEquals('callback',$compris_controller->getSubmittedAction());
    }

    function testFaxCallBack()
    {
        //curl -v http://device.order140-test.com/app2/messagemanager/9973iofwiv5w/map_id.8888/service.Phaxio/fax/callback.txt
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id);
        $alphanumeric = $merchant_resource->alphanumeric_id;
        $numeric_id = $merchant_resource->numeric_id;

        $url = "http://device.order140-test.com/app2/messagemanager/$alphanumeric/map_id.123456/service.Phaxio/fax/callback.txt";
        $request = createRequestObject($url,'POST');
        $fax_controller = ControllerFactory::getMessageControllerFromRequest($request);
        $this->assertEquals('FaxController',get_class($fax_controller));
        $this->assertEquals($alphanumeric,$fax_controller->getSubmittedIdentifier());
        $this->assertEquals('callback',$fax_controller->getSubmittedAction());

    }

    function testChinaIPCallBack()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id);
        $alphanumeric = $merchant_resource->alphanumeric_id;
        $numeric_id = $merchant_resource->numeric_id;

        $url = "http://device.splickit.com/app2/messagemanager/$numeric_id/chinaipprinter/callback.txt";
        $request = createRequestObject($url,'POST');
        $china_controller = ControllerFactory::getMessageControllerFromRequest($request);
        $this->assertEquals('ChinaIPPrinterController',get_class($china_controller));
        $this->assertEquals($numeric_id,$china_controller->getSubmittedIdentifier());
        $this->assertEquals('callback',$china_controller->getSubmittedAction());
    }

    function testGetControllerNameFromUrl()
    {
        $controller_factory = new ControllerFactory();
        $url = 'http://device.splickit.com/app2/messagemanager/getnextmessagebymerchantid/53436974/epsonprinter/message.txt';
        $name = $controller_factory->getUrlNameFromUrlString($url);
        $this->assertEquals('epsonprinter',$name);


        $url = 'http://device.splickit.com/app2/messagemanager/getnextmessagebymerchantid/66en54gpsl5t/winapp/';
        $name = $controller_factory->getUrlNameFromUrlString($url);
        $this->assertEquals('winapp',$name);

        $url = 'http://device.splickit.com/app2/messagemanager/foundry/getnextmessagebymerchantid';
        $name = $controller_factory->getUrlNameFromUrlString($url);
        $this->assertEquals('foundry',$name);

    }

    function testGetControllerFromRequest()
    {

        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id);
        $alphanumeric = $merchant_resource->alphanumeric_id;
        $numeric_id = $merchant_resource->numeric_id;

        $url = "http://device.splickit.com/app2/messagemanager/getnextmessagebymerchantid/$numeric_id/epsonprinter/message.txt";
        $request = createRequestObject($url,'GET');
        $epson_controller = ControllerFactory::getMessageControllerFromRequest($request);
        $this->assertEquals('EpsonPrinterController',get_class($epson_controller));
        $this->assertEquals($numeric_id,$epson_controller->getSubmittedIdentifier());
        $this->assertEquals('getnextmessagebymerchantid',$epson_controller->getSubmittedAction());

        $url = "http://device.splickit.com/app2/messagemanager/getnextmessagebymerchantid/$alphanumeric/winapp/";
        $request = createRequestObject($url,'GET');
        $winapp_controller = ControllerFactory::getMessageControllerFromRequest($request);
        $this->assertEquals('WindowsServiceController',get_class($winapp_controller));
        $this->assertEquals($alphanumeric,$winapp_controller->getSubmittedIdentifier());
        $this->assertEquals('getnextmessagebymerchantid',$winapp_controller->getSubmittedAction());

        $url = "http://device.splickit.com/app2/messagemanager/foundry/getnextmessagebymerchantid?reqtype=02&siteid=$alphanumeric&reqfmt=01";
        $request = createRequestObject($url,'GET');
        $foundry_controller = ControllerFactory::getMessageControllerFromRequest($request);
        $this->assertEquals('FoundryController',get_class($foundry_controller));
        $this->assertEquals($alphanumeric,$foundry_controller->getSubmittedIdentifier());
        $this->assertEquals('getnextmessagebymerchantid',$foundry_controller->getSubmittedAction());
    }

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        ini_set('max_execution_time', 0);
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction(); ;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main()
    {
        $suite = new PHPUnit_Framework_TestSuite(__CLASS__);
        PHPUnit_TextUI_TestRunner::run($suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    MessageFactoryTest::main();
}

?>