<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class OpieTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $user;
	var $ids;

	function setUp()
	{
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
		$this->ids = $_SERVER['unit_test_ids'];
		$this->user = logTestUserIn($this->ids['user_id']);
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';		
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->user);
		unset($this->stamp);
		unset($this->ids);
    }

    function testOpieOrder()
    {
    	
    	//$tz = getProperty("default_server_timezone");
    	//date_default_timezone_set($tz);
    	$ids = $this->ids;
    	$merchant_id = $ids['merchant_id'];
		$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id,'pickup','skip hours');
    	$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
		$order_id = $order_resource->order_id;
		$this->assertNotNull($order_resource);
		$this->assertNull($order_resource->error);
		$this->assertTrue($order_id > 1000,"Bad order id of: ".$order_id);  
		$order_dt_tm = $order_resource->order_dt_tm;
		$pickup_dt_tm = $order_resource->pickup_dt_tm;
		$tip = $order_resource->tip_amt;
		$grand_total = $order_resource->grand_total;
    	$grand_total_to_merchant = $order_resource->grand_total_to_merchant;
    	
    	$help_date_1 = date("Y-m-d H:i:s",$order_dt_tm);

		//sleep(1);
		
		$opie_controller = new OpieController($mt, $u, $r, 5);

        $order_message_resources = $opie_controller->getAllMessagesForOrderId($order_id);
        foreach ($order_message_resources as $order_message_resource) {
            if ($order_message_resource->message_format == 'O') {
                $order_message_resource->next_message_dt_tm = time()-10;
                $order_message_resource->save();
            }
        }

        $opie_formatted_resource = $opie_controller->pullNextMessageResourceByMerchant($merchant_id);
		$this->assertNotNull($opie_formatted_resource);
		#$opie_controller->markMessageDeliveredById($opie_formatted_resource->map_id);
        $opie_controller->markMessageDeliveredById($opie_formatted_resource->message_id);
		$message_text = $opie_formatted_resource->message_text;
		$order_data = json_decode($message_text);
		
		$help_date_2 = date("Y-m-d H:i:s",$order_data->order_dt_tm);
		myerror_log("order dt time from order: ".$help_date_1);
		myerror_log("order dt time from message: ".$help_date_2);
		
		$this->assertEquals($order_resource->order_id, $order_data->order_id);
		$this->assertEquals($order_resource->order_dt_tm, getMySqlFormattedDateTimeFromTimeStampAndTimeZone($order_data->order_dt_tm));
		$this->assertEquals($order_resource->pickup_dt_tm, getMySqlFormattedDateTimeFromTimeStampAndTimeZone($order_data->pickup_dt_tm));
		$this->assertEquals($order_resource->order_amt, $order_data->order_amt);
		$this->assertEquals($order_resource->user_id, $order_data->user_id);
		
		$new_order_resource = Resource::find($order_adapter,''.$order_id);
		$this->assertEquals("E", $new_order_resource->status);
		
		// check that message has not been viewed
		$message_resource = SplickitController::getResourceFromId($opie_formatted_resource->message_id, "MerchantMessageHistory");
		$this->assertEquals("N", $message_resource->viewed);
		
		// now mark it as viewed
		$opie_controller->markMessageAsViewed($opie_formatted_resource->message_id);
		$message_resource2 = SplickitController::getResourceFromId($opie_formatted_resource->message_id, "MerchantMessageHistory");
		$this->assertEquals("V", $message_resource2->viewed);
		
		//$expected_message_text = "{\"order_id\":\"".$order_id."\",\"stamp\":\"\",\"merchant_id\":\"1105\",\"order_dt_tm\":".$order_dt_tm.",\"user_id\":\"20000\",\"pickup_dt_tm\":".$pickup_dt_tm.",\"order_amt\":\"3.50\",\"promo_code\":null,\"promo_id\":null,\"promo_amt\":\"0.00\",\"total_tax_amt\":\"0.28\",\"trans_fee_amt\":\"0.00\",\"delivery_amt\":\"0.00\",\"tip_amt\":\"".$tip."\",\"customer_donation_amt\":\"0.00\",\"grand_total\":\"".$grand_total."\",\"grand_total_to_merchant\":\"".$grand_total_to_merchant."\",\"cash\":\"N\",\"merchant_donation_amt\":\"0.00\",\"note\":\"skip hours\",\"status\":\"O\",\"order_qty\":\"1\",\"payment_file\":null,\"order_type\":\"R\",\"phone_no\":null,\"user_delivery_location_id\":null,\"requested_delivery_time\":null,\"device_type\":null,\"app_version\":\"\",\"skin_id\":\"68\",\"distance_from_store\":\"0.00\",\"created\":1350693103,\"modified\":943945200,\"logical_delete\":\"N\",\"advance_order\":\"false\",\"first_name\":\"Test\",\"customer_first_name\":\"Test\",\"full_name\":\"Test Testing User\",\"customer_full_name\":\"Test Testing User\",\"user_email\":\"testable.user+1350679050@gmail.com\",\"user_phone_no\":\"13038844083\",\"pickup_date_time\":\"Oct 19, 2012   18:46\",\"pickup_time\":\"6:46\",\"pickup_time_military\":\"18:46\",\"pickup_date\":\"10\/19\/12\",\"pickup_day\":\"Friday\",\"order_day\":\"Fri\",\"order_time\":\"6:31\",\"merchant_name\":\"Moes Southwest Grill\",\"merchant_addr\":\"1530 16th st.\",\"merchant_phone_no\":\"(123) 456 7890\",\"brand_id\":\"162\",\"merchant_city_st_zip\":\"Denver, CO 80202\",\"day_of_week\":\"Friday\",\"order_identifier\":\"200\",\"order_details\":[{\"order_id\":\"221200\",\"order_detail_id\":\"455209\",\"item_size_id\":\"324199\",\"external_id\":null,\"menu_type_name\":\"Breakfast (until 10:30am Daily)\",\"size_name\":\"Kids\",\"size_print_name\":\"Kids\",\"item_name\":\"Breakfast Burrito\",\"item_print_name\":\"Breakfast Burrito\",\"name\":\"\",\"note\":\"\",\"quantity\":\"1\",\"price\":\"3.50\",\"item_total\":\"3.50\",\"item_total_w_mods\":\"3.50\",\"item_tax\":\"0.28\",\"created\":1350693103,\"modified\":1350693103,\"logical_delete\":\"N\",\"mods\":[[{\"order_detail_id\":\"455209\",\"order_detail_mod_id\":\"11981\",\"modifier_size_id\":\"2513363\",\"external_id\":null,\"modifier_item_id\":\"1602015\",\"modifier_item_priority\":\"0\",\"modifier_group_id\":\"235541\",\"modifier_group_name\":null,\"mod_name\":\"Eggs\",\"mod_print_name\":\"Eggs\",\"modifier_type\":\"H\",\"comes_with\":\"H\",\"hold_it\":true,\"mod_quantity\":\"0\",\"mod_price\":\"0.000\",\"mod_total_price\":\"0.000\",\"created\":1350693103,\"modified\":943945200,\"logical_delete\":\"N\"},{\"order_detail_id\":\"455209\",\"order_detail_mod_id\":\"11982\",\"modifier_size_id\":\"2513365\",\"external_id\":null,\"modifier_item_id\":\"1602017\",\"modifier_item_priority\":\"0\",\"modifier_group_id\":\"235541\",\"modifier_group_name\":null,\"mod_name\":\"Potatoes \",\"mod_print_name\":\"Potatoes \",\"modifier_type\":\"H\",\"comes_with\":\"H\",\"hold_it\":true,\"mod_quantity\":\"0\",\"mod_price\":\"0.000\",\"mod_total_price\":\"0.000\",\"created\":1350693103,\"modified\":943945200,\"logical_delete\":\"N\"}],[],[]],\"mod_titles\":[\"Holds\",\"Mods\",\"Sides\"]}],\"amt_billed_to_cc\":\"9.04\",\"balance_before\":\"0.00\",\"balance_after\":\"-9.04\",\"message_id\":\"17979\"}";
    	//$this->assertEquals($expected_message_text, $message_text);
    }

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

    	$_SERVER['request_time1'] = microtime(true);
    	$tz = date_default_timezone_get();
    	$_SERVER['default_tz'] = $tz;
    	date_default_timezone_set("America/Denver");
    	ini_set('max_execution_time',300);
    	      SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();$mysqli->begin_transaction(); ;
    	
    	$skin_resource = createWorldHqSkin();
    	$ids['skin_id'] = $skin_resource->skin_id;
    	
		//map it to a menu
    	$menu_id = createTestMenuWithNnumberOfItems(5);
    	$ids['menu_id'] = $menu_id;
    	
    	$modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 10);
    	$modifier_group_id = $modifier_group_resource->modifier_group_id;
    	$item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 2);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[1]['item_id'], $modifier_group_id, 4);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[2]['item_id'], $modifier_group_id, 1);

    	$merchant_resource = createNewTestMerchant($menu_id);
    	$merchant_id = $merchant_resource->merchant_id;
    	$ids['merchant_id'] = $merchant_id;
    	
    	$map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_id,"message_format"=>'O',"delivery_addr"=>"opie","message_type"=>"X"));
    	attachMerchantToSkin($merchant_id, $ids['skin_id']);
    	
    	$user_resource = createNewUser(array("flags"=>"1C20000001"));
    	$ids['user_id'] = $user_resource->user_id;
    	
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    OpieTest::main();
}

?>