<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class PaymentIntelityPassthroughTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $code = generateCode(7);
        $_SERVER['STAMP'] = __CLASS__ . '-' . $code;
        $_SERVER['RAW_STAMP'] = $code;
        $this->ids = $_SERVER['unit_test_ids'];
        setContext('com.splickit.ipassskin');

    }

    function tearDown()
    {
        //delete your instance
        unset($this->ids);
        unset($_SERVER['max_lead']);
    }



    function testPayment()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user_resource->flags = '1000000001';
        $user_resource->save();
        $user = logTestUserResourceIn($user_resource);

        $merchant_id = $this->ids['merchant_id'];
        $order_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $order_data['tip_amt'] = 8.88;

        $json_encoded_data = json_encode($order_data);
        $request = createRequestObject("app2/apiv2/carts","POST",$json_encoded_data);
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $cart_resource = $place_order_controller->processV2Request();
        $ucid = $cart_resource->ucid;

        $url = "app2/apiv2/carts/$ucid/checkout";
        $request = createRequestObject($url,"GET");
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_resource = $place_order_controller->processV2Request();

        $this->assertNull($checkout_resource->error);
        $accepted_payment_types = $checkout_resource->accepted_payment_types;
        $this->assertCount(1,$accepted_payment_types);
        $this->assertEquals('Intelity Pass Through',$accepted_payment_types[0]['name']);
        $this->assertEquals(10010,$accepted_payment_types[0]['splickit_accepted_payment_type_id']);

        $place_order_json = '{"merchant_payment_type_map_id":"'.$accepted_payment_types[0]['merchant_payment_type_map_id'].'","requested_time":null}';
        $request = createRequestObject("app2/apiv2/orders/$ucid","POST",$place_order_json);
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $order_resource = $place_order_controller->processV2Request();


     //   $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $bcrecords = BalanceChangeAdapter::staticGetRecords(array("order_id"=>$order_id),'BalanceChangeAdapter');

        $hash = createHashmapFromArrayOfArraysByFieldName($bcrecords,'process');
        $lu_payment_record = $hash['IntelityPassThrough'];
        $this->assertCount(1,$bcrecords,"it should show the intelity passthrough payment");
        $this->assertNotNull($lu_payment_record,'It should have the intelity pass through payment record');

    }

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        ini_set('max_execution_time', 0);
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction();;


        $skin_resource = getOrCreateSkinAndBrandIfNecessary('IpassSkin','IpassBrand',null,null);
        SkinSplickitAcceptedPaymentTypeMapsAdapter::staticAddRecord($skin_resource->skin_id,SplickitPaymentService::INTELITYPASSTHROUGHPAYMENTID);
        $ids['skin_id'] = $skin_resource->skin_id;
        $ids['passthrough_skin_resource'] = $skin_resource;

//map it to a menu
        $menu_id = createTestMenuWithNnumberOfItems(3);
        $ids['menu_id'] = $menu_id;

        $modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 3);
        $modifier_group_id = $modifier_group_resource->modifier_group_id;
        $item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', getM());
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 2);

//*
        $sides_modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 1, 'Sides', 'S');
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[1]['item_id'], $sides_modifier_group_resource->modifier_group_id, 0);

        $mealdeal_modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 1, 'Meal Deal', 'I');
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[2]['item_id'], $mealdeal_modifier_group_resource->modifier_group_id, 0);
//*/
        //$data = ['no_payment'=>true];
        $merchant_resource = createNewTestMerchant($menu_id,$data);
        attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
        $ids['merchant_id'] = $merchant_resource->merchant_id;

        //create level up passthrough merchant payment type map
        MerchantPaymentTypeMapsAdapter::createMerchantPaymentTypeMap($merchant_resource->merchant_id, 10010, null);

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main()
    {
        $suite = new PHPUnit_Framework_TestSuite(__CLASS__);
        PHPUnit_TextUI_TestRunner::run($suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    PaymentIntelityPassthroughTest::main();
}

?>