<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class PaymentLevelupPassthroughTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $code = generateCode(7);
        $_SERVER['STAMP'] = __CLASS__ . '-' . $code;
        $_SERVER['RAW_STAMP'] = $code;
        $this->ids = $_SERVER['unit_test_ids'];
        setContext('com.splickit.lupassskin');

    }

    function tearDown()
    {
        //delete your instance
        unset($this->ids);
        unset($_SERVER['max_lead']);
    }

    function testBroadcastPayment()
    {
        $skin_resource = getOrCreateSkinAndBrandIfNecessary("lubroadcastskin-lub","lubroadcastbrand",$skin_id,$brand_id);
        SkinSplickitAcceptedPaymentTypeMapsAdapter::staticAddRecord($skin_resource->skin_id,SplickitPaymentService::LEVELUPBROADCASTPAYMENTID);
        setContext($skin_resource->external_identifier);

        $user_resource = createNewUserWithCCNoCVV();
        $user_resource->flags = '1000000001';
        $user_resource->save();
        $user = logTestUserResourceIn($user_resource);

        $merchant_id = $this->ids['merchant_id'];
        $order_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);

        $json_encoded_data = json_encode($order_data);
        $request = createRequestObject("app2/apiv2/carts","POST",$json_encoded_data);
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $cart_resource = $place_order_controller->processV2Request();
        $ucid = $cart_resource->ucid;

        $url = "app2/apiv2/carts/$ucid/checkout";
        $request = createRequestObject($url,"GET");
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_resource = $place_order_controller->processV2Request();

        //  $checkout_resource = getCheckoutResourceFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $accepted_payment_types = $checkout_resource->accepted_payment_types;
        $this->assertCount(1,$accepted_payment_types);
        $this->assertEquals('LevelUp Broadcast',$accepted_payment_types[0]['name']);
        $this->assertEquals(11000,$accepted_payment_types[0]['splickit_accepted_payment_type_id']);

        $place_order_json = '{"merchant_payment_type_map_id":"'.$accepted_payment_types[0]['merchant_payment_type_map_id'].'","requested_time":null}';
        $request = createRequestObject("app2/apiv2/orders/$ucid","POST",$place_order_json);
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $order_resource = $place_order_controller->processV2Request();


        //   $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $bcrecords = BalanceChangeAdapter::staticGetRecords(array("order_id"=>$order_id),'BalanceChangeAdapter');

        $hash = createHashmapFromArrayOfArraysByFieldName($bcrecords,'process');
        $lu_payment_record = $hash['LevelUpBroadcast'];
        $this->assertCount(1,$bcrecords,"it should show the level up broadcast payment");
        $this->assertNotNull($lu_payment_record,'It should have the LU broadcast payment record');

    }

    function testPayment()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user_resource->flags = '1000000001';
        $user_resource->save();
        $user = logTestUserResourceIn($user_resource);

        $merchant_id = $this->ids['merchant_id'];
        $order_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $order_data['tip_amt'] = 8.88;

        $json_encoded_data = json_encode($order_data);
        $request = createRequestObject("app2/apiv2/carts","POST",$json_encoded_data);
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $cart_resource = $place_order_controller->processV2Request();
        $ucid = $cart_resource->ucid;

        $url = "app2/apiv2/carts/$ucid/checkout";
        $request = createRequestObject($url,"GET");
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_resource = $place_order_controller->processV2Request();

      //  $checkout_resource = getCheckoutResourceFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $accepted_payment_types = $checkout_resource->accepted_payment_types;
        $this->assertCount(1,$accepted_payment_types);
        $this->assertEquals('LevelUp Pass Through',$accepted_payment_types[0]['name']);
        $this->assertEquals(10000,$accepted_payment_types[0]['splickit_accepted_payment_type_id']);

        $place_order_json = '{"merchant_payment_type_map_id":"'.$accepted_payment_types[0]['merchant_payment_type_map_id'].'","requested_time":null}';
        $request = createRequestObject("app2/apiv2/orders/$ucid","POST",$place_order_json);
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $order_resource = $place_order_controller->processV2Request();


     //   $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $bcrecords = BalanceChangeAdapter::staticGetRecords(array("order_id"=>$order_id),'BalanceChangeAdapter');

        $hash = createHashmapFromArrayOfArraysByFieldName($bcrecords,'process');
        $lu_payment_record = $hash['LevelUpPassThrough'];
        $this->assertCount(1,$bcrecords,"it should show the level up passthrough payment");
        $this->assertNotNull($lu_payment_record,'It should have the LU pass through payment record');

    }


    function testWebSkinOnlyCCandCash()
    {
        $web_skin_resource = $this->ids['web_skin_resource'];
        setContext($web_skin_resource->external_identifier);

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $merchant_id = $this->ids['merchant_id'];
        $order_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());

        $this->assertNull($checkout_resource->error);
        $accepted_payment_types = $checkout_resource->accepted_payment_types;
        //$this->assertCount(2,$accepted_payment_types);
        $apt_hash = createHashmapFromArrayOfArraysByFieldName($accepted_payment_types,'splickit_accepted_payment_type_id');
        return $apt_hash;
        $this->assertTrue(isset($apt_hash['1000']),'It should have a cash payment type');
        $this->assertTrue(isset($apt_hash['2000']),'It should have a CC payment type');
        $this->assertFalse(isset($apt_hash['11000']),'It should have a CC payment type');
    }

    /**
     * @depends testWebSkinOnlyCCandCash
     */
    function testItShouldHaveCCPaymentType($apt_hash)
    {
        $this->assertTrue(isset($apt_hash['2000']),'It should have a CC payment type');
    }

    /**
     * @depends testWebSkinOnlyCCandCash
     */
    function testItShouldHaveCashPaymentType($apt_hash)
    {
        $this->assertTrue(isset($apt_hash['1000']),'It should have a cash payment type');
    }

    /**
     * @depends testWebSkinOnlyCCandCash
     */
    function testItShouldNOTHavePassthroughPaymentType($apt_hash)
    {
        $this->assertFalse(isset($apt_hash['10000']),'It should NOT have a LEVEL UP PASS THROUGH payment type');
    }

    function testVariablePromo()
    {
        $merchant_id = $this->ids['merchant_id'];

        $promo_data = [];
        $promo_data['key_word'] = 'variable';
        $promo_data['promo_type'] = 6;
        $promo_data['description'] = 'LU discount';
        $promo_data['start_date'] = '2010-01-01';
        $promo_data['end_date'] = '2030-01-01';
        $promo_data['max_use'] = 1000;
        $promo_data['brand_id'] = getBrandIdFromCurrentContext();
        $request = createRequestObject("/app2/admin/promo",'POST',json_encode($promo_data));
        $promo_controller = new PromoController(getM(),null,$request,5);
        $promo_resource = $promo_controller->createPromo();
        $key_word = $promo_resource->promo_key_words[0]->promo_key_word;

        $user_resource = createNewUserWithCCNoCVV();
        $user_resource->flags = '1000000001';
        $user_resource->save();
        $user = logTestUserResourceIn($user_resource);
        $order_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','the note',5);
//        $order_data['promo_code'] = $key_word;
//        $order_data['variable_promo_amt'] = 2.88;

        $pre_checkout_resource = getCheckoutResourceFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $cart_ucid = $pre_checkout_resource->ucid;
        $this->assertNull($pre_checkout_resource->error);
        $pre_tax_amt = $pre_checkout_resource->total_tax_amt;
        $this->assertEquals(0.00,$pre_checkout_resource->promo_tax_amt);
        $this->assertEquals(0.00,$pre_checkout_resource->promo_amt);

        $order_data_promo['promo_code'] = $key_word;
        $order_data_promo['variable_promo_amt'] = 2.88;

        $request = createRequestObject("/app2/apiv2/cart/$cart_ucid/checkout", 'post', json_encode($order_data_promo), 'application/json');
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $promo_checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($promo_checkout_resource->error);
        $this->assertNull($promo_checkout_resource->error);
        $this->assertEquals(-2.88,$promo_checkout_resource->promo_amt);
        $post_tax_amt = $promo_checkout_resource->total_tax_amt;
        $promo_tax_amt = $promo_checkout_resource->promo_tax_amt;
        $this->assertEquals(-.288,$promo_tax_amt);

        $this->assertEquals(round($pre_tax_amt+$promo_tax_amt,2), $post_tax_amt);


        $order_data2 = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
//        $order_data2['promo_code'] = $key_word;
//        $order_data2['variable_promo_amt'] = 2.88;
        $request = createRequestObject("/app2/apiv2/cart/$cart_ucid/checkout", 'post', json_encode($order_data2), 'application/json');
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $new_checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($new_checkout_resource->error);
        $this->assertEquals(0.00,$new_checkout_resource->promo_amt);
        $this->assertNull($new_checkout_resource->promo_code);

        // now hit it back in again
        $order_data3['promo_code'] = $key_word;
        $order_data3['variable_promo_amt'] = 2.88;
        $request = createRequestObject("/app2/apiv2/cart/$cart_ucid/checkout", 'post', json_encode($order_data3), 'application/json');
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $final_checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($final_checkout_resource->error);

        $this->assertEquals(-2.88,$final_checkout_resource->promo_amt);

        $order_resource = placeOrderFromCheckoutResource($final_checkout_resource);
        $this->assertNull($order_resource->error);

        $this->assertEquals(-2.88,$order_resource->promo_amt);
        $this->assertEquals($promo_resource->promo_id,$order_resource->promo_id);
        $this->assertEquals('variable',$order_resource->promo_code);

        return $promo_resource;

    }

    /**
     * @depends testVariablePromo
     */
    function testVariablePromoVivonet($promo_resource)
    {
        $key_word = $promo_resource->promo_key_words[0]->promo_key_word;
        $merchant_id = $this->ids['merchant_id'];
        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_id,'V','vivonet','X');
        $mvima_data = ['merchant_id'=>$merchant_id,'store_id'=>1616688,'promo_charge_id'=>88888,'tender_id'=>436,'service_tip_id'=>12345];
        $this->assignThirdPartyValues($mvima_data);

        $user_resource = createNewUserWithCCNoCVV();
        $user_resource->flags = '1000000001';
        $user_resource->save();
        $user = logTestUserResourceIn($user_resource);
        $order_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','the note',5);
//        $order_data['promo_code'] = $key_word;
//        $order_data['variable_promo_amt'] = 2.88;

        $pre_checkout_resource = getCheckoutResourceFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $cart_ucid = $pre_checkout_resource->ucid;
        $this->assertNull($pre_checkout_resource->error);
        $pre_tax_amt = $pre_checkout_resource->total_tax_amt;
        $this->assertEquals(0.00,$pre_checkout_resource->promo_tax_amt);
        $this->assertEquals(0.00,$pre_checkout_resource->promo_amt);

        $order_data_promo['promo_code'] = $key_word;
        $order_data_promo['variable_promo_amt'] = 2.88;

        $request = createRequestObject("/app2/apiv2/cart/$cart_ucid/checkout", 'post', json_encode($order_data_promo), 'application/json');
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $promo_checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($promo_checkout_resource->error);
        $this->assertNull($promo_checkout_resource->error);
        $this->assertEquals(-2.88,$promo_checkout_resource->promo_amt);
        $post_tax_amt = $promo_checkout_resource->total_tax_amt;
        $promo_tax_amt = $promo_checkout_resource->promo_tax_amt;
        $this->assertEquals(-.288,$promo_tax_amt);
        $this->assertEquals(round($pre_tax_amt+$promo_tax_amt,2), $post_tax_amt);


    }

    function assignThirdPartyValues($vivonet_data)
    {
        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $parameters = $tpppa->getParametersFromProviderId(1003);
        $tpppmma = new ThirdPartyProviderParameterMerchantMapsAdapter(getM());
        foreach ($parameters as $parameter_row) {
            $data = [];
            $data['third_party_provider_parameter_id'] = $parameter_row['third_party_provider_parameter_id'];
            $data['merchant_id'] = $vivonet_data['merchant_id'];
            $parameter_name = $parameter_row['name'];
            switch ($parameter_name) {
                case "store_id":
                    $data['value'] =  $vivonet_data['store_id'];
                    Resource::createByData($tpppmma,$data);
                    break;
                case "merchant_key":
                    $data['value'] = '736de47f4cdee9e8fe9ab8a0bc0eb37e';
                    Resource::createByData($tpppmma,$data);
                    break;
                default:
                    if ($value = $vivonet_data[$parameter_name]) {
                        $data['value'] = $value;
                        Resource::createByData($tpppmma,$data);
                    }
            }
        }
    }



    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        ini_set('max_execution_time', 0);
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction();;


        $skin_resource = getOrCreateSkinAndBrandIfNecessary('LUpassSkin','LUpassBrand',158,null);
        SkinSplickitAcceptedPaymentTypeMapsAdapter::staticAddRecord($skin_resource->skin_id,SplickitPaymentService::LEVELUPPASSTHROUGHPAYMENTID);
        $ids['skin_id'] = $skin_resource->skin_id;
        $ids['passthrough_skin_resource'] = $skin_resource;

        $web_skin_resource = getOrCreateSkinAndBrandIfNecessary('LUWebSkin','LUWebBrand',null,$skin_resource->brand_id);
        SkinSplickitAcceptedPaymentTypeMapsAdapter::staticAddRecord($web_skin_resource->skin_id,SplickitPaymentService::CASHSPLICKITPAYMENTID);
        SkinSplickitAcceptedPaymentTypeMapsAdapter::staticAddRecord($web_skin_resource->skin_id,SplickitPaymentService::CREDITCARDSPLICKITPAYMENTID);
        $ids['web_skin_resource'] = $web_skin_resource;

//map it to a menu
        $menu_id = createTestMenuWithNnumberOfItems(3);
        $ids['menu_id'] = $menu_id;

        $modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 3);
        $modifier_group_id = $modifier_group_resource->modifier_group_id;
        $item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', getM());
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 2);

//*
        $sides_modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 1, 'Sides', 'S');
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[1]['item_id'], $sides_modifier_group_resource->modifier_group_id, 0);

        $mealdeal_modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 1, 'Meal Deal', 'I');
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[2]['item_id'], $mealdeal_modifier_group_resource->modifier_group_id, 0);
//*/
        //$data = ['no_payment'=>true];
        $merchant_resource = createNewTestMerchant($menu_id,$data);
        attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
        attachMerchantToSkin($merchant_resource->merchant_id,$web_skin_resource->skin_id);
        $ids['merchant_id'] = $merchant_resource->merchant_id;

        //create level up passthrough merchant payment type map
        MerchantPaymentTypeMapsAdapter::createMerchantPaymentTypeMap($merchant_resource->merchant_id, 10000, null);
        MerchantPaymentTypeMapsAdapter::createMerchantPaymentTypeMap($merchant_resource->merchant_id, 1000, null);

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main()
    {
        $suite = new PHPUnit_Framework_TestSuite(__CLASS__);
        PHPUnit_TextUI_TestRunner::run($suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    PaymentLevelupPassthroughTest::main();
}

?>