<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class PitaPitCateringTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
        $this->ids = $_SERVER['unit_test_ids'];
        setContext('com.splickit.pitapit');
    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }

    function testGetLeadTimesArrayForCateringDuringOpenHours()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        setProperty("global_throttling", "false");
        $merchant_id = $this->ids['merchant_id'];
        $current_time = getTomorrowTwelveNoonTimeStampDenver();
        //$current_time = time();
        $merchant_advanced_ordering_adapter = new MerchantAdvancedOrderingInfoAdapter(getM());
        $madata['merchant_id'] = $merchant_id;
        $madata['max_days_out'] = 7;
        //$pp_catering_lead = 120;
        //$madata['catering_minimum_lead_time'] = $pp_catering_lead;
        $ma_resource = Resource::createByData($merchant_advanced_ordering_adapter,$madata);
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_data_resource = getCheckoutResourceFromOrderData($cart_data,$current_time);
        $lead_times_array = $checkout_data_resource->lead_times_array;
        $this->assertNotNull($lead_times_array);
        $lead_time = new LeadTime($merchant_id);
        $expected_first_pickup = $current_time+(60 * 60);
        myerror_log("expected first pickup time: ".date('Y-m-d H:i:s',$expected_first_pickup));
        myerror_log("actual first pickup time: ".date('Y-m-d H:i:s',$lead_times_array[0]));
        $this->assertEquals($expected_first_pickup,$lead_times_array[0]);
       // $this->assertEquals("Please note, your first available pickup time for this order is 2 hours from now.",$checkout_data_resource->user_message);

        $last_time = array_pop($lead_times_array);

        myerror_log("actual last pickup time: ".date('Y-m-d H:i:s',$last_time));
        $expected_last_ts = $current_time + (7*24*3600) + $lead_time->pitapit_catering_lead;
        myerror_log("expected last pickup time: ".date('Y-m-d H:i:s',$expected_last_ts));
        $this->assertEquals(date('Y-m-d',$expected_last_ts),date('Y-m-d',$last_time),"It should have the last time as 6 days away");

    }

    function testGetLeadTimesArrayForCateringDuringOpenHours2Items()
    {
        $menu_id = createTestMenuWithNnumberOfItemsAndMenuTypes(1,$menu_id,2);
        $options[TONIC_FIND_BY_METADATA]["menu_id"] = $menu_id;
        $menu_type_resources = Resource::findAll(new MenuTypeAdapter($m),null,$options);
        $menu_type_resource = $menu_type_resources[0];
        $menu_type_resource->cat_id = 'C';
        $menu_type_resource->save();
        $menu = CompleteMenu::getCompleteMenu($menu_id);
        $merchant_resource = createNewTestMerchant($menu_id);
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        setProperty("global_throttling", "false");
        $merchant_id = $merchant_resource->merchant_id;
        $current_time = getTomorrowTwelveNoonTimeStampDenver();
        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleCartArrayByMerchantId($merchant_id, 'pickup', 'some dumb note',1);
        $second_item['item_id'] = $menu['menu_types'][1]['menu_items'][0]['item_id'];
        $second_item['size_id'] = $menu['menu_types'][1]['sizes'][0]['size_id'];
        $second_item['quantity'] = 1;
        $second_item['mods'] = array();
        $order_data['items'][] = $second_item;
        $checkout_data_resource = getCheckoutDataWithThrottling($order_data, $merchant_id, $current_time);
        $lead_times_array = $checkout_data_resource->lead_times_array;
        $this->assertNull($checkout_data_resource->error);
        $this->assertNotNull($lead_times_array);
        $lead_time = new LeadTime($merchant_id);
        $expected_first_pickup = $current_time+($lead_time->pitapit_catering_lead * 60);
        myerror_log("expected first pickup time: ".date('Y-m-d H:i:s',$expected_first_pickup));
        myerror_log("actual first pickup time: ".date('Y-m-d H:i:s',$lead_times_array[0]));
        $this->assertEquals($expected_first_pickup,$lead_times_array[0]);
        $this->assertEquals("Please note, your first available pickup time for this order is 60 minutes from now.",$checkout_data_resource->user_message);
    }




    function testAPIv2()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $merchant_id = $this->ids['merchant_id'];
        $order_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note');

        $json_encoded_data = json_encode($order_data);
        $request = new Request();
        $request->url = '/app2/apiv2/cart/checkout';
        $request->method = "post";
        $request->body = $json_encoded_data;
        $request->mimetype = 'application/json';
        $request->_parseRequestBody();
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        $current_time = getTomorrowTwelveNoonTimeStampDenver();
        $place_order_controller->setCurrentTime($current_time);
        $checkout_data_resource = $place_order_controller->processV2Request();
        $this->assertNull($checkout_data_resource->error);
        $lead_times_array = $checkout_data_resource->lead_times_array;
        $this->assertNotNull($lead_times_array);

        $lead_time = new LeadTime($merchant_id);
        $expected_first_pickup = $current_time+($lead_time->pitapit_catering_lead * 60);

        myerror_log("expected first pickup time: ".date('Y-m-d h:i:s',$expected_first_pickup));
        myerror_log("actual first pickup time: ".date('Y-m-d h:i:s',$lead_times_array[0]));
        $this->assertEquals(date('Y-m-d h:i:s',$expected_first_pickup),date('Y-m-d h:i:s',$lead_times_array[0]));
        $this->assertEquals($expected_first_pickup,$lead_times_array[0]);
        $this->assertEquals("Please note, your first available pickup time for this order is 60 minutes from now.",$checkout_data_resource->user_message);

    }


    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time',300);
              SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();$mysqli->begin_transaction(); ;

        setContext('com.splickit.pitapit');
        $ids['skin_id'] = $skin_resource->skin_id;



        // create catering menu with single regular menu type
        $menu_id = createTestCateringMenuWithOneItem();

        // add a regular section
//        $menu_type_resource = createNewMenuType($menu_id, 'Regular Menu Type','E');
//        $size_resource = createNewSize($menu_type_resource->menu_type_id, 'Test Size 1');
//        createItem("Regular Item", $size_resource->size_id, $menu_type_resource->menu_type_id);

        $ids['menu_id'] = $menu_id;


        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_resource->advanced_ordering = 'Y';
        $merchant_resource->save();
        attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
        $ids['merchant_id'] = $merchant_resource->merchant_id;

        $user_resource = createNewUser(array("flags"=>"1C20000001"));
        $ids['user_id'] = $user_resource->user_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    PitaPitCateringTest::main();
}

?>