<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class PitaPitStoredValueTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
		$this->ids = $_SERVER['unit_test_ids'];
		setContext("com.splickit.pitapit");
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }
    
    function testGetSvaNumberFromUserBrandPointsMapRecord()
    {
    	$_SERVER['REQUEST_METHOD'] = 'POST';
    	$loyalty_number = '123456789055555';
    	$user_resource = createNewUser(array("loyalty_number"=>$loyalty_number,"pin"=>'1234'));
    	
    	$pitcard_payment_service = new PitCardStoredValuePaymentService();
    	$pitcard_payment_service->setBillingUserIdAndResourceFromUserId($user_resource->user_id);
    	$saved_loyalty_number = $pitcard_payment_service->getBillingUsersAccountNumber();
    	$this->assertEquals($loyalty_number.':1234:;5022440100000000118=380100071527KATANA?', $saved_loyalty_number);

       	$loyalty_number = '123456789088888';
    	$user_resource = createNewUser(array("loyalty_number"=>$loyalty_number,"pin"=>'1234'));
    	
    	$pitcard_payment_service = new PitCardStoredValuePaymentService();
    	$pitcard_payment_service->setBillingUserIdAndResourceFromUserId($user_resource->user_id);
    	$saved_loyalty_number = $pitcard_payment_service->getBillingUsersAccountNumber();
    	$this->assertEquals($loyalty_number.':1234:;5022440100000000118=380100071527KATANA?', $saved_loyalty_number);
    }
    
    function testProcessPaymentWithPitCardStoredValuePaymentService()
    {
		//create user with heartland account
    	$_SERVER['REQUEST_METHOD'] = 'POST';
    	$loyalty_number = '123456789055555';
    	$user_resource = createNewUser(array("loyalty_number"=>$loyalty_number,"pin"=>'1234'));
    	
    	$user = logTestUserResourceIn($user_resource);
    	
    	$pcsvps = new PitCardStoredValuePaymentService();
    	$pcsvps->setBillingUserIdAndResourceFromUserId($user['user_id']);
    	$results = $pcsvps->processPayment(30.25);
    	$this->assertEquals(100, $results['response_code']);
    	$this->assertNotNull($results['order']);
    	$this->assertEquals($results['order'], $results['transactionid']);
    	$this->assertEquals("Points 500,USD 19.75", $results['sva.balances']);
    }

	function testPayWithStoredValueCard()
    {
    	//create merchant
    	$merchant_resource = createNewTestMerchant($this->ids['menu_id'],array("no_payment"=>true));
    	$merchant_resource->brand_id = 282;
    	$merchant_resource->save();
    	Resource::createByData(new MerchantHeartlandInfoMapsAdapter($mimetypes), array("merchant_id"=>$merchant_resource->merchant_id,"heartland_store_id"=>"us-1000"));
    	$merchant_id = $merchant_resource->merchant_id;
    	
    	//set merchant to take stored value
    	$merchant_payment_type_map_adapter = new MerchantPaymentTypeMapsAdapter($mimetypes);
    	$merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_id, 3000, $billing_entity_id);
    	
		//create user with heartland account
    	$_SERVER['REQUEST_METHOD'] = 'POST';
    	$loyalty_number = '123456789055555';
    	$user_resource = createNewUser(array("loyalty_number"=>$loyalty_number,"pin"=>'1234',"flags"=>"1C20000001"));
    	
    	$user = logTestUserResourceIn($user_resource);
    	$user_id = $user['user_id'];
    	
    	$balance_before = $user_resource->balance;
    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours');
    	$checkout_data_resource = getCheckoutDataWithThrottling($order_data, $merchant_id, $time_stamp);
    	
    	$payments_array = $checkout_data_resource->accepted_payment_types;
    	$payments_array = createHashmapFromArrayOfArraysByFieldName($payments_array, 'name');
    	$this->assertNotNull($payments_array['Pit Card']);
    	$pit_card_payment_type = $payments_array['Pit Card'];
    	$order_data['merchant_payment_type_map_id'] = $pit_card_payment_type['merchant_payment_type_map_id'];
    	$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
    	$order_id = $order_resource->order_id;
		$this->assertNull($order_resource->error);
		$this->assertTrue($order_resource->order_id > 1000);
		$this->assertEquals('PitCardStoredValuePaymentService', $order_resource->payment_service_used);
		
		$balance_change_adapter = new BalanceChangeAdapter($mimetypes);
		if ($balance_change_records = $balance_change_adapter->getRecords(array("order_id"=>$order_id), $options)) {
			$balance_change_rows_by_user_id = setBalanceChangeHashFromBalanceChangeArrayFromOrder($balance_change_records);
		}
		$this->assertCount(2, $balance_change_rows_by_user_id);
		$this->assertEquals($balance_before, $balance_change_rows_by_user_id["$user_id-Order"]['balance_before']);
		$this->assertEquals($order_resource->grand_total, -$balance_change_rows_by_user_id["$user_id-Order"]['charge_amt']);
		$this->assertEquals($balance_before-$order_resource->grand_total, $balance_change_rows_by_user_id["$user_id-Order"]['balance_after']);
		
		$this->assertEquals($balance_before-$order_resource->grand_total, $balance_change_rows_by_user_id["$user_id-StoredValuePayment"]['balance_before']);
		$this->assertEquals($balance_change_rows_by_user_id["$user_id-StoredValuePayment"]['balance_before'], -$balance_change_rows_by_user_id["$user_id-StoredValuePayment"]['charge_amt']);
		$this->assertEquals('PitCardStoredValuePaymentService', $balance_change_rows_by_user_id["$user_id-StoredValuePayment"]['cc_processor']);
		
    }
    
	function testPayWithStoredValueCardExistingNegativeBalanceOnUser()
    {
    	//create merchant
    	$merchant_resource = createNewTestMerchant($this->ids['menu_id'],array("no_payment"=>true));
    	$merchant_resource->brand_id = 282;
    	$merchant_resource->save();
    	Resource::createByData(new MerchantHeartlandInfoMapsAdapter($mimetypes), array("merchant_id"=>$merchant_resource->merchant_id,"heartland_store_id"=>"us-1000"));
    	$merchant_id = $merchant_resource->merchant_id;
    	
    	//set merchant to take stored value
    	$merchant_payment_type_map_adapter = new MerchantPaymentTypeMapsAdapter($mimetypes);
    	$merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_id, 3000, $billing_entity_id);
    	
		//create user with heartland account
    	$_SERVER['REQUEST_METHOD'] = 'POST';
    	$loyalty_number = '123456789055555';
    	$user_resource = createNewUser(array("loyalty_number"=>$loyalty_number,"pin"=>'1234',"flags"=>"1C20000001"));
    	$user_resource->balance = "-5.35";
    	$user_resource->save();
    	
    	$user = logTestUserResourceIn($user_resource);
    	$user_id = $user['user_id'];
    	
    	$balance_before = $user_resource->balance;
    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours');
    	$checkout_data_resource = getCheckoutDataWithThrottling($order_data, $merchant_id, $time_stamp);
    	
    	$payments_array = $checkout_data_resource->accepted_payment_types;
    	$payments_array = createHashmapFromArrayOfArraysByFieldName($payments_array, 'name');
    	$this->assertNotNull($payments_array['Pit Card']);
    	$pit_card_payment_type = $payments_array['Pit Card'];
    	
    	$order_data['merchant_payment_type_map_id'] = $pit_card_payment_type['merchant_payment_type_map_id'];
    	$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
    	$order_id = $order_resource->order_id;
		$this->assertNull($order_resource->error);
		$this->assertTrue($order_resource->order_id > 1000);
		$this->assertEquals('PitCardStoredValuePaymentService', $order_resource->payment_service_used);
		
		$balance_change_adapter = new BalanceChangeAdapter($mimetypes);
		if ($balance_change_records = $balance_change_adapter->getRecords(array("order_id"=>$order_id), $options)) {
			$balance_change_rows_by_user_id = setBalanceChangeHashFromBalanceChangeArrayFromOrder($balance_change_records);
		}
		$this->assertCount(2, $balance_change_rows_by_user_id);
		$this->assertEquals($balance_before, $balance_change_rows_by_user_id["$user_id-Order"]['balance_before']);
		$this->assertEquals($order_resource->grand_total, -$balance_change_rows_by_user_id["$user_id-Order"]['charge_amt']);
		$this->assertEquals($balance_before-$order_resource->grand_total, $balance_change_rows_by_user_id["$user_id-Order"]['balance_after']);
		
		$this->assertEquals($balance_before-$order_resource->grand_total, $balance_change_rows_by_user_id["$user_id-StoredValuePayment"]['balance_before']);
		$this->assertEquals($order_resource->grand_total, $balance_change_rows_by_user_id["$user_id-StoredValuePayment"]['charge_amt']);
		$this->assertEquals('PitCardStoredValuePaymentService', $balance_change_rows_by_user_id["$user_id-StoredValuePayment"]['cc_processor']);
		$this->assertEquals($balance_before, $balance_change_rows_by_user_id["$user_id-StoredValuePayment"]['balance_after'],"previous balance should not have been affected");
		
		$user_resource_after = SplickitController::getResourceFromId($user_id, 'User');
		$this->assertEquals($balance_before, $user_resource_after->balance,"balance on user record should not have been affected");
		
    }
    
 	function testPayWithStoredValueCardExistingCreditBalanceOnUser()
    {
    	//create merchant
    	$merchant_resource = createNewTestMerchant($this->ids['menu_id'],array("no_payment"=>true));
    	$merchant_resource->brand_id = 282;
    	$merchant_resource->save();
    	Resource::createByData(new MerchantHeartlandInfoMapsAdapter($mimetypes), array("merchant_id"=>$merchant_resource->merchant_id,"heartland_store_id"=>"us-1000"));
    	$merchant_id = $merchant_resource->merchant_id;
    	
    	//set merchant to take stored value
    	$merchant_payment_type_map_adapter = new MerchantPaymentTypeMapsAdapter($mimetypes);
    	$merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_id, 3000, $billing_entity_id);
    	
		//create user with heartland account
    	$_SERVER['REQUEST_METHOD'] = 'POST';
    	$loyalty_number = '123456789055555';
    	$user_resource = createNewUser(array("loyalty_number"=>$loyalty_number,"pin"=>'1234',"flags"=>"1C20000001"));
    	$user_resource->balance = "5.00";
    	$user_resource->save();
    	
    	$user = logTestUserResourceIn($user_resource);
    	$user_id = $user['user_id'];
    	
    	$balance_before = $user_resource->balance;
    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours');
    	$checkout_data_resource = getCheckoutDataWithThrottling($order_data, $merchant_id, $time_stamp);
    	
    	$payments_array = $checkout_data_resource->accepted_payment_types;
    	$payments_array = createHashmapFromArrayOfArraysByFieldName($payments_array, 'name');
    	$this->assertNotNull($payments_array['Pit Card']);
    	$pit_card_payment_type = $payments_array['Pit Card'];
    	
    	$order_data['merchant_payment_type_map_id'] = $pit_card_payment_type['merchant_payment_type_map_id'];
    	$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
    	$order_id = $order_resource->order_id;
		$this->assertNull($order_resource->error);
		$this->assertTrue($order_resource->order_id > 1000);
		$this->assertEquals('PitCardStoredValuePaymentService', $order_resource->payment_service_used);
		
		$balance_change_adapter = new BalanceChangeAdapter($mimetypes);
		if ($balance_change_records = $balance_change_adapter->getRecords(array("order_id"=>$order_id), $options)) {
			$balance_change_rows_by_user_id = setBalanceChangeHashFromBalanceChangeArrayFromOrder($balance_change_records);
		}
		$this->assertCount(2, $balance_change_rows_by_user_id);
		$this->assertEquals($balance_before, $balance_change_rows_by_user_id["$user_id-Order"]['balance_before']);
		$this->assertEquals($order_resource->grand_total, -$balance_change_rows_by_user_id["$user_id-Order"]['charge_amt']);
		$this->assertEquals($balance_before-$order_resource->grand_total, $balance_change_rows_by_user_id["$user_id-Order"]['balance_after']);
		
		$this->assertEquals($balance_before-$order_resource->grand_total, $balance_change_rows_by_user_id["$user_id-StoredValuePayment"]['balance_before']);
		$this->assertEquals($order_resource->grand_total - $balance_before, $balance_change_rows_by_user_id["$user_id-StoredValuePayment"]['charge_amt']);
		$this->assertEquals('PitCardStoredValuePaymentService', $balance_change_rows_by_user_id["$user_id-StoredValuePayment"]['cc_processor']);
		$this->assertEquals(0.00, $balance_change_rows_by_user_id["$user_id-StoredValuePayment"]['balance_after'],"previous balance should not have been wiped out");
		
		$user_resource_after = SplickitController::getResourceFromId($user_id, 'User');
		$this->assertEquals(0.00, $user_resource_after->balance,"balance on user record should not have been set to zero");
		
    }
    
	function testPayWithStoredValueCardInsufficientFunds()
    {
    	//create merchant
    	$merchant_resource = createNewTestMerchant($this->ids['menu_id'],array("no_payment"=>true));
    	$merchant_resource->brand_id = 282;
    	$merchant_resource->save();
    	Resource::createByData(new MerchantHeartlandInfoMapsAdapter($mimetypes), array("merchant_id"=>$merchant_resource->merchant_id,"heartland_store_id"=>"us-1000"));
    	$merchant_id = $merchant_resource->merchant_id;
    	
    	//set merchant to take stored value
    	$merchant_payment_type_map_adapter = new MerchantPaymentTypeMapsAdapter($mimetypes);
    	$merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_id, 3000, $billing_entity_id);
    	
		//create user with heartland account
    	$_SERVER['REQUEST_METHOD'] = 'POST';
    	$loyalty_number = '123456789033333';
    	$user_resource = createNewUser(array("loyalty_number"=>$loyalty_number,"pin"=>'1234',"flags"=>"1C20000001"));
    	
    	$user = logTestUserResourceIn($user_resource);
    	$user_id = $user['user_id'];
    	
    	$balance_before = $user_resource->balance;
    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours');
    	$checkout_data_resource = getCheckoutDataWithThrottling($order_data, $merchant_id, $time_stamp);
    	$total = $checkout_data_resource->order_amt + $checkout_data_resource->total_tax_amt;
    	$payments_array = $checkout_data_resource->accepted_payment_types;
    	$payments_array = createHashmapFromArrayOfArraysByFieldName($payments_array, 'name');
    	$this->assertNotNull($payments_array['Pit Card']);
    	$pit_card_payment_type = $payments_array['Pit Card'];
    	
    	$order_data['merchant_payment_type_map_id'] = $pit_card_payment_type['merchant_payment_type_map_id'];
    	$order_data['tip'] = '0.00';
    	$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
    	$order_id = $order_resource->order_id;
		$this->assertNotNull($order_resource->error);
		$expected_message = 'For the account [123456789033333] cannot pay the amount ['.$total.' USD] against the current balance [1.50 USD].';
		$this->assertEquals($expected_message, $order_resource->error);		
    }

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

    	ini_set('max_execution_time',0);
    	$_SERVER['request_time1'] = microtime(true);
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	ini_set('max_execution_time',300);
    	      SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();$mysqli->begin_transaction(); ;
    	
    	$skin_resource = createWorldHqSkin();
    	$ids['skin_id'] = $skin_resource->skin_id;
    	
		//map it to a menu
    	$menu_id = createTestMenuWithNnumberOfItems(5);
    	$ids['menu_id'] = $menu_id;
    	
    	$modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 10);
    	$modifier_group_id = $modifier_group_resource->modifier_group_id;
    	$item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 2);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[1]['item_id'], $modifier_group_id, 4);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[2]['item_id'], $modifier_group_id, 1);

    	$merchant_resource = createNewTestMerchant($menu_id);
    	attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
    	$ids['merchant_id'] = $merchant_resource->merchant_id;
    	Resource::createByData(new MerchantHeartlandInfoMapsAdapter($mimetypes), array("merchant_id"=>$merchant_resource->merchant_id,"heartland_store_id"=>"us-1000"));

    	$user_resource = createNewUser(array("flags"=>"1C20000001"));
    	$ids['user_id'] = $user_resource->user_id;
    	
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    PitaPitStoredValueTest::main();
}

?>