<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class PitapitLoyaltyTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
		$this->ids = $_SERVER['unit_test_ids'];
		setContext('com.splickit.pitapit');
		
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
		//setSessionProperty("fake_prod", false);
    }

//    function testRealFutureOrder()
//    {
//        $complete_order = CompleteOrder::staticGetCompleteOrder(280451);
//    }
//
//    function testFutureOrder()
//    {
//        $user_resource = createNewUserWithCCNoCVV();
//        $user = logTestUserResourceIn($user_resource);
//        $merchant_id = $this->ids['merchant_id'];
//        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
//        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
//        $this->assertNull($checkout_resource->error);
//        $order_resource = placeOrderFromCheckoutResource($checkout_resource);
//        $this->assertNull($order_resource->error);
//    }

	function testGetPitaPitValues()
	{
		$domain = 'pitapit';
		$terminal = '0001';
		$terminal_order = '0001';
		$terminal_version = 'v17';
		$api_key = "qwertyuiop";
		$heartland_loyalty_service = new HeartlandLoyaltyService($api_key,$domain,$terminal,$terminal_order,$terminal_version);
		$this->assertEquals('pitapit',$heartland_loyalty_service->domain,"should have grabbed the domain from the context");
		$this->assertEquals('0001',$heartland_loyalty_service->terminal);
		$this->assertEquals('0001',$heartland_loyalty_service->terminal_order);
		$this->assertEquals('v17',$heartland_loyalty_service->terminal_version);
	}

	function testHeartlandLoyaltyServiceFactory()
	{
		$heartland_loyalty_service = PitapitLoyaltyController::loyaltyServiceFactory();
		$this->assertEquals('pitapit',$heartland_loyalty_service->domain);
		$this->assertEquals('0001',$heartland_loyalty_service->terminal);
		$this->assertEquals('0001',$heartland_loyalty_service->terminal_order);
		$this->assertEquals('v17',$heartland_loyalty_service->terminal_version);
	}

    function testGetPinAndSVAFromLoyaltyNubmerNoPin()
    {
    	$loyalty_number = '45456ert546';
    	$pplc = new PitapitLoyaltyController($mimetypes, $user, $request);
    	$pplc->loyalty_number = $loyalty_number;
    	$creds = $pplc->getSvaAndPinFromLoyaltyNumber();
    	$this->assertEquals($loyalty_number, $creds['sva']);
    	$this->assertnull($creds['pin']);
    }

    function testGetPinSVAQrcodeFromLoyaltyNubmer()
    {
    	$loyalty_number = '45456ert546:8888:;6277204114009518=380100071527KATANA?';
    	$pplc = new PitapitLoyaltyController($mimetypes, $user, $request);
    	$pplc->loyalty_number = $loyalty_number;
    	$creds = $pplc->getSvaAndPinFromLoyaltyNumber();
    	$this->assertEquals('45456ert546', $creds['sva']);
    	$this->assertEquals('8888', $creds['pin']);
    	$this->assertEquals(';6277204114009518=380100071527KATANA?', $creds['qrcode_number']);	
    }
    
    function testJoinLoyalty()
    {
    	$user_resource = createNewUser();
    	$user = logTestUserResourceIn($user_resource);
    	$_SERVER['PHP_AUTH_PW'] = 'welcome';
    	
    	$request = new Request();
    	$data['create_loyalty_account'] = true;
    	$request->body = json_encode($data);
    	$request->method = 'POST';
    	$request->mimetype = 'application/json';
    	$request->url = "apiv2/users/".$user['uuid'];
    	$request->_parseRequestBody();
    	$user_controller = new UserController($mt, $user, $request);
    	$response = $user_controller->processV2Request();
    	
    	$user = logTestUserIn($user['user_id']);
    	$ubpma = new UserBrandPointsMapAdapter($mimetypes);
    	$ubpm_record = $ubpma->getRecord(array("user_id"=>$user['user_id'],"brand_id"=>282));
		$this->assertNotNull($ubpm_record,"should have found a user brand points map record");
		$loyalty_number = $ubpm_record['loyalty_number'];
		$ln = explode(":", $loyalty_number);
		$this->assertCount(3, $ln);
		$this->assertNotNull($ln[0]);
		$this->assertNotNull($ln[1]);
		$this->assertNotNull($ln[2]);
		$this->assertContains('KATANA?', $ln[2]);
		
    }

    function testValidationOfBadLoyaltyNumber()
    {
    	//setSessionProperty("fake_prod", true);
    	$pitapit_loyalty_controler = LoyaltyControllerFactory::getLoyaltyControllerForContext();
    	$pitapit_loyalty_controler->setLoyaltyData(array("loyalty_number"=>'123456765432','pin'=>'1324'));
    	$this->assertEquals('123456765432', $pitapit_loyalty_controler->getLoyaltyNumber());
    	$this->assertTrue(is_a($pitapit_loyalty_controler, 'PitapitLoyaltyController'));
    	$this->assertFalse($pitapit_loyalty_controler->validateLoadedLoyaltyNumber());
    }

    function testFailLinkingOfCardOnNoPin()
    {
        $user_resource = createNewUser();
        $user = logTestUserResourceIn($user_resource);
        $request = new Request();
        $request->body = '{"loyalty_number":"6277204111736287","pin":""}';
        $request->method = 'POST';
        $request->mimetype = 'application/json';
        $request->url = "apiv2/users/".$user['uuid'];
        $request->_parseRequestBody();
        $user_controller = new UserController($mt, $user, $request);
        $response = $user_controller->processV2Request();
        $this->assertEquals(401,$response->http_code);
        $this->assertEquals("Sorry you must enter both loyalty number and pin in order to link to an existing loyalty account.",$response->error);

    }
    
    function testUserAdapterCreateSplickitAccountLinkWithGoodLoyaltyNumber()
    {
    	//setSessionProperty("fake_prod", true);
    	$_SERVER['REQUEST_METHOD'] = 'POST';
    	$loyalty_number = '123456789088888';
    	$user_resource = createNewUser(array("loyalty_number"=>$loyalty_number,"pin"=>'4444'));
    	$this->assertNull($user_resource->error);
    	$this->assertTrue($user_resource->_exists);

    	// should have created a user brand loyalty record
    	$user_brand_points_map_adapter = new UserBrandPointsMapAdapter($mimetypes);
    	$user_brand_loyalty_record = $user_brand_points_map_adapter->getRecord(array("user_id"=>$user_resource->user_id,"brand_id"=>282));
    	$this->assertNotNull($user_brand_loyalty_record);
    	$this->assertEquals($loyalty_number.':4444:;1441441441441441=380100071527KATANA?',$user_brand_loyalty_record['loyalty_number']);
    	return $user_resource->user_id;
    }    
    
    /**
     * @depends testUserAdapterCreateSplickitAccountLinkWithGoodLoyaltyNumber
     */
    function testGetUserSessionWithLoyalty($user_id)
    {
    	$user_brand_points_map_adapter = new UserBrandPointsMapAdapter($mimetypes);
    	$user_brand_loyalty_record = $user_brand_points_map_adapter->getRecord(array("user_id"=>$user_id,"brand_id"=>282));
    	$saved_points = $user_brand_loyalty_record['points'];
    	$loyalty_number_field = $user_brand_loyalty_record['loyalty_number'];
    	$ubl = explode(":", $loyalty_number_field);
    	$loyalty_number = $ubl[0];
    	$pin = $ubl[1];
    	$qrcode = $ubl[2];
    	
    	$user = logTestUserIn($user_id);
    	$user_resource = SplickitController::getResourceFromId($user_id, 'User');
    	$user_session_controller = new UsersessionController($mt, $user, $r, 5);
    	$user_session = $user_session_controller->getUserSession($user_resource);
    	$this->assertNotNull($user_session->brand_loyalty);
    	$brand_loyalty = $user_session->brand_loyalty;
    	$this->assertEquals($saved_points, $brand_loyalty['loyalty_points']);
    	$this->assertEquals($loyalty_number,$brand_loyalty['loyalty_number']);
    	$this->assertEquals($pin,$brand_loyalty['pin']);
    	$this->assertEquals($qrcode, $brand_loyalty['barcode_number']);
    	$this->assertEquals($qrcode, $brand_loyalty['qrcode_number']);
    	$this->assertTrue(is_array($brand_loyalty['loyalty_transactions']));
    	return $user_id;
    }
    
    /**
     * @depends testGetUserSessionWithLoyalty
     */
    function testGetUserSessionWithOldLoyaltyNumberWithDeleteOfRecord($user_id)
    {
    	$user_brand_points_map_adapter = new UserBrandPointsMapAdapter($mimetypes);
    	$ubpm_resource = $user_brand_points_map_adapter->getExactResourceFromData(array("user_id"=>$user_id,"brand_id"=>282));
    	$loyalty_number_field = $ubpm_resource->loyalty_number;
    	$ubl = explode(":", $loyalty_number_field);
    	$loyalty_number = $ubl[0];
    	$ubpm_resource->loyalty_number = $loyalty_number;
    	$ubpm_resource->save();
    	
    	$user = logTestUserIn($user_id);
    	$user_resource = SplickitController::getResourceFromId($user_id, 'User');
    	$user_session_controller = new UsersessionController($mt, $user, $r, 5);
    	$user_session = $user_session_controller->getUserSession($user_resource);
    	$this->assertNull($user_session->brand_loyalty,"Should no longer have found a loyalty seciton on the user session");
		
    	// now to check to see if the local loyalty record was deleted
    	$user_brand_points_map_adapter = new UserBrandPointsMapAdapter($mimetypes);
    	$user_brand_loyalty_record = $user_brand_points_map_adapter->getRecord(array("user_id"=>$user_id,"brand_id"=>282));
    	$this->assertNull($user_brand_loyalty_record,"Local loyalty record should have been deleted");
    	
    	$this->assertNotNull($user_session->user_message);
    	$this->assertContains("Please Note: We have fully integrated our loyalty functionality now, BUT....you will need to link your account again with both loyalty number AND pin. Sorry for the inconvenience.", $user_session->user_message);
    }

    function testUserAdapterUpdateSplickitAccountLinkWithGoodLoyaltyNumber()
    {
    	//setSessionProperty("fake_prod", true);
    	$_SERVER['REQUEST_METHOD'] = 'POST';
    	$loyalty_number = 'old-3456788888';
    	$user_resource = createNewUserWithCC();
    	setSessionProperty('is_a_user_update', 'true');
    	$user_resource->loyalty_number = $loyalty_number;
    	$user_resource->pin = '1234';
    	$user_resource->save();
    	
    	// should have created a user brand loyalty record
    	$user_brand_points_map_adapter = new UserBrandPointsMapAdapter($mimetypes);
    	$user_brand_loyalty_record = $user_brand_points_map_adapter->getRecord(array("user_id"=>$user_resource->user_id,"brand_id"=>282));
    	$this->assertNotNull($user_brand_loyalty_record);
    	$this->assertEquals($loyalty_number.':1234:;5022440100000000118=380100071527KATANA?',$user_brand_loyalty_record['loyalty_number']);
    	
    	$loyalty_number2 = 'new-ju7656g77777';
    	setSessionProperty('is_a_user_update', 'true');
    	$user_resource->loyalty_number = $loyalty_number2;
    	$user_resource->pin = '3333';
    	$result = $user_resource->save();
    	$this->assertFalse($result);
    	$this->assertEquals("We're sorry, the pit card number you entered, new-ju7656g77777:3333, appears to be invalid, please check your entry.", $user_resource->error);
    	
    	$loyalty_number2 = 'new-ju7656g88888';
    	setSessionProperty('is_a_user_update', 'true');
    	$user_resource->loyalty_number = $loyalty_number2;
    	$user_resource->pin = '3333';
    	$result = $user_resource->save();
    	$this->assertTrue($result);
    	
    	$user_brand_points_map_adapter = new UserBrandPointsMapAdapter($mimetypes);
    	$user_brand_loyalty_record = $user_brand_points_map_adapter->getRecord(array("user_id"=>$user_resource->user_id,"brand_id"=>282));
    	$this->assertNotNull($user_brand_loyalty_record);
    	$this->assertEquals($loyalty_number2.':3333:;1331331331331331=380100071527KATANA?',$user_brand_loyalty_record['loyalty_number']);
    	return $user_resource;
    }
    
    /**
     * @depends testUserAdapterUpdateSplickitAccountLinkWithGoodLoyaltyNumber
     */
    function testGetUserSessionWithPitaPitLoyaltyInfo($user_resource)
    {
    	$user = logTestUserResourceIn($user_resource);
    	$user_id = $user['user_id'];
    	$usc = new UsersessionController($mt, $user, $r, 5);
    	$user_session_resource = $usc->getUserSession($user_resource);
    	$this->assertNotNull($user_session_resource);
    	$this->assertNotNull($user_session_resource->brand_loyalty);
    	$brand_loyalty = $user_session_resource->brand_loyalty;
    	$this->assertEquals('new-ju7656g88888', $brand_loyalty['loyalty_number']);
    	$this->assertEquals('3333', $brand_loyalty['pin']);
    	$this->assertEquals(';1331331331331331=380100071527KATANA?', $brand_loyalty['qrcode_number']);

    	$loyalty_controller = LoyaltyControllerFactory::getLoyaltyControllerForContext();
    	$this->assertTrue(is_a($loyalty_controller, 'PitapitLoyaltyController'));
    	$loyalty_user_session_data = $loyalty_controller->getAccountInfoForUserSession();
    	$this->assertEquals('new-ju7656g88888', $loyalty_user_session_data['loyalty_number']);
    	$this->assertEquals('3333', $loyalty_user_session_data['pin']);
    	$this->assertEquals(';1331331331331331=380100071527KATANA?', $loyalty_user_session_data['qrcode_number']);
    	$this->assertEquals($skin_resource->brand_id, $loyalty_user_session_data['brand_id']);
    	$this->assertEquals(1500,$loyalty_user_session_data['points']);
    	$this->assertTrue(is_array($loyalty_user_session_data['loyalty_transactions']));
    	$this->assertEquals($user_session_resource->brand_loyalty, $loyalty_user_session_data);
    	return $user_id;
    }
    
    /**
     * @depends testGetUserSessionWithPitaPitLoyaltyInfo
     */
    function testGetPointsForOrder($user_id)
    {
    	
    	$user = logTestUserIn($user_id);
    	$merchant_id = $this->ids['merchant_id'];
    	
    	$id = MerchantHeartlandInfoMapsAdapter::getHeartlandStoreId($merchant_id);
    	
    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', $note);
    	$order_resource = placeOrderFromOrderData($order_data, getTodayTwelveNoonTimeStampDenver());
    	$this->assertNull($order_resource->error);
    	$order_id = $order_resource->order_id;
    	$order_amt = $order_resource->order_amt;
    	$expected_earned_points = round($order_amt); 
    	$this->assertEquals("You earned $expected_earned_points points on this order.", $order_resource->loyalty_message);
    }
    
    function testPayWithLoyaltyPoints()
    {
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$_SERVER['REQUEST_METHOD'] = 'POST';
    	$loyalty_number = '12345623488888';
    	$user_resource = createNewUser();
    	setSessionProperty('is_a_user_update', 'true');
    	$user_resource->loyalty_number = $loyalty_number;
    	$user_resource->pin = '1234';
    	$user_resource->save();
		$user = logTestUserResourceIn($user_resource);
		$user_id = $user['user_id'];
		
		$ubpma = new UserBrandPointsMapAdapter($mimetypes);
    	$ubpm_record = $ubpma->getRecord(array("user_id"=>$user_id,"brand_id"=>282));

		// now adjust point values
		$user_brand_points_map_resource = Resource::find(new UserBrandPointsMapAdapter($mimetypes),$ubpm_record['map_id']);
		$this->assertEquals(1500, $user_brand_points_map_resource->points);
		    	
		$order_adapter = new OrderAdapter($mimetypes);
    	//$order = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'skip hours');
    	$order = $order_adapter->getSimpleCartArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'skip hours');
    	
    	// now add the points stuff to the item
    	$item = $order['items'][0];
//    	$item['uuid'] = 10;
    	$item['points_used'] = 50;
 //   	$item['amt_off'] = 1.65;
    	$item['brand_points_id'] = $this->ids['brand_points_id'];
    	$order['items'][0] = $item;
    	$order['total_points_used'] = 50;
    	$order['loyalty_number'] = $loyalty_number;
    	
    	$order_json = json_encode($order);

    	$pickup_string = date('g:ia',$pickup_time);

    	$checkout_resource = getCheckoutDataWithThrottling($order, $merchant_id, getTodayTwelveNoonTimeStampDenver());
    	$this->assertTrue(isset($checkout_resource->lead_times_array));
    	$this->assertEquals(0.00, $checkout_resource->order_amt);
    	$this->assertEquals(0.00, $checkout_resource->total_tax_amt);
    	
    	$request = new Request();
    	$request->method = 'POST';
    	$request->url = 'localhost/app2/apiv2/cart/checkout';
    	$request->data = $order;
    	$place_order_controller = new PlaceOrderController($mt, $_SERVER['AUTHENTICATED_USER'], $request,5);
    	$place_order_controller->setCurrentTime(getTodayTwelveNoonTimeStampDenver());
    	$checkout_data_resource_api_v2 = $place_order_controller->processV2Request();
    	$this->assertTrue(isset($checkout_data_resource_api_v2->lead_times_array));
    	$this->assertEquals(0.00, $checkout_data_resource_api_v2->order_amt,"subtotal should have been zero because of pay with points");
    	$this->assertEquals(0.00, $checkout_data_resource_api_v2->total_tax_amt,"tax should have been zero because of pay with points");
    	
    	$order['tip'] = '0.00';
    	$order['tax_amt'] = '0.00';
    	$order['sub_total'] = '0.00';
    	$order['grand_total'] = '0.00';
    	$order['lead_time'] = 20;
    	$pickup_time = getTodayTwelveNoonTimeStampDenver()+1200;
    	$order['actual_pickup_time'] = $pickup_time;
    	
    	$order_resource = placeOrderFromOrderData($order, getTodayTwelveNoonTimeStampDenver());
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_resource->order_id > 1000,"should have created a good order id but got: ".$order_resource->error);
    	$this->assertNull($order_resource->error);
    	
    	$this->assertEquals(0.00, $order_resource->order_amt);
    	$this->assertEquals(0.00, $order_resource->total_tax_amt);
    	
		// now check to see if the correct amount was recorded in the history		
		$ublh_adapter = new UserBrandLoyaltyHistoryAdapter($mimetypes);
		$ublh_record = $ublh_adapter->getRecord(array("user_id"=>$user_id,"order_id"=>$order_id));
		$this->assertNotNull($ublh_record);
		$this->assertEquals(50, $ublh_record['points_redeemed']);		
   	}
   	
   	function testPayWithPointsWithSecondCashItem()
   	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$_SERVER['REQUEST_METHOD'] = 'POST';
    	$loyalty_number = '12345623488888';
    	$user_resource = createNewUserWithCC();
    	setSessionProperty('is_a_user_update', 'true');
    	$user_resource->loyalty_number = $loyalty_number;
    	$user_resource->pin = '1234';
    	$user_resource->save();
		$user = logTestUserResourceIn($user_resource);
		$user_id = $user['user_id'];
		$order_adapter = new OrderAdapter($mimetypes);
    	//$order = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'skip hours');
    	$order = $order_adapter->getSimpleCartArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'skip hours',2);
    	
    	// now add the points stuff to the item
    	$item = $order['items'][0];
    	$item['points_used'] = 50;
    	$item['brand_points_id'] = $this->ids['brand_points_id'];
    	$order['items'][0] = $item;
    	$order['total_points_used'] = 50;
    	$order['loyalty_number'] = $loyalty_number;

    	$checkout_resource = getCheckoutDataWithThrottling($order, $merchant_id, getTodayTwelveNoonTimeStampDenver());
    	$this->assertTrue(isset($checkout_resource->lead_times_array));
    	$this->assertEquals(1.50, $checkout_resource->order_amt);
    	$this->assertEquals(0.15, $checkout_resource->total_tax_amt);
    	
    	$request = new Request();
    	$request->method = 'POST';
    	$request->url = 'localhost/app2/apiv2/cart/checkout';
    	$request->data = $order;
    	$place_order_controller = new PlaceOrderController($mt, $_SERVER['AUTHENTICATED_USER'], $request,5);
    	$place_order_controller->setCurrentTime(getTodayTwelveNoonTimeStampDenver());
    	$checkout_data_resource_api_v2 = $place_order_controller->processV2Request();
    	$this->assertTrue(isset($checkout_data_resource_api_v2->lead_times_array));
    	$this->assertEquals(1.50, $checkout_data_resource_api_v2->order_amt,"subtotal should have been zero because of pay with points");
    	$this->assertEquals(0.15, $checkout_data_resource_api_v2->total_tax_amt,"tax should have been zero because of pay with points");
    	
    	$order['tip'] = '0.00';
    	$order['tax_amt'] = '0.15';
    	$order['sub_total'] = '1.50';
    	$order['grand_total'] = '1.65';
    	$order['lead_time'] = 20;
    	$pickup_time = getTodayTwelveNoonTimeStampDenver()+1200;
    	$order['actual_pickup_time'] = $pickup_time;
    	
    	$order_resource = placeOrderFromOrderData($order, getTodayTwelveNoonTimeStampDenver());
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_resource->order_id > 1000,"should have created a good order id but got: ".$order_resource->error);
    	$this->assertNull($order_resource->error);
    	
    	$this->assertEquals(1.50, $order_resource->order_amt);
    	$this->assertEquals(0.15, $order_resource->total_tax_amt);
    	
		// now check to see if the correct amount was recorded in the history		
		$ublh_adapter = new UserBrandLoyaltyHistoryAdapter($mimetypes);
		$ublh_record = $ublh_adapter->getRecord(array("user_id"=>$user_id,"order_id"=>$order_id));
		$this->assertNotNull($ublh_record);
		$this->assertEquals(50, $ublh_record['points_redeemed']);		
   		
   	}
    
    function testPayWithLoyaltyPointsFailure()
    {
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$_SERVER['REQUEST_METHOD'] = 'POST';
    	$loyalty_number = '12345623422222';
    	$user_resource = createNewUser();
    	setSessionProperty('is_a_user_update', 'true');
    	$user_resource->loyalty_number = $loyalty_number;
    	$user_resource->pin = '1234';
    	$user_resource->save();
		$user = logTestUserResourceIn($user_resource);
		$user_id = $user['user_id'];
		
		$ubpma = new UserBrandPointsMapAdapter($mimetypes);
    	$ubpm_record = $ubpma->getRecord(array("user_id"=>$user_id,"brand_id"=>282));

		// now adjust point values
		$user_brand_points_map_resource = Resource::find(new UserBrandPointsMapAdapter($mimetypes),$ubpm_record['map_id']);
		$this->assertEquals(25, $user_brand_points_map_resource->points);
		$user_brand_points_map_resource->points = 100;
		$user_brand_points_map_resource->save();
		
		// now get user session to make sure it sets it back
		$user_session_controller = new UsersessionController($mt, $user, $r);
		$user_session_resource = $user_session_controller->getUserSession($user_resource);
		
		$order_adapter = new OrderAdapter($mimetypes);
    	$order = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'skip hours');
    	
    	// now add the points stuff to the item
    	$item = $order['items'][0];
    	$item['uuid'] = 10;
    	$item['points_used'] = 50;
    	$item['amt_off'] = 1.65;
    	$item['brand_points_id'] = $this->ids['brand_points_id'];
    	$order['items'][0] = $item;
    	$order['tip'] = '0.00';
    	$order['tax_amt'] = '0.00';
    	$order['sub_total'] = '0.00';
    	$order['grand_total'] = '0.00';
    	$order['total_points_used'] = 50;
    	$order['lead_time'] = 20;
    	$order['loyalty_number'] = $loyalty_number;
    	$pickup_time = time()+1200;
    	$order['actual_pickup_time'] = $pickup_time;
    	$pickup_string = date('g:ia',$pickup_time);
    	$order_resource = placeOrderFromOrderData($order, getTodayTwelveNoonTimeStampDenver());
		$this->assertNotNull($order_resource->error);
		$this->assertEquals("We're sorry, but it appears you do not have enough points in your account to place this order.  If you feel you have received this message in error, please contact customer support", $order_resource->error);
    }
    
    function testGetLoyaltyHistoryNoHistory()
	{
		setContext('com.splickit.pitapit');
		$user_resource = createNewUser();
		$user = logTestUserResourceIn($user_resource);
		$loyalty_controller = LoyaltyControllerFactory::getLoyaltyControllerForContext();
		$_SERVER['REQUEST_METHOD'] = 'GET';
		$loyalty_history = $loyalty_controller->getLoyaltyHistory();
		$this->assertNotNull($loyalty_history,"loyalty history should not be null, shoudl be an empty array");
		$this->assertTrue(is_array($loyalty_history)," loyalty history should have been an array");
		$this->assertCount(0, $loyalty_history,"loyalty history should have no records. count 0");
		return $user;
	}

	/**
	 * @depends testGetLoyaltyHistoryNoHistory
	 */
	function testUserControllerGetEmptyLoyalty($user)
	{
		logTestUserIn($user['user_id']);
		$uuid = $user['uuid'];
    	$request = new Request();
    	$request->url = "apiv2/users/$uuid/loyalty_history";
    	$request->method = 'GET';
    	$user_controller = new UserController($mt, $user, $request);
    	$history_resource = $user_controller->processV2Request();
    	$this->assertNull($history_resource->error);
		return $history_resource;
	}
	
	/**
     * @depends testUserControllerGetEmptyLoyalty
     */
    function testUserControllerGetEmptyLoyaltyMakeSureResourceIsNotNull($history_resource)
    {
    	$this->assertNotNull($history_resource);
    }
    
    /**
     * @depends testUserControllerGetEmptyLoyalty
     */
    function testUserControllerGetEmptyLoyaltyForResponse($history_resource)
    {
    	$response = getV2ResponseWithJsonFromResource($history_resource, $headers);
    	$this->assertTrue(is_a($response, 'Response'));
    }

    function testFilterLoyaltyHistoryForPointsOnlyRecords()
    {
        $history_json = '{"history":[{"id":"1xxxxxxxxxxx726909","type":"Balance Inquiry","date":"2015-05-19T16:36:18Z","store":null,"transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726934","type":"Balance Inquiry","date":"2015-05-19T16:36:17Z","store":null,"transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726910","type":"Promotion","date":"2015-05-19T09:36:09-07:00","store":"us-075-c","transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726910","type":"Promotion","date":"2015-05-19T09:36:09-07:00","store":"us-075-c","transactionAmount":{"currency":"Points","amount":14,"formatted":"14"},"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726910","type":"Promotion","date":"2015-05-19T09:36:09-07:00","store":"us-075-c","transactionAmount":{"currency":"Points","amount":14,"formatted":"14"},"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726987","type":"Balance Inquiry","date":"2015-05-19T16:36:06Z","store":null,"transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726918","type":"Balance Inquiry","date":"2015-05-19T16:35:58Z","store":null,"transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726918","type":"Balance Inquiry","date":"2015-05-19T16:35:47Z","store":null,"transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726906","type":"Balance Inquiry","date":"2015-05-19T16:35:09Z","store":null,"transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726969","type":"Balance Inquiry","date":"2015-05-19T16:35:04Z","store":null,"transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726964","type":"Balance Inquiry","date":"2015-05-19T16:34:56Z","store":null,"transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726973","type":"Balance Inquiry","date":"2015-05-19T16:34:52Z","store":null,"transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726966","type":"Balance Inquiry","date":"2015-05-19T16:32:41Z","store":null,"transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726937","type":"Balance Inquiry","date":"2015-05-19T16:32:40Z","store":null,"transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726965","type":"Balance Inquiry","date":"2015-05-19T16:32:36Z","store":null,"transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726948","type":"Balance Inquiry","date":"2015-05-19T16:32:34Z","store":null,"transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726902","type":"Balance Inquiry","date":"2015-05-18T19:07:55Z","store":null,"transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726985","type":"Balance Inquiry","date":"2015-05-18T19:07:54Z","store":null,"transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726912","type":"Promotion","date":"2015-05-18T12:07:46-07:00","store":"us-075-c","transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726912","type":"Promotion","date":"2015-05-18T12:07:46-07:00","store":"us-075-c","transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726912","type":"Promotion","date":"2015-05-18T12:07:46-07:00","store":"us-075-c","transactionAmount":{"currency":"Points","amount":8,"formatted":"8"},"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726991","type":"Balance Inquiry","date":"2015-05-18T19:07:43Z","store":null,"transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726977","type":"Balance Inquiry","date":"2015-05-18T19:07:36Z","store":null,"transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726962","type":"Balance Inquiry","date":"2015-05-18T19:07:25Z","store":null,"transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}},{"id":"1xxxxxxxxxxx726958","type":"Balance Inquiry","date":"2015-05-18T19:07:17Z","store":null,"transactionAmount":null,"status":{"code":0,"name":"okay","description":"okay"}}]}';
        $history_array = json_decode($history_json,true);
        $hlc = PitapitLoyaltyController::loyaltyServiceFactory();
        $better_history = $hlc->cleanHistory($history_array['history']);
        $this->assertCount(3,$better_history);
        foreach($better_history as $history_record) {
            if (trim($history_record['amount']) == "")  {
                $this->assertTrue(false,"should not have found a history record returned with no points");
            }
        }
    }

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

    	$_SERVER['request_time1'] = microtime(true);
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	ini_set('max_execution_time',300);
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction();
    	    	
		//map it to a menu
    	$menu_id = createTestMenuWithNnumberOfItems(2);
    	$ids['menu_id'] = $menu_id;
    	
    	$modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 1);
    	$modifier_group_id = $modifier_group_resource->modifier_group_id;
    	$item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 1);

    	// set up points stuff
    	$complete_menu = CompleteMenu::getCompleteMenu($menu_id);
    	//Test Size 1
    	$size_id = $complete_menu['menu_types'][0]['sizes'][0]['size_id'];
    	$ids['size_id'] = $size_id;
    	
    	$bp_resource = Resource::createByData(new BrandPointsAdapter($mimetypes), array("brand_id"=>282,"points"=>50,"description"=>"Test Size 1 Points"));
    	$brand_points_id = $bp_resource->brand_points_id;
    	$ids['brand_points_id'] = $brand_points_id;
    	$bpom_resource = Resource::createByData(new BrandPointsObjectMapAdapter($mimetypes), array("brand_points_id"=>$brand_points_id,"object_type"=>'size',"object_id"=>$size_id));
		$blr_resource = Resource::createByData(new BrandLoyaltyRulesAdapter($mimetypes), array("brand_id"=>282));
    	$ids['brand_loyalty_rules_id'] = $blr_resource->brand_loyalty_rules_id;

    	$merchant_resource = createNewTestMerchant($menu_id);
    	$merchant_resource->brand_id = 282;
    	$merchant_resource->save();
    	
    	$mhima = new MerchantHeartlandInfoMapsAdapter($mimetypes);
    	Resource::createByData($mhima, array("merchant_id"=>$merchant_resource->merchant_id,"heartland_store_id"=>'us-1000'));
    	
    	$id = MerchantHeartlandInfoMapsAdapter::getHeartlandStoreId($merchant_resource->merchant_id);
    	
    	attachMerchantToSkin($merchant_resource->merchant_id,13);
    	$ids['merchant_id'] = $merchant_resource->merchant_id;
    	
    	$user_resource = createNewUser(array("flags"=>"1C20000001"));
    	$ids['user_id'] = $user_resource->user_id;
    	
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }

	static function tearDownAfterClass()
    {
    	SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    PitapitLoyaltyTest::main();
}

?>