<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class PortalFunctionsTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;
    var $merchant_id;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__ . '-' . $_SERVER['STAMP'];
        $this->ids = $_SERVER['unit_test_ids'];
        setContext("com.splickit.worldhq");

    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }

    function testAddTipOnAuthorizedOrder()
    {
        $merchant_id = $this->ids['merchant_id'];
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $user_id = $user['user_id'];
        $balance_before = $user['balance'];
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $original_order_grand_total = $order_resource->grand_total;
        $this->assertEquals('VioPaymentService', $order_resource->payment_service_used);
        $ucid = $order_resource->ucid;

        $balance_change_adapter = new BalanceChangeAdapter(getM());
        if ($balance_change_records = $balance_change_adapter->getRecords(array("order_id"=>$order_id), null)) {
            $balance_change_rows_by_user_id = setBalanceChangeHashFromBalanceChangeArrayFromOrder($balance_change_records);
        }
        $this->assertCount(2, $balance_change_rows_by_user_id);
        $this->assertTrue(isset($balance_change_rows_by_user_id["$user_id-Authorize"]),"Should have found the authorize row");

        $tip_amt = 1.00;
        $url = "http://127.0.0.1:" . $this->api_port . "/app2/portal/orders/$ucid";
        $data = ['tip_amt'=>$tip_amt];

        $request =  createRequestObject($url,'POST',json_encode($data));
        $order_controller = new OrderController(getM(),null,$request);
        $response  = makeMockPortalRequest($order_controller);
        $response_as_array = json_decode($response,true);
        $this->assertEquals(200,$response_as_array['http_code']);
        $response_data = $response_as_array['data'];
        $this->assertEquals('success',$response_data['result']);
        $this->assertEquals(OrderController::ORDER_HAS_BEEN_UPDATED_MESSAGE,$response_data['message']);



        $balance_change_records = BalanceChangeAdapter::staticGetRecords(array("order_id"=>$order_id),"BalanceChangeAdapter");
        $this->assertCount(3,$balance_change_records,"There should have been 3 balance change records");


        $request =  createRequestObject($url,'POST',json_encode($data));
        $order_controller = new OrderController(getM(),null,$request);
        $response2  = makeMockPortalRequest($order_controller);
        $response2_as_array = json_decode($response2,true);

        $this->assertEquals(422,$response2_as_array['http_code']);

    }

    function testAddTipToCapturedPayment()
    {
        $merchant_id = $this->ids['merchant_id'];
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $order_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,null,0.00,time());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $ucid = $order_resource->ucid;

        $url = "/app2/portal/orders/$ucid/captureauthorizedpayment";
        $request = createRequestObject($url,'POST');
        $order_controller = new OrderController(getM(),null,$request,5);
        $response = makeMockPortalRequest($order_controller);
        $response_as_array = json_decode($response,true);
        $response_data = $response_as_array['data'];
        $this->assertEquals('success',$response_data['result']);

        $balance_change_adapter = new BalanceChangeAdapter(getM());
        if ($balance_change_records = $balance_change_adapter->getRecords(array("order_id"=>$order_id), null)) {
            $balance_change_rows_by_user_id = setBalanceChangeHashFromBalanceChangeArrayFromOrder($balance_change_records);
        }
        $this->assertCount(3,$balance_change_records);

        $tip_amt = 1.50;
        $url = "http://127.0.0.1:" . $this->api_port . "/app2/portal/orders/$ucid";
        $data = ['tip_amt'=>$tip_amt];

        $request =  createRequestObject($url,'POST',json_encode($data));
        $order_controller = new OrderController(getM(),null,$request);
        $response  = makeMockPortalRequest($order_controller);
        $response_as_array = json_decode($response,true);
        $this->assertEquals(200,$response_as_array['http_code']);
        $response_data = $response_as_array['data'];
        $this->assertEquals('success',$response_data['result']);
        $this->assertEquals(OrderController::ORDER_HAS_BEEN_UPDATED_MESSAGE,$response_data['message']);

        $balance_change_adapter = new BalanceChangeAdapter(getM());
        if ($balance_change_records = $balance_change_adapter->getRecords(array("order_id"=>$order_id), null)) {
            $balance_change_rows_by_user_id = setBalanceChangeHashFromBalanceChangeArrayFromOrder($balance_change_records);
        }
        $this->assertCount(4,$balance_change_records);

    }



    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time',300);
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction(); ;

        createWorldHqSkin();
        //map it to a menu
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $ids['menu_id'] = $menu_id;

        $merchant_resource = createNewTestMerchant($menu_id,['authorize'=>true]);
        $merchant_id = $merchant_resource->merchant_id;
        $ids['merchant_id'] = $merchant_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;

    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    PortalFunctionsTest::main();
}

?>