<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname']; 
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class PunchIntegrationTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
		$this->ids = $_SERVER['unit_test_ids'];
		setContext('com.splickit.moes');
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
		unset($_SERVER['punch_test_user_info']);
    }

	function testAuthenticateWithPunchTokenNewUserBadCharacters()
	{
		$simulated_punch_user_id = rand(111111,9999999);
		$simulated_punch_authentication_token = str_replace('-','',generateUUID());
		$_SERVER['punch_test_user_info'] = array("simulated_punch_user_id"=>$simulated_punch_user_id,"simulated_punch_authentication_token"=>$simulated_punch_authentication_token);
		$_SERVER['punch_test_user_info']['first_name'] = "Hello?";
		$_SERVER['punch_test_user_info']['last_name'] = "bob-newheart";
		$request_all['Punch_authentication_token'] = "new_user_token_badcharacters";
		$loginAdapter = LoginAdapterFactory::getLoginAdapterForContext();
		$user_resource = $loginAdapter->doAuthorizeWithSpecialUserValidation($email, $password,$request_all);
		$this->assertNotFalse($user_resource,"should have found a user resource but authorize came back false");

		// need to validate the creation of the user loyalty number and setting of balance
		$ubpma = new UserBrandPointsMapAdapter($m);
		$user_record = $ubpma->getRecord(array("user_id"=>$user_resource->user_id,"brand_id"=>getBrandIdFromCurrentContext()));
		$this->assertNotNull($user_record);
		$this->assertEquals($simulated_punch_user_id.':'.$simulated_punch_authentication_token,$user_record['loyalty_number'],'It should have set the loyalty number to the concatination of punch id and auth token');
		$this->assertNotNull($user_resource->splickit_authentication_token,"It should have a splickit authenticaton token on the returned user_resource");

		$punch_loyalty_controller = new PunchLoyaltyController($m,$user_resource->getDataFieldsReally(),$r,5);
		$retrieved_token = $punch_loyalty_controller->getPunchAuthenticationToken($user_resource->user_id);
		$this->assertEquals($simulated_punch_authentication_token,$retrieved_token,"It should have retrieved the auth token from the user brand points map record");
	}


    function testPaymentOfCompleteSubtotalButFailThePunchCharge()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user_resource->last_four = '8888';
        $user_resource->save();

        $ubpmd['user_id'] = $user_resource->user_id;
        $ubpmd['brand_id'] = getBrandIdFromCurrentContext();
        $simulated_punch_user_id = rand(111111,9999999);
        $simulated_punch_authentication_token = str_replace('-','',generateUUID());
        $ubpmd['loyalty_number'] = "$simulated_punch_user_id:$simulated_punch_authentication_token";
        $ubpmd['points'] = 55;
        $ubpmd['dollar_balance'] = 10.50;
        $resource = Resource::createByData(new UserBrandPointsMapAdapter($m), $ubpmd);

        $user = logTestUserResourceIn($user_resource);

        $menu_id = $this->ids['menu_id'];

        //$merchant_resource = createNewTestMerchant($menu_id);
        $merchant_resource = getOrCreateNewTestMerchantBasedOnExternalId("123456-FF",$menu_id);
        attachMerchantToSkin($merchant_resource->merchant_id, getSkinIdForContext());

        $billing_entity_resource = createSageBillingEntity($merchant_resource->brand_id);

        $merchant_payment_type_map_adapter = new MerchantPaymentTypeMapsAdapter($mimetypes);
        $cc_merchant_payment_type_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 2000, $billing_entity_resource->id);

        // create cash merchant payment type record
        $cash_merchant_payment_type_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 1000, $billing_entity_id);

        // punch loyalty merchant payment type record
        $punch_merchant_payment_type_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 8000, $billing_entity_id);

        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','thenote',1);
        $order_data['items'][0]['mods'] = array();
        $json_encoded_data = json_encode($order_data);
        $request = createRequestObject('/app2/apiv2/cart',"POST",$json_encoded_data,'application/json',$m);
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $cart_resource = $place_order_controller->processV2Request();
        $this->assertNull($cart_resource->error);
        $cart_ucid = $cart_resource->ucid;

        $request = createRequestObject("/apiv2/cart/$cart_ucid/checkout",'GET');
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_resource = $place_order_controller->processV2Request();

        $payment_array = $checkout_resource->accepted_payment_types;
        $this->assertCount(3,$payment_array,'there should be 3 choices in the payment array');

        $base_order_data = CompleteOrder::getBaseOrderData($checkout_resource->oid_test_only);
        $user = logTestUserIn($base_order_data['user_id']);

        //force use of punch payment
        $new_payment_array = array(array_pop($payment_array));
        $checkout_resource->accepted_payment_types = $new_payment_array;
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$base_order_data['merchant_id'],0.00,$time);
        $this->assertNotNull($order_resource->error);
        $this->assertEquals(LoyaltyBalancePaymentService::FAILED_LOYALTY_REDEMPTION_MESSAGE,$order_resource->error);

        //validate users balance is zero
        $user = getStaticRecord(array("user_id"=>$user['user_id']),"UserAdapter");
        $this->assertEquals(0.00,$user['balance'],"should have a zero balance");
        
    }

    function testPaymentOfPartialSubtotalOnly()
    {
        $user_resource = createNewUserWithCCNoCVV();

        $ubpmd['user_id'] = $user_resource->user_id;
        $ubpmd['brand_id'] = getBrandIdFromCurrentContext();
        $simulated_punch_user_id = rand(111111,9999999);
        $simulated_punch_authentication_token = str_replace('-','',generateUUID());
        $ubpmd['loyalty_number'] = "$simulated_punch_user_id:$simulated_punch_authentication_token";
        $ubpmd['points'] = 55;
        $ubpmd['dollar_balance'] = 2.50;
        $resource = Resource::createByData(new UserBrandPointsMapAdapter($m), $ubpmd);

        $user = logTestUserResourceIn($user_resource);

        $menu_id = $this->ids['menu_id'];

        //$merchant_resource = createNewTestMerchant($menu_id);
        $merchant_resource = getOrCreateNewTestMerchantBasedOnExternalId("123456-FF",$menu_id);
        attachMerchantToSkin($merchant_resource->merchant_id, getSkinIdForContext());

        $billing_entity_resource = createSageBillingEntity($merchant_resource->brand_id);

        $merchant_payment_type_map_adapter = new MerchantPaymentTypeMapsAdapter($mimetypes);
        $cc_merchant_payment_type_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 2000, $billing_entity_resource->id);

        // create cash merchant payment type record
        $cash_merchant_payment_type_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 1000, $billing_entity_id);

        // punch loyalty merchant payment type record
        $punch_merchant_payment_type_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 8000, $billing_entity_id);

        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','thenote',1);
        $order_data['items'][0]['mods'] = array();
        $json_encoded_data = json_encode($order_data);
        $request = new Request();
        $request->url = '/app2/apiv2/cart';
        $request->method = "post";
        $request->body = $json_encoded_data;
        $request->mimetype = 'application/json';
        $request->_parseRequestBody();
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $cart_resource = $place_order_controller->processV2Request();
        $this->assertNull($cart_resource->error);
        $cart_ucid = $cart_resource->ucid;

        $request = new Request();
        $request->url = "/apiv2/cart/$cart_ucid/checkout";
        $request->method = "get";
        $request->mimetype = 'application/json';
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_resource = $place_order_controller->processV2Request();

        $payment_array = $checkout_resource->accepted_payment_types;
        $this->assertCount(3,$payment_array,'there should be 3 choices in the payment array');
        $payment_hash = createHashmapFromArrayOfArraysByFieldName($payment_array,'splickit_accepted_payment_type_id');
        //$this->assertEquals('Rockin Rewards($1.50) + CC',$payment_hash['8000']['name']);

        $base_order_data = CompleteOrder::getBaseOrderData($checkout_resource->oid_test_only);
        $user = logTestUserIn($base_order_data['user_id']);
        $new_cart_note = "the new cart note";
        $order_data = array();
        $order_data['merchant_id'] = $base_order_data['merchant_id'];
        $order_data['note'] = $new_cart_note;
        $order_data['user_id'] = $base_order_data['user_id'];
        $order_data['cart_ucid'] = $base_order_data['ucid'];
        $order_data['tip'] = 0.00;
        $base_order_data['grand_total'] = $base_order_data['grand_total'] + $order_data['tip'];
        $payment_array = $checkout_resource->accepted_payment_types;
        $order_data['merchant_payment_type_map_id'] = $payment_array[2]['merchant_payment_type_map_id'];
        $lead_times_array = $checkout_resource->lead_times_array;
        $order_data['actual_pickup_time'] = $lead_times_array[0];
        // this should be ignored;
        $order_data['lead_time'] = 100000;

        $json_encoded_data = json_encode($order_data);
        $request = new Request();
        $request->url = '/apiv2/orders/'.$base_order_data['ucid'];
        $request->method = "post";
        $request->body = $json_encoded_data;
        $request->mimetype = 'application/json';
        $request->_parseRequestBody();
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $order_resource = $place_order_controller->processV2Request();
        $this->assertNull($order_resource->error);

        $balance_change_records = BalanceChangeAdapter::staticGetRecords(array("order_id"=>$order_resource->order_id),'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($balance_change_records,'process');
        $this->assertEquals(1.50,$bcr_hash['PunchLoyaltyBalancePayment']['charge_amt'],"It should have the punch charge");
        $expected_cc_charge = $base_order_data['grand_total'] - 1.50;
        $this->assertNull($bcr_hash['CCpayment'],"It should NOT have a cc row");
        //$this->assertEquals($expected_cc_charge,$bcr_hash['CCpayment']['charge_amt'],"It should have the reduced CC charge");


    }

    function testCreatePunchLoyaltyMessagesOnOrderForModOnlyRule()
	{
		setContext('com.splickit.moes');
		$menu_id = createTestMenuWithNnumberOfItems(1);
		$modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 5);
		$modifier_group_id = $modifier_group_resource->modifier_group_id;
		$modifier_item_records = ModifierItemAdapter::staticGetRecords(array("modifier_group_id"=>$modifier_group_id),'ModifierItemAdapter');
		$modifier_size_records = ModifierSizeMapAdapter::staticGetRecords(array("modifier_item_id"=>$modifier_item_record['modifier_item_id']),'ModifierSizeMapAdapter');

		$item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
		assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 0);

		$test_id = '88888';
		$test_name = 'Item Only Name';
		$test_group = 'sumdumgroup';
		$moia = new AdmMoesAlohaInfoAdapter($m);
		$data['item_id'] = $item_records[0]['item_id'];
		$data['name'] = $test_name;
		$data['aloha_id'] = $test_id;
		$data['rule'] = 'item';
		$data['major_group'] = $test_group;
		$resource = Resource::createByData($moia,$data);

		$mod_test_id = '99999';
		$mod_test_name = 'Mod Only Name';
		$mod_test_group = 'sumdummodgroup';
		$moia = new AdmMoesAlohaInfoAdapter($m);
		$data = array();
		$data['item_id'] = null;
		$data['size_id'] = null;
		$data['modifier_id'] = $modifier_item_records[4]['modifier_item_id'];
		$data['name'] = $mod_test_name;
		$data['aloha_id'] = $mod_test_id;
		$data['rule'] = 'mod';
		$data['major_group'] = $mod_test_group;
		$resource = Resource::createByData($moia,$data);

		$complete_menu = new CompleteMenu($menu_id);
		$price_resources = $complete_menu->getAllMenuItemSizeMapResources($menu_id,'Y',0);
		$external_for_item = $price_resources[0]->external_id;

		$modifier_price_resource = Resource::find(new ModifierSizeMapAdapter($m),null,array(TONIC_FIND_BY_METADATA=>array("modifier_item_id"=>$data['modifier_id'])));
		$modifier_price_resource->modifier_price = .88;
		$modifier_price_resource->save();

		$merchant_resource = getOrCreateNewTestMerchantBasedOnExternalId("123456-EF",$menu_id);
		attachMerchantToSkin($merchant_resource->merchant_id, getSkinIdForContext());

		$user = logTestUserIn($this->ids['user_id']);

		$order_adapter = new OrderAdapter($m);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','thenote');

		$order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
		$this->assertNull($order_resource->error);
		$complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id);

		$punch_controller = new PunchLoyaltyController($m,$user,$r,5);
		$punch_order_payload_array = $punch_controller->getPunchPayloadArrayForOrder($complete_order);
		$punch_payload_as_json = json_encode($punch_order_payload_array);
		$offset = 7 - date('I',getTomorrowTwelveNoonTimeStampDenver());

		// need to contact Gauresh about what the correct payload is for this
		// OK!  premium mods get their own node, reduce the price of the parent item the amount of the premium mod

		$expected_payload_listed_price_of_item = $complete_order['order_details'][0]['item_total_w_mods'] - $modifier_price_resource->modifier_price;

		$expected_parent_item_node = cleanUpDoubleSpacesCRLFTFromString('{
					  "item_name":"Item Only Name",
					  "item_qty":"1",
					  "item_amount":"'.number_format($expected_payload_listed_price_of_item,2).'",
					  "menu_item_type":"M",
					  "menu_item_id":"'.$test_id.'",
					  "menu_family":"800",
					  "menu_major_group":"'.$test_group.'"}');
		$expected_item_array = json_decode($expected_parent_item_node,true);
		$this->assertEquals($expected_item_array,$punch_order_payload_array['menu_items'][0]);


		$expected_premium_modifier_node = cleanUpDoubleSpacesCRLFTFromString('{
					  "item_name":"'.$mod_test_name.'",
					  "item_qty":"1",
					  "item_amount":"'.$modifier_price_resource->modifier_price.'",
					  "menu_item_type":"M",
					  "menu_item_id":"'.$mod_test_id.'",
					  "menu_family":"800",
					  "menu_major_group":"'.$mod_test_group.'"}');
		$expected_modifier_array = json_decode($expected_premium_modifier_node,true);
		$this->assertEquals($expected_modifier_array,$punch_order_payload_array['menu_items'][1]);

		$this->assertContains($expected_parent_item_node,$punch_payload_as_json,"It should have the parent item node with the reduced amount");
		$this->assertContains($expected_premium_modifier_node,$punch_payload_as_json,"It should have the premium modifier item node with the price of the premium modifier");


		$expected_payload = cleanUpDoubleSpacesCRLFTFromString('{
				"receipt_amount":'.$complete_order['grand_total'].',
				"cc_last4":"'.$user['last_four'].'",
				"store_number":"123456-EF",
				"employee_id":1,
				"employee_name":"Splickit Server",
				"menu_items":['.$expected_parent_item_node.','.$expected_premium_modifier_node.'],
				"subtotal_amount":'.$complete_order['order_amt'].',
				"receipt_datetime":"'.date('Y-m-d',getTomorrowTwelveNoonTimeStampDenver()).'T12:00:00-0'.$offset.':00",
				"transaction_no":'.$order_resource->order_id.',
				"external_uid":"'.$order_resource->ucid.'",
				"client":"'.getProperty('punch_client_id').'"
				}');
		$expected_payload_array = json_decode($expected_payload,true);
		$this->assertEquals($expected_payload_array,$punch_order_payload_array,"It should have generated the expected array");
	}

	function testGetCorrectLoginAdapterForMoesSkin()
	{
		$login_adapter = LoginAdapterFactory::getLoginAdapterForContext();
		$this->assertEquals('MoesLoginAdapter',get_class($login_adapter));
	}

	function testValidatePunchAuthenticationTokenNameAsValidHeader()
	{
		$login_adapter = LoginAdapterFactory::getLoginAdapterForContext();
		$login_headers = array();
		$login_headers['Punch_authentication_token'] = 'sumdumtoken';
		$this->assertTrue($login_adapter->isValidHeaderTokenSet($login_headers),'It should accept the punch token');
	}

	function testValidateSplickitAuthenticationTokenNameAsValidHeader()
	{
		$login_adapter = LoginAdapterFactory::getLoginAdapterForContext();
		$login_headers = array();
		$login_headers['splickit_authentication_token'] = 'sumdumtoken';
		$this->assertTrue($login_adapter->isValidHeaderTokenSet($login_headers),'It shoudl still accept a spclikit auth token');
	}



	function testAuthenticateWithPunchTokenNewUser()
	{
		$simulated_punch_user_id = rand(111111,9999999);
		$simulated_punch_authentication_token = str_replace('-','',generateUUID());
		$_SERVER['punch_test_user_info'] = array("simulated_punch_user_id"=>$simulated_punch_user_id,"simulated_punch_authentication_token"=>$simulated_punch_authentication_token);
		$request_all['Punch_authentication_token'] = "new_user_token_1234567890";
		$loginAdapter = LoginAdapterFactory::getLoginAdapterForContext();
		$user_resource = $loginAdapter->doAuthorizeWithSpecialUserValidation($email, $password,$request_all);
		$this->assertNotFalse($user_resource,"should have found a user resource but authorize came back false");

		// need to validate the creation of the user loyalty number and setting of balance
		$ubpma = new UserBrandPointsMapAdapter($m);
		$user_record = $ubpma->getRecord(array("user_id"=>$user_resource->user_id,"brand_id"=>getBrandIdFromCurrentContext()));
		$this->assertNotNull($user_record);
		$this->assertEquals($simulated_punch_user_id.':'.$simulated_punch_authentication_token,$user_record['loyalty_number'],'It should have set the loyalty number to the concatination of punch id and auth token');
		$this->assertNotNull($user_resource->splickit_authentication_token,"It should have a splickit authenticaton token on the returned user_resource");
		
		$punch_loyalty_controller = new PunchLoyaltyController($m,$user_resource->getDataFieldsReally(),$r,5);
		$retrieved_token = $punch_loyalty_controller->getPunchAuthenticationToken($user_resource->user_id);
		$this->assertEquals($simulated_punch_authentication_token,$retrieved_token,"It should have retrieved the auth token from the user brand points map record");
	}

	function testAuthenticateWithPunchTokenExistingUser()
	{
		$simulated_punch_user_id = rand(111111,9999999);
		$simulated_punch_authentication_token = str_replace('-','',generateUUID());
		$user_id = $this->ids['user_id'];
		$user = UserAdapter::staticGetRecordByPrimaryKey($user_id,'UserAdapter');
		$user['simulated_punch_user_id'] = $simulated_punch_user_id;
		$user['simulated_punch_authentication_token'] = $simulated_punch_authentication_token;
		$_SERVER['punch_test_user_info'] = $user;
		$request_all['Punch_authentication_token'] = "existing_user_token_1234567890";
		$loginAdapter = LoginAdapterFactory::getLoginAdapterForContext();
		$user_resource = $loginAdapter->doAuthorizeWithSpecialUserValidation($email, $password,$request_all);
		$this->assertNotFalse($user_resource,"should have found a user resource but authorize came back false");
		$this->assertEquals($user_id, $user_resource->user_id,"should have grabbed the user resource associated wih the correct user but it did not");

		// need to validate the creation or update of the user loyalty number and balance
		$ubpma = new UserBrandPointsMapAdapter($m);
		$user_record = $ubpma->getRecord(array("user_id"=>$user_id,"brand_id"=>getBrandIdFromCurrentContext()));
		$this->assertNotNull($user_record);
		$this->assertEquals($simulated_punch_user_id.':'.$simulated_punch_authentication_token,$user_record['loyalty_number'],'It should have set the loyalty number to the concatination of punch id and auth token');
		$data['user_id'] = $user_id;
		$data['punch_user_id'] = $simulated_punch_user_id;
		return $data;
	}

	/**
	 * @depends testAuthenticateWithPunchTokenExistingUser
	 */
	function testUpdateOfUserBrandPointsMapRecordWithNewPunchAuthenticationToken($data)
	{
		$simulated_punch_user_id = $data['punch_user_id'];
		$simulated_punch_authentication_token = str_replace('-','',generateUUID());
		$user_id = $data['user_id'];
		$user = UserAdapter::staticGetRecordByPrimaryKey($user_id,'UserAdapter');
		$user['simulated_punch_user_id'] = $simulated_punch_user_id;
		$user['simulated_punch_authentication_token'] = $simulated_punch_authentication_token;
		$_SERVER['punch_test_user_info'] = $user;
		$request_all['Punch_authentication_token'] = "existing_user_token_1234567890";
		$loginAdapter = LoginAdapterFactory::getLoginAdapterForContext();
		$user_resource = $loginAdapter->doAuthorizeWithSpecialUserValidation($email, $password,$request_all);
		$this->assertNotFalse($user_resource,"should have found a user resource but authorize came back false");
		$this->assertEquals($user_id, $user_resource->user_id,"should have grabbed the user resource associated wih the correct user but it did not");

		// need to validate the update of the user loyalty number
		$ubpma = new UserBrandPointsMapAdapter($m);
		$user_record = $ubpma->getRecord(array("user_id"=>$user_id,"brand_id"=>getBrandIdFromCurrentContext()));
		$this->assertNotNull($user_record);
		$this->assertEquals($simulated_punch_user_id.':'.$simulated_punch_authentication_token,$user_record['loyalty_number'],'It should have set the loyalty number to the concatination of punch id and auth token');
		return $data;
	}

	/**
	 * @depends testUpdateOfUserBrandPointsMapRecordWithNewPunchAuthenticationToken
	 */
	function testUpdateOfPointsBalanceOnGetUserSession($data)
	{
		$user = logTestUserIn($data['user_id']);
		$usc = new UsersessionController($mt, $user, $r, 5);
		$user_resource = SplickitController::getResourceFromId($data['user_id'], 'User');
		$user_session_resource = $usc->getUserSession($user_resource);
		// need to validate the update of the users points fromt the remote system
		$ubpma = new UserBrandPointsMapAdapter($m);
		$user_record = $ubpma->getRecord(array("user_id"=>$user['user_id'],"brand_id"=>getBrandIdFromCurrentContext()));
		$this->assertNotNull($user_record);
		$this->assertEquals(89,$user_record['points'],"Should have updated the users points balance with the value from the remote system");
		$this->assertEquals(1.50,$user_record['dollar_balance'],"Should have updated the users dollar balance with the value from the remote system");

		$this->assertNotNull($user_session_resource->brand_loyalty,"Brand loyalty record should come back on user session");
		$this->assertEquals('$1.50',$user_session_resource->brand_loyalty['points']);
		$this->assertEquals(1.50,$user_session_resource->brand_loyalty['usd'],"There should be a USD section on the user session for backwards compatability");

		return $data;
	}

	/**
	 * @depends testUpdateOfPointsBalanceOnGetUserSession
	 */
	function testGetPayWithLoyaltyOptionOnCheckoutSinceUserHasBalance($data)
	{
		$user = logTestUserIn($data['user_id']);
		$menu_id = $this->ids['menu_id'];

		//$merchant_resource = createNewTestMerchant($menu_id);
		$merchant_resource = getOrCreateNewTestMerchantBasedOnExternalId("123456-AB",$menu_id);
		attachMerchantToSkin($merchant_resource->merchant_id, getSkinIdForContext());

		$billing_entity_resource = createSageBillingEntity($merchant_resource->brand_id);

		$merchant_payment_type_map_adapter = new MerchantPaymentTypeMapsAdapter($mimetypes);
		$cc_merchant_payment_type_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 2000, $billing_entity_resource->id);

		// create cash merchant payment type record
		$cash_merchant_payment_type_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 1000, $billing_entity_id);

		// punch loyalty merchant payment type record
		$punch_merchant_payment_type_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 8000, $billing_entity_id);

		$order_adapter = new OrderAdapter($m);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','thenote',4);

		$json_encoded_data = json_encode($order_data);
		$request = new Request();
		$request->url = '/app2/apiv2/cart';
		$request->method = "post";
		$request->body = $json_encoded_data;
		$request->mimetype = 'application/json';
		$request->_parseRequestBody();
		$place_order_controller = new PlaceOrderController($mt, $user, $request);
		$place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
		$cart_resource = $place_order_controller->processV2Request();
		$this->assertNull($cart_resource->error);
		$cart_ucid = $cart_resource->ucid;

		$request = new Request();
		$request->url = "/apiv2/cart/$cart_ucid/checkout";
		$request->method = "get";
		$request->mimetype = 'application/json';
		$place_order_controller = new PlaceOrderController($mt, $user, $request);
		$place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
		$checkout_resource = $place_order_controller->processV2Request();

		$payment_array = $checkout_resource->accepted_payment_types;
		$this->assertCount(3,$payment_array,'there should be 3 choices in the payment array');
		$payment_hash = createHashmapFromArrayOfArraysByFieldName($payment_array,'splickit_accepted_payment_type_id');
		$this->assertEquals('Use $1.50 Rewards ('.LoyaltyBalancePaymentService::BALANCE_ON_CARD_TEXT.')',$payment_hash['8000']['name']);
		return $checkout_resource;
	}
	
	/**
	 * @depends testGetPayWithLoyaltyOptionOnCheckoutSinceUserHasBalance
	 */
	function testPlaceOrderWithLoyaltyBalanceAndCC($checkout_resource)
	{

		$base_order_data = CompleteOrder::getBaseOrderData($checkout_resource->oid_test_only);
		$user = logTestUserIn($base_order_data['user_id']);
		$new_cart_note = "the new cart note";
		$order_data['merchant_id'] = $base_order_data['merchant_id'];
		$order_data['note'] = $new_cart_note;
		$order_data['user_id'] = $base_order_data['user_id'];
		$order_data['cart_ucid'] = $base_order_data['ucid'];
		//$order_data['tip'] = (rand(100, 200))/100;
		$order_data['tip'] = 0.00;
		$base_order_data['grand_total'] = $base_order_data['grand_total'] + $order_data['tip'];
		$payment_array = $checkout_resource->accepted_payment_types;
		$order_data['merchant_payment_type_map_id'] = $payment_array[2]['merchant_payment_type_map_id'];
		$lead_times_array = $checkout_resource->lead_times_array;
		$order_data['actual_pickup_time'] = $lead_times_array[0];
		// this should be ignored;
		$order_data['lead_time'] = 100000;

		$json_encoded_data = json_encode($order_data);
		$request = new Request();
		$request->url = '/apiv2/orders/'.$base_order_data['ucid'];
		$request->method = "post";
		$request->body = $json_encoded_data;
		$request->mimetype = 'application/json';
		$request->_parseRequestBody();
		$place_order_controller = new PlaceOrderController($mt, $user, $request);
		$place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
		$order_resource = $place_order_controller->processV2Request();
		$this->assertNull($order_resource->error);
		$order_id = $order_resource->order_id;

		//$earned_amount = round(10*$order_resource->order_amt);
		//$this->assertEquals("$earned_amount Points",$order_resource->loyalty_earned_message);
		// if the below tests fail, its probabaly because we are not calling out to get balance after we send the rewards payload.
		// so that code may need to be written.
		//$this->assertEquals("$1.50",$order_resource->loyalty_balance_message,"Should get the balance from the remote server. which defaults to $1.50 in development");

		// check to see if the redemption payload is correct
		$redemption_payload_array = $place_order_controller->getPaymentService()->punch_loyalty_controller->getPayloadArray();
		$this->assertEquals('discount_amount',$redemption_payload_array['discount_type'],"Should have had the discount amount as the discount type");
		$this->assertEquals(1.50,$redemption_payload_array['redeemed_points'],"It should have had $1.50 in redeemed points");

		// check to see if the earn payload is correct
		$earn_payload_array = $place_order_controller->getLoyaltyController()->getPayloadArray();
		$this->assertEquals($order_resource->order_amt-1.50,$earn_payload_array['subtotal_amount'],"The subtotal should be the order amount minus the award redemption");
		$this->assertEquals($order_resource->grand_total-1.50,$earn_payload_array['receipt_amount'],"The subtotal should be the grand total minus the award redemption");
		$last_item_node = array_pop($earn_payload_array['menu_items']);
		$this->assertEquals('Redemption Discount',$last_item_node['item_name']);
		$this->assertEquals(1.50,$last_item_node['item_amount'],'The discount node should have an amount equal to the discount amount');
		$expected_discount_order_detail = array();
		$expected_discount_order_detail['item_name'] = 'Redemption Discount';
		$expected_discount_order_detail['item_qty'] = '1';
		$expected_discount_order_detail['item_amount'] = "1.50";
		$expected_discount_order_detail['menu_item_type'] = 'D';
		$expected_discount_order_detail['menu_item_id'] = '00';
		$expected_discount_order_detail['menu_family'] = '00';
		$expected_discount_order_detail['menu_major_group'] = '00';
		$this->assertEquals($expected_discount_order_detail, $last_item_node);

		//check to see if all records were added to balance change table
		$balance_change_records = BalanceChangeAdapter::staticGetRecords(array("order_id"=>$order_id),'BalanceChangeAdapter');
		$bcr_hash = createHashmapFromArrayOfArraysByFieldName($balance_change_records,'process');
		$this->assertEquals(1.50,$bcr_hash['PunchLoyaltyBalancePayment']['charge_amt'],"It should have the punch charge");
		$expected_cc_charge = $order_resource->grand_total_to_merchant;
		$this->assertEquals($expected_cc_charge,$bcr_hash['CCpayment']['charge_amt'],"It should have the reduced CC charge");

		$complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
		$expected_grand_total_to_merchant = round($base_order_data['grand_total'] - 1.50 - .15 - $order_data['tip'],2);
		$this->assertEquals($expected_grand_total_to_merchant,$complete_order['grand_total_to_merchant']);
		return $order_id;
	}
	
	/**
	 * @depends testPlaceOrderWithLoyaltyBalanceAndCC
	 */
	function testRewardsIsShownOnOrderConfirmation($order_id)
	{
		$message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'Econf');
		$this->assertNotNull($message_resource);
		$message_text = $message_resource->message_text;
		myerror_log($message_text);
		//$this->assertContains("Rocking Rewards: $1.50",$message_text);
	}
	
//	function testCorrectPayloadIfBalanceIsMoreThanGrandTotal()
//	{
//		$this->assertTrue(false,"Write test to make sure payload only has amount needed, but do we really need this?");
//	}


	
	function testVoidRedemptionOnCCfail()
	{
		$user_resource = createNewUserWithCC();
		$user_resource->uuid = '12340-56789-INVLD-NUMBR';
		$user_resource->save();

		$ubpmd['user_id'] = $user_resource->user_id;
		$ubpmd['brand_id'] = getBrandIdFromCurrentContext();
		$simulated_punch_user_id = rand(111111,9999999);
		$simulated_punch_authentication_token = str_replace('-','',generateUUID());
		$ubpmd['loyalty_number'] = "$simulated_punch_user_id:$simulated_punch_authentication_token";
		$ubpmd['points'] = 55;
		$ubpmd['dollar_balance'] = 2.50;
		$resource = Resource::createByData(new UserBrandPointsMapAdapter($m), $ubpmd);

		$user = logTestUserResourceIn($user_resource);

		$menu_id = $this->ids['menu_id'];

		//$merchant_resource = createNewTestMerchant($menu_id);
		$merchant_resource = getOrCreateNewTestMerchantBasedOnExternalId("123456-FF",$menu_id);
		attachMerchantToSkin($merchant_resource->merchant_id, getSkinIdForContext());

		$billing_entity_resource = createSageBillingEntity($merchant_resource->brand_id);

		$merchant_payment_type_map_adapter = new MerchantPaymentTypeMapsAdapter($mimetypes);
		$cc_merchant_payment_type_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 2000, $billing_entity_resource->id);

		// create cash merchant payment type record
		$cash_merchant_payment_type_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 1000, $billing_entity_id);

		// punch loyalty merchant payment type record
		$punch_merchant_payment_type_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 8000, $billing_entity_id);

		$order_adapter = new OrderAdapter($m);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','thenote',4);

		$json_encoded_data = json_encode($order_data);
		$request = new Request();
		$request->url = '/app2/apiv2/cart';
		$request->method = "post";
		$request->body = $json_encoded_data;
		$request->mimetype = 'application/json';
		$request->_parseRequestBody();
		$place_order_controller = new PlaceOrderController($mt, $user, $request);
		$place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
		$cart_resource = $place_order_controller->processV2Request();
		$this->assertNull($cart_resource->error);
		$cart_ucid = $cart_resource->ucid;

		$request = new Request();
		$request->url = "/apiv2/cart/$cart_ucid/checkout";
		$request->method = "get";
		$request->mimetype = 'application/json';
		$place_order_controller = new PlaceOrderController($mt, $user, $request);
		$place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
		$checkout_resource = $place_order_controller->processV2Request();

		$payment_array = $checkout_resource->accepted_payment_types;
		$this->assertCount(3,$payment_array,'there should be 3 choices in the payment array');
		$payment_hash = createHashmapFromArrayOfArraysByFieldName($payment_array,'splickit_accepted_payment_type_id');
		$this->assertEquals('Use $2.50 Rewards ('.LoyaltyBalancePaymentService::BALANCE_ON_CARD_TEXT.')',$payment_hash['8000']['name']);

		$base_order_data = CompleteOrder::getBaseOrderData($checkout_resource->oid_test_only);
		$user = logTestUserIn($base_order_data['user_id']);
		$new_cart_note = "the new cart note";
		$order_data['merchant_id'] = $base_order_data['merchant_id'];
		$order_data['note'] = $new_cart_note;
		$order_data['user_id'] = $base_order_data['user_id'];
		$order_data['cart_ucid'] = $base_order_data['ucid'];
		$order_data['tip'] = (rand(100, 200))/100;
		$base_order_data['grand_total'] = $base_order_data['grand_total'] + $order_data['tip'];
		$payment_array = $checkout_resource->accepted_payment_types;
		$order_data['merchant_payment_type_map_id'] = $payment_array[2]['merchant_payment_type_map_id'];
		$lead_times_array = $checkout_resource->lead_times_array;
		$order_data['actual_pickup_time'] = $lead_times_array[0];
		// this should be ignored;
		$order_data['lead_time'] = 100000;

		$json_encoded_data = json_encode($order_data);
		$request = new Request();
		$request->url = '/apiv2/orders';
		$request->method = "post";
		$request->body = $json_encoded_data;
		$request->mimetype = 'application/json';
		$request->_parseRequestBody();
		$place_order_controller = new PlaceOrderController($mt, $user, $request);
		$place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
		$order_resource = $place_order_controller->processV2Request();
		$this->assertNotNull($order_resource->error);

		// check to make sure money was put back on the account? but how?  its all remote.


	}

	function testAuthenticateWithSplickitAuthenticationToken()
	{
		$simulated_punch_user_id = rand(111111,9999999);
		$simulated_punch_authentication_token = str_replace('-','',generateUUID());
		$user_id = $this->ids['user_id'];
		$user = UserAdapter::staticGetRecordByPrimaryKey($user_id,'UserAdapter');
		$user['simulated_punch_user_id'] = $simulated_punch_user_id;
		$user['simulated_punch_authentication_token'] = $simulated_punch_authentication_token;
		$_SERVER['punch_test_user_info'] = $user;
		$request_all['Punch_authentication_token'] = "existing_user_token_1234567890";
		$loginAdapter = LoginAdapterFactory::getLoginAdapterForContext();
		$user_resource = $loginAdapter->doAuthorizeWithSpecialUserValidation($email, $password,$request_all);
		$this->assertNotFalse($user_resource,"should have found a user resource but authorize came back false");
		$this->assertEquals($user_id, $user_resource->user_id,"should have grabbed the user resource associated wih the correct user but it did not");
	}


//	function testAuthenticateFromDispatchWithTokenExistingUser()
//	{
//		$user_resource = createNewUser();
//		$user = $user_resource->getDataFieldsReally();
//		$simulated_punch_user_id = rand(111111,9999999);
//		$simulated_punch_authentication_token = str_replace('-','',generateUUID());
//		$user['simulated_punch_user_id'] = $simulated_punch_user_id;
//		$user['simulated_punch_authentication_token'] = $simulated_punch_authentication_token;
//		$_SERVER['punch_test_user_info'] = $user;
//		$url = "http://localhost/app2/apiv2/users";
//
//		$request = new Request();
//		$request->method = 'GET';
//		$request->url = $url;
//		$request->data = array("Punch_authentication_token"=>"existing_user_token_1234567890");
//
//		$user_controller = new UserController($mt, getLoggedInUser(), $request);
//		$resource = $user_controller->processV2Request();
//		$response = ApiDispatchTest::makeRequest($url, "splickit_authentication_token:$splickit_authentication_token");
//		$this->assertEquals(200, $this->info['http_code']);
//	}


	function testCreatePunchLoyaltyMessagesOnOrderForItemRule()
	{
		setContext('com.splickit.moes');
		$menu_id = $this->ids['menu_id'];

		//$merchant_resource = createNewTestMerchant($menu_id);
		$merchant_resource = getOrCreateNewTestMerchantBasedOnExternalId("123456-BC",$menu_id);
		attachMerchantToSkin($merchant_resource->merchant_id, getSkinIdForContext());

		$user = logTestUserIn($this->ids['user_id']);

		$order_adapter = new OrderAdapter($m);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','thenote');

		$order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
		$this->assertNull($order_resource->error);
		$complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id);

		$punch_controller = new PunchLoyaltyController($m,$user,$r,5);
		$punch_order_payload_array = $punch_controller->getPunchPayloadArrayForOrder($complete_order);
		$offset = 7 - date('I',getTomorrowTwelveNoonTimeStampDenver());

		$expected_payload = cleanUpDoubleSpacesCRLFTFromString('{
				"receipt_amount":'.$complete_order['grand_total'].',
				"cc_last4":"'.$user['last_four'].'",
				"store_number":"123456-BC",
				"employee_id":1,
				"employee_name":"Splickit Server",
				"menu_items":[
				   {
					  "item_name":"'.$this->ids['adm_moes_aloha_record']['name'].'",
					  "item_qty":"1",
					  "item_amount":"'.$complete_order['order_details'][0]['item_total_w_mods'].'",
					  "menu_item_type":"M",
					  "menu_item_id":'.$this->ids['adm_moes_aloha_record']['aloha_id'].',
					  "menu_family":"800",
					  "menu_major_group":"'.$this->ids['adm_moes_aloha_record']['major_group'].'"
				   }
				],
				"subtotal_amount":'.$complete_order['order_amt'].',
				"receipt_datetime":"'.date('Y-m-d',getTomorrowTwelveNoonTimeStampDenver()).'T12:00:00-0'.$offset.':00",
				"transaction_no":'.$order_resource->order_id.',
				"external_uid":"'.$order_resource->ucid.'",
				"client":"'.getProperty('punch_client_id').'"
				}');
		$expected_payload_array = json_decode($expected_payload,true);

		$this->assertEquals($expected_payload_array,$punch_order_payload_array,"It should have generated the expected array");

	}

	function testCreatePunchLoyaltyMessagesOnOrderForItemSizeRule()
	{
		setContext('com.splickit.moes');
		$menu_id = createTestMenuWithNnumberOfItems(1);
		$modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 5);
		$modifier_group_id = $modifier_group_resource->modifier_group_id;

		$item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
		assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 0);

		$item_size_record = ItemSizeAdapter::staticGetRecord(array("item_id"=>$item_records[0]['item_id']),'ItemSizeAdapter');

		$test_id = '88887';
		$test_name = 'Item Size Name';
		$test_group = 'Size Group';
		$moia = new AdmMoesAlohaInfoAdapter($m);
		$data['item_id'] = $item_records[0]['item_id'];
		$data['size_id'] = $item_size_record['size_id'];
		$data['name'] = $test_name;
		$data['aloha_id'] = $test_id;
		$data['rule'] = 'item-size';
		$data['major_group'] = $test_group;
		$resource = Resource::createByData($moia,$data);

		//$merchant_resource = createNewTestMerchant($menu_id);
		$merchant_resource = getOrCreateNewTestMerchantBasedOnExternalId("123456-CD",$menu_id);
		attachMerchantToSkin($merchant_resource->merchant_id, getSkinIdForContext());

		$user = logTestUserIn($this->ids['user_id']);

		$order_adapter = new OrderAdapter($m);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','thenote');

		$order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
		$this->assertNull($order_resource->error);
		$complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id);

		$punch_controller = new PunchLoyaltyController($m,$user,$r,5);
		$punch_order_payload_array = $punch_controller->getPunchPayloadArrayForOrder($complete_order);
		$offset = 7 - date('I',getTomorrowTwelveNoonTimeStampDenver());

		$expected_payload = cleanUpDoubleSpacesCRLFTFromString('{
				"receipt_amount":'.$complete_order['grand_total'].',
				"cc_last4":"'.$user['last_four'].'",
				"store_number":"123456-CD",
				"employee_id":1,
				"employee_name":"Splickit Server",
				"menu_items":[
				   {
					  "item_name":"'.$test_name.'",
					  "item_qty":"1",
					  "item_amount":"'.$complete_order['order_details'][0]['item_total_w_mods'].'",
					  "menu_item_type":"M",
					  "menu_item_id":'.$test_id.',
					  "menu_family":"800",
					  "menu_major_group":"'.$test_group.'"
				   }
				],
				"subtotal_amount":'.$complete_order['order_amt'].',
				"receipt_datetime":"'.date('Y-m-d',getTomorrowTwelveNoonTimeStampDenver()).'T12:00:00-0'.$offset.':00",
				"transaction_no":'.$order_resource->order_id.',
				"external_uid":"'.$order_resource->ucid.'",
				"client":"'.getProperty('punch_client_id').'"
				}');
		$expected_payload_array = json_decode($expected_payload,true);

		$this->assertEquals($expected_payload_array,$punch_order_payload_array,"It should have generated the expected array");

	}


	function testCreatePunchLoyaltyMessagesOnOrderForItemModRule()
	{
		setContext('com.splickit.moes');
		$menu_id = createTestMenuWithNnumberOfItems(1);
		$modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 5);
		$modifier_group_id = $modifier_group_resource->modifier_group_id;
		$modifier_item_records = ModifierItemAdapter::staticGetRecords(array("modifier_group_id"=>$modifier_group_id),'ModifierItemAdapter');
		$modifier_size_records = ModifierSizeMapAdapter::staticGetRecords(array("modifier_item_id"=>$modifier_item_record['modifier_item_id']),'ModifierSizeMapAdapter');

		$item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
		assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 0);

		$test_id = '88889';
		$moia = new AdmMoesAlohaInfoAdapter($m);
		$data['item_id'] = $item_records[0]['item_id'];
		$data['modifier_id'] = $modifier_item_records[2]['modifier_item_id'];
		$data['name'] = "My Special Name";
		$data['aloha_id'] = $test_id;
		$data['rule'] = 'item-mod';
		$data['major_group'] = 'burritos';
		$resource = Resource::createByData($moia,$data);




		$complete_menu = new CompleteMenu($menu_id);
		$price_resources = $complete_menu->getAllMenuItemSizeMapResources($menu_id,'Y',0);
		$external_for_item = $price_resources[0]->external_id;

		//$merchant_resource = createNewTestMerchant($menu_id);
		$merchant_resource = getOrCreateNewTestMerchantBasedOnExternalId("123456-AA",$menu_id);
		attachMerchantToSkin($merchant_resource->merchant_id, getSkinIdForContext());

		$user = logTestUserIn($this->ids['user_id']);

		$order_adapter = new OrderAdapter($m);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','thenote');

		$order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
		$this->assertNull($order_resource->error);
		$complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id);

		$punch_controller = new PunchLoyaltyController($m,$user,$r,5);
		$punch_order_payload_array = $punch_controller->getPunchPayloadArrayForOrder($complete_order);
		$offset = 7 - date('I',getTomorrowTwelveNoonTimeStampDenver());

		$expected_payload = cleanUpDoubleSpacesCRLFTFromString('{
				"receipt_amount":'.$complete_order['grand_total'].',
				"cc_last4":"'.$user['last_four'].'",
				"store_number":"123456-AA",
				"employee_id":1,
				"employee_name":"Splickit Server",
				"menu_items":[
				   {
					  "item_name":"My Special Name",
					  "item_qty":"1",
					  "item_amount":"'.$complete_order['order_details'][0]['item_total_w_mods'].'",
					  "menu_item_type":"M",
					  "menu_item_id":'.$test_id.',
					  "menu_family":"800",
					  "menu_major_group":"burritos"
				   }
				],
				"subtotal_amount":'.$complete_order['order_amt'].',
				"receipt_datetime":"'.date('Y-m-d',getTomorrowTwelveNoonTimeStampDenver()).'T12:00:00-0'.$offset.':00",
				"transaction_no":'.$order_resource->order_id.',
				"external_uid":"'.$order_resource->ucid.'",
				"client":"'.getProperty('punch_client_id').'"
				}');
		$expected_payload_array = json_decode($expected_payload,true);

		$this->assertEquals($expected_payload_array,$punch_order_payload_array,"It should have generated the expected array");

	}




	static function setUpBeforeClass()
    {
        setContext('com.splickit.moes');
    	$_SERVER['request_time1'] = microtime(true);
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	ini_set('max_execution_time',300);
    	      SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();$mysqli->begin_transaction(); ;
		$_SERVER['no-welcome'];

		$brand_resource = Resource::find(new BrandAdapter($m),112,$option);
		$brand_resource->loyalty = 'Y';
		$brand_resource->save();

		$blr_data['brand_id'] = 112;
		$blr_data['loyalty_type'] = 'remote';
		$blr_data['charge_tax'] = false;
		$brand_loyalty_rules_resource = Resource::factory(new BrandLoyaltyRulesAdapter($m),$blr_data);
		$brand_loyalty_rules_resource->save();

		$menu_id = createTestMenuWithNnumberOfItems(1);
		$modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 5);
		$modifier_group_id = $modifier_group_resource->modifier_group_id;

		$item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
		assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 0);

		$test_id = '88888';
		$test_name = 'Item Only Name';
		$test_group = 'LTO';
		$moia = new AdmMoesAlohaInfoAdapter($m);
		$data['item_id'] = $item_records[0]['item_id'];
		$data['name'] = $test_name;
		$data['aloha_id'] = $test_id;
		$data['rule'] = 'item';
		$data['major_group'] = $test_group;
		$resource = Resource::createByData($moia,$data);
		$ids['adm_moes_aloha_record'] = $resource->getDataFieldsReally();
		$ids['menu_id'] = $menu_id;


		$user_resource = createNewUserWithCC();
		$user_resource->flags = '1C20000001';
		$user_resource->save();
    	$ids['user_id'] = $user_resource->user_id;
    	
    	$_SERVER['log_level'] = 5;
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
	PunchIntegrationTest::main();
}

?>