<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class RemoteLoyaltyTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
        $this->ids = $_SERVER['unit_test_ids'];
        setContext($this->ids['context']);

    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->stamp);
        unset($this->ids);
    }

    function testSubmitInstorePurchaseAmountCreateUserIfFlagIsSetOnLoyalty()
    {
        clearAuthenticatedUserParametersForSession();
        $phone = ''.rand(1111111111,9999999999);
        $data['order_amount'] = 8.88;
        $remote_order_number = rand(111111,999999);
        $data['order_number'] = $remote_order_number;
        $data['phone_number'] = $phone;
        $data['location_id'] = "99999";
        $data['first_name'] = 'sumdumname';
        $data['last_name'] = 'sumdumlastname';
        $data['new_user'] = true;
        $url = "http://localhost/app2/pos/loyalty/$phone";
        $request = createRequestObject($url,'POST',json_encode($data),'application/json');
        $pos_controller = new PosController($m,$user,$request,5);
        $response = makeMockPortalRequest($pos_controller);
        $this->assertContains('{"success":"true"}',$response);


        $lplra = new LoyaltyParkingLotRecordsAdapter($m);
        $expected_phone = substr($phone,1);
        $lplr_record = $lplra->getRecord(array("brand_id"=>getBrandIdFromCurrentContext(),"phone_number"=>$expected_phone));
        $this->assertNull($lplr_record,'No more parking lot');

        //user should exist now with phone number as email
        $user_resource = Resource::find(new UserAdapter(getM()),null,[TONIC_FIND_BY_METADATA=>['email'=>$phone]]);
        $this->assertEquals($phone,$user_resource->email);
        $this->assertEquals('Sumdumname',$user_resource->first_name);


        //loyalty record should exist with history
        $ublh_adapter = new UserBrandLoyaltyHistoryAdapter(getM());
        $ublh_records = $ublh_adapter->getRecords(array("order_id"=>$remote_order_number));
        $this->assertCount(1,$ublh_records,"There should be 1 record for this order");
        $ublh_record = $ublh_records[0];
        $this->assertEquals(LoyaltyController::IN_STORE_PURCHASE_LABEL,$ublh_record['process']);
        $this->assertEquals(89,$ublh_record['points_added']);
        $this->assertEquals('location 99999',$ublh_record['notes']);

        return $phone;
    }

    /**
     * @depends testSubmitInstorePurchaseAmountCreateUserIfFlagIsSetOnLoyalty
     */
    function testGetLoyaltyInfo($phone)
    {
        $request = createRequestObject("/app2/pos/loyalty/$phone");
        $pos_controller = new PosController(getM(),null,$request);
        $response = makeMockPortalRequest($pos_controller);
        $response_as_array = json_decode($response,true);
        $this->assertEquals(89,$response_as_array['data']['points']);
        $this->assertEquals(.89,$response_as_array['data']['dollar_balance']);
        $this->assertEquals(getBrandIdFromCurrentContext(),$response_as_array['data']['brand_id']);
        return $phone;
    }

    /**
     * @depends testGetLoyaltyInfo
     */
    function testSubmitInstorePurchaseAmountToAPItNewUserButUserWithSamePhoneNumberExists($phone)
    {
        clearAuthenticatedUserParametersForSession();
        $data['order_amount'] = 3.33;
        $remote_order_number = rand(111111,999999);
        $data['order_number'] = $remote_order_number;
        $data['phone_number'] = $phone;
        $data['new_user'] = true;
        $url = "http://localhost/app2/pos/loyalty/$phone";
        $request = createRequestObject($url,'POST',json_encode($data),'application/json');
        $pos_controller = new PosController(getM(),null,$request,5);
        $response = makeMockPortalRequest($pos_controller);
        $response_as_array = json_decode($response,true);
        $user_resource = Resource::find(new UserAdapter(getM()),null,[TONIC_FIND_BY_METADATA=>['email'=>$phone]]);
        $user = $user_resource->getDataFieldsReally();
        $first_name = $user['first_name'];
        $last_name = $user['last_name'];
        $this->assertEquals(LoyaltyController::LOYALTY_NUMBER_EXISTS_FOR_REMOTE_JOIN_ERROR."$first_name $last_name",$response_as_array['error']['error_message']);
        $this->assertEquals(LoyaltyController::LOYALTY_NUMBER_EXISTS_FOR_REMOTE_JOIN_ERROR_CODE,$response_as_array['error']['error_code']);

        return $phone;
    }

    /**
     * @depends testGetLoyaltyInfo
     */
    function testSubmitInstorePurchaseAmountToAPIPhoneNumberExists($phone)
    {
        clearAuthenticatedUserParametersForSession();
        $data['order_amount'] = 45.00;
        $remote_order_number = rand(111111,999999);
        $data['order_number'] = $remote_order_number;
        $data['phone_number'] = $phone;
        $url = "http://localhost/app2/pos/loyalty/$phone";
        $request = createRequestObject($url,'POST',json_encode($data),'application/json');
        $pos_controller = new PosController(getM(),null,$request,5);
        $response = makeMockPortalRequest($pos_controller);
        $response_as_array = json_decode($response,true);
        $this->assertEquals(200,$response_as_array['http_code']);

        $request = createRequestObject($url);
        $pos_controller = new PosController(getM(),null,$request);
        $response = makeMockPortalRequest($pos_controller);
        $response_as_array = json_decode($response,true);
        $this->assertEquals(539,$response_as_array['data']['points']);
        $this->assertEquals(5.39,$response_as_array['data']['dollar_balance']);
        $this->assertEquals(getBrandIdFromCurrentContext(),$response_as_array['data']['brand_id']);
        return $phone;
    }

    /**
     * @depends testSubmitInstorePurchaseAmountToAPIPhoneNumberExists
     */
    function testJoinOnWebAndconvertExistingRemoteLoyaltyOnlyAccount($phone)
    {
        $existing_remote_user_resource = Resource::find(new UserAdapter(getM()),null,[TONIC_FIND_BY_METADATA=>['email'=>$phone]]);
        $existing_uuid = $existing_remote_user_resource->uuid;
        $existing_flags = $existing_remote_user_resource->flags;
        $this->assertEquals(UserController::STARTING_REMOTEUSER_FLAGS,$existing_flags,'It should have the remote user flags');

        $user = logTestUserIn(1);
        $url = "http://localhost/app2/apiv2/users?log_level=5";
        $new_user_data = createNewUserDataFields();
        $new_user_data['contact_no'] = $phone;
        $request = createRequestObject($url,'POST',json_encode($new_user_data));
        $user_controller = new UserController(getM(), $user, $request);
        $response = makeMockRequest($user_controller);
        $response_as_array = json_decode($response,true);
        $uuid = $response_as_array['data']['uuid'];

        $this->assertEquals($existing_uuid,$uuid,'it should have converted the existing remote user record to an full online ordering record');

        $user = getUserFromId($uuid);
        $user_id = $user['user_id'];

        $ubpm_adapter = new UserBrandPointsMapAdapter(getM());
        $ubpm_record = $ubpm_adapter->getRecord(array("user_id"=>$user['user_id'],"brand_id"=>getBrandIdFromCurrentContext()));
        $this->assertNotNull($ubpm_record,"it should have retrieved the existing loyalty record");
        $this->assertEquals($phone,$ubpm_record['loyalty_number'],"It should have set the loyalty number to the phone number");
        $this->assertEquals(539,$ubpm_record['points']);
        $this->assertEquals(5.39,$ubpm_record['dollar_balance']);


    }

    function testSubmitInstorePurchaseCreateUserButFailOnOrderSave()
    {
        clearAuthenticatedUserParametersForSession();
        $phone = ''.rand(1111111111,9999999999);
        $data['order_amount'] = -1.11;
        $remote_order_number = rand(111111,999999);
        $data['order_number'] = $remote_order_number;
        $data['phone_number'] = $phone;
        $data['location_id'] = "99999";
        $data['first_name'] = 'sumdumname';
        $data['last_name'] = 'sumdumlastname';
        $data['new_user'] = true;
        $url = "http://localhost/app2/pos/loyalty/$phone";
        $request = createRequestObject($url,'POST',json_encode($data),'application/json');
        $pos_controller = new PosController($m,$user,$request,5);
        $response = makeMockPortalRequest($pos_controller);
        $respnose_as_array = json_decode($response,true);
        $error = $respnose_as_array['error']['error_message'];
        $this->assertContains('User account has been created',$error);

        //user should exist now with phone number as email
        $user_resource = Resource::find(new UserAdapter(getM()),null,[TONIC_FIND_BY_METADATA=>['email'=>$phone]]);
        $this->assertEquals($phone,$user_resource->email);
        $this->assertEquals('Sumdumname',$user_resource->first_name);


        //loyalty record should exist with history
        $ublh_adapter = new UserBrandLoyaltyHistoryAdapter(getM());
        $ublh_records = $ublh_adapter->getRecords(array("order_id"=>$remote_order_number));
        $this->assertCount(0,$ublh_records,"There should be 0 records for this order");
    }

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        ini_set('max_execution_time',300);




        $goodcents_skin_resource = getOrCreateSkinAndBrandIfNecessary("Goodcents Subs","Goodcents Subs",140,430);
        $brand_resource = Resource::find(new BrandAdapter($m),"".$goodcents_skin_resource->brand_id);
        $brand_resource->loyalty = 'Y';
        $brand_resource->save();

        $blra = new BrandLoyaltyRulesAdapter($m);
        $gcblr = array("brand_id"=>430);
        $goodcents_brand_loyalty_rules_resource = Resource::findOrCreateIfNotExistsByData($blra,$gcblr);
        $goodcents_brand_loyalty_rules_resource->starting_point_value = 100;
        $goodcents_brand_loyalty_rules_resource->earn_value_amount_multiplier = 10;
        $goodcents_brand_loyalty_rules_resource->loyalty_type = 'splickit_earn';
        $goodcents_brand_loyalty_rules_resource->save();

        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction(); ;
        $_SERVER['request_time1'] = microtime(true);

        $brand_id = 88888;
        $skin_resource = getOrCreateSkinAndBrandIfNecessary("loy unit skin", "loyunitbrand", $skin_id, $brand_id);
        $brand_id = $skin_resource->brand_id;
        $brand_resource = Resource::find(new BrandAdapter($mimetypes),"$brand_id");
        $brand_resource->loyalty = 'Y';
        $brand_resource->save();

        $blr_data['brand_id'] = $brand_id;
        $blr_data['loyalty_type'] = 'splickit_earn';
        $brand_loyalty_rules_resource = Resource::factory(new BrandLoyaltyRulesAdapter($m),$blr_data);
        $brand_loyalty_rules_resource->save();
        $ids['blr_resource'] = $brand_loyalty_rules_resource->getRefreshedResource();
        setContext($skin_resource->external_identifier);
        $ids['context'] = $skin_resource->external_identifier;
        $menu_id = createTestMenuWithNnumberOfItems(5);
        $ids['menu_id'] = $menu_id;

        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $ids['merchant_id'] = $merchant_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;

    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    RemoteLoyaltyTest::main();
}

?>