<?php


$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';


class STSStoredValueTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
		$this->ids = $_SERVER['unit_test_ids'];
        setContext('com.splickit.stsskin2');
        $_SERVER['STS_PREVIOUS_BALANCE'] = 0.00;
	}

    function tearDown()
    {
        //delete your instance
        unset($this->ids);
        unset($_SERVER['max_lead']);
    }

    function testGuestPlaceOrderWithStoredValueAndResetAfterOrder()
    {
        $user_resource = createGuestUser();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($this->ids['merchant_id']);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data, getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $ucid = $checkout_resource->ucid;

        //validate that the stored value option is there
        $hash = createHashmapFromArrayOfArraysByFieldName($checkout_resource->accepted_payment_types,'name');
        $this->assertNotNull($hash['STS Stored Value'],'It should  have stored value payment option');

        //now add the stored value card to the user
        $card_number = '1234567890';
        $uuid = $user_resource->uuid;
        $url = "http://127.0.0.1/app2/apiv2/users/$uuid/stored_value/12000/add";
        $data = [];
        $data['card_number'] = $card_number;
        $request_object = createRequestObject($url,'POST',json_encode($data));
        $user_controller = new UserController(getM(),$user,$request_object,5);
        $response = makeMockRequest($user_controller);
        $response_array = json_decode($response,true);
        $this->assertEquals(200,$response_array['http_code']);

        $ussvma = new UserSkinStoredValueMapsAdapter(getM());
        $stored_card_number = $ussvma->getCardNumber($user['user_id'],getSkinIdForContext(),12000);
        $this->assertEquals($card_number,$stored_card_number,"It should have stored the card number");

        $stored_value_record = $ussvma->getStoredValueRecord();
        $this->assertEquals(77.77,$stored_value_record['balance']);

        $this->assertEquals(UserController::STORED_VALUE_ADD_SUCCESS_MESSAGE_FOR_GUEST."$77.77",$response_array['message']);

        // now call checkout again
        $checkout_resource2 = getCheckoutResourceFromCartUcid($ucid,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource2->error);
        $hash = createHashmapFromArrayOfArraysByFieldName($checkout_resource2->accepted_payment_types,'name');
        $this->assertNotNull($hash['STS Stored Value'],'It should  have stored value payment option');
        $checkout_resource2->accepted_payment_types = [$hash['STS Stored Value']];

        $order_resource = placeOrderFromCheckoutResource($checkout_resource2,$user,$this->ids['merchant_id'],0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order_id],'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
        $stored_value_row = $bcr_hash['StoredValuePayment'];
        $this->assertEquals('StsPaymentService',$stored_value_row['cc_processor']);

        // now check that the row was removed from the stored value map
        $ussvma = new UserSkinStoredValueMapsAdapter(getM());
        $stored_card_number = $ussvma->getCardNumber($user['user_id'],getSkinIdForContext(),12000);
        $this->assertNull($stored_card_number,"It should no longer have the stored value record");

    }


    function testResetStoredValueForGuestAfterOneHour()
    {
        $user_resource = createGuestUser();
        $user = logTestUserResourceIn($user_resource);
        $user_id = $user['user_id'];


        //now add the stored value card to the user
        $card_number = '1234567890';
        $uuid = $user_resource->uuid;
        $url = "http://127.0.0.1/app2/apiv2/users/$uuid/stored_value/12000/add";
        $data = [];
        $data['card_number'] = $card_number;
        $request_object = createRequestObject($url,'POST',json_encode($data));
        $user_controller = new UserController(getM(),$user,$request_object,5);
        $response = makeMockRequest($user_controller);
        $response_array = json_decode($response,true);
        $this->assertEquals(200,$response_array['http_code']);

        $ussvma = new UserSkinStoredValueMapsAdapter(getM());
        $stored_card_number = $ussvma->getCardNumber($user['user_id'],getSkinIdForContext(),12000);
        $this->assertEquals($card_number,$stored_card_number,"It should have stored the card number");

        $options[TONIC_FIND_BY_METADATA] = ['user_id'=>$user_id];
        $stored_value_resource = Resource::find($ussvma,null,$options);
        $this->assertEquals(77.77,$stored_value_resource->balance);

        $cctua = new CreditCardUpdateTrackingAdapter($m);
        $new_created = getMySqlFormattedDateTimeFromTimeStampAndTimeZone(time() - 3601,date_default_timezone_get());
        $this->assertFalse($stored_value_resource->created < (time() - 3598));

        // now run service and the user shoudl still have a CC attached.
        $guccca = new GuestUserCreditCardCheckActivity();
        $guccca->doIt();

        $stored_value_resource = Resource::find($ussvma,null,$options);
        $this->assertEquals(77.77,$stored_value_resource->balance);


        // update cc update record to show it was updated over an hour ago
        $sql = "UPDATE User_Skin_Stored_Value_Maps SET created = '$new_created' WHERE id = ".$stored_value_resource->id;
        $ussvma->_query($sql);

        $stored_value_resource = $stored_value_resource->refreshResource();
        $this->assertTrue($stored_value_resource->created < (time() - 3598));

        // now run service and the user should have the CC erased
        $guccca = new GuestUserCreditCardCheckActivity();
        $guccca->doIt();

        $stored_value_resource = Resource::find($ussvma,null,$options);
        $this->assertNull($stored_value_resource,"It should no longer have the record in the Table");
    }

    function testFullPaymentWithoutCC()
    {
        $user_resource = createNewUser();
        $user_id = $user_resource->user_id;
        $user = logTestUserResourceIn($user_resource);
        $ubpa = new UserBrandPointsMapAdapter(getM());
        $ubpa->createUserLoyaltyAccount($user_id, $this->ids['brand_id'], $user_resource->contact_no);
        // create account
        $ubsvma = new UserSkinStoredValueMapsAdapter(getM());
        $ubsvma->saveCardNumber('1234588888',$user_id,getSkinIdForContext(),12000);

        $merchant_id = $this->ids['merchant_id'];
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','sum dum note',10);

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $hash = createHashmapFromArrayOfArraysByFieldName($checkout_resource->accepted_payment_types,'name');
        $this->assertNotNull($hash['STS Stored Value'],'It should  have stored value payment option');

        $checkout_resource->accepted_payment_types = [$hash['STS Stored Value']];
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);

        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order_resource->order_id],'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
        $this->assertCount(2,$bcr_hash,"It should have 2 BalanceChange records");

        // now check users balance
        $user_resource = $user_resource->getRefreshedResource();
        $this->assertEquals(0.00,$user_resource->balance);

        $this->assertNotNull($bcr_hash['StoredValuePayment']);
        $this->assertEquals($order_resource->grand_total,$bcr_hash['StoredValuePayment']['charge_amt']);

    }

    function testDenyPartialPaymentWithoutCC()
    {
        $user_resource = createNewUser();
        $user_id = $user_resource->user_id;
        $user = logTestUserResourceIn($user_resource);
        $ubpa = new UserBrandPointsMapAdapter(getM());
        $ubpa->createUserLoyaltyAccount($user_id, $this->ids['brand_id'], $user_resource->contact_no);
        // create account
        $ubsvma = new UserSkinStoredValueMapsAdapter(getM());
        $ubsvma->saveCardNumber('1234522222',$user_id,getSkinIdForContext(),12000);

        $merchant_id = $this->ids['merchant_id'];
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','sum dum note',10);

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $hash = createHashmapFromArrayOfArraysByFieldName($checkout_resource->accepted_payment_types,'name');
        $this->assertNotNull($hash['STS Stored Value'],'It should  have stored value payment option');

        $checkout_resource->accepted_payment_types = [$hash['STS Stored Value']];
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNotNull($order_resource->error);

    }

    function testPartialPaymentWithCCOnFile()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user_id = $user_resource->user_id;
        $user = logTestUserResourceIn($user_resource);
        $ubpa = new UserBrandPointsMapAdapter(getM());
        $ubpa->createUserLoyaltyAccount($user_id, $this->ids['brand_id'], $user_resource->contact_no);
        // create account
        $ubsvma = new UserSkinStoredValueMapsAdapter(getM());
        $ubsvma->saveCardNumber('1234522222',$user_id,getSkinIdForContext(),12000);



        $merchant_id = $this->ids['merchant_id'];
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','sum dum note',10);

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $hash = createHashmapFromArrayOfArraysByFieldName($checkout_resource->accepted_payment_types,'name');
        $this->assertNotNull($hash['STS Stored Value'],'It should  have stored value payment option');

        $checkout_resource->accepted_payment_types = [$hash['STS Stored Value']];
        $_SERVER['STS_PREVIOUS_BALANCE'] = 5.00;
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);

        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order_resource->order_id],'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
        $this->assertCount(3,$bcr_hash,"It should have 3 BalanceChange records");

        $this->assertNotNull($bcr_hash['StoredValuePayment']);
        $cc_payment_row = $bcr_hash['Authorize'];
        $this->assertNotNull($cc_payment_row,"It should have a cc payment row");
        $expected_cc_charge_amt = $order_resource->grand_total - 5.00;
        $this->assertEquals($expected_cc_charge_amt,$cc_payment_row['charge_amt']);

        // now check users balance
        $user_resource = $user_resource->getRefreshedResource();
        $this->assertEquals(0.00,$user_resource->balance);
    }

    function testPaymentWithStsButNotCardOnFile()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user_id = $user_resource->user_id;
        $user = logTestUserResourceIn($user_resource);
//        $ubpa = new UserBrandPointsMapAdapter(getM());
//        $ubpa->createUserLoyaltyAccount($user_id, $this->ids['brand_id'], $user_resource->contact_no);
        // create account
//        $ubsvma = new UserSkinStoredValueMapsAdapter(getM());
//        $ubsvma->saveCardNumber('1234511111',$user_id,getSkinIdForContext(),12000);



        $merchant_id = $this->ids['merchant_id'];
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','sum dum note',10);

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $hash = createHashmapFromArrayOfArraysByFieldName($checkout_resource->accepted_payment_types,'name');
        $this->assertNotNull($hash['STS Stored Value'],'It should have stored value payment option');
        //$this->assertNotNull($hash['Stored Value + CC'],'It should have the stored value + cc payment option');

        $cc_payment_type = $checkout_resource->accepted_payment_types[0];
        $sts_payment_type = $checkout_resource->accepted_payment_types[1];
        $checkout_resource->accepted_payment_types = [$sts_payment_type];
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNotNull($order_resource->error,'It should have thrown an error since the user does not have a stored value card.');
        $this->assertEquals(StsPaymentService::USER_DOES_NOT_HAVE_A_CARD_ON_RECORD_ERROR,$order_resource->error);

        // lets make sure that cart is still in checkout mode
        $order_id = $checkout_resource->oid_test_only;
        $order = new Order($order_id);
        $this->assertEquals('P',$order->get('status'),"It should still be in a pending state");

    }

    function testDoNotAllowPaymentWithStoredValueCardIfBalanceIsntEnoughToCoverTheCharge()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user_id = $user_resource->user_id;
        $user = logTestUserResourceIn($user_resource);
        $ubpa = new UserBrandPointsMapAdapter(getM());
        $ubpa->createUserLoyaltyAccount($user_id, $this->ids['brand_id'], $user_resource->contact_no);
        // create account
        $ubsvma = new UserSkinStoredValueMapsAdapter(getM());
        $ubsvma->saveCardNumber('1234511111',$user_id,getSkinIdForContext(),12000);



        $merchant_id = $this->ids['merchant_id'];
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','sum dum note',10);

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $hash = createHashmapFromArrayOfArraysByFieldName($checkout_resource->accepted_payment_types,'name');
        $this->assertNotNull($hash['STS Stored Value'],'It should not have stored value payment option');
        //$this->assertNotNull($hash['Stored Value + CC'],'It should have the stored value + cc payment option');

        $cc_payment_type = $checkout_resource->accepted_payment_types[0];
        $sts_payment_type = $checkout_resource->accepted_payment_types[1];
        $checkout_resource->accepted_payment_types = [$sts_payment_type];
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNotNull($order_resource->error,'It should have thrown an error since the stored value card does not have enough balance on it.');
        $this->assertEquals(StsPaymentService::CARD_DOES_NOT_HAVE_ENOUGH_VALUE_TO_COVER_CHARGE_ERROR,$order_resource->error);

        $checkout_resource->accepted_payment_types = [$cc_payment_type];
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);

        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order_resource->order_id],'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
        $this->assertNull($bcr_hash['StoredValuePayment']);
        $cc_payment_row = $bcr_hash['Authorize'];
        $this->assertNotNull($cc_payment_row,"It should have a cc payment row");
        $expected_cc_charge_amt = $order_resource->grand_total;
        $this->assertEquals($expected_cc_charge_amt,$cc_payment_row['charge_amt']);

    }


    function testFailVoidRolloverToRefund()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user_id = $user_resource->user_id;
        $user = logTestUserResourceIn($user_resource);
        $ubpa = new UserBrandPointsMapAdapter(getM());
        $ubpa->createUserLoyaltyAccount($user_id, $this->ids['brand_id'], $user_resource->contact_no);

        $uuid = $user_resource->uuid;
        $url = "http://127.0.0.1/app2/apiv2/users/$uuid/stored_value/12000/add";
        $data = [];
        $data['card_number'] = '9876543210';
        $request_object = createRequestObject($url,'POST',json_encode($data));
        $user_controller = new UserController(getM(),$user,$request_object,5);
        $response = makeMockRequest($user_controller);
        $response_array = json_decode($response,true);

        $merchant_id = $this->ids['merchant_id'];

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','the note',5);

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $sts_payment_type = $checkout_resource->accepted_payment_types[1];
        $checkout_resource->accepted_payment_types = [$sts_payment_type];

        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order_id],'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
        $stored_value_row = $bcr_hash['StoredValuePayment'];
        $this->assertEquals('StsPaymentService',$stored_value_row['cc_processor']);
        $stored_value_resource = Resource::find(new BalanceChangeAdapter(getM()),$stored_value_row['id']);
        $stored_value_resource->cc_transaction_id = '0088:8888888';
        $stored_value_resource->save();
        $request = new Request();
        $request->data['note'] = "Sum dum note";
        $request->data['employee_name'] = 'sara';
        $order_controller = new OrderController(getM(), $user, $request, 5);
        $refund_results = $order_controller->issueOrderRefund($order_id, "0.00");
        $this->assertEquals("success", $refund_results['result']," should have gotten a success but: ".$refund_results['message']);

        // balance on card should go back up 2.50
//        $ubsvma = new UserSkinStoredValueMapsAdapter(getM());
//        $record = $ubsvma->getRecord(["user_id"=>$user_id,"skin_id"=>getSkinIdForContext(),"splickit_accepted_payment_type_id"=>12000]);
//        $expected_balance = 100 - $order_resource->grand_total + 2.50;
//        $this->assertEquals($expected_balance,$record['balance'],"It should have the correct balance on the card");

        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order_id],'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
        $this->assertCount(2,$bcr);
        $this->assertEquals('REFUNDED',$bcr_hash['StoredValuePayment']['notes']);

        $aora = new AdmOrderReversalAdapter(getM());
        $record = $aora->getRecord(['order_id'=>$order_id]);
        $this->assertEquals($order_resource->grand_total,$record['amount']);


        $request = new Request();
        $request->data['note'] = "Sum dum note again";
        $order_controller = new OrderController(getM(), $user, $request, 5);
        $refund_results = $order_controller->issueOrderRefund($order_id, "1.25");
        $this->assertEquals("failure", $refund_results['result']," should have gotten a failure since you cant do two partials on the same order.");

    }

    //  ***** currently not allowing split charges ******

//    function testSplitPaymentWithSTSButNotEnoughToCoverEntireCharge()
//    {
//        $user_resource = createNewUserWithCCNoCVV();
//        $user_id = $user_resource->user_id;
//        $user = logTestUserResourceIn($user_resource);
//        $ubpa = new UserBrandPointsMapAdapter(getM());
//        $ubpa->createUserLoyaltyAccount($user_id, $this->ids['brand_id'], $user_resource->contact_no);
//        // create account
//        $ubsvma = new UserSkinStoredValueMapsAdapter(getM());
//        $ubsvma->saveCardNumber('1111111111',$user_id,getSkinIdForContext(),12000);
//
//
//
//        $merchant_id = $this->ids['merchant_id'];
//        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','sum dum note',10);
//
//        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
//        $this->assertNull($checkout_resource->error);
//        $hash = createHashmapFromArrayOfArraysByFieldName($checkout_resource->accepted_payment_types,'name');
//        $this->assertNull($hash['STS Stored Value'],'It should not have stored value payment option');
//        $this->assertNotNull($hash['Stored Value + CC'],'It should have the stored value + cc payment option');
//
//        $sts_payment_type = $checkout_resource->accepted_payment_types[1];
//        $checkout_resource->accepted_payment_types = [$sts_payment_type];
//        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
//        $this->assertNull($order_resource->error);
//
//        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order_resource->order_id],'BalanceChangeAdapter');
//        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
//        $stored_value_row = $bcr_hash['StoredValuePayment'];
//        $this->assertEquals('StsPaymentService',$stored_value_row['cc_processor']);
//        $this->assertEquals(5.25,$stored_value_row['charge_amt']);
//        $cc_payment_row = $bcr_hash['Authorize'];
//        $this->assertNotNull($cc_payment_row,"It should have a cc payment row");
//        $expected_cc_charge_amt = $order_resource->grand_total - 5.25;
//        $this->assertEquals($expected_cc_charge_amt,$cc_payment_row['charge_amt']);
//
//    }


    function testBetterErrorForBadCard()
    {
        $card_number = '1001001';
        $user_resource = createNewUser();
        $user = logTestUserResourceIn($user_resource);

        $uuid = $user_resource->uuid;
        $url = "http://127.0.0.1/app2/apiv2/users/$uuid/stored_value/12000/add";
        $data = [];
        $data['card_number'] = $card_number;
        $request_object = createRequestObject($url,'POST',json_encode($data));
        $user_controller = new UserController(getM(),$user,$request_object,5);
        $response = makeMockRequest($user_controller);
        $response_as_array = json_decode($response,true);
        $this->assertEquals(422,$response_as_array['http_code']);
        $this->assertEquals(StsPaymentService::CARD_NUMBER_IS_INVALID_ERROR_MESSAGE,$response_as_array['error']['error']);

        $ussvma = new UserSkinStoredValueMapsAdapter(getM());
        $records = $ussvma->getRecords(['user_id'=>$user['user_id']]);
        $this->assertCount(0,$records);

    }

    function testDoNotCreateSecondRowForAUserSkinCombination()
    {
        $card_number = rand(111111111,999999999);
        $user_resource = createNewUser();
        $user = logTestUserResourceIn($user_resource);

        $uuid = $user_resource->uuid;
        $url = "http://127.0.0.1/app2/apiv2/users/$uuid/stored_value/12000/add";
        $data = [];
        $data['card_number'] = $card_number;
        $request_object = createRequestObject($url,'POST',json_encode($data));
        $user_controller = new UserController(getM(),$user,$request_object,5);
        $response = makeMockRequest($user_controller);

        $ussvma = new UserSkinStoredValueMapsAdapter(getM());
        $records = $ussvma->getRecords(['user_id'=>$user['user_id']]);
        $this->assertCount(1,$records);

        // now try adding it again
        $user_controller = new UserController(getM(),$user,$request_object,5);
        $response = makeMockRequest($user_controller);

        $ussvma = new UserSkinStoredValueMapsAdapter(getM());
        $records = $ussvma->getRecords(['user_id'=>$user['user_id']]);
        $this->assertCount(1,$records,"It shoiuld still only have one record");

        // now try with different card number and it shoould replace it
        $card_number2 = rand(111111111,999999999);

        $data = [];
        $data['card_number'] = $card_number2;
        $request_object = createRequestObject($url,'POST',json_encode($data));
        $user_controller = new UserController(getM(),$user,$request_object,5);
        $response = makeMockRequest($user_controller);

        $ussvma = new UserSkinStoredValueMapsAdapter(getM());
        $records = $ussvma->getRecords(['user_id'=>$user['user_id']]);
        $this->assertCount(1,$records);

        $this->assertEquals($card_number2,$records[0]['card_number']);




    }


    function testAddCardToUserAccount()
    {
        $card_number = '1234567890';
        $user_resource = createNewUser();
        $user = logTestUserResourceIn($user_resource);




        $uuid = $user_resource->uuid;
        $url = "http://127.0.0.1/app2/apiv2/users/$uuid/stored_value/12000/add";
        $data = [];
        $data['card_number'] = $card_number;
        $request_object = createRequestObject($url,'POST',json_encode($data));
        $user_controller = new UserController(getM(),$user,$request_object,5);
        $response = makeMockRequest($user_controller);
        $response_array = json_decode($response,true);

        $ussvma = new UserSkinStoredValueMapsAdapter(getM());
        $stored_card_number = $ussvma->getCardNumber($user['user_id'],getSkinIdForContext(),12000);
        $this->assertEquals($card_number,$stored_card_number,"It should have stored the card number");

        $stored_value_record = $ussvma->getStoredValueRecord();
        $this->assertEquals(77.77,$stored_value_record['balance']);

        $this->assertEquals("Your card info was saved successfully. You have a balance of $77.77",$response_array['message']);
    }

    ///  ************  tests to figure out when and what to show for payment options *************

    function testShowSTSPaymentTypeWhenSkinAndMerchantAreAssociatedOnly()
    {
        // create new skin with no STS association
        $skin_resource = getOrCreateSkinAndBrandIfNecessaryWithLoyalty("nonstsskin","nonstsbrand",null, null);
        $merchant_id = $this->ids['merchant_id'];
        attachMerchantToSkin($merchant_id, $skin_resource->skin_id);
        setContext($skin_resource->external_identifier);

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $user_id = $user_resource->user_id;
        $uuid = $user_resource->uuid;

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $payment_hash = createHashmapFromArrayOfArraysByFieldName($checkout_resource->accepted_payment_types,'splickit_accepted_payment_type_id');
        $this->assertFalse(isset($payment_hash[StsPaymentService::SPLICKIT_ACCEPTED_PAYMENT_TYPE_ID_FOR_STS]),"It should not have the sts stored value Payment type");

        // now change skins and it should appear
        setContext('com.splickit.stsskin2');
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $payment_hash = createHashmapFromArrayOfArraysByFieldName($checkout_resource->accepted_payment_types,'splickit_accepted_payment_type_id');
        $this->assertTrue(isset($payment_hash[StsPaymentService::SPLICKIT_ACCEPTED_PAYMENT_TYPE_ID_FOR_STS]),"It should not have the sts stored value Payment type");

        $user_info = $checkout_resource->user_info;
        $this->assertTrue('' === $user_info['stored_value_card_number'],'It should have a blank field for the stored value card number' );

        $ussvma = new UserSkinStoredValueMapsAdapter(getM());
        $svm_resource = $ussvma->saveCardNumber('88888888',$user_id,getSkinIdForContext(),StsPaymentService::SPLICKIT_ACCEPTED_PAYMENT_TYPE_ID_FOR_STS);
        $svm_resource->balance = 10.10;
        $svm_resource->save();

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $user_info = $checkout_resource->user_info;
        $this->assertTrue('88888888' === $user_info['stored_value_card_number'],'It should have a blank field for the stored value card number' );
        $this->assertEquals(88.88,$user_info['stored_value_card_balance'],"It should have retrieved the balance from the remote system");
    }

    function testEndToEndCharge()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user_id = $user_resource->user_id;
        $user = logTestUserResourceIn($user_resource);
        $ubpa = new UserBrandPointsMapAdapter(getM());
        $ubpa->createUserLoyaltyAccount($user_id, $this->ids['brand_id'], $user_resource->contact_no);

        $merchant_id = $this->ids['merchant_id'];

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $cart_ucid = $checkout_resource->ucid;
        $sts_payment_type = $checkout_resource->accepted_payment_types[1];
        $checkout_resource->accepted_payment_types = [$sts_payment_type];
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        // order should fail since user does not have a card number on file with us
        $this->assertNotNull($order_resource->error);
        $this->assertEquals(StsPaymentService::USER_DOES_NOT_HAVE_A_CARD_ON_RECORD_ERROR,$order_resource->error);

        // create account
        $ubsvma = new UserSkinStoredValueMapsAdapter(getM());
        $ubsvma->saveCardNumber('1234567890',$user_id,getSkinIdForContext(),12000);
        $request = createRequestObject("/apiv2/cart/$cart_ucid/checkout", 'GET');
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($checkout_resource->error);
        $sts_payment_type = $checkout_resource->accepted_payment_types[1];
        $checkout_resource->accepted_payment_types = [$sts_payment_type];
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);

        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order_resource->order_id],'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
        $stored_value_row = $bcr_hash['StoredValuePayment'];
        $this->assertEquals('StsPaymentService',$stored_value_row['cc_processor']);
        return $order_resource->order_id;
    }

    /**
     * @depends testEndToEndCharge
     */
    function testRefundStoredValueCharge($order_id)
    {
        $order_resource = SplickitController::getResourceFromId($order_id, "Order");
        $user = logTestUserIn($order_resource->user_id);
        $request = new Request();
        $request->data['note'] = "Sum dum note";
        $order_controller = new OrderController(getM(), $user, $request, 5);
        $refund_results = $order_controller->issueOrderRefund($order_id, "50.00");
        $this->assertEquals('failure',$refund_results['result']);
        $refund_results = $order_controller->issueOrderRefund($order_id, $order_resource->grand_total);
        $this->assertEquals("success", $refund_results['result']," should have gotten a success but: ".$refund_results['message']);

        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order_id],'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
        $this->assertCount(2,$bcr);
        $this->assertEquals('VOIDED',$bcr_hash['StoredValuePayment']['notes']);

        $aora = new AdmOrderReversalAdapter(getM());
        $record = $aora->getRecord(['order_id'=>$order_id]);
        $this->assertEquals($order_resource->grand_total,$record['amount']);


        // now try refunding again
        $refund_results = $order_controller->issueOrderRefund($order_id, "0.00");
        $this->assertEquals("failure", $refund_results['result'],"Should have gotten a failure since order had already been voided");
        $this->assertEquals(OrderController::ORDER_HAS_ALREADY_BEEN_REFUNDED_MESSAGE,$refund_results['message']);
    }

    function testPartialRefund()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user_id = $user_resource->user_id;
        $user = logTestUserResourceIn($user_resource);
        $ubpa = new UserBrandPointsMapAdapter(getM());
        $ubpa->createUserLoyaltyAccount($user_id, $this->ids['brand_id'], $user_resource->contact_no);

        $uuid = $user_resource->uuid;
        $url = "http://127.0.0.1/app2/apiv2/users/$uuid/stored_value/12000/add";
        $data = [];
        $data['card_number'] = '9876543210';
        $request_object = createRequestObject($url,'POST',json_encode($data));
        $user_controller = new UserController(getM(),$user,$request_object,5);
        $response = makeMockRequest($user_controller);
        $response_array = json_decode($response,true);

        $merchant_id = $this->ids['merchant_id'];

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','the note',4);

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $sts_payment_type = $checkout_resource->accepted_payment_types[1];
        $checkout_resource->accepted_payment_types = [$sts_payment_type];

        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order_id],'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
        $stored_value_row = $bcr_hash['StoredValuePayment'];
        $this->assertEquals('StsPaymentService',$stored_value_row['cc_processor']);
        $_SERVER['STS_PREVIOUS_BALANCE'] = $_SERVER['STS_PREVIOUS_BALANCE'] - $stored_value_row['charge_amt'];
        $request = new Request();
        $request->data['note'] = "Sum dum note";
        $order_controller = new OrderController(getM(), $user, $request, 5);
        $refund_results = $order_controller->issueOrderRefund($order_id, "2.50");
        $this->assertEquals("success", $refund_results['result']," should have gotten a success but: ".$refund_results['message']);

        // balance on card should go back up 2.50
//        $ubsvma = new UserSkinStoredValueMapsAdapter(getM());
//        $record = $ubsvma->getRecord(["user_id"=>$user_id,"skin_id"=>getSkinIdForContext(),"splickit_accepted_payment_type_id"=>12000]);
//        $expected_balance = 100 - $order_resource->grand_total + 2.50;
//        $this->assertEquals($expected_balance,$record['balance'],"It should have the correct balance on the card");

        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order_id],'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
        $this->assertCount(3,$bcr);
        $this->assertEquals(2.50,$bcr_hash['PartialRefund']['charge_amt']);

        $aora = new AdmOrderReversalAdapter(getM());
        $record = $aora->getRecord(['order_id'=>$order_id]);
        $this->assertEquals(2.50,$record['amount']);


        $request = new Request();
        $request->data['note'] = "Sum dum note again";
        $order_controller = new OrderController(getM(), $user, $request, 5);
        $refund_results = $order_controller->issueOrderRefund($order_id, "1.25");
        $this->assertEquals("failure", $refund_results['result']," should have gotten a failure since you cant do two partials on the same order.");


    }

    function testBetterErrorForBadAPIKey()
    {
        $resource = $this->ids['skin_sts_info_map_resource'];
//        $resource->api_key = "4567ujhnbvfrty";
//        $resource->save();
        $id = $resource->id;
        $sql = "UPDATE Skin_Sts_Info_Maps set api_key = '4567yujhbvfrtyu' WHERE id = $id";
        $ssima = new SkinStsInfoMapsAdapter(getM());
        $ssima->_query($sql);
        $card_number = rand(111111111,999999999);
        $user_resource = createNewUser();
        $user = logTestUserResourceIn($user_resource);

        $uuid = $user_resource->uuid;
        $url = "http://127.0.0.1/app2/apiv2/users/$uuid/stored_value/12000/add";
        $data = [];
        $data['card_number'] = $card_number;
        $request_object = createRequestObject($url,'POST',json_encode($data));
        $user_controller = new UserController(getM(),$user,$request_object,5);
        $response = makeMockRequest($user_controller);
        $response_as_array = json_decode($response,true);
        $this->assertEquals(500,$response_as_array['http_code']);
        $this->assertEquals(StsPaymentService::CONNECTON_ERROR_TO_REMOTE_SERVER,$response_as_array['error']['error']);


    }



    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        $skin_resource = getOrCreateSkinAndBrandIfNecessaryWithLoyalty("stsskin2","stsbrand2",null, null);
        ini_set('max_execution_time', 0);
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction(); ;

        setContext('com.splickit.stsskin2');

    	$simple_menu_id = createTestMenuWithOneItem("item_one");
    	$ids['menu_id'] = $simple_menu_id;
    	
    	$merchant_resource = createNewTestMerchant($simple_menu_id,['authorize'=>true]);
    	attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
    	$ids['merchant_id'] = $merchant_resource->merchant_id;

    	$msa = new SkinStsInfoMapsAdapter(getM());
    	$data['skin_id'] = $skin_resource->skin_id;
    	$data['merchant_number'] = '111111111111';
    	$data['terminal_id'] = '275';
    	$data['api_key'] = 'h*B7833Mp1tZ';
    	$ids['skin_sts_info_map_resource'] = Resource::createByData($msa,$data);

    	$merchant_payment_type_map_resource = MerchantPaymentTypeMapsAdapter::createMerchantPaymentTypeMap($merchant_resource->merchant_id, 12000, null);

    	$_SERVER['log_level'] = 5;
		$_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    STSStoredValueTest::main();
}
?>