<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class SkinSpecificMenuTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $code = generateCode(7);
        $_SERVER['STAMP'] = __CLASS__ . '-' . $code;
        $_SERVER['RAW_STAMP'] = $code;
        $this->ids = $_SERVER['unit_test_ids'];
        setContext('com.splickit.worldhq');
        setProperty('use_merchant_caching','false');
    }

    function tearDown()
    {
        //delete your instance
        unset($this->ids);
        unset($_SERVER['max_lead']);
    }

    function testGetMenuForSpecificSkin()
    {
        $brand_id = $this->ids['brand_id'];
        $merchant_id = $this->ids['merchant_id'];

        $alt_skin_resource = getOrCreateSkinAndBrandIfNecessary('altskin','localbrand',null,$brand_id);
        $alt_skin_id = $alt_skin_resource->skin_id;

        attachMerchantToSkin($merchant_id,$alt_skin_id);
        $alt_menu_id = createTestMenuWithNnumberOfItems(1);


        // now create the second menu's daughter records
        createDaughterRecordsForMerchantMenuCombination($merchant_id,$alt_menu_id);

        MerchantMenuMapAdapter::createMerchantMenuMap($merchant_id, $alt_menu_id, 'pickup', $alt_skin_id);
        MerchantMenuMapAdapter::createMerchantMenuMap($merchant_id, $alt_menu_id, 'delivery', $alt_skin_id);

        setContext('com.splickit.localskin');

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $url = "/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,"GET");
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $response = makeMockRequest($merchant_controller);
        $response_as_array = json_decode($response,true);
        $pulled_menu_id = $response_as_array['data']['menu_id'];
        $this->assertEquals($this->ids['default_menu_id'],$pulled_menu_id);


        //now change the context
        removeContext();
        setContext('com.splickit.altskin');

        $request = createRequestObject($url,"GET");
        $merchant_controller = new MerchantController(getM(), null, $request, 5);
        $response = makeMockRequest($merchant_controller);
        $response_as_array = json_decode($response,true);
        $pulled_menu_id = $response_as_array['data']['menu_id'];
        $this->assertEquals($alt_menu_id,$pulled_menu_id);

        return true;

    }

    function testPlaceOrderInOrginalSkin()
    {
        setContext('com.splickit.worldhq');
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $merchant_id = $this->ids['merchant_id'];
        $menu_id = $this->ids['default_menu_id'];

        //$cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($this->ids['merchant_id']);
        $order_adapter = new OrderAdapter(getM());
        $raw_cart_data = $order_adapter->getSimpleOrderArray($menu_id, $merchant_id, 'sumdumnuote',1);
        $cart_data = [];
        $cart_data['items'] = $raw_cart_data['items'];
        $cart_data['merchant_id'] = $raw_cart_data['merchant_id'];
        $cart_data['user_id'] = $raw_cart_data['user_id'];


        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$this->ids['merchant_id'],0.00);
        $this->assertNull($order_resource->error);

    }


    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        ini_set('max_execution_time', 0);
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction(); ;

        $skin_resource = getOrCreateSkinAndBrandIfNecessary('localskin','localbrand');
        $ids['skin_id'] = $skin_resource->skin_id;

        $brand_id = $skin_resource->brand_id;
        $ids['brand_id'] = $brand_id;

        $menu_id = createTestMenuWithNnumberOfItems(1);
        $ids['default_menu_id'] = $menu_id;

        $merchant_resource = createNewTestMerchant($menu_id);
        attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
        $ids['merchant_id'] = $merchant_resource->merchant_id;

        $alt_skin_resource = getOrCreateSkinAndBrandIfNecessary('altskin','localbrand',null,$brand_id);
        $ids['alt_skin_id'] = $alt_skin_resource->skin_id;

        MerchantPaymentTypeMapsAdapter::createMerchantPaymentTypeMap($merchant_resource->merchant_id, 1000, $billing_entity_id);
//        $billing_entity_resource = createSageBillingEntity($merchant_resource->brand_id);
//        $merchant_payment_type_map_resource = MerchantPaymentTypeMapsAdapter::createMerchantPaymentTypeMap($merchant_resource->merchant_id, 2000, $billing_entity_resource->id);

        $user_resource = createNewUser(array("flags" => "1C20000001"));
        $ids['user_id'] = $user_resource->user_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main()
    {
        $suite = new PHPUnit_Framework_TestSuite(__CLASS__);
        PHPUnit_TextUI_TestRunner::run($suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    SkinSpecificMenuTest::main();
}

?>