<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class TelegramControllerTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $mmha;
    var $ids;

    function setUp()
    {
        $_SERVER['STAMP'] = __CLASS__.'-'.setNewStamp();
        $this->ids = $_SERVER['unit_test_ids'];
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        setContext('com.splickit.telegramskin');
    }

    function tearDown()
    {
        //delete your instance
        unset($this->ids);
    }

    function testSendMessageTelegram()
    {
        $telegram_user_id = getProperty('telegram_test_user_id');
        $data['message_data'] = ['message_format'=>'TTM','delivery_addr'=>"$telegram_user_id"];
        $data['cash'] = true;
        $merchant_resource = createNewTestMerchantDelivery($this->ids['menu_id'],$data);
        $merchant_id = $merchant_resource->merchant_id;

        $mdpd_resource = Resource::find(new MerchantDeliveryPriceDistanceAdapter(getM()),$merchant_resource->merchant_delivery_price_distance_id);
        $mdpd_resource->price = "5.00";
        $mdpd_resource->save();


        attachMerchantToSkin($merchant_resource->merchant_id, getSkinIdForContext());

        $user_resource = createNewUserWithCCNoCVV();
        $user_resource->first_name = 'Bob';
        $user_resource->last_name = 'Roberts';
        $user_resource->save();
        $user = logTestUserResourceIn($user_resource);
        $user_id = $user['user_id'];
        $udla = new UserDeliveryLocationAdapter(getM());
        $data = array("user_id" => $user_id, "address1" => "1633 18th street", "city" => "Boulder", "state" => "CO", "zip" => "80302", "phone_no" => "1234567890", "lat" => 40.016101, "lng" => -105.271900);
        $udla_resource = Resource::createByData($udla, $data);
        $udl_id = $udla_resource->user_addr_id;


        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','sum dum note',3);
        $cart_data['user_addr_id'] = $udl_id;
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $checkout_resource->note = "sum dum note";
        $order_resource = placeOrderFromCheckoutResource($checkout_resource);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'TTM');
        $text_controller = ControllerFactory::generateFromMessageResource($message_resource,getM());
        $text_controller->setTestDeliveryAddress(null);
        $text_controller->sendThisMessage($message_resource);

        $sent_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'TTM');
        $this->assertNotNull($sent_message_resource->response);

    }

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        set_time_limit(30);
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['default_tz'] = $tz;
        date_default_timezone_set("America/Denver");
        $sql = 'TRUNCATE TABLE `db_message_log`';
        $mmha = new MerchantMessageHistoryAdapter(getM());
        $mmha->_query($sql);
        ini_set('max_execution_time',300);

        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction(); ;

        $skin_resource = getOrCreateSkinAndBrandIfNecessary('telegramskin','telegrambrand',null,null);
        setContext($skin_resource->external_identifier);
        $ids['skin_id'] = $skin_resource->skin_id;

        $menu_id = createTestMenuWithNnumberOfItems(3);
        $ids['menu_id'] = $menu_id;

        $items = CompleteMenu::getAllMenuItemsAsResources($menu_id);
        $items[0]->item_name = "Black Cherry Cookies 14g";
        $items[0]->item_print_name = $items[0]->item_name;
        $items[0]->save();
        $items[1]->item_name = "Cuvee Cookies Pre-Roll";
        $items[1]->item_print_name = $items[1]->item_name;
        $items[1]->save();
        $items[2]->item_name = "Zkittles 3.5g";
        $items[2]->item_print_name = $items[2]->item_name;
        $items[2]->save();

        $sizes = CompleteMenu::getAllSizesAsResources($menu_id);
        $sizes[0]->size_name = 'one size';
        $sizes[0]->size_print_name = 'one size';
        $sizes[0]->save();


        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $parameters = $tpppa->getParametersFromProviderId(8001);
        foreach ($parameters as $parameter_row) {
            $data = [];
            $data['third_party_provider_parameter_id'] = $parameter_row['third_party_provider_parameter_id'];
            if ($parameter_row['name'] == 'bot_token') {
                $data['brand_id'] = getBrandIdFromCurrentContext();
                $data['value'] = getProperty('telegram_test_bot_token');
                $tpppbma = new ThirdPartyProviderParameterBrandMapsAdapter(getM());
                Resource::createByData($tpppbma,$data);
            }

        }

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }


    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
    }

    /* main method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    TelegramControllerTest::main();
}

?>