<?php


$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';


class ThanxIntegrationTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__ . '-' . $_SERVER['STAMP'] . '-' . generateCode(5);
        $this->ids = $_SERVER['unit_test_ids'];
        setContext('com.splickit.rrbbq');
    }

    function tearDown()
    {
        //delete your instance
        unset($this->ids);
        unset($_SERVER['max_lead']);
        $_SERVER['FORCE_THANX_REWARD_FAIL'] = false;
    }

    function testUserExistsInThanxButNotOrder140()
    {
        $user_fields = createNewUserDataFields();
        $_SERVER['thanx_email'] = $user_fields['email'];
        $_SERVER['thanx_id'] = rand(111111,999999);
        $_SERVER['thanx_first_name'] = $user_fields['first_name'];
        $_SERVER['thanx_last_name'] = $user_fields['last_name'];
        $_SERVER['thanx_phone'] = '+13038885555';
        $code = 'fb8948bfffe77e0fd1f926e10248a4d505342b540466271e8a7e7da64e3d7414';
        $request_all['thanx_authentication_token'] = $code;
        $loginAdapter = new LoginAdapter(getM());
        $logged_in_user_resource = $loginAdapter->doAuthorizeWithSpecialUserValidation('thanx_authentication_token', $code,$request_all);
        $this->assertNotFalse($logged_in_user_resource,"should have created a user resource but authorize came back false");
        $this->assertEquals($user_fields['email'], $logged_in_user_resource->email,"should have created the user resource associated wih the correct user but it did not");
        $this->assertEquals($user_fields['first_name'],$logged_in_user_resource->first_name);
        $this->assertEquals($user_fields['last_name'],$logged_in_user_resource->last_name);
        $this->assertEquals('303-888-5555',$logged_in_user_resource->contact_no);


        $user_resource = UserAdapter::getUserResourceFromId($logged_in_user_resource->user_id);
        $loyalty_controller = LoyaltyControllerFactory::getLoyaltyControllerForContext($user_resource->getDataFieldsReally());
        $loyalty_number = $loyalty_controller->loyalty_number;
        $this->assertContains('id=',$loyalty_number);
        $this->assertContains(':access=',$loyalty_number);
        $this->assertContains(':refresh=',$loyalty_number);

        // try to order
        $user_resource->flags = '1C20000001';
        $user_resource->save();
        $user = logTestUserResourceIn($user_resource);
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id);
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_resource->merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data);
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource);
        $this->assertNull($order_resource->error);

    }

    function testUserExistsInThanxButNotOrder140NoEmail()
    {
        $user_fields = createNewUserDataFields();
        $_SERVER['thanx_email'] = null;
        $_SERVER['thanx_id'] = rand(111111,999999);
        $code = 'fb8948bfffe77e0fd1f926e10248a4d505342b540466271e8a7e7da64e3d7414';
        $request_all['thanx_authentication_token'] = $code;
        $loginAdapter = new LoginAdapter(getM());
        $logged_in_user_resource = $loginAdapter->doAuthorizeWithSpecialUserValidation('thanx_authentication_token', $code,$request_all);
        $this->assertFalse($logged_in_user_resource,"should NOT have created a user resource");

    }

    function testCreatUserButExistingInOurSystemNotInThanx()
    {
        setContext('com.splickit.worldhq');
        $user_resource = createNewUserWithCCNoCVV();
        setContext('com.splickit.rrbbq');
        $loyalty_controller = LoyaltyControllerFactory::getLoyaltyControllerForContext($user_resource->getDataFieldsReally());
        $this->assertNull($loyalty_controller->loyalty_number,'It should not have a loyalty number for RRBBQ');
        $url = "/app2/apiv2/users?log_level=5";
        $new_user_data['first_name'] = $user_resource->first_name;
        $new_user_data['last_name'] = $user_resource->last_name;
        $new_user_data['email'] = $user_resource->email;
        $new_user_data['contact_no'] = $user_resource->contact_no;
        $user = logTestUserIn(1);
        $request = createRequestObject($url,"POST",json_encode($new_user_data),'application/json');
        $user_controller = new UserController(getM(), $user, $request, 5);
        $response = makeMockRequest($user_controller);

        $new_user_response = json_decode($response,true);
        $this->assertNull($new_user_response['error'],"should NOT have gotten an error");
        $this->assertEquals("Your profile has authenticated. You are now logged in.",$new_user_response['message']);
        $user = $new_user_response['data'];
        $this->assertNotNull($user['user_id']);
        $this->assertEquals($user_resource->uuid,$user['user_id']);
        $this->assertEquals($user_resource->email,$user['email']);
        $this->assertEquals($user_resource->contact_no,$user['contact_no']);

        $expires_at = $user['splickit_authentication_token_expires_at'];
        $this->assertTrue($expires_at > (time()+(179*24*3600)));

        $user_resource = UserAdapter::getUserResourceFromId($user['uuid']);
        $loyalty_controller = LoyaltyControllerFactory::getLoyaltyControllerForContext($user_resource->getDataFieldsReally());
        $loyalty_number = $loyalty_controller->loyalty_number;
        $this->assertContains('id=',$loyalty_number);
        $this->assertContains(':access=',$loyalty_number);
        $this->assertContains(':refresh=',$loyalty_number);



    }

    function testRemoveReward()
    {
        $url = "/app2/apiv2/users?log_level=5";
        $new_user_data = createNewUserDataFields();
        $user = logTestUserIn(1);
        $request = createRequestObject($url,"POST",json_encode($new_user_data),'application/json');
        $user_controller = new UserController(getM(), $user, $request, 5);
        $response = makeMockRequest($user_controller);

        $new_user_response = json_decode($response,true);
        $this->assertNull($new_user_response['error'],"should NOT have gotten an error");
        $user = $new_user_response['data'];
        $this->assertNotNull($user['user_id']);
        $user_resource = UserAdapter::getUserResourceFromId($user['uuid']);
        $user_resource->flags = '1C20000001';
        $user_resource->save();

        clearAuthenticatedUserParametersForSession();
        $user = logTestUserResourceIn($user_resource);
        $merchant_id = $this->ids['merchant_id'];
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note',5);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data);
        $this->assertNull($checkout_resource->error);

        $additional_loyalty_checkout_info = $checkout_resource->additional_loyalty_checkout_info;
        $this->assertNotNull($additional_loyalty_checkout_info,'It should have the rewards on the checkout info');

        $reward = $additional_loyalty_checkout_info['data'][2];

        $cart_ucid = $checkout_resource->ucid;
        $pre_tax_amt = $checkout_resource->total_tax_amt;
        $this->assertEquals(0.00,$checkout_resource->promo_tax_amt);
        $this->assertEquals(0.00,$checkout_resource->promo_amt);

        $order_data_promo['promo_code'] = $reward['id'];
        $promo_code = $reward['id'];
        $remote_promo_service = $additional_loyalty_checkout_info['service'];
        $order_data_promo['remote_promo_service'] = $additional_loyalty_checkout_info['service'];

        //$request = createRequestObject("/app2/apiv2/cart/$cart_ucid/checkout", 'post', json_encode($order_data_promo), 'application/json');
        $request = createRequestObject("/app2/apiv2/cart/$cart_ucid/checkout?promo_code=$promo_code&remote_promo_service=$remote_promo_service");
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $promo_checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($promo_checkout_resource->error);
        $this->assertEquals(-5.00,$promo_checkout_resource->promo_amt);
        $post_tax_amt = $promo_checkout_resource->total_tax_amt;
        $promo_tax_amt = $promo_checkout_resource->promo_tax_amt;
        $this->assertEquals(-.50,$promo_tax_amt);
        $this->assertEquals($pre_tax_amt+$promo_tax_amt, $post_tax_amt);

        // now remove it
        $request = createRequestObject("/app2/apiv2/cart/$cart_ucid/checkout?promo_code=remove_promo&remote_promo_service=$remote_promo_service");
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $no_promo_checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($no_promo_checkout_resource->error);
        $this->assertEquals(0.00,$no_promo_checkout_resource->promo_amt);
        $no_promo_post_tax_amt = $no_promo_checkout_resource->total_tax_amt;
        $no_promo_tax_amt = $no_promo_checkout_resource->promo_tax_amt;
        $this->assertEquals(0.00,$no_promo_tax_amt);
        $this->assertEquals($pre_tax_amt, $no_promo_post_tax_amt);
        $this->assertNull($no_promo_checkout_resource->promo_code);

        $order_resource = placeOrderFromCheckoutResource($promo_checkout_resource);
        $this->assertNull($order_resource->error);
        $this->assertEquals(0.00,$order_resource->promo_amt);
        $this->assertEquals(8.25,$order_resource->grand_total);

        // see if the thanx row was NOT added to balance_change
        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order_resource->order_id],'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
        $cc_payment_row = $bcr_hash['Authorize'];
        $this->assertNotNull($cc_payment_row,"It should have a cc payment row");
        $expected_cc_charge_amt = $order_resource->grand_total;
        $this->assertEquals($expected_cc_charge_amt,$cc_payment_row['charge_amt']);

        $loyalty_payment_row = $bcr_hash['LoyaltyBalancePayment'];
        $this->assertNull($loyalty_payment_row,'It should not have a loyalty payment row');
    }

    function testFailureOnPHoneNumber()
    {
        $url = "/app2/apiv2/users?log_level=5";
        $new_user_data = createNewUserDataFields();
        $new_user_data['contact_no'] = '8888888888';
        $new_user_data['birthday'] = "05/04/1975";
        $user = logTestUserIn(1);
        $request = createRequestObject($url,"POST",json_encode($new_user_data),'application/json');
        $user_controller = new UserController(getM(), $user, $request, 5);
        $response = makeMockRequest($user_controller);

        $new_user_response = json_decode($response,true);
        $this->assertNotNull($new_user_response['error']['error'],"Should have gotten an error");
        $this->assertEquals('Phone is invalid',$new_user_response['error']['error']);

        $correct_phone = '3038841438';
        $new_user_data['contact_no'] = $correct_phone;
        $request = createRequestObject($url,"POST",json_encode($new_user_data),'application/json');
        $user_controller = new UserController(getM(), $user, $request, 5);
        $response = makeMockRequest($user_controller);

        $new_user_response = json_decode($response,true);
        $this->assertNull($new_user_response['error']['error'],"Should NOT have gotten an error");


        $user_resource = getUserResourceFromId($new_user_response['data']['user_id']);
        $this->assertEquals('303-884-1438',$user_resource->contact_no);


    }


    function testFailureAfterOrderHasBeenProcessedLocally()
    {
        $url = "/app2/apiv2/users?log_level=5";
        $new_user_data = createNewUserDataFields();
        $user = logTestUserIn(1);
        $request = createRequestObject($url,"POST",json_encode($new_user_data),'application/json');
        $user_controller = new UserController(getM(), $user, $request, 5);
        $response = makeMockRequest($user_controller);

        $new_user_response = json_decode($response,true);
        $this->assertNull($new_user_response['error'],"should NOT have gotten an error");
        $user = $new_user_response['data'];
        $this->assertNotNull($user['user_id']);
        $user_resource = UserAdapter::getUserResourceFromId($user['uuid']);
        $user_resource->flags = '1C20000001';
        $user_resource->save();

        clearAuthenticatedUserParametersForSession();
        $user = logTestUserResourceIn($user_resource);
        $merchant_id = $this->ids['merchant_id'];
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note',5);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data);
        $this->assertNull($checkout_resource->error);
        $order_id = $checkout_resource->oid_test_only;

        $additional_loyalty_checkout_info = $checkout_resource->additional_loyalty_checkout_info;
        $this->assertNotNull($additional_loyalty_checkout_info,'It should have the rewards on the checkout info');

        $reward = $additional_loyalty_checkout_info['data'][2];

        $cart_ucid = $checkout_resource->ucid;
        $pre_tax_amt = $checkout_resource->total_tax_amt;
        $this->assertEquals(0.00,$checkout_resource->promo_tax_amt);
        $this->assertEquals(0.00,$checkout_resource->promo_amt);

        $order_data_promo['promo_code'] = $reward['id'];
        $promo_code = $reward['id'];
        $remote_promo_service = $additional_loyalty_checkout_info['service'];
        $order_data_promo['remote_promo_service'] = $additional_loyalty_checkout_info['service'];

        $request = createRequestObject("/app2/apiv2/cart/$cart_ucid/checkout?promo_code=$promo_code&remote_promo_service=$remote_promo_service");
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $promo_checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($promo_checkout_resource->error);
        $this->assertEquals(-5.00,$promo_checkout_resource->promo_amt);
        $post_tax_amt = $promo_checkout_resource->total_tax_amt;
        $promo_tax_amt = $promo_checkout_resource->promo_tax_amt;
        $this->assertEquals(-.50,$promo_tax_amt);
        $this->assertEquals($pre_tax_amt+$promo_tax_amt, $post_tax_amt);

        $_SERVER['FORCE_THANX_REWARD_FAIL'] = true;
        $order_resource = placeOrderFromCheckoutResource($promo_checkout_resource);
        $this->assertNotNull($order_resource->error);
        $this->assertEquals('This reward can no longer be used',$order_resource->error);

        // so need to do all the checks to see if the order is int eh correct state.
        $base_order_resource = CompleteOrder::getBaseOrderData($order_id);
        $this->assertEquals(OrderAdapter::ORDER_PAYMENT_FAILED,$base_order_resource['status'],"It should have a status of payment failure");
        $this->assertEquals(0.00,$base_order_resource['promo_tax_amt']);
        $this->assertEquals(0.00,$base_order_resource['promo_amt']);
        $this->assertEquals(null,$base_order_resource['promo_code']);
        $this->assertEquals(null,$base_order_resource['promo_id']);

        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$base_order_resource['order_id']],'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
        $cc_payment_row = $bcr_hash['AuthCancelled'];
        $this->assertNotNull($cc_payment_row,"It should have a cc payment row");
        $this->assertEquals('CANCELED',$cc_payment_row['notes']);
        $this->assertNull($bcr_hash['LoyaltyBalancePayment']);


    }

    function testUpdateCartWithExistingAwardStillValidForCart()
    {
        $thanx_promo_resource = $this->ids['thanx_promo_resource'];
        $thanx_promo_id = $thanx_promo_resource->promo_id;
        $url = "/app2/apiv2/users?log_level=5";
        $new_user_data = createNewUserDataFields();
        $user = logTestUserIn(1);
        $request = createRequestObject($url,"POST",json_encode($new_user_data),'application/json');
        $user_controller = new UserController(getM(), $user, $request, 5);
        $response = makeMockRequest($user_controller);

        $new_user_response = json_decode($response,true);
        $this->assertNull($new_user_response['error'],"should NOT have gotten an error");
        $user = $new_user_response['data'];
        $this->assertNotNull($user['user_id']);
        $user_resource = UserAdapter::getUserResourceFromId($user['uuid']);
        $user_resource->flags = '1C20000001';
        $user_resource->save();

        clearAuthenticatedUserParametersForSession();
        $user = logTestUserResourceIn($user_resource);
        $merchant_id = $this->ids['merchant_id'];
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note',5);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data);
        $this->assertNull($checkout_resource->error);

        $additional_loyalty_checkout_info = $checkout_resource->additional_loyalty_checkout_info;
        $this->assertNotNull($additional_loyalty_checkout_info,'It should have the rewards on the checkout info');

        $reward = $additional_loyalty_checkout_info['data'][3];

        $cart_ucid = $checkout_resource->ucid;
        $pre_tax_amt = $checkout_resource->total_tax_amt;
        $this->assertEquals(0.00,$checkout_resource->promo_tax_amt);
        $this->assertEquals(0.00,$checkout_resource->promo_amt);

        $promo_code = $reward['id'];
        $remote_promo_service = $additional_loyalty_checkout_info['service'];

        $request = createRequestObject("/app2/apiv2/cart/$cart_ucid/checkout?promo_code=$promo_code&remote_promo_service=$remote_promo_service");
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $promo_checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($promo_checkout_resource->error);
        $this->assertEquals(-5.00,$promo_checkout_resource->promo_amt);
        $post_tax_amt = $promo_checkout_resource->total_tax_amt;
        $promo_tax_amt = $promo_checkout_resource->promo_tax_amt;
        $base_order_data = CompleteOrder::getBaseOrderData($cart_ucid);
        $this->assertEquals($thanx_promo_id,$base_order_data['promo_id']);
        $this->assertEquals(-.50,$promo_tax_amt);
        $this->assertEquals($pre_tax_amt+$promo_tax_amt, $post_tax_amt);

        $complete_order = CompleteOrder::staticGetCompleteOrder($cart_ucid);
        // now lets call checkout again as if the user went back to the menu
        $request = createRequestObject("/app2/apiv2/cart/$cart_ucid/checkout",'POST');
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $promo_checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($promo_checkout_resource->error);
        $this->assertEquals(-5.00,$promo_checkout_resource->promo_amt);
        $post_tax_amt = $promo_checkout_resource->total_tax_amt;
        $promo_tax_amt = $promo_checkout_resource->promo_tax_amt;
        $this->assertEquals(-.50,$promo_tax_amt);
        $this->assertEquals($pre_tax_amt+$promo_tax_amt, $post_tax_amt);

        // now lets add another item and checkout again
        $order_resource = Resource::find(new OrderAdapter(getM()),$checkout_resource->oid_test_only);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note',1);
        $cart_data['ucid'] = $order_resource->ucid;
        $new_checkout_resource = getCheckoutResourceFromOrderData($cart_data);
        $this->assertNull($new_checkout_resource->error);
        $this->assertEquals($promo_code,$new_checkout_resource->promo_code);
        $this->assertEquals(-5.00,$new_checkout_resource->promo_amt);
    }


    function testUpdateCartWithExistingAwardThatIsNoLongerValidForCart()
    {
        $url = "/app2/apiv2/users?log_level=5";
        $new_user_data = createNewUserDataFields();
        $user = logTestUserIn(1);
        $request = createRequestObject($url,"POST",json_encode($new_user_data),'application/json');
        $user_controller = new UserController(getM(), $user, $request, 5);
        $response = makeMockRequest($user_controller);

        $new_user_response = json_decode($response,true);
        $this->assertNull($new_user_response['error'],"should NOT have gotten an error");
        $user = $new_user_response['data'];
        $this->assertNotNull($user['user_id']);
        $user_resource = UserAdapter::getUserResourceFromId($user['uuid']);
        $user_resource->flags = '1C20000001';
        $user_resource->save();

        clearAuthenticatedUserParametersForSession();
        $user = logTestUserResourceIn($user_resource);
        $merchant_id = $this->ids['merchant_id'];
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note',5);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data);
        $this->assertNull($checkout_resource->error);

        $additional_loyalty_checkout_info = $checkout_resource->additional_loyalty_checkout_info;
        $this->assertNotNull($additional_loyalty_checkout_info,'It should have the rewards on the checkout info');

        $reward = $additional_loyalty_checkout_info['data'][2];

        $cart_ucid = $checkout_resource->ucid;
        $pre_tax_amt = $checkout_resource->total_tax_amt;
        $this->assertEquals(0.00,$checkout_resource->promo_tax_amt);
        $this->assertEquals(0.00,$checkout_resource->promo_amt);

        $promo_code = $reward['id'];
        $remote_promo_service = $additional_loyalty_checkout_info['service'];

        $request = createRequestObject("/app2/apiv2/cart/$cart_ucid/checkout?promo_code=$promo_code&remote_promo_service=$remote_promo_service");
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $promo_checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($promo_checkout_resource->error);
        $this->assertEquals(-5.00,$promo_checkout_resource->promo_amt);
        $post_tax_amt = $promo_checkout_resource->total_tax_amt;
        $promo_tax_amt = $promo_checkout_resource->promo_tax_amt;
        $this->assertEquals(-.50,$promo_tax_amt);
        $this->assertEquals($pre_tax_amt+$promo_tax_amt, $post_tax_amt);

        // now lets add another item and checkout agin with a failure
        $order_resource = Resource::find(new OrderAdapter(getM()),$checkout_resource->oid_test_only);
        $order_resource->promo_code = 'ABCD1234';
        $order_resource->save();

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note',1);
        $cart_data['ucid'] = $order_resource->ucid;
        $new_checkout_resource = getCheckoutResourceFromOrderData($cart_data);
        $this->assertNull($new_checkout_resource->error);
        $this->assertEquals('',$new_checkout_resource->promo_code);
        $this->assertEquals(null,$new_checkout_resource->promo_id);
        $this->assertEquals(0.00,$new_checkout_resource->promo_amt);
    }

    function testBadRewardChoice()
    {
        $url = "/app2/apiv2/users?log_level=5";
        $new_user_data = createNewUserDataFields();
        $user = logTestUserIn(1);
        $request = createRequestObject($url,"POST",json_encode($new_user_data),'application/json');
        $user_controller = new UserController(getM(), $user, $request, 5);
        $response = makeMockRequest($user_controller);

        $new_user_response = json_decode($response,true);
        $this->assertNull($new_user_response['error'],"should NOT have gotten an error");
        $user = $new_user_response['data'];
        $this->assertNotNull($user['user_id']);
        $user_resource = UserAdapter::getUserResourceFromId($user['uuid']);
        $user_resource->flags = '1C20000001';
        $user_resource->save();

        clearAuthenticatedUserParametersForSession();
        $user = logTestUserResourceIn($user_resource);
        $merchant_id = $this->ids['merchant_id'];
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note',5);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data);
        $this->assertNull($checkout_resource->error);

        $additional_loyalty_checkout_info = $checkout_resource->additional_loyalty_checkout_info;
        $this->assertNotNull($additional_loyalty_checkout_info,'It should have the rewards on the checkout info');

        $reward = $additional_loyalty_checkout_info['data'][0];

        $cart_ucid = $checkout_resource->ucid;
        $this->assertEquals(0.00,$checkout_resource->promo_amt);

        $promo_code = $reward['id'];
        $remote_promo_service = $additional_loyalty_checkout_info['service'];

        $request = createRequestObject("/app2/apiv2/cart/$cart_ucid/checkout?promo_code=$promo_code&remote_promo_service=$remote_promo_service");
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $promo_checkout_resource = $place_order_controller->processV2Request();
        $this->assertNotNull($promo_checkout_resource->error);
        $this->assertEquals('This reward cannot be applied to any items in your purchase',$promo_checkout_resource->error);
        $this->assertEquals(0.00,$promo_checkout_resource->promo_amt);
        $this->assertEquals('',$promo_checkout_resource->promo_code);

    }


    function testRewards()
    {
        $url = "/app2/apiv2/users?log_level=5";
        $new_user_data = createNewUserDataFields();
        $user = logTestUserIn(1);
        $request = createRequestObject($url,"POST",json_encode($new_user_data),'application/json');
        $user_controller = new UserController(getM(), $user, $request, 5);
        $response = makeMockRequest($user_controller);

        $new_user_response = json_decode($response,true);
        $this->assertNull($new_user_response['error'],"should NOT have gotten an error");
        $user = $new_user_response['data'];
        $this->assertNotNull($user['user_id']);
        $user_resource = UserAdapter::getUserResourceFromId($user['uuid']);
        $user_resource->flags = '1C20000001';
        $user_resource->save();

        clearAuthenticatedUserParametersForSession();
        $user = logTestUserResourceIn($user_resource);
        $merchant_id = $this->ids['merchant_id'];
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note',5);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data);
        $this->assertNull($checkout_resource->error);

        $additional_loyalty_checkout_info = $checkout_resource->additional_loyalty_checkout_info;
        $this->assertNotNull($additional_loyalty_checkout_info,'It should have the rewards on the checkout info');

        $reward = $additional_loyalty_checkout_info['data'][2];

        $cart_ucid = $checkout_resource->ucid;
        $pre_tax_amt = $checkout_resource->total_tax_amt;
        $this->assertEquals(0.00,$checkout_resource->promo_tax_amt);
        $this->assertEquals(0.00,$checkout_resource->promo_amt);

        $order_data_promo['promo_code'] = $reward['id'];
        $promo_code = $reward['id'];
        $remote_promo_service = $additional_loyalty_checkout_info['service'];
        $order_data_promo['remote_promo_service'] = $additional_loyalty_checkout_info['service'];

        //$request = createRequestObject("/app2/apiv2/cart/$cart_ucid/checkout", 'post', json_encode($order_data_promo), 'application/json');
        $request = createRequestObject("/app2/apiv2/cart/$cart_ucid/checkout?promo_code=$promo_code&remote_promo_service=$remote_promo_service");
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $promo_checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($promo_checkout_resource->error);
        $this->assertEquals(-5.00,$promo_checkout_resource->promo_amt);
        $post_tax_amt = $promo_checkout_resource->total_tax_amt;
        $promo_tax_amt = $promo_checkout_resource->promo_tax_amt;
        $this->assertEquals(-.50,$promo_tax_amt);
        $this->assertEquals($pre_tax_amt+$promo_tax_amt, $post_tax_amt);

        $order_resource = placeOrderFromCheckoutResource($promo_checkout_resource);
        $this->assertNull($order_resource->error);
        $this->assertEquals(-5.00,$order_resource->promo_amt);
        $this->assertEquals(2.75,$order_resource->grand_total);

        // see if the thanx row was added to balance_change
        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order_resource->order_id],'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
        $cc_payment_row = $bcr_hash['Authorize'];
        $this->assertNotNull($cc_payment_row,"It should have a cc payment row");
        $expected_cc_charge_amt = $order_resource->grand_total;
        $this->assertEquals($expected_cc_charge_amt,$cc_payment_row['charge_amt']);

        $loyalty_payment_row = $bcr_hash['LoyaltyBalancePayment'];
        $this->assertEquals('ThanxLoyaltyController',$loyalty_payment_row['cc_processor'],"It should ahve thanx listed");
        $this->assertEquals(5.00,$loyalty_payment_row['charge_amt']);

        return $order_resource;
    }

    /**
     * @depends testRewards
     */
    function testRefund($order_resource)
    {
        $ucid = $order_resource->ucid;
        $url = "http://127.0.0.1:" . $this->api_port . "/app2/portal/orders/$ucid/refundorder";
        $body = '{"refund_amount":"0.00","employee_name":"Michael","note":"customer canceled","user_id":'.$order_resource->user_id.',"submit":true}';
        $request = createRequestObject($url,'POST',$body);
        $order_controller = new OrderController(getM(),null,$request);
        $response = makeMockPortalRequest($order_controller);
        $response_array = json_decode($response,true);
        $this->assertEquals(200,$response_array['http_code']);
        $this->assertEquals('success',$response_array['data']['result']);

        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order_resource->order_id],'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');

        $loyalty_payment_row = $bcr_hash['LoyaltyBalancePayment'];
        $this->assertEquals('ThanxLoyaltyController',$loyalty_payment_row['cc_processor'],"It should ahve thanx listed");
        $this->assertContains('REFUNDED:',$loyalty_payment_row['notes']);
    }

    function testCreateNewUserFromWebUserAlreadyExistsInOrder140()
    {
        removeContext();
        $user_resource = createNewUserWithCCNoCVV();
//        $user_resource->first_name = 'exists';
//        $user_resource->save();
        setContext('com.splickit.rrbbq');
        $url = "/app2/apiv2/users?log_level=5";
        $user_data['first_name'] = $user_resource->first_name;
        $user_data['last_name'] = $user_resource->last_name;
        $user_data['email'] = $user_resource->email;
        $user_data['password'] = 'welcome';
        $user_data['contact_no'] = $user_resource->contact_no;

        $user = logTestUserIn(1);
        $request = createRequestObject($url,"POST",json_encode($user_data),'application/json');
        $user_controller = new UserController(getM(), $user, $request, 5);
        $response = makeMockRequest($user_controller);

        $new_user_response = json_decode($response,true);
        $this->assertNull($new_user_response['error'],"should NOT have gotten an error");
        $user = $new_user_response['data'];
        $this->assertNotNull($user['user_id']);

        $user_resource = UserAdapter::getUserResourceFromId($user['uuid']);
        $loyalty_controller = LoyaltyControllerFactory::getLoyaltyControllerForContext($user_resource->getDataFieldsReally());
        $loyalty_number = $loyalty_controller->loyalty_number;
        $this->assertNotNull($loyalty_number,"It should have a loyalty number");
        $this->assertContains('id=',$loyalty_number);
        $this->assertContains(':access=',$loyalty_number);
        $this->assertContains(':refresh=',$loyalty_number);

        $expires_at = $user['splickit_authentication_token_expires_at'];
        $this->assertTrue($expires_at > (time()+(179*24*3600)));
        return $user;
    }

    function testCreateNewUserFromWebUserAlreadyExistsInThanx()
    {
        $url = "/app2/apiv2/users?log_level=5";
        $new_user_data = createNewUserDataFields();
        $new_user_data['first_name'] = 'exists';
        $user = logTestUserIn(1);
        $request = createRequestObject($url,"POST",json_encode($new_user_data),'application/json');
        $user_controller = new UserController(getM(), $user, $request, 5);
        $response = makeMockRequest($user_controller);

        $new_user_response = json_decode($response,true);
        $this->assertNotNull($new_user_response['error'],"should have gotten an error");
        $this->assertEquals(ThanxLoyaltyController::ERROR_FOR_THANX_EXISTING_USER_MESSAGE,$new_user_response['error']['error']);
    }


    function testCreateUserFromWeb()
    {
        $url = "/app2/apiv2/users?log_level=5";
        $new_user_data = createNewUserDataFields();
        $user = logTestUserIn(1);
        $request = createRequestObject($url,"POST",json_encode($new_user_data),'application/json');
        $user_controller = new UserController(getM(), $user, $request, 5);
        $response = makeMockRequest($user_controller);

        $new_user_response = json_decode($response,true);
        $this->assertNull($new_user_response['error'],"should NOT have gotten an error");
        $user = $new_user_response['data'];
        $this->assertNotNull($user['user_id']);
        $expires_at = $user['splickit_authentication_token_expires_at'];
        $this->assertTrue($expires_at > (time()+(179*24*3600)));

        $user_resource = UserAdapter::getUserResourceFromId($user['uuid']);
        $loyalty_controller = LoyaltyControllerFactory::getLoyaltyControllerForContext($user_resource->getDataFieldsReally());
        $loyalty_number = $loyalty_controller->loyalty_number;
        $this->assertContains('id=',$loyalty_number);
        $this->assertContains(':access=',$loyalty_number);
        $this->assertContains(':refresh=',$loyalty_number);
        $user_resource->set('loyalty_number',$loyalty_number);
        return $user_resource;
    }

    /**
     * @depends testCreateUserFromWeb
     */
    function testAuthenticateUserWithThanxCode($user_resource)
    {
        $l = explode(':',$user_resource->loyalty_number);
        foreach ($l as $component) {
            $c = explode('=',$component);
            $hash[$c[0]] = $c[1];
        }
        $_SERVER['thanx_email'] = $user_resource->email;
        $_SERVER['thanx_id'] = $hash['id'];
        clearAuthenticatedUserParametersForSession();
        $code = 'fb8948bfffe77e0fd1f926e10248a4d505342b540466271e8a7e7da64e3d7414';
        $request_all['thanx_authentication_token'] = $code;
        $loginAdapter = new LoginAdapter(getM());
        $logged_in_user_resource = $loginAdapter->doAuthorizeWithSpecialUserValidation('thanx_authentication_token', $code,$request_all);
        $this->assertNotFalse($logged_in_user_resource,"should have found a user resource but authorize came back false");
        $this->assertEquals($user_resource->email, $logged_in_user_resource->email,"should have grabbed the user resource associated wih the correct user but it did not");
        return $user_resource;
    }

    /**
     * @depends testAuthenticateUserWithThanxCode
     */
    function testPlaceOrder($user_resource)
    {
        $user_resource->flags = '1C20000001';
        $user_resource->save();
        $user = logTestUserResourceIn($user_resource);
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id);
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_resource->merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data);
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource);
        $this->assertNull($order_resource->error);
    }

    /**
     * @depends testAuthenticateUserWithThanxCode
     */
    function testUpdateUserNameWithThanx($user_resource)
    {
        $user = logTestUserResourceIn($user_resource);
        $uuid = $user_resource->uuid;
        $url = "/app2/apiv2/users/$uuid";
        $user_data = ['first_name'=>'sasquatch','birthday'=>"05/04/1975"];
        $request = createRequestObject($url,'POST',json_encode($user_data));
        $user_controller = new UserController(getM(),$user,$request);
        $response = makeMockRequest($user_controller);

        setSessionProperty('is_a_user_update', 'true');
        $user_adapter = new UserAdapter(getM());
        $this->assertTrue($user_adapter->update($user_resource));
    }

    function testAuthenticateUserWithThanxCodeFailure()
    {
        clearAuthenticatedUserParametersForSession();
        $code = '12345678900987654321';
        $request_all['thanx_authentication_token'] = $code;
        $loginAdapter = new LoginAdapter(getM());
        $logged_in_user_resource = $loginAdapter->doAuthorizeWithSpecialUserValidation('thanx_authentication_token', $code,$request_all);
        $this->assertFalse($logged_in_user_resource,"should have failed");
    }


    static function setUpBeforeClass()
    {
        //client_id = 2aa4ac527ee97dd4322dfc40e3fe18242c8d4f7ed1c02588eee924c5df5c2003
        //SECRET = 421ca1269e50bb167db5a4bf9467a5c8d4dcec04e7a786fb91cffbb3afc715d8
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time',300);
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction();

        $skin_resource = getOrCreateSkinAndBrandIfNecessaryWithLoyalty("rrbbq","rrbbq",335, 598);

        $blr_data['brand_id'] = 598;
        $blr_data['loyalty_type'] = 'remote';
        $blr_data['charge_tax'] = false;
        $brand_loyalty_rules_resource = Resource::factory(new BrandLoyaltyRulesAdapter(getM()),$blr_data);
        $brand_loyalty_rules_resource->save();

        $ids['skin_id'] = $skin_resource->skin_id;
        setContext('com.splickit.rrbbq');

        $simple_menu_id = createTestMenuWithOneItem("item_one");
        $ids['menu_id'] = $simple_menu_id;

        $merchant_resource = createNewTestMerchant($simple_menu_id,['authorize'=>true]);
        $merchant_resource->save();
        $ids['merchant_id'] = $merchant_resource->merchant_id;

        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $parameters = $tpppa->getParametersFromProviderId(3003);
        foreach ($parameters as $parameter_row) {
            $data = [];
            $data['third_party_provider_parameter_id'] = $parameter_row['third_party_provider_parameter_id'];
            if ($parameter_row['name'] == 'client_id') {
                $data['value'] = '2aa4ac527ee97dd4322dfc40e3fe18242c8d4f7ed1c02588eee924c5df5c2003';
                $data['skin_id'] = getSkinIdForContext();
                $tpppsma = new ThirdPartyProviderParameterSkinMapsAdapter(getM());
                Resource::createByData($tpppsma,$data);
            } else if ($parameter_row['name'] == 'secret') {
                $data['value'] = '421ca1269e50bb167db5a4bf9467a5c8d4dcec04e7a786fb91cffbb3afc715d8';
                $data['skin_id'] = getSkinIdForContext();
                $tpppsma = new ThirdPartyProviderParameterSkinMapsAdapter(getM());
                Resource::createByData($tpppsma,$data);
            } else if ($parameter_row['name'] == 'merchant_key') {
                $data['value'] = 'gkmy334henlyvx6';
                $data['skin_id'] = getSkinIdForContext();
                $tpppsma = new ThirdPartyProviderParameterSkinMapsAdapter(getM());
                Resource::createByData($tpppsma,$data);
            } else if ($parameter_row['name'] == 'redirect_url') {
                $data['value'] = 'snarfs.order140-test.com';
                $data['skin_id'] = getSkinIdForContext();
                $tpppsma = new ThirdPartyProviderParameterSkinMapsAdapter(getM());
                Resource::createByData($tpppsma,$data);
            }
        }

        $promo_data = [];
        $promo_data['key_word'] = 'thanx';
        $promo_data['promo_type'] = 6;
        $promo_data['description'] = 'Thanx Loyalty Variable Promo';
        $promo_data['start_date'] = '2010-01-01';
        $promo_data['end_date'] = '2040-01-01';
        $promo_data['max_use'] = 10000;
        $promo_data['brand_id'] = getBrandIdFromCurrentContext();
        $request = createRequestObject("/app2/admin/promo",'POST',json_encode($promo_data));
        $promo_controller = new PromoController(getM(),null,$request,5);
        $promo_resource = $promo_controller->createPromo();
        $ids['thanx_promo_resource'] = $promo_resource;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }


    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    ThanxIntegrationTest::main();
}
?>