<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';


class ThirdPartyFunctionalityTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
		$this->ids = $_SERVER['unit_test_ids'];
        setContext('com.splickit.thirdpartyskin');
	}

    function tearDown()
    {
        //delete your instance
        unset($this->ids);
        unset($_SERVER['max_lead']);
    }

    function testGetMerchantParameters()
    {
        $merchant_id = $this->ids['merchant_id'];
        $tpp = new ThirdPartyProviderParameterMerchantMapsAdapter(getM());
        $results = $tpp->getMerchantParameters($merchant_id);
        $this->assertCount(2,$results,'it should have 2 TPP it is associated with');
        $vivonet_parameters = $results['Vivonet'];
        $this->assertCount(11,$vivonet_parameters,"It should have 11 vivonet parameters");
        $vantive_parameters = $results['Vantive'];
        $this->assertCount(1,$vantive_parameters,"It should have 1 vantive parameters");
    }

    function testGetMerchantParametersForSpecificProvider()
    {
        $merchant_id = $this->ids['merchant_id'];
        $tpp = new ThirdPartyProviderParameterMerchantMapsAdapter(getM());
        $results = $tpp->getMerchantParameters($merchant_id,1003);
        $this->assertCount(1,$results,'it should have 1 it is associated with');
        $vivonet_parameters = $results['Vivonet'];
        $this->assertCount(11,$vivonet_parameters,"It should have 11 vivonet parameters");
    }

    function testGetBrandParameters()
    {
        $brand_id = getBrandIdFromCurrentContext();
        $tppbma = new ThirdPartyProviderParameterBrandMapsAdapter(getM());
        $results = $tppbma->getBrandParameters($brand_id);
        $this->assertCount(1,$results,'it should have 1 TPP it is associated with');
        $vivonet_parameters = $results['Vivonet'];
        $this->assertCount(1,$vivonet_parameters,"It should have 1 vivonet parameters");
    }

    function testGetBrandParametersForSpecificProvider()
    {
        $brand_id = getBrandIdFromCurrentContext();
        $tppbma = new ThirdPartyProviderParameterBrandMapsAdapter(getM());
        $results = $tppbma->getBrandParameters($brand_id,1003);
        $this->assertCount(1,$results,'it should have 1 TPP it is associated with');
        $vivonet_parameters = $results['Vivonet'];
        $this->assertCount(1,$vivonet_parameters,"It should have 1 vivonet parameters");

        $results = $tppbma->getBrandParameters($brand_id,5001);
        $this->assertCount(0,$results,'it should have no TPP it is associated with');
    }

    function testGetSkinParameters()
    {
        $skin_id = getSkinIdForContext();
        $tppsma = new ThirdPartyProviderParameterSkinMapsAdapter(getM());
        $results = $tppsma->getSkinParameters($skin_id);
        $this->assertCount(2,$results,'it should have 2 TPP it is associated with');
        $vivonet_parameters = $results['Vivonet'];
        $this->assertCount(2,$vivonet_parameters,"It should have 2 vivonet parameters");
        $vantive_parameters = $results['Vantive'];
        $this->assertCount(1,$vantive_parameters,"It should have 2 vantive parameter");
    }

    function testGetSkinParametersForSpecificProvider()
    {
        $skin_id = getSkinIdForContext();
        $tppsma = new ThirdPartyProviderParameterSkinMapsAdapter(getM());
        $results = $tppsma->getSkinParameters($skin_id,1003);
        $this->assertCount(1,$results,'it should have 2 TPP it is associated with');
        $vivonet_parameters = $results['Vivonet'];
        $this->assertCount(2,$vivonet_parameters,"It should have 2 vivonet parameters");
    }

    function testGetAllParametersWithOverrides()
    {
        $results = getAllThirdPartyParametersForMerchantSkinCombination(getSkinIdForContext(),$this->ids['merchant_id']);
        $this->assertCount(2,$results,'it should have 2 TPP it is associated with');
        $vivonet_parameters = $results['Vivonet'];
        $this->assertCount(12,$vivonet_parameters,"It should have 12 vivonet parameters");
        $this->assertEquals('1003-skin_0',$vivonet_parameters['alternate_url']);
        $this->assertEquals('1003-merch_1',$vivonet_parameters['merchant_key']);
        $vantive_parameters = $results['Vantive'];
        $this->assertCount(2,$vantive_parameters,"It should have 2 vantive parameters");
        $this->assertEquals('5001-skin_value',$vantive_parameters['OperatorID']);
        $this->assertEquals('5001-merch_value',$vantive_parameters['MerchantID']);
    }

    function testGetParametersWithOverridesForSpecificProvider()
    {
        $vivonet_parameters = getThirdPartyParametersForMerchantSkinCombination(1003,getSkinIdForContext(),$this->ids['merchant_id']);
        $this->assertCount(12,$vivonet_parameters,"It should have 12 vivonet parameters");
        $this->assertEquals('1003-skin_0',$vivonet_parameters['alternate_url']);
        $this->assertEquals('1003-merch_1',$vivonet_parameters['merchant_key']);

        $vantive_parameters = getThirdPartyParametersForMerchantSkinCombination(5001,getSkinIdForContext(),$this->ids['merchant_id']);
        $this->assertCount(2,$vantive_parameters,"It should have 2 vantive parameters");
        $this->assertEquals('5001-skin_value',$vantive_parameters['OperatorID']);
        $this->assertEquals('5001-merch_value',$vantive_parameters['MerchantID']);
    }


    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        $skin_resource = getOrCreateSkinAndBrandIfNecessaryWithLoyalty("thirdpartyskin","thirdpartybrand",null, null);
        ini_set('max_execution_time', 0);
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction(); ;

        setContext('com.splickit.thirdpartyskin');

    	$simple_menu_id = createTestMenuWithOneItem("item_one");
    	$ids['menu_id'] = $simple_menu_id;

    	$merchant_resource = createNewTestMerchant($simple_menu_id,['authorize'=>true]);
    	attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
    	$ids['merchant_id'] = $merchant_resource->merchant_id;

        foreach (ThirdPartyProviderParametersAdapter::staticGetRecords(['third_party_provider_id'=>1003],'ThirdPartyProviderParametersAdapter') as $index=>$tpp) {
            $data = [];
            $data['third_party_provider_parameter_id'] = $tpp['id'];
            $data['value'] = "1003-merch_$index";
            $data['merchant_id'] = $merchant_resource->merchant_id;
            $data['skin_id'] = getSkinIdForContext();
            $data['brand_id'] = getBrandIdFromCurrentContext();
            if ($index > 0) {
                $resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()),$data);
            }
            if ($index < 2) {
                $data['value'] = "1003-skin_$index";
                $resource = Resource::createByData(new ThirdPartyProviderParameterSkinMapsAdapter(getM()),$data);
            }
            if ($index == 1) {
                $data['value'] = "1003-brand_$index";
                $resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()),$data);
            }
            myerror_log("inset");
        }

        foreach (ThirdPartyProviderParametersAdapter::staticGetRecords(['third_party_provider_id'=>5001],'ThirdPartyProviderParametersAdapter') as $index=>$tpp) {
            $data = [];
            $data['third_party_provider_parameter_id'] = $tpp['id'];
            $data['value'] = "5001-merch_value";
            $data['merchant_id'] = $merchant_resource->merchant_id;
            $data['skin_id'] = getSkinIdForContext();
            if ($tpp['name'] == 'MerchantID') {
                $resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()), $data);
            }
            if ($tpp['name'] == 'OperatorID') {
                $data['value'] = "5001-skin_value";
                $resource = Resource::createByData(new ThirdPartyProviderParameterSkinMapsAdapter(getM()),$data);
            }
            myerror_log("inset");
        }



        $_SERVER['log_level'] = 5;
		$_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    ThirdPartyFunctionalityTest::main();
}
?>