<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class ThrottelingTest extends PHPUnit_Framework_TestCase
{

    function setUp()
    {
        setProperty('do_not_call_out_to_aws', 'true');
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__ . '-' . $_SERVER['STAMP'];
        setProperty("global_throttling", "true");
        setContext("com.splickit.worldhq");

        // we dont want to call to inspirepay
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
    }

    function testGreaterThanFiveMinutePrepTime()
    {
        $menu_id = createTestMenuWithOneItem("Test Item 1");
        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $prep_resource = Resource::createByData(new MerchantPreptimeInfoAdapter(getM()), array("merchant_id" => $merchant_resource->merchant_id, "entree_preptime_seconds" => 400));

        for ($i = 0; $i < 7; $i++) {
            $user_resource = createNewUserWithCCNoCVV();
            $user = logTestUserResourceIn($user_resource);

            $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note',3);
            $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
            $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id);
        }
        $order_adapter = new OrderAdapter(getM());
        $orders = $order_adapter->getRecords(['merchant_id'=>$merchant_id]);
        foreach ($orders as $order) {
            myerror_log($order['order_qty'].'  '.$order['pickup_dt_tm']);
        }
        myerror_log("done");
    }

    function testGetLeadTimesWithThrottlingChangeConcurrent()
    {
        $menu_id = createTestMenuWithOneItem("Test Item 1");
        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $prep_resource = Resource::createByData(new MerchantPreptimeInfoAdapter(getM()), array("merchant_id" => $merchant_resource->merchant_id, "entree_preptime_seconds" => 300));

        $hours_adapter = new HourAdapter(getM());
        $sql = "UPDATE Hour SET open = '12:59:00' WHERE merchant_id = $merchant_id ";
        $hours_adapter->_query($sql);

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $expected_first_pickup_time = date("Y-m-d H:i:s",getTomorrowTwelveNoonTimeStampDenver()+3600);
        $actual_first_pickup_time = date("Y-m-d H:i:s",$checkout_resource->lead_times_array[0]);
        $this->assertEquals($expected_first_pickup_time,$actual_first_pickup_time);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id);
        $this->assertNull($order_resource->error);

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $expected_first_pickup_time = date("Y-m-d H:i:s",getTomorrowTwelveNoonTimeStampDenver()+3600+300);
        $actual_first_pickup_time = date("Y-m-d H:i:s",$checkout_resource->lead_times_array[0]);
        $this->assertEquals($expected_first_pickup_time,$actual_first_pickup_time);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id);
        $this->assertNull($order_resource->error);


        $lead_time = new LeadTime($merchant_resource);
        $time_period_in_minutes = 5;
        $able_to_make_in_time_period = $lead_time->getNumberEntresThisMerchantIsAbleToMakeInTimePeriod($time_period_in_minutes);
        $existing_order_data = $lead_time->getExistingOrderResources(getTodayTwelveNoonTimeStampDenver());
        $existing_order_data = $lead_time->redistributeBucketsToAccountForOverloadedTimeSlots($existing_order_data,$able_to_make_in_time_period);
        $existing_order_data_by_strings = [];
        foreach ($existing_order_data as $key=>$value) {
            $key_as_string = date('H:i',$key);
            $existing_order_data_by_strings[$key_as_string] = $value;
        }



        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','the note',2);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $expected_first_pickup_time = date("Y-m-d H:i:s",getTomorrowTwelveNoonTimeStampDenver()+3600+900);
        $actual_first_pickup_time = date("Y-m-d H:i:s",$checkout_resource->lead_times_array[0]);
        $this->assertEquals($expected_first_pickup_time,$actual_first_pickup_time);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id);
        $this->assertNull($order_resource->error);

        $lead_time = new LeadTime($merchant_resource);
        $time_period_in_minutes = 5;
        $able_to_make_in_time_period = $lead_time->getNumberEntresThisMerchantIsAbleToMakeInTimePeriod($time_period_in_minutes);
        $existing_order_data = $lead_time->getExistingOrderResources(getTodayTwelveNoonTimeStampDenver());
        $existing_order_data = $lead_time->redistributeBucketsToAccountForOverloadedTimeSlots($existing_order_data,$able_to_make_in_time_period);
        $existing_order_data_by_strings = [];
        foreach ($existing_order_data as $key=>$value) {
            $key_as_string = date('H:i',$key);
            $existing_order_data_by_strings[$key_as_string] = $value;
        }
        $this->assertEquals(1,$existing_order_data_by_strings['13:15']);
        $this->assertCount(4,$existing_order_data_by_strings);

        // now change the concurrent to 2
        $prep_resource->concurrently_able_to_make = 2;
        $prep_resource->save();

        $lead_time = new LeadTime($merchant_resource);
        $time_period_in_minutes = 5;
        $able_to_make_in_time_period = $lead_time->getNumberEntresThisMerchantIsAbleToMakeInTimePeriod($time_period_in_minutes);
        $existing_order_data = $lead_time->getExistingOrderResources(getTodayTwelveNoonTimeStampDenver());
        $existing_order_data = $lead_time->redistributeBucketsToAccountForOverloadedTimeSlots($existing_order_data,$able_to_make_in_time_period);
        $existing_order_data_by_strings2 = [];
        foreach ($existing_order_data as $key=>$value) {
            $key_as_string = date('H:i',$key);
            $existing_order_data_by_strings2[$key_as_string] = $value;
        }

        $this->assertEquals(2,$existing_order_data_by_strings2['13:15'],'It should have moved both to the same bucket because of 2 concurrent');
        $this->assertCount(3,$existing_order_data_by_strings2);


        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','the note',2);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $expected_first_pickup_time = date("Y-m-d H:i:s",getTomorrowTwelveNoonTimeStampDenver()+3600);
        $actual_first_pickup_time = date("Y-m-d H:i:s",$checkout_resource->lead_times_array[0]);
        // $this->assertEquals($expected_first_pickup_time,$actual_first_pickup_time);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id);
        $this->assertNull($order_resource->error);

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','the note',2);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $expected_first_pickup_time = date("Y-m-d H:i:s",getTomorrowTwelveNoonTimeStampDenver()+3600);
        $actual_first_pickup_time = date("Y-m-d H:i:s",$checkout_resource->lead_times_array[0]);
        // $this->assertEquals($expected_first_pickup_time,$actual_first_pickup_time);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id);
        $this->assertNull($order_resource->error);

    }

    function testRedistributeBucketsForLargeNumberOfEntres()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $merchant_resource->advanced_ordering = 'Y';
        $merchant_resource->save();
        $mao_data = ["merchant_id"=>$merchant_id,'max_days_out'=>4];
        $maoia = new MerchantAdvancedOrderingInfoAdapter(getM());
        $maoi_resource = Resource::createByData($maoia,$mao_data);
        $prep_resource = Resource::createByData(new MerchantPreptimeInfoAdapter(getM()), array("merchant_id" => $merchant_resource->merchant_id, "entree_preptime_seconds" => 75));

        $hours_adapter = new HourAdapter(getM());
        $sql = "UPDATE Hour SET open = '13:00:00' WHERE merchant_id = $merchant_id ";
        $hours_adapter->_query($sql);
        $day = date('w') + 1;
        $day = $day == 8 ? 0 : $day;
        $sql = "UPDATE Hour SET day_open = 'N' WHERE day_of_week = $day";
        $hours_adapter->_query($sql);


        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note',10);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTodayTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $checkout_resource->lead_times_array = [getTomorrowTwelveNoonTimeStampDenver() + (2*3600)];
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id);
        $this->assertNull($order_resource->error);

        //validate that the buckets are getting set correctly
        $lead_time = new LeadTime($merchant_resource);
        $able_to_make_in_time_period = $lead_time->getNumberEntresThisMerchantIsAbleToMakeInTimePeriod($lead_time->getThrottlingBucketSizeInMinutes());
        $existing_order_data = $lead_time->getExistingOrderResources(getTodayTwelveNoonTimeStampDenver());
        $existing_order_data = $lead_time->redistributeBucketsToAccountForOverloadedTimeSlots($existing_order_data,$able_to_make_in_time_period);
        $existing_order_data_by_strings = [];
        foreach ($existing_order_data as $key=>$value) {
            $key_as_string = date('H:i',$key);
            $existing_order_data_by_strings[$key_as_string] = $value;
        }
        $this->assertEquals(4,$existing_order_data_by_strings['14:00']);
        $this->assertEquals(4,$existing_order_data_by_strings['13:55']);
        $this->assertEquals(2,$existing_order_data_by_strings['13:50']);


    }

    function testGetLeadTimesWithThrottlingSingleConcurrentButClosedToday()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $merchant_resource->advanced_ordering = 'Y';
        $merchant_resource->save();
        $mao_data = ["merchant_id"=>$merchant_id,'max_days_out'=>4];
        $maoia = new MerchantAdvancedOrderingInfoAdapter(getM());
        $maoi_resource = Resource::createByData($maoia,$mao_data);
        $prep_resource = Resource::createByData(new MerchantPreptimeInfoAdapter(getM()), array("merchant_id" => $merchant_resource->merchant_id, "entree_preptime_seconds" => 75));

        $hours_adapter = new HourAdapter(getM());
        $sql = "UPDATE Hour SET open = '13:00:00' WHERE merchant_id = $merchant_id ";
        $hours_adapter->_query($sql);
        $day = date('w') + 1;
        $day = $day == 8 ? 0 : $day;
        $sql = "UPDATE Hour SET day_open = 'N' WHERE day_of_week = $day";
        $hours_adapter->_query($sql);

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTodayTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $expected_second_pickup_time = date("Y-m-d H:i:s",getTomorrowTwelveNoonTimeStampDenver()+3600+300);
        $actual_second_pickup_time = date("Y-m-d H:i:s",$checkout_resource->lead_times_array[1]);
        $this->assertEquals($expected_second_pickup_time,$actual_second_pickup_time);
        $checkout_resource->lead_times_array = [$checkout_resource->lead_times_array[1]];
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id);
        $this->assertNull($order_resource->error);

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note',2);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTodayTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $expected_second_pickup_time = date("Y-m-d H:i:s",getTomorrowTwelveNoonTimeStampDenver()+3600+300);
        $actual_second_pickup_time = date("Y-m-d H:i:s",$checkout_resource->lead_times_array[1]);
        $this->assertEquals($expected_second_pickup_time,$actual_second_pickup_time);
        $checkout_resource->lead_times_array = [$checkout_resource->lead_times_array[1]];
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id);
        $this->assertNull($order_resource->error);

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note',2);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTodayTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id);
        $this->assertNull($order_resource->error);


        //validate that the buckets are getting set correctly
        $lead_time = new LeadTime($merchant_resource);
        $time_period_in_minutes = 5;
        $able_to_make_in_time_period = $lead_time->getNumberEntresThisMerchantIsAbleToMakeInTimePeriod($time_period_in_minutes);
        $existing_order_data = $lead_time->getExistingOrderResources(getTodayTwelveNoonTimeStampDenver());
        $existing_order_data = $lead_time->redistributeBucketsToAccountForOverloadedTimeSlots($existing_order_data,$able_to_make_in_time_period);
        $existing_order_data_by_strings = [];
        foreach ($existing_order_data as $key=>$value) {
            $key_as_string = getMySqlFormattedDateTimeFromTimeStampAndTimeZone($key);
            $existing_order_data_by_strings[$key_as_string] = $value;
        }

        //check for large order
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note',6);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTodayTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $expected_second_pickup_time = date("Y-m-d H:i:s",getTomorrowTwelveNoonTimeStampDenver()+3600+900);
        $actual_second_pickup_time = date("Y-m-d H:i:s",$checkout_resource->lead_times_array[1]);
        $this->assertEquals($expected_second_pickup_time,$actual_second_pickup_time);


        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note',4);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTodayTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $expected_second_pickup_time = date("Y-m-d H:i:s",getTomorrowTwelveNoonTimeStampDenver()+3600+600);
        $actual_second_pickup_time = date("Y-m-d H:i:s",$checkout_resource->lead_times_array[1]);
        $this->assertEquals($expected_second_pickup_time,$actual_second_pickup_time);
        $checkout_resource->lead_times_array = [$checkout_resource->lead_times_array[1]];
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id);
        $this->assertNull($order_resource->error);
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        // now add an extra order to the first bucket
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTodayTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id);
        $this->assertNull($order_resource->error);


        //validate that the buckets are getting set correctly
        $lead_time = new LeadTime($merchant_resource);
        $time_period_in_minutes = 5;
        $able_to_make_in_time_period = $lead_time->getNumberEntresThisMerchantIsAbleToMakeInTimePeriod($time_period_in_minutes);
        $existing_order_data = $lead_time->getExistingOrderResources(getTodayTwelveNoonTimeStampDenver());
        $existing_order_data = $lead_time->redistributeBucketsToAccountForOverloadedTimeSlots($existing_order_data,$able_to_make_in_time_period);
        $existing_order_data_by_strings = [];
        foreach ($existing_order_data as $key=>$value) {
            $key_as_string = getMySqlFormattedDateTimeFromTimeStampAndTimeZone($key);
            $existing_order_data_by_strings[$key_as_string] = $value;
        }

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note',3);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTodayTwelveNoonTimeStampDenver());
        $expected_second_pickup_time = date("Y-m-d H:i:s",getTomorrowTwelveNoonTimeStampDenver()+3600+900);
        $actual_second_pickup_time = date("Y-m-d H:i:s",$checkout_resource->lead_times_array[1]);
        $this->assertEquals($expected_second_pickup_time,$actual_second_pickup_time);
        $checkout_resource->lead_times_array = [$checkout_resource->lead_times_array[1]];
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id);
        $this->assertNull($order_resource->error);

        //validate that the buckets are getting set correctly
        $lead_time = new LeadTime($merchant_resource);
        $time_period_in_minutes = 5;
        $able_to_make_in_time_period = $lead_time->getNumberEntresThisMerchantIsAbleToMakeInTimePeriod($time_period_in_minutes);
        $existing_order_data = $lead_time->getExistingOrderResources(getTodayTwelveNoonTimeStampDenver());
        $existing_order_data = $lead_time->redistributeBucketsToAccountForOverloadedTimeSlots($existing_order_data,$able_to_make_in_time_period);
        $existing_order_data_by_strings = [];
        foreach ($existing_order_data as $key=>$value) {
            $key_as_string = getMySqlFormattedDateTimeFromTimeStampAndTimeZone($key);
            $existing_order_data_by_strings[$key_as_string] = $value;
        }

        // now try a more than 2 bucket order
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note',10);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTodayTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $expected_first_pickup_time = date("Y-m-d H:i:s",getTomorrowTwelveNoonTimeStampDenver()+3600+1800);
        $actual_first_pickup_time = date("Y-m-d H:i:s",$checkout_resource->lead_times_array[0]);
        $this->assertEquals($expected_second_pickup_time,$actual_second_pickup_time);
        $checkout_resource->lead_times_array = [$checkout_resource->lead_times_array[0]];
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id);
        $this->assertNull($order_resource->error);


        // add another large order in the future
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note',10);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTodayTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $checkout_resource->lead_times_array = [getTomorrowTwelveNoonTimeStampDenver() + (2*3600)];
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id);
        $this->assertNull($order_resource->error);

        //validate that the buckets are getting set correctly
        $lead_time = new LeadTime($merchant_resource);
        $time_period_in_minutes = 5;
        $able_to_make_in_time_period = $lead_time->getNumberEntresThisMerchantIsAbleToMakeInTimePeriod($time_period_in_minutes);
        $existing_order_data = $lead_time->getExistingOrderResources(getTodayTwelveNoonTimeStampDenver());
        $existing_order_data = $lead_time->redistributeBucketsToAccountForOverloadedTimeSlots($existing_order_data,$able_to_make_in_time_period);
        $existing_order_data_by_strings = [];
        foreach ($existing_order_data as $key=>$value) {
            $key_as_string = date('H:i',$key);
            $existing_order_data_by_strings[$key_as_string] = $value;
        }
        $this->assertEquals(4,$existing_order_data_by_strings['14:00']);
        $this->assertEquals(4,$existing_order_data_by_strings['13:55']);
        $this->assertEquals(2,$existing_order_data_by_strings['13:50']);
        $this->assertEquals(4,$existing_order_data_by_strings['13:30']);
        $this->assertEquals(4,$existing_order_data_by_strings['13:25']);
        $this->assertEquals(2,$existing_order_data_by_strings['13:20']);
        $this->assertEquals(3,$existing_order_data_by_strings['13:15']);

    }



    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        setProperty('do_not_call_out_to_aws', 'true');
        ini_set('max_execution_time', 300);
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction();
        setContext("com.splickit.worldhq");
        $brand_id = getBrandIdFromCurrentContext();
        $brand_resource = Resource::find(new BrandAdapter(getM()), $brand_id);
        $brand_resource->loyalty = 'Y';
        $brand_resource->save();

        $_SERVER['request_time1'] = microtime(true);

        $_SERVER['log_level'] = 5;

    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
    }

    /* mail method for testing */
    static function main()
    {
        $suite = new PHPUnit_Framework_TestSuite(__CLASS__);
        PHPUnit_TextUI_TestRunner::run($suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    ThrottelingTest::main();
}

?>