<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class TokyoJoesTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
        $this->ids = $_SERVER['unit_test_ids'];
        setContext('com.splickit.tokyojoes');
    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }

    function testGetLeadTimesArrayForCateringDuringOpenHours()
    {

        $menu_id = createTestCateringMenuWithOneItem();
        $merchant_resource = createNewTestMerchantWithCatering($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        setProperty("global_throttling", "false");
        //$merchant_id = $this->ids['merchant_id'];
        $current_time = getTomorrowTwelveNoonTimeStampDenver();
        $order_adapter = new OrderAdapter();
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'some dumb note');
        $checkout_data_resource = getCheckoutDataWithThrottling($order_data, $merchant_id, $current_time);
        //$checkout_data_resource = getCheckoutDataWithThrottling($order_data, $merchant_id, $time);
        $lead_times_array = $checkout_data_resource->lead_times_array;
        $this->assertNotNull($lead_times_array);
        $expected_first_pickup = mktime(17, 0, '0', date("m",$current_time)  , date("d",$current_time), date("Y",$current_time));
        //$expected_first_pickup = time()+(3*3600);
        myerror_log("expected first pickup time: ".date('Y-m-d H:i:s',$expected_first_pickup));
        myerror_log("actual first pickup time: ".date('Y-m-d H:i:s',$lead_times_array[0]));
        $this->assertEquals(date('Y-m-d H:i:s',$expected_first_pickup),date('Y-m-d H:i:s',$lead_times_array[0]));
        $this->assertEquals("Please note, your first available pickup time for this order is over 5 hours from now.",$checkout_data_resource->user_message);
    }

    function testSetMessageSendTimeForCatering()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        setProperty("global_throttling", "false");
        $merchant_id = $this->ids['merchant_id'];
        $current_time = getTomorrowTwelveNoonTimeStampDenver();
        $order_adapter = new OrderAdapter(getM());
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'some dumb note');
        $order_data['lead_time'] = 120;
        $order_resource = placeOrderFromOrderData($order_data,$current_time);
        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'E');
        $send_time = $message_resource->next_message_dt_tm;
        $send_time_as_string = date('Y-m-d H:i:s',$send_time);
        $pickup_time_as_string = $order_resource->pickup_dt_tm;
        myerror_log("send time: $send_time_as_string");
        myerror_log("pickup time: $pickup_time_as_string");
        $this->assertEquals($current_time,$send_time);
    }

    function testAPIv2()
    {
        $menu_id = createTestCateringMenuWithOneItem();
        $merchant_resource = createNewTestMerchantWithCatering($menu_id);
        $merchant_id = $merchant_resource->merchant_id;

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $order_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','note');

        $json_encoded_data = json_encode($order_data);
        $request = new Request();
        $request->url = '/app2/apiv2/cart/checkout';
        $request->method = "post";
        $request->body = $json_encoded_data;
        $request->mimetype = 'application/json';
        $request->_parseRequestBody();
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        $current_time = getTomorrowTwelveNoonTimeStampDenver();
        $place_order_controller->setCurrentTime($current_time);
        $checkout_data_resource = $place_order_controller->processV2Request();
        $this->assertNull($checkout_data_resource->error);
        $lead_times_array = $checkout_data_resource->lead_times_array;
        $this->assertNotNull($lead_times_array);
        $expected_first_pickup = mktime(17, 0, '0', date("m",$current_time)  , date("d",$current_time), date("Y",$current_time));
        //$expected_first_pickup = time()+(3*3600);
        myerror_log("expected first pickup time: ".date('Y-m-d h:i:s',$expected_first_pickup));
        myerror_log("actual first pickup time: ".date('Y-m-d h:i:s',$lead_times_array[0]));
        $this->assertEquals(date('Y-m-d h:i:s',$expected_first_pickup),date('Y-m-d h:i:s',$lead_times_array[0]));
        $this->assertEquals($expected_first_pickup,$lead_times_array[0]);
        $this->assertEquals("Please note, your first available pickup time for this order is over 5 hours from now.",$checkout_data_resource->user_message);

    }


    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time',300);
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction(); ;

        $skin_resource = getOrCreateSkinAndBrandIfNecessary('tokyojoes','tokyojoes',31,200);
        setContext('com.splickit.tokyojoes');
        $ids['skin_id'] = $skin_resource->skin_id;



        // create catering menu with single regular menu type
        $menu_id = createTestCateringMenuWithOneItem();

        // add a regular section
//        $menu_type_resource = createNewMenuType($menu_id, 'Regular Menu Type','E');
//        $size_resource = createNewSize($menu_type_resource->menu_type_id, 'Test Size 1');
//        createItem("Regular Item", $size_resource->size_id, $menu_type_resource->menu_type_id);

        $ids['menu_id'] = $menu_id;


        $merchant_resource = createNewTestMerchant($menu_id);
        attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
        $ids['merchant_id'] = $merchant_resource->merchant_id;

        $user_resource = createNewUser(array("flags"=>"1C20000001"));
        $ids['user_id'] = $user_resource->user_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    TokyoJoesTest::main();
}

?>