<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class VioAVSTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
		$this->ids = $_SERVER['unit_test_ids'];
        setContext('com.splickit.worldhq');
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }

    function testPlaceOrderUseNewServiceCheckAVSWithFail()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user_resource->uuid = substr($user_resource->uuid,0,17).'NOAVS';
        $user_resource->save();
        $user = logTestUserResourceIn($user_resource);

        $merchant_id = $this->ids['merchant_id'];
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNotNull($order_resource->error,'It should have thrown an error because the zip doesnt match');
        $this->assertEquals(VioPaymentService::FAILED_AVS_ERROR_MESSAGE,$order_resource->error);
        return $order_resource;
    }

    /**
     * @depends testPlaceOrderUseNewServiceCheckAVSWithFail
     */
    function testvalidateOrderChangedBackFromPendingToFailed($order_resource)
    {
        $order_id = $order_resource->cancelled_order_id;
        $order_record = OrderAdapter::staticGetRecordByPrimaryKey($order_id,"OrderAdapter");
        $this->assertEquals('N',$order_record['status'],"should have listed the cart as a failed payment");
    }


    /**
     * @depends testPlaceOrderUseNewServiceCheckAVSWithFail
     */
    function testvalidateOrderFailure($order_resource)
    {
        $this->assertNotNull($order_resource->error,"Order should have failed due to a mismatched AVS");
        $vio_payment_service = new VioPaymentService($data);
        $this->assertEquals(VioPaymentService::FAILED_AVS_ERROR_MESSAGE, $order_resource->error);
    }

    /**
     * @depends testPlaceOrderUseNewServiceCheckAVSWithFail
     */
    function testvalidateFlagsAreUnchangedOnUser($order_resource)
    {
        $base_order = CompleteOrder::getBaseOrderData($order_resource->cancelled_order_id, getM());
        $user = UserAdapter::staticGetRecordByPrimaryKey($base_order['user_id'], 'UserAdapter');
        $this->assertEquals('1C20000001', $user['flags'],"Flags should have stayed with CVV check off");
    }

    /**
     * @depends testPlaceOrderUseNewServiceCheckAVSWithFail
     */
    function testvalidateChargeWasRecorded($order_resource)
    {
        $user_id = $order_resource->user_id;
        $order_id = $order_resource->cancelled_order_id;
        $records = BalanceChangeAdapter::staticGetRecords(array("user_id"=>$user_id,"order_id"=>$order_id), 'BalanceChangeAdapter');
        $hash = createHashmapFromArrayOfArraysByFieldName($records,'process');
        $this->assertNotNull($hash['AuthCancelledAVS'],'It should have the auth cancelled record');
        $this->assertEquals('CANCELED',$hash['AuthCancelledAVS']['notes']);
        //$this->assertNotNull($hash['CCvoidAVS'],'It should have the CCVoid record');
        return $hash['AuthCancelledAVS'];
    }

    /**
     * @depends testvalidateChargeWasRecorded
     */
    function testCCProcessorWasRecorded($record)
    {
        $this->assertEquals($this->ids['billing_entity_external_id'], $record['cc_processor'],"cc processor should have recorded as the destination id");
    }

    /**
     * @depends testvalidateChargeWasRecorded
     */
    function testCCTransactionIdWasRecorded($record)
    {
        $this->assertTrue($record['cc_transaction_id'] != null,"cc transaction id should not be null");
    }

    /**
     * @depends testPlaceOrderUseNewServiceCheckAVSWithFail
     */
    function testvalidateChargeWasVoided($order_resource)
    {
        $user_id = $order_resource->user_id;
        $order_id = $order_resource->cancelled_order_id;
        $process = 'CCvoidAVS';
        $records = BalanceChangeAdapter::staticGetRecords(array("user_id"=>$user_id,"order_id"=>$order_id), 'BalanceChangeAdapter');
        $record = BalanceChangeAdapter::staticGetRecord(array("user_id"=>$user_id,"order_id"=>$order_id,"process"=>$process), 'BalanceChangeAdapter');
        $this->assertNull($record,"There should not be a CCvoid row since this was done on the Initial Auth call do to an AVS failure");

    }

    /**
     * @depends testPlaceOrderUseNewServiceCheckAVSWithFail
     */
    function testvalidateNoRecordInAdminReversal($order_resource)
    {
        $order_id = $order_resource->cancelled_order_id;
        $record = AdmOrderReversalAdapter::staticGetRecord(array("order_id"=>$order_id), 'AdmOrderReversalAdapter');
        $this->assertNull($record,"There should not be a record for a void");
        //$this->assertEquals("X",$record['credit_type'],"should have found the pending adm order reversal record");
    }

    /**
     * @depends testPlaceOrderUseNewServiceCheckAVSWithFail
     */
    function testvalidateNoEmailToUserOnRefund($order_resource)
    {
        $order_id = $order_resource->cancelled_order_id;
        $cancelled_order_record = OrderAdapter::staticGetRecordByPrimaryKey($order_id,'Order');
        $user_id = $cancelled_order_record['user_id'];
        $user = UserAdapter::getUserResourceFromId($user_id);
        $mmha = new MerchantMessageHistoryAdapter(getM());
        $records = $mmha->getRecords(array("message_delivery_addr"=>$user->email));
        $this->assertCount(0,$records,"should not have found an email");
    }

    /**
     * @depends testPlaceOrderUseNewServiceCheckAVSWithFail
     */
    function testValidateSecondFail($order_resource)
    {
        $base_order = CompleteOrder::getBaseOrderData($order_resource->cancelled_order_id);
        $merchant_id = $base_order['merchant_id'];
        $user = logTestUserIn($base_order['user_id']);

        $ucid = $base_order['ucid'];

        $url = "/app2/apiv2/cart/$ucid/checkout";
        $request = createRequestObject($url,'GET');
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_data_resource = $place_order_controller->processV2Request();
        $order_resource = placeOrderFromCheckoutResource($checkout_data_resource,$user,$merchant_id,0.00,$t);
//
//		$place_order_data['merchant_id'] = $merchant_id;
//		$place_order_data['note'] = "skip hours";
//		$place_order_data['user_id'] = $user['user_id'];
//		$place_order_data['cart_ucid'] = $ucid;
//		$place_order_data['tip'] = 0.00;
//		$place_order_data['merchant_payment_type_map_id'] = $checkout_data_resource->accepted_payment_types[0]['merchant_payment_type_map_id'];
//		$place_order_data['requested_time'] = $checkout_data_resource->lead_times_array[0];
//		$request = createRequestObject("/apiv2/orders/$ucid",'post',json_encode($place_order_data),'application/json');
//		$place_order_controller = new PlaceOrderController($mt, $user, $request);
//		$place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
//		$order_resource = $place_order_controller->processV2Request();
        $this->assertNotNull($order_resource->error);
        $this->assertEquals(VioPaymentService::FAILED_AVS_ERROR_MESSAGE,$order_resource->error);
        $this->assertEquals(422,$order_resource->http_code);

        //check to make sure both charges were voided
        $allbcrecords = BalanceChangeAdapter::staticGetRecords(array("order_id"=>$base_order['order_id']),'BalanceChangeAdapter');
        $bcrecords = BalanceChangeAdapter::staticGetRecords(array("order_id"=>$base_order['order_id'],"process"=>'AuthCancelledAVS'),'BalanceChangeAdapter');
        $this->assertCount(2,$bcrecords,"it shoudl have two auth canceled records");
        foreach($bcrecords as $record) {
            //notes should contain the 'voided'
            $notes = $record['notes'];
            $this->assertContains("CANCELED",$notes,'Should have shown the authcode was cancelled');
        }
        // validate only 1 order row
        $bcrecords = BalanceChangeAdapter::staticGetRecords(array("order_id"=>$base_order['order_id'],"process"=>'Order'),'BalanceChangeAdapter');
        $this->assertCount(1,$bcrecords,"it should have only 1 order record");
        return $order_resource;
    }

    /**
     * @depends testValidateSecondFail
     */
    function testOnlyOneOrderRowAfterASuccessfulPlaceOrder($order_resource)
    {
        $base_order = CompleteOrder::getBaseOrderData($order_resource->cancelled_order_id);
        $merchant_id = $base_order['merchant_id'];
        $user_resource = getUserResourceFromId($base_order['user_id']);
        $user_resource->uuid = substr($user_resource->uuid,0,17).'12345';
        $user_resource->save();
        $user = logTestUserResourceIn($user_resource);
        $ucid = $base_order['ucid'];
        $checkout_resource = getCheckoutResourceFromCartUcid($ucid,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);

        // validate only 1 order row
        $bcrecords = BalanceChangeAdapter::staticGetRecords(array("order_id"=>$base_order['order_id'],"process"=>'Order'),'BalanceChangeAdapter');
        $this->assertCount(1,$bcrecords,"it should have only 1 order record");

        $bcrecords = BalanceChangeAdapter::staticGetRecords(array("order_id"=>$base_order['order_id'],"process"=>'Authorize'),'BalanceChangeAdapter');
        $this->assertCount(1,$bcrecords,"it should have only 1 Authorization record");

    }

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

    	ini_set('max_execution_time',0);
    	$_SERVER['request_time1'] = microtime(true);
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	ini_set('max_execution_time',300);
    	SplickitCache::flushAll();
    	$db = DataBase::getInstance();
    	$mysqli = $db->getConnection();
    	$mysqli->begin_transaction(); ;

        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id,["authorize"=>true]);
        $ids['cc_billing_entity_id'] = $merchant_resource->cc_billing_entity_id;
        $ids['billing_entity_external_id'] = $merchant_resource->billing_entity_external_id;

        $ids['merchant_id'] = $merchant_resource->merchant_id;


        $_SERVER['log_level'] = 5;
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	SplickitCache::flushAll();
    	$db = DataBase::getInstance();
    	$mysqli = $db->getConnection();
    	$mysqli->rollback();
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    VioAVSTest::main();
}

?>