<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class VIOMerchantAccountTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
		$this->ids = $_SERVER['unit_test_ids'];
		
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }

    function testCreateMerchantAccountHappy()
    {
        setContext('com.splickit.worldhq');
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id,['no_payment'=>true]);
        $merchant_id = $merchant_resource->merchant_id;

        $sql = "INSERT INTO `payment_merchant_applications` VALUES (null, $merchant_id, 'BIG INVESTMENTS & TRUST COMPANY', '', 'biginvestmentcompany@gmail.com', 'BIG INVESTMENTS & TRUST CO', '83-0758546', '43247812', '107000262', 'checking', 'business', 'Bob', 'Newheart', '1981-06-24', '234-67-9854', '1234567890', '', 'https://s3.amazonaws.com/splickit.merchant.processing.documents/OwnerInfo/107427/driver_license.png', 'https://s3.amazonaws.com/splickit.merchant.processing.documents/OwnerInfo/107427/voided_check.png',null, null, NOW(), NOW(), 'N')";
        $pmaa = new PaymentMerchantApplicationsAdapter(getM());
        $pmaa->_query($sql);

        $data["first_name"] = "Chris";
        $data["last_name"] = "Terry";
        $data["date_of_birth"] = "1970-1-23";
        $data["social_security_number"] = "555-55-5555";
        $data["phone"] = "555-555-5555";
        $data["business_legal_name"] = "Booming Biz";  // business name
        $data["doing_business_as"] = "sum dum name";
        $data["ein"] = "444-44-4444"; // federal tax id
        $data["bank_account_account_number"] = "55555555555";
        $data["bank_account_routing_number"] = "011401533";
        $data["merchant_id"] = $merchant_resource->merchant_id;

        $url = "http://localhost/app2/portal/payments/merchant_account";
        $request = createRequestObject($url,'POST',json_encode($data));
        $card_gateway_controller = new CardGatewayController(getM(),null,$request,5);
        $response = makeMockPortalRequest($card_gateway_controller);

        $response_array = json_decode($response,true);
        $this->assertEquals(200,$response_array['http_code']);
        $response_data = $response_array['data'];
        $this->assertTrue(isset($response_data['vio_credit_card_processor_id']),'It should have a credit card processor id');
        $this->assertTrue(isset($response_data['merchant_payment_type_map']),'It should have a merchant_payment_type_map' );


    }

    function testCreateMerchantAccountFullFailure()
    {
        setContext('com.splickit.worldhq');
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id,['no_payment'=>true]);
        $merchant_id = $merchant_resource->merchant_id;

        $sql = "INSERT INTO `payment_merchant_applications` VALUES (null, $merchant_id, 'BIG INVESTMENTS & TRUST COMPANY', '', 'biginvestmentcompany@gmail.com', 'BIG INVESTMENTS & TRUST CO', '83-0758546', '43247812', '107000262', 'checking', 'business', 'Bob', 'Newheart', '1981-06-24', '234-67-9854', '1234567890', '', 'https://s3.amazonaws.com/splickit.merchant.processing.documents/OwnerInfo/107427/driver_license.png', 'https://s3.amazonaws.com/splickit.merchant.processing.documents/OwnerInfo/107427/voided_check.png',null, null, NOW(), NOW(), 'N')";
        $pmaa = new PaymentMerchantApplicationsAdapter(getM());
        $pmaa->_query($sql);


        $data["first_name"] = "Chris";
        $data["last_name"] = "Terry";
        $data["date_of_birth"] = "1970-1-23";
        $data["social_security_number"] = "555-55-5555";
        $data["phone"] = "555-555-5555";
        $data["business_legal_name"] = "Booming Biz";  // business name
        $data["doing_business_as"] = "sum dum name";
        $data["ein"] = "444-44-4444"; // federal tax id
        $data["bank_account_account_number"] = "55555555555";
        $data["bank_account_routing_number"] = "555555555";
        $data["merchant_id"] = $merchant_resource->merchant_id;

        $url = "http://localhost/app2/portal/payments/merchant_account";
        $request = createRequestObject($url,'POST',json_encode($data));
        $card_gateway_controller = new CardGatewayController(getM(),null,$request,5);
        $response = makeMockPortalRequest($card_gateway_controller);

        $response_array = json_decode($response,true);
        $this->assertEquals(422,$response_array['http_code']);
        $error = $response_array['error'];
        $this->assertEquals(CardGatewayController::MERCHANT_ACCOUNT_NOT_CREATED_ERROR_MESSAGE." Invalid Routing Number.",$error['error_message']);
    }

    function testCreateMerchantAccountPartialFailureGoodAccountNumberButBadStatusCode()
    {
        setContext('com.splickit.worldhq');
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id,['no_payment'=>true]);

        $merchant_id = $merchant_resource->merchant_id;

        $sql = "INSERT INTO `payment_merchant_applications` VALUES (null, $merchant_id, 'BIG INVESTMENTS & TRUST COMPANY', '', 'biginvestmentcompany@gmail.com', 'BIG INVESTMENTS & TRUST CO', '83-0758546', '43247812', '107000262', 'checking', 'business', 'Bob', 'Newheart', '1981-06-24', '234-67-9854', '1234567890', '', 'https://s3.amazonaws.com/splickit.merchant.processing.documents/OwnerInfo/107427/driver_license.png', 'https://s3.amazonaws.com/splickit.merchant.processing.documents/OwnerInfo/107427/voided_check.png',null, null, NOW(), NOW(), 'N')";
        $pmaa = new PaymentMerchantApplicationsAdapter(getM());
        $pmaa->_query($sql);


        $data["first_name"] = "Chris";
        $data["last_name"] = "Terry";
        $data["date_of_birth"] = "1970-1-23";
        $data["social_security_number"] = "555-55-5555";
        $data["phone"] = "555-555-5555";
        $data["business_legal_name"] = "Booming Biz";  // business name
        $data["doing_business_as"] = "sum dum name";
        $data["ein"] = "444-44-4444"; // federal tax id
        $data["bank_account_account_number"] = "55555555555";
        $data["bank_account_routing_number"] = "888888888";
        $data["merchant_id"] = $merchant_resource->merchant_id;

        $url = "http://localhost/app2/portal/payments/merchant_account";
        $request = createRequestObject($url,'POST',json_encode($data));
        $card_gateway_controller = new CardGatewayController(getM(),null,$request,5);
        $response = makeMockPortalRequest($card_gateway_controller);

        $response_array = json_decode($response,true);
        $this->assertEquals(200,$response_array['http_code']);
        $user_message = $response_array['message'];
        $this->assertEquals(CardGatewayController::MERCHANT_ACCOUNT_CREATED_BUT_ERROR_MESSAGE,$user_message);
    }


    static function setUpBeforeClass()
    {
        $code = setNewStamp();

    	$_SERVER['request_time1'] = microtime(true);
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	ini_set('max_execution_time',300);
    	SplickitCache::flushAll();
    	$db = DataBase::getInstance();
    	$mysqli = $db->getConnection();
    	$mysqli->begin_transaction(); ;

    	$_SERVER['log_level'] = 5;
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	SplickitCache::flushAll();
    	$db = DataBase::getInstance();
    	$mysqli = $db->getConnection();
    	$mysqli->rollback();
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    VIOMerchantAccountTest::main();
}

?>