<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class VioPaymentTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
		$this->ids = $_SERVER['unit_test_ids'];
		
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }

    function testGetVioWriteMethodTest()
    {
        $this->assertNotNull(getProperty('vio_write_username_password'),"shojld have a value");
        $this->assertEquals(getProperty('vio_write_username_password'),VioPaymentService::getVioWriteCredentials(),"Should get a valid username and password");
    }

    function testGetVioWriteCredentials()
    {
        $user_resource = $this->ids['user_resource'];
        $user = logTestUserResourceIn($user_resource);
        $user_controller = new UserController($mt,$user,$request);
        $resource = $user_controller->getVioWriteCredentials();
        $this->assertNotNull($resource->vio_write_credentials,"THere shoudl be a good valud for vio write creds");
        $this->assertEquals(getProperty('vio_write_username_password'),$resource->vio_write_credentials);
    }
    
    function testIsMonthYearInThePastUsingPastDate()
    {
    	$vio_payment_service = new VioPaymentService($data);
    	$this->assertTrue($vio_payment_service->isMonthYearInThePast(6, 2013),"should have returned true since june 2013 is in the past");
    }
    
    function testIsMonthYearInThePastCurrentDate()
    {
    	$month = date('m');
    	$year = date('Y');
    	$vio_payment_service = new VioPaymentService($data);
    	$this->assertFalse($vio_payment_service->isMonthYearInThePast($month, $year),"shoudl have returned false becuase CC should be good for the entire last month");
    }
    
    function testIsMonthYearInThePastUsingFutureDate()
    {
    	$vio_payment_service = new VioPaymentService($data);
    	$this->assertFalse($vio_payment_service->isMonthYearInThePast(6, 2029),"SHould have returned false since 2019 is far in the future");
    }
    
    function testIsVaultedCardExpiredCardExpired()
    {
    	$_SERVER['HTTP_NO_CC_CALL'] = 'false';
    	$uuid = 'asdf5-asdf5-asdf5-asdf5-zzzzz';
    	$this->assertTrue(VioPaymentService::isVaultedCardExpired($uuid));
    }
    
    function testIsVaultedCardExpiredCardValid()
    {
    	$_SERVER['HTTP_NO_CC_CALL'] = 'false';
    	$uuid = 'asdf5-asdf5-asdf5-asdf5-asdf5';
    	$this->assertFalse(VioPaymentService::isVaultedCardExpired($uuid));
    }

    function testGetVioPaymentErrorMessage()
    {
    	$vio_payment_service = new VioPaymentService($data);
    	$returned_error_message = "SOM DUM ERROR";
    	$this->assertEquals($returned_error_message, $vio_payment_service->formatVioResponseMessage($returned_error_message));
    }

    function testGetVioPaymentErrorMessageForAVSFailure()
    {
    	$vio_payment_service = new VioPaymentService($data);
    	$returned_error_message = "AVS FAILURE - AYXWZ 064809";
    	$this->assertEquals("AVS FAILURE", $vio_payment_service->formatVioResponseMessage($returned_error_message));
    }

    function testFormatVioResponseMessageTSYSErrorFormat()
    {
        $response_message = 'Address Verification Failed';
        $json = '{"params":{ "transaction_id":"123852909", "response_code":"A0000", "avs_code":"N", "cvv_code":"M"},"cvv_result":{ "message":null, "code":null},"message":{ "cvvVerificationCode":"M", "authCode":"028430", "hostReferenceNumber":"516617406501", "taskID":"131808573", "transactionID":"123852909", "transactionTimestamp":"2015-06-15T11:07:00", "transactionAmount":"255", "processedAmount":"255", "totalAmount":"255", "addressVerificationCode":"N", "responseMessage":"'.$response_message.'", "maskedCardNumber":"3410", "customerReceipt":"SNARFS MOBILE ORDERING \\n1035 PEARL ST 5TH FLOOR \\nBOULDER CO 80302\\n06/15/2015 11:07:00\\n\\nCREDIT - SALE\\n\\nCARD # : **** **** **** 3410\\nCARD TYPE : VISA\\nREF # : 123852909 \\nInvoice number : 123852909\\nAUTH CODE : 028430\\nSubtotal: $02.55\\n--------------------------------------\\nTotal:$02.55\\n--------------------------------------\\n\\n\\nCary Russell\\n\\n CUSTOMER ACKNOWLEDGES RECEIPT OF \\n GOODS AND/OR SERVICES IN THE AMOUNT \\n OF THE TOTAL SHOWN HEREON AND AGREES \\n TO PERFORM THE OBLIGATIONS SET FORTH \\n BY THE CUSTOMER`S AGREEMENT WITH THE \\n ISSUER\\nAPPROVED\\n\\n\\n\\n\\nCustomer Copy", "merchantReceipt":"SNARFS MOBILE ORDERING \\n1035 PEARL ST 5TH FLOOR \\nBOULDER CO 80302\\n06/15/2015 11:07:00\\n\\nCREDIT - SALE\\n\\nCARD # : **** **** **** 3410\\nCARD TYPE : VISA\\nREF # : 123852909 \\nInvoice number : 123852909\\nAUTH CODE : 028430\\nSubtotal: $02.55\\n--------------------------------------\\nTotal:$02.55\\n--------------------------------------\\n\\nSignature :\\n __________________________ \\n\\nCary Russell\\n\\n CUSTOMER ACKNOWLEDGES RECEIPT OF \\n GOODS AND/OR SERVICES IN THE AMOUNT \\n OF THE TOTAL SHOWN HEREON AND AGREES \\n TO PERFORM THE OBLIGATIONS SET FORTH \\n BY THE CUSTOMER`S AGREEMENT WITH THE \\n ISSUER\\nAPPROVED\\n\\n\\n\\n\\nMerchant Copy", "responseCode":"A0000", "status":"PASS", "cardType":"V"},"avs_result":{ "code":null, "message":null, "street_match":null, "postal_match":null},"authorization":null,"success?":false,"test":false }';
        $json_array = json_decode($json, true);
        $vio_payment_service = new VioPaymentService($data);
        $vio_error_text = $vio_payment_service->formatVioResponseMessage($json_array['message']);
        $this->assertEquals($response_message,$vio_error_text,"should have been able to pull out the error message from the TSYS format");
    }

    function testProcessTSYSError()
    {

        $vio_response_message = 'Address Verification Failed';
        $error_message = "Sorry but your credit card did not pass addresss validation, please re-enter or try a diffenent card";
        $sql = "INSERT INTO `Lookup` VALUES(null, 'vio_payment_error', '$vio_response_message', '$error_message', 'Y', '0000-00-00 00:00:00', '0000-00-00 00:00:00', 'N')";
        $lookup_adapter = new LookupAdapter($mimetypes);
        $lookup_adapter->_query($sql);
        $vio_json = '{"path":"/payments","route":"/payments","mode":"post","status":"422 Unprocessable Entity","errors":{"payment":{"gateway_error":[{"message":"Address Verification Failed"}]}},"data":{"payment":{"captures":[],"fraudulence":{"_id":"557f0633a9e7c0588f00008e","confidence":-1,"blocked":false,"flagged":false},"refunds":[],"response":{"params":{"transaction_id":"123852909","response_code":"A0000","avs_code":"N","cvv_code":"M"},"cvv_result":{"message":null,"code":null},"message":{"cvvVerificationCode":"M","authCode":"028430","hostReferenceNumber":"516617406501","taskID":"131808573","transactionID":"123852909","transactionTimestamp":"2015-06-15T11:07:00","transactionAmount":"255","processedAmount":"255","totalAmount":"255","addressVerificationCode":"N","responseMessage":"Address Verification Failed","maskedCardNumber":"3410","customerReceipt":"SNARFS MOBILE ORDERING \\n1035 PEARL ST 5TH FLOOR \\nBOULDER CO 80302\\n06/15/2015 11:07:00\\n\\nCREDIT - SALE\\n\\nCARD # : **** **** **** 3410\\nCARD TYPE : VISA\\nREF # : 123852909 \\nInvoice number : 123852909\\nAUTH CODE : 028430\\nSubtotal: $02.55\\n--------------------------------------\\nTotal:$02.55\\n--------------------------------------\\n\\n\\nCary Russell\\n\\n CUSTOMER ACKNOWLEDGES RECEIPT OF \\n GOODS AND/OR SERVICES IN THE AMOUNT \\n OF THE TOTAL SHOWN HEREON AND AGREES \\n TO PERFORM THE OBLIGATIONS SET FORTH \\n BY THE CUSTOMER`S AGREEMENT WITH THE \\n ISSUER\\nAPPROVED\\n\\n\\n\\n\\nCustomer Copy","merchantReceipt":"SNARFS MOBILE ORDERING \\n1035 PEARL ST 5TH FLOOR \\nBOULDER CO 80302\\n06/15/2015 11:07:00\\n\\nCREDIT - SALE\\n\\nCARD # : **** **** **** 3410\\nCARD TYPE : VISA\\nREF # : 123852909 \\nInvoice number : 123852909\\nAUTH CODE : 028430\\nSubtotal: $02.55\\n--------------------------------------\\nTotal:$02.55\\n--------------------------------------\\n\\nSignature :\\n __________________________ \\n\\nCary Russell\\n\\n CUSTOMER ACKNOWLEDGES RECEIPT OF \\n GOODS AND/OR SERVICES IN THE AMOUNT \\n OF THE TOTAL SHOWN HEREON AND AGREES \\n TO PERFORM THE OBLIGATIONS SET FORTH \\n BY THE CUSTOMER`S AGREEMENT WITH THE \\n ISSUER\\nAPPROVED\\n\\n\\n\\n\\nMerchant Copy","responseCode":"A0000","status":"PASS","cardType":"V"},"avs_result":{"code":null,"message":null,"street_match":null,"postal_match":null},"authorization":null,"success?":false,"test":false},"_id":"557f0633a9e7c0588f00008d","account_id":"51a8dd097a0bfba2e8000003","amount":"2.55","authorization_id":"123852909","cents":255,"created_at":"2015-06-15T17:06:59Z","credit_card_brand":"visa","credit_card_id":"557f054175f9eba41000062c","credit_card_identifier":"6448-ywx8z-ma8et-b8x8b","credit_card_last_four":"3410","currency":"USD","destination_id":"555f754675f9eb1385000425","destination_identifier":"MULTI908258000042","destination_kind":"tsys","destination_name":"Snarfs Mobile Ordering","destination_request_time":3.008736115,"first_name":"Cary","void_status":"unvoided","updated_at":"2015-06-15T17:07:02Z","transaction_id":"123852909","transacted":false,"gateway_order_id":"7380767","identifier":"5aeb72ad-b394-40ea-bd9d-d2caa71b57d4","kind":"purchase","last_name":"Russell","require_cvv":false,"status":"failure","zip":"80302","destination":"MULTI908258000042","credit_card":"6448-ywx8z-ma8et-b8x8b","links":[{"rel":"self","href":"https://api.value.io/v1/payments/557f0633a9e7c0588f00008d"},{"rel":"index","href":"https://api.value.io/v1/payments"}]}}}';
        $full_vio_payment_results = json_decode($vio_json,true);
        $vio_payment_results = $full_vio_payment_results['data'];
        $vio_payment_results['http_code'] = 422;
        $vio_payment_results['status'] = 'failure';

        $vio_payment_service = new VioPaymentService($data);
        $result = $vio_payment_service->processVioError($vio_payment_results);
        $this->assertEquals($error_message, $result['response_text']);
    }
    
    function testProcessVioAVSError()
    {
    	$vio_json = '{"path":"/payments","route":"/payments","mode":"post","status":"422 Unprocessable Entity","errors":{"payment":{"gateway_error":[{"message":"AVS FAILURE - AYXWZ 021221"}]}},"data":{"payment":{"captures":[],"refunds":[],"response":{"cvv_result":{"code":"M","message":"Match"},"avs_result":{"street_match":"N","code":"N","message":"Street address and postal code do not match.","postal_match":"N"},"params":{"recurring":"0","risk":"00","order_number":"1xxxxxxxxxxx3026","reference":"FADELCBff0","avs_result":"N","cvv_result":"M","front_end":"45","code":"650103","success":"X","message":"AVS FAILURE - AYXWZ 021221"},"authorization":"FADELCBff0;bankcard","success?":false,"test":false,"message":"AVS FAILURE - AYXWZ 021221"},"void_response":{},"_id":"543c786e75f9eba18a000199","account_id":"51a8dd097a0bfba2e8000003","amount":"16.98","authorization_id":"FADELCBff0","cents":1698,"created_at":"2014-10-14T01:12:14Z","credit_card_brand":"visa","credit_card_id":"543c786b75f9eba18a000198","credit_card_identifier":"6895-1fd9m-ej256-62h16","currency":"USD","destination_id":"543553e275f9eb24c500015c","destination_identifier":"W9HV5184OQ","destination_kind":"sage-gateway-7","destination_name":"sage_gateway_7","destination_request_time":4.192775329,"first_name":"Paul","gateway_order_id":"1xxxxxxxxxxx3026","identifier":"4c7d5a2a-b6cc-44c5-858b-531e591c03c5","kind":"purchase","last_name":"Potocki","require_cvv":false,"void_status":"unvoided","updated_at":"2014-10-14T01:12:18Z","status":"failure","transaction_id":"FADELCBff0","transacted":false,"zip":"54115","destination":"W9HV5184OQ","credit_card":"6895-1fd9m-ej256-62h16","links":[{"rel":"self","href":"https://api.value.io/v1/payments/543c786e75f9eba18a000199"},{"rel":"index","href":"https://api.value.io/v1/payments"}]}}}';
    	$full_vio_payment_results = json_decode($vio_json,true);
    	$vio_payment_results = $full_vio_payment_results['data'];
    	$vio_payment_results['http_code'] = 422;
    	$vio_payment_results['status'] = 'failure';
    	
    	$vio_payment_service = new VioPaymentService($data);
    	$result = $vio_payment_service->processVioError($vio_payment_results);
    	$this->assertEquals($this->ids['error_message'], $result['response_text']);
    	
    }

    function testProcessVIOCurlError()
    {
        $vio_payment_results['error'] = "SSL peer certificate or SSH remote key was not OK";
        $vio_payment_results['error_no'] = 51;
        $vio_payment_results['http_code'] = 0;
        $vio_payment_service = new VioPaymentService($data);
        $result = $vio_payment_service->processVioError($vio_payment_results);
        $this->assertEquals($vio_payment_service->curl_error_user_message, $result['response_text'],"Should have gotten the default curl error message");
    }
    
    static function setUpBeforeClass()
    {
        $code = setNewStamp();

    	ini_set('max_execution_time',0);
    	$_SERVER['request_time1'] = microtime(true);
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	ini_set('max_execution_time',300);
    	      SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();$mysqli->begin_transaction(); ;
    	$error_message = "Your address did not match";
    	$ids['error_message'] = $error_message;
    	$sql = "INSERT INTO `Lookup` VALUES(null, 'vio_payment_error', 'AVS FAILURE', '$error_message', 'Y', '0000-00-00 00:00:00', '0000-00-00 00:00:00', 'N')";
    	$lookup_adapter = new LookupAdapter($mimetypes);
    	$lookup_adapter->_query($sql);
/*    	$skin_resource = createWorldHqSkin();
    	$ids['skin_id'] = $skin_resource->skin_id;
    	
    	$menu_id = createTestMenuWithNnumberOfItems(1);
    	$ids['menu_id'] = $menu_id;

    	$merchant_resource = createNewTestMerchant($menu_id);
    	attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
    	$ids['merchant_id'] = $merchant_resource->merchant_id;
*/    	
    	$user_resource = createNewUser(array("flags"=>"1C20000001"));
    	$ids['user_id'] = $user_resource->user_id;
    	$ids['user_resource'] = $user_resource;
    	
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    VioPaymentTest::main();
}

?>