<?php
error_reporting(E_ERROR | E_COMPILE_ERROR | E_COMPILE_WARNING | E_PARSE);
$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');
$db_info->database = 'smaw_unittest';
$db_info->username = 'root';
$db_info->password = 'splickit';
if (isset($_SERVER['XDEBUG_CONFIG'])) {
    putenv("SMAW_ENV=unit_test_ide");
    $db_info->hostname = "127.0.0.1";
    $db_info->port = 13306;
} else {
    putenv("SMAW_ENV=unit_test");
    $db_info->hostname = "db_container";
    $db_info->port = 3306;
}
$_SERVER['DB_INFO'] = $db_info;

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';


class WorldPayTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;
    var $api_port = "80";

    function setUp()
    {
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'].'-'.generateCode(5);
        $this->ids = $_SERVER['unit_test_ids'];
        setContext('com.splickit.maryjane');
        if (isset($_SERVER['XDEBUG_CONFIG'])) {
            $this->api_port = "10080";
        }
    }

    function tearDown()
    {
        //delete your instance
        unset($this->ids);
        unset($_SERVER['max_lead']);
    }

    function testEndToEndCharge()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user_id = $user_resource->user_id;
        $user = logTestUserResourceIn($user_resource);
        $ubpa = new UserBrandPointsMapAdapter(getM());
        $ubpa->createUserLoyaltyAccount($user_id, $this->ids['brand_id'], $user_resource->contact_no);

        $merchant_id = $this->ids['merchant_id'];

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $checkout_order_resource = CompleteOrder::getBaseOrderData($checkout_resource->oid_test_only);
        $this->assertNull($checkout_resource->error);
        $cart_ucid = $checkout_resource->ucid;
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);

        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order_resource->order_id],'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
        $payment_row = $bcr_hash['P2P'];
        $this->assertEquals('WorldPay',$payment_row['cc_processor']);
        $this->assertEquals('PENDING',$payment_row['notes']);
        $this->assertTrue($payment_row['cc_transaction_id'] > 0);

        // order will be in an open state but messages will all be set to 'X'
        $this->assertEquals('O',$order_resource->status,"It shoudl be in an open state");
        $order_message = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'E');
        $this->assertEquals('X',$order_message->locked,"It should have locked as 'X' for all order message");
        return $payment_row;
    }

    /**
     * @depends testEndToEndCharge
     */
    function testHaveActivityCheckForStatusOfOrder($payment_row)
    {
        $id = $payment_row['id'];
        $_SERVER['WORLD_PAY_RESPONSE'] = 'received';
        $activity_history_adapter = new ActivityHistoryAdapter(getM());
        $activity_history_resource = $activity_history_adapter->createActivityReturnActivityResource('P2PStatusCheck', time()-2, null,null,300);
        $activity = $activity_history_adapter->getActivityFromUnlockedActivityHistoryResource($activity_history_resource);
        $activity->executeThisActivity();

        $payment_resource = Resource::find(new BalanceChangeAdapter(getM()),$id);
        $this->assertEquals('PENDING',$payment_resource->notes);
        $order_message = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($payment_row['order_id'],'E');
        $this->assertEquals('X',$order_message->locked,"It should have locked as 'X' for all order message");

        $_SERVER['WORLD_PAY_RESPONSE'] = 'approved';
        $activity_history_adapter = new ActivityHistoryAdapter(getM());
        $activity_history_resource = $activity_history_adapter->createActivityReturnActivityResource('P2PStatusCheck', time()-2, null,null,300);
        $activity = $activity_history_adapter->getActivityFromUnlockedActivityHistoryResource($activity_history_resource);
        $activity->executeThisActivity();

        $payment_resource = Resource::find(new BalanceChangeAdapter(getM()),$id);
        $this->assertEquals('COMPLETED-' . getRawStamp(),$payment_resource->notes);
        $order_message = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($payment_row['order_id'],'E');
        $this->assertEquals('N',$order_message->locked,"It should have locked as 'N' for all order message");

    }

    function testWithUserCancel()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user_id = $user_resource->user_id;
        $user = logTestUserResourceIn($user_resource);
        $ubpa = new UserBrandPointsMapAdapter(getM());
        $ubpa->createUserLoyaltyAccount($user_id, $this->ids['brand_id'], $user_resource->contact_no);

        $merchant_id = $this->ids['merchant_id'];

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);

        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order_resource->order_id],'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
        $payment_row = $bcr_hash['P2P'];
        $this->assertEquals('WorldPay',$payment_row['cc_processor']);
        $this->assertEquals('PENDING',$payment_row['notes']);
        $this->assertTrue($payment_row['cc_transaction_id'] > 0);

        // order will be in an open state but messages will all be set to 'X'
        $this->assertEquals('O',$order_resource->status,"It shoudl be in an open state");
        $order_message = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'E');
        $this->assertEquals('X',$order_message->locked,"It should have locked as 'X' for all order message");
        $id = $payment_row['id'];

        $_SERVER['WORLD_PAY_RESPONSE'] = 'declined';
        $activity_history_adapter = new ActivityHistoryAdapter(getM());
        $activity_history_resource = $activity_history_adapter->createActivityReturnActivityResource('P2PStatusCheck', time()-2, null,null,300);
        $activity = $activity_history_adapter->getActivityFromUnlockedActivityHistoryResource($activity_history_resource);
        $activity->executeThisActivity();

        $payment_resource = Resource::find(new BalanceChangeAdapter(getM()),$id);
        $this->assertEquals('DECLINED' . getRawStamp(),$payment_resource->notes);
        $order_message = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($payment_row['order_id'],'E');
        $this->assertEquals('C',$order_message->locked,"It should have locked as 'C' for all order message");
    }


    function testExpired()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user_id = $user_resource->user_id;
        $user = logTestUserResourceIn($user_resource);
        $ubpa = new UserBrandPointsMapAdapter(getM());
        $ubpa->createUserLoyaltyAccount($user_id, $this->ids['brand_id'], $user_resource->contact_no);

        $merchant_id = $this->ids['merchant_id'];

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);

        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order_resource->order_id],'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
        $payment_row = $bcr_hash['P2P'];
        $this->assertEquals('WorldPay',$payment_row['cc_processor']);
        $this->assertEquals('PENDING',$payment_row['notes']);
        $this->assertTrue($payment_row['cc_transaction_id'] > 0);

        // order will be in an open state but messages will all be set to 'X'
        $this->assertEquals('O',$order_resource->status,"It shoudl be in an open state");
        $order_message = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'E');
        $this->assertEquals('X',$order_message->locked,"It should have locked as 'X' for all order message");
        $id = $payment_row['id'];

        $_SERVER['WORLD_PAY_RESPONSE'] = 'received';
        $activity_history_adapter = new ActivityHistoryAdapter(getM());
        $activity_history_resource = $activity_history_adapter->createActivityReturnActivityResource('P2PStatusCheck', time()-2, null,null,300);
        $activity = $activity_history_adapter->getActivityFromUnlockedActivityHistoryResource($activity_history_resource);
        $activity->executeThisActivity();

        $payment_resource = Resource::find(new BalanceChangeAdapter(getM()),$id);
        $this->assertEquals('PENDING',$payment_resource->notes);
        $order_message = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($payment_row['order_id'],'E');
        $this->assertEquals('X',$order_message->locked,"It should have locked as 'X' for all order message");

        $new_created = getMySqlFormattedDateTimeFromTimeStampAndTimeZone(  time() - 7200);
        $sql = "UPDATE Balance_Change SET `created` = '$new_created' WHERE `id` = $id";
        $bca = new BalanceChangeAdapter(getM());
        $bca->_query($sql);
        $payment_resource = Resource::find(new BalanceChangeAdapter(getM()),$id);

        $activity_history_adapter = new ActivityHistoryAdapter(getM());
        $activity_history_resource = $activity_history_adapter->createActivityReturnActivityResource('P2PStatusCheck', time()-2, null,null,300);
        $activity = $activity_history_adapter->getActivityFromUnlockedActivityHistoryResource($activity_history_resource);
        $activity->executeThisActivity();

        $payment_resource = Resource::find(new BalanceChangeAdapter(getM()),$id);
        $this->assertEquals('EXPIRED',$payment_resource->notes);
        $order_message = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($payment_row['order_id'],'E');
        $this->assertEquals('C',$order_message->locked,"It should have locked as 'C' for all order message");

    }

//    /**
//     * @depends testEndToEndCharge
//     */
//    function testCallBackFromWorldPay($transaction_id)
//    {
//
//        $data = ['purchasecode_id'=>$transaction_id,'success'=>1];
//        $url = "http://127.0.0.1:".$this->api_port."/app2/apiv2/payments";
//
////        $user = logTestUserIn(1);
////        $request = createRequestObject($url,'POST',json_encode($data));
////        $order_controller = new OrderController(getM(),$user,$request);
////        $response = makeMockRequest($order_controller);
//        $external_id = getPublicClientIdForContext();
//        $headers = array("X_ORDER140_CLIENT_ID"=>"$external_id","X_ORDER140_CLIENT_DEVICE"=>"unit_testing","HTTP_X_ORDER140_CLIENT_VERSION"=>"10.5.0","X_ORDER140_CLIENT"=>"APIDispatchTest","NO_CC_CALL"=>"true");
//        $header_array = array();
//        foreach ($headers as $key=>$value) {
//            $header_array[] = $key.":".$value;
//        }
//        $response = $this->makeRequest($url,"admin:Test1234",'POST',$header_array,$data);
//        $rar = json_decode($response,true);
//        $this->assertEquals(422,$rar['http_code']);
//        $this->assertContains(OrderController::UNABLE_TO_VALIDATE_P2P_TRANSACTION_ID_ERROR_MESSAGE,$rar['error']['error']);
//
//        clearAuthenticatedUserParametersForSession();
//        removeContext();
//
//        $world_pay_skin = $this->ids['worldpay_skin_resource'];
//        $wpi = $world_pay_skin->external_identifier;
//        setContext($wpi);
//
////        $user = logTestUserIn(1);
////        $request = createRequestObject($url,'POST',json_encode($data));
////        $order_controller = new OrderController(getM(),$user,$request);
////        $response = makeMockRequest($order_controller);
//        $external_id = getPublicClientIdForContext();
//        $headers = array("X_ORDER140_CLIENT_ID"=>"$external_id","X_ORDER140_CLIENT_DEVICE"=>"unit_testing","HTTP_X_ORDER140_CLIENT_VERSION"=>"10.5.0","X_ORDER140_CLIENT"=>"APIDispatchTest","NO_CC_CALL"=>"true");
//        $header_array = array();
//        foreach ($headers as $key=>$value) {
//            $header_array[] = $key.":".$value;
//        }
//        $response = $this->makeRequest($url,"admin:Test1234",'POST',$header_array,$data);
//
//        $rar = json_decode($response,true);
//        $this->assertEquals(200,$rar['http_code']);
//        $this->assertEquals('success',$rar['data']['result']);
//
//        $response_stamp = $rar['stamp'];
//        // now confirm payment row was updated
//        $ucid = $rar['data']['order_id'];
//        $order = new Order($ucid);
//        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order->getOrderId()],'BalanceChangeAdapter');
//        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
//        $payment_row = $bcr_hash['P2P'];
//        $this->assertEquals('WorldPay',$payment_row['cc_processor']);
//        $this->assertEquals("COMPLETED-$response_stamp",$payment_row['notes']);
//
//        // confirm that message were released
//        $order_message = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order->getOrderId(),'E');
//        $this->assertEquals('N',$order_message->locked,"It should have un-locked the message");
//
//    }

    function makeRequest($url,$userpassword,$method = 'GET', $headers = array(),$data = null)
    {
        unset($this->info);
        $method = strtoupper($method);
        $curl = curl_init($url);
        if ($userpassword) {
            curl_setopt($curl, CURLOPT_USERPWD, $userpassword);
        }
        if ($method == 'POST') {
            $json = json_encode($data);
            curl_setopt($curl, CURLOPT_POST, 1);
            curl_setopt($curl, CURLOPT_POSTFIELDS,$json);
            $headers[] = 'Content-Type: application/json';
            $headers[] = 'Content-Length: ' . strlen($json);
        } else if ($method == 'DELETE') {
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "DELETE");
        }
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        logCurl($url,$method,$userpassword,$headers,$json);
        $result = curl_exec($curl);
        $this->info = curl_getinfo($curl);
        curl_close($curl);
        return $result;
    }


    static function setUpBeforeClass()
    {
        $code = setNewStamp();
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time',300);
        SplickitCache::flushAll();

        $sql = 'DELETE FROM Balance_Change';
        $bca = new BalanceChangeAdapter(getM());
        $bca->_query($sql);
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction();




        $worldpay_skin_resource = getOrCreateSkinAndBrandIfNecessaryWithLoyalty("WorldPay","WorldPay",null, null);
        $worldpay_skin_resource->password = Encrypter::Encrypt("Test1234");
        $worldpay_skin_resource->save();
        $ids['worldpay_skin_resource'] = $worldpay_skin_resource;
        $skin_resource = getOrCreateSkinAndBrandIfNecessaryWithLoyalty("maryjane","maryjane",null, null);
        $skin_resource->password = Encrypter::Encrypt("Test1234");
        $skin_resource->save();
        $ids['skin_id'] = $skin_resource->skin_id;
        setContext('com.splickit.maryjane');

        $simple_menu_id = createTestMenuWithOneItem("item_one");
        $ids['menu_id'] = $simple_menu_id;

        $merchant_resource = createNewTestMerchant($simple_menu_id,['no_payment'=>true]);
        $merchant_resource->save();
        $ids['merchant_id'] = $merchant_resource->merchant_id;

        $merchant_payment_type_map_resource = MerchantPaymentTypeMapsAdapter::createMerchantPaymentTypeMap($merchant_resource->merchant_id, 13010, null);

        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $parameters = $tpppa->getParametersFromProviderId(7001);
        foreach ($parameters as $parameter_row) {
            $data = [];
            $data['third_party_provider_parameter_id'] = $parameter_row['third_party_provider_parameter_id'];
            if ($parameter_row['name'] == 'worldpay_merchant_id') {
                $data['value'] = 'world12345';
                $data['merchant_id'] = $merchant_resource->merchant_id;
                $tpppmma = new ThirdPartyProviderParameterMerchantMapsAdapter(getM());
                Resource::createByData($tpppmma,$data);
            }
        }

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    WorldPayTest::main();
}
?>