<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class XoikosTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $code = generateCode(7);
        $_SERVER['STAMP'] = __CLASS__ . '-' . $code;
        $_SERVER['RAW_STAMP'] = $code;
        $this->ids = $_SERVER['unit_test_ids'];
        setContext('com.splickit.goodcentssubs');
    }

    function tearDown()
    {
        //delete your instance
        unset($this->ids);
        unset($_SERVER['max_lead']);
    }

    function testGetActiveStausByStore()
    {

        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = getOrCreateNewTestMerchantBasedOnExternalId('22222',$menu_id,['no_payment'=>true]);
        $merchant_resource->ordering_on = 'Y';
        $merchant_resource->save();
        $merchant_id = $merchant_resource->merchant_id;
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_resource->merchant_id,"message_format"=>'X',"delivery_addr"=>"Xoikos","message_type"=>"X"));

        $xoikos_service = new XoikosService(["merchant"=>$merchant_resource->getDataFieldsReally()]);
        $this->assertTrue($xoikos_service->testStoreIsActive());

        unset_server_session_data();

        $url = "http://127.0.0.1:80/app2/apiv2/merchants/$merchant_id";
        $request = createRequestObject($url,'GET');
        $merchant_controller = new MerchantController(getM(),null, $request);
        $response_resource = $merchant_controller->processV2Request();
        $result_as_array = $response_resource->getDataFieldsReally();
        $this->assertNotEquals(PlaceOrderController::ORDERING_OFFLINE_MESSAGE,$result_as_array['user_message']);

        unset_server_session_data();

        $bad_external = '88888';
        $xoikos_service->merchant['merchant_external_id'] = "".$bad_external;
        $this->assertFalse($xoikos_service->testStoreIsActive());

        unset_server_session_data();

        $merchant_resource->merchant_external_id = $bad_external;
        $merchant_resource->save();

        $request = createRequestObject($url,'GET');
        $merchant_controller = new MerchantController(getM(),null, $request);
        $response_resource = $merchant_controller->processV2Request();

        $result_as_array = $response_resource->getDataFieldsReally();
        $this->assertEquals("Alert!",$result_as_array['user_message_title']);
        $this->assertEquals(PlaceOrderController::ORDERING_OFFLINE_MESSAGE,$result_as_array['user_message']);

        unset_server_session_data();

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertEquals(PlaceOrderController::STORE_DROPPED_OFFLINE_MESSAGE,$checkout_resource->error);
        return $merchant_resource;
    }

    /**
     * @depends testGetActiveStausByStore
     */
    function testAddToCart($merchant_resource)
    {
        unset_server_session_data();

        $merchant_resource->ordering_on = 'Y';
        $merchant_resource->merchant_external_id = '88888';
        $merchant_resource->save();
        $merchant_id = $merchant_resource->merchant_id;

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id, 'pickup', 'the note', 1);
        $cart_resource = getCartResourceFromOrderData($cart_data);
        $this->assertNull($cart_resource->error);

        $checkout_resource = getCheckoutResourceFromCartUcid($cart_resource->ucid,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNotNull($checkout_resource->error);

        $merchant_resource->merchant_external_id = '22222';
        $merchant_resource->save();

        unset_server_session_data();

        $checkout_resource = getCheckoutResourceFromCartUcid($cart_resource->ucid,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);


    }


    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        ini_set('max_execution_time', 0);
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $skin_id = 140;
        $brand_id = 430;
        $skin_resource = getOrCreateSkinAndBrandIfNecessary("Goodcents Subs","Goodcents Subs",$skin_id,$brand_id);
        $brand_resource = Resource::find(new BrandAdapter(getM()),$brand_id,null);
        $brand_resource->support_email = 'xoikos_support@email.com';
        $brand_resource->save();
        setContext('com.splickit.goodcentssubs');
        $mysqli->begin_transaction(); ;

        $ids['skin_id'] = $skin_resource->skin_id;
        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main()
    {
        $suite = new PHPUnit_Framework_TestSuite(__CLASS__);
        PHPUnit_TextUI_TestRunner::run($suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    XoikosTest::main();
}

?>