<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class ZeroTaxTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $code = generateCode(7);
        $_SERVER['STAMP'] = __CLASS__ . '-' . $code;
        $_SERVER['RAW_STAMP'] = $code;
        $this->ids = $_SERVER['unit_test_ids'];
    }

    function tearDown()
    {
        //delete your instance
        unset($this->ids);
        unset($_SERVER['max_lead']);
    }

    function testChargeTaxForWorldHq()
    {
        setContext("com.splickit.worldhq");
        $merchant_id = $this->ids['merchant_id'];

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','some note',3);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);

        $this->assertTrue($checkout_resource->total_tax_amt > 0 ,"It shoudl have a non zero tax amount");
    }

    function testDoNotChargTaxForZeroTaxSkin()
    {
        setContext("com.splickit.zerotaxskin");
        $merchant_id = $this->ids['merchant_id'];

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','some note',3);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);

        $this->assertTrue($checkout_resource->total_tax_amt == 0 ,"It shoudl have a zero tax amount but it had a tax amount of: ".$checkout_resource->total_tax_amt);

        $order_resource = placeOrderFromCheckoutResource($checkout_resource);
        $this->assertNull($order_resource->error);

        $this->assertTrue($order_resource->total_tax_amt == 0 ,"It shoudl have a zero tax amount but it had a tax amount of: ".$checkout_resource->total_tax_amt);
        $this->assertEquals($order_resource->grand_total,$order_resource->order_amt,'It should have the amounts the same since no tax');
        $this->assertEquals($order_resource->grand_total,$order_resource->grand_total_to_merchant,'It should have the amounts the same since no tax');
    }

    static function setUpBeforeClass()
    {
        $code = setNewStamp();

        ini_set('max_execution_time', 0);
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));

        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction(); ;


        $skin_resource = getOrCreateSkinAndBrandIfNecessary("zerotaxskin","zerotaxbrand",null,null);
        $ids['skin_id'] = $skin_resource->skin_id;

        //map it to a menu
        $menu_id = createTestMenuWithNnumberOfItems(3);
        $ids['menu_id'] = $menu_id;

        $merchant_resource = createNewTestMerchant($menu_id);
        $resource = attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id'],1);
        attachMerchantToSkin($merchant_resource->merchant_id, 250);

        $ids['merchant_id'] = $merchant_resource->merchant_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main()
    {
        $suite = new PHPUnit_Framework_TestSuite(__CLASS__);
        PHPUnit_TextUI_TestRunner::run($suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    ZeroTaxTest::main();
}

?>