<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class MicrosControllerTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
		
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }

    function testCreateMicrosControllerFromFactory()
    {
        $message_resource = Resource::dummyfactory(array("message_format" => 'M'));
        $controller_name = ControllerFactory::getControllerNameFromMessageResource($message_resource);
        $this->assertEquals('Micros',$controller_name,"It should return Micros as the controller name");
        $controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $this->assertEquals('MicrosController', get_class($controller), "It should return a MicrosController");
    }

    function testCreateMicrosMessageOrder()
    {
        $user_resource = createNewUserWithCC();
        $user = logTestUserResourceIn($user_resource);
        $order_data = OrderAdapter::staticGetSimpleOrderArrayByMerchantId($this->ids['merchant_id'],'pickup','the note');
        $order_data['tip'] = 0.00;
        $time_stamp = getTimeStampForDateTimeAndTimeZone(12, 0, 0, 2, 14, 2014 , "America/Baltimore");
        $order_response = placeOrderFromOrderData($order_data,$time_stamp);
        $this->assertNull($order_response->error);
        $this->assertTrue($order_response->order_id > 1000);
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_response->order_id,$m);
        return $order_response->order_id;
    }

    /**
     * @depends testCreateMicrosMessageOrder
     */
    function testMicrosSymphonyMessageCreated($order_id)
    {
        $messages = MerchantMessageHistoryAdapter::getAllOrderMessages($order_id);
        $messages_hash = createHashmapFromArrayOfResourcesByFieldName($messages,'message_format');
        $this->assertNotNull($messages_hash['MS'],"SHould have created a message with the micros format");
        return $messages_hash['MS'];
    }

    /**
     * @depends testMicrosSymphonyMessageCreated
     */
    function testGetMicrosSymphonyTemplate($message_resource)
    {
        $micros_controller = new MicrosController($m,$u,$r,5);
        $template_file_path = $micros_controller->getRepresentationFromMessageFormatOfMessageResource($message_resource);
        $this->assertEquals('/order_templates/micros/execute_order_micros_symphony.xml',$template_file_path);
        $message_resource->_representation = $template_file_path;
        $representation = $message_resource->loadRepresentation(new FileAdapter($this->mimetypes, 'resources'));
        $this->assertTrue(is_a($representation,'Resource'),"should have created a representation");
    }

    /**
     * @depends testMicrosSymphonyMessageCreated
     */
    function testMicrosControllerGenerated($message_resource)
    {
        $micros_message_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $this->assertTrue(is_a($micros_message_controller,'MicrosController'),"Should have created a micros message controller");
        return $micros_message_controller;
    }

    /**
     * @depends testMicrosControllerGenerated
     */
    function testFormatMessageForMicros($micros_message_controller)
    {
        //$this->assertTrue(false);
        $message_resource = $micros_message_controller->prepMessageForSending($micros_message_controller->message_resource);
        $message_text = $message_resource->message_text;
        $this->assertContains("http://htng.org/PWS/2008B/FoodBevOrdering",$message_text,"should contain the food and bev xmlns reference");
        $expected_string = cleanUpXML(file_get_contents("./unit_tests/resources/expected_hmshost_demo_micros.txt"));
        $expected_string = str_replace("  ",'', $expected_string);
        $actual_string = cleanUpXML($message_text);
        $actual_string = str_replace("  ",'', $actual_string);
        $this->assertEquals($expected_string, $actual_string,"Foramtted message does not equal the expected. Check template for errors");
    }

    /**
     * @depends testMicrosControllerGenerated
     */
    function testSendMessageForMicros($micros_message_controller)
    {
        $this->assertTrue(false,"currently not set up");
        //$result = $micros_message_controller->sendThisMessage($micros_message_controller->message_resource);
        //  $this->assertTrue($result,"should have successfully sent the message");
    }

    /**
     * @depends testCreateMicrosMessageOrder
     */
    function testMicros3200MessageCreated($order_id)
    {
        $messages = MerchantMessageHistoryAdapter::getAllOrderMessages($order_id);
        $messages_hash = createHashmapFromArrayOfResourcesByFieldName($messages,'message_format');
        $this->assertNotNull($messages_hash['MT'],"SHould have created a message with the micros 3200 format");
        return $messages_hash['MT'];
    }

    /**
     * @depends testMicros3200MessageCreated
     */
    function testGetMicros3200Template($message_resource)
    {
        $micros_controller = new MicrosController($m,$u,$r,5);
        $template_file_path = $micros_controller->getRepresentationFromMessageFormatOfMessageResource($message_resource);
        $this->assertEquals('/order_templates/micros/execute_order_micros_3200.xml',$template_file_path);
        $message_resource->_representation = $template_file_path;
        $representation = $message_resource->loadRepresentation(new FileAdapter($this->mimetypes, 'resources'));
        $this->assertTrue(is_a($representation,'Resource'),"should have created a representation");
    }

    /**
     * @depends testMicros3200MessageCreated
     */
    function testFormatMessageForMicros3200($message_resource)
    {
        $micros_message_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $this->assertTrue(is_a($micros_message_controller,'MicrosController'),"Should have created a micros message controller");
        $message_resource = $micros_message_controller->prepMessageForSending($micros_message_controller->message_resource);
        $message_text = $message_resource->message_text;
        //$expected_message_text = '<POSREQUEST><CHECKREQUESTS><ADDCHECK TABLENUMBER="" EXTCHECKID="SPLICK-IT-'.$message_resource->order_id.'" READYTIME="" READYDATETIME="" NOTE="'.$message_resource->note.'" ORDERMODEFEE="" ORDERID="" PRICEONLY="" ORDERMODE="2">';
        //$expected_message_text = '<POSREQUEST><CHECKREQUESTS><ADDCHECK TABLENUMBER="" EXTCHECKID="SPLICK-IT-'.$message_resource->order_id.'" READYTIME="" READYDATETIME="" NOTE="'.$message_resource->note.'" ORDERMODEFEE="" ORDERID="" PRICEONLY="" ORDERMODE="2">';
        $expected_message_text = '<POSREQUEST><CHECKREQUESTS><ADDCHECK TABLENUMBER="" EXTCHECKID="SPLICK-IT" READYTIME="" READYDATETIME="" NOTE="'.$message_resource->note.'" ORDERMODEFEE="" ORDERID="" PRICEONLY="" ORDERMODE="2">';
        $this->assertContains($expected_message_text,cleanUpDoubleSpacesCRLFTFromString($message_text),"should contain the top level stuff");
        /*$expected_string = cleanUpXML(file_get_contents("./unit_tests/resources/expected_hmshost_demo_micros.txt"));
        $expected_string = str_replace("  ",'', $expected_string);
        $actual_string = cleanUpXML($message_text);
        $actual_string = str_replace("  ",'', $actual_string);
        $this->assertEquals($expected_string, $actual_string,"Foramtted message does not equal the expected. Check template for errors");*/
        return $micros_message_controller;
    }

    /**
     * @depends testFormatMessageForMicros3200
     */
    function testSendMessageForMicros3200($micros_message_controller)
    {
        $result = $micros_message_controller->sendThisMessage($micros_message_controller->message_resource);
        $this->assertTrue($result,"should have successfully sent the message");
        $message_id = $micros_message_controller->message_resource->map_id;
        $mmh_record = MerchantMessageHistoryAdapter::staticGetRecordByPrimaryKey($message_id,'MerchantMessageHistoryAdapter');
        $message_text = $mmh_record['message_text'];
        $this->assertNotNull($message_text,"XML message should have been saved");
        $response = $mmh_record['response'];
        $this->assertNotNull($response,"Micros POS response should have been saved");
    }

    function testUnhappyPathForMicros3200()
    {
        //an error still returns a 200 for so we need to write good tests for this at some point
        $this->assertTrue(false,"Need unhappy path tests for micros");
    }



    static function setUpBeforeClass()
    {
        $menu_id = 102614;
        $brand_id = 307;
        $menu_adapter = new MenuAdapter($mimetypes);
        if ($menu_adapter->getRecord(array("menu_id"=>$menu_id))){
            ; // do nothing
        } else {
            //create menu
            // why do i have to do this twice. please help :(
            $menu_adapter->importMenu($menu_id,'prod','local',$brand_id);
            $menu_adapter->importMenu($menu_id,'prod','local',$brand_id);
        }
        $_SERVER['log_level'] = 5;
        mysqli_query("BEGIN");
        $_SERVER['request_time1'] = microtime(true);
        $skin_id = 118;
        $name = "hmshost";
        $skin_resource = getOrCreateSkinAndBrandIfNecessaryWithLoyalty($name, $name, $skin_id, $brand_id);
        $ids['context'] = "com.splickit.$name";
        $ids['brand_id'] = $brand_id;
        $ids['skin_id'] = $skin_id;

        $ids['menu_id'] = $menu_id;

        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_resource->brand_id = $brand_id;
        $merchant_resource->save();

        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'MS','micros','X');
        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'MT','micros','O');
        $ids['merchant_id'] = $merchant_resource->merchant_id;

        setContext($ids['context']);
        $user_resource = createNewUser(array('flags'=>'1C20000001'));
        $ids['user_id'] = $user_resource->user_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;    }
    
	static function tearDownAfterClass()
    {
    	mysqli_query("ROLLBACK");
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    MicrosControllerTest::main();
}

?>