<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class YourPieTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
        $this->ids = $_SERVER['unit_test_ids'];
        setContext('com.splickit.yourpie');

    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }

    function testMenuExists()
    {
        $menu_resource = Resource::find(new MenuAdapter($mimetypes), "".$this->ids['menu_id'], $options);
        $this->assertNotNull($menu_resource);
    }

    private function createOrder($user,$merchant_id)
    {
        $menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$merchant_id);
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getOrderArrayFromFullMenuWithOneModiferPerModifierGroup($menu,$merchant_id,"some note",1,0,1);
        $request = createRequestObject("app2/apiv2/cart/checkout","POST",json_encode($order_data),'application/json');
        $place_order_controller = new PlaceOrderController($m,$user,$request,5);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($checkout_resource->error,"It should not throw an error");
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$this->ids['merchant_id'],0.00,$time);
        return $order_resource;
    }

    function testFavorites()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $merchant_id = $this->ids['merchant_id'];
        $menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$merchant_id,2,'pickup');
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getOrderArrayFromFullMenuWithOneModiferPerModifierGroup($menu,$merchant_id,"some note",1,0,5);
        $this->assertCount(5,$order_data['items'][0]['mods'],"there shuold be 5 mods");
        $request = createRequestObject("app2/apiv2/cart/checkout","POST",json_encode($order_data),'application/json');
        $place_order_controller = new PlaceOrderController($m,$user,$request,5);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($checkout_resource->error,"It should not throw an error");
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$this->ids['merchant_id'],0.00,$time);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);

        $request = new Request();
        $request->url = "http://localhost/app2/apiv2/favorites?log_level=5";
        $request->body = json_encode(array("order_id"=>$order_id,"favorite_name"=>'sumdumname'));
        $request->method = 'post';
        $request->mimetype = 'application/json';

        $favorite_controller = new FavoriteController($m,$user,$request,5);
        $favorite = $favorite_controller->processV2Request();
        //$favorite = $favorite_controller->saveFavoriteFromOrderId($order_id,'sumdumname');
        $this->assertNotNull($favorite->favorite_id,"should have gotten back a resource with the favorite id");
        $this->assertEquals("Your favorite was successfully stored.",$favorite->user_message);

        setProperty('use_merchant_caching',"false");
        $user_id = $user['user_id'];
        $user = logTestUserIn($user_id);
        $request = new Request();
        $request->url = "/app2/apiv2/merchants/$merchant_id?log_level=5";
        $request->method = 'GET';
        $merchant_controller = new MerchantController($mt, $user, $request, 5);
        $resource = $merchant_controller->processV2Request();
        $this->assertNotNull($resource->user_favorites,"Should have found favorites section");
        $this->assertCount(1,$resource->user_favorites,"Should have found one record for this user menu combination");
        $favorite_hash = createHashmapFromArrayOfArraysByFieldName($resource->user_favorites,'favorite_name');
        $favorite_one = $favorite_hash['sumdumname'];

    }

    static function setUpBeforeClass()
    {
        $brand_id = 333;
        $skin_id = 80;
        getOrCreateSkinAndBrandIfNecessary('yourpie','yourpie',$skin_id,$brand_id);
        $menu_id = 102430;
        $ids['menu_id'] = $menu_id;
        $menu_adapter = new MenuAdapter($mimetypes);
        if ($menu_adapter->getRecord(array("menu_id"=>$menu_id))){
            ; // do nothing
        } else {
            //create menu
            $menu_adapter->importMenu($menu_id,'prod','local');
            $menu_resource = Resource::find($menu_adapter,"$menu_id",$options);
            $menu_resource->version = 2.00;
            $menu_resource->save();
        }
        setContext('com.splickit.yourpie');
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        mysqli_query("BEGIN");
        $merchant_resource = createNewTestMerchant($menu_id);
        $ids['merchant_id'] = $merchant_resource->merchant_id;

        //$user_resource = createNewUser(array("flags"=>"1C20000001"));
        //$ids['user_id'] = $user_resource->user_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        mysqli_query("ROLLBACK");
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    YourPieTest::main();
}

?>