<?php
error_reporting(E_ERROR | E_COMPILE_ERROR | E_COMPILE_WARNING | E_PARSE);
$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

$_SERVER['ENVIRONMENT'] = isset($_SERVER['XDEBUG_CONFIG']) ? "unit_test_ide" : "unit_test";
$_SERVER['DB_INFO_FILE_PATH'] = "config/smaw_database_unittest.conf";

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';


class BibibopBrinkIntegrationTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;
    var $api_port;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
        $this->ids = $_SERVER['unit_test_ids'];
        if (isset($_SERVER['XDEBUG_CONFIG'])) {
            $this->api_port = "10080";
        }

    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }


//    function testGetDestinations()
//    {
//        $brink_importer = new BrinkImporter();
//        $method = "GetDestinations";
//        $response0 = $brink_importer->curlIt($method,$merchant_resource);
//        myerror_log($response0);
//    }



    function testStageAllImport()
    {
        setContext("com.splickit.bibibop");
        $merchant_resource = createNewTestMerchant($this->ids['menu_id']);
        $merchant_resource->merchant_external_id = generateCode(10);
        $merchant_resource->save();

        $merchant_resource2 = createNewTestMerchant($this->ids['menu_id']);
        $merchant_resource2->merchant_external_id = generateCode(10);
        $merchant_resource2->save();

        $request = createRequestObject("/apiv2/pos/import/brinkbibibop/all",'POST');
        $pos_controller = new PosController(getM(),null,$request,5);
        $resource = $pos_controller->processV2request();
        $this->assertContains("The imports have been staged. there were",$resource->message);
    }

    function testStageImport()
    {
        setContext("com.splickit.bibibop");
        $external_id = $this->ids['merchant_resource']->merchant_external_id;
        $request = createRequestObject("/apiv2/pos/import/brinkbibibop/$external_id",'POST');
        $pos_controller = new PosController(getM(),null,$request,5);
        $resource = $pos_controller->processV2request();
        $this->assertEquals("The import has been staged",$resource->message);
        return $pos_controller->getId();
    }

    /**
     * @depends testStageImport
     */
    function testActivityImport($activity_id)
    {
        removeContext();
        $external_id = $this->ids['merchant_resource']->merchant_external_id;
        $activity_resource = Resource::find(new ActivityHistoryAdapter(getM()),"$activity_id",null);

        $this->assertNotNull($activity_resource);
        $this->assertEquals("object=BrinkbibibopImporter;method=import;thefunctiondatastring=$external_id",$activity_resource->info);
        $activity_resource->locked = 'Y';
        $activity_resource->save();
        $activity = SplickitActivity::getActivity($activity_resource);
        $result = $activity->doit();

        $menu_id = $this->ids['menu_id'];
        $complete_menu = new CompleteMenu($menu_id);
        $item_sizes = $complete_menu->getAllMenuItemSizePrices($menu_id,'Y',$this->ids['merchant_id'],getM());
        $count = sizeof($item_sizes);
        $this->assertTrue($count>10,"It shoudl have imported a bunch of prices for the merchant but it only imported: $count");

    }


//    function testLoadPricesForMerchant()
//    {
//        $ids = $this->ids;
//        $menu_id = $ids['menu_id'];
//        $external_id = $this->ids['merchant_resource']->merchant_external_id;
//
//        $brink_importer = ImporterFactory::getImporterFromUrl("http://localhost/app2/apiv2/pos/import/brinkbibibop/$external_id");
//        $brink_importer->create_zero_records = false;
//        $this->assertTrue($brink_importer->importRemoteMerchantMetaDataForLoadedMerchant());
//
//        //$menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$this->ids['merchant_id'],2,'pickup');
//        $complete_menu = new CompleteMenu($menu_id);
//        $item_sizes = $complete_menu->getAllMenuItemSizePrices($menu_id,'Y',$ids['merchant_id'],getM());
//        $count = sizeof($item_sizes);
//        $this->assertTrue($count>10,"It shoudl have imported a bunch of prices for the merchant but it only imported: $count");
//
//        $mi = CompleteMenu::getAllMenuItemsAsArray($menu_id);
//    }

    function testBrinkBiBiBopImportThroughPOSEndpoint()
    {
        setContext('com.splickit.bibibop');
        $external_id = $this->ids['merchant_resource']->merchant_external_id;
        $url = "http://127.0.0.1:".$this->api_port."/app2/pos/import/brinkbibibop/$external_id";
        $response = makeUnitTestRequest($url,'admin:welcome','POST');
        $response_array = json_decode($response,true);
        $this->assertContains('The import has been completed.',$response_array['data']['message']);
    }

    function testPlaceOrderBowlWithChicken()
    {

        $size_id = 94887;
        $item_id = 303317;
        
        $chicken_modifier_item_id = 2353022;


        $merchant_id = $this->ids['merchant_id'];
        $user_resource = createNewUserWithCCNoCVV();

        $user_resource->balance = 100;
        $user_resource->save();

        $user = logTestUserResourceIn($user_resource);
        $usersession_controller = new UsersessionController(getM(),$user,$request);
        $usersession_controller->getUserSession($user_resource);

        $order_adapter = new OrderAdapter(getM());
        $order_data = $order_adapter->getCartArrayWithOneModierPerModifierGroup($merchant_id,'pickup',1);

//        $order_data['items'][0]['item_id'] = $item_id;
//        $order_data['items'][0]['size_id'] = $size_id;
//        //$order_data['items'][0] = array('mods'=>array(),'quantity'=>1,'item_id'=>$item_id,'size_id'=>$size_id,'item_total_price'=>2.0);
        $order_data['items'][0] = array('mods'=>array(array('modifier_item_id'=>$chicken_modifier_item_id,'mod_quantity'=>1),array('modifier_item_id'=>2352979,'mod_quantity'=>1)),'quantity'=>1,'item_id'=>$item_id,'size_id'=>$size_id,'item_total_price'=>6.75);
        $request = createRequestObject('app2/apiv2/cart','post',json_encode($order_data),'application/json');
        $place_order_controller = new PlaceOrderController(getM(),$user,$request,5);
        $create_cart_response = $place_order_controller->processV2Request();
        $this->assertNull($create_cart_response->error);
        $ucid = $create_cart_response->ucid;

        $brink_controller = new BrinkController(getM(),$u,$r,5);
        $order_id = $create_cart_response->oid_test_only;
        $calculate_order_message_resource = $brink_controller->prepMessageForSending(Resource::dummyfactory(array("order_id"=>$order_id,"message_format"=>'BC')));
        $body = $calculate_order_message_resource->message_text;
        myerror_log($body);

        $this->assertContains('<ItemId>206789</ItemId>',$body);



        $request = createRequestObject("app2/apiv2/cart/$ucid/checkout","GET");
        $place_order_controller = new PlaceOrderController(getM(),$user,$request,5);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($checkout_resource->error,"It should not throw an error");

        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,getM(),0.00,time());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $brink_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'B');
        $brink_message_controller = ControllerFactory::generateFromMessageResource($brink_message_resource);
        $ready_to_send_message_resource = $brink_message_controller->prepMessageForSending($brink_message_resource);
        $body2 = $ready_to_send_message_resource->message_text;
        myerror_log($body2);
        $this->assertContains('<ItemId>206789</ItemId>',$body2);
    }

    function testItShouldGetTheTaxFromBrinkOnGetCheckoutData()
    {
        $merchant_id = $this->ids['merchant_id'];
        $user_resource = createNewUserWithCCNoCVV(array("email"=>"bobnewhart@order140.com","first_name"=>"bob","last_name"=>"newhart","contact_no"=>'8888888888'));
        $user_resource->balance = 100;
        $user_resource->save();
        $user = logTestUserResourceIn($user_resource);
        $order_adapter = new OrderAdapter(getM());
        $menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$merchant_id,2,'pickup');
        $order_data = $order_adapter->getOrderArrayFromFullMenuWithOneModiferPerModifierGroup($menu,$merchant_id,"some note",1,0,1);
        $order_data['items'][0]['note'] = 'the big item note';
        $request = createRequestObject('app2/apiv2/cart','post',json_encode($order_data),'application/json');
        $place_order_controller = new PlaceOrderController($m,$user,$request,5);
        $create_cart_response = $place_order_controller->processV2Request();
        $ucid = $create_cart_response->ucid;

        $request = createRequestObject("app2/apiv2/cart/$ucid/checkout","GET");
        $place_order_controller = new PlaceOrderController($m,$user,$request,5);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($checkout_resource->error,"It should not throw an error");
        $checkout_receipt_items = createHashmapFromArrayOfArraysByFieldName($checkout_resource->order_summary['receipt_items'],'title');
//        $checkout_tax = $checkout_receipt_items['Tax'];
//        $this->assertEquals('$0.74',$checkout_tax['amount'],"Tax should have been set by the brink pull");
//        $total = $checkout_receipt_items['Total']['amount'];
//        $this->assertEquals("$8.13",$total,"total should have been set to the brink values");
////        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
//        $order_amt = floatval($checkout_resource->order_amt);
//        $total_tax_amt = floatval($checkout_resource->total_tax_amt);
//        $this->assertTrue($total_tax_amt > 0.00,"It should have sent back a valid total tax amount but sent back: ".$checkout_resource->total_tax_amt);
//        $tax_rate = intval(100*$total_tax_amt/$order_amt);
//        $this->assertNotEquals(20,$tax_rate,"tax rate should be calculated by brink, not the 20% that is in our system");

        //make sure totals are correct.
//        $complete_order = CompleteOrder::staticGetCompleteOrder($checkout_resource->oid_test_only,$m);
//        $grand_total_from_math = $complete_order['order_amt']+$complete_order['total_tax_amt']+$complete_order['delivery_amt']+$complete_order['tip_amt']+$complete_order['promo_amt'];
//        $this->assertEquals($grand_total_from_math,$complete_order['grand_total'],"the set grand total should equal the sum of the parts");

        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$m,0.00,time());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $bibibop_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'B');
        $this->assertNotNull($bibibop_message_resource);
        $brink_controller = ControllerFactory::generateFromMessageResource($bibibop_message_resource);
        $result = $brink_controller->sendThisMessage($bibibop_message_resource);

        $sent_bibibop_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'B');
        $this->assertEquals('S',$sent_bibibop_message_resource->locked);
        $response = $sent_bibibop_message_resource->response;
        $this->assertContains('0</ResultCode>',$response);
        $this->assertContains('<OrderId>',$response);


    }

    function makePOSRequest($url,$method = 'GET',$body = null,$headers = array())
    {
        unset($this->info);
        $method = strtoupper($method);
        $curl = curl_init($url);
        $client_id = getPublicClientIdForContext();
        $headers = array_merge($headers,array("SPLICKIT_CLIENT_ID:$client_id","X_SPLICKIT_CLIENT_DEVICE:unit_testing","X_SPLICKIT_CLIENT:POSDispatchTest","NO_CC_CALL:true"));
        if ($method == 'POST') {
            curl_setopt($curl, CURLOPT_POST, 1);
            curl_setopt($curl, CURLOPT_POSTFIELDS,$body);
            $headers[] = 'Content-Type: application/json';
            $headers[] = 'Content-Length: ' . strlen($body);
        }
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        logCurl($url,$method,$userpassword,$headers,$body);
        $result = curl_exec($curl);
        $this->info = curl_getinfo($curl);
        curl_close($curl);
        return $result;
    }

    static function setUpBeforeClass()
    {
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time',300);

        $menu_id = 102754;
        $brand_id = 437;
        $skin_id = 150;
        $skin_resource = getOrCreateSkinAndBrandIfNecessary("bibibop","bibibop",$skin_id,$brand_id);
        $brand_resource = Resource::find(new BrandAdapter(getM()),"$brand_id");
        $brand_resource->loyalty = 'Y';
        $brand_resource->save();

        $blr_data['brand_id'] = $brand_id;
        $blr_data['loyalty_type'] = 'splickit_earn';
        $blr_data['starting_point_value'] = 5000;
        $blr_data['earn_value_amount_multiplier'] = 100;
        $blr_data['charge_modifiers_loyalty_purchase'] = 1;
        $brand_loyalty_rules_resource = Resource::factory(new BrandLoyaltyRulesAdapter($m),$blr_data);
        $brand_loyalty_rules_resource->save();
        $ids['blr_resource'] = $brand_loyalty_rules_resource->getRefreshedResource();

        $menu_adapter = new MenuAdapter(getM());
        if ($menu_resource = Resource::find($menu_adapter,"$menu_id")){
//            $menu_resource->version = 2.0;
//            $menu_resource->save();
        } else {
            //create menu
            $menu_adapter->importMenu($menu_id,'prod','local',$brand_id);
//            $menu_resource = Resource::find($menu_adapter,"$menu_id");
//            $menu_resource->version = 2.0;
//            $menu_resource->save();
        }


        setContext('com.splickit.bibibop');
        $merchant_resource = getOrCreateNewTestMerchantBasedOnExternalId('Lab');
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_resource->merchant_id,"message_format"=>'B',"delivery_addr"=>"Brink","message_type"=>"X"));

//        $db = DataBase::getInstance();
//        $mysqli = $db->getConnection();
//        $mysqli->begin_transaction();

        Resource::createByData(new MerchantMenuMapAdapter($m),array("merchant_id"=>$merchant_resource->merchant_id,"menu_id"=>$menu_id));
        $ids['merchant_resource'] = $merchant_resource;
        $ids['merchant_id'] = $merchant_resource->merchant_id;
        Resource::createByData(new MerchantBrinkInfoMapsAdapter($m),array("merchant_id"=>$merchant_resource->merchant_id,"brink_location_token"=>"ppgH7PTz9kmac3W0xsp9MQ=="));

        $ids['menu_id'] = $menu_id;

        //$map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_resource->merchant_id,"message_format"=>'B',"delivery_addr"=>"Brink","message_type"=>"X"));

        $user_resource = createNewUser(array("flags"=>"1C20000001"));
        $ids['user_resource'] = $user_resource;
        $ids['user_id'] = $user_resource->user_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }


    static function tearDownAfterClass()
    {
//        $db = DataBase::getInstance();
//        $mysqli = $db->getConnection();
//        $mysqli->rollback();

        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    BibibopBrinkIntegrationTest::main();
}

?>
