<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';


class BrinkImporterTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
        $this->ids = $_SERVER['unit_test_ids'];

    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }

//    function testItshouldImportASecondMerchantIntoTheSameMenuOrNot()
//    {
//        $previous_menu_id = createTestMenuWithOneItem("The Item");
//        $merchant_resource = getOrCreateNewTestMerchantBasedOnExternalId('pitapit-11111');
//        $mmm_resource = MerchantMenuMapAdapter::createMerchantMenuMap($merchant_resource->merchant_id, $previous_menu_id, 'pickup');
//        $this->assertEquals($previous_menu_id,$mmm_resource->menu_id);
//        $mbima_resource = Resource::createByData(new MerchantBrinkInfoMapsAdapter($m),array("merchant_id"=>$merchant_resource->merchant_id,"brink_location_token"=>getProperty('brink_test_location_token')));
//
//        $activity_history_resource = ActivityHistoryAdapter::createActivityReturnActivityResource('ExecuteObjectFunction', time()-10, "object=BrinkImporter;method=import;thefunctiondatastring=pitapit-11111", $activity_text);
//        $splickit_activity = new ExecuteObjectFunctionActivity($activity_history_resource);
//        $splickit_activity->executeThisActivity();
//
//        $refeshed_activity_resource = $activity_history_resource->getRefreshedResource();
//        $this->assertEquals('E',$refeshed_activity_resource->locked,"Should have set the activity to executed");
//
//        $mmm_record = MerchantMenuMapAdapter::staticGetRecord(array("merchant_id"=>$merchant_resource->merchant_id,"merchant_menu_type"=>'pickup'),'MerchantMenuMapAdapter');
//        $this->assertEquals($this->ids['menu_id'],$mmm_record['menu_id']);
//    }

//    function testGetMenuWithoutCombosBecauseNoActivePrices()
//    {
//        $menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$this->ids['merchant_id'],2,'pickup');
//        $menu_type_hash = createHashmapFromArrayOfArraysByFieldName($menu['menu_types'],'menu_type_name');
//        $this->assertFalse(isset($menu_type_hash['Combos']),"It should not have combos");
//    }

    function testImporterFactory()
    {
        $url = 'http://localhost/app2/pos/import/brink/'.$this->ids['merchant_resource']->merchant_external_id.'?log_level=6';
        $importer = ImporterFactory::getImporterFromUrl($url);
        $this->assertTrue(is_a($importer,'BrinkImporter'),"should have gotten back a brink importer");
        $merchant_resource = $importer->getLoadedMerchantResource();
        $this->assertEquals($this->ids['merchant_id'],$merchant_resource->merchant_id);
    }

    function testImporterFactoryWithBrinkLocationId()
    {
        $brink_location_id = getProperty('brink_test_location_token');
        $url = "http://localhost/app2/pos/import/brink/$brink_location_id?log_level=6";
        $importer = ImporterFactory::getImporterFromUrl($url);
        $this->assertTrue(is_a($importer,'BrinkImporter'),"should have gotten back a brink importer");
        $merchant_resource = $importer->getLoadedMerchantResource();
        $this->assertEquals($this->ids['merchant_id'],$merchant_resource->merchant_id);
    }

    function testStageTheImportBadMerchantId()
    {
        $public_client_id = $this->ids['pita_pit_public_client_id'];
        $headers = array("SPLICKIT_CLIENT_ID:$public_client_id","X_SPLICKIT_CLIENT_DEVICE:unit_testing","X_SPLICKIT_CLIENT:".get_class($this),"NO_CC_CALL:true");
        $response = makeLocalhostRequest('http://localhost/app2/pos/import/brink/pitapit-1234567?log_level=6',$UP,'POST',$data,'text',$headers);
        $response_array = json_decode($response['result'],true);
        $this->assertEquals(422,$response_array['http_code']);
    }

    function testStageTheImportFromRemoteCall()
    {
        $public_client_id = $this->ids['pita_pit_public_client_id'];
        $headers = array("SPLICKIT_CLIENT_ID:$public_client_id");
        $response = makeLocalhostRequest('http://localhost/app2/pos/import/brink/pitapit-88888?log_level=6',$UP,'POST',$data,'text',$headers);
        $response_array = json_decode($response['result'],true);
        $this->assertEquals(200,$response_array['http_code']);
        $activity_id = $response_array['data']['activity_id'];
        $this->assertTrue($activity_id > 1000,"It should have created a valid activity id");
    }

    function testItShouldFailOnAMerchantThatIsNotABrinkMerchant()
    {
        setContext('com.splickit.pitapit');
        $curl = curl_init('http://localhost/app2/apiv2/importbrinkmerchant/pitapit/pitapit-99999');
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        $result = curl_exec($curl);
        $info = curl_getinfo($curl);
        curl_close($curl);
        $result_array = json_decode($result,true);
        $this->assertEquals(422,$result_array['http_code']);
        $this->assertEquals("Merchant does not appear to be a brink merchant",$result_array['error']['error']);
    }

    function testItShouldImportTheMenu()
    {
        $merchant_resource = $this->ids['merchant_resource'];
        $complete_menu = $this->ids['complete_menu'];
        $this->assertTrue($complete_menu['menu_id'] > 1000,"it should have created a valid menu id");
        return $complete_menu;
    }

    /**
     * @depends testItShouldImportTheMenu
     */
    function testItShouldNotContainIsDeletedMenuTypes($complete_menu)
    {
        $menu_type_hash_by_name = createHashmapFromArrayOfArraysByFieldName($complete_menu['menu_types'],'menu_type_name');
        $this->assertNull($menu_type_hash_by_name['Combos'],'It should not contain the menu type combos because IsDeleted is set to True in brink');
    }

    function testItShouldHaveCreatedTheDefaultPriceListMenu()
    {
        $complete_menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'N');
        $this->assertTrue(count($complete_menu['menu_types']) > 4," we should have the full menu for the zero merchant");
        $item = $complete_menu['menu_types'][0]['menu_items'][0];
        $this->assertNotNull($item['size_prices'],"there shoudl be default, zero record, sizes");
        $size_price_id = $item['size_prices'][0]['item_size_id'];
        $record = ItemSizeAdapter::staticGetRecordByPrimaryKey($size_price_id,'ItemSizeAdapter');
        $this->assertEquals(0,$record['merchant_id'],"should have found the zero record");
    }

    /**
     * @depends testItShouldImportTheMenu
     */
    function testItShouldHaveBuiltAValidMenuStructure($complete_menu)
    {
        $this->assertTrue(count($complete_menu['menu_types']) > 4, "there should be a few menu types");
        $menu_type = $complete_menu['menu_types'][0];
    }

    /**
     * @depends testItShouldImportTheMenu
     */
    function testItShouldHaveBuiltAValidMenuTypesStructure($complete_menu)
    {
        $menu_type = $complete_menu['menu_types'][0];
        $this->assertTrue(count($menu_type['menu_items']) > 4, "there should be a few items");
    }

    /**
     * @depends testItShouldImportTheMenu
     */
    function testItShouldHaveBuiltACorrectSizesStructure($complete_menu)
    {
        $menu_type = $complete_menu['menu_types'][0];
        $menu_type_id = $menu_type['menu_type_id'];
        $sizes = SizeAdapter::staticGetRecords(array("menu_type_id"=>$menu_type_id),'SizeAdapter');
        $sizes_by_name = createHashmapFromArrayOfArraysByFieldName($sizes,'size_name');
        $this->assertCount(2,$sizes_by_name,"There shoudl be 2 sizes for pitas");
        $this->assertTrue(isset($sizes_by_name['Regular Pita']),"It should have found a regular size");
        $this->assertTrue(isset($sizes_by_name['Small Pita']),"It should ahve found a small size");

        // now validate teh item has both sizes
        $this->assertCount(2,$complete_menu['menu_types'][0]['menu_items'][0]['size_prices'],"should have found both sizes on the item");
    }

    /**
     * @depends testItShouldImportTheMenu
     */
    function testItShouldHaveAValidStructureForAnItem($complete_menu)
    {
        $item = $complete_menu['menu_types'][0]['menu_items'][0];
        $this->assertTrue(strlen(trim($item['item_name'])) > 2,"should be a valid item name");
        $this->assertEquals($item['item_name'],$item['description'],'item name and description should be the same');
        $this->assertEquals($item['item_name'],$item['item_print_name'],'item name and item_print_name should be the same');
        $this->assertEquals('Y',$item['active']);
        $this->assertNotNull($item['modifier_groups']);
        $this->assertNotNull($item['size_prices']);
    }

    /**
     * @depends testItShouldImportTheMenu
     */
    function testItShouldHaveAValidModifierGroupStructureForAnItem($complete_menu)
    {
        $menu_id = $complete_menu['menu_id'];
        $complete_menu_object = new CompleteMenu($menu_id);
        $complete_menu = $complete_menu_object->getCompletePickupMenu($menu_id,'Y',$this->ids['merchant_id'],2);
        $modifier_groups = $complete_menu['menu_types'][0]['menu_items'][0]['modifier_groups'];
        $this->assertTrue(count($modifier_groups) > 1,"THere should be a few modifier groups for a pita item");
        $modifier_groups_by_name = createHashmapFromArrayOfArraysByFieldName($modifier_groups,'modifier_group_display_name');
        $extras = $modifier_groups_by_name['Extras'];
//        $this->assertEquals(0,$extras['modifier_group_min_modifier_count']);
//        $this->assertEquals(count($extras['modifier_items']),$extras['modifier_group_max_modifier_count'],"the max count shoudl equal the number of items in teh group for extras");

        $pita_choice = $modifier_groups_by_name['Pita Choice'];
        $this->assertEquals(1,$pita_choice['modifier_group_max_modifier_count'],"the max count should equal 1 for pita choice");
        $this->assertEquals(1,$pita_choice['modifier_group_min_modifier_count'],"the min count should equal 1 for pita choice");

        $modifier_items = $extras['modifier_items'];
        $this->assertCount(2,$modifier_items[0]['modifier_prices_by_item_size_id'],"there should be two sizes for this since there are two pita sizes");
    }

    /**
     * @depends testItShouldImportTheMenu
     */
    function testItShouldNotDuplicatePitaChoices($complete_menu)
    {
        $pita_choice_count = 0;
        $complete_menu = CompleteMenu::getCompleteMenu($complete_menu['menu_id'],'Y',0);
        $modifier_groups = $complete_menu['modifier_groups'];
        foreach ($modifier_groups as $mg) {
            if ($mg['modifier_group_name'] == 'Pita Choice') {
                $pita_choice_count++;
            }
        }
        $this->assertEquals(1,$pita_choice_count,"there should only be one modifier group called Pita Choice");
    }

    function testPromo()
    {
        $merchant_id = $this->ids['merchant_id'];

        $user_resource = createNewUserWithCCNoCVV(array("email"=>"bobnewhart@order140.com","first_name"=>"bob","last_name"=>"newhart","contact_no"=>'8888888888'));
        $user = logTestUserResourceIn($user_resource);
        $order_adapter = new OrderAdapter($m);
        $menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$merchant_id,2,'pickup');
        $order_data = $order_adapter->getOrderArrayFromFullMenuWithOneModiferPerModifierGroup($menu,$merchant_id,"some note",1,0,1);
        $order_data['tip'] = 1.00;
        $request = createRequestObject('app2/apiv2/cart','post',json_encode($order_data),'application/json');
        $place_order_controller = new PlaceOrderController($m,$user,$request,5);
        $create_cart_response = $place_order_controller->processV2Request();
        $complete_order = CompleteOrder::staticGetCompleteOrder($create_cart_response->oid_test_only,$m);
        $splickit_sub_total = $complete_order['order_details'][0]['item_total_w_mods'];
        $ucid = $create_cart_response->ucid;
        $request = createRequestObject("app2/apiv2/cart/$ucid/checkout?promo_code=type1promo","GET");
        $place_order_controller = new PlaceOrderController($m,$user,$request,5);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($checkout_resource->error,"It should not throw an error");
        $complete_order = CompleteOrder::staticGetCompleteOrder($create_cart_response->oid_test_only,$m);
        $this->assertEquals($splickit_sub_total,$checkout_resource->order_amt,"When promo is used, we need OUR subtotal");
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$this->ids['merchant_id'],0.00,$time);
        $this->assertNull($order_resource->error,"It should not throw an error on place order with promo");
        $this->assertEquals(-$this->ids['promo_amt'],$order_resource->promo_amt,"It should have the promo amount");

        $brink_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'B');
        $this->assertNotNull($brink_message_resource,"we shojdl have a message resource");
        $brink_controller = new BrinkController($m, $u, $r, 5);
        $ready_to_send_message_resource = $brink_controller->prepMessageForSending($brink_message_resource);
        myerror_log($ready_to_send_message_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);
        //check to see that promo is in the XML
        $expected_promo_node = cleanUpDoubleSpacesCRLFTFromString('<Discounts>
						<NewOrderDiscount>
							<Amount>'.number_format($this->ids['promo_amt'],2).'</Amount>
							<DiscountId>4009161</DiscountId>
							<Id>1</Id>
							<LoyaltyRewardId>0</LoyaltyRewardId>
							<Name i:nil="true"/>
							<OrderItemIds i:nil="true" xmlns:a="http://schemas.microsoft.com/2003/10/Serialization/Arrays"/>
							<Percent>0</Percent>
						</NewOrderDiscount>
					</Discounts>');
        $this->assertContains($expected_promo_node,$body,"It should have the promo node");
        $this->assertEquals($splickit_sub_total,$order_resource->order_amt,"When promo is used, we need OUR subtotal");

        //make sure totals are correct.
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id,$m);
        $grand_total_from_math = $complete_order['order_amt']+$complete_order['total_tax_amt']+$complete_order['delivery_amt']+$complete_order['tip_amt']+$complete_order['promo_amt'];
        $this->assertEquals($grand_total_from_math,$complete_order['grand_total'],"the set grand total should equal the sum of the parts");

    }
    
    function testItShouldGetTheTaxFromBrinkOnGetCheckoutData()
    {
        $merchant_id = $this->ids['merchant_id'];
        $user_resource = createNewUserWithCCNoCVV(array("email"=>"bobnewhart@order140.com","first_name"=>"bob","last_name"=>"newhart","contact_no"=>'8888888888'));
        $user = logTestUserResourceIn($user_resource);
        $order_adapter = new OrderAdapter($m);
        $menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$merchant_id,2,'pickup');
        $order_data = $order_adapter->getOrderArrayFromFullMenuWithOneModiferPerModifierGroup($menu,$merchant_id,"some note",1,0,1);
        $order_data['tip'] = 1.00;
        $request = createRequestObject('app2/apiv2/cart','post',json_encode($order_data),'application/json');
        $place_order_controller = new PlaceOrderController($m,$user,$request,5);
        $create_cart_response = $place_order_controller->processV2Request();
        $ucid = $create_cart_response->ucid;

        $request = createRequestObject("app2/apiv2/cart/$ucid/checkout","GET");
        $place_order_controller = new PlaceOrderController($m,$user,$request,5);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($checkout_resource->error,"It should not throw an error");
        $checkout_receipt_items = createHashmapFromArrayOfArraysByFieldName($checkout_resource->order_summary['receipt_items'],'title');
        $checkout_tax = $checkout_receipt_items['Tax'];
        $this->assertEquals('$0.74',$checkout_tax['amount'],"Tax should have been set by the brink pull");
        $total = $checkout_receipt_items['Total']['amount'];
        $this->assertEquals("$8.13",$total,"total should have been set to the brink values");
//        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
        $order_amt = floatval($checkout_resource->order_amt);
        $total_tax_amt = floatval($checkout_resource->total_tax_amt);
        $this->assertTrue($total_tax_amt > 0.00,"It should have sent back a valid total tax amount but sent back: ".$checkout_resource->total_tax_amt);
        $tax_rate = intval(100*$total_tax_amt/$order_amt);
        $this->assertNotEquals(20,$tax_rate,"tax rate should be calculated by brink, not the 20% that is in our system");

        //make sure totals are correct.
        $complete_order = CompleteOrder::staticGetCompleteOrder($checkout_resource->oid_test_only,$m);
        $grand_total_from_math = $complete_order['order_amt']+$complete_order['total_tax_amt']+$complete_order['delivery_amt']+$complete_order['tip_amt']+$complete_order['promo_amt'];
        $this->assertEquals($grand_total_from_math,$complete_order['grand_total'],"the set grand total should equal the sum of the parts");
    }

    function testDoubleMeat()
    {
        $merchant_id = $this->ids['merchant_id'];
        $user_resource = createNewUserWithCCNoCVV(array("email"=>"bobnewhart@order140.com","first_name"=>"bob","last_name"=>"newhart","contact_no"=>'8888888888'));
        $user = logTestUserResourceIn($user_resource);
        $order_adapter = new OrderAdapter($m);
        $menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$merchant_id,2,'pickup');
        $order_data = $order_adapter->getOrderArrayFromFullMenuWithOneModiferPerModifierGroup($menu,$merchant_id,"some note",1,0,1);
        $record = ModifierItemAdapter::staticGetRecord(array("modifier_item_name"=>'Double Meat ($2.29)'),'ModifierItemAdapter');
        $order_data['items'][0]['mods'][] = array("mod_quantity"=>1,"modifier_item_id"=>$record['modifier_item_id']);
        $order_data['tip'] = 1.00;
        $request = createRequestObject('app2/apiv2/cart/checkout','post',json_encode($order_data),'application/json');
        $place_order_controller = new PlaceOrderController($m,$user,$request,5);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $create_cart_checkout_response = $place_order_controller->processV2Request();
        $this->assertNull($create_cart_checkout_response->error,"It should not throw an error due to double meat");

        // now try to place the order
        $order_resource = placeOrderFromCheckoutResource($create_cart_checkout_response,$user,$merchant_id,1.00,$time);
        $this->assertNull($order_resource->error);
        

        $brink_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'B');
        $brink_controller = new BrinkController($m, $u, $r, 5);
        $ready_to_send_message_resource = $brink_controller->prepMessageForSending($brink_message_resource);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);
        $brink_controller = new BrinkController($m,$u,$r,5);
        $this->assertTrue($brink_controller->sendThisMessage($brink_message_resource));
        $message_history_record = MerchantMessageHistoryAdapter::staticGetRecordByPrimaryKey($brink_message_resource->map_id,'MerchantMessageHistoryAdapter');
        $this->assertNotNull($message_history_record['message_text'],"should have recorded the sent XML");
        $this->assertNotNull($message_history_record['response'],"should have recorded the response XML");
        $this->assertContains("<SubmitOrder ",$message_history_record['message_text']);
        $this->assertContains("<OrderId>",$message_history_record['response']);
        $this->assertNotContains('<OrderId>0</OrderId>',$message_history_record['response']);
    }

//    function testAddComboToAnOrderedItem()
//    {
//        $lead_time_minutes = 20;
//        $merchant_id = $this->ids['merchant_id'];
//        $user_resource = createNewUserWithCC(array("email"=>"bobnewhart@order140.com","first_name"=>"bob","last_name"=>"newhart","contact_no"=>'8888888888'));
//        $user = logTestUserResourceIn($user_resource);
//        $order_adapter = new OrderAdapter($m);
//        $menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$merchant_id);
//        $order_data = $order_adapter->getOrderArrayFromFullMenuWithOneModiferPerModifierGroup($menu,$merchant_id,"skip hours",1,0,1);
//        $order_data['items'][0]['mods'][] = array("mod_quantity"=>1,"modifier_item_id"=>2281407);
//        $order_data['tip'] = 1.00;
//        $tomorrow_12_noon_denver = getTomorrowTwelveNoonTimeStampDenver();
//        //$time_stamp_for_noon_on_feb14_denver = getTimeStampForDateTimeAndTimeZone(12, 0, 0, 2, 14, 2015, 'America/Denver');
//        $order_data['lead_time'] = $lead_time_minutes;
//        $pickup_ts = $tomorrow_12_noon_denver+($lead_time_minutes * 60);
//        $order_data['actual_pickup_time'] = $pickup_ts;
//        $request = createRequestObject('/app2/apiv2/cart/checkout','post',json_encode($order_data),'application/json');
//        $place_order_controller = new PlaceOrderController($m,$user,$request,5);
//        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
//        $checkout_data_response = $place_order_controller->processV2Request();
//        //$checkout_data_response = getCheckoutDataV2FromOrderData($order_data, $time);
//        $order_id = $checkout_data_response->order_id;
//    }

    function testUserGetsChargeTheCorrectAmountWithATipAndOurPayloadToBrinkHasCorrectAmount()
    {
        $lead_time_minutes = 20;
        $merchant_id = $this->ids['merchant_id'];
        $user_resource = createNewUserWithCC(array("email"=>"bobnewhart@order140.com","first_name"=>"bob","last_name"=>"newhart","contact_no"=>'8888888888'));
        $user = logTestUserResourceIn($user_resource);
        $order_adapter = new OrderAdapter($m);
        $menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$merchant_id,2,'pickup');
        $order_data = $order_adapter->getOrderArrayFromFullMenuWithOneModiferPerModifierGroup($menu,$merchant_id,"some note",1,0,1);
        $order_data['tip'] = 1.00;
        $tomorrow_12_noon_denver = getTomorrowTwelveNoonTimeStampDenver();
        //$time_stamp_for_noon_on_feb14_denver = getTimeStampForDateTimeAndTimeZone(12, 0, 0, 2, 14, 2015, 'America/Denver');
        $order_data['lead_time'] = $lead_time_minutes;
        $pickup_ts = $tomorrow_12_noon_denver+($lead_time_minutes * 60);
        $order_data['actual_pickup_time'] = $pickup_ts;

        $order_resource = placeOrderFromOrderData($order_data,$time_stamp_for_noon_on_feb14_denver);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $this->assertTrue($order_id > 1000,"should have created a valid order id");

        $complete_order = CompleteOrder::getBaseOrderData($order_id,$m);
        $calculated_grand_total = $complete_order['order_amt']+$complete_order['tip_amt']+$complete_order['total_tax_amt'];
        $this->assertEquals($calculated_grand_total,$complete_order['grand_total']);

        $brink_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'B');
        $brink_controller = new BrinkController($m, $u, $r, 5);
        $ready_to_send_message_resource = $brink_controller->prepMessageForSending($brink_message_resource);

        $expected_grand_total_to_brink = number_format($complete_order['grand_total']-$complete_order['tip_amt'],2);
        $this->assertContains("<Amount>$expected_grand_total_to_brink</Amount>",$ready_to_send_message_resource->message_text);

    }

    function testItShouldBeAbleToPlaceAnOrderAgainstTheImportedMenu()
    {
        $lead_time_minutes = 20;
        $merchant_id = $this->ids['merchant_id'];
        $user_resource = createNewUserWithCC(array("email"=>"bobnewhart@order140.com","first_name"=>"bob","last_name"=>"newhart","contact_no"=>'8888888888'));
        $user = logTestUserResourceIn($user_resource);
        $order_adapter = new OrderAdapter($m);
        $menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$merchant_id,2,'pickup');
        $order_data = $order_adapter->getOrderArrayFromFullMenuWithOneModiferPerModifierGroup($menu,$merchant_id,"some note",1,0,1);
        $order_data['tip'] = 1.00;
        $tomorrow_12_noon_denver = getTomorrowTwelveNoonTimeStampDenver();
        //$time_stamp_for_noon_on_feb14_denver = getTimeStampForDateTimeAndTimeZone(12, 0, 0, 2, 14, 2015, 'America/Denver');
        $order_data['lead_time'] = $lead_time_minutes;
        $pickup_ts = $tomorrow_12_noon_denver+($lead_time_minutes * 60);
        $order_data['actual_pickup_time'] = $pickup_ts;

        $order_resource = placeOrderFromOrderData($order_data,$time_stamp_for_noon_on_feb14_denver);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $this->assertTrue($order_id > 1000,"should have created a valid order id");
        return $order_id;
    }

    function testDeliveryOrderZoneTwo()
    {
        $merchant_id = $this->ids['merchant_id'];
        $menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$merchant_id,2,'pickup');

        $user_resource = createNewUserWithCC(array("email"=>"bobnewhart_two@order140.com","first_name"=>"bob","last_name"=>"newhart","contact_no"=>'8888888888'));
        $user = logTestUserResourceIn($user_resource);
        $json = '{"user_addr_id":null,"user_id":"'.$user['user_id'].'","name":"","address1":"180 800 S","address2":"apt 2","city":"Salt Lake City","state":"UT","zip":"84111","phone_no":"1234567890","lat":40.7496327,"lng":-111.8872291}';
        $request = new Request();
        $request->body = $json;
        $request->mimetype = "Application/json";
        $request->_parseRequestBody();
        $request->method = 'POST';
        $request->url = "/users/".$user['uuid']."/userdeliverylocation";
        $user_controller = new UserController($mt, $user, $request,5);
        $response = $user_controller->processV2Request();
        $this->assertNull($response->error,"should not have gotten a delivery save error but did");
        $this->assertNotNull($response->user_addr_id);
        $user_address_id = $response->user_addr_id;
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getCartArrayFromFullMenuWithOneModiferPerModifierGroup($menu, $merchant_id,'sum dum note',10,0,1);
        $order_data['tip'] = 1.00;
        $order_data['user_addr_id'] = $user_address_id;

        $request = createRequestObject('app2/apiv2/cart/checkout','post',json_encode($order_data),'application/json');
        $place_order_controller = new PlaceOrderController($m,$user,$request,5);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($checkout_resource->error,"It should not throw an error");
        $this->assertEquals(5.00,$checkout_resource->delivery_amt,"It should get the delivery amount from zone 2 of brink");
        $this->assertTrue($checkout_resource->total_tax_amt > 1.00,"It should have tax");

        //make sure totals are correct.
        $complete_order = CompleteOrder::staticGetCompleteOrder($checkout_resource->oid_test_only,$m);
        $grand_total_from_math = $complete_order['order_amt']+$complete_order['total_tax_amt']+$complete_order['delivery_amt']+$complete_order['tip_amt']+$complete_order['promo_amt'];
        $this->assertEquals($grand_total_from_math,$complete_order['grand_total'],"the set grand total should equal the sum of the parts");

        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id);
        $this->assertEquals(5.00,$complete_order['delivery_amt']);
        $this->assertTrue($complete_order['grand_total'] == $complete_order['order_amt'] + $complete_order['total_tax_amt'] + $complete_order['delivery_amt'],"Our total shoudl include delivery amount");

        $brink_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'B');
        $brink_controller = new BrinkController($m, $u, $r, 5);
        $ready_to_send_message_resource = $brink_controller->prepMessageForSending($brink_message_resource);
        myerror_log($ready_to_send_message_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);

        $expected_delivery_surcharge_node = cleanUpDoubleSpacesCRLFTFromString('<Surcharges>
                  <OrderSurcharge>
                     <Amount>5</Amount>
                     <Id>1</Id>
                     <IsSystemApplied>true</IsSystemApplied>
                     <Name>Delivery Fee 2</Name>
                     <SurchargeId>4024023</SurchargeId>
                     <Taxes />
                  </OrderSurcharge>
               </Surcharges>');
        $expected_delivery_surcharge_node = str_ireplace('> <','><',$expected_delivery_surcharge_node);
        $this->assertContains($expected_delivery_surcharge_node,$body,"It should contain the delivery surcharge node");

    }

    function testSendDeliveryOrder()
    {
        $user_resource = createNewUserWithCC(array("email"=>"bobnewhart_two@order140.com","first_name"=>"bob","last_name"=>"newhart","contact_no"=>'8888888888'));
        $user = logTestUserResourceIn($user_resource);
        $json = '{"user_addr_id":null,"user_id":"'.$user['user_id'].'","name":"","address1":"104 N 4th St","address2":"apt 2","city":"Coeur d\'Alene","state":"ID","zip":"83814","phone_no":"9709262121","lat":47.6735024,"lng":-116.7834097}';
        //$json = '{"user_addr_id":null,"user_id":"'.$user['user_id'].'","name":"","address1":"1045 Pine Street","address2":"apt 2","city":"Boulder","state":"CO","zip":"80302","phone_no":"1234567890","lat":40.0197891,"lng":-105.2846977}';
        //$json = '{"user_addr_id":null,"user_id":"'.$user['user_id'].'","name":"","address1":"180 800 S","address2":"apt 2","city":"Salt Lake City","state":"UT","zip":"84111","phone_no":"1234567890","lat":40.7496327,"lng":-111.8872291}';
        $request = new Request();
        $request->body = $json;
        $request->mimetype = "Application/json";
        $request->_parseRequestBody();
        $request->method = 'POST';
        $request->url = "/users/".$user['uuid']."/userdeliverylocation";
        $user_controller = new UserController($mt, $user, $request,5);
        //$response = $user_controller->setDeliveryAddr();
        $response = $user_controller->processV2Request();
        $this->assertNull($response->error,"should not have gotten a delivery save error but did");
        $this->assertNotNull($response->user_addr_id);
        $user_address_id = $response->user_addr_id;

        $merchant_id = $this->ids['merchant_id'];
        $merchant_record = MerchantAdapter::staticGetRecordByPrimaryKey($merchant_id,'MerchantAdapter');
        $mdpd_resource = Resource::find(new MerchantDeliveryPriceDistanceAdapter($m),'',array(3=>array("merchant_id"=>$merchant_id)));
        $menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$merchant_id,2,'pickup');
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getCartArrayFromFullMenuWithOneModiferPerModifierGroup($menu, $merchant_id,'sum dum note',10,0,1);
        $order_data['tip'] = 1.00;
        //$order_data = OrderAdapter::staticGetSimpleOrderArrayByMerchantId($merchant_id, 'delivery', 'sum dum note');
        $order_data['user_addr_id'] = $user_address_id;

        $request = createRequestObject('app2/apiv2/cart/checkout','post',json_encode($order_data),'application/json');
        $place_order_controller = new PlaceOrderController($m,$user,$request,5);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($checkout_resource->error,"It should not throw an error");

        //make sure totals are correct.
        $complete_order = CompleteOrder::staticGetCompleteOrder($checkout_resource->oid_test_only,$m);
        $grand_total_from_math = $complete_order['order_amt']+$complete_order['total_tax_amt']+$complete_order['delivery_amt']+$complete_order['tip_amt']+$complete_order['promo_amt'];
        $this->assertEquals($grand_total_from_math,$complete_order['grand_total'],"the set grand total should equal the sum of the parts");


        // now try to place the order
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);

        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id);
        $this->assertEquals(1.00,$complete_order['delivery_amt']);
        $this->assertTrue($complete_order['grand_total'] == $complete_order['order_amt'] + $complete_order['total_tax_amt'] + $complete_order['delivery_amt'],"Our total shoudl include delivery amount");

        $brink_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'B');
        $brink_controller = new BrinkController($m, $u, $r, 5);
        $ready_to_send_message_resource = $brink_controller->prepMessageForSending($brink_message_resource);
        myerror_log($ready_to_send_message_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);
        
        $expected_delivery_surcharge_node = cleanUpDoubleSpacesCRLFTFromString('<Surcharges>
                  <OrderSurcharge>
                     <Amount>1</Amount>
                     <Id>1</Id>
                     <IsSystemApplied>true</IsSystemApplied>
                     <Name>Delivery Fee 1</Name>
                     <SurchargeId>4004802</SurchargeId>
                     <Taxes />
                  </OrderSurcharge>
               </Surcharges>');
        $expected_delivery_surcharge_node = str_ireplace('> <','><',$expected_delivery_surcharge_node);
        $this->assertContains($expected_delivery_surcharge_node,$body,"It should contain the delivery surcharge node");
//        // make sure that body send total without delivery charge
//        $brink_total = $complete_order['order_amt'] + $complete_order['tax_amt'];
//
//        $this->assertContains("<Total>$brink_total</Total>",$body,"It should have the total without the delivery charge");


        $brink_controller = new BrinkController($m,$u,$r,5);
        $this->assertTrue($brink_controller->sendThisMessage($brink_message_resource));
        $message_history_record = MerchantMessageHistoryAdapter::staticGetRecordByPrimaryKey($brink_message_resource->map_id,'MerchantMessageHistoryAdapter');
        $this->assertNotNull($message_history_record['message_text'],"should have recorded the sent XML");
        $this->assertNotNull($message_history_record['response'],"should have recorded the response XML");
        $this->assertContains("<SubmitOrder ",$message_history_record['message_text']);
        $this->assertContains("<OrderId>",$message_history_record['response']);
        $this->assertNotContains('<OrderId>0</OrderId>',$message_history_record['response']);

    }


    /**
     * @depends testItShouldBeAbleToPlaceAnOrderAgainstTheImportedMenu
     */
    function testItShouldNotUseOurTaxValueButShouldOverrideWithTheBrinkValue($order_id)
    {
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
        $order_amt = floatval($complete_order['order_amt']);
        $total_tax_amt = floatval($complete_order['total_tax_amt']);
        $tip_amt = floatval($complete_order['tip_amt']);
        $tax_rate = intval(100*$total_tax_amt/$order_amt);
        $this->assertNotEquals(20,$tax_rate,"tax rate should not equal the set rate of 20 but should have been calculated from brink call");
        $this->assertTrue($tax_rate < 11);
        $this->assertEquals($complete_order['grand_total'],$order_amt+$total_tax_amt+$tip_amt,"grand total should equal the brink tax plus our order amount");
    }

    /**
     * @depends testItShouldBeAbleToPlaceAnOrderAgainstTheImportedMenu
     */
    function testItShouldCreateAValidBrinkPayload($order_id)
    {
        $merchant_id = $this->ids['merchant_id'];
        $order_adapter = new OrderAdapter($m);
        $menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$merchant_id,2,'pickup');
        $order_data = $order_adapter->getOrderArrayFromFullMenuWithOneModiferPerModifierGroup($menu,$merchant_id,"skip hours",1,0,1);
        $item = $order_data['items'][0];
        $item_size_record = ItemSizeAdapter::staticGetRecord(array("item_id"=>$item['item_id'],"size_id"=>$item['size_id'],"merchant_id"=>$merchant_id),'ItemSizeAdapter');
        $item_external_id = $item_size_record['external_id'];
        $modifier = $item['mods'][0];
        $modifer_size_record = ModifierSizeMapAdapter::staticGetRecord(array("modifier_item_id"=>$modifier['modifier_item_id'],"size_id"=>$item['size_id'],"merchant_id"=>$merchant_id),'ModifierSizeMapAdapter');
        $modifier_item_external_id = $modifer_size_record['external_id'];
        $m = explode('-',$modifier_item_external_id);
        $expected_payload = cleanUpDoubleSpacesCRLFTFromString(file_get_contents("./unit_tests_pos_integrations/resources/expected_brink_single_item_with_pita_modifier.xml"));
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
        $expected_payload = str_replace('%%pitapitsmbltprice%%',$complete_order['order_details'][0]['item_total'],$expected_payload);
        $expected_payload = str_replace('%%ordergrandtotal%%',$complete_order['grand_total']-$complete_order['tip_amt'],$expected_payload);
        $expected_payload = str_replace('%%tip_amt%%',$complete_order['tip_amt'],$expected_payload);
        $expected_payload = str_replace('%%item_external_id%%',$item_external_id,$expected_payload);
        $expected_payload = str_replace('%%modifier_item_external_id%%',$m[1],$expected_payload);
        $expected_payload = str_replace('%%modifier_group_external_id%%',$m[0],$expected_payload);

        $tz = date_default_timezone_get();
        date_default_timezone_set("GMT");
        $brink_formatted_pickup = date("Y-m-d\TH:i:00.0000\Z",getTomorrowTwelveNoonTimeStampDenver()+(20 * 60));
        $expected_payload = str_replace('%%tomorrowpickup%%',$brink_formatted_pickup,$expected_payload);
        date_default_timezone_set($tz);

        $brink_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'B');
        $brink_controller = new BrinkController($m, $u, $r, 5);
        $ready_to_send_message_resource = $brink_controller->prepMessageForSending($brink_message_resource);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);
        $this->assertEquals($expected_payload,$body,"should have created the expected brink payload");
        return $brink_message_resource;
    }

    /**
     * @depends testItShouldCreateAValidBrinkPayload
     */
    function testItShouldSendTheMessageAndGetAValidResponse($message_resource)
    {
        $brink_controller = new BrinkController($m,$u,$r,5);
        $this->assertTrue($brink_controller->sendThisMessage($message_resource));
        $message_history_record = MerchantMessageHistoryAdapter::staticGetRecordByPrimaryKey($message_resource->map_id,'MerchantMessageHistoryAdapter');
        $this->assertNotNull($message_history_record['message_text'],"should have recorded the sent XML");
        $this->assertNotNull($message_history_record['response'],"should have recorded the response XML");
        $this->assertContains("<SubmitOrder ",$message_history_record['message_text']);
        $this->assertContains("<OrderId>",$message_history_record['response']);
        $this->assertNotContains('<OrderId>0</OrderId>',$message_history_record['response']);
    }

    /**
     * @expectedException UnsuccessfulBrinkPushException
     */
    function testMessageSendWIthoutAValidResponse()
    {
        $merchant_id = $this->ids['merchant_id'];
        $user = logTestUserIn($this->ids['user_id']);
        $order_adapter = new OrderAdapter($m);
        $menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$merchant_id,2,'pickup');
        $order_data = $order_adapter->getOrderArrayFromFullMenuWithOneModiferPerModifierGroup($menu,$merchant_id,"skip hours",1,0,1);
        $order_data['tip'] = 1.00;
        $tomorrow_12_noon_denver = getTomorrowTwelveNoonTimeStampDenver();
        $lead_time_minutes = 20;
        $order_data['lead_time'] = $lead_time_minutes;
        $pickup_ts = $tomorrow_12_noon_denver+($lead_time_minutes * 60);
        $order_data['actual_pickup_time'] = $pickup_ts;
        $order_resource = placeOrderFromOrderData($order_data,$time_stamp_for_noon_on_feb14_denver);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $brink_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'B');
        $brink_controller = new BrinkController($m, $u, $r, 5);
        $ready_to_send_message_resource = $brink_controller->prepMessageForSending($brink_message_resource);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);
        $brink_service = new BrinkService('456rtdfg456RTdfg==');
        $response = $brink_service->send($body);

    }

    function testFailedBrinkPush()
    {
        $this->assertTrue(false,"need to build test that checks the response for no order id, and fails the message appropriately");
    }

//    function testItshouldImportASecondMerchantIntoTheSameMenuOrNot()
//    {
//        $previous_menu_id = createTestMenuWithOneItem("The Item");
//        $merchant_resource = getOrCreateNewTestMerchantBasedOnExternalId('pitapit-11111');
//        $mmm_resource = MerchantMenuMapAdapter::createMerchantMenuMap($merchant_resource->merchant_id, $previous_menu_id, 'pickup');
//        $this->assertEquals($previous_menu_id,$mmm_resource->menu_id);
//        $mbima_resource = Resource::createByData(new MerchantBrinkInfoMapsAdapter($m),array("merchant_id"=>$merchant_resource->merchant_id,"brink_location_token"=>getProperty('brink_test_location_token')));
//
//        $activity_history_resource = ActivityHistoryAdapter::createActivityReturnActivityResource('ExecuteObjectFunction', time()-10, "object=BrinkImporter;method=import;thefunctiondatastring=pitapit-11111", $activity_text);
//        $splickit_activity = new ExecuteObjectFunctionActivity($activity_history_resource);
//        $splickit_activity->executeThisActivity();
//
//        $refeshed_activity_resource = $activity_history_resource->getRefreshedResource();
//        $this->assertEquals('E',$refeshed_activity_resource->locked,"Should have set the activity to executed");
//
//        $mmm_record = MerchantMenuMapAdapter::staticGetRecord(array("merchant_id"=>$merchant_resource->merchant_id,"merchant_menu_type"=>'pickup'),'MerchantMenuMapAdapter');
//        $this->assertEquals($this->ids['menu_id'],$mmm_record['menu_id']);
//    }

    static function setUpBeforeClass()
    {
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time',300);

        $skin_adapter = new SkinAdapter($m);
        $public_client_id = generateUUID();
        $sql = "UPDATE Skin SET public_client_id = '$public_client_id' WHERE skin_id = 13";
        $skin_adapter->_query($sql);
        $ids['pita_pit_public_client_id'] = $public_client_id;


        setContext('com.splickit.pitapit');
        $merchant_resource2 = getOrCreateNewTestMerchantBasedOnExternalId('pitapit-99999');
        $merchant_resource = getOrCreateNewTestMerchantBasedOnExternalId('pitapit-88888');
        $merchant_resource->address1 = '105 North 4th St';
        $merchant_resource->city = "Coeur d'Alene";
        $merchant_resource->state = 'ID';
        $merchant_resource->zip = '83814';
        $merchant_resource->lat = '47.673240';
        $merchant_resource->lng = '-116.781634';
        $merchant_resource->save();

        $merchant_resource3 = getOrCreateNewTestMerchantBasedOnExternalId('US-075-C:05-008-ID');
        $merchant_resource3->address1 = '105 North 4th St';
        $merchant_resource3->city = "Coeur d'Alene";
        $merchant_resource3->state = 'ID';
        $merchant_resource3->zip = '83814';
        $merchant_resource3->lat = '47.673240';
        $merchant_resource3->lng = '-116.781634';
        $merchant_resource3->save();
        Resource::createByData(new MerchantBrinkInfoMapsAdapter($m),array("merchant_id"=>$merchant_resource3->merchant_id,"brink_location_token"=>"4Uo9rh1ZOUKambW059BLoA=="));

        unset($merchant_resource);
        $merchant_resource = $merchant_resource3;

        $merchant_id = $merchant_resource->merchant_id;

        $ids['mbim_resource'] = Resource::createByData(new MerchantBrinkInfoMapsAdapter($m),array("merchant_id"=>$merchant_resource->merchant_id,"brink_location_token"=>getProperty('brink_test_location_token')));

        $ids['merchant_resource'] = $merchant_resource;
        $ids['merchant_id'] = $merchant_resource->merchant_id;

        $options[TONIC_FIND_BY_METADATA]['merchant_id'] = $merchant_resource->merchant_id;
        $tax_resource = Resource::find(new TaxAdapter($m),'',$options);
        $tax_resource->rate = 20.0;
        $tax_resource->save();

        if ($record = MerchantMenuMapAdapter::staticGetRecord(array("merchant_id"=>$merchant_resource->merchant_id,"merchant_menu_type"=>'pickup'),'MerchantMenuMapAdapter')) {
            $menu_id = $record['menu_id'];
            $brink_importer = ImporterFactory::getImporterFromUrl("http://localhost/app2/apiv2/pos/import/brink/US-075-C:05-008-ID");
            $brink_importer->create_zero_records = false;
            $menu_id =  $brink_importer->importRemoteMerchantMetaDataForLoadedMerchant();
        } else {
            // takes almost 4X as long to do it with the API vs method call. WTF?
//            //$curl = curl_init('http://localhost/app2/admin/importbrinkmerchant?merchant_id=pitapit-88888');
//            $curl = curl_init('http://localhost/app2/apiv2/importbrinkmerchant/pitapit/pitapit-88888');
//            curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
//            $result = curl_exec($curl);
//            $info = curl_getinfo($curl);
//            curl_close($curl);
//            $result_array = json_decode($result,true);
//            $menu_id = $result_array['menu_id'];
            $brink_importer = ImporterFactory::getImporterFromUrl("http://localhost/app2/apiv2/pos/import/brink/US-075-C:05-008-ID");
            $brink_importer->create_zero_records = true;
            $menu_id =  $brink_importer->importRemoteMerchantMetaDataForLoadedMerchant();

            addDeliveryToMerchant($merchant_resource, $menu_id);
            $mdpda = new MerchantDeliveryPriceDistanceAdapter($m);
            $mdpd_resource = $mdpda->getExactResourceFromData(array("merchant_id"=>$merchant_resource->merchant_id));
            $mdpd_resource->name = "zone_1";
            $mdpd_resource->distance_up_to = 100;
            $mdpd_resource->price = 1.50;
            $mdpd_resource->save();
            $dp_resource = Resource::findOrCreateIfNotExistsByData(new MerchantDeliveryPriceDistanceAdapter($m),array("merchant_id"=>$merchant_resource->merchant_id,"distance_up_to"=>"3000","name"=>"zone_2"));
            $dp_resource->price = 3.50;
            $dp_resource->save();

        }

        //now try a second
//        $brink_importer = ImporterFactory::getImporterFromUrl("http://localhost/app2/apiv2/pos/import/brink/US-075-C?brand_name=pitapit");
//        $menu_id =  $brink_importer->importRemoteMerchantMetaDataForLoadedMerchant();


        $udlmpma = new UserDeliveryLocationMerchantPriceMapsAdapter($m);
        $sql = "DELETE FROM User_Delivery_Location_Merchant_Price_Maps WHERE merchant_id = $merchant_id";
        $udlmpma->_query($sql);

        $complete_menu = CompleteMenu::getCompleteMenu($menu_id,'Y',$merchant_resource->merchant_id,2,'pickup');
        $ids['complete_menu'] = $complete_menu;
        $ids['menu_id'] = $menu_id;

        //mysqli_query("BEGIN");
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_resource->merchant_id,"message_format"=>'B',"delivery_addr"=>"Brink","message_type"=>"X"));

        $user_resource = createNewUser(array("flags"=>"1C20000001"));
        $ids['user_id'] = $user_resource->user_id;

//create the type 1 promo
        $promo_adapter = new PromoAdapter($m);
        $promo_id = 111;
        $promo_amt = 5.00;
        $ids['promo_amt'] = $promo_amt;
        if ($promo_record = $promo_adapter->getRecordFromPrimaryKey($promo_id)) {
            ;
        } else {
            $ids['promo_id_type_1'] = $promo_id;
            $sql = "INSERT INTO `Promo` VALUES($promo_id, 'The Type1 Promo', 'Get Up to $5 off', 1, 'Y', 'N', 0, 2, 'N', 'N', '2010-01-01', '2020-01-01', 100000, 0, 0.00, 0, 0.00, 'Y', 'N', NOW(), NOW(), 'N')";
            $promo_adapter->_query($sql);
//            $sql = "INSERT INTO `Promo_Merchant_Map` VALUES(null, 201, $merchant_resource->merchant_id, '2013-10-05', '2020-01-01', NULL, now())";
//            $promo_adapter->_query($sql);
            $pmm_resource = Resource::createByData(new PromoMerchantMapAdapter($mimetypes), array("merchant_id"=>$merchant_resource->merchant_id,"promo_id"=>$promo_id));
            $ids['promo_merchant_map_id_type_1'] = $pmm_resource->map_id;
            $sql = "INSERT INTO `Promo_Message_Map` VALUES(null, $promo_id, 'Congratulations! You''re getting %%amt%% off your order!', NULL, NULL, NULL, NULL, now())";
            $promo_adapter->_query($sql);
            $sql = "INSERT INTO `Promo_Type1_Amt_Map` VALUES(null, $promo_id, 1.00, $promo_amt, 0,50.00, NOW())";
            $promo_adapter->_query($sql);
            $pkwm_adapter = new PromoKeyWordMapAdapter($m);
            Resource::createByData($pkwm_adapter, array("promo_id"=>$promo_id,"promo_key_word"=>"type1promo","brand_id"=>300));

        }


        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;

    }

    static function tearDownAfterClass()
    {
        mysqli_query("ROLLBACK");
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    BrinkImporterTest::main();
}

?>