<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';


class EPLFoundryImporterTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
        $this->ids = $_SERVER['unit_test_ids'];
        setContext('com.splickit.elpolloloco');

    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }

//    function testGetPayloadFromDump()
//    {
//        $raw = file_get_contents("./unit_tests_pos_integrations/resources/EPLMenu.xml");
//        $epli = new EplImporter($this->ids['merchant_resource']);
//        $epli->processRawExport($raw);
//        $result = $epli->imported_prices;
//        $this->assertTrue(sizeof($result) > 500);
//        return $result;
//    }

    function testCustomEmailConfPickup()
    {
        $merchant_resource = $this->ids['merchant_resource'];
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $order_data = OrderAdapter::staticGetSimpleOrderArrayByMerchantId($merchant_resource->merchant_id);
        $place_order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($place_order_resource->error);
        $emailconf_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'Econf');
        $body = $emailconf_message_resource->message_text;
        myerror_log($body);
        $this->assertContains('Please go directly to the pick-up counter',$body,"It should contain the pickup food info");
    }

    function testCustomEmailConfDelivery()
    {
        $menu_id = createTestMenuWithOneItem('delivery item');
        $merchant_resource = createNewTestMerchantDelivery($menu_id);
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $json = '{"user_addr_id":null,"user_id":"'.$user['user_id'].'","name":"","address1":"1045 Pine Street","address2":"","city":"boulder","state":"co","zip":"80302","phone_no":"9709262121","lat":40.0197891,"lng":-105.284703}';
        $request = new Request();
        $request->body = $json;
        $request->mimetype = "Application/json";
        $request->_parseRequestBody();
        $request->method = 'POST';
        $request->url = "/users/".$user['uuid']."/userdeliverylocation";
        $user_controller = new UserController($mt, $user, $request,5);
        $response = $user_controller->processV2Request();
        $this->assertNull($response->error,"should not have gotten a delivery save error but did");
        $this->assertNotNull($response->user_addr_id);
        $user_address_id = $response->user_addr_id;

        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id, 'delivery', 'some note');
        $order_data['user_addr_id'] = $user_address_id;
        $delivery_time = date('Y-m-d H:i:s',getTomorrowTwelveNoonTimeStampDenver()+3600);
        $order_data['delivery_time'] = $delivery_time;

        $place_order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($place_order_resource->error);
        $emailconf_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'Econf');
        $body = $emailconf_message_resource->message_text;
        myerror_log($body);
        $this->assertContains('Your order will be delivered at '.$delivery_time.' to the address below',$body,"It should contain the delivery info");
        $this->assertContains('1045 Pine Street',$body,"It should contain the delivery location info");
    }

    function testImportComboPriceToMerchant()
    {
        $merchant_resource = $this->ids['merchant_resource'];
        // check prices
        $result = $this->ids['imported_prices'];
        $regular_plu = '8050081';
        $combo_plu = '8050451';
        $regular_price = $result[$regular_plu]['PRICE'];
        $expected_price_because_of_forced_combo = $result[$combo_plu]['PRICE'];
        $item_id = 299864;
        $size_id = 93827;
        $data['item_id'] = $item_id;
        $data['size_id'] = $size_id;
        $data['merchant_id'] = $merchant_resource->merchant_id;
        $price_record = ItemSizeAdapter::staticGetRecord($data,'ItemSizeAdapter');
        $this->assertEquals($expected_price_because_of_forced_combo,$price_record['price'],"It should have picked up the combo price");
        $this->assertNotEquals($regular_price,$price_record['price'],"It should NOT have picked up the combo price");
    }

    function testImportComboBeveragePrice()
    {
        $merchant_resource = $this->ids['merchant_resource'];
        $regular_plu = '8048061';
        $combo_plu = '8048062';
        $drink_plu = '8019002-8019300:C';
        // get prices for each of the plus
        $result = $this->ids['imported_prices'];
        $regular_price = $result[$regular_plu]['PRICE'];
        $combo_price = $result[$combo_plu]['PRICE'];

        //check modifier price
        $modifier_item_id = 2343498;
        $size_id_of_bowl = 93843;
        $merchant_id = $merchant_resource->merchant_id;
        $mod_data['modifier_item_id'] = $modifier_item_id;
        $mod_data['merchant_id'] = $merchant_id;
        $mod_data['size_id'] = $size_id_of_bowl;
        $mod_price_record = ModifierSizeMapAdapter::staticGetRecord($mod_data,'ModifierSizeMapAdapter');
        $this->assertNotNull($mod_price_record);
        $drink_price_as_part_of_combo = $combo_price - $regular_price+$this->ids['large_drink_upcharge'];
        $this->assertTrue($mod_price_record['modifier_price'] > 0.00,"Price should have been greater than 0.00 but it was: ".$mod_price_record['modifier_price']);
        $this->assertEquals($drink_price_as_part_of_combo,$mod_price_record['modifier_price'],"The Drink price should be the difference");
        $this->assertEquals($drink_plu,$mod_price_record['external_id'],"It should have had the combo flag external id");
    }

    function testBonelessSub()
    {
        $merchant_resource = $this->ids['merchant_resource'];
        $merchant_id = $merchant_resource->merchant_id;

        //$menu = CompleteMenu::getCompleteMenu(102711,'Y',$merchant_id);
        $user_resource = createNewUserWithCC();
        $user = logTestUserResourceIn($user_resource);
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','the note');
        $order_data['items'][0]['item_id'] = 299872;
        $order_data['items'][0]['size_id'] = 93834;
        $order_data['items'][0]['mods'] = array();
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2343718,'mod_quantity'=>1);
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2343726,'mod_quantity'=>1);
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2345657,'mod_quantity'=>1);
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2345655,'mod_quantity'=>1);

        $order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        //$complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id);

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'UE');
        $foundry_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $foundry_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        myerror_log($message_to_send_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);



        $item_xml = cleanUpDoubleSpacesCRLFTFromString('
<COMBOMODMAINITEM ITEMID="8026307" PRICE="" FOR="" />
<MODITEM QTY="1" ITEMID="8048002" MODCODE="1"/>
<MODITEM QTY="1" ITEMID="8013090" MODCODE="1"/>
<COMBOMODMAINITEM ITEMID="8029586" PRICE="" FOR="" />');
        $this->assertContains($item_xml,$body,"It Should have the expected item xml as whats sent");
    }

    function testAllThighModifier()
    {
        $merchant_resource = $this->ids['merchant_resource'];
        $merchant_id = $merchant_resource->merchant_id;

        $user_resource = createNewUserWithCC();
        $user = logTestUserResourceIn($user_resource);
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','the note');
        $order_data['items'][0]['item_id'] = 300214;
        $order_data['items'][0]['size_id'] = 93927;
        $order_data['items'][0]['mods'] = array();
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2342984,'mod_quantity'=>1);
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2342981,'mod_quantity'=>1);
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2343650,'mod_quantity'=>1);
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2343687,'mod_quantity'=>1);

        $order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id);

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'UE');
        $foundry_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $foundry_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        myerror_log($message_to_send_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);



        $item_xml = cleanUpDoubleSpacesCRLFTFromString('<ITEMREQUESTS><ADDITEM QTY="1" ITEMID="8050452" COMBOMEALGROUPID="4002" COMBOMEALMENUITEMID="8050082" ORDERNOW="Y" PRICE="3.98" FOR="">
              <COMBOMODMAINITEM ITEMID="8050512" PRICE="" FOR="" />
              <MODITEM QTY="1" ITEMID="8786353" MODCODE="1"/>
              <MODITEM QTY="1" ITEMID="8786349" MODCODE="1"/>
              <MODITEM QTY="1" ITEMID="8005088" MODCODE="1"/>
            </ADDITEM>
        </ITEMREQUESTS>
');
        $this->assertContains($item_xml,$body,"It Should have the expected item xml as whats sent");
    }

    function testType4PromoWithFoundry()
    {
        $merchant_resource = $this->ids['merchant_resource'];
        $merchant_id = $merchant_resource->merchant_id;

        $user_resource = createNewUserWithCC();
        $user = logTestUserResourceIn($user_resource);
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','the note');
        $item_id = $order_data['items'][0]['item_id'];
        $size_id = $order_data['items'][0]['size_id'];
        $price_record = ItemSizeAdapter::staticGetRecord(array("item_id"=>$item_id,"size_id"=>$size_id,"merchant_id"=>$merchant_id),'ItemSizeAdapter');

        //create the type 4 promo

        $promo_id = 400;
        $ids['promo_id_type_4'] = $promo_id;
        $sql = "INSERT INTO `Promo` VALUES($promo_id, 'The Promo 4', 'Get $ off your item', 4, 'Y', 'N', 0, 2, 'N', 'N', '2010-01-01', '2020-01-01', 100, 0, 0.00, 0, 0.00, 'Y', 'N', NOW(), NOW(), 'N')";
        $promo_adapter = new PromoAdapter($m);
        $promo_adapter->_query($sql);
        $brand_id = getBrandIdFromCurrentContext();
        $sql = "INSERT INTO `Promo_Brand_External_Id_Map` VALUES (null,$promo_id,$brand_id,'812',NOW(),NOW(),'N')";
        $promo_adapter->_query($sql);
        $pmm_resource = Resource::createByData(new PromoMerchantMapAdapter($mimetypes), array("merchant_id"=>$merchant_id,"promo_id"=>$promo_id));
        $ids['promo_merchant_map_id_type_4'] = $pmm_resource->map_id;
        $sql = "INSERT INTO `Promo_Message_Map` VALUES(null, $promo_id, 'Congratulations! You''re getting %%amt%% off of your %%item_name%%!', NULL, NULL, NULL, 'Here''s the deal, order a large this, and you''ll get a discount! Limit 1', now())";
        $promo_adapter->_query($sql);
        $sql = "INSERT INTO `Promo_Type4_Item_Amount_Maps` VALUES(null, $promo_id, '".$price_record['external_id']."',1.00,40,.75,NOW(),NOW(),'N' )";
        $promo_adapter->_query($sql);

        $pkwm_adapter = new PromoKeyWordMapAdapter($mimetypes);
        Resource::createByData($pkwm_adapter, array("promo_id"=>$promo_id,"promo_key_word"=>"TestPromoFour","brand_id"=>430));

        $request = createRequestObject('/apiv2/cart/checkout','POST',json_encode($order_data),'application/json');
        $placeorder_controller = new PlaceOrderController($m,$user,$request,5);
        $checkout_resource = $placeorder_controller->processV2Request();
        $ucid = $checkout_resource->cart_ucid;
        $request = createRequestObject("/apiv2/cart/$ucid/checkout?promo_code=TestPromoFour",'get');
        $placeorder_controller = new PlaceOrderController($m,$user,$request,5);
        $promo_checkout_resource = $placeorder_controller->processV2Request();
        $this->assertNull($promo_checkout_resource->error);
        $this->assertEquals("-1.00", $promo_checkout_resource->promo_amt);
        $this->assertEquals("Congratulations! You're getting $1.00 off of your Chicken Fajita!", $promo_checkout_resource->user_message);


//        $order_data['tip'] = 0.00;
//        $order_data['promo_code'] = 'Test Promo Four';
        $order_data = getPlaceOrderDataFromCheckoutResource($checkout_resource);
        $request = createRequestObject("/apiv2/orders/$ucid","POST",json_encode($order_data),$m);
        $placeorder_controller = new PlaceOrderController($m,$user,$request,5);
        $order_resource = $placeorder_controller->processV2Request();
        $order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id);

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'UE');
        $foundry_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $foundry_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        myerror_log($message_to_send_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);

        $promo_discount = -$complete_order['promo_amt'];
        $external_id = $complete_order['order_details'][0]['external_id'];
        $this->assertNotNull($external_id,"It should have an external id");
        $this->assertTrue($promo_discount > 0.00,"Promo Discount shoudl be greater than zero");
//        $expected_promo_node = cleanUpDoubleSpacesCRLFTFromString('<ADDITEM QTY="1" ITEMID="'.$external_id.'" PRICE="" FOR="">
//					<ADDDISCOUNT ID="208" AMOUNT="'.number_format($promo_discount,2).'" REFERENCE="ITEM DISC" />
//				</ADDITEM>');
//        $this->assertContains($expected_promo_node,$body,"It should have contained the promo node");

        $promo_discount = -$complete_order['promo_amt'];
        $this->assertTrue($promo_discount > 0.00,"Promo Discount shoudl be greater than zero");
        $expected_promo_node = cleanUpDoubleSpacesCRLFTFromString('<DISCOUNTREQUESTS>
				<ADDDISCOUNT ID="812" AMOUNT="'.number_format($promo_discount,2).'" REFERENCE="SUBTOTAL DISC" />
			</DISCOUNTREQUESTS>');
        $this->assertContains($expected_promo_node,$body,"It should have contained the promo node");
    }

    function testType1PromoWithFoundry()
    {
        $merchant_resource = $this->ids['merchant_resource'];
        $merchant_id = $merchant_resource->merchant_id;
        $ids['promo_id_type_1'] = 201;
        $sql = "INSERT INTO `Promo` VALUES(201, 'The Type1 Promo', 'Get 25% off', 1, 'Y', 'N', 0, 2, 'N', 'N', '2010-01-01', '2020-01-01', 1, 0, 0.00, 0, 0.00, 'Y', 'N', NOW(), NOW(), 'N')";
        $promo_adapter = new PromoAdapter($m);
        $promo_adapter->_query($sql);
        $brand_id = getBrandIdFromCurrentContext();
        $sql = "INSERT INTO `Promo_Brand_External_Id_Map` VALUES (null,201,$brand_id,'888',NOW(),NOW(),'N')";
        $promo_adapter->_query($sql);

        $sql = "INSERT INTO `Promo_Merchant_Map` VALUES(null, 201, $merchant_id, '2013-10-05', '2020-01-01', NULL, now())";
        $pmm_resource = Resource::createByData(new PromoMerchantMapAdapter($mimetypes), array("merchant_id"=>$merchant_id,"promo_id"=>201));
        $ids['promo_merchant_map_id_type_1'] = $pmm_resource->map_id;
        $sql = "INSERT INTO `Promo_Message_Map` VALUES(null, 201, 'Congratulations! You''re getting a 25% off your order!', NULL, NULL, NULL, NULL, now())";
        $promo_adapter->_query($sql);
        $sql = "INSERT INTO `Promo_Type1_Amt_Map` VALUES(null, 201, 1.00, 0.00, 25,50.00, NOW())";
        $promo_adapter->_query($sql);
        Resource::createByData(new PromoKeyWordMapAdapter($m), array("promo_id"=>201,"promo_key_word"=>"type1promo","brand_id"=>430));


        $user_resource = createNewUserWithCC();
        $user = logTestUserResourceIn($user_resource);
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','the note');
        $order_data['tip'] = 0.00;
        $order_data['promo_code'] = 'type1promo';
        $order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id);

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'UE');
        $foundry_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $foundry_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        myerror_log($message_to_send_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);

        $promo_discount = -$complete_order['promo_amt'];
        $this->assertTrue($promo_discount > 0.00,"Promo Discount shoudl be greater than zero");
        $expected_promo_node = cleanUpDoubleSpacesCRLFTFromString('<DISCOUNTREQUESTS>
				<ADDDISCOUNT ID="888" AMOUNT="'.number_format($promo_discount,2).'" REFERENCE="SUBTOTAL DISC" />
			</DISCOUNTREQUESTS>');
        $this->assertContains($expected_promo_node,$body,"It should have contained the promo node");
    }

    // so we need to check all the various combo configs

    function testForcedComboNoBevItem()
    {
        //8pc + 2 large sides    8050081:8050451:FC
        $merchant_resource = $this->ids['merchant_resource'];
        $regular_plu = '8050081';
        $combo_plu = '8050451';
        // get prices for each of the plus
        $result = $this->ids['imported_prices'];
        $regular_price = $result[$regular_plu]['PRICE'];
        $expected_price_because_of_forced_combo = $result[$combo_plu]['PRICE'];
        $item_id = 299864;
        $size_id = 93827;

        // now order it
        $user = logTestUserIn($this->ids['user_id']);
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','the note');
        $order_data['tip'] = 2.22;
        $order_data['items'][0]['item_id'] = $item_id;
        $order_data['items'][0]['size_id'] = $size_id;
        $order_data['items'][0]['mods'] = array();
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2343650,'mod_quantity'=>1);
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2342984,'mod_quantity'=>1);
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2342981,'mod_quantity'=>1);
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2345641,'mod_quantity'=>1);

        $order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id);
        $this->assertEquals($expected_price_because_of_forced_combo,$order_resource->order_amt,"It should have a subtotal equal to the amount of the combo");
        $this->assertNotEquals($regular_price,$order_resource->order_amt,"It should NOT have a subtotal equal to the amount of the combo");

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'UE');
        $foundry_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $foundry_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        myerror_log($message_to_send_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);



        $item_xml = cleanUpDoubleSpacesCRLFTFromString('<ITEMREQUESTS>
                <ADDITEM QTY="1" ITEMID="8050451" COMBOMEALGROUPID="4001" COMBOMEALMENUITEMID="8050081" ORDERNOW="Y" PRICE="'.$order_resource->order_amt.'" FOR="">
                        <COMBOMODMAINITEM ITEMID="8027514" PRICE="" FOR="" />
                        <MODITEM QTY="1" ITEMID="8786353" MODCODE="1"/>
                        <MODITEM QTY="1" ITEMID="8786349" MODCODE="1"/>
                        <MODITEM QTY="1" ITEMID="8005088" MODCODE="1"/>
                </ADDITEM>
            </ITEMREQUESTS>');

        $this->assertContains($item_xml,$body,"It Should have the expected item xml as whats sent");

        $expected_token = "<token>".$merchant_resource->alphanumeric_id."</token>";
        $this->assertContains($expected_token,$body,"It should have the merchant alphanumeric id set as the token");


        $expected_payment = '<PAYMENTREQUESTS><ADDTENDER TENDERID="407" AMOUNT="'.$order_resource->grand_total_to_merchant.'" TIP="'.$order_resource->tip_amt.'"/></PAYMENTREQUESTS>';
        $this->assertContains($expected_payment,$body,"It should have had the payment node correct");
    }

    function testForcedMealComboOptionNotChosen()
    {
        //2pc Combo   8050021:8050480:8050486
        $merchant_resource = $this->ids['merchant_resource'];
        $regular_plu = '8050021';
        $meal_plu = '8050480';
        $combo_plu = '8050486';
        // get prices for each of the plus
        $result = $this->ids['imported_prices'];
        $regular_price = $result[$regular_plu]['PRICE'];
        $expected_price_because_of_meal = $result[$meal_plu]['PRICE'];
        $combo_price = $result[$combo_plu]['PRICE'];
        $item_id = 299869;
        $size_id = 93831;

        $user = logTestUserIn($this->ids['user_id']);
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','the note');
        $order_data['items'][0]['item_id'] = $item_id;
        $order_data['items'][0]['size_id'] = $size_id;
        $order_data['items'][0]['mods'] = array();
        //2343739,2343738,2343727
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2343739,'mod_quantity'=>1);
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2343738,'mod_quantity'=>1);
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2343727,'mod_quantity'=>1);


        $order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertEquals($expected_price_because_of_meal,$order_resource->order_amt,"It should have a subtotal equal to the amount of the combo UNLESS!!!!!  we start charging more for large soda choice");

        $this->assertNotEquals($regular_price,$order_resource->order_amt,"It should NOT have a subtotal equal to the regular price of the item");
        $this->assertNotEquals($combo_price,$order_resource->order_amt,"It should NOT have a subtotal equal to the amount of the combo");

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'UE');
        $foundry_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $foundry_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        myerror_log($message_to_send_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);


        $item_xml = cleanUpDoubleSpacesCRLFTFromString('<ITEMREQUESTS>
                <ADDITEM QTY="1" ITEMID="8050480" COMBOMEALGROUPID="4030" COMBOMEALMENUITEMID="8050021" ORDERNOW="Y" PRICE="'.$order_resource->order_amt.'" FOR="">
                        <MODITEM QTY="1" ITEMID="8786047" MODCODE="1"/>
                        <MODITEM QTY="1" ITEMID="8786043" MODCODE="1"/>
                        <MODITEM QTY="1" ITEMID="8005047" MODCODE="1"/>
                </ADDITEM>
            </ITEMREQUESTS>');

        $this->assertContains($item_xml,$body,"It Should have the expected item xml as whats sent");
    }

    function testForcedMealComboOptionChosen()
    {
        //2pc Combo   8050021:8050480:8050486
        $merchant_resource = $this->ids['merchant_resource'];
        $regular_plu = '8050021';
        $meal_plu = '8050480';
        $combo_plu = '8050486';
        // get prices for each of the plus
        $result = $this->ids['imported_prices'];
        $regular_price = $result[$regular_plu]['PRICE'];
        $meal_price = $result[$meal_plu]['PRICE'];
        $combo_price = $result[$combo_plu]['PRICE']+$this->ids['large_drink_upcharge'];
        $item_id = 299869;
        $size_id = 93831;

        $user = logTestUserIn($this->ids['user_id']);
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','the note');
        $order_data['items'][0]['item_id'] = $item_id;
        $order_data['items'][0]['size_id'] = $size_id;
        $order_data['items'][0]['mods'] = array();
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2343739,'mod_quantity'=>1);
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2343738,'mod_quantity'=>1);
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2343727,'mod_quantity'=>1);
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2343528,'mod_quantity'=>1);

        $order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertEquals($combo_price,$order_resource->order_amt,"It should have a subtotal equal to the amount of the combo UNLESS!!!!!  we start charging more for large soda choice");

        $this->assertNotEquals($regular_price,$order_resource->order_amt,"It should NOT have a subtotal equal to the regular price of the item");
        $this->assertNotEquals($meal_price,$order_resource->order_amt,"It should NOT have a subtotal equal to the amount of the meal");

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'UE');
        $foundry_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $foundry_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        myerror_log($message_to_send_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);


        $item_xml = cleanUpDoubleSpacesCRLFTFromString('<ITEMREQUESTS>
                <ADDITEM QTY="1" ITEMID="8050486" COMBOMEALGROUPID="4036" COMBOMEALMENUITEMID="8050021" ORDERNOW="Y" PRICE="'.$order_resource->order_amt.'" FOR="">
                        <MODITEM QTY="1" ITEMID="8786047" MODCODE="1"/>
                        <MODITEM QTY="1" ITEMID="8786043" MODCODE="1"/>
                        <MODITEM QTY="1" ITEMID="8005047" MODCODE="1"/>
                        <MODITEM QTY="1" ITEMID="8019002" MODCODE="1">
                            <MODMOD QTY="1" ITEMID="8019300" MODCODE="1"/>
                        </MODITEM>
                </ADDITEM>
            </ITEMREQUESTS>');

        $this->assertContains($item_xml,$body,"It Should have the expected item xml as whats sent");
    }

    function testItmeWithComboOptionNOTForcedNOTChosen()
    {
        //Chicken Black Bean Bowl

        //8048061:8048062
        $merchant_resource = $this->ids['merchant_resource'];
        $regular_plu = '8048061';
        $combo_plu = '8048062';
        // get prices for each of the plus
        $result = $this->ids['imported_prices'];
        $regular_price = $result[$regular_plu]['PRICE'];
        $combo_price = $result[$combo_plu]['PRICE'];
        $item_id = 299881;
        $size_id = 93843;
        $data['item_id'] = $item_id;
        $data['size_id'] = $size_id;
        $data['merchant_id'] = $merchant_resource->merchant_id;
        $price_record = ItemSizeAdapter::staticGetRecord($data,'ItemSizeAdapter');
        $this->assertEquals($regular_price,$price_record['price'],"It should have picked up the regular price");

        $user = logTestUserIn($this->ids['user_id']);
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','the note');
        $order_data['items'][0]['item_id'] = $item_id;
        $order_data['items'][0]['size_id'] = $size_id;
        $order_data['items'][0]['mods'] = array();
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2342996,'mod_quantity'=>1);

        $order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertEquals($regular_price,$order_resource->order_amt,"It should have a subtotal equal to the amount of the combo UNLESS!!!!!  we start charging more for large soda choice");

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'UE');
        $foundry_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $foundry_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        myerror_log($message_to_send_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);

        $this->assertContains('EXTCHECKID="12:20 PM"',$body);
        $this->assertContains('ORDERID="'.$order_resource->order_id.'"',$body);
        $item_xml = cleanUpDoubleSpacesCRLFTFromString('<ITEMREQUESTS>
                <ADDITEM QTY="1" ITEMID="8048061" ORDERNOW="Y" PRICE="'.$order_resource->order_amt.'" FOR="">
                        <MODITEM ITEMID="9999931" PRICE="" FOR="" />
                        <MODITEM QTY="0" ITEMID="8026311" MODCODE="1"/>
                        <MODITEM ITEMID="9999931" PRICE="" FOR="" />
                        <MODITEM QTY="0" ITEMID="8026002" MODCODE="1"/>
                        <MODITEM ITEMID="9999929" PRICE="" FOR="" />
                        <MODITEM QTY="1" ITEMID="9100322" MODCODE="1"/>
                        <MODITEM ITEMID="9999931" PRICE="" FOR="" />
                        <MODITEM QTY="0" ITEMID="8026014" MODCODE="1"/>
                        <MODITEM ITEMID="9999931" PRICE="" FOR="" />
                        <MODITEM QTY="0" ITEMID="9100006" MODCODE="1"/>
                        <MODITEM ITEMID="9999931" PRICE="" FOR="" />
                        <MODITEM QTY="0" ITEMID="9100508" MODCODE="1"/>
                        <MODITEM ITEMID="9999931" PRICE="" FOR="" />
                        <MODITEM QTY="0" ITEMID="8010117" MODCODE="1"/>
                        <MODITEM ITEMID="9999931" PRICE="" FOR="" />
                        <MODITEM QTY="0" ITEMID="9100501" MODCODE="1"/>
                </ADDITEM>
            </ITEMREQUESTS>');

        $this->assertContains($item_xml,$body,"It Should have the expected item xml as whats sent");

    }

    function testItmeWithComboOptionNOTForcedComboChosen()
    {
        //Chicken Black Bean Bowl
        // large drink: 8019002

        //8048061:8048062
        $merchant_resource = $this->ids['merchant_resource'];
        $regular_plu = '8048061';
        $combo_plu = '8048062';
        // get prices for each of the plus
        $result = $this->ids['imported_prices'];
        $regular_price = $result[$regular_plu]['PRICE'];
        $combo_price = $result[$combo_plu]['PRICE']+$this->ids['large_drink_upcharge'];
        $item_id = 299881;
        $size_id = 93843;
        $data['item_id'] = $item_id;
        $data['size_id'] = $size_id;
        $data['merchant_id'] = $merchant_resource->merchant_id;
        $price_record = ItemSizeAdapter::staticGetRecord($data,'ItemSizeAdapter');
        $this->assertEquals($regular_price,$price_record['price'],"It should have picked up the regular price");

        $user = logTestUserIn($this->ids['user_id']);
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','the note');
        $order_data['items'][0]['item_id'] = $item_id;
        $order_data['items'][0]['size_id'] = $size_id;
        $order_data['items'][0]['mods'] = array();
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2343498,'mod_quantity'=>1);



        $order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertEquals($combo_price,$order_resource->order_amt,"It should have a subtotal equal to the amount of the combo UNLESS!!!!!  we start charging more for large soda choice");
        $this->assertNOTEquals($regular_price,$order_resource->order_amt,"It should NOT have a subtotal equal to the amount of the combo");
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id,$m);
        $order_detail = $complete_order['order_details'][0];
        $this->assertEquals($regular_price,$order_detail['price'],'base price should be price without combo');
        $this->assertEquals($combo_price,$order_detail['item_total_w_mods'],"Item total should be the combo price");
        //$this->assertTrue(false,'Build test to validate order template with combos');

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'UE');
        $foundry_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $foundry_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        myerror_log($message_to_send_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);


        $item_xml = '<ADDITEM QTY="1" ITEMID="8048062" COMBOMEALGROUPID="12230" COMBOMEALMENUITEMID="8048061" ORDERNOW="Y" PRICE="9.08" FOR="">';
        $this->assertContains($item_xml,$body,"It Should have the expected item xml as whats sent");

        $mod_item_xml = cleanUpDoubleSpacesCRLFTFromString('<MODITEM QTY="1" ITEMID="8019002" MODCODE="1"><MODMOD QTY="1" ITEMID="8019300" MODCODE="1"/></MODITEM>');
        $this->assertContains($mod_item_xml,$body,"It Should have the expected mod item xml as whats sent");
    }

    function testSkinlessBreastException()
    {
        $item_id = 299872;
        $size_id = 93834;
        $merchant_resource = $this->ids['merchant_resource'];
        $user = logTestUserIn($this->ids['user_id']);
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','the note');
        $order_data['items'][0]['item_id'] = $item_id;
        $order_data['items'][0]['size_id'] = $size_id;
        $order_data['items'][0]['mods'] = array();
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2343718,'mod_quantity'=>1);
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2343726,'mod_quantity'=>1);
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2345657,'mod_quantity'=>1);
        $order_data['items'][0]['mods'][] = array('modifier_item_id'=>2345655,'mod_quantity'=>1);

        $order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id);
        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'UE');
        $foundry_controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $message_to_send_resource = $foundry_controller->prepMessageForSending($message_resource);
        $this->assertNotNull($message_to_send_resource,"Should have generated the message to send resource");
        myerror_log($message_to_send_resource->message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($message_to_send_resource->message_text);
        $this->assertNotContains('8026308',$body,"It should not have the regular item in the mod list");
        $this->assertNotContains('9999931',$body,"It should not have the ADD item in the mod list");
    }


    function testComboNOTForcedWeird()
    {
        $this->assertTrue(false,'Build test for Chicken Avocado Burrito');
    }



    static function setUpBeforeClass()
    {
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time',300);

        $menu_id = EplImporter::MENU_ID;
        $menu_adapter = new MenuAdapter($mimetypes);
        if ($menu_resource = Resource::find($menu_adapter,"$menu_id")){
            $menu_resource->version = 3.0;
            $menu_resource->save();
        } else {
            //create menu
            $menu_adapter->importMenu($menu_id,'prod','local',$brand_id);
            $menu_resource = Resource::find($menu_adapter,"$menu_id");
            $menu_resource->version = 3.0;
            $menu_resource->save();
        }

        getOrCreateSkinAndBrandIfNecessary('ElPolloLoco','ElPolloLoco',51,307);
        setContext('com.splickit.elpolloloco');

        mysqli_query("BEGIN");
        $merchant_resource = createNewTestMerchant();
        $merchant_resource->alphanumeric_id = '33r2vclgso59';
        $merchant_resource->alphanumeric_id = '700t2mvn209k';
        $merchant_resource->save();
        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'UE','Foundry','X');
        $data['merchant_id'] = $merchant_resource->merchant_id;
        $data['menu_id'] = $menu_id;
        // we do it this way to prevent the creation of daughter records till the import
        Resource::createByData(new MerchantMenuMapAdapter($m),$data);
        $raw = file_get_contents("./unit_tests_pos_integrations/resources/epl.xml");
        $request = createRequestObject('/messagemanager/foundry/getnextmessagebymerchantid','POST',$raw,'application/xml');
        $message_controller = ControllerFactory::generateFromUrl($request->url, $mimetypes, $user, $request, 5);
        $message_controller->pullNextMessageResourceByMerchant($a);

        $records = ModifierSizeMapAdapter::staticGetRecords(array("modifier_item_id"=>2344907),'ModifierSizeMapAdapter');



        $merchant_resource = createNewTestMerchant();
        $merchant_resource->alphanumeric_id = 'epl8888test';
        $merchant_resource->save();
        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'UE','Foundry','X');
        $data['merchant_id'] = $merchant_resource->merchant_id;
        $data['menu_id'] = $menu_id;
        // we do it this way to prevent the creation of daughter records till the import
        Resource::createByData(new MerchantMenuMapAdapter($m),$data);
        $ids['merchant_resource'] = $merchant_resource;
        $ids['merchant_id'] = $merchant_resource->merchant_id;

        $raw = file_get_contents("./unit_tests_pos_integrations/resources/EPLMenu.xml");

//      **********  THIS IS JUST TO PUSH A MENU TO STAGING
//        $raw = file_get_contents("./unit_tests_pos_integrations/resources/new_epl_menu_with_groups.xml");
//        $curl = curl_init("https://device.splickit-dev.com/app2/messagemanager/foundry/getnextmessagebymerchantid");
//        if (strtoupper('POST') == 'POST') {
//            curl_setopt($curl, CURLOPT_POSTFIELDS, $raw);
//            curl_setopt($curl, CURLOPT_POST, 1);
//            $headers[] = "Content-type: application/xml; charset=utf-8";
//            $headers[] = 'Content-Length: '.strlen($raw);
//            curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
//        }
//        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
//        $result = curl_exec($curl);
//        $info = curl_getinfo($curl);
//        curl_close($curl);
/*      ****************  END STAGING PUSH ******************/
        
        $request = new Request();
        $request->body = $raw;
        $request->method = 'POST';
        $request->mimetype = 'application/xml';
        $request->url = '/messagemanager/foundry/getnextmessagebymerchantid';
        $request->_parseRequestBody();
        $message_controller = ControllerFactory::generateFromUrl($request->url, $mimetypes, $user, $request, 5);
        $message_controller->pullNextMessageResourceByMerchant($a);
        $ids['imported_prices'] = $message_controller->importer->imported_prices;
        $ids['large_drink_upcharge'] = $message_controller->importer->large_drink_combo_upcharge;

        $user_resource = createNewUserWithCC();
        $ids['user_id'] = $user_resource->user_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        mysqli_query("ROLLBACK");
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    EPLFoundryImporterTest::main();
}

?>