<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');
require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class ElPolloLocoMerchantDataTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
        $this->ids = $_SERVER['unit_test_ids'];

    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }

    function testLoadImporter()
    {
        $url = "/apiv2/pos/import/elpolloloco/3214";
        $elpolloloco_importer = ImporterFactory::getImporterFromUrl($url);

        $this->assertTrue(is_a($elpolloloco_importer,'ElPolloLocoImporter'),"should have created a elpolloloco importer");
        $merchant_resource = $elpolloloco_importer->getLoadedMerchantResource();
        $this->assertEquals($this->ids['merchant_id'],$merchant_resource->merchant_id,"should have loaded the merchant as part of the factory");

        return $elpolloloco_importer;
    }
    /**
     * @depends testLoadImporter
     */
    function testImportRemoteData($importer){
        $merchant = MerchantAdapter::getMerchantFromExternalIdBrandId(3214, 307);
        $this->assertEquals("1234567890", $merchant->phone_no);
        $this->assertEquals("1234567890", $merchant->fax_no);
        $this->assertEquals("dummy@order140.com", $merchant->shop_email);
        $this->assertEquals("Y", $merchant->ordering_on);
        $this->isNull($merchant->custom_menu_message);
        $this->assertEquals("40.014726", $merchant->lat);
        $this->assertEquals("-105.274479", $merchant->lng);
        $this->assertEquals("-7", $merchant->time_zone);

        $importer->importRemoteMerchantMetaDataForLoadedMerchant();

        $merchant = MerchantAdapter::getMerchantFromExternalIdBrandId(3214, 307);
        $this->assertEquals("310-674-7924", $merchant->phone_no);
        $this->assertEquals("310-674-8037", $merchant->fax_no);
        $this->assertEquals("epl3214@wksusa.com", $merchant->shop_email);
        $this->assertEquals("Y", $merchant->ordering_on);
        $this->isNull($merchant->custom_menu_message);
        $this->assertEquals("33.945690", $merchant->lat);
        $this->assertEquals("-118.327370", $merchant->lng);
        $this->assertEquals("-8", $merchant->time_zone);
    }

    static function setUpBeforeClass()
    {
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time',300);

        mysqli_query("BEGIN");
        getOrCreateSkinAndBrandIfNecessary("elpolloloco","El Pollo Loco",142,307);
        $merchant_resource = getOrCreateNewTestMerchantBasedOnExternalId('3214');
        $merchant_resource->set("brand_id", 307);
        $merchant_resource->save();
        $ids['merchant_id'] = $merchant_resource->merchant_id;


        $hour_adapter = new HourAdapter($mimetypes);
        Resource::createByData($hour_adapter, array("merchant_id"=>$merchant_resource->merchant_id,"hour_type"=>'R','open'=>'10:00','close'=>'14:30'));

        $user_resource = createNewUser(array("flags"=>"1C20000001"));
        $ids['user_id'] = $user_resource->user_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        mysqli_query("ROLLBACK");
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    ElPolloLocoMerchantDataTest::main();
}

?>