<?php

error_reporting(E_ERROR | E_COMPILE_ERROR | E_COMPILE_WARNING | E_PARSE);
$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

$_SERVER['ENVIRONMENT'] = isset($_SERVER['XDEBUG_CONFIG']) ? "unit_test_ide" : "unit_test";
$_SERVER['DB_INFO_FILE_PATH'] = "config/smaw_database_unittest.conf";

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';


class EmagineTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $_SERVER['DO_NOT_RUN_CC'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
        $this->ids = $_SERVER['unit_test_ids'];

    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }

    function testGroupOrdervsSingleOrder()
    {
        $user = logTestUserIn($this->ids['user_id']);
        $merchant_id = $this->ids['merchant_id'];
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $cart_data['items'][] = $cart_data['items'][0];
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'NE');
        $emagine_controller = ControllerFactory::generateFromMessageResource($message_resource, $m, $u, $r);
        $resource = $emagine_controller->prepMessageForSending($message_resource);
        $body = $resource->emagine_order;

        $emails = "sumdumemail1@order140.com";

        $request = new Request();
        $request->data = array("merchant_id" => $merchant_id, "note" => "sum dum", "participant_emails" => $emails, "group_order_type" => 2, "submit_at_ts" => (getTomorrowTwelveNoonTimeStampDenver() + 900));
        $request->url = "app2/apiv2/grouporders";
        $request->method = "POST";
        $group_order_controller = new GroupOrderController($mt, $user, $request, 5);
        $group_order_controller->current_time = getTomorrowTwelveNoonTimeStampDenver();
        $group_order_resource = $group_order_controller->processV2Request();
        $this->assertNull($group_order_resource->error);
        $this->assertNotNull($group_order_resource);

        $this->assertNotNull($group_order_resource->group_order_token);
        $this->assertTrue($group_order_resource->group_order_id > 1000, "shouljd have a group order id");
        $this->assertEquals($user['user_id'], $group_order_resource->admin_user_id);

        $group_order_adapter = new GroupOrderAdapter($mimetypes);
        $group_order_record = $group_order_adapter->getRecordFromPrimaryKey($group_order_resource->group_order_id);

        $base_order_data = OrderAdapter::staticGetRecordByPrimaryKey($group_order_resource->order_id, 'OrderAdapter');

        $group_order_record['order_id'] = $group_order_resource->order_id;

        $group_order_token = $group_order_record['group_order_token'];
        $group_order_id = $group_order_record['group_order_id'];

        unset($cart_data['items'][1]);

        $cart_data['group_order_token'] = $group_order_token;

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $first_user_order_resource = placeOrderFromCheckoutResource($checkout_resource);
        $this->assertNull($first_user_order_resource->error);


        $user_resource = createNewUserWithCCNoCVV();
        $user_resource->first_name = 'Other';
        $user_resource->last_name = 'Guy';
        $user_resource->save();
        $user = logTestUserResourceIn($user_resource);

        $cart_data['user_id'] = $user['uuid'];
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $second_user_order_resource = placeOrderFromCheckoutResource($checkout_resource);
        $this->assertNull($second_user_order_resource->error);


        $group_order_activity_id = $group_order_record['auto_send_activity_id'];
        $send_group_order_activity = SplickitActivity::findActivityResourceAndReturnActivityObjectByActivityId($group_order_activity_id);
        $this->assertNotNull($send_group_order_activity);
        $class_name = get_class($send_group_order_activity);
        $this->assertEquals("SendGroupOrderActivity", $class_name);
        $this->assertTrue($send_group_order_activity->doit());

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($group_order_resource->order_id,'NE');
        $emagine_controller = ControllerFactory::generateFromMessageResource($message_resource, $m, $u, $r);
        $resource = $emagine_controller->prepMessageForSending($message_resource);
        $group_order_body = $resource->emagine_order;



    }


    function testSendOrder()
    {
        $user = logTestUserIn($this->ids['user_id']);
        $merchant_id = $this->ids['merchant_id'];
        $order_adapter = new OrderAdapter($m);
        $order_adapter->setMenuTypeIndexForTestOrders(2);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'sum dum note');
        $order_data['items'][0]['mods'] = array();
        $checkout_resource = getCheckoutResourceFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);

     //   $order_resource = placeOrderFromOrderData($order_data, getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $messages = MerchantMessageHistoryAdapter::getAllOrderMessages($order_id);
        $messages_hash = createHashmapFromArrayOfResourcesByFieldName($messages, 'message_format');
        $this->assertNotNull($messages_hash['NE'], "Should have created a message with the Emagine format");
        $message_resource = $messages_hash['NE'];
        $this->assertNotNull($message_resource);
        $this->assertEquals("N",$message_resource->locked,"the locked should have been set to 'N'");
        $this->assertTrue(true);
        $order_id = $message_resource->order_id;
        $this->assertNotNull($order_id);

        $emagine_controller = ControllerFactory::generateFromMessageResource($message_resource, $m, $u, $r);
        $response = $emagine_controller->sendThisMessage($message_resource);
        $this->assertTrue($response);

        // now check to see if the message was actually sent
        $new_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'N');
        $this->assertEquals("S",$new_message_resource->locked,"The locked should have been set to 'S'");
        $this->assertNotNull($new_message_resource->response,"the response should have been saved");
        $this->assertNotNull($new_message_resource->message_text,"the message text should hvae been saved");
        $pos_response = json_decode($new_message_resource->response);
        $this->assertTrue($pos_response->ok, "successful Order is sent to Emagine");
        $this->assertEquals("", $pos_response->failureMessage);
        $this->assertEquals(0, $pos_response->failureCode);
        $this->assertTrue($pos_response->porder_uid !== "", "Emagine response external order id");
        return $order_id;

    }
    /**
     * @depends testSendOrder
     */
    function testSendTwoOrdersWithSameOrderId($order_id)
    {
        $messages = MerchantMessageHistoryAdapter::getAllOrderMessages($order_id);
        $messages_hash = createHashmapFromArrayOfResourcesByFieldName($messages, 'message_format');
        $this->assertNotNull($messages_hash['N'], "Should have created a message with the Emagine format");
        $message_resource =  $messages_hash['N'];

        $emagine_controller = ControllerFactory::generateFromMessageResource($message_resource, $m, $u, $r);
        $response = $emagine_controller->sendThisMessage($message_resource);
        $this->assertTrue($response);
        $this->assertEquals("Emagine message failure: 'This order id has already been submitted: $order_id'. Emagine Error Code: 10398", $emagine_controller->getErrorMessage());
    }


    function testSendOrderFailure()
    {
        $user = logTestUserIn($this->ids['user_id']);
        $merchant_id = $this->ids['merchant_id'];
        $order_adapter = new OrderAdapter($m);
        $order_adapter->setMenuTypeIndexForTestOrders(2);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'sum dum note');
        $order_data['items'][0]['mods'] = array();
        $item_size_record = getStaticRecords($order_data['items'][0],'ItemSizeAdapter');
        $order_resource = placeOrderFromOrderData($order_data, getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'N');
        $emagine_controller = ControllerFactory::generateFromMessageResource($message_resource, $m, $u, $r);
        $readytosend_message_resource = $emagine_controller->prepMessageForSending($message_resource);
        $body = $readytosend_message_resource->message_text;
        $this->assertContains($item_size_record[0]['external_id'],$body);

        //now replace the external id with a random string to force a failure
        $fake_id = "qwertyu234567";
        $body = str_replace($item_size_record[0]['external_id'],$fake_id,$body);
        $this->assertContains($fake_id,$body,"It should now have the fake external id");
        $message_resource->set('message_text', $body);

        $response = $emagine_controller->sendThisMessage($message_resource);
        $this->assertTrue($response);

        $new_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'N');
        $this->assertEquals("N",$new_message_resource->locked,"the locked should have been set to 'N'");

        $this->assertEquals("Emagine message failure: 'MenuItem 'Prime Rib and Provolone' not found with external_id 'qwertyu234567''. Emagine Error Code: 10355", $emagine_controller->getErrorMessage());

    } 
    
    static function setUpBeforeClass()
    {
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time', 300);


      /************* pushing curitto menu to staging *************************
        $menu_id = 102762;
        $brand_id = 438;
        $skin_id = 151;

        $menu_adapter = new MenuAdapter($mimetypes);
        if ($menu_resource = Resource::find($menu_adapter,"$menu_id")){
        } else {
            $menu_adapter->importMenu($menu_id,'prod','test',$brand_id);
        }
     //  */



        $menu_id = 102431;
        $brand_id = 150;
        $skin_id = 5;

        $menu_adapter = new MenuAdapter($mimetypes);
        if ($menu_resource = Resource::find($menu_adapter,"$menu_id")){
        } else {
            $menu_adapter->importMenu($menu_id,'prod','local',$brand_id);
        }

        getOrCreateSkinAndBrandIfNecessary("Snarfs", "Snarf's", $skin_id, $brand_id);

        setContext('com.splickit.snarfs');
        $ids['skin_id'] = getSkinIdForContext();

        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_resource->group_ordering_on = 1;
        $merchant_resource->save();


        attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
        $ids['merchant_id'] = $merchant_resource->merchant_id;
        MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id, 'NE', 'emagine', 'X');

        Resource::createByData(new MerchantEmagineInfoMapsAdapter($mimetypes), array('merchant_id' => $merchant_resource->merchant_id, 'location' => "Location731", 'token' => "1234567890"));

        //$user_resource = createNewUser(array("flags" => "1C20000001"));
        $user_resource = createNewUserWithCCNoCVV();
        // create loyalty record
        $user_session_controller = new UsersessionController($m,$user_resource->getDataFieldsReally(),$r,5);
        $user_session_controller->getUserSession($user_resource);
        $ids['user_id'] = $user_resource->user_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }


    static function tearDownAfterClass()
    {
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    EmagineTest::main();
}