<?php
error_reporting(E_ERROR | E_COMPILE_ERROR | E_COMPILE_WARNING | E_PARSE);
$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

$db_info->database = 'smaw_unittest';
$db_info->username = 'root';
$db_info->password = 'splickit';
if (isset($_SERVER['XDEBUG_CONFIG'])) {
    putenv("SMAW_ENV=unit_test_ide");
    $db_info->hostname = "127.0.0.1";
    $db_info->port = 13306;
} else {
    $db_info->hostname = "db_container";
    $db_info->port = 3306;
}
$_SERVER['DB_INFO'] = $db_info;

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class PitaPitLoyaltyTestPOS extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
		setContext("com.splickit.pitapit");
    	setProperty('use_new_pita_pit_loyalty', 'true');
		
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }
    
    function testMenuExists()
    {
    	$menu_resource = Resource::find(new MenuAdapter($mimetypes), "".$this->ids['menu_id'], $options);
    	$this->assertNotNull($menu_resource);
    }
    
    function testDoNotReturnPointsOnMenuIfOldPitapitLoyaltyIsTrue()
    {
    	setProperty('use_new_pita_pit_loyalty', 'false');
    	$menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$this->ids['merchant_id']);
    	$menu_as_string = print_r($menu,true);
    	$this->assertNotContains('brand_points_id', $menu_as_string);
    }
    
    function testGetMenuWithPoints()
    {
        setProperty('use_new_pita_pit_loyalty', 'true');
    	$menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$this->ids['merchant_id']);
		$menu_as_string = print_r($menu,true);
		$menu_as_string = cleanUpCRLFTFromString($menu_as_string);
		$menu_as_string = str_replace("  ", '', $menu_as_string);
    	$this->assertContains('brand_points_id', $menu_as_string);
    	$item_1_data = $menu['menu_types'][1]['menu_items'][1]['size_prices'][0];
		$this->assertEquals(90, $item_1_data['points']);
		$this->assertEquals(1003, $item_1_data['brand_points_id']);
		
		$item_2_data = $menu['menu_types'][2]['menu_items'][1]['size_prices'][0];
		$this->assertEquals(90, $item_2_data['points']);
		$this->assertEquals(1003, $item_2_data['brand_points_id']);
		
		$item_3_data = $menu['menu_types'][3]['menu_items'][1]['size_prices'][0];
		$this->assertEquals(70, $item_3_data['points']);
		$this->assertEquals(1004, $item_3_data['brand_points_id']);
		
		$item_4_data = $menu['menu_types'][5]['menu_items'][1]['size_prices'][0];
		$this->assertEquals(15, $item_4_data['points']);
		$this->assertEquals(1006, $item_4_data['brand_points_id']);
    }
    
    function testLinkLoyaltyWithPin()
    {
    	$code = generateCode(10);
    	$email = $code."_sumdumemail@order140.com";
    	$hls = PitapitLoyaltyController::loyaltyServiceFactory();
    	$password = generateCode(8);
    	$heartland_user_result = $hls->createUserKatana($email,"$password");
    	$this->assertEquals('success', $heartland_user_result['status']);
    	$this->assertEquals(204,$heartland_user_result['http_code']);
    	$account_creation_result = $hls->createAccountKatana($email,"$password");
    	$this->assertEquals('success', $account_creation_result['status']);
    	$account_number = $account_creation_result['id'];
    	$this->assertNotNull($account_number);
    	$pin = $account_creation_result['pin'];

    	$user_resource = createNewUser(array("email"=>$email));
    	$user = logTestUserResourceIn($user_resource);
    	$uuid = $user['uuid'];
		$request = new Request();
    	$request->url = "apiv2/users/$uuid?log_level=5";
    	$request->method = 'POST';
    	$data['loyalty_number'] = "$account_number";
    	$data['pin'] = "$pin";
    	$request->data = $data;
    	$user_controller = new UserController($mt, $user, $request);
    	$resource = $user_controller->processV2Request();
    	$this->assertNull($resource->error,"should NOT have gotten an error.");
    	
    	$user = logTestUserIn($user['user_id']);
    	$ubpma = new UserBrandPointsMapAdapter($mimetypes);
    	$ubpm_record = $ubpma->getRecord(array("user_id"=>$user['user_id'],"brand_id"=>282));
		$this->assertNotNull($ubpm_record,"should have found a user brand points map record");
		$loyalty_number = $ubpm_record['loyalty_number'];
		$ln = explode(":", $loyalty_number);
		$this->assertCount(3, $ln);
		$this->assertNotNull($ln[0]);
		$this->assertEquals($account_number, $ln[0]);
		$this->assertNotNull($ln[1]);
		$this->assertEquals($pin, $ln[1]);
		$this->assertNotNull($ln[2]);
		return $user;
    }
    
    /**
     * @depends testLinkLoyaltyWithPin
     */
    function testGetUserSessionWithFields($user)
    {
    	logTestUserIn($user['user_id']);
    	$uuid = $user['uuid'];
    	$request = new Request();
    	$request->url = "apiv2/users/$uuid?log_level=5";
    	$request->method = 'GET';
    	$user_controller = new UserController($mt, $user, $request);
    	$usersession_resource = $user_controller->processV2Request();
		$this->assertNotNull($usersession_resource->user_id);
		$this->assertNotNull($usersession_resource->brand_loyalty);
		Resource::encodeResourceIntoTonicFormat($usersession_resource);
    }
    
    function testLinkLoyaltyWithBadPin()
    {
    	$code = generateCode(10);
    	$email = $code."_sumdumemail@order140.com";
    	$hls = PitapitLoyaltyController::loyaltyServiceFactory();
    	$password = generateCode(8);
    	$heartland_user_result = $hls->createUserKatana($email,"$password");
    	$this->assertEquals('success', $heartland_user_result['status']);
    	$this->assertEquals(204,$heartland_user_result['http_code']);
    	$account_creation_result = $hls->createAccountKatana($email,"$password");
    	$this->assertEquals('success', $account_creation_result['status']);
    	$account_number = $account_creation_result['id'];
    	$this->assertNotNull($account_number);
    	$pin = $account_creation_result['pin'];
    	
    	$user_resource = createNewUser(array("email"=>$email));
    	$user = logTestUserResourceIn($user_resource);
    	$uuid = $user['uuid'];
		$request = new Request();
    	$request->url = "apiv2/users/$uuid?log_level=5";
    	$request->method = 'POST';
    	$data['loyalty_number'] = "$account_number";
    	$data['pin'] = "1234";
    	$request->data = $data;
    	$user_controller = new UserController($mt, $user, $request);
    	$resource = $user_controller->processV2Request();
    	$this->assertNotNull($resource->error,"should have gotten an error due to bad pin");
    	$this->assertEquals("Incorrect PIN/Account combination.", $resource->error);
    	$this->assertEquals(400,$resource->http_code);
    }

	function testJoinLoyaltyFailure()
	{
		$user_resource = createNewUser();
		$user = logTestUserResourceIn($user_resource);

		$hls = PitapitLoyaltyController::loyaltyServiceFactory();
		$response = $hls->createPitCardUser($user_resource->email, "welcome");

		$request = new Request();
		$data['create_loyalty_account'] = true;
		$request->body = json_encode($data);
		$request->method = 'POST';
		$request->mimetype = 'application/json';
		$request->url = "apiv2/users/".$user['uuid'];
		$request->_parseRequestBody();
		$user_controller = new UserController($mt, $user, $request);
		$response = $user_controller->processV2Request();
		$this->assertNotNull($response->error,"should have gotten an error");
		$this->assertContains("email is already registered",$response->error);
	}


	function testJoinLoyalty()
    {
    	$user = logTestUserIn($this->ids['user_id']);

    	$request = new Request();
    	$data['create_loyalty_account'] = true;
    	$request->body = json_encode($data);
    	$request->method = 'POST';
    	$request->mimetype = 'application/json';
    	$request->url = "apiv2/users/".$user['uuid'];
    	$request->_parseRequestBody();
    	$user_controller = new UserController($mt, $user, $request);
    	$response = $user_controller->processV2Request();
    	
    	$user = logTestUserIn($user['user_id']);
    	$ubpma = new UserBrandPointsMapAdapter($mimetypes);
    	$ubpm_record = $ubpma->getRecord(array("user_id"=>$user['user_id'],"brand_id"=>282));
		$this->assertNotNull($ubpm_record,"should have found a user brand points map record");
		$loyalty_number = $ubpm_record['loyalty_number'];
		$ln = explode(":", $loyalty_number);
		$this->assertCount(3, $ln);
		$this->assertNotNull($ln[0]);
		$this->assertNotNull($ln[1]);
		$this->assertNotNull($ln[2]);
    }

    function testPlaceOrderGetLoyaltyPointsOnRemoteSystem()
    {
    	$user = logTestUserIn($this->ids['user_id']);
    	$order_data = OrderAdapter::staticGetSimpleOrderArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'skip hours');
        $_SERVER['DO_NOT_RUN_CC'] = 'true';
    	$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
        $_SERVER['DO_NOT_RUN_CC'] = 'false';
    	$this->assertNull($order_resource->error);
    	$this->assertTrue($order_resource->order_id > 999);
    	$complete_order = CompleteOrder::getBaseOrderData($order_resource->order_id, $mimetypes);
    	$pita_pit_loyalty_controller = new PitapitLoyaltyController($mimetypes, $user, $request);
    	$point_value_of_order = $pita_pit_loyalty_controller->getPointsEarnedFromCompleteOrder($complete_order);
    	
    	// now get a usersession and see if there is a loyalty section
    	$user = logTestUserIn($this->ids['user_id']);
    	$usersession_controller = new UsersessionController($mt, $user, $r);
    	$user_resource = SplickitController::getResourceFromId($user['user_id'], 'User');
    	$user_session = $usersession_controller->getUserSession($user_resource);
    	$this->assertNotNull($user_session->brand_loyalty);
    	$brand_loyalty = $user_session->brand_loyalty;
    	$this->assertEquals($point_value_of_order, $brand_loyalty['points']);
    }
    
    function testRedeemPointsFromTheMenuFailureThenSuccess()
    {
    	$user = logTestUserIn($this->ids['user_id']);
    	
    	// fake out our system so it will fail on the remote system
    	// NO, i think we're just going to go with our system has the true amount since it was gotten during the user session call.
/*    	$ubpma = new UserBrandPointsMapAdapter($mimetypes);
    	$ubpm_resource = $ubpma->getExactResourceFromData(array('user_id'=>$user['user_id'],'brand_id'=>282));
    	$ubpm_resource->points = 100;
    	$ubpm_resource->save();
*/    	
		$order_adapter = new OrderAdapter($mimetypes);
    	$order = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'skip hours');
    	$menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$merchant_resource->merchant_id);
    	$pita_item = $menu['menu_types'][1]['menu_items'][0]['size_prices'][0];
    	// now add the points stuff to the item
    	$item = $order['items'][0];
    	$item['item_id'] = $pita_item['item_id'];
    	$item['size_id'] = $pita_item['size_id'];
    	$item['mods'] = array();
    	
    	//$item['uuid'] = 10;
    	$item['points_used'] = 90;
    	$item['amt_off'] = $pita_item['price'];
    	$item['brand_points_id'] = 1003;
    	$order['items'][0] = $item;
    	$order['tip'] = '0.00';
    	$order['tax_amt'] = '0.00';
    	$order['sub_total'] = '0.00';
    	$order['grand_total'] = '0.00';
    	$order['total_points_used'] = 90;
    	$order['lead_time'] = 20;
    	$order['loyalty_number'] = $loyalty_number;
    	//$order['user_id'] = 20000;
    	$pickup_time = time()+1200;
    	$order['actual_pickup_time'] = $pickup_time;
    	$pickup_string = date('g:ia',$pickup_time);
    	$order_resource = placeOrderFromOrderData($order, getTodayTwelveNoonTimeStampDenver());
    	$this->assertNotNull($order_resource->error,"user does not have enough points so should have thrown an error");
    	$points_message = "We're sorry, but it appears you do not have enough points in your account to place this order.  If you feel you have received this message in error, please contact customer support";
    	$this->assertEquals($points_message, $order_resource->error);    	
    	
    	// so now add points the remote user log them in and try go palce the same order
    	$ubpma = new UserBrandPointsMapAdapter($mimetypes);
    	$ubpm_record = $ubpma->getRecord(array('user_id'=>$user['user_id'],'brand_id'=>282), $options);
    	$loyalty_number = $ubpm_record['loyalty_number'];
    	$original_balance = $ubpm_record['points'];
    	$s = explode(':',$loyalty_number);
    	$sva = $s[0];
    	if ($s[1]) {
    		$pin = $s[1];
    	}
    	$points = 100;
		$hls = PitapitLoyaltyController::loyaltyServiceFactory();
		$hls->setStore('splickitstore');
		$result = $hls->addPoints($sva,$pin,$points);
		$this->assertNull($result['error']);
		$this->assertEquals('success',$result['status'],"should have had success at adding points");
    	
		//now log in again and get a user session to reset points in our system (which is why we are going to avoid dealing with failures on redemptions)
		$user = logTestUserIn($user['user_id']);
		$user_session_controller = new UsersessionController($mt, $user, $r);
		$user_session = $user_session_controller->getUserSession(SplickitController::getResourceFromId($user['user_id'], 'User'));
		$this->assertNotNull($user_session->brand_loyalty,"should have found a brand loyalty");
		$brand_loyalty = $user_session->brand_loyalty;
		$points = $brand_loyalty['points'];
		$this->assertEquals(100+$original_balance, $points," shold have found 100 + $original_balance points");
		
		$pickup_time = time()+1200;
    	$order['actual_pickup_time'] = $pickup_time;
    	$pickup_string = date('g:ia',$pickup_time);
    	$order_resource = placeOrderFromOrderData($order, getTodayTwelveNoonTimeStampDenver());
    	$this->assertNull($order_resource->error);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 999,"shoudl have foudn a good order id");
    	
    }
    
    function testGetLoyaltyHistoryNoHistory()
    {
    	$user_resource = createNewUser();
    	$user = logTestUserResourceIn($user_resource);
    	$uuid = $user['uuid'];
    	$request = new Request();
    	$request->url = "apiv2/users/$uuid/loyalty_history?log_level=5";
    	$request->method = 'GET';
    	$user_controller = new UserController($mt, $user, $request);
    	$history_resource = $user_controller->processV2Request();
    	return $history_resource;
    }

    /**
     * @depends testGetLoyaltyHistoryNoHistory
     */
    function testNoHistoryIsStillAResource($history_resource)
    {
    	$this->assertNotNull($history_resource);
    }
    
    /**
     * @depends testGetLoyaltyHistoryNoHistory
     */
    function testNoHistoryCanBeProcessed($history_resource)
    {
    	$response = getV2ResponseWithJsonFromResource($history_resource, $headers);
    	$this->assertTrue(is_a($response, 'Response'));

        //check for no double data
        $reponse_body_as_array = json_decode($response->body,true);
        $this->assertCount(0,$reponse_body_as_array['data'],"should have found an empty array");

    }
    
    function testPayWithPitCardStoredValue()
    {
    	$user_resource = UserAdapter::getUserResourceFromId($this->ids['user_id']);
    	$user_resource->flags = '1000000001';
    	$user_resource->save();
		$user = logTestUserIn($user_resource->user_id);
		$this->assertEquals('1000000001', $user['flags']);
    	$ubpma = new UserBrandPointsMapAdapter(getM());
    	$ubpm_record = $ubpma->getRecord(array("user_id"=>$user['user_id'],"brand_id"=>282));
		$this->assertNotNull($ubpm_record,"should have found a user brand points map record");
		$loyalty_number = $ubpm_record['loyalty_number'];
		$ln = explode(":", $loyalty_number);
		$result = $this->loadStoredValue($user, 50.00, $ln[0], $ln[1]);
		$this->assertEquals(200, $result['http_code']);
		
		$order_adapter = new OrderAdapter(getM());
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'skip hours');
    	$order_data['merchant_payment_type_map_id'] = $this->ids['merchant_payment_map_id_for_pitcard'];
    	$checkout_resource = getCheckoutResourceFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
    	$this->assertNull($checkout_resource->error);
    	$checkout_resource->accepted_payment_types = [array_pop($checkout_resource->accepted_payment_types)];
    	$order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$this->ids['merchant_id'],0.00);
		$this->assertNull($order_resource->error,"Should have placed a good order");
    	$order_id = $order_resource->order_id;
    	$this->assertEquals('PitCardStoredValuePaymentService', $order_resource->payment_service_used,"Payment should have been made with PitCardStoredValuePaymentService");

    }
    
    function loadStoredValue($user,$amount,$sva,$pin)
    {
    	$hls = PitapitLoyaltyController::loyaltyServiceFactory();
        $hls->setStore('splickitstore');
        $result = $hls->addDollars($sva,$pin,$amount);
    	return $result;
    }
    
/*    
    function testPlaceOrderWithPoints()
    {
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$_SERVER['REQUEST_METHOD'] = 'POST';
    	$loyalty_number = '12345623488888';
    	$user_resource = createNewUser();
    	setSessionProperty('is_a_user_update', 'true');
    	$user_resource->loyalty_number = $loyalty_number;
    	$user_resource->save();
		$user = logTestUserResourceIn($user_resource);
		$user_id = $user['user_id'];
		
		$ubpma = new UserBrandPointsMapAdapter($mimetypes);
    	$ubpm_record = $ubpma->getRecord(array("user_id"=>$user_id,"brand_id"=>282));

		// now adjust point values
		$user_brand_points_map_resource = Resource::find(new UserBrandPointsMapAdapter($mimetypes),$ubpm_record['map_id']);
		$this->assertEquals(1500, $user_brand_points_map_resource->points);
		    	
		$order_adapter = new OrderAdapter($mimetypes);
    	$order = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'skip hours');

    	$menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$this->ids['merchant_id']);
		$item_1_data = $menu['menu_types'][1]['menu_items'][1]['size_prices'][0];
		$price = $item_1_data['price'];
		$item_id = $item_1_data['item_id'];
		$size_id = $item_1_data['size_id'];
		
    	// now add the points stuff to the item
    	$item = $order['items'][0];
    	
    	$item['uuid'] = 10;
    	$item['item_id'] = $item_id;
    	$item['size_id'] = $size_id;
    	$item['mods'] = array();
    	$item['points_used'] = 90;
    	$item['amt_off'] = $price;
    	$item['brand_points_id'] = 1003;
    	$order['items'][0] = $item;
    	$order['tip'] = '0.00';
    	$order['tax_amt'] = '0.00';
    	$order['sub_total'] = '0.00';
    	$order['grand_total'] = '0.00';
    	$order['total_points_used'] = 90;
    	$order['lead_time'] = 20;
    	$order['loyalty_number'] = $loyalty_number;
    	$pickup_time = time()+1200;
    	$order['actual_pickup_time'] = $pickup_time;
    	$pickup_string = date('g:ia',$pickup_time);
    	$order_resource = placeOrderFromOrderData($order, getTodayTwelveNoonTimeStampDenver());
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_resource->order_id > 1000,"should have created a good order id but got: ".$order_resource->error);
    	$this->assertNull($order_resource->error);
    	
    	$this->assertEquals(0.00, $order_resource->order_amt);
    	$this->assertEquals(0.00, $order_resource->total_tax_amt);
    	
		// now check to see if the correct amount was recorded in the history		
		$ublh_adapter = new UserBrandLoyaltyHistoryAdapter($mimetypes);
		$ublh_record = $ublh_adapter->getRecord(array("user_id"=>$user_id,"order_id"=>$order_id));
		$this->assertNotNull($ublh_record);
		$this->assertEquals(90, $ublh_record['points_redeemed']);		
    	
    }
*/

    function testGetLoyaltyHistory()
    {
        sleep(4); // to let the history on the remote server update
        $user = logTestUserIn($this->ids['user_id']);
        $uuid = $user['uuid'];
        $request = new Request();
        $request->url = "apiv2/users/$uuid/loyalty_history?log_level=5";
        $request->method = 'GET';
        $user_controller = new UserController($mt, $user, $request);
        $history_resource = $user_controller->processV2Request();
        $this->assertNotNull($history_resource->data);
        $history = $history_resource->data;
        $this->assertTrue(count($history) > 1,"Should have found more than 1 history rows but found: ".count($history));
        foreach ($history as $transaction_record) {
            $this->assertCount(3, $transaction_record);
            $this->assertEquals(date('Y-m-d'), $transaction_record['transaction_date']);
            //$this->assertNotNull($transaction_record['transaction_date']);
            $this->assertNotNull($transaction_record['description']);
            if (strtolower($transaction_record['description']) == 'balance inquiry') {
                $this->assertTrue(false,"Should not have found a balance inquiry record in the history but did");
            }
        }
        //Resource::encodeResourceIntoTonicFormat($usersession_resource);
    }


    static function setUpBeforeClass()
    {
    	$menu_id = 102326;
    	$brand_id = 282;
    	$ids['menu_id'] = $menu_id;
    	$menu_adapter = new MenuAdapter(getM());
		if ($menu_adapter->getRecord(array("menu_id"=>$menu_id))){
			; // do nothing
		} else {
			//create menu
	    	$menu_adapter->importMenu($menu_id,'prod','local',$brand_id);
		}
    	
    	$_SERVER['request_time1'] = microtime(true);
    	$_SERVER['NO_MOCKS'] = true;
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));

		mysqli_query("BEGIN");
		$menu_resource = Resource::find($menu_adapter,"$menu_id");
		$menu_resource->version = 2.0;
		$menu_resource->save();
		$merchant_resource = createNewTestMerchant($menu_id);
    	$merchant_resource->brand_id = $brand_id;
    	$merchant_resource->save();
    	$mhima = new MerchantHeartlandInfoMapsAdapter($mimetypes);
    	Resource::createByData($mhima, array("merchant_id"=>$merchant_resource->merchant_id,"heartland_store_id"=>"us-1000"));
    	$merchant_payment_type_map_resource = MerchantPaymentTypeMapsAdapter::createMerchantPaymentTypeMap($merchant_resource->merchant_id, 3000, $billing_entity_id);
    	$ids['merchant_payment_map_id_for_pitcard'] = $merchant_payment_type_map_resource->id;
    	$ids['merchant_id'] = $merchant_resource->merchant_id;

		$sqls[] = "UPDATE Brand2 SET loyalty='Y' WHERE brand_id = $brand_id";
    	//now add rows if they dont exist because shut off
        $sqls[] = "INSERT INTO `Brand_Points` VALUES (1003,282,90,'Pita standard/forkstyle','2014-07-25 16:11:37','2015-05-18 22:50:24','N')";
        $sqls[] = "INSERT INTO `Brand_Points` VALUES (1004,282,70,'Kids Pita','2014-07-25 16:12:09','2015-05-18 22:50:24','N')";
        $sqls[] = "INSERT INTO `Brand_Points` VALUES (1005,282,30,'Fountain Drink','2014-07-25 16:12:53','2015-05-18 22:50:24','N')";
        $sqls[] = "INSERT INTO `Brand_Points` VALUES (1006,282,15,'chips','2014-07-25 16:12:56','2015-05-18 22:50:24','N')";
        $sqls[] = "INSERT INTO `Brand_Points` VALUES (1007,282,10,'Cookie','2014-07-25 16:14:25','2015-05-18 22:50:24','N')";

        $sqls[] = "INSERT INTO Brand_Points_Object_Map VALUES (null,1003,'size',70337,now(),now(),'N')";
		$sqls[] = "INSERT INTO Brand_Points_Object_Map VALUES (null,1003,'size',70339,now(),now(),'N')";
		$sqls[] = "INSERT INTO Brand_Points_Object_Map VALUES (null,1003,'size',70343,now(),now(),'N')";
		$sqls[] = "INSERT INTO Brand_Points_Object_Map VALUES (null,1004,'size',70351,now(),now(),'N')";
		$sqls[] = "INSERT INTO Brand_Points_Object_Map VALUES (null,1005,'item',280977,now(),now(),'N')";
		$sqls[] = "INSERT INTO Brand_Points_Object_Map VALUES (null,1006,'size',70371,now(),now(),'N')";
		$sqls[] = "INSERT INTO Brand_Points_Object_Map VALUES (null,1007,'size',70357,now(),now(),'N')";
		$bpoma = new BrandPointsObjectMapAdapter($mimetypes);
		foreach ($sqls as $sql) {
			$bpoma->_query($sql);
		}

        setContext('com.splickit.pitapit');
    	$user_resource = createNewUser(array("flags"=>"1C20000001"));
    	$ids['user_id'] = $user_resource->user_id;
    	
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	mysqli_query("ROLLBACK");
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    PitaPitLoyaltyTestPOS::main();
}

?>