<?php
error_reporting(E_ERROR | E_COMPILE_ERROR | E_COMPILE_WARNING | E_PARSE);
$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

$_SERVER['ENVIRONMENT'] = isset($_SERVER['XDEBUG_CONFIG']) ? "unit_test_ide" : "unit_test";
$_SERVER['DB_INFO_FILE_PATH'] = "config/smaw_database_unittest.conf";


require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class SnarfsLoyaltyTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $merchant_id;
	var $menu_id;
	var $user_id;
	var $brand_points_id;
	var $user_brand_loyalty_map_id;
	var $size_id;
	var $ids;
	
	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $_SERVER['DO_NOT_RUN_CC'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
		setContext($this->ids['context']);
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->stamp);
		unset($this->ids);
    }

    function testEmagineFormatForLoyaltyItem()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $ubpma = new UserBrandPointsMapAdapter($mimetypes);
        $options[TONIC_FIND_BY_METADATA] = array("user_id"=>$user_resource->user_id,"brand_id"=>$brand_id);
        $ubpmr = Resource::find($ubpma,null,$options);
        $ubpmr->points = 1300;
        $ubpmr->save();

        setProperty("skip_first_menu_type",true);
        $order_adapter = new OrderAdapter($mimetypes);
        $cart_data = $order_adapter->getCartArrayWithOneModierPerModifierGroup($this->ids['merchant_id']);
        // now add the points stuff to the item
        $cart_data['items'][0]['points_used'] = 1200;
        $cart_data['items'][0]['amt_off'] = $cart_data['items'][0]['item_total_price'];
        $cart_data['items'][0]['brand_points_id'] = 1009;
        $cart_data['items'][0]['mods'][] = array('modifier_item_id'=>2258762,'mod_quantity'=>1);
        $cart_data['total_points_used'] = 1200;

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $this->assertEquals("1.50",$checkout_resource->order_amt,"Order amout shoudl only be the added extra modifier");

        $order_resource = placeOrderFromCheckoutResourceWithoutCCCall($checkout_resource);
        $this->assertNull($order_resource->error);

        $this->assertEquals(1.90,$order_resource->grand_total,"should be a grand total of 1.90 since only charged for the added modifier");

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'N');
        $emagine_controller = ControllerFactory::generateFromMessageResource($message_resource);
        $ready_to_send_message_resource = $emagine_controller->prepMessageForSending($message_resource);
        $message_text = $ready_to_send_message_resource->message_text;
        $message_array = json_decode($message_text,true);
        $item = $message_array['order']['order_details'][0];
        $hashmap_modifiers_by_name = createHashmapFromArrayOfArraysByFieldName($item['order_detail_modifiers'],'mod_name');
        $this->assertCount(5,$hashmap_modifiers_by_name,"There should be 5 modifier records");
        $this->assertEquals(0.00,$hashmap_modifiers_by_name['Bacon']['mod_price']);
        $this->assertEquals(1.50,$hashmap_modifiers_by_name['Artichoke']['mod_price']);

        // now validate that the item price reflects the amount taken off by points
        $this->assertEquals('0.00',$item['price']);
        $this->assertEquals("0.00",$item['item_total']);
        $this->assertEquals("1.50",$item['item_total_w_mods']);
        $this->assertEquals("0.15",$item['item_tax']);


    }

    function testGetLoyaltyHistoryNoHistory()
    {
        $user = logTestUserIn($this->ids['user_id']);
        $uuid = $user['uuid'];
        $url = "http://localhost/app2/apiv2/users/$uuid/loyalty_history?log_level=5";
        $request = new Request();
        $request->url = $url;
        $request->method = "get";
        $request->mimetype = 'application/json';
        $user_controller = new UserController($m,$user,$request,5);
        $response = $user_controller->processV2Request();
        $this->assertTrue(isset($response->data));
        $this->assertTrue(is_array($response->data));
        $this->assertTrue(count($response->data) == 0);
    }

    function testGetUserSessionWithLoyaltyInfo()
    {
        $user = logTestUserIn($this->ids['user_id']);
        $uuid = $user['uuid'];
        $url = "http://localhost/app2/apiv2/users/$uuid?log_level=5";
        $request = new Request();
        $request->url = $url;
        $request->method = "get";
        $request->mimetype = 'application/json';
        $user_controller = new UserController($m,$user,$request,5);
        $user_session_resource = $user_controller->processV2Request();
        $loyalty_user_session_data = $user_session_resource->brand_loyalty;
        $loyalty_number = $loyalty_user_session_data['loyalty_number'];
        $this->assertEquals(str_replace(' ','',$user['contact_no']),$loyalty_number," loyalty nubmer shoudl be phone nyubmer");
        $this->assertEquals(getBrandIdFromCurrentContext(), $loyalty_user_session_data['brand_id']);
        $this->assertTrue($loyalty_user_session_data['points'] == '0');
        $this->assertTrue(is_array($loyalty_user_session_data['loyalty_transactions']));

    }

    function testRecordPointsFromOrder()
    {
		$user = logTestUserIn($this->ids['user_id']);		
		$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'skip hours');
		$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
		$order_id = $order_resource->order_id;
		$this->assertTrue($order_id > 1000,"Should have created a valid order by got an error: ".$order_resource->error);
		return $order_resource;
    }

    /**
     * @depends testRecordPointsFromOrder
     */
    function testCorrectPointsAdded($order_resource)
    {
    	$user_id = $order_resource->user_id;
    	$ubpm_adapter = new UserBrandPointsMapAdapter($mimetypes);
    	$ubpm_record = $ubpm_adapter->getRecord(array("user_id"=>$user_id,"brand_id"=>$this->ids['brand_id']));
        $expected_points = round($order_resource->order_amt * 10);
		$this->assertEquals($expected_points, $ubpm_record['points'],"should have had 10x subtotal added");
    }
		
	/**
     * @depends testRecordPointsFromOrder
     */
    function testCorrectLoyaltyHistoryRecord($order_resource)
	{
		$order_id = $order_resource->order_id;
		$user_id = $order_resource->user_id;
		$ublh_adapter = new UserBrandLoyaltyHistoryAdapter(getM());
		$ublh_record = $ublh_adapter->getRecord(array("user_id"=>$user_id,"order_id"=>$order_id));
      $expected_points = round($order_resource->order_amt * 10);
		$this->assertEquals($expected_points, $ublh_record['points_added'],"should have a record of a single point being added");
    }

    function testPointsOnMenu()
    {
        $menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$this->ids['merchant_id'],2,'pickup');
        $better_menu = createHashmapFromArrayOfArraysByFieldName($menu['menu_types'],'menu_type_name');
        $sandwiches = $better_menu["Snarf's Sandwiches"];
        $this->assertEquals("1200",$sandwiches['menu_items'][0]['size_prices'][0]['points'],"sandwich should have had a 1200 points value");
        $this->assertEquals("1009",$sandwiches['menu_items'][0]['size_prices'][0]['brand_points_id'],"Tea should have brand points id of 1008");
     }

    function testPlaceOrderWithPointsAPIv1()
    {
        // we dont want to call to inspirepay
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';

        $user_id = $this->ids['user_id'];
        $user = logTestUserIn($user_id);

        // now adjust point values
        if ($user_brand_points_map_resource = Resource::find(new UserBrandPointsMapAdapter($mimetypes),"".$this->ids['user_brand_loyalty_map_id'])) {
            $user_brand_points_map_resource->points = 500;
            $user_brand_points_map_resource->save();
        }

        setProperty("skip_first_menu_type",true);
        $order_adapter = new OrderAdapter($mimetypes);
        $cart_data = $order_adapter->getCartArrayWithOneModierPerModifierGroup($this->ids['merchant_id']);
        $cart_data['items'][0]['mods'] = array(array_shift($cart_data['items'][0]['mods']));
        // now add the points stuff to the item
        $cart_data['items'][0]['points_used'] = 1200;
        $cart_data['items'][0]['amt_off'] = $cart_data['items'][0]['item_total_price'];
        $cart_data['items'][0]['brand_points_id'] = 1009;
        $cart_data['total_points_used'] = 1200;
        $cart_data['user_id'] = $user_id;

        $order_resource = placeOrderFromOrderDataAPIV1($cart_data,getTomorrowTwelveNoonTimeStampDenver());

        // order should fail;
        $this->assertNotNull($order_resource->error, "should have gotten an error");
        $this->assertEquals("We're sorry, but it appears you do not have enough points in your account to place this order.  If you feel you have received this message in error, please contact customer support", $order_resource->error);

        $user_brand_points_map_resource->points = 1250;
        $user_brand_points_map_resource->save();

        $order_resource = placeOrderFromOrderDataAPIV1($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $this->assertTrue($order_resource->order_id > 1000,"should have created a good order id but got: ".$order_resource->error);
        $this->assertNull($order_resource->error);

        $this->assertEquals(0.00, $order_resource->order_amt);
        $this->assertEquals(0.00, $order_resource->total_tax_amt);

        // now check to see if the correct amount was recorded in the history
        $brand_points_id = 1009;
        $brand_points_resource = Resource::find(new BrandPointsAdapter($mimetypes), "$brand_points_id", $options);

        $ublh_adapter = new UserBrandLoyaltyHistoryAdapter($mimetypes);
        $ublh_record = $ublh_adapter->getRecord(array("user_id"=>$user_id,"order_id"=>$order_id));
        $this->assertNotNull($ublh_record);
        $this->assertEquals($brand_points_resource->points, $ublh_record['points_redeemed']);

        $user_brand_points_map_resource = Resource::find(new UserBrandPointsMapAdapter($mimetypes),"".$this->ids['user_brand_loyalty_map_id']);
        $this->assertEquals(50, $user_brand_points_map_resource->points);
        return $order_resource->order_id;
    }


    function testPlaceOrderWithPoints()
    {
		// we dont want to call to inspirepay
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';

		$user_id = $this->ids['user_id'];
		$user = logTestUserIn($user_id);
		
		// now adjust point values
		if ($user_brand_points_map_resource = Resource::find(new UserBrandPointsMapAdapter($mimetypes),"".$this->ids['user_brand_loyalty_map_id'])) {
            $user_brand_points_map_resource->points = 500;
            $user_brand_points_map_resource->save();
        }

        setProperty("skip_first_menu_type",true);
		$order_adapter = new OrderAdapter($mimetypes);
    	$cart_data = $order_adapter->getCartArrayWithOneModierPerModifierGroup($this->ids['merchant_id']);
        $cart_data['items'][0]['mods'] = array(array_shift($cart_data['items'][0]['mods']));

    	// now add the points stuff to the item
    	$item = $cart_data['items'][0];
        $item['item_id'] = 276612;
        $item['size_id'] = 89543;
    	$item['points_used'] = 1200;
    	$item['amt_off'] = $amount_off;
    	$item['brand_points_id'] = 1009;
        $cart_data['items'][0] = $item;
        $cart_data['total_points_used'] = 1200;
        $request = createRequestObject('/app2/apiv2/cart/checkout','POST',json_encode($cart_data),'application/json');
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_resource = $place_order_controller->processV2Request();

    	// order should fail;
    	$this->assertNotNull($checkout_resource->error, "should have gotten an error");
    	$this->assertEquals("We're sorry, but it appears you do not have enough points in your account to place this order.  If you feel you have received this message in error, please contact customer support", $checkout_resource->error);
    	
    	$user_brand_points_map_resource->points = 1250;
    	$user_brand_points_map_resource->save();

    	$user = logTestUserIn($user['user_id']);
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_resource = $place_order_controller->processV2Request();
        $this->assertEquals(0.25,$checkout_resource->convenience_fee,"It should have a convenience_fee.");
        $this->assertEquals(2000,$checkout_resource->accepted_payment_types[0]['splickit_accepted_payment_type_id'],"Should have a CC payment type choice only");
        $ucid = $checkout_resource->cart_ucid;

        $complete_order = CompleteOrder::staticGetCompleteOrder($ucid);

        $order_data['merchant_id'] = $this->ids['merchant_id'];
        $order_data['note'] = 'sum dum note';
        $order_data['user_id'] = $user['user_id'];
        $order_data['cart_ucid'] = $ucid;
        $order_data['tip'] = 0.00;
        $order_data['merchant_payment_type_map_id'] = $checkout_resource->accepted_payment_types[0]['merchant_payment_type_map_id'];
        $order_data['actual_pickup_time'] = $checkout_resource->lead_times_array[0];
        // this should be ignored;
        $order_data['lead_time'] = 100000;

        $request = createRequestObject("/apiv2/orders/$ucid",'post',json_encode($order_data),'application/json');
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $order_resource = $place_order_controller->processV2Request();
        $this->assertNull($order_resource->error);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_resource->order_id > 1000,"should have created a good order id but got: ".$order_resource->error);
    	$this->assertNull($order_resource->error);
    	
    	$this->assertEquals(0.00, $order_resource->order_amt);
    	$this->assertEquals(0.00, $order_resource->total_tax_amt);
    	
		// now check to see if the correct amount was recorded in the history
		$brand_points_id = 1009;
		$brand_points_resource = Resource::find(new BrandPointsAdapter($mimetypes), "$brand_points_id", $options);
		
		$ublh_adapter = new UserBrandLoyaltyHistoryAdapter($mimetypes);
		$ublh_record = $ublh_adapter->getRecord(array("user_id"=>$user_id,"order_id"=>$order_id));
		$this->assertNotNull($ublh_record);
		$this->assertEquals($brand_points_resource->points, $ublh_record['points_redeemed']);		
    	
		$user_brand_points_map_resource = Resource::find(new UserBrandPointsMapAdapter($mimetypes),"".$this->ids['user_brand_loyalty_map_id']);
		$this->assertEquals(50, $user_brand_points_map_resource->points);
        return $order_resource->order_id;
    }

    /**
     * @depends testPlaceOrderWithPoints
     */
    function testListPointsInSummary($order_id)
    {
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
        $cart_items = $complete_order['order_summary']['cart_items'];
        $cart_item = $cart_items[0];
        $this->assertEquals('1200 pts',$cart_item['item_price']);
    }

    function testAddToCartWithPoints()
    {
        $user_id = $this->ids['user_id'];
        $user = logTestUserIn($user_id);

        // now adjust point values
        if ($user_brand_points_map_resource = Resource::find(new UserBrandPointsMapAdapter($mimetypes), "" . $this->ids['user_brand_loyalty_map_id'])) {
            $user_brand_points_map_resource->points = 1300;
            $user_brand_points_map_resource->save();

        }

        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleCartArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'skip hours');
        $amount_off = $order_data['sub_total'];
        // now add the points stuff to the item
        $item = $order_data['items'][0];
        $item['item_id'] = 276612;
        $item['size_id'] = 89543;
        $item['points_used'] = 1200;
        $item['amt_off'] = $amount_off;
        $item['brand_points_id'] = 1009;
        $order_data['items'][0] = $item;
        $order_data['total_points_used'] = 1200;

        $json_encoded_data = json_encode($order_data);
        $request = new Request();
        $request->url = '/app2/apiv2/cart';
        $request->method = "post";
        $request->body = $json_encoded_data;
        $request->mimetype = 'application/json';
        $request->_parseRequestBody();
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        //$cart_resource = $place_order_controller->createNewCart();
        $cart_resource = $place_order_controller->processV2Request();
        $this->assertNotNull($cart_resource, "should have gotten a cart resource back");
        //$this->assertTrue($cart_resource->insert_id > 999,"should have a valid cart id");
        $this->assertNotNull($cart_resource->ucid, "cart should have a unique identifier");
        return $cart_resource;
    }

    /**
     * @depends testAddToCartWithPoints
     */
    function testAddSecondItemWithoutPoints($cart_resource)
    {
        $user_id = $this->ids['user_id'];
        $user = logTestUserIn($user_id);

        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleCartArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'skip hours');

        $json_encoded_data = json_encode($order_data);
        $request = new Request();
        $request->url = '/app2/apiv2/cart/'.$cart_resource->ucid;
        $request->method = "post";
        $request->body = $json_encoded_data;
        $request->mimetype = 'application/json';
        $request->_parseRequestBody();
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        $cart_resource2 = $place_order_controller->processV2Request();
        $this->assertNotNull($cart_resource2, "should have gotten a cart resource back");
        $this->assertNotNull($cart_resource2->ucid, "cart should have a unique identifier");

        $order_summary = $cart_resource2->order_summary['cart_items'];
        $this->assertCount(2, $order_summary);
        $item = $order_summary[0];
        $this->assertNotNull($item['item_price'],"shoudl have found an item price");
        $this->assertEquals("1200 pts",$item['item_price']);
        $receipt_items_array = $cart_resource2->order_summary['receipt_items'];
        $receipt_items = createHashOfRecieptItemsByTitle($receipt_items_array);
        $this->assertEquals(1200,$receipt_items['Points Used'],"Should have had a points used section on the receipt summary");

        $item_two = $order_summary[1];
        $item_two_price = $item_two['item_price'];

        $this->assertEquals("$item_two_price", $receipt_items['Subtotal']);
        return $cart_resource2;
    }

    /**
     * @depends testAddSecondItemWithoutPoints
     */
    function testSummaryWithPointsFromAddToCart($cart_resource)
    {
        $order_summary = $cart_resource->order_summary['cart_items'];
        $this->assertCount(2, $order_summary);
        $item = $order_summary[0];
        $this->assertNotNull($item['item_price'],"shoudl have found an item price");
        $this->assertEquals("1200 pts",$item['item_price']);
        $receipt_items_array = $cart_resource->order_summary['receipt_items'];
        $receipt_items = createHashOfRecieptItemsByTitle($receipt_items_array);
        $this->assertEquals(1200,$receipt_items['Points Used'],"Should have had a points used section on the receipt summary");
    }

    /**
     * @depends testAddSecondItemWithoutPoints
     */
    function testGetCheckoutDataWithPoints($cart_resource)
    {
        $cart_ucid = $cart_resource->ucid;
        $user_id = $this->ids['user_id'];
        $user = logTestUserIn($user_id);

        $request = new Request();
        $request->url = "/apiv2/cart/$cart_ucid/checkout";
        $request->method = "get";
        $request->mimetype = 'application/json';

        $placeorder_controller = new PlaceOrderController($mt, $user, $request);
        $placeorder_controller->setCurrentTime(getTodayTwelveNoonTimeStampDenver());
        $checkout_data_resource = $placeorder_controller->processV2Request();
        $order_summary = $checkout_data_resource->order_summary['cart_items'];
        $this->assertCount(2, $order_summary);
        $item = $order_summary[0];
        $this->assertNotNull($item['item_price'],"shoudl have found an item price");
        $this->assertEquals("1200 pts",$item['item_price']);
        $receipt_items_array = $checkout_data_resource->order_summary['receipt_items'];
        $receipt_items = createHashOfRecieptItemsByTitle($receipt_items_array);
        $this->assertEquals(1200,$receipt_items['Points Used'],"Should have had a points used section on the receipt summary");
    }

// */    

    static function setUpBeforeClass()
    {
    	$menu_id = 102431;
    	$brand_id = 150;
    	$menu_adapter = new MenuAdapter($mimetypes);
        $blr_resource = Resource::findOrCreateIfNotExistsByData(new BrandLoyaltyRulesAdapter($m),array("brand_id"=>150,"loyalty_type"=>"splickit_points"));
        if ($menu_adapter->getRecord(array("menu_id"=>$menu_id))){
          ; // do nothing
        } else {
          //create menu
          $menu_adapter->importMenu($menu_id,'prod','local',$brand_id);

            //lets add bacon to the comes with for the italian sandwich sandwich
            $sql = "INSERT INTO Item_Modifier_Item_Map (`item_id`,`modifier_item_id`) VALUES (276612,2258762)";
            $imima = new ItemModifierItemMapAdapter($m);
            $imima->_query($sql);

        }
        $_SERVER['log_level'] = 5;
		//mysql_query("BEGIN");
    	$_SERVER['request_time1'] = microtime(true);
    	$skin_id = 5;
    	$name = "snarfs";
    	$ids['context'] = "com.splickit.$name";
    	$ids['brand_id'] = $brand_id;
    	$ids['skin_id'] = $skin_id;  

        $brand_resource = Resource::find(new BrandAdapter($m),"$brand_id");
        $brand_resource->loyalty = 'Y';
        $brand_resource->save();
        $ids['brand_loyalty_rules_id'] = $blr_resource->brand_loyalty_rules_id;
    	
    	$bp_records = BrandPointsAdapter::staticGetRecords(array("brand_id"=>$brand_id), 'BrandPointsAdapter');
    	$ids['brand_points_records'] = $bp_records;
    	
    	$ids['menu_id'] = $menu_id;
    	
    	$merchant_resource = createNewTestMerchant($menu_id,array("no_payment"=>true));
        $merchant_resource->external_id = createCode(10);
    	$merchant_resource->trans_fee_type = 'F';
        $merchant_resource->trans_fee_rate = .25;
        $merchant_resource->save();
        $ids['merchant_resource'] = $merchant_resource;
    	$ids['merchant_id'] = $merchant_resource->merchant_id;

        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id, 'N', 'emagine', 'X');
        $emagine_map = Resource::createByData(new MerchantEmagineInfoMapsAdapter($mimetypes), array('merchant_id' => $merchant_resource->merchant_id, 'location' => "Location888", 'token' => "9876543210"));


        setContext($ids['context']);
    	$user_resource = createNewUser(array('flags'=>'1C20000001'));
    	$ids['user_id'] = $user_resource->user_id;

        $ubpma = new UserBrandPointsMapAdapter($mimetypes);
    	$record = $ubpma->getRecord(array("user_id"=>$user_resource->user_id,"brand_id"=>$brand_id), $options);
    	$ids['user_brand_loyalty_map_id'] = $record['map_id'];


            $type = 'sage';
            $identifier = generateCode(20);
            $data['merchant_id_number'] = '469197833948';
            $data['merchant_id_key'] = 'T8L3P8O2X7I1';
            $vio_service = new VioPaymentService($data);
            $response = $vio_service->createDestination($type, $identifier, $data);
            $destination = $response['destination'];
            $be = BillingEntitiesAdapter::createBillingEntity(2002, 'test billing entity', 'test billing entity', $brand_id, $identifier, $credentials);
        $merchant_payment_type_map_resource = MerchantPaymentTypeMapsAdapter::createMerchantPaymentTypeMap($merchant_resource->merchant_id, 2000, $be->id);
            //$_SERVER['unit_test_ids']['destination_id'] = $identifier;
            $ids['vio_id'] = $identifier;

        $uuid = $user_resource->uuid;

        //{"brand": "visa", "number":"4242424242424242","cvv":"123","month":4,"year":2015,"first_name":"Ara","last_name":"Howard"}
        $data['identifier'] = $uuid;
        $data['brand'] = "visa";
        $data['number'] = "4111111111111111";
        $data['cvv'] = "123";
        $data['zip'] = "12345";
        $data['month'] = 10;
        $data['year'] = 2019;
        $data['first_name'] = "sumdum";
        $data['last_name'] = "guy";
        $vio_payment_service = new VioPaymentService($data);
        $response = $vio_payment_service->saveCreditCard($data);

    	$_SERVER['log_level'] = 5;
		$_SERVER['unit_test_ids'] = $ids;

    }
    
	static function tearDownAfterClass()
    {
    	mysqli_query("ROLLBACK");
    }    
    
    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    SnarfsLoyaltyTest::main();
}

?>