
<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class TullysLoyaltyTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $merchant_id;
	var $menu_id;
	var $user_id;
	var $brand_points_id;
	var $user_brand_loyalty_map_id;
	var $size_id;
	var $ids;
	
	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
		setContext($this->ids['context']);
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->stamp);
		unset($this->ids);
    }
    
    function testStartingPointsForTullys()
    {
    	$ubpma = new UserBrandPointsMapAdapter($mimetypes);
    	$record = $ubpma->getRecordFromPrimaryKey($this->ids['user_brand_loyalty_map_id']);
    	$this->assertEquals(10, $record['points']);
    }
    
    function testRecordPointsFromOrder()
    {
		$user = logTestUserIn($this->ids['user_id']);		
		$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'skip hours');
		$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
		$order_id = $order_resource->order_id;
		$this->assertTrue($order_id > 1000,"Should have created a valid order by got an error: ".$order_resource->error);
		return $order_resource;
    }
    
    /**
     * @depends testRecordPointsFromOrder
     */
    function testCorrectPointsAdded($order_resource)
    {
    	$user_id = $order_resource->user_id;
    	$ubpm_adapter = new UserBrandPointsMapAdapter($mimetypes);
    	$ubpm_record = $ubpm_adapter->getRecord(array("user_id"=>$user_id,"brand_id"=>$this->ids['brand_id']));
		$this->assertEquals(11, $ubpm_record['points'],"should have had a single point added");
    }
		
	/**
     * @depends testRecordPointsFromOrder
     */
    function testCorrectLoyaltyHistoryRecord($order_resource)
	{
		$order_id = $order_resource->order_id;
		$user_id = $order_resource->user_id;
		$ublh_adapter = new UserBrandLoyaltyHistoryAdapter($mimetypes);
		$ublh_record = $ublh_adapter->getRecord(array("user_id"=>$user_id,"order_id"=>$order_id));
		$this->assertEquals(1, $ublh_record['points_added'],"should have a record of a single point being added");		
    }
    
    function testRecordPointsFromOrderBySize()
    {
    	$user = logTestUserIn($this->ids['user_id']);
		$ubpm_adapter = new UserBrandPointsMapAdapter($mimetypes);
    	$ubpm_record_start = $ubpm_adapter->getRecord(array("user_id"=>$user['user_id'],"brand_id"=>$this->ids['brand_id']));
		$points_start = $ubpm_record_start['points'];		
		$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'skip hours');
    	// set it to the redbull item
    	$order_data['items'][0]['item_id']=287779;
    	$order_data['items'][0]['size_id']=93197;
		$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
		$order_id = $order_resource->order_id;
		$this->assertTrue($order_id > 1000,"Should have created a valid order but got an error: ".$order_resource->error);
		$ubpm_record = $ubpm_adapter->getRecord(array("user_id"=>$user['user_id'],"brand_id"=>$this->ids['brand_id']));
		$this->assertEquals($points_start+1, $ubpm_record['points'],"should have had a single point added");
    }
    
    function testRecordPointsFromOrderByItemNoPointsForIceWater()
    {
		$user = logTestUserIn($this->ids['user_id']);		
		$ubpm_adapter = new UserBrandPointsMapAdapter($mimetypes);
    	$ubpm_record_start = $ubpm_adapter->getRecord(array("user_id"=>$user['user_id'],"brand_id"=>$this->ids['brand_id']));
		$points_start = $ubpm_record_start['points'];		
		$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'skip hours');
    	// set it to the ice water item
    	$order_data['items'][0]['item_id']=288587;
    	$order_data['items'][0]['size_id']=93196;
		$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
		$order_id = $order_resource->order_id;
		$this->assertTrue($order_id > 1000,"Should have created a valid order but got an error: ".$order_resource->error);
		$ubpm_record = $ubpm_adapter->getRecord(array("user_id"=>$user['user_id'],"brand_id"=>$this->ids['brand_id']));
		$this->assertEquals($points_start, $ubpm_record['points'],"should have added NO points");
    }

    function testRecordPointsFromOrderByMenuTypeTea()
    {
        $user = logTestUserIn($this->ids['user_id']);
        $ubpm_adapter = new UserBrandPointsMapAdapter($mimetypes);
        $ubpm_record_start = $ubpm_adapter->getRecord(array("user_id"=>$user['user_id'],"brand_id"=>$this->ids['brand_id']));
        $points_start = $ubpm_record_start['points'];
        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'skip hours');
        // set it to the ice water item
        $order_data['items'][0]['item_id']=287749;
        $order_data['items'][0]['size_id']=92964;
        $order_resource = placeOrderFromOrderData($order_data, $time_stamp);
        $order_id = $order_resource->order_id;
        $this->assertTrue($order_id > 1000,"Should have created a valid order but got an error: ".$order_resource->error);
        $ubpm_record = $ubpm_adapter->getRecord(array("user_id"=>$user['user_id'],"brand_id"=>$this->ids['brand_id']));
        $this->assertEquals($points_start+1, $ubpm_record['points'],"should have added 1 point");
    }

//*

    function testPointsOnMenu()
    {
        $menu = CompleteMenu::getCompleteMenu($this->ids['menu_id'],'Y',$this->ids['merchant_id'],2,'pickup');
        $better_menu = createHashmapFromArrayOfArraysByFieldName($menu['menu_types'],'menu_type_name');
        $teas = $better_menu['Teas'];
        $this->assertEquals("10",$teas['menu_items'][0]['size_prices'][0]['points'],"tea should have had a 10 points value");
        $this->assertEquals("1008",$teas['menu_items'][0]['size_prices'][0]['brand_points_id'],"Tea should have brand points id of 1008");
     }

    function testPlaceOrderWithPoints()
    {
		// we dont want to call to inspirepay 
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		
		$user_id = $this->ids['user_id'];
		$user = logTestUserIn($user_id);		
		
		// now adjust point values
		$user_brand_points_map_resource = Resource::find(new UserBrandPointsMapAdapter($mimetypes),"".$this->ids['user_brand_loyalty_map_id']);
		$user_brand_points_map_resource->points = 5;
		$user_brand_points_map_resource->save();
		    	
		$order_adapter = new OrderAdapter($mimetypes);
    	$order = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'skip hours');
    	
    	// now add the points stuff to the item
    	$item = $order['items'][0];
    	$item['points_used'] = 10;
    	$item['amt_off'] = 1.65;
    	$item['brand_points_id'] = $this->ids['brand_points_id'];
    	$order['items'][0] = $item;
    	$order['tip'] = '0.00';
    	$order['grand_total'] = '0.00';
    	$order['total_points_used'] = 10;
    	$order['lead_time'] = 20;
    	$pickup_time = time()+1200;
    	$order['actual_pickup_time'] = $pickup_time;
    	$pickup_string = date('g:ia',$pickup_time);
    	$order_resource = placeOrderFromOrderData($order, $time_stamp);
    	// order should fail;
    	$this->assertNotNull($order_resource->error, "should have gotten an error");
    	$this->assertEquals("We're sorry, but it appears you do not have enough points in your account to place this order.  If you feel you have received this message in error, please contact customer support", $order_resource->error);
    	
    	$user_brand_points_map_resource->points = 11;
    	$user_brand_points_map_resource->save();

		$order_resource = placeOrderFromOrderData($order, $time_stamp);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_resource->order_id > 1000,"should have created a good order id but got: ".$order_resource->error);
    	$this->assertNull($order_resource->error);
    	
    	$this->assertEquals(0.00, $order_resource->order_amt);
    	$this->assertEquals(0.00, $order_resource->total_tax_amt);
    	
		// now check to see if the correct amount was recorded in the history
		$brand_points_id = $this->ids['brand_points_id'];
		$brand_points_resource = Resource::find(new BrandPointsAdapter($mimetypes), "$brand_points_id", $options);
		
		$ublh_adapter = new UserBrandLoyaltyHistoryAdapter($mimetypes);
		$ublh_record = $ublh_adapter->getRecord(array("user_id"=>$user_id,"order_id"=>$order_id));
		$this->assertNotNull($ublh_record);
		$this->assertEquals($brand_points_resource->points, $ublh_record['points_redeemed']);		
    	
		$user_brand_points_map_resource = Resource::find(new UserBrandPointsMapAdapter($mimetypes),"".$this->ids['user_brand_loyalty_map_id']);
		$this->assertEquals(1, $user_brand_points_map_resource->points);
    }

    function testAddToCartWithPoints()
    {
        $user_id = $this->ids['user_id'];
        $user = logTestUserIn($user_id);

        // now adjust point values
        $user_brand_points_map_resource = Resource::find(new UserBrandPointsMapAdapter($mimetypes), "" . $this->ids['user_brand_loyalty_map_id']);
        $user_brand_points_map_resource->points = 100;
        $user_brand_points_map_resource->save();

        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleCartArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'skip hours');

        // now add the points stuff to the item
        $item = $order_data['items'][0];
        $item['points_used'] = 10;
        $item['amt_off'] = 1.65;
        $item['brand_points_id'] = $this->ids['brand_points_id'];
        $order_data['items'][0] = $item;
        $order_data['total_points_used'] = 10;

        $json_encoded_data = json_encode($order_data);
        $request = new Request();
        $request->url = '/app2/apiv2/cart';
        $request->method = "post";
        $request->body = $json_encoded_data;
        $request->mimetype = 'application/json';
        $request->_parseRequestBody();
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        //$cart_resource = $place_order_controller->createNewCart();
        $cart_resource = $place_order_controller->processV2Request();
        $this->assertNotNull($cart_resource, "should have gotten a cart resource back");
        //$this->assertTrue($cart_resource->insert_id > 999,"should have a valid cart id");
        $this->assertNotNull($cart_resource->ucid, "cart should have a unique identifier");
        return $cart_resource;
    }

    /**
     * @depends testAddToCartWithPoints
     */
    function testAddSecondItemWithoutPoints($cart_resource)
    {
        $user_id = $this->ids['user_id'];
        $user = logTestUserIn($user_id);

        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleCartArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'skip hours');

        $json_encoded_data = json_encode($order_data);
        $request = new Request();
        $request->url = '/app2/apiv2/cart/'.$cart_resource->ucid;
        $request->method = "post";
        $request->body = $json_encoded_data;
        $request->mimetype = 'application/json';
        $request->_parseRequestBody();
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        $cart_resource2 = $place_order_controller->processV2Request();
        $this->assertNotNull($cart_resource2, "should have gotten a cart resource back");
        $this->assertNotNull($cart_resource2->ucid, "cart should have a unique identifier");

        $order_summary = $cart_resource2->order_summary['cart_items'];
        $this->assertCount(2, $order_summary);
        $item = $order_summary[0];
        $this->assertNotNull($item['item_price'],"shoudl have found an item price");
        $this->assertEquals("10 pts",$item['item_price']);
        $receipt_items_array = $cart_resource2->order_summary['receipt_items'];
        $receipt_items = createHashOfRecieptItemsByTitle($receipt_items_array);
        $this->assertEquals(10,$receipt_items['Points Used'],"Should have had a points used section on the receipt summary");
        $this->assertEquals('$2.00', $receipt_items['Subtotal']);
        $this->assertEquals('$0.20',$receipt_items['Tax']);
        return $cart_resource2;
    }

    /**
     * @depends testAddSecondItemWithoutPoints
     */
    function testSummaryWithPointsFromAddToCart($cart_resource)
    {
        $order_summary = $cart_resource->order_summary['cart_items'];
        $this->assertCount(2, $order_summary);
        $item = $order_summary[0];
        $this->assertNotNull($item['item_price'],"shoudl have found an item price");
        $this->assertEquals("10 pts",$item['item_price']);
        $receipt_items_array = $cart_resource->order_summary['receipt_items'];
        $receipt_items = createHashOfRecieptItemsByTitle($receipt_items_array);
        $this->assertEquals(10,$receipt_items['Points Used'],"Should have had a points used section on the receipt summary");
        $this->assertEquals('$2.00', $receipt_items['Subtotal']);
        $this->assertEquals('$0.20',$receipt_items['Tax']);
    }

    /**
     * @depends testAddSecondItemWithoutPoints
     */
    function testGetCheckoutDataWithPoints($cart_resource)
    {
        $cart_ucid = $cart_resource->ucid;
        $user_id = $this->ids['user_id'];
        $user = logTestUserIn($user_id);

        $request = new Request();
        $request->url = "/apiv2/cart/$cart_ucid/checkout";
        $request->method = "get";
        $request->mimetype = 'application/json';

        $placeorder_controller = new PlaceOrderController($mt, $user, $request);
        $placeorder_controller->setCurrentTime(getTodayTwelveNoonTimeStampDenver());
        $checkout_data_resource = $placeorder_controller->processV2Request();
        $order_summary = $checkout_data_resource->order_summary['cart_items'];
        $this->assertCount(2, $order_summary);
        $item = $order_summary[0];
        $this->assertNotNull($item['item_price'],"shoudl have found an item price");
        $this->assertEquals("10 pts",$item['item_price']);
        $receipt_items_array = $checkout_data_resource->order_summary['receipt_items'];
        $receipt_items = createHashOfRecieptItemsByTitle($receipt_items_array);
        $this->assertEquals(10,$receipt_items['Points Used'],"Should have had a points used section on the receipt summary");
        $this->assertEquals('$2.00', $receipt_items['Subtotal']);
        $this->assertEquals('$0.20',$receipt_items['Tax']);
    }

// */    

    static function setUpBeforeClass()
    {
    	ini_set('max_execution_time',300);
    	$menu_id = 102587;
    	$brand_id = 174;
    	$menu_adapter = new MenuAdapter($mimetypes);
		if ($menu_adapter->getRecord(array("menu_id"=>$menu_id))){
			; // do nothing
		} else {
			//create menu
			$menu_adapter->importMenu($menu_id,'prod','local',$brand_id);
		}
        $_SERVER['log_level'] = 5;
		mysqli_query("BEGIN");
    	$_SERVER['request_time1'] = microtime(true);
    	$skin_id = 500;
    	$name = "tullys";
    	$skin_resource = getOrCreateSkinAndBrandIfNecessaryWithLoyalty($name, $name, $skin_id, $brand_id);	
    	$ids['context'] = "com.splickit.$name";  
    	$ids['brand_id'] = $brand_id;
    	$ids['skin_id'] = $skin_id;  

		$blr_record = BrandLoyaltyRulesAdapter::staticGetRecord(array("brand_id"=>$brand_id), 'BrandLoyaltyRulesAdapter');
    	$ids['brand_loyalty_rules_id'] = $blr_record['brand_loyalty_rules_id'];
    	
    	$bp_record = BrandPointsAdapter::staticGetRecord(array("brand_id"=>$brand_id), 'BrandPointsAdapter');
    	$ids['brand_points_id'] = $bp_record['brand_points_id'];
    	
    	$ids['menu_id'] = $menu_id;
    	
    	$merchant_resource = createNewTestMerchant($menu_id);
    	$ids['merchant_id'] = $merchant_resource->merchant_id;
    	    	
    	setContext($ids['context']);
    	$user_resource = createNewUser(array('flags'=>'1C20000001'));
    	$ids['user_id'] = $user_resource->user_id;
    	
    	//get a user session to create eh loyalty record
    	$usc = new UsersessionController($mt, $user_resource->getDataFieldsReally(), $r, 5);
    	$user_session_resource = $usc->getUserSession($user_resource);

    	$ubpma = new UserBrandPointsMapAdapter($mimetypes);
    	$record = $ubpma->getRecord(array("user_id"=>$user_resource->user_id,"brand_id"=>$brand_id), $options);
    	$ids['user_brand_loyalty_map_id'] = $record['map_id'];

    	$_SERVER['log_level'] = 5;
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	mysqli_query("ROLLBACK");
    }    
    
    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    TullysLoyaltyTest::main();
}

?>