<?php
error_reporting(E_ERROR | E_COMPILE_ERROR | E_COMPILE_WARNING | E_PARSE);
$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

$_SERVER['ENVIRONMENT'] = isset($_SERVER['XDEBUG_CONFIG']) ? "unit_test_ide" : "unit_test";
$_SERVER['DB_INFO_FILE_PATH'] = "config/smaw_database_unittest.conf";

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';


class XoikosOrderTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $_SERVER['DO_NOT_RUN_CC'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
        $this->ids = $_SERVER['unit_test_ids'];
        setContext('com.splickit.goodcentssubs');

    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }

    function testGetActiveStausByStore()
    {

        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = getOrCreateNewTestMerchantBasedOnExternalId('20',$menu_id);
        $data = ["merchant"=>$merchant_resource->getDataFieldsReally()];
        $xoikos_service = new XoikosService($data);
        $xoikos_service->url = "http://orders.subshop.com/order.asmx";
        $this->assertTrue($xoikos_service->testStoreIsActive());

        $merchant_id = $merchant_resource->merchant_id;
        $url = "http://127.0.0.1:".$this->api_port."/app2/apiv2/merchants/$merchant_id";
        $response = $this->makeRequestWithDefaultHeaders($url, "store_tester@order140.com:st");


        $xoikos_service->merchant['merchant_external_id'] = '234567890';
        $this->assertFalse($xoikos_service->testStoreIsActive());
    }


    function testQuantityEnhancements()
    {
        $merchant_id = $this->ids['drive_through_merchant_id'];
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $order_data_other = OrderAdapter::staticGetSimpleOrderArrayByMerchantId($merchant_id);
        $order_data = $this->getOrderWithGoodcentsOriginal($merchant_id);
        $order_data['items'][0]['quantity'] = 3;
        $order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $this->assertTrue($order_id > 1000);
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
        $xoikos_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'XD');
        $xoikos_controller = ControllerFactory::generateFromMessageResource($xoikos_message_resource);
        $ready_to_send_message_resource = $xoikos_controller->prepMessageForSending($xoikos_message_resource);
        $message_text = $ready_to_send_message_resource->message_text;
        myerror_log($message_text);
        $expected_item_block = cleanUpDoubleSpacesCRLFTFromString("<Price>
                                    <ItemPrice>5.99</ItemPrice>
                                    <PriceBeforeDiscount>5.99</PriceBeforeDiscount>
                                    <BasePrice>5.99</BasePrice>
                                    <Tax>0.60</Tax>
                                </Price>");
        // just check the lines
        $this->assertContains("<ItemPrice>5.99</ItemPrice>",$message_text,"It should show the single price not the mulitple");
        $this->assertContains("<PriceBeforeDiscount>5.99</PriceBeforeDiscount>",$message_text,"It should show the single price not the mulitple");
        $this->assertContains("<BasePrice>5.99</BasePrice>",$message_text,"It should show the single price not the mulitple");
        $this->assertContains("<Tax>0.60</Tax>",$message_text,"It should show the single price not the mulitple");
    }

    function testMealDealForDriveThroughRestaurant()
    {
        $user = logTestUserIn($this->ids['user_id']);
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['drive_through_merchant_id'],'pickup',$note);
        $order_data['items'][0]['item_id'] = 299858;
        $order_data['items'][0]['size_id'] = 93826;
        $order_data['items'][0]['mods']  = array(array("mod_quantity"=>1,"modifier_item_id"=>2342933),array("mod_quantity"=>1,"modifier_item_id"=>2342917));
        $order_data['items'][0]['note'];
        $order_data['items'][1]['item_id'] = 299858;
        $order_data['items'][1]['quantity'] = 1;
        $order_data['items'][1]['size_id'] = 93826;
        $order_data['items'][1]['mods']  = array(array("mod_quantity"=>1,"modifier_item_id"=>2342933),array("mod_quantity"=>1,"modifier_item_id"=>2342917));
        $order_data['items'][1]['note']= "Item Drink Chips comment";
        $order_data['lead_time'] = 20;
        $order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $xoikos_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'XD');
        $this->assertNotNull($xoikos_message_resource);
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
        $xoikos_controller = ControllerFactory::generateFromMessageResource($xoikos_message_resource);
        $ready_to_send_message_resource = $xoikos_controller->prepMessageForSending($xoikos_message_resource);
        $message_text = $ready_to_send_message_resource->message_text;
        myerror_log($message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);
        $body_as_array = parseXMLintoHashmap(str_replace('&','+',$body));

        $this->assertCount(4,$body_as_array['OrderItems']['OrderItem'],"It should have 2 items");

        $expected_top_level_drink_node = cleanUpDoubleSpacesCRLFTFromString("<OrderItem>
<Qty>1</Qty>
<ItemName>Regular Coke </ItemName>
<Recipient></Recipient>
<ItemComments></ItemComments>
<POS>
<ROPE />
<FlexPOS />
<InfoKING>
<PLU>393</PLU>
</InfoKING>
</POS>
<Price>
<ItemPrice>1.95</ItemPrice>
<PriceBeforeDiscount>1.95</PriceBeforeDiscount>
<BasePrice>1.95</BasePrice>
<Tax>0.20</Tax>
</Price>
<Taxable>true</Taxable>
<TaxExempt>false</TaxExempt>
<Options>
<Option>
<OptionText>Drink</OptionText>
<SelectionText>Regular Coke</SelectionText>
<Price>0.00</Price>
<PriceEach>$0.00</PriceEach>
<Quantity>Regular</Quantity>
<Side>Full</Side>
<Status>Added</Status>
<POS>
<ROPE/>
<FlexPOS/>
<InfoKING>
<PLU>M45</PLU>
</InfoKING>
</POS>
</Option>
</Options>
</OrderItem>");
        $this->assertContains($expected_top_level_drink_node,$body,"It Should have the top level drink");

        $drive_through_top_level_mealdeal_node_with_chips = '<OrderItem><Qty>1</Qty><ItemName>Drink &amp; Chips</ItemName><Recipient></Recipient><ItemComments></ItemComments><POS><ROPE /><FlexPOS /><InfoKING><PLU>423</PLU></InfoKING></POS><Price><ItemPrice>0.90</ItemPrice><PriceBeforeDiscount>0.90</PriceBeforeDiscount><BasePrice>0.90</BasePrice><Tax>0.09</Tax></Price><Taxable>true</Taxable><TaxExempt>false</TaxExempt><Options><Option><OptionText>Chips</OptionText><SelectionText>Baked Lays</SelectionText><Price>0.00</Price><PriceEach>$0.00</PriceEach><Quantity>Regular</Quantity><Side>Full</Side><Status>Added</Status><POS><ROPE/><FlexPOS/><InfoKING><PLU>M36</PLU></InfoKING></POS></Option></Options></OrderItem>';
   //     $this->assertContains($drive_through_top_level_mealdeal_node_with_chips,$body,"It Should have a drive through mealdeal node");
        $this->assertNotContains("<PLU>393:M45</PLU>",$body,"It should not have the cobined node");

        //the following asserts check if the 'Note:' has been added correctly
        $this->assertContains("<ItemComments></ItemComments>", $body);
        $this->assertContains("<OrderComments>Note: skip hours</OrderComments>", $body);
        $this->assertContains("<ItemComments>Note: Item Drink Chips comment</ItemComments>", $body);
    }


    function testProcessPriceResource()
    {
        //test set the price to 4.98
        $options[TONIC_FIND_BY_METADATA] = array("size_id"=>93792,"item_id"=>299783,"merchant_id"=>$this->ids['merchant_id']);
        $price_resource = Resource::find(new ItemSizeAdapter($m),null,$options);
        if ($price_resource == null) {
            $options_other[TONIC_FIND_BY_METADATA] = array("size_id"=>93792,"item_id"=>299783,"merchant_id"=>"0");
            $price_resource = Resource::find(new ItemSizeAdapter($m),null,$options_other);
            $price_resource->_exists = false;
            $price_resource->item_size_id = null;
        }

        $xoikos_importer = new XoikosImporter('8500');
        $merchant = $xoikos_importer->getMerchant();
        $this->assertEquals($this->ids['merchant_id'],$merchant['merchant_id']);

        $plu_list["1"] = array("store"=>"8500","plu"=>"1","description"=>"8\" Goodcents Original","price"=>"4.98","taxable"=>"0");

        $xoikos_importer->imported_prices = $plu_list;

        $result = $xoikos_importer->processPriceResource($price_resource);

        $this->assertTrue($result,"should have resulted in a true");

        $price_resource = Resource::find(new ItemSizeAdapter($m),null,$options);
        $this->assertEquals(4.98,$price_resource->price);
        $this->assertEquals(0,$price_resource->tax_group);
    }

    function testProcessPriceResourceMealDealItemDrink()
    {
        $options[TONIC_FIND_BY_METADATA] = array("size_id"=>0,"modifier_item_id"=>2342679,"merchant_id"=>$this->ids['merchant_id']);
        $price_resource = Resource::find(new ModifierSizeMapAdapter($m),null,$options);
        if ($price_resource == null) {
            $options_other[TONIC_FIND_BY_METADATA] = array("size_id"=>0,"modifier_item_id"=>2342679,"merchant_id"=>"0");
            $price_resource = Resource::find(new ModifierSizeMapAdapter($m),null,$options_other);
            $price_resource->_exists = false;
            $price_resource->modifier_size_id = null;
        }
        $xoikos_importer = new XoikosImporter('8500');

        $plu_list["383"] = array("store"=>"8500","plu"=>"383","description"=>"Medium Drink Meal Deal","price"=>"0.20","taxable"=>"1");

        $xoikos_importer->imported_prices = $plu_list;

        $result = $xoikos_importer->processPriceResource($price_resource);

        $this->assertTrue($result,"should have resulted in a true");

        $price_resource = Resource::find(new ModifierSizeMapAdapter($m),null,$options);
        $this->assertEquals(0.20,$price_resource->modifier_price);
    }

    function testImportPricesBadID()
    {
        $external_id = '88888';
        $merchant_resource2 = getOrCreateNewTestMerchantBasedOnExternalId($external_id);
        Resource::createByData(new MerchantMenuMapAdapter($m),array("merchant_id"=>$merchant_resource2->merchant_id,"menu_id"=>$menu_id));
        $url = "/apiv2/pos/import/xoikos/$external_id";
        $xoikos_importer = ImporterFactory::getImporterFromUrl($url);
        $this->assertFalse($xoikos_importer->importRemoteMerchantMetaDataForLoadedMerchant());
    }

    function testImportPrices()
    {

        //$url = "/apiv2/pos/import/xoikos/8500";
        setContext('com.splickit.goodcentssubs');
        $created_merchant_resource = getOrCreateNewTestMerchantBasedOnExternalId('206');
        $created_merchant_resource->brand_id = 430;
        $created_merchant_resource->save();
        Resource::createByData(new MerchantMenuMapAdapter(getM()),array("merchant_id"=>$created_merchant_resource->merchant_id,"menu_id"=>$this->ids['menu_id']));


        $activity_history_adapter = new ActivityHistoryAdapter(getM());
        //$info = 'object=XoikosImporter;method=import;thefunctiondatastring=206';
        $info = 'object=XoikosImporter;method=import;thefunctiondatastring='.$created_merchant_resource->alphanumeric_id;
        $original_activity_history_resource = $activity_history_adapter->createActivityReturnActivityResource('ExecuteObjectFunction', time() - 5, $info, null);
        $id = $original_activity_history_resource->activity_id;

        $activity = $activity_history_adapter->getNextActivityToDo(null);
        $this->assertEquals($id, $activity->getActivityHistoryId());
        $activity->executeThisActivity();
        $activity_history_resource = SplickitController::getResourceFromId($id, "ActivityHistory");
        $this->assertEquals('E', $activity_history_resource->locked);


        $url = "/apiv2/pos/import/xoikos/206";
        $xoikos_importer = ImporterFactory::getImporterFromUrl($url);
        $this->assertTrue(is_a($xoikos_importer,'XoikosImporter'),"should have created a xoikos importer");
        $merchant_resource = $xoikos_importer->getLoadedMerchantResource();
        //$this->assertEquals($this->ids['merchant_id'],$merchant_resource->merchant_id,"should have loaded the merchant as part of the factory");

        $xoikos_importer->importRemoteMerchantMetaDataForLoadedMerchant();
        $number_item_prices = $xoikos_importer->getNumberOfItemPricesImported();
        $number_modifier_prices = $xoikos_importer->getNumberOfModifierPricesImported();

        $this->assertTrue($number_item_prices > 55,"should have imported a good number of item but only imported: $number_item_prices");
        $this->assertTrue($number_modifier_prices > 100,"Should have imported a good number of modifiers but only imported: $number_modifier_prices");

        $options[TONIC_FIND_BY_METADATA]['item_id'] = 299783;
        $options[TONIC_FIND_BY_METADATA]['size_id'] = 93792;
        $options[TONIC_FIND_BY_METADATA]['merchant_id'] = $this->ids['merchant_id'];

        $price_resource = Resource::find(new ItemSizeAdapter($m),'',$options);
        $this->assertTrue($price_resource->price > 5.00,"should have imported a good price");


        $sql = "SELECT a.map_id FROM Item_Modifier_Group_Map a JOIN Item b ON a.item_id = b.item_id JOIN Menu_Type c ON b.menu_type_id = c.menu_type_id WHERE c.menu_id = 102710 AND a.merchant_id = 0";
        $options2[TONIC_FIND_BY_SQL] = $sql;
        $resources = Resource::findAll(new ItemModifierGroupMapAdapter($m),'',$options2);
        $this->assertTrue(count($resources) > 50);

        //validate the IMGM records were created
        $records = ItemModifierGroupMapAdapter::staticGetRecords(array("merchant_id"=>$this->ids['merchant_id']),'ItemModifierGroupMapAdapter');
        $this->assertEquals(count($resources),count($records),"imported merchant should have had the same number of IMGM records");

    }

    function testASAPDelivery()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $json = '{"user_addr_id":null,"user_id":"' . $user['user_id'] . '","name":"","address1":"1045 Pine Street","address2":"","city":"boulder","state":"co","zip":"80302","phone_no":"9709262121","lat":40.0197891,"lng":-105.284703}';
        $request = new Request();
        $request->body = $json;
        $request->mimetype = "Application/json";
        $request->_parseRequestBody();
        $request->method = 'POST';
        $request->url = "/users/" . $user['uuid'] . "/userdeliverylocation";
        $user_controller = new UserController($mt, $user, $request, 5);
        $response = $user_controller->processV2Request();
        $this->assertNull($response->error, "should not have gotten a delivery save error but did");
        $this->assertNotNull($response->user_addr_id);
        $user_address_id = $response->user_addr_id;
        $user_delivery_location_resource = SplickitController::getResourceFromId($user_address_id, 'UserDeliveryLocation');

        $ids = $this->ids;
        $merchant_id = $ids['merchant_id'];

        $mdi_adapter = new MerchantDeliveryInfoAdapter($this->mimetypes);
        $mdi_options[TONIC_FIND_BY_METADATA]['merchant_id'] = $merchant_id;
        $mdi_resource = Resource::findExact($mdi_adapter,'',$mdi_options);
        $mdi_resource->allow_asap_on_delivery = 'Y';
        $mdi_resource->save();


        // url = isindeliveryarea
        $request = createRequestObject("/apiv2/merchants/$merchant_id/isindeliveryarea/$user_address_id?log_level=5",'GET');
        $merchant_controller = new MerchantController($mt, $user, $request, 5);
        $resource = $merchant_controller->processV2Request();

        $this->assertTrue(isset($resource->is_in_delivery_range),"should have found the 'is in delivery range' field");
        $this->assertTrue($resource->is_in_delivery_range," the is in delivery range should be true");

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'delivery');
        $cart_data['user_addr_id'] = $user_address_id;

        $json_encoded_data = json_encode($cart_data);
        $url = '/app2/apiv2/cart/checkout';
        $request = createRequestObject($url,'post',$json_encoded_data,'application/json');
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        //$place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($resource->error);

        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00,time());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;

        $xoikos_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'X');
        $xoikos_controller = ControllerFactory::generateFromMessageResource($xoikos_message_resource);
        $ready_to_send_message_resource = $xoikos_controller->prepMessageForSending($xoikos_message_resource);
        $message_text = $ready_to_send_message_resource->message_text;
        myerror_log($message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);

        $expected_asap_node = "<FulfillmentASAP>true</FulfillmentASAP>";
        $this->assertContains($expected_asap_node,$body,"should have created ASAP node");


    }



    function testCancelOrder()
    {
        //$user = logTestUserIn($this->ids['user_id']);
        $user_resource = createNewUserWithCCNoCVV();
        $user_resource->balance = 100;
        $user_resource->save();
        $user = logTestUserResourceIn($user_resource);
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'],'pickup',$note);
        $order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $ucid = $order_resource->ucid;
        $order_id = $order_resource->order_id;
        $this->assertTrue($order_id > 1000);

        $_SERVER['SOAPAction'] = "http://www.xoikos.com/webservices/CancelOrder";
        $xml = '<?xml version="1.0" encoding="utf-8"?><soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema"><soap:Body><CancelOrder xmlns="http://www.xoikos.com/webservices/"><OrderID>'.$ucid.'</OrderID></CancelOrder></soap:Body></soap:Envelope>';
        $request = new Request();
        $request->url = "/pos/xoikos";
        $request->method = "POST";
        $request->body = $xml;
        $request->mimetype = 'Applicationxml';


        $pos_controller = new PosController($mt,$u,$request,5);
        $resource = $pos_controller->processV2request();
        $this->assertTrue(isset($resource->send_soap_response),"should have set the soap flag on the resource");
        $expected_response = '<?xml version="1.0" encoding="utf-8"?><soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema"><soap:Body><CancelOrderResponse xmlns="http://www.xoikos.com/webservices/"><CancelOrderResult><Success>true</Success><ErrorMessage>Order with ID '.$ucid.' has been canceled.</ErrorMessage></CancelOrderResult></CancelOrderResponse></soap:Body></soap:Envelope>';
        $this->assertEquals($expected_response,$resource->soap_body,"Should have the expected soap body");

    }

    function testGetCompleteMenu()
    {
        $complete_menu = CompleteMenu::getCompleteMenu(102710,'Y',$this->ids['merchant_id'],2,'pickup');
        return $complete_menu;
    }

    /**
     * @depends testGetCompleteMenu
     */
    function testImportOfCookiesPrices($complete_menu)
    {
        $the_types = getRecordFromArrayOfArraysFromFieldAndValueOfSubArray($complete_menu['menu_types'],'menu_type_name','Sides');
        $menu_type = $the_types['menu_items'];
        $cookie_item = getRecordFromArrayOfArraysFromFieldAndValueOfSubArray($menu_type,'item_name','Cookies');
        $cookie_item_price = $cookie_item['size_prices'][0]['price'];
        $cookie_types = $cookie_item['modifier_groups'][0]['modifier_items'];
        foreach ($cookie_types as $cookie_type) {
            $this->assertEquals($cookie_item_price,$cookie_type['modifier_prices_by_item_size_id'][0]['modifier_price'],"price of modifier should have been price of cookie");
        }
        $chips_and_coookies_menu_type = getRecordFromArrayOfArraysFromFieldAndValueOfSubArray($complete_menu['menu_types'],'menu_type_id','47765');
        $price_override = $chips_and_coookies_menu_type['menu_items'][1]['modifier_groups'][0]['modifier_group_credit'];
        $this->assertEquals($cookie_item_price,$price_override,"Should have had a price override of the price of one cookie");
    }

    /**
     * @depends testGetCompleteMenu
     */
    function testImportOfChipsPrices($complete_menu)
    {
        $the_types = getRecordFromArrayOfArraysFromFieldAndValueOfSubArray($complete_menu['menu_types'],'menu_type_name','Sides');
        $menu_type = $the_types['menu_items'];
        $chip_item = getRecordFromArrayOfArraysFromFieldAndValueOfSubArray($menu_type,'item_name','Chips');
        $chip_item_price = $chip_item['size_prices'][0]['price'];
        $chip_types = $chip_item['modifier_groups'][0]['modifier_items'];
        foreach ($chip_types as $chip_type) {
            $this->assertEquals($chip_item_price,$chip_type['modifier_prices_by_item_size_id'][0]['modifier_price'],"price of modifier should have been price of chip");
        }
        $chips_and_coookies_menu_type = getRecordFromArrayOfArraysFromFieldAndValueOfSubArray($complete_menu['menu_types'],'menu_type_id','47765');
        $price_override = $chips_and_coookies_menu_type['menu_items'][0]['modifier_groups'][0]['modifier_group_credit'];
        $this->assertEquals($chip_item_price,$price_override,"Should have had a price override of the price of one cookie");
        //$this->assertEquals($chip_item_price,$complete_menu['menu_types']['7']['menu_items'][0]['modifier_groups'][0]['modifier_group_credit'],"Should have had a price override of the price of one chip");
    }

    /**
     * @depends testGetCompleteMenu
     */
    function testImportOfCookiesPricesForGoldBox($complete_menu)
    {
        $the_c = getRecordFromArrayOfArraysFromFieldAndValueOfSubArray($complete_menu['menu_types'],'menu_type_name','Catering');
        $catering_menu_items = $the_c['menu_items'];
        $the_mg = getRecordFromArrayOfArraysFromFieldAndValueOfSubArray($catering_menu_items,'item_name','Gold Box Lunch');
        $gold_box_modifier_groups = $the_mg['modifier_groups'];
        $the_cookies = getRecordFromArrayOfArraysFromFieldAndValueOfSubArray($gold_box_modifier_groups,'modifier_group_display_name','Cookies');
        $cookie_types = $the_cookies['modifier_items'];
        foreach ($cookie_types as $cookie_type) {
            $this->assertTrue(isset($cookie_type['modifier_prices_by_item_size_id'][0]['modifier_price']),"should have had the price record");
            $this->assertEquals(0.00,$cookie_type['modifier_prices_by_item_size_id'][0]['modifier_price'],"price of modifier should have been set to 0.00 for gold box size");
        }
    }

    /**
     * @depends testGetCompleteMenu
     */
    function testImportOfChipsPricesForGoldBox($complete_menu)
    {
        $the_c = getRecordFromArrayOfArraysFromFieldAndValueOfSubArray($complete_menu['menu_types'],'menu_type_name','Catering');
        $catering_menu_items = $the_c['menu_items'];
        $the_mg = getRecordFromArrayOfArraysFromFieldAndValueOfSubArray($catering_menu_items,'item_name','Gold Box Lunch');
        $gold_box_modifier_groups = $the_mg['modifier_groups'];
        $the_chips = getRecordFromArrayOfArraysFromFieldAndValueOfSubArray($gold_box_modifier_groups,'modifier_group_display_name','Chips');
        $chip_types = $the_chips['modifier_items'];
        foreach ($chip_types as $chip_type) {
            $this->assertTrue(isset($chip_type['modifier_prices_by_item_size_id'][0]['modifier_price']),"should have had the price record");
            $this->assertEquals(0.00,$chip_type['modifier_prices_by_item_size_id'][0]['modifier_price'],"price of modifier should have been set to 0.00 for gold box size: ".$chip_type['modifier_item_name']);
        }
    }


//    function testOverrideOfChipsAndCookiesOnGoldBox($complete_menu)
//    {
//        $chip_item_price = $complete_menu['menu_types']['7']['menu_items'][0]['size_prices'][0]['price'];
//        $cookie_item_price = $complete_menu['menu_types']['7']['menu_items'][1]['size_prices'][0]['price'];
//        $this->assertEquals($chip_item_price,$complete_menu['menu_types']['9']['menu_items'][2]['modifier_groups'][3]['modifier_group_credit'],"Should have had a price override of the price of one chip");
//        $this->assertEquals($cookie_item_price,$complete_menu['menu_types']['9']['menu_items'][2]['modifier_groups'][4]['modifier_group_credit'],"Should have had a price override of the price of one cookie");
//    }



    function testCreateXoikosMessageFromGoodCentsOrder()
    {
        $user_resource = Resource::find(new UserAdapter($m),$this->ids['user_id']);
        $user_resource->contact_no = "(212) 555-3333";
        $user_resource->first_name = "adam";
        $user_resource->last_name = "development";
        $user_resource->save();
        $user = logTestUserResourceIn($user_resource);
        $order_data = $this->getOrderWithGoodcentsOriginal($this->ids['merchant_id']);
        $order_resource = placeOrderFromOrderData($order_data,$time);
        $order_id = $order_resource->order_id;
        $this->assertTrue($order_id > 1000);
        $this->assertNull($order_resource->error);
        $xoikos_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'X');
        $this->assertNotNull($xoikos_message_resource);
        return $order_id;
    }

    function testPromoOnXoikosOrder()
    {
        $user = logTestUserIn($this->ids['user_id']);
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'],'pickup','skip hours',1);
        $order_data['items'][0]['mods'] = array();
        $order_data['items'][0]['item_id'] = 299800;
        $order_data['items'][0]['size_id'] = 93800;
        $order_data['tip'] = 0.00;
        //$order_data['note'] = 'skip hours';
        $checkout_resource = getCheckoutResourceFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $full_tax_amount = number_format(floatval($checkout_resource->total_tax_amt),2);
        $full_grand_total = number_format(floatval($checkout_resource->order_amt) + floatval($checkout_resource->total_tax_amt),2);

        $order_data['promo_code'] = 'Test Promo';
        $request = createRequestObject('/apiv2/cart/'.$checkout_resource->ucid.'?promo_code=Test+Promo','GET');
        $place_order_controller = new PlaceOrderController($m,$user,$request);
        $resource = $place_order_controller->processV2Request();
//        $checkout_resource = getCheckoutResourceFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
//        $checkout_data = $checkout_resource->getDataFieldsReally();
        $discount_tax_amount = number_format(floatval($resource->total_tax_amt),2);
        $this->assertNotEquals($full_tax_amount,$discount_tax_amount);


        $order_resource = placeOrderFromOrderData($order_data,$time);
        $order_id = $order_resource->order_id;
        $this->assertNull($order_resource->error);
        $this->assertEquals(-3.00,$order_resource->promo_amt);

        $xoikos_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'X');
        $xoikos_controller = ControllerFactory::generateFromMessageResource($xoikos_message_resource);
        $ready_to_send_message_resource = $xoikos_controller->prepMessageForSending($xoikos_message_resource);
        $message_text = $ready_to_send_message_resource->message_text;
        myerror_log($message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);

        $expected_discount_payload =   cleanUpDoubleSpacesCRLFTFromString("  <Discounts>
                                            <Discount>
                                                <DiscountCode>Test Promo</DiscountCode>
                                                <Amount>3.00</Amount>
                                            </Discount>
                                        </Discounts>");

        $this->assertContains($expected_discount_payload,$body,"should have created Discount payload");

        // now check to see if the full amounts are shown
        $this->assertContains("<TotalPrice>$full_grand_total</TotalPrice>",$body,"should have the full price of $full_grand_total");
        $this->assertContains("<Tax>$discount_tax_amount</Tax>",$body,"should have the full tax of $full_tax_amount");
        $this->assertContains("<PaymentType>CreditCard</PaymentType><Amount>".$order_resource->grand_total."</Amount>",$body,"should have the discounted amoutn for the payment amount");

        return $order_id;
    }

    function testXoikosOrderPayloadWithLoyalty()
    {
        $user_id = $this->ids['user_id'];
        $user_resource = Resource::find(new UserAdapter(getM()),"$user_id");
        $user_session_controller = new UsersessionController(getM(),$user_resource->getDataFieldsReally(),null,5);
        $user_session_controller->getUserSession($user_resource);

        $ubp_resource = Resource::find(new UserBrandPointsMapAdapter(getM()),"",array(TONIC_FIND_BY_METADATA=>array("user_id"=>$user_id,"brand_id"=>getBrandIdFromCurrentContext())));
        $ubp_resource->dollar_balance = 15.00;
        $ubp_resource->points = 1500;
        $ubp_resource->save();

        $item_size_adapter = new ItemSizeAdapter(getM());
        $sql = "select * From Item_Size_Map where merchant_id = ".$this->ids['merchant_id']." and active = 'Y' order by price desc";
        $options[TONIC_FIND_BY_SQL] = $sql;
        $resources = Resource::findAll($item_size_adapter,null,$options);
        $first_item = $resources[0];

        $user = logTestUserIn($user_id);
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getSimpleCartArrayByMerchantId($this->ids['merchant_id'],'pickup','skip hours',1);
        $order_data['items'][0]['mods'] = array();
        $order_data['items'][0]['item_id'] = $first_item->item_id;
        $order_data['items'][0]['size_id'] = $first_item->size_id;
        $order_data['tip'] = 0.00;

        $request = createRequestObject('/app2/apiv2/cart','post',json_encode($order_data),'application/json');
        $place_order_controller = new PlaceOrderController(getM(), $user, $request,5);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $cart_resource = $place_order_controller->processV2Request();
        $this->assertNull($cart_resource->error);
        $cart_ucid = $cart_resource->ucid;

        $request = createRequestObject("/app2/apiv2/cart/$cart_ucid/checkout",'get');
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $checkout_resource = $place_order_controller->processV2Request();

        $new_cart_note = "the new cart note";
        $order_data = array();
        $order_data['note'] = $new_cart_note;;
        $order_data['tip'] = 0.00;
        $payment_array = createHashmapFromArrayOfArraysByFieldName($checkout_resource->accepted_payment_types,'splickit_accepted_payment_type_id');
        // use  the loylaty plus credit card map
        $order_data['merchant_payment_type_map_id'] = $payment_array['8000']['merchant_payment_type_map_id'];
        $lead_times_array = $checkout_resource->lead_times_array;
        $order_data['actual_pickup_time'] = $lead_times_array[0];

        $request = createRequestObject("/apiv2/orders/$cart_ucid",'post',json_encode($order_data),'application/json');
        $place_order_controller = new PlaceOrderController($mt, $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $order_resource = $place_order_controller->processV2Request();
        $this->assertNull($order_resource->error);
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id);

        $order_summary = $order_resource->order_summary;
        $receipt_items_hash = createHashmapFromArrayOfArraysByFieldName($order_summary['receipt_items'],'title');

        $expected_subtotal = $first_item->price - 15.00;
        $expected_tax = .1 * $expected_subtotal;
        $expected_grand_total = $expected_subtotal + $expected_tax;
        $this->assertEquals("$".number_format($expected_subtotal,2),$receipt_items_hash['Subtotal']['amount']);
        $this->assertEquals("$".number_format($expected_tax,2),$receipt_items_hash['Tax']['amount']);
        $this->assertEquals("$".number_format($expected_grand_total,2),$receipt_items_hash['Total']['amount']);

        $payment_items = $order_summary['payment_items'];
        $this->assertNotNull($payment_items,'there should be the payment items section in the summary');
        $payment_items_hash = createHashmapFromArrayOfArraysByFieldName($payment_items,'title');
        //$this->assertEquals('$2.15',$payment_items_hash['Loyalty Balance']['amount']);
        $this->assertEquals('$'.number_format($order_resource->grand_total,2),$payment_items_hash[CompleteOrder::CC_CHARGED_LABEL]['amount']);
        $this->assertCount(1,$payment_items,"there should be one payemnt item");

        $loyalty_payment_service = $place_order_controller->getPaymentService();
        $expected_grand_total_to_merchant = $checkout_resource->order_amt + $checkout_resource->total_tax_amt - 15.00 - $loyalty_payment_service->tax_adjustment;
        $this->assertEquals($expected_grand_total_to_merchant,$order_resource->grand_total_to_merchant,"there should have been a grand total to merchant that was the difference");
        $order_id = $order_resource->order_id;

        //check to see if all records were added to balance change table
        $balance_change_records = BalanceChangeAdapter::staticGetRecords(array("order_id"=>$order_id),'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($balance_change_records,'process');
        $this->assertEquals(15.00,$bcr_hash['LoyaltyBalancePayment']['charge_amt'],"It should have the loyalty charge");

        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
        $discount_order_detail = array_pop($complete_order['order_details']);
        $this->assertEquals(-15.00,$discount_order_detail['item_total_w_mods']);
        $this->assertEquals(LoyaltyBalancePaymentService::DISCOUNT_NAME,$discount_order_detail['item_name']);

        // check to see if loyalty record was updated
        $ubp_resource = Resource::find(new UserBrandPointsMapAdapter($m),"",array(TONIC_FIND_BY_METADATA=>array("user_id"=>$user_id,"brand_id"=>getBrandIdFromCurrentContext())));

        $expected_points_added = number_format($expected_subtotal * 10,0);
        $expected_dollar_balance = number_format($expected_points_added/100,2);

        $this->assertEquals($expected_dollar_balance,$ubp_resource->dollar_balance,'It should hae reduced the dollar balance to zero bu then added the 3.10 for the 30.99 spend on the CC');
        $this->assertEquals($expected_points_added,$ubp_resource->points,"It should have 24 points.");
        
        
        
        
        
//        //$checkout_data = getCheckoutDataV2FromOrderData($order_data);
//        $order_resource = placeOrderFromOrderData($order_data,$time);
//        $order_id = $order_resource->order_id;
//        $this->assertNull($order_resource->error);
//        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
//        $order_detail_id = $complete_order['order_details'][0]['order_detail_id'];
//        $order_resource = Resource::find(new OrderAdapter($m),"$order_id");
//        $order_resource->order_amt = 5.99;
//        $order_resource->total_tax_amt = .60;
//        $order_resource->grand_total = 7.59;
//        $order_resource->grand_total_to_merchant = 6.59;
//        $order_resource->save();
//        $order_detail_resource = Resource::find(new OrderDetailAdapter($m),"$order_detail_id");
//        $order_detail_resource->item_total_w_mods = 5.99;
//        $order_detail_resource->item_tax = .60;
//        $order_detail_resource->save();
//
//        $sql = "INSERT INTO Order_Detail_Modifier SET order_detail_id = ".$order_detail_resource->order_detail_id.", mod_name = 'price adjustment', mod_print_name = 'points', modifier_type = 'P', comes_with = 'N', hold_it = 'N', mod_quantity = 50, mod_price = -2.00, mod_total_price = -2.00 ";
//        $odma = new OrderDetailModifierAdapter($m);
//        $odma->_query($sql);
//
//        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
//        $this->assertEquals(50,$complete_order['points_used']);
//        $this->assertEquals(2,$complete_order['amount_off_from_points']);

        $xoikos_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'X');
        $xoikos_controller = ControllerFactory::generateFromMessageResource($xoikos_message_resource);
        $ready_to_send_message_resource = $xoikos_controller->prepMessageForSending($xoikos_message_resource);
        $message_text = $ready_to_send_message_resource->message_text;
        myerror_log($message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);

        $expected_discount_payload =   cleanUpDoubleSpacesCRLFTFromString("  <Discounts>
                                            <Discount>
                                                <DiscountCode>Loyalty Purchase</DiscountCode>
                                                <Amount>15.00</Amount>
                                            </Discount>
                                        </Discounts>");

        $this->assertContains($expected_discount_payload,$body,"should have created Discount payload");
        // now check to see if the full amounts are shown
        $this->assertContains("<TotalPrice>".number_format($expected_grand_total,2)."</TotalPrice>",$body,"should have the reduced full price");
        $this->assertContains("<Tax>".number_format($expected_tax,2)."</Tax>",$body,"should have the reduced tax amount");
        $this->assertContains("<PaymentType>CreditCard</PaymentType><Amount>".$order_resource->grand_total."</Amount>",$body,"should have the discounted amoutn for the payment amount");



    }

    function testSplitUpSubOptionsOnPartyTrays()
    {
        $user = logTestUserIn($this->ids['user_id']);
        $merchant_id = $this->ids['merchant_id'];
        $complete_menu = CompleteMenu::getCompleteMenu(102710,'Y',$merchant_id,2);

        $ismr = ItemSizeAdapter::staticGetRecord(array("item_id"=>299825,"size_id"=>93814,"merchant_id"=>$merchant_id),'ItemSizeAdapter');
        $order_data = array();
        $order_data['user_id'] = getLoggedInUserId();
        $order_data['merchant_id'] = $merchant_id;
        $order_data['note'] = "Order Level Note";
        $order_data['items'][0]['quantity'] = 1;
        $order_data['items'][0]['item_id'] = 299825;
        $order_data['items'][0]['size_id'] = 93814;
        $order_data['items'][0]['mods'][] = array("mod_quantity"=>2,"modifier_item_id"=>2342701);
        $order_data['items'][0]['mods'][] = array("mod_quantity"=>3,"modifier_item_id"=>2342702);
        $order_data['items'][0]['note'];
        $order_data['actual_pickup_time'] = getTomorrowTwelveNoonTimeStampDenver() + (20*60);
        $order_data['tip'] = 0.00;
        $order_data['sub_total'] = floatval($ismr['price']*5);
        $order_data['tax_amt'] = $order_data['sub_total'] * .1;
        $order_data['grand_total'] = $order_data['sub_total'] + $order_data['tax_amt'];

        $order_resource = placeOrderFromOrderData($order_data,$time);
        $order_id = $order_resource->order_id;
        $this->assertTrue($order_id > 1000);
        $this->assertNull($order_resource->error);
        $xoikos_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'X');
        $this->assertNotNull($xoikos_message_resource);
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
        $xoikos_controller = ControllerFactory::generateFromMessageResource($xoikos_message_resource);
        $ready_to_send_message_resource = $xoikos_controller->prepMessageForSending($xoikos_message_resource);
        $message_text = $ready_to_send_message_resource->message_text;
        myerror_log($message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);

        $body_as_array = parseXMLintoHashmap(str_replace('&','+',$body));
        $option_nodes = $body_as_array['OrderItems']['OrderItem']['Options']['Option'];
        $this->assertCount(5,$option_nodes,"It should have 5 option nodes for the 5 sandwichs");
        $italian_count = 0;
        $penny_club_count = 0;
        foreach ($option_nodes as $option_node) {
            if ($option_node['SelectionText'] == 'Italian'){
                $italian_count++;
            } else if ($option_node['SelectionText'] == 'Penny Club') {
                $penny_club_count++;
            }
        }
        $this->assertEquals(3,$italian_count,"It should have 3 italian nodes");
        $this->assertEquals(2,$penny_club_count,"It should have 2 penny club noes");
    }

    function testSplitUpOpionsWithQuantityCookieNode()
    {
        $user = logTestUserIn($this->ids['user_id']);
        $merchant_id = $this->ids['merchant_id'];

        $ismr = ItemSizeAdapter::staticGetRecord(array("item_id"=>299813,"size_id"=>93804,"merchant_id"=>$merchant_id),'ItemSizeAdapter');
        $order_data = array();
        $order_data['user_id'] = getLoggedInUserId();
        $order_data['merchant_id'] = $merchant_id;
        $order_data['note'] = "Order Level Note";
        $order_data['items'][0]['quantity'] = 1;
        $order_data['items'][0]['item_id'] = 299813;
        $order_data['items'][0]['size_id'] = 93804;
        $order_data['items'][0]['mods'][] = array("mod_quantity"=>2,"modifier_item_id"=>2342343);
        $order_data['items'][0]['mods'][] = array("mod_quantity"=>3,"modifier_item_id"=>2342344);
        $order_data['items'][0]['note'];
        $order_data['actual_pickup_time'] = getTomorrowTwelveNoonTimeStampDenver() + (20*60);
        $order_data['tip'] = 0.00;
        $order_data['sub_total'] = floatval($ismr['price']*5);
        $order_data['tax_amt'] = $order_data['sub_total'] * .1;
        $order_data['grand_total'] = $order_data['sub_total'] + $order_data['tax_amt'];

        $order_resource = placeOrderFromOrderData($order_data,$time);
        $order_id = $order_resource->order_id;
        $this->assertTrue($order_id > 1000);
        $this->assertNull($order_resource->error);
        $xoikos_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'X');
        $this->assertNotNull($xoikos_message_resource);
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
        $xoikos_controller = ControllerFactory::generateFromMessageResource($xoikos_message_resource);
        $ready_to_send_message_resource = $xoikos_controller->prepMessageForSending($xoikos_message_resource);
        $message_text = $ready_to_send_message_resource->message_text;
        myerror_log($message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);

        $body_as_array = parseXMLintoHashmap(str_replace('&','+',$body));
        $this->assertCount(2,$body_as_array['OrderItems']['OrderItem'],"It should have produced 2 items");
        foreach ($body_as_array['OrderItems']['OrderItem'] as $cookie_item_node) {
            $this->assertCount(1,$cookie_item_node['Options'],"It should only have 1 flavor per cookie item");
        }


        $this->assertContains($this->getMultiCookieExpectedPayload("Chocolate Chip",$ismr['price'],"M52",2),$body,"should have created the expected Xoikos payload for 2 chocolate chips cookies");
        $this->assertContains($this->getMultiCookieExpectedPayload("Peanut Butter",$ismr['price'],"M53",3),$body,"should have created the expected Xoikos payload for 3 peanut butter cookies");
    }

    function testSplitUpOpionsWithQuantityChipNode()
    {
        $user = logTestUserIn($this->ids['user_id']);
        $merchant_id = $this->ids['merchant_id'];

        $ismr = ItemSizeAdapter::staticGetRecord(array("item_id"=>299812,"size_id"=>93804,"merchant_id"=>$merchant_id),'ItemSizeAdapter');
        $order_data = array();
        $order_data['user_id'] = getLoggedInUserId();
        $order_data['merchant_id'] = $merchant_id;
        $order_data['note'] = "Order Level Note";
        $order_data['items'][0]['quantity'] = 1;
        $order_data['items'][0]['item_id'] = 299812;
        $order_data['items'][0]['size_id'] = 93804;
        $order_data['items'][0]['mods'][] = array("mod_quantity"=>2,"modifier_item_id"=>2342334);
        $order_data['items'][0]['mods'][] = array("mod_quantity"=>3,"modifier_item_id"=>2342336);
        $order_data['items'][0]['note'];
        $order_data['actual_pickup_time'] = getTomorrowTwelveNoonTimeStampDenver() + (20*60);
        $order_data['tip'] = 0.00;
        $order_data['sub_total'] = floatval($ismr['price']*5);
        $order_data['tax_amt'] = $order_data['sub_total'] * .1;
        $order_data['grand_total'] = $order_data['sub_total'] + $order_data['tax_amt'];

        $order_resource = placeOrderFromOrderData($order_data,$time);
        $order_id = $order_resource->order_id;
        $this->assertTrue($order_id > 1000);
        $this->assertNull($order_resource->error);
        $xoikos_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'X');
        $this->assertNotNull($xoikos_message_resource);
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
        $xoikos_controller = ControllerFactory::generateFromMessageResource($xoikos_message_resource);
        $ready_to_send_message_resource = $xoikos_controller->prepMessageForSending($xoikos_message_resource);
        $message_text = $ready_to_send_message_resource->message_text;
        myerror_log($message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);

        $body_as_array = parseXMLintoHashmap(str_replace('&','+',$body));
        $this->assertCount(2,$body_as_array['OrderItems']['OrderItem'],"It should have produced 2 items");
        foreach ($body_as_array['OrderItems']['OrderItem'] as $chip_item_node) {
            $this->assertCount(1,$chip_item_node['Options'],"It should only have 1 flavor per cookie item");
        }


        $this->assertContains($this->getMultiChipExpectedPayload("Baked Lays",$ismr['price'],"M36",2),$body,"should have created the expected Xoikos payload for 2 chocolate chips cookies");
        $this->assertContains($this->getMultiChipExpectedPayload("Cheetos",$ismr['price'],"M43",3),$body,"should have created the expected Xoikos payload for 3 peanut butter cookies");
    }

    function getMultiCookieExpectedPayload($name,$single_cookie_price,$mlu,$quantity)
    {
        return $this->getMultiExpectedPayload('Cookies',$name,"336",$single_cookie_price,$mlu,$quantity);
    }

    function getMultiChipExpectedPayload($name,$single_chip_price,$mlu,$quantity)
    {
        return $this->getMultiExpectedPayload('Chips',$name,"270",$single_chip_price,$mlu,$quantity);
    }


    function getMultiExpectedPayload($group_name,$name,$plu,$single_item_price,$mlu,$quantity)
    {
        $total = floatval($single_item_price * $quantity);
        $tax = number_format($total*.1,2);
        $item_node = "<OrderItem>
                            <Qty>$quantity</Qty>
                            <ItemName>$group_name</ItemName>
                            <Recipient></Recipient>
                            <ItemComments></ItemComments>
                            <POS>
                                <ROPE />
                                <FlexPOS />
                                <InfoKING>
                                    <PLU>$plu</PLU>
                                </InfoKING>
                            </POS>
                            <Price>
                                <ItemPrice>$single_item_price</ItemPrice>
                                <PriceBeforeDiscount>$single_item_price</PriceBeforeDiscount>
                                <BasePrice>$single_item_price</BasePrice>
                                <Tax>$tax</Tax>
                            </Price>
                            <Taxable>true</Taxable>
                            <TaxExempt>false</TaxExempt>
                            <Options>
                                <Option>
                                    <OptionText>$group_name</OptionText>
                                    <SelectionText>$name</SelectionText>
                                    <Price>0</Price>
                                    <PriceEach>$0</PriceEach>
                                    <Quantity>Regular</Quantity>
                                    <Side>Full</Side>
                                    <Status>Added</Status>
                                    <POS>
                                        <ROPE/>
                                        <FlexPOS/>
                                        <InfoKING>
                                            <PLU>$mlu</PLU>
                                        </InfoKING>
                                    </POS>
                                </Option>
                            </Options>
                        </OrderItem>";
        return cleanUpDoubleSpacesCRLFTFromString($item_node);
    }

    function getOrderWithGoodcentsOriginal($merchant_id)
    {
        $ismr = ItemSizeAdapter::staticGetRecord(array("item_id"=>299783,"size_id"=>93792,"merchant_id"=>$merchant_id),'ItemSizeAdapter');
        $order_data = array();
        $order_data['user_id'] = getLoggedInUserId();
        $order_data['merchant_id'] = $merchant_id;
        $order_data['note'] = "Order Level Note";
        $order_data['items'][0]['quantity'] = 1;
        $order_data['items'][0]['item_id'] = 299783;
        $order_data['items'][0]['size_id'] = 93792;
        $order_data['items'][0]['mods'] = array(array("mod_quantity"=>1,"modifier_item_id"=>2342276));
        $order_data['items'][0]['note'] = "Item Comments";
        $order_data['actual_pickup_time'] = getTomorrowTwelveNoonTimeStampDenver() + (20*60);
        $order_data['tip'] = 0.00;
        $order_data['sub_total'] = floatval($ismr['price']);
        $order_data['tax_amt'] = floatval($ismr['price']) * .1;
        $order_data['grand_total'] = $order_data['sub_total'] + $order_data['tax_amt'];
        return $order_data;
    }


    /**
     * @depends testCreateXoikosMessageFromGoodCentsOrder
     */
    function testCreateXoikosOrderMessageTemplate($order_id)
    {
        $expected_payload = cleanUpDoubleSpacesCRLFTFromString(file_get_contents("./unit_tests_pos_integrations/resources/expected_xoikos_payload.xml"));
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
        $expected_payload = str_replace('%%singleitempriceprice%%',$complete_order['order_details'][0]['item_total'],$expected_payload);
        $expected_payload = str_replace('%%ordergrandtotal%%',$complete_order['grand_total'],$expected_payload);
        $expected_payload = str_replace('%%price%%',$complete_order['order_details'][0]['price'],$expected_payload);
        $expected_payload = str_replace('%%tax%%',number_format($complete_order['order_details'][0]['item_tax'],2),$expected_payload);
        $tz = date_default_timezone_get();
        date_default_timezone_set("GMT");
        $xoikos_formatted_pickup = date("Y-m-d\TH:i:00.0000\Z",getTomorrowTwelveNoonTimeStampDenver()+(20 * 60));
        $expected_payload = str_replace('%%tomorrowpickup%%',$xoikos_formatted_pickup,$expected_payload);
        date_default_timezone_set($tz);


        $xoikos_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'X');
        $xoikos_controller = ControllerFactory::generateFromMessageResource($xoikos_message_resource);
        $ready_to_send_message_resource = $xoikos_controller->prepMessageForSending($xoikos_message_resource);
        $message_text = $ready_to_send_message_resource->message_text;
        myerror_log($message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);
        $this->assertContains($expected_payload,$body,"should have created the expected Xoikos payload");
        $this->assertContains($complete_order['ucid'],$body,"should have found the UCID in the order payload");

        $expected_payment_node = cleanUpDoubleSpacesCRLFTFromString("<Payment>
            <PaymentType>CreditCard</PaymentType>
            <Amount>".$complete_order['grand_total']."</Amount>
            <AccountNumber>XXXXXXXXXXXX1234</AccountNumber>
        </Payment>");
        $this->assertContains($expected_payment_node,$body,"should have created the expected Xoikos payment node");

        $this->assertNotContains("(212) 555-3333",$body,"should not contain the formatted phone number");
        $this->assertContains('2125553333',$body,"should contain the unformatted phone nubmer");
        $expected_ucid_node = "<OrderID>".$complete_order['ucid']."</OrderID>";
        $this->assertContains($expected_ucid_node,$body,"shoudl contain the UCID");
        /*
         * TODO  write test for pickup and order
         */

        return $complete_order['ucid'];
    }

    function testCreateOrderWithExtraCheese()
    {
        $user = logTestUserIn($this->ids['user_id']);
        $order_adapter = new OrderAdapter($m);
        $order_data = $this->getOrderWithGoodcentsOriginal($this->ids['merchant_id']);
        // add extra cheese
        $order_data['items'][0]['mods'][] = array("mod_quantity"=>1,"modifier_item_id"=>2342293);
        $order_resource = placeOrderFromOrderData($order_data,$time);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $xoikos_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'X');
        $this->assertNotNull($xoikos_message_resource);
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
        $xoikos_controller = ControllerFactory::generateFromMessageResource($xoikos_message_resource);
        $ready_to_send_message_resource = $xoikos_controller->prepMessageForSending($xoikos_message_resource);
        $message_text = $ready_to_send_message_resource->message_text;
        myerror_log($message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);
        $body_as_array = parseXMLintoHashmap(str_replace('&','+',$body));

        $modifiers = $body_as_array['OrderItems']['OrderItem']['Options']['Option'];
        $this->assertCount(3,$modifiers,"there shoudl have been 3 modifier nodes because of the cheese");

        //check for cheese size node
        $expected_extra_cheese_item_plu = '<POS><ROPE/><FlexPOS/><InfoKING><PLU>251</PLU></InfoKING></POS>';

        $this->assertContains($expected_extra_cheese_item_plu,$body,"should have created the expected Xoikos payload for the extra cheese PLU node");

        $expected_cheese_mlu_node = cleanUpDoubleSpacesCRLFTFromString('<Option>
                        <OptionText>Extras</OptionText>
                        <SelectionText>American Cheese ($)</SelectionText>
                        <Price>0.00</Price>
                        <PriceEach>$0.00</PriceEach>
                        <Quantity>Regular</Quantity>
                        <Side>Full</Side>
                        <Status>Added</Status>
                        <POS>
                            <ROPE/>
                            <FlexPOS/>
                            <InfoKING>
                                <PLU>M3</PLU>
                            </InfoKING>
                        </POS>
                    </Option>');
        $this->assertContains($expected_cheese_mlu_node,$body,"should have created the expected Xoikos payload for the extra cheese MLU node");

    }

    function testCreateOrderWithBottledDrink()
    {
        $_SERVER['DO_NOT_RUN_CC'] = 'true';
        $user = logTestUserIn($this->ids['user_id']);
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'],'pickup',$note);
        $order_data['items'][0] = array("item_id"=>299845,"size_id"=>93821,"quantity"=>1,"mods"=>array());
        $order_data['lead_time'] = 20;
        $order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $xoikos_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'X');
        $this->assertNotNull($xoikos_message_resource);

        $xoikos_controller = ControllerFactory::generateFromMessageResource($xoikos_message_resource);
        $ready_to_send_message_resource = $xoikos_controller->prepMessageForSending($xoikos_message_resource);
        $message_text = $ready_to_send_message_resource->message_text;
        myerror_log($message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);

        $expected_bottled_drink_item_node = cleanUpDoubleSpacesCRLFTFromString("<ItemName>Coke</ItemName>
        <Recipient></Recipient>
        <ItemComments></ItemComments>
        <POS>
            <ROPE />
            <FlexPOS />
            <InfoKING>
                <PLU>386</PLU>
            </InfoKING>
        </POS>");
        $this->assertContains($expected_bottled_drink_item_node,$body,"should have created the expected Xoikos payload for the bottled drink item node");

//        $body_as_array = parseXMLintoHashmap($body);
//        $modifiers = $body_as_array['OrderItems']['OrderItem']['Options']['Option'];
//        $this->assertNotNull($modifiers,"there should have been a modifier node");
//        $this->assertCount(1,$modifiers,"there should have been only 1 modifier node");




        $expected_bottled_drink_modifier_node = cleanUpDoubleSpacesCRLFTFromString("<Option>
                <OptionText></OptionText>
                <SelectionText></SelectionText>
                <Price>0.00</Price>
                <PriceEach>$0.00</PriceEach>
                <Quantity>Regular</Quantity>
                <Side>Full</Side>
                <Status>Added</Status>
                <POS>
                    <ROPE/>
                    <FlexPOS/>
                    <InfoKING>
                        <PLU>M45</PLU>
                    </InfoKING>
                </POS>
            </Option>");
        $this->assertContains($expected_bottled_drink_modifier_node,$body,"should have created the expected Xoikos payload for the bottled drink modifier node");
    }

    function testCreateOrderWithMealDealRegularTopLevel()
    {
        $user = logTestUserIn($this->ids['user_id']);
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'],'pickup',$note);
        $order_data['items'][0]['item_id'] = 299858;
        $order_data['items'][0]['size_id'] = 93826;
        $order_data['items'][0]['mods']  = array(array("mod_quantity"=>1,"modifier_item_id"=>2342933),array("mod_quantity"=>1,"modifier_item_id"=>2342917));
        $order_data['items'][0]['note'] = "Item Comments";
        $order_data['lead_time'] = 20;
        $order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $xoikos_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'X');
        $this->assertNotNull($xoikos_message_resource);
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
        $xoikos_controller = ControllerFactory::generateFromMessageResource($xoikos_message_resource);
        $ready_to_send_message_resource = $xoikos_controller->prepMessageForSending($xoikos_message_resource);
        $message_text = $ready_to_send_message_resource->message_text;
        myerror_log($message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);
        $body_as_array = parseXMLintoHashmap(str_replace('&','+',$body));

        $modifiers = $body_as_array['OrderItems']['OrderItem']['Options']['Option'];
        $this->assertCount(2,$modifiers,"there shoudl have been 2 modifier nodes");

        //check for drink node
        $expected_drink_node_as_part_of_meal_deal = cleanUpDoubleSpacesCRLFTFromString(file_get_contents("./unit_tests_pos_integrations/resources/expected_meal_deal_drink_node.xml"));
        $this->assertContains($expected_drink_node_as_part_of_meal_deal,$body,"should have created the expected Xoikos payload for the drink node");
        // check for chips node
        $expected_chips_node_as_part_of_meal_deal = cleanUpDoubleSpacesCRLFTFromString(file_get_contents("./unit_tests_pos_integrations/resources/expected_meal_deal_chip_node.xml"));
        $this->assertContains($expected_chips_node_as_part_of_meal_deal,$body,"should have created the expected Xoikos payload for the chips node");
    }

    function testCreateOrderWithMealDealLargeAndCookieTopLevel()
    {
        $user_resource = Resource::find(new UserAdapter($m),$this->ids['user_id']);
        $user_resource->balance = 50.00;
        $user_resource->save();
        $user = logTestUserIn($this->ids['user_id']);
        $order_adapter = new OrderAdapter($m);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'],'pickup',$note);
        $order_data['items'][0]['item_id'] = 299859;
        $order_data['items'][0]['size_id'] = 93826;
        $order_data['items'][0]['mods']  = array(array("mod_quantity"=>1,"modifier_item_id"=>2342947),array("mod_quantity"=>1,"modifier_item_id"=>2342927));
        $order_data['items'][0]['note'] = "Item Comments";
        $order_data['lead_time'] = 20;
        $order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $xoikos_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'X');
        $this->assertNotNull($xoikos_message_resource);

        $xoikos_controller = ControllerFactory::generateFromMessageResource($xoikos_message_resource);
        $ready_to_send_message_resource = $xoikos_controller->prepMessageForSending($xoikos_message_resource);
        $message_text = $ready_to_send_message_resource->message_text;
        myerror_log($message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);
        $body_as_array = parseXMLintoHashmap(str_replace('&','+',$body));

        $modifiers = $body_as_array['OrderItems']['OrderItem']['Options']['Option'];
        $this->assertCount(3,$modifiers,"there shoudl have been 5 modifier nodes");

        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);


        //check for drink node
        $expected_drink_node_as_part_of_meal_deal = cleanUpDoubleSpacesCRLFTFromString("<Option>
    <OptionText>
        Drink
    </OptionText>
    <SelectionText>
        Large Coke
    </SelectionText>
    <Price>
        0.00
    </Price>
    <PriceEach>
        $0.00
    </PriceEach>
    <Quantity>
        Regular
    </Quantity>
    <Side>
        Full
    </Side>
    <Status>
        Added
    </Status>
    <POS>
        <ROPE/>
        <FlexPOS/>
        <InfoKING>
            <PLU>
                M45
            </PLU>
        </InfoKING>
    </POS>
</Option>");
        $this->assertContains($expected_drink_node_as_part_of_meal_deal,$body,"should have created the expected Xoikos payload for the drink node");

        // check for cookie node
        $expected_cookie_node_as_part_of_meal_deal = cleanUpDoubleSpacesCRLFTFromString(file_get_contents("./unit_tests_pos_integrations/resources/expected_meal_deal_cookie_node.xml"));
        $this->assertContains($expected_cookie_node_as_part_of_meal_deal,$body,"should have created the expected Xoikos payload for the cookie node");

        $modifiers = $complete_order['order_details'][0]['order_detail_complete_modifier_list_no_holds'];
        $modifiers_hash = createHashmapFromArrayOfArraysByFieldName($modifiers,"external_id");

        //check for drink size
        //$expected_meal_deal_drink_size_node = cleanUpDoubleSpacesCRLFTFromString(file_get_contents("./unit_tests_pos_integrations/resources/expected_meal_deal_drink_size_node.xml"));

        $meal_deal_drink_size_price = number_format($modifiers_hash['384:M45']['mod_price'],2);
        $expected_meal_deal_drink_size_node = cleanUpDoubleSpacesCRLFTFromString('<Option>
               <OptionText>Drink Size</OptionText>
               <SelectionText>Large</SelectionText>
               <Price>%%price%%</Price>
               <PriceEach>$%%price%%</PriceEach>
               <Quantity>Regular</Quantity>
               <Side>Full</Side>
               <Status>Added</Status>
               <POS>
                  <ROPE/>
                  <FlexPOS/>
                  <InfoKING>
                     <PLU>384</PLU>
                  </InfoKING>
               </POS>
            </Option>');

        $expected_meal_deal_drink_size_node = str_replace('> <','><',$expected_meal_deal_drink_size_node);
        $expected_meal_deal_drink_size_node = str_replace('%%price%%',$meal_deal_drink_size_price,$expected_meal_deal_drink_size_node);
        $this->assertContains($expected_meal_deal_drink_size_node,$body,"should have created the expected Xoikos drink size payload");

    }





    /*
    These tests are from the original way of adding a meal deal which i'm sure they will going back to so i'm saving the tests
    */

//    function testCreateOrderWithMealDealRegular()
//    {
//        $user_resource = createNewUserWithCC();
//        $user = logTestUserResourceIn($user_resource);
//        $order_adapter = new OrderAdapter($m);
//        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'],'pickup',$note);
//        $order_data['items'][0]['mods'] = array($order_data['items'][0]['mods'][0],array("mod_quantity"=>1,"modifier_item_id"=>2342661),array("mod_quantity"=>1,"modifier_item_id"=>2342663));
//        $order_data['items'][0]['note'] = "Item Comments";
//        $order_data['lead_time'] = 20;
//        $order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
//        $this->assertNull($order_resource->error);
//        $order_id = $order_resource->order_id;
//        $xoikos_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'X');
//        $this->assertNotNull($xoikos_message_resource);
//
//        $xoikos_controller = ControllerFactory::generateFromMessageResource($xoikos_message_resource);
//        $ready_to_send_message_resource = $xoikos_controller->prepMessageForSending($xoikos_message_resource);
//        $message_text = $ready_to_send_message_resource->message_text;
//        myerror_log($message_text);
//        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);
//        $body_as_array = parseXMLintoHashmap($body);
//
//        $modifiers = $body_as_array['OrderItems']['OrderItem']['Options']['Option'];
//        $this->assertCount(4,$modifiers,"there shoudl have been 4 modifier nodes");
//
//        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
//
//
//        //check for drink node
//        $expected_drink_node_as_part_of_meal_deal = cleanUpDoubleSpacesCRLFTFromString(file_get_contents("./unit_tests_pos_integrations/resources/expected_meal_deal_drink_node.xml"));
//        $this->assertContains($expected_drink_node_as_part_of_meal_deal,$body,"should have created the expected Xoikos payload for the drink node");
//        // check for chips node
//        $expected_chips_node_as_part_of_meal_deal = cleanUpDoubleSpacesCRLFTFromString(file_get_contents("./unit_tests_pos_integrations/resources/expected_meal_deal_chip_node.xml"));
//        $this->assertContains($expected_chips_node_as_part_of_meal_deal,$body,"should have created the expected Xoikos payload for the chips node");
//        // check for meal deal node
//        $expected_meal_deal_node = cleanUpDoubleSpacesCRLFTFromString(file_get_contents("./unit_tests_pos_integrations/resources/expected_meal_deal_node.xml"));
//
//        $modifiers = $complete_order['order_details'][0]['order_detail_complete_modifier_list_no_holds'];
//        $modifiers_hash = createHashmapFromArrayOfArraysByFieldName($modifiers,"external_id");
//        $meal_deal_price = $modifiers_hash['423:M36']['mod_price'];
//        $expected_meal_deal_node = str_replace('%%price%%',$meal_deal_price,$expected_meal_deal_node);
//        $this->assertContains($expected_meal_deal_node,$body,"should have created the expected Xoikos payload");
//    }
//
//    function testCreateOrderWithMealDealLargeAndCookie()
//    {
//        $user_resource = createNewUserWithCC();
//        $user = logTestUserResourceIn($user_resource);
//        $order_adapter = new OrderAdapter($m);
//        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'],'pickup',$note);
//        $order_data['items'][0]['mods'] = array($order_data['items'][0]['mods'][0],array("mod_quantity"=>1,"modifier_item_id"=>2342684),array("mod_quantity"=>1,"modifier_item_id"=>2342672));
//        $order_data['items'][0]['note'] = "Item Comments";
//        $order_data['lead_time'] = 20;
//        $order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
//        $this->assertNull($order_resource->error);
//        $order_id = $order_resource->order_id;
//        $xoikos_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'X');
//        $this->assertNotNull($xoikos_message_resource);
//
//        $xoikos_controller = ControllerFactory::generateFromMessageResource($xoikos_message_resource);
//        $ready_to_send_message_resource = $xoikos_controller->prepMessageForSending($xoikos_message_resource);
//        $message_text = $ready_to_send_message_resource->message_text;
//        myerror_log($message_text);
//        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);
//        $body_as_array = parseXMLintoHashmap($body);
//
//        $modifiers = $body_as_array['OrderItems']['OrderItem']['Options']['Option'];
//        $this->assertCount(5,$modifiers,"there shoudl have been 5 modifier nodes");
//
//        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
//
//
//        //check for drink node
//        $expected_drink_node_as_part_of_meal_deal = cleanUpDoubleSpacesCRLFTFromString(file_get_contents("./unit_tests_pos_integrations/resources/expected_meal_deal_drink_node.xml"));
//        $this->assertContains($expected_drink_node_as_part_of_meal_deal,$body,"should have created the expected Xoikos payload for the drink node");
//
//        // check for chips node
//        $expected_cookie_node_as_part_of_meal_deal = cleanUpDoubleSpacesCRLFTFromString(file_get_contents("./unit_tests_pos_integrations/resources/expected_meal_deal_cookie_node.xml"));
//        $this->assertContains($expected_cookie_node_as_part_of_meal_deal,$body,"should have created the expected Xoikos payload for the cookie node");
//        // check for meal deal node
//        $expected_meal_deal_node = cleanUpDoubleSpacesCRLFTFromString(file_get_contents("./unit_tests_pos_integrations/resources/expected_meal_deal_node.xml"));
//
//        $modifiers = $complete_order['order_details'][0]['order_detail_complete_modifier_list_no_holds'];
//        $modifiers_hash = createHashmapFromArrayOfArraysByFieldName($modifiers,"external_id");
//        $meal_deal_price = $modifiers_hash['424:M52']['mod_price'];
//        $expected_meal_deal_node = str_replace('%%price%%',$meal_deal_price,$expected_meal_deal_node);
//        $expected_meal_deal_node = str_replace('With Chips','With a Cookie',$expected_meal_deal_node);
//        $expected_meal_deal_node = str_replace('423','424',$expected_meal_deal_node);
//        $this->assertContains($expected_meal_deal_node,$body,"should have created the expected Xoikos meal deal payload");
//
//        //check for drink size
//        $expected_meal_deal_drink_size_node = cleanUpDoubleSpacesCRLFTFromString(file_get_contents("./unit_tests_pos_integrations/resources/expected_meal_deal_drink_size_node.xml"));
//
//        $meal_deal_drink_size_price = $modifiers_hash['384:M45']['mod_price'];
//        $expected_meal_deal_drink_size_node = str_replace('%%price%%',$meal_deal_drink_size_price,$expected_meal_deal_drink_size_node);
//        $this->assertContains($expected_meal_deal_drink_size_node,$body,"should have created the expected Xoikos drink size payload");
//
//    }
//

    function testCashPayment()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $mptm_resource = MerchantPaymentTypeMapsAdapter::createMerchantPaymentTypeMap($this->ids['merchant_id'],1000,null);
        //$user = logTestUserIn($this->ids['user_id']);
        $order_data = OrderAdapter::getSimpleCartArrayByMerchantId($this->ids['merchant_id']);
        $checkout_resource = getCheckoutResourceFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $payment_hash = createHashmapFromArrayOfArraysByFieldName($checkout_resource->accepted_payment_types,'name');
        $checkout_resource->accepted_payment_types = array($payment_hash['Cash']);
        //$order_data['merchant_payment_type_map_id'] = $mptm_resource->id;
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$this->ids['merchant_id'],0.00,getTomorrowTwelveNoonTimeStampDenver());
        //$order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $this->assertEquals('Y',$order_resource->cash,"Should have been a cash order");

        $xoikos_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id, 'X');
        $xoikos_controller = ControllerFactory::generateFromMessageResource($xoikos_message_resource);
        $ready_to_send_message_resource = $xoikos_controller->prepMessageForSending($xoikos_message_resource);
        $message_text = $ready_to_send_message_resource->message_text;
        myerror_log($message_text);
        $body = cleanUpDoubleSpacesCRLFTFromString($ready_to_send_message_resource->message_text);

        $expected_payment_node = cleanUpDoubleSpacesCRLFTFromString("<Payment>
            <PaymentType>POD</PaymentType>
            <Amount>".$order_resource->grand_total."</Amount>
        </Payment>");
        $this->assertContains($expected_payment_node,$body,"should have created the expected Xoikos payment node");

    }

    /**
     * @depends testCreateXoikosMessageFromGoodCentsOrder
     */
    function testSendOrderToXoikos($order_id)
    {
        $xoikos_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'X');
        $xoikos_controller = ControllerFactory::generateFromMessageResource($xoikos_message_resource);
        $this->assertTrue($xoikos_controller->sendThisMessage($xoikos_message_resource));
        $message_history_record = MerchantMessageHistoryAdapter::staticGetRecordByPrimaryKey($xoikos_message_resource->map_id,'MerchantMessageHistoryAdapter');
        $this->assertNotNull($message_history_record['message_text'],"should have recorded the sent XML");
        $this->assertNotNull($message_history_record['response'],"should have recorded the response XML");
        $this->assertContains("<success>true</success>",$message_history_record['response']);
    }

    /**
     * @depends testPromoOnXoikosOrder
     */
    function testSendPromoOrderToXoikos($order_id)
    {
        $xoikos_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id, 'X');
        $xoikos_controller = ControllerFactory::generateFromMessageResource($xoikos_message_resource);
        $this->assertTrue($xoikos_controller->sendThisMessage($xoikos_message_resource));
        $message_history_record = MerchantMessageHistoryAdapter::staticGetRecordByPrimaryKey($xoikos_message_resource->map_id,'MerchantMessageHistoryAdapter');
        $this->assertNotNull($message_history_record['message_text'],"should have recorded the sent XML");
        $this->assertNotNull($message_history_record['response'],"should have recorded the response XML");
        $this->assertContains("<success>true</success>",$message_history_record['response']);
    }

    static function setUpBeforeClass()
    {
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time',300);

        $menu_id = 102710;
        $skin_id = 140;
        $brand_id = 430;
        $menu_adapter = new MenuAdapter(getM());
        if ($menu_resource = Resource::find($menu_adapter,"$menu_id")){
            myerror_log("we have the menu");
        } else {
            //create menu
            $menu_adapter->importMenu($menu_id,'prod','local',$brand_id);
        }

        $skin_resource = getOrCreateSkinAndBrandIfNecessary("good cents subs","Goodcents Subs",$skin_id,$brand_id);
        setContext('com.splickit.goodcentssubs');


//        $merchant_resource2 = getOrCreateNewTestMerchantBasedOnExternalId('131');
//        Resource::createByData(new MerchantMenuMapAdapter($m),array("merchant_id"=>$merchant_resource2->merchant_id,"menu_id"=>$menu_id));
//        $url = "/apiv2/pos/import/xoikos/131";
//        $xoikos_importer = ImporterFactory::getImporterFromUrl($url);
//        $xoikos_importer->importRemoteMerchantMetaDataForLoadedMerchant();




        $merchant_resource = getOrCreateNewTestMerchantBasedOnExternalId('8500');
        Resource::createByData(new MerchantMenuMapAdapter($m),array("merchant_id"=>$merchant_resource->merchant_id,"menu_id"=>$menu_id));

        $merchant_resource2 = getOrCreateNewTestMerchantBasedOnExternalId('1031');
        Resource::createByData(new MerchantMenuMapAdapter($m),array("merchant_id"=>$merchant_resource2->merchant_id,"menu_id"=>$menu_id));
        $url = "/apiv2/pos/import/xoikos/1031";

        $merchant_resource2 = getOrCreateNewTestMerchantBasedOnExternalId('50');
        Resource::createByData(new MerchantMenuMapAdapter($m),array("merchant_id"=>$merchant_resource2->merchant_id,"menu_id"=>$menu_id));
        $url = "/apiv2/pos/import/xoikos/50";
        $xoikos_importer = ImporterFactory::getImporterFromUrl($url);
        $xoikos_importer->importRemoteMerchantMetaDataForLoadedMerchant();

        // delivery meal setup
        $msma = new ModifierSizeMapAdapter($m);
        $plu_array = array(2342917=>"393:M45",2342922=>"394:M45",2342927=>"395:M45");
        $options[TONIC_FIND_BY_METADATA]['size_id'] = 0;
        $options[TONIC_FIND_BY_METADATA]['merchant_id'] = $merchant_resource2->merchant_id;

        $price = 1.95;
        foreach ($plu_array as $id=>$value) {
            $options[TONIC_FIND_BY_METADATA]['modifier_item_id'] = $id;
            if ($msm_resource = Resource::find($msma,'',$options)) {
                $msm_resource->external_id = $value;
                $msm_resource->modifier_price = $price;
                $msm_resource->save();
                $price = $price + .25;
            }
        }



//        $number_item_prices = $xoikos_importer->getNumberOfItemPricesImported();
//        $number_modifier_prices = $xoikos_importer->getNumberOfModifierPricesImported();
//
//        $complete_menu = new CompleteMenu($menu_id);
//        $complete_menu = $complete_menu->getCompleteMenu($menu_id,'Y',$merchant_resource2->merchant_id,2,'pickup');

        addDeliveryToMerchant($merchant_resource,$menu_id);

        $ids['drive_through_merchant_resource'] = $merchant_resource2;
        $ids['drive_through_merchant_id'] = $merchant_resource2->merchant_id;

        $ids['merchant_resource'] = $merchant_resource;
        $ids['merchant_id'] = $merchant_resource->merchant_id;

        $ids['menu_id'] = $menu_id;

        $map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_resource->merchant_id,"message_format"=>'X',"delivery_addr"=>"Xoikos","message_type"=>"X"));
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_resource2->merchant_id,"message_format"=>'XD',"delivery_addr"=>"Xoikos","message_type"=>"X"));


        $brand_id = getBrandIdFromCurrentContext();
        $brand_resource = Resource::find(new BrandAdapter($mimetypes),"$brand_id");
        $brand_resource->loyalty = 'Y';
        $brand_resource->save();

        $blr_data['brand_id'] = $brand_id;
        $blr_data['loyalty_type'] = 'splickit_earn';
        $brand_loyalty_rules_resource = Resource::factory(new BrandLoyaltyRulesAdapter($m),$blr_data);
        $brand_loyalty_rules_resource->save();

        setContext('com.splickit.goodcentssubs');


        $billing_entity_resource = createSageBillingEntity($brand_id);

        $type = 'sage';
        $identifier = generateCode(20);
        $data['merchant_id_number'] = '469197833948';
        $data['merchant_id_key'] = 'T8L3P8O2X7I1';
        $vio_service = new VioPaymentService($data);
        $response = $vio_service->createDestination($type, $identifier, $data);
        $destination = $response['destination'];
        $billing_entity_resource = BillingEntitiesAdapter::createBillingEntity(2002, 'test billing entity', 'test billing entity', $brand_id, $identifier, $credentials);
        $merchant_payment_type_map_resource = MerchantPaymentTypeMapsAdapter::createMerchantPaymentTypeMap($merchant_resource->merchant_id, 2000, $billing_entity_resource->id);
        //$_SERVER['unit_test_ids']['destination_id'] = $identifier;
        $ids['vio_id'] = $identifier;


        $merchant_payment_type_map_adapter = new MerchantPaymentTypeMapsAdapter($mimetypes);

        // loyalty merchant payment type record
        $merchant_payment_type_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 8000, $billing_entity_id);

        $promo_adapter = new PromoAdapter($m);
        $promo_id = 401;
        $ids['promo_id_type_4'] = $promo_id;
        $sql = "INSERT INTO `Promo` VALUES($promo_id, 'The Promo 4', 'Get $ off your item', 4, 'Y', 'N', 0, 2, 'N', 'N', '2010-01-01', '2020-01-01', 100, 0, 0.00, 0, 0.00, 'Y', 'N', NOW(), NOW(), 'N')";
        $promo_adapter->_query($sql);
        $pmm_resource = Resource::createByData(new PromoMerchantMapAdapter($mimetypes), array("merchant_id"=>$merchant_id,"promo_id"=>$promo_id));
        $ids['promo_merchant_map_id_type_4'] = $pmm_resource->map_id;
        $sql = "INSERT INTO `Promo_Message_Map` VALUES(null, $promo_id, 'Congratulations! You''re getting %%amt%% off of your %%item_name%%!', NULL, NULL, NULL, 'Here''s the deal, order a large this, and you''ll get a discount! Limit 1', now())";
        $promo_adapter->_query($sql);
        $sql = "INSERT INTO `Promo_Type4_Item_Amount_Maps` VALUES(null, $promo_id, '1134',3.00,0,0,NOW(),NOW(),'N' )";
        $promo_adapter->_query($sql);

        $pkwm_adapter = new PromoKeyWordMapAdapter($mimetypes);
        Resource::createByData($pkwm_adapter, array("promo_id"=>$promo_id,"promo_key_word"=>"Test Promo","brand_id"=>430));




        $user_resource = createNewUser(array("flags"=>"1C20000001"));
        $ids['user_id'] = $user_resource->user_id;

        $uuid = $user_resource->uuid;

        $data['identifier'] = $uuid;
        $data['brand'] = "visa";
        $data['number'] = "4111111111111111";
        $data['cvv'] = "123";
        $data['zip'] = "12345";
        $data['month'] = 10;
        $data['year'] = 2019;
        $data['first_name'] = "sumdum";
        $data['last_name'] = "guy";
        $vio_payment_service = new VioPaymentService($data);
        $response = $vio_payment_service->saveCreditCard($data);

        $user_resource = Resource::find(new UserAdapter($m),$user_resource->user_id,$options);
        $user_resource->flags = "1C20000001";
        $user_resource->last_four = "1234";
        $user_resource->save();

        $sql = "INSERT INTO Billing_Entities VALUES (NULL,2001,'sumdum','sumdum',430,'fakeprocessor','dum','capture',now(),now(),'N')";
        $bea = new BillingEntitiesAdapter(getM());
        $bea->_query($sql);

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    XoikosOrderTest::main();
}

?>