<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class FpnLoyaltyTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $service;

	function setUp()
	{
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->service = new FpnLoyaltyService();
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
    }
    
    function testDeterminSuccessFromJsonPayload()
    {
    	// success
    	$json_payload = '{"xml":{"Reward":"REWARD:0.00","status":"success","Approved":"29916540","Clerk":"27","Check":"9780653","PointsAdded":"10.50","TotalPoints":["369.50","369.50"],"TotalSaved":"0.00","TotalVisits":"37","GiftCardBalance":"0.00","RewardCashBalance":"0.00","CustomReceiptMessages":"Register your Rewards Number and receive\na R5.00 instant reward\n"}}';
    	$result = $this->service->determineSuccessFromJsonPayload($json_payload);
    	$this->assertTrue($result);
    	
    	//failed probabaly need a way to get the actual failed reason
    	$json_payload = '{"xml":{"Reward":"REWARD:0.00","status":"failed","Approved":"29916540","Clerk":"27","Check":"9780653","PointsAdded":"10.50","TotalPoints":["369.50","369.50"],"TotalSaved":"0.00","TotalVisits":"37","GiftCardBalance":"0.00","RewardCashBalance":"0.00","CustomReceiptMessages":"Register your Rewards Number and receive\na R5.00 instant reward\n"}}';
    	$result = $this->service->determineSuccessFromJsonPayload($json_payload);
    	$this->assertFalse($result);
    	
    }

	//{"xml":{"status":"success","RewardAvailable":"5.00"}}

	function testAnotherResponseJsonPayload()
	{
		$json_payload = '{"xml":{"status":"success","RewardAvailable":"5.00"}}';
		$result = $this->service->determineSuccessFromJsonPayload($json_payload);
		$this->assertTrue($result);

	}

	function testDetermineSuccessFromOtherXMLSoapReturn()
	{
		$loyalty_return ='<?xml version="1.0" encoding="utf-8"?><soap:Envelope xmlns:soap="http://www.w3.org/2003/05/soap-envelope" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema"><soap:Body><AddPointsResponse xmlns="http://tempuri.org/"><AddPointsResult>{"xml":{"status":"success","RewardAvailable":"5.00"}}</AddPointsResult></AddPointsResponse></soap:Body></soap:Envelope>';
		$return = $this->service->processRawReturn($loyalty_return,'AddPointsResult');
		$this->assertTrue($return,"should have processed as true");
	}
    
    function testDeterminSuccessFromXMLSoapReturn()
    {
		$loyalty_return ='<?xml version="1.0" encoding="utf-8"?><soap:Envelope xmlns:soap="http://www.w3.org/2003/05/soap-envelope" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema"><soap:Body><AddPointsResponse xmlns="http://tempuri.org/"><AddPointsResult>{"xml":{"Reward":"REWARD:0.00","status":"success","Approved":"29916540","Clerk":"27","Check":"9780653","PointsAdded":"10.50","TotalPoints":["369.50","369.50"],"TotalSaved":"0.00","TotalVisits":"37","GiftCardBalance":"0.00","RewardCashBalance":"0.00","CustomReceiptMessages":"Register your Rewards Number and receive\na R5.00 instant reward\n"}}</AddPointsResult></AddPointsResponse></soap:Body></soap:Envelope>';
		$return = $this->service->processRawReturn($loyalty_return,'AddPointsResult');
		$this->assertTrue($return);   	

		$loyalty_return ='<?xml version="1.0" encoding="utf-8"?><soap:Envelope xmlns:soap="http://www.w3.org/2003/05/soap-envelope" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema"><soap:Body><AddPointsResponse xmlns="http://tempuri.org/"><AddPointsResult>{"xml":{"Reward":"REWARD:0.00","status":"failed","Approved":"29916540","Clerk":"27","Check":"9780653","PointsAdded":"10.50","TotalPoints":["369.50","369.50"],"TotalSaved":"0.00","TotalVisits":"37","GiftCardBalance":"0.00","RewardCashBalance":"0.00","CustomReceiptMessages":"Register your Rewards Number and receive\na R5.00 instant reward\n"}}</AddPointsResult></AddPointsResponse></soap:Body></soap:Envelope>';
		$return = $this->service->processRawReturn($loyalty_return,'AddPointsResult');
		$this->assertFalse($return);   	
    }

    function testCreateXMLSoapMessage()
    {
    	$new_order['order_amt'] = '10.50';
		$new_order['brand_loyalty_number'] = '900011062412';
		$new_order['merchant_rock_comm_id'] = 'TES92130';
		$new_order['order_id'] = rand(1111111, 9999999);
		$xml = $this->service->createLoyaltySoapXmlDocument($new_order);
		$expected = '<?xml version="1.0" encoding="utf-8"?><soap12:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap12="http://www.w3.org/2003/05/soap-envelope"><soap12:Header><UserCredentials xmlns="http://tempuri.org/"><userName>splickit</userName><password>Pcn9CCCr</password> </UserCredentials></soap12:Header><soap12:Body><AddPoints xmlns="http://tempuri.org/"><MerchantRockCommID>TES92130</MerchantRockCommID><CustomerCardNumber>900011062412</CustomerCardNumber><NumPoints>10.50</NumPoints><CardAction>CardKeyed</CardAction><ClerkID>27</ClerkID><CheckNumber>'.$new_order['order_id'].'</CheckNumber><AccountType>CardNumber</AccountType><SurveyFlag>false</SurveyFlag><ResponseFormat>Json</ResponseFormat></AddPoints></soap12:Body></soap12:Envelope>';
		$this->assertEquals($expected, $xml);
    }

	function testPlaceOrderWithFPNMerchantButNoLoyaltyNumberValidateNoCreationOfFpnLoyaltyMessage()
	{
		setContext("com.splickit.eastcoastwings");
		$menu_id = createTestMenuWithOneItem("Test Item 1");
		$merchant_resource = createNewTestMerchant($menu_id);
		$merchant_resource->brand_id = 379;
		$merchant_resource->save();

		//assign FPN stuff
		$mfpnm_adapter = new MerchantFPNMapAdapter($mimetypes);
		Resource::createByData($mfpnm_adapter, array("merchant_id" => $merchant_id, "fpn_merchant_key" => '41Mk65OH04krz32O4ZG2vy3Oj884882A', "fpn_merchant_pin" => '1405', "merchant_rock_comm_id" => "TES92130"));

		$merchant_id = $merchant_resource->merchant_id;

		$user_resource = createNewUser(array('flags' => '1C20000001'));
		$user_id = $user_resource->user_id;

		logTestUserIn($user_id);

        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id,'pickup','dum note',1);
        $order_resource = placeOrderFromOrderData($order_data);
        $this->assertNotNull($order_resource, "should have created an order");
		$this->assertNull($order_resource->error, "should not have gotten an error: " . $order_resource->error);

		// now verify the message was NOT created
		$mmha = new MerchantMessageHistoryAdapter($mimetypes);
		$fpn_loyalty_message_record = $mmha->getRecord(array("order_id" => $order_resource->order_id, "message_format" => 'C'));
		$this->assertNull($fpn_loyalty_message_record, "shold NOT have found the FPN loyalty message record");
	}

	function testPlaceOrderWithFPNMerchantValidateCreationOfFpnLoyaltyMessage()
    {
    	setContext("com.splickit.eastcoastwings");
    	$menu_id = createTestMenuWithOneItem("Test Item 1");
    	$merchant_resource = createNewTestMerchant($menu_id);
    	$merchant_resource->brand_id = 379;
    	$merchant_resource->save();
    	
    	//assign FPN stuff
    	$mfpnm_adapter = new MerchantFPNMapAdapter($mimetypes);
    	Resource::createByData($mfpnm_adapter,array("merchant_id"=>$merchant_id,"fpn_merchant_key"=>'41Mk65OH04krz32O4ZG2vy3Oj884882A',"fpn_merchant_pin"=>'1405',"merchant_rock_comm_id"=>"TES92130"));
    	
    	$merchant_id = $merchant_resource->merchant_id;
    	
    	$user_resource = createNewUser(array('flags'=>'1C20000001'));
    	$user_id = $user_resource->user_id;
    	
    	//give them FPN loyalty number
    	$ubpma = new UserBrandPointsMapAdapter($mimetypes);
    	Resource::createByData($ubpma,array("user_id"=>$user_id,"brand_id"=>379,"loyalty_number"=>"900011062412"));
    	
    	logTestUserIn($user_id);

        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id,'pickup','dum note',1);
        $order_resource = placeOrderFromOrderData($order_data);

        $this->assertNotNull($order_resource,"should have created an order");
    	$this->assertNull($order_resource->error,"should not have gotten an error: ".$order_resource->error);
    	
    	// now verify the message was created
    	$mmha = new MerchantMessageHistoryAdapter($mimetypes);
    	$fpn_loyalty_message_record = $mmha->getRecord(array("order_id"=>$order_resource->order_id,"message_format"=>'C'));
    	$this->assertNotNull($fpn_loyalty_message_record,"shold have found the FPN loyalty message record");
    	$this->assertEquals("service=FpnLoyalty", $fpn_loyalty_message_record['info']);
    	$actual_body = $fpn_loyalty_message_record['message_text'];
    	$expected_body = '<?xml version="1.0" encoding="utf-8"?><soap12:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap12="http://www.w3.org/2003/05/soap-envelope"><soap12:Header><UserCredentials xmlns="http://tempuri.org/"><userName>splickit</userName><password>Pcn9CCCr</password> </UserCredentials></soap12:Header><soap12:Body><AddPoints xmlns="http://tempuri.org/"><MerchantRockCommID>TES92130</MerchantRockCommID><CustomerCardNumber>900011062412</CustomerCardNumber><NumPoints>'.$order_resource->order_amt.'</NumPoints><CardAction>CardKeyed</CardAction><ClerkID>27</ClerkID><CheckNumber>'.$order_resource->order_id.'</CheckNumber><AccountType>CardNumber</AccountType><SurveyFlag>false</SurveyFlag><ResponseFormat>Json</ResponseFormat></AddPoints></soap12:Body></soap12:Envelope>';
		$this->assertEquals($expected_body, $actual_body);
    	return $fpn_loyalty_message_record['map_id'];
    }

    /**
     * @depends testPlaceOrderWithFPNMerchantValidateCreationOfFpnLoyaltyMessage
     * @param $map_id
     */
    function testSendFpnLoyaltyMessage($map_id)
    {
    	$mmha = new MerchantMessageHistoryAdapter($mimetypes);
    	$message_resource = Resource::find($mmha,''.$map_id);
    	$this->assertNotNull($message_resource);
    	$curl_controller = new CurlController($mt, $u, $r,5);
    	$response = $curl_controller->sendThisMessage($message_resource);
    	
    	// verify it was sent
    	$sent_message_resource = Resource::find($mmha,''.$map_id);
    	$this->assertEquals('S', $sent_message_resource->locked);
    	$expected_response = '{"xml":{"Reward":"REWARD:0.00","status":"success","Approved":"29916540","Clerk":"27","Check":"9780653","PointsAdded":"10.50","TotalPoints":["369.50","369.50"],"TotalSaved":"0.00","TotalVisits":"37","GiftCardBalance":"0.00","RewardCashBalance":"0.00","CustomReceiptMessages":"Register your Rewards Number and receive\na R5.00 instant reward\n"}}';
    	$actual_response = $sent_message_resource->response;
		$this->assertEquals($expected_response, $actual_response);    	
    }
    
    static function setUpBeforeClass()
    {
    	ini_set('max_execution_time',300);
    	      $db = DataBase::getInstance(); $mysqli = $db->getConnection();$mysqli->begin_transaction(); ;
    	$_SERVER['request_time1'] = microtime(true);    	
    }
    
	static function tearDownAfterClass()
    {
    	$db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    FpnLoyaltyTest::main();
}

?>