<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class FpnTest extends PHPUnit_Framework_TestCase
{
	var $stamp;

	function setUp()
	{
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
			
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
    }
       
    function testPlaceOrderWithFPNMerchant()
    {
    	setContext("com.splickit.eastcoastwings");
    	$menu_id = createTestMenuWithOneItem("Test Item 1");
    	$merchant_resource = createNewTestMerchant($menu_id);
    	$merchant_resource->brand_id = 379;
    	$merchant_resource->cc_processor = 'F';
    	$merchant_resource->save();
    	
    	//assign FPN stuff
    	$mfpnm_adapter = new MerchantFPNMapAdapter($mimetypes);
    	Resource::createByData($mfpnm_adapter,array("merchant_id"=>$merchant_id,"fpn_merchant_key"=>'41Mk65OH04krz32O4ZG2vy3Oj884882A',"fpn_merchant_pin"=>'1405',"merchant_rock_comm_id"=>"TES92130"));
    	
    	$merchant_id = $merchant_resource->merchant_id;
    	
    	$user_resource = createNewUser();
    	$user = $user_resource->getDataFieldsReally();
    	$user_id = $user_resource->user_id;
    	logTestUserIn($user_id);
    	    	
    	$json_encoded_cc_data = "{\"jsonVal\":{\"cc_exp_date\":\"03/2020\",\"cc_number\":\"4005562233445564\",\"cvv\":\"123\",\"zip\":\"90036\"}}"; 
    	$request = new Request();
    	$request->url = '/app2/phone/users';
    	$request->method = "post";
    	$request->body = $json_encoded_cc_data;
    	$request->mimetype = 'Applicationjson';
    	$request->_parseRequestBody();

    	$user_controller = new UserController($mt, $user, $request,5);
    	// save cc
    	$response = $user_controller->updateUser();
    	$this->assertNull($response->error);

    	// now get logged in user again since their flags have changed
    	$user_resource = Resource::find(new UserAdapter($mimetypes),"$user_id");
    	$user = $user_resource->getDataFieldsReally();
    	$_SERVER['AUTHENTICATED_USER'] = $user;

        $order_data = getSimpletCartArrayByMerchantId($merchant_id);
        $order_resource = placeOrderFromOrderData($order_data);

        $order_id = $order_resource->order_id;
		$this->assertNotNull($order_resource);
		$this->assertNull($order_resource->error);
		$this->assertTrue($order_resource->order_id > 1000,"Bad order id of: ".$order_resource->order_id);  
		
		// check balance change table
		$bc_adapter = new BalanceChangeAdapter($mimetypes);
		$data['process'] = 'CCpayment';
		$data['order_id'] = $order_id;
		$bc_records = $bc_adapter->getRecordsForOrderId($data);
		$cc_record = $bc_records[0];
		$this->assertNotNull($cc_record);
		return $order_id;
    }
    
    /**
     * @depends testPlaceOrderWithFPNMerchant
     */
    function testCreditVoid($order_id)
    {
		$bc_adapter = new BalanceChangeAdapter($mimetypes);
		$data['process'] = 'CCpayment';
		$data['order_id'] = $order_id;
		$bc_records = $bc_adapter->getRecordsForOrderId($data);
		$balance_change_row = $bc_records[0];
    	$ref_num_string = $balance_change_row['cc_transaction_id'];
    	$s = explode("=", $ref_num_string);
    	$ref_no = $s[1];
    	$cc_functions = CreditCardFunctions::creditCardFunctionsFactory('fpnpay');
    	$return_data = $cc_functions->creditVoidTransaction($balance_change_row, "0.00", $merchant_resource);
    	$this->assertEquals(100, $return_data['response_code']);
    }
    
    function testFPNGoodCard()
	{
		$card_data['cc_number'] = "4005562233445564";
		$card_data['cc_exp'] = '0314';
		$card_data['zip'] = "90036";
		$card_data['cv2'] = "435";
		
		$user_resource = Resource::dummyFactory(array("first_name"=>"Test T","last_name"=>"Jones"));
		
		$fz_payments = new FranchisePayments();
		$result_data = $fz_payments->process(1.00, $user_resource, 1234, $merchant , $card_data);
		$this->assertEquals("Approved", $result_data['responsetext']);
		
	}
	
    function testFPNBadCardExp()
	{
		$card_data['cc_number'] = "4005562233445564";
		$card_data['cc_exp'] = '0312';
		$card_data['zip'] = "90036";
		$card_data['cv2'] = "435";
		
		$user_resource = Resource::dummyFactory(array("first_name"=>"Test T","last_name"=>"Jones"));
		
		$fz_payments = new FranchisePayments();
		$result_data = $fz_payments->process(2.00, $user_resource, 1234, $merchant , $card_data);
		$this->assertEquals("Declined", $result_data['responsetext']);
		$this->assertEquals("Credit card has expired.",$result_data['error']);
	}

	static function setUpBeforeClass()
    {
    	ini_set('max_execution_time',300);
    	      $db = DataBase::getInstance(); $mysqli = $db->getConnection();$mysqli->begin_transaction(); ;
    	$_SERVER['request_time1'] = microtime(true);
    }
    
	static function tearDownAfterClass()
    {
    	$db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
    }
	
    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    FpnTest::main();
}

?>