<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class GiftTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $user;
	var $merchant_id;
	var $menu_id;
	var $giftee_user_id;
	var $giftee_user_resource;

	function setUp()
	{
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		
		setContext("com.splickit.order");		
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		
    	$giftee_user_resource = createNewUser();
    	$this->giftee_user_id = $giftee_user_resource->user_id;
    	$this->giftee_user_resource = $giftee_user_resource;
    	
    	$this->user = logTestUserIn($_SERVER['unit_test_ids']['user_id']);
    	$this->merchant_id = $_SERVER['unit_test_ids']['merchant_id'];
    	$this->menu_id = $_SERVER['unit_test_ids']['menu_id'];

        setContext('com.splickit.worldhq');
	}
	
	function tearDown() 
	{
		//delete your instance
		setProperty("test_mandril_fail",'false');
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->user);
    }

    function testGetGiftList()
    {
    	$gift_adapter = new GiftAdapter($mimetypes);
    	$resource = $gift_adapter->getGiftHistory(20000);
    	$this->assertNotNull($resource->gifts);
    	
    }
    
    function testPlaceOrderWithUseGiftButNoActiveGifts()
    {
    	//over ride loged in user
		$this->user = logTestUserIn($this->giftee_user_id);
    	
		$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->merchant_id,'pickup','skip hours');
		$tip = $order_data['tip'];
		$grand_total = $order_data['grand_total'];
		
		$order_data['promo_code'] = 'usegift';
		$json['jsonVal'] = $order_data;
    	$json_encoded_data = json_encode($json);
 
		$order_resource = placeTheOrder($json_encoded_data);
		$this->assertNotNull($order_resource);
		$this->assertEquals("Sorry, you do not appear to have any active gifts", $order_resource->error);	
    }

    function testPlaceSmallOrderWithUseGiftGoodGift()
    {
    	$giftee_user_resource = $this->giftee_user_resource;
    	$giftee_user_id = $giftee_user_resource->user_id;
    	$gift_amt = rand(2000, 3000)/100; 	
    	$gift_resource = createGift($giftee_user_resource,$gift_amt);
    	$this->assertNull($gift_resource->error,"there should not have been an error: ".$gift_resource->error);
    	$this->assertEquals($this->giftee_user_id, $gift_resource->receiver_user_id);
    	$this->assertEquals($gift_amt, $gift_resource->amt);
       	
/*    	$user_adapter = new UserAdapter($mimetypes);
    	$user_resource = Resource::find($user_adapter,'20011');
    	$user_adapter->setPassword($user_resource, 'welcome');
*/    	
  //  	$user = logTestUserIn($giftee_user_resource->user_id);
//		$user_session_controller = new UsersessionController($mt, $user, $r, 5);
//		$user_session = $user_session_controller->getUserSession($giftee_user_resource);
		//$user_
		
		$login_adapter  = new LoginAdapter($mimetypes);
		$logged_in_giftee_user_resource = $login_adapter->doAuthorizeWithSpecialUserValidation($this->giftee_user_id, 'welcome', $request_data);
		$this->assertNull($logged_in_giftee_user_resource->error);
		$this->assertNotNull($logged_in_giftee_user_resource->gift_resource,"should have found a gift");
		$gift_resource_on_user = $logged_in_giftee_user_resource->gift_resource;
		$this->assertNull($gift_resource_on_user->_adapter);
		$this->assertNull($gift_resource_on_user->_exists);
		$this->assertNull($gift_resource_on_user->_representation);
		
		$user = $logged_in_giftee_user_resource->getDataFieldsReally();
		//$user['gift_resource'] = $resource;
		$user['gift_resource'] = $logged_in_giftee_user_resource->gift_resource;
		$_SERVER['AUTHENTICATED_USER'] = $user;
		$_SERVER['AUTHENTICATED_USER_ID'] = $user_id;
		$this->user = $user;
		$user_id = $user['user_id'];
    	
		$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->merchant_id,'pickup','skip hours');
		$tip = $order_data['tip'];
		$grand_total = $order_data['grand_total'];
		
		$order_data['promo_code'] = 'usegift';
		//$json['jsonVal'] = $order_data;
    	//$json_encoded_data = json_encode($json);
 
		//$order_resource = placeTheOrder($json_encoded_data);
		$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
		$this->assertNotNull($order_resource);
		$this->assertNull($order_resource->error);
		$this->assertTrue($order_resource->order_id > 1000);
		
		$order_id = $order_resource->order_id;
		
		$gifter_user_id = $gift_resource->gifter_user_id;
		$new_user_resource = Resource::find(new UserAdapter($mimetypes), "$user_id", $options);
		$this->assertTrue($new_user_resource->_exists);
		$this->assertEquals(0.00, $new_user_resource->balance);
		$balance_change_adapter = new BalanceChangeAdapter($mimetypes);
		if ($balance_change_records = $balance_change_adapter->getRecords(array("order_id"=>$order_id), $options)) {
			$balance_change_rows_by_user_id = setBalanceChangeHashFromBalanceChangeArrayFromOrder($balance_change_records);
		}
		$this->assertCount(3, $balance_change_rows_by_user_id);
		$this->assertEquals($balance_before, -$balance_change_rows_by_user_id["$user_id-Order"]['balance_before'],"order balance before error");
		$this->assertEquals($order_resource->grand_total, -$balance_change_rows_by_user_id["$user_id-Order"]['charge_amt'],"order charge amt error");
		$this->assertEquals($balance_before+$order_resource->grand_total, -$balance_change_rows_by_user_id["$user_id-Order"]['balance_after'],"order balance after error");
		
		$this->assertEquals($order_resource->grand_total, -$balance_change_rows_by_user_id["$user_id-GIFT USED"]['balance_before'],"Gift balance before error");
		$this->assertEquals($order_resource->grand_total, $balance_change_rows_by_user_id["$user_id-GIFT USED"]['charge_amt'],"gift charge amount error");
		$this->assertEquals($new_user_resource->balance, $balance_change_rows_by_user_id["$user_id-GIFT USED"]['balance_after'],"gift balance after error");    	
		    	
		$this->assertEquals(0, -$balance_change_rows_by_user_id["$gifter_user_id-CCpayment"]['balance_before']," cc payment balance before error");
		$this->assertEquals($order_resource->grand_total, $balance_change_rows_by_user_id["$gifter_user_id-CCpayment"]['charge_amt'],"cc payment charge amt error");
		$this->assertEquals(0, $balance_change_rows_by_user_id["$gifter_user_id-CCpayment"]['balance_after'],"cc payment balance after error");    	
		    	
    }
        
    function testPlaceOrderWithUseGiftBigOrderNoCC()
    {
    	$giftee_user_resource = $this->giftee_user_resource;
	 	$gift_amt = rand(500,700)/100;
    	$gift_resource = createGift($giftee_user_resource,$gift_amt);
    	$this->assertNull($gift_resource->error,"there should not have been an error: ".$gift_resource->error);
    	$this->assertEquals($giftee_user_resource->user_id, $gift_resource->receiver_user_id);
    	$this->assertEquals($gift_amt, $gift_resource->amt);
       	
/*    	$user_adapter = new UserAdapter($mimetypes);
    	$user_resource = Resource::find($user_adapter,'20011');
    	$user_adapter->setPassword($user_resource, 'welcome');
*/    	
		$login_adapter  = new LoginAdapter($mimetypes);
		$logged_in_giftee_user_resource = $login_adapter->doAuthorizeWithSpecialUserValidation($this->giftee_user_id, 'welcome', $request_data);
		$this->assertNull($logged_in_giftee_user_resource->error);
		$this->assertNotNull($logged_in_giftee_user_resource->gift_resource,"should have found a gift");
		
		$user = $logged_in_giftee_user_resource->getDataFieldsReally();
		$user['gift_resource'] = $gift_resource;
		$_SERVER['AUTHENTICATED_USER'] = $user;
		$_SERVER['AUTHENTICATED_USER_ID'] = $user_id;
		$this->user = $user;
    	
		$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->merchant_id,'pickup','skip hours',7);
		$tip = $order_data['tip'];
		$grand_total = $order_data['grand_total'];
		
		$order_data['promo_code'] = 'usegift';
		$json['jsonVal'] = $order_data;
    	$json_encoded_data = json_encode($json);
 
		$order_resource = placeTheOrder($json_encoded_data);
		$this->assertNotNull($order_resource);
		$this->assertNotNull($order_resource->error,"Should have found an error");
		$this->assertEquals('Your purchase amount is greater than your gift. Please enter your credit card info', $order_resource->error);
		$this->assertEquals(150,$order_resource->error_code);
    	
    }
        
    function testPlaceOrderThatIsLargerThanGiftSoUseCreditCard()
    {
    	$giftee_user_resource = createNewUserWithCC();
    	logTestUserIn($this->user['user_id']);
    	$gift_amt = rand(500,700)/100;
    	$gift_resource = createGift($giftee_user_resource,$gift_amt);
    	$this->assertNull($gift_resource->error,"there should not have been an error: ".$gift_resource->error);
    	$this->assertEquals($giftee_user_resource->user_id, $gift_resource->receiver_user_id,"gift user ids are wrong");
    	$this->assertEquals($gift_amt, $gift_resource->amt,"gift amount is wrong");
    	
		$login_adapter  = new LoginAdapter($mimetypes);
		$logged_in_giftee_user_resource = $login_adapter->doAuthorizeWithSpecialUserValidation($giftee_user_resource->user_id, 'welcome', $request_data);
		$this->assertNull($logged_in_giftee_user_resource->error);
		$this->assertNotNull($logged_in_giftee_user_resource->gift_resource,"should have found a gift");
    	
		$user = $logged_in_giftee_user_resource->getDataFieldsReally();
		$user['gift_resource'] = $gift_resource;
		$_SERVER['AUTHENTICATED_USER'] = $user;
		$_SERVER['AUTHENTICATED_USER_ID'] = $user_id;
		$this->user = $user;
   	
		$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->merchant_id,'pickup','skip hours',10);
		$tip = $order_data['tip'];
		$grand_total = $order_data['grand_total'];
		
		// first try with too much $
		//$order_data['tip'] = $tip + $gift_amt;
		//$order_data['grand_total'] = $grand_total + $gift_amt;
		$order_data['promo_code'] = 'usegift';
		$json['jsonVal'] = $order_data;
    	$json_encoded_data = json_encode($json);
 
		$order_resource = placeTheOrder($json_encoded_data);
		$this->assertNotNull($order_resource,"should have created a resource");
		$this->assertNull($order_resource->error);
		$order_id = $order_resource->order_id;
		$this->assertTrue($order_id > 1000,"Bad order id of: ".$order_resource->order_id);  

		// GIFT resource should have gotten updated
		$gift_id = $gift_resource->gift_id;
		$new_gift_resource = Resource::find(new GiftAdapter($mimetypes),''.$gift_id);
		$this->assertEquals($order_id, $new_gift_resource->order_id,"shouljd have found an order id on the gift");
		myerror_log("used on is: ".$new_gift_resource->used_on);
		
		//check balance change table to see if CC was only run for the order amount.
		$balance_change_adapter = new BalanceChangeAdapter($mimetypes);
		$bc_data['user_id'] = $new_gift_resource->gifter_user_id;
		$bc_data['process'] = 'CCpayment';
		$bc_data['order_id'] = $order_id;
		$bc_options[TONIC_FIND_BY_METADATA] = $bc_data;
		$bc_resource = Resource::find($balance_change_adapter,null,$bc_options);
		$this->assertNotNull($bc_resource);
		$this->assertEquals($gift_amt, $bc_resource->charge_amt,"gift amount should have matched balance change row charge amount in CCpayment row");
		
		// now check to see the additional row from the extra charge
		$bc_data['user_id'] = $giftee_user_resource->user_id;
		$bc_data['process'] = 'CCpayment';
		$bc_data['order_id'] = $order_id;
		$bc_options[TONIC_FIND_BY_METADATA] = $bc_data;
		$bc_resource2 = Resource::find($balance_change_adapter,null,$bc_options);
		$this->assertNotNull($bc_resource2,"should have found a CC payment row for the double billing amount");
		$second_charge = $order_resource->grand_total - $gift_amt;
		$this->assertEquals($second_charge, $bc_resource2->charge_amt,"second charge amount should match");
		
		// finally check the balance change row to make sure we set the user balance back to where it was.
		$bc_data['user_id'] = $giftee_user_resource->user_id;
		$bc_data['process'] = 'GIFTandCC';
		$bc_data['order_id'] = $order_id;
		$bc_options[TONIC_FIND_BY_METADATA] = $bc_data;
		$bc_resource3 = Resource::find($balance_change_adapter,null,$bc_options);
		$this->assertNotNull($bc_resource3,"should have found the GIFTandCC balance change row");
		$second_charge = $order_resource->grand_total - $gift_amt;
		$this->assertEquals($order_resource->grand_total, $bc_resource3->charge_amt,"charge amount should equal order grand total for GiftandCC row");
		$this->assertEquals(0.00,$bc_resource3->balance_after);
    }
// */
    
    function testGetActiveGifts()
    {

    	$giftee_user_resource = $this->giftee_user_resource;
    	$giftee_user_resource->flags = '1C20000001';
    	$giftee_user_resource->save();
    	
    	$gift_resource1 = createGift($giftee_user_resource);
    	$gift_resource1->expires_on = date('Y-m-d',time()+432000);
  		$gift_resource1->save();  	

    	$gift_resource2 = createGift($giftee_user_resource);
     	$gift_resource2->expires_on = date('Y-m-d',time()+1232000);
  		$gift_resource2->save();  	

    	$gift_data['receiver_user_id'] = $this->giftee_user_resource->user_id;
    	$gift_data['used_on'] = '0000-00-00 00:00:00';
    	$today = date("Y-m-d");
    	$gift_data['expires_on'] = array(">"=>$today);
		$gift_options[TONIC_FIND_BY_METADATA] = $gift_data;
		$gift_options[TONIC_SORT_BY_METADATA] = " expires_on DESC ";
		$gift_adapter = new GiftAdapter($mimetypes);
		$resources = Resource::findAll($gift_adapter,'',$gift_options);
		$this->assertEquals(2, sizeof($resources, $mode));	
		
		$soonest_expiring_gift_resource = $gift_adapter->getSoonestExpiringGiftResource($this->giftee_user_resource->user_id);
		$this->assertNotNull($soonest_expiring_gift_resource);
		$this->assertEquals($gift_resource1->gift_id,$soonest_expiring_gift_resource->gift_id);
			
    }

    function testCreateGiftExistingUser()
    {
    	$giftee_user_resource = $this->giftee_user_resource;
    	$gift_resource = createGift($giftee_user_resource,20.00);
    	$this->assertNull($gift_resource->error);
    	$this->assertTrue($gift_resource->receiver_user_id > 19999);
    	$this->assertEquals(20.00, $gift_resource->amt);
    	
    	//now test to make sure the menu has the correct message
    	$user = logTestUserInWithGift($giftee_user_resource->user_id,$gift_resource);
		$this->user = $user;
    	
		$request = new Request();
		$request->url = "/merchants/".$this->merchant_id;
		
		$merchant_controller = new MerchantController($mt, $user,$request,5);
		$resource_merchant = $merchant_controller->getMerchant();
		//myerror_log("resource: ".$resource->__toString());
		$this->assertNull($resource_merchant->error,"there was an error returned on the get Menu call");
		$this->assertEquals($this->menu_id, $resource_merchant->menu['menu_id']);
		$this->assertContains('You have gift of up to $20', $resource_merchant->user_message);
    }

    function testCreateGiftNOTExistingUser()
    {
    	$id = rand(1000, 9999);
    	$gift_data['receiver_email'] = $id.'adam@order140.com';
    	$gift_data['amt'] = 20.00;

    	$gift_data['expires_on'] = date("Y-m-d", time()+(14*24*60*60));
    	$gift_data['personal_note'] = "here is the note";
    	$json_encoded_data = json_encode(array("jsonVal"=>$gift_data));

   		$request = new Request();
    	$request->method = "post";
    	$request->body = $json_encoded_data;
    	$request->mimetype = 'Applicationjson';
    	$request->_parseRequestBody();
    	$gift_adapter = new GiftAdapter($mimetypes);
    	$gift_resource = $gift_adapter->createGiftFromRequest($request);
    	$this->assertNull($gift_resource->error);
    	$this->assertTrue($gift_resource->receiver_user_id == 0);
    	$this->assertEquals(20.00, $gift_resource->amt);
    	
    	return $gift_resource;
    }
    
    /**
     * @depends testCreateGiftNOTExistingUser
	 *
     * @param $gift_resource
     */
    function testClaimGift($gift_resource)
    {
    	// so this tests a user claiming a gift that is using a differnt email than was used for the gift.
    	$token = $gift_resource->gift_token;
    	
    	// get the user that was logged in
    	$user = $this->user;
		
		$gift_adapter = new GiftAdapter($mimetypes);
		$new_gift_resource = $gift_adapter->claimGift($token);
		$this->assertNull($new_gift_resource->error,"should not have found an error: ".$new_resource->error);
		$this->assertEquals($gift_resource->gift_id, $new_gift_resource->gift_id,"gift_id's should have been equal");
		$this->assertEquals($user['user_id'],$new_gift_resource->receiver_user_id);
		
		$user['gift_resource'] = $new_gift_resource;
		
		return $user;
    }
    
    /**
     * 
     * @depends testClaimGift
     * @param $user
     */
    function testPlaceOrderUsingGift($user)
    {
    	$user_id = $user['user_id'];

    	$gift_resource = $user['gift_resource'];
    	$user['flags'] = '1000000001';
    	$_SERVER['AUTHENTICATED_USER'] = $user;
		$_SERVER['AUTHENTICATED_USER_ID'] = $user['user_id'];
    	
		$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->merchant_id,'pickup','skip hours');
		$tip = $order_data['tip'];
		$grand_total = $order_data['grand_total'];
		
		// first try with too much $
		$order_data['tip'] = $tip + 20.00;
		$order_data['grand_total'] = $grand_total + 20.00;
		$order_data['promo_code'] = 'usegift';
		$json['jsonVal'] = $order_data;
    	$json_encoded_data = json_encode($json);
 
		$order_resource = placeTheOrder($json_encoded_data);
		$this->assertNotNull($order_resource);
		$this->assertNotNull($order_resource->error);
		$this->assertEquals("Sorry, this gift has a maximum value of $20.00.  Please add a Credit Card to your account or remove something from your cart", $order_resource->error);

		// now try with a good amount but expired gift
		$gift_resource = $user['gift_resource'];
		$expires_on = date('Y-m-d',time()-(7*24*60*60));
		$gift_resource->expires_on = $expires_on;
		$gift_resource->save();
		
		$order_data['tip'] = $tip;
		$order_data['grand_total'] = $grand_total;
		$json['jsonVal'] = $order_data;
    	$json_encoded_data = json_encode($json);
		$order_resource = placeTheOrder($json_encoded_data);
		$this->assertNotNull($order_resource);
		$this->assertNotNull($order_resource->error);
		$this->assertEquals("Sorry this gift expired on ".$expires_on, $order_resource->error);
    	
		// reset expires on and we should be good to go.
		
		//lets add acredit to the user to make sure it doesn't get used.
		$user_resource = Resource::find(new UserAdapter($mimetypes),''.$user['user_id']);
		$user_resource->balance = 50.00;
		$user_resource->save();
		
		$gift_resource->expires_on = date("Y-m-d", time()+(14*24*60*60));
		$gift_resource->save();
		
    	$order_resource = placeTheOrder($json_encoded_data);
    	$this->assertNotNull($order_resource);
		$this->assertNull($order_resource->error);
		$order_id = $order_resource->order_id;
		$this->assertTrue($order_id > 1000,"Bad order id of: ".$order_resource->order_id);  

		// GIFT resource should have gotten updated
		$gift_id = $gift_resource->gift_id;
		$new_gift_resource = Resource::find(new GiftAdapter($mimetypes),''.$gift_id);
		$this->assertEquals($order_id, $new_gift_resource->order_id);
		myerror_log("used on is: ".$new_gift_resource->used_on);
		
		//check balance change table to see if CC was only run for the order amount.
		$balance_change_adapter = new BalanceChangeAdapter($mimetypes);
		$bc_data['user_id'] = $new_gift_resource->gifter_user_id;
		$bc_data['process'] = 'CCpayment';
		$bc_data['order_id'] = $order_id;
		$bc_options[TONIC_FIND_BY_METADATA] = $bc_data;
		$bc_resource = Resource::find($balance_change_adapter,null,$bc_options);
		$this->assertNotNull($bc_resource);
		$this->assertEquals($order_resource->grand_total, $bc_resource->charge_amt);

    }

    function testCreateGiftNOTExistingUserBadEmail()
    {
    	$id = rand(1000, 9999);
    	setProperty("test_mandril_fail",'true');
    	$gift_data['receiver_email'] = $id.'adam@adam@order140.com';
    	$gift_data['amt'] = 22.00;

    	$gift_data['expires_on'] = date("Y-m-d", time()+(14*24*60*60));
    	$gift_data['personal_note'] = "here is the note";
    	$json_encoded_data = json_encode(array("jsonVal"=>$gift_data));

   		$request = new Request();
    	$request->method = "post";
    	$request->body = $json_encoded_data;
    	$request->mimetype = 'Applicationjson';
    	$request->_parseRequestBody();
    	$gift_adapter = new GiftAdapter($mimetypes);
    	$resource = $gift_adapter->createGiftFromRequest($request);
    	$this->assertNotNull($resource->error);
    	$this->assertEquals('There was an error trying to send the gift to the listed email, please check your entry.', $resource->error);
    }
    
    function testCreateGiftNoCCForGiver()
    {
    	// set the flags to appear as if the user does not have a cc on file
    	$user_resource = createNewUser();

    	$this->user = logTestUserIn($user_resource->user_id);

    	$gift_data['receiver_email'] = $this->giftee_user_resource->email;
    	$gift_data['amt'] = 20.00;

    	$gift_data['expires_on'] = date("Y-m-d", time()+(14*24*60*60));
    	$gift_data['personal_note'] = "here is the note";
    	$json_encoded_data = json_encode($gift_data);

   		$request = new Request();
    	$request->method = "post";
    	$request->body = $json_encoded_data;
    	$request->mimetype = 'Applicationjson';
    	$request->_parseRequestBody();
    	$gift_adapter = new GiftAdapter($mimetypes);
    	$resource = $gift_adapter->createGiftFromRequest($request);
    	$this->assertNotNull($resource->error,"should have gotten an error since the gifter does not have a CC on file");
    	$this->assertEquals("Sorry, You must have a working credit card on file to send a gift.",$resource->error);
    	
    	// now reset so we can save the gift and then try to order, should not get a failure
		$user_resource->flags = "1C00000001";
		$user_resource->save();
		$this->user = logTestUserIn($user_resource->user_id);
    	$gift_resource = $gift_adapter->createGiftFromRequest($request);
    	$this->assertNull($gift_resource->error,"should not have gotten an error");
    	$giftee_user = $this->giftee_user_resource->getDataFieldsReally();
    	$giftee_user['gift_resource'] = $gift_resource;
		return $giftee_user;
    }
    
    /**
     * 
     * @depends testCreateGiftNoCCForGiver
     * @param $giftee_user
     */
    /*testPlaceOrderWithGiftButBadCC($giftee_user)
    {
    	$gift_resource = $giftee_user['gift_resource'];
    	$_SERVER['HTTP_NO_CC_CALL_WITH_FAIL'] = true;

    	$_SERVER['AUTHENTICATED_USER'] = $giftee_user;
    	$_SERVER['AUTHENTICATED_USER_ID'] = $giftee_user['user_id'];
    	
    	$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->merchant_id,'pickup','skip hours');
		$order_data['promo_code'] = 'usegift';
    	//$json['jsonVal'] = $order_data;
    	$json_encoded_data = json_encode($order_data);
		$order_resource = placeTheOrder($json_encoded_data);
		$this->assertNotNull($order_resource);
		$this->assertNotNull($order_resource->error);
		myerror_log("the error is: ".$order_resource->error);
		$this->assertEquals("We're sorry but there was an error charging the credit card from your gifter.  We have notified the individual who sent you the gift and it will hopefully be taken care of shortly. Sorry for the inconvenience.", $order_resource->error);
    	
		//now check to make sure the gift is still valid
		$gift_id = $gift_resource->gift_id;
		$new_gift_resource = Resource::find(new GiftAdapter($mimetypes),''.$gift_id);
		$this->assertEquals(null, $new_gift_resource->order_id);
		$this->assertEquals('0000-00-00 00:00:00',$new_gift_resource->used_on);
		$this->assertTrue($new_gift_resource->used_amt < 0.01);
    }*/

    static function setUpBeforeClass()
    {
    	ini_set('max_execution_time',300);
    	      $db = DataBase::getInstance(); $mysqli = $db->getConnection();$mysqli->begin_transaction(); ;
    	$_SERVER['request_time1'] = microtime(true);    	
		    	
		//map it to a menu
    	$menu_id = createTestMenuWithNnumberOfItems(5);
    	$ids['menu_id'] = $menu_id;
    	
    	$merchant_resource = createNewTestMerchant($menu_id);
    	$ids['merchant_id'] = $merchant_resource->merchant_id;
    	  	    	
    	$user_resource = createNewUser();
    	$user_resource->flags = '1C00000001';
    	$user_resource->save();
    	$ids['user_id'] = $user_resource->user_id;

    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    	
    }
    
	static function tearDownAfterClass()
    {
    	$db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
    }
        
    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    GiftTest::main();
}

?>