<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class GprsMessageTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $user;
	var $merchant_id;
	var $menu_id;

	function setUp()
	{
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		
		setContext("com.splickit.order");
		setProperty("gprs_tunnel_merchant_shutdown", "false");
		
		$user_resource = SplickitController::getResourceFromId($_SERVER['unit_test_ids']['user_id'], 'User');
		$this->user = $user_resource->getDataFieldsReally();
    	$this->merchant_id = $_SERVER['unit_test_ids']['merchant_id'];
    	$this->menu_id = $_SERVER['unit_test_ids']['menu_id'];
    	
    	logTestUserIn($user_resource->user_id);
		// set to true since the mock object will take care of it.
		setProperty("test_sms_messages_on",'true');
	}
	
	function tearDown() 
	{
		//delete your instance
		unset($this->user);
		unset($this->merchant_id);
		unset($this->menu_id);
		$_SERVER['STAMP'] = $this->stamp;
		setProperty("gprs_tunnel_merchant_shutdown", "false");
    }

    function testCleanMessageForSending() {
      $message_resource = Resource::createByData(new MySQLAdapter($m),$data);
      $order_details = array();
      $order_detail = array('note' => "Boo#gers@", 'item_print_name' => "Bo@ogers", 'size_print_name' => "Boo*g@ers");
      $order_detail['order_detail_modifiers'] = array(array('mod_print_name' => "In# @My*; Nose"));
      $order_detail['order_detail_hold_it_modifiers'] = array(array('mod_print_name' => "In# @My*; Nose"));
      $order_detail['order_detail_mealdeal'] = array(array('mod_print_name' => "In# @My*; Nose"));
      $order_detail['order_detail_sides'] = array(array('mod_print_name' => "In# @My*; Nose"));
      $order_detail['order_detail_added_modifiers'] = array(array('mod_print_name' => "In# @My*; Nose"));
      $order_detail['order_detail_comeswith_modifiers'] = array(array('mod_print_name' => "In# @My*; Nose"));
      $order_details[] = $order_detail;
      $message_resource->order_details = $order_details;
      $gprs_controller = new GprsController($m,$u,$r);
      
      $cleaned_resource = $gprs_controller->cleanMessageForSending($message_resource);

      foreach($cleaned_resource->order_details as $cleaned_item) {
        $this->assertEquals($cleaned_item['note'], "Boogers", "We should be stripping characters from the item notes.");
        $this->assertEquals($cleaned_item['item_print_name'], "Boogers", "We should be stripping characters from item print name.");
        $this->assertEquals($cleaned_item['size_print_name'], "Boogers", "We should be stripping characters from the size print name.");
        
        foreach($cleaned_item['order_detail_modifiers'] as $mod) {
  	       $this->assertEquals($mod['mod_print_name'], "In My Nose", "We should be stripping characters from the modifier print name.");
  	    }
  	    
  	    foreach($cleaned_item['order_detail_hold_it_modifiers'] as $mod) {
  	      $this->assertEquals($mod['mod_print_name'], "In My Nose", "We should be stripping characters from the modifier print name.");
  	    }
  	    
  	    foreach($cleaned_item['order_detail_sides'] as $mod) {
  	      $this->assertEquals($mod['mod_print_name'], "In My Nose", "We should be stripping characters from the modifier print name.");
  	    }
  	    
  	    foreach($cleaned_item['order_detail_mealdeal'] as $mod) {
  	      $this->assertEquals($mod['mod_print_name'], "In My Nose", "We should be stripping characters from the modifier print name.");
  	    }
  	    
  	    foreach($cleaned_item['order_detail_comeswith_modifiers'] as $mod) {
  	      $this->assertEquals($mod['mod_print_name'], "In My Nose", "We should be stripping characters from the modifier print name.");
  	    }
  	    
  	    foreach($cleaned_item['order_detail_added_modifiers'] as $mod) {
  	      $this->assertEquals($mod['mod_print_name'], "In My Nose", "We should be stripping characters from the modifier print name.");
  	    }	    
      }
    }    
    
    function testGetSMSNumberFOrMerchantId()
    {
    	$set_number = '1234567890';
    	$merchant_resource = createNewTestMerchant();
    	$map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_resource->merchant_id,"message_format"=>'T',"delivery_addr"=>"$set_number","message_type"=>"A","info"=>"firmware=8.0"));
    	$gprs_controller = new GprsController($mt, $u, $r, 5);
    	$sms_no = $gprs_controller->getSMSNumberForMerchant($merchant_resource->merchant_id);
    	$this->assertEquals($set_number, $sms_no);	
    }
    
    function testHasPrinterCalledInRecently()
    {
    	$merchant_id = $this->merchant_id;
    	$gpciha = new GprsPrinterCallInHistoryAdapter($mimetypes);
		$id = Resource::createByData($gpciha,array("merchant_id"=>$merchant_id,"last_call_in"=>time()-180));
    	
    	$result = $gpciha->hasPrinterCalledInRecently($merchant_id,120);
    	$this->assertFalse($result);
    	
    	$gpciha->createRecord($merchant_id);
		$result = $gpciha->hasPrinterCalledInRecently($merchant_id,120);
    	$this->assertTrue($result);
    }
    
    function testTsobFailureToCallInFunctionality()
    {
    	$merchant_id = $this->merchant_id;
    	$gpciha = new GprsPrinterCallInHistoryAdapter($mimetypes);
    	$resource = Resource::find($gpciha,"$merchant_id",$options);
    	$resource->last_call_in = time()-600;
    	$resource->save();

    	$mmha = new MerchantMessageHistoryAdapter($mimetypes);
    	$map_id = $mmha->createMessage($merchant_id, null, 'Tsob', '3038844083', time(), 'A', $info, $message_text,'S',time(),1);
    	
    	$gpciha->checkForRecentCallInFromSobTextAndSendRoboCallIfNeeded($merchant_id);
    	
    	// since this was the first failure a robo call should have been created.
    	$sql = "SELECT * FROM Merchant_Message_History WHERE merchant_id = $merchant_id AND message_format = 'IA' AND map_id > $map_id";
    	$options[TONIC_FIND_BY_SQL] = $sql;
    	$message_resource = Resource::find($mmha,'',$options);
    	$this->assertNotNull($message_resource,"An IVR robo call message should have been created");

    	$gpciha->checkForRecentCallInFromSobTextAndAlertSupportIfNeeded($merchant_id);
    	$sql = "SELECT * FROM Merchant_Message_History WHERE message_format = 'E' AND map_id > ".$message_resource->map_id." AND message_text LIKE 'We have a new version printer that has not returned a morning call in request%'";
    	$options[TONIC_FIND_BY_SQL] = $sql;
    	$support_message_resource = Resource::find($mmha,'',$options);
    	$this->assertNotNull($support_message_resource,"A support email should have been created.");
    }
    
    function testCreateTsobAndActivityCheck()
    {
    	$merchant_id = $this->merchant_id;
    	// first reset hte call in history
    	$gmciha = new GprsPrinterCallInHistoryAdapter($mimetypes);
    	$gmciha_resource = Resource::find($gmciha,"$merchant_id");
    	$last_call_in = time() - 1200;
    	$gmciha_resource->last_call_in = $last_call_in;
    	$gmciha_resource->save();
    	
    	// erase the activity history

    	$mmha = new MerchantMessageHistoryAdapter($mimetypes);
    	$next_message_dt_tm = time()-5;
    	$tsob_map_id = $mmha->createMessage($merchant_id, null, 'Tsob', '3038844083', $next_message_dt_tm, 'A', $info, '***');
    	$message_resource = Resource::find($mmha,''.$tsob_map_id);
    	$text_controller = new TextController($mt, $u, $r);
    	$text_controller->sendThisMessage($message_resource);
    	
    	// now check to see if the activity got created.
    	$activity_history_adapter = new ActivityHistoryAdapter($mimetypes);
    	$activity_id = $text_controller->sob_activity_id;
		$this->assertTrue($activity_id > 1000,"Should have created an activity but no id was registered");    	
    	$activity_history_resource = Resource::find($activity_history_adapter,''.$activity_id);
    	$execute_object_method_activity = new ExecuteObjectFunctionActivity($activity_history_resource);
    	$execute_object_method_activity->doit();
    	
    	// should have scheduled a robo call
    	$sql = "SELECT * FROM Merchant_Message_History WHERE merchant_id = $merchant_id AND message_format = 'IA' AND map_id > $tsob_map_id";
    	$options[TONIC_FIND_BY_SQL] = $sql;
    	$message_resource = Resource::find($mmha,'',$options);
    	$this->assertNotNull($message_resource,"An IVR robo call message should have been created");
    	
    	// should have also scheduled another activity
    	$sql = "SELECT * FROM Activity_History WHERE activity='ExecuteObjectFunction' AND activity_id > $activity_id AND info LIKE '%checkForRecentCallInFromSobTextAndAlertSupportIfNeeded%'";
    	$options[TONIC_FIND_BY_SQL] = $sql;
    	$new_activity_history_resource = Resource::find($activity_history_adapter,'',$options);
    	$this->assertTrue($new_activity_history_resource->activity_id > 1000,"Should have created a check for call in and alert support activity but none was found");

    	// now run the new activity and see if it creates the support email
    	$new_execute_object_method_activity = new ExecuteObjectFunctionActivity($new_activity_history_resource);
    	$new_execute_object_method_activity->doit();
    	
    	$sql = "SELECT * FROM Merchant_Message_History WHERE message_format = 'E' AND map_id > ".$message_resource->map_id." AND message_text LIKE 'We have a new version printer that has not returned a morning call in request%'";
    	$options[TONIC_FIND_BY_SQL] = $sql;
    	$support_message_resource = Resource::find($mmha,'',$options);
    	$this->assertNotNull($support_message_resource,"A support email should have been created.");
    }

    function testCreateMessages()
    {
        setContext("com.splickit.snarfs");
        logTestUserIn($this->user['user_id']);

        setProperty('new_shadow_device_on','false');
        $merchant_resource = createNewTestMerchant($this->menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_id,"message_format"=>'GUA',"delivery_addr"=>"gprs","message_type"=>"X","info"=>"firmware=7.0"));
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_id,"message_format"=>'T',"delivery_addr"=>"7777777777","message_type"=>"A","info"=>"firmware=7.0","message_text"=>'***'));

        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id,'pickup','dum note',1);
        $order_resource = placeOrderFromOrderData($order_data);


        $this->assertNotNull($order_resource);
        $this->assertNull($order_resource->error);
        $this->assertTrue($order_resource->order_id > 1000,"Bad order id of: ".$order_resource->order_id);
        $order_id = $order_resource->order_id;
        $mmha = new MerchantMessageHistoryAdapter($m);
        $messages = $mmha->getAllOrderMessages($order_id);
        $messages_array = createHashmapFromArrayOfResourcesByFieldName($messages,'message_format');
        $text_message = $messages_array['T'];
        $gprs_message = $messages_array['GUA'];
        $this->assertEquals("7777777777",$gprs_message->message_delivery_addr,"should have the text address as part of the gprs");
        $this->assertEquals("***",$text_message->message_text,"should have created a ***");
        return $merchant_id;
    }

    /**
     * @depends testCreateMessages
     */
    function testMakeSurelineBreaksDontClipLongNotes($merchant_id)
    {
        setContext("com.splickit.worldhq");
        $skin = getSkinIdForContext();

        logTestUserIn($this->user['user_id']);

        $note = "This is a longlonglonglong really long notes entry";
        $order_data = OrderAdapter::staticGetSimpleOrderArrayByMerchantId($merchant_id,'pickup',$note,1);
        $order_resource = placeOrderFromOrderData($order_data, getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNotNull($order_resource);
        $this->assertNull($order_resource->error);
        $this->assertTrue($order_resource->order_id > 1000,"Bad order id of: ".$order_resource->order_id);
        $order_id = $order_resource->order_id;
        $mmha = new MerchantMessageHistoryAdapter($m);
        $messages = $mmha->getAllOrderMessages($order_id);
        $messages_array = createHashmapFromArrayOfResourcesByFieldName($messages,'message_format');
        $gprs_message_resource = $messages_array['GUA'];
        $gprs_controller = new GprsController($m,$u,$r,5);
        $message_resource = $gprs_controller->prepMessageForSending($gprs_message_resource);

        $this->assertContains("Instructions: This is a longlo::nglonglong really long notes e::ntry",$message_resource->message_text);
    }

    /**
     * @depends testCreateMessages
     */
    function testMakeSureNotesIsCleanWithGoodCarriageReturns($merchant_id)
    {
        setContext("com.splickit.worldhq");
        logTestUserIn($this->user['user_id']);

        $note = "first line
This is a return
another return.";
        $item_note = "item note
This is an item return
another item return.";

        $order_data = OrderAdapter::staticGetSimpleOrderArrayByMerchantId($merchant_id,'pickup',$note,1);
        $order_data['items'][0]['note'] = $item_note;
        $order_resource = placeOrderFromOrderData($order_data, getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNotNull($order_resource);
        $this->assertNull($order_resource->error);
        $this->assertTrue($order_resource->order_id > 1000,"Bad order id of: ".$order_resource->order_id);
        $order_id = $order_resource->order_id;
        $mmha = new MerchantMessageHistoryAdapter($m);
        $messages = $mmha->getAllOrderMessages($order_id);
        $messages_array = createHashmapFromArrayOfResourcesByFieldName($messages,'message_format');
        $gprs_message_resource = $messages_array['GUA'];
        $gprs_controller = new GprsController($m,$u,$r,5);
        $message_resource = $gprs_controller->prepMessageForSending($gprs_message_resource);

        $this->assertContains("Instructions: first line::This is a return::another return.",$message_resource->message_text);

        $this->assertContains("Note: item note::This is an item return::another item return",$message_resource->message_text);


    }

    function testResendOfOrderToGPRSprinter()
    {
		$merchant_resource = createNewTestMerchant($this->menu_id);
    	$merchant_id = $merchant_resource->merchant_id;    	
    	$map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_id,"message_format"=>'GUA',"delivery_addr"=>"1234567890","message_type"=>"X","info"=>"firmware=7.0"));

        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id,'pickup','dum note',1);
        $order_resource = placeOrderFromOrderData($order_data);


        $this->assertNotNull($order_resource);
		$this->assertNull($order_resource->error);
		$this->assertTrue($order_resource->order_id > 1000,"Bad order id of: ".$order_resource->order_id);  
		$order_id = $order_resource->order_id;
		
		// now get the order
    	$request = new Request();
    	$request->url = "/m/g/$merchant_id/gprs/f2.txt";
    	$gprs_controller = new GprsController($mt, $u, $request,5);
    	
    	$message_resource = $gprs_controller->pullNextMessageResourceByMerchant($merchant_id);
    	$message_text = $message_resource->message_text;
    	myerror_log("pulled message is: ".$message_resource->message_text);
    	myerror_log("trimmed: ".substr($message_text, 0,16));
    	$this->assertEquals("#".$merchant_id."*1*".$order_id."**", substr($message_text, 0,18));
    	$gprs_controller->markMessageDelivered();

    	// now call the resend
    	$order_controller = new OrderController($mt, $u, $r,5);
    	$order_controller->resendOrder($order_id);
    	
    	$message_resource2 = $gprs_controller->pullNextMessageResourceByMerchant($merchant_id);
    	myerror_log("We have pulled the next message");
    	$this->assertNotNull($message_resource2);
    	$message_text = $message_resource2->message_text;
    	myerror_log("pulled message is: ".$message_text);
 		$this->assertEquals("#".$merchant_id."*1*".$order_id."-", substr($message_text, 0,17));
 		$append = substr($message_text, 17,2);
 		$append2 = substr($message_text, 19,2);
 		myerror_log("the append is: ".$append);
 		myerror_log("the append2 is: ".$append2);
 		$this->assertTrue(1 == preg_match("/[a-z]{2}/", $append,$matches1),"should have found a two letter string but found: ".$append);
 		$this->assertFalse(1 == preg_match("/[a-z]{2}/", $append2,$matches2),"should not have found two letter string: ".$append2);
 		$this->assertEquals("**", $append2);
    }
    
    function testNewPrinterReceiptItems()
    {   
    	$user_resource = createNewUserWithCC();
    	$user = logTestUserResourceIn($user_resource); 	
    	$json = '{"user_id":"'.$user['user_id'].'","name":"","instructions":"Ring the &^%$#buzzer twice","address1":"1045 pine street #3","address2":"Buzzer #234","city":"Boulder","state":"CO","zip":"80302","phone_no":"(123)45#6.7890","lat":40.019785,"lng":-105.282509}';
    	$request = new Request();
    	$request->body = $json;
    	$request->mimetype = "Application/json";
    	$request->_parseRequestBody(); 
    	$user_controller = new UserController($mt, $user, $request,5);
    	$response = $user_controller->setDeliveryAddr();
    	
    	$merchant_resource = createNewTestMerchant($this->menu_id);
    	$merchant_resource->delivery = 'Y';
    	$merchant_resource->immediate_message_delivery = 'Y';
    	$merchant_resource->save();
    	$merchant_id = $merchant_resource->merchant_id;
    	$mdia = new MerchantDeliveryInfoAdapter($mimetypes);
    	$mdia_resource = $mdia->getExactResourceFromData(array("merchant_id"=>$merchant_id));	
    	$mdia_resource->minimum_order = 0.01;
    	$mdia_resource->delivery_cost = 1.00;
    	$mdia_resource->delivery_increment = 15;
    	$mdia_resource->max_days_out = 3;
    	$mdia_resource->save();

        $map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_id,"message_format"=>'GUA',"delivery_addr"=>"gprs","message_type"=>"X","info"=>"firmware=7.0"));
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_id,"message_format"=>'T',"delivery_addr"=>"7777777777","message_type"=>"A","info"=>"firmware=7.0","message_text"=>'***'));


        $order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id,'pickup','skip hours',20);
    	$order_data['lead_time'] = 90;
    	$order_data['delivery'] = 'yes';
    	$order_data['user_addr_id'] = $response->user_addr_id;
    	$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
    	$order_id = $order_resource->order_id;
    	$this->assertNull($order_resource->error,"should not have gotten an error: ".$order_resource->error);
    	$this->assertTrue($order_resource->order_id > 1000);
    	$complete_order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$this->assertNotNull($complete_order['receipt_items_for_merchant_printout']);
    	$better_receipt_items = createHashOfRecieptItemsByTitle($complete_order['receipt_items_for_merchant_printout']);
    	$expected_total = number_format($complete_order['tip_amt'] + $complete_order['grand_total_to_merchant'],2);
    	$this->assertNotNull($better_receipt_items['Tax']);
    	$this->assertEquals("$".$complete_order['grand_total_to_merchant'], $better_receipt_items['Total']);
    	$this->assertEquals("$".$complete_order['tip_amt'], $better_receipt_items['Tip']);
    	$this->assertEquals("$".$expected_total, $better_receipt_items['Grand Total']);

        // check that # is striped out
        $mmha = new MerchantMessageHistoryAdapter($m);
        $messages = $mmha->getAllOrderMessages($order_id);
        $messages_array = createHashmapFromArrayOfResourcesByFieldName($messages,'message_format');
        $gprs_message_resource = $messages_array['GUA'];
        $gprs_controller = new GprsController($m,$u,$r,5);
        $message_resource = $gprs_controller->prepMessageForSending($gprs_message_resource);
        $this->assertContains("Buzzer 234",$message_resource->message_text);
        $this->assertContains("1045 pine street 3",$message_resource->message_text);
        $this->assertContains("1234567890",$message_resource->message_text);
        $this->assertContains("Ring the buzzer twice",$message_resource->message_text);

    }

	function testBrokenUPOrderForOldFirmware()
    {
    	
    	$json = '{"user_id":"'.$this->user['user_id'].'","name":"","address1":"1045 pine street","address2":"","city":"Boulder","state":"CO","zip":"80302","phone_no":"1234566890","lat":40.019785,"lng":-105.282509}';
    	$request = new Request();
    	$request->body = $json;
    	$request->mimetype = "Application/json";
    	$request->_parseRequestBody(); 
    	$user_controller = new UserController($mt, $this->user, $request,5);
    	$response = $user_controller->setDeliveryAddr();
    	
    	$merchant_resource = createNewTestMerchant($this->menu_id);
    	$merchant_resource->delivery = 'Y';
    	$merchant_resource->immediate_message_delivery = 'Y';
    	$merchant_resource->save();
    	$merchant_id = $merchant_resource->merchant_id;
    	$mdia = new MerchantDeliveryInfoAdapter($mimetypes);
    	$mdia_resource = $mdia->getExactResourceFromData(array("merchant_id"=>$merchant_id));	
    	$mdia_resource->minimum_order = 0.01;
    	$mdia_resource->delivery_cost = 1.00;
    	$mdia_resource->delivery_increment = 15;
    	$mdia_resource->max_days_out = 3;
    	$mdia_resource->save();
    	
    	$merchant_id = $merchant_resource->merchant_id;    	
    	$map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_id,"message_format"=>'GUA',"delivery_addr"=>"1234567890","message_type"=>"X"));

		$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id,'pickup','skip hours',20);
    	$order_data['lead_time'] = 90;
    	$order_data['delivery'] = 'yes';
    	$order_data['user_addr_id'] = $response->user_addr_id;
    	$order_resource = placeOrderFromOrderData($order_data,$the_time);
    	$order_id = $order_resource->order_id;
    	$complete_order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$this->assertNull($order_resource->error,"should not have gotten an error: ".$order_resource->error);
    	$this->assertTrue($order_resource->order_id > 1000);
    	$request = new Request();
    	$request->url = "/m/g/$merchant_id/gprs/f.txt";
    	$gprs_controller = new GprsController($mt, $u, $request,5);
    	
    	$message_resource = $gprs_controller->pullNextMessageResourceByMerchant($merchant_id);
    	$message_text = $message_resource->message_text;
    	myerror_log("pulled message is: ".$message_resource->message_text);
    	
    	$this->assertEquals('(page 1 of 13)::xx#', substr($message_text, -19));

    	$message_data['merchant_id'] = $merchant_id;
    	$message_data['order_id'] = $order_resource->order_id;
    	$message_data['message_format'] = 'GUA';
    	$message_data['map_id'] = array(">"=>$message_resource->map_id);
    	$message_options[TONIC_FIND_BY_METADATA] = $message_data;
    	$message_options[TONIC_SORT_BY] = " map_id ASC ";
    	$mmh_adapter = new MerchantMessageHistoryAdapter($mimetypes);
    	$message_resources = Resource::findAll($mmh_adapter,'',$message_options);
    	
    	$this->assertCount(12,$message_resources);
    	$message2 = $message_resources[0]; 
    	$message2_text = $message2->message_text;
    	$expected_message2_start = "#$merchant_id*$order_id*::::------- SUMMARY --------::::Items:  20::::- Test Item 1 $1.50::";
        $this->assertContains($expected_message2_start,$message2_text);
    	
    	$message_last = array_pop($message_resources);
    	$tip = $order_resource->tip_amt;
    	
    	$expected_message_last = "::::(page 13 of 13)::xx#";
    	$actual_last = $message_last->message_text;
    	$this->assertEquals($expected_message_last, substr($actual_last,-24));
    }
         
    function testResendOfTextToNonLateMessages()
    {
    	// get rid of all open messages
    	$sql = "UPDATE Merchant_Message_History SET locked = 'C' WHERE locked='P'";

    	$mmha = new MerchantMessageHistoryAdapter($mimetypes);
    	$mmha->_query($sql);
    	
    	//set up data. 2 messages not late the other late
    	$map_id1 = $mmha->createMessage(1001, 1001, "GUC",'123456789', time()-60, 'X', "firmware=8.0", $message_text,'P');
    	$map_id2 = $mmha->createMessage(1002, 1002, "GUC",'123456789', time()-120, 'X', null, $message_text,'P');
    	$map_id3 = $mmha->createMessage(1003, 1003, "GUC",'123456789', time()-600, 'X', null, $message_text,'P');
    	
    	$failover_time = 240;
		$failover_date_string = date('Y-m-d H:i:s',(time()-$failover_time));    	
    	$result = $mmha->resendActivationTextsToNonLateGPRSMessages($failover_date_string);
    	$this->assertNotNull($result);
    	$this->assertEquals('***   queued', $result[$map_id1]);
    	$this->assertEquals('***5   queued', $result[$map_id2]);
    	$this->assertNull($result[$map_id3]);
    }

    function testGPRSTunnelMerchantShutdownTrue() {
    	setProperty("gprs_tunnel_merchant_shutdown", "true");
    	
		$merchant_resource = createNewTestMerchant($this->menu_id);
		$merchant_id = $merchant_resource->merchant_id;    	
    	$map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_id,"message_format"=>'GUA',"delivery_addr"=>"1234567890","message_type"=>"X"));

		$request = new Request();
		$request->url = "/merchants/$merchant_id";
		$merchant_controller = new MerchantController($mt, $this->user, $request,5);
		$resource = $merchant_controller->getMerchant();
		$this->assertEquals(999, $resource->error_code);
		$this->assertEquals("We're sorry but due to a T-Mobile outage, we cannot deliver orders to this merchant at this time, please try again soon.",$resource->error);

        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id,'pickup','dum note',1);
        $order_resource = placeOrderFromOrderData($order_data);

        $this->assertEquals(500, $order_resource->http_code);
		$this->assertEquals("We're sorry but due to a T-Mobile outage, we cannot deliver orders to this merchant at this time, please try again soon.",$order_resource->error);

		// now try with a firmware 8.0 merchant
		$version_70_merchant_id = $this->merchant_id;
		$request = new Request();
		$request->url = "/merchants/$version_70_merchant_id";
		$merchant_controller = new MerchantController($mt, $this->user, $request,5);
		$resource = $merchant_controller->getMerchant();
		$this->assertNull($resource->error_code);
		$this->assertNotNull($resource->menu,"Should have found a menu since this is a version 8.0 mercahnt");

        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($version_70_merchant_id,'pickup','dum note',1);
        $order_resource = placeOrderFromOrderData($order_data);

        $this->assertNull($order_resource->error_code,"should have been a good order since this is a version 8 mercahnt");
		$this->assertTrue($order_resource->order_id > 1000);
		
    }
        
    function testGPRSTotalMerchantShutdownTrue()
    {
    	setProperty("gprs_tunnel_merchant_shutdown", "false");
    	setProperty("gprs_total_merchant_shutdown", "true");
    	
		$merchant_resource = createNewTestMerchant($this->menu_id);
		$merchant_id = $merchant_resource->merchant_id;    	
    	$map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_id,"message_format"=>'GUA',"delivery_addr"=>"1234567890","message_type"=>"X"));

		$request = new Request();
		$request->url = "/merchants/$merchant_id";
		$merchant_controller = new MerchantController($mt, $this->user, $request,5);
		$resource = $merchant_controller->getMerchant();
		$this->assertEquals(999, $resource->error_code);
		$this->assertEquals("We're sorry but due to a T-Mobile outage, we cannot deliver orders to this merchant at this time, please try again soon.",$resource->error);

        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id,'pickup','dum note',1);
        $order_resource = placeOrderFromOrderData($order_data);

        $this->assertEquals(500, $order_resource->http_code);
		$this->assertEquals("We're sorry but due to a T-Mobile outage, we cannot deliver orders to this merchant at this time, please try again soon.",$order_resource->error);

		// now try with a firmware 8.0 merchant
		$version_70_merchant_id = $this->merchant_id;
		$request = new Request();
		$request->url = "/merchants/$version_70_merchant_id";
		$merchant_controller = new MerchantController($mt, $this->user, $request,5);
		$resource = $merchant_controller->getMerchant();
		$this->assertEquals(999, $resource->error_code);
		$this->assertEquals("We're sorry but due to a T-Mobile outage, we cannot deliver orders to this merchant at this time, please try again soon.",$resource->error);
		
		$order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($version_70_merchant_id,'pickup','skip hours');
        $checkout_data_resource = getCheckoutDataWithThrottling($order_data, $version_70_merchant_id, $time_stamp);
		$this->assertEquals(500, $checkout_data_resource->http_code);
		$this->assertEquals("We're sorry but due to a T-Mobile outage, we cannot deliver orders to this merchant at this time, please try again soon.",$checkout_data_resource->error);

		$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
		$this->assertEquals(500, $order_resource->http_code);
		$this->assertEquals("We're sorry but due to a T-Mobile outage, we cannot deliver orders to this merchant at this time, please try again soon.",$order_resource->error);
		
    }
    
    function testGetNextMessage()
    {
    	$merchant_resource = createNewTestMerchant($this->menu_id);
    	$merchant_id = $merchant_resource->merchant_id;
    	$map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_id,"message_format"=>'GUA',"delivery_addr"=>"1234567890","message_type"=>"X","info"=>"firmware=7.0"));

        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id,'pickup','dum note',1);
        $order_resource = placeOrderFromOrderData($order_data);


        $this->assertNotNull($order_resource);
		$this->assertNull($order_resource->error);
		$this->assertTrue($order_resource->order_id > 1000,"Bad order id of: ".$order_resource->order_id);  
		
		$request = new Request();
    	$request->url = "/m/g/$merchant_id/gprs/f2.txt";
    	$gprs_controller = new GprsController($mt, $u, $request,5);
    	
    	$message_resource = $gprs_controller->pullNextMessageResourceByMerchant($merchant_id);
    	myerror_log("pulled message is: ".$message_resource->message_text);
    	$this->assertEquals('#'.$merchant_id.'*1*'.$order_resource->order_id.'**::::Order For: First Last::Order Id:  '.$order_resource->order_id.'::', substr($message_resource->message_text, 0,64));
    	
    	$gprs_controller->message_resource->message_text = 'nullit';
    	$gprs_controller->message_resource->locked = 'P';
    	$gprs_controller->message_resource->save();
    	
    	$request = new Request();
    	$request->url = "/m/g/$merchant_id/gprs/f.txt";
    	$gprs_controller = new GprsController($mt, $u, $request,5);
    	
    	$message_resource = $gprs_controller->pullNextMessageResourceByMerchant($merchant_id);
    	myerror_log("pulled message is: ".$message_resource->message_text);
    	
    	$this->assertEquals('#'.$merchant_id.'*'.$order_resource->order_id.'*::::Order For: First Last::Order Id:  '.$order_resource->order_id.'::', substr($message_resource->message_text, 0,61));
    }
    
    function testCheckForGPRSMessageInStateReadyToBeSent()
    {
    	$merchant_resource = createNewTestMerchant();
    	// first create GPRS message not ready to be sent
    	$mmha = new MerchantMessageHistoryAdapter($mimetypes);
    	$gprs_message_resource = $mmha->createMessageReturnResource($merchant_resource->merchant_id, 1000, 'GUC', '1234567890', time()+5, 'X', 'firmware=8.0', $message_text,'P');
    	$gprs_controller = new GprsController($mt, $u, $r,5);
    	$this->assertFalse($gprs_controller->isThereAGPRSMessageReadyForThisMerchant($merchant_resource->merchant_id));
    	
    	// now changet the send time
    	$gprs_message_resource->next_message_dt_tm = time()-5;
    	$gprs_message_resource->save();
    	$this->assertTrue($gprs_controller->isThereAGPRSMessageReadyForThisMerchant($merchant_resource->merchant_id));
    	
    }
    
    function testCreateTEXTMessageFirmware8()
    {
    	$merchant_id = 1000;
    	$gprs_controller = new GprsController($mt, $u, $r,5);
    	$message_resource = $gprs_controller->createTEXTMessageFirmware8($merchant_id, '1234567890', $test_order_id, time()+5);
    	$this->assertNotNull($message_resource);
    	$this->assertEquals('1234567890',$message_resource->message_delivery_addr);
    	$this->assertEquals('T', $message_resource->message_format,'Should have created a T type message');
    	$this->assertEquals(1000, $message_resource->merchant_id);
    }
    
   	function testGprsCallBackWithTextMap()
    {
    	//create sent message
    	$merchant_resource = createNewTestMerchant();
    	$merchant_id = $merchant_resource->merchant_id;
    	$map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_id,"message_format"=>'T',"delivery_addr"=>"1234567890","message_type"=>"A","info"=>"firmware=7.0"));
    	
    	$mmha = new MerchantMessageHistoryAdapter($mimetypes);
    	$picked_up_message_resource = $mmha->createMessageReturnResource($merchant_id, 1000, 'GUC', '1234567890', time()-10, 'X', 'firmware=8.0', $message_text,'S',time()-1,1);
    	$picked_up_message_resource->viewed = 'N';
    	$picked_up_message_resource->save();
    	
    	//now create pending
    	$ready_to_be_picked_up_message_resource = $mmha->createMessage($merchant_id, 1001, 'GUC', '1234567890', time()-1, 'X', 'firmware=8.0', $message_text);
    	
    	// now do callback
    	$request = new Request();
    	$data['o'] = 1000;
    	$request->data = $data; 
    	$gprs_controller = new GprsController($mt, $u, $request,5);
    	
    	$gprs_controller->callBack($merchant_resource->numeric_id);
    	$refreshed = $picked_up_message_resource->refreshResource($picked_up_message_resource->map_id);
    	$this->assertEquals('V', $refreshed->viewed);
    	
    	// now check to see if a text was scheduled 5 seconds in the future for this merchant;
    	$record = $mmha->getRecord(array('merchant_id'=>$merchant_id,'message_format'=>'T','locked'=>'N','next_message_dt_tm'=>array('>'=>(time()+3))));
    	$this->assertTrue($record['map_id'] > 1000,'should have created a message resource 5 seconds in teh future');
    	$this->assertEquals('1234567890',$record['message_delivery_addr']);

    }
    
   	function testGprsCallBackWithNOTextMap()
    {
    	//create sent message
    	$merchant_resource = createNewTestMerchant();
    	$merchant_id = $merchant_resource->merchant_id;
    	$mmha = new MerchantMessageHistoryAdapter($mimetypes);
    	$picked_up_message_resource = $mmha->createMessageReturnResource($merchant_id, 1000, 'GUC', '1234567890', time()-10, 'X', 'firmware=8.0', $message_text,'S',time()-1,1);
    	$picked_up_message_resource->viewed = 'N';
    	$picked_up_message_resource->save();
    	
    	//now create pending
    	$ready_to_be_picked_up_message_resource = $mmha->createMessage($merchant_id, 1001, 'GUC', '1234567890', time()-1, 'X', 'firmware=8.0', $message_text);
    	
    	// now do callback
    	$request = new Request();
    	$data['o'] = 1000;
    	$request->data = $data; 
    	$gprs_controller = new GprsController($mt, $u, $request,5);
    	
    	$gprs_controller->callBack($merchant_resource->numeric_id);
    	$refreshed = $picked_up_message_resource->refreshResource($picked_up_message_resource->map_id);
    	$this->assertEquals('V', $refreshed->viewed);
    	
    	// now check to see that no text message was created
    	$record = $mmha->getRecord(array('merchant_id'=>$merchant_id,'message_format'=>'T','locked'=>'N','next_message_dt_tm'=>array('>'=>(time()+3))));
    	$this->assertNull($record);
    }

    static function setUpBeforeClass()
    {
    	ini_set('max_execution_time',300);
    	      SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();$mysqli->begin_transaction(); ;
    	$_SERVER['request_time1'] = microtime(true);    	
		$_SERVER['log_level'] = 5; 
		$menu_id = createTestMenuWithNnumberOfItems(1);
    	
    	$modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 10);
    	$modifier_group_id = $modifier_group_resource->modifier_group_id;
    	$item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 10);

    	$ids['menu_id'] = $menu_id;
    	$merchant_resource = createNewTestMerchant($menu_id);
    	$ids['merchant_id'] = $merchant_resource->merchant_id;
    	
    	$map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_resource->merchant_id,"message_format"=>'GUA',"delivery_addr"=>"1234567890","message_type"=>"X","info"=>"firmware=7.0"));

		$user_resource = createNewUser(array('flags'=>'1C20000001'));
    	$ids['user_id'] = $user_resource->user_id;
    	    	
    	$_SERVER['unit_test_ids'] = $ids;

    }
    
	static function tearDownAfterClass()
    {
    	SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
    }

    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    GprsMessageTest::main();
}

?>