<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class JerseyMikesControllerTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
		setContext('com.splickit.jerseymikes');
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }
    
    function testFailOrderFromExpiredCardInVault()
    {
    	$_SERVER['HTTP_NO_CC_CALL'] = 'false';
    	setContext('com.splickit.jerseymikes');
		$merchant_id = $this->ids['merchant_id'];
		
    	$user_resource = createNewUser(array("flags"=>"1C20000001"));
    	$user_resource->uuid = '12345-12345-12345-zzzzz';
    	$user_resource->save();
    	$user_id = $user_resource->user_id;
    	$user = logTestUserIn($user_id);
    	
    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours');
    	$order_data['tip'] = 0.00;
    	
    	$order_resource = placeOrderFromOrderData($order_data, getTodayTwelveNoonTimeStampDenver());
    	$this->assertNotNull($order_resource->error,"Should have gotten an error since card is expired in the vault");
    	$this->assertEquals("Sorry, your credit card has expired, please upload a new one.",$order_resource->error);
    }
    
    function testBackwardsCompatibilityOfJMWithNewPayment()
    {
		setContext('com.splickit.jerseymikes');
		$merchant_id = $this->ids['merchant_id'];
		
    	$user_resource = createNewUser(array("flags"=>"1C20000001"));
    	$user_id = $user_resource->user_id;
    	$user = logTestUserIn($user_id);
    	
    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours');
    	$order_data['tip'] = 0.00;
    	
    	$order_resource = placeOrderFromOrderData($order_data, getTodayTwelveNoonTimeStampDenver());
    	$this->assertNull($order_resource->error);
    	$order_id = $order_resource->order_id;

    	$this->assertEquals('JerseyMikesPaymentService', $order_resource->payment_service_used);
    	
		$balance_change_adapter = new BalanceChangeAdapter($mimetypes);
		$balance_change_records = $balance_change_adapter->getRecords(array("order_id"=>$order_id));
		$this->assertCount(0, $balance_change_records);
		$this->assertNotNull($order_resource->points_current);
		$this->assertNotNull($order_resource->points_lifetime);
    }
    
    function testNewPromoProcessor()
    {
    	$jm_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jm['taxable'] = 20;
    	$jm['non_taxable'] = 10;
    	$jm['additional_tax'] = 0.00;
    	$jm['tax'] = 2.00;
    	
    	$complete_order['promo_external_id_for_brand'] = "asdefg";
    	$complete_order['promo_id'] = 201;

    	$return = $jm_controller->processJMPromo($jm, $complete_order);
    	
    	$this->assertEquals(15.00, $jm['taxable']);
    	$this->assertEquals(7.50, $jm['non_taxable']);
    	$this->assertEquals(1.50, $jm['tax']);
    	$this->assertEquals(5.00, $jm['taxable_discount']);
    	$this->assertEquals(2.50, $jm['non_taxable_discount']);    	
    }
    
	function testGetCheckoutDataWithPromo()
    {
    	setContext('com.splickit.jerseymikes');
		$merchant_id = $this->ids['merchant_id'];
		$jm_promo_code = $this->ids['external_promo_code'];
    	
    	$user_resource = createNewUser(array("flags"=>"1C20000001"));
    	$user_id = $user_resource->user_id;
    	$user = logTestUserIn($user_id);
    	
    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours');
    	$order_data['tip'] = 0.00;
    	$order_data['promo_code'] = $this->ids['promo_code'];
    	
    	$promo_controller = new PromoController($mt, $user, $r, 5);
    	$promo_resource = $promo_controller->validatePromo($order_data);
    	$this->assertNull($promo_resource->error,"should not have gotten an error");
    	$this->assertEquals('Promo Code Validated', $promo_resource->user_message_title);
    	$this->assertEquals(201,$promo_resource->promo_id);
    	
    	$place_order_controller = new PlaceOrderController($mt, $user, $r,5);
    	$checkout_data_resource = $place_order_controller->getCheckoutDataFromOrderData($order_data);
    	$this->assertEquals(.75, $checkout_data_resource->total_tax_amt);
    }

    function testPromo()
    {
    	$merchant_id = $this->ids['merchant_id'];
		$jm_promo_code = $this->ids['external_promo_code'];
    	
    	$user_resource = createNewUser(array("flags"=>"1C20000001"));
    	$user_id = $user_resource->user_id;
    	$user = logTestUserIn($user_id);
    	
    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours');
    	$order_data['tip'] = 0.00;
    	$order_data['promo_code'] = $this->ids['promo_code'];
    	
    	$promo_controller = new PromoController($mt, $user, $r, 5);
    	$promo_resource = $promo_controller->validatePromo($order_data);
    	$this->assertNull($promo_resource->error,"should not have gotten an error");
    	$this->assertEquals('Promo Code Validated', $promo_resource->user_message_title);
    	$this->assertEquals(201,$promo_resource->promo_id);
    	$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
    	$this->assertNull($order_resource->error);
    	$this->assertTrue($order_resource->order_id > 1000,"shoudl have created a good order id");
    	
    	$complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id, $mimetypes);
    	$this->assertEquals(.75, $complete_order['total_tax_amt']);
    	$this->assertEquals(-2.50, $complete_order['promo_amt']);
    	$this->assertEquals(10.00, $complete_order['order_amt']);
    	$this->assertEquals(8.25,$complete_order['grand_total']);
    	$this->assertEquals(8.25, $order_resource->grand_total);
    	
    	// to make sure the values are coming back correctly from the place order for display to the customer
    //	$this->assertEquals(.75, $order_resource->total_tax_amt);
    	$this->assertEquals(-2.50, $order_resource->promo_amt);
    	$this->assertEquals(10.00, $order_resource->order_amt);
   // 	$this->assertEquals(8.25,$order_resource->grand_total);
    	
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($complete_order);
    	
    	// WHAT I NEED IS PRE_TAXABLE AND PRE_NON_TAXABLE.  the taxable and non taxable should reflect the dicount
    	$this->assertNotNull($jersey_mikes_formatted_order['discount_number'],"should have found the discount_number since we had a promo");
    	$this->assertEquals($jm_promo_code, $jersey_mikes_formatted_order['discount_number']);
    	$this->assertEquals(7.50, $jersey_mikes_formatted_order['taxable']);
		$this->assertEquals(.75, $jersey_mikes_formatted_order['tax']);    	
    	$this->assertEquals(2.50, $jersey_mikes_formatted_order['taxable_discount']); 
    	$this->assertTrue(isset($jersey_mikes_formatted_order['non_taxable_discount']));
    			    	
    }

    function testPromoWithMikesMarketer()
    {
    	$merchant_id = $this->ids['merchant_id'];
		$jm_promo_code = $this->ids['external_promo_code'];
    	
    	$new_user_resource = createNewUser(array("flags"=>"1C20000001"));
    	$user_id = $new_user_resource->user_id;

    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours');
    	$order_data['tip'] = 0.00;
    	$order_data['user_id'] = $user_id;
    	$json_encoded_data = json_encode($order_data);
    	
		$mm_data['username'] = "$user_id";
    	$mm_data['valid_until_time_stamp'] = time()+60;
    	$auth_data['mikes_marketer_id'] = "12345";
    	$auth_data['some_key'] = "some_value";
    	$auth_data['promo_code'] = $this->ids['promo_code'];
    	$mm_data['auth_data'] = $auth_data;
    	$mm_json = json_encode($mm_data);
    	$mm_json_encrypted = SplickitCrypter::doEncryption($mm_json,'mikesmarketer');
    	
    	$headers['Content-Type'] = "application/json";
    	$headers['Content-Length'] = strlen($json_encoded_data);
    	$headers['Auth_token'] = $mm_json_encrypted;
    	
    	$url = "http://localhost/app2/phone/placeorder?log_level=6";
    	$request = new Request();
    	$request->url = $url;
    	
    	$request->body = $json_encoded_data;
    	$request->_parseRequestBody();
    	$request_all = array_merge($request->data,$headers);
    	
    	$user_resource = LoginAdapter::staticDoAuthorizeWithSpecialUserValidation("mikesmarketer", "Z2xTtj5w1qz", $request_all);
    	$this->assertEquals($new_user_resource->email, $user_resource->email);
    	$this->assertNotNull($user_resource->auth_data);
    	$user = logTestUserResourceIn($user_resource);
    	$this->assertNotNull($user['auth_data']);

    	$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
    	$this->assertNull($order_resource->error);
    	$this->assertTrue($order_resource->order_id > 1000,"shoudl have created a good order id");
    	
    	$complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id, $mimetypes);
    	$this->assertEquals(.75, $complete_order['total_tax_amt']);
    	$this->assertEquals(-2.50, $complete_order['promo_amt']);
    	$this->assertEquals(10.00, $complete_order['order_amt']);
    	$this->assertEquals(8.25,$complete_order['grand_total']);
    	$this->assertEquals(8.25, $order_resource->grand_total);
    	
    	// to make sure the values are coming back correctly from the place order for display to the customer
    //	$this->assertEquals(.75, $order_resource->total_tax_amt);
    	$this->assertEquals(-2.50, $order_resource->promo_amt);
    	$this->assertEquals(10.00, $order_resource->order_amt);
   // 	$this->assertEquals(8.25,$order_resource->grand_total);
    	
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($complete_order);
    	
    	// WHAT I NEED IS PRE_TAXABLE AND PRE_NON_TAXABLE.  the taxable and non taxable should reflect the dicount
    	$this->assertNotNull($jersey_mikes_formatted_order['discount_number'],"should have found the discount_number since we had a promo");
    	$this->assertEquals($jm_promo_code, $jersey_mikes_formatted_order['discount_number']);
    	$this->assertEquals(7.50, $jersey_mikes_formatted_order['taxable']);
		$this->assertEquals(.75, $jersey_mikes_formatted_order['tax']);    	
    	$this->assertEquals(2.50, $jersey_mikes_formatted_order['taxable_discount']); 
    	$this->assertTrue(isset($jersey_mikes_formatted_order['non_taxable_discount']));    	
    }
    
    function testGetCheckoutDataForMikesMarketer()
    {
    	$merchant_id = $this->ids['merchant_id'];
    	
    	$new_user_resource = createNewUser(array("flags"=>"1C20000001"));
    	$user_id = $new_user_resource->user_id;

    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'my note');
    	$order_data['tip'] = 0.00;
    	$order_data['user_id'] = $user_id;
    	$json_encoded_data = json_encode($order_data);
    	
		$mm_data['username'] = "$user_id";
    	$mm_data['valid_until_time_stamp'] = time()+60;
    	$auth_data['mikes_marketer_id'] = "12345";
    	$auth_data['some_key'] = "some_value";
    	$mm_data['auth_data'] = $auth_data;
    	$mm_json = json_encode($mm_data);
    	$mm_json_encrypted = SplickitCrypter::doEncryption($mm_json,'mikesmarketer');
    	
    	$headers['Content-Type'] = "application/json";
    	$headers['Content-Length'] = strlen($json_encoded_data);
    	$headers['Auth_token'] = $mm_json_encrypted;
    	
    	$url = "http://localhost/app2/phone/placeorder?log_level=6";
    	$request = new Request();
    	$request->url = $url;
    	
    	$request->body = $json_encoded_data;
    	$request->_parseRequestBody();
    	$request_all = array_merge($request->data,$headers);
    	
    	$user_resource = LoginAdapter::staticDoAuthorizeWithSpecialUserValidation("mikesmarketer", "Z2xTtj5w1qz", $request_all);
    	$this->assertEquals($new_user_resource->email, $user_resource->email);
    	$this->assertNotNull($user_resource->auth_data);
    	$user = logTestUserResourceIn($user_resource);
    	$this->assertNotNull($user['auth_data']);

    	$time_stamp_for_noon_on_feb14_denver = getTimeStampForDateTimeAndTimeZone(12, 0, 0, 2, 14, 2014, 'America/Denver');
    	$expected_final_time = getTimeStampForDateTimeAndTimeZone(20, 0, 0, 3, 15, 2014, 'America/Denver');
    	$checkout_resource = getCheckoutDataWithThrottling($order_data, $merchant_id, $time_stamp_for_noon_on_feb14_denver);
    	$lead_times = $checkout_resource->lead_times_array;
    	$last_time = array_pop($lead_times);
    	$this->assertEquals(getFullyFormattedDateAndTimeStringForTimeStamp($expected_final_time), getFullyFormattedDateAndTimeStringForTimeStamp($last_time),"last available time should have been 4 weeks in the future");
    	
    }
    
    static function setUpBeforeClass()
    {
    	$_SERVER['request_time1'] = microtime(true);
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	ini_set('max_execution_time',300);
    	mysqli_query("BEGIN");
    	
    	$skin_resource = createWorldHqSkin();
    	$ids['skin_id'] = $skin_resource->skin_id;
    	
		//map it to a menu
    	$menu_id = createTestMenuWithNnumberOfItems(1,102433);

    	$ids['menu_id'] = $menu_id;
    	
    	$merchant_resource = createNewTestMerchant($menu_id);
    	$merchant_resource->merchant_external_id = 1023;
    	$merchant_resource->brand_id = 326;
    	$merchant_resource->order_del_type = 'K';
    	$merchant_resource->immediate_message_delivery = 'Y';
    	$merchant_resource->save();
    	$merchant_id = $merchant_resource->merchant_id;
    	attachMerchantToSkin($merchant_id, $ids['skin_id']);
    	$ids['merchant_id'] = $merchant_id;
    	
    	$modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 2);
    	$modifier_group_id = $modifier_group_resource->modifier_group_id;
    	$item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
    	// set PLU
    	$item = $item_records[0];
    	$isa = new ItemSizeAdapter($mimetypes);
    	$is_resource = $isa->getExactResourceFromData(array("item_id"=>$item['item_id']));
    	$is_resource->external_id = 'PLU1206';
    	$is_resource->merchant_id = $merchant_id;
    	$is_resource->price = 10.00;
    	$is_resource->save();
    	
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 2);
    	
    	$mia = new ModifierItemAdapter($mimetypes);
    	$msa = new ModifierSizeMapAdapter($mimetypes);
    	$modifier_records = $mia->getRecords(array("modifier_group_id"=>$modifier_group_id), $options);
    	$modifier_size_resource1 = $msa->getExactResourceFromData(array("modifier_item_id"=>$modifier_records[0]['modifier_item_id'],"size_id"=>"0"));
    	$modifier_size_resource1->external_id = 'M63';
    	$modifier_size_resource1->merchant_id = $merchant_id;
    	$modifier_size_resource1->save();
    	$modifier_size_resource2 = $msa->getExactResourceFromData(array("modifier_item_id"=>$modifier_records[1]['modifier_item_id'],"size_id"=>"0"));
    	$modifier_size_resource2->external_id = 'M120';
    	$modifier_size_resource2->merchant_id = $merchant_id;
    	$modifier_size_resource2->save();
    	 
    	// set to version 3.0
    	$menu_resource = Resource::find(new MenuAdapter($mimetypes), "$menu_id", $options);
    	$menu_resource->version = 3.0;
    	$menu_resource->save();

    	//create the type 1 promo
    	$promo_adapter = new PromoAdapter($mimetypes); 
    	$ids['promo_id'] = 201;
	   	$sql = "INSERT INTO `Promo` VALUES(201, 'The Type1 Promo', 'Get 25% off', 1, 'Y', 'N', 0, 2, 'N', 'N', '2010-01-01', '2020-01-01', 1, 0, 0.00, 0, 0.00, 'Y', 'N', NOW(), NOW(), 'N')";
    	$promo_adapter->_query($sql);
    	$sql = "INSERT INTO `Promo_Merchant_Map` VALUES(null, 201, $merchant_id, '2013-10-05', '2020-01-01', NULL, now())";
    	$pmm_resource = Resource::createByData(new PromoMerchantMapAdapter($mimetypes), array("merchant_id"=>$merchant_id,"promo_id"=>201));
    	$ids['promo_merchant_map_id'] = $pmm_resource->map_id;
    	$sql = "INSERT INTO `Promo_Message_Map` VALUES(null, 201, 'Congratulations! Youre getting a 25% off your order!', NULL, NULL, NULL, NULL, now())";
    	$promo_adapter->_query($sql);
    	$sql = "INSERT INTO `Promo_Type1_Amt_Map` VALUES(null, 201, 1.00, 0.00, 25,50.00, NOW())";
    	$promo_adapter->_query($sql);
    	
    	$promo_code = 'jmtype1promo';
    	$ids['promo_code'] = $promo_code;
    	$pkwm_adapter = new PromoKeyWordMapAdapter($mimetypes);
    	Resource::createByData($pkwm_adapter, array("promo_id"=>201,"promo_key_word"=>"$promo_code","brand_id"=>326));
    	
		// create promo brand extrenal map
		$external_promo_code = 'JM_12345';
		$ids['external_promo_code'] = $external_promo_code;
		$pbeima = new PromoBrandExternalIdMapAdapter($mimetypes);
		$resource = Resource::createByData($pbeima, array("promo_id"=>201,"brand_id"=>326,"external_id"=>$external_promo_code));
    	
    	$user_resource = createNewUser(array("flags"=>"1C20000001"));
    	$ids['user_id'] = $user_resource->user_id;
    	
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	mysqli_query("ROLLBACK");
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    JerseyMikesControllerTest::main();
}

?>