<?php
$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class JerseyMikesImporterTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;
	
	function setUp()
	{
        setContext('com.splickit.jerseymikes');
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
		unset($this->stamp);
    }

    function testMakeSureZeroPriceLogicDoesntSetComboSmallDrinkToInnactiveBecauseItsPriceIsZeroInOurSystem()
    {
        $merchant_id = $this->ids['regular_store'];
        $menu = $this->loadMenu($merchant_id);
        $msma = new ModifierSizeMapAdapter($m);
        $record = $msma->getRecord(array("modifier_item_id"=>2270133, "merchant_id"=>$merchant_id));
        $this->assertEquals('Y', $record['active'], "22oz fountain drink shoudl be active");
    }

    function testZeroPriceSettingRecordToInnactive()
    {
        $raw_result = file_get_contents("unit_tests/resources/jm_zero_price_thing.txt");
        $jm_importer = new JerseyMikesImporter();
        $result = $jm_importer->processDataPull($raw_result,18017);
        $merchant_id = $jm_importer->imported_merchant_ids[0];
        $price_record = ItemSizeAdapter::staticGetRecord(array("external_id"=>"PLU1102","merchant_id"=>$merchant_id),'ItemSizeAdapter');
        $initial_price = $price_record['price'];
        $this->assertEquals(1.23,$price_record['price']);
        $this->assertEquals('Y',$price_record['active']);
        $raw_result = str_replace("1.2345",".0000",$raw_result);
        $result = $jm_importer->processDataPull($raw_result,18017);

        $price_record = ItemSizeAdapter::staticGetRecord(array("external_id"=>"PLU1102","merchant_id"=>$merchant_id),'ItemSizeAdapter');
        $this->assertEquals(0,$price_record['price']);
        $this->assertNotEquals($initial_price,$price_record['price']);
        $this->assertEquals('N',$price_record['active'],"should have set the active to 'N' since price was set to .00");

    }
  
    function testGetLinkFromDispatcher()
    {
    	$url = "http://localhost/app2/phone/jerseymikesstore/33001/link";
    	$curl = curl_init($url);
    	curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
		$result = curl_exec($curl);
		$info = curl_getinfo($curl);
		curl_close($curl);
		$this->assertEquals(200, $info['http_code']);
		$result_array = json_decode($result,true);
		$this->assertNull($result_array['ERROR']);
		$this->assertNotNull($result_array['link']);
		$merchant_id = $this->ids['delivery_merchant_id'];
		$expected_link = "https://ordering.jerseymikes.com/NJ/Lincoln/210-N-14th-Street/$merchant_id";
		$this->assertEquals($expected_link, $result_array['link']);
    }
    
    function testImportFromImporter()
    {
    	$request = new Request();
    	$request->url = "http://localhost/app2/phone/importjerseymikesprices/1025/";
    	$response = JerseyMikesImporter::stageImportFromRequest($request);
    	$this->assertEquals(200, $response->statusCode);
    	$this->assertContains('Store id 1025 has been staged for import.', $response->body);
    }
    
    function testJerseyMikesOrderDescription()
    {
    	setContext('com.splickit.jerseymikes');
		$merchant_id = $this->ids['regular_store'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		
		$_SERVER['HTTP_NO_CC_CALL'] = 'false';
    	$user = logTestUserResourceIn($user_resource);
    	addDummyCCDataToResource($user_resource);
    	$splickit_vault = new SplickitVault();
    	$this->assertTrue($splickit_vault->save($user_resource));
    	
		$user_id = $user['user_id'];
		$user = logTestUserIn($user_id);
		
		$tip = rand(100, 1000)/100;
    	$pickup_time = getTomorrowTwelveNoonTimeStampDenver();
    	$json_encoded_order_data = '{"merchant_id":"'.$merchant_id.'","items":[{"quantity":1,"note":"","item_id":"278172","size_id":"89585","mods":[{"modifier_item_id":"2258930","mod_quantity":1},{"modifier_item_id":"2258931","mod_quantity":1}]},{"quantity":1,"note":"","item_id":"277816","size_id":"89668","mods":[{"modifier_item_id":"2296492","mod_quantity":1}]}],"total_points_used":0,"note":"","lead_time":15,"promo_code":"","tip":"'.$tip.'","user_id":"'.$user_id.'","loyalty_number":"1234567890","sub_total":"8.24","grand_total":"8.99","actual_pickup_time":"'.$pickup_time.'"}';
    	$order_data = json_decode($json_encoded_order_data,true);
		$order_controller = new PlaceOrderController($mt, $user, $r);
		$order_resource = $order_controller->placeOrderForUnitTestWithOrderData($order_data);
		$this->assertNull($order_resource->error);
		$order_return_array = $order_resource->getDataFieldsReally();
		$order_items = $order_resource->order_items;
		foreach ($order_items as $order_item) {
			if ($order_item['item_name'] == '#7 Turkey and Provolone') {
				$turkey_item = $order_item;
			} else if ($order_item['item_name'] == 'Schweppes') {
				$drink_item = $order_item;
			}
		}
		$this->assertNotNull($turkey_item,"should have found the turkey item");
		$this->assertNotNull($drink_item,"shold have found a drink item");
		$this->assertEquals(" Onions, Lettuce", $turkey_item['item_description']);
		$this->assertEquals("", $drink_item['item_description']);
    }
    
    function testJerseyMikesMultiComboOrder()
    {
    	setContext('com.splickit.jerseymikes');
		$merchant_id = $this->ids['regular_store'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
		$menu = $this->loadMenu($merchant_id);

    	$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	$json_encoded_order_data = "{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2262511\",\"size_id\":\"0\"}],\"item_id\":\"277815\",\"size_id\":\"90054\"},{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2262511\",\"size_id\":\"0\"}],\"item_id\":\"277815\",\"size_id\":\"90055\"},{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":3,\"modifier_item_id\":\"2271915\",\"size_id\":\"0\"}],\"item_id\":\"276963\",\"size_id\":\"89668\"},{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2258945\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258930\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258931\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258932\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258933\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258934\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258935\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258936\",\"size_id\":\"0\"}],\"item_id\":\"276772\",\"size_id\":\"89585\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"11.58\",\"grand_total\":\"12.48\",\"actual_pickup_time\":\"$pickup_time\"}";
    	$order_data = json_decode($json_encoded_order_data,true);
    	$place_order_controller = new PlaceOrderController($mt, $user, $r, 5);
    	$checkout_data_resource = $place_order_controller->getCheckoutDataFromOrderData($order_data);
		$order_resource = placeOrderFromOrderData($order_data, $time_stamp);

    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);
    	
    	//check display stuff
		$cart_items = $order_resource->order_summary['cart_items'];
		foreach ($cart_items as $cart_item)
		{
			if ($cart_item['item_name'] == 'Chips') {
				$chip_display = $cart_item;
			} else if ($cart_item['item_name'] == 'Regular Chip Drink Combo') {
				$regular_display = $cart_item;
			} else if ($cart_item['item_name'] == 'Large Chip Drink Combo') {
				$large_display = $cart_item;
			} else if ($cart_item['item_name'] == '#3 The American Classic') {
				$sub_display = $cart_item;
			}
		}
		
		$this->assertEquals(' White, Onions, Lettuce, Tomato, Red Wine Vinegar, Olive Oil Blend, Oregano, Salt', $sub_display['item_description']);
		$this->assertEquals('',$chip_display['item_description']);
    	
    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$this->assertNotNUll($jersey_mikes_formatted_order);
    	//$this->assertEquals(10.73, $jersey_mikes_formatted_order['taxable']); 
    	$items = $jersey_mikes_formatted_order['items'];
    	foreach ($items as $the_item)
    	{
    		if ($the_item['item_name'] == 'Large Chip Drink Combo')
    			$large_CD_item = $the_item;
    		else if ($the_item['item_name'] == 'Regular Chip Drink Combo')
    			$regular_CD_item = $the_item;
    		else if ($the_item['item_name'] == "Bags Of Chips")
    			$chips = $the_item;
    		else if ($the_item['item_name'] == "#3 The American Classic")
    			$sandwich_item = $the_item;
    			
    	}
    	$this->assertEquals('GPLU100',$regular_CD_item['vplu']);
    	$this->assertEquals('GPLU200',$large_CD_item['vplu']);
    	$this->assertEquals('PLU11702',$chips['vplu']);
    	$this->assertEquals('PLU202',$sandwich_item['vplu']);
    	
		$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
    	$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    }
    
    function testNoFountainDrinksAndCombosOnDeliveryMenu()
    {
    	setContext("com.splickit.jerseymikes");
    	$merchant_id = $this->ids['delivery_merchant_id'];
    	// use admin to avoid caching
		$user = logTestUserIn(2);
		
		$request = new Request();
		$request->url = "/merchants/$merchant_id";
		$merchant_controller = new MerchantController($mt, $user, $request, 5);
		$merchant_resource = $merchant_controller->getMerchant();
		$menu = $merchant_resource->menu;
		unset($menu['combos']);
		$menu_to_string = cleanUpCRLFTFromString(var_export($menu,true));
		$this->assertContains('Fountain Drink', $menu_to_string);
		$this->assertContains('Chip Drink Combo', $menu_to_string);
		$this->assertContains('Bags Of Chips', $menu_to_string,"should contain bags of chips modifier");
		$this->assertNotContains('Cheetos', $menu_to_string," should not contain chip flavor");
		
		$request = new Request();
		$request->url = "/merchants/$merchant_id/delivery";
		$merchant_controller = new MerchantController($mt, $user, $request, 5);
		$merchant_resource = $merchant_controller->getMerchant();
		$menu = $merchant_resource->menu;
		// unset the combo section for now
		unset($menu['combos']);
		$menu_to_string = cleanUpCRLFTFromString(var_export($menu,true));
		myerror_log($menu_to_string);
		//$this->assertNotContains('Fountain Drink', $menu_to_string);
		$this->assertTrue(substr_count($menu_to_string, 'Fountain Drink') < 1,"should not contain foiuntain drink");
		//$this->assertNotContains('Chip Drink Combo', $menu_to_string);
		$this->assertTrue(substr_count($menu_to_string, 'Chip Drink Combo') < 1,"shoujld not contain combo");
		//$this->assertNotContains('22oz Fountain',$menu_to_string);
		$this->assertTrue(substr_count($menu_to_string, '22oz Fountain') < 1,"should not contain foiuntain drink 22oz");
		//$this->assertNotContains('32oz Fountain',$menu_to_string);
		$this->assertTrue(substr_count($menu_to_string, '32oz Fountain') < 1,"should not contain foiuntain drink 32 oz");
		//$this->assertNotContains('combo pricing',$menu_to_string);
		$this->assertTrue(substr_count($menu_to_string, 'combo pricing') < 1,"should not contain any combos");
		//$this->assertNotContains('Bags Of Chips', $menu_to_string,"Should not contain bags of chips modifier");
		$this->assertTrue(substr_count($menu_to_string, 'Bags Of Chips') < 1,"Should not contain bags of chips modifier");
		//$this->assertContains('Cheetos',$menu_to_string,"Should contain chip flavor modifier"); 
		$this->assertTrue(substr_count($menu_to_string, 'Cheetos') > 0,"Should contain chip flavor modifier");

    }
    
    function testBottledDrinkOnDeliveryMenu()
    {
    	setContext("com.splickit.jerseymikes");
    	$merchant_id = $this->ids['delivery_merchant_id2'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
		$menu = $this->loadMenu($merchant_id,102433);
		$price = "2.00";
		$json = '{"user_id":"'.$user['user_id'].'","name":"","address1":"1412 East O Street","address2":"","city":"Lincoln","state":"NE","zip":"68508","phone_no":"1234567890","lat":40.813621,"lng":-96.700979}';
    	$request = new Request();
    	$request->body = $json;
    	$request->mimetype = "Application/json";
    	$request->_parseRequestBody(); 
    	$user_controller = new UserController($mt, $user, $request,5);
    	$response = $user_controller->setDeliveryAddr();
    	$this->assertNull($response->error,"should not have gotten a delivery save error but did");
    	$user_address_id = $response->user_addr_id;

		$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
     	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"item_id\":\"277816\",\"size_id\":\"89668\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2270806\",\"size_id\":\"0\"}]}],\"total_points_used\":0,\"note\":\"skip hours.\",\"lead_time\":104,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_number\":\"3036677493\",\"sub_total\":\"1.99\",\"grand_total\":\"3.17\",\"actual_pickup_time\":\"1391973893\",\"delivery\":\"yes\",\"user_addr_id\":\"$user_address_id\",\"delivery_time\":\"1:24 PM\"}}";    	
		$array = json_decode($json_encoded_data,true);
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);
    	    	
    	$complete_menu_object = new CompleteMenu(102433);
    	$complete_menu_object->merchant_menu_type = 'Delivery';
    	$complete_menu_object->getAllMenuItemSizePrices(102433, 'Y', $merchant_id, $mimetypes, $ltos);
    	$item_sizes = $complete_menu_object->getFullPriceListByItemSize();
    	$bottled_drink_price = $item_sizes['277816-89668'];

    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$this->assertEquals($bottled_drink_price, $jersey_mikes_formatted_order['taxable']);  
    	$items = $jersey_mikes_formatted_order['items'];
    	$bottled_drink_item = $items[0];
    	$this->assertEquals(1, $bottled_drink_item['quantity']);
    	$this->assertEquals($bottled_drink_price,$bottled_drink_item['price']);
    	$this->assertEquals(92,$bottled_drink_item['withs'][0]);  	 

    	// now lets try it with 2 bottles
    	$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
     	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"item_id\":\"277816\",\"size_id\":\"89668\",\"mods\":[{\"mod_quantity\":2,\"modifier_item_id\":\"2270806\",\"size_id\":\"0\"}]}],\"total_points_used\":0,\"note\":\"skip hours.\",\"lead_time\":104,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_number\":\"3036677493\",\"sub_total\":\"1.99\",\"grand_total\":\"3.17\",\"actual_pickup_time\":\"1391973893\",\"delivery\":\"yes\",\"user_addr_id\":\"$user_address_id\",\"delivery_time\":\"1:24 PM\"}}";    	
		$array = json_decode($json_encoded_data,true);
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);
    	
    	$order2 = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_formatted_order2 = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order2);
    	$this->assertEquals(2*$bottled_drink_price, $jersey_mikes_formatted_order2['taxable']);  
    	$items = $jersey_mikes_formatted_order2['items'];
    	$bottled_drink_item = $items[0];
       	$this->assertEquals(2, $bottled_drink_item['quantity']);
    	$this->assertEquals($bottled_drink_price,$bottled_drink_item['price']);
    	$this->assertEquals(2*$bottled_drink_price,$bottled_drink_item['item_total']);
    	$this->assertEquals(92,$bottled_drink_item['withs'][0]);

    	//now lets try with 2 different bottles
    	$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
     	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"item_id\":\"277816\",\"size_id\":\"89668\",\"mods\":[{\"mod_quantity\":2,\"modifier_item_id\":\"2270806\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2296491\",\"size_id\":\"0\"}]}],\"total_points_used\":0,\"note\":\"skip hours.\",\"lead_time\":104,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_number\":\"3036677493\",\"sub_total\":\"1.99\",\"grand_total\":\"3.17\",\"actual_pickup_time\":\"1391973893\",\"delivery\":\"yes\",\"user_addr_id\":\"$user_address_id\",\"delivery_time\":\"1:24 PM\"}}";    	
		$array = json_decode($json_encoded_data,true);
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);
    	
    	$order3 = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_formatted_order3 = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order3);
    	$this->assertEquals(3*$bottled_drink_price, $jersey_mikes_formatted_order3['taxable']);  
    	$items = $jersey_mikes_formatted_order3['items'];
    	$this->assertCount(3, $items);
    	foreach ($items as $item)
    	{
    		if ($item['vplu'] == PLU52313) {
    			continue;
    		}
    		if ($item['quantity'] == 1) {
    			$gatorade_item = $item;
    		} else if ($item['quantity'] == 2) {
    			$sobe_item = $item;
    		}
    	}
    	$this->assertEquals($bottled_drink_price,$sobe_item['price']);
    	$this->assertEquals(2*$bottled_drink_price,$sobe_item['item_total']);
    	$this->assertEquals(92,$sobe_item['withs'][0]);
       	
    	$this->assertEquals($bottled_drink_price,$gatorade_item['price']);
    	$this->assertEquals($bottled_drink_price,$gatorade_item['item_total']);
    	$this->assertEquals(251,$gatorade_item['withs'][0]);
    	
    }

/*    
    function testBottledDrinkChipSub()
    {
    	$merchant_id = $this->ids['regular_store'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
		$menu = $this->loadMenu($merchant_id);
    	
		$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	$json_encoded_data = "{\"jsonVal\":{\"$merchant_id\":\"104496\",\"items\":[{\"quantity\":1,\"note\":\"\",\"item_id\":\"283326\",\"size_id\":\"91746\",\"sizeprice_id\":\"1013193\",\"mods\":[{\"mod_sizeprice_id\":\"4410906\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4410870\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4410872\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4410873\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4410874\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4410875\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4410927\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4470211\",\"mod_quantity\":1}]}],\"total_points_used\":0,\"note\":\"\",\"lead_time\":313,\"promo_code\":\"\",\"tip\":\"0.00\",\"user_id\":\"20000\",\"loyalty_number\":\"1234567890\",\"sub_total\":\"6.93\",\"grand_total\":\"8.56\",\"actual_pickup_time\":\"1391972400\",\"delivery\":\"yes\",\"user_addr_id\":\"120812\",\"delivery_time\":\"1:00 PM\"}}";
    }
*/  
  
    function testSubsByTheBoxMultiSub()
    {
    	$merchant_id = $this->ids['regular_store'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
		$menu = $this->loadMenu($merchant_id);
    	
    	$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2261182\",\"size_id\":\"0\"},{\"mod_quantity\":2,\"modifier_item_id\":\"2261200\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2261191\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2261197\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258930\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258931\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258932\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258933\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258934\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258935\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258940\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2262080\",\"size_id\":\"0\"}],\"item_id\":\"277704\",\"size_id\":\"89970\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"49.95\",\"grand_total\":\"53.83\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);
    	    	
    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$this->assertNotNull($jersey_mikes_formatted_order);
    	$items = $jersey_mikes_formatted_order['items'];
    	$this->assertEquals(1, sizeof($items));
    	$item = $items[0];
    	$this->assertEquals("GPLU143500",$item['vplu']);
    	$this->assertEquals(0,sizeof($item['withs']));
    	$components = $item['components'];
    	$this->assertEquals(4,sizeof($components, $mode)); 
    	$component0 = $components[0];
    	$this->assertEquals(2, sizeof($component0['withs']));
    	$this->assertEquals(2, sizeof($component0['holds']));
    	$component2 = $components[2];
    	$this->assertEquals(2, sizeof($component2['withs']));
    	$this->assertEquals(2, sizeof($component2['holds']));
    	
		$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
    	$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    	
    }
    
    function testSubsByTheBox4SubsTheSame()
    {
    	$merchant_id = $this->ids['regular_store'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
		$menu = $this->loadMenu($merchant_id);
    	
    	$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2261182\",\"size_id\":\"0\"},{\"mod_quantity\":4,\"modifier_item_id\":\"2261200\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258930\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258931\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258932\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258933\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258934\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258935\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258940\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2262080\",\"size_id\":\"0\"}],\"item_id\":\"277704\",\"size_id\":\"89970\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"49.95\",\"grand_total\":\"53.83\",\"actual_pickup_time\":\"$pickup_time\"}}";//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103478\",\"items\":[{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"751842\",\"mods\":[{\"mod_sizeprice_id\":\"4063418\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063434\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063435\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063426\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063432\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063358\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063359\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063360\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063361\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063362\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063363\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063368\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063453\",\"mod_quantity\":1}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"49.95\",\"grand_total\":\"53.83\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);
    	    	
    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$this->assertNotNull($jersey_mikes_formatted_order);
    	$items = $jersey_mikes_formatted_order['items'];
    	$this->assertEquals(1, sizeof($items));
    	$item = $items[0];
    	$this->assertEquals("GPLU143500",$item['vplu']);
    	$this->assertEquals(0,sizeof($item['withs']));
    	$components = $item['components'];
    	$this->assertEquals(4,sizeof($components, $mode)); 
    	$component0 = $components[0];
    	$this->assertEquals('PLU1003', $component0['vplu']);
    	$this->assertEquals(2, sizeof($component0['withs']));
    	$this->assertEquals(2, sizeof($component0['holds']));
    	$component2 = $components[2];
    	$this->assertEquals('PLU1003', $component2['vplu']);
    	$this->assertEquals(2, sizeof($component2['withs']));
    	$this->assertEquals(2, sizeof($component2['holds']));
    	
		$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
    	$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    	
    }

    function testJerseyMikesComboOnlyFromMainMenu()
    {
		$merchant_id = $this->ids['regular_store'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
		$menu = $this->loadMenu($merchant_id);
    	
    	$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
		$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2262076\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2271915\",\"size_id\":\"0\"}],\"item_id\":\"277814\",\"size_id\":\"89668\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"2.54\",\"grand_total\":\"2.74\",\"actual_pickup_time\":\"$pickup_time\"}}";
		$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);
    	
    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$this->assertEquals(1,sizeof($jersey_mikes_formatted_order['items']));
    	$this->assertEquals('GPLU100',$jersey_mikes_formatted_order['items'][0]['vplu']);
    	$components = $jersey_mikes_formatted_order['items'][0]['components'];
    	$this->assertEquals(2, sizeof($components, $mode));
    	
		$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
    	$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    }
   
    function testSubWithAddedBacon()
    {
		$merchant_id = $this->ids['regular_store'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
		$menu = $this->loadMenu($merchant_id);
    	
    	$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"tip\":\"$tip\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2258945\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258930\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258931\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258932\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258933\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258934\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258935\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258936\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258942\",\"size_id\":\"89585\"}],\"item_id\":\"276771\",\"size_id\":\"89585\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"5.95\",\"grand_total\":\"6.42\",\"actual_pickup_time\":\"$pickup_time\"}}";//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103478\",\"tip\":$tip,\"items\":[{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"751787\",\"mods\":[{\"mod_sizeprice_id\":\"4063391\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063358\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063359\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063360\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063361\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063362\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063363\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063364\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063371\",\"mod_quantity\":1}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"5.95\",\"grand_total\":\"6.42\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);
    	
    	//get bacon price
    	$msma = new ModifierSizeMapAdapter($mimetypes);
    	$record = $msma->getRecord(array("modifier_item_id"=>2258942,"size_id"=>89585,"merchant_id"=>$merchant_id));
		$bacon_price = $record['modifier_price'];
		
		$sandwich_price = $menu['PLU102']['price'];
		
		$total_sandwich_price_with_bacon = $bacon_price + $sandwich_price;
    	
    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$this->assertNotNull($jersey_mikes_formatted_order);
    	$items = $jersey_mikes_formatted_order['items'];
    	$this->assertEquals(1, sizeof($items));
    	$item = $items[0];
    	$this->assertEquals("PLU102",$item['vplu']);
    	$withs = $item['withs'];
    	$this->assertEquals(1,sizeof($withs, $mode));
    	$this->assertEquals("1", $withs[0]);
    	$price = $item['price'];
    	$this->assertEquals($total_sandwich_price_with_bacon,$price);
    	
		$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
    	$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    }
    
    function testSubsByTheBox()
    {
    	$merchant_id = $this->ids['regular_store'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
		$menu = $this->loadMenu($merchant_id);
    	
    	$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2261182\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2261199\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2261200\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2261191\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2261197\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258930\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258931\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258932\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258933\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258934\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258935\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258940\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2262080\",\"size_id\":\"0\"}],\"item_id\":\"277704\",\"size_id\":\"89970\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"49.95\",\"grand_total\":\"53.83\",\"actual_pickup_time\":\"$pickup_time\"}}";//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103478\",\"items\":[{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"751842\",\"mods\":[{\"mod_sizeprice_id\":\"4063418\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063434\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063435\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063426\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063432\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063358\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063359\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063360\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063361\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063362\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063363\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063368\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063453\",\"mod_quantity\":1}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"49.95\",\"grand_total\":\"53.83\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);
    	    	
    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$this->assertNotNull($jersey_mikes_formatted_order);
    	$items = $jersey_mikes_formatted_order['items'];
    	$this->assertEquals(1, sizeof($items));
    	$item = $items[0];
    	$this->assertEquals("GPLU143500",$item['vplu']);
    	$this->assertEquals(0,sizeof($item['withs']));
    	$components = $item['components'];
    	$this->assertEquals(4,sizeof($components, $mode)); 
    	$component0 = $components[0];
    	$this->assertEquals(2, sizeof($component0['withs']));
    	$this->assertEquals(2, sizeof($component0['holds']));
    	$component2 = $components[2];
    	$this->assertEquals(2, sizeof($component2['withs']));
    	$this->assertEquals(2, sizeof($component2['holds']));
    	
		$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
    	$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    	
    }

	function testLunchBox()
    {
    	$merchant_id = $this->ids['regular_store'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
		$menu = $this->loadMenu($merchant_id);
    	
    	$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2261182\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258946\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2261196\",\"size_id\":\"89972\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258934\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258935\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258936\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258941\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2261205\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2271921\",\"size_id\":\"0\"}],\"item_id\":\"277705\",\"size_id\":\"89972\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"5.95\",\"grand_total\":\"6.42\",\"actual_pickup_time\":\"$pickup_time\"}}";//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103478\",\"items\":[{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"751844\",\"mods\":[{\"mod_sizeprice_id\":\"4063418\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063392\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4066775\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063362\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063363\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063364\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063369\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063448\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063415\",\"mod_quantity\":1}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"5.95\",\"grand_total\":\"6.42\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);
    	
    	$price = $menu['277705-89972']['price'];

    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$this->assertNotNull($jersey_mikes_formatted_order);
    	$this->assertEquals($price,$jersey_mikes_formatted_order['taxable']);
    	$items = $jersey_mikes_formatted_order['items'];
    	$this->assertEquals(1, sizeof($items));
    	$item = $items[0];
    	$this->assertEquals($price, $item['price']);
    	
    	$this->assertEquals("GPLU400",$item['vplu']);
    	$this->assertEquals(0,sizeof($item['withs']));
    	$components = $item['components'];
     	$this->assertEquals(3, sizeof($items[0]['components'], $mode));
    	// make sure each component has the withs and the holds	
    	foreach ($items[0]['components'] as $component)
    	{
    		if ($component['vplu'] == 'PLU601')
				$sub_component = $component;
    		else if ($component['vplu'] == 'PLU11702')
				$chip_component = $component;
    		else if ($component['vplu'] == 'PLU11301')
				$cookie_component = $component;	    		
    	}
    	$this->assertEquals(5, sizeof($sub_component['holds']));
    	$this->assertEquals(2, sizeof($sub_component['withs']));
    	$this->assertEquals(1, sizeof($chip_component['withs']));
    	$this->assertEquals(1, sizeof($cookie_component['withs']));
    	
		$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
    	$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    }
    
    function testQuantityOfLunchBox()
    {
		$quantity = 5;
    	$merchant_id = $this->ids['regular_store'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
		$menu = $this->loadMenu($merchant_id);
    	
    	$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":$quantity,\"modifier_item_id\":\"2261182\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258945\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2261199\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258930\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258931\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258932\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258933\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258934\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258935\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258936\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2261206\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2271921\",\"size_id\":\"0\"}],\"item_id\":\"277705\",\"size_id\":\"89971\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"8.45\",\"grand_total\":\"9.11\",\"actual_pickup_time\":\"$pickup_time\"}}";//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103478\",\"items\":[{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"751843\",\"mods\":[{\"mod_sizeprice_id\":\"4063418\",\"mod_quantity\":5},{\"mod_sizeprice_id\":\"4063391\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063434\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063358\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063359\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063360\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063361\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063362\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063363\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063364\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063449\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063417\",\"mod_quantity\":1}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"8.45\",\"grand_total\":\"9.11\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);
    	
    	$price = $menu['277705-89971']['price'];
    	$total_price = $quantity * $price;
    	
    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$this->assertNotNull($jersey_mikes_formatted_order);
    	$this->assertEquals($total_price,$jersey_mikes_formatted_order['taxable']);
    	$items = $jersey_mikes_formatted_order['items'];
    	$this->assertEquals(1, sizeof($items));
    	$item = $items[0];
    	$this->assertEquals($price, $item['price']);
 		$this->assertEquals($total_price, $item['item_total']);
 		
		$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
 		$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
 
    }

    function testQuantityOfBrownies()
    {
    	$quantity = 10;
    	$merchant_id = $this->ids['regular_store'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
		$menu = $this->loadMenu($merchant_id);
    	
    	$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":$quantity,\"modifier_item_id\":\"2263862\",\"size_id\":\"0\"}],\"item_id\":\"276965\",\"size_id\":\"89668\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"0.75\",\"grand_total\":\"0.81\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);
    	    	
    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	
    	$price = $menu['PLU11502']['price'];
    	$total_price  = $quantity * $price;
    	
    	$this->assertNotNull($jersey_mikes_formatted_order);
    	$this->assertEquals($total_price,$jersey_mikes_formatted_order['taxable']);
    	$items = $jersey_mikes_formatted_order['items'];
    	$this->assertEquals(1, sizeof($items));
    	$item = $items[0];
    	$this->assertEquals($quantity, $item['quantity']);
    	$this->assertEquals($price, $item['price']);
 		$this->assertEquals($total_price, $item['item_total']);
 		
		$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
 		$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    }
    
    function testQuantityOfCookies()
    {
    	$quantity = 4;
    	$merchant_id = $this->ids['regular_store'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
		$menu = $this->loadMenu($merchant_id);
    	
	    $tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103478\",\"items\":[{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"751836\",\"mods\":[{\"mod_sizeprice_id\":\"4063457\",\"mod_quantity\":4},{\"mod_sizeprice_id\":\"4063448\",\"mod_quantity\":1}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"0.5\",\"grand_total\":\"0.54\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":$quantity,\"modifier_item_id\":\"2263861\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2272054\",\"size_id\":\"0\"}],\"item_id\":\"276964\",\"size_id\":\"91012\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"5.00\",\"grand_total\":\"5.54\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);
    	   	
		$price = $menu['PLU11301']['price'];
    	$total_price  = $quantity * $price;
    	
    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$this->assertNotNull($jersey_mikes_formatted_order);
    	$this->assertEquals($total_price,$jersey_mikes_formatted_order['taxable']);
    	$items = $jersey_mikes_formatted_order['items'];
    	$this->assertEquals(1, sizeof($items));
    	$item = $items[0];
    	$this->assertEquals($quantity, $item['quantity']);
    	$this->assertEquals($price, $item['price']);
 		$this->assertEquals($total_price, $item['item_total']);
 		
		$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
 		$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    }
    
	function testLunchBoxWithHolds()
    {
    	$merchant_id = $this->ids['regular_store'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
		$menu = $this->loadMenu($merchant_id);
    	
    	$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
		//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103478\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":2,\"modifier_item_id\":\"2263861\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2271923\",\"size_id\":\"0\"}],\"item_id\":\"276964\",\"size_id\":\"90816\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"5.00\",\"grand_total\":\"5.54\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2261182\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258946\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2261191\",\"size_id\":\"89971\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258933\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258934\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258935\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258936\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258940\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258937\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258938\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2261205\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2271921\",\"size_id\":\"0\"}],\"item_id\":\"277705\",\"size_id\":\"89971\"},{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2258945\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258930\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258931\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258932\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258933\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258934\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258935\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258936\",\"size_id\":\"0\"}],\"item_id\":\"276772\",\"size_id\":\"89586\"},{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2263861\",\"size_id\":\"0\"}],\"item_id\":\"277705\",\"size_id\":\"89971\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"18.9\",\"grand_total\":\"20.37\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);
    	    	
    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$this->assertNotNull($jersey_mikes_formatted_order);
    	//$this->assertEquals(18.90,$jersey_mikes_formatted_order['taxable']);
    	$items = $jersey_mikes_formatted_order['items'];
    	$this->assertEquals(3, sizeof($items));
    	foreach ($items as $the_item)
    	{
    		if ($the_item['vplu'] == 'GPLU600')
    			$item = $the_item;
    	}
    	$price = $menu['277705-89971']['price'];
    	$this->assertEquals($price, $item['price']);
    	
    	//$this->assertEquals("GPLU600",$item['vplu']);
    	$this->assertEquals(0,sizeof($item['withs']));
    	$components = $item['components'];
     	$this->assertEquals(3, sizeof($items[0]['components'], $mode));
    	// make sure each component has the withs and the holds	
    	foreach ($items[0]['components'] as $component)
    	{
    		if ($component['vplu'] == 'PLU102')
				$sub_component = $component;
    		else if ($component['vplu'] == 'PLU11702')
				$chip_component = $component;
    		else if ($component['vplu'] == 'PLU11301')
				$cookie_component = $component;	    		
    	}
    	$this->assertEquals(4, sizeof($sub_component['holds']));
    	$this->assertEquals(4, sizeof($sub_component['withs']));
    	$this->assertEquals(1, sizeof($chip_component['withs']));
    	$this->assertEquals('222',$chip_component['withs'][0]);
    	$this->assertEquals(1, sizeof($cookie_component['withs']));
    	
		$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
    	$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    }
    
	function testChickenVsSteakSandwichCaliforniaInNYC()
    {
		$merchant_id = $this->ids['nyc_store'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
//		$menu = CompleteMenu::getCompleteMenu(102433,'Y',$merchant_id);
//		$order_adapter = new OrderAdapter($mimetypes);
//		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours');
    	$tip = rand(100,1000)/100;
    	$pickup_time = time()+900;
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2284418\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2262079\"}],\"item_id\":\"283292\",\"size_id\":\"89590\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"6.75\",\"grand_total\":\"7.28\",\"actual_pickup_time\":\"$pickup_time\"}}";//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103478\",\"items\":[{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"751817\",\"mods\":[{\"mod_sizeprice_id\":\"4063409\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063391\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063399\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063400\",\"mod_quantity\":1}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"6.75\",\"grand_total\":\"7.28\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created a order. ".$order_resource->error);

    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$this->assertNotNull($jersey_mikes_formatted_order);
    	$this->assertNull($jersey_mikes_formatted_order['controller_error']);
    	$items = $jersey_mikes_formatted_order['items'];
    	$this->assertEquals(1, sizeof($items));
    	$this->assertEquals("PLU47202",$items[0]['vplu']);
    	$withs = $items[0]['withs'];
    	$this->assertEquals(0,sizeof($withs, $mode));
    	
		$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
    	$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    }
    
    function testBottledDrinksWiths()
    {
    	$merchant_id = $this->ids['no_tax_on_cold_subs_merchant_id'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];

		$menu = CompleteMenu::getCompletePickupMenu(102433,'Y',$merchant_id);
		$modifier_groups_hash = createHashmapFromArrayOfArraysByFieldName($menu['modifier_groups'],'modifier_group_name');
		$bottled_drinks_hash = createHashmapFromArrayOfArraysByFieldName($modifier_groups_hash['Bottled Drink']['modifier_items'],'modifier_item_name');

    	$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103586\",\"items\":[{\"quantity\":1,\"note\":\"skip hours\",\"sizeprice_id\":\"753251\",\"mods\":[{\"mod_sizeprice_id\":\"4067455\",\"mod_quantity\":2},{\"mod_sizeprice_id\":\"4067457\",\"mod_quantity\":3}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_number\":null,\"sub_total\":\"3.27\",\"grand_total\":\"3.53\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"skip hours\",\"mods\":[{\"mod_quantity\":2,\"modifier_item_id\":\"2270803\",\"size_id\":\"0\"},{\"mod_quantity\":3,\"modifier_item_id\":\"2270804\",\"size_id\":\"0\"}],\"item_id\":\"277816\",\"size_id\":\"89668\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_number\":null,\"sub_total\":\"3.27\",\"grand_total\":\"3.53\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created and order. ".$order_resource->error);
    	$order_items_for_display = $order_resource->order_summary;
    	$this->assertCount(2, $order_items_for_display);
    	$the_item = $order_items_for_display['cart_items'][0];
    	//$this->assertEquals("Aquafina(x2),Pepsi(x3)", $the_item['item_description']);
    	
    	$complete_menu = new CompleteMenu(102433);
    	$complete_menu->getAllMenuItemSizePrices(102433, 'Y', $merchant_id, $mimetypes, $ltos);
    	$price_list = $complete_menu->getFullPriceListByItemSize();
    	$base_bottled_drink_price = $price_list['277816-89668'];
    	
    	$order_items = $order_resource->order_items;
    	$this->assertEquals(2, sizeof($order_items, $mode));
    	$this->assertEquals(3*$base_bottled_drink_price,$order_items[0]['item_price']);
    	$this->assertEquals(2*$base_bottled_drink_price,$order_items[1]['item_price']);
    	
    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$items = $jersey_mikes_formatted_order['items'];
    	$this->assertEquals(2,sizeof($items));
    	$this->assertEquals("PLU15307", $items[0]['vplu']);
    	$this->assertEquals("Pepsi", $items[0]['item_name']);
    	$this->assertEquals("3", $items[0]['quantity']);
    	$this->assertEquals("PLU15307", $items[1]['vplu']);
    	$this->assertEquals("Aquafina", $items[1]['item_name']);
    	$this->assertEquals("2", $items[1]['quantity']);
    	
    			$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
    	$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    }
    
    function testNewCookieConfigPackOf3()
    {
    	$merchant_id = $this->ids['regular_store'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
		
		$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103478\",\"items\":[{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"751836\",\"mods\":[{\"mod_sizeprice_id\":\"4063457\",\"mod_quantity\":4},{\"mod_sizeprice_id\":\"4063448\",\"mod_quantity\":1}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"0.5\",\"grand_total\":\"0.54\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":4,\"modifier_item_id\":\"2263861\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2272054\",\"size_id\":\"0\"}],\"item_id\":\"276964\",\"size_id\":\"91011\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"5.00\",\"grand_total\":\"5.54\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);

    	$item_size_adapter = new ItemSizeAdapter($mimetypes);
    	$record = $item_size_adapter->getRecord(array("size_id"=>91011,"item_id"=>276964,"merchant_id"=>$merchant_id));
    	$price = $record['price'];
    	$total_taxable = 4 * $price;
    	
    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$this->assertNotNull($jersey_mikes_formatted_order);
    	$this->assertEquals($total_taxable, $jersey_mikes_formatted_order['taxable']);
    	$items = $jersey_mikes_formatted_order['items'];
    	$this->assertTrue(sizeof($items, $mode) == 1);
    	$item = $items[0];
    	$this->assertEquals('PLU11325', $item['vplu']);
    	$withs = $item['withs'];
    	$this->assertEquals(82, $withs[0]);
    }

    function testNewCookieConfigSingleCookie()
    {
    	$merchant_id = $this->ids['regular_store'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
    	    	
	    $tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103478\",\"items\":[{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"751836\",\"mods\":[{\"mod_sizeprice_id\":\"4063457\",\"mod_quantity\":4},{\"mod_sizeprice_id\":\"4063448\",\"mod_quantity\":1}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"0.5\",\"grand_total\":\"0.54\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":2,\"modifier_item_id\":\"2263861\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2272054\",\"size_id\":\"0\"}],\"item_id\":\"276964\",\"size_id\":\"91012\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"5.00\",\"grand_total\":\"5.54\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);

    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$this->assertNotNull($jersey_mikes_formatted_order);

    	$item_size_adapter = new ItemSizeAdapter($mimetypes);
    	$record = $item_size_adapter->getRecord(array("size_id"=>91012,"item_id"=>276964,"merchant_id"=>$merchant_id));
    	$price = $record['price'];
    	$total_taxable = 2 * $price;
    	
    	$this->assertEquals($total_taxable, $jersey_mikes_formatted_order['taxable']);
    	$items = $jersey_mikes_formatted_order['items'];
    	$this->assertTrue(sizeof($items, $mode) == 1);
    	$item = $items[0];
    	$this->assertEquals('PLU11301', $item['vplu']);
    	$withs = $item['withs'];
    	$this->assertEquals(82, $withs[0]);
    	
    }
    
	function testAdditionalTaxForBottledDrinksChicago()
    {    	
    	$merchant_id = $this->ids['chicago_merchant_id'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
    	
		$menu = $this->loadMenu($merchant_id);	
    	$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103504\",\"items\":[{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"753186\",\"mods\":[{\"mod_sizeprice_id\":\"4067111\",\"mod_quantity\":1}]},{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"753132\",\"mods\":[{\"mod_sizeprice_id\":\"4067016\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4066983\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4066984\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4066985\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4066986\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4066987\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4066988\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4066989\",\"mod_quantity\":1}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"7.74\",\"grand_total\":\"8.34\",\"actual_pickup_time\":\"$pickup_time\"}}";    	
   		$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2262511\",\"size_id\":\"0\"}],\"item_id\":\"277816\",\"size_id\":\"89668\"},{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2258945\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258930\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258931\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258932\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258933\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258934\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258935\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258936\",\"size_id\":\"0\"}],\"item_id\":\"276771\",\"size_id\":\"89585\"}],\"note\":\"skip hours\",\"user_id\":\"$user_id\",\"delivery\":\"N\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$this->assertNotNull($checkout_data_resource);
    	$this->assertNull($checkout_data_resource->error); 
    	
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2262511\",\"size_id\":\"0\"}],\"item_id\":\"277816\",\"size_id\":\"89668\"},{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2258945\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258930\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258931\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258932\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258933\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258934\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258935\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258936\",\"size_id\":\"0\"}],\"item_id\":\"276771\",\"size_id\":\"89585\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"7.74\",\"grand_total\":\"8.34\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);
    	    	
    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$this->assertNotNull($jersey_mikes_formatted_order);
    	// aditional tax should be 3% of the bottled drink 
    	
    	$tax_adapter = new TaxAdapter($mimetypes);
    	$taxes = $tax_adapter->getTotalTaxRates($merchant_id);
    	$tax_rate1 = $taxes[1];
    	$order_array = json_decode($json_encoded_data,true);
    	foreach ($order_array['jsonVal']['items'] as $order_item)
    	{
    		$item_price = $menu[$order_item['item_id'].'-'.$order_item['size_id']]['price'];
    		$total_price = $total_price + $item_price;
    	}
    	$total_tax = number_format($total_price * $tax_rate1,2);
    	
    	$this->assertEquals(.07,$jersey_mikes_formatted_order['additional_tax']);
    	$this->assertEquals($total_tax, $jersey_mikes_formatted_order['tax']);
  
		$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
    	$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    }
    
 	function testMultiChip()
    {
    	$merchant_id = $this->ids['regular_store'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
    	$menu = $this->loadMenu($merchant_id);
    	
    	$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103478\",\"items\":[{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"751835\",\"mods\":[{\"mod_sizeprice_id\":\"4063417\",\"mod_quantity\":5}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"4.25\",\"grand_total\":\"4.58\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":5,\"modifier_item_id\":\"2271915\",\"size_id\":\"0\"}],\"item_id\":\"276963\",\"size_id\":\"89668\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"4.25\",\"grand_total\":\"4.58\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created and order. ".$order_resource->error);

    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$items = $jersey_mikes_formatted_order['items'];
    	$this->assertEquals(1,sizeof($items));
    	$item = $items[0];
    	$this->assertEquals(5, $item['quantity']);
    	
    	$price = $menu['276963-89668']['price']; 
    	$item_total = $price * 5;
    	$this->assertEquals($item_total, $item['item_total']);
    	
		$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
    	$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    }
    
    function testMultiDrinkWithChip()
    {
    	$merchant_id = $this->ids['regular_store'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
		$menu = $this->loadMenu($merchant_id);
    	
		$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103478\",\"items\":[{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"751835\",\"mods\":[{\"mod_sizeprice_id\":\"4063417\",\"mod_quantity\":1}]},{\"quantity\":2,\"note\":\"\",\"sizeprice_id\":\"751839\",\"mods\":[{\"mod_sizeprice_id\":\"4063456\",\"mod_quantity\":2}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"4.23\",\"grand_total\":\"4.56\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2271915\",\"size_id\":\"0\"}],\"item_id\":\"276963\",\"size_id\":\"89668\"},{\"quantity\":2,\"note\":\"\",\"mods\":[{\"mod_quantity\":2,\"modifier_item_id\":\"2262511\",\"size_id\":\"0\"}],\"item_id\":\"277815\",\"size_id\":\"90054\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"$tip\":\"7.44\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"4.23\",\"grand_total\":\"4.56\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created and order. ".$order_resource->error);

    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$json_jm_order = json_encode($jersey_mikes_formatted_order);
    	$items = $jersey_mikes_formatted_order['items'];
    	$this->assertEquals(2, sizeof($items));
    	
    	$item1 = $items[0];
    	$item2 = $items[1];
    	$this->assertEquals("GPLU100", $item1['vplu']);
    	$price = $menu['277814-89668']['price'];
    	$item1_total = $price * $item1['quantity'];
    	$this->assertEquals($item1_total,$item1['item_total']);
    	$this->assertEquals('PLU8702',$item2['vplu']);
    	$price = $menu['PLU8702']['price'];
    	$item2_total = $price * $item2['quantity'];
    	$this->assertEquals($item2_total,$item2['item_total']);
    	
    	$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
    	$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    }
    
	function testJerseyMikesSandwhichPlusComboOrder()
    {
    	$merchant_id = $this->ids['regular_store'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
		$menu = $this->loadMenu($merchant_id);
    	
		$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103478\",\"items\":[{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"751787\",\"mods\":[{\"mod_sizeprice_id\":\"4063411\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063391\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063358\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063359\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063360\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063361\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063362\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063363\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063364\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063417\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063450\",\"mod_quantity\":1}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"7.99\",\"grand_total\":\"8.61\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2271915\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2262076\",\"size_id\":\"0\"}],\"item_id\":\"276771\",\"size_id\":\"89585\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"7.99\",\"grand_total\":\"8.61\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created and order. ".$order_resource->error);

    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	
    	$gplu100_price = $menu['277814-89668']['price'];
    	$plu102_price = $menu['PLU102']['price'];
    	
    	$total_taxable = $gplu100_price+$plu102_price;
    	$base_rate = TaxAdapter::staticGetTotalBaseTaxRate($merchant_id);
    	$total_tax = number_format($total_taxable*$base_rate/100,2);
    	
    	$this->assertEquals($total_taxable, $jersey_mikes_formatted_order['taxable']);
    	$this->assertEquals($total_tax, $jersey_mikes_formatted_order['tax']);
    	$this->assertEquals(2,sizeof($jersey_mikes_formatted_order['items']));
    	$this->assertEquals('GPLU100',$jersey_mikes_formatted_order['items'][0]['vplu']);
    	$this->assertEquals(0, sizeof($jersey_mikes_formatted_order['items'][0]['withs'], $mode));
    	$this->assertEquals(0, sizeof($jersey_mikes_formatted_order['items'][0]['holds'], $mode));
    	$this->assertEquals(2, sizeof($jersey_mikes_formatted_order['items'][0]['components'], $mode));

    	$this->assertEquals($gplu100_price, $jersey_mikes_formatted_order['items'][0]['price']);
    	$components = $jersey_mikes_formatted_order['items'][0]['components'];
    	foreach ($components as $component) {
    		if ($component['item_name'] == 'Bags Of Chips') {
    			$chips = $component;
    		}
    		if ($component['item_name'] == '22oz Fountain') {
    			$fountain = $component;
    		}
       	}
    	$this->assertEquals('PLU11702',$chips['vplu']);
    	$this->assertEquals('PLU8702',$fountain['vplu']);
    	$sandwich_item = $jersey_mikes_formatted_order['items'][1];
    	$this->assertEquals('PLU102',$sandwich_item['vplu']);
    	$this->assertEquals($plu102_price,$sandwich_item['price']);
    	
		$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
    	$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    }
    
	function testJerseyMikesComboOrderWithExtraChips()
    {
    	$merchant_id = $this->ids['regular_store'];
		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
		$menu = $this->loadMenu($merchant_id);
    	
		$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103478\",\"items\":[{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"751787\",\"mods\":[{\"mod_sizeprice_id\":\"4063411\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063391\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063358\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063359\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063360\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063361\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063362\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063363\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063364\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063417\",\"mod_quantity\":2},{\"mod_sizeprice_id\":\"4063450\",\"mod_quantity\":1}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"7.99\",\"grand_total\":\"8.61\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":2,\"modifier_item_id\":\"2271915\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2262076\",\"size_id\":\"0\"}],\"item_id\":\"276771\",\"size_id\":\"89585\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"7.99\",\"grand_total\":\"8.61\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);

    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	    	
    	$gplu100_price = $menu['277814-89668']['price'];
    	if ($menu['277814-89668']['tax_group'] == 0) {
    		$total_non_taxable = $gplu100_price;
    	} else {
    		$total_taxable = $gplu100_price;
    	}
    	
    	$plu102_price = $menu['PLU102']['price'];
    	if ($menu['PLU102']['tax_group'] == 0) {
    		$total_non_taxable = $total_non_taxable + $plu102_price;
    	} else {
    		$total_taxable = $total_taxable + $plu102_price;
    	}
    	
    	$plu11702_price = $menu['PLU11702']['price'];
    	if ($menu['PLU11702']['tax_group'] == 0) {
    		$total_non_taxable = $total_non_taxable + $plu11702_price;
    	} else {
    		$total_taxable = $total_taxable + $plu11702_price;
    	}
    	
    	$base_rate = TaxAdapter::staticGetTotalBaseTaxRate($merchant_id);
    	$total_tax = number_format($total_taxable*$base_rate/100,2);
    	
    	$this->assertEquals($total_taxable, $jersey_mikes_formatted_order['taxable']);
    	$this->assertEquals($total_non_taxable, $jersey_mikes_formatted_order['non_taxable']);
    	$this->assertEquals($total_tax, $jersey_mikes_formatted_order['tax']);
    	$this->assertEquals(3,sizeof($jersey_mikes_formatted_order['items']));
    	foreach ($jersey_mikes_formatted_order['items'] as $item)
    	{
    		if ($item['vplu'] == 'GPLU100')
    			$combo_item = $item;
    		if ($item['vplu'] == 'PLU102')
    			$sandwich_item = $item;
    		if ($item['vplu'] == 'PLU11702')
    			$extra_chips_items = $item;
    	}
    	$this->assertEquals(0, sizeof($combo_item['withs'], $mode));
    	$this->assertEquals(0, sizeof($combo_item['holds'], $mode));
    	$this->assertEquals(2, sizeof($combo_item['components'], $mode));
    	$this->assertEquals($gplu100_price, $combo_item['price']);
    	$components = $combo_item['components'];
    	$components = $jersey_mikes_formatted_order['items'][0]['components'];
    	foreach ($components as $component) {
    		if ($component['item_name'] == 'Bags Of Chips') {
    			$chips = $component;
    		}
    		if ($component['item_name'] == '22oz Fountain') {
    			$fountain = $component;
    		}
       	}
    	$this->assertEquals('PLU11702',$chips['vplu']);
    	$this->assertEquals('PLU8702',$fountain['vplu']);
    	
    	// check sandwhich
    	$this->assertEquals('PLU102',$sandwich_item['vplu']);
    	$this->assertEquals($plu102_price,$sandwich_item['price']);
    	//check extra chips
    	$this->assertEquals('PLU11702',$extra_chips_items['vplu']);
    	$this->assertEquals($plu11702_price,$extra_chips_items['price']);
    	
		$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
    	$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    	
    }

/*************************************************************************/
        
    function testSetDeliveryTaxRecordNontaxable()
    {
    	$merchant_resource = createNewTestMerchant();
    	$merchant_id = $merchant_resource->merchant_id;
    	$tax_adapter = new TaxAdapter($mimetypes);
    	$tax_resource = $tax_adapter->getExactResourceFromData(array("merchant_id"=>$merchant_id,"tax_group"=>1));
    	$tax_resource->rate = 25;
    	$tax_resource->save();
    	
    	$jm_price_records[JerseyMikesController::$non_taxable_delivery_plu] = array("plu_id"=>52313,"size_id"=>13,"price"=>1.00,"taxable"=>0);   
    	$jersey_mikes_importer = new JerseyMikesImporter();	
    	$jersey_mikes_importer->checkForDeliveryPluAndSetTaxRecordIfPresent($jm_price_records,$merchant_id);
    	
    	$tax_records = $tax_adapter->getTotalTaxRates($merchant_id);
    	$delivery_tax_rate = $tax_records[99];
    	$this->assertNotNull($delivery_tax_rate);
    	$this->assertEquals(0.00, $delivery_tax_rate); 
    }
   
    function testSetDeliveryTaxRecordTaxable()
    {
    	$merchant_resource = createNewTestMerchant();
    	$merchant_id = $merchant_resource->merchant_id;
    	$tax_adapter = new TaxAdapter($mimetypes);
    	$tax_resource = $tax_adapter->getExactResourceFromData(array("merchant_id"=>$merchant_id,"tax_group"=>1));
    	$tax_resource->rate = 25;
    	$tax_resource->save();
    	
    	$jm_price_records[JerseyMikesController::$taxable_delivery_plu] = array("plu_id"=>52413,"size_id"=>13,"price"=>1.00,"taxable"=>0);   
    	$jersey_mikes_importer = new JerseyMikesImporter();	
    	$jersey_mikes_importer->checkForDeliveryPluAndSetTaxRecordIfPresent($jm_price_records,$merchant_id);
    	
    	$tax_records = $tax_adapter->getTotalTaxRates($merchant_id);
    	$delivery_tax_rate = $tax_records[99];
    	$this->assertNotNull($delivery_tax_rate);
    	$this->assertEquals(0.25, $delivery_tax_rate); 
    }
   
    function testDeliveryInformationImport()
    {
    	//using other nebraska merchant with delivery
//    	$jersey_mikes_importer = new JerseyMikesImporter();
 //   	$result = $jersey_mikes_importer->importFullStore("33001");
    	$merchant_adapter = new MerchantAdapter($mimetypes);
    	$merchant = $merchant_adapter->getRecord(array("merchant_id"=>$this->ids['delivery_merchant_id']));
    	$this->assertNotNull($merchant);
    	$merchant_id = $merchant['merchant_id'];
    	
    	$tax_adapter = new TaxAdapter($mimetypes);
    	$tax_records = $tax_adapter->getTotalTaxRates($merchant_id);
    	$base_tax_rate = $tax_records['1'];
    	$this->assertTrue($base_tax_rate > 0.00);
    	$this->assertEquals($base_tax_rate, $tax_records['99']);
    	
    	$delivery_tax_record = $tax_adapter->getRecord(array("merchant_id"=>$merchant_id,"tax_group"=>99));
    	$this->assertEquals(JerseyMikesController::$taxable_delivery_plu, $delivery_tax_record['locale_description']);
    }

    function testCreateJerseyMikesItemNoWiths()
    {
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r, 5);
    	$jm_item = $jersey_mikes_controller->createJerseyMikesItem("12345", "PLU12345", "The Item", 1, 'the note', 5.55, 5.55, $withs, $holds, $components);
    	$this->assertEquals("12345", $jm_item['plu']);
    	$this->assertEquals("the note", $jm_item['note']);
    	$this->assertTrue(is_array($jm_item['withs']));
    	$this->assertCount(0, $jm_item['withs']);
    }
    
    function testCreateJerseyMikesItemWithWiths()
    {
    	$withs = array("this","that","and the other thing");
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r, 5);
    	$jm_item = $jersey_mikes_controller->createJerseyMikesItem("12345", "PLU12345", "The Item", 1, 'the note', 5.55, 5.55, $withs, $holds, $components);
    	$this->assertEquals("12345", $jm_item['plu']);
    	$this->assertEquals("the note", $jm_item['note']);
    	$this->assertTrue(is_array($jm_item['withs']));
    	$this->assertCount(3, $jm_item['withs']);
    }
    
    function testDeliveryIntegrationTaxable()
    {
    	$merchant_id = $this->ids['delivery_merchant_id'];
    	
    	//reset tax rate to 10% for ease of testing
    	$tax_adapter = new TaxAdapter($mimetypes);
    	$tax_resource = $tax_adapter->getExactResourceFromData(array("merchant_id"=>$merchant_id,"tax_group"=>1));
    	$tax_resource->rate = 10;
    	$tax_resource->save();

    	// should already be set to taxable from the import
    	$delivery_tax_resource = $tax_adapter->getExactResourceFromData(array("merchant_id"=>$merchant_id,"tax_group"=>99));
    	// set to 20 for ease of testing
    	$delivery_tax_resource->rate = '20';
    	$delivery_tax_resource->save();

    	$user_resource = createNewUser(array("flags"=>'1C20000001'));
    	$user = logTestUserIn($user_resource->user_id);
    	$json = '{"user_id":"'.$user['user_id'].'","name":"","address1":"1412 East O Street","address2":"","city":"Lincoln","state":"NE","zip":"68508","phone_no":"1234567890","lat":40.813621,"lng":-96.700979}';
    	$request = new Request();
    	$request->body = $json;
    	$request->mimetype = "Application/json";
    	$request->_parseRequestBody(); 
    	$user_controller = new UserController($mt, $user, $request,5);
    	$response = $user_controller->setDeliveryAddr();
    	$this->assertNull($response->error,"should not have gotten a delivery save error but did");
    	$user_address_id = $response->user_addr_id;
    	
    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'delivery', 'skip hours',5);
       	$order_data['lead_time'] = '45';
    	$order_data['delivery'] = 'yes';
    	$order_data['tip'] = 0.00;
    	$order_data['user_addr_id'] = $user_address_id;
    	$order_data['delivery_time'] = "As soon as possible.";
    	
    	$checkout_data_resource = getCheckoutDataWithThrottling($order_data, $merchant_id, $time_stamp);
    	$this->assertEquals(4.66,$checkout_data_resource->total_tax_amt);
		$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
		$this->assertContains("Your order to Jersey Mikes has been scheduled for delivery at ", $order_resource->user_message);
		$order_id = $order_resource->order_id;
    	$this->assertNull($order_resource->error);
    	$this->assertTrue($order_id > 1000);
    	
    	// make sure delivery is not part of the items for display
    	$this->assertCount(5, $order_resource->order_items);
    	$this->assertEquals(4.66, $order_resource->total_tax_amt);

    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r, 5);
    	$jm_order_data = $jersey_mikes_controller->convertOrderToJerseyMikesDataSetFromOrderId($order_id);
    	$this->assertEquals('41.55', $jm_order_data['taxable']);
    	$this->assertEquals('0.00', $jm_order_data['non_taxable']);
    	$this->assertEquals('4.66', $jm_order_data['tax']);
    	$this->assertCount(6, $jm_order_data['items']);
    	foreach($jm_order_data['items'] as $item)
    	{
    		if ($item['vplu'] == 'PLU52313') {
    			$delivery_item = $item;
    		}
    	}
    	
    	$this->assertNotNull($delivery_item);
    	$this->assertEquals('52313', $delivery_item['plu']);
    	$this->assertEquals(5.00, $delivery_item['price']);
    	$this->assertEquals('Delivery Charge', $delivery_item['item_name']);
    	$this->assertCount(0, $delivery_item['withs']);
    	$this->assertCount(0, $delivery_item['holds']);
    	$this->assertCount(0, $delivery_item['components']);
    	
    }
    
    function testDeliveryIntegrationNoTax()
    {
    	$merchant_id = $this->ids['delivery_merchant_id'];
    	
    	//$delivery_tax_resource = $tax_adapter->createTaxRecord($merchant_id, "Delivery",'10', 99);
    	$tax_adapter = new TaxAdapter($mimetypes);
    	$delivery_tax_resource = $tax_adapter->getExactResourceFromData(array("merchant_id"=>$merchant_id,"tax_group"=>99));
    	$delivery_tax_resource->rate = 0;
    	$delivery_tax_resource->locale_description = 'PLU52413';
    	$delivery_tax_resource->save();    	
    	
    	$user_resource = createNewUser(array("flags"=>'1C20000001'));
    	$user = logTestUserIn($user_resource->user_id);
    	$json = '{"user_id":"'.$user['user_id'].'","name":"","address1":"1412 East O Street","address2":"","city":"Lincoln","state":"NE","zip":"68508","phone_no":"1234567890","lat":40.813621,"lng":-96.700979}';
    	$request = new Request();
    	$request->body = $json;
    	$request->mimetype = "Application/json";
    	$request->_parseRequestBody(); 
    	$user_controller = new UserController($mt, $user, $request,5);
    	$response = $user_controller->setDeliveryAddr();
    	$this->assertNull($response->error,"should not have gotten a delivery save error but did");
    	$user_address_id = $response->user_addr_id;
    	
    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'delivery', 'skip hours',5);
       	$order_data['lead_time'] = '45';
    	$order_data['delivery'] = 'yes';
    	$order_data['tip'] = 0.00;
    	$order_data['user_addr_id'] = $user_address_id;
    	$order_data['delivery_time'] = "As soon as possible.";
		$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
		$order_id = $order_resource->order_id;
    	$this->assertNull($order_resource->error);
    	$this->assertTrue($order_id > 1000);

    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r, 5);
    	$jm_order_data = $jersey_mikes_controller->convertOrderToJerseyMikesDataSetFromOrderId($order_id);
    	$this->assertEquals('36.55', $jm_order_data['taxable']);
    	$this->assertEquals('5.00', $jm_order_data['non_taxable']);
    	$this->assertEquals('3.66', $jm_order_data['tax']);
    	$this->assertCount(6, $jm_order_data['items']);
    	foreach($jm_order_data['items'] as $item)
    	{
    		if ($item['vplu'] == 'PLU52413') {
    			$delivery_item = $item;
    		}
    	}
    	
    	$this->assertNotNull($delivery_item);
    	$this->assertEquals(5.00, $delivery_item['price']);
    	$this->assertEquals('Delivery Charge', $delivery_item['item_name']);
    	$this->assertCount(0, $delivery_item['withs']);
    	$this->assertCount(0, $delivery_item['holds']);
    	$this->assertCount(0, $delivery_item['components']);
    	
    }

    function testNoTaxOnItemForOrder()
    {
    	$user_resource = createNewUser(array('flags'=>'1C20000001'));
    	$user_id = $user_resource->user_id;
    	$return_array['user_id'] = $user_id;
    	logTestUserIn($user_id);
		$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	$merchant_id = $this->ids['no_tax_on_cold_subs_merchant_id'];
    	
    	//get item_size_id of mini turkey
    	$item_size_map_adapter = new ItemSizeAdapter($mimetypes);
    	$record = $item_size_map_adapter->getRecord(array("external_id"=>'PLU501',"merchant_id"=>$merchant_id));
    	$this->assertNotNull($record);

    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayFromFullMenu(102433, $merchant_id, 'skip hours',1,0,array($record['item_size_id']));    	
    	$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
    	$order_id = $order_resource->order_id;
    	$this->assertNull($order_resource->error);
    	$this->assertEquals(0.00, $order_resource->total_tax_amt);
    	
    	$jerser_mikes_controller = new JerseyMikesController($mt, $u, $r, 6);
    	$jersey_mikes_formatted_order = $jerser_mikes_controller->convertOrderToJerseyMikesDataSetFromOrderId($order_id);
    	$this->assertEquals(0.00, $jersey_mikes_formatted_order->taxable);
    	$this->assertEquals($price,$jersey_mikes_formatted_order->non_taxable);
    }
    
    function testCheeseSteakBasic()
    {
    	$user_resource = createNewUser(array('flags'=>'1C20000001'));
    	$user_id = $user_resource->user_id;
    	$return_array['user_id'] = $user_id;
    	logTestUserIn($user_id);
		$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	$merchant_id = $this->ids['nyc_store'];
    	
    	//get item_size_id of cheese steak
    	$item_size_map_adapter = new ItemSizeAdapter($mimetypes);
    	$record = $item_size_map_adapter->getRecord(array("external_id"=>'PLU47002:PLU47202',"merchant_id"=>$merchant_id));
    	$this->assertNotNull($record);

    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayFromFullMenu(102433, $merchant_id, 'skip hours',1,0,array($record['item_size_id']));  
    	// add the steak or chicken choice
    	$msm_adapter = new ModifierSizeMapAdapter($mimetypes);
    	$modifier_sm_record = $msm_adapter->getRecord(array("external_id"=>"PLU47002:noprice:noprice","merchant_id"=>$merchant_id));
    	$mods[] = array("mod_quantity"=>1,"mod_sizeprice_id"=>$modifier_sm_record['modifier_size_id']);  	
    	
    	$item = $order_data['items'][0];
    	$item['mods'] = $mods;
    	$order_data['items'][0] = $item;
    	$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
    	$order_id = $order_resource->order_id;
    	$this->assertNull($order_resource->error);
    	
    	$jerser_mikes_controller = new JerseyMikesController($mt, $u, $r, 6);
    	$jersey_mikes_formatted_order = $jerser_mikes_controller->convertOrderToJerseyMikesDataSetFromOrderId($order_id);
		$this->assertNull($jersey_mikes_formatted_order['controller_error']);	    	
    }

/*************  these tests below all require connection to the interweb ****************/    
    
	function testMessageSend()
    {
    	setContext("com.splickit.jerseymikes");
    	$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		$user_id = $user['user_id'];
		
    	$merchant_id = $this->ids['regular_store'];
    	//create JM message map
    	$map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_id,"message_format"=>'K',"delivery_addr"=>"JerseyMikes","message_type"=>"X"));
    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id,'pickup','the note skip hours');
    	$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created and order. ".$order_resource->error);
    	
    	$mmha = new MerchantMessageHistoryAdapter($mimetypes);
    	$mr_data['order_id'] = $order_id;
    	$mr_data['message_format'] = 'K';
    	$mr_options[TONIC_FIND_BY_METADATA] = $mr_data;
    	$message_resource = Resource::find($mmha,null,$mr_options);
    	$message_id = $message_resource->map_id;
    	$this->assertNotNull($message_resource);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$response = $jersey_mikes_controller->sendThisMessage($message_resource);
    	$this->assertTrue($response);
    	$new_message_resource = Resource::find($mmha,''.$message_id);
    	$message_response = $new_message_resource->response;
    	$response_array = json_decode($message_response,true);
    	$this->assertEquals("success",strtolower($response_array['status']),"error sending order to JM: ".$response_array['message']); 
    	$order_id = $response_array['response']['order_id'];
    	$this->assertNotNull($order_id);
    }
    
	function testTheImport()

    {    	
/*    	$o_store_basic['merchant_external_id']='HD';
		$o_store_basic['brand_id'] = 326;
		$options_store_basic[TONIC_FIND_BY_METADATA] = $o_store_basic;
		$merchant_adapter = new MerchantAdapter($mimetypes);
		$merchant_resource = Resource::find($merchant_adapter,'',$options_store_basic);
		$merchant_resource->lat = 10.000001;
		$merchant_resource->lng = -10.000001;
		$merchant_resource->save(); 
		
		$merchant_resource2 = SplickitController::getResourceFromId($merchant_resource, 'Merchant');
		$this->assertEquals(10.000001, $merchant_resource2->lat); 
		$this->assertEquals(-10.000001, $merchant_resource2->lng); 
		
*/		
    	$jersey_mikes_importer = new JerseyMikesImporter();
    	$result = $jersey_mikes_importer->importFullStore(5025);
    	
    	$merchant_id = $jersey_mikes_importer->imported_merchant_ids[0];
    	$this->assertTrue($merchant_id > 1000);
    	
    	$jm_prices_array = $jersey_mikes_importer->getJmPricesArray();
    	$small_combo_price = $jm_prices_array['GPLU100']['price'];
    	$large_combo_price = $jm_prices_array['GPLU200']['price'];
    	$lunch_box_with_drink_price = $jm_prices_array['GPLU700']['price'];
    	
    	$menu = CompleteMenu::getCompleteMenu(102433,'Y',$merchant_id);
    	$this->assertNotNull($menu);
    	$this->assertEquals($small_combo_price, $menu['combos'][0]['price']);
    	$this->assertEquals($large_combo_price, $menu['combos'][1]['price']);
    	$this->assertTrue($menu['combos'][0]['price'] > 0.00);
    	$this->assertTrue($menu['combos'][1]['price'] > 0.00);
    	
    	//now check override for the chips
    	$imgma = new ItemModifierGroupMapAdapter($mimetypes);
    	$base_imgm_record = $imgma->getRecord(array("item_id"=>277814,"modifier_group_id"=>328207,"merchant_id"=>0));
    	$this->assertNotNull($base_imgm_record);
    	$this->assertEquals(2.00, $base_imgm_record['price_override']);
    	
    	$merchant_imgm_record = $imgma->getRecord(array("item_id"=>277814,"modifier_group_id"=>328207,"merchant_id"=>$merchant_id));
    	$this->assertNotNull($merchant_imgm_record);
    	$this->assertEquals(2.00, $merchant_imgm_record['price_override']);
    	
    	$return_array['merchant_id'] = $merchant_id;
    	
    	$return_array['small_combo_price'] = $small_combo_price;
    	$return_array['large_combo_price'] = $large_combo_price;
    	$return_array['lunch_box_with_drink_price'] = $lunch_box_with_drink_price;
    	return $return_array;
    }
    
    /**
     * @depends testTheImport
     */

    function testItemLevelSmallComboOrder($return_array)
    {
    	$user_resource = createNewUser(array('flags'=>'1C20000001'));
    	$user_id = $user_resource->user_id;
    	$return_array['user_id'] = $user_id;
    	logTestUserIn($user_id);
		$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	$small_combo_price = $return_array['small_combo_price'];
    	$merchant_id = $return_array['merchant_id'];
    	
    	//$menu = CompleteMenu::getCompleteMenu(102433,'Y',$merchant_id);
    	
    	//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103586\",\"items\":[{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"753248\",\"mods\":[{\"mod_sizeprice_id\":\"4067302\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4067218\",\"mod_quantity\":1}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"2.38\",\"grand_total\":\"2.57\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2270133\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2271915\"}],\"item_id\":\"277814\",\"size_id\":\"89668\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"$small_combo_price\",\"grand_total\":\"0.00\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$tax = $checkout_data_resource->total_tax_amt;
    	$grand_total = $tip+$small_combo_price+tax;
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2270133\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2271915\",\"size_id\":\"0\"}],\"item_id\":\"277814\",\"size_id\":\"89668\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"$small_combo_price\",\"grand_total\":\"$grand_total\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created and order. ".$order_resource->error);

    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$items = $jersey_mikes_formatted_order['items'];
    	$this->assertEquals(1,sizeof($items));
    	$item = $items[0];
    	$this->assertEquals("GPLU100", $item['vplu']);
    	$this->assertEquals($small_combo_price, $item['item_total']);
    	
		$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
    	$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    	
    	return $return_array;
    }	
    
    /**
     * @depends testItemLevelSmallComboOrder
     */
    
    function testItemLevelLargeComboOrder($return_array)
    {
    	$user_id = $return_array['user_id'];
    	$user = logTestUserIn($user_id);
		$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	$large_combo_price = $return_array['large_combo_price'];
    	$merchant_id = $return_array['merchant_id'];
    	
    	//$menu = CompleteMenu::getCompleteMenu(102433,'Y',$merchant_id);
    	
    	//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103586\",\"items\":[{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"753248\",\"mods\":[{\"mod_sizeprice_id\":\"4067302\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4067218\",\"mod_quantity\":1}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"2.38\",\"grand_total\":\"2.57\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2270134\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2271915\",\"size_id\":\"0\"}],\"item_id\":\"277814\",\"size_id\":\"89668\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"$large_combo_price\",\"grand_total\":\"0.00\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$tax = $checkout_data_resource->total_tax_amt;
    	$grand_total = $tip+$large_combo_price+tax;
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2270134\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2271915\",\"size_id\":\"0\"}],\"item_id\":\"277814\",\"size_id\":\"89668\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"$large_combo_price\",\"grand_total\":\"$grand_total\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created and order. ".$order_resource->error);

    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$items = $jersey_mikes_formatted_order['items'];
    	$this->assertEquals(1,sizeof($items));
    	$item = $items[0];
    	$this->assertEquals("GPLU200", $item['vplu']);
    	$this->assertEquals($large_combo_price, $item['item_total']);
    	
		$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
    	$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    	return $return_array;
	}

    /**
     * @depends testItemLevelLargeComboOrder
     */
    
	function testLunchBoxWithDrink($return_array)
    {
    	$user_id = $return_array['user_id'];
    	$user = logTestUserIn($user_id);
		$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	$merchant_id = $return_array['merchant_id'];
    	
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2261182\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258945\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2261198\",\"size_id\":\"89971\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258930\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258931\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258932\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258933\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258934\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258935\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258936\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2261201\",\"size_id\":\"89971\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2261205\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2271921\",\"size_id\":\"0\"}],\"item_id\":\"277705\",\"size_id\":\"89971\"}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"9.99\",\"grand_total\":\"10.77\",\"actual_pickup_time\":\"$pickup_time\"}}";//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103478\",\"items\":[{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"751843\",\"mods\":[{\"mod_sizeprice_id\":\"4063418\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063391\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4066780\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063358\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063359\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063360\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063361\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063362\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063363\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063364\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063437\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063448\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063417\",\"mod_quantity\":1}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"9.99\",\"grand_total\":\"10.77\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$checkout_data_resource = getCheckoutData($json_encoded_data);
    	$order_resource = placeTheOrder($json_encoded_data);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);
    	
    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$this->assertNotNull($jersey_mikes_formatted_order);
    	$this->assertEquals($return_array['lunch_box_with_drink_price'],$jersey_mikes_formatted_order['taxable']);
    	$items = $jersey_mikes_formatted_order['items'];
    	$this->assertEquals(1, sizeof($items));
    	$item = $items[0];
    	$this->assertEquals($return_array['lunch_box_with_drink_price'], $item['price']);
    	
    	$this->assertEquals("GPLU700",$item['vplu']);
    	$this->assertEquals(0,sizeof($item['withs']));
    	$components = $item['components'];
     	$this->assertEquals(4, sizeof($items[0]['components'], $mode));
    	// make sure each component has the withs and the holds	
    	foreach ($items[0]['components'] as $component)
    	{
    		if ($component['vplu'] == 'PLU802')
				$sub_component = $component;
    		else if ($component['vplu'] == 'PLU11702')
				$chip_component = $component;
    		else if ($component['vplu'] == 'PLU11301')
				$cookie_component = $component;	    		
    		else if ($component['vplu'] == 'PLU15707')
				$drink_component = $component;	    		
    	}
    	$this->assertEquals(0, sizeof($sub_component['holds']));
    	$this->assertEquals(0, sizeof($sub_component['withs']));
    	$this->assertEquals(1, sizeof($chip_component['withs']));
    	$this->assertEquals(222,$chip_component['withs'][0]);
    	$this->assertEquals(1, sizeof($cookie_component['withs']));
    	$this->assertEquals(82,$cookie_component['withs'][0]);
    	$this->assertEquals(1, sizeof($drink_component['withs']));
       	$this->assertEquals(86,$drink_component['withs'][0]);
       	
		$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
       	$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    	
    }

    static function loadStoreIdIfNotCurrentlyLoaded($store_id,$menu_id = 102433)
    {
    	$merchant_adapter = new MerchantAdapter($mimetypes);
    	if ($record = $merchant_adapter->getRecord(array("merchant_external_id"=>$store_id))) {
    		return $record['merchant_id'];
    	}
    	$jersey_mikes_importer = new JerseyMikesImporter();
    	if ($menu_id != 102433) {
    		$jersey_mikes_importer->setJerseyMikesMenuId($menu_id);
    	}
    	$result = $jersey_mikes_importer->importFullStore($store_id);
    	
    	$merchant_id = $jersey_mikes_importer->imported_merchant_ids[0];
    	$merchant_resource = $merchant_adapter->getExactResourceFromData(array("merchant_id"=>$merchant_id));
    	$merchant_resource->active = 'Y';
    	$merchant_resource->ordering_on = 'Y';
    	$merchant_resource->save();
    	return $merchant_id;
    }
    
    function loadMenu($merchant_id,$menu_id = 102433)
    {
    	$item_size_adapter = new ItemSizeAdapter($mimetypes);
    	$item_size_data = array();
		$item_size_options[TONIC_FIND_BY_METADATA] = $item_size_data;
		$item_size_options[TONIC_JOIN_STATEMENT] = " JOIN Item ON Item.item_id = Item_Size_Map.item_id JOIN Menu_Type ON Item.menu_type_id = Menu_Type.menu_type_id ";
		$item_size_options[TONIC_FIND_BY_STATIC_METADATA] = " Menu_Type.menu_id = $menu_id AND Menu_Type.logical_delete = 'N' AND Item_Size_Map.merchant_id = $merchant_id";
		//$item_size_options[TONIC_SORT_BY_METADATA] = ' Item_Size_Map.priority DESC ';
		$item_size_resources = Resource::findAll($item_size_adapter,'',$item_size_options);
		$better_item_size_array = array();
		foreach ($item_size_resources as $is_resource)
		{
			$plu = $is_resource->external_id;
			$key = $is_resource->item_id.'-'.$is_resource->size_id;
			$tax_group = $is_resource->tax_group;
			$price = $is_resource->price; 
			$record = array("price"=>$price,"tax_group"=>$tax_group);
			$better_item_size_array[$key] = $record;
			if ($plu) {
				$better_item_size_array[$plu] = $record;
			}
		}
		$this->regular_menu = $better_item_size_array;
    	return $better_item_size_array;
    }   
    
    static function setUpBeforeClass()
    {
    	//$_SERVER['ENVIRONMENT'] = "local";
    	
    	// import the menu
    	$_SERVER['NO_MOCKS'] = true;
    	
    	$menu_adapter = new MenuAdapter($mimetypes);
		if ($menu_adapter->getRecord(array("menu_id"=>102433))){
			; // do nothing
		} else {
			//create menu
			$menu_adapter->importMenu(102433,'test','local',326);

			$sqls[] = "UPDATE Item SET active = 'N', logical_delete = 'Y' WHERE item_id = 276775;";
			$sqls[] = "UPDATE Item SET active = 'N', logical_delete = 'Y' WHERE item_id = 276776;";
			$sqls[] = "UPDATE Item SET active = 'N', logical_delete = 'Y' WHERE item_id = 277820;";
			foreach ($sqls as $sql) {
				$menu_adapter->_query($sql);
			}

            $sqls[] = "UPDATE Brand_Points SET points = 48 WHERE brand_points_id = 1000";
            $sqls[] = "UPDATE Brand_Points SET points = 72 WHERE brand_points_id = 1001";
            $sqls[] = "UPDATE Brand_Points SET points = 144 WHERE brand_points_id = 1002";
            foreach ($sqls as $sql) {
                $menu_adapter->_query($sql);
            }

		}

		//load nebraska delivery store
		$_SERVER['FORCE_JM_DELIVERY_FOR_STORE'] = "33001";
		$delivery_merchant_id = JerseyMikesImporterTest::loadStoreIdIfNotCurrentlyLoaded("33001");
		$ids['delivery_merchant_id'] = $delivery_merchant_id;
		
		//load other nebraska store with delivery menu
		$_SERVER['FORCE_JM_DELIVERY_FOR_STORE'] = "33002";
		$delivery_merchant_id2 = JerseyMikesImporterTest::loadStoreIdIfNotCurrentlyLoaded("33002");
		$ids['delivery_merchant_id2'] = $delivery_merchant_id2;
		
		//now load california store
		$no_tax_on_cold_subs_merchant_id = JerseyMikesImporterTest::loadStoreIdIfNotCurrentlyLoaded("20107");
		$ids['no_tax_on_cold_subs_merchant_id'] = $no_tax_on_cold_subs_merchant_id;
		
		//now load Chicago store
		$chicago_merchant_id = JerseyMikesImporterTest::loadStoreIdIfNotCurrentlyLoaded("27014");
		$ids['chicago_merchant_id'] = $chicago_merchant_id;
		
		//now load regular jersey store
		$regular_store_merchant_id = JerseyMikesImporterTest::loadStoreIdIfNotCurrentlyLoaded("1025");
		$ids['regular_store'] = $regular_store_merchant_id;

		//now load new york city store
		$nyc_store_id = JerseyMikesImporterTest::loadStoreIdIfNotCurrentlyLoaded("34001");
		$ids['nyc_store'] = $nyc_store_id;

		mysqli_query("BEGIN");
		//set up delivery
/*		MerchantMenuMapAdapter::createMerchantMenuMap($delivery_merchant_id, 102433, 'delivery');
    	$merchant_resource = SplickitController::getResourceFromId($delivery_merchant_id, 'Merchant');
    	$merchant_resource->delivery = 'Y';
    	$merchant_resource->save();
    	
    	$merchant_controller = new MerchantController($mt, $u, $r, 5);
    	$merchant_controller->stubOutMerchantDelivery($delivery_merchant_id);
*/    	
    	$mdpd_adapter = new MerchantDeliveryPriceDistanceAdapter($mimetypes);
    	$mdpd_resources = $mdpd_adapter->getMerchantDeliveryPriceDistanceResourcesOrderedByPriceAscending($delivery_merchant_id);
    	$mdpd_resource = $mdpd_resources[0];
    	$mdpd_resource->price = 5.00;
    	$mdpd_resource->distance_up_to = 5;
    	$mdpd_resource->save();

    	//set up delivery for other store
/*    	MerchantMenuMapAdapter::createMerchantMenuMap($delivery_merchant_id2, 102433, 'delivery');
    	$merchant_resource = SplickitController::getResourceFromId($delivery_merchant_id2, 'Merchant');
    	$merchant_resource->delivery = 'Y';
    	$merchant_resource->save();
    	
    	$merchant_controller = new MerchantController($mt, $u, $r, 5);
    	$merchant_controller->stubOutMerchantDelivery($delivery_merchant_id2);
 */   	
    	$mdpd_adapter = new MerchantDeliveryPriceDistanceAdapter($mimetypes);
    	$mdpd_resources = $mdpd_adapter->getMerchantDeliveryPriceDistanceResourcesOrderedByPriceAscending($delivery_merchant_id2);
    	$mdpd_resource = $mdpd_resources[0];
    	$mdpd_resource->price = 0;
    	$mdpd_resource->distance_up_to = 5;
    	$mdpd_resource->save();

    	$blr_adapter = new BrandLoyaltyRulesAdapter($mimetypes);
		$blr_resource = Resource::find($blr_adapter,"1001");
		$blr_resource->max_points_per_order = 88888;
		$blr_resource->save();
		$ids['brand_loyalty_rules_id'] = $blr_resource->brand_loyalty_rules_id;
    	
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;

    }
    
	static function tearDownAfterClass()
    {
    	mysqli_query("ROLLBACK");
    }
     
    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    JerseyMikesImporterTest::main();
}

?>