<?php
$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class JerseyMikesLoyaltyPurchaseTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;
	
	function setUp()
	{
		setContext("com.splickit.jerseymikes");
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
		unset($this->stamp);
    }
    
    function testSubWithAddedBaconUsePoints()
    {
		$merchant_id = $this->ids['regular_store'];
		$user_resource = createNewUser(array("loyalty_number"=>"1234567890","flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		
		$ubpma = new UserBrandPointsMapAdapter($mimetypes);
		$user_brand_points_map_resource = $ubpma->getExactResourceFromData(array("user_id"=>$user['user_id'],"brand_id"=>326));
		// now adjust point values
		$user_brand_points_map_resource->points = 100;
		$user_brand_points_map_resource->save();
		
		$menu = $this->loadMenu($merchant_id);
		
		$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['regular_store'], 'pickup', 'skip hours');
    	
    	$item = $order_data['items'][0];
    	$sandwich_price = $menu[$item['item_id'].'-'.$item['size_id']]['price'];
    	
    	// add the bacon
    	unset($item['mods']);
    	$item['mods'][] = array("mod_quantity"=>1,"modifier_item_id"=>2258942,"size_id"=>$item['size_id']);

    	// now add the points stuff to the item
    	$item['uuid'] = 10;
    	$item['points_used'] = 72;
    	//$item['amt_off'] = $sandwich_price;
    	$item['brand_points_id'] = 1001;
    	$order_data['items'][0] = $item;
		$order_data['total_points_used'] = 72;
 
		$checkout_data_resource = getCheckoutDataWithThrottling($order_data, $merchant_id, $time_stamp);
		
    	$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
    	$this->assertNull($order_resource->error,"should not have gotten an error");
       	$order_id = $order_resource->order_id;
       	$complete_order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);
    	
    	//get bacon price
    	$msma = new ModifierSizeMapAdapter($mimetypes);
    	$record = $msma->getRecord(array("modifier_item_id"=>2258942,"size_id"=>89585,"merchant_id"=>$merchant_id));
		$bacon_price = $record['modifier_price'];

    	$this->assertEquals($bacon_price, $order_resource->order_amt);
    	$this->assertEquals(0.11, $order_resource->total_tax_amt);
   	
    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$this->assertNotNull($jersey_mikes_formatted_order);
    	
    	$item = $jersey_mikes_formatted_order['items'][0];
    	$price = $item['price'];
    	$this->assertEquals($bacon_price,$price);
    	$this->assertEquals($bacon_price,$jersey_mikes_formatted_order['taxable']); 
    	
		$zzcheck = $checkout_data_resource->total_tax_amt;
    	$zzorder = $order_resource->total_tax_amt;
    	$this->assertEquals($checkout_data_resource->total_tax_amt, $order_resource->total_tax_amt);
    }    
    
    function testIsThisWorking()
    {
		setContext("com.splickit.jerseymikes");

		// we dont want to call to inspirepay 
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$user_resource = createNewUser(array("loyalty_number"=>"1234567890","flags"=>"1C20000001"));
		$this->assertNull($user_resource->error);
		$user = logTestUserIn($user_resource->user_id);
		$usc = new UsersessionController($mt, $user, $r, 5);
		$user_session = $usc->getUserSession($user_resource);
    }  
    
    function testGetJMMenuWithPoints()
    {
    	$menu = CompleteMenu::getCompleteMenu(102433,'Y',$this->ids['regular_store']);
		$medium_american_classic_size_price_menu_data = $menu['menu_types'][0]['menu_items'][1]['size_prices'][0];
		$this->assertEquals(72, $medium_american_classic_size_price_menu_data['points']);
		$this->assertEquals(1001, $medium_american_classic_size_price_menu_data['brand_points_id']);
		$giant_tuna_size_price_record = $menu['menu_types'][0]['menu_items'][5]['size_prices'][1];
		$this->assertEquals(144, $giant_tuna_size_price_record['points']);
		$this->assertEquals(1002, $giant_tuna_size_price_record['brand_points_id']);
		$mini_jersey_shore_favorite_size_price_record = $menu['menu_types'][0]['menu_items'][0]['size_prices'][2];
		$this->assertEquals(48, $mini_jersey_shore_favorite_size_price_record['points']);
		$this->assertEquals(1000, $mini_jersey_shore_favorite_size_price_record['brand_points_id']);		
    }
 
    function testPlaceOrderWithPointsJerseyMikes()
    {

		// we dont want to call to inspirepay 
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$user_resource = createNewUser(array("loyalty_number"=>"1234567890","flags"=>"1C20000001"));
		$user = logTestUserIn($user_resource->user_id);
		
		$ubpma = new UserBrandPointsMapAdapter($mimetypes);
		$user_brand_points_map_resource = $ubpma->getExactResourceFromData(array("user_id"=>$user['user_id'],"brand_id"=>326));
		// now adjust point values
		$user_brand_points_map_resource->points = 50;
		$user_brand_points_map_resource->save();
		    	
		$order_adapter = new OrderAdapter($mimetypes);
    	$order = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['regular_store'], 'pickup', 'skip hours');
    	
    	// now add the points stuff to the item
    	$item = $order['items'][0];
    	$item['uuid'] = 10;
    	$item['points_used'] = 72;
    	$item['amt_off'] = $order['sub_total'];
    	$item['brand_points_id'] = 1001;
    	$order['items'][0] = $item;
    	
    	$order['total_points_used'] = 72;
    	$order['lead_time'] = 20;
    	$order['user_id'] = $user['user_id'];
    	$pickup_time = time()+1200;
    	$order['actual_pickup_time'] = $pickup_time;
    	$pickup_string = date('g:ia',$pickup_time);
    	$order_resource = placeOrderFromOrderData($order, $time_stamp);
    	// order should fail;
    	$this->assertNotNull($order_resource->error);
    	$this->assertEquals("We're sorry, but it appears you do not have enough points in your account to place this order.  If you feel you have received this message in error, please contact customer support", $order_resource->error);
    	
    	$user_brand_points_map_resource->points = 100;
    	$user_brand_points_map_resource->save();
		$order_resource = placeOrderFromOrderData($order, $time_stamp);
    	$order_id = $order_resource->order_id;
    	$this->assertTrue($order_resource->order_id > 1000,"should have created a good order id but got: ".$order_resource->error);
    	$this->assertNull($order_resource->error);
    	
    	$this->assertEquals(0.00, $order_resource->order_amt);
    	$this->assertEquals(0.00, $order_resource->total_tax_amt);
    	
    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	$this->assertEquals(28, $jersey_mikes_formatted_order['loyalty_balance']);
    	$this->assertEquals(72, $jersey_mikes_formatted_order['loyalty_points_used']);
    	$item = $jersey_mikes_formatted_order['items'][0];
    	$this->assertEquals('yes', $item['use_points']);
    }  

    static function loadStoreIdIfNotCurrentlyLoaded($store_id,$menu_id = 102433)
    {
    	$merchant_adapter = new MerchantAdapter($mimetypes);
    	if ($record = $merchant_adapter->getRecord(array("merchant_external_id"=>$store_id))) {
    		return $record['merchant_id'];
    	}
    	$jersey_mikes_importer = new JerseyMikesImporter();
    	if ($menu_id != 102433) {
    		$jersey_mikes_importer->setJerseyMikesMenuId($menu_id);
    	}
    	$result = $jersey_mikes_importer->importFullStore($store_id);
    	
    	$merchant_id = $jersey_mikes_importer->imported_merchant_ids[0];
    	return $merchant_id;
    }
    
    function loadMenu($merchant_id,$menu_id = 102433)
    {
    	$item_size_adapter = new ItemSizeAdapter($mimetypes);
    	$item_size_data = array();
		$item_size_options[TONIC_FIND_BY_METADATA] = $item_size_data;
		$item_size_options[TONIC_JOIN_STATEMENT] = " JOIN Item ON Item.item_id = Item_Size_Map.item_id JOIN Menu_Type ON Item.menu_type_id = Menu_Type.menu_type_id ";
		$item_size_options[TONIC_FIND_BY_STATIC_METADATA] = " Menu_Type.menu_id = $menu_id AND Menu_Type.logical_delete = 'N' AND Item_Size_Map.merchant_id = $merchant_id";
		//$item_size_options[TONIC_SORT_BY_METADATA] = ' Item_Size_Map.priority DESC ';
		$item_size_resources = Resource::findAll($item_size_adapter,'',$item_size_options);
		$better_item_size_array = array();
		foreach ($item_size_resources as $is_resource)
		{
			$plu = $is_resource->external_id;
			$key = $is_resource->item_id.'-'.$is_resource->size_id;
			$tax_group = $is_resource->tax_group;
			$price = $is_resource->price; 
			$record = array("price"=>$price,"tax_group"=>$tax_group);
			$better_item_size_array[$key] = $record;
			if ($plu) {
				$better_item_size_array[$plu] = $record;
			}
		}
		$this->regular_menu = $better_item_size_array;
    	return $better_item_size_array;
    }   
    
    static function setUpBeforeClass()
    {
    	//$_SERVER['ENVIRONMENT'] = "local";
    	
    	// import the menu
    	$_SERVER['NO_MOCKS'] = true;
    	
    	$menu_adapter = new MenuAdapter($mimetypes);
		if ($menu_adapter->getRecord(array("menu_id"=>102433))){
			; // do nothing
		} else {
			//create menu
			// why do i have to do this twice. please help :(
	    	$menu_adapter->importMenu(102433,'prod','local',326);
		}
		
		//correct for their system being offline
		$sqls[] = "UPDATE Brand_Points SET points = 48 WHERE brand_points_id = 1000";
		$sqls[] = "UPDATE Brand_Points SET points = 72 WHERE brand_points_id = 1001";
		$sqls[] = "UPDATE Brand_Points SET points = 144 WHERE brand_points_id = 1002";
		foreach ($sqls as $sql) {
			$menu_adapter->_query($sql);
		}

		//now load regular jersey store
		$regular_store_merchant_id = JerseyMikesLoyaltyPurchaseTest::loadStoreIdIfNotCurrentlyLoaded("1025");
		$ids['regular_store'] = $regular_store_merchant_id;
		mysqli_query("BEGIN");
		
		//set up delivery    	
    	
    	$blr_adapter = new BrandLoyaltyRulesAdapter($mimetypes);
		$blr_resource = Resource::find($blr_adapter,"1001");
		$blr_resource->max_points_per_order = 88888;
		$blr_resource->save();
		$ids['brand_loyalty_rules_id'] = $blr_resource->brand_loyalty_rules_id;
    	
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;

    }
    
	static function tearDownAfterClass()
    {
    	mysqli_query("ROLLBACK");
    }
     
    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    JerseyMikesLoyaltyPurchaseTest::main();
}

?>