<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class JerseyMikesLoyaltyTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		setContext('com.splickit.jerseymikes');
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
		loadProperties();
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }
        
    function testUserAdapterCreateSplickitAccountWithBadLoyaltyNumber()
    {
    	$_SERVER['REQUEST_METHOD'] = 'POST';
    	$loyalty_number = '1239087654';
    	$user_resource = createNewUser(array("loyalty_number"=>$loyalty_number,"loyalty_phone_number"=>$loyalty_number));
    	$this->assertNotNull($user_resource->error);
    	$this->assertEquals('Sorry but the loyalty number you entered, 1239087654, is not valid', $user_resource->error);
    	$this->assertFalse($user_resource->_exists);	
    }
    
    function testUserAdapterUpdateSplickitAccountLinkWithGoodLoyaltyNumber()
    {
    	$_SERVER['REQUEST_METHOD'] = 'POST';
    	$loyalty_number = '1234567890';
    	$user_resource = createNewUser();
    	setSessionProperty('is_a_user_update', 'true');
    	$user_resource->loyalty_number = $loyalty_number;
    	$user_resource->loyalty_phone_number = $loyalty_number;
    	$user_resource->save();
    	
    	// should have created a user brand loyalty record
    	$user_brand_points_map_adapter = new UserBrandPointsMapAdapter($mimetypes);
    	$user_brand_loyalty_record = $user_brand_points_map_adapter->getRecord(array("user_id"=>$user_resource->user_id,"brand_id"=>326));
    	$this->assertNotNull($user_brand_loyalty_record);
    	$this->assertEquals($loyalty_number,$user_brand_loyalty_record['loyalty_number']);
    	$this->assertNotEquals('0000-00-00 00:00:00',$user_brand_loyalty_record['created']);
    	
    	// now check usersession fields
    	$user = logTestUserIn($user_resource->user_id);
		$usc = new UsersessionController($mt, $user, $r, 5);
		$user_session = $usc->getUserSession($user_resource);
		$this->assertNotNull($user_session->brand_loyalty);
		$brand_loyalty = $user_session->brand_loyalty;
		$this->assertArrayHasKey('card_number', $brand_loyalty);
		$this->assertArrayNotHasKey('error', $brand_loyalty);
    	$this->assertArrayNotHasKey('status', $brand_loyalty);
    	$this->assertArrayNotHasKey('error_no', $brand_loyalty);
    	$this->assertArrayNotHasKey('http_code', $brand_loyalty);
    	
    }

    function testUserAdapterUpdateSplickitAccountLinkWithBADBADBADLoyaltyNumber()
    {
    	$_SERVER['REQUEST_METHOD'] = 'POST';
    	$loyalty_number = '1239087654';
    	$user_resource = createNewUser();
    	setSessionProperty('is_a_user_update', 'true');
    	$user_resource->loyalty_number = $loyalty_number;
    	$user_resource->loyalty_phone_number = $loyalty_number;
    	$user_resource->save();
    	
    	$this->assertEquals("Sorry but the loyalty number you entered, $loyalty_number, is not valid", $user_resource->error);	
    }
    
    function testUserAdapterJoinShorePoints()
    {
    	$_SERVER['REQUEST_METHOD'] = 'POST';
    	$loyalty_number = '1'.rand(111111111, 999999999);
    	$user_resource = createNewUser();
    	setSessionProperty('is_a_user_update', 'true');
    	$user_resource->create_loyalty_account = 'yes';
    	$user_resource->loyalty_number = $loyalty_number;
    	$user_resource->loyalty_phone_number = $loyalty_number;
    	$user_resource->save();
    	
    	// should have created a user brand loyalty record
    	$user_brand_points_map_adapter = new UserBrandPointsMapAdapter($mimetypes);
    	$user_brand_loyalty_record = $user_brand_points_map_adapter->getRecord(array("user_id"=>$user_resource->user_id,"brand_id"=>326));
    	$this->assertNotNull($user_brand_loyalty_record);
    	$this->assertEquals($loyalty_number,$user_brand_loyalty_record['loyalty_number']);
    }
    
    function testUserAdapterCreateSplickitAccountWithExistingLoyaltyNumber()
    {
    	$_SERVER['REQUEST_METHOD'] = 'POST';
    	$loyalty_number = '1234567890';
    	$user_resource = createNewUser(array("loyalty_number"=>$loyalty_number,"loyalty_phone_number"=>$loyalty_number));
    	$this->assertNull($user_resource->error);
    	$this->assertTrue($user_resource->_exists);
    	
    	// should have created a user brand loyalty record
    	$user_brand_points_map_adapter = new UserBrandPointsMapAdapter($mimetypes);
    	$user_brand_loyalty_record = $user_brand_points_map_adapter->getRecord(array("user_id"=>$user_resource->user_id,"brand_id"=>326));
    	$this->assertNotNull($user_brand_loyalty_record);
    	$this->assertEquals($loyalty_number,$user_brand_loyalty_record['loyalty_number']);
    	return $user_resource;
    }
//*/    

    /**
     * @depends  testUserAdapterCreateSplickitAccountWithExistingLoyaltyNumber
     */
    function testJMLoyaltySessionData($user_resource)
    {
    	$user_brand_points_map_adapter = new UserBrandPointsMapAdapter($mimetypes);
    	$user_brand_loyalty_record = $user_brand_points_map_adapter->getRecord(array("user_id"=>$user_resource->user_id,"brand_id"=>326));
    	$saved_points = $user_brand_loyalty_record['points'];
    	$loyalty_number = $user_brand_loyalty_record['loyalty_number'];
    	
    	$user = logTestUserIn($user_resource->user_id);
    	$user_session_controller = new UsersessionController($mt, $user, $r, 5);
    	$user_session = $user_session_controller->getUserSession($user_resource);
    	$this->assertNotNull($user_session->brand_loyalty);
    	$brand_loyalty = $user_session->brand_loyalty;
    	$this->assertEquals($saved_points, $brand_loyalty['loyalty_points']);
    	$this->assertEquals('3C154325E7544F', $brand_loyalty['barcode_number']);
    	//$this->assertNull($brand_loyalty['qrcode_number']);
    	$this->assertTrue(is_array($brand_loyalty['loyalty_transactions']));
    }
    
    function testJMLoyaltyControllerToSendCorrectErrorOnBadPhoneNumberForCreatedShorePointsAccount()
    {
    	setContext('com.splickit.jerseymikes');
    	$loyalty_number = "14567";
    	$user_resource = createNewUser();
	   	$user = logTestUserIn($user_resource->user_id);
    	$loyalty_controller = new JerseymikesLoyaltyController($mimetypes, $user, $request);
    	$data = array("create_loyalty_account"=>1,"loyalty_phone_number"=>$loyalty_number);
    	$loyalty_controller->setLoyaltyData($data);
	   	$result = $loyalty_controller->createOrLinkAccount($user_resource->user_id);
	   	//$this->assertFalse($result);
	   	$sre = $loyalty_controller->getServiceResponseError();
	   	$this->assertEquals($sre, 'The number provided was not a valid phone number.');

    }
    
    function testJMLoyaltyControllerRecordLoyaltyFails()
    {
    	$user_resource = createNewUser();
    	$user = logTestUserIn($user_resource->user_id);
    	$loyalty_controller = new JerseymikesLoyaltyController($mimetypes, $user, $request);
    	$blf_save_result = $loyalty_controller->recordLoyaltyFail();
    	$this->assertTrue($blf_save_result);
    	$blf_adapter = new BrandLoyaltyFailsAdapter($mimetypes);
    	$brand_loyalty_fails_resource = $blf_adapter->getExactResourceFromData(array("device_id"=>$user['device_id'],"brand_id"=>326));
    	$this->assertNotNull($brand_loyalty_fails_resource);
    	$this->assertEquals(1, $brand_loyalty_fails_resource->failed_attempts);
    	$blf_save_result = $loyalty_controller->recordLoyaltyFail();
    	$blf_save_result = $loyalty_controller->recordLoyaltyFail();
    	$this->assertEquals($loyalty_controller->too_many_failed_loyalty_attempts_message, $loyalty_controller->service_response['error']);
    }
    
	function testJMLoyaltyControllerDoesUserHaveTooManyFailedAttempts()
	{
    	$user_resource = createNewUser();
    	$user = logTestUserIn($user_resource->user_id);
    	$loyalty_controller = new JerseymikesLoyaltyController($mimetypes, $user, $request);
    	$this->assertFalse($loyalty_controller->doesUserHaveTooManyFailedAttempts());
    	$loyalty_controller->recordLoyaltyFail();
    	
    	$blf_adapter = new BrandLoyaltyFailsAdapter($mimetypes);
    	$brand_loyalty_fails_resource = $blf_adapter->getExactResourceFromData(array("device_id"=>$user['device_id'],"brand_id"=>326));
		$this->assertNotNull($brand_loyalty_fails_resource);
    	$this->assertEquals(1, $brand_loyalty_fails_resource->failed_attempts);
    	$this->assertFalse($loyalty_controller->doesUserHaveTooManyFailedAttempts());
    	$brand_loyalty_fails_resource->failed_attempts = 3;
    	$brand_loyalty_fails_resource->save();
    	$this->assertTrue($loyalty_controller->doesUserHaveTooManyFailedAttempts());
	}    
// */	    
    function testUserAdapterFailLogicOnBadLoyaltyNumberOnLinkAccount()
    {
    	//should be line 102
    	$bad_loyalty_number = '1239087654';
    	$_SERVER['REQUEST_METHOD'] = 'POST';
    	$user_resource = createNewUser();
    	$user_id = $user_resource->user_id;
    	$user = logTestUserIn($user_id);
    	
    	setSessionProperty('is_a_user_update', 'true');
    	$user_resource->loyalty_number = $bad_loyalty_number;
    	$user_resource->loyalty_phone_number = $bad_loyalty_number;
    	$result = $user_resource->save();
    	//$this->assertFalse($result);
    	$this->assertNotNull($user_resource->error);
    	$this->assertEquals("Sorry but the loyalty number you entered, $bad_loyalty_number, is not valid", $user_resource->error);
    	unset($user_resource);
    	$user_resource = Resource::find(new UserAdapter($mimetypes), "$user_id", $options);
    	$user_resource->loyalty_number = $bad_loyalty_number;
    	$user_resource->loyalty_phone_number = $bad_loyalty_number;
    	$result = $user_resource->save();
    	unset($user_resource);
    	$user_resource = Resource::find(new UserAdapter($mimetypes), "$user_id", $options);
    	$user_resource->loyalty_number = $bad_loyalty_number;
    	$user_resource->loyalty_phone_number = $bad_loyalty_number;
    	$result = $user_resource->save();
    	unset($user_resource);

    	$user_resource = Resource::find(new UserAdapter($mimetypes), "$user_id", $options);
    	$user_resource->loyalty_number = $bad_loyalty_number;
    	$user_resource->loyalty_phone_number = $bad_loyalty_number;
    	$result = $user_resource->save();
    	$jmlc = new JerseymikesLoyaltyController($mimetypes, $user, $request);
    	$this->assertEquals($jmlc->too_many_failed_loyalty_attempts_message, $user_resource->error);	
    }

    static function setUpBeforeClass()
    {
    	$_SERVER['request_time1'] = microtime(true);
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	ini_set('max_execution_time',300);
    	mysqli_query("BEGIN");
    	$sql = "INSERT INTO Property (name,value) values ('jm_username','splickit')";
    	mysqli_query($sql);
    	$sql = "INSERT INTO Property (name,value) values ('jm_password','Spelunk$5marble')";
    	mysqli_query($sql);
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	mysqli_query("ROLLBACK");
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['argv'][1]) && $_SERVER['argv'][1] == 'use_main'  && !defined('PHPUnit_MAIN_METHOD')) {
    JerseyMikesLoyaltyTest::main();
}

?>