<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class JerseyMikesMarketerTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
		setSessionProperty('do_not_call_out_to_aws', 'true');
		setContext('com.splickit.jerseymikes');
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
		unset($this->stamp);
    }
		
	function testCreateUserWithToken()
    {
    	$user_data = createNewUserDataFields();
    	$json_user_data = json_encode($user_data);
    	$encrypted_user_data = SplickitCrypter::doEncryption($json_user_data,'mikesmarketer');
    	$create_user_data['create_user_token'] = $encrypted_user_data;
    	
    	$_SERVER['CONTENT_TYPE'] = "application/json";
    	$request = new Request();
    	$request->body = json_encode($create_user_data);    	
    	
    	$admin_user_resource = Resource::find(new UserAdapter($mimetypes),"12");
    	$user = $admin_user_resource->getDataFieldsReally();
    	
    	$mt = getMimetypes();
    	$user_controller = new UserController($mt, $user, $request,5);
    	$user_resource = $user_controller->createUser();
    	
    	$this->assertNull($user_resource->error);
    	$this->assertTrue($user_resource->user_id > 1000);
    	$this->assertEquals(72, $user_resource->skin_id);
    	
    	$user_brand_points_map_adapter = new UserBrandPointsMapAdapter($mimetypes);
    	$record = $user_brand_points_map_adapter->getRecord(array("user_id"=>$user_resource->user_id,"brand_id"=>326));
    	$this->assertNull($record);
    	
    }
    
	function testCreateUserWithTokenWithLoyalty()
    {
    	$user_data = createNewUserDataFields();
    	$user_data['loyalty_number'] = "1234567890";
    	$json_user_data = json_encode($user_data);
    	$encrypted_user_data = SplickitCrypter::doEncryption($json_user_data,'mikesmarketer');
    	$create_user_data['create_user_token'] = $encrypted_user_data;
    	
    	$_SERVER['CONTENT_TYPE'] = "application/json";
    	$request = new Request();
    	$request->body = json_encode($create_user_data);    	
    	
    	$admin_user_resource = Resource::find(new UserAdapter($mimetypes),"12");
    	$user = $admin_user_resource->getDataFieldsReally();
    	
    	$mt = getMimetypes();
    	$user_controller = new UserController($mt, $user, $request,5);
    	$user_resource = $user_controller->createUser();
    	
    	$this->assertNull($user_resource->error);
    	$this->assertTrue($user_resource->user_id > 1000);
    	$this->assertEquals(72, $user_resource->skin_id);
    	
    	$user_brand_points_map_adapter = new UserBrandPointsMapAdapter($mimetypes);
    	$record = $user_brand_points_map_adapter->getRecord(array("user_id"=>$user_resource->user_id,"brand_id"=>326));
    	$this->assertNotNull($record);
    	$this->assertEquals("1234567890", $record['loyalty_number']);
    	return $user_resource->uuid;
    }
    
	/**
	 * @depends testCreateUserWithTokenWithLoyalty
	 */
    function testMikesMarketerOrderWithAuth($uuid)
    {
    	$user_adapter = new UserAdapter($mimetypes);
    	$user_resource_to_modify = $user_adapter->getExactResourceFromData(array("uuid"=>$uuid));
    	
    	$user_resource_to_modify->flags = '1C20000001';
    	$user_resource_to_modify->save();
    	
    	$merchant_id = $this->ids['regular_store'];
    	
    	$user_id = $user_resource_to_modify->user_id;
    	$tip = rand(100, 1000)/100;
    	$pickup_time = getTodayTwelveNoonTimeStampDenver()+900;
    	//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103478\",\"items\":[{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"751787\",\"mods\":[{\"mod_sizeprice_id\":\"4063411\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063391\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063358\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063359\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063360\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063361\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063362\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063363\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063364\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063417\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063450\",\"mod_quantity\":1}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"7.99\",\"grand_total\":\"8.61\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$json_encoded_data = "{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2258930\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258931\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258932\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258933\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258934\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258935\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258936\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2271915\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2270133\",\"size_id\":\"0\"}],\"item_id\":\"278172\",\"size_id\":\"89585\"}],\"note\":\"Here is the note.\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"7.99\",\"grand_total\":\"8.61\",\"actual_pickup_time\":\"$pickup_time\"}";
		$order_data = json_decode($json_encoded_data,true);
    	
		$mm_data['username'] = "$user_id";
    	$mm_data['valid_until_time_stamp'] = time()+60;
    	$auth_data['mikes_marketer_id'] = "12345";
    	$auth_data['some_key'] = "some_value";
    	//$auth_data['promo_code'] = "promo_code";
    	$mm_data['auth_data'] = $auth_data;
    	$mm_json = json_encode($mm_data);
    	$mm_json_encrypted = SplickitCrypter::doEncryption($mm_json,'mikesmarketer');
    	
    	$headers['Content-Type'] = "application/json";
    	$headers['Content-Length'] = strlen($json_encoded_data);
    	$headers['Auth_token'] = $mm_json_encrypted;
    	
    	$url = "http://localhost/app2/phone/placeorder?log_level=6";
    	$request = new Request();
    	$request->url = $url;
    	
    	$request->body = $json_encoded_data;
    	$request->_parseRequestBody();
    	$request_all = array_merge($request->data,$headers);
    	
    	$user_resource = LoginAdapter::staticDoAuthorizeWithSpecialUserValidation("mikesmarketer", "Z2xTtj5w1qz", $request_all);
    	$this->assertEquals($user_resource_to_modify->email, $user_resource->email);
    	$this->assertNotNull($user_resource->auth_data);
    	$user = logTestUserResourceIn($user_resource);
    	$this->assertNotNull($user['auth_data']);

    	$order_resource = placeOrderFromOrderData($order_data, getTodayTwelveNoonTimeStampDenver());
    	$this->assertNull($order_resource->error);
    	$order_id = $order_resource->order_id;
		$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);

    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, new Request(),5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	
    	$catering_crm_data = $jersey_mikes_formatted_order['catering_crm'];
    	$this->assertEquals("12345", $catering_crm_data['mikes_marketer_id']);
    	$this->assertEquals("some_value", $catering_crm_data['some_key']);
    	$this->assertEquals("Here is the note. ", $jersey_mikes_formatted_order['note']);
    	
    	$complete_menu = new CompleteMenu(102433);
    	$complete_menu->merchant_menu_type = 'Pickup';
    	$complete_menu->getAllMenuItemSizePrices(102433, 'Y', $merchant_id, $mimetypes, $ltos);
    	$price_list = $complete_menu->getFullPriceListByExternalId();
    	$item1_price = $price_list['PLU502']['price'];
    	$item2_price = $price_list['GPLU100:GPLU200']['price'];
    	
    	$rate = TaxAdapter::staticGetTotalBaseTaxRate($merchant_id);
    	
    	$tax = number_format(($item1_price+$item2_price)*($rate/100),2);
    	$this->assertEquals($item1_price+$item2_price, $jersey_mikes_formatted_order['taxable']);
    	$this->assertEquals($tax, $jersey_mikes_formatted_order['tax']);
    	$this->assertEquals(2,sizeof($jersey_mikes_formatted_order['items']));
    	$this->assertEquals('GPLU100',$jersey_mikes_formatted_order['items'][0]['vplu']);
    	$this->assertEquals(0, sizeof($jersey_mikes_formatted_order['items'][0]['withs'], $mode));
    	$this->assertEquals(0, sizeof($jersey_mikes_formatted_order['items'][0]['holds'], $mode));
    	$this->assertEquals(2, sizeof($jersey_mikes_formatted_order['items'][0]['components'], $mode));
    	$this->assertEquals($item2_price, $jersey_mikes_formatted_order['items'][0]['price']);
    	$components = $jersey_mikes_formatted_order['items'][0]['components'];
    	$component_plu_string = $components[0]['vplu'].'-'.$components[1]['vplu'];
    	$this->assertContains('PLU11702', $component_plu_string);
    	$this->assertContains('PLU8702', $component_plu_string);
    	$sandwich_item = $jersey_mikes_formatted_order['items'][1];
    	$this->assertEquals('PLU502',$sandwich_item['vplu']);
    	$this->assertEquals($item1_price,$sandwich_item['price']);
    }	

    function testCreateUserWithTokenFromDispatchBadEmail()
	{
		
		$user_data = createNewUserDataFields();
		$user_data['email'] = 'myveryfavoritehorse';
    	$json_user_data = json_encode($user_data);
    	$encrypted_user_data = SplickitCrypter::doEncryption($json_user_data,'mikesmarketer');
    	$create_user_data['create_user_token'] = $encrypted_user_data;
    	$request_json = json_encode($create_user_data);

    	$url = "http://localhost/app2/phone/users/mikesmarketercreate?log_level=5";
    	$curl = curl_init($url);
    	curl_setopt($curl, CURLOPT_USERPWD, "mikesmarketer:Z2xTtj5w1qz");
    	curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
    	curl_setopt($curl, CURLOPT_VERBOSE,0);
		$headers = array('Content-Type: application/json','Content-Length: ' . strlen($request_json),'X_SPLICKIT_CLIENT_ID:com.splickit.jerseymikes');
		curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
		curl_setopt($curl, CURLOPT_POST, 1);
		curl_setopt($curl, CURLOPT_POSTFIELDS,$request_json);
		$result = curl_exec($curl);
		curl_close($curl);
		$create_user_response = json_decode($result,true);
		$this->assertNotNull($create_user_response['ERROR'],'Should have gotten an error');
		$this->assertEquals('Sorry but the email address you entered is not valid', $create_user_response['ERROR']);
		$this->assertEquals('500',$create_user_response['ERROR_CODE']);
	}

	function testCreateUserWithTokenFromDispatch()
	{
		
		$user_data = createNewUserDataFields();
    	$json_user_data = json_encode($user_data);
    	$encrypted_user_data = SplickitCrypter::doEncryption($json_user_data,'mikesmarketer');
    	$create_user_data['create_user_token'] = $encrypted_user_data;
    	$request_json = json_encode($create_user_data);

    	$url = "http://localhost/app2/phone/users/mikesmarketercreate?log_level=6";
    	$curl = curl_init($url);
    	curl_setopt($curl, CURLOPT_USERPWD, "mikesmarketer:Z2xTtj5w1qz");
    	curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
    	curl_setopt($curl, CURLOPT_VERBOSE,0);
		$headers = array('Content-Type: application/json','Content-Length: ' . strlen($request_json),'X_SPLICKIT_CLIENT_ID:com.splickit.jerseymikes');
		curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
		curl_setopt($curl, CURLOPT_POST, 1);
		curl_setopt($curl, CURLOPT_POSTFIELDS,$request_json);
		$result = curl_exec($curl);
		curl_close($curl);
		$create_user_response = json_decode($result,true);
		$this->assertNull($create_user_response['error']);
		$this->assertEquals('success', $create_user_response['status']);
		$uuid = $create_user_response['uuid'];
		$this->assertNotNull($uuid);	
		return $uuid;
	}
	
/*	function testCreateUserWithTokenFromDispatchTweb()
	{
		
		$user_data = createNewUserDataFields();
		$json_user_data = json_encode($user_data);
    	$encrypted_user_data = SplickitCrypter::doEncryption($json_user_data,'mikesmarketer');
    	$create_user_data['create_user_token'] = $encrypted_user_data;
    	$request_json = json_encode($create_user_data);

    	$url = "https://test.splickit.com/app2/phone/users/mikesmarketercreate?log_level=5";
    	$curl = curl_init($url);
    	curl_setopt($curl, CURLOPT_USERPWD, "mikesmarketer:Z2xTtj5w1qz");
    	curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
    	curl_setopt($curl, CURLOPT_VERBOSE,0);
		$headers = array('Content-Type: application/json','Content-Length: ' . strlen($request_json));
		curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
		curl_setopt($curl, CURLOPT_POST, 1);
		curl_setopt($curl, CURLOPT_POSTFIELDS,$request_json);
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
		$result = curl_exec($curl);
		curl_close($curl);
		$create_user_response = json_decode($result,true);
		$this->assertNull($create_user_response['error']);
		$this->assertEquals('success', $create_user_response['status']);
		$uuid = $create_user_response['uuid'];
		$this->assertNotNull($uuid);	
		return $uuid;
	}
*/	
/*	function testCreateUserWithTokenFromDispatchBadLoyaltyNumber()
	{
		
		$user_data = createNewUserDataFields();
		$user_data['loyalty_number'] = "575638290348";
    	$json_user_data = json_encode($user_data);
    	$encrypted_user_data = SplickitCrypter::doEncryption($json_user_data);
    	$create_user_data['create_user_token'] = $encrypted_user_data;
    	$request_json = json_encode($create_user_data);

    	$url = "http://localhost:8888/app2/phone/users/mikesmarketercreate?log_level=5";
    	$curl = curl_init($url);
    	curl_setopt($curl, CURLOPT_USERPWD, "mikesmarketer:welcome");
    	curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
    	curl_setopt($curl, CURLOPT_VERBOSE,1);
		$headers = array('Content-Type: application/json','Content-Length: ' . strlen($request_json));
		curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
		curl_setopt($curl, CURLOPT_POST, 1);
		curl_setopt($curl, CURLOPT_POSTFIELDS,$request_json);
		$result = curl_exec($curl);
		curl_close($curl);
		$create_user_response = json_decode($result,true);
		$this->assertNotNull($create_user_response['error']);
		//$this->assertTrue($create_user_response['user_id'] < 1000);	
	}
*/	
	/**
	 * @depends testCreateUserWithTokenFromDispatch
	 */
    function testMikesMarketerOrderFromDispatch($uuid)
    {
    	$user_adapter = new UserAdapter($mimetypes);
    	$user_resource = $user_adapter->getExactResourceFromData(array("uuid"=>$uuid));
    	$this->assertTrue(is_a($user_resource, 'Resource'));
		$_SERVER['HTTP_NO_CC_CALL'] = 'false';
    	$user = logTestUserResourceIn($user_resource);
    	addDummyCCDataToResource($user_resource);
    	$splickit_vault = new SplickitVault();
    	$this->assertTrue($splickit_vault->save($user_resource));
    	
    	$user_resource->flags = '1C20000001';
    	$user_resource->save();
    	
    	$merchant_id = $this->ids['regular_store'];
    	
    	$user_id = $user_resource->user_id;
    	$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103478\",\"items\":[{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"751787\",\"mods\":[{\"mod_sizeprice_id\":\"4063411\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063391\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063358\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063359\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063360\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063361\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063362\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063363\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063364\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063417\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063450\",\"mod_quantity\":1}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"7.99\",\"grand_total\":\"8.61\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2258930\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258931\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258932\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258933\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258934\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258935\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2258936\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2271915\",\"size_id\":\"0\"},{\"mod_quantity\":1,\"modifier_item_id\":\"2270133\",\"size_id\":\"0\"}],\"item_id\":\"278172\",\"size_id\":\"89585\"}],\"note\":\"skip hoursThis is the note that i'm writing.\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"7.99\",\"grand_total\":\"8.61\",\"actual_pickup_time\":\"$pickup_time\"}}";

		$mm_data['username'] = "$user_id";
    	$mm_data['valid_until_time_stamp'] = time()+60;
    	$auth_data['mikes_marketer_id'] = "12345";
    	$auth_data['some_key'] = "some_value";
    	//$auth_data['promo_code'] = "promo_code";
    	$mm_data['auth_data'] = $auth_data;
    	$mm_json = json_encode($mm_data);
    	$mm_json_encrypted = SplickitCrypter::doEncryption($mm_json,'mikesmarketer');
    	
    	$url = "http://localhost/app2/phone/placeorder?log_level=6";
    	$curl = curl_init($url);
    	curl_setopt($curl, CURLOPT_USERPWD, "mikesmarketer:Z2xTtj5w1qz"); 
    	curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($curl, CURLOPT_VERBOSE,0);
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
		//$headers = array('Content-Type: application/json','Content-Length: ' . strlen($json_encoded_data));
		curl_setopt($curl, CURLOPT_HTTPHEADER, array("Content-Type: application/json","Content-Length: " . strlen($json_encoded_data),"X_SPLICKIT_CLIENT_ID:com.splickit.jerseymikes","X_SPLICKIT_CLIENT_DEVICE:unit_testing","X_SPLICKIT_CLIENT:LoginTest","Auth_token:$mm_json_encrypted"));
    	curl_setopt($curl, CURLOPT_POST, 1);
		curl_setopt($curl, CURLOPT_POSTFIELDS,$json_encoded_data);
    	
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);	
		$result = curl_exec($curl);
		curl_close($curl);
		$r = json_decode($result,true);
		$this->assertNull($r['error']);
		$order_id = $r['order_id'];
		
		$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);

    	$order = CompleteOrder::staticGetCompleteOrder($order_id, $mimetypes);
    	$jersey_mikes_controller = new JerseyMikesController($mt, $u, new Request(),5);
    	$jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
    	
    	$jm_order_json = json_encode($jersey_mikes_formatted_order);
    	myerror_log("$jm_order_json");
    	
    	$catering_crm_data = $jersey_mikes_formatted_order['catering_crm'];
    	$this->assertEquals("12345", $catering_crm_data['mikes_marketer_id']);
    	$this->assertEquals("some_value", $catering_crm_data['some_key']);
    	$this->assertEquals("skip hoursThis is the note that i'm writing. skip hours ", $jersey_mikes_formatted_order['note']);
    	
    	$complete_menu = new CompleteMenu(102433);
    	$complete_menu->getAllMenuItemSizePrices(102433, 'Y', $merchant_id, $mimetypes, $ltos);
    	$price_list = $complete_menu->getFullPriceListByExternalId();
    	$item1_price = $price_list['PLU502']['price'];
    	$item2_price = $price_list['GPLU100:GPLU200']['price'];
    	
    	$rate = TaxAdapter::staticGetTotalBaseTaxRate($merchant_id);
    	$tax = number_format(($item1_price+$item2_price)*($rate/100),2);
    	$this->assertEquals($item1_price+$item2_price, $jersey_mikes_formatted_order['taxable']);
    	$this->assertEquals($tax, $jersey_mikes_formatted_order['tax']);
    	$this->assertEquals(2,sizeof($jersey_mikes_formatted_order['items']));
    	$this->assertEquals('GPLU100',$jersey_mikes_formatted_order['items'][0]['vplu']);
    	$this->assertEquals(0, sizeof($jersey_mikes_formatted_order['items'][0]['withs'], $mode));
    	$this->assertEquals(0, sizeof($jersey_mikes_formatted_order['items'][0]['holds'], $mode));
    	$this->assertEquals(2, sizeof($jersey_mikes_formatted_order['items'][0]['components'], $mode));
    	$this->assertEquals($item2_price, $jersey_mikes_formatted_order['items'][0]['price']);
    	$components = $jersey_mikes_formatted_order['items'][0]['components'];
    	$component_plu_string = $components[0]['vplu'].'-'.$components[1]['vplu'];
    	$this->assertContains('PLU11702', $component_plu_string);
    	$this->assertContains('PLU8702', $component_plu_string);
    	$sandwich_item = $jersey_mikes_formatted_order['items'][1];
    	$this->assertEquals('PLU502',$sandwich_item['vplu']);
    	$this->assertEquals($item1_price,$sandwich_item['price']);
    	    	
		return $uuid;
    }

    function testMikesMarketerOrderFromDispatchTweb()
    {
    	
    	$user_id = 511724;
    	$user_adapter = new UserAdapter($mimetypes);
    	$user_resource = $user_adapter->getExactResourceFromData(array("user_id"=>$user_id));
    	
    	$merchant_id = $this->ids['regular_store'];
    	
    	$tip = rand(100, 1000)/100;
    	$pickup_time = time()+900;
    	//$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"103478\",\"items\":[{\"quantity\":1,\"note\":\"\",\"sizeprice_id\":\"751787\",\"mods\":[{\"mod_sizeprice_id\":\"4063411\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063391\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063358\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063359\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063360\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063361\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063362\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063363\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063364\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063417\",\"mod_quantity\":1},{\"mod_sizeprice_id\":\"4063450\",\"mod_quantity\":1}]}],\"note\":\"skip hours\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"20000\",\"loyalty_no\":\"\",\"sub_total\":\"7.99\",\"grand_total\":\"8.61\",\"actual_pickup_time\":\"$pickup_time\"}}";
    	$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2258930\",\"size_id\":\"0\"}],\"item_id\":\"278172\",\"size_id\":\"89585\"}],\"note\":\"This is the note that i'm writing.\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"7.99\",\"grand_total\":\"8.61\"}}";

		$mm_data['username'] = "$user_id";
    	$mm_data['valid_until_time_stamp'] = time()+60;
    	$auth_data['mikes_marketer_id'] = "12345";
    	$auth_data['some_key'] = "some_value";
    	$auth_data['promo_code'] = "CRM10";
    	$mm_data['auth_data'] = $auth_data;
    	$mm_json = json_encode($mm_data);
    	$mm_json_encrypted = SplickitCrypter::doEncryption($mm_json,'mikesmarketer');
    	
    	$url = "https://75.101.136.231/app2/phone/getcheckoutdata?log_level=6";
    	$curl = curl_init($url);
    	curl_setopt($curl, CURLOPT_USERPWD, "mikesmarketer:Z2xTtj5w1qz"); 
    	curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($curl, CURLOPT_VERBOSE,0);
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
		//$headers = array('Content-Type: application/json','Content-Length: ' . strlen($json_encoded_data));
		curl_setopt($curl, CURLOPT_HTTPHEADER, array("Content-Type: application/json","Content-Length: " . strlen($json_encoded_data),"X_SPLICKIT_CLIENT_ID:com.splickit.jerseymikes","X_SPLICKIT_CLIENT_DEVICE:unit_testing","X_SPLICKIT_CLIENT:LoginTest","Auth_token:$mm_json_encrypted"));
    	curl_setopt($curl, CURLOPT_POST, 1);
		curl_setopt($curl, CURLOPT_POSTFIELDS,$json_encoded_data);
    	
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);	
		$result = curl_exec($curl);
		curl_close($curl);
		$r = json_decode($result,true);
		$this->assertNull($r['error']);
		
		// check for advanced lead times
		$lead_times_array = $r['lead_times_array'];
		$last = array_pop($lead_times_array);
		$future_ts = getTodayTwelveNoonTimeStampDenver() + (28*24*60*60);
		$this->assertTrue($last > $future_ts);
		
    	// value correctly
    	$subtotal = $r['order_amt'];
    	$grand_total = $r['grand_total'];
		$json_encoded_data = "{\"jsonVal\":{\"merchant_id\":\"$merchant_id\",\"items\":[{\"quantity\":1,\"note\":\"\",\"mods\":[{\"mod_quantity\":1,\"modifier_item_id\":\"2258930\",\"size_id\":\"0\"}],\"item_id\":\"278172\",\"size_id\":\"89585\"}],\"note\":\"This is the note that i'm writing.\",\"lead_time\":15,\"promo_code\":\"\",\"tip\":\"$tip\",\"user_id\":\"$user_id\",\"loyalty_no\":\"\",\"sub_total\":\"$subtotal\",\"grand_total\":\"$grand_total\",\"actual_pickup_time\":\"$future_ts\"}}";

    	$url = "https://75.101.136.231/app2/phone/placeorder?log_level=6";
    	$curl = curl_init($url);
    	curl_setopt($curl, CURLOPT_USERPWD, "mikesmarketer:Z2xTtj5w1qz"); 
    	curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($curl, CURLOPT_VERBOSE,0);
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
		//$headers = array('Content-Type: application/json','Content-Length: ' . strlen($json_encoded_data));
		curl_setopt($curl, CURLOPT_HTTPHEADER, array("Content-Type: application/json","Content-Length: " . strlen($json_encoded_data),"X_SPLICKIT_CLIENT_ID:com.splickit.jerseymikes","X_SPLICKIT_CLIENT_DEVICE:unit_testing","X_SPLICKIT_CLIENT:LoginTest","Auth_token:$mm_json_encrypted"));
    	curl_setopt($curl, CURLOPT_POST, 1);
		curl_setopt($curl, CURLOPT_POSTFIELDS,$json_encoded_data);
    	
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);	
		$result = curl_exec($curl);
		curl_close($curl);
		$r = json_decode($result,true);
		$this->assertNull($r['ERROR'],"Should not have gotten an order error");
		$order_id = $r['order_id'];
		$this->assertTrue($order_id > 1000,"should have created an order. ".$order_resource->error);
		
		$item_price = $r['order_summary']['cart_items'][0]['item_price'];
		$discount = number_format($item_price * .1,2);
		$receipt_items = $r['order_summary']['receipt_items'];
		$better_recipt_items = createHashOfRecieptItemsByTitle($receipt_items);
		$promo_amount = $better_recipt_items['Promo Discount'];
		$this->assertEquals('$-'.$discount, $promo_amount);
    }
	
    static function loadStoreIdIfNotCurrentlyLoaded($store_id,$menu_id = 102433)
    {
    	$merchant_adapter = new MerchantAdapter($mimetypes);
    	if ($record = $merchant_adapter->getRecord(array("merchant_external_id"=>$store_id))) {
    		return $record['merchant_id'];
    	}
    	$jersey_mikes_importer = new JerseyMikesImporter();
    	if ($menu_id != 102433) {
    		$jersey_mikes_importer->setJerseyMikesMenuId($menu_id);
    	}
    	$result = $jersey_mikes_importer->importFullStore($store_id);
    	
    	$merchant_id = $jersey_mikes_importer->imported_merchant_ids[0];
    	return $merchant_id;
    }
    
    function loadMenu($merchant_id,$menu_id = 102433)
    {
    	$item_size_adapter = new ItemSizeAdapter($mimetypes);
    	$item_size_data = array();
		$item_size_options[TONIC_FIND_BY_METADATA] = $item_size_data;
		$item_size_options[TONIC_JOIN_STATEMENT] = " JOIN Item ON Item.item_id = Item_Size_Map.item_id JOIN Menu_Type ON Item.menu_type_id = Menu_Type.menu_type_id ";
		$item_size_options[TONIC_FIND_BY_STATIC_METADATA] = " Menu_Type.menu_id = $menu_id AND Menu_Type.logical_delete = 'N' AND Item_Size_Map.merchant_id = $merchant_id";
		//$item_size_options[TONIC_SORT_BY_METADATA] = ' Item_Size_Map.priority DESC ';
		$item_size_resources = Resource::findAll($item_size_adapter,'',$item_size_options);
		$better_item_size_array = array();
		foreach ($item_size_resources as $is_resource)
		{
			$plu = $is_resource->external_id;
			$key = $is_resource->item_id.'-'.$is_resource->size_id;
			$tax_group = $is_resource->tax_group;
			$price = $is_resource->price; 
			$record = array("price"=>$price,"tax_group"=>$tax_group);
			$better_item_size_array[$key] = $record;
			if ($plu) {
				$better_item_size_array[$plu] = $record;
			}
		}
		$this->regular_menu = $better_item_size_array;
    	return $better_item_size_array;
    }   
    
    static function setUpBeforeClass()
    {
    	//$_SERVER['ENVIRONMENT'] = "local";
    	
    	// import the menu
    	$_SERVER['request_time1'] = microtime(true);
    	//$_SERVER['NO_MOCKS'] = true;
    	$_SERVER['HTTP_NO_CC_CALL'] = 'true';
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	
    	$menu_adapter = new MenuAdapter($mimetypes);
		if ($menu_adapter->getRecord(array("menu_id"=>102433))){
			; // do nothing
		} else {
			//create menu
			// why do i have to do this twice. please help :(
	    	$menu_adapter->importMenu(102433,'prod','local',326);
			$menu_adapter->importMenu(102433,'prod','local');
		}
				
		//now load regular jersey store
		$regular_store_merchant_id = JerseyMikesMarketerTest::loadStoreIdIfNotCurrentlyLoaded("1023");
		$ids['regular_store'] = $regular_store_merchant_id;
        $merchant_resource = SplickitController::getResourceFromId($regular_store_merchant_id, 'Merchant');
		$merchant_resource->brand_id = 326;
		$merchant_resource->active = 'Y';
		$merchant_resource->save();

    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;

    }
    
	static function tearDownAfterClass()
    {
    	//mysqli_query("ROLLBACK");
    }
     
    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    JerseyMikesMarketerTest::main();
}

?>