<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class JerseyMikesSingleMenuImportTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
		setContext("com.splickit.jerseymikes");
		// admin so as not to cache
		logTestUserIn(2);
    	
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }
    
    function testSingleMenuImport()
    {
    	unset($_SERVER['FORCE_JM_DELIVERY_FOR_STORE']);
		$jersey_mikes_importer = new JerseyMikesImporter();
    	
    	$this->assertTrue(file_exists('unit_tests_pos_integrations/jerseymikes3302import.txt'));
		$import_result = file_get_contents('unit_tests_pos_integrations/jerseymikes3302import.txt');
		myerror_log($import_result);
    	$this->assertTrue($jersey_mikes_importer->processDataPull($import_result,'33002'));
    	
    	$merchant_id = $jersey_mikes_importer->imported_merchant_ids[0];
		
    	// use admin to avoid caching
		$user = logTestUserIn(2);
		
		$request = new Request();
		$request->url = "/merchants/$merchant_id";
		$request->method = 'GET';
		$merchant_controller = new MerchantController($mt, $user, $request, 5);
		$merchant_resource = $merchant_controller->processV2Request();
		$menu = $merchant_resource->menu;
		unset($menu['combos']);
		$menu_to_json = json_encode($menu);
		myerror_log($menu_to_json);
		$menu_to_string = cleanUpCRLFTFromString(var_export($menu,true));
		myerror_log($menu_to_string);
		$this->assertContains('Fountain Drink', $menu_to_string);
		$this->assertContains('Chip Drink Combo', $menu_to_string);
		$this->assertContains('Bags Of Chips', $menu_to_string,"should contain bags of chips modifier");
        $this->assertNotContains('Cheetos', $menu_to_string," should not contain chip flavor");
        $this->assertContains('Number Of Bottles',$menu_to_string,"should contain number of bottles modifier");
        $this->assertNotContains('Naked - Berry Blast',$menu_to_string,"should not contain Naked - Berry Blast modifier");
        $this->assertNotContains('Black Cherry Seltzer',$menu_to_string,"should not contain Black Cherry Seltzer modifier");
        return $merchant_id;
    }

    /**
     * @depends testSingleMenuImport
     */
    function testDoNotSend8888ingredientId($merchant_id)
    {
        setContext('com.splickit.jerseymikes');
        $user_resource = createNewUserWithCC();
        $user = logTestUserResourceIn($user_resource);
        $user_id = $user['user_id'];
        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours', 1);
        $bags_of_chips_mods = array('mod_quantity'=>1,'modifier_item_id'=>2271915);
        $item['quantity'] = 3;
        $item['item_id'] = 276963;
        $item['size_id'] = 89668;
        $item['mods'][] = $bags_of_chips_mods;
        $items[] = $item;
        $order_data['items'] = $items;
        $order_resource = placeOrderFromOrderData($order_data, $time_stamp);
        $order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id, $mimetypes);
        $jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
        $jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
        $chip_item = $jersey_mikes_formatted_order['items'][0];
        $this->assertCount(0,$chip_item['withs'],"withs should not contain the 8888 modifier");
        $this->assertEquals(3,$chip_item['quantity'],"should have been 3 bags of chips");
    }

    /**
     * @depends testSingleMenuImport
     */
    function testNumberOfBotlesSetsQuantityCorrectlyOnJmPayload($merchant_id)
    {
        //item:277816  modifier:2317098   size:89668
        setContext('com.splickit.jerseymikes');
        $user_resource = createNewUserWithCC();
        $user = logTestUserResourceIn($user_resource);
        $user_id = $user['user_id'];
        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours', 1);
        $number_of_bottles = array('mod_quantity'=>5,'modifier_item_id'=>2317503);
        $item['quantity'] = 1;
        $item['item_id'] = 277816;
        $item['size_id'] = 89668;
        $item['mods'][] = $number_of_bottles;
        $items[] = $item;
        $order_data['items'] = $items;
        $order_resource = placeOrderFromOrderData($order_data, $time_stamp);
        $order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id, $mimetypes);
        $jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
        $jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
        $bottle_item = $jersey_mikes_formatted_order['items'][0];
        $this->assertCount(0,$bottle_item['withs'],"withs should not contain the 8888 modifier");
        $this->assertEquals(5,$bottle_item['quantity'],"should have been 5 bottles");
        $this->assertEquals(9.95,$order_resource->order_amt,"order total should have been for 5 bottles");
    }


    /**
     * @depends testSingleMenuImport
     */
    function testGetDeliveryMenuOfNonDeliveryMerchant($merchant_id)
    {
    	$request = new Request();
		$request->url = "/merchants/$merchant_id/delivery";
		$request->method = 'GET';
		$merchant_controller = new MerchantController($mt, $user, $request, 5);
		$merchant_resource = $merchant_controller->processV2Request();
		$this->assertNotNull($merchant_resource->error,"should have gotten an error because merhcant is not set up for delivery");
		$this->assertEquals("We're sorry, but this location does not appear to be accepting delivery orders right now.",$merchant_resource->error);
    }

    /**
     * @depends testSingleMenuImport
     */
    function testTurnOnDelivery($merchant_id)
    {
		$this->assertTrue(file_exists('unit_tests_pos_integrations/jerseymikes3302import_delivery_on.txt'));
		$import_result = file_get_contents('unit_tests_pos_integrations/jerseymikes3302import_delivery_on.txt');
		myerror_log($import_result);
		$jersey_mikes_importer = new JerseyMikesImporter();
    	$this->assertTrue($jersey_mikes_importer->processDataPull($import_result,'33002'));
    	
    	$this->assertEquals($merchant_id, $jersey_mikes_importer->imported_merchant_ids[0]);
    	return $merchant_id;
    }
    
    /**
     * @depends testTurnOnDelivery
     */
    function testDeliveryActiveForMerchant($merchant_id)
    {
    	$merchant_record = MerchantAdapter::staticGetRecordByPrimaryKey($merchant_id, 'MerchantAdapter');
    	$this->assertNotNull($merchant_record);
    	$this->assertEquals('Y', $merchant_record['delivery']);
    }
    
    /**
     * @depends testTurnOnDelivery
     */
    function testDeliveryMenuMapping($merchant_id)
    {
    	$merchant_menu_map_adapter = new MerchantMenuMapAdapter($mimetypes);
    	$record = $merchant_menu_map_adapter->getRecord(array("merchant_id"=>$merchant_id,"menu_id"=>102433,"merchant_menu_type"=>"delivery"), $options);	
    	$this->assertNotNull($record,"should have found a delivery menu mapping");
    }
    
    /**
     * @depends testTurnOnDelivery
     */
    function testDefaultDeliveryHourMapping($merchant_id)
    {
    	$hour_adapter = new HourAdapter($mimetypes);
    	$hours = $hour_adapter->getHours($merchant_id, 'D');
    	$this->assertNotNull($hours,"should have found delivery hours");
    	$this->assertCount(7, $hours,"should have found 7 delivery hours");
    }
    
    /**
     * @depends testTurnOnDelivery
     */
    function testDefaultDeliveryInfoMapping($merchant_id)
    {
    	$merchant_delivery_info_adapter = new MerchantDeliveryInfoAdapter($mimetypes);
    	$record = $merchant_delivery_info_adapter->getRecord(array("merchant_id"=>$merchant_id), $options);
    	$this->assertNotNull($record,"Should have found a delivery info record");
    	$this->assertEquals(1.00, $record['minimum_order'],"minimum order should have been defaulted to 1.00");
    	$this->assertEquals('Y', $record['active'],"delivery info record should be active");
    	$this->assertEquals('driving',$record['delivery_price_type']);
    }
    
    /**
     * @depends testTurnOnDelivery
     */
    function testDefaultDeliveryPriceDistanceMapping($merchant_id)
    {
    	$merchant_delivery_price_distance_adapter = new MerchantDeliveryPriceDistanceAdapter($mimetypes);
    	$record = $merchant_delivery_price_distance_adapter->getRecord(array("merchant_id"=>$merchant_id), $options);
    	$this->assertNotNull($record,"should have found a delivery price record");
    	$this->assertEquals(1.00, $record['price'],"price should be defaulted to 1.00");
    	$this->assertEquals(1.00, $record['distance_up_to'],"distance should be defaulted to 1 miles driving");	
    }

    /**
     * @depends testTurnOnDelivery
     */
    function testDeliveryMenuItems($merchant_id)
    {
    	$request = new Request();
		$request->url = "/merchants/$merchant_id/delivery";
		$request->method = 'GET';
		$merchant_controller = new MerchantController($mt, $user, $request, 5);
		$merchant_resource = $merchant_controller->processV2Request();
		$this->assertNotNull($merchant_resource);
		$this->assertNull($merchant_resource->error,"Should not have found an error but did");
		$menu = $merchant_resource->menu;
		// unset the combo section for now
		unset($menu['combos']);
		$menu_to_json = json_encode($menu);
		myerror_log($menu_to_json);
		$menu_to_string = cleanUpCRLFTFromString(var_export($menu,true));
		myerror_log($menu_to_string);
		//$this->assertNotContains('Fountain Drink', $menu_to_string);
		$this->assertTrue(substr_count($menu_to_string, 'Fountain Drink') < 1,"should not contain fountain drink");
		//$this->assertNotContains('Chip Drink Combo', $menu_to_string);
		$this->assertTrue(substr_count($menu_to_string, 'Chip Drink Combo') < 1,"shoujld not contain combo");
		//$this->assertNotContains('22oz Fountain',$menu_to_string);
		$this->assertTrue(substr_count($menu_to_string, '22oz Fountain') < 1,"should not contain foiuntain drink 22oz");
		//$this->assertNotContains('32oz Fountain',$menu_to_string);
		$this->assertTrue(substr_count($menu_to_string, '32oz Fountain') < 1,"should not contain foiuntain drink 32 oz");
		//$this->assertNotContains('combo pricing',$menu_to_string);
		$this->assertTrue(substr_count($menu_to_string, 'combo pricing') < 1,"should not contain any combos");
		//$this->assertNotContains('Bags Of Chips', $menu_to_string,"Should not contain bags of chips modifier");
		$this->assertTrue(substr_count($menu_to_string, 'Bags Of Chips') < 1,"Should not contain bags of chips modifier");
		$this->assertTrue(substr_count($menu_to_string, 'Black Cherry Seltzer') > 0,"should contain Black Cherry Seltzer modifier");
		$this->assertTrue(substr_count($menu_to_string, 'Number Of Bottles') < 1,"Should not contain bags of Number Of Bottles");
		//$this->assertContains('Cheetos',$menu_to_string,"Should contain chip flavor modifier"); 
		$this->assertTrue(substr_count($menu_to_string, 'Cheetos') > 0,"Should contain chip flavor modifier");
		//$this->assertNotContains('Cool Ranch Doritos',$menu_to_string,"Should NOT contain chip flavor modifier since it was not in the dump");
		$this->assertTrue(substr_count($menu_to_string, 'Cool Ranch Doritos') < 1,"Should NOT contain chip flavor modifier since it was not in the dump");
		   	    	
    }

    /**
     * @depends testTurnOnDelivery
     */
    function testHoldItDoesntContainThe8888Modifier($merchant_id)
    {
        setContext('com.splickit.jerseymikes');
        $user_resource = createNewUserWithCC();
        $user = logTestUserResourceIn($user_resource);
        $user_id = $user['user_id'];
        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'delivery', 'skip hours', 1);
        $bags_of_chips_mods = array('mod_quantity'=>1,'modifier_item_id'=>2307542);
        $item['quantity'] = 1;
        $item['item_id'] = 276963;
        $item['size_id'] = 89668;
        $item['mods'][] = $bags_of_chips_mods;
        $items[] = $item;
        $order_data['items'] = $items;
        $order_resource = placeOrderFromOrderData($order_data, $time_stamp);
        $order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id, $mimetypes);
        $jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
        $jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
        $chip_item = $jersey_mikes_formatted_order['items'][0];
        $this->assertCount(0,$chip_item['holds'],"holds should not contain the 8888 modifier");
    }

    /**
     * @depends testSingleMenuImport
     */
    function testDifferentBotlesSetsItemsWithsCorrectlyOnJmPayload($merchant_id)
    {
        //item:277816  modifier:2317098   size:89668
        setContext('com.splickit.jerseymikes');
        $user_resource = createNewUserWithCC();
        $user = logTestUserResourceIn($user_resource);
        $user_id = $user['user_id'];
        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'delivery', 'skip hours', 1);
        $mods[] = array('mod_quantity'=>1,'modifier_item_id'=>2270803);
        $mods[] = array('mod_quantity'=>1,'modifier_item_id'=>2270804);
        $mods[] = array('mod_quantity'=>1,'modifier_item_id'=>2270805);

        $item['quantity'] = 1;
        $item['item_id'] = 277816;
        $item['size_id'] = 89668;
        $item['mods'] = $mods;
        $items[] = $item;
        $order_data['items'] = $items;
        $order_resource = placeOrderFromOrderData($order_data, $time_stamp);
        $order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id, $mimetypes);
        $jersey_mikes_controller = new JerseyMikesController($mt, $u, $r,5);
        $jersey_mikes_formatted_order = $jersey_mikes_controller->convertOrderToJerseyMikesDataSet($order);
        $jm_order_as_json_string = json_encode($jersey_mikes_formatted_order);
        $this->assertNotContains('8888',$jm_order_as_json_string,"should not contain the 8888 modifier anywhere");
        $this->assertCount(3,$jersey_mikes_formatted_order['items'],"Should have resulted in 3 items in the payload since they are 3 differnt bottles.");
        $all_withs = array();
        foreach ($jersey_mikes_formatted_order['items'] as $jm_item) {
            $withs = $jm_item['withs'];
            $this->assertCount(1,$withs,"should only be 1 with for each item");
            $this->assertNull($all_withs[$withs[0]],"shoudl be 3 differnt bottles");
            $all_withs[$withs[0]] = 1;
        }
        $this->assertEquals(5.97,$order_resource->order_amt,"order total should have been for 5 bottles");
    }


    //now test custom values
    // jerseymikes3302import_delivery_on_with_custom_values
    /**
     * @depends testTurnOnDelivery
     */
    function testCustomDeliveryValues($merchant_id)
    {
		$this->assertTrue(file_exists('unit_tests_pos_integrations/jerseymikes3302import_delivery_on_with_custom_values.txt'));
		$import_result = file_get_contents('unit_tests_pos_integrations/jerseymikes3302import_delivery_on_with_custom_values.txt');
		$jersey_mikes_importer = new JerseyMikesImporter();
    	$this->assertTrue($jersey_mikes_importer->processDataPull($import_result,'33002'));
    	$this->assertEquals($merchant_id, $jersey_mikes_importer->imported_merchant_ids[0]);
    	return $merchant_id;
    }
    
    /**
     * @depends testCustomDeliveryValues
     */
    function testCustomDeliveryHourMapping($merchant_id)
    {
    	$hour_adapter = new HourAdapter($mimetypes);
    	$hours = $hour_adapter->getHours($merchant_id, 'D');
    	$this->assertNotNull($hours,"should have found delivery hours");
    	$this->assertCount(7, $hours,"should have found 7 delivery hours");
    	$day = $hours[0];
    	$this->assertEquals('11:00:00', $day['open']);
    	$this->assertEquals('15:00:00', $day['close']);
    }
    
    /**
     * @depends testCustomDeliveryValues
     */
    function testHoursMessageForDeliveryMenu1($merchant_id)
    {
		$complete_merchant = new CompleteMerchant($merchant_id);
		$time_stamp = getTimeStampForDateTimeAndTimeZone(5, 0, 0, 9, 9, 2014, 'America/New_York');
		$complete_merchant->setTimeForTesting($time_stamp);
		$merchant_data = $complete_merchant->getTheMerchantData('delivery');
		$weeks_open_close_ts = $complete_merchant->open_close_ts_for_next_7_days_including_today;
		
		$expected_next_open = getTimeStampForDateTimeAndTimeZone(11, 0, 0, 9, 9, 2014, 'America/New_York');
		$this->assertEquals($expected_next_open, $weeks_open_close_ts[0]['open'],"should have had next open at ".date('h:i a',$expected_next_open)." but it was: ".date('h:i a',$weeks_open_close_ts[0]['open']));
		
		$user_message = $complete_merchant->isMerchantOpenAtThisTime($weeks_open_close_ts, $time_stamp);
		$this->assertEquals("Please note: This merchant's delivery hours start at 11:00 am", $user_message);		    	
    }
    
    /**
     * @depends testCustomDeliveryValues
     */
    function testHoursMessageForDeliveryMenu2($merchant_id)
    {
		$complete_merchant = new CompleteMerchant($merchant_id);
		$time_stamp = getTimeStampForDateTimeAndTimeZone(16, 0, 0, 9, 9, 2014, 'America/New_York');
		$complete_merchant->setTimeForTesting($time_stamp);
		$merchant_data = $complete_merchant->getTheMerchantData('delivery');
		$weeks_open_close_ts = $complete_merchant->open_close_ts_for_next_7_days_including_today;
		
		$user_message = $complete_merchant->isMerchantOpenAtThisTime($weeks_open_close_ts, $time_stamp);
		$this->assertEquals("Please note: This merchant's delivery hours have ended for today. Next available delivery is Wed 11:00 am", $user_message);		    	
    }
    
    /**
     * @depends testCustomDeliveryValues
     */
    function testHoursMessageForDeliveryMenu3($merchant_id)
    {
		$complete_merchant = new CompleteMerchant($merchant_id);
		$time_stamp = getTimeStampForDateTimeAndTimeZone(14, 01, 0, 9, 9, 2014, 'America/New_York');
		$complete_merchant->setTimeForTesting($time_stamp);
		$merchant_data = $complete_merchant->getTheMerchantData('delivery');
		$weeks_open_close_ts = $complete_merchant->open_close_ts_for_next_7_days_including_today;
		
		$user_message = $complete_merchant->isMerchantOpenAtThisTime($weeks_open_close_ts, $time_stamp);
		$this->assertEquals("Please note: This merchant's delivery hours end today at 3:00 pm", $user_message);		    	
    }
    
	/**
     * @depends testCustomDeliveryValues
     */
    function testCustomDeliveryInfoMapping($merchant_id)
    {
    	$merchant_delivery_info_adapter = new MerchantDeliveryInfoAdapter($mimetypes);
    	$record = $merchant_delivery_info_adapter->getRecord(array("merchant_id"=>$merchant_id), $options);
    	$this->assertNotNull($record,"Should have found a delivery info record");
    	$this->assertEquals(5.00, $record['minimum_order'],"minimum order should have been set to 5");
    	$this->assertEquals('Y', $record['active'],"delivery info record should be active");
    	$this->assertEquals('polygon',$record['delivery_price_type']);
    }
    
    /**
     * @depends testCustomDeliveryValues
     */
    function testCustomDeliveryPriceDistanceMapping($merchant_id)
    {
    	$merchant_delivery_price_distance_adapter = new MerchantDeliveryPriceDistanceAdapter($mimetypes);
    	$record = $merchant_delivery_price_distance_adapter->getRecord(array("merchant_id"=>$merchant_id), $options);
    	$this->assertNotNull($record,"should have found a delivery price record");
    	$this->assertEquals(2.50, $record['price'],"price should be set to 2.50");
    	$this->assertEquals("40.833294 -96.720200,40.799127 -96.720543,40.798607 -96.682262,40.834074 -96.682091",$record['polygon_coordinates'],"Polygon coordinates should have been set");	
    }
    
    // now try importing a without values to make sure they dont get overwritten
        /**
     * @depends testSingleMenuImport
     */
    function testTurnOnDeliveryWithoutCustomValuesToMakeSUreNotToOVerwriteValues($merchant_id)
    {
		$this->assertTrue(file_exists('unit_tests_pos_integrations/jerseymikes3302import_delivery_on.txt'));
		$import_result = file_get_contents('unit_tests_pos_integrations/jerseymikes3302import_delivery_on.txt');
		myerror_log($import_result);
		$jersey_mikes_importer = new JerseyMikesImporter();
    	$this->assertTrue($jersey_mikes_importer->processDataPull($import_result,'33002'));
    	
    	$this->assertEquals($merchant_id, $jersey_mikes_importer->imported_merchant_ids[0]);
    	return $merchant_id;
    }
    
    /**
     * @depends testCustomDeliveryValues
     */
    function testCustomDeliveryHourMapping2($merchant_id)
    {
    	$hour_adapter = new HourAdapter($mimetypes);
    	$hours = $hour_adapter->getHours($merchant_id, 'D');
    	$this->assertNotNull($hours,"should have found delivery hours");
    	$this->assertCount(7, $hours,"should have found 7 delivery hours");
    	$day = $hours[0];
    	$this->assertEquals('11:00:00', $day['open']);
    	$this->assertEquals('15:00:00', $day['close']);
    }
    
    /**
     * @depends testCustomDeliveryValues
     */
    function testCustomDeliveryInfoMapping2($merchant_id)
    {
    	$merchant_delivery_info_adapter = new MerchantDeliveryInfoAdapter($mimetypes);
    	$record = $merchant_delivery_info_adapter->getRecord(array("merchant_id"=>$merchant_id), $options);
    	$this->assertNotNull($record,"Should have found a delivery info record");
    	$this->assertEquals(1.00, $record['minimum_order'],"minimum order should have been set to 1");
    	$this->assertEquals('Y', $record['active'],"delivery info record should be active");
    	$this->assertEquals('polygon',$record['delivery_price_type']);
    }
    
    /**
     * @depends testCustomDeliveryValues
     */
    function testCustomDeliveryPriceDistanceMapping2($merchant_id)
    {
    	$merchant_delivery_price_distance_adapter = new MerchantDeliveryPriceDistanceAdapter($mimetypes);
    	$record = $merchant_delivery_price_distance_adapter->getRecord(array("merchant_id"=>$merchant_id), $options);
    	$this->assertNotNull($record,"should have found a delivery price record");
    	$this->assertEquals(2.50, $record['price'],"price should be set to 2.50");
    	$this->assertEquals("40.833294 -96.720200,40.799127 -96.720543,40.798607 -96.682262,40.834074 -96.682091",$record['polygon_coordinates'],"Polygon coordinates should have been set");	
    }

    static function setUpBeforeClass()
    {
   		myerror_log("we are starting the single menu import");
    	$_SERVER['request_time1'] = microtime(true);
    	$_SERVER['NO_MOCKS'] = true;
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	
		$menu_adapter = new MenuAdapter($mimetypes);
		if ($menu_adapter->getRecord(array("menu_id"=>102433))){
			; // do nothing
		} else {
			//create menu
	    	$menu_adapter->importMenu(102433,'test','local',326);
		}
		
		if ($merchant_resource = MerchantAdapter::getMerchantFromExternalIdBrandId('33002', 326)) {
			$merchant_id_for_33002 = $merchant_resource->merchant_id;
	    	myerror_log("we ahve the merchant 33002.  merchant_id: ".$merchant_id_for_33002);
			$sql = "call SMAWSP_ADMIN_DEL_MERCHANT($merchant_id_for_33002)";
	    	$menu_adapter->_query($sql);
		} else {
			myerror_log("couldn't find 33002");
		}
    	mysqli_query("BEGIN");
    	
    	$sql = "UPDATE Modifier_Size_Map a JOIN Modifier_Item b ON a.modifier_item_id = b.modifier_item_id SET a.active = 'Y', a.included_merchant_menu_types = 'Delivery' WHERE b.modifier_group_id = 328207 and b.modifier_item != 2260280";
    	$menu_adapter->_query($sql);
    	$sql = "UPDATE Modifier_Size_Map a JOIN Modifier_Item b ON a.modifier_item_id = b.modifier_item_id SET a.active = 'Y', a.included_merchant_menu_types = 'Pickup' WHERE b.modifier_group_id = 328207 and b.modifier_item = 2260280";
    	$menu_adapter->_query($sql);
    	    	
    //	$user_resource = createNewUser(array("flags"=>"1C20000001"));
    //	$ids['user_id'] = $user_resource->user_id;
    	
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	mysqli_query("ROLLBACK");
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    JerseyMikesSingleMenuImportTest::main();
}

?>