<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class LateMessageAlerterTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $late_message_alerter;
	var $merchant_id;
	var $ids;

	function setUp()
	{
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->late_message_alerter = new LateMessageAlerter();
		$this->merchant_id = $_SERVER['unit_test_ids']['merchant_id'];
		$this->ids = $_SERVER['unit_test_ids'];
		
	}
	
	function tearDown() 
	{
		//delete your instance
		unset($this->late_message_alerter);
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->stamp);
		unset($this->merchant_id);
		unset($this->ids);
    }

    
    function createMessageData($minutes_late,$message_format = 'GUC',$locked = 'P',$with_delete = true)
    {
    	$mmh_adapter = new MerchantMessageHistoryAdapter($mimetypes);
    	if ($with_delete)
    	{
	    	$sql = "DELETE FROM Merchant_Message_History WHERE order_id = ".$this->ids['order_id'];
	    	$mmh_adapter->_query($sql);
	    	$sql = "DELETE FROM customer_service_results WHERE order_id = ".$this->ids['order_id'];
	    	$mmh_adapter->_query($sql);
    	}
    	    	
    	// now create the message to be sent $minutes_late min ago
    	$five_min_ago = time() - ($minutes_late * 60) - 10;
    	$mmh_data['merchant_id'] = $this->merchant_id;
    	$mmh_data['order_id'] = $this->ids['order_id'];
    	$mmh_data['message_format'] = $message_format;
    	$mmh_data['message_delivery_addr'] = '1234567890';
    	$mmh_data['next_message_dt_tm'] = $five_min_ago;
    	$mmh_data['locked'] = 'P';
    	$mmh_data['viewed'] = 'N';
    	$mmh_data['message_type'] = 'X';
    	$mmh_data['tries'] = '0';
    	$mmh_resource = Resource::factory($mmh_adapter,$mmh_data);
    	if ($mmh_resource->save())
    		return $mmh_resource;
		return false;
    }
    
    function testGetLateMessages()
    {
        $mmha = new MerchantMessageHistoryAdapter($mimetypes);
        $sql = "DELETE FROM Merchant_Message_History WHERE 1=1";
        $mmha->_query($sql);
        $message_resources[1] = $this->createMessageData(3);
    	$message_resources[2] = $this->createMessageData(4,'GUC','P',false);
    	$message_resources[3] = $this->createMessageData(5,'SUW','P',false);
    	$message_resources[4] = $this->createMessageData(6,'W','P',false);
    	$message_resources[5] = $this->createMessageData(7,'O','P',false);
    	$message_resources[6] = $this->createMessageData(2,'GUA','P',false);
    	$message_resources[7] = $this->createMessageData(1,'GUA','P',false);
    	
    	// lets add a cob message to make sure it doesn't get picked up.
    	$mmha->createMessage($this->merchant_id, $order_id, 'GCob', '1234567890', time()-200, 'I', 'Firmware=8.0', 'this is the cob message','P');
    	
    	// check for anything 2.5 minuts late
    	$ts = time()-150;
    	$messages = $mmha->getLatePulledMessagesFromTimeStamp($ts);
    	$this->assertEquals(5, count($messages));
    	
    	$ts2 = time()-330;
    	$messages2 = $mmha->getLatePulledMessagesFromTimeStamp($ts2);
    	$this->assertEquals(2, count($messages2));
    	
    }
    
    function testFailLateMessage()
    {
    	$message_resource = $this->createMessageData(21);
    	$mmha = new MerchantMessageHistoryAdapter($mimetypes);
    	$result = $mmha->checkIfMessageIsLateByThisManyMinutesAndFailIfSo(25, $message_resource);
    	$this->assertFalse($result);
    	$result2 = $mmha->checkIfMessageIsLateByThisManyMinutesAndFailIfSo(20, $message_resource);
    	$this->assertTrue($result2);
    }
    
    //*****test unviewed messages******
    function testUnViewedMessage()
    {
    	$message_resource = $this->createMessageData(3);
    	$message_resource->sent_dt_tm = date("Y-m-d H:i:s",time()-30);
    	$message_resource->locked = "S";
    	$message_resource->viewed = "N";
    	$message_resource->tries = 1;
    	$message_resource->save();
    	$message_id = $message_resource->map_id;
    	
    	myerror_log("about to do fist check for late message alerter. Should not get any messages");
    	$unviewed_message_resources = LateMessageAlerter::getUnviewedMessages();
    	$this->assertFalse($unviewed_message_resources);
    	
    	$message_resource->sent_dt_tm = date("Y-m-d H:i:s",time()-130);
    	$message_resource->save();
    	
    	myerror_log("about to do fist check for late message alerter. Should get one message, message_id: ".$message_id);
    	$unviewed_message_resources = LateMessageAlerter::getUnviewedMessages();
    	$this->assertEquals(1, count($unviewed_message_resources));
    	$unviewed_message_resource = $unviewed_message_resources[0];
    	
    	myerror_log("about to process the message.   should be setting the sent to 0000-00-00");
    	LateMessageAlerter::processUnviewedMessages();
    	$updated_message_resource = Resource::find(new MerchantMessageHistoryAdapter($mimetypes),''.$message_id);
    	$this->assertEquals('0000-00-00 00:00:00', $updated_message_resource->sent_dt_tm);
    	$this->assertEquals('P', $updated_message_resource->locked);
    	$this->assertEquals('101', $updated_message_resource->tries);
    	$this->assertNull($updated->message_text);

    	$updated_message_resource->sent_dt_tm = date("Y-m-d H:i:s",time()-630);
    	$updated_message_resource->locked = "S";
    	$updated_message_resource->save();
    	$unviewed_message_resources = LateMessageAlerter::getUnviewedMessages();
    	$this->assertFalse($unviewed_message_resources);

    }


//    function testEpsonMessageFiveMinutesLateSendAlert()
//    {
//        $message_resource = $this->createMessageData(5,'SUW','P',true);
//        $result = $this->late_message_alerter->checkAndProcessLatePulledMessages();
//    }
//
//    function testEpsonMessageUnviewed()
//    {
//        $message_resource = $this->createMessageData(5,'SUW','P',true);
//        $message_resource->locked = 'S';
//        $message_resource->viewed = 'N';
//        $message_resource->sent_dt_tm = getMySqlFormattedDateTimeFromTimeStampAndTimeZone(time()-120);
//        $message_resource->save();
//
//        $unviewed_messages = LateMessageAlerter::getUnviewedMessages();
//        $this->assertCount(1,$unviewed_messages);
//
//        $result = LateMessageAlerter::processUnviewedMessages();
//
//
//    }

    
    function testMessageFiveMinLateSecondAttemptOldFirmware()
    {
    	
    	setProperty('call_center_on', 'false');
    	$message_resource = $this->createMessageData(5);
    	$message_resource->info = "firmware=8.0";
    	$message_resource->tries = 1;
    	$message_resource->save();
    	
    	$mmh_adapter = new MerchantMessageHistoryAdapter($mimetypes);
    	
		$this->late_message_alerter->chooseActionForLateNewFirmwarePrinterMessage($message_resource);
		$this->assertEquals('processLateMessage', $this->late_message_alerter->action);
    	
		$result = $this->late_message_alerter->getResult();
		$this->assertEquals(1, sizeof($result, $mode),"results should have had 1 and only 1 record but instead it had: ".sizeof($result, $mode));
		$this->assertNotNull($result[1],"Wrong result: ".array_pop($result)."    SHOULD HAVE BEEN: Call Center Off. Support Alerted.");
		
		$cc_options[TONIC_FIND_BY_METADATA]['merchant_id'] = $this->merchant_id;
		$cc_options[TONIC_FIND_BY_METADATA]['order_id'] = $this->ids['order_id'];
		$cc_options[TONIC_FIND_BY_METADATA]['message_format'] = 'CC';
		$cc_resources = Resource::findAll($mmh_adapter,'',$cc_options);
		$this->assertEquals(1, sizeof($cc_resources, $mode));
		$cc_resource = $cc_resources[0];
		$this->assertEquals('Call Center Off! Support Alerted!',$cc_resource->info);

		sleep(1);
		
		// do it again, should not add another row to the db for the intervention
		//$this->late_message_alerter->processLateMessage($message_resource);
		$this->late_message_alerter->checkAndProcessLatePulledMessages();
		$result = $this->late_message_alerter->getResult();
		$this->assertEquals(1, sizeof($result, $mode),"results should have had 1 and only 1 record but instead it had: ".sizeof($result, $mode));
		$this->assertNotNull($result[1],"Wrong result: ".array_pop($result)."    SHOULD HAVE BEEN: Call Center Off. Support Alerted.");
		
		$cc_resources2 = Resource::findAll($mmh_adapter,'',$cc_options);
		$this->assertEquals(1, sizeof($cc_resources2, $mode), "should still have only been one call center row but there were 2");
		
		return $message_resource;
		
    }
    
    /**
     * 5 minute late message with call center off
     * 
     * $this->result[1] = "Call Center Off. Support Alerted";
     */
    
    function testMessageFiveMinLateFirstAttemptNewFirmware()
    {
    	$message_resource = $this->createMessageData(5);
    	$message_resource->info = "firmware=8.0";
    	$message_resource->save();
       	$this->late_message_alerter->chooseActionForLateNewFirmwarePrinterMessage($message_resource);
    	$this->assertEquals('stageNewGPRSactiviationMessage', $this->late_message_alerter->action);
    	$this->assertEquals(1, $message_resource->tries);
    }
    
    function testMessageFiveMinLateSecondAttemptNewFirmware()
    {
    	
    	setProperty('call_center_on', 'false');
    	$message_resource = $this->createMessageData(5);
    	$message_resource->info = "firmware=8.0";
    	$message_resource->tries = 1;
    	$message_resource->save();
    	
    	$mmh_adapter = new MerchantMessageHistoryAdapter($mimetypes);
    	
		$this->late_message_alerter->chooseActionForLateNewFirmwarePrinterMessage($message_resource);
		$this->assertEquals('processLateMessage', $this->late_message_alerter->action);
    	
		$result = $this->late_message_alerter->getResult();
		$this->assertEquals(1, sizeof($result, $mode),"results should have had 1 and only 1 record but instead it had: ".sizeof($result, $mode));
		$this->assertNotNull($result[1],"Wrong result: ".array_pop($result)."    SHOULD HAVE BEEN: Call Center Off. Support Alerted.");
		
		$cc_options[TONIC_FIND_BY_METADATA]['merchant_id'] = $this->merchant_id;
		$cc_options[TONIC_FIND_BY_METADATA]['order_id'] = $this->ids['order_id'];
		$cc_options[TONIC_FIND_BY_METADATA]['message_format'] = 'CC';
		$cc_resources = Resource::findAll($mmh_adapter,'',$cc_options);
		$this->assertEquals(1, sizeof($cc_resources, $mode));
		$cc_resource = $cc_resources[0];
		$this->assertEquals('Call Center Off! Support Alerted!',$cc_resource->info);

		sleep(1);
		
		// do it again, should not add another row to the db for the intervention
		//$this->late_message_alerter->processLateMessage($message_resource);
		$this->late_message_alerter->checkAndProcessLatePulledMessages();
		$result = $this->late_message_alerter->getResult();
		$this->assertEquals(1, sizeof($result, $mode),"results should have had 1 and only 1 record but instead it had: ".sizeof($result, $mode));
		$this->assertNotNull($result[1],"Wrong result: ".array_pop($result)."    SHOULD HAVE BEEN: Call Center Off. Support Alerted.");
		
		$cc_resources2 = Resource::findAll($mmh_adapter,'',$cc_options);
		$this->assertEquals(1, sizeof($cc_resources2, $mode), "should still have only been one call center row but there were 2");
		
		return $message_resource;
		
    }
    
	function testMessageFiveMinLateFirstAttemptNewFirmwareIPversion()
    {
    	$message_resource = $this->createMessageData(5);
    	$message_resource->info = "firmware=1.1.0.9ip";
    	$message_resource->save();
    	$this->late_message_alerter->extractAndSetMessageInfoData($message_resource);
       	$this->late_message_alerter->chooseActionForLateNewFirmwarePrinterMessage($message_resource);
    	$this->assertEquals('processLateMessage', $this->late_message_alerter->action);
    	$this->assertEquals(1, $message_resource->tries);
    }

	function testOldFirmwareSecondPartMessageLate()
    {
    	$sql = "DELETE FROM Merchant_Message_History WHERE order_id = ".$this->ids['order_id'];
    	$mmh_adapter = new MerchantMessageHistoryAdapter($mimetypes);
	    $mmh_adapter->_query($sql);
    	
    	$user = logTestUserIn($this->ids['user_id']);
    	
    	$json = '{"user_id":"'.$this->ids['user_id'].'","name":"","address1":"1045 pine street","address2":"","city":"Boulder","state":"CO","zip":"80302","phone_no":"1234566890","lat":40.019785,"lng":-105.282509}';
    	$request = new Request();
    	$request->body = $json;
    	$request->mimetype = "Application/json";
    	$request->_parseRequestBody(); 
    	$user_controller = new UserController($mt, $this->user, $request,5);
    	$response = $user_controller->setDeliveryAddr();
    	
    	$merchant_resource = createNewTestMerchant($this->ids['menu_id']);
    	$merchant_resource->delivery = 'Y';
    	$merchant_resource->immediate_message_delivery = 'Y';
    	$merchant_resource->save();
    	$merchant_id = $merchant_resource->merchant_id;
    	$mdia = new MerchantDeliveryInfoAdapter($mimetypes);
    	$mdia_resource = $mdia->getExactResourceFromData(array("merchant_id"=>$merchant_id));	
    	$mdia_resource->minimum_order = 0.01;
    	$mdia_resource->delivery_cost = 1.00;
    	$mdia_resource->delivery_increment = 15;
    	$mdia_resource->max_days_out = 3;
    	$mdia_resource->save();
    	
    	$merchant_id = $merchant_resource->merchant_id;    	
    	$map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_id,"message_format"=>'GUA',"delivery_addr"=>"1234567890","message_type"=>"X"));
    	$map_resource = Resource::createByData(new MerchantMessageMapAdapter($mimetypes),array("merchant_id"=>$merchant_id,"message_format"=>'T',"delivery_addr"=>"1234567890","message_type"=>"A"));

		$order_data = getSimpletCartArrayByMerchantId($merchant_id,'delivery','dum note',20);
        $order_data['delivery'] = 'yes';
        $order_data['user_addr_id'] = $response->user_addr_id;
    	$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
    	$order_id = $order_resource->order_id;
    	$this->assertNull($order_resource->error,"should not have gotten an error: ".$order_resource->error);
    	$this->assertTrue($order_resource->order_id > 1000);
    	//sleep(1);  	
    	$request = new Request();
    	$request->url = "/m/g/$merchant_id/gprs/f.txt";
    	$gprs_controller = new GprsController($mt, $u, $request,5);
    	
    	$message_resource = $gprs_controller->pullNextMessageResourceByMerchant($merchant_id);
    	$message_text = $message_resource->message_text;
    	myerror_log("pulled message is: ".$message_resource->message_text);
    	$this->assertEquals('(page 1 of 3)::xx#', substr($message_text, -18));

    	$message_data['merchant_id'] = $merchant_id;
    	$message_data['order_id'] = $order_resource->order_id;
    	$message_data['message_format'] = 'GUA';
    	$message_data['map_id'] = array(">"=>$message_resource->map_id);
    	$message_options[TONIC_FIND_BY_METADATA] = $message_data;
    	$message_options[TONIC_SORT_BY] = " map_id ASC ";
    	$mmh_adapter = new MerchantMessageHistoryAdapter($mimetypes);
    	$message_resources = Resource::findAll($mmh_adapter,'',$message_options);
    	
    	$this->assertCount(2, $message_resources, "Should have found 2 other messages");
    	$message2 = $message_resources[0]; 
    	$message2_text = $message2->message_text;
    	$expected_message2_start = "#$merchant_id*$order_id*::::------- SUMMARY --------::::Items:  20::::- Test Item 1 $1.50::";  //84
    	$this->assertContains($expected_message2_start,$message2_text);
    	
    	//now lets make the unsent message late
    	foreach ($message_resources as $mr)
    	{
    		$mr->next_message_dt_tm = $mr->next_message_dt_tm - 330;
    		$mr->save();
    	}
    	
    	$this->late_message_alerter->checkAndProcessLatePulledMessages();
    	$this->assertEquals('We have a second part message not being picked up, alert the team!', $this->late_message_alerter->action);
    }
    
    static function setUpBeforeClass()
    {
    	ini_set('max_execution_time',300);
        setProperty('gprs_total_merchant_shutdown',"false");
        SplickitCache::flushAll();         $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction(); ;
    	$_SERVER['request_time1'] = microtime(true);    	
		//map it to a menu
    	$menu_id = createTestMenuWithNnumberOfItems(5);
    	$ids['menu_id'] = $menu_id;
    	setContext('com.splickit.worldhq');
    	$merchant_resource = createNewTestMerchant($menu_id);
    	$ids['merchant_id'] = $merchant_resource->merchant_id;
    	  	    	
    	$user_resource = createNewUser();
    	$user_resource->flags = '1C00000001';
    	$user_resource->save();
    	$ids['user_id'] = $user_resource->user_id;
    	logTestUserIn($ids['user_id']);
    	
    	$_SERVER['HTTP_NO_CC_CALL'] = 'true';
    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($ids['merchant_id'], 'pickup', 'skip hours');
    	//$time_stamp = getTodayTwelveNoonTimeStampDenver();
    	$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
    	$ids['order_id'] = $order_resource->order_id;
    	
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    	
    }
    
	static function tearDownAfterClass()
    {
    	SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    LateMessageAlerterTest::main();
}

?>