<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class LevelupPosIntegrationTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
		
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }

	function testProcessPaymentWithLevelup()
	{
		$level_up_payment_service = new LevelupPaymentService(array("merchant_id"=>$this->ids['merchant_id']));
		$data['levelup_user_token'] = "1140-aaa509de3012cb40884753ffff4a04e0f7470349b0fbaca4665bbd9b238c72";
		$level_up_payment_service->loadAdditionalDataFieldsIfNeeded($data);
		$level_up_payment_service->order_id = $this->ids['order_id'];
		$results = $level_up_payment_service->processPayment(.50);
        $this->assertEquals(200,$level_up_payment_service->curl_response['http_code']);
		$this->assertTrue(isset($results['uuid']),"should have processsed a valid order but didn't. error: ".$results['error']);
        return $results['uuid'];
	}

    /**
     * @depends testProcessPaymentWithLevelup
     */
    function testProcessRefundWithLevelUp($uuid)
    {
        $level_up_payment_service = new LevelupPaymentService(array("merchant_id"=>$this->ids['merchant_id']));
        $level_up_payment_service->order_id = $this->ids['order_id'];
        $level_up_payment_service->uuid = $uuid;
        $results = $level_up_payment_service->processRefundForUUID($uuid);
        $this->assertEquals(200,$level_up_payment_service->curl_response['http_code']);
        $this->assertEquals($uuid,$results['uuid']);
    }

    /**
     * @depends testProcessPaymentWithLevelup
     */
    function testProcessRefundWithLevelUpErrorAlreadyRefunded($uuid)
    {
        $level_up_payment_service = new LevelupPaymentService(array("merchant_id"=>$this->ids['merchant_id']));
        $level_up_payment_service->order_id = $this->ids['order_id'];
        $level_up_payment_service->uuid = $uuid;
        $results = $level_up_payment_service->processRefundForUUID($uuid);
        $this->assertEquals(422,$results['http_code']);
        $this->assertEquals("The refund was unsuccessful. Perhaps this order has already been refunded or was unsuccessful.",$results['response_text']);
    }

    function testProcessRefundWithLevelUpErrorBadUUID()
    {
        $level_up_payment_service = new LevelupPaymentService(array("merchant_id"=>$this->ids['merchant_id']));
        $level_up_payment_service->order_id = $this->ids['order_id'];
        $level_up_payment_service->uuid = "123456789876543234567";
        $results = $level_up_payment_service->processRefundForUUID($uuid);
        $this->assertEquals(404,$results['http_code']);
        $this->assertEquals("Unable to refund, Levelup UUID not validated by Levelup server.",$results['response_text']);
    }


    static function setUpBeforeClass()
	{
		$_SERVER['request_time1'] = microtime(true);
		$tz = date_default_timezone_get();
		$_SERVER['starting_tz'] = $tz;
		date_default_timezone_set(getProperty("default_server_timezone"));
		ini_set('max_execution_time',300);

		$menu_id = 102430;
		$brand_id = 833;
		$skin_resource = getOrCreateSkinAndBrandIfNecessary('yourpie','yourpie',80,$brand_id);
		$menu_adapter = new MenuAdapter($mimetypes);
		if ($menu_adapter->getRecord(array("menu_id"=>$menu_id))){
			; // do nothing
		} else {
			//create menu
			$menu_adapter->importMenu($menu_id,'prod','local',$brand_id);
		}
		mysqli_query("BEGIN");


		setContext('com.splickit.yourpie');
		$ids['skin_id'] = $skin_resource->skin_id;

		//map it to a menu

		$ids['menu_id'] = $menu_id;

		$merchant_resource = createNewTestMerchant($menu_id);
		attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
		$ids['merchant_id'] = $merchant_resource->merchant_id;


		$billing_entity_resource = createSageBillingEntity($merchant_resource->brand_id);

		$merchant_payment_type_map_adapter = new MerchantPaymentTypeMapsAdapter($mimetypes);
		$cc_merchant_payment_type_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 2000, $billing_entity_resource->id);
		$ids['cc_billing_entity_id'] = $cc_merchant_payment_type_resource->billing_entity_id;

		// create cash merchang payment type record
		$mptma_cash_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 1000, null);

		// create level up
		$sql = "INSERT INTO Splickit_Accepted_Payment_Types VALUES (6000,'Level Up','Level Up','leveluppaymentservice',NOW(),NOW(),'N')";
		$merchant_payment_type_map_adapter->_query($sql);
		$levelup_merchant_payment_type_map_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id,6000,null);
		$ids['levelup_mptm_id'] = $levelup_merchant_payment_type_map_resource->id;

		$mlima = new MerchantLevelupInfoMapsAdapter($m);
		//$merchant_token = "1988759-bd5b68cc176b70534303074f3e972b3bd30cc03adf5f914f8542027e200a81";
		//$ids['levelup_merchant_token'] = $merchant_token;
		$merchant_levelup_info_resource = Resource::createByData($mlima,array("merchant_id"=>$merchant_resource->merchant_id,"levelup_location_id"=>83,"levelup_merchant_token"=>$merchant_token));
		$ids['merchant_levelup_info_id'] = $merchant_levelup_info_resource->id;

		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$ids['user_id'] = $user_resource->user_id;

/*		$expected_payload = cleanUpDoubleSpacesCRLFTFromString(file_get_contents("./unit_tests/resources/expected_levelup_message_body.txt"));
		$expected_payload_array = json_decode($expected_payload,true);
		$ids['expected_payload_array'] = $expected_payload_array;
*/
		logTestUserResourceIn($user_resource);

		$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id, 'pickup', 'skip hours',2);
		$order_data['items'][1]['note'] = "Sum Dum Item Note";
		$order_data['tip'] = 0.00;
		$order_data['merchant_payment_type_map_id'] = $mptma_cash_resource->id;
		$order_resource = placeOrderFromOrderData($order_data,$time);
		$ids['order_id'] = $order_resource->order_id;

		$_SERVER['log_level'] = 5;
		$_SERVER['unit_test_ids'] = $ids;
	}
    
	static function tearDownAfterClass()
    {
    	mysqli_query("ROLLBACK");
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
	LevelupPosIntegrationTest::main();
}

?>