<?php
$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class LevelupTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
        $this->ids = $_SERVER['unit_test_ids'];
    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
        unset($this->stamp);
    }


    function testProcessPaymentWithLevelup()
    {
/*    this was only used during development to make sure the level up payment worked in production

        $level_up_payment_service = new LevelupPaymentService(array("merchant_id"=>$this->ids['merchant_id']));
        $level_up_payment_service->levelup_info['levelup_location_id'] = 9180;
        $data['levelup_user_token'] = "1976492-1d2c4f1c196668db863090055ced7a91167013444066bff036a763948a1a25";
        $filename = "./config/prod.conf";
        $txt = file_get_contents($filename);
        $prod_env_properties_array = json_decode($txt,true);
        foreach ($prod_env_properties_array as $row) {
            $prod_env_properties[$row['name']] = $row['value'];
        }
        $level_up_payment_service->url = $prod_env_properties['levelup_url'];
        $level_up_payment_service->merchant_token = $prod_env_properties['levelup_merchant_token'];
        $level_up_payment_service->loadAdditionalDataFieldsIfNeeded($data);
        $level_up_payment_service->order_id = $this->ids['order_id'];
        $results = $level_up_payment_service->processPayment(.50);
        $this->assertTrue(isset($results['uuid']),"should have processsed a valid order but didn't. error: ".$results['response_text']);
*/
    }



    static function setUpBeforeClass()
    {
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time',300);
        mysqli_query("BEGIN");

        $skin_resource = getOrCreateSkinAndBrandIfNecessary('yourpie','yourpie',80,333);
        setContext('com.splickit.yourpie');
        $ids['skin_id'] = $skin_resource->skin_id;

        //map it to a menu
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $ids['menu_id'] = $menu_id;

        $modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 2);
        $modifier_group_id = $modifier_group_resource->modifier_group_id;
        $item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[1]['item_id'], $modifier_group_id, 1);

        $merchant_resource = createNewTestMerchant($menu_id);
        attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
        $ids['merchant_id'] = $merchant_resource->merchant_id;

        $cm = new CompleteMenu($menu_id);
        $item_size_map_resources = $cm->getAllMenuItemSizeMapResources($menu_id,'Y',0);
        foreach ($item_size_map_resources as $item_size_resource) {
            $item_size_resource->price = .02;
            $item_size_resource->save();
        }

        $modifier_size_map_resources = $cm->getAllModifierItemSizeResources($menu_id,'Y',0);
        foreach ($modifier_size_map_resources as $mod_resource) {
            $mod_resource->modifier_price = .01;
            $mod_resource->save();
        }


        $billing_entity_resource = createSageBillingEntity($merchant_resource->brand_id);

        $merchant_payment_type_map_adapter = new MerchantPaymentTypeMapsAdapter($mimetypes);
        $cc_merchant_payment_type_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 2000, $billing_entity_resource->id);
        $ids['cc_billing_entity_id'] = $cc_merchant_payment_type_resource->billing_entity_id;

        // create cash merchang payment type record
        $mptma_cash_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 1000, null);

        // create level up
        $sql = "INSERT INTO Splickit_Accepted_Payment_Types VALUES (6000,'Level Up','Level Up','leveluppaymentservice',NOW(),NOW(),'N')";
        $merchant_payment_type_map_adapter->_query($sql);
        $levelup_merchant_payment_type_map_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id,6000,null);
        $ids['levelup_mptm_id'] = $levelup_merchant_payment_type_map_resource->id;

        $mlima = new MerchantLevelupInfoMapsAdapter($m);
        //$merchant_token = "1988759-bd5b68cc176b70534303074f3e972b3bd30cc03adf5f914f8542027e200a81";
        $ids['levelup_merchant_token'] = $merchant_token;
        $merchant_levelup_info_resource = Resource::createByData($mlima,array("merchant_id"=>$merchant_resource->merchant_id,"levelup_location_id"=>9180,"levelup_merchant_token"=>$merchant_token));
        $ids['merchant_levelup_info_id'] = $merchant_levelup_info_resource->id;

        $expected_payload = cleanUpDoubleSpacesCRLFTFromString(file_get_contents("./unit_tests/resources/expected_levelup_message_body.txt"));
        $expected_payload_array = json_decode($expected_payload,true);
        $ids['expected_payload_array'] = $expected_payload_array;

        $user_resource = createNewUser(array("flags"=>"1C20000001"));
        $ids['user_id'] = $user_resource->user_id;

        logTestUserResourceIn($user_resource);

        $order_adapter = new OrderAdapter($mimetypes);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id, 'pickup', 'skip hours',2);
        $order_data['items'][1]['note'] = "Sum Dum Item Note";
        $order_data['items'][1]['mods'][0]['mod_quantity'] = 2;
        $order_data['tip'] = 0.00;
        $order_data['merchant_payment_type_map_id'] = $mptma_cash_resource->id;
        $order_resource = placeOrderFromOrderData($order_data,$time);
        $ids['order_id'] = $order_resource->order_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        mysqli_query("ROLLBACK");
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    LevelupTest::main();
}