<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class PaymentLevelupTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
		setContext('com.splickit.yourpie');
		
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }
	
	function testGetPaymentServiceFromPaymentGod()
	{
		$payment_service = PaymentGod::paymentServiceFactoryByMerchantPaymentTypeMapId($this->ids['levelup_mptm_id'],null);
		$this->assertTrue(is_a($payment_service, 'LevelupPaymentService'),"Should have returned a level up payment service object");
	}

	function testGetLevelupMerchantInfo()
	{
		$levelup_payment_service = new LevelupPaymentService($data);
		$this->assertNull($levelup_payment_service->getMerchantLevelupInfo());
		$levelup_payment_service->merchant_id = $this->ids['merchant_id'];
		$levelup_info = $levelup_payment_service->getMerchantLevelupInfo();
		$this->assertEquals(83,$levelup_info['levelup_location_id']);
		$this->assertEquals($this->ids['levelup_merchant_token'],$levelup_info['levelup_merchant_token']);
	}

	function testSetSkuAndUpc()
	{
		$levelup_payment_service = new LevelupPaymentService(array("merchant_id"=>$merchant_id));
		$item = array("dummy"=>"value");
		$order_detail['external_id'] = "sku123:upc456";
		$levelup_payment_service->setSkuAndUpcIfAvailable($order_detail,$item);
		$this->assertCount(3,$item,"should have added the fields");
		$this->assertEquals("sku123",$item['sku']);
		$this->assertEquals("upc456",$item['upc']);
	}

	function testSetSkuAndUpcButOnlySku()
	{
		$levelup_payment_service = new LevelupPaymentService(array("merchant_id"=>$merchant_id));
		$item = array("dummy"=>"value");
		$order_detail['external_id'] = "sku890";
		$levelup_payment_service->setSkuAndUpcIfAvailable($order_detail,$item);
		$this->assertCount(2,$item,"should have added the fields");
		$this->assertEquals("sku890",$item['sku']);
		$this->assertFalse(isset($item['upc']));
	}

	function testSetSkuAndUpcNoExternal()
	{
		$levelup_payment_service = new LevelupPaymentService(array("merchant_id"=>$merchant_id));
		$item = array("dummy"=>"value");
		$order_detail['external_id'] = "";
		$levelup_payment_service->setSkuAndUpcIfAvailable($order_detail,$item);
		$this->assertCount(1,$item,"should not have added the fields");
	}

	function testGetLevelupPaymentPayload()
	{
		$merchant_id = $this->ids['merchant_id'];
		$order_id = $this->ids['order_id'];

		$levelup_payment_service = new LevelupPaymentService(array("merchant_id"=>$merchant_id));
		$payload_array = $levelup_payment_service->buildLevelupOrderPaymentPayloadFromOrderId($order_id);
		$this->assertTrue(is_array($payload_array),"Payload should have been an array");
		$this->assertCount(1,$payload_array,"Should have been an array with 1 top level");

		return $payload_array;
	}

	/**
	 * @depends testGetLevelupPaymentPayload
	 */
	function testCheckNoModifierChild($payload_array)
	{
		$expected_payload_array = $this->ids['expected_payload_array'];
		$this->assertEquals($expected_payload_array['items'][0],$payload_array['items'][0]);
	}

	/**
	 * @depends testGetLevelupPaymentPayload
	 */
	function testCheckChildWithModifiers($payload_array)
	{
		$expected_payload_array = $this->ids['expected_payload_array'];
		$this->assertEquals($expected_payload_array['items'][1],$payload_array['items'][1]);
	}

	/**
	 * @depends testGetLevelupPaymentPayload
	 */
	function testCheckTotalPayload($payload_array)
	{
		$expected_payload_array = $this->ids['expected_payload_array'];
		$expected_payload_array['order']['identifier_from_merchant'] = $this->ids['order_id'];
        $this->assertNotNull($payload_array['order']['items'][1]['item']['children'],"there should be a child section");
        $this->assertEquals($expected_payload_array['order']['items'][1]['item']['children'],$payload_array['order']['items'][1]['item']['children'],"the children array shoudl be equal");
		$this->assertEquals($expected_payload_array,$payload_array);
	}

	function testSetLevelUpResponse()
	{
		$lups = new LevelupPaymentService($data);
		$json = '{"order": {"uuid": "1a2b3c4d5e6f7g8h9i9h8g7f6e5d4c3b2a1","spend_amount": 10,"tip_amount": 0,"total_amount": 10}}';
		$response['raw_result'] = $json;
		$response['http_code'] = 200;
		$result_array = $lups->processCurlResponse($response);
		$return_array = $lups->formatSuccessForBackwardsCompatabilty($result_array);
		$this->assertEquals("1a2b3c4d5e6f7g8h9i9h8g7f6e5d4c3b2a1",$return_array['transactionid']);
		$this->assertEquals(100,$return_array['response_code']);
	}

	function testGetLevelupAuthInfo()
	{
		$merchant_token = generateCode(20);
		$user_token = generateCode(15);
		$lups = new LevelupPaymentService($data);
		$auth_string = $lups->createAuthHeader($merchant_token,$user_token);
		$expected_auth_string = 'Authorization: token merchant="'.$merchant_token.'",user="'.$user_token.'"';
		$this->assertEquals($expected_auth_string,$auth_string,"Auth string creation error");
	}

	function testGetLevelupPaymentTypeForMerchant()
	{
		$merchant_id = $this->ids['merchant_id'];

		$user = logTestUserIn($this->ids['user_id']);

		$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours',10);
		$place_order_controller = new PlaceOrderController($mt, $user, $r,5);
		$checkout_data_resource = $place_order_controller->getCheckoutDataFromOrderData($order_data);
		$this->assertNotNull($checkout_data_resource->accepted_payment_types,"Should have found the payment array");
		$payments_array = $checkout_data_resource->accepted_payment_types;
		$payments_hash = createHashmapFromArrayOfArraysByFieldName($payments_array,'name');
		$this->assertFalse(isset($payments_hash['LevelUp']),'there should NOT have been a level up payment sent becuase no levelup_user_token was submitted with the order');

        $order_data['levelup_user_token'] = generateCode(20);
        $checkout_data_resource = $place_order_controller->getCheckoutDataFromOrderData($order_data);
        $this->assertNotNull($checkout_data_resource->accepted_payment_types,"Should have found the payment array");
        $payments_array = $checkout_data_resource->accepted_payment_types;
        $payments_hash = createHashmapFromArrayOfArraysByFieldName($payments_array,'name');
        $this->assertTrue(isset($payments_hash['LevelUp']),'there should have been a level up payment type associated with this merchant');
        $this->assertCount(3, $payments_hash, "there should have been three payment typs available");
	}

	function testOrderWithLevelupNoUserToken()
	{
		$merchant_id = $this->ids['merchant_id'];
		$user = logTestUserIn($this->ids['user_id']);
		$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours');
//		$order_data['tip'] = 0.00;
//		$order_data['merchant_payment_type_map_id'] = $this->ids['levelup_mptm_id'];
		$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
		$this->assertNotNull($order_resource->error,"should have thrown an error");
		$this->assertEquals("We're sorry, but there is a problem with your LevelUp authentication and payment cannot be processed at this time.",$order_resource->error);
	}


	function testCompleteOrderWithLevelupAsPayment()
	{
		$merchant_id = $this->ids['merchant_id'];

		$starting_balance = -15.45;
		$user_resource = createNewUser();
		$user_id = $user_resource->user_id;
		setUserBalance($user_id,$starting_balance);
		$user = logTestUserIn($user_id);
		$balance_before = $user['balance'];
		$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours');
		$order_data['tip'] = 0.00;
		$order_data['levelup_user_token'] = "ASDFGHJKL123456789";

		$order_data['merchant_payment_type_map_id'] = $this->ids['levelup_mptm_id'];
		$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
		$order_id = $order_resource->order_id;
		$this->assertNull($order_resource->error,"should not have gotten an order error");
		$this->assertTrue($order_resource->order_id > 1000);
		$this->assertEquals('LevelupPaymentService', $order_resource->payment_service_used);

		$balance_change_adapter = new BalanceChangeAdapter($mimetypes);
		if ($balance_change_records = $balance_change_adapter->getRecords(array("order_id"=>$order_id), $options)) {
			$balance_change_rows_by_user_id = setBalanceChangeHashFromBalanceChangeArrayFromOrder($balance_change_records);
		}
		$this->assertCount(2, $balance_change_rows_by_user_id);
		$this->assertEquals($balance_before, $balance_change_rows_by_user_id["$user_id-Order"]['balance_before']);
		$this->assertEquals($order_resource->grand_total, -$balance_change_rows_by_user_id["$user_id-Order"]['charge_amt']);
		$this->assertEquals($balance_before-$order_resource->grand_total, $balance_change_rows_by_user_id["$user_id-Order"]['balance_after']);

		$this->assertEquals($balance_before-$order_resource->grand_total, $balance_change_rows_by_user_id["$user_id-Levelup"]['balance_before']);
		$this->assertEquals($order_resource->grand_total, number_format($balance_change_rows_by_user_id["$user_id-Levelup"]['charge_amt'],2));
        $this->assertEquals(LevelupPaymentService::SERVICE_NAME, $balance_change_rows_by_user_id["$user_id-Levelup"]['process']);
        $this->assertEquals(LevelupPaymentService::SERVICE_NAME, $balance_change_rows_by_user_id["$user_id-Levelup"]['cc_processor'],"Level up shoudl be set at the cc processor");

        $new_user_resource = SplickitController::getResourceFromId($user_id, 'User');
		$this->assertEquals($starting_balance, $balance_change_rows_by_user_id["$user_id-Levelup"]['balance_after'],"Balance before should have stayed since level up should take care of credit");
		$this->assertEquals($starting_balance, $new_user_resource->balance);
		return $order_id;
	}

    /**
     * @depends testCompleteOrderWithLevelupAsPayment
     */
    function testRefundOfLevelupPayment($order_id)
    {
        $order_resource = SplickitController::getResourceFromId($order_id, "Order");
        $user = logTestUserIn($order_resource->user_id);
        $request = new Request();
        $the_refund_note = "Guy didn't like the pizza.";
        $request->data['note'] = $the_refund_note;
        $order_controller = new OrderController($mt, $user, $request, 5);
        $refund_results = $order_controller->issueOrderRefund($order_id, "0.00");
        $this->assertEquals("success", $refund_results['result']," should have gotten a success but: ".$refund_results['message']);
        $this->assertEquals(100,$refund_results['response_code']);

        // check to see if order was changed to cancelled
        $order_adapter = new OrderAdapter($mimetypes);
        $new_order_resource = Resource::find($order_adapter,''.$order_id);
        $this->assertEquals('N', $new_order_resource->status);

        // check to see if the messages were cancelled

        $message_data['order_id'] = $order_id;
        $message_options[TONIC_FIND_BY_METADATA] = $message_data;
        $message_resources = Resource::findAll(new MerchantMessageHistoryAdapter($mimetypes),'',$message_options);
        $this->assertTrue(sizeof($message_resources) > 0);
        foreach ($message_resources as $message_resource) {
            $this->assertEquals('C', $message_resource->locked);
        }

        // now check the balance change table and the order_reversal table

        $balance_change_resource = Resource::find(new BalanceChangeAdapter($mimetypes),''.$refund_results['balance_change_id']);
        $this->assertEquals($expected, $actual);
        $this->assertEquals($order_resource->grand_total,$balance_change_resource->charge_amt);
        $this->assertEquals('LevelupRefund',$balance_change_resource->process);
        $this->assertEquals("Issuing a LevelupPaymentService REFUND from the API: ".$the_refund_note,$balance_change_resource->notes);

		// There SHOULD be a record in the admin reversal for external refunds
        $adm_reversal_resource = Resource::find(new AdmOrderReversalAdapter($mimetypes),''.$refund_results['order_reversal_id']);
        $this->assertNotNull($adm_reversal_resource);
    }

	function testFailureOfLevelupCharge()
	{
		$merchant_id = $this->ids['merchant_id'];


		$user = logTestUserIn($this->ids['user_id']);
		$balance_before = $user['balance'];
		$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours');
		$order_data['tip'] = 20.00;
		$order_data['levelup_user_token'] = "ASDFGHJKL123456789";

		$order_data['merchant_payment_type_map_id'] = $this->ids['levelup_mptm_id'];
		$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
		$this->assertNotNull($order_resource->error,"should have thrown an error");
		$this->assertEquals("Sorry. We cannot charge the credit card at this time.",$order_resource->error);

	}

	function testOrderWithLevelUpNoMerchantLocationId()
	{
		$mlui_resource = Resource::find(new MerchantLevelupInfoMapsAdapter($m),''.$this->ids['merchant_levelup_info_id']);
		$mlui_resource->levelup_location_id = "";
		$mlui_resource->save();

		$merchant_id = $this->ids['merchant_id'];


		$user = logTestUserIn($this->ids['user_id']);
		$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours');
		$order_data['tip'] = 0.00;
		$order_data['levelup_user_token'] = "1234NOLOCATIONIDERROR1234";

		$order_data['merchant_payment_type_map_id'] = $this->ids['levelup_mptm_id'];
		$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
		$this->assertNotNull($order_resource->error,"should have thrown an error");
		$this->assertEquals("We're sorry, there is a configuration problem with this merchant. We have now been notified and will address it immediately.",$order_resource->error);
	}



	static function setUpBeforeClass()
    {
    	$_SERVER['request_time1'] = microtime(true);
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	ini_set('max_execution_time',300);
    	      $db = DataBase::getInstance(); $mysqli = $db->getConnection();$mysqli->begin_transaction(); ;

		$skin_resource = getOrCreateSkinAndBrandIfNecessary('yourpie','yourpie',80,333);
    	setContext('com.splickit.yourpie');
    	$ids['skin_id'] = $skin_resource->skin_id;
    	
		//map it to a menu
    	$menu_id = createTestMenuWithNnumberOfItems(2);
    	$ids['menu_id'] = $menu_id;

		$cm = new CompleteMenu($menu_id);
		$item_size_map_resources = $cm->getAllMenuItemSizeMapResources($menu_id,'Y',0);
		$item_size_map_resources[0]->external_id = "sku12345:upc67890";
		$item_size_map_resources[0]->save();
		$item_size_map_resources[1]->external_id = "sku88888";
		$item_size_map_resources[1]->save();

		$modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 2);
    	$modifier_group_id = $modifier_group_resource->modifier_group_id;
    	$item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[1]['item_id'], $modifier_group_id, 1);

    	$merchant_resource = createNewTestMerchant($menu_id,array("no_payment"=>true));
    	attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
    	$ids['merchant_id'] = $merchant_resource->merchant_id;

		$billing_entity_resource = createSageBillingEntity($merchant_resource->brand_id);

		$merchant_payment_type_map_adapter = new MerchantPaymentTypeMapsAdapter($mimetypes);
		$cc_merchant_payment_type_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 2000, $billing_entity_resource->id);
		$ids['cc_billing_entity_id'] = $cc_merchant_payment_type_resource->billing_entity_id;

		// create cash merchang payment type record
		$merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id, 1000, null);

		// create level up
		$sql = "INSERT INTO Splickit_Accepted_Payment_Types VALUES (6000,'LevelUp','LevelUp','leveluppaymentservice',NOW(),NOW(),'N')";
		$merchant_payment_type_map_adapter->_query($sql);
		$levelup_merchant_payment_type_map_resource = $merchant_payment_type_map_adapter->createMerchantPaymentTypeMap($merchant_resource->merchant_id,6000,null);
		$ids['levelup_mptm_id'] = $levelup_merchant_payment_type_map_resource->id;

		$mlima = new MerchantLevelupInfoMapsAdapter($m);
		//$merchant_token = generateCode(20);
		//$ids['levelup_merchant_token'] = $merchant_token;
		$merchant_levelup_info_resource = Resource::createByData($mlima,array("merchant_id"=>$merchant_resource->merchant_id,"levelup_location_id"=>99,"levelup_merchant_token"=>$merchant_token));
		$ids['merchant_levelup_info_id'] = $merchant_levelup_info_resource->id;

		$expected_payload = cleanUpDoubleSpacesCRLFTFromString(file_get_contents("./unit_tests/resources/expected_levelup_message_body.txt"));
		$expected_payload_array = json_decode($expected_payload,true);
		$ids['expected_payload_array'] = $expected_payload_array;

		$user_resource = createNewUser(array("flags"=>"1C20000001"));
    	$ids['user_id'] = $user_resource->user_id;

		logTestUserResourceIn($user_resource);

		$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id, 'pickup', 'skip hours',2);
		$order_data['items'][1]['note'] = "Sum Dum Item Note";
		$order_data['items'][1]['mods'][1]['mod_quantity'] = 2;
        $mods = $order_data['items'][1]['mods'];
        if ($mods[0]['modifier_item_id'] > $mods[1]['modifier_item_id']) {
            $new_mods[] = $mods[1];
            $new_mods[] = $mods[0];
            $order_data['items'][1]['mods'] = $new_mods;
        }
		$order_data['tip'] = 0.00;
		$order_resource = placeOrderFromOrderData($order_data,$time);
		$ids['order_id'] = $order_resource->order_id;
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id,$m);
		$_SERVER['log_level'] = 5;
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	$db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
	PaymentLevelupTest::main();
}

?>