<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class PrinterSwapTest extends PHPUnit_Framework_TestCase
{
	var $swap_map_adapter;
	var $stamp;	
	var $ids;
	
	function setUp()
	{
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->ids = $_SERVER['unit_test_ids'];
		$this->swap_map_adapter = new MerchantPrinterSwapMapAdapter($mimetypes);
		setProperty("new_shadow_device_on", "false");
	}
	
	function tearDown() 
	{
		// delete your instance
		unset($this->swap_map_adapter);
		unset($this->stamp);
		unset($this->ids);
		$_SERVER['STAMP'] = $this->stamp;
		setProperty("new_shadow_device_on", "true");
	}
		
    function testSetItUp()
    {
        $merchant_id = $this->ids['merchant_id'];
		$sql = "INSERT INTO `Merchant_Message_Map` (
					`map_id` ,
					`merchant_id` ,
					`message_format` ,
					`delivery_addr` ,
					`delay` ,
					`message_type` ,
					`info` ,
					`message_text` ,
					`created` ,
					`modified` ,
					`logical_delete`
					)
					VALUES (
					NULL , '$merchant_id', 'P', '173.194.73.103', '-1', 'A', NULL , NULL , NOW(), '0000-00-00 00:00:00', 'N'
					), (
					NULL , '$merchant_id', 'T', '1234567890', '-1', 'A', NULL , '***5', NOW(), '0000-00-00 00:00:00', 'N'
					), (
					NULL , '$merchant_id', 'GUW', 'gprs', '-1', 'X', NULL , NULL , NOW(), '0000-00-00 00:00:00', 'N'
					)";
		$result = $this->swap_map_adapter->_query($sql);
		
		$sql = "DELETE FROM Merchant_Printer_Swap_Map WHERE merchant_id = $merchant_id";
		$result = $this->swap_map_adapter->_query($sql);
		
		// delete all message for merchant_id = 1003
		$sql = "DELETE FROM Merchant_Message_History WHERE merchant_id = $merchant_id";
		$this->swap_map_adapter->_query($sql);
		
		// add a record to the swap table.
		$data['merchant_id'] = $merchant_id;
		$data['new_sms_no'] = '3038844083';
		$data['live'] = 'N';
		$resource = Resource::factory($this->swap_map_adapter,$data);
		$resource->save();
		$this->assertTrue($resource->_exists);
		
		return true;
    }
    
     /** 
     * @depends testSetItUp
     */
 
    function testNoOrderReturnBio($result)
    {    	
    	$merchant_id = $this->ids['merchant_id'];
    	$numeric_id = $this->ids['numeric_id'];
    	
    	$request = new Request();
    	$request->url = "/m/g/$numeric_id/gprs/f2.txt";
    	$gprs_controller = new GprsController($mt, $u, $request,5);
    	
    	$message_resource = $gprs_controller->pullNextMessageResourceByMerchant($merchant_id);
    	// in addition to pulling a bio message a new message should  have gotten created for the new firmware printer to pick up next.  TEXT and GPRS 
    	// new message text would be "This is a test message::to make sure your printer::is functioning correctly::Please ignore."
    	$this->assertEquals("GUB", $message_resource->message_format);
  
    	$ots = $message_resource->bio_message_order_id;
    	$this->assertEquals("#$merchant_id*1*bio".$ots."**::name:  Unit Test Merchant::addr:  1505 Arapaho Ave::city:  boulder::phone: 1234567890::email: dummy@order140.com:: ::;*#", $message_resource->message_text);
    	
    	return $ots;
    	
    }
    
    /** 
     * @depends testNoOrderReturnBio
     */
    function testCreatedGPRSMessageAndTextMessage($order_id)
    {
    	$merchant_id = $this->ids['merchant_id'];
    	$numeric_id = $this->ids['numeric_id'];
		
    	$text_controller = new TextController($mt, $u, $r, 5);
    	$mmha_options[TONIC_FIND_BY_METADATA]['merchant_id'] = $merchant_id;
    	$text_message_resource = $text_controller->getNextMessageResourceForSend($mmha_options);
    	
   		$this->assertEquals('3038844083', $text_message_resource->message_delivery_addr);
   		//$this->assertEquals($order_id, $text_message_resource->order_id);
   		
   		$text_controller->markMessageDelivered($text_message_resource);

   		$request = new Request();
    	$request->url = "/m/g/$numeric_id/gprs/f2.txt";
    	$gprs_controller = new GprsController($mt, $u, $request,5);
    	$message_resource = $gprs_controller->pullNextMessageResourceByMerchant($merchant_id);
    	$this->assertEquals('Y', $message_resource->locked);
    	// had to do this because fo the possible 1 second delay
    	$message_order_id = $message_resource->order_id;
    	$this->assertTrue($order_id == $message_order_id || $order_id == $message_order_id-1);
//    	$this->assertEquals($order_id, $message_resource->order_id);
   		
    	$gprs_controller->markMessageDelivered($message_resource);
    	
    	$request = new Request();
    	$request->url = "/messagemanager/$numeric_id/gprs/callback.txt";
    	$request->data['o'] = $order_id;
    	$gprs_controller = new GprsController($mt, $u, $request,5);
    	$gprs_controller->callBack($numeric_id);
    	
    	// test to make sure the message got marked as viewed.
    	$mmha = new MerchantMessageHistoryAdapter($mimetypes);
    	$completed_gprs_resource = Resource::find($mmha,$message_resource->map_id);
    	
    	$this->assertNotNull($completed_gprs_resource);
    	$this->assertEquals('V', $completed_gprs_resource->viewed);
    	
    	return true;
    	
    }
    
    /** 
     * @depends testCreatedGPRSMessageAndTextMessage
     */
    function testNoMessageSteal($result)
    {		
    	$merchant_id = $this->ids['merchant_id'];
    	$numeric_id = $this->ids['numeric_id'];
    	
		$user = logTestUserIn(101);    	
    	// first create an order with messages
    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours');
    	$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
		$this->assertNotNull($order_resource);
		$this->assertTrue($order_resource->order_id > 1000,"errer: ".$order_resource->error);
		
		$order_id = $order_resource->order_id;
		$order_id_length = strlen($order_id);
    	
    	$merchant_message_history_adapter = new MerchantMessageHistoryAdapter($mimetypes);
		$mmh_options[TONIC_FIND_BY_METADATA]['order_id'] = $order_id;
		$mmh_options[TONIC_FIND_BY_METADATA]['message_format'] = array("NOT LIKE"=>"E%"); // need to account for the conf and orders all emails now
		$results = Resource::findAll($merchant_message_history_adapter,'',$mmh_options);
		
		$this->assertEquals(3, sizeof($results, $mode));
    		
    	//sleep(1);
    	$request = new Request();
    	$request->url = "/m/g/$numeric_id/gprs/f2.txt";
    	$gprs_controller = new GprsController($mt, $u, $request,5);
    	$message_resource = $gprs_controller->pullNextMessageResourceByMerchant($merchant_id);

    	myerror_log("F2 pulled message is: ".$message_resource->message_text);
 		$this->assertEquals('P', $gprs_controller->message_resource->locked);   
 		$this->assertEquals('#'.$merchant_id.'*1*'.$order_id.'**', substr($message_resource->message_text,0,12+$order_id_length));

 		//now simulate the real printer picking up the message
 		$request = new Request();
 		$request->url = "/m/g/$numeric_id/gprs/f.txt";
 		$gprs_controller = new GprsController($mt, $u, $request,5);
    	$message_resource = $gprs_controller->pullNextMessageResourceByMerchant($merchant_id);
    	
    	$this->assertEquals('Y', $gprs_controller->message_resource->locked);
    	
    	$body = $message_resource->message_text;
    	$this->assertEquals('#'.$merchant_id.'*'.$order_id.'*', substr($body,0,9+$order_id_length));
		
    	$gprs_controller->markMessageDelivered();
		$this->assertEquals('S', $gprs_controller->message_resource->locked, 'Should have marked the message as SENT');
		
		return true;
    }
    
    // now test the swap.  first set the flag to live on the swap table.  and call in for a message.
    /** 
     * @depends testNoMessageSteal
     */
    function testReconfigureMessageMap($result)
    {
    	$merchant_id = $this->ids['merchant_id'];
    	$numeric_id = $this->ids['numeric_id'];

    	// first set record to live
    	$swap_map_resource = Resource::find($this->swap_map_adapter,"".$merchant_id);
    	$swap_map_resource->live = 'Y';
    	$swap_map_resource->save();
    	
    	// now call in for a message
    	$request = new Request();
    	$request->url = "/m/g/$numeric_id/gprs/f2.txt";
    	$gprs_controller = new GprsController($mt, $u, $request,5);
    	
    	//this shoujld get the bio, reconfigure the maps, and set the merchatn to active.
    	$ts = time();

        // unset the values to mimic what happens when a printer calls in
        unset($_SERVER['SKIN']);
        unset($_SERVER['SKIN_ID']);
        unset($_SERVER['BRAND']);
        unset($_SERVER['HTTP_X_SPLICKIT_CLIENT_ID']);

        $gprs_bio_message = $gprs_controller->pullNextMessageResourceByMerchant($merchant_id);
    	$ots = $gprs_bio_message->bio_message_order_id;
    	
    	$merchant_resource = SplickitController::getResourceFromId($merchant_id, "Merchant");
    	$this->assertEquals('Y', $merchant_resource->active);
    	
		$this->assertEquals("GUB", $gprs_bio_message->message_format);
    	//$this->assertEquals("#1003*1*bio".$ots."**::name:  Corner Coffee Bar::addr:  2130 Mountain View Ave::city:  Longmont::phone: 720-684-6003::email: rtsnooze@aol.com:: ::;*#", $gprs_bio_message->message_text);
 		$this->assertEquals("#$merchant_id*1*bio".$ots."**::name:  Unit Test Merchant::addr:  1505 Arapaho Ave::city:  boulder::phone: 1234567890::email: dummy@order140.com:: ::;*#", $gprs_bio_message->message_text);
    	
    	$test_order_id = $gprs_bio_message->test_order_id;
    	$this->assertTrue($test_order_id > 1000);
    	
    	// check to see if the maps have been reconfigured.
    	$merchant_message_map_adapter = new MerchantMessageMapAdapter($mimetypes);
		$messages = $merchant_message_map_adapter->getRecords(array("merchant_id"=>$merchant_id));
		foreach ($messages as $message_resource)
		{
			$message_format = $message_resource->message_format;
			if ($message_format == 'T')
			{
				$this->assertEquals('3038844083', $message_resource->delivery_addr);
				$this->assertEquals('0', $message_resource->delay);
				$this->assertEquals('firmware=8.0', $message_resource->info);
			} else if (substr($message_format, 0,1) == 'G') {
				$this->assertEquals('GUW', $message_format);
				$this->assertEquals('0', $message_resource->delay);
				$this->assertEquals('firmware=8.0', $message_resource->info);
			} else if ($message_format == 'P') {
				$this->assertEquals('Y', $message_resource->logical_delete);
			}
		}
		
		// now check to see if the text and gprs messages were actually created
		$mmh_adapter = new MerchantMessageHistoryAdapter($this->mimetypes);
		$new_order_message_data['order_id'] = $test_order_id;
		$ts = time() + 5;
		$date_string = date('Y-m-d H:i:s',$ts);
		$new_order_message_data['next_message_dt_tm'] = array("<"=>$date_string);
		$nom_options[TONIC_FIND_BY_METADATA] = $new_order_message_data;
		//$new_messages = Resource::findAll($mmh_adapter,null,$nom_options);
		$new_messages = MerchantMessageHistoryAdapter::getAllOrderMessages($test_order_id);
        $this->assertTrue(sizeof($new_messages) > 1);
		foreach ($new_messages as $new_message_resource)
		{
            $send_on_dt_string = date("Y-m-d H:i:s",$new_message_resource->next_message_dt_tm);
            myerror_log("send on for test message: $send_on_dt_string");
            $this->assertTrue($new_message_resource->next_message_dt_tm < time() + 10,"it should schedule the message for now");
			$message_format = $new_message_resource->message_format;
			if ($message_format == 'T')
			{
				$this->assertEquals('3038844083', $new_message_resource->message_delivery_addr);
				$text_exists = true;
			} else if (substr($message_format, 0,1) == 'G') {
				$this->assertEquals('GUW', $message_format);
				$gprs_exists = true;
			}
		}
		
		$this->assertTrue($text_exists);
		$this->assertTrue($gprs_exists);
		
		return true;
    }
    
    // now test the fail of a version 5 printer calling in for version 8 messages
    /** 
     * @depends testReconfigureMessageMap
     */
   
    function testBadConfiguration($result)
    {
    	$merchant_id = $this->ids['merchant_id'];
    	$numeric_id = $this->ids['numeric_id'];
    	
    	// first create a test order
    	$place_order_controller = new PlaceOrderController($mimetypes, $user, $request,5);
		$order_resource = $place_order_controller->placeSimpleTestOrder($merchant_id, 'pickup', 'skip hours');
	   	$request = new Request();
    	$request->url = "/m/g/$numeric_id/gprs/f.txt";
    	$gprs_controller = new GprsController($mt, $u, $request,5);
    	$message_resource = $gprs_controller->pullNextMessageResourceByMerchant($merchant_id);
    	// email should have gotten sent out to me and resource should not have the 2 in it
    	$format_index = substr($message_resource->message_format, -1);
    	
    	$this->assertNotEquals('2', $format_index);	
    }

    static function setUpBeforeClass()
    {
    	$_SERVER['request_time1'] = microtime(true);
    	$tz = date_default_timezone_get();
    	$_SERVER['default_tz'] = $tz;
    	date_default_timezone_set("America/Denver");
    	ini_set('max_execution_time',300);
    	      SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();$mysqli->begin_transaction(); ;
    	
    	$skin_resource = createWorldHqSkin();
    	$ids['skin_id'] = $skin_resource->skin_id;
    	
		//map it to a menu
    	$menu_id = createTestMenuWithNnumberOfItems(5);
    	$ids['menu_id'] = $menu_id;
    	
    	$modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 10);
    	$modifier_group_id = $modifier_group_resource->modifier_group_id;
    	$item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 2);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[1]['item_id'], $modifier_group_id, 4);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[2]['item_id'], $modifier_group_id, 1);

    	$merchant_resource = createNewTestMerchant($menu_id);
    	$merchant_resource->active = 'N';
    	$merchant_resource->save();
    	attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
    	$ids['merchant_id'] = $merchant_resource->merchant_id;
    	$ids['numeric_id'] = $merchant_resource->numeric_id;
    	
    	$user_resource = createNewUser(array("flags"=>"1C20000001"));
    	$ids['user_id'] = $user_resource->user_id;
    	
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
    }
    
	/* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (true && !defined('PHPUnit_MAIN_METHOD')) {
    PrinterSwapTest::main();
}

?>