<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class PrinterSwapTestEpsonIP extends PHPUnit_Framework_TestCase
{
	var $swap_map_adapter;
	var $stamp;	
	var $ids;
	
	function setUp()
	{
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->ids = $_SERVER['unit_test_ids'];
		$this->swap_map_adapter = new MerchantPrinterSwapMapAdapter($mimetypes);
		setProperty("new_shadow_device_on", "false");
	}
	
	function tearDown() 
	{
		// delete your instance
		unset($this->swap_map_adapter);
		unset($this->stamp);
		unset($this->ids);
		$_SERVER['STAMP'] = $this->stamp;
		setProperty("new_shadow_device_on", "true");
	}
		
    function testSetItUp()
    {
    	$merchant_id = $this->ids['merchant_id'];
		$sql = "INSERT INTO `Merchant_Message_Map` (
					`map_id` ,
					`merchant_id` ,
					`message_format` ,
					`delivery_addr` ,
					`delay` ,
					`message_type` ,
					`info` ,
					`message_text` ,
					`created` ,
					`modified` ,
					`logical_delete`
					)
					VALUES (
					NULL , '$merchant_id', 'P', '173.194.73.103', '-1', 'A', NULL , NULL , NOW(), '0000-00-00 00:00:00', 'N'
					), (
					NULL , '$merchant_id', 'T', '1234567890', '-1', 'A', NULL , '***5', NOW(), '0000-00-00 00:00:00', 'N'
					), (
					NULL , '$merchant_id', 'GUW', 'gprs', '-1', 'X', NULL , NULL , NOW(), '0000-00-00 00:00:00', 'N'
					)";
		$result = $this->swap_map_adapter->_query($sql);
		
		$sql = "DELETE FROM Merchant_Printer_Swap_Map WHERE merchant_id = $merchant_id";
		$result = $this->swap_map_adapter->_query($sql);
		
		// delete all message for merchant_id = 1003
		$sql = "DELETE FROM Merchant_Message_History WHERE merchant_id = $merchant_id";
		$this->swap_map_adapter->_query($sql);
		
		// add a record to the swap table.
		$data['merchant_id'] = $merchant_id;
		$data['new_sms_no'] = 'ip printer';
		$data['live'] = 'N';
		$resource = Resource::factory($this->swap_map_adapter,$data);
		$resource->save();
		$this->assertTrue($resource->_exists);
		
		return true;
    }
    
     /** 
     * @depends testSetItUp
     */
 
    function testNoOrderReturnBio($result)
    {    	
    	$merchant_id = $this->ids['merchant_id'];
    	$numeric_id = $this->ids['numeric_id'];
    	
    	$request = new Request();
    	$request->url = "/messagemanager/getnextmessagebymerchantid/$numeric_id/epsonprinter/message.txt";
    	$message_controller = ControllerFactory::generateFromUrl($request->url, $mimetypes, $user, $request, 5);
    	
    	$message_resource = $message_controller->pullNextMessageResourceByMerchant($numeric_id);
    	// in addition to pulling a bio message a new message should  have gotten created for the new firmware printer to pick up next. 
    	// new message text would be "This is a test message::to make sure your printer::is functioning correctly::Please ignore."
    	$this->assertEquals("SUB", $message_resource->message_format);
  
    	$ots = $message_resource->bio_message_order_id;
        $this->assertContains('<text>name:  Unit Test Merchant&#10;</text>',$message_resource->message_text);
        $this->assertContains('<text>addr:  1505 Arapaho Ave&#10;</text>',$message_resource->message_text);
        $this->assertContains('<text>city:  boulder&#10;</text>',$message_resource->message_text);
        $this->assertContains('<text>phone: 1234567890&#10;</text>',$message_resource->message_text);
        $this->assertContains('<text>email: dummy@order140.com&#10;</text>',$message_resource->message_text);

        // make sure we did not create a test message.
        $next_message_resource = $message_controller->pullNextMessageResourceByMerchant($numeric_id);
        $this->assertEquals('SUB',$next_message_resource->message_format,"should just get another bio");

        return $ots;
    }
    
    /** 
     * @depends testNoOrderReturnBio
     */
    function testCreatedEpsonIPMessage($order_id)
    {
        // not sure what this test would do for the epson

//    	$merchant_id = $this->ids['merchant_id'];
//    	$numeric_id = $this->ids['numeric_id'];
//
//   		$request = new Request();
//    	$request->url = "/messagemanager/getnextmessagebymerchantid/$numeric_id/epsonprinter/message.txt";
//    	$message_controller = ControllerFactory::generateFromUrl($request->url, $mimetypes, $user, $request, 5);
//   		$message_resource = $message_controller->pullNextMessageResourceByMerchant($numeric_id);
//    	$this->assertEquals('Y', $message_resource->locked);
//    	// had to do this because fo the possible 1 second delay
//    	$message_order_id = $message_resource->order_id;
//    	$this->assertTrue($order_id == $message_order_id || $order_id == $message_order_id-1);
//    	$this->assertEquals($order_id, $message_resource->order_id);
   		
//    	$message_controller->markMessageDelivered($message_resource);
    	
//    	$request = new Request();
//    	$request->url = "/messagemanager/$numeric_id/epsonprinter/callback.txt";
//    	$request->data['o'] = $order_id;
//    	$message_controller2 = ControllerFactory::generateFromUrl($request->url, $mimetypes, $user, $request, 5);
//    	$message_controller2->callBack($numeric_id);
//
//    	// test to make sure the message got marked as viewed.
//    	$mmha = new MerchantMessageHistoryAdapter($mimetypes);
//    	$completed_gprs_resource = Resource::find($mmha,$message_resource->map_id);
//
//    	$this->assertNotNull($completed_gprs_resource);
//    	$this->assertEquals('V', $completed_gprs_resource->viewed);
    	
    	return true;
    	
    }
    
    /** 
     * @depends testCreatedEpsonIPMessage
     */
    function testNoMessageSteal($result)
    {		
    	$merchant_id = $this->ids['merchant_id'];
    	$numeric_id = $this->ids['numeric_id'];
    	
		$user = logTestUserIn(101);    	
    	// first create an order with messages
    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours');
    	$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
        $this->assertNotNull($order_resource);
        $this->assertNull($order_resource->error);
		$this->assertTrue($order_resource->order_id > 1000,"errer: ".$order_resource->error);
		
		$order_id = $order_resource->order_id;
		$order_id_length = strlen($order_id);
    	
    	$merchant_message_history_adapter = new MerchantMessageHistoryAdapter($mimetypes);
		$mmh_options[TONIC_FIND_BY_METADATA]['order_id'] = $order_id;
		$mmh_options[TONIC_FIND_BY_METADATA]['message_format'] = array("NOT LIKE"=>"E%"); // need to account for the conf and orders all emails now
		$results = Resource::findAll($merchant_message_history_adapter,'',$mmh_options);
		
		$this->assertEquals(3, sizeof($results, $mode));
    		
    	//sleep(1);
    	$request = new Request();
    	$request->url = "/messagemanager/getnextmessagebymerchantid/$numeric_id/epsonprinter/message.txt";
    	$message_controller = ControllerFactory::generateFromUrl($request->url, $mimetypes, $user, $request, 5);
    	// should grab a bio and not pick up the message (which it really couldn't since they're differnt formats now)
   		$message_resource = $message_controller->pullNextMessageResourceByMerchant($numeric_id);
   		$this->assertEquals('SUB', $message_resource->message_format);

 		//now simulate the real printer picking up the message
 		$request = new Request();
 		$request->url = "/m/g/$numeric_id/gprs/f.txt";
 		$gprs_controller = new GprsController($mt, $u, $request,5);
    	$message_resource = $gprs_controller->pullNextMessageResourceByMerchant($numeric_id);
    	
    	$this->assertEquals('Y', $gprs_controller->message_resource->locked);
    	
    	$body = $message_resource->message_text;
    	$this->assertEquals('#'.$merchant_id.'*'.$order_id.'*', substr($body,0,9+$order_id_length));
		
    	$gprs_controller->markMessageDelivered();
		$this->assertEquals('S', $gprs_controller->message_resource->locked, 'Should have marked the message as SENT');
		
		return true;
    }
    
    // now test the swap.  first set the flag to live on the swap table.  and call in for a message.
    /** 
     * @depends testNoMessageSteal
     */
    function testReconfigureMessageMap($result)
    {
    	$merchant_id = $this->ids['merchant_id'];
    	$numeric_id = $this->ids['numeric_id'];

    	// first set record to live
    	$swap_map_resource = Resource::find($this->swap_map_adapter,"".$merchant_id);
    	$swap_map_resource->live = 'Y';
    	$swap_map_resource->save();
    	
    	// now call in for a message
    	$request = new Request();
    	$request->url = "/messagemanager/getnextmessagebymerchantid/$numeric_id/epsonprinter/message.txt";
    	$message_controller = ControllerFactory::generateFromUrl($request->url, $mimetypes, $user, $request, 5);
    	    	
    	//this should get the bio, reconfigure the maps, and set the merchatn to active.
    	$ts = time();
    	$gprs_bio_message = $message_controller->pullNextMessageResourceByMerchant($numeric_id);
    	$ots = $gprs_bio_message->bio_message_order_id;
    	
    	$merchant_resource = SplickitController::getResourceFromId($merchant_id, "Merchant");
    	$this->assertEquals('Y', $merchant_resource->active);
    	
		$this->assertEquals("SUB", $gprs_bio_message->message_format);

        $this->assertContains('<text>name:  Unit Test Merchant&#10;</text>',$gprs_bio_message->message_text);
        $this->assertContains('<text>addr:  1505 Arapaho Ave&#10;</text>',$gprs_bio_message->message_text);
        $this->assertContains('<text>city:  boulder&#10;</text>',$gprs_bio_message->message_text);
        $this->assertContains('<text>phone: 1234567890&#10;</text>',$gprs_bio_message->message_text);
        $this->assertContains('<text>email: dummy@order140.com&#10;</text>',$gprs_bio_message->message_text);
    	
    	$test_order_id = $gprs_bio_message->test_order_id;
    	$this->assertTrue($test_order_id > 1000);

    	// check to see if the maps have been reconfigured.
    	$merchant_message_map_adapter = new MerchantMessageMapAdapter($mimetypes);
		$messages = $merchant_message_map_adapter->getRecords(array("merchant_id"=>$merchant_id));
		foreach ($messages as $message_record)
		{
			$message_format = $message_record['message_format'];
			$base_format = substr($message_format, 0,1);
			if ($base_format == 'T')
			{
				$text_message_record = $message_record;
				$this->assertTrue(false,"should not have found a Text message map since we are now an IP printer");
			} else if ($base_format == 'S') {
				$ip_message_record = $message_record;
			} else if ($base_format == 'P') {
				$ping_message_record = $message_record;
				$this->assertTrue(false,"should not have found a PING message map since we are now an IP printer");
			}
		}
		$this->assertEquals('SUW', $ip_message_record['message_format']);
		$this->assertEquals('0', $ip_message_record['delay']);
		$this->assertEquals('Epson Ip', $ip_message_record['info']);
		
		// now check to see if the new message was created.
		$mmh_adapter = new MerchantMessageHistoryAdapter($this->mimetypes);
		$new_order_message_data['order_id'] = $test_order_id;
		$ts = time() + 5;
		$date_string = date('Y-m-d H:i:s',$ts);
		$new_order_message_data['next_message_dt_tm'] = array("<"=>$date_string);
		$nom_options[TONIC_FIND_BY_METADATA] = $new_order_message_data;
		$new_messages = Resource::findAll($mmh_adapter,null,$nom_options);
		$this->assertTrue(sizeof($new_messages) > 0);
		$text_exists = false;
		$epsonip_printer_message = false;
		foreach ($new_messages as $new_message_resource)
		{
			$message_format = $new_message_resource->message_format;
			if ($message_format == 'T') {
				$text_exists = true;
			} else if (substr($message_format, 0,1) == 'S') {
				$this->assertEquals('SUT', $message_format);
				$epsonip_printer_message = true;
			}
		}
		
		$this->assertFalse($text_exists);
		$this->assertTrue($epsonip_printer_message);

		// lets check the test message

        $epson_test_message_resource = $message_controller->pullNextMessageResourceByMerchant($numeric_id);
        $this->assertEquals("SUT",$epson_test_message_resource->message_format);

        $this->assertContains('<text>This is a test message&#10;</text>',$epson_test_message_resource->message_text);
        $this->assertContains('<text>to make sure your printer&#10;</text>',$epson_test_message_resource->message_text);
        $this->assertContains('<text>is functioning correctly.&#10;</text>',$epson_test_message_resource->message_text);
        $this->assertContains('<text>If you see this, all is 200 OK.&#10;</text>',$epson_test_message_resource->message_text);


        return true;
    }
    
    // now test the fail of a version 5 printer calling in for an epson message
    /** 
     * @depends testReconfigureMessageMap
     */
   
    function testBadConfiguration($result)
    {
    	$merchant_id = $this->ids['merchant_id'];
    	$numeric_id = $this->ids['numeric_id'];
    	
    	// first create a test order
    	$place_order_controller = new PlaceOrderController($mimetypes, $user, $request,5);
		$order_resource = $place_order_controller->placeSimpleTestOrder($merchant_id, 'pickup', 'skip hours');
		$this->assertNull($order_resource->error);
		$this->assertTrue($order_resource->order_id > 0,"It should have a good order id");
	   	$request = new Request();
    	$request->url = "/m/g/$numeric_id/gprs/f.txt";
    	$gprs_controller = new GprsController($mt, $u, $request,5);
    	$message_resource = $gprs_controller->pullNextMessageResourceByMerchant($numeric_id);
    	// email should have gotten sent out to me and resource should not have the 2 in it
    	$format_index = substr($message_resource->message_format, -1);
    	
    	$this->assertNotEquals('2', $format_index);	
    }

    static function setUpBeforeClass()
    {
    	$_SERVER['request_time1'] = microtime(true);
    	$tz = date_default_timezone_get();
    	$_SERVER['default_tz'] = $tz;
    	date_default_timezone_set("America/Denver");
    	ini_set('max_execution_time',300);
        SplickitCache::flushAll();         $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->begin_transaction();
    	
    	$skin_resource = createWorldHqSkin();
    	$ids['skin_id'] = $skin_resource->skin_id;
    	
		//map it to a menu
    	$menu_id = createTestMenuWithNnumberOfItems(5);
    	$ids['menu_id'] = $menu_id;
    	
    	$modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 10);
    	$modifier_group_id = $modifier_group_resource->modifier_group_id;
    	$item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 2);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[1]['item_id'], $modifier_group_id, 4);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[2]['item_id'], $modifier_group_id, 1);

    	$merchant_resource = createNewTestMerchant($menu_id);
    	$merchant_resource->active = 'N';
    	$merchant_resource->save();
    	attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
    	$ids['merchant_id'] = $merchant_resource->merchant_id;
    	$ids['numeric_id'] = $merchant_resource->numeric_id;
    	
    	$user_resource = createNewUser(array("flags"=>"1C20000001"));
    	$ids['user_id'] = $user_resource->user_id;
    	
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
        SplickitCache::flushAll();         $db = DataBase::getInstance();
        $mysqli = $db->getConnection();
        $mysqli->rollback();
    }
    
	/* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    PrinterSwapTestEpsonIP::main();
}

?>