<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/unit_test_functions.inc';
require_once 'lib/utilities/functions.inc';

class SplickitVaultTest extends PHPUnit_Framework_TestCase
{
	var $stamp;

	function setUp()
	{
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		setContext("com.splickit.order");
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
    }
    
    function testCleanCCdata()
    {
    	$user_resource = createNewUserWithCC();
    	$user = $user_resource->getDataFieldsReally();
    	$user['cc_number'] = "8888888888888888";
    	$user['cc_exp_date'] = "12/2022";
    	$user['cvv'] = "023";
    	$user['zip'] = "12345";
    	$clean_data = SplickitVault::cleanCCdata($user);
    	$this->assertTrue(isset($clean_data['number']),"shojld have found cc number in payload");
    	$this->assertTrue(isset($clean_data['cvv']),"should have found cvv in payload");
    	$this->assertEquals($user['cvv'], $clean_data['cvv']);
    	$this->assertTrue(is_string($clean_data['cvv']),"cvv should be a string");
    	$json_val = json_encode($clean_data);
    	$json = SplickitVault::convertCCNumberInJsonToNumber($json_val);
    	
    	$this->assertContains('"number":8888888888888888,', $json);
    }
    
    function testCCSaveFails()
    {
    	$resource = Resource::dummyfactory($data);
    	$cc_functions = new CreditCardFunctions();
    	$this->assertFalse($cc_functions->cc_save($resource));
    	$this->assertEquals('Credit Card save error, zip cannot be blank.', $resource->error);
    	$resource->set('zip','12345');
    	
    	$this->assertFalse($cc_functions->cc_save($resource));
    	$this->assertEquals('Credit Card save error, expiration date cannot be blank.', $resource->error);
    	$resource->set('cc_exp_date','1418');
    	
    	$this->assertFalse($cc_functions->cc_save($resource));
    	$this->assertEquals('Credit Card save error, CVV cannot be blank.', $resource->error);
    	$resource->set('cvv','123y');
    	
    	$this->assertFalse($cc_functions->cc_save($resource));
    	$this->assertEquals('Credit Card save error, CVV must be 3 or 4 digits only.', $resource->error);
    	$resource->cvv = '456';
    	
    	$this->assertFalse($cc_functions->cc_save($resource));
    	$this->assertEquals('Credit Card save error, invalid expiration date: 1418', $resource->error);

    }

    function testSaveCardInVaultWithPrimaryFail()
    {
    	// get new user
    	$zip = rand(11111,19999);
    	$data['uuid'] = "test-$zip-0316-123-zd0c";
    	$user_resource = createNewUser($data);
    	$this->assertEquals('1000000001', $user_resource->flags);
    	$user_id = $user_resource->user_id;
    	$user = logTestUserIn($user_resource->user_id);

    	$json_encoded_data = "{\"jsonVal\":{\"cc_exp_date\":\"12/2022\",\"cc_number\":\"4111111111111111\",\"cvv\":\"123\",\"zip\":\"$zip\"}}"; 
    	$request = new Request();
    	$request->url = '/app2/apiv2/users/';
    	$request->method = "post";
    	$request->body = $json_encoded_data;
    	$request->mimetype = 'Applicationjson';
    	$request->_parseRequestBody();

    	$user_controller = new UserController($mt, $user, $request,5);
    	// save cc
    	$_SERVER['REQUEST_METHOD'] = 'POST';
    	$response_resource = $user_controller->processV2Request();
    	$this->assertNotNull($response_resource->error,'Should have gotten a credit card error');
    	$this->assertEquals("We're sorry, but we did not reach the credit card storage facility, please try again.", $response_resource->error); 
    	$updated_user_resource = SplickitController::getResourceFromId($user_id, 'User');   	
    	$this->assertEquals('1000000001', $updated_user_resource->flags);    
    }
    
    function testSaveCardInVaultWithLoyaltyOn()
    {
    	// get new user
    	$zip = rand(11111,19999);
    	$data['uuid'] = "test-$zip-0316-123-zd0c";
    	$user_resource = createNewUser($data);
    	$this->assertEquals('1000000001', $user_resource->flags);
    	$user_id = $user_resource->user_id;
    	$user = logTestUserIn($user_resource->user_id);

    	$json_encoded_data = "{\"jsonVal\":{\"cc_exp_date\":\"12/2018\",\"cc_number\":\"4111111111111111\",\"cvv\":\"123\",\"zip\":\"$zip\"}}";
    	$request = new Request();
    	$request->url = '/app2/apiv2/users';
    	$request->method = "post";
    	$request->body = $json_encoded_data;
    	$request->mimetype = 'Applicationjson';
    	$request->_parseRequestBody();

    	$user_controller = new UserController($mt, $user, $request,5);
    	// save cc
    	setContext('com.splickit.pitapit');
    	$_SERVER['REQUEST_METHOD'] = 'POST';
    	$response_resource = $user_controller->processV2Request();
    	$updated_user_resource = SplickitController::getResourceFromId($user_id, 'User');   	
    	$this->assertEquals('1C21000001', $updated_user_resource->flags);    
    	$this->assertEquals('insert', $response_resource->vault_process);	
    }

	function testSaveCCbadNumberDoNotFailOverToIPVault()
    {
    	$user_resource = createNewUser();
    	$user_id = $user_resource->user_id;
    	$user = logTestUserIn($user_resource->user_id);
    	
		$zip = rand(11111,19999);
    	$json_encoded_data = "{\"jsonVal\":{\"cc_exp_date\":\"03/2030\",\"cc_number\":\"4111114567111111\",\"cvv\":\"123\",\"zip\":\"$zip\"}}";
    	$request = new Request();
    	$request->url = '/app2/apiv2/users';
    	$request->method = "post";
    	$request->body = $json_encoded_data;
    	$request->mimetype = 'Applicationjson';
    	$request->_parseRequestBody();

    	$user_controller = new UserController($mt, $user, $request,5);
    	// save cc
    	$resource = $user_controller->processV2Request();
    	$this->assertEquals("Error saving credit card info: CC number is not valid", $resource->error);
    	
    	$user_resource2 = UserController::getUserResourceFromUserId($user_id);
    	$this->assertEquals('1000000001', $user_resource2->flags);
    }

    function testSaveCCbadExpDateDoNotFailOverToIPVault()
    {
    	$user_resource = createNewUser();
    	$user_id = $user_resource->user_id;
    	$user = logTestUserIn($user_resource->user_id);
    	    	
		$zip = rand(11111,19999);
    	$json_encoded_data = "{\"jsonVal\":{\"cc_exp_date\":\"03/2012\",\"cc_number\":\"4111111111111111\",\"cvv\":\"123\",\"zip\":\"$zip\"}}"; 
    	$request = new Request();
    	$request->url = '/app2/apiv2/users';
    	$request->method = "post";
    	$request->body = $json_encoded_data;
    	$request->mimetype = 'Applicationjson';
    	$request->_parseRequestBody();

    	$user_controller = new UserController($mt, $user, $request,5);
    	// save cc
    	$resource = $user_controller->processV2Request();
    	$this->assertEquals("Credit Card save error, expired expiration date: 0312", $resource->error);
    	
    	$user_resource2 = UserController::getUserResourceFromUserId($user_id);
    	$this->assertEquals('1000000001', $user_resource2->flags);  	
    }

    function testSaveCardInVault()
    {
    	// get new user
    	$zip = rand(11111,19999);
    	$data['uuid'] = "test-$zip-0316-123-zd0c";
    	$user_resource = createNewUser($data);
    	$this->assertEquals('1000000001', $user_resource->flags);
    	$user_id = $user_resource->user_id;
    	$user = logTestUserIn($user_resource->user_id);

    	$json_encoded_data = "{\"jsonVal\":{\"cc_exp_date\":\"03/2030\",\"cc_number\":\"4111111111111111\",\"cvv\":\"123\",\"zip\":\"$zip\"}}";
    	$request = new Request();
    	$request->url = '/app2/apiv2/users';
    	$request->method = "post";
    	$request->body = $json_encoded_data;
    	$request->mimetype = 'Applicationjson';
    	$request->_parseRequestBody();

    	$user_controller = new UserController($mt, $user, $request,5);
    	// save cc
    	$response_resource = $user_controller->processV2Request();
    	$updated_user_resource = SplickitController::getResourceFromId($user_id, 'User');   	
    	$this->assertEquals('1C21000001', $updated_user_resource->flags);    
    	$this->assertEquals('insert', $response_resource->vault_process);	
    	return $user['user_id'];
    }

    /**
     * @depends testSaveCardInVault
     */
    function testSaveCreditCardWithFlagsCorrect($user_id)
    {
    	$user_resource = UserAdapter::getUserResourceFromId($user_id);
    	$user_resource->flags = '1C20000001';
    	$user_resource->save();
    	$user = logTestUserIn($user_id);
    	$uuid = $user['uuid'];
    	$this->assertEquals('1C20000001', $user['flags']);
    	
    	$request = new Request();
    	$request->url = "apiv2/users/$uuid?log_level=5";
    	$request->method = 'POST';
    	$request->mimetype = 'application/json';
    	$request->body = "{\"cc_exp_date\":\"03/2030\",\"cc_number\":\"4111111111111111\",\"cvv\":\"123\",\"zip\":\"12345\"}";
    	$request->_parseRequestBody();
    	$user_controller = new UserController($mt, $user, $request);
    	$resource = $user_controller->processV2Request();
    	$user = logTestUserIn($user_id);
		$this->assertEquals('1C21000001', $user['flags']);
		return $user_id;    	
    }

    /**
     * @depends testSaveCreditCardWithFlagsCorrect
     */
     function testFailoverToUpdateOnExistingUserInVault($user_id)
    {
    	$user_adapter = new UserAdapter($mimetypes);
    	$user_resource = Resource::find($user_adapter,''.$user_id);
    	$this->assertEquals('1C21000001', $user_resource->flags);
		$user = $user_resource->getDataFieldsReally();
		$_SERVER['AUTHENTICATED_USER'] = $user;    	
    	$uuid = $user['uuid'];
    	$d = explode('-', $uuid);
    	$zip = $d[1];
    	
    	$user_resource->set("cc_exp_date","03/2030");
    	$user_resource->set("cc_number","4111111111111111");
    	$user_resource->set("cvv","123");
    	$user_resource->set("zip","$zip");
    	$user_resource->flags = '1000000001';
    	$user_resource->uuid = 'existing-3906-4l68-00o8-zd0c';
    	$cc_functions = new CreditCardFunctions();
    	$result = $cc_functions->cc_save($user_resource);
    	$status = $cc_functions->getStatus();
    	$this->assertTrue($result);
    	$this->assertEquals('200 OK', $status);
    	$this->assertEquals('update', $cc_functions->splickit_vault_save_process);
    	$this->assertEquals('1C21000001', $user_resource->flags);
    	
    	$card_data = $cc_functions->pullCreditCardDataFromUserId($user_id);
    	$this->assertEquals($zip, $card_data['zip']);
    	$this->assertEquals($zip, $card_data['postal_code']);
    	return $card_data;
     }

    /**
     * @depends testFailoverToUpdateOnExistingUserInVault
     */
    
    function testRunCCAtInspirePayWithNoVault($card_data)
    {
		$inspire_pay = New inspirePay();
		$amount = rand(100,1000)/100;
		$order_id = rand(10000,1000000);
		$inspire_pay->runCardNoVault($card_data, $amount, $order_id);
		$return_fields = $inspire_pay->getReturnData();
		$this->assertEquals(100, $return_fields['response_code']);    	
    }

    function testPullCCInfoFromInspirePayAndResaveInNewVault()
    {
    	$user_resource = createNewUser();
    	$user_id = $user_resource->user_id;
    	$user = logTestUserIn($user_resource->user_id);

    	$inspireObj = new InspirePay();
    	$zip = rand(11111,19999);
    	$result_boolean = $inspireObj->createCustomerVaultRecord($user['user_id'], "4111111111111111", "123", "06/2015", $zip, "first", "last");
    	$inspire_pay_object = new InspirePay();
		$inspire_pay_object->getCustomerVaultRecord($user['user_id']);
    	$card_data = $inspire_pay_object->getReturnData();
		$return_data2 = $inspire_pay_object->getCustomerVaultRecordCCReally($user['user_id']);
		$card_data['cc_number'] = $return_data2['cc_number'];
		$splickit_vault = new SplickitVault();
		$result = $splickit_vault->saveCardFromInspirePayReturnAndUserResource($card_data, $user_resource);
		$this->assertTrue($result);
    }
    
    static function setUpBeforeClass()
    {
    	ini_set('max_execution_time',300);
    	      SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();$mysqli->begin_transaction(); ;
    }
    
	static function tearDownAfterClass()
    {
    	SplickitCache::flushAll();         $db = DataBase::getInstance(); $mysqli = $db->getConnection();       $mysqli->rollback();
    }
        
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['argv'][1]) && $_SERVER['argv'][1] == 'use_main'  && !defined('PHPUnit_MAIN_METHOD')) {
    SplickitVaultTest::main();
}

?>