<?php

/**
 * Created by PhpStorm.
 * User: Elvis
 * Date: 8/24/16
 * Time: 3:25 PM
 */

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';


class EmagineServiceTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
        $this->ids = $_SERVER['unit_test_ids'];

    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }

    function testSendOrderToEmagineService()
    {
        $user = logTestUserIn($this->ids['user_id']);
        $merchant_id = $this->ids['merchant_id'];
        $order_adapter = new OrderAdapter($m);
        $order_adapter->setMenuTypeIndexForTestOrders(2);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'sum dum note');
        $order_data['items'][0]['mods'] = array();
        $item_size_record = getStaticRecords($order_data['items'][0],'ItemSizeAdapter');
        $order_resource = placeOrderFromOrderData($order_data, getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'N');
        $emagine_controller = ControllerFactory::generateFromMessageResource($message_resource, $m, $u, $r);
        $readytosend_message_resource = $emagine_controller->prepMessageForSending($message_resource);
        $body = $readytosend_message_resource->message_text;
        $this->assertContains($item_size_record[0]['external_id'],$body);
        $emagine_service = new EmagineService($data);
        $response = $emagine_service->send($body);
        $this->assertNotNull($response);
        $this->assertEquals(200,$response['http_code'],"It should have the OK 200 response");
        $this->assertEquals("success",$response['status'],"It shoudl have had a success");
        $this->assertTrue($response['ok'],"It should have returned an ok of true");

    }    

    /**
     * @expectedException UnsuccessfulEmaginePushException
     */
    function testEmagineServiceFailure()
    {
        $user = logTestUserIn($this->ids['user_id']);
        $merchant_id = $this->ids['merchant_id'];
        $order_adapter = new OrderAdapter($m);
        $order_adapter->setMenuTypeIndexForTestOrders(2);
        $order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'sum dum note');
        $order_data['items'][0]['mods'] = array();
        $item_size_record = getStaticRecords($order_data['items'][0],'ItemSizeAdapter');
        $order_resource = placeOrderFromOrderData($order_data, getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'N');
        $emagine_controller = ControllerFactory::generateFromMessageResource($message_resource, $m, $u, $r);
        $readytosend_message_resource = $emagine_controller->prepMessageForSending($message_resource);
        $body = $readytosend_message_resource->message_text;
        $this->assertContains($item_size_record[0]['external_id'],$body);
        //now replace the external id with a random string to force a failure
        $fake_id = "qwertyu234567";
        $body = str_replace($item_size_record[0]['external_id'],$fake_id,$body);
        $this->assertContains($fake_id,$body,"It should now have the fake external id");
        $emagine_service = new EmagineService($data);
        $response = $emagine_service->send($body);
        $this->assertNotNull($response);
        $this->assertEquals(200,$response['http_code'],"It should have the OK 200 response");
        $this->assertEquals("failure",$response['status'],"It shoudl have had a failure");
        $this->assertFalse($response['Ok'],"It should have returned an ok of false");
    } 
    
    static function setUpBeforeClass()
    {
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time', 300);

        $menu_id = 102431;
        $brand_id = 150;
        $skin_id = 5;

        $menu_adapter = new MenuAdapter($mimetypes);
        if ($menu_resource = Resource::find($menu_adapter,"$menu_id")){
        } else {
            $menu_adapter->importMenu($menu_id,'prod','local',$brand_id);
        }

        getOrCreateSkinAndBrandIfNecessary("Snarfs", "Snarf's", $skin_id, $brand_id);

        setContext('com.splickit.snarfs');
        $ids['skin_id'] = getSkinIdForContext();

        $merchant_resource = createNewTestMerchant($menu_id);

        attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
        $ids['merchant_id'] = $merchant_resource->merchant_id;
        MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id, 'N', 'emagine', 'X');

        Resource::createByData(new MerchantEmagineInfoMapsAdapter($mimetypes), array('merchant_id' => $merchant_resource->merchant_id, 'location' => "Location731", 'token' => "1234567890"));

        $user_resource = createNewUser(array("flags" => "1C20000001"));
        // create loyalty record
        $user_session_controller = new UsersessionController($m,$user_resource->getDataFieldsReally(),$r,5);
        $user_session_controller->getUserSession($user_resource);
        $ids['user_id'] = $user_resource->user_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }


    static function tearDownAfterClass()
    {
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    EmagineServiceTest::main();
}