<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class FaxServicesTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
		
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }

    function testPhaxioService()
    {
    	$fax_data['fax_no'] = '7204384799';
    	$fax_data['fax_text'] = "<html><head><title>order stuff</title></head><body>hello world</body></html>";
    	$phaxio_fax_service = new PhaxioFaxService($message_resource);
    	$result = $phaxio_fax_service->send($fax_data);
    	$this->assertEquals(true, $result);
    	$this->assertEquals(200, $phaxio_fax_service->curl_response_array['http_code']);
    	$raw_result_json = $phaxio_fax_service->curl_response_array['raw_result'];
    	$result_data = json_decode($raw_result_json,true);
    	$this->assertTrue(isset($result_data['faxId']));
    	$this->assertEquals('Fax queued for sending', $result_data['message']);
    }
    
    function testFaxageFaxService()
    {
    	$fax_data['fax_no'] = '7204384799';
    	$fax_data['fax_text'] = "<html><head><title>order stuff2</title></head><body>hello world2</body></html>";
    	$faxage_fax_service = new FaxageFaxService($message_resource);
    	$result = $faxage_fax_service->send($fax_data);
    	$this->assertEquals(true, $result);
    	$this->assertEquals(200, $faxage_fax_service->curl_response_array['http_code']);
    	$raw_result = $faxage_fax_service->curl_response_array['raw_result'];
    	$this->assertContains('JOBID: ', $raw_result);
    }

    static function setUpBeforeClass()
    {
    	$_SERVER['request_time1'] = microtime(true);
    	$_SERVER['NO_MOCKS'] = true;
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	mysqli_query("BEGIN");
    	
    	$skin_resource = createWorldHqSkin();
    	$ids['skin_id'] = $skin_resource->skin_id;
    	
		//map it to a menu
    	$menu_id = createTestMenuWithNnumberOfItems(5);
    	$ids['menu_id'] = $menu_id;
    	
    	$modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 10);
    	$modifier_group_id = $modifier_group_resource->modifier_group_id;
    	$item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 2);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[1]['item_id'], $modifier_group_id, 4);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[2]['item_id'], $modifier_group_id, 1);

    	$merchant_resource = createNewTestMerchant($menu_id);
    	attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
    	$ids['merchant_id'] = $merchant_resource->merchant_id;
    	
    	$user_resource = createNewUser(array("flags"=>"1C20000001"));
    	$ids['user_id'] = $user_resource->user_id;
    	
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	mysqli_query("ROLLBACK");
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['argv'][1]) && $_SERVER['argv'][1] == 'use_main'  && !defined('PHPUnit_MAIN_METHOD')) {
    FaxServicesTest::main();
}

?>