<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class FishBowlTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
		setContext('com.splickit.moes');
		
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }

    function testFishBowlServiceConnectivity()
    {
    	$email = 'sumdumemail@order140.com';
    	$fishbowl_service = new FishBowlService();
    	$response = $fishbowl_service->checkExistence($email);
    	$this->assertEquals(200, $response['http_code']);
		$this->assertContains("Object reference not set to an instance of an object.",$response['raw_result']);
    }

	function testValidateGoodBirthDate()
	{
		$fbs = new FishBowlService();
		$result = $fbs->validateBirthday("06/08");
		$this->assertTrue($result,"Birthday shoujdl have been valid but was not");
	}

	function testValidateBadBirthDate()
	{
		$fbs = new FishBowlService();
		$result = $fbs->validateBirthday("608");
		$this->assertFalse($result);
		$this->assertEquals(FishBowlService::BIRTHDAY_ERROR_MESSAGE,join(",", $fbs->getError()));
	}

	function testJoinFishbowl()
    {
    	$user_resource = createNewUser();
    	$fishbowl_service = new FishBowlService();
    	$response = $fishbowl_service->send(array_merge($fishbowl_service->fishbowl_data, array("Birthdate"=>'06/08', "Zip"=>'10029')), $user_resource);
    	$this->assertTrue($fishbowl_service->isSuccessfulResponse($response));
		return $user_resource;
	}

	/**
	 * @depends testJoinFishbowl
	 */
	function testFishBowlServiceForExistingUser($user_resource)
	{
		$fishbowl_service = new FishBowlService();
		$response = $fishbowl_service->checkExistence($user_resource->email);
		$this->assertEquals(302, $response['http_code']);
		$this->assertContains('NewMember=False',$response['raw_result'],"should have found new member false");
	}
    
    static function setUpBeforeClass()
    {
    	$_SERVER['request_time1'] = microtime(true);
    	$_SERVER['NO_MOCKS'] = true;
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	mysqli_query("BEGIN");
    	    	
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	mysqli_query("ROLLBACK");
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    FishBowlTest::main();
}

?>