<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class FpnServiceTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }
	       
    function testProcessWithFPN()
    {
    	$fp = new FranchisePayments($this->ids['merchant_id']);
    	$user_data['first_name'] = 'firstname';
    	$user_data['last_name'] = 'lastname';
    	$user_resource = Resource::dummyfactory($user_data);
    	$neworder_id = time();
    	$card_data['cc_number'] = "4005562233445564";
		$card_data['cc_exp'] = '0320';
		$card_data['zip'] = "90036";
    	$result_data = $fp->process(10.00, $billing_user_resource, $neworder_id, $merchant, $card_data);
    	$this->assertNull($result_data['error']);
    	$this->assertEquals(100, $result_data['response_code']);  	
    }
    
    function testPlaceOrderWithFPNMerchant()
    {
    	$merchant_id = $this->ids['merchant_id'];
		
		setContext('com.splickit.order');
    	$user_resource = createNewUser();
		$new_user_id = $user_resource->user_id;
		$user = logTestUserIn($new_user_id);
    	
    	$json_encoded_cc_data = "{\"jsonVal\":{\"cc_exp_date\":\"03/2020\",\"cc_number\":\"4005562233445564\",\"cvv\":\"123\",\"zip\":\"90036\"}}"; 
    	$request = new Request();
    	$request->url = '/app2/phone/users';
    	$request->method = "post";
    	$request->body = $json_encoded_cc_data;
    	$request->mimetype = 'Applicationjson';
    	$request->_parseRequestBody();

    	$user_controller = new UserController($mt, $user, $request,5);
    	// save cc
    	$resopnse = $user_controller->updateUser();
		$this->assertNull($resopnse->error);
    	// now get logged in user again since their flags have changed
    	$user = logTestUserIn($user_resource->user_id);
				
		$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id, 'pickup', 'skip hours');
		
		$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
		$order_id = $order_resource->order_id;    	    	
		$this->assertNotNull($order_resource);
		$this->assertNull($order_resource->error);
		$this->assertTrue($order_resource->order_id > 1000,"Bad order id of: ".$order_resource->order_id);  
		
		// check balance change table
		$bc_adapter = new BalanceChangeAdapter($mimetypes);
		$data['process'] = 'CCpayment';
		$data['order_id'] = $order_id;
		$bc_records = $bc_adapter->getRecordsForOrderId($data);
		$cc_record = $bc_records[0];
		$this->assertNotNull($cc_record);
		return $order_id;
    }
    
    /**
     * @depends testPlaceOrderWithFPNMerchant
     */
    function testCreditVoid($order_id)
    {
		$bc_adapter = new BalanceChangeAdapter($mimetypes);
		$data['process'] = 'CCpayment';
		$data['order_id'] = $order_id;
		$bc_records = $bc_adapter->getRecordsForOrderId($data);
		$balance_change_row = $bc_records[0];
    	$ref_num_string = $balance_change_row['cc_transaction_id'];
    	$s = explode("=", $ref_num_string);
    	$ref_no = $s[1];
    	$cc_functions = CreditCardFunctions::creditCardFunctionsFactory('F');
    	$merchant_resource = SplickitController::getResourceFromId($this->ids['merchant_id'], 'Merchant');
    	$return_data = $cc_functions->creditVoidTransaction($balance_change_row, "0.00", $merchant_resource);
    	$this->assertEquals(100, $return_data['response_code']);
    }
    
    function testFPNGoodCard()
	{
		$card_data['cc_number'] = "4005562233445564";
		$card_data['cc_exp'] = '0320';
		$card_data['zip'] = "90036";
		$card_data['cv2'] = "435";
		
		$user_resource = Resource::dummyFactory(array("first_name"=>"Test T","last_name"=>"Jones"));
		
		$fz_payments = new FranchisePayments();
		$result_data = $fz_payments->process(1.00, $user_resource, 1234, $merchant , $card_data);
		$this->assertEquals("Approved", $result_data['responsetext']);
		
	}
	
    function testFPNBadCardExp()
	{
		$card_data['cc_number'] = "4005562233445564";
		$card_data['cc_exp'] = '0314';
		$card_data['zip'] = "90036";
		$card_data['cv2'] = "435";
		
		$user_resource = Resource::dummyFactory(array("first_name"=>"Test T","last_name"=>"Jones"));
		
		$fz_payments = new FranchisePayments();
		$result_data = $fz_payments->process(2.00, $user_resource, 1234, $merchant , $card_data);
		$this->assertEquals("Declined", $result_data['responsetext']);
		$this->assertEquals("Credit card has expired.",$result_data['error']);
	}

	static function setUpBeforeClass()
    {
    	$_SERVER['request_time1'] = microtime(true);
    	$_SERVER['NO_MOCKS'] = true;
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	mysqli_query("BEGIN");

    	$ids['skin_id'] = 88; // ECW
    	$ids['brand_id'] = 379;  // ECW
    	
    	$simple_menu_id = createTestMenuWithOneItem("item_one");
    	$ids['menu_id'] = $simple_menu_id;
    	
    	$merchant_resource = createNewTestMerchant($simple_menu_id);
    	$merchant_resource->cc_processor = 'F';
    	$merchant_resource->save();
    	attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
    	$ids['merchant_id'] = $merchant_resource->merchant_id;
    	
    	$ids['merchant_rock_comm_id'] = 'ECW92130';
    	$mfpnma = new MerchantFPNMapAdapter($mimetypes);
    	$data['merchant_id'] = $ids['merchant_id'];
    	$data['fpn_merchant_key'] = '_1Mk65OH04krz32O4ZG2vy3Oj884882A';
    	$data['fpn_merchant_pin'] = 1405;
    	$data['merchant_rock_comm_id'] = $ids['merchant_rock_comm_id'];
    	Resource::createByData($mfpnma, $data);

    	$user_resource = createNewUser(array("flags"=>"1C20000001"));
    	$ids['user_id'] = $user_resource->user_id;
    	//$ids['loyalty_number'] = '900292740611';
    	$ids['loyalty_number'] = '900292740629';
    	$ubpa = new UserBrandPointsMapAdapter($mimetypes);
    	$ubpa->createUserLoyaltyAccount($ids['user_id'], $ids['brand_id'], $ids['loyalty_number']);

    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
		    
    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

//if (isset($_SERVER['argv'][1]) && $_SERVER['argv'][1] == 'use_main'  && !defined('PHPUnit_MAIN_METHOD')) {
if (isset($_SERVER['argv'][1]) && $_SERVER['argv'][1] == 'use_main'  && !defined('PHPUnit_MAIN_METHOD')) {
    FpnServiceTest::main();
}

?>