<?php
$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class HeartlandPitaPitLoyaltyIntegrationTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;
	
	function setUp()
	{
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
	}
	
	function tearDown() {
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
		unset($this->stamp);
  }
    
    function testSetReadableBalances() {
    	$results['sva.balances'] = "Points 1000,USD 5025";
    	$results['status'] = 'success';
    	$hls = PitapitLoyaltyController::loyaltyServiceFactory();
    	$hls->setReadableBalances($results);
    	$this->assertEquals(1000, $results['Points']);
    	$this->assertEquals(50.25, $results['USD']);    	
    }
    
    function testSetReadableBalancesNoPoints() {
    	$results['sva.balances'] = "USD 5025";
    	$results['status'] = 'success';
    	$hls = PitapitLoyaltyController::loyaltyServiceFactory();
    	$hls->setReadableBalances($results);
    	$this->assertEquals(null, $results['Points']);
    	$this->assertEquals(50.25, $results['USD']);
    }
    
    function testSetReadableBalancesNoCash() {
    	$results['sva.balances'] = "Points 1000";
    	$results['status'] = 'success';
    	$hls = PitapitLoyaltyController::loyaltyServiceFactory();
    	$hls->setReadableBalances($results);
    	$this->assertEquals(1000, $results['Points']);
    	$this->assertEquals(null, $results['USD']);
    }

    function testCreateUserKatana()
    {
    	$user = $this->ids['user'];
    	$hls = PitapitLoyaltyController::loyaltyServiceFactory();
    	$heartland_user_result = $hls->createUserKatana($user['email'],'password');
    	$this->assertEquals('success', $heartland_user_result['status']);
    	$this->assertEquals(204,$heartland_user_result['http_code']);
    	return $user;
    }
    
    /**
     * @depends testCreateUserKatana 
     */
    function testCreateAccountKatana($user)
    {
    	$hls = PitapitLoyaltyController::loyaltyServiceFactory();
    	$account_creation_result = $hls->createAccountKatana($user['email'],'password');
    	$this->assertEquals('success', $account_creation_result['status']);
    	$account_number = $account_creation_result['id'];
    	$this->assertNotNull($account_number);
    	$pin = $account_creation_result['pin'];
    	$this->assertNotNull($pin);
    	$account['account_number'] = $account_number;
    	$account['pin'] = $pin;
    	return $account;
    }
    
    /**
     * @depends testCreateAccountKatana
     */
    function testGetAccountInfo($account)
    {
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
    	
		$hls = PitapitLoyaltyController::loyaltyServiceFactory();
		$result = $hls->getBalance($sva, $pin);
		$this->assertNull($result['error']);
		$this->assertEquals('success',$result['status']);
		
		$qr_result = $hls->getKatanaAccountInfoWithQRCodeNumber($sva, $pin);
		$this->assertNull($qr_result['error']);
		$this->assertEquals('success',$qr_result['status']);
		$track2 = $qr_result['track2'];
		$this->assertContains($qr_result['number'], $track2);
		//$this->assertContains("KATANA?", $track2);
		
		// check on bad loylaty number
		$result = $hls->getBalance('1234567890', $pin);
		$this->assertEquals('failure',$result['status']);

		return $account;
    }

    /**
     * @depends testGetAccountInfo
     */
    function testAddPoints($account)
    {
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
    	$points = 1000;
		$hls = PitapitLoyaltyController::loyaltyServiceFactory();
		$hls->setStore('splickitstore');
		$result = $hls->addPoints($sva,$pin,$points);
		$this->assertNull($result['error']);
		$this->assertEquals('success',$result['status']);
		$balance = $result['sva.balances'];
		$b = explode(',', $balance);
		$points_balance = $b[0];
		$this->assertEquals("Points 1000", $points_balance);
		return $account;
    }
    
    /**
     * @depends testAddPoints
     */
    function testRewardPoints($account)
    {
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
    	$amount_of_purchase = 33.33;
		$hls = PitapitLoyaltyController::loyaltyServiceFactory();
		$hls->setStore('splickitstore');
		$result = $hls->rewardPoints($sva,$pin,$amount_of_purchase);
		$this->assertNull($result['error']);
		$this->assertEquals('success',$result['status']);
		$this->assertEquals('Points 33',$result['rewards']);
	return $account;
    }
    
    /**
     * @depends testRewardPoints
     */
    function testGetBalance($account)
    {
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
		$hls = PitapitLoyaltyController::loyaltyServiceFactory();
		$result = $hls->getBalance($sva,$pin);
		$this->assertNull($result['error']);
		$this->assertEquals('success',$result['status']);
		$balance = $result['sva.balances'];
		$b = explode(',', $balance);
		$points_balance = $b[0];
		$this->assertEquals("Points 1033", $points_balance);
		return $account;
    }
    
    /**
     * @depends testGetBalance
     */
    function testRedeemPoints($account)
    {
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
		$hls = PitapitLoyaltyController::loyaltyServiceFactory();
		$hls->setStore('splickitstore');
    	$result = $hls->redeemPoints($sva, $pin, 500);
    	$this->assertNull($result['error']);
		$this->assertEquals('success',$result['status']);
		$balance = $result['sva.balances'];
		$b = explode(',', $balance);
		$points_balance = $b[0];
		$this->assertEquals("Points 533", $points_balance);
		return $account;
    }
    
    /**
     * @depends testRedeemPoints
     */
    function testRedeemPointsFailure($account)
    {
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
		$hls = PitapitLoyaltyController::loyaltyServiceFactory();
		$hls->setStore('splickitstore');
    	$result = $hls->redeemPoints($sva, $pin, 1000);
    	$this->assertNotNull($result['error']);
		$this->assertEquals('failure',$result['status']);
    	$this->assertEquals(400, $result['status.code']);
    	$this->assertEquals('InsufficientFunds',$result['status.name']);
		return $account;
    }
    
    /**
     * @depends testRedeemPointsFailure
     */
    function testGetHistory($account)
    {
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
    	$user=$this->ids['user'];
		$hls = PitapitLoyaltyController::loyaltyServiceFactory();
		$result = $hls->getHistory($sva,$pin,$user['email']);
		$this->assertNull($result['error']);
		$this->assertNotNull($result['history']);
		$history = $result['history'];
		foreach ($history as $transaction_record) {
			$this->assertCount(3, $transaction_record);
			$this->assertNotNull($transaction_record['transaction_date']);
			$this->assertNotNull($transaction_record['description']);
			if (strtolower($transaction_record['description']) == 'balance inquiry') {
				$this->assertTrue(false,"Should not have found a balance inquiry record in the history but did");
			}
		}
		$this->assertEquals('success',$result['status']);
		return $account;
    }

//** start stored value tests **//

    /**
     * @depends testGetHistory
     */
    function testLoadStoredValueFromCC($account)
    {

        $user = $this->ids['user'];
        $sva = $account['account_number'];
        $pin = $account['pin'];

        $dollars = 17.77;
        $hls = PitapitLoyaltyController::loyaltyServiceFactory();
        $hls->setStore('splickitstore');
        $result = $hls->addDollars($sva,$pin,$dollars);
        $this->assertNull($result['error']);
        $this->assertEquals('success',$result['status']);
        $balance = $result['sva.balances'];
        $b = explode(',', $balance);
        $stored_value_balance = $b[1];
        $this->assertEquals("USD ".$dollars*100, $stored_value_balance);
        return $account;


/*    	$user = $this->ids['user'];
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
    	$cc_number = '4788250000028291';
    	$exp_date = '1014';
    	$cvv = '111';
    	$full_name = $user['first_name'].' '.$user['last_name'];
    	$hls = PitapitLoyaltyController::loyaltyServiceFactory();
    	$amount = '17.77';
    	$result = $hls->addStoredValueFromCC($sva,$pin,$amount,$cc_number,$exp_date,$cvv,$full_name,$user['email'],'password');
    	$this->assertEquals(200, $result['http_code']);
    	
    	$balance_result = $hls->getBalance($sva, $pin);
    	$this->assertEquals(17.77, $balance_result['USD']);
    	return $account;
*/
    }
    
//    /**
//     * @depends testLoadStoredValueFromCC
 //    */
/*    function testStoreCC($account)
    {
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
    	$hls = PitapitLoyaltyController::loyaltyServiceFactory();
    	$result = $hls->storeCC($cc_info);
    	$this->assertTrue($result);
    	return $account;
    }
 */   
/*    function testLoadStoredValueFromStoredCC()
    {
    	//NOPE we never do this.  any load will come from our vault with the CC number i think
    	$this->assertTrue(true);
    }
 */   
/*    /**
     * @depends testStoreCC
     */
/*    function testSetUpAutoLoad($account)
    {
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
    	$hls = PitapitLoyaltyController::loyaltyServiceFactory();
    	$result = $hls->setAutoLoad($info);
    	$this->assertTrue($result);
    	return $account;
    }
*/    
    /**
     * @depends testLoadStoredValueFromCC
     */
    function testChargeStoredValue($account)
    {
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
    	
    	$hls = PitapitLoyaltyController::loyaltyServiceFactory();
    	$balance_result = $hls->getBalance($sva, $pin);
    	$starting_balance = $balance_result['USD'];

    	$amount_in_pennies = rand(900, 1600);
    	$amount = $amount_in_pennies/100;
    	$hls->setStore('us-1000');
    	$result = $hls->chargeAgainstStoredValue($sva, $pin, $amount);
    	$this->assertEquals(200, $result['status.code']);

    	$expected_ending_balance = number_format($starting_balance-$amount,2);
    	
    	$hls2 = PitapitLoyaltyController::loyaltyServiceFactory();
    	$balance_result = $hls2->getBalance($sva,$pin);
    	$this->assertEquals($expected_ending_balance, $balance_result['USD']);
    	//$balance = $balance_result['sva.balances'];
    	//$this->assertEquals('Points 500,USD 252', $balance);
    	return $account;
    }
        
    /**
     * @depends testChargeStoredValue
     */
    function testChargeStoredValueFailure($account)
    {
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
    	
    	$hls = PitapitLoyaltyController::loyaltyServiceFactory();
    	$balance_result = $hls->getBalance($sva, $pin);
    	$starting_balance = $balance_result['USD'];
    	
    	$amount = $starting_balance + 5;
    	$hls->setStore('us-1000');
    	$result = $hls->chargeAgainstStoredValue($sva, $pin, $amount);
    	$this->assertEquals(400, $result['status.code']);
    	$this->assertEquals('InsufficientFunds',$result['status.name']);
    }
    
    static function setUpBeforeClass()
    {
    	$_SERVER['request_time1'] = microtime(true);
    	$_SERVER['NO_MOCKS'] = true;
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	mysqli_query("BEGIN");
    	
		$user_resource = createNewUser();
		$ids['user_id'] = $user_resource->user_id;
		$ids['user'] = $user_resource->getDataFieldsReally();
		$ids['user_resource'] = $user_resource;
    	
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	mysqli_query("ROLLBACK");
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
	HeartlandPitaPitLoyaltyIntegrationTest::main();
}

?>

