<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';


class IvrNewTwilioTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__ . '-' . $_SERVER['STAMP'];
        $this->ids = $_SERVER['unit_test_ids'];
        setProperty('test_sms_messages_on', 'true');

    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }

    function testCallOnly()
    {
        $message = '<?xml version="1.0" encoding="UTF-8"?>
            <Response>
                <Message>280743</Message>
                <Message>1234</Message>
                <Play loop="0">https://s3.amazonaws.com/splickit-api-assets/audio/you_have_a_pending_order.mp3</Play>
            </Response>';
        $url_encoded_message = urlencode(cleanUpDoubleSpacesCRLFTFromString($message));
        $message_response_url= getProperty('twimlets_server').$url_encoded_message;
        $response = TwilioService::InitializeOutboundCall('4107754092', $message_response_url);
        $this->assertEquals('queued',$response['twilio_status']);
    }

    function testSendMessage()
    {
        // first create a message assiated to the order and merchant
        $mha = new MerchantMessageHistoryAdapter($m);
        $message_resource = $mha->createMessageReturnResource($this->ids['merchant_id'],$this->ids['order_id'],'IB','4107754092',time()-1);
        $ivr_controller = new IvrController($m,$u,$r,5);
        $ivr_controller->setTestDeliveryAddress('4107754092'); // change this number for the number with you do the test
        $results = $ivr_controller->sendThisMessage($message_resource);
        $this->assertTrue($results);
        $new_message_resource = $message_resource->refreshResource($message_resource->map_id);
        $response = $new_message_resource->response;
        $this->assertNotEmpty($response);
    }

    static function setUpBeforeClass()
    {
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time', 300);
        mysqli_query("BEGIN");
        $skin_resource = createWorldHqSkin();
        $ids['skin_id'] = $skin_resource->skin_id;

        $menu_id = createTestMenuWithOneItem("item_one");
        $ids['menu_id'] = $menu_id;

        $merchant_resource = createNewTestMerchant($menu_id);
        attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
        $ids['merchant_id'] = $merchant_resource->merchant_id;

        $user_resource = createNewUser(array("flags"=>"1C20000001"));
        logTestUserResourceIn($user_resource);
        $ids['user_id'] = $user_resource->user_id;

        $order_data = OrderAdapter::staticGetSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','some note');
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        if ($order_resource->order_id < 1000) {
            die("error creating order");
        }
        $ids['order_id'] = $order_resource->order_id;


        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        mysqli_query("ROLLBACK");
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main()
    {
        $suite = new PHPUnit_Framework_TestSuite(__CLASS__);
        PHPUnit_TextUI_TestRunner::run($suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    IvrNewTwilioTest::main();
}

?>