<?php
error_reporting(E_ERROR | E_COMPILE_ERROR | E_COMPILE_WARNING | E_PARSE);
$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');
$db_info->database = 'smaw_unittest';
$db_info->username = 'root';
$db_info->password = 'splickit';
if (isset($_SERVER['XDEBUG_CONFIG'])) {
    putenv("SMAW_ENV=unit_test_ide");
    $db_info->hostname = "127.0.0.1";
    $db_info->port = 13306;
} else {
    $db_info->hostname = "db_container";
    $db_info->port = 3306;
}
$_SERVER['DB_INFO'] = $db_info;
require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class MessageDispatchTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;
    var $info;

    function setUp()
    {
        $_SERVER['no_new_merchant_payment'] = true;
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $_SERVER['DO_NOT_RUN_CC'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
        $this->ids = $_SERVER['unit_test_ids'];
        $this->api_port = "80";
        if (isset($_SERVER['XDEBUG_CONFIG'])) {
            $this->api_port = "10080";
        }
    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
        unset($this->info);
    }

    function testAdapter()
    {
        $id = rand(1000,9999);
        $mmra = new MessageManagerRequestsAdapter(getM());
        $mmra->recordMessageManangerRequest($id,"/app2/messagemanager/getnextmessagebymerchantid/$id/json",'get');
    }

    function testResetOfMessageIfWrongLocked()
    {
        setContext('com.splickit.worldhq');
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $merchant_numeric = $merchant_resource->numeric_id;
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'J',"delivery_addr"=>"Ghost Kitchen","message_type"=>"X"));

//        $url = "http://127.0.0.1:".$this->api_port."/app2/messagemanager/getnextmessagebymerchantid/$merchant_numeric/json";
//        $response = $this->makeRequest($url,null);


        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data);
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00,null);
        $this->assertNull($order_resource->error);


        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'J');
        $message_resource->next_message_dt_tm = time()-100;
        $message_resource->locked = 'N';
        $message_resource->save();

        $url = "http://127.0.0.1:".$this->api_port."/app2/messagemanager/sendnextmessage";
        $response = $this->makeRequest($url,null);

        $message_resource = $message_resource->getRefreshedResource();
        $this->assertEquals('P',$message_resource->locked);


    }

    function testComprisStatus()
    {
        $menu_id = createTestMenuWithNnumberOfItems(1);

        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $merchant_alphanumeric = $merchant_resource->alphanumeric_id;

        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_resource->merchant_id,"message_format"=>'O',"delivery_addr"=>"Compris","message_type"=>"X"));

        $url = "http://127.0.0.1:".$this->api_port."/app2/messagemanager/getnextmessagebymerchantid/$merchant_alphanumeric/compris";

        $response = $this->makeRequest($url,null);
        myerror_log($response);
        $packet_id = "STATUS-".time();
        $expected_body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Request Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus/></Request></CommunicationCenter>';
        $this->assertContains($expected_body,$response);

        // test again shoudl get null
        $response = $this->makeRequest($url,null);
        $this->assertEquals('',$response);

        //now send back a good.
        $callback_url = "http://127.0.0.1:".$this->api_port."/app2/messagemanager/$merchant_alphanumeric/compris/callback.txt";
        $body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Response Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus Turn="BAT-A.00001" PosTerminal="mws" PosId="3" ResultCode="0" ResultText="Pos is ready"/></Response></CommunicationCenter>';
        $response = $this->makeRequest($callback_url,null,'POSTXML',null,$body);

        $merchant_resource = $merchant_resource->getRefreshedResource();
        $this->assertEquals('Y',$merchant_resource->ordering_on,"ordering_on should have been set to 'Y'");

        $dciha = new DeviceCallInHistoryAdapter(getM());
        $last_call_in_resource = $dciha->getCallInResourceByMerchantId($merchant_id);
        $this->assertEquals(0,$last_call_in_resource->auto_turned_off);




        // now send back a bad
        $packet_id = "STATUS-".time();
        $body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Response Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus Turn="BAT-A.00013" PosTerminal="mws" ResultCode="50" ResultText="Pos is not runnning"/></Response></CommunicationCenter>';
        $response = $this->makeRequest($callback_url,null,'POSTXML',null,$body);

        $merchant_resource = $merchant_resource->getRefreshedResource();
        $this->assertEquals('N',$merchant_resource->ordering_on,"ordering_on should have been set to 'N'");
        $this->assertEquals('F',$merchant_resource->inactive_reason,"inactive_reason should have been set to 'F'");

        $dciha = new DeviceCallInHistoryAdapter(getM());
        $last_call_in_resource = $dciha->getCallInResourceByMerchantId($merchant_id);
        $this->assertEquals(1,$last_call_in_resource->auto_turned_off);


        // send back a second bad and make sure no email gets sent
        $packet_id = "STATUS-".time();
        $body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Response Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus Turn="BAT-A.00013" PosTerminal="mws" ResultCode="50" ResultText="Pos is not runnning"/></Response></CommunicationCenter>';
        $response = $this->makeRequest($callback_url,null,'POSTXML',null,$body);




        // now send back a good
        $body = '<?xml version="1.0" encoding="UTF-16"?><CommunicationCenter MessageCenter="123456" PacketId="'.$packet_id.'" Asynchronous="0"><Response Type="OrderHandler" Id="'.$packet_id.'" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus Turn="BAT-A.00001" PosTerminal="mws" PosId="3" ResultCode="0" ResultText="Pos is ready"/></Response></CommunicationCenter>';
        $response = $this->makeRequest($callback_url,null,'POSTXML',null,$body);

        $merchant_resource = $merchant_resource->getRefreshedResource();
        $this->assertEquals('Y',$merchant_resource->ordering_on,"ordering_on should have been set to 'N'");

        $dciha = new DeviceCallInHistoryAdapter(getM());
        $last_call_in_resource = $dciha->getCallInResourceByMerchantId($merchant_id);
        $this->assertEquals(0,$last_call_in_resource->auto_turned_off);

        // now send back a weird
        $body = '<?xml version="1.0" encoding="UTF-8"?><CommunicationCenter MessageCenter="123456" PacketId="STATUS-1615580180" Asynchronous="0"><Response Type="OrderHandler" Id="STATUS-1615580180" ExpirationTime="2055-08-15T12:05:00.000"><PosStatus ResultCode="10" ResultText="Error Creating SendOrderHandler component"/></Response></CommunicationCenter>';
        $response = $this->makeRequest($callback_url,null,'POSTXML',null,$body);

        $merchant_resource = $merchant_resource->getRefreshedResource();
        $this->assertEquals('N',$merchant_resource->ordering_on,"ordering_on should have been set to 'N'");

        $dciha = new DeviceCallInHistoryAdapter(getM());
        $last_call_in_resource = $dciha->getCallInResourceByMerchantId($merchant_id);
        $this->assertEquals(1,$last_call_in_resource->auto_turned_off);



    }


    function testComprisMessageScheduling()
    {
        setContext('com.splickit.vtwoapi');
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $merchant_alphanumeric = $merchant_resource->alphanumeric_id;

        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_resource->merchant_id,"message_format"=>'O',"delivery_addr"=>"Compris","message_type"=>"X"));

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $url = "http://127.0.0.1:".$this->api_port."/app2/apiv2/cart/checkout";
        $external_id = getContext();
        $headers = array("X_ORDER140_CLIENT_ID"=>"$external_id","X_ORDER140_CLIENT_DEVICE"=>"unit_testing","HTTP_X_ORDER140_CLIENT_VERSION"=>"10.5.0","X_ORDER140_CLIENT"=>"APIDispatchTest","NO_CC_CALL"=>"true");
        $header_array = array();
        foreach ($headers as $key=>$value) {
            $header_array[] = $key.":".$value;
        }
        $response = $this->makeRequest($url, $user['email'].":welcome","POST",$header_array,$cart_data);

        $checkout_result_as_array = json_decode($response,true);
        $this->assertEquals('200', $this->info['http_code'],"an error was thrown: ".$checkout_result_as_array['error']['error']);
        $checkout_data = $checkout_result_as_array['data'];
        $this->assertNull($checkout_result_as_array['error']);
        $cart_ucid = $checkout_data['ucid'];

        $order_data = array();
        $order_data['tip'] = 0.00;
        $payment_array = $checkout_data['accepted_payment_types'];
        $order_data['merchant_payment_type_map_id'] = $payment_array[0]['merchant_payment_type_map_id'];
        $order_data['requested_time'] = getTomorrowTwelveNoonTimeStampDenver();

        $url = "http://127.0.0.1:".$this->api_port."/app2/apiv2/orders/$cart_ucid";
        $response = $this->makeRequest($url, $user['email'].":welcome","POST",$header_array,$order_data);
        $request_result_as_array = json_decode($response,true);
        $this->assertEquals('200', $this->info['http_code'],"an error was thrown: ".$request_result_as_array['error']['error']);
        $this->assertNull($request_result_as_array['error'],"an error was thrown: ".$request_result_as_array['error']['error']);
        $order_id = $request_result_as_array['data']['order_id'];
        $first_order_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'O');

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $url = "http://127.0.0.1:".$this->api_port."/app2/apiv2/cart/checkout";
        $external_id = getContext();
        $headers = array("X_ORDER140_CLIENT_ID"=>"$external_id","X_ORDER140_CLIENT_DEVICE"=>"unit_testing","HTTP_X_ORDER140_CLIENT_VERSION"=>"10.5.0","X_ORDER140_CLIENT"=>"APIDispatchTest","NO_CC_CALL"=>"true");
        $header_array = array();
        foreach ($headers as $key=>$value) {
            $header_array[] = $key.":".$value;
        }
        $response = $this->makeRequest($url, $user['email'].":welcome","POST",$header_array,$cart_data);

        $checkout_result_as_array = json_decode($response,true);
        $this->assertEquals('200', $this->info['http_code'],"an error was thrown: ".$checkout_result_as_array['error']['error']);
        $checkout_data = $checkout_result_as_array['data'];
        $this->assertNull($checkout_result_as_array['error']);
        $cart_ucid = $checkout_data['ucid'];

        $order_data = array();
        $order_data['tip'] = 0.00;
        $payment_array = $checkout_data['accepted_payment_types'];
        $order_data['merchant_payment_type_map_id'] = $payment_array[0]['merchant_payment_type_map_id'];
        $order_data['requested_time'] = getTomorrowTwelveNoonTimeStampDenver();

        $url = "http://127.0.0.1:".$this->api_port."/app2/apiv2/orders/$cart_ucid";
        $response = $this->makeRequest($url, $user['email'].":welcome","POST",$header_array,$order_data);
        $request_result_as_array = json_decode($response,true);
        $this->assertEquals('200', $this->info['http_code'],"an error was thrown: ".$request_result_as_array['error']['error']);
        $this->assertNull($request_result_as_array['error'],"an error was thrown: ".$request_result_as_array['error']['error']);
        $order_id = $request_result_as_array['data']['order_id'];
        $second_order_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'O');

        $diff = $second_order_message_resource->next_message_dt_tm - $first_order_message_resource->next_message_dt_tm;
        $this->assertEquals(10,$diff,"It should have a difference equal to the buffer");


    }


    function testCompris()
    {

        $menu_id = createTestMenuWithNnumberOfItems(1);

        $modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 5);
        $modifier_group_id = $modifier_group_resource->modifier_group_id;
        $item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', getM());
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id);
        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $merchant_alphanumeric = $merchant_resource->alphanumeric_id;

        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_resource->merchant_id,"message_format"=>'O',"delivery_addr"=>"Compris","message_type"=>"X"));

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $compris_message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'O');
        $this->assertNotNull($compris_message_resource);
        $compris_message_resource->next_message_dt_tm = time() - 10;
        $compris_message_resource->save();



        $url = "http://127.0.0.1:".$this->api_port."/app2/messagemanager/getnextmessagebymerchantid/$merchant_alphanumeric/compris";

        $response = $this->makeRequest($url,null);
        $this->assertContains('COMPRIS TEMPLATE',$response);

    }

    function testEmailSend()
    {
        setContext('com.splickit.worldhq');
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        setProperty('test_addr_email','adam@order140.com');

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data);
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00,null);
        $this->assertNull($order_resource->error);


        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'Econf');
        $message_resource->next_message_dt_tm = time()-100;
        $message_resource->save();

        setProperty('mandrill_key','vZ6mjy4OhHNZXI0tip3ckA');
        $email_controller = ControllerFactory::generateFromMessageResource($message_resource);
        $email_controller->sendThisMessage($message_resource);

    }

    function testPasswordProtectSomeMessages()
    {
        $password_skin_resource = getOrCreateSkinAndBrandIfNecessary("cloudrestrictiveskin",'cloudrestrictivebrand',$skin_id,$brand_id);
        $password_skin_resource->mobile_app_type = 'R';
        $password_skin_resource->password = 'nullit';
        $password_skin_resource->save();
        $public_client_id = $password_skin_resource->public_client_id;
        setContext('com.splickit.worldhq');
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $data['message_data'] = ["message_format"=>"J",'delivery_addr'=>"Ghost Kitchen"];
        $merchant_resource = createNewTestMerchant($menu_id,$data);
        $merchant_id = $merchant_resource->merchant_id;
        $merchant_numeric = $merchant_resource->numeric_id;
        SkinMerchantMapAdapter::createSkinMerchantMapRecord($merchant_resource->merchant_id,$password_skin_resource->skin_id);

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data);
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'J');
        $message_resource->next_message_dt_tm = time()-660;
        $message_resource->save();

        $url = "http://127.0.0.1:".$this->api_port."/app2/messagemanager/getnextmessagebymerchantid/$merchant_numeric/json";

        $headers = ["X_SPLICKIT_CLIENT_ID: $public_client_id"];
        $response = $this->makeRequest($url,null,'GET',$headers);
        $this->assertEquals("Unauthorized Request",$response);
        $response = $this->makeRequest($url,'message_admin:test12345','GET',$headers);
        $this->assertEquals("Unauthorized Request",$response);
        $password_skin_resource->password = Encrypter::Encrypt('test12345');
        $password_skin_resource->save();
        $response = $this->makeRequest($url,'message_admin:test54321','GET',$headers);
        $this->assertEquals("Unauthorized Request",$response);
        $response = $this->makeRequest($url,'message_admin:test12345','GET',$headers);
        $this->assertNotEquals("Unauthorized Request",$response);
        $message_json_as_array = json_decode($response,true);
        $this->assertEquals($order_resource->order_id,$message_json_as_array['order_id'],'It should have retrieved the message for the order');
    }

    function testJsonController()
    {
        setContext('com.splickit.worldhq');
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id);
        $merchant_id = $merchant_resource->merchant_id;
        $merchant_numeric = $merchant_resource->numeric_id;
        $map_resource = Resource::createByData(new MerchantMessageMapAdapter(getM()),array("merchant_id"=>$merchant_id,"message_format"=>'J',"delivery_addr"=>"Ghost Kitchen","message_type"=>"X"));

        $url = "http://127.0.0.1:".$this->api_port."/app2/messagemanager/getnextmessagebymerchantid/$merchant_numeric/json";
        $response = $this->makeRequest($url,null);

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data);
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00,null);
        $this->assertNull($order_resource->error);


        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'J');
        $message_resource->next_message_dt_tm = time()-100;
        $message_resource->save();
        $expected_message_text = $message_resource->portal_order_json;

        $response = $this->makeRequest($url,null);

        $this->assertEquals($expected_message_text,cleanUpCRLFTFromString($response),"It should have the expected message text");
    }


    function testStarPrinter()
    {
        setContext('com.splickit.worldhq');
        $merchant_resource = createNewTestMerchant($this->ids['menu_id']);
        $merchant_numeric_id = $merchant_resource->numeric_id;
        $merchant_id = $merchant_resource->merchant_id;
        attachMerchantToSkin($merchant_resource->merchant_id, $this->ids['skin_id']);
        $map_resource = MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'RUC','Starmicros','X');

        $url = "http://127.0.0.1:" . $this->api_port . "/app2/messagemanager/getnextmessagebymerchantid/$merchant_numeric_id/starmicros/message.txt";

        $body = '{"status": "23 6 0 0 0 4 0 0 0 ","printerMAC": "00:11:62:0d:77:9f","statusCode": "211%20OK%20Paper%20Low","printingInProgress": false,"clientAction": null}';
        $data = json_decode($body,true);
        $response = $this->makeRequest($url,null,'POST',NULL,$data);

        $this->assertEquals('{"jobReady": false,"mediaTypes": ["text/plain"]}',$response);

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $order_data = OrderAdapter::staticGetSimpleOrderArrayByMerchantId($merchant_resource->merchant_id, 'pickup', 'sum dum note');
        $order_data['tip'] = 0.00;
        $order_resource = placeOrderFromOrderData($order_data, $time);
        $this->assertNull($order_resource->error);
        $order_id = $order_resource->order_id;
        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'RUC');
        $message_resource->next_message_dt_tm = time() - 100;
        $message_resource->save();

        $response = $this->makeRequest($url,null,'POST',NULL,$data);
        $this->assertEquals('{"jobReady": true,"mediaTypes": ["text/plain"]}',$response);

        // validate that the message was not picked up
        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'RUC');
        $this->assertEquals('P',$message_resource->locked);

        $url = "http://127.0.0.1:" . $this->api_port . "/app2/messagemanager/getnextmessagebymerchantid/$merchant_numeric_id/starmicros/message.txt?mac=00:11:62:0d:77:9f&type=text/plain";
        $response = $this->makeRequest($url,'GET');

        $this->assertContains("Order Id:  $order_id",$response);

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_resource->order_id,'RUC');
        $this->assertEquals('S',$message_resource->locked);
    }


    function testFoundryGetNextMessageBadAlphanumeric()
    {
        $bad_alpha = 'yh76gt54r3ew';
        $url = "http://127.0.0.1:".$this->api_port."/app2/messagemanager/foundry/getnextmessagebymerchantid?reqtype=01&siteid=$bad_alpha&reqfmt=01";
        $response = $this->makeRequest($url,null);
        $this->assertEquals(422,$this->info['http_code'],"it should have sent back a unprocessable entitiy error");
        $this->assertEquals("No matching merchant for: $bad_alpha",$response);
    }

    function testFoundryGetSendMenuResponse()
    {
        $skin_resource = getOrCreateSkinAndBrandIfNecessary("foundskin",'foundbrand',null,null);
        setContext($skin_resource->external_identifier);
        $merchant_resource = createNewTestMerchant();
        $data['merchant_id'] = $merchant_resource->merchant_id;
        $data['menu_id'] = $this->ids['menu_id'];
        // we do it this way to prevent the creation of daughter records till the import
        Resource::createByData(new MerchantMenuMapAdapter($m),$data);
        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $foundry_third_party_parameters = $tpppa->getRecords(['third_party_provider_id' => 1005]);
        foreach ($foundry_third_party_parameters as $foundry_parameter) {
            $name = $foundry_parameter['name'];
            if ($name == 'modecode') {
                $value = '1';
            } else if ($name == 'modecode_holds') {
                $value = '2';
            } else if ($name == 'promo_id') {
                $value = '12345';
            } else if ($name == 'tender_id_visa') {
                $value = '215';
            } else if ($name == 'tender_id_mastercard') {
                $value = '216';
            } else if ($name == 'tender_id_american_express') {
                $value = '217';
            } else if ($name == 'tender_id_discover') {
                $value = '218';
            } else if ($name == 'brand_order_mode_delivery') {
                $value = '3';
            } else if ($name == 'brand_order_mode_pickup') {
                $value = '3';
            } else if ($name == 'get_menu') {
                $value = '0';
                $tpppmm_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()), ["merchant_id" => $merchant_resource->merchant_id, "third_party_provider_parameter_id" => $foundry_parameter['id'], "value" => $value]);
                continue;
            }
            $tpppbm_resource = Resource::createByData(new ThirdPartyProviderParameterBrandMapsAdapter(getM()), ["brand_id" => $brand_id, "third_party_provider_parameter_id" => $foundry_parameter['id'], "value" => $value]);
        }

        //$maps_resource = Resource::createByData(new MerchantFoundryInfoMapsAdapter($m),$data);

        $url = "http://127.0.0.1:".$this->api_port."/app2/messagemanager/foundry/getnextmessagebymerchantid?reqtype=01&siteid=".$merchant_resource->alphanumeric_id."&reqfmt=01";
        $response = makeLocalhostRequest($url,$up);
        $this->assertEquals("<SVRRESPONSE>1</SVRRESPONSE>",$response['result']);

        $tpppmm_resource->value = 1;
        $tpppmm_resource->save();

        $url = "http://127.0.0.1:".$this->api_port."/app2/messagemanager/foundry/getnextmessagebymerchantid?reqtype=01&siteid=".$merchant_resource->alphanumeric_id."&reqfmt=01";
        $response = makeLocalhostRequest($url,$up);
        $expected = '<SVRRESPONSE REQTYPE="01" SITEID="'.$merchant_resource->alphanumeric_id.'" SVRREQ="1"  >1</SVRRESPONSE>';
        $this->assertEquals("$expected",$response['result']);

        $tpppmm_resource = $tpppmm_resource->getRefreshedResource();
        $this->assertEquals(0,$tpppmm_resource->value,"shoudl have set the flag back to false");

    }


    function testCheckForFoundryMessagesReadyToBePickedUpNoMessageReady()
    {
        $merchant_id = $this->ids['merchant_alphanumeric_id'];
        $url = "http://127.0.0.1:".$this->api_port."/app2/messagemanager/foundry/getnextmessagebymerchantid?reqtype=01&siteid=$merchant_id&reqfmt=01";
        $response = $this->makeRequest($url,$up);
        $this->assertEquals("<SVRRESPONSE>1</SVRRESPONSE>",$response);

    }

    function testCheckForFoundryMessagesReadyToBePickedUpWithMessageReady()
    {
        $merchant_resource = createNewTestMerchant();
        $maps_resource = Resource::createByData(new MerchantFoundryInfoMapsAdapter($m),array("merchant_id"=>$merchant_resource->merchant_id));
        $url = "http://127.0.0.1:".$this->api_port."/app2/messagemanager/foundry/getnextmessagebymerchantid?reqtype=01&siteid=".$merchant_resource->alphanumeric_id."&reqfmt=01";
        $response = $this->makeRequest($url,$up);
        $this->assertEquals("<SVRRESPONSE>1</SVRRESPONSE>",$response);
        createMessages(1,$merchant_resource->merchant_id,'U','P');
        $url = "http://127.0.0.1:".$this->api_port."/app2/messagemanager/foundry/getnextmessagebymerchantid?reqtype=01&siteid=".$merchant_resource->alphanumeric_id."&reqfmt=01";
        $response = $this->makeRequest($url,$up);
        $this->assertEquals("<SVRRESPONSE>2</SVRRESPONSE>",$response);
    }

    function testPullFoundryMessage()
    {
        $skin_resource = getOrCreateSkinAndBrandIfNecessary("foundskin",'foundbrand',null,null);
        setContext($skin_resource->external_identifier);
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 1);
        $modifier_size_map = $modifier_group_resource->modifier_items[0]->modifier_size_map;
        $modifier_size_map->external_id = '12345:'.$modifier_size_map->external_id;
        //$modifier_size_map->external_id = '98765:2-12345:'.$modifier_size_map->external_id;
        $modifier_size_map->save();
        $item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', getM());
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_resource->modifier_group_id, 0);

        $merchant_resource = createNewTestMerchant($menu_id,['authorize'=>true]);
        Resource::createByData(new MerchantMenuMapAdapter($m),$data);
        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $foundry_third_party_parameters = $tpppa->getRecords(['third_party_provider_id' => 1005]);
        $test_merchant_tender_id = '76543';
        foreach ($foundry_third_party_parameters as $foundry_parameter) {
            $name = $foundry_parameter['name'];
            if ($name == 'modecode') {
                $value = '1';
            } else if ($name == 'modecode_holds') {
                $value = '2';
            } else if ($name == 'tender_id_visa') {
                $value = $test_merchant_tender_id;
            } else if ($name == 'brand_order_mode_delivery') {
                $value = '6';
            } else if ($name == 'brand_order_mode_pickup') {
                $value = '3';
            } else if ($name == 'get_menu') {
                $value = '0';
            } else {
                continue;
            }
            $tpppmm_resource = Resource::createByData(new ThirdPartyProviderParameterMerchantMapsAdapter(getM()), ["merchant_id" => $merchant_resource->merchant_id, "third_party_provider_parameter_id" => $foundry_parameter['id'], "value" => $value]);
        }




        MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id,'U','Foundry','X');
        $url = "http://127.0.0.1:".$this->api_port."/app2/messagemanager/foundry/getnextmessagebymerchantid?reqtype=01&siteid=".$merchant_resource->alphanumeric_id."&reqfmt=01";
        $response = $this->makeRequest($url,$up);
        $this->assertEquals("<SVRRESPONSE>1</SVRRESPONSE>",$response);

        $user_resource = createNewUserWithCCNoCVV();
        $user = logTestUserResourceIn($user_resource);
        $note = 'sum dum note';
        $cart_data = OrderAdapter::staticGetSimpleOrderArrayByMerchantId($merchant_resource->merchant_id, 'pickup', $note);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,time());
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_resource->merchant_id,0.00,time());
        $order_id = $order_resource->order_id;
        $this->assertNull($order_resource->error);


        $balance_change_adapter = new BalanceChangeAdapter(getM());
        if ($balance_change_resources = Resource::findAll($balance_change_adapter,'',[3=>["order_id"=>$order_id]])) {
            foreach ($balance_change_resources as $bcr) {
                if ($bcr->process == 'CCpayment') {
                    $bcr->process = 'authorize';
                    $bcr->card_info = 'visa-1111';
                    $bcr->save();
                }
            }
        }
        sleep(2);

        //set the balance chage recrods corerctly

        $response = $this->makeRequest($url,$userpassword);
        $this->assertEquals("<SVRRESPONSE>2</SVRRESPONSE>",$response);

        $url = "http://127.0.0.1:".$this->api_port."/app2/messagemanager/foundry/getnextmessagebymerchantid?reqtype=02&siteid=".$merchant_resource->alphanumeric_id."&reqfmt=01";
        $order_message_payload = cleanUpDoubleSpacesCRLFTFromString($this->makeRequest($url,$userpassword));
        $order_message_payload = str_replace("> <","><",$order_message_payload);

        $expected_payload = cleanUpDoubleSpacesCRLFTFromString(file_get_contents("./unit_tests/resources/expected_Foundry_message_body.txt"));
        $order_id = $order_resource->order_id;
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);

        $user = $complete_order['user'];

        $expected_payload = str_replace("%%ready_time%%",$complete_order['pickup_time_military_with_seconds'],$expected_payload);
        $expected_payload = str_replace("%%ready_date_time%%",$complete_order['pickup_date_time_foundry'],$expected_payload);
        $expected_payload = str_replace("%%pickup_time_ampm%%",$complete_order['pickup_time_ampm'],$expected_payload);
        $expected_payload = str_replace("%%order_id%%",$order_id,$expected_payload);
        $expected_payload = str_replace("%%note%%",$note,$expected_payload);
        $expected_payload = str_replace("%%user_email%%",$user['email'],$expected_payload);
        $expected_payload = str_replace("%%user_phone_no%%",$user['contact_no'],$expected_payload);
        $expected_payload = str_replace("%%item_external_id%%",$complete_order['order_details'][0]['external_id'],$expected_payload);

        $mods_stuff = explode(':',$complete_order['order_details'][0]['order_detail_complete_modifier_list_no_holds'][0]['external_id']);
        $expected_payload = str_replace("%%modifier_external_id%%",$mods_stuff[1],$expected_payload);
        $expected_payload = str_replace("%%modifier_group_external_id%%",$mods_stuff[0],$expected_payload);
        $expected_payload = str_replace('TENDERID="215"','TENDERID="'.$test_merchant_tender_id.'"',$expected_payload);
//
//        $expected_payload = str_replace("%%ready_time%%",$complete_order['pickup_time_military_with_seconds'],$expected_payload);
//        $expected_payload = str_replace("%%ready_date_time%%",$complete_order['pickup_date_time4'],$expected_payload);
//
//        $expected_payload = str_replace("%%order_id%%",$order_id,$expected_payload);
//        $expected_payload = str_replace("%%user_email%%",$user['email'],$expected_payload);
//        $expected_payload = str_replace("%%item_external_id%%",$complete_order['order_details'][0]['external_id'],$expected_payload);
//        $expected_payload = str_replace("%%modifier_external_id%%",$complete_order['order_details'][0]['order_detail_complete_modifier_list_no_holds'][0]['external_id'],$expected_payload);
        $expected_payload = str_replace("> <","><",$expected_payload);
        //$this->assertEquals($expected_payload,$order_message_payload);

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'U');
        $this->assertEquals($expected_payload,cleanUpDoubleSpacesCRLFTFromString($message_resource->message_text),"It should have saved the message text on the message resource");

        return $order_resource->order_id;
    }

    /**
     * @depends testPullFoundryMessage
     */
    function testFoundryCallBack($order_id)
    {
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_id,$m);
        $alphanumeric_id = $complete_order['merchant']['alphanumeric_id'];
        $url = "http://127.0.0.1:".$this->api_port."/app2/messagemanager/foundry/getnextmessagebymerchantid?reqtype=05&siteid=$alphanumeric_id&reqfmt=01";
        $call_back_payload = cleanUpDoubleSpacesCRLFTFromString('<POSRESPONSE EXTTERMID="" EXTREQUESTID="">
	<CHECKRESPONSES>
		<ADDCHECK ORDERID="'.$order_id.'" EXTCHECKID="SPLICKIT-'.$order_id.'" INTCHECKID="37" POSCHECKTOTAL="'.$complete_order['grand_total'].'" POSCHECKSUBTOTAL="'.$complete_order['order_amt'].'" POSCHECKTAX="'.$complete_order['total_tax'].'" ITEMERRORS="0" TENDERERRORS="0" SYSTEMERRORS="0" DONOTRESEND="TRUE">
			<LOGS COUNT="0" />
			<DEBUGS COUNT="2">
				<DEBUG TEXT="D150, Enhanced External Check ID: MYORDER-123" />
				<DEBUG TEXT="D150, MicrosTransaction.PostTransaction ChkNum=1924, ChkId=MYORDER-123, SubTtl=14.95, Tax=1.30, TtlDue=16.25" />
			</DEBUGS><ERRORS COUNT="0" />
		</ADDCHECK>
	</CHECKRESPONSES>
	<PRINTRESPONSES />
	<ERRORS COUNT="0" />
	<DEBUGS COUNT="2">
		<DEBUG TEXT="D150, Enhanced External Check ID: MYORDER-123" />
		<DEBUG TEXT="D150, MicrosTransaction.PostTransaction ChkNum=1924, ChkId=MYORDER-123, SubTtl=14.95, Tax=1.30, TtlDue=16.25" />
	</DEBUGS>
	<LOGS COUNT="0" />
</POSRESPONSE>');

        $result = makeLocalhostRequest($url,$up,'POST',$call_back_payload,'application/xml');
        $info = $result['curl_info'];
        $this->assertEquals(422,$info['http_code'],"It should have resulted in an unprocessable entitiry since reqtype=5 is not valid");

//        $curl = curl_init($url);
//        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
//
//        curl_setopt($curl, CURLOPT_POSTFIELDS, $call_back_payload);
//        curl_setopt($curl, CURLOPT_POST, 1);
//        $headers[] = 'Content-type: application/xml; charset=utf-8';
//        $headers[] = 'Content-Length: '.strlen($call_back_payload);
//        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
//        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
//        $result = curl_exec($curl);
//        $this->info = curl_getinfo($curl);
//        curl_close($curl);
        $url = "http://127.0.0.1:".$this->api_port."/app2/messagemanager/foundry/getnextmessagebymerchantid?reqtype=04&siteid=$alphanumeric_id&reqfmt=01";
        $result = makeLocalhostRequest($url,$up,'POST',$call_back_payload,'application/xml');
        $info = $result['curl_info'];
        $this->assertEquals(200,$info['http_code']);

        $message_resource = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'U');
        $this->assertEquals($call_back_payload,$message_resource->response,"It should have saved the response on the message resource");
        $this->assertEquals('V',$message_resource->viewed);
    }

    function testFoundryCallBackBadPost()
    {
        $alphanumeric_id = null;
        $url = "http://127.0.0.1:".$this->api_port."/app2/messagemanager/foundry/getnextmessagebymerchantid?reqtype=04&siteid=$alphanumeric_id&reqfmt=01";
        $call_back_payload = cleanUpDoubleSpacesCRLFTFromString('<POSRESPONSE EXTTERMID="" EXTREQUESTID="">
	<CHECKRESPONSES>
		<ADDCHECK ORDERID="" EXTCHECKID="SPLICKIT-88888888" INTCHECKID="37" POSCHECKTOTAL="10.88" POSCHECKSUBTOTAL="5.99" POSCHECKTAX="5.77" ITEMERRORS="0" TENDERERRORS="0" SYSTEMERRORS="0" DONOTRESEND="TRUE">
			<LOGS COUNT="0" />
			<DEBUGS COUNT="2">
				<DEBUG TEXT="D150, Enhanced External Check ID: MYORDER-123" />
				<DEBUG TEXT="D150, MicrosTransaction.PostTransaction ChkNum=1924, ChkId=MYORDER-123, SubTtl=14.95, Tax=1.30, TtlDue=16.25" />
			</DEBUGS><ERRORS COUNT="0" />
		</ADDCHECK>
	</CHECKRESPONSES>
	<PRINTRESPONSES />
	<ERRORS COUNT="0" />
	<DEBUGS COUNT="2">
		<DEBUG TEXT="D150, Enhanced External Check ID: MYORDER-123" />
		<DEBUG TEXT="D150, MicrosTransaction.PostTransaction ChkNum=1924, ChkId=MYORDER-123, SubTtl=14.95, Tax=1.30, TtlDue=16.25" />
	</DEBUGS>
	<LOGS COUNT="0" />
</POSRESPONSE>');
        $result = makeLocalhostRequest($url,$up,'POST',$call_back_payload,'application/xml');
//        $curl = curl_init($url);
//        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
//        curl_setopt($curl, CURLOPT_POSTFIELDS, $call_back_payload);
//        curl_setopt($curl, CURLOPT_POST, 1);
//        $headers[] = 'Content-type: application/xml; charset=utf-8';
//        $headers[] = 'Content-Length: '.strlen($call_back_payload);
//        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
//        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
//        $result = curl_exec($curl);
//        $info = curl_getinfo($curl);
//        curl_close($curl);

        $this->assertEquals(422,$result['curl_info']['http_code']);
    }



    function makeRequest($url,$userpassword,$method = 'GET', $headers = array(),$data = null)
    {
        unset($this->info);
        $method = strtoupper($method);
        $curl = curl_init($url);
        if ($userpassword) {
            curl_setopt($curl, CURLOPT_USERPWD, $userpassword);
        }
        if ($method == 'POST') {
            $json = json_encode($data);
            curl_setopt($curl, CURLOPT_POST, 1);
            curl_setopt($curl, CURLOPT_POSTFIELDS,$json);
            $headers[] = 'Content-Type: application/json';
            $headers[] = 'Content-Length: ' . strlen($json);
        } else if ($method == 'POSTXML') {
            $method = 'POST';
            curl_setopt($curl, CURLOPT_POST, 1);
            curl_setopt($curl, CURLOPT_POSTFIELDS,$data);
            $headers[] = 'Content-Type: application/xml';
            $headers[] = 'Content-Length: ' . strlen($data);
        } else if ($method == 'DELETE') {
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "DELETE");
        }
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        logCurl($url,$method,$userpassword,$headers,$json);
        $result = curl_exec($curl);
        $this->info = curl_getinfo($curl);
        curl_close($curl);
        return $result;
    }

    static function setUpBeforeClass()
    {
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time',300);
        createWorldHqSkin();

        $menu_id = createTestMenuWithOneItem('item_one');
        $ids['menu_id'] = $menu_id;
        $_SERVER['no_new_merchant_payment'] = true;
        $modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 1);
        $item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
        assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_resource->modifier_group_id, 0);

        $merchant_resource = createNewTestMerchant($menu_id);
        $maps_resource = Resource::createByData(new MerchantFoundryInfoMapsAdapter($m),array("merchant_id"=>$merchant_resource->merchant_id));
        $ids['merchant_id'] = $merchant_resource->merchant_id;
        $ids['merchant_alphanumeric_id'] = $merchant_resource->alphanumeric_id;
        $user_resource = createNewUser(array("flags"=>"1C20000001"));
        $ids['user_id'] = $user_resource->user_id;
        $ids['user'] = $user_resource->getDataFieldsReally();
        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
        $mmha = new MerchantMessageHistoryAdapter(getM());
        $sql = "TRUNCATE Merchant_Message_History";
        $mmha->_query($sql);
    }

    static function tearDownAfterClass()
    {
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }



}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    MessageDispatchTest::main();
}

?>