<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class PunchServiceTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
        $this->ids = $_SERVER['unit_test_ids'];

    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }

    function testAuthenticateWithTokenBadToken()
    {
        $punch_loyalty_service = new PunchLoyaltyService();
        $results = $punch_loyalty_service->authenticate("456uijhgfrtyuikmnbgtyuikm");
        $this->assertEquals(401,$results['http_code']);
    }

    function testAuthenticateWithGoodToken()
    {
        // token must be retrieved from the logs of the web server when you run this. Token will be good for 1 hour
//        $punch_token = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJodHRwczovL3B1bmNoaC5jb20iLCJlbWFpbF92ZXJpZmllZCI6ZmFsc2UsInN1YiI6MTA5NTg1NSwiZW1haWwiOiJhZGFtQHNwbGlja2l0LmNvbSIsImlhdCI6MTQ2MDQ4ODUzNiwiZXhwIjoxNDYwNDkyMTM2fQ.J7iiSugI72h6RfWJD8xKlVZZLadB0rDTe3_xoGMHdg0";
//        $punch_loyalty_service = new PunchLoyaltyService();
//        $results = $punch_loyalty_service->authenticate($punch_token);
//        $this->assertEquals(200,$results['http_code']);
//        $result_array = json_decode($results['raw_result'],true);
//        $this->assertNotNull($result_array['user'],"successfull response should have the user section");
//        $user_info['authentication_token'] = $result_array['user']['authentication_token'];
        $user_info['authentication_token'] = 'VRYyUCD4JP9tDcyeYzPn';
        $user_info['first_name'] = 'Tarek';
        //$user_info['last_name'] = 'Dim';
        return $user_info;
    }

    /**
     * @depends testAuthenticateWithGoodToken
     */
    function testGetBalanceForExisting($user_info)
    {
        $punch_user_authentication_token = $user_info['authentication_token'];
        $punch_loyalty_service = new PunchLoyaltyService();
        $results = $punch_loyalty_service->getBalance($punch_user_authentication_token);
        logData($results,"punch results");
        $this->assertEquals(200,$punch_loyalty_service->curl_response['http_code']);
        $this->assertContains('banked_rewards',$results);
        $user_info['results'] = $results;
        return $user_info;
    }

    /**
     * @depends testGetBalanceForExisting
     */
    function testRewardAdam($user_info)
    {
        $punch_user_authentication_token = $user_info['authentication_token'];
        $order_id = generateUUID();
        $json = cleanUpDoubleSpacesCRLFTFromString('{
				"receipt_amount":29.51,
				"cc_last4":"1234",
				"store_number":"123",
				"employee_id":1,
				"employee_name":"Splickit Server",
				"menu_items":[
				   {
					  "item_name":"Ancho Chicken Bowl",
					  "item_qty":1,
					  "item_amount":8.88,
					  "menu_item_type":"M",
					  "menu_item_id":1940,
					  "menu_family":"800",
					  "menu_major_group":"152"
				   },{
					  "item_name":"Ancho Chicken Bowl",
					  "item_qty":1,
					  "item_amount":8.88,
					  "menu_item_type":"M",
					  "menu_item_id":1940,
					  "menu_family":"800",
					  "menu_major_group":"152"
				   },{
					  "item_name":"Ancho Chicken Bowl",
					  "item_qty":1,
					  "item_amount":8.88,
					  "menu_item_type":"M",
					  "menu_item_id":1940,
					  "menu_family":"800",
					  "menu_major_group":"152"
				   }
				],
				"subtotal_amount":25.96,
				"receipt_datetime":"'.date('Y-m-d',time()).'T'.date('H:i:sP',time()).'",
				"transaction_no":'.rand(10000,999999).',
				"external_uid":"'.$order_id.'",
				"client":"'.getProperty('punch_client_id').'"
				}');
        $payload_array = json_decode($json,true);
        $punch_loyalty_service = new PunchLoyaltyService();
        $results = $punch_loyalty_service->rewardPoints($payload_array,$punch_user_authentication_token);
        $this->assertEquals(200,$results['http_code']);
        $raw_results = $results['raw_result'];
        $results_array = json_decode($raw_results,true);
        $this->assertEquals($user_info['first_name'],$results_array['first_name'],"It should have been the same user");
        $checkin_data = $results_array['checkin'];
        $this->assertEquals($order_id,$checkin_data['external_uid'],"It should have had the same external id as the order");
    }

    /**
     * @depends testGetBalanceForExisting
     */
    function testRedeemBalance($user_info)
    {
        $order_id = generateUUID();
        $json = cleanUpDoubleSpacesCRLFTFromString('{
                "discount_type":"discount_amount",
                "redeemed_points":5.00,
				"receipt_amount":8.50,
				"cc_last4":"1234",
				"employee_id":1,
				"employee_name":"Splickit Server",
				"menu_items":[
				   {
					  "item_name":"Ancho Chicken Bowl",
					  "item_qty":1,
					  "item_amount":7.00,
					  "menu_item_type":"M",
					  "menu_item_id":1940,
					  "menu_family":"800",
					  "menu_major_group":"152"
				   }
				],
				"subtotal_amount":7.00,
				"receipt_datetime":"'.date('Y-m-d',time()).'T'.date('H:i:sP',time()).'",
				"transaction_no":'.rand(10000,999999).',
				"external_uid":"'.$order_id.'",
				"client":"'.getProperty('punch_client_id').'"
				}');
        $payload_array = json_decode($json,true);
        $punch_loyalty_service = new PunchLoyaltyService();
        $punch_user_authentication_token = $user_info['authentication_token'];
        $results = $punch_loyalty_service->redemptionPurchase($payload_array,$punch_user_authentication_token);
        $this->assertEquals(200,$results['http_code']);
        $raw_results = $results['raw_result'];
        $results_array = json_decode($raw_results,true);
        $user_info['redemption_id'] = $results_array['redemption_id'];
        return $user_info;
    }

    /**
     * @depends testRedeemBalance
     */
    function testVoidRedemtion($user_info)
    {
        $data['redemption_id'] = $user_info['redemption_id'];
        $data['authentication_token'] = $user_info['authentication_token'];
        $punch_loyalty_service = new PunchLoyaltyService();
        $results = $punch_loyalty_service->voidRedemption($data,$user_info['authentication_token']);
        $this->assertEquals(202,$results['http_code']);

    }



    function testCreateUserWithServiceNeverUsed()
    {
        $user_resource = createNewUser();
        $user['email'] = $user_resource->email;
        $user['password'] = "punchh<3splickit";
        $user['first_name'] = $user_resource->first_name;
        $user['last_name'] = $user_resource->last_name;
        $user['zip_code'] = '90210';
        $user['birthday'] = '10/12/1999';
        $punch_loyalty_service = new PunchLoyaltyService();
        $results = $punch_loyalty_service->createUserOnPunchSystemNeverUsed($user);
        $this->assertEquals(201,$results['http_code']);
        $raw_results = $results['raw_result'];
        $results_array = json_decode($raw_results,true);
        $this->assertNotNull($results_array['user_id'],"should have a user id on the returned object");
        $this->assertNotNull($results_array['authentication_token'],"shojld have found an authentication token on the returned object");
        return $results_array;
    }

    /**
     * @depends testCreateUserWithServiceNeverUsed
     */
    function testRewardPurchase($user_info)
    {
        $punch_user_authentication_token = $user_info['authentication_token'];
        $order_id = generateUUID();
        $json = cleanUpDoubleSpacesCRLFTFromString('{
				"receipt_amount":8.88,
				"cc_last4":"1234",
				"employee_id":1,
				"employee_name":"Splickit Server",
				"menu_items":[
				   {
					  "item_name":"Ancho Chicken Bowl",
					  "item_qty":1,
					  "item_amount":8.88,
					  "menu_item_type":"M",
					  "menu_item_id":1940,
					  "menu_family":"800",
					  "menu_major_group":"152"
				   },{
					  "item_name":"Ancho Chicken Bowl",
					  "item_qty":1,
					  "item_amount":8.88,
					  "menu_item_type":"M",
					  "menu_item_id":1940,
					  "menu_family":"800",
					  "menu_major_group":"152"
				   }
				],
				"subtotal_amount":17.96,
				"payable":20.51,
				"receipt_datetime":"'.date('Y-m-d',time()).'T'.date('H:i:sP',time()).'",
				"transaction_no":'.rand(10000,999999).',
				"external_uid":"'.$order_id.'",
				"client":"'.getProperty('punch_client_id').'"
				}');
        $payload_array = json_decode($json,true);
        $punch_loyalty_service = new PunchLoyaltyService();
        $results = $punch_loyalty_service->rewardPoints($payload_array,$punch_user_authentication_token);
        $this->assertEquals(200,$results['http_code']);
        $raw_results = $results['raw_result'];
        $results_array = json_decode($raw_results,true);
        $this->assertEquals($user_info['first_name'],$results_array['first_name'],"It should have been the same user");
        $checkin_data = $results_array['checkin'];
        $this->assertEquals($order_id,$checkin_data['external_uid'],"It should have had the same external id as the order");
    }

    /**
     * @depends testCreateUserWithServiceNeverUsed
     */
    function testGetBalance($user_info)
    {
        $punch_user_authentication_token = $user_info['authentication_token'];
        $punch_loyalty_service = new PunchLoyaltyService();
        $results = $punch_loyalty_service->getBalance($punch_user_authentication_token);
        logData($results,"punch results");
        $this->assertEquals(200,$punch_loyalty_service->curl_response['http_code']);
        $this->assertContains('points_balance',$results);
        $this->assertContains('banked_rewards',$results);
    }

    static function setUpBeforeClass()
    {
        $_SERVER['request_time1'] = microtime(true);
        $_SERVER['NO_MOCKS'] = true;
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    PunchServiceTest::main();
}

?>