<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';


class PushMessageTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    var $apnsHost = 'gateway.push.apple.com';
    var $apnsPort = 2195;


    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__ . '-' . $_SERVER['STAMP'];
        $this->ids = $_SERVER['unit_test_ids'];
        setProperty('test_sms_messages_on', 'true');

    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }

    function testPushAndroidMessageWithPushController()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $umsma = new UserMessagingSettingMapAdapter($mimetypes);
        $token = "APA91bFEfy6eOqT5Z_r3haJDKNQzbG3ARwXPis7UAUQTP6mWCXdDq5nGbd-hpSdW0Qq8dW-yr2aO8j2hS6RAqvFsS_VZc4h5GdtXHZ88qeZT8s4K4JKy769eQ8V9qKl_GZCErx8CVsmd";
        $id = $umsma->createRecord($user_resource->user_id, 13, "push", "gcm", 'sumdumdeviceid1234', $token, "Y");
        $this->assertTrue($id > 0);
        $user_id = $user_resource->user_id;
        $message = "Hello".rand(1000,9999);
        $url = "http://localhost/app2/admin/pushmessage/?users=$user_id&message=$message&skin=com.splickit.pitapit";
        $curl = curl_init($url);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        $result = curl_exec($curl);
        $this->info = curl_getinfo($curl);
        curl_close($curl);

        $mmha = new MerchantMessageHistoryAdapter($m);
        $message_resource = Resource::find($mmha,null,array(TONIC_FIND_BY_METADATA=>array("message_text"=>$message)));
        $info = $message_resource->info;
        $this->assertContains('skin_id=13',$info,"It should have the skin id");

        $push_controller = ControllerFactory::generateFromMessageResource($message_resource);
        $this->assertInstanceOf('PushMessageController',$push_controller,"It should have generated a push controller");

        $result = $push_controller->sendThisMessage($message_resource);
        $this->assertTrue($result);

        $new_message_resource = Resource::find($mmha,null,array(TONIC_FIND_BY_METADATA=>array("message_text"=>$message)));
        $this->assertContains("multicast_id",$new_message_resource->response,"It should have the response");
    }

    function testCreatePushMessageControllerFromFactory()
    {
        $message_resource = Resource::dummyfactory(array("message_format" => 'Y'));
        $controller_name = ControllerFactory::getControllerNameFromMessageResource($message_resource);
        $this->assertEquals('PushMessage',$controller_name,"It should return PushMessage as the controller name");
        $controller = ControllerFactory::generateFromMessageResource($message_resource,$m,$u,$r,5);
        $this->assertEquals('PushMessageController', get_class($controller), "It should return a PushMessageController");
    }

    function testPushAndroidServiceSingleRecipient()
    {
        $device_token = "APA91bFEfy6eOqT5Z_r3haJDKNQzbG3ARwXPis7UAUQTP6mWCXdDq5nGbd-hpSdW0Qq8dW-yr2aO8j2hS6RAqvFsS_VZc4h5GdtXHZ88qeZT8s4K4JKy769eQ8V9qKl_GZCErx8CVsmd"; //trueforce android device

        $url = "https://fcm.googleapis.com/fcm/send";
        $fields = array(
            'to'  => $device_token,
            'data'              => array(
                "title" => "The Title",
                "message" => "You Just Won A New Car"
            )
        );
        $response = PushAndroidCurl::curlIt($url, $fields);
        $this->assertEquals(200,$response['http_code']);
        $response_array = json_decode($response['raw_result'],true);
        $this->assertEquals(1,$response_array['success']);
        $this->assertNotNull($response_array['results'][0]['message_id'],"It should have the message id");
    }

    /**
     * @expectedException UnsuccessfulAndroidPushException
     */
    function testPushAndroidServiceFailure()
    {
        $user_resource = createNewUser();
        $umsma = new UserMessagingSettingMapAdapter($mimetypes);
        $token = 'sdfghjklkjhgfvbnkl;jhgfgvhbjnklkjhjgf-UzMUKEc0eP_gInWmQNLAP6brQ9naXEnWDTvMFEPrsZk5nQtXdj9rWPuuCw-s6zDMw21zun4M3U7rfU77coX9lAMnA';
        $id = $umsma->createRecord($user_resource->user_id, 13, "push", "gcm", generateCode(20), $token, "Y");

        $message_data['user_messaging_setting_map_id'] = $id;
        $message_data['user_id'] = $user_resource->user_id;
        $message_data['test'] = 'false';
        

        $message_title = "Notification";
        $message_body = "failthismessage";
        $push_android_service = new PushAndroidService();
        $push_android_service->prepareService($message_data);
        $response = $push_android_service->send(array( 'title' => $message_title, 'message' => $message_body));
        $this->assertContains('error',$response['raw_result']);
        $this->assertContains('InvalidRegistration',$response['raw_result']);
        $response_array = $push_android_service->results_array;
        $this->assertEquals(0,$response_array['success'],"It should have a success of 0 becuase it was a bad token");
        $this->assertEquals("failure",strtolower($response_array['status']));
    }

    function testPushAndroidService(){

        $user_resource = createNewUser();
        $umsma = new UserMessagingSettingMapAdapter($mimetypes);
        //$token = "APA91bHG31bGdiI7xXiLiLwzPaUX2weuLsCaX8PzHR1B_1vGmnP-isCIuXqu2xVf5ejbfCeBXQA9wd9yi_3Z9C8E_Kc2Tkiuf_cGCKmQz8fKUCu6yJYqvsj5151q4__Nz3cKzDK6KCEn"; //trueforce android device
        $token = "APA91bFEfy6eOqT5Z_r3haJDKNQzbG3ARwXPis7UAUQTP6mWCXdDq5nGbd-hpSdW0Qq8dW-yr2aO8j2hS6RAqvFsS_VZc4h5GdtXHZ88qeZT8s4K4JKy769eQ8V9qKl_GZCErx8CVsmd"; //trueforce android device
        $id = $umsma->createRecord($user_resource->user_id, 13, "push", "gcm", generateCode(20), $token, "Y");

        $message_data['user_messaging_setting_map_id'] = $id;
        $message_data['user_id'] = $user_resource->user_id;
        $message_data['test'] = 'false';

        $message_title = "Notification";
        $message_body = "Hello world";
        $push_android_service = new PushAndroidService();
        $push_android_service->prepareService($message_data);
        $response_service = $push_android_service->send(array( 'title' => $message_title, 'message' => $message_body));
        $this->assertNotNull($response_service);
        $this->assertEquals(200, $response_service['http_code']);
        $this->assertNotNull($response_service['raw_result']);
        $this->assertEquals(0, $response_service['error_no']);
        $this->assertNotContains('error', $response_service['raw_result']);
    }

    function testPushAppleNotificationService(){

        $user_resource = createNewUser();
        $umsma = new UserMessagingSettingMapAdapter($mimetypes);
        $token = "1eb20b0cf1ce9c98e3ff1d933ccd55211e0c379d81b945344b81507ddd954559"; //trueforce ios device
        $id = $umsma->createRecord($user_resource->user_id, 13, "push", "iphone", '3B305486-E432-4968-B002-33BEFC1BBEA3', $token, "Y");

        $message_data['user_messaging_setting_map_id'] = $id;
        $message_data['user_id'] = $user_resource->user_id;
        $message_data['skin_id'] = 13;
        $message_data['test'] = 'false';
        $message_data['force_prod_cert'] = false;// change to true to run in local and send notification without apple sandbox

        $message_title = "Notification";
        $message_body = "Hello world";
        $apple_push_notification_service = new ApplePushNotificationService();
        $apple_push_notification_service->prepareService($message_data);
        $response_service = $apple_push_notification_service->send(array( 'title' => $message_title, 'message' => $message_body));
        $this->assertNotNull($response_service);
        $this->assertEquals(200, $response_service['http_code']);
        $this->assertNotNull($response_service['raw_result']);
        $this->assertEquals(0, $response_service['error_no']);
        $this->assertNotContains('error', $response_service['raw_result']);
    }

    function testSendIosPush()
    {
        //$user_resource = createNewUserWithCCNoCVV();
        //PushMessageController::pushMessageToUser(20000,"Hello World");

        $device_token = '786f79fc49bd1502c7f98e82fbcf058377ce29b23af3a1e98c433cce960e22d4';
        $message = "You just won a new Car!";

        $payload['aps'] = array('alert' => $message,'sound'=>'default');
        $payload = json_encode($payload);
        $streamContext = stream_context_create();
        stream_context_set_option($streamContext, 'ssl', 'local_cert', 'lib/utilities/push_certificates/com.splickit.pitapit.pem');
        for( $z = 0; $z < 10; ++ $z ) {
            // prod test exception is done here.
            if( ($apns = $this->getApns($streamContext)) ) {
                break;
            }
            myerror_log("unable to connect to apple. sleep for 2 seconds and try again");
            sleep(2);
        }

        if ($apns)
            myerror_log("we have a valid connection to apple!");
        else
        {
            myerror_log("UNABLE TO CONNECT TO APPLE FOR PUSH MESSAGE!");
            die;
        }

        $apnsMessage = chr(0) . chr(0) . chr(32) . pack('H*', str_replace(' ', '', $device_token)) . chr(0) . chr(strlen($payload)) . $payload;
        $i = 0;
        while ( ! fwrite($apns, $apnsMessage) && $i < 100)
        {
            myerror_log("trying it again");
            $error_array = error_get_last();
            if ($error_array['message'] == 'fwrite(): SSL: Broken pipe')
                $apns = $this->getApns($streamContext);
            //$string = print_r($error_array,true);
            $string = $error_array['type'].'  '.$error_array['message'];
            myerror_log($string);
            $i++;
        }
        if ($i > 99)
        {
            myerror_log("ERROR! Message not pushed to user is: ".$record['user_id']."  Response: ".$response);
            $fails++;
        }
        else
        {
            myerror_log("iphone message pushed to user_id: ".$record['user_id']);
            $num_users_sent_to++;
        }
    }

    private function getApns($streamContext)
    {
        // always use apple production for testing real pushes
//        if (isProd())
        $apns = stream_socket_client('ssl://' . $this->apnsHost . ':' . $this->apnsPort, $error, $errorString, 2, STREAM_CLIENT_CONNECT, $streamContext);
//        else
//            $apns = stream_socket_client('ssl://'.$this->apnsHost_test.':' . $this->apnsPort, $error, $errorString, 2, STREAM_CLIENT_CONNECT, $streamContext);

        return $apns;
    }


    static function setUpBeforeClass()
    {
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time', 300);
        //mysql_query("BEGIN");
        $skin_resource = createWorldHqSkin();
        $ids['skin_id'] = $skin_resource->skin_id;

        $menu_id = createTestMenuWithOneItem("item_one");
        $ids['menu_id'] = $menu_id;

        $merchant_resource = createNewTestMerchant($menu_id);
        attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
        $ids['merchant_id'] = $merchant_resource->merchant_id;

        $user_resource = createNewUser(array("flags"=>"1C20000001"));
        logTestUserResourceIn($user_resource);
        $ids['user_id'] = $user_resource->user_id;

        MerchantMessageMapAdapter::createMerchantMessageMap($merchant_resource->merchant_id, 'PM', '786f79fc49bd1502c7f98e82fbcf058377ce29b23af3a1e98c433cce960e22d4');

        $order_data = OrderAdapter::staticGetSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','some note');
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $order_resource = placeOrderFromOrderData($order_data,getTomorrowTwelveNoonTimeStampDenver());
        if ($order_resource->order_id < 1000) {
            die("error creating order");
        }
        $ids['order_id'] = $order_resource->order_id;

        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        //mysql_query("ROLLBACK");
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main()
    {
        $suite = new PHPUnit_Framework_TestSuite(__CLASS__);
        PHPUnit_TextUI_TestRunner::run($suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    PushMessageTest::main();
}

?>