<?php

chdir("/Users/radamnyc/code/smaw");

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class RefundCreditServiceTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $user;
	var $ids;

	function setUp()
	{		
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		setContext("com.splickit.order");
		$this->ids = $_SERVER['unit_test_ids'];
		$this->user = logTestUserIn($this->ids['user_id']);
	}
	
	function tearDown() 
	{
		//delete your instance
		unset($this->ids);
		unset($this->user);
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->stamp);
    }
    
    function testChargeMercuryWithoutTwoDecimalFloat()
    {
    	$mercury_cc_functions = new MercuryCreditCardFunctions();
    	$mercury_cc_functions->cardProcessor($amount_to_run_card_for, $billing_user_resource, $neworder_id, $merchant, $card_data);
    	
    	$card_data['cc_number'] = "4111111111111111";
    	$card_data['charge_amt'] = "2.1";
		$card_data['order_id'] = rand(111111,999999);
		$card_data['merchant_id'] = ''.$this->ids['merchant_id'];
		$card_data['tran_code'] = 'Sale';
		$card_data['ref_no'] = '1';
		$mercury_payment = new MercuryPayments();
		$mercury_payment->setFileAdapter(getFileAdapter());
		$return_fields = $mercury_payment->runcard($card_data);
		$this->assertEquals(100, $return_fields['response_code']);

    }

    function testRefundOrderMercuryPay()
    {
    	$merchant_id = $this->ids['merchant_id'];
		$order_adapter = new OrderAdapter($mimetypes);
		$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_id,'pickup','skip hours');
 		$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
    	    	
		$this->assertNotNull($order_resource);
		$this->assertNull($order_resource->error);
		$this->assertTrue($order_resource->order_id > 1000,"Bad order id of: ".$order_resource->order_id);  
    	
		$order_id = $order_resource->order_id;
		$grand_total = $order_resource->grand_total;
		$order_controller = new OrderController($mt, $this->user, $r,5);

    	$refund_results = $order_controller->issueOrderRefund($order_id, 100.00);
    	//should throw an error since the amount is too much
    	$this->assertEquals('failure',$refund_results['result']);
    	$resp_actual = $refund_results['message'];
    	$resp_expected = "Error! The refund amount, 100, cannot be more than the total amount the card was originally run for: $grand_total";
		$this->assertContains($resp_expected,$resp_actual);
    	//$this->assertEquals($resp_expected, $resp_actual);
    	// value of zero means do the whole thing
    	$refund_results = $order_controller->issueOrderRefund($order_id, "0.00");
    	$this->assertEquals("success", $refund_results['result']," should have gotten a success but: ".$refund_results['message']);
    	$this->assertEquals(100,$refund_results['response_code']);
    	
    	// check to see if order was changed to cancelled
    	$new_order_resource = Resource::find($order_adapter,''.$order_id);
    	$this->assertEquals('N', $new_order_resource->status);
    	
    	// check to see if the messages were cancelled
    	$message_data['order_id'] = $order_id;
    	$message_options[TONIC_FIND_BY_METADATA] = $message_data;
    	$message_resources = Resource::findAll(new MerchantMessageHistoryAdapter($mimetypes),'',$message_options);
		$this->assertTrue(sizeof($message_resources) > 0);
		foreach ($message_resources as $message_resource)
		{
			$this->assertEquals('C', $message_resource->locked);
		}    	

    	// now check the balance change table and the order_reversal table
    	
    	$balance_change_resource = Resource::find(new BalanceChangeAdapter($mimetypes),''.$refund_results['balance_change_id']);
    	$this->assertEquals($expected, $actual);
    	$this->assertEquals($balance_change_resource->charge_amt, $grand_total);
    	$this->assertEquals($balance_change_resource->process, 'CCvoid');
    	$this->assertEquals($balance_change_resource->notes, 'Issuing a MercuryCreditCardFunctions VOID from the API: ');
    	
    	$adm_reversal_resource = Resource::find(new AdmOrderReversalAdapter($mimetypes),''.$refund_results['order_reversal_id']);
    	$this->assertNull($adm_reversal_resource);
    }

    function testRefundOrderInspirePay()
    {
    	$merchant_resource = createNewTestMerchant($this->ids['menu_id']);
		$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($merchant_resource->merchant_id,'pickup','skip hours');
 		$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
    	    	
		$this->assertNotNull($order_resource);
		$this->assertNull($order_resource->error);
		$this->assertTrue($order_resource->order_id > 1000,"Bad order id of: ".$order_resource->order_id);  
    	
		$order_id = $order_resource->order_id;
		$grand_total = $order_resource->grand_total;
		$order_controller = new OrderController($mt, $this->user, $r,5);
		$order_controller->updateOrderStatusById($order_id,'E');
    	$refund_results = $order_controller->issueOrderRefund($order_id, 100.00);
    	//should throw an error since the amount is too much
    	$this->assertEquals('failure',$refund_results['result']);
    	$this->assertEquals("Error! The refund amount, 100, cannot be more than the total amount the card was originally run for: ".$grand_total, $refund_results['message']);
    	
    	// value of zero means do the whole thing
    	$refund_results = $order_controller->issueOrderRefund($order_id, "0.00");
    	$this->assertEquals("success", $refund_results['result']);
    	$this->assertEquals(100,$refund_results['response_code']);
    	
    	// check to see if order stayed E
    	$new_order_resource = Resource::find($order_adapter,''.$order_id);
    	$this->assertEquals('E', $new_order_resource->status);
    	
    	// now check the balance change table and the order_reversal table
    	
    	$balance_change_resource = Resource::find(new BalanceChangeAdapter($mimetypes),''.$refund_results['balance_change_id']);
    	$this->assertEquals($expected, $actual);
    	$this->assertEquals($balance_change_resource->charge_amt, $grand_total);
    	$this->assertEquals($balance_change_resource->process, 'CCvoid');
    	$this->assertEquals($balance_change_resource->notes, 'Issuing a InspireCreditCardFunctions VOID from the API: ');
    	
    	$adm_reversal_resource = Resource::find(new AdmOrderReversalAdapter($mimetypes),''.$refund_results['order_reversal_id']);
    	$this->assertNotNull($adm_reversal_resource);
    	$this->assertEquals($adm_reversal_resource->order_id, $order_id);
    	$this->assertEquals($adm_reversal_resource->amount, $grand_total);
    	$this->assertEquals($adm_reversal_resource->credit_type, 'G');
    	$this->assertEquals($adm_reversal_resource->note, 'Issuing a InspireCreditCardFunctions void from the API: ');
    }

/*
    function testRefundBlockOfOrders()
    {
    	//$create_table_sql = "CREATE TABLE `smaw_prod`.`xxrefund_data` (`order_id` INT( 11 ) NOT NULL ,`user_id` INT( 11 ) NOT NULL ,UNIQUE (`order_id`)) ENGINE = MYISAM";
    	$delete_records = "DELETE FROM xxrefund_data WHERE 1=1";
    	$insert_data_sql = "INSERT INTO `xxrefund_data` (`order_id`, `user_id`) VALUES (240805, 20000), (240803, 20000)";
    	$delete_sql = "DELETE FROM adm_order_reversal  WHERE (adm_order_reversal.`order_id` IN (240805,240803)";
    	$aor_adapter = new AdmOrderReversalAdapter($mimetypes);
    	$aor_adapter->_query($delete_records);
    	$aor_adapter->_query($insert_data_sql);
    	$aor_adapter->_query($delete_sql);
    	$user_controller = new UserController($mt, $u,new Request(), 5);
    	$table_name = "xxrefund_data";
    	$refund_data = $user_controller->refundBlockOfOrders($table_name);
    	$this->assertTrue(true);
    }
*/    

//*/    
    static function setUpBeforeClass()
    {
    	$_SERVER['request_time1'] = microtime(true);
    	$_SERVER['NO_MOCKS'] = true;
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	mysqli_query("BEGIN");
    	
    	$user_resource = createNewUser();
    	$user = logTestUserResourceIn($user_resource);
    	
		$zip = rand(11111,19999);
    	$json_encoded_data = "{\"jsonVal\":{\"cc_exp_date\":\"05/2016\",\"cc_number\":\"4111111111111111\",\"cvv\":\"789\",\"zip\":\"$zip\"}}"; 
    	$request = new Request();
    	$request->url = '/app2/phone/users';
    	$request->method = "post";
    	$request->body = $json_encoded_data;
    	$request->mimetype = 'Applicationjson';
    	$request->_parseRequestBody();

    	$user_controller = new UserController($mt, $user, $request,5);
    	$user_controller->updateUser();    	
    	$ids['user_id'] = $user_resource->user_id;
    	
    	$skin_resource = createWorldHqSkin();
    	$ids['skin_id'] = $skin_resource->skin_id;
    	
		//map it to a menu
    	$menu_id = createTestMenuWithNnumberOfItems(5);
    	$ids['menu_id'] = $menu_id;
    	
    	$modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 10);
    	$modifier_group_id = $modifier_group_resource->modifier_group_id;
    	$item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 2);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[1]['item_id'], $modifier_group_id, 4);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[2]['item_id'], $modifier_group_id, 1);

    	$merchant_resource = createNewTestMerchant($menu_id);
    	$merchant_resource->cc_processor = 'M';
    	$merchant_resource->save();
    	
    	attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
    	$ids['merchant_id'] = $merchant_resource->merchant_id;

    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	mysqli_query("ROLLBACK");
    	date_default_timezone_set($_SERVER['starting_tz']);
    }    

    /* main method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    RefundCreditServiceTest::main();
}

?>