<?php

error_reporting(E_ERROR | E_COMPILE_ERROR | E_COMPILE_WARNING | E_PARSE);
$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');
$db_info->database = 'smaw_unittest';
$db_info->username = 'root';
$db_info->password = 'splickit';
if (isset($_SERVER['XDEBUG_CONFIG'])) {
    putenv("SMAW_ENV=unit_test_ide");
    $db_info->hostname = "127.0.0.1";
    $db_info->port = 13306;
} else {
    $db_info->hostname = "db_container";
    $db_info->port = 3306;
}
$_SERVER['DB_INFO'] = $db_info;

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class STSTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }

    function testForceFailFromNotEnoughBalance()
    {
        $user_resource = createNewUser();
        $user = logTestUserResourceIn($user_resource);
        $user_id = $user['user_id'];
        $uuid = $user_resource->uuid;
        $url = "http://127.0.0.1:".$this->api_port."/app2/apiv2/users/$uuid/stored_value/12000/create";
        $card_number = "".rand(1111111111,9999999999);
        $data['card_number'] = $card_number;
        $data['merchant_id'] = $this->ids['merchant_id'];

//        $request = createRequestObject($url,'POST',json_encode($data));
//        $user_controller = new UserController(getM(),$user,$request);
//        $response = makeMockRequest($user_controller);

        $ubpa = new UserBrandPointsMapAdapter(getM());
        $ubpa->createUserLoyaltyAccount($user_id, $this->ids['brand_id'], $user_resource->contact_no);
        // create account
        $ubsvma = new UserSkinStoredValueMapsAdapter(getM());
        $card_resource = $ubsvma->saveCardNumber($card_number,$user_id,getSkinIdForContext(),12000);
        $card_resource->balance = 15.00;
        $card_resource->save();

        $data['user_id'] = $user_resource->user_id;
        $sts_service = new StsPaymentService($data);
        $results = $sts_service->addCashValueToCard($card_number,5.00);
        $this->assertEquals('00',$results['Response_Code']);

        $sts_service = new StsPaymentService($data);
        $sts_service->billing_user_id = $user_resource->user_id;
        $sts_service->processPayment(10.00);

    }

    function testAddStoredValueCardToUserAccount()
    {
        // we still dont have a way to do this through the app.  What merchant do we charge the money to?
        $user_resource = createNewUser();
        $uuid = $user_resource->uuid;
        $url = "http://127.0.0.1:".$this->api_port."/app2/apiv2/users/$uuid/stored_value/12000/create";
        $card_number = "".rand(1111111111,9999999999);
        $data['card_number'] = $card_number;
        $data['merchant_id'] = $this->ids['merchant_id'];


        $data['user_id'] = $user_resource->user_id;
        $sts_service = new StsPaymentService($data);
        $results = $sts_service->addCashValueToCard($card_number,88.00);
        $this->assertEquals('00',$results['Response_Code']);

        $sts_service = new StsPaymentService($data);
        $results2 = $sts_service->getRemoteCardInfo($card_number);
        $this->assertEquals('00',$results2['Response_Code']);
        $this->assertEquals("88.00",$results2['Amount_Balance']);
        $this->assertNotEquals($results2['Transaction_ID'],$results['Transaction_ID'],'It should be 2 different transaction ids');
    }

    function testCheckBalanceWithCurl()
    {

        // first load up a card
        $initial_amount = "100.00";
        $url = "https://smarttransactions.net/gateway_no_lrc.php";
        $string = '<Request><Merchant_Number>111111111111</Merchant_Number><Terminal_ID>275</Terminal_ID><API_Key>h*B7833Mp1tZ</API_Key><Action_Code>02</Action_Code><Trans_Type>N</Trans_Type><POS_Entry_Mode></POS_Entry_Mode><Card_Number>NewAccountRQ</Card_Number><Transaction_Amount>'.$initial_amount.'</Transaction_Amount></Request>';
        $result_array = StsCurl::curlIt($url,$string);
        $result = $result_array['raw_result'];
        $this->assertContains("<Amount_Balance>$initial_amount</Amount_Balance>",$result);

        // get the created card number back
        $start = strpos($result,'<Card_Number>');
        $end = strpos($result,'</Card_Number>');
        $diff = $end - $start;
        $card_number = substr($result,$start+13,$diff-13);

        // call to get balance
        $string = '<Request><Merchant_Number>111111111111</Merchant_Number><Terminal_ID>275</Terminal_ID><API_Key>h*B7833Mp1tZ</API_Key><Action_Code>05</Action_Code><Trans_Type>N</Trans_Type><POS_Entry_Mode>S</POS_Entry_Mode><Card_Number>'.$card_number.'</Card_Number></Request>';
        $result_array2 = StsCurl::curlIt($url,$string);
        $result2 = $result_array2['raw_result'];
        $this->assertContains("<Amount_Balance>$initial_amount</Amount_Balance>",$result2);

        //try to get bad card number
        $string = '<Request><Merchant_Number>111111111111</Merchant_Number><Terminal_ID>275</Terminal_ID><API_Key>h*B7833Mp1tZ</API_Key><Action_Code>05</Action_Code><Trans_Type>N</Trans_Type><POS_Entry_Mode>S</POS_Entry_Mode><Card_Number>345676545677654</Card_Number></Request>';
        $result_array2 = StsCurl::curlIt($url,$string);

        // charge 8.88 agains the new card
        $string = '<Request><Merchant_Number>111111111111</Merchant_Number><Terminal_ID>275</Terminal_ID><API_Key>h*B7833Mp1tZ</API_Key><Action_Code>19</Action_Code><Trans_Type>N</Trans_Type><POS_Entry_Mode>S</POS_Entry_Mode><Card_Number>'.$card_number.'</Card_Number><Transaction_Amount>8.88</Transaction_Amount></Request>';
        $result_array3 = StsCurl::curlIt($url,$string);
        $result3 = $result_array3['raw_result'];
        $this->assertContains("<Response_Code>00</Response_Code>",$result3);
        $this->assertContains("<Approved_Amount>8.88</Approved_Amount>",$result3);
        $raw_result_as_array = parseXMLintoHashmap($result3);

        //void the charge
        $auth_reference = $raw_result_as_array['Auth_Reference'];
        $transaction_id = $raw_result_as_array['Transaction_ID'];
        $string_v = '<Request><Action_Code>11</Action_Code><Auth_Reference>'.$auth_reference.'</Auth_Reference><Merchant_Number>111111111111</Merchant_Number><Terminal_ID>275</Terminal_ID><API_Key>h*B7833Mp1tZ</API_Key><Trans_Type>L</Trans_Type><POS_Entry_Mode>M</POS_Entry_Mode><Transaction_ID>'.$transaction_id.'</Transaction_ID></Request>';
        $result_array4 = StsCurl::curlIt($url,$string_v);
        $result4 = $result_array4['raw_result'];
        $raw_result_as_array = parseXMLintoHashmap($result4);
        $this->assertEquals('00',$raw_result_as_array['Response_Code']);
        $this->assertEquals('100.00',$raw_result_as_array['Amount_Balance']);

        return $card_number;
    }

    /**
     * @depends testCheckBalanceWithCurl
     */
    function testAddCredit($card_number)
    {
        $credit_amount = 8.88;
        $url = "https://smarttransactions.net/gateway_no_lrc.php";
        $string = '<Request><Merchant_Number>111111111111</Merchant_Number><Terminal_ID>275</Terminal_ID><API_Key>h*B7833Mp1tZ</API_Key><Action_Code>08</Action_Code><Trans_Type>N</Trans_Type><POS_Entry_Mode></POS_Entry_Mode><Card_Number>'.$card_number.'</Card_Number><Transaction_Amount>'.$credit_amount.'</Transaction_Amount></Request>';
        $result_array = StsCurl::curlIt($url,$string);
        $result = $result_array['raw_result'];
        $this->assertContains("<Amount_Balance>108.88</Amount_Balance>",$result);

    }

    /**
     * @depends testCheckBalanceWithCurl
     */
    function testEndToEndCharge($card_number)
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user_id = $user_resource->user_id;
        $user = logTestUserResourceIn($user_resource);
        $ussvma = new UserSkinStoredValueMapsAdapter(getM());
        $ussvma->saveCardNumber($card_number,$user_id,getSkinIdForContext(),12000);

        $merchant_id = $this->ids['merchant_id'];

        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);

        $checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
        $this->assertNull($checkout_resource->error);
        $sts_payment_type = $checkout_resource->accepted_payment_types[1];
        $checkout_resource->accepted_payment_types = [$sts_payment_type];
        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);

        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order_resource->order_id],'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
        $stored_value_row = $bcr_hash['StoredValuePayment'];
        $this->assertEquals('StsPaymentService',$stored_value_row['cc_processor']);
        return $order_resource->order_id;
    }

    /**
     * @depends testEndToEndCharge
     */
    function testPartailRefundByAddingCredit($order_id)
    {

    }

    function testVoidPreviousCharge()
    {
       /*  <Request> <Action_Code>11</Action_Code> <Auth_Reference>0005</Auth_Reference><Merchant_Number>111111111111</Merchant_Number><Terminal_ID>002</Terminal_ID>
            <API_Key>j*gkNkUv55Csw</API_Key> <Trans_Type>L</Trans_Type><POS_Entry_Mode>M</POS_Entry_Mode><Transaction_ID>108</Transaction_ID></Request> */
    }




    function curlIt($string)
    {
        $url = "https://smarttransactions.net/gateway_no_lrc.php";
        unset($this->info);
        $method = strtoupper('POST');
        $curl = curl_init($url);

        curl_setopt($curl, CURLOPT_POST, 1);
        curl_setopt($curl, CURLOPT_POSTFIELDS,$string);
        $headers[] = 'Content-Type: application/XML';
        $headers[] = 'Content-Length: ' . strlen($string);
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        logCurl($url,$method,null,$headers,$string);
        $result = curl_exec($curl);
        $this->info = curl_getinfo($curl);
        curl_close($curl);
        myerror_log("RESPONSE: $result");
        return $result;
    }

	static function setUpBeforeClass()
    {
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time',300);

        $skin_resource = getOrCreateSkinAndBrandIfNecessaryWithLoyalty("stsskin","stsbrand",null, null);
        $ids['skin_id'] = $skin_resource->skin_id;
        setContext('com.splickit.stsskin');

    	$simple_menu_id = createTestMenuWithOneItem("item_one");
    	$ids['menu_id'] = $simple_menu_id;
    	
    	$merchant_resource = createNewTestMerchant($simple_menu_id);
    	$merchant_resource->save();
    	//attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
    	$ids['merchant_id'] = $merchant_resource->merchant_id;

    	$msa = new SkinStsInfoMapsAdapter(getM());
    	$data['skin_id'] = getSkinIdForContext();
    	$data['merchant_number'] = '111111111111';
    	$data['terminal_id'] = '275';
    	$data['api_key'] = 'h*B7833Mp1tZ';
    	$ids['skin_sts_info_map_resource'] = Resource::findOrCreateIfNotExistsByData($msa,$data);

    	$merchant_payment_type_map_resource = MerchantPaymentTypeMapsAdapter::createMerchantPaymentTypeMap($merchant_resource->merchant_id, 12000, null);
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        date_default_timezone_set($_SERVER['starting_tz']);
    }


    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    STSTest::main();
}
?>