<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';


class SmsSenderTestServices extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;

    function setUp()
    {
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__ . '-' . $_SERVER['STAMP'];
        $this->ids = $_SERVER['unit_test_ids'];
        setProperty('test_sms_messages_on', 'true');

    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }

    function testSendWithCdyne()
    {
    	$result = SmsSender2::send_with_cdyne(getProperty("test_sms_number"), "cdyne message");
		$this->assertNotNull($result['response_id']);
		$this->assertEquals("NoError",$result['response_text']);
		$this->assertEquals(9,count($result),"Should have been 9 fields returned");
    }

    function testSendWithTwilio()
    {
    	$result = SmsSender2::send_with_twilio(getProperty("test_sms_number"), "twilio messages");
		$this->assertNotNull($result['response_id']);
		$this->assertEquals("queued",$result['response_text']);
		$this->assertEquals(2,count($result),"Should have been 2 fields returned");
    }

	function testSendWithTwilioFail()
	{
		$result = SmsSender2::send_with_twilio("1234567890", "twilio messages");
		$this->assertFalse(false);
	}


    static function setUpBeforeClass()
    {
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time', 300);
        mysqli_query("BEGIN");
        $user_resource = createNewUser(array("flags" => "1C20000001"));
        $ids['user_id'] = $user_resource->user_id;
        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        mysqli_query("ROLLBACK");
        date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main()
    {
        $suite = new PHPUnit_Framework_TestSuite(__CLASS__);
        PHPUnit_TextUI_TestRunner::run($suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    SmsSenderTestServices::main();
}

?>