<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class SplickitVaultServiceTest extends PHPUnit_Framework_TestCase
{
	var $stamp;

	function setUp()
	{
		setContext("com.splickit.order");
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
    }
    
    function testSaveCardInVault()
    {
    	// get new user
    	$user_resource = createNewUser();
    	$user = logTestUserResourceIn($user_resource);
    	
		$zip = rand(11111,19999);
    	$json_encoded_data = "{\"jsonVal\":{\"cc_exp_date\":\"03/2016\",\"cc_number\":\"4111111111111111\",\"cvv\":\"123\",\"zip\":\"$zip\"}}"; 
    	$request = new Request();
    	$request->url = '/app2/phone/users';
    	$request->method = "post";
    	$request->body = $json_encoded_data;
    	$request->mimetype = 'Applicationjson';
    	$request->_parseRequestBody();

    	$user_controller = new UserController($mt, $user, $request,5);
    	// save cc
    	$user_controller->updateUser();    	
    	
    	// pull new cc info and verify
    	$splickit_vault = new SplickitVault();
    	$card_data = $splickit_vault->getCardInfoOnly($user_resource->uuid);
    	$this->assertEquals("4111111111111111",$card_data['number']);
    	$this->assertEquals($zip,$card_data['zip']);
    	
    	//check for backward compatability
    	$this->assertEquals("4111111111111111",$card_data['cc_number']);
    	$this->assertEquals($zip,$card_data['postal_code']);
    	$this->assertEquals("0316",$card_data['cc_exp']);    	
    	return $user['user_id'];
    }

    /**
     * @depends testSaveCardInVault
     */
     function testFailoverToUpdateOnExistingUserInVault($user_id)
    {
    	$user_resource = Resource::find(new UserAdapter($mimetypes),"$user_id");
    	$this->assertEquals('1C21000001', $user_resource->flags);
    	$user = logTestUserResourceIn($user_resource);
    	$zip = rand(11111,19999);
    	//$json_encoded_data = "{\"jsonVal\":{\"cc_exp_date\":\"03/2016\",\"cc_number\":\"4111114567111111\",\"cvv\":\"123\",\"zip\":\"$zip\"}}"; 
    	$user_resource->set("cc_exp_date","03/2016");
    	$user_resource->set("cc_number","4111111111111111");
    	$user_resource->set("cvv","123");
    	$user_resource->set("zip","$zip");
    	$user_resource->flags = '1000000001';
    	$cc_functions = new CreditCardFunctions();
    	$result = $cc_functions->cc_save($user_resource);
    	$status = $cc_functions->getStatus();
    	$this->assertTrue($result);
    	$this->assertEquals('200 OK', $status);
    	
    	$splickit_vault = new SplickitVault();
    	$card_dataa = $splickit_vault->getCardInfoOnly($user_resource->uuid);
    	
    	$card_data = $cc_functions->pullCreditCardDataFromUserId($user_id);
    	$this->assertEquals($zip, $card_data['zip']);
    	$this->assertEquals($zip, $card_data['postal_code']);
    	return $card_data;
     }

    /**
     * @depends testFailoverToUpdateOnExistingUserInVault
     */
    
    function testRunCCAtInspirePayWithNoVault($card_data)
    {
		$inspire_pay = New inspirePay();
		$amount = rand(100,1000)/100;
		$order_id = rand(10000,1000000);
		$inspire_pay->runCardNoVault($card_data, $amount, $order_id);
		$return_fields = $inspire_pay->getReturnData();
		$this->assertEquals(100, $return_fields['response_code']);    	
    }
    
    function testSaveGoodCCnumber()
    {
    	$user_resource = createNewUser();
    	$user = logTestUserResourceIn($user_resource);
    	
		$zip = rand(11111,19999);
    	$json_encoded_data = "{\"jsonVal\":{\"cc_exp_date\":\"05/2016\",\"cc_number\":\"4111111111111111\",\"cvv\":\"789\",\"zip\":\"$zip\"}}"; 
    	$request = new Request();
    	$request->url = '/app2/phone/users';
    	$request->method = "post";
    	$request->body = $json_encoded_data;
    	$request->mimetype = 'Applicationjson';
    	$request->_parseRequestBody();

    	$user_controller = new UserController($mt, $user, $request,5);
    	// save cc
    	$user_controller->updateUser();    	
    	
    	// pull new cc info and verify
    	$splickit_vault = new SplickitVault();
    	$card_data = $splickit_vault->getCardInfoOnly($user_resource->uuid);
    	$this->assertEquals("4111111111111111",$card_data['number']);
    	$this->assertEquals($zip,$card_data['zip']);
    	
    	//check for backward compatability
    	$this->assertEquals("4111111111111111",$card_data['cc_number']);
    	$this->assertEquals($zip,$card_data['postal_code']);
    	$this->assertEquals("0516",$card_data['cc_exp']);    	
    	
    }
   
	function testSaveCCbadNumberDoNotFailOverToIPVault()
    {
    	$user_resource = createNewUser();
		$user = $user_resource->getDataFieldsReally();
		$user_id = $user['user_id'];
		$_SERVER['AUTHENTICATED_USER'] = $user;    	
    	
		$zip = rand(11111,19999);
    	$json_encoded_data = "{\"jsonVal\":{\"cc_exp_date\":\"03/2018\",\"cc_number\":\"4111114567111111\",\"cvv\":\"123\",\"zip\":\"$zip\"}}";
    	$request = new Request();
    	$request->url = '/app2/phone/users';
    	$request->method = "post";
    	$request->body = $json_encoded_data;
    	$request->mimetype = 'Applicationjson';
    	$request->_parseRequestBody();

    	$user_controller = new UserController($mt, $user, $request,5);
    	// save cc
    	$resource = $user_controller->updateUser();    	
    	$this->assertEquals("Error saving credit card info: CC number is not valid", $resource->error);
    	
    	$user_resource2 = UserController::getUserResourceFromUserId($user_id);
    	$this->assertEquals('1000000001', $user_resource2->flags);
    }

    function testSaveCCbadExpDateDoNotFailOverToIPVault()
    {
    	$user_resource = createNewUser();
		$user = $user_resource->getDataFieldsReally();
		$user_id = $user['user_id'];
		$_SERVER['AUTHENTICATED_USER'] = $user;    	
    	
		$zip = rand(11111,19999);
    	$json_encoded_data = "{\"jsonVal\":{\"cc_exp_date\":\"03/2012\",\"cc_number\":\"4111111111111111\",\"cvv\":\"123\",\"zip\":\"$zip\"}}"; 
    	$request = new Request();
    	$request->url = '/app2/phone/users';
    	$request->method = "post";
    	$request->body = $json_encoded_data;
    	$request->mimetype = 'Applicationjson';
    	$request->_parseRequestBody();

    	$user_controller = new UserController($mt, $user, $request,5);
    	// save cc
    	$resource = $user_controller->updateUser();
    	$this->assertEquals("Credit Card save error, expired expiration date: 0312", $resource->error);
    	
    	$user_resource2 = UserController::getUserResourceFromUserId($user_id);
    	$this->assertEquals('1000000001', $user_resource2->flags);  	
    }

    function testPullCCInfoFromInspirePayAndResaveInNewVault()
    {
    	$user_resource = createNewUser();
    	$user = $user_resource->getDataFieldsReally();
		$_SERVER['AUTHENTICATED_USER'] = $user;    	

    	$inspireObj = new InspirePay();
    	$zip = rand(11111,19999);
    	$result_boolean = $inspireObj->createCustomerVaultRecord($user['user_id'], "4111111111111111", "123", "06/2018", $zip, "first", "last");
		//$this->assertTrue($result_boolean,"should have created an new inspire pay vault record");
    	$inspire_pay_object = new InspirePay();
		$inspire_pay_object->getCustomerVaultRecord($user['user_id']);
    	$card_data = $inspire_pay_object->getReturnData();
		$inspire_pay_object->getCustomerVaultRecordCC($user['user_id']);
		$return_data2 = $inspire_pay_object->getReturnData();
		// acces wil be denied here so we'll fake it
		$return_data2['cc_number'] = "4111111111111111";
		$card_data['cc_number'] = $return_data2['cc_number'];
		$splickit_vault = new SplickitVault();
		$splickit_vault->saveCardFromInspirePayReturnAndUserResource($card_data, $user_resource);
		
		// now get the record and see if it all lines up
    	$splickit_vault2 = new SplickitVault();
    	$card_data = $splickit_vault2->getCardInfoOnly($user_resource->uuid);
    	$this->assertEquals("4111111111111111",$card_data['number']);
    	$this->assertEquals($zip,$card_data['zip']);
    	
    	//check for backward compatability
    	$this->assertEquals("4111111111111111",$card_data['cc_number']);
    	$this->assertEquals($zip,$card_data['postal_code']);
    	$this->assertEquals("0618",$card_data['cc_exp']);

    }
        
    function testWholeNewUserThingFromDispatch()
    {
    	mysqli_query("ROLLBACK");
    	// get new user
    	$user_resource = createNewUser();
		$user = $user_resource->getDataFieldsReally();
		$this->assertEquals('100', substr($user['flags'], 0,3));
		$zip = rand(11111,19999);
    	$json_encoded_data = "{\"jsonVal\":{\"cc_exp_date\":\"03/2016\",\"cc_number\":\"4111111111111111\",\"cvv\":\"123\",\"zip\":\"$zip\"}}"; 
		$url = "http://localhost/app2/phone/users?log_level=5";
    	$curl = curl_init($url);
    	curl_setopt($curl, CURLOPT_USERPWD, $user['email'].":welcome");
    	curl_setopt($curl, CURLOPT_HTTPHEADER, array("X_SPLICKIT_CLIENT_ID:com.splickit.order","X_SPLICKIT_CLIENT_DEVICE:unit_testing","X_SPLICKIT_CLIENT:SplickitVaultTest",'Content-Type: application/json','Content-Length: ' . strlen($json_encoded_data))); 
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($curl, CURLOPT_POST, 1);
		curl_setopt($curl, CURLOPT_POSTFIELDS,$json_encoded_data);
		$result = curl_exec($curl);
		myerror_log("we have returned from calling dispatch");
		curl_close($curl);
		$result_array = json_decode($result,true);
		
		$user_resource_after = Resource::find(new UserAdapter($mimetypes),''.$user['user_id']);
		$this->assertEquals('1C2', substr($user_resource_after->flags, 0,3));
		
    	$splickit_vault2 = new SplickitVault();
    	$card_data = $splickit_vault2->getCardInfoOnly($user_resource_after->uuid);
    	$this->assertEquals("4111111111111111",$card_data['number']);
    	$this->assertEquals($zip,$card_data['zip']);	
    }

    static function setUpBeforeClass()
    {
    	$_SERVER['request_time1'] = microtime(true);
    	$_SERVER['NO_MOCKS'] = true;
    	$_SERVER['HTTP_NO_CC_CALL'] = 'false';
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	mysqli_query("BEGIN");

    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
    static function tearDownAfterClass()
    {
    	mysqli_query("ROLLBACK");
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    SplickitVaultServiceTest::main();
}

?>