<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class SplickitVaultTestNew extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		setContext('com.splickit.order');
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
		
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }
    
    function addDummyCCDataToResource(&$resource)
    {
    	addDummyCCDataToResource($resource);	
    }

    function testSplickitVaultSaveResponse()
    {
    	$user_resource = createNewUser();
    	$user = logTestUserResourceIn($user_resource);
    	$this->addDummyCCDataToResource($user_resource);
    	$splickit_vault = new SplickitVault();
    	$this->assertTrue($splickit_vault->save($user_resource));
    	
    	$curl_response = $splickit_vault->curl_response_array;
    	$this->assertEquals("200", $curl_response['http_code']);
    	$curl_info = $curl_response['curl_info'];
    	$this->assertNotNull($curl_info['time_of_request_in_seconds']);
    	return $user;
    }
    
	/**
     * @depends testSplickitVaultSaveResponse
     */
    function testSplickitVaultPullCardInfo($user)
    {
    	$splickit_vault = new SplickitVault();
    	$card_data = $splickit_vault->getCardInfoOnly($user['uuid']);
    	$this->assertEquals($user['uuid'], $card_data['identifier']);
    	$this->assertEquals("45678", $card_data['zip']);
    	$this->assertEquals("2029", $card_data['year']);
    }
    
    /**
     * @depends testSplickitVaultSaveResponse
     */
    function testSplickitVaultPullCardData($user)
    {
    	$splickit_vault = new SplickitVault();
    	$card_data = $splickit_vault->getCardInfoOnly($user['uuid']);
    	$this->assertEquals("4111111111111111", $card_data['number']);
    	$this->assertEquals($user['uuid'], $card_data['identifier']);
    	$this->assertEquals("45678", $card_data['zip']);
    	$this->assertEquals("2029", $card_data['year']);
    }
    
    function testSaveGoodCCnumber()
    {
    	$user_resource = createNewUser();
    	$user = logTestUserResourceIn($user_resource);
    	
		$zip = rand(11111,19999);
    	$json_encoded_data = "{\"jsonVal\":{\"cc_exp_date\":\"05/2016\",\"cc_number\":\"4111111111111111\",\"cvv\":\"789\",\"zip\":\"$zip\"}}"; 
    	$request = new Request();
    	$request->url = '/app2/phone/users';
    	$request->method = "post";
    	$request->body = $json_encoded_data;
    	$request->mimetype = 'Applicationjson';
    	$request->_parseRequestBody();

    	$user_controller = new UserController($mt, $user, $request,5);
    	// save cc
    	$user_controller->updateUser();  

    	// validate that flags were updated
    	$user2 = logTestUserIn($user['user_id']);
    	$this->assertEquals('1C21000001', $user2['flags']);
    	
    	// pull new cc info and verify
    	$splickit_vault = new SplickitVault();
    	$card_data = $splickit_vault->getCardInfoOnly($user_resource->uuid);
    	$this->assertEquals("4111111111111111",$card_data['number']);
    	$this->assertEquals($zip,$card_data['zip']);
    	$this->assertTrue($card_data['vaulted'],"card data should have a vaulted vaulted value of true");
    	
    	//check for backward compatability
    	$this->assertEquals("4111111111111111",$card_data['cc_number']);
    	$this->assertEquals($zip,$card_data['postal_code']);
    	$this->assertEquals("0516",$card_data['cc_exp']);    	
    	return $user;
    }
    
    /**
     * @depends testSaveGoodCCnumber
     */
    function testPlaceOrderWithRunningCC($user)
    {
    	logTestUserIn($user['user_id']);
    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'], 'Pickup', 'skip hours');
    	$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
    	$this->assertNull($order_resource->error);
    }
    
    function testStoreTestersAbilityToPlaceAnOrder()
    {
    	logTestUserIn(101);
    	$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'], 'Pickup', 'skip hours');
    	$order_resource = placeOrderFromOrderData($order_data, $time_stamp);
    	$this->assertNull($order_resource->error);
    }
    
	function testSaveCCbadNumberDoNotFailOverToIPVault()
    {
    	$user_resource = createNewUser();
    	$user = logTestUserResourceIn($user_resource);
    	
		$zip = rand(11111,19999);
    	$json_encoded_data = "{\"jsonVal\":{\"cc_exp_date\":\"03/2016\",\"cc_number\":\"4111114567111111\",\"cvv\":\"123\",\"zip\":\"$zip\"}}"; 
    	$request = new Request();
    	$request->url = '/app2/phone/users';
    	$request->method = "post";
    	$request->body = $json_encoded_data;
    	$request->mimetype = 'Applicationjson';
    	$request->_parseRequestBody();

    	$user_controller = new UserController($mt, $user, $request,5);
    	// save cc
    	$resource = $user_controller->updateUser();    	
    	$this->assertEquals("Error saving credit card info: CC number is not valid", $resource->error);
    	
    	$user_resource2 = UserController::getUserResourceFromUserId($user['user_id']);
    	$this->assertEquals('1000000001', $user_resource2->flags);
    }

    function testSaveCCbadExpDateDoNotFailOverToIPVault()
    {
    	$user_resource = createNewUser();
		$user = $user_resource->getDataFieldsReally();
		$user_id = $user['user_id'];
		$_SERVER['AUTHENTICATED_USER'] = $user;    	
    	
		$zip = rand(11111,19999);
    	$json_encoded_data = "{\"jsonVal\":{\"cc_exp_date\":\"03/2012\",\"cc_number\":\"4111111111111111\",\"cvv\":\"123\",\"zip\":\"$zip\"}}"; 
    	$request = new Request();
    	$request->url = '/app2/phone/users';
    	$request->method = "post";
    	$request->body = $json_encoded_data;
    	$request->mimetype = 'Applicationjson';
    	$request->_parseRequestBody();

    	$user_controller = new UserController($mt, $user, $request,5);
    	// save cc
    	$resource = $user_controller->updateUser();
    	$this->assertEquals("Credit Card save error, expired expiration date: 0312", $resource->error);
    	
    	$user_resource2 = UserController::getUserResourceFromUserId($user_id);
    	$this->assertEquals('1000000001', $user_resource2->flags);  	
    }

    function testSplickitVaultUpdateFailRollToInsert()
    {
    	$user_resource = createNewUserWithCC();
    	$user = logTestUserResourceIn($user_resource);
    	$this->addDummyCCDataToResource($user_resource);
    	$splickit_vault = new SplickitVault();
    	$result = $splickit_vault->save($user_resource,true);
    	$this->assertTrue($result);
    	$this->assertEquals('insert', $splickit_vault->vault_save_process);
    }
    
    function testPullDuringOrderWithNoRecord()
    {
    	$user_resource = createNewUserWithCC();
    	$user = logTestUserResourceIn($user_resource);
		$order_adapter = new OrderAdapter($mimetypes);
    	$order_data = $order_adapter->getSimpleOrderArrayByMerchantId($this->ids['merchant_id'], 'pickup', 'skip hours');
    	$order_result = placeOrderFromOrderData($order_data, $time_stamp);
    	$this->assertNotNull($order_result->error);
    	$this->assertEquals("We're sorry, your credit card info has expired, please re-enter your information.  Thanks!", $order_result->error);
		    	
    }
    
    function testWholeNewUserThingFromDispatch()
    {
    	mysqli_query("ROLLBACK");
    	// get new user
    	$user_resource = createNewUser();
    	$user = logTestUserResourceIn($user_resource);
		$this->assertEquals('100', substr($user['flags'], 0,3));
		$zip = rand(11111,19999);
    	$json_encoded_data = "{\"jsonVal\":{\"cc_exp_date\":\"03/2016\",\"cc_number\":\"4111111111111111\",\"cvv\":\"123\",\"zip\":\"$zip\"}}"; 
		$url = "http://localhost/app2/phone/users?log_level=5";
    	$curl = curl_init($url);
    	curl_setopt($curl, CURLOPT_USERPWD, $user['email'].":welcome");
    	curl_setopt($curl, CURLOPT_HTTPHEADER, array("X_SPLICKIT_CLIENT_ID:com.splickit.order","X_SPLICKIT_CLIENT_DEVICE:unit_testing","X_SPLICKIT_CLIENT:SplickitVaultTest",'Content-Type: application/json','Content-Length: ' . strlen($json_encoded_data))); 
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($curl, CURLOPT_POST, 1);
		curl_setopt($curl, CURLOPT_POSTFIELDS,$json_encoded_data);
		$result = curl_exec($curl);
		myerror_log("we have returned from calling dispatch");
		curl_close($curl);
		$result_array = json_decode($result,true);
		
		$user_resource_after = Resource::find(new UserAdapter($mimetypes),''.$user['user_id']);
		$this->assertEquals('1C2', substr($user_resource_after->flags, 0,3));
		
    	$splickit_vault2 = new SplickitVault();
    	$card_data = $splickit_vault2->getCardInfoOnly($user_resource_after->uuid);
    	$this->assertEquals("4111111111111111",$card_data['number']);
    	$this->assertEquals($zip,$card_data['zip']);	
    }
        
    static function setUpBeforeClass()
    {
    	ini_set('max_execution_time',0);
    	$_SERVER['request_time1'] = microtime(true);
    	$_SERVER['NO_MOCKS'] = true;
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	mysqli_query("BEGIN");
    	
    	$skin_resource = createWorldHqSkin();
    	$ids['skin_id'] = $skin_resource->skin_id;
    	
		//map it to a menu
    	$menu_id = createTestMenuWithNnumberOfItems(1);
    	$ids['menu_id'] = $menu_id;
    	
    	$merchant_resource = createNewTestMerchant($menu_id);
    	attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
    	$ids['merchant_id'] = $merchant_resource->merchant_id;
      	
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	mysqli_query("ROLLBACK");
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    SplickitVaultTestNew::main();
}

?>