<?php
error_reporting(E_ERROR | E_COMPILE_ERROR | E_COMPILE_WARNING | E_PARSE);
$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');
$db_info->database = 'smaw_unittest';
$db_info->username = 'root';
$db_info->password = 'splickit';
if (isset($_SERVER['XDEBUG_CONFIG'])) {
    putenv("SMAW_ENV=unit_test_ide");
    $db_info->hostname = "127.0.0.1";
    $db_info->port = 13306;
} else {
    putenv("SMAW_ENV=unit_test");
    $db_info->hostname = "db_container";
    $db_info->port = 3306;
}
$_SERVER['DB_INFO'] = $db_info;

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class ThanxServiceTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;
    var $api_port = "80";

	function setUp()
	{
        $_SERVER['HTTP_NO_CC_CALL'] = 'true';
        $_SERVER['DO_NOT_RUN_CC'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
        if (isset($_SERVER['XDEBUG_CONFIG'])) {
            $this->api_port = "10080";
        }
        setContext('com.splickit.rrbbq');
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }

//        INSERT INTO Configuration_By_Environment_Variables (`name`,`value`,`environment`) VALUES ('thanx_url','https://api.thanxsandbox.com','default');
//        INSERT INTO Configuration_By_Environment_Variables (`name`,`value`,`environment`) VALUES ('thanx_url','https://api.thanx.com','production');
//        INSERT INTO Third_Party_Providers (id,third_party_provider_type_id,name,description) VALUES (3003,3000,'Thanx','Thanx Loyalty');
//        INSERT INTO Third_Party_Provider_Parameters (third_party_provider_id,name,default_context,required) VALUES (3003,'client_id','skin',1),(3003,'secret','skin',1);

//    function testCreateUserForStagingTesting()
//    {
//        $user_data = [];
//        $user_data['first_name'] = 'adam';
//        $user_data['last_name'] = 'rosenthal';
//        $user_data['email'] = 'adam@order140.com';
//
//        $thanx_loyalty_service = new ThanxLoyaltyService();
//        $response = $thanx_loyalty_service->createUser($user_data);
//        $this->assertEquals(200,$response['http_code']);
//    }

    function testCreateUser()
    {
        $user_data = [];
        $randomizer_string = createCode(3);
        $user_data['first_name'] = 'first';
        $user_data['last_name'] = 'last';
        $user_data['email'] = getCreateTestUserEmail($randomizer_string);
        $user_data['phone'] = '+1212799'.rand(1111,9999);
        $birthday_array = ['year'=>1975,'month'=>8,'day'=>14];
        $user_data['birth_date'] = $birthday_array;

        $thanx_loyalty_service = new ThanxLoyaltyService();
        $response = $thanx_loyalty_service->createUser($user_data);
        $this->assertEquals(201,$response['http_code']);
        $response['user_data'] = $user_data;
        return $response;
    }

    /**
     * @depends testCreateUser
     */
    function testGetUserFromId($data)
    {
        $id = $data['user']['id'];
        $access_token = $data['authorization']['access_token'];
        $thanx_loyalty_service = new ThanxLoyaltyService();
        $response = $thanx_loyalty_service->getRemoteUserInfo($id,$access_token);
        $this->assertEquals('first', $response['user']['first_name']);

        $up_data = [];
        $up_data['thanx_id'] = $id;
        $up_data['first_name'] = 'last';
        $up_data['last_name'] = 'first';
        $birthday_array = ['year'=>1975,'month'=>9,'day'=>14];
        $up_data['birth_date'] = $birthday_array;
        // TODO: Put in my update test here.
        $update_response = $thanx_loyalty_service->updateUser($up_data, $access_token);

        $post_update_response = $thanx_loyalty_service->getRemoteUserInfo($id,$access_token);

        $this->assertEquals('last', $post_update_response['user']['first_name']);
    }

    /**
     * @depends testCreateUser
     */
    function testAuthenticateWithValidToken($data)
    {
        $remote_user_info = $data['user'];
        $authorization_info = $data['authorization'];

        // first get code
        $thanx_loyalty_service = new ThanxLoyaltyService();
        $user = ['email'=>$remote_user_info['email']];
        $response = $thanx_loyalty_service->initiateAuthenticationProcess($user);
        $this->assertEquals(200,$response['http_code']);
    }

    function testCreateUserFromSimulatedWeb()
    {
        $url = "http://127.0.0.1:" . $this->api_port . "/app2/apiv2/users?log_level=5";
        $new_user_data = createNewUserDataFields();
        $new_user_data['contact_no'] = '+1212787'.rand(1111,9999);
        $user = logTestUserIn(1);
//        $request = createRequestObject("/apiv2/users","POST",json_encode($new_user_data),'application/json');
//        $user_controller = new UserController(getM(), $user, $request, 5);
//        $response = makeMockRequest($user_controller);

        $response = $this->makeRequestWithDefaultHeaders($url, "admin:welcome","POST",$new_user_data);
        $this->assertEquals('200', $this->info['http_code']);

        $new_user_response = json_decode($response,true);
        $this->assertNull($new_user_response['error'],"should NOT have gotten an error");
        $user = $new_user_response['data'];
        $this->assertNotNull($user['user_id']);
        $expires_at = $user['splickit_authentication_token_expires_at'];
        $this->assertTrue($expires_at > (time()+(179*24*3600)));

        $user_resource = UserAdapter::getUserResourceFromId($user['uuid']);
        $loyalty_controller = LoyaltyControllerFactory::getLoyaltyControllerForContext($user_resource->getDataFieldsReally());
        $loyalty_number = $loyalty_controller->loyalty_number;
        $this->assertContains('id=',$loyalty_number);
        $this->assertContains(':access=',$loyalty_number);
        $this->assertContains(':refresh=',$loyalty_number);
        return $user_resource;
    }

    /**
     * @depends testCreateUserFromSimulatedWeb
     */
    function testSendOrderInfo($user_resource)
    {
        $user_resource->flags = '1C20000001';
        $user_resource->save();
        $user = logTestUserResourceIn($user_resource);
        $menu_id = createTestMenuWithNnumberOfItems(1);
        $merchant_resource = createNewTestMerchant($menu_id);
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_resource->merchant_id);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data);
        $this->assertNull($checkout_resource->error);
        $order_resource = placeOrderFromCheckoutResource($checkout_resource);
        $this->assertNull($order_resource->error);

        // now refund the order
        $thanks_loyalty_controller = new ThanxLoyaltyController(getM(),$user,null);
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id);
        $response = $thanks_loyalty_controller->sendThanxLoyaltyOrderEvent($complete_order,'refunded');
        $this->assertEquals(201,$response['http_code'],'It should have a successfull refund');
    }

    function testAquireAccessToken()
    {
        $user_resource = UserAdapter::doesUserExist('arosenthal@order140.com');
        $user_id = $user_resource->user_id;
        $brand_id = getBrandIdFromCurrentContext();
        $sql = 'INSERT INTO User_Brand_Points_Map (`user_id`, `brand_id`, `loyalty_number`, `points`, `created`, `modified`) VALUES ('.$user_id.', '.$brand_id.', "id=xk3zqr9cddnpw64:access=6ca7f0399cbfc81bb20615574d36c18eaf8dc916375e75bd06eef961d315a608:refresh=f66ff45fb635fe7466619dcdb95c9d7536cb5a570cbb7c9d30b2aaf8f2f5d7ba", 0, NOW(), NOW())';
        $ubpma = new UserBrandPointsMapAdapter(getM());
        $ubpma->_query($sql);

        $merchant_id = $this->ids['merchant_id'];
        $thanx_info = [];
//        // first get code
//        $thanx_loyalty_service = new ThanxLoyaltyService();
//        $user = ['email'=>'arosenthal@order140.com'];
//        $response = $thanx_loyalty_service->initiateAuthenticationProcess($user);
//        $this->assertEquals(200,$response['http_code']);

        $code = '16e5546752038d23c275192708e4cad743918c880d2ba759e41c094ed8a4304d';
//        $thanx_loyalty_service = new ThanxLoyaltyService();
//        $response = $thanx_loyalty_service->aquireAccessToken($code);
//        $response_as_array = json_decode($response['raw_result'],true);
//        $this->assertEquals(200,$response['http_code']);

//        $url = "http://127.0.0.1:".$this->api_port."/app2/apiv2/usersession";
//        $response = $this->makeRequestWithDefaultHeaders($url, "thanx_authentication_token:$code");
//        $this->assertEquals(200, $this->info['http_code']);
//        $response_as_array = json_decode($response,true);
//        $splickit_authentication_token = $response_as_array['data']['splickit_authentication_token'];
//        $user_id = $response_as_array['data']['brand_loyalty']['user_id'];
//        $loyalty_number = $response_as_array['data']['brand_loyalty']['loyalty_number'];
//        $l = explode(':',$loyalty_number);
//
//        foreach ($l as $info) {
//            $i = explode("=",$info);
//            $thanx_info[$i[0]] = $i[1];
//        }

        $thanks_access_token = $thanx_info['access'];
        if ($thanks_access_token == null) {
            $user_id = $user_resource->user_id;
            $thanks_access_token = '6ca7f0399cbfc81bb20615574d36c18eaf8dc916375e75bd06eef961d315a608';
        }

        $thanks_loyalty_service = new ThanxLoyaltyService();
        $response = $thanks_loyalty_service->getRemoteAccount($thanks_access_token);
        //$response = $thanks_loyalty_service->getRewards($thanks_access_token);
        $this->assertEquals(200,$response['http_code']);
        $raw_rewards = $response['rewards'];
        $this->assertTrue(sizeof($raw_rewards) > 0, 'It should have rewards');
        $thanks_loyalty_controller = new ThanxLoyaltyController(getM(),$user_resource->getDataFieldsReally(),null,5);
        $rewards = $thanks_loyalty_controller->processRewardsListForCheckout($raw_rewards);


        $user = logTestUserIn($user_id);


//        $thanks_loyalty_controller = new ThanxLoyaltyController(getM(),$user,null);
//        $complete_order = CompleteOrder::staticGetCompleteOrder(280497);
//        $response = $thanks_loyalty_controller->sendThanxLoyaltyOrderEvent($complete_order,'refunded');
//        $complete_order = CompleteOrder::staticGetCompleteOrder(280498);
//        $response = $thanks_loyalty_controller->sendThanxLoyaltyOrderEvent($complete_order,'refunded');
//        $complete_order = CompleteOrder::staticGetCompleteOrder(280465);
//        $response = $thanks_loyalty_controller->sendThanxLoyaltyOrderEvent($complete_order,'refunded');




        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id,'pickup','noete',3);
        $checkout_resource = getCheckoutResourceFromOrderData($cart_data);
        $this->assertNull($checkout_resource->error);
        $checkout_rewards = $checkout_resource->additional_loyalty_checkout_info;
        $this->assertEquals('ThanxLoyalty',$checkout_rewards['service']);
        $this->assertEquals($rewards,$checkout_rewards['data'],'It should have the rewards on the checkout info');

        // lets check on a reward with the cart first try one that should fail.
        $reward = $checkout_rewards['data'][0];

        $cart_ucid = $checkout_resource->ucid;
        $pre_tax_amt = $checkout_resource->total_tax_amt;
        $this->assertEquals(0.00,$checkout_resource->promo_tax_amt);
        $this->assertEquals(0.00,$checkout_resource->promo_amt);

        $promo_code = $reward['id'];
        $remote_promo_service = $checkout_rewards['service'];
        $request = createRequestObject("/app2/apiv2/cart/$cart_ucid/checkout?promo_code=$promo_code&remote_promo_service=$remote_promo_service");
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $promo_checkout_resource = $place_order_controller->processV2Request();
        $this->assertNotNull($promo_checkout_resource->error);

        // now try with a good one.
        $reward = $checkout_rewards['data'][3];

        $promo_code = $reward['id'];
        $remote_promo_service = $checkout_rewards['service'];
        $request = createRequestObject("/app2/apiv2/cart/$cart_ucid/checkout?promo_code=$promo_code&remote_promo_service=$remote_promo_service");
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $promo_checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($promo_checkout_resource->error);
        //$this->assertEquals(-5.00,$promo_checkout_resource->promo_amt);
        $this->assertTrue($promo_checkout_resource->promo_amt < 0);
        $post_tax_amt = $promo_checkout_resource->total_tax_amt;
        $promo_tax_amt = $promo_checkout_resource->promo_tax_amt;
        //$this->assertEquals(-.50,$promo_tax_amt);
        $this->assertEquals($pre_tax_amt+$promo_tax_amt, $post_tax_amt);

        // now clear it
        $request = createRequestObject("/app2/apiv2/cart/$cart_ucid/checkout?promo_code=remove_promo&remote_promo_service=$remote_promo_service");
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $no_promo_checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($no_promo_checkout_resource->error);
        $this->assertEquals(0.00,$no_promo_checkout_resource->promo_amt);
        $this->assertEquals($pre_tax_amt,$no_promo_checkout_resource->total_tax_amt);

        // now add it back in again
        $request = createRequestObject("/app2/apiv2/cart/$cart_ucid/checkout?promo_code=$promo_code&remote_promo_service=$remote_promo_service");
        $place_order_controller = new PlaceOrderController(getM(), $user, $request);
        $place_order_controller->setCurrentTime(getTomorrowTwelveNoonTimeStampDenver());
        $promo_checkout_resource = $place_order_controller->processV2Request();
        $this->assertNull($promo_checkout_resource->error);
        //$this->assertEquals(-5.00,$promo_checkout_resource->promo_amt);
        $this->assertTrue($promo_checkout_resource->promo_amt < 0);
        $post_tax_amt = $promo_checkout_resource->total_tax_amt;
        $promo_tax_amt = $promo_checkout_resource->promo_tax_amt;
        //$this->assertEquals(-.50,$promo_tax_amt);
        $this->assertEquals($pre_tax_amt+$promo_tax_amt, $post_tax_amt);


        $order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
        $this->assertNull($order_resource->error);

        // now refund the order
        $thanks_loyalty_controller = new ThanxLoyaltyController(getM(),$user,null);
        $complete_order = CompleteOrder::staticGetCompleteOrder($order_resource->order_id);
        $response = $thanks_loyalty_controller->sendThanxLoyaltyOrderEvent($complete_order,'refunded');
        $this->assertEquals(201,$response['http_code'],'It should have a successfull refund');



    }

    // grant $10 off coupon   campaign_id = 15vx3eghngplg2k

    function testCreateUserExisting()
    {
        $user_data = [];
        $user_data['first_name'] = 'adam';
        $user_data['last_name'] = 'rosenthal';
        $user_data['email'] = 'arosenthal@order140.com';

        $thanx_loyalty_service = new ThanxLoyaltyService();
        $response = $thanx_loyalty_service->createUser($user_data);
        $this->assertFalse($response,"It should respond with false because we could not create the remote user. they already exist");
        $response = $thanx_loyalty_service->curl_response;
        $this->assertEquals(400,$response['http_code']);
        $this->assertEquals("User account exists",$response['error']);
    }



    function testGetUser()
    {
        $id = '1eebe0f0f77d';
        $access_token = '4fcbadc790a62b41dfc1d7f1a777fc600060deb1df332a31d03786ffad3d092b';
        $thanx_loyalty_controller = new ThanxLoyaltyController();
        $response = $thanx_loyalty_controller->getRemoteUserInfo($id,$access_token);
        $this->assertEquals(200,$response['http_code']);
        $this->assertEquals($id,$response['user']['id']);
        $this->assertTrue(isset($response['user']['email']));
    }

    function testGetAccount()
    {
        $access_token = '4fcbadc790a62b41dfc1d7f1a777fc600060deb1df332a31d03786ffad3d092b';
        $thanx_loyalty_controller = new ThanxLoyaltyController();
        $response = $thanx_loyalty_controller->getRemoteAccount($access_token);
        $this->assertEquals(200,$response['http_code']);
        $this->assertTrue(isset($response['email']));

    }

    function testInitiateAuthenticationWithoutExitingUser()
    {
        $user = createNewUserDataFields();
        unset($user['password']);
        unset($user['account_hash']);
        $thanx_loyalty_service = new ThanxLoyaltyService();
        $response = $thanx_loyalty_service->initiateAuthenticationProcess($user);
        $this->assertEquals(401,$response['http_code']);
    }


    function makeRequestWithDefaultHeaders($url,$userpassword,$method = 'GET',$data = null,$external_id = null)
    {
        if ($external_id == null) {
            $external_id = getContext();
        }
        $headers = array("X_ORDER140_CLIENT_ID"=>"$external_id","X_ORDER140_CLIENT_DEVICE"=>"unit_testing","HTTP_X_ORDER140_CLIENT_VERSION"=>"10.5.0","X_ORDER140_CLIENT"=>"APIThanxTest","NO_CC_CALL"=>"true");
        if ($authentication_token = $data['splickit_authentication_token']) {
            $headers['splickit_authentication_token'] = $authentication_token;
            unset($data['splickit_authentication_token']);
        }
        if ($data['headers']) {
            $headers = array_merge($headers,$data['headers']);
            unset($data['headers']);
        }
        $header_array = array();
        foreach ($headers as $key=>$value) {
            $header_array[] = $key.":".$value;
        }
        return $this->makeRequest($url,$userpassword,$method,$header_array,$data);
    }

    function makeRequest($url,$userpassword,$method = 'GET', $headers = array(),$data = null)
    {
        unset($this->info);
        $method = strtoupper($method);
        $curl = curl_init($url);
        if ($userpassword) {
            curl_setopt($curl, CURLOPT_USERPWD, $userpassword);
        }
        if ($method == 'POST') {
            $json = json_encode($data);
            curl_setopt($curl, CURLOPT_POST, 1);
            curl_setopt($curl, CURLOPT_POSTFIELDS,$json);
            $headers[] = 'Content-Type: application/json';
            $headers[] = 'Content-Length: ' . strlen($json);
        } else if ($method == 'DELETE') {
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "DELETE");
        }
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        logCurl($url,$method,$userpassword,$headers,$json);
        $result = curl_exec($curl);
        $this->info = curl_getinfo($curl);
        curl_close($curl);
        return $result;
    }


	static function setUpBeforeClass()
    {
        //client_id = 2aa4ac527ee97dd4322dfc40e3fe18242c8d4f7ed1c02588eee924c5df5c2003
        //SECRET = 421ca1269e50bb167db5a4bf9467a5c8d4dcec04e7a786fb91cffbb3afc715d8
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time',300);

        $skin_resource = getOrCreateSkinAndBrandIfNecessaryWithLoyalty("rrbbq","rrbbq",335, 598);
        $ids['skin_id'] = $skin_resource->skin_id;
        setContext('com.splickit.rrbbq');

    	$simple_menu_id = createTestMenuWithOneItem("item_one");
    	$simple_menu_id = createTestMenuWithNnumberOfItems(3);
    	$item_sizes = CompleteMenu::getAllItemSizesAsResources($simple_menu_id,0);
    	$item_size = $item_sizes[0];
    	$item_size->external_id = "483541";
    	$item_size->price = 5.00;
    	$item_size->save();

        $item_size = $item_sizes[1];
        $item_size->price = 7.00;
        $item_size->save();
        $item_size = $item_sizes[2];
        $item_size->price = 3.25;
        $item_size->save();


        $ids['menu_id'] = $simple_menu_id;
    	
    	$merchant_resource = createNewTestMerchant($simple_menu_id);
    	$merchant_resource->save();
    	$ids['merchant_id'] = $merchant_resource->merchant_id;

        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $parameters = $tpppa->getParametersFromProviderId(3003);
        foreach ($parameters as $parameter_row) {
            $data = [];
            $data['third_party_provider_parameter_id'] = $parameter_row['third_party_provider_parameter_id'];
            if ($parameter_row['name'] == 'client_id') {
                $data['value'] = '2aa4ac527ee97dd4322dfc40e3fe18242c8d4f7ed1c02588eee924c5df5c2003';
                $data['skin_id'] = getSkinIdForContext();
                $tpppsma = new ThirdPartyProviderParameterSkinMapsAdapter(getM());
                Resource::createByData($tpppsma,$data);
            } else if ($parameter_row['name'] == 'secret') {
                $data['value'] = '421ca1269e50bb167db5a4bf9467a5c8d4dcec04e7a786fb91cffbb3afc715d8';
                $data['skin_id'] = getSkinIdForContext();
                $tpppsma = new ThirdPartyProviderParameterSkinMapsAdapter(getM());
                Resource::createByData($tpppsma,$data);
            } else if ($parameter_row['name'] == 'merchant_key') {
                $data['value'] = 'gkmy334henlyvx6';
                $data['skin_id'] = getSkinIdForContext();
                $tpppsma = new ThirdPartyProviderParameterSkinMapsAdapter(getM());
                Resource::createByData($tpppsma,$data);
            } else if ($parameter_row['name'] == 'redirect_url') {
                $data['value'] = 'rrbbq.order140-test.com';
                $data['skin_id'] = getSkinIdForContext();
                $tpppsma = new ThirdPartyProviderParameterSkinMapsAdapter(getM());
                Resource::createByData($tpppsma,$data);
            }
        }

        $_SERVER['log_level'] = 5;
		$_SERVER['unit_test_ids'] = $ids;
    }

    static function tearDownAfterClass()
    {
        SplickitCache::flushAll();
        date_default_timezone_set($_SERVER['starting_tz']);
    }


    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    ThanxServiceTest::main();
}
?>