<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';


class TwilioTest extends PHPUnit_Framework_TestCase
{
    var $url_encoded_message;

    function setUp()
    {
        $message = '<?xml version="1.0" encoding="UTF-8"?>
            <Response>
                <Message>280743</Message>
                <Message>1234</Message>
                <Play loop="2">https://s3.amazonaws.com/splickit-api-assets/audio/restart_printer.mp3</Play>
            </Response>';
        $this->url_encoded_message = urlencode(cleanUpDoubleSpacesCRLFTFromString($message));
    }

    function testSMSWithTwillio()
    {
        $phone_number = '4107754092';
        $twillio_service = new TwilioService();
        $response = $twillio_service->doSMSMessage($phone_number,"hello world");
        $this->assertEquals("queued",$response['response_text']);
    }
    
    function testSMSWithTwilioLongMessage()
    {
        $phone_number = '4107754092';
        $twillio_service = new TwilioService();
        $message = "I am that merry wanderer of the night. I jest to Oberon and make him smile When I a fat and bean-fed horse beguile, Neighing in likeness of a filly foal: And sometime lurk I in a gossip's bowl, In very likeness of a roasted crab, And when she drinks, against her lips I bob And on her wither'd dewlap pour the ale.";
        $response = $twillio_service->doSMSMessage($phone_number,$message);
        $this->assertEquals("queued",$response['response_text']);
    }

    /**
     * Run this test to test the twilio calls
    */
    function testOutboundCallsWithTwilio()
    {
        $phone_number = "4107754092";//TODO: This is the number to call, should has ten digits only
        $message_response_url= getProperty('twimlets_server').$this->url_encoded_message;
        $response = TwilioService::InitializeOutboundCall($phone_number, $message_response_url);
        $this->assertEquals('queued',$response['twilio_status']);
    }

    function testTwimletsServerAndMessageEncode(){
        $this->assertNotNull(getProperty('twimlets_server'));
        $this->assertEquals("http://twimlets.com/echo?Twiml=",getProperty('twimlets_server'));
        $message_response_url= getProperty('twimlets_server').$this->url_encoded_message;
        $this->assertEquals(str_replace(array("\n", " "),"","http://twimlets.com/echo?Twiml=%3C%3Fxml+version%
        3D%221.0%22+encoding%3D%22UTF-8%22%3F%3E%3CResponse%3E%3CMessage%3E280743%3C%2FMessage%3E%3CMessage%
        3E1234%3C%2FMessage%3E%3CPlay+loop%3D%222%22%3Ehttps%3A%2F%2Fs3.amazonaws.com%2Fsplickit-api-assets%
        2Faudio%2Fyou_have_a_pending_order.mp3%3C%2FPlay%3E%3C%2FResponse%3E"),$message_response_url);
    }

    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    TwilioTest::main();
}

