<?php
error_reporting(E_ERROR | E_COMPILE_ERROR | E_COMPILE_WARNING | E_PARSE);
$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');
$db_info->database = 'smaw_unittest';
$db_info->username = 'root';
$db_info->password = 'splickit';
if (isset($_SERVER['XDEBUG_CONFIG'])) {
    putenv("SMAW_ENV=unit_test_ide");
    $db_info->hostname = "127.0.0.1";
    $db_info->port = 13306;
} else {
    $db_info->hostname = "db_container";
    $db_info->port = 3306;
}
$_SERVER['DB_INFO'] = $db_info;
require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class ValueIOTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;
	
	function setUp()
	{
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
		$this->api_port = "80";
		if (isset($_SERVER['XDEBUG_CONFIG'])) {
            $this->api_port = "10080";
        }
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
		unset($this->stamp);
    }

	/**************** sage **********************/

	function testCreateSageMerchantGateway()
	{
		$type = 'sage';
		$identifier = generateCode(20);
		$data['merchant_id_number'] = '469197833948';
		$data['merchant_id_key'] = 'T8L3P8O2X7I1';
		$vio_service = new VioPaymentService($data);
		$response = $vio_service->createDestination($type, $identifier, $data);
		$this->assertNotNull($response);
		$this->assertEquals('success', $response['status']);
		$this->assertNotNull($response['destination']);
		$destination = $response['destination'];
		$this->assertEquals($identifier, $destination['identifier']);
		$this->assertNotNull($destination['id']);
		//return $destination['id'];
		//BillingEntitiesAdapter::createBillingEntity($vio_credit_card_processor_id, 'test billing entity', 'test billing entity', 100, $identifier, $credentials);
		//$_SERVER['unit_test_ids']['destination_id'] = $identifier;
		return $identifier;
	}

	/**
	 * @depends testCreateSageMerchantGateway
	 */
	function testProcessCard($identifier)
	{
		$user_resource = $this->ids['user_resource'];
		//$data = array("billing_entity_external_id"=>$identifier,"user_id"=>$user_resource->user_id);
		$data = array("user_id"=>$user_resource->user_id);
		$vio_payment_service = new VioPaymentService($data);
		$vio_payment_service->setVioPaymentIdentifier($identifier);
		$order_id = time();
		$vio_payment_service->setOrderId($order_id);
		$response = $vio_payment_service->processPayment("1.00");
		$this->assertEquals('success', $response['status']);
		$this->assertEquals($order_id,$response['payment']['response']['params']['order_number']);
		$bcr['cc_processor'] = $identifier;
		$bcr['cc_transaction_id'] = $response['transactionid'];
		return $bcr;
		//return $response['transactionid'];
	}

	/**
	 * @depends testProcessCard
	 */
	function testVoidPayment($bcr)
	{
		//F6FEFt2B90
		$vio_payment_service = new VioPaymentService($data);
		$user_resource = $this->ids['user_resource'];
		$vio_payment_service->setBillingUserIdAndResourceFromUserId($user_resource->user_id);
		$response = $vio_payment_service->voidTransaction($bcr);
		$this->assertEquals('success', $response['status']);
		$this->assertEquals(100, $response['response_code']);
	}

	/**
	 * @depends testCreateSageMerchantGateway
	 */
	function testRefundPayment($identifier)
	{
		$user_resource = $this->ids['user_resource'];
		//$data = array("billing_entity_external_id"=>$identifier,"user_id"=>$user_resource->user_id);
		$data = array("user_id"=>$user_resource->user_id);
		$vio_payment_service = new VioPaymentService($data);
		$vio_payment_service->setVioPaymentIdentifier($identifier);
		$vio_payment_service->setOrderId(time());
		$response = $vio_payment_service->processPayment("1.00");
		$this->assertEquals('success', $response['status']);
		$bcr['cc_processor'] = $identifier;
		$bcr['cc_transaction_id'] = $response['transactionid'];

		$vio_payment_service2 = new VioPaymentService($data);
		$user_resource = $this->ids['user_resource'];
		$vio_payment_service2->setBillingUserIdAndResourceFromUserId($user_resource->user_id);
		$response = $vio_payment_service2->refundTransaction($bcr,1.00);
		$this->assertEquals('success', $response['status']);
		$this->assertEquals(100, $response['response_code']);
	}

	/**
	 * @depends testCreateSageMerchantGateway
	 */
	function testProcessCardWithFailure($identifier)
	{
		$user_resource = createNewUser();
		$uuid = $user_resource->uuid;

		//{"brand": "visa", "number":"4242424242424242","cvv":"123","month":4,"year":2015,"first_name":"Ara","last_name":"Howard"}
		$data['identifier'] = $uuid;
		$data['brand'] = "visa";
		//$data['number'] = "4003000176567651";
		$data['number'] = "4111111111111111";
		$data['cvv'] = "123";
		$data['zip'] = "12345";
		$data['month'] = 10;
		$data['year'] = 2020;
		$data['first_name'] = "sumdum";
		$data['last_name'] = "guy2";
		$vio_payment_service = new VioPaymentService();
		$response = $vio_payment_service->saveCreditCard($data);
		$this->assertNotNull($response);
		$this->assertEquals('success', $response['status']);
		$this->assertNotNull($response['credit_card']);
		$card_return = $response['credit_card'];
		$this->assertNotNull($card_return['credit_card_token_single_use']);
		//$data = array("billing_entity_external_id"=>$identifier,"user_id"=>$user_resource->user_id);
		$data = array("user_id"=>$user_resource->user_id);
		$vio_payment_service = new VioPaymentService($data);
		$vio_payment_service->setVioPaymentIdentifier($identifier);
		$response = $vio_payment_service->processPayment("2.00");
		$this->assertEquals('failure', $response['status']);
		$this->assertEquals("We're sorry but your credit card was declined.", $response['response_text']);
	}

	/**
	 * @depends testCreateSageMerchantGateway
	 */
	function testProcessCardWithAuthorizeAndThenCapture($destination_identifier)
	{


		$user_resource = $this->ids['user_resource'];
		//$data = array("billing_entity_external_id"=>$identifier,"user_id"=>$user_resource->user_id);
		$data = array("user_id"=>$user_resource->user_id);
		$vio_payment_service = new VioPaymentService($data);
		$vio_payment_service->billing_entity_record = array("process_type"=>'authorize');
		$vio_payment_service->setVioPaymentIdentifier($destination_identifier);
		$order_id = time()+1;
		$vio_payment_service->setOrderId($order_id);
		$response = $vio_payment_service->processPayment("1.00");
		$this->assertEquals('success', $response['status']);
		$this->assertEquals($order_id,$response['payment']['response']['params']['order_number']);
		$identifier = $response['payment']['identifier'];

		$vio_payment_service = new VioPaymentService($data);
		$vio_payment_service->setVioPaymentIdentifier($destination_identifier);
		$response = $vio_payment_service->capture(1.00,$identifier);
		$this->assertEquals(200,$response['http_code']);

		$fields = array("destination_identifier"=>$destination_identifier,"identifier"=>$identifier);
		return $fields;
	}

	/**
	 * @depends testProcessCardWithAuthorizeAndThenCapture
	 */
	function testProcessCardWithAuthorizeAndThenCaptureFailure($fields)
	{
		$user_resource = $this->ids['user_resource'];
		$data = array("user_id"=>$user_resource->user_id);
		$vio_payment_service = new VioPaymentService($data);
		$vio_payment_service->setVioPaymentIdentifier($fields['destination_identifier']);
		$response = $vio_payment_service->capture(1.00,$fields['identifier']);
		$this->assertEquals(422,$response['http_code']);
	}

	function testCreateUserVaultRecordForMecuryCharge()
	{
		$user_resource = $this->ids['mercury_user_resource'];
		$uuid = $user_resource->uuid;

		//{"brand": "visa", "number":"4242424242424242","cvv":"123","month":4,"year":2015,"first_name":"Ara","last_name":"Howard"}
		$data['identifier'] = $uuid;
		$data['brand'] = "visa";
		$data['number'] = "4003000123456781";
		$data['cvv'] = "123";
		$data['zip'] = "12345";
		$data['month'] = 10;
		$data['year'] = 2020;
		$data['first_name'] = "sumdum2";
		$data['last_name'] = "guy";
		$vio_payment_service = new VioPaymentService();
		$response = $vio_payment_service->saveCreditCard($data);
		$this->assertNotNull($response);
		$this->assertEquals('success', $response['status']);
		$this->assertNotNull($response['credit_card']);
		$card_return = $response['credit_card'];
		$this->assertNotNull($card_return['credit_card_token_single_use']);
	}

	function testSetupInstantMerchantAccount()
	{

		setContext('com.splickit.worldhq');
		$menu_id = createTestMenuWithNnumberOfItems(1);
		$merchant_resource = createNewTestMerchant($menu_id,['no_payment'=>true]);



		$data["merchant_id"] = $merchant_resource->merchant_id;
		$data["test"] = "true";
		$data["first_name"] = "Chris";
		$data["last_name"] = "Terry";
		$data["date_of_birth"] = "1970-1-23";
		$data["social_security_number"] = "555-55-5555";
		$data["phone"] = "555-555-5555";
		$data["currency"] = "USD";
		$data["business_legal_name"] = "Booming Biz";
		$data["doing_business_as"] = "";
		$data["ein"] = "444-44-4444";
		$data["monthly_bank_card_volume"] = 0;
		$data["average_ticket"] = 0;
		$data["highest_ticket"] = 0;
		$data["apartment_number"] = "#3W";
		$data["address1"] = $merchant_resource->address1;
		$data["address2"] = "";
		$data["city"] = $merchant_resource->city;
		$data["state"] = $merchant_resource->state;
		$data["country"] = "USA";
		$data["zip"] = $merchant_resource->zip;
		$data["bank_account_country_code"] = "USA";
		$data["bank_account_account_number"] = "55555555555";
        $data["bank_account_routing_number"] = "011401533";
        //$data["bank_account_routing_number"] = "555555555";
        $data["bank_account_bank_name"] = "Greatown Bank";

        Resource::createByData(new PaymentMerchantApplicationsAdapter(getM()),$data);


		$card_gateway_controoler = new CardGatewayController(getM(),$u,$r);
		$merchant_billing_resource = $card_gateway_controoler->createNewMerchantAccountAndBillingEntity($data);
		$this->assertTrue($merchant_billing_resource->_exists,"It should have created a billing destination ");
		$this->assertTrue(isset($merchant_billing_resource->merchant_payment_type_map),"It should have created the billing map");



	}


	function testCreateSecureNetMerchantGateway()
	{
		setContext('com.splickit.worldhq');
		$menu_id = createTestMenuWithNnumberOfItems(1);
		$merchant_resource = createNewTestMerchant($menu_id,['no_payment'=>true]);

		$secure_net_id = generateAlphaCode(10);
		$secure_key = generateAlphaCode(20);

		$data['vio_selected_server'] = 'Secure-Net';
		$data['vio_merchant_id'] = $merchant_resource->merchant_id;
		$data['description'] = 'An entity to test with';
		$data['secure_net_id'] = $secure_net_id;
		$data['secure_key'] = $secure_key;
		$data['identifier'] = $merchant_resource->alphanumeric_id;
		$data['brand_id'] = $merchant_resource->brand_id;
		$data['process_type'] = 'Authorize';

		$card_gateway_controller = new CardGatewayController($mt, $u, $r);
		$resource = $card_gateway_controller->createPaymentGateway($data);
		$this->assertTrue(is_a($resource, 'Resource'),'Should have returned a resource');
		$this->assertNull($resource->error);
		$this->assertNotNull($resource->id);
		$this->assertNotNull($resource->merchant_payment_type_map);

		$expected_string = "secure_net_id=$secure_net_id|secure_key=$secure_key";
		$this->assertEquals($expected_string,$resource->credentials);
		$created_merchant_payment_type_map_id = $resource->merchant_payment_type_map->id;

		$billing_entity_resource = SplickitController::getResourceFromId($resource->id, 'BillingEntities');
		$this->assertNotNull($billing_entity_resource);
		$this->assertEquals('2008',$billing_entity_resource->vio_credit_card_processor_id);
		$this->assertEquals('Secure-Net='.$merchant_resource->merchant_id, $billing_entity_resource->name);
		$this->assertEquals($merchant_resource->brand_id, $billing_entity_resource->brand_id);
		$this->assertEquals($merchant_resource->alphanumeric_id, $billing_entity_resource->external_id);

		$merchant_payment_type_map_resource = SplickitController::getResourceFromId($created_merchant_payment_type_map_id, 'MerchantPaymentTypeMaps');
		$this->assertNotNull($merchant_payment_type_map_resource);
		//$this->assertEquals($resource->identifier, $merchant_payment_type_map_resource->vio_destination_id);
		$this->assertEquals('2000', $merchant_payment_type_map_resource->splickit_accepted_payment_type_id);
		$this->assertEquals($billing_entity_resource->id,$merchant_payment_type_map_resource->billing_entity_id);
		return $merchant_payment_type_map_resource;
	}

	/**
	 * @depends testCreateSecureNetMerchantGateway
	 */
	function testPlaceOrderWithSecureNet($merchant_payment_type_map_resource)
	{
		setContext('com.splickit.worldhq');
		$merchant_id = $merchant_payment_type_map_resource->merchant_id;
		$user_resource = createNewUserWithCCNoCVV();
		$user = logTestUserResourceIn($user_resource);

		$cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
		$checkout_resource = getCheckoutResourceFromOrderData($cart_data,getTomorrowTwelveNoonTimeStampDenver());
		$this->assertNull($checkout_resource->error);

		$order_resource = placeOrderFromCheckoutResource($checkout_resource,$user,$merchant_id,0.00);
		$this->assertNull($order_resource->error);

	}
		/**
	 * @depends testCreateSecureNetMerchantGateway
	 */
	function testProcessCardSecureNet($merchant_payment_type_map_resource)
	{
		$user_resource = createNewUser();
		$data = array("billing_entity_id"=>$merchant_payment_type_map_resource->billing_entity_id,"user_id"=>$user_resource->user_id);
		$vio_payment_service = new VioPaymentService($data);
		$response = $vio_payment_service->processPayment("2.00");
		$this->assertEquals('success', $response['status']);
	}


    function testDeleteWithNoRecord()
    {
        $vio_payment_service = new VioPaymentService($d);
        $uuid = generateUUID();
        $delete_response = $vio_payment_service->deleteVaultRecord($uuid);
        $this->assertEquals(404,$delete_response['http_code']);
    }

    function testDeleteVaultRecord()
    {
        $user_resource = $this->ids['user_resource'];
        $uuid = $user_resource->uuid;

        //{"brand": "visa", "number":"4242424242424242","cvv":"123","month":4,"year":2015,"first_name":"Ara","last_name":"Howard"}
        $data['identifier'] = $uuid;
        $data['brand'] = "visa";
        $data['number'] = "4111111111111111";
        $data['cvv'] = "123";
        $data['zip'] = "12345";
        $data['month'] = 10;
        $data['year'] = 2029;
        $data['first_name'] = "sumdum";
        $data['last_name'] = "guy";
        $vio_payment_service = new VioPaymentService($data);
        $response = $vio_payment_service->saveCreditCard($data);
        $this->assertNotNull($response);
        $this->assertEquals('success', $response['status']);
        $this->assertNotNull($response['credit_card']);
        $card_return = $response['credit_card'];
        $this->assertNotNull($card_return['credit_card_token_single_use']);

        $delete_response = $vio_payment_service->deleteVaultRecord($uuid);
        $this->assertNotNull($delete_response,"It should have gotten a response from the delete call");
        $this->assertEquals('success', $delete_response['status']);

    }

    function testUpdateVaultRecord()
    {
        $user_resource = $this->ids['user_resource'];
        $uuid = $user_resource->uuid;

        //{"brand": "visa", "number":"4242424242424242","cvv":"123","month":4,"year":2015,"first_name":"Ara","last_name":"Howard"}
        //$data['identifier'] = $uuid;
        $data['brand'] = "visa";
        $data['number'] = "4012888888881881";
        $data['cvv'] = "123";
        $data['zip'] = "12345";
        $data['month'] = 10;
        $data['year'] = 2029;
        $data['first_name'] = "sumdum";
        $data['last_name'] = "guy";
        $vio_payment_service = new VioPaymentService($data);
        $response = $vio_payment_service->saveCreditCard($data);
        $this->assertNotNull($response);
        $this->assertEquals('success', $response['status']);
        $this->assertNotNull($response['credit_card']);
        $card_return = $response['credit_card'];
        $this->assertNotNull($card_return['credit_card_token_single_use']);

        $url = "http://127.0.0.1:".$this->api_port."/app2/apiv2/users/$uuid";
        $new_user_data['credit_card_token_single_use'] = $card_return['credit_card_token_single_use'];
        $new_user_data['credit_card_saved_in_vault'] = false;
        $new_user_data['last_four'] = "1881";
        $response = $this->makeRequestWithDefaultHeaders($url, $user_resource->email.":welcome","POST",$new_user_data);


//        $new_uuid = generateUUID();
//        $update_response = $vio_payment_service->updateExistingRecordWithNewUUID($card_return['credit_card_token_single_use'],$new_uuid);
//        $this->assertNotNull($update_response,"It should have gotten a response from the delete call");
//        $this->assertEquals('success', $update_response['status']);

        $vio_payment_service2 = new VioPaymentService($d);
        $cc_info = $vio_payment_service2->getCreditCardInfoNonSensitive($uuid);
        $this->assertNotNull($cc_info);
        $this->assertEquals('XXXX-XXXX-XXXX-1881',$cc_info['credit_card']['number']);
    }
    
    function testCreateUserVaultRecordForSageCharge()
    {
    	$user_resource = createNewUser();
    	//$user_resource = $this->ids['user_resource'];
    	$uuid = $user_resource->uuid;

    	//{"brand": "visa", "number":"4242424242424242","cvv":"123","month":4,"year":2015,"first_name":"Ara","last_name":"Howard"}
    	$data['identifier'] = $uuid;
    	$data['brand'] = "visa";
    	$data['number'] = "4111111111111111";
    	$data['cvv'] = "123";
    	$data['zip'] = "12345";
    	$data['month'] = 10;
    	$data['year'] = 2020;
    	$data['first_name'] = "sumdum";
    	$data['last_name'] = "guy";
    	$vio_payment_service = new VioPaymentService($data);
    	$response = $vio_payment_service->saveCreditCard($data);
    	$this->assertNotNull($response);
    	$this->assertEquals('success', $response['status']);
    	$this->assertNotNull($response['credit_card']);
    	$card_return = $response['credit_card'];
    	$this->assertNotNull($card_return['credit_card_token_single_use']);
    }
    
    function testGetUserNonSensitiveInfoFromVault()
    {
    	$user_resource = $this->ids['user_resource'];
    	$uuid = $user_resource->uuid;
    	$vio_payment_service = new VioPaymentService($data);
    	$response = $vio_payment_service->getCreditCardInfoNonSensitive($uuid);
    	$month = $response['credit_card']['month'];
    	$year = $response['credit_card']['year'];
    	$this->assertEquals(10, $month);
    	$this->assertEquals(2020, $year);
        return $response;
    }

    function testUpdateCCwithExistingValueRecord()
    {
        $user_resource = createNewUser();
        $uuid = $user_resource->uuid;

        //{"brand": "visa", "number":"4242424242424242","cvv":"123","month":4,"year":2015,"first_name":"Ara","last_name":"Howard"}
        $data['identifier'] = $uuid;
        $data['brand'] = "visa";
        $data['number'] = "4111111111111111";
        $data['cvv'] = "123";
        $data['zip'] = "12345";
        $data['month'] = 10;
        $data['year'] = 2020;
        $data['first_name'] = "sumdum";
        $data['last_name'] = "guy";
        $vio_payment_service = new VioPaymentService($d);
        $response = $vio_payment_service->saveCreditCard($data);
        $this->assertNotNull($response);
        $this->assertEquals('success', $response['status']);
        $this->assertNotNull($response['credit_card']);
        $card_return1 = $response['credit_card'];
        $this->assertNotNull($card_return1['credit_card_token_single_use']);

        // now create a new cc at VIO (simulating how the web does update)
        unset($data['identifier']);
        $data['number'] = "4012888888881881";
        $data['year'] = 2022;
        $data['zip'] = "56789";
        $vio_payment_service = new VioPaymentService($d);
        $response = $vio_payment_service->saveCreditCard($data);
        $this->assertNotNull($response);
        $this->assertEquals('success', $response['status']);
        $this->assertNotNull($response['credit_card']);
        $card_return2 = $response['credit_card'];
        $this->assertNotNull($card_return2['credit_card_token_single_use']);
        $this->assertNotEquals($card_return2['credit_card_token_single_use'],$card_return1['credit_card_token_single_use'],"The CC single use tokens shoudl not be the same");

        // the stage is set
        $user = logTestUserResourceIn($user_resource);
        $data = array ('credit_card_saved_in_vault'=>false,'credit_card_token_single_use'=>$card_return2['credit_card_token_single_use']);
        $request = new Request();
        $request->data = $data;
        $user_controller = new UserController($mt, $user, $request,5);
        $user_resource = $user_controller->updateUser();
        $this->assertNull($user_resource->error);
        $this->assertEquals('1C21000001',$user_resource->flags);
        $vio_payment_service2 = new VioPaymentService($d);
        $cc_info2 = $vio_payment_service2->getCreditCardInfoNonSensitive($uuid);
        $this->assertEquals("XXXX-XXXX-XXXX-1881",$cc_info2['credit_card']['number']);
        $this->assertEquals(2022,$cc_info2['credit_card']['year']);
        $this->assertEquals('1881',$user_resource->last_four,"last four shoudl have been saved by calling VIO after flag update");

        //test to see if the user had the last 4 updated
        $user = UserAdapter::staticGetRecordByPrimaryKey($user['user_id'],'UserAdapter');
        $this->assertEquals('1881',$user['last_four'],"Last 4 of user should have been updated");

		//test to see if the record was added to the update table
		$record = CreditCardUpdateTrackingAdapter::staticGetRecord(array("user_id"=>$user_resource->user_id),'CreditCardUpdateTrackingAdapter');
		$this->assertEquals('1881',$record['last_four']);
    }

    function testUpdateCCwithNoCurrentVaultRecord()
    {
        $data['brand'] = "visa";
        $data['number'] = "4111111111111111";
        $data['cvv'] = "123";
        $data['zip'] = "12345";
        $data['month'] = 10;
        $data['year'] = 2021;
        $data['first_name'] = "sumdum";
        $data['last_name'] = "guy";

        $vio_payment_service = new VioPaymentService($d);
        $response = $vio_payment_service->saveCreditCard($data);


        $this->assertNotNull($response);
        $this->assertEquals('success', $response['status']);
        $this->assertNotNull($response['credit_card']);
        $card_return1 = $response['credit_card'];
        $this->assertNotNull($card_return1['credit_card_token_single_use']);

        $user_resource = createNewUser();
        $user = logTestUserResourceIn($user_resource);
        $data = array ('credit_card_saved_in_vault'=>false,'credit_card_token_single_use'=>$card_return1['credit_card_token_single_use']);
        $request = new Request();
        $request->data = $data;
        $user_controller = new UserController($mt, $user, $request,5);
        $user_resource = $user_controller->updateUser();
        //$this->assertNull($user_resource->error);
        $this->assertEquals('1C21000001',$user_resource->flags);
        $vio_payment_service2 = new VioPaymentService($d);
        $cc_info2 = $vio_payment_service2->getCreditCardInfoNonSensitive($user_resource->uuid);
        $this->assertEquals("XXXX-XXXX-XXXX-1111",$cc_info2['credit_card']['number']);
        $this->assertEquals(2021,$cc_info2['credit_card']['year']);
        $this->assertEquals('1111',$user_resource->last_four,"last four shoudl have been saved by calling VIO after flag update");
        return $card_return1['credit_card_token_single_use'];
    }

    /**
     * @depends testUpdateCCwithNoCurrentVaultRecord
     */
    function testGetCorrectErrorIfSingleUserTokenDoesntExistAtVio($single_use_token)
    {
        $user_resource = createNewUser();
        $user = logTestUserResourceIn($user_resource);
        //$single_use_token = generateAlphaCode(20);
        $data = array ('credit_card_saved_in_vault'=>false,'credit_card_token_single_use'=>$single_use_token);
        $request = new Request();
        $request->data = $data;
        $user_controller = new UserController($mt, $user, $request,5);
        $user_resource = $user_controller->updateUser();
        $this->assertNotNull($user_resource->error);
        $this->assertEquals(500,$user_resource->error_code);

    }

    /**
     * @depends testGetUserNonSensitiveInfoFromVault
     */
    function testGetLastFourOfCC($response)
    {
        $vio_payment_service = new VioPaymentService($data);
        $last_4 = $vio_payment_service->getLast4FromNonSensitiveResponse($response);
        $this->assertEquals('1881',$last_4,"should have returned the last four of 1111");
    }

   	function testCreateUserVaultRecordForFPNCharge()
    {
    	$user_resource = $this->ids['fpn_user_resource'];
    	$uuid = $user_resource->uuid;
    	
    	//{"brand": "visa", "number":"4242424242424242","cvv":"123","month":4,"year":2015,"first_name":"Ara","last_name":"Howard"}
    	$data['identifier'] = $uuid;
    	$data['brand'] = "visa";
    	$data['number'] = "4111111111111111";
    	$data['cvv'] = "123";
    	$data['zip'] = "12345";
    	$data['month'] = 10;
    	$data['year'] = 2020;
    	$data['first_name'] = "sumdum";
    	$data['last_name'] = "guy";
    	$vio_payment_service = new VioPaymentService($d);
    	$response = $vio_payment_service->saveCreditCard($data);
    	$this->assertNotNull($response);
    	$this->assertEquals('success', $response['status']);
    	$this->assertNotNull($response['credit_card']);
    	$card_return = $response['credit_card'];
    	$this->assertNotNull($card_return['credit_card_token_single_use']);
    }
    
    function testCreateFPNMerchantGateway()
    {
    	// this is generated on the FPN site
    	$source_key = '_1Mk65OH04krz32O4ZG2vy3Oj884882A';
    	$source_key = '_D5wSUO25tm0o1Yb4m11VRPpqAt7o9u3';
    	 
    	$merchant_resource = $this->ids['fpn_merchant_resource'];
    	$merchant_id_key = generateCode(10);
    	$merchant_id_number = generateCode(5);
    	$data['vio_selected_server'] = 'Fpn';
    	$data['vio_merchant_id'] = $merchant_resource->merchant_id;
    	$data['description'] = 'An entity to test with';
    	$data['source_key'] = $source_key;
    	$data['identifier'] = $merchant_resource->alphanumeric_id;
    	$data['brand_id'] = $merchant_resource->brand_id;
    	
    	$card_gateway_controller = new CardGatewayController($mt, $u, $r);
    	$resource = $card_gateway_controller->createPaymentGateway($data);
    	$this->assertTrue(is_a($resource, 'Resource'),'Should have returned a resource');
    	$this->assertNull($resource->error);
    	$this->assertNotNull($resource->id);
    	$this->assertNotNull($resource->merchant_payment_type_map);
    	$expected_string = "source_key=$source_key";
    	$this->assertEquals($expected_string,$resource->credentials);
    	$created_merchant_payment_type_map_id = $resource->merchant_payment_type_map->id;

    	$billing_entity_resource = SplickitController::getResourceFromId($resource->id, 'BillingEntities');
    	$this->assertNotNull($billing_entity_resource);
    	$this->assertEquals('2003',$billing_entity_resource->vio_credit_card_processor_id);
    	$this->assertEquals('Fpn='.$merchant_resource->merchant_id, $billing_entity_resource->name);
    	$this->assertEquals($merchant_resource->brand_id, $billing_entity_resource->brand_id);
    	$this->assertEquals($merchant_resource->alphanumeric_id, $billing_entity_resource->external_id);

    	$merchant_payment_type_map_resource = SplickitController::getResourceFromId($created_merchant_payment_type_map_id, 'MerchantPaymentTypeMaps');
    	$this->assertNotNull($merchant_payment_type_map_resource);
    	//$this->assertEquals($resource->identifier, $merchant_payment_type_map_resource->vio_destination_id);
    	$this->assertEquals('2000', $merchant_payment_type_map_resource->splickit_accepted_payment_type_id);
    	$this->assertEquals($billing_entity_resource->id,$merchant_payment_type_map_resource->billing_entity_id);
    	return $merchant_payment_type_map_resource;
    }
       	
   	/**
   	 * @depends testCreateFPNMerchantGateway
   	 */
   	function testProcessCardFPN($merchant_payment_type_map_resource)
   	{
   		$user_resource = $this->ids['fpn_user_resource'];
		$data = array("billing_entity_id"=>$merchant_payment_type_map_resource->billing_entity_id,"user_id"=>$user_resource->user_id);
		$vio_payment_service = new VioPaymentService($data);
		$response = $vio_payment_service->processPayment("2.00");
   		$this->assertEquals('success', $response['status']);
   	}
   	

    /***************  now do mercury ***************/
    
   	function testCreateMercuryMerchantGateway()
   	{
   		$type = 'mercury';
   		$identifier = generateCode(20);
   		$data['merchant_id'] = '023358150511666';
   		$data['password'] = 'xyz';
   		
   		$vio_service = new VioPaymentService();
   		$response = $vio_service->createDestination($type, $identifier, $data);
   		$this->assertNotNull($response);
    	$this->assertEquals('success', $response['status']);
    	$this->assertNotNull($response['destination']);
   		$destination = $response['destination'];
   		$this->assertEquals($identifier, $destination['identifier']);
   		$this->assertNotNull($destination['id']);
   		//return $destination['id'];
   		//BillingEntitiesAdapter::createBillingEntity($vio_credit_card_processor_id, 'test billing entity', 'test billing entity', 100, $identifier, $credentials);
   		//$_SERVER['unit_test_ids']['destination_id'] = $identifier;
   		return $identifier;
   	}
   	
   	/**
   	 * @depends testCreateMercuryMerchantGateway
   	 */
   	function testProcessCardMercury($identifier)
   	{
		$user_resource = $this->ids['mercury_user_resource'];
		//$data = array("billing_entity_external_id"=>$identifier,"user_id"=>$user_resource->user_id);
   		$data = array("user_id"=>$user_resource->user_id);
		$vio_payment_service = new VioPaymentService($data);
		$vio_payment_service->setVioPaymentIdentifier($identifier);
   		$vio_payment_service->setOrderId(time());
   		$response = $vio_payment_service->processPayment("1.00");
   		$this->assertEquals('success', $response['status']);
   		$bcr['cc_processor'] = $identifier;
   		$bcr['cc_transaction_id'] = $response['transactionid'];
   		return $bcr;
   		//return $response['transactionid'];
   	}
   	
   	/**
   	 * @depends testProcessCardMercury
   	 */
   	function testVoidPaymentMercury($bcr)
   	{
   		//F6FEFt2B90
		$vio_payment_service = new VioPaymentService($data);
		$user_resource = $this->ids['mercury_user_resource'];
		$vio_payment_service->setBillingUserIdAndResourceFromUserId($user_resource->user_id);
		$response = $vio_payment_service->voidTransaction($bcr);
		$this->assertEquals('success', $response['status']);
   		$this->assertEquals(100, $response['response_code']);
   	}

    function getExternalId()
    {
        if ($external_id = getContext()) {
            // use it
        } else {
            $external_id = "com.splickit.vtwoapi";
        }
        return $external_id;
    }

    function makeRequestWithDefaultHeaders($url,$userpassword,$method = 'GET',$data = null)
    {
        $external_id = $this->getExternalId();
        $headers = array("X_SPLICKIT_CLIENT_ID"=>"$external_id","X_SPLICKIT_CLIENT_DEVICE"=>"unit_testing","HTTP_X_SPLICKIT_CLIENT_VERSION"=>"10.5.0","X_SPLICKIT_CLIENT"=>"APIDispatchTest");
        if ($authentication_token = $data['splickit_authentication_token']) {
            $headers['splickit_authentication_token'] = $authentication_token;
            unset($data['splickit_authentication_token']);
        }
        if ($data['headers']) {
            $headers = array_merge($headers,$data['headers']);
            unset($data['headers']);
        }
        $header_array = array();
        foreach ($headers as $key=>$value) {
            $header_array[] = $key.":".$value;
        }
        return $this->makeRequest($url,$userpassword,$method,$header_array,$data);
    }

    function makeRequest($url,$userpassword,$method = 'GET', $headers = array(),$data = null)
    {
        unset($this->info);
        $method = strtoupper($method);
        $curl = curl_init($url);
        if ($userpassword) {
            curl_setopt($curl, CURLOPT_USERPWD, $userpassword);
        }
        if ($method == 'POST') {
            $json = json_encode($data);
            curl_setopt($curl, CURLOPT_POST, 1);
            curl_setopt($curl, CURLOPT_POSTFIELDS,$json);
            $headers[] = 'Content-Type: application/json';
            $headers[] = 'Content-Length: ' . strlen($json);
        } else if ($method == 'DELETE') {
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "DELETE");
        }
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        logCurl($url,$method,$userpassword,$headers,$json);
        $result = curl_exec($curl);
        $this->info = curl_getinfo($curl);
        curl_close($curl);
        return $result;
    }

    static function setUpBeforeClass()
    {
    	$_SERVER['request_time1'] = microtime(true);
    	$_SERVER['NO_MOCKS'] = true;
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	mysqli_query("BEGIN");
    	$user_resource = createNewUser();
		$ids['user_resource'] = $user_resource;
		
		$mercury_user_resource = createNewUser();
		$ids['mercury_user_resource'] = $mercury_user_resource;
    	
		$fpn_user_resource = createNewUser();
		$ids['fpn_user_resource'] = $fpn_user_resource;
		
		$fpn_merchant_resource = createNewTestMerchant(0,['no_payment'=>true]);
		$ids['fpn_merchant_resource'] = $fpn_merchant_resource;
		
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	mysqli_query("ROLLBACK");
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    ValueIOTest::main();
}
?>
