<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';


class VivonetTest extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
		
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }

	/**
	 * @expectedException UnsuccessfulVivonetPushException
	 */
	function testVivonetServiceMerchantIsNotEnableForMerchant()
	{
		$vivonet_service = new VivonetService(123456);
		$this->assertEquals("vivonet", $vivonet_service->service_name);
	}

    function testVivonetServiceMerchantIsEnableForMerchant()
    {
    	$vivonet_service = new VivonetService($this->ids["merchant_external_id"]);
		$this->assertEquals("vivonet", $vivonet_service->service_name);
		$this->assertEquals($this->ids["merchant_external_id"], $vivonet_service->store_id);

    }
	/**
	 * @expectedException InvalidVivonetConfigurationException
	 */
	function testIvalidConfigurationVivonetService(){
		$vivonet_service = new VivonetService($this->ids["merchant_external_id"]);
		$this->assertEquals("vivonet", $vivonet_service->service_name);
		$this->assertEquals($this->ids["merchant_external_id"], $vivonet_service->store_id);
		$vivonet_service->configure(
			VivonetService::PLACE_ORDER,
			VivonetService::PLACE_ORDER,
			array("storeId" => $vivonet_service->store_id),
			array("storeId"=> $vivonet_service->store_id)
		);
	}

	function testConfigureVivonetService(){
		$vivonet_service = new VivonetService($this->ids["merchant_external_id"]);
		$this->assertEquals("vivonet", $vivonet_service->service_name);
		$this->assertEquals($this->ids["merchant_external_id"], $vivonet_service->store_id);

		//check default configuration
		$vivonet_service->configure();
		$settings = $vivonet_service->getCurrentConfiguration();
		$this->assertEquals("https://api.vivonet.com/v1/apiKeys/stores", $settings['url']);
		$this->assertTrue(in_array("x-api-key:736de47f4cdee9e8fe9ab8a0bc0eb37e", $settings['headers']));

		//check if the configuratios is correct for post order
		$vivonet_service->configure(
			VivonetService::ORDERING,
			VivonetService::PLACE_ORDER,
			array("storeId" => $vivonet_service->store_id),
			array("storeId"=> $vivonet_service->store_id)
		);
		$settings = $vivonet_service->getCurrentConfiguration();
		$this->assertEquals("https://api.vivonet.com/v1/stores/161668/orders", $settings['url']);
		$this->assertTrue(in_array("x-api-key:736de47f4cdee9e8fe9ab8a0bc0eb37e", $settings['headers']));
		$this->assertTrue(in_array("storeId:161668", $settings['headers']));

		$json = '{
			"orderId": 0,
			"externalSystemOrderId": "string",
			"orderPlacedBy": "splickit consulmer",
			"orderPlacerId": "161668",
			"orderLineItems": [{
				"price": 2.69,
				"remark": "Extra hot",
				"orderTypeId": 0,
				"quantityUnit": "item",
				"discounts": [],
				"ignorePrice": false,
				"quantity": 1,
				"modifiers": [{
					"price": 0.5,
					"remark": "fried",
					"orderTypeId": 0,
					"quantityUnit": "item",
					"discounts": [],
					"ignorePrice": false,
					"quantity": 1,
					"modifiers": [{
						"price": 0,
						"remark": "",
						"orderTypeId": 0,
						"quantityUnit": "item",
						"discounts": [],
						"ignorePrice": false,
						"quantity": 1,
						"modifiers": [],
						"productName": "Level 2 modifier",
						"orderLineItemId": 1,
						"productId": 5052759
					}],
					"productName": "Almond",
					"orderLineItemId": 1,
					"productId": 5052758
				}],
				"productName": "Canadiano 12oz",
				"orderLineItemId": 0,
				"productId": 5053321
			}],
			"charges": [],
			"payments": [{
				"amount": 0,
				"tenderId": 8767339,
				"paymentId": 1,
				"lineItemIds": [],
				"paymentMethod": {
					"securityCode": null,
					"expirationDate": "0000",
					"nameOnCard": "NOC Vince-NIT",
					"paymentMethodId": 0,
					"type": "RawCard",
					"cardNumber": "10000000000"
				}
			}],
			"pickupTime": '.(time()+600).'
		}';
		$result = $vivonet_service->send($json);

		$this->assertTrue($result['http_code'] === 200);

	}
    
    static function setUpBeforeClass()
    {
		$_SERVER['request_time1'] = microtime(true);
		$_SERVER['NO_MOCKS'] = false;
		$tz = date_default_timezone_get();
		$_SERVER['starting_tz'] = $tz;
		date_default_timezone_set(getProperty("default_server_timezone"));
		mysqli_query("BEGIN");

		$skin_resource = getOrCreateSkinAndBrandIfNecessary("Juice It Up!","Juice It Up!", 26, 198);
		$ids['skin_id'] = $skin_resource->skin_id;

		//map it to a menu
		$menu_id = createTestMenuWithNnumberOfItems(5);
		$ids['menu_id'] = $menu_id;

		$modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 10);
		$modifier_group_id = $modifier_group_resource->modifier_group_id;
		$item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
		assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 2);
		assignModifierGroupToItemWithFirstNAsComesWith($item_records[1]['item_id'], $modifier_group_id, 4);
		assignModifierGroupToItemWithFirstNAsComesWith($item_records[2]['item_id'], $modifier_group_id, 1);

		$merchant_resource = createNewTestMerchant($menu_id);
		attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
		$merchant_resource->set("merchant_external_id", 161668);
		$ids['merchant_id'] = $merchant_resource->merchant_id;
		$ids['merchant_external_id'] = $merchant_resource->merchant_external_id;

		$user_resource = createNewUser(array("flags"=>"1C20000001"));
		$ids['user_id'] = $user_resource->user_id;

		$_SERVER['log_level'] = 5;
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	mysqli_query("ROLLBACK");
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    VivonetTest::main();
}

?>