<?php

error_reporting(E_ERROR | E_COMPILE_ERROR | E_COMPILE_WARNING | E_PARSE);
$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');
$db_info->database = 'smaw_unittest';
$db_info->username = 'root';
$db_info->password = 'splickit';
if (isset($_SERVER['XDEBUG_CONFIG'])) {
    putenv("SMAW_ENV=unit_test_ide");
    $db_info->hostname = "127.0.0.1";
    $db_info->port = 13306;
} else {
    $db_info->hostname = "db_container";
    $db_info->port = 3306;
}
$_SERVER['DB_INFO'] = $db_info;

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class WorldPayServiceTest extends PHPUnit_Framework_TestCase
{
    var $stamp;
    var $ids;
    var $api_port = "80";

    function setUp()
    {
        $this->stamp = $_SERVER['STAMP'];
        $_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
        $this->ids = $_SERVER['unit_test_ids'];
        if (isset($_SERVER['XDEBUG_CONFIG'])) {
            $this->api_port = "10080";
        }
    }

    function tearDown()
    {
        //delete your instance
        $_SERVER['STAMP'] = $this->stamp;
        unset($this->ids);
    }

    function testMakePaymentRequest()
    {
        $user_resource = createNewUserWithCCNoCVV();
        $user_resource->contact_no = '303-884-4083';
        $user_resource->save();
        $merchant_id = $this->ids['merchant_id'];
        $world_pay_service = new WorldpayPaymentService(['merchant_id'=>$merchant_id]);
        $world_pay_service->setBillingUserIdAndResourceFromUserId($user_resource->user_id);
        $world_pay_service->order_id = rand(1111111111,9999999999);
        $amount = rand(5000,15000)/100;
        setProperty('verbose_curl','true');
        $response = $world_pay_service->processPayment($amount);
        $this->assertNotNull($response['purchasecode_id'],"It should have a purchasecode_id field");
        $this->assertTrue($response['purchasecode_id'] > 0,"It should have a valid purchasecode id that is an integer greater than 0");
        $this->assertEquals(2,$response['error'],"Error code should be 2");
        return $response;
    }

    /**
     * @depends testMakePaymentRequest
     */
    function testCheckStatus($response)
    {
        $merchant_id = $this->ids['merchant_id'];
        $world_pay_service = new WorldpayPaymentService(['merchant_id'=>$merchant_id]);
        setProperty('verbose_curl','true');
        $response2 = $world_pay_service->retrieveStatusOfPurchaseCode($response['purchasecode_id'],$response['code_table']);
        $this->assertEquals(1,$response2['code_status']);

        // confirm the purchase
        $response3 = $world_pay_service->retrieveStatusOfPurchaseCode($response['purchasecode_id'],$response['code_table']);
        $this->assertEquals(4,$response3['code_status']);

    }

    function testEndToEndCharge()
    {
        $user = logTestUserIn($this->ids['user_id']);

        $merchant_id = $this->ids['merchant_id'];
        $cart_data = OrderAdapter::getSimpleCartArrayByMerchantId($merchant_id);
        clearAuthenticatedUserParametersForSession();

        $url = "http://127.0.0.1:".$this->api_port."/app2/apiv2/cart/checkout";
        $response = $this->makeRequestWithDefaultHeaders($url, $user['email'].":welcome","POST",$cart_data);
        $checkout_resource = json_decode($response);
        $this->assertNull($checkout_resource->error);
        $ucid = $checkout_resource->data->ucid;

        $url = "http://127.0.0.1:".$this->api_port."/app2/apiv2/orders/$ucid";
        $order_data = array();
        $order_data['tip'] = 0.00;
        $payment_array = $checkout_resource->data->accepted_payment_types;
        $order_data['merchant_payment_type_map_id'] = $payment_array[0]->merchant_payment_type_map_id;

        $lead_times_array = $checkout_resource->data->lead_times_array;
        $order_data['requested_time'] = $lead_times_array[0];
        $response = $this->makeRequestWithDefaultHeaders($url, $user['email'].':welcome','POST',$order_data);
        $request_result_as_array = json_decode($response,true);
        $this->assertEquals('200', $this->info['http_code'],"an error was thrown: ".$request_result_as_array['error']['error']);
        $this->assertNull($request_result_as_array['error'],"an error was thrown: ".$request_result_as_array['error']['error']);
        $order_id = $request_result_as_array['data']['order_id'];

        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order_id],'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
        $payment_row = $bcr_hash['P2P'];
        $this->assertEquals('WorldPay',$payment_row['cc_processor']);
        $this->assertEquals('PENDING',$payment_row['notes']);
        $this->assertTrue($payment_row['cc_transaction_id'] > 0);

        // order will be in an open state but messages will all be set to 'X'
        $base_order_data = CompleteOrder::getBaseOrderData($order_id);
        $this->assertEquals('O',$base_order_data['status'],"It shoudl be in an open state");
        $order_message = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order_id,'E');
        $this->assertEquals('X',$order_message->locked,"It should have locked as 'X' for all order message");
        return $payment_row['cc_transaction_id'];
    }

    /**
     * @depends testEndToEndCharge
     */
    function testCallBackFromWorldPay($transaction_id)
    {
        $this->assertTrue(false,"NOT DOING THIS TEST ANYMORE");
        $user_resource = getUserResourceFromId($this->ids['user_id']);
        $data = ['purchasecode_id'=>$transaction_id,'success'=>1,'phonesale'=>$user_resource->contact_no,'amount'=>96.80];
        $url = "http://127.0.0.1:".$this->api_port."/app2/apiv2/payments";

//        $user = logTestUserIn(1);
//        $request = createRequestObject($url,'POST',json_encode($data));
//        $order_controller = new OrderController(getM(),$user,$request);
//        $response = makeMockRequest($order_controller);
        $external_id = getPublicClientIdForContext();
        $headers = array("X_ORDER140_CLIENT_ID"=>"$external_id","X_ORDER140_CLIENT_DEVICE"=>"unit_testing","HTTP_X_ORDER140_CLIENT_VERSION"=>"10.5.0","X_ORDER140_CLIENT"=>"APIDispatchTest","NO_CC_CALL"=>"true");
        $header_array = array();
        foreach ($headers as $key=>$value) {
            $header_array[] = $key.":".$value;
        }
        $response = $this->makeRequest($url,"admin:Test1234",'POST',$header_array,$data);
        $rar = json_decode($response,true);
        $this->assertEquals(422,$rar['http_code']);
        $this->assertContains(OrderController::UNABLE_TO_VALIDATE_P2P_TRANSACTION_ID_ERROR_MESSAGE,$rar['error']['error']);

        clearAuthenticatedUserParametersForSession();
        removeContext();

        $world_pay_skin = $this->ids['worldpay_skin_resource'];
        $wpi = $world_pay_skin->external_identifier;
        setContext($wpi);

//        $user = logTestUserIn(1);
//        $request = createRequestObject($url,'POST',json_encode($data));
//        $order_controller = new OrderController(getM(),$user,$request);
//        $response = makeMockRequest($order_controller);
        $external_id = getPublicClientIdForContext();
        $headers = array("X_ORDER140_CLIENT_ID"=>"$external_id","X_ORDER140_CLIENT_DEVICE"=>"unit_testing","HTTP_X_ORDER140_CLIENT_VERSION"=>"10.5.0","X_ORDER140_CLIENT"=>"APIDispatchTest","NO_CC_CALL"=>"true");
        $header_array = array();
        foreach ($headers as $key=>$value) {
            $header_array[] = $key.":".$value;
        }
        $response = $this->makeRequest($url,"admin:Test1234",'POST',$header_array,$data);

        $rar = json_decode($response,true);
        $this->assertEquals(200,$rar['http_code']);
        $this->assertEquals('success',$rar['data']['result']);

        $response_stamp = $rar['stamp'];
        // now confirm payment row was updated
        $ucid = $rar['data']['order_id'];
        $order = new Order($ucid);
        $bcr = BalanceChangeAdapter::staticGetRecords(['order_id'=>$order->getOrderId()],'BalanceChangeAdapter');
        $bcr_hash = createHashmapFromArrayOfArraysByFieldName($bcr,'process');
        $payment_row = $bcr_hash['P2P'];
        $this->assertEquals('WorldPay',$payment_row['cc_processor']);
        $this->assertEquals("COMPLETED-$response_stamp",$payment_row['notes']);

        // confirm that message were released
        $order_message = MerchantMessageHistoryAdapter::getMessageByOrderIdAndFormat($order->getOrderId(),'E');
        $this->assertEquals('N',$order_message->locked,"It should have un-locked the message");

    }

    function makeRequestWithDefaultHeaders($url,$userpassword,$method = 'GET',$data = null,$external_id = null)
    {
        if ($external_id == null) {
            $external_id = getContext();
        }
        $headers = array("X_ORDER140_CLIENT_ID"=>"$external_id","X_ORDER140_CLIENT_DEVICE"=>"unit_testing","HTTP_X_ORDER140_CLIENT_VERSION"=>"10.5.0","X_ORDER140_CLIENT"=>"APIGiveXTest","NO_CC_CALL"=>"true");
        if ($authentication_token = $data['splickit_authentication_token']) {
            $headers['splickit_authentication_token'] = $authentication_token;
            unset($data['splickit_authentication_token']);
        }
        if ($data['headers']) {
            $headers = array_merge($headers,$data['headers']);
            unset($data['headers']);
        }
        $header_array = array();
        foreach ($headers as $key=>$value) {
            $header_array[] = $key.":".$value;
        }
        return $this->makeRequest($url,$userpassword,$method,$header_array,$data);
    }

    function makeRequest($url,$userpassword,$method = 'GET', $headers = array(),$data = null)
    {
        unset($this->info);
        $method = strtoupper($method);
        $curl = curl_init($url);
        if ($userpassword) {
            curl_setopt($curl, CURLOPT_USERPWD, $userpassword);
        }
        if ($method == 'POST') {
            $json = json_encode($data);
            curl_setopt($curl, CURLOPT_POST, 1);
            curl_setopt($curl, CURLOPT_POSTFIELDS,$json);
            $headers[] = 'Content-Type: application/json';
            $headers[] = 'Content-Length: ' . strlen($json);
        } else if ($method == 'DELETE') {
            curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "DELETE");
        }
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        logCurl($url,$method,$userpassword,$headers,$json);
        $result = curl_exec($curl);
        $this->info = curl_getinfo($curl);
        curl_close($curl);
        return $result;
    }


    static function setUpBeforeClass()
    {
        $code = setNewStamp();
        $_SERVER['request_time1'] = microtime(true);
        $tz = date_default_timezone_get();
        $_SERVER['starting_tz'] = $tz;
        date_default_timezone_set(getProperty("default_server_timezone"));
        ini_set('max_execution_time',300);

        $worldpay_skin_resource = getOrCreateSkinAndBrandIfNecessaryWithLoyalty("WorldPay","WorldPay",null, null);
        $worldpay_skin_resource->password = Encrypter::Encrypt("Test1234");
        $worldpay_skin_resource->save();
        $ids['worldpay_skin_resource'] = $worldpay_skin_resource;
        $skin_resource = getOrCreateSkinAndBrandIfNecessaryWithLoyalty("maryjane","maryjane",null, null);
        $skin_resource->password = Encrypter::Encrypt("Test1234");
        $skin_resource->save();
        $ids['skin_id'] = $skin_resource->skin_id;
        setContext('com.splickit.maryjane');

        $simple_menu_id = createTestMenuWithOneItem("Virgin Mary");
        $ids['menu_id'] = $simple_menu_id;

        $item_sizes = CompleteMenu::getAllItemSizesAsResources($simple_menu_id,0);
        foreach ($item_sizes as $item_size) {
            $item_size->price = 88.00;
            $item_size->save();
        }
        $merchant_resource = createNewTestMerchant($simple_menu_id,['no_payment'=>true]);
        $merchant_resource->save();
        $ids['merchant_id'] = $merchant_resource->merchant_id;

        $merchant_payment_type_map_resource = MerchantPaymentTypeMapsAdapter::createMerchantPaymentTypeMap($merchant_resource->merchant_id, 13010, null);

        $tpppa = new ThirdPartyProviderParametersAdapter(getM());
        $parameters = $tpppa->getParametersFromProviderId(7001);
        foreach ($parameters as $parameter_row) {
            $data = [];
            $data['third_party_provider_parameter_id'] = $parameter_row['third_party_provider_parameter_id'];
            if ($parameter_row['name'] == 'worldpay_merchant_id') {
                $data['value'] = '3';
                $data['merchant_id'] = $merchant_resource->merchant_id;
                $tpppmma = new ThirdPartyProviderParameterMerchantMapsAdapter(getM());
                Resource::createByData($tpppmma,$data);
            }
        }

        $user_resource = createNewUserWithCCNoCVV();
        $user_id = $user_resource->user_id;
        $ids['user_id'] = $user_id;


        $_SERVER['log_level'] = 5;
        $_SERVER['unit_test_ids'] = $ids;
    }


    static function tearDownAfterClass()
    {

    }


    /* mail method for testing */
    static function main() {
        $suite = new PHPUnit_Framework_TestSuite( __CLASS__);
        PHPUnit_TextUI_TestRunner::run( $suite);
    }

}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    WorldPayServiceTest::main();
}
?>