<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class HeartlandValidationScript extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
		
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }
    
    function createAccount()
    {
        //$url = "https://puat.splickit.com/app2/apiv2/users?log_level=5";
        $url = "https://10.63.47.222/app2/apiv2/users?log_level=5";
    	$new_user_data = createNewUserDataFields();
		$new_user_data['first_name'] = "Joe";
		$new_user_data['last_name'] = "Pita";
        $code = strtolower(generateCode(4));
        $new_user_data['email'] = "tester-$code@pitapit.com";
		$response = $this->makeRequest($url, "admin:welcome","POST",$new_user_data);
		$this->assertEquals('200', $this->info['http_code']);
		$new_user_response = json_decode($response,true);
		$this->assertNull($new_user_response['error'],"should NOT have gotten an error");
		$user = $new_user_response['data'];
		$this->assertNotNull($user['user_id']);

    	$hls = PitapitLoyaltyController::loyaltyServiceFactory();
    	$heartland_user_result = $hls->createUserKatana($user['email'],'welcome');
    	$this->assertEquals(204,$heartland_user_result['http_code']);
    	$account_creation_result = $hls->createAccountKatana($user['email'],'welcome');
    	$this->assertEquals('success', $account_creation_result['status']);
    	$account_number = $account_creation_result['id'];
    	$this->assertNotNull($account_number);
    	$pin = $account_creation_result['pin'];
    	$this->assertNotNull($pin);
    	$account['account_number'] = $account_number;
    	$account['pin'] = $pin;
    	$account['user'] = $user;
    	myerror_log("HEARTLAND SCRIPT created account: ".$account_number);
    	echo "\r\nuserpass: ".$user['email'].":welcome \r\n";
    	echo "account number: $account_number \r\n";
    	echo "pin: $pin \r\n";
    	return $account;    	
    }
    
    function loadStoredValueFromCC($account,$amount)
    {
    	$user = $account['user'];
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
    	$cc_number = '4788250000028291';
    	$exp_date = '1014';
    	$cvv = '111';
    	$full_name = $user['first_name'].' '.$user['last_name'];
    	$hls = PitapitLoyaltyController::loyaltyServiceFactory();
    	//$result = $hls->addStoredValueFromCC($sva,$pin,$amount,$cc_number,$exp_date,$cvv,$full_name,$user['email'],'welcome');
        $hls->setStore('splickitstore');
        $result = $hls->addDollars($sva,$pin,$amount);
    	return (200 == $result['http_code']);
    }
    
    function chargeStoredValue($account,$amount,$pin)
    {
    	$sva = $account['account_number'];
    	$pin = ($pin == null) ? $account['pin'] : $pin;
    	$hls = PitapitLoyaltyController::loyaltyServiceFactory();
    	$hls->setStore('us-1000');
    	$result = $hls->chargeAgainstStoredValue($sva, $pin, $amount);
    	return (200 == $result['http_code']);
    }
    
    function getAccountInfoArcher($account)
    {
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
    	$hls = PitapitLoyaltyController::loyaltyServiceFactory();
		$result = $hls->getBalance($sva, $pin);
		return (200 == $result['http_code']);
    }
    
    function rewardPoints($account,$amount_of_purchase)
    {
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
		$hls = PitapitLoyaltyController::loyaltyServiceFactory();
		$hls->setStore('splickitstore');
		$result = $hls->rewardPoints($sva,$pin,$amount_of_purchase);
		return (200 == $result['http_code']);
    }
    
    function addPoints($account,$points)
    {
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
		$hls = PitapitLoyaltyController::loyaltyServiceFactory();
		$hls->setStore('splickitstore');
		$result = $hls->addPoints($sva,$pin,$points);
		return (200 == $result['http_code']);    	
    }

    function testDoIt()
    {
        for($i=0;$i<4;$i++) {
            $account = $this->createAccount();

            $this->assertTrue($this->loadStoredValueFromCC($account, 100.00));
            $this->assertTrue($this->addPoints($account, 1000));

            $account_info = $this->getAccountInfoKatana($account);
            $this->assertEquals(200, $account_info['http_code']);
            myerror_log("account info recieved");
            echo "card loaded with 1000 points \r\n";
            echo "card loaded with 100 USD \r\n";
            echo "\r\n";
        }
    }
    
    function getAccountInfoKatana($account)
    {
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
    	$hls = PitapitLoyaltyController::loyaltyServiceFactory();
		$result = $hls->getKatanaAccountInfoWithQRCodeNumber($sva, $pin);
		//return (200 == $result['http_code']);
		return $result;
    }
    
    function getHistory($account)
    {
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
    	$user = $account['user'];
		$hls = PitapitLoyaltyController::loyaltyServiceFactory();
		$result = $hls->getHistory($sva,$pin,$user['email']);
		return (200 == $result['http_code']);
    }
    
    function makeRequest($url,$userpassword,$method = 'GET',$data = null)
    {
    	unset($this->info);
    	$method = strtoupper($method);
    	$curl = curl_init($url);
    	if ($userpassword) {
    		curl_setopt($curl, CURLOPT_USERPWD, $userpassword);
    	}
    	//if ($external_id = getContext()) {
    		// use it
    //	} else {
    		$external_id = "com.splickit.pitapit";
    //	}
    	$headers = array("X_SPLICKIT_CLIENT_ID:$external_id","X_SPLICKIT_CLIENT_DEVICE:unit_testing","X_SPLICKIT_CLIENT:APIDispatchTest","NO_CC_CALL:true");
    	if ($authentication_token = $data['splickit_authentication_token']) {
    		$headers[] = "splickit_authentication_token:$authentication_token";
    	}
    	if ($method == 'POST') {
    		$json = json_encode($data);
    		curl_setopt($curl, CURLOPT_POST, 1);
			curl_setopt($curl, CURLOPT_POSTFIELDS,$json);
			$headers[] = 'Content-Type: application/json';
			$headers[] = 'Content-Length: ' . strlen($json);
    	} else if ($method == 'DELETE') {
   			curl_setopt($curl, CURLOPT_CUSTOMREQUEST, "DELETE"); 		
    	}
		curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
		curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($curl, CURLOPT_VERBOSE, 0);
		//curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
		curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
		$result = curl_exec($curl);
		$this->info = curl_getinfo($curl);
		curl_close($curl);
    	return $result;
    }

    static function setUpBeforeClass()
    {
    	ini_set('max_execution_time',300);
    	$_SERVER['request_time1'] = microtime(true);
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	mysqli_query("BEGIN");
    	
    	$skin_resource = createWorldHqSkin();
    	$ids['skin_id'] = $skin_resource->skin_id;
    	
		//map it to a menu
    	$menu_id = createTestMenuWithNnumberOfItems(5);
    	$ids['menu_id'] = $menu_id;
    	
    	$modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 10);
    	$modifier_group_id = $modifier_group_resource->modifier_group_id;
    	$item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 2);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[1]['item_id'], $modifier_group_id, 4);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[2]['item_id'], $modifier_group_id, 1);

    	$merchant_resource = createNewTestMerchant($menu_id);
    	attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
    	$ids['merchant_id'] = $merchant_resource->merchant_id;
    	
    	$user_resource = createNewUser(array("flags"=>"1C20000001"));
    	$ids['user_id'] = $user_resource->user_id;
    	
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	mysqli_query("ROLLBACK");
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['XDEBUG_CONFIG']) && !defined('PHPUnit_MAIN_METHOD')) {
    HeartlandValidationScript::main();
}

?>