<?php

$filepathParts = pathinfo(__FILE__);
$path = $filepathParts['dirname'];
chdir($path . '/../');

require_once 'lib/utilities/functions.inc';
require_once 'lib/utilities/unit_test_functions.inc';

class HeartlandValidationScript extends PHPUnit_Framework_TestCase
{
	var $stamp;
	var $ids;

	function setUp()
	{
		$_SERVER['HTTP_NO_CC_CALL'] = 'true';
		$this->stamp = $_SERVER['STAMP'];
		$_SERVER['STAMP'] = __CLASS__.'-'.$_SERVER['STAMP'];
		$this->ids = $_SERVER['unit_test_ids'];
		
	}
	
	function tearDown() 
	{
		//delete your instance
		$_SERVER['STAMP'] = $this->stamp;
		unset($this->ids);
    }
    
    function createAccount()
    {
    	$user_resource = createNewUser();
    	$user = $user_resource->getDataFieldsReally();
    	$hls = new HeartlandLoyaltyService();
    	$heartland_user_result = $hls->createUserKatana($user['email'],'password');
    	$this->assertEquals(204,$heartland_user_result['http_code']);
    	$account_creation_result = $hls->createAccountKatana($user['email'],'password');
    	$this->assertEquals('success', $account_creation_result['status']);
    	$account_number = $account_creation_result['id'];
    	$this->assertNotNull($account_number);
    	$pin = $account_creation_result['pin'];
    	$this->assertNotNull($pin);
    	$account['account_number'] = $account_number;
    	$account['pin'] = $pin;
    	$account['user'] = $user;
    	myerror_log("HEARTLAND SCRIPT created account: ".$account_number);
    	return $account;    	
    }
    
    function loadStoredValueFromCC($account,$amount)
    {
    	$user = $account['user'];
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
    	$cc_number = '4788250000028291';
    	$exp_date = '1014';
    	$cvv = '111';
    	$full_name = $user['first_name'].' '.$user['last_name'];
    	$hls = new HeartlandLoyaltyService();
    	$result = $hls->addStoredValueFromCC($sva,$pin,$amount,$cc_number,$exp_date,$cvv,$full_name,$user['email'],'password');
    	return (200 == $result['http_code']);
    }
    
    function chargeStoredValue($account,$amount,$pin)
    {
    	$sva = $account['account_number'];
    	$pin = ($pin == null) ? $account['pin'] : $pin;
    	$hls = new HeartlandLoyaltyService();
    	$hls->setStore('us-1000');
    	$result = $hls->chargeAgainstStoredValue($sva, $pin, $amount);
    	return (200 == $result['http_code']);
    }
    
    function getAccountInfoArcher($account)
    {
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
    	$hls = new HeartlandLoyaltyService();
		$result = $hls->getBalance($sva, $pin);
		return (200 == $result['http_code']);
    }
    
    function rewardPoints($account,$amount_of_purchase)
    {
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
		$hls = new HeartlandLoyaltyService();
		$hls->setStore('splickitstore');
		$result = $hls->rewardPoints($sva,$pin,$amount_of_purchase);
		return (200 == $result['http_code']);
    }

    function testScriptArcherButNotReally()
    {
    	myerror_log("HEARTLAND SCRIPT starting archer tests");
    	$account1 = $this->createAccount();
    	$account2 = $this->createAccount();
    	$account3 = $this->createAccount();
    	
    	$this->assertTrue($this->loadStoredValueFromCC($account1, 20.00));
    	$this->assertTrue($this->loadStoredValueFromCC($account2, 10.00));
    	
		$this->assertTrue($this->chargeStoredValue($account2, 10.00));    	
    	$this->assertFalse($this->chargeStoredValue($account3, 5.00));  
		$this->assertFalse($this->chargeStoredValue($account2, 8.00));    	
		$this->assertFalse($this->chargeStoredValue($account1, 8.00,'8888'));
		
		$this->assertTrue($this->loadStoredValueFromCC($account2, 10.00));
    	$this->assertFalse($this->loadStoredValueFromCC($account2, 0.01));
    	
    	// ******** partials go here ********
    	  // no partials for now   V2
		//   ******  end partials ******
    	
    	$this->assertTrue($this->getAccountInfoArcher($account1));
    	
    	$this->assertTrue($this->rewardPoints($account3, 10.00));
    	$this->assertTrue($this->rewardPoints($account3, 15.00));
    	
    }
    
    function getAccountInfoKatana($account)
    {
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
    	$hls = new HeartlandLoyaltyService();
		$result = $hls->getKatanaAccountInfoWithQRCodeNumber($sva, $pin);
		return (200 == $result['http_code']);
    }
    
    function getHistory($account)
    {
    	$sva = $account['account_number'];
    	$pin = $account['pin'];
    	$user = $account['user'];
		$hls = new HeartlandLoyaltyService();
		$result = $hls->getHistory($sva,$pin,$user['email']);
		return (200 == $result['http_code']);
    }
    
    function testScriptKatana()
    {
    	myerror_log("HEARTLAND SCRIPT starting katana tests");
    	$account = $this->createAccount();
    	$this->assertTrue($this->getAccountInfoKatana($account));
    	$this->assertTrue($this->getHistory($account));
    }
    
    private function reverseRequest($stamp,$username_password)
    {
    	$url = 'https://api.cert.chockstone.com/archer/v1/reverse';
		$archer_data = array('domain'=>'pitapit','terminal'=>'0001','terminal.order'=>'0001','terminal.version'=>'v17');
		$archer_data['store'] = 'splickitstore';		
		$archer_data['reversal'] = "$stamp";
		$i = 1;
		do {
			$archer_data['request'] = "$stamp-$i";
			$response = HeartlandCurl::curl($url, 'post', $archer_data, "$username_password");
			logData($response, 'reverse');
			$i++;
		} while ($response['http_code'] != 200);
    }
    
	function testScriptTimeOutReversalRedeemLoyalty()
    {
    	
		$stamp = generateCode(10);
    	
		$sva = '5022440000014154303';
    	$username = 'test_user';
    	$password = 'test_password';    	
    	
		$url = 'https://api.cert.chockstone.com/archer/v1/redeem';
		$archer_data = array('domain'=>'pitapit','terminal'=>'0001','terminal.order'=>'0001','terminal.version'=>'v17');
		$archer_data['sva'] = $sva;
		$archer_data['amount'] = 101;
		$archer_data['currency'] = "Points";
		$archer_data['store'] = 'splickitstore';		
		//$archer_data['pin'] = $pin;
		$archer_data['acquired'] = 'MANUAL';
		$archer_data['partial'] = false;
		$archer_data['request'] = "$stamp";
		
		$response = HeartlandCurl::curl($url, 'post', $archer_data, "$username:$password");
		$this->assertContains('ServiceUnavailable', $response['raw_result']);
		$this->assertEquals(503, $response['http_code']);
		$this->reverseRequest($stamp,"$username:$password");
    }
    
	function testScriptTimeOutReversalRedeemGift()
    {
    	$stamp = generateCode(10);    	
    	$sva = '5022440000014154303';
    	$username = 'test_user';
    	$password = 'test_password';
		$amount_in_pennies = 101;
		$url = 'https://api.cert.chockstone.com/archer/v1/redeem';
		$archer_data = array('domain'=>'pitapit','terminal'=>'0001','terminal.order'=>'0001','terminal.version'=>'v17');
		$archer_user_password = 'splickit:12345';
		$archer_data['sva'] = $sva;
		$archer_data['amount'] = "$amount_in_pennies";
		$archer_data['currency'] = "USD";
		$archer_data['store'] = 'splickitstore';		
		//$archer_data['pin'] = $pin;
		$archer_data['acquired'] = 'MANUAL';
		$archer_data['partial'] = false;
		$archer_data['request'] = "$stamp";
		
		$response = HeartlandCurl::curl($url, 'post', $archer_data, "$username:$password");
		$this->assertContains('ServiceUnavailable', $response['raw_result']);
		$this->assertEquals(503, $response['http_code']);
		$this->reverseRequest($stamp,"$username:$password");
    }
        
    function testScriptTimeOutReversalReward()
    {
    	$stamp = generateCode(10);    	
    	$sva = '5022440000014154303';
    	$username = 'test_user';
    	$password = 'test_password';
		$url = 'https://api.cert.chockstone.com/archer/v1/reward';
		$archer_data = array('domain'=>'pitapit','terminal'=>'0001','terminal.order'=>'0001','terminal.version'=>'v17');
		$archer_user_password = 'splickit:12345';
		$archer_data['sva'] = $sva;
		$archer_data['amount'] = 101;
		$archer_data['currency'] = "USD";
		$archer_data['store'] = 'splickitstore';		
		//$archer_data['pin'] = $pin;
		$archer_data['acquired'] = 'MANUAL';
		$archer_data['partial'] = false;
		$archer_data['request'] = "$stamp";
		
		$response = HeartlandCurl::curl($url, 'post', $archer_data, "$username:$password");
		$this->assertContains('ServiceUnavailable', $response['raw_result']);
		$this->assertEquals(503, $response['http_code']);
		$this->reverseRequest($stamp,"$username:$password");		
    }
    
    static function setUpBeforeClass()
    {
    	ini_set('max_execution_time',300);
    	$_SERVER['request_time1'] = microtime(true);
    	$tz = date_default_timezone_get();
    	$_SERVER['starting_tz'] = $tz;
    	date_default_timezone_set(getProperty("default_server_timezone"));
    	mysqli_query("BEGIN");
    	
    	$skin_resource = createWorldHqSkin();
    	$ids['skin_id'] = $skin_resource->skin_id;
    	
		//map it to a menu
    	$menu_id = createTestMenuWithNnumberOfItems(5);
    	$ids['menu_id'] = $menu_id;
    	
    	$modifier_group_resource = createModifierGroupWithNnumberOfItems($menu_id, 10);
    	$modifier_group_id = $modifier_group_resource->modifier_group_id;
    	$item_records = CompleteMenu::getAllMenuItemsAsArray($menu_id, 'Y', $mimetypes);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[0]['item_id'], $modifier_group_id, 2);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[1]['item_id'], $modifier_group_id, 4);
    	assignModifierGroupToItemWithFirstNAsComesWith($item_records[2]['item_id'], $modifier_group_id, 1);

    	$merchant_resource = createNewTestMerchant($menu_id);
    	attachMerchantToSkin($merchant_resource->merchant_id, $ids['skin_id']);
    	$ids['merchant_id'] = $merchant_resource->merchant_id;
    	
    	$user_resource = createNewUser(array("flags"=>"1C20000001"));
    	$ids['user_id'] = $user_resource->user_id;
    	
    	$_SERVER['log_level'] = 5; 
		$_SERVER['unit_test_ids'] = $ids;
    }
    
	static function tearDownAfterClass()
    {
    	mysqli_query("ROLLBACK");
    	date_default_timezone_set($_SERVER['starting_tz']);
    }

    /* mail method for testing */
    static function main() {
		$suite = new PHPUnit_Framework_TestSuite( __CLASS__);
  		PHPUnit_TextUI_TestRunner::run( $suite);
 	}
    
}

if (isset($_SERVER['argv'][1]) && $_SERVER['argv'][1] == 'use_main'  && !defined('PHPUnit_MAIN_METHOD')) {
    HeartlandValidationScript::main();
}

?>